<?php
/**
 * stanislav.elbl, 18.12.2019
 * Zajistuje automaticke schvaleni/zamitnuti absence z linku (napr. z emailu) bez prihlaseni
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\Model;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\RonPortalSettings;

class AbsenceLinkController extends AbsenceController
{
    const SETTINGS_SCHVALOVANI_BEZ_PRIHLASENI = 'schvalabs_bezprihlaseni';

    /**
     * AbsenceController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct($database);
        if (RonPortalSettings::getIntValue(self::SETTINGS_SCHVALOVANI_BEZ_PRIHLASENI)) {
            $this ->enableAccessWithoutLogin = true;
        }
    }


    public function provedAkciZLinku($akce)
    {
        $rec = $this ->nactiAbsenciRecord((int)$_GET['idpp'], $_GET['datum'], (int)$_GET['poradi']);

        if (!$rec ->overKlicProSchvaleni($akce, $_GET['linkid'], $_GET['key'])) {
            throw new \Exception('Spatny kod pro schvaleni!');
        }

        $okInfo = '';

        switch ($akce) {
            case 'schvalitabsenci' :
                $hodnota = 1;
                $jizSchvalenaInfo = 'Absence_je_jiz_schvalena';
                $okInfo = 'Schvaleni_absence_probehlo';
                break;

            case 'zamitnoutabsenci' :
                $hodnota = 2;
                $jizSchvalenaInfo = 'Absence_je_jiz_zamitnuta';
                $okInfo = 'Zamitnuti_absence_probehlo';
                break;

            default :
                throw new \Exception('Chyba ve funkci provedAkciZLinku: neznama akce!');
        }

        if ($rec ->fSchvaleni ->value == $hodnota) {
            MessageManager::addInfoMessage(\Ron\ronText($jizSchvalenaInfo));
            return;
        }

        AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $rec ->fDatumOd ->value, $rec ->fDatumDo ->value, $rec ->pp, false);

        $rec ->addLogText("Prime schvaleni z odkazu\nURL: " . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);

        $rec->nastavSchvaleni($hodnota);

        $rec->schvaleni_Email($hodnota);

        MessageManager::addInfoMessage(\Ron\ronText($okInfo));
    }


    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');

        try {
            if (!RonPortalSettings::getIntValue(self::SETTINGS_SCHVALOVANI_BEZ_PRIHLASENI)) {
                throw new NeniPravoException();
            }

            if (!isset($_GET['key']) || !isset($_GET['linkid']) || $_GET['linkid'] == '') {
                throw new \Exception('Neni zadan kod pro schvaleni absence!');
            }

            switch ($_GET['action']) {
                case 'schvalitabsenci' :
                case 'zamitnoutabsenci' :
                    $this->provedAkciZLinku($_GET['action']);
                    break;

                default:
                    throw new NeniPravoException();
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        Model::addViewObject(new MessageManagerViewObject());
        return 'view/appview_nouser.php';
    }
}


return new AbsenceLinkController($this ->database);
