<?php
/**
 * UTF8
 * stanislav.elbl, 30.03.2016
 */

namespace RonPortal\Editace;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\TextManager;
use RonPortal\Controls\RozdeleniShiftFormView;
use RonPortal\RonPortalSettings;
use RonPortal\TypRozdeleni;
use RonPortal\XML\RonPortalXMLForm;

class EditaceDneController extends HttpController
{
    /** @var Database */
    public $database;

    /**
     * EditaceDochazkyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    private function redirect($idpp, $den)
    {
        if (($_SESSION['editace_backaction'] ?? null) === 'rozdelenidv') {
            return new Redirect(null, '?menuid=rozdelenidv&idpp=' . $idpp . '&obdobi=' . DateUtils::dateToUrlString(DateUtils::firstDayOfMonth($den)));
        }

        return new Redirect(null, '?menuid=zpracovanadochazka&idpp=' . $idpp . '&datum=' . DateUtils::dateToUrlString($den));
    }


    public function handleAction($action)
    {
        TextManager::getInstance()
            ->loadStrings('dochazka.ini')
            ->loadStrings('vyroba.ini');

        HtmlHeaders::getInstance()
            ->setTitle(\Ron\ronText('RON_Portal') . ' - ' . \Ron\ronText('Editace_dochazky'))
            ->addScript('script/dochazka.js')
            ->addScript('script/vyroba.js')
            ->addCss('css/dochazka.css');

        $this ->addTranslationsScript('editace');

        GuiUtils::scrollPageHeader();

        $f = new EditaceDneFormular($this ->database);

        if (isset($_GET['action'])  &&  $_GET['action']) {
            return $f ->provedAkci($_GET['action']);
        }

        if ( (isset($_POST['ulozzaznamyproden'])  &&  $_POST['ulozzaznamyproden'])  ||  (isset($_POST['ulozrozdelenivysledku'])  &&  $_POST['ulozrozdelenivysledku']) ) {
            // nacteni a zpracovani dat odeslanych uzivatelem
            try {
                if ($f->save()) {
                    return RonPortalXMLForm::$eventRedirect ?: $this->redirect($f->pp->idPP, $f->den);
                }
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
            }
        } else {
            if (!$_GET['datum'] || !$_GET['idpp']) {
                return new Redirect(null, '?menuid=zpracovanadochazka');
            }

            // nacteni dat z databaze
            $f ->nactiDataProDen((int)$_GET['idpp'], DateUtils::urlStringToDate($_GET['datum'], true)); // enableTimestamp=true - prozatim
        }

        return $f;
    }
}


return new EditaceDneController($this ->database);
