<?php
/**
 * stanislav.elbl, 20.05.2016
 */

namespace RonPortal\Editace;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;


class EditaceMesicnichViewObject implements ViewObject
{
    /** @var RonPortalFilterForm */
    var $filterForm;
    /** @var  EditaceMesicnichHromadna */
    var $editaceMesicnich;

    /**
     * EditaceMesicnichViewObject constructor.
     * @param RonPortalFilterForm $filterForm
     * @param EditaceMesicnichHromadna $editaceMesicnich
     */
    public function __construct(RonPortalFilterForm $filterForm, EditaceMesicnichHromadna $editaceMesicnich)
    {
        $this->filterForm = $filterForm;
        $this->editaceMesicnich = $editaceMesicnich;
    }


    public function render()
    {
        echo '<h1>' . TextManager::_('Editace_mesicnich_vysledku') . '</h1>';

        $this ->filterForm ->render();

        $url =
            '?menuid=editacemesicnich' .
            '&obdobi=' . DateUtils::dateToUrlString($this ->editaceMesicnich ->obdobi ->zacatek) .
            '&rozdeleni=' . ($this ->filterForm ->getFilterValue('rozdeleni') !== null
                ? StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni'))
                : ''
            ) .
            '&rozdeleni2=' . ($this ->filterForm ->getFilterValue('rozdeleni2') !== null
                ? StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni2'))
                : ''
            );

        $this ->editaceMesicnich ->zobrazFormular($url);
    }
}


class EditaceMesicnichController extends HttpController
{
    /** @var Database */
    var $database;

    /** @var RonPortalFilterForm */
    var $filterForm;
    /** @var  EditaceMesicnichHromadna */
    var $editaceMesicnich;

    /**
     * EditaceMesicnichController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    private function zobrazEditaciMesicnich()
    {
        $this ->editaceMesicnich ->nactiSeznamOsobPodlePrav($this ->filterForm);

        return new EditaceMesicnichViewObject($this ->filterForm, $this ->editaceMesicnich);
    }


    /**
     * Zapis dat - volano z controlleru pro standardni akci menuid=editacemesicnich
     */
    private function zapisData()
    {
        $this ->editaceMesicnich ->nactiSeznamOsobPodlePrav($this ->filterForm);
        $this ->editaceMesicnich ->nactiPost();

//        $this ->editaceMesicnich ->nactiSeznamOsobPodlePost(RonPortalSession::obdobi(), false); // povolitVse = false - prava dle uzivatele
//        $this ->editaceMesicnich ->nactiLimityRozdeleni();

        try {
            if ($this->editaceMesicnich->jeChyba) {
                throw new \Exception(TextManager::_('Spatna_hodnota'));
            }

            if (!$this->editaceMesicnich->kontrolaObdobi()) {
                throw new \Exception(TextManager::_('Obdobi_uzavreno'));
            }

            if ($this->editaceMesicnich->zkontrolujLimity()) {
                $this->editaceMesicnich->zapisData();
            } else {
                MessageManager::addErrorMessage(TextManager::_('Prekrocen_limit_MP'));

                if ($this->editaceMesicnich->jeChybaNeprekrocitelnyLimit && $this->editaceMesicnich->textChybyNeprekrocitelnyLimit) {
                    MessageManager::addErrorMessage($this->editaceMesicnich->textChybyNeprekrocitelnyLimit);
                }
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        if (!MessageManager::getInstance() ->isError()) {
            return new Redirect(null, '?menuid=editacemesicnich&obdobi=' . DateUtils::dateToUrlString($this ->editaceMesicnich ->obdobi ->zacatek));
        }

        return new EditaceMesicnichViewObject($this ->filterForm, $this ->editaceMesicnich);
    }


    /**
     * @throws NeniPravoException
     * @throws DBException
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance()
            ->addCss('css/dochazka.css')
            ->setAppTitle('Editace_mesicnich_vysledku');
        $this ->addTranslationsScript('editacemesicnich');

        if (isset($_GET['action']) && $_GET['action'] == 'translations') {
            return $this ->translations();
        }


        if (!Prava::vratHodnotuPrava('vkladani_mzdpol') // )
            && !Prava::vratHodnotuPrava('editace_mv')     // predelano 19.5.2016 puvodne tu bylo 'editace'
        ) {
            throw new NeniPravoException();
        }

        $this ->filterForm = new RonPortalFilterForm($this ->database, 'editacemesicnich', true);
        $this ->filterForm ->addFilterObdobi();
        $filterRozdeleni = $this ->filterForm ->addFilterRozdeleni1(null, Prava::vratPrava('prohlizeni'));
        $this ->filterForm ->addFilterRozdeleni2(null, Prava::vratPrava('prohlizeni'));

        $this ->filterForm ->loadData();

        $loadPost = (isset($_POST['zapiseditacimesicnich']) && $_POST['zapiseditacimesicnich']);

        $this ->editaceMesicnich = new EditaceMesicnichHromadna($this ->database,
            $this ->filterForm ->getFilterValue('rozdeleni', 'typRozdeleni'),
            $loadPost ? null : $this ->filterForm ->getFilterValue('rozdeleni'));  // rozdeleni pro mzdove polozky - pro POST je potreba nejdrive nacist nastaveni

        // od 06-2018 nacitam prednostne z SYS_BVAR_GLOBAL
        $settings = RonPortalSettings::getBvarSettings($this ->database,
            'RonPortal_Editace_mesicnich',
            RonPortalSettings::getMultilineValue('editacemesicnich')); // puvodni hodnota z nastaveni intrawebu

        $this ->editaceMesicnich->nactiNastaveni($settings);
        if (!isset($this ->editaceMesicnich ->vcetnePodrizenych)) {
            // "vcetne podrizenych" neni nastaveno primo ve formulari - pouzije se nastaveni z filtru
            $this ->editaceMesicnich ->vcetnePodrizenych = $this ->filterForm ->getFilterValue('rozdeleni', 'vcetnePodrizenych');
        } else {
            // "vcetne podrizenych"  je nastaveno primo ve formulari - ve filtru se uz nezobrazuje
            if ($filterRozdeleni) {
                $filterRozdeleni->setShowVcetnePodrizenych(false);
            }
            //$this ->filterForm ->showVcetnePodrizenychRozdeleni = false;
        }

        if ($loadPost) {
            $this->editaceMesicnich->f_Rozdeleni = $_POST[$this ->editaceMesicnich ->prefixHTML . 'ROZDELENI'];
            if ($this->editaceMesicnich->f_Rozdeleni === '') {
                $this->editaceMesicnich->f_Rozdeleni = null;
            }
            $this->editaceMesicnich->vcetnePodrizenych = $_POST[$this ->editaceMesicnich ->prefixHTML . 'VCETNE_PODRIZENYCH'];

//            RonPortalSession::setRozdeleni($this->editaceMesicnich->f_Rozdeleni);
//            RonPortalSession::setRozdeleniVcetnePodrizenych($this->editaceMesicnich->vcetnePodrizenych);    // TODO: odstranit
        }

        if ($loadPost) {
            return $this ->zapisData();
        }

        return $this ->zobrazEditaciMesicnich();
    }

}


return new EditaceMesicnichController($this ->database);
