<?php
/** pavel.rudol 9.4.2020 */

namespace RonPortal\Sklad;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\RonPortalSettings;
use RonPortal\System\ReportOutputObject;
use RonPortal\TypRozdeleni;
use Ron\TextManager;

class FirmaController extends HttpController
{
    /** @var Database */
    private $database;

    public function __construct($database)
    {
        $this->database = $database;
    }

    /**
     * @param $kodFirmy
     * @return BaseViewObject
     * @throws DBException
     */
    private function detailFirmy($kodFirmy)
    {
        $firma = Firma::nactiFirmu($this->database, $kodFirmy);

        Model::addData('firma', $firma);
        Model::addData('database', $this->database);

        return new BaseViewObject('ronportal/sklad/view_firmadetail.php');
    }

    /**
     * @param $action
     * @return FirmaBrowser
     * @throws Exception
     */
    private function browserFirmy($action)
    {
        $typRozdeleni = $this->vratTypRozdeleniProFiltr();
        $pomTyp = null;
        if ($typRozdeleni)
            $pomTyp = TypRozdeleni::getTypRozdeleni($this->database, $typRozdeleni);

        $indexFiltr = RonPortalSettings::getIntValue('dodavatele_filtrovat_dle_uzivatelska');

        $b = new FirmaBrowser($this->database, $action, $indexFiltr, $pomTyp);
        $b->load();
        return $b;
    }

    /**
     * @return ReportOutputObject
     * @throws Exception
     */
    private function report()
    {
        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, 'Firma', false);

        $result->integerParams['TypRozdeleniFirmy'] = $this->vratTypRozdeleniProFiltr();
        $result->integerParams['Aktivni'] = $_GET['aktivni'];
        $result->stringParams =
            ['RozdeleniFirmy' => $_GET['rozdeleni'],
                'Mesto' => $_GET['mesto'],
                'KlicovaSlova' => $_GET['klicova-slova']];

        return $result;
    }

    /**
     * @return int|mixed
     * @throws DBException
     */
    private function vratTypRozdeleniProFiltr()
    {
        $params = $this->database->createParameters();

        switch (RonPortalSettings::getIntValue('vydanaobjednavka_dodavatel_dle_rozdeleni')) {
            case 1 :
                $typRozdeleniName = 'ObjednavkaVydana_TypRozdeleni1';
                break;
            case 2 :
                $typRozdeleniName = 'ObjednavkaVydana_TypRozdeleni2';
                break;
            case 3 :
                $typRozdeleniName = 'ObjednavkaVydana_TypRozdeleni3';
                break;
        }

        if (isset($typRozdeleniName)) {
            $sql =
                "SELECT Name, Val FROM SYS_VAR_GLOBAL WHERE Name = " . $params->addParameter($typRozdeleniName);

            $cursor = $this->database->getSqlResult($sql, $params);

            if ($row = $cursor->fetch()) {
                return $row[1];
            }
        }
        return 0;
    }

    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|OutputObject|Redirect|ViewObject|DokumentOutputObject|FirmaBrowser|ReportOutputObject|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('sklad.ini');

        HtmlHeaders::getInstance()
            ->addCss('css/sklad.css');

        $firmaAction = isset($_GET['action']) ? $_GET['action'] : null;

        switch ($firmaAction) {
            case 'download' : // download dokumentu
                return DokumentyFirmyBrowser::downloadDokument($this ->database, $_GET['firma'], (int)$_GET['record']);

            case 'report' :
                return $this->report();

            case 'detailfirmy' :
                return $this->detailFirmy($_GET['record']);

            case 'seznamfirem' :
            default :
                return $this->browserFirmy($action);
        }
    }

}

return new FirmaController($this->database);
