<?php
/**
 * UTF8
 * stanislav.elbl, 28.08.2015
 * Rozsirena funkcnost vkladani historie - nedokumentovana funkce (zakazka) - pro klasicke vkladani se nepouziva
 *
 */

namespace RonPortal\Dochazka;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxZamestnanec;
use RonPortal\Log;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;


class RozsirenaHistorieController implements Controller
{
    /** @var  Database */
    var $database;

    public function __construct($database)
    {
        $this ->database = $database;
    }


    /**
     * @param int $datum
     * @param bool $desc
     * @return \Ron\DB\Cursor
     * @throws \Ron\DB\DBException
     */
    function nactiHistoriiProOsobu($datum, $desc)
    {
        $sql =
            "SELECT HistoriePruchodu.DatumCasPruchodu, HistoriePruchodu.kodOperace, HistoriePruchodu.Karta, OperaceDochazky.Popis " .
            " FROM HistoriePruchodu, PovolenaOperaceTerminalu, PrirazeniKarty, OperaceDochazky, PracovniPomer " .
            // odfiltruje jen operace, ktere lze zadat na "obedovem" terminalu
            " WHERE PovolenaOperaceTerminalu.terminal = '" . $this ->database ->addSlashes( RonPortalSettings::getValue(RonPortalSettings::HISTORIE_TERMINAL) ) . "' " .
            " AND HistoriePruchodu.kodOperace = PovolenaOperaceTerminalu.Operace " .
            " AND OperaceDochazky.kod = PovolenaOperaceTerminalu.Operace " .
            // odfiltruje zaznamy pro vybraneho cloveka
            " AND PrirazeniKarty.kodKarty = HistoriePruchodu.Karta " .
            ' AND PracovniPomer.Id = ' . RonPortalSession::vybranaOsoba() .
            " AND PrirazeniKarty.RC = PracovniPomer.RC" .
            " AND PrirazeniKarty.osobniCislo = PracovniPomer.OsobniCislo" .
            " AND PrirazeniKarty.datumNastupu = PracovniPomer.DatumNastupu" .
            " AND PrirazeniKarty.datumPrideleni <= HistoriePruchodu.DatumCasPruchodu" .
            " AND (PrirazeniKarty.datumVraceni >= HistoriePruchodu.DatumCasPruchodu OR PrirazeniKarty.datumVraceni IS NULL)";

        // pokud je zadan datum tak odfiltruje operace pro zadany den
        if ($datum)
            $sql .=
                " AND HistoriePruchodu.DatumCasPruchodu >= " . $this ->database ->timestampToString(DateUtils::startOfTheDay($datum)) .
                " AND HistoriePruchodu.DatumCasPruchodu < " . $this ->database ->timestampToString(DateUtils::startOfTheDay(DateUtils::incDay($datum, 1)));
        else
            $sql .=
                " AND HistoriePruchodu.DatumCasPruchodu >= " . $this ->database ->timestampToString(DateUtils::startOfTheDay(time()));

        // serazeni podle data - pokud je $desc, tak sestupne (prvni radek je nejnovejsi zaznam)
        $sql .= " ORDER BY HistoriePruchodu.DatumCasPruchodu ";
        if ($desc) $sql .= " DESC";

        return $this ->database ->getSqlResult($sql);
    }


    // ----------------------------------------------------------------------------
    // Vraci posledni zaznam v danem dnu
    // $result[0] - cas
    // $result[1] - operace
    // $result[2] - karta
    // $result[3] - popis operace
    /**
     * @param $datum
     * @return array|null
     */
    function posledniZaznamNaDen($datum)
    {
        $cursor = $this ->nactiHistoriiProOsobu($datum, true); // seradit sestupne
        $row = $cursor ->fetch();

        return $row;
    }



    function vlozZaznam()
    {
        // Urci datum a cas operace - pokud se vklada zaznam pro dnesni den, tak
        // se zvoli aktualni datum (aby fungovaly i terminaly), jinak se najde
        // posledni zaznam pro dany den a vlozi se o sekundu vetsi cas
        $datum = DateUtils::dateStringToTimestamp($_POST['datumoperace']);

        $prava = Prava::vratPrava('historie');

        //  VyberOsobyHistorie($connection);
        $ppomer = new PracovniPomer();
        if (!$ppomer->nactiPPPodleId($this ->database, $_POST['osoba'])) {
            throw new \Exception('Nelze najit zadany pracovni pomer'); // nemelo by nastat
        }

        if (!$ppomer->vyberNactenyPP($this ->database, $prava)) {
            throw new NeniPravoException();
        }

        // Zkontroluje, jestli se nevklada po uzavirce
        if (!$prava->priznak)
            if ($datum + 3600 * RonPortalSettings::getIntValue(RonPortalSettings::HISTORIE_POCET_HODIN) < time()) {
                throw new \Exception( \Ron\ronText('Nelze_vkladat_po_uzavirce') );
            }

        // Pokud se vklada pro aktualni den, tak zvoli aktualni cas,
        // jinak zvoli  posledni zadany cas + 1 minuta (sekunda nefunguje)
        if ($datum == DateUtils::startOfTheDay(time())) {
            // Nastavi aktualni cas
            $datum = time();
        } else {
            // Najde posledni zaznam pro zadane datum
            $row = $this ->posledniZaznamNaDen($datum);
            if ($row) $datum = $this ->database ->stringToTimestamp($row[0]) + 60;
        }

        $prirazeniKarty = PrirazeniKarty::vratCisloKarty($this ->database, $datum, $ppomer ->rc);

        if (!$prirazeniKarty) {
            throw new \Exception( \Ron\ronText('Neni_prirazena_karta') );
        }

        $karta = $prirazeniKarty ->kod;

        $sql =
            'INSERT INTO HistoriePruchodu ' .
            ' (Terminal, DatumCasPruchodu, Karta, kodOperace, uspesny, RC, osobniCislo, datumNastupu, vlozenoManualne, Uzivatel) ' .
            ' VALUES ' .
            " ('" . $this ->database ->addSlashes( RonPortalSettings::getValue(RonPortalSettings::HISTORIE_TERMINAL) ) . "'," .
            $this ->database ->timestampToString($datum) . ", '" . $this ->database ->addSlashes($karta) . "', '" . $this ->database ->addSlashes($_POST['operace']) . "', 1, " .
            "'" . $this ->database ->addSlashes($ppomer->rc) . "', '" . $this ->database ->addSlashes($ppomer->osobniCislo) . "', " .
            $this ->database ->timestampToString($ppomer->datumNastupu) . ", 1, '" . $this ->database ->addSlashes(RonPortalUtils::getLogUserName()) . "')";

        Log::addRecordToDBLog($this ->database, 'HistoriePruchodu', 'INSERT', 'Historie pruchodu', $sql);

        $result = $this ->database ->sql($sql);

        if (!$result) {
            throw new DBException($this ->database, $sql, \Ron\ronText('Nelze_vlozit_do_historie') );
        }

        return new Redirect(null, '?menuid=historierozsirena');
    }


    /**
     * @param string $action
     * @return Redirect|ViewObject|string
     * @throws DBException
     * @throws NeniPravoException
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        if (!RonPortalSettings::getIntValue('rozsirenahistoriepruchodu')) {
            throw new \Exception('TATO FUNKCE NENI POVOLENA!');
        }

        Prava::nastavObdobiProPrava(time(), time());

        if (isset($_POST['vkladaniaction'])  &&  $_POST['vkladaniaction'] == 'vlozdohistorie') {
            return $this ->vlozZaznam();
        }

        $prava = Prava::vratPrava('historie');
        if (!($prava->jsouNejakePrava())) {
            RonPortalUtils::neniPravoException();
        }

        // vyber osoby
        $combo = new ComboBoxZamestnanec('osoba'); // jmeno promenne
        $combo ->htmlParams = 'data-ron-autosubmit="1"'; // 'onchange="this.form.submit()"';
        $combo->loadDataPP($this ->database,
            time(), time(),
            $prava, ' ', RonPortalSession::vybranaOsoba(), false); // $emptyrow = true, $selectedvalue, $vcetneexternich = false
        $combo->nastavOsobuDoSession();

        Model::addData('comboOsoba', $combo);

        // seznam existujicich zaznamu
        $historie = $this ->nactiHistoriiProOsobu(0, false);  // vsechny zaznamy od dneska, vzestupne
        Model::addData('historie', $historie);


        $sql =
            'SELECT OperaceDochazky.Kod, OperaceDochazky.PopisProDochazku ' .
            '  FROM OperaceDochazky, PovolenaOperaceTerminalu ' .
            ' WHERE OperaceDochazky.kod = PovolenaOperaceTerminalu.Operace ' .
            " AND PovolenaOperaceTerminalu.terminal = '" . $this ->database ->addSlashes( RonPortalSettings::getValue(RonPortalSettings::HISTORIE_TERMINAL) ) . "' " .
            ' ORDER BY OperaceDochazky.PopisProDochazku';
        $combo = new ComboBox('operace');
        $combo ->emptyRow = ' ';
        $combo ->loadData($this ->database, $sql, isset($_POST['operace']) ? $_POST['operace'] : null);
        Model::addData('comboOperace', $combo);
        Model::addData('database', $this ->database);

        return new BaseViewObject('ronportal/dochazka/view_vkladani_historie.php');
        //return new RozsirenaHistorieViewObject($this ->database);
    }


}

return new RozsirenaHistorieController($this ->database);
