<?php
/**
 * Roman.Szablatura RON Software 27.12.2017
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\Redirect;
use Ron\TextManager;
use RonPortal\Editace\KalendarOsobyForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Class KalendarOsobyController
 * @package Ron\RonPortal\Dochazka
 */
class KalendarOsobyController implements Controller
{
    /** @var  Database $database */
    private $database;

    /** @var  PravaNaOsoby $pravo */
    public $pravo;


    /**
     * KalendarOsobyController constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        $this->database = $database;
        Prava::nastavObdobiProPrava(time(), time());
        $this->pravo = Prava::vratPrava('prirazenikalendare');
    }


    /**
     * @param int $osoba
     * @return BaseViewObject
     * @throws \Exception
     */
    private function detailKalendareOsoby($osoba)
    {
        $idPP = isset($_POST['osoba'])
            ? (int)$_POST['osoba']
            : (isset($_GET['osoba']) ? (int)$_GET['osoba'] : null);

        if (!$idPP) {
            throw new \Exception('Neni zadana osoba!');
        }

        $pracovniPomer = PracovniPomer::nactiPP($this ->database, $idPP);
        if (!Prava::lzeZobrazitPP('prirazenikalendare', $pracovniPomer ->idPP, time()));

        $f = new KalendarOsobyForm($this ->database, $pracovniPomer);
        if ($f ->handleAction()) {
            return new Redirect(null, '?menuid=kalendarosoby&action=detailkalendare&osoba=' . $osoba);
        }
        return $f;
    }




    /**
     * @param string $action
     * @return BaseViewObject|\Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|Redirect|\Ron\MVC\ViewObject|string
     * @throws \Exception
     */
    public function handleAction($action)
    {
        $idpp = isset($_POST['osoba'])
            ? (int)$_POST['osoba']
            : (int)$_GET['osoba'];

        if (!$this->pravo->lzeZobrazitPP($idpp)){
            throw new NeniPravoException();
        }

        TextManager::getInstance() ->loadStrings('dochazka.ini');

        switch ($_GET['action']){
            case 'detailkalendare':
                return $this->detailKalendareOsoby($idpp);
                break;

            default:
                return new Redirect(null, '?menuid=zpracovanadochazka');
        }
    }
}

return new KalendarOsobyController($this->database);
