<?php
/**
 * stanislav.elbl, 18.09.2015
 * Uzivatelske nastaveni - napr. povoleni stahovani kalendaru
 */

namespace RonPortal;

use Ron\DB\Database;
use Ron\HttpUtils;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Controller;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\TextManager;


class NastaveniController implements Controller
{
    /** @var Database */
    var $database;

    /**
     * NastaveniController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }



    function nastaveniKalendaru()
    {
        $nastaveni = NastaveniUzivatele::getInstance();

        $doSave = false;
        if ( !$nastaveni ->getItem(NastaveniUzivatele::ICAL_KEY) ) {
            $nastaveni ->vygenerujKlic();
            $doSave = true;
        }


        if (isset($_POST['ulozitosobninastaveni'])) {
            $nastaveni ->setItem(NastaveniUzivatele::ICAL_SMENY, (isset($_POST['ical_smeny']) && $_POST['ical_smeny']) ? 1 : 0, 1);
            $nastaveni ->setItem(NastaveniUzivatele::ICAL_SCHVALENE_ABSENCE, (isset($_POST['ical_schvaleneabsence']) && $_POST['ical_schvaleneabsence']) ? 1 : 0, 1);

            $nastaveni ->setItem(NastaveniUzivatele::ICAL_VSECHNY_ABSENCE, (isset($_POST['ical_vsechnyabsence']) && $_POST['ical_vsechnyabsence']) ? 1 : 0, 0);
            $nastaveni ->setItem(NastaveniUzivatele::ICAL_SLUZEBNI_CESTY, (isset($_POST['ical_sluzebnicesty']) && $_POST['ical_sluzebnicesty']) ? 1 : 0, 1);

            $doSave = true;
        }

        if ($doSave) {
            try {
                $nastaveni ->save($this ->database, true);
                MessageManager::addInfoMessage( \Ron\ronText('Nastaveni_ulozeno'));

                return new Redirect(null, '?menuid=nastaveni');
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        }

        global $ical_url;
        $baseUrl = $ical_url ?:
            (HttpUtils::isSSL() ? 'https://' : 'http://') .
            $_SERVER['SERVER_NAME'] . ':' . $_SERVER['SERVER_PORT'] .
            $_SERVER['SCRIPT_NAME'];

        $baseUrl .=
            '?menuid=download&login=' . ($nastaveni ->loginName ?: User::getLoginName()) .
            '&key=' . $nastaveni ->getItem(NastaveniUzivatele::ICAL_KEY) .
            '&';

        Model::addData('baseUrl', $baseUrl);

        return new BaseViewObject('view/nastaveniuzivatele.php');
    }


    /**
     * @param string $action
     * @return ViewObject
     * @throws NeniPravoException
     */
    public function handleAction($action)
    {
        if (User::jeZastup()) {
            throw new NeniPravoException();
        }

        TextManager::getInstance() ->loadStrings('nastaveni.ini');

        NastaveniUzivatele::getInstance() ->load($this ->database);

        return $this ->nastaveniKalendaru();
    }

}


return new NastaveniController($this ->database);
