<?php
/**
 * stanislav.elbl, 25.01.2021
 */

namespace RonPortal\Common;

use Exception;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;

class PoznamkyKeDniController extends JsonController
{
    /** @var Database */
    public $database;

    /**
     * PoznamkyKeDniController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function loadData($output)
    {
        $form = new PoznamkyKeDniFormular($this ->database);
        $output ->addData('data', $form ->getJsonData());
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function storeData($output)
    {
        $form = new PoznamkyKeDniFormular($this ->database);
        $form ->utf8 = true;
        $form ->handleAction();

        $output ->addMessages(MessageManager::getInstance());
    }


    /**
     * @throws Exception
     */
    public function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $this ->loadData($output);
                break;

            case 'storedata' :
                $this ->storeData($output);
                break;
        }
    }


    /**
     * @throws NeniPravoException
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('dochazka.ini');
        if (!Prava::jsouNejakePrava('poznamkykedni')) {
            throw new NeniPravoException();
        }

        if (isset($_GET['action'])) {
            return parent::handleAction($action);
        }

        $result = new PoznamkyKeDniBrowser($this ->database, 'poznamkykedni');
        $result ->load();
        return $result;
    }
}

return new PoznamkyKeDniController($this ->database);
