<?php
/**
 * stanislav.elbl, 06.03.2016
 */

namespace RonPortal\Editace;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RozdeleniShiftFormView;
use function Ron\ronText;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\User;


class RozdeleniDVController extends JsonController
{
    /** @var Database */
    public $database;

    /**
     * RozdeleniDVController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * @param JsonOutputObject $output
     * @throws DBException
     * @throws Exception
     */
    private function nactiDataRozdeleni($output)
    {
        $kodRozdeleni = $_GET['rozdeleni'];
        $datumOd = DateUtils::dateStringToTimestamp($_GET['datumod']);
        $datumDo = DateUtils::dateStringToTimestamp($_GET['datumdo']);

        if (RonPortalSettings::getValue('dv_casovaslozka2')) {
            // nacte hodnoty pro casovou slozku 2
            $pp = PracovniPomer::nactiPP($this->database, (int)$_GET['idpp']);
            $items = ZaznamRozdeleniVysledku::nactiZaznamyOsoby($this->database,
                RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI), $pp, [strtolower(RonPortalSettings::getValue('dv_casovaslozka2'))], $datumOd, $datumDo);

            $rozdeleni2 = [];
            foreach ($items as $item) {
                if ($item ->rozdeleni == $kodRozdeleni) {
                    $rozdeleni2[] = [
                        'datum' => StringUtils::jsonDate($item->den),
                        'hodnota' => $item->hodinyStr
                    ];
                }
            }
            $output ->addData('rozdeleni2', $rozdeleni2);
        }

        // nacte schvaleni rozdeleni
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Den, KodTypuRozdeleni, Rozdeleni, Schvaleno' .
            ' FROM SchvaleniRozdeleniDoch' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI)) .
            ' AND Rozdeleni = ' . $params ->addParameter($kodRozdeleni) .
            ' AND Den >= ' . $params ->addParameter($this ->database ->dateToParam($datumOd)) .
            ' AND Den <= ' . $params ->addParameter($this ->database ->dateToParam($datumDo));

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $output ->addData('KodTypuRozdeleni', RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI));
        $output ->addData('Rozdeleni', StringUtils::jsonText($kodRozdeleni));
        $lzeSchvalit = Prava::vratPrava('schval_rozdeleni_dv') ->jePravoNaRozdeleni(RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI), $kodRozdeleni, $datumOd, $datumDo);
        $output ->addData('lzeSchvalit', $lzeSchvalit ? 1 : 0);

        $schvaleni = [];
        while ($row = $cursor ->fetch()) {
            $schvaleni[] = [
                'datum' => StringUtils::jsonDate($this ->database ->stringToTimestamp($row[0])),
                'hodnota'=> (int)$row[3]
            ];
        }
        $cursor ->close();

        $output ->addData('schvaleni', $schvaleni);
    }

    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'dataRozdeleni' :
                $this ->nactiDataRozdeleni($output);
                break;
        }
    }


    /**
     * @return Redirect|ViewObject
     * @throws NeniPravoException
     * @throws Exception
     */
    private function prehledRozdeleniDV()
    {
        if (!isset($_GET['obdobi'])) {
            throw new Exception('Chyba: neni zadano obdobi!');
        }

        $obdobi = Obdobi::vratObdobi($this ->database,
            'ADS',
            DateUtils::urlStringToDate($_GET['obdobi'])
        );

        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI), $_GET['prehledrozdeleni']);

        $prava = Prava::vratPrava('schval_rozdeleni_dv');
        if (!$prava ->jePravoNaRozdeleni($rozdeleni ->typ, $rozdeleni ->kod, $obdobi ->zacatek, $obdobi ->konec)) {
            throw new NeniPravoException();
        }

        $f = new PrehledRozdeleniDVFormular($this ->database, $rozdeleni, $obdobi);

        if (isset($_POST['schvaleniRozdeleniDV'])  &&  (int)$_POST['schvaleniRozdeleniDV']) {
            try {
                $f->zpracujPost();
                return new Redirect(null, '?menuid=rozdelenidv&prehledrozdeleni=' . StringUtils::htmlTextConvert($rozdeleni ->kod) . '&obdobi=' . DateUtils::dateToUrlString($obdobi ->zacatek));
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
                return $f;
            }
        }


        $f->nactiData();
        return $f;
    }

    /**
     * @param null $form
     * @return RozdeleniDVForm|null
     * @throws Exception
     */
    private function zobrazFormular($form = null)
    {
        $prava = Prava::vratPrava('prohlizeni');

        $filterForm = new RonPortalFilterForm($this ->database, 'rozdelenidv', true);

        $filterForm ->addFilterObdobi();
        $filterForm ->addFilterPP(false, $prava);

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $filterForm ->addFilterRozdeleni1(null, $prava);
            $filterForm ->addFilterRozdeleni2(null, $prava);
        }

        $filterForm ->loadData();

        $pp = PracovniPomer::nactiPP($this ->database, $filterForm ->getFilterValue('osoba'));
        $datumOd = $filterForm ->getOmezitObdobiOd();
        //$datumDo = DateUtils::urlStringToDate($_POST['datumDo']);

        if (!$form) {
            $form = new RozdeleniDVForm($this->database, $pp, DateUtils::firstDayOfMonth($datumOd), DateUtils::lastDayOfMonth($datumOd));
            $form->load();
        }

        Model::addViewObject( new TextViewObject('<h1>' . ronText('Rozdeleni_dennich_vysledku') . '</h1>') );
        Model::addViewObject($filterForm);

        if (RonPortalSettings::getPouzivatRozdeleniDV() && RonPortalSettings::getIntValue(RonPortalSettings::DV_POVOLIT_PRIRAZENI_ROZDELENI)) {
            $url = '?menuid=editace&action=rozdeleniosoby&idpp=' . $pp ->idPP;

            $shiftForm = new RozdeleniShiftFormView($form ->typRozdeleni, $url);
            Model::addViewObject($shiftForm);
        }

        return $form;
    }


    /**
     * @return Redirect|RozdeleniDVForm|null
     * @throws Exception
     */
    private function zpracujPost()
    {
        $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $datumOd = DateUtils::urlStringToDate($_POST['obdobi']);
        $datumDo = DateUtils::lastDayOfMonth($datumOd);


        $f = new RozdeleniDVForm($this->database, $pp, $datumOd, $datumDo);
        $f ->load();    // nacte originalni data a zkontroluje prava
        $f ->process(); // zpracuje POST

        if (MessageManager::getInstance() ->isError()) {
            return $this ->zobrazFormular($f);
        }

        return new Redirect(null,
            '?menuid=rozdelenidv&osoba=' . $pp ->idPP .
            '&obdobi=' . DateUtils::dateToUrlString($datumOd) .
            (isset($_POST['novyRadek']) ? '&novyRadek=1' : '')
        );
    }


    /**
     * @param $action
     * @return bool|InternalRedirect|JsonOutputObject|OutputObject|Redirect|ViewObject|HromadneRozdeleniDVFormular|RozdeleniDVForm|string|null
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            throw new NeniPravoException('V nastaveni portalu neni povolena editace!');
        }

        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->addCss('css/dochazka.css');

        if (isset($_GET['action'])) {
            // json requesty maji nastaveno 'action'
            return parent::handleAction($action);
        }

        if (isset($_GET['prehledrozdeleni'])) {
            return $this ->prehledRozdeleniDV();
        }

        $noveZobrazeni = RonPortalSettings::getIntValue('dv_noverozdeleni', 1);

        HtmlHeaders::getInstance()->setTitle(ronText('RON_Portal') . ' - ' . ronText('Title_Rozdeleni_DV'));

        if ($noveZobrazeni) {
            // nove zobrazeni - seznam rozdeleni na radcich a jednotlive dny ve sloupcich
            if (isset($_POST['rozdelenidv_rows_count'])  &&  isset($_POST['osoba'])) {
                return $this ->zpracujPost();
            }

            return $this ->zobrazFormular();
        }

        // stare zob
        $pocatekObdobi = isset($_POST['obdobi']) ?
            DateUtils::urlStringToDate($_POST['obdobi']) :
            (isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : RonPortalSession::obdobi());
        $obdobi = Obdobi::vratObdobi($this ->database, 'ADS', $pocatekObdobi);

        $idpp = $_POST['osoba'] ?? ($_GET['idpp'] ?? RonPortalSession::vybranaOsoba());
        $pp = PracovniPomer::nactiPP($this->database, $idpp);

        $f = new HromadneRozdeleniDVFormular($this->database, $pp, $obdobi);
        $f->nactiData();

        if (isset($_POST['ulozitRozdeleniDV']) && $_POST['ulozitRozdeleniDV']) {
            return $f->zpracujPost();
        }

        return $f;

    }

}

return new RozdeleniDVController($this ->database);
