<?php
/**
 * UTF8
 * stanislav.elbl, 01.11.2015 * roman.szablatura, 19.10.2016
 */

namespace RonPortal\Common;

use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\User;


/**
 * Class UkolyController
 * @package Ron\Personalistika
 */
class UkolyController extends JsonController
{
    /** @var  Database */
    var $database;

    /**
     * UkolyController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }


    /**
     * @return UkolyBrowser
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    public function seznamUkolu()
    {
        Prava::nastavObdobiProPrava(time(), time());

        $b = new UkolyBrowser($this ->database);
        $b ->load();
        return $b;
    }


    public function ukolFormular($kodSerial)
    {
        $ukol = new Ukol();
        $ukol ->load($this ->database, $kodSerial);

        $f = new UkolFormular($this ->database, $ukol);
        if (!$f ->handleAction()) {
            return $f;
        }

        return new Redirect(null,
            'menuid=ukoly' .
            (RonPortalSettings::getIntValue('typ_tlacitka') ? '&record=' . $ukol ->kodSerial : ''));
    }


    /**
     * Nacte obsah pro dashboard "Moje ukoly"
     * @param JsonOutputObject $output
     * @throws \Ron\DB\DBException
     */
    public function mojeUkoly($output)
    {
        $html = '<table class="dashboardTable" data-ron-href="?menuid=ukoly">';
        $html .='<tr>';
        $html .= "<td>" . \Ron\ronText('') . '</td>';
        $html .= "<td>" . \Ron\ronText('typ_ukolu') . '</td>';
        $html .= "<td>" . \Ron\ronText('popis') . '</td>';
        $html .= "<td>" . \Ron\ronText('termin') . '</td>';
        $html .= '</tr>';


        $params = $this ->database ->createParameters();
        $sql =
            'SELECT UkolOsoby.KodSerial, UkolOsoby.RC_Osoby, UkolOsoby.StavUkolu, UkolOsoby.RC_Hodnotitele_Def, UkolOsoby.Hodnoceno,' .             //0..4
            ' UkolOsoby.RC_Schvalovatele1_Def, UkolOsoby.Schvaleno1, UkolOsoby.RC_Schvalovatele2_Def, UkolOsoby.Schvaleno2, KategorieUkolu.Popis,' .//5..9
            ' UkolOsoby.ZkracenyPopis, datumSplneni_Narizene' .                                                                                     //10..11

            ' FROM UkolOsoby LEFT JOIN KategorieUkolu ON (KategorieUkolu.kod = UkolOsoby.kodKategorieUkolu) ' .

            'WHERE StavUkolu <> -1 AND (RC_Osoby = ' . $params->addParameter(User::getRC()) . '
                OR RC_Hodnotitele_Def = ' . $params->addParameter(User::getRC()) . ' 
                OR RC_Schvalovatele1_Def = ' . $params->addParameter(User::getRC()) . ' 
                OR RC_Schvalovatele2_Def = ' . $params->addParameter(User::getRC()) . ')';

        $cursor = $this->database->getSqlResult($sql,$params);

        $prihlasenaOsoba = User::getRC();

        while ($row = $cursor->fetch()) {
            $html .="<tr>";

                //Ukol ke splneni
                if ($row[1] == $prihlasenaOsoba && $row[2] == 0 ){
                    $html .="<td>";
                    $html .="<a href='?menuid=ukoly&record={$row[0]}'>";
                    $html .= \Ron\ronText('mam_splnit') ;
                    $html .="</a>";
                    $html .="</td>";
                    $html .= '<td>' . ($row[9]) . '</td>';
                    $html .= '<td>' . ($row[10]) . '</td>';
                    $html .= '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[11])) . '</td>';
                }

                //Ukol pripraven k hodnoceni
                if ($row[3] == $prihlasenaOsoba && $row[4] == 0 && $row[2] == 1){
                    $html .="<td>";
                    $html .="<a href='?menuid=ukoly&record={$row[0]}'>";
                    $html .= \Ron\ronText('mam_hodnotit') ;
                    $html .="</a>";
                    $html .="</td>";
                    $html .= '<td>' . ($row[9]) . '</td>';
                    $html .= '<td>' . ($row[10]) . '</td>';
                    $html .= '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[11])) . '</td>';
                }

                //Ukol pripraven ke schvaleni 1
                //Ukol pripraven ke schvaleni 2
                if (($row[5] == $prihlasenaOsoba && $row[2] == 1 && $row[4] == 1 && $row[6] == 0) || ($row[7] == $prihlasenaOsoba && $row[2] == 1 && $row[6] == 1 && $row[8] == 0)){
                    $html .="<td>";
                    $html .="<a href='?menuid=ukoly&record={$row[0]}'>";
                    $html .= \Ron\ronText('mam_schvalit') ;
                    $html .="</a>";
                    $html .="</td>";
                    $html .= '<td>' . ($row[9]) . '</td>';
                    $html .= '<td>' . ($row[10]) . '</td>';
                    $html .= '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[11])) . '</td>';
                }

            $html .="</tr>";
        }

        $html .= '</table>';

        $output ->addText('html', $html);
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws \Ron\DB\DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']){
            case 'mojeukoly':
                $this->mojeUkoly($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|\Ron\MVC\InternalRedirect|\Ron\MVC\OutputObject|Redirect|\Ron\MVC\ViewObject|string
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance() ->loadStrings('ukoly.ini');
        HtmlHeaders::getInstance()
            ->setAppTitle('Ukoly');

        Prava::nastavObdobiProPrava(time(), time());

        if (isset($_POST['formaction'])  &&  $_POST['formaction']) {
            return $this ->ukolFormular((int)$_POST['KodSerial']);
        }

        if (isset($_GET['action'])) {
            //json action
            return parent::handleAction($action);
        }

        if (isset($_GET['record'])) {
            return $this->ukolFormular((int)$_GET['record']);
        }

        return $this->seznamUkolu();
    }
}

return new UkolyController($this ->database);
