<?php
/**
 * stanislav.elbl, 21.09.2018
 */

namespace RonPortal\Vyroba;

use Ron\DateUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\TextViewObject;
use Ron\MVC\ViewObject;
use Ron\TextManager;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;


class VyrobaOsobyController extends HttpController
{
    var $database;

    /**
     * VyrobaOsobyController constructor.
     * @param $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * @param VyrobaOsobyForm $form
     * @return ViewObject
     * @throws \Exception
     */
    private function zobrazFormular($form = null)
    {
        $prava = Prava::vratPrava('vyroba');

        $filterForm = new RonPortalFilterForm($this ->database, 'vyrobaosoby', true);
        $filterForm ->addFilterObdobi();
        $filterForm ->addFilterPP(false, $prava);

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $filterForm ->addFilterRozdeleni1(null, $prava);
            $filterForm ->addFilterRozdeleni2(null, $prava);
        }

        $filterForm ->loadData();

        Model::addViewObject( new TextViewObject('<h1>' . \Ron\ronText('Editace_vyroby_osoby') . '</h1>') );
        Model::addViewObject($filterForm);

        if (!$filterForm ->getFilterValue('osoba')) {
            return new TextViewObject('');// zobrazuje jen filtrovaci formular a chybovou hlasku "nemate PP ..."
        }

        $pp = PracovniPomer::nactiPP($this ->database, $filterForm ->getFilterValue('osoba'));
        $datumOd = $filterForm ->getOmezitObdobiOd(); // isset($_GET['obdobi']) ? DateUtils::urlStringToDate($_GET['obdobi']) : RonPortalSession::obdobi();
        //$datumDo = DateUtils::urlStringToDate($_POST['datumDo']);

        if (!$form) {
            $form = new VyrobaOsobyForm($this->database, $pp, DateUtils::firstDayOfMonth($datumOd), DateUtils::lastDayOfMonth($datumOd));
            $form->load();
        }


        return $form;
    }


    private function zpracujPost()
    {
        $pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $datumOd = DateUtils::urlStringToDate($_POST['obdobi']);
        //$datumDo = DateUtils::urlStringToDate($_POST['datumDo']);
        $datumDo = DateUtils::lastDayOfMonth($datumOd);

//        if (DateUtils::daysBetween($datumOd, $datumDo) > 31) {
//            throw new \Exception('Spatne zadane obdobi!');
//        }

        Prava::nastavObdobiProPrava($datumOd, $datumDo);
        if (!Prava::lzeZobrazitPP('editacevyroby', $pp ->idPP)) {
            throw new NeniPravoException();
        }

        $f = new VyrobaOsobyForm($this ->database, $pp, $datumOd, $datumDo);
        $f ->load();    // nacte originalni data

        try {
            $f->process(); // zpracuje POST
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        if (MessageManager::getInstance() ->isError()) {
            return $this ->zobrazFormular($f);
        }

        return new Redirect(null,
            '?menuid=vyrobaosoby&osoba=' . $pp ->idPP .
            '&obdobi=' . DateUtils::dateToUrlString($datumOd) .
            (isset($_POST['novyRadek']) ? '&novyRadek=1' : '')
        );
    }


    public function handleAction($action)
    {
        // TODO: zkontrolovat prava vyroba a editacevyroby ! ! !

        TextManager::getInstance()
            ->loadStrings('vyroba.ini')
            ->loadStrings('dochazka.ini');

        HtmlHeaders::getInstance()
            ->addCss('css/dochazka.css')
            ->addScript('script/vyroba.js');

        if (isset($_POST['vyroba_rows_count'])  &&  isset($_POST['osoba'])  &&  isset($_POST['osoba'])) {
            return $this ->zpracujPost();
        }

        return $this ->zobrazFormular();
    }
}


return new VyrobaOsobyController($this ->database);
