<?php

/**
 * UTF-8
 * Roman.Szablatura 19.5.2016
 */
namespace Ron\RonPortal\Zamestnanci;

use Ron\Controls\ComboBox;
use Ron\DB\DBRecord;
use Ron\FormField;
use Ron\RonPortal\RonPortalForm;
use Ron\StringUtils;

require_once ('adsapp/ronportalform.php');

class VyrazeneMediumField extends FormField
{
    private $url;

    function __construct($prava, $edit, $url)
    {
        parent::__construct();
        $this->editable = $edit;
        $this->url = $url;
        $this->caption = '<STRONG>' . \Ron\ronText('medium') . '</STRONG>';
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        if($canEdit){
            $sql =
                "SELECT  Karta.Kod, Karta.OznaceniMedia + ' - " . \Ron\ronText('Kod') . ":'" .
                'FROM Karta  ' .
                'WHERE DatumVyrazeni IS NOT NULL';

            $combo = new ComboBox('medium');
            $combo->emptyRow = \Ron\ronText('vyberte_medium');
            $combo->className .= 'app_combo adsfilterforminput_popis';
            $combo->alwaysShowSelected = isset($value) && $value != '';

            $jeVybraneMedium = isset($_GET['medium']) ? $_GET['medium'] : 0;

            $combo->loadData($database,$sql,$jeVybraneMedium ? $jeVybraneMedium : $rowData[0]);
            $combo->showCode = 1;
            $combo->htmlParams = 'data-ron-select2="medium" onChange="window.location.href = &quot;' . $this->url . '&amp;medium=&quot;+this.value;"';
            $combo->render();
        }
    }
}

class UpozorneniNaObnovuVyrazenehoMediaField extends FormField
{
    function __construct($options)
    {
        parent::__construct($options);
        $this->caption = \Ron\ronText('upozorneni');
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);

        if($rowData[5] == 0 || $rowData[5] == 2){
            echo '<strong>' . \Ron\ronText('obnova_poskozeneho_vadneho') . '</strong>';
        }
    }
}

class ObnoveniMedia extends RonPortalForm
{
    var $kodKarty;

    function __construct( $database, $kodKarty)
    {
        parent::__construct($database);

        $this->kodKarty = $kodKarty;

        $this->title = \Ron\ronText('obnovit_medium');
        $this->htmlAction = '?menuid=zamestnanec&action=obnovenimedia';
        $this->stornoAction = '?menuid=' . $_SESSION['Ron.Zamestnanci.Browser'];
        $this->dbLogSrcDescription = StringUtils::fromUTF8('Vyřazení ID media');

        $this->sqlTableName = 'Karta';
        $this->sqlSelectPart = 'Kod, OznaceniMedia, DatumPorizeni, DatumVyrazeni, ' .    //0 - 3
                                'Blokovana, ZpusobVyrazeni, ZalohaNaKartu, Poznamka';  //4 - 7

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SECTION, FORM_FIELD_CAPTION_TEXT => 'vyrazena_blokovana_media']);

        $this->addField(new VyrazeneMediumField(null,true,$this->htmlAction . '&formaction=edit'));

        $this->addField(new UpozorneniNaObnovuVyrazenehoMediaField($database));

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_SQL_NAME => 'Kod', FORM_FIELD_SQL_INDEX => 0, FORM_FIELD_NAME => 'Kod', FORM_FIELD_EDIT => false,
            FORM_FIELD_CAPTION_TEXT => 'Kod']);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_SQL_NAME => 'OznaceniMedia', FORM_FIELD_SQL_INDEX => 1, FORM_FIELD_NAME => 'OznaceniMedia', FORM_FIELD_EDIT => false,
            FORM_FIELD_CAPTION_TEXT => 'identifikacni_medium']);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_NAME => 'DatumPorizeni',
            FORM_FIELD_SQL_INDEX => 2, FORM_FIELD_SQL_NAME => 'DatumPorizeni', FORM_FIELD_CAPTION_TEXT => 'datum_porizeni', FORM_FIELD_EDIT => 0]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_NAME => 'DatumVyrazeni',
            FORM_FIELD_SQL_INDEX => 3, FORM_FIELD_SQL_NAME => 'DatumVyrazeni', FORM_FIELD_CAPTION_TEXT => 'datum_vyrazeni', FORM_FIELD_EDIT => 1, FORM_FIELD_REQUIRED => $_GET['action'] == 'obnovenimedia' ? 0 : 1]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_NAME => 'Blokovana', FORM_FIELD_SQL_NAME => 'Blokovana', FORM_FIELD_SQL_INDEX => 4,
            FORM_FIELD_EDIT => 0, FORM_FIELD_CAPTION_TEXT => 'zablokovane_medium', FORM_FIELD_SQL => 'SELECT Kod, Popis FROM S_AnoNe', FORM_FIELD_EDIT => 1]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_NAME => 'ZpusobVyrazeni', FORM_FIELD_SQL_NAME => 'ZpusobVyrazeni', FORM_FIELD_SQL_INDEX => 5,
            FORM_FIELD_EDIT => 0, FORM_FIELD_CAPTION_TEXT => 'zpusob_vyrazeni', FORM_FIELD_SQL => 'SELECT Kod, Popis FROM S_ZpusobVyrazeniKarty', FORM_FIELD_EDIT => 1]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_SQL_NAME => 'ZalohaNaKartu', FORM_FIELD_NAME => 'ZalohaNaKartu', FORM_FIELD_EDIT => false,
            FORM_FIELD_SQL_INDEX => 6, FORM_FIELD_CAPTION_TEXT => 'zalohaNaKartu']);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_SQL_INDEX => 7, FORM_FIELD_SQL_NAME => 'Poznamka', FORM_FIELD_CAPTION_TEXT => 'Poznamka',
            FORM_FIELD_EDIT => 1, FORM_FIELD_NAME => 'Poznamka']);
    }

    function getWhereCondition()
    {
        if (isset($_POST['medium'])) $medium = $_POST['medium'];
        else if (isset($_GET['medium'])) $medium = $_GET['medium'];
        else $medium = '';

        if($medium) {
            $result = "Kod = '" . $this->database->addSlashes($medium) . "'";
        } else {
            $result = "DatumVyrazeni IS NOT NULL";
        }
        return $result;
    }


    /**
     * @param DBRecord $rec
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);
        $rec->where = $this->getWhereCondition();
    }

    /**
     * @param DBRecord $rec
     * @return bool
     * @throws \Exception
     */
    function beforeUpdateRecord($rec)
    {
        if($rec['datumVyrazeni'] != ''){
            throw new \Exception(\Ron\ronText('obnoveni_media_datem'));
        }

        if($rec['datumVyrazeni'] == ''){
            if($rec['ZpusobVyrazeni'] != ''){
                $rec['ZpusobVyrazeni'] = '';
            }
        }
        return parent::beforeUpdateRecord($rec);
    }
}
