<?php

/** UTF-8
 * Roman.Szablatura 19.5.2016
 */
namespace Ron\RonPortal\Zamestnanci;

use Ron\DB\Database;
use Ron\DB\DBRecord;
use Ron\DB\DBUtils;
use Ron\FormField;
use Ron\RonPortal\RonPortalDBRecord;
use Ron\RonPortal\RonPortalForm;
use Ron\StringUtils;

require_once ('adsapp/ronportalform.php');

class datumPrirazeniForm extends FormField
{

    function __construct($options)
    {
        parent::__construct($options);

        $this->caption = \Ron\ronText('datum_prirazeni');
    }

    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $sql =
            'SELECT datumPrideleni ' .
            'FROM Karta LEFT JOIN PrirazeniKarty ON (Karta.Kod = PrirazeniKarty.kodKarty)' .
            "WHERE kodKarty = '" . $database->addSlashes($_GET['karta']) .  "' AND datumVraceni IS NULL";

        $cursor = $database->getSqlResult($sql);
        $row = $cursor->fetch();


        foreach($row as $r){
            echo StringUtils::format('date', $database->stringToTimestamp($r));
        }
    }
}

class VyrazeniMedia extends RonPortalForm
{
    var $kodKarty;

    function __construct( $database, $kodKarty)
    {
        parent::__construct($database);

        $this->kodKarty = $kodKarty;

        $this->title = \Ron\ronText('vyrazeni_media');
        $this->htmlAction = '?menuid=zamestnanec&action=vyrazenimedia';
        $this->stornoAction = '?menuid=' . $_SESSION['Ron.Zamestnanci.Browser'];
        $this->dbLogSrcDescription = StringUtils::fromUTF8('Vyřazení ID media');

        $this->sqlTableName = 'Karta';
        $this->sqlSelectPart = 'kod, kod2, id, typ, datumPorizeni, datumVyrazeni, blokovana, zpusobVyrazeni, zalohaNaKartu, Poznamka';  //0 - 9

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SECTION, FORM_FIELD_CAPTION_TEXT => 'medium']);

        $this->addField(new datumPrirazeniForm(null));

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_SQL_NAME =>'kod', FORM_FIELD_NAME => 'kod', FORM_FIELD_EDIT => false,
            FORM_FIELD_SQL_INDEX => 0,  FORM_FIELD_CAPTION_TEXT => 'Kod']);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_EDIT, FORM_FIELD_SQL_NAME =>'zalohaNaKartu', FORM_FIELD_NAME => 'zalohaNaKartu', FORM_FIELD_EDIT => false,
            FORM_FIELD_SQL_INDEX => 8,  FORM_FIELD_CAPTION_TEXT => 'zalohaNaKartu']);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_NAME => 'blokovana', FORM_FIELD_SQL_NAME => 'blokovana', FORM_FIELD_SQL_INDEX => 6,
            FORM_FIELD_EDIT => 0, FORM_FIELD_CAPTION_TEXT =>'zablokovane_medium', FORM_FIELD_SQL => 'SELECT Kod, Popis FROM S_AnoNe', FORM_FIELD_EDIT => 1]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_DATE, FORM_FIELD_NAME => 'datumVyrazeni',
            FORM_FIELD_SQL_INDEX => 5, FORM_FIELD_SQL_NAME =>'datumVyrazeni', FORM_FIELD_CAPTION_TEXT => 'datum_vyrazeni', FORM_FIELD_EDIT => 1]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_SELECT, FORM_FIELD_SQL_INDEX => 7, FORM_FIELD_SQL_NAME => 'zpusobVyrazeni', FORM_FIELD_NAME => 'zpusobVyrazeni',
            FORM_FIELD_SQL => 'SELECT Kod, Popis FROM S_ZpusobVyrazeniKarty', FORM_FIELD_CAPTION_TEXT => 'zpusob_vyrazeni', FORM_FIELD_EDIT => 1 ]);

        $this->addField([FORM_FIELD_TYPE => FORM_FIELD_TYPE_MEMO, FORM_FIELD_SQL_INDEX => 9, FORM_FIELD_NAME => 'Poznamka', FORM_FIELD_SQL_NAME => 'Poznamka', FORM_FIELD_CAPTION_TEXT => 'Poznamka',
            FORM_FIELD_EDIT => 1]);

    }

    function getWhereCondition()
    {
//        $result = "kod = '" . $this->database->addSlashes($this->kodKarty) . "'";
        $result = "kod = '" . ($this->database->addSlashes($this->kodKarty) ? $this->database->addSlashes($this->kodKarty) : $_POST['kod']) . "'";

        return $result;
    }


    function createDBRecord()
    {
        $result = parent::createDBRecord();
//        $result->addField('Poznamka', 'memo',false, 0, $rec['Poznamka']);
//        $this->addField('datumVyrazeni', 'date', '' );
        return $result;
    }


    /**
     * @param DBRecord $rec
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);

        $rec->where = $this->getWhereCondition();
    }

    /**
     *
     */
    function renderCommonFields()
    {
        parent::renderCommonFields();

        $datumPrirazeni = DBUtils::getSqlValue($this->database, "SELECT datumPrideleni FROM PrirazeniKarty WHERE kodKarty='" . $this->kodKarty . "'" );

        echo '<input type="hidden" value="' . $this->database->addSlashes($this->kodKarty) . '" name="kod" />';
        echo '<input type="hidden" value="' . $datumPrirazeni . '" name="datumPrirazeni" />';
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function afterUpdateRecord($rec)
    {
        //TODO: ukoncit prirazeni vybrane karty ke dni vyrazeni

        if (!self::ukonciPrirazeniKarty($this->database, $rec['kod'], $rec['datumVyrazeni'])) {
            return false;
        }
        return parent::afterUpdateRecord($rec);
    }

    /**
     * @param DBRecord $rec
     * @return bool
     * @throws \Exception
     */
    function beforeUpdateRecord($rec)
    {
        if($rec['datumVyrazeni'] == '' || ($rec['datumVyrazeni'] != '' && $rec['datumVyrazeni'] < $rec['datumPrirazeni'] )){
            throw new \Exception(\Ron\ronText('datum_vyrazeni_media'));
        }

        if($rec['zpusobVyrazeni'] == ''){
            throw new \Exception(\Ron\ronText('pozadovan_zpusob_vyrazeni'));
        }
        return parent::beforeUpdateRecord($rec);
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function beforeInsertRecord($rec)
    {
        return parent::beforeInsertRecord($rec); // TODO: Change the autogenerated stub
    }


    /**
     * @param Database $database
     * @param string $idKarty
     * @param int $datumUkonceni
     * @return bool|int
     */
    public static function ukonciPrirazeniKarty($database, $idKarty, $datumUkonceni )
    {
        $recPrirazeni = new RonPortalDBRecord($database, 'PrirazeniKarty');
        $recPrirazeni->logDescription = StringUtils::fromUTF8('Vyřazení karty');

        $recPrirazeni->addField('datumVraceni', 'timestamp', false, 0, $datumUkonceni);

        $recPrirazeni->where =
            " kodKarty = '" . $database->addSlashes($idKarty) . "'" .
            " AND (datumVraceni > " . $database->timestampToString($datumUkonceni) . " OR datumVraceni IS NULL)";

        return $recPrirazeni->loadAndUpdateRecord();
    }

}
