<?php

/**
 * UTF-8
 * Pavel Rudol, RON Software
 *  4.9.2015
 */

namespace Ron\RonPortal\Zamestnanci;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MVC\DataOutputObject;
use Ron\MVC\JsonController;
use Ron\MVC\Model;
use Ron\MVC\TextViewObject;
use Ron\MVC\ViewObject;
use Ron\RonPortal\PracovniPomer as PPomer;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\Redirect;
use Ron\MVC\JsonOutputObject;
use RonPortal\Controller\PritomnostController;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\NastaveniUzivatele;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use RonPortal\System\FotoOutputObject;
use RonPortal\Zamestnanci\SeznamOsobBrowser;
use RonPortal\Zamestnanci\SeznamPPBrowser;

require_once('pracovnipomery/zamestnanec.php');

class ZamestnanecControler extends PritomnostController
{
    /** @var Zamestnanec $zamestnanec */
    private $zamestnanec;
    /** @var PracovniPomerOsoby $pracovniPomer */
    private $pracovniPomer;

    private $zalozky;

    /** @var PravaNaOsoby[] $seznamPrav */
    private $seznamPrav;

    public function __construct($database)
    {
        parent::__construct($database, Prava::vratPrava('zamest_prohl'));

        HtmlHeaders::getInstance()->addScript('pracovnipomery/zamestnanciscripts.js');
        HtmlHeaders::getInstance()->addCss('pracovnipomery/zamestnanci.css');

        TextManager::getInstance()->loadStrings('zamestnanci.ini');

        $this->seznamPrav = [
            'zobrazitZamestnance' => Prava::vratPrava('zamest_prohl'),
            'vlozitZamestnance' => Prava::vratPrava('zamest_prohl'), //editace pres priznak ?
            'editovatRozdeleni' => Prava::vratPrava('zamest_prohl'),
            'editovatKalendar' => Prava::vratPrava('zamest_prohl'),
            'editovatKartu' => Prava::vratPrava('zamest_prohl'),
        ];

        $this->zalozky = [
            'hlavni' => ['text' => \Ron\ronText('HlavniUdaje'), 'section' => 'hlavni', 'view' => FALSE, 'edit' => FALSE],
            'ostatni' => ['text' => \Ron\ronText('OstatniUdaje'), 'section' => 'ostatni', 'view' => FALSE, 'edit' => FALSE],
            'ppomer' => ['text' => \Ron\ronText('PracovniPomer'), 'section' => 'ppomer', 'view' => FALSE, 'edit' => FALSE],
            'ostatnipp' => ['text' => \Ron\ronText('OstatniUdajePP'), 'section' => 'ostatnipp', 'view' => FALSE, 'edit' => FALSE],
            'rozdeleni' => ['text' => \Ron\ronText('Rozdeleni'), 'section' => 'rozdeleni', 'view' => FALSE, 'edit' => FALSE],

        ];

    }

    protected function nastavModel($section = '', $zamestnanecaction = '')
    {
        Model::addData('zamestnanec', $this->zamestnanec);
        Model::addData('vybranypp', $this->pracovniPomer);
        Model::addData('detailosobylink', 'menuid=zamestnanec&idpp=');

        Model::addData('section', $section);
        Model::addData('osobaformaction', $zamestnanecaction);
        Model::addData('zalozky', $this->zalozky);
    }


    /**
     * @param boolean $novyZamestnanec je vkladan novy zamestnanec
     * nacte zamestnance z databaze/zalozi noveho
     * @throws Exception
     */
    protected function loadZamestnanec($novyZamestnanec = FALSE)
    {
        $idPP = $_POST['idpp'] ?? ($_GET['idpp'] ?? ($_GET['record'] ?? 0));
        $rc = $_POST['rc'] ?? ($_GET['rc'] ?? '');

        if ($idPP or ($rc != '')) {
            $this->zamestnanec = new Zamestnanec($this->database, $idPP, $rc);

        } elseif ($novyZamestnanec) {
            $this->zamestnanec = new Zamestnanec($this->database);
        }

        if ($idPP) {
            $this->pracovniPomer = PracovniPomerOsoby::vratPP($this->database, $idPP)[$idPP];

            $this->pridejComboboxyPP($this->pracovniPomer);
        }
    }

    protected function nactiDataProView($novyPP)
    {
        $section = isset($_GET['session@section']) ? $_GET['session@section'] : '';
        $zamestnanecaction = isset($_GET['action']) ? $_GET['action'] : 'edit';

        $this->loadZamestnanec();
        $this->nastavPrava();
        if ($novyPP) {
            $this->pracovniPomer = new PracovniPomerOsoby($this->database);
        } elseif (($zamestnanecaction != 'insert')) {
            if ($this->zalozky['rozdeleni']['view'] || $this->zalozky['rozdeleni']['edit']) {
                $this->pracovniPomer->nactiRozdeleniPP($this->database);
            }
        }

        $this->nastavModel($section, $zamestnanecaction);
    }

    /**
     * @param PracovniPomerOsoby $pracovniPomer
     */
    protected function pridejComboboxyPP($pracovniPomer = null)
    {
        $comboTypOsoby = new ComboBox('TypOsoby');
        $comboTypOsoby->loadData($this->database,
            'SELECT Kod, Popis FROM S_TypOsoby ORDER BY Popis', isset($pracovniPomer) ? $pracovniPomer->zamestnanec : '1');
        Model::setData('comboTypOsoby', $comboTypOsoby);

        $comboTypPP = new ComboBox('TypPracovnihoPomeru');
        $comboTypPP->loadData($this->database,
            'SELECT Kod, Popis FROM TypPracovnihoPomeru ORDER BY Popis', isset($pracovniPomer) ? $pracovniPomer->typPP : '');
        Model::setData('comboTypPP', $comboTypPP);
    }

    protected function pridejComboboxyRozdeleni($typRozdeleni = 1, $rozdeleni = '')
    {
        $comboTypRozdeleni = new ComboBox('TypRozdeleni');
        $comboTypRozdeleni->loadData($this->database,
            'SELECT Kod, Popis FROM TypRozdeleni WHERE Aktivni = 1 ORDER BY Kod', $typRozdeleni);
        $comboTypRozdeleni->htmlParams = 'id="detailosoby_typrozdeleni"';
        Model::setData('comboTypRozdeleni', $comboTypRozdeleni);

        $comboRozdeleni = new ComboBox('Rozdeleni');
        $comboRozdeleni->loadData($this->database,
            'SELECT Kod, Popis FROM Rozdeleni ' .
            'WHERE DatumVytvoreni <= ' . $this->database->timestampToString(time()) .
            '  AND (DatumZruseni >= ' . $this->database->timestampToString(time()) . ' OR DatumZruseni IS NULL)' .
            '  AND KodTypuRozdeleni = ' . ($rozdeleni != '' ? $typRozdeleni : '0') .
            ' ORDER BY Popis', $rozdeleni);
        $comboRozdeleni->htmlParams = 'id="detailosoby_rozdeleni_kod"';
        Model::setData('comboRozdeleni', $comboRozdeleni);
    }

    /**
     * Vrati pole se seznamem rozdeleni: kod, popis zvoleneho typu,
     * rozdeleni je filtrovano na platnost podle zadaneho obdobi, nebo aktualniho data
     * a pokud je zadan typ prava, tak je seznam odfiltrovan i podle prava prihlaseneho uzivatele.
     * @param int $typRozdeleni typ rozdeleni
     * @param int $datumOd zacatek obdobi ve kterem ma byt rozdeleni platne
     * @param int $datumDo konec obdobi ve kterem ma byt rozdeleni platne
     * @param string $kodPrava kod prava pro omezeni rozdeleni
     * @return array pole se seznamem rozdeleni kod, popis
     */
    public function vratSeznamRozdeleni($typRozdeleni, $datumOd = 0, $datumDo = 0, $kodPrava = '')
    {
        $result = [];

        $prava = ($kodPrava != '')
            ? Prava::vratPrava($kodPrava)
            : null;

        if ($datumOd == 0) {
            $datumOd = time();
        }

        if ($datumDo == 0) {
            $datumDo = time();
        }

        $sql = 'SELECT Kod, Popis FROM Rozdeleni ' .
            'WHERE KodTypuRozdeleni = ' . (int)$typRozdeleni .
            '  AND DatumVytvoreni <= ' . $this->database->timestampToString($datumDo) .
            '  AND (DatumZruseni >= ' . $this->database->timestampToString($datumOd) . ' OR DatumZruseni IS NULL) ';

        if ($prava) {
            $sql .= $prava->sqlfiltrRozdeleni($datumOd, $datumDo, 1);
        }

        $sql .= ' ORDER BY Kod';

        $cursor = $this->database->select($sql);

        while ($row = $cursor->fetch()) {
            $result[] = ['kod' => StringUtils::jsonText($row[0]),
                'popis' => StringUtils::jsonText($row[1])];
        }

        return $result;
    }

    protected function novyZamestnanec()
    {
        $this->loadZamestnanec(TRUE);
        /*if ($this->zamestnanec->nactiZamestnancePost()) {
            try {
                $this->zamestnanec->vlozOsobu();
                $idVlozenehoPP = $this->zamestnanec->seznamPP[0]->vlozPP();
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e->getMessage(), ($e->getCode() == 1 ? 'RC' : NULL));
                $this->nastavModel();
                $this->pridejComboboxyPP($this->zamestnanec->seznamPP[0]);
                return new BaseViewObject('pracovnipomery/view_zamestnanecnovy.php');
            }

            return $idVlozenehoPP;
        } else {
            $this->nastavModel();
            $this->pridejComboboxyPP($this->zamestnanec->seznamPP[0]);
            return new BaseViewObject('pracovnipomery/view_zamestnanecnovy.php');
        }*/
    }

    protected function novyPP()
    {
        $this->loadZamestnanec();
        $this->pracovniPomer = new PracovniPomerOsoby($this->database);

        if ($this->pracovniPomer->nactiPPzPostu($this->database, $this->zamestnanec->rc)) {
            try {
                $idPP = $this->pracovniPomer->vlozPP(StringUtils::fromUTF8('Vložení prac. poměru'));
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e->getMessage());
                $this->nastavModel();
                $this->pridejComboboxyPP($this->pracovniPomer);
                return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
            }

            return new Redirect('?menuid=zamestnanec&idpp=' . $idPP);
        } else {
            $this->nastavPrava();
            $this->nastavModel('ppomer');
            $this->pridejComboboxyPP($this->pracovniPomer);
            return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
        }
    }

    protected function noveRozdeleniOsoby()
    {
        $this->loadZamestnanec();
        $rozdeleniOsoby = new RozdeleniOsoby($this->database);

        if ($rozdeleniOsoby->nactiRozdeleniZPostu(false)) {
            try {
                $rozdeleniOsoby->zapisRozdeleni();
            } catch (Exception $ex) {
                MessageManager::addErrorMessage($ex->getMessage());
                $this->nastavModel();
                $this->pridejComboboxyPP($this->pracovniPomer);
                $this->pridejComboboxyRozdeleni();
                return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
            }
            return new Redirect('?menuid=zamestnanec&idpp=' . $this->zamestnanec->idPP);
        } else {
            $this->nastavPrava();
            $this->nastavModel('rozdeleni');
            $this->pridejComboboxyPP($this->pracovniPomer);
            $this->pridejComboboxyRozdeleni();
            return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
        }
    }

    protected function aktualizovatOsobu($section, $jenDetaily = 0)
    {
        if ($this->zamestnanec->nactiOsobuPost($jenDetaily)) {
            try {
//                $this->zamestnanec->aktualizujOsobu($jenDetaily);
            } catch (Exception $exc) {
                MessageManager::addErrorMessage($exc->getMessage());
                $this->nastavModel($section);
                return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
            }

            return TRUE;
        } else {
            $this->nastavModel($section);
            return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
        }
    }

    protected function aktualizovatPP($section)
    {
        //k zamestnanci se pridava novy pracovni pomer, objekt PP jeste neexistuje
        if (!is_object($this->pracovniPomer)) {
            $idPP = isset($_GET['idpp']) ? $_GET['idpp'] : 0;
            if ($idPP) {
                //TODO: zkontrolovat
//                $this->zamestnanec->nactiZamestnanceID($idPP);
            }
            $this->pracovniPomer = new PracovniPomerOsoby($this->database);
        }
        if ($this->pracovniPomer->nactiPPzPostu($this->database)) {
            try {
                $this->pracovniPomer->aktualizujPP($this->database);
            } catch (Exception $exc) {
                MessageManager::addErrorMessage($exc->getMessage());
                $this->nastavModel($section);
                $this->pridejComboboxyPP($this->pracovniPomer);
                return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
            }

            return TRUE;
        } else {
            $this->nastavModel($section);
            $this->pridejComboboxyPP($this->pracovniPomer);
            return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
        }
    }

    protected function aktualizovatDetailyPP()
    {
        if ($this->pracovniPomer->nactiDetailyPPzPostu()) {
            try {
                $this->pracovniPomer->aktualizujPP($this->database, TRUE);
            } catch (Exception $exc) {
                MessageManager::addErrorMessage($exc->getMessage());
                return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
            }

            return TRUE;
        }
        return false;
    }

    protected function smazRozdeleni()
    {
        $this->loadZamestnanec();
        $this->nastavPrava();

        if (!$this->zalozky['rozdeleni']['edit']) {
            throw new NeniPravoException();
        }

        $typRozdeleni = isset($_GET['typrozdeleni']) ? $_GET['typrozdeleni'] : '';
        $kodRozdeleni = isset($_GET['rozdeleni']) ? $_GET['rozdeleni'] : '';
        $platnostod = isset($_GET['platnostod']) ? $_GET['platnostod'] : 0;

        $this->pracovniPomer->seznamRozdeleni[$typRozdeleni];

        $rozdeleni = new RozdeleniOsoby($this->database);

        $rozdeleni->nactiPracovniPomer($this->pracovniPomer);

        $rozdeleni->typRozdeleni = (int)$typRozdeleni;
        $rozdeleni->kodRozdeleni = $kodRozdeleni;
        $rozdeleni->platnostOd = $platnostod;

        try {
            $rozdeleni->zrusRozdeleni();
        } catch (Exception $exc) {
            MessageManager::addErrorMessage($exc->getMessage());
            return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
        }

        return new Redirect(NULL, '?menuid=zamestnanec&idpp=' . $this->zamestnanec->idPP);
    }


    /**
     * nastavi do session pro kazdou sekci/zalozku jestli ma byt skryta, nebo zobrazena
     */
    protected function nactiNastaveni()
    {
        foreach ($this->zalozky as $zalozka) {
            $_SESSION['detailosoby_' . $zalozka['section']] = NastaveniUzivatele::getUserSettingsItem($this->database, 'detailosoby_' . $zalozka['section'], 1);
        }
    }

    /**
     *
     * @return mixed false|JSonOutputObject
     */
    protected function ulozNastaveni()
    {
        foreach ($this->zalozky as $zalozka) {
            if (isset($_GET['detailosoby_' . $zalozka['section']])) {
                NastaveniUzivatele::setUserSettingsItem($this->database, 'detailosoby_' . $zalozka['section'], $_GET['detailosoby_' . $zalozka['section']], 1);
                return new JsonOutputObject(['text' => ($_GET['detailosoby_' . $zalozka['section']] ? \Ron\ronText('Skryt') : \Ron\ronText('Zobrazit')),
                    'result' => JsonOutputObject::RESULT_OK]);
            }
        }
    }

    protected function overPravo($action)
    {
        Prava::nastavObdobiProPrava(time(), time());

        if (!($this->seznamPrav[$action]->jsouNejakePrava())) {
            throw new NeniPravoException();
        }
    }

    protected function nastavPrava()
    {
        Prava::nastavObdobiProPrava(time(), time());

        $this->zalozky['hlavni']['view'] = ($this->seznamPrav['zobrazitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP) or
            $this->seznamPrav['zobrazitZamestnance']->lzeZobrazitRC($this->zamestnanec->rc));
        $this->zalozky['hlavni']['edit'] = ($this->seznamPrav['vlozitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP) or
            $this->seznamPrav['vlozitZamestnance']->lzeZobrazitRC($this->zamestnanec->rc));

        $this->zalozky['ostatni']['view'] = ($this->seznamPrav['zobrazitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP) or
            $this->seznamPrav['zobrazitZamestnance']->lzeZobrazitRC($this->zamestnanec->rc));
        $this->zalozky['ostatni']['edit'] = ($this->seznamPrav['vlozitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP) or
            $this->seznamPrav['vlozitZamestnance']->lzeZobrazitRC($this->zamestnanec->rc));

        $this->zalozky['ppomer']['view'] = $this->seznamPrav['zobrazitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP);
        $this->zalozky['ppomer']['edit'] = $this->seznamPrav['vlozitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP);

        $this->zalozky['ostatnipp']['view'] = $this->seznamPrav['zobrazitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP);
        $this->zalozky['ostatnipp']['edit'] = $this->seznamPrav['vlozitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP);

        $this->zalozky['rozdeleni']['view'] = $this->seznamPrav['zobrazitZamestnance']->lzeZobrazitPP($this->zamestnanec->idPP);
        $this->zalozky['rozdeleni']['edit'] = $this->seznamPrav['editovatRozdeleni']->lzeZobrazitPP($this->zamestnanec->idPP);
    }

    protected function zobrazFormularZamestnance($novyzamestnanec = FALSE, $novyPP = FALSE)
    {
        $this->nactiNastaveni();

        $this->nactiDataProView($novyPP);

        if ($novyzamestnanec) {
            return new BaseViewObject('pracovnipomery/view_zamestnanecnovy.php');
        } else {
            return new BaseViewObject('pracovnipomery/view_zamestnanec.php');
        }
    }

    protected function handleInsert($section) //, $novyPP
    {
        $this->overPravo('vlozitZamestnance');

        switch ($section) {
            case 'ppomer':
                return $this->novyPP();
            case 'rozdeleni':
                return $this->noveRozdeleniOsoby();
            default:
                $vlozenyZamestnanec = $this->novyZamestnanec();
                if ($vlozenyZamestnanec instanceof ViewObject) {
                    return $vlozenyZamestnanec;
                } elseif ($vlozenyZamestnanec > 0) {
                    return new Redirect('?menuid=zamestnanec&idpp=' . $vlozenyZamestnanec);
                }
        }

    }

    protected function handleUpdate($section)
    {
        $this->loadZamestnanec(TRUE);
        $this->nastavPrava();

        if (!$this->zalozky[$section]['edit']) {
            throw new NeniPravoException();
        }
        //var_dump($section, $_POST, $this);

        switch ($section) {
            case 'hlavni':
                $aktualizace = $this->aktualizovatOsobu($section);
                if ($aktualizace === TRUE) {
                    return new Redirect('?menuid=zamestnanec&idpp=' . $this->zamestnanec->idPP);
                } else {
                    return $aktualizace;
                }
                break;
            case 'ostatni':
                $aktualizace = $this->aktualizovatOsobu($section, 1); //jen uz. pole, eMail a Telefon
                if ($aktualizace === TRUE) {
                    return new Redirect('?menuid=zamestnanec&idpp=' . $this->zamestnanec->idPP);
                } else {
                    return $aktualizace;
                }
                break;
            case 'ostatnipp':
                $aktualizace = $this->aktualizovatDetailyPP();
                if ($aktualizace === TRUE) {
                    return new Redirect('?menuid=zamestnanec&idpp=' . $this->zamestnanec->idPP);
                } else {
                    return $aktualizace;
                }
                break;
            case 'ppomer':
                $aktualizace = $this->aktualizovatPP($section);
                if ($aktualizace === TRUE) {
                    return new Redirect('?menuid=zamestnanec&idpp=' . $this->zamestnanec->idPP);
                } else {
                    return $aktualizace;
                }
                break;
        }
    }

    protected function handlePost($novyPP = FALSE)
    {
        $formsection = isset($_POST['formsection']) ? $_POST['formsection'] : '';

        switch ($_POST['formaction']) {
            case 'insert':
                return $this->handleInsert($formsection); // , $novyPP ? ?

            case 'update':
            default:
                return $this->handleUpdate($formsection);
        }

    }

    protected function handleJSON()
    {
        $action = isset($_GET['jsonaction']) ? $_GET['jsonaction'] : '';

        switch ($action) {
            case 'seznamrozdeleni':
                if (isset($_GET['typrozdeleni'])) {
                    $jsonRozdeleni = new JsonOutputObject(['rozdeleni' => $this->vratSeznamRozdeleni($_GET['typrozdeleni'], $_GET['datumod'], $_GET['datumdo'], $_GET['kodprava']),
                        'result' => JsonOutputObject::RESULT_OK]);
                    return $jsonRozdeleni;
                }
                break;

            case 'nastaveni':
            default:
                return $this->ulozNastaveni();
        }

        return false;
    }

    private function vydaniIDMedia($vyrazenaMedia)
    {
        //TODO: zkontrolovat prava na osobu
        $idOsoby = isset($_POST['osoba']) ? (int)$_POST['osoba'] : (int)$_GET['osoba'];
        $kodKarty = isset($_POST['kodkarty']) ? $_POST['kodkarty'] : $_GET['kodkarty'];
        $datumPrideleni = isset($_POST['datumprideleni']) ? (int)$_POST['datumprideleni'] : (int)$_GET['datumprideleni'];

        require_once('pracovnipomery/f_vydejidentifikacnihomedia.php');

        $pracPomer = new PPomer();
        $pracPomer->nactiPPPodleId($this->database,$idOsoby);

        $f = new VydejIDMediaForm($this->database, $pracPomer, $kodKarty, $datumPrideleni, $vyrazenaMedia);

        if ($f->handleAction()) {
//            echo 'vydani_id media';
            return new Redirect(null, '?menuid='.$_SESSION['Ron.Zamestnanci.Browser']);
        }

        return $f;
    }

    private function vyrazeniIDMedia()
    {
        //TODO: zkontrolovat prava na osobu

        $kodKarty = isset($_POST['karta']) ? $_POST['karta'] : $_GET['karta'];

        require_once('pracovnipomery/f_vyrazenimedia.php');

        $f = new VyrazeniMedia($this->database, $kodKarty);

        if ($f->handleAction()) {
            return new Redirect(null, '?menuid='.$_SESSION['Ron.Zamestnanci.Browser']);
        }

        return $f;
    }

    private function obnovenimedia($kodKarty)
    {
//        $kodKarty = isset($_POST['karta']) ? $_POST['karta'] : $_GET['karta'];

        require_once('pracovnipomery/f_obnovenimedia.php');

        $f = new ObnoveniMedia($this->database, $kodKarty);

//        echo('?menuid='.$_SESSION['Ron.Zamestnanci.Browser'] );
        if ($f->handleAction()) {
            return new Redirect(null, '?menuid='.$_SESSION['Ron.Zamestnanci.Browser'] . $_GET['medium']);
        }

        return $f;
    }


    /** Zobrazi fotku osoby, kontroluji se prava na Osobu podle RC.
     * @param int $idOsoby
     * @param PravaNaOsoby $prava
     * @return FotoOutputObject
     */
    private function fotoOsoby($idOsoby, $prava)
    {
        try {
            $rcOsoby = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . $idOsoby);
            if (!$prava->lzeZobrazitRC($rcOsoby, time(), time())) {
                return new DataOutputObject(TextManager::_('Nemate_pravo'));
            }

            $foto = new FotoOutputObject($this->database);
            $foto->nactiIdFotkyPracovnihoPomeru($idOsoby);
            return $foto;
        } catch (Exception $e) {
            return new DataOutputObject('CHYBA!');
        }
    }

//    /** Vrati JSON objekt s informacemi o poslednim pruchodu zvolene osoby/konkretniho prac. pomeru.
//     * @param JsonOutputObject $json
//     * @param int $idPPosoby id libovolneho pracovniho pomeru osoby (budou se kontrolovat vsechny PP osoby)
//     * @param int $idPP id konkretniho pracovniho pomeru
//     * @param PravaNaOsoby $prava prava pro kontrolu opravneni na zvolenou osobu/prac. pomer
//     * @throws Exception
//     * @throws NeniPravoException
//     */
//    private function pritomnostZamestnance($json, $idPPosoby, $idPP, $prava)
//    {
//        //vyhledani posledniho pruchodu pro osobu, vyhledava se podle RC (RC se najde podle libovolneho ID prac. pomeru osoby)
//        if ($idPPosoby > 0) {
//            $osobaRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . (int)$idPPosoby);
//            if (!$prava->lzeZobrazitRC($osobaRC, time(), time())) {
//                throw new NeniPravoException();
//            }
//
//            $posledniOperace = DochazkaUtils::vratPosledniPruchodSQL($this->database, time(), $osobaRC, null, null, false);
//        } //vyhledani posledniho pruchodu pro konkretni prac. pomer
//        elseif ($idPP > 0) {
//            if (!$prava->lzeZobrazitPP($idPP, time(), time())) {
//                throw new NeniPravoException();
//            }
//
//            $pracovniPomer = PracovniPomerOsoby::vratPP($this->database, $idPP);
//
//            if (count($pracovniPomer) != 1) {
//                throw new Exception('Zamestnanec nenalezen.');
//            } elseif (count($pracovniPomer) == 1) {
//                $pracovniPomer = current($pracovniPomer);
//            }
//
//            $posledniOperace = DochazkaUtils::vratPosledniPruchodSQL($this->database, time(), $pracovniPomer->rc, $pracovniPomer->osobniCislo, $pracovniPomer->datumNastupu);
//        }
//
//        if (!isset($posledniOperace)) {
//            throw new Exception(StringUtils::fromUTF8('Nebyl nalezen posledni pruchod'));
//        }
//
//        //$json->addData('pritomnost', ['pritomen' => $posledniOperace[4], 'datumcasoperace' => $posledniOperace[0], 'popisoperace' => StringUtils::jsonText($posledniOperace[3])]);
//        $data = [$idPP => [
//            'idpp' => $idPP,
//            'pritomnost' => (int)$posledniOperace[4],
//            'datumcasoperace' => $posledniOperace[0],
//            'popisoperace' => StringUtils::jsonText($posledniOperace[3])]
//        ];
//
//        $json->addData('data', $data);
//    }


//    /**
//     * @param JsonOutputObject $json
//     * @param string $kodRozdeleni1
//     * @param int $vcetnePodrizenych1
//     * @param string $kodRozdeleni2
//     * @param int $vcetnePodrizenych2
//     * @param bool $jenRC
//     * @param PravaNaOsoby $prava
//     * @throws Exception
//     * @throws \Ron\DB\DBException
//     */
//    private function pritomnostZamestnancu($json, $kodRozdeleni1, $vcetnePodrizenych1, $kodRozdeleni2, $vcetnePodrizenych2, $jenRC, $prava)
//    {
//        $sql =
//            'SELECT PracovniPomer.Id, PuvDochProMonitorovani.Pritomnost, PuvDochProMonitorovani.DatumCasOperace, OperaceDochazky.Popis,' .
//            '  Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.OsobniCislo, ' .
//            '  (SELECT MAX(PP.ID) FROM PracovniPomer PP WHERE PP.RC = Osoba.RC)' . // idpp pro osoby
//            ' FROM PracovniPomer INNER JOIN PuvDochProMonitorovani' .
//            '  ON PuvDochProMonitorovani.RC = PracovniPomer.RC' .
//            '  AND PuvDochProMonitorovani.OsobniCislo = PracovniPomer.OsobniCislo' .
//            '  AND PuvDochProMonitorovani.DatumNastupu = PracovniPomer.DatumNastupu' .
//            ' INNER JOIN Osoba' .
//            '  ON Osoba.RC = PracovniPomer.RC' .
//            ' LEFT OUTER JOIN OperaceDochazky' .
//            '  ON OperaceDochazky.Kod = PuvDochProMonitorovani.KodOperace' .
//            ' WHERE 1 = 1';
//
//        // filtr rozdeleni 1
//        $filtrRozdeleni1 = ($kodRozdeleni1 != '') && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)
//            ? RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $kodRozdeleni1,
//                time(), time(), $vcetnePodrizenych1)
//            : '';
//
//        if ($filtrRozdeleni1) {
//            $sql .= "\nAND " . $filtrRozdeleni1;
//        }
//
//        // filtr rozdeleni 2
//        $filtrRozdeleni2 = ($kodRozdeleni2 != '') && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)
//            ? RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $kodRozdeleni2,
//                time(), time(), $vcetnePodrizenych2)
//            : '';
//
//        if ($filtrRozdeleni2) {
//            $sql .= "\nAND " . $filtrRozdeleni2;
//        }
//
//        // filtr prava
//        $filtrPrava = Prava::sqlFiltrPodlePrav($prava, 'PracovniPomer');
//        if ($filtrPrava) {
//            $sql .= "\nAND " . $filtrPrava;
//        }
//
//        $sql .= ' ORDER BY PuvDochProMonitorovani.DatumCasOperace';
//
//
//        $cursor = $this ->database ->getSqlResult($sql);
//
//        $data = [];
//        while ($row = $cursor ->fetch()) {
//            $casOd = RonPortalSettings::getIntValue('monitorovani_dnu') > 0
//                ? DateUtils::incDay(time(), -RonPortalSettings::getIntValue('monitorovani_dnu'))
//                : DateUtils::startOfTheDay(time());
//
//            $casOperace = $this ->database ->stringToTimestamp($row[2]);
//
//            $data[$jenRC ? (int)$row[7] : (int)$row[0]] = [
//                'idpp' => $row[0],
//                'pritomnost' => ($casOperace >= $casOd) ? (int)$row[1] : 0,
//                'datumcasoperace' => StringUtils::format('timestamp', $casOperace),
//                'popisoperace' => StringUtils::jsonText($row[3]),
//                'jmeno' => StringUtils::jsonText($row[4] . ' ' . $row[5]),
//                'osobnicislo' => StringUtils::jsonText($row[6])
//            ];
//        }
//
//        $json ->addData('data', $data);
//    }


//    /**
//     * @param JsonOutputObject $output
//     * @throws Exception
//     */
//    private function pritomnost($output)
//    {
//        if (isset($_GET['osoba'])) {
//            $this->pritomnostZamestnance($output, (int)$_GET['osoba'], (int)$_GET['osobapp'], $this->seznamPrav['zobrazitZamestnance']);
//        } else {
//            $this->pritomnostZamestnancu($output,
//                $_GET['rozdeleni1'], (int)$_GET['vcetnepodrizenych1'], $_GET['rozdeleni2'], (int)$_GET['vcetnepodrizenych2'],
//                isset($_GET['jenrc']) && (int)$_GET['jenrc'],
//                $this->seznamPrav['zobrazitZamestnance']);
//        }
//    }


    private function hlavickaVyberAkce($typSeznamu)
    {
        Model::addViewObject(new TextViewObject(TextManager::_('MI_Seznam_PP'), '', 'h1'));

        $sekce = [
            'seznampp' => TextManager::_('Prehled_PP'),
            'seznamosob' => TextManager::_('Seznam_osob'),
        ];

        if (Prava::jsouNejakePrava('zamest_novy')) {
            $sekce['new'] = TextManager::_('Zapis_noveho_zamestnance');
        }

        $list = new LinksList($sekce, $typSeznamu, '?menuid=zamestnanec&action=');
        Model::addViewObject($list);
    }


    /**
     * Zobrazi seznam osob/pracovnich pomeru.
     * @param string $action
     * @return SeznamOsobBrowser|SeznamPPBrowser
     * @throws Exception Pokud nejsou nastaveny sloupce tabulky.
     */
    private function handleSeznamZamestnancu($action = null)
    {
        $typSeznamu = $action ?? NastaveniUzivatele::getUserSettingsItem($this->database, 'zamestnanci_seznam', 'seznampp');

//        if (!RonPortalSettings::getValue('pracovnipomery_sloupce')) { - presunuto primo do seznamu
//            throw new Exception( \Ron\ronText('Zamestnanci_neni_nastaveni_sloupcu_tabulky') );
//        }

        $_SESSION['Ron.Zamestnanci.Browser'] = $typSeznamu;
        NastaveniUzivatele::setUserSettingsItem($this->database, 'zamestnanci_seznam', $typSeznamu, 'seznampp');

        $this->hlavickaVyberAkce($typSeznamu);

        switch ($typSeznamu) {
            case 'seznamosob' :
                $b = new SeznamOsobBrowser($this->database, $typSeznamu);
                $b->load();
                return $b;
            case 'seznampp' :
            default :
                $b = new SeznamPPBrowser($this->database, $typSeznamu);
                $b->load();
                return $b;
        }


    }


//    function handleJsonAction($action, $output)
//    {
//        if (isset($_GET['action'])) {
//            switch (isset($_GET['action'])) {
//                case 'pritomnost' :
//                    $this ->pritomnost($output);
//                    break;
//            }
//        }
//    }


    /** Zpracuje akci zamestnance (vlozeni, aktualizaci, zobrazeni)
     * @return mixed
     */
    private function handleZamestnanec()
    {
        //pokud neni zadana action, nastavi se na edit
        $zamestnanecaction = isset($_GET['action']) ? $_GET['action'] : 'edit';

        $novyPP = isset($_GET['novypp']);

        //TODO: odstranit po dokonceni formulare
        //2.10.2019 kontrola neni nutna, ostatni moznosti nejsou ve switch podporovany
//        global $enable_seznamPP;
//        if (!($zamestnanecaction == 'foto' || $zamestnanecaction == 'pritomnost' || $zamestnanecaction == 'seznam') and !$enable_seznamPP) {
//            RonPortalUtils::neniPravoException();
//        }

        switch ($zamestnanecaction) {
            case 'foto' :
                return $this->fotoOsoby((int)$_GET['osoba'], $this->seznamPrav['zobrazitZamestnance']);

//            case 'pritomnost' :
//                return $this->pritomnostZamestnance((int)$_GET['osoba'], (int)$_GET['osobapp'], $this->seznamPrav['zobrazitZamestnance']);

            case 'seznam' :
            default :
                return $this->handleSeznamZamestnancu(isset($_GET['action']) ? $_GET['action'] : null);

             /*
            case 'prirazenikarty':
                return $this->vydaniIDMedia($_GET['volnamedia']);
            case 'vyrazenimedia':
                return $this->vyrazeniIDMedia();
            case 'obnovenimedia':
                return $this->obnovenimedia($_GET['medium']);
            case 'json':
                return $this->handleJSON();
            case 'insert':
                if (isset($_POST['formaction'])) {
                    return $this->handlePost($novyPP);
                }
                $this->pridejComboboxyPP();
                return $this->zobrazFormularZamestnance(TRUE, $novyPP);//new BaseViewObject('pracovnipomery/view_zamestnanecnovy.php');

            case 'smazatrozdeleni':
                return $this->smazRozdeleni();
            case 'editrozdeleni':
                $this->pridejComboboxyRozdeleni($_GET['typrozdeleni'], $_GET['rozdeleni']);
                return $this->zobrazFormularZamestnance();
            case 'edit':
            default:
                if (isset($_POST['formaction'])) {
                    return $this->handlePost($novyPP);
                }
                $this->pridejComboboxyRozdeleni(); //TODO: zmenit nacitani (zpracovani postu/GETu
                return $this->zobrazFormularZamestnance(FALSE, $novyPP);//new BaseViewObject('pracovnipomery/view_zamestnanec.php');
            */
        }
    }

    public function handleAction($action)
    {
        $this->overPravo('zobrazitZamestnance');

        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'pritomnost' :
                    return parent::handleAction($action);
            }
        }

        HtmlHeaders::getInstance() ->setAppTitle(TextManager::_('Zamestnanci'));

        switch ($action) {
            case 'zamestnanec' :
                return $this->handleZamestnanec();
            case 'seznamosob' :
            case 'seznampp' :
                return $this->handleSeznamZamestnancu($action);
            default :
                throw new Exception('Chyba: neznama akce v metode ZamestnanecController:handeAction!');
        }
    }
}



