<?php

/**
 * UTF-8
 * Pavel Rudol, RON Software
 *  11.9.2015
 */

namespace Ron\RonPortal\Zamestnanci;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\DB\DBRecordField;
use Ron\MessageManager;
use RonPortal\RonPortalDBRecord;
use Ron\DateUtils;
use RonPortal\RonPortalUtils;
use Ron\StringUtils;
use RonPortal\Zamestnanci\ZamestnanciUtils;
use function Ron\ronText;

class RozdeleniOsoby
{
    /** @var Database */
    private $database;

    protected $idPP;
    protected $rc;
    protected $osobniCislo;
    protected $datumNastupu;

    public $typRozdeleni;
    public $typRozdeleniPopis;
    public $kodRozdeleni;
    public $kodRozdeleniPopis;
    public $platnostOd;
    public $platnostDo;

    public function __construct($database = null)
    {
        if ($database) {
            $this->database = $database;
        }
    }

    /**
     *
     * @param Database $database
     * @param PracovniPomerOsoby $pracovniPomer
     * @param int $typRozdeleni
     * @return RozdeleniOsoby[]
     * @throws DBException
     */
    public static function nactiRozdeleniPP($database, $pracovniPomer, $typRozdeleni)
    {
        $params = $database->createParameters();
        $sql =
            "SELECT RozdeleniOsob.KodTypuRozdeleni, RozdeleniOsob.Rozdeleni, RozdeleniOsob.PlatnostOd, RozdeleniOsob.PlatnostDo, " . // 0 - 3
            " Rozdeleni.Popis, TypRozdeleni.Popis " . // 4 - 5
            "FROM RozdeleniOsob JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni) " .
            " JOIN TypRozdeleni ON (TypRozdeleni.Kod = RozdeleniOsob.KodTypuRozdeleni) " .
            "WHERE RozdeleniOsob.RC = " . $params->addParameter($pracovniPomer->rc) .
            "  AND RozdeleniOsob.OsobniCislo = " . $params->addParameter($pracovniPomer->osobniCislo) .
            "  AND RozdeleniOsob.DatumNastupu = " . $params->addParameter($database->dateToParam($pracovniPomer->datumNastupu));
        if ($typRozdeleni != 0) {
            $sql .= "  AND RozdeleniOsob.KodTypuRozdeleni = " . (int)$typRozdeleni;
        }

        $sql .= " ORDER BY RozdeleniOsob.KodTypuRozdeleni, RozdeleniOsob.PlatnostOd";

        $cursor = $database->getSqlResult($sql, $params);

        $seznamRozdeleni = [];

        while ($row = $cursor->fetch()) {
            $rozdeleniOsoby = new RozdeleniOsoby();
            $rozdeleniOsoby->typRozdeleni = (int)$row[0];
            $rozdeleniOsoby->kodRozdeleni = $row[1];
            $rozdeleniOsoby->platnostOd = $database->stringToTimestamp($row[2]);
            $rozdeleniOsoby->platnostDo = $database->stringToTimestamp($row[3]);
            $rozdeleniOsoby->kodRozdeleniPopis = $row[4];
            $rozdeleniOsoby->typRozdeleniPopis = $row[5];
            $rozdeleniOsoby->nactiPracovniPomer($pracovniPomer);

            if ($typRozdeleni != 0) {
                $seznamRozdeleni[] = $rozdeleniOsoby;
            } else {
                $seznamRozdeleni[$row[0]][] = $rozdeleniOsoby;
            }
        }

        return $seznamRozdeleni;
    }

    /**
     * @param bool $kontrolovatDuplicitu
     * @param bool $kontrolovatPovinnaPole
     * @param int $index index prvku v postu
     * @return bool
     * @throws Exception
     */
    public function nactiRozdeleniZPostu($kontrolovatDuplicitu = true, $kontrolovatPovinnaPole = true, $index = null)
    {
        $poradiPrvku = '';
        if (!is_null($index)) {
            $poradiPrvku = '_' . $index;
        }

        if (isset($_POST['idPP'])) {
            $this->idPP = filter_input(INPUT_POST, 'idPP', FILTER_SANITIZE_SPECIAL_CHARS);
        }

        $this->typRozdeleni = filter_input(INPUT_POST, 'rozdeleni_typ_rozdeleni' . $poradiPrvku, FILTER_SANITIZE_SPECIAL_CHARS);
        $this->kodRozdeleni = filter_input(INPUT_POST, 'rozdeleni_kod' . $poradiPrvku, FILTER_SANITIZE_SPECIAL_CHARS);
        if (isset($_POST['rozdeleni_platnostOd' . $poradiPrvku]) and ($_POST['rozdeleni_platnostOd' . $poradiPrvku] != '')) {
            $this->platnostOd = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'rozdeleni_platnostOd' . $poradiPrvku, FILTER_SANITIZE_SPECIAL_CHARS));
        }
        if (isset($_POST['rozdeleni_platnostDo' . $poradiPrvku]) and ($_POST['rozdeleni_platnostDo' . $poradiPrvku] != '')) {
            $this->platnostDo = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'rozdeleni_platnostDo' . $poradiPrvku, FILTER_SANITIZE_SPECIAL_CHARS));
        }

        if ($kontrolovatPovinnaPole && !Zamestnanec::overPovinnePole([
            ['pole' => 'rozdeleni_typ_rozdeleni' . $poradiPrvku, 'chyba' => 'NezadanyTypRozdeleni'],
            ['pole' => 'rozdeleni_kod' . $poradiPrvku, 'chyba' => 'NezadaneRozdeleni'],
            ['pole' => 'rozdeleni_platnostOd' . $poradiPrvku, 'chyba' => 'NezadanaPlatnostOd'],
        ])
        ) {
            return FALSE;
        }

        if ($kontrolovatDuplicitu && !$this->zkontrolujDuplicituRozdeleni()) {
            return FALSE;
        }

        return TRUE;
    }

    /**
     * @param int $platnostOd
     * @throws Exception
     */
    public function zapisRozdeleni($platnostOd = 0)
    {
        $rec = new RonPortalDBRecord($this->database, 'RozdeleniOsob');
        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', false, 15, $this->rc);

        $rec->addField('OsobniCislo', '', false, 15, $this->osobniCislo);
        $rec->addField('DatumNastupu', 'datetime', false, 0, $this->datumNastupu);
        $rec->addField('KodTypuRozdeleni', 'int', false, 0, $this->typRozdeleni);
        $rec->addField('Rozdeleni', '', false, 30, $this->kodRozdeleni);
        $rec->addField('PlatnostOd', 'datetime', false, 0, $platnostOd > 0 ? $platnostOd : $this->platnostOd);
        $rec->addField('PlatnostDo', 'datetime', false, 0, $this->platnostDo);

        $params = $this->database->createParameters();

        $rec->setWhere(
            " RC = " . $params->addParameter($this->rc) .
            " AND OsobniCislo = " . $params->addParameter($this->osobniCislo) .
            " AND DatumNastupu = " . $params->addParameter($this->database->timestampToParam($this->datumNastupu)) .
            " AND KodTypuRozdeleni = " . $params->addParameter($this->typRozdeleni) .
            " AND PlatnostOd = " . $params->addParameter($this->database->timestampToParam($this->platnostOd)), $params);

        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri vkladani rozdeleni' . "\n" . $exception->getMessage());
        }
    }

    /**
     * @param Database $database
     */
    public function setDatabase($database)
    {
        $this->database = $database;
    }

    /**
     *
     * @param PracovniPomerOsoby $pracPomer
     * @return bool
     */
    public function nactiPracovniPomer($pracPomer)
    {
        $this->rc = $pracPomer->rc;
        $this->osobniCislo = $pracPomer->osobniCislo;
        $this->datumNastupu = $pracPomer->datumNastupu;
        $this->idPP = $pracPomer->idPP;

        if (!$this->idPP || !$this->rc) {
            return false;
        }
        return true;
    }

    /**
     * @throws Exception
     */
    public function zrusRozdeleni()
    {
        $rec = new RonPortalDBRecord($this->database, 'RozdeleniOsob');
        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', false, 15, $this->rc);

        $rec->addField('OsobniCislo', '', false, 15, $this->osobniCislo);
        $rec->addField('DatumNastupu', 'datetime', false, 0, $this->datumNastupu);
        $rec->addField('KodTypuRozdeleni', 'int', false, 0, $this->typRozdeleni);
        $rec->addField('Rozdeleni', '', false, 30, $this->kodRozdeleni);
        $rec->addField('PlatnostOd', 'datetime', false, 0, $this->platnostOd);

        $params = $this->database->createParameters();

        $rec->setWhere(
            " RozdeleniOsob.RC = " . $params->addParameter($this->rc) .
            " AND RozdeleniOsob.OsobniCislo = " . $params->addParameter($this->osobniCislo) .
            " AND RozdeleniOsob.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($this->datumNastupu)) .
            " AND RozdeleniOsob.KodTypuRozdeleni = " . $params->addParameter($this->typRozdeleni) .
            " AND RozdeleniOsob.Rozdeleni = " . $params->addParameter($this->kodRozdeleni) .
            " AND RozdeleniOsob.PlatnostOd = " . $params->addParameter($this->database->timestampToParam($this->platnostOd)), $params);

        if (!$rec->deleteRecord()) {
            throw new DBException($this->database, $rec->getSelectSQL());
        }

    }

    /**
     * @return bool
     * @throws Exception
     */
    protected function zkontrolujDuplicituRozdeleni()
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT RozdeleniOsob.RC " .
            "FROM RozdeleniOsob " .
            "WHERE 1 = 1 " .
            "  AND RozdeleniOsob.RC = " . $params->addParameter($this->rc) .
            "  AND RozdeleniOsob.OsobniCislo = " . $params->addParameter($this->osobniCislo) .
            "  AND RozdeleniOsob.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($this->datumNastupu)) .
            "  AND RozdeleniOsob.KodTypuRozdeleni = " . $params->addParameter($this->typRozdeleni) .
            "  AND RozdeleniOsob.Rozdeleni = " . $params->addParameter($this->kodRozdeleni) .
            "  AND RozdeleniOsob.PlatnostOd = " . $params->addParameter($this->database->timestampToParam($this->platnostOd));

        //var_dump($sql, $this);
        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            MessageManager::getInstance()->clear();

            MessageManager::addErrorMessage(ronText('Duplicitni_rozdeleni'), 'Rozdeleni');
            MessageManager::addErrorMessage(ronText('Duplicitni_rozdeleni'), 'PlatnostOd');
            return FALSE;
        } else {
            return TRUE;
        }
    }

}

/**
 * Class PracovniPomerOsoby
 * @package Ron\RonPortal\Zamestnanci
 */
class PracovniPomerOsoby
{
    public $idPP;
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $datumUkonceni;
    public $vybrany;

    public $duvodVzniku;
    public $duvodVznikuPopis;
    public $duvodUkonceni;
    public $duvodUkonceniPopis;
    public $poradoveCisloPP;
    public $zamestnanec;
    public $zamestnanecStr;
    public $typPP;
    public $typPPStr;
    public $zpracovatDochazku;
    public $uzivatelskaPole;

    /** @var RozdeleniOsoby[] */
    public $seznamRozdeleni;

    /** @var array Seznam povinnych poli, ktere se budou kontrolovat pri nacitani z POSTu (nazev pole v POSTu). <br />
     *  Pole jsou ve formatu ['pole' => 'nazev pole POSTu', 'chyba' => 'kod textu chyby'] <br />
     *  Povinna pole dle databazove definice.
     */
    private $seznamPovinnychPoliDB;
    /** @var array Seznam povinnych poli, ktere se budou kontrolovat pri nacitani z POSTu (nazev pole v POSTu). <br />
     *  Pole jsou ve formatu ['pole' => 'nazev pole POSTu', 'chyba' => 'kod textu chyby'] <br />
     *  Povinna pole mozno definovat pred pouzitim ve formulari.
     */
    public $seznamPovinnychPoliUzivatel;

    public function __construct()
    {
        $this->uzivatelskaPole = array_fill(1, 20, NULL);

        $this->seznamPovinnychPoliDB =
            [
                [Zamestnanec::NAZEV_POLE_POST => 'OsobniCislo', Zamestnanec::TEXT_CHYBA => 'NezadaneOsobniCislo'],
                [Zamestnanec::NAZEV_POLE_POST => 'DatumNastupu', Zamestnanec::TEXT_CHYBA => 'NezadaneDatumNastupu']
            ];
    }

    /**
     * @param $database
     * @param int $typRozdeleni
     * @throws Exception
     */
    public function nactiRozdeleniPP($database, $typRozdeleni = 0)
    {
        $this->seznamRozdeleni = RozdeleniOsoby::nactiRozdeleniPP($database, $this, $typRozdeleni);
    }

//    /** Vrátí nové osobní číslo podle zadané řady.
//     * @param string $pocatecniOsCislo osobní čísla číslovat od
//     * @param string $maxPoradi poslední osobní číslo
//     * @param int $delkaOsCisla požadovaná délka osobního čísla
//     * @return string
//     */
//    protected static function getNextOsobniCislo($pocatecniOsCislo, $maxPoradi, $delkaOsCisla)
//    {
//        if (is_null($maxPoradi) || ($maxPoradi == '0')) {
//            $result = $pocatecniOsCislo;
//        } else {
//            try {
//
//                $pomOsCislo = '';
//                $result = '';
//
//                $pomString = $maxPoradi;
//
//                $pomStringArray = str_split($pomString);
//
//                for ($index = 0; $index < count($pomStringArray); $index++) {
//                    if (($pomStringArray[$index] >= '0') && ($pomStringArray[$index] <= '9')) {
//                        $pomOsCislo .= $pomStringArray[$index];
//                    }
//                }
//
//                $index2 = strlen($pomOsCislo);
//
//                $pomOsCislo = (int)$pomOsCislo + 1;
//                $pomOsCislo = str_pad($pomOsCislo, $index2, '0', STR_PAD_LEFT);
//
//                $index2 = 0;
//
//                $pomOsCisloArray = str_split($pomOsCislo);
//
//                for ($index = 0; $index <= max(count($pomStringArray), count($pomOsCisloArray)); $index++) {
//                    if ($index > count($pomStringArray)) {
//                        $result .= $pomOsCisloArray[$index2];
//                        $index2++;
//                    } elseif (($pomStringArray[$index] >= '0') && ($pomStringArray[$index] <= '9')) {
//                        $result .= $pomOsCisloArray[$index2];
//                        $index2++;
//                    } else {
//                        $result .= $pomStringArray[$index];
//                    }
//                }
//
//            } catch (\Exception $e) {
//                $result = '0';
//            }
//
//        }
//
//        $result = str_pad($result, $delkaOsCisla, '0', STR_PAD_LEFT);
//
//        return $result;
//    }

    /** Vrati nove osobni cislo podle nastavene rady u typu pracovniho pomeru.
     * @param Database $database
     * @param string $kodTypuPP
     * @return string
     * @throws Exception
     * @deprecated
     */
    public static function getNextOsobniCisloTypuPP($database, $kodTypuPP)
    {
        return ZamestnanciUtils::getNextOsobniCisloTypuPP($database, $kodTypuPP);
//        $result = '';
//
//        if ($kodTypuPP == '') {
//            return $result;
//        }
//
//        $params = $database->createParameters();
//
//        $sql = "SELECT OsobniCislaOd, DelkaOsobnihoCisla FROM TypPracovnihoPomeru WHERE (Kod = " . $params->addParameter($kodTypuPP) . ")";
//
//        $cursor = $database->getSqlResult($sql, $params);
//
//        $row = $cursor->fetch();
//
//        $pomString = $row[0];
//        $delkaOsCisla = $row[1];
//
//        $params = $database->createParameters();
//
//        $maxPoradi = DBUtils::getSqlValue($database,
//            "SELECT MAX(PracovniPomer.OsobniCislo) AS MaxPoradi FROM PracovniPomer, TypPracovnihoPomeru " .
//            "WHERE (PracovniPomer.TypPracovnihoPomeru = TypPracovnihoPomeru.Kod)" .
//            "  AND (TypPracovnihoPomeru.OsobniCislaOd =  " . $params->addParameter($pomString) . ")", $params);
//            //21.12.2016 RON-PR do rady osobnich cisel se berou vsechny typy osob
//            //"  AND (PracovniPomer.Zamestnanec <> 101)");
//
//        return self::getNextOsobniCislo($pomString, $maxPoradi, $delkaOsCisla);
    }

    /** Vrati nove osobni cislo pro typ osoby.
     * @param Database $database
     * @param int $typOsoby
     * @param string $pocatecniOsCislo
     * @param int $delkaOsCisla
     * @return string
     * @throws Exception
     * @deprecated
     */
    public static function getNextOsobniCisloTypOsoby($database, $typOsoby, $pocatecniOsCislo, $delkaOsCisla)
    {
        return ZamestnanciUtils::getNextOsobniCisloTypOsoby($database, $typOsoby, $pocatecniOsCislo, $delkaOsCisla);
//        $result = '';
//
//        if ($typOsoby < 0) {
//            return $result;
//        }
//
//        $params = $database->createParameters();
//
//        $maxPoradi = DBUtils::getSqlValue($database,
//            "SELECT MAX(PracovniPomer.OsobniCislo) AS MaxPoradi FROM PracovniPomer " .
//            "WHERE (Zamestnanec =  " . $params->addParameter($typOsoby) . ")", $params);
//
//        return self::getNextOsobniCislo($pocatecniOsCislo, $maxPoradi, $delkaOsCisla);
    }

    /** Nacte data z radku databaze.
     * @param Database $database
     * @param array $row
     */
    private function nactiRadek($database, $row)
    {
        $this->rc = $row[0];
        $this->osobniCislo = $row[1];
        $this->datumNastupu = $database->stringToTimestamp($row[2]);
        $this->datumUkonceni = $row[3] ? $database->stringToTimestamp($row[3]) : 0;

        $this->uzivatelskaPole = [];

        $this->poradoveCisloPP = $row[4];
        $this->zamestnanec = (int)$row[5];
        $this->zamestnanecStr = $row[28];
        $this->typPP = $row[6];
        $this->typPPStr = $row[29];
        $this->zpracovatDochazku = $row[7];

        $this->duvodVzniku = $row[30];
        $this->duvodVznikuPopis = $row[32];
        $this->duvodUkonceni = $row[31];
        $this->duvodUkonceniPopis = $row[33];

        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelskaPole[$i] = $row[$i + 7];
        }
    }

    /** Nacte pracovni pomer z databaze
     * @param Database $database
     * @param int $idPP
     * @param string $rc
     * @param bool $nactiSeznamPPOsoby
     * @return PracovniPomerOsoby[]
     * @throws Exception
     */
    public static function vratPP($database, $idPP = null, $rc = null, $nactiSeznamPPOsoby = false)
    {
        $sql =
            "SELECT PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.DatumUkonceni, " . // 0 - 3
            "   PracovniPomer.PoradoveCisloPP, PracovniPomer.Zamestnanec, PracovniPomer.TypPracovnihoPomeru, PracovniPomer.ZpracovatDochazku, " . // 4 - 7
            "   PracovniPomer.uzivatelska1, PracovniPomer.uzivatelska2, PracovniPomer.uzivatelska3, PracovniPomer.uzivatelska4, PracovniPomer.uzivatelska5, " . // 8 - 12
            "   PracovniPomer.uzivatelska6, PracovniPomer.uzivatelska7, PracovniPomer.uzivatelska8, PracovniPomer.uzivatelska9, PracovniPomer.uzivatelska10, " . // 13 - 17
            "   PracovniPomer.uzivatelska11, PracovniPomer.uzivatelska12, PracovniPomer.uzivatelska13, PracovniPomer.uzivatelska14, PracovniPomer.uzivatelska15, " . // 18 - 22
            "   PracovniPomer.uzivatelska16, PracovniPomer.uzivatelska17, PracovniPomer.uzivatelska18, PracovniPomer.uzivatelska19, PracovniPomer.uzivatelska20, " . // 23 - 27
            "   S_TypOsoby.Popis, TypPracovnihoPomeru.Popis, PracovniPomer.DuvodVznikuPP, PracovniPomer.DuvodUkonceniPP, DuvodVznikuPP.Popis, DuvodUkonceniPP.Popis, " . // 28 - 33
            "   PracovniPomer.Id " . // 34
            "FROM PracovniPomer " .
            " JOIN S_TypOsoby ON (S_TypOsoby.Kod = PracovniPomer.Zamestnanec) " .
            " JOIN TypPracovnihoPomeru ON (TypPracovnihoPomeru.Kod = PracovniPomer.TypPracovnihoPomeru) " .
            " LEFT JOIN DuvodVznikuPP ON (DuvodVznikuPP.Kod = PracovniPomer.DuvodVznikuPP)" .
            " LEFT JOIN DuvodUkonceniPP ON (DuvodUkonceniPP.Kod = PracovniPomer.DuvodUkonceniPP) ";

        $params = $database->createParameters();

        if (!$nactiSeznamPPOsoby && $idPP) {
            $sql .=
                "WHERE PracovniPomer.ID = " . $params->addParameter($idPP);
        }
        elseif ($nactiSeznamPPOsoby && ($rc != '')) {
            $sql .=
                "WHERE PracovniPomer.RC = " . $params->addParameter($rc);
        }
        else {
            throw new Exception('Zaměstnanec nenalezen!');
        }

        $sql .=
            " ORDER BY PracovniPomer.DatumNastupu";

        $cursor = $database->getSqlResult($sql, $params);


        $seznamPP = [];

        while ($row = $cursor->fetch()) {
            $pracovniPomer = new PracovniPomerOsoby();
            $pracovniPomer->nactiRadek($database, $row);
            //pokud je predane ID PP shodne z akt. radkem databaze, oznaci se PP jako vybrany
            $pracovniPomer->vybrany = ($row[34] == $idPP);
            $seznamPP[$row[34]] = $pracovniPomer;
        }

        if (count($seznamPP) == 0) {
            throw new Exception("Zamestnanec s pracovnim pomerem ID: " . (int)$idPP . " nebyl nalezen!");
        }

        return $seznamPP;

    }

    /** Overi zda existuje pracovni pomer se stejnym osobnim cislem (RC, Datem nastupu)
     * @param Database $database
     * @param bool $jenOsobniCislo pokud je true, kontroluje se jen duplicita os. cisla, jinak celeho PP
     * @return bool vraci false, pokud existuje stejny PP (stejne os. cislo)
     * @throws Exception
     */
    protected function zkontrolujDuplicituPP($database, $jenOsobniCislo = FALSE)
    {
        $params = $database->createParameters();

        $sql = "SELECT PracovniPomer.RC, PracovniPomer.OsobniCislo, PracovniPomer.DatumNastupu, PracovniPomer.ID " . // 0 - 3
            "FROM PracovniPomer " .
            "WHERE 1 = 1 " .
            "  AND PracovniPomer.OsobniCislo = " . $params->addParameter($this->osobniCislo);
        if ($jenOsobniCislo == FALSE) {
            $sql .= "  AND PracovniPomer.RC = " . $params->addParameter($this->rc) .
                "  AND PracovniPomer.DatumNastupu = " . $params->addParameter($database->timestampToParam($this->datumNastupu));
        }

        $sql .= " AND PracovniPomer.ID <> " . $params->addParameter($this->idPP);
        // var_dump($sql, $this);
        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            MessageManager::getInstance()->clear();
            if ($jenOsobniCislo) {
                MessageManager::addErrorMessage(ronText('DuplicitniOsobniCislo'), 'OsobniCislo');
                return FALSE;
            }
            MessageManager::addErrorMessage(ronText('DuplicitniPP'), 'OsobniCislo');
            MessageManager::addErrorMessage(ronText('DuplicitniPP'), 'DatumNastupu');
            return FALSE;
        } else {
            return TRUE;
        }

    }

    /**
     * @param Database $database
     * @param string $rc
     * @return bool
     * @throws Exception
     */
    public function nactiPPzPostu($database, $rc = '')
    {
        if ($rc != '') {
            $this->rc = $rc;
        } else {
            $this->rc = filter_input(INPUT_POST, 'RC', FILTER_SANITIZE_SPECIAL_CHARS);
        }


        if (!$this->rc) {
            MessageManager::addErrorMessage(ronText('NezadaneRC'), 'RC');
            return false;
        }

        $this->idPP = filter_input(INPUT_POST, 'idPP', FILTER_SANITIZE_SPECIAL_CHARS);

        $this->zamestnanec = filter_input(INPUT_POST, 'TypOsoby', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->osobniCislo = filter_input(INPUT_POST, 'OsobniCislo', FILTER_SANITIZE_SPECIAL_CHARS);
        if (isset($_POST['DatumNastupu']) and ($_POST['DatumNastupu'] != '')) {
            $this->datumNastupu = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'DatumNastupu', FILTER_SANITIZE_SPECIAL_CHARS));
        }
        if (isset($_POST['DatumUkonceni']) and ($_POST['DatumUkonceni'] != '')) {
            $this->datumUkonceni = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'DatumUkonceni', FILTER_SANITIZE_SPECIAL_CHARS));
        }
        $this->poradoveCisloPP = filter_input(INPUT_POST, 'PoradoveCisloPP', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->typPP = filter_input(INPUT_POST, 'TypPracovnihoPomeru', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zpracovatDochazku = filter_input(INPUT_POST, 'ZpracovatDochazku', FILTER_SANITIZE_SPECIAL_CHARS) == 'on';

        $this->duvodVzniku = filter_input(INPUT_POST, 'DuvodVzniku');
        $this->duvodUkonceni = filter_input(INPUT_POST, 'DuvodUkonceni');

        if (!$this->zkontrolujDuplicituPP($database)) {
            return FALSE;
        }

        return TRUE;
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function nactiDetailyPPzPostu()
    {
        if (!empty($this->seznamPovinnychPoliUzivatel)) {
            if (!Zamestnanec::overPovinnePole($this->seznamPovinnychPoliUzivatel)) {
                return false;
            }
        }

        for ($i = 1; $i <= 20; $i++) {
            if (isset($_POST['pracovnipomer_uzivatelska'.$i . '_pocet_poli'])) {
                for ($n = 1; $n <= $_POST['pracovnipomer_uzivatelska'.$i . '_pocet_poli']; $n++) {
                    $this->uzivatelskaPole[$i] .= filter_input(INPUT_POST, 'pracovnipomer_uzivatelska' . $i . '_' . $n, FILTER_SANITIZE_SPECIAL_CHARS);
                }
            }
            else {
                $this->uzivatelskaPole[$i] = filter_input(INPUT_POST, 'pracovnipomer_uzivatelska' . $i, FILTER_SANITIZE_SPECIAL_CHARS);
            }
        }

        return TRUE;
    }

    /**
     * @param $database
     * @param string $logDesc
     * @return mixed
     * @throws Exception
     */
    public function vlozPP($database, $logDesc = 'Vlozeni zamestnance')
    {
        $rec = new RonPortalDBRecord($database, 'PracovniPomer');
        $rec->logDescription = $logDesc;

        $rec->addField('RC', '', false, 15, $this->rc);

        $rec->addField('OsobniCislo', '', false, 15, $this->osobniCislo);
        $rec->addField('DatumNastupu', 'datetime', false, 0, $this->datumNastupu);
        $rec->addField('DatumUkonceni', 'datetime', false, 0, $this->datumUkonceni ?: null);
        $rec->addField('ID', 'int', false, 0, SQLUtils::newSerialNumber($database, 'PracovniPomer', 'ID'));
        $rec->addField('Zamestnanec', 'int', false, 0, $this->zamestnanec);
        $rec->addField('TypPracovnihoPomeru', '', false, 10, $this->typPP);
        $rec->addField('ZpracovatDochazku', 'int', false, 0, $this->zpracovatDochazku);
        $rec->addField('DuvodVznikuPP', '', false, 10, $this->duvodVzniku);
        $rec->addField('DuvodUkonceniPP', '', false, 10, $this->duvodUkonceni);

        $rec->addField('Insert_UserName', '', TRUE, 25, RonPortalUtils::getLogUserName());
        $rec->addField('Insert_Timestamp', 'timestamp', TRUE, 0, time());

        try {
            $rec->insertRecord();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri vkladani zamestnance' . "\n" . $exception->getMessage());
        }

        $this->idPP = $rec['ID'];

        return $rec['ID'];
    }

    /**
     * @param Database $database
     * @param int $jenDetaily 0 = hlavni udaje osoby (vse mimo uz. pole), 1 = jen detaily (uz. pole), 2 = vse
     * @return mixed|string
     */
    public function aktualizujPP($database, $jenDetaily = 0)
    {
        $rec = new RonPortalDBRecord($database, 'PracovniPomer');
        $rec->logDescription = 'Aktualizace pracovniho pomeru';

        if ($jenDetaily == 1 || $jenDetaily == 2) {
            for ($i = 1; $i <= 20; $i++) {
                $rec->addField('uzivatelska' . $i, '', false, 100, $this->uzivatelskaPole[$i]);
            }
        }
        if ($jenDetaily == 0 || $jenDetaily == 2) {
            $rec->addField('OsobniCislo', '', false, 15, $this->osobniCislo);
            $rec->addField('DatumNastupu', 'datetime', false, 0, $this->datumNastupu);
            $rec->addField('DatumUkonceni', 'datetime', false, 0, $this->datumUkonceni ?: null);
            $rec->addField('Zamestnanec', 'int', false, 0, $this->zamestnanec);
            $rec->addField('TypPracovnihoPomeru', '', false, 10, $this->typPP);
            $rec->addField('ZpracovatDochazku', 'int', false, 0, $this->zpracovatDochazku);
            $rec->addField('DuvodVznikuPP', '', false, 10, $this->duvodVzniku);
            $rec->addField('DuvodUkonceniPP', '', false, 10, $this->duvodUkonceni);
        }

        $rec->addField('Update_UserName', '', FALSE, 25, RonPortalUtils::getLogUserName())
            ->updateLog = true;
        $rec->addField('Update_Timestamp', 'timestamp', FALSE, 0, time())
            ->updateLog = true;

        $rec->where = "PracovniPomer.ID = " . (int)$this->idPP;

        try {
            $rec->loadAndUpdateRecord();
        } catch (Exception $ex) {
            return $ex->getMessage();
        }

        return $rec['ID'];
    }

}

/**
 *
 */
class Zamestnanec
{
    const NAZEV_POLE_POST = 'pole';
    const TEXT_CHYBA = 'chyba';

    /** @var \Ron\Db\Database */
    private $database;

    protected $noveRC;
    public $idPP;
    public $rc;
    public $jmeno;
    public $prijmeni;
    public $rodneCislo;

    /** @var PracovniPomerOsoby[] $seznamPP pole pracovních pomeru zaměstnance */
    public $seznamPP;

    public $idFotografie;

    public $titul;
    public $titulZa;
    public $datumNarozeni;
    public $email;
    public $telefon;
    public $poznamka;
    public $uzivatelskaPole;

    //personální údaje
    public $mistoNarozeni;
    public $okresNarozeni;
    public $narodnost;
    public $statniPrislusnost;
    public $rodnePrijmeni;

    public $pohlavi;
    public $pohlaviPopis;

    /** @var array Seznam povinnych poli, ktere se budou kontrolovat pri nacitani z POSTu (nazev pole v POSTu). <br />
     *  Pole jsou ve formatu ['pole' => 'nazev pole POSTu', 'chyba' => 'kod textu chyby'] <br />
     *  Povinna pole dle databazove definice.
     */
    private $seznamPovinnychPoliDB;
    /** @var array Seznam povinnych poli, ktere se budou kontrolovat pri nacitani z POSTu (nazev pole v POSTu). <br />
     *  Pole jsou ve formatu ['pole' => 'nazev pole POSTu', 'chyba' => 'kod textu chyby'] <br />
     *  Povinna pole mozno definovat pred pouzitim ve formulari.
     */
    public $seznamPovinnychPoliUzivatel;

    /**
     *
     * @param \Ron\Db\Database $database
     * @param int $id id pracovniho pomeru
     * @param string $rc
     * @throws Exception
     */
    public function __construct($database, $id = 0, $rc = '')
    {
        $this->database = $database;
        $this->seznamPP = [];

        if ($id) {
            $this->idPP = (int)$id;
        } elseif ($rc != '') {
            $this->rc = $rc;
        }

        if ($this->idPP || ($this->rc != '')) {
            $this->nactiZamestnance();
        }

        $this->seznamPovinnychPoliDB =
        [
            [Zamestnanec::NAZEV_POLE_POST => 'Prijmeni', Zamestnanec::TEXT_CHYBA => 'NezadanePrijmeni'],
        ];
    }

    /**
     *
     * @param Database $database
     * @param string $rc
     * @return string
     * @throws DBException
     */
    public static function vratPrijmeniJmenoOsoby($database, $rc)
    {
        $params = $database->createParameters();

        $sql = 'SELECT Prijmeni, Jmeno, Titul  FROM Osoba WHERE RC = ' . $params->addParameter($rc);

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        if ($row) {
            return $row[0] . ' ' . $row[1] . ($row[2] ? ', ' . $row[2] : '');
        }
        return '';
    }


    /**
     * @param bool $jenDetaily
     * @return bool
     * @throws Exception
     */
    public function nactiOsobuPost($jenDetaily = FALSE)
    {
        $result = true;

        if (!empty($this->seznamPovinnychPoliUzivatel)) {
            if (!self::overPovinnePole($this->seznamPovinnychPoliUzivatel)) {
                $result = false;
            }
        }

        if ($jenDetaily) {
            for ($i = 1; $i <= 20; $i++) {
                if (isset($_POST['osoba_uzivatelska'.$i . '_pocet_poli'])) {
                    for ($n = 1; $n <= $_POST['osoba_uzivatelska'.$i . '_pocet_poli']; $n++) {
                        $this->uzivatelskaPole[$i] .= filter_input(INPUT_POST, 'osoba_uzivatelska' . $i . '_' . $n, FILTER_SANITIZE_SPECIAL_CHARS);
                    }
                }
                else {
                    $this->uzivatelskaPole[$i] = filter_input(INPUT_POST, 'osoba_uzivatelska' . $i, FILTER_SANITIZE_SPECIAL_CHARS);
                }
            }
        } else {
            if (!empty($this->seznamPovinnychPoliDB)) {
                if (!self::overPovinnePole($this->seznamPovinnychPoliDB)) {
                    $result = false;
                }
            }

            if ($this->rc == '') {
                $this->rc = filter_input(INPUT_POST, 'RC', FILTER_SANITIZE_SPECIAL_CHARS);
            } else {
                $this->noveRC = filter_input(INPUT_POST, 'RC', FILTER_SANITIZE_SPECIAL_CHARS);
            }
            $this->rodneCislo = filter_input(INPUT_POST, 'rodnecislo', FILTER_SANITIZE_SPECIAL_CHARS);
            $this->titul = filter_input(INPUT_POST, 'Titul', FILTER_SANITIZE_SPECIAL_CHARS);
            $this->jmeno = filter_input(INPUT_POST, 'Jmeno', FILTER_SANITIZE_SPECIAL_CHARS);
            $this->prijmeni = filter_input(INPUT_POST, 'Prijmeni', FILTER_SANITIZE_SPECIAL_CHARS);
            if (isset($_POST['DatumNarozeni']) and ($_POST['DatumNarozeni'] != '')) {
                $this->datumNarozeni = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'DatumNarozeni', FILTER_SANITIZE_SPECIAL_CHARS));
            }
            $this->titulZa = filter_input(INPUT_POST, 'TitulZa', FILTER_SANITIZE_SPECIAL_CHARS);
            $this->poznamka = filter_input(INPUT_POST, 'Poznamka', FILTER_SANITIZE_SPECIAL_CHARS);

            $this->email = filter_input(INPUT_POST, 'eMail', FILTER_SANITIZE_SPECIAL_CHARS);

            //telefon muze byt rozdelen na 2 pole: telefon_predvolba a Telefon
            if (isset($_POST['telefon_predvolba'])) {
                $this->telefon = filter_input(INPUT_POST, 'telefon_predvolba', FILTER_SANITIZE_SPECIAL_CHARS);
            }

            $this->telefon .= filter_input(INPUT_POST, 'Telefon', FILTER_SANITIZE_SPECIAL_CHARS);

            for ($i = 1; $i <= 20; $i++) {
                $this->uzivatelskaPole[$i] = filter_input(INPUT_POST, 'osoba_uzivatelska' . $i, FILTER_SANITIZE_SPECIAL_CHARS);
            }

            $this->mistoNarozeni = filter_input(INPUT_POST, 'MistoNarozeni', FILTER_SANITIZE_SPECIAL_CHARS);

            if ((strlen($this->mistoNarozeni) > 0) && (strlen($this->mistoNarozeni) < 11)) {
                $this->nactiMestoOkresZDB();
            }

            $this->narodnost = filter_input(INPUT_POST, 'Narodnost', FILTER_SANITIZE_SPECIAL_CHARS);
            $this->statniPrislusnost = filter_input(INPUT_POST, 'StatniPrislusnost', FILTER_SANITIZE_SPECIAL_CHARS);
            $this->rodnePrijmeni = filter_input(INPUT_POST, 'RodnePrijmeni', FILTER_SANITIZE_SPECIAL_CHARS);

            $this->pohlavi = filter_input(INPUT_POST, 'Pohlavi', FILTER_SANITIZE_SPECIAL_CHARS);
        }
        return $result;
    }


    /**
     * @param Database $database
     * @param string $rodneCislo
     * @return bool|string vraci RC, pokud jiz existuje osoba s rodnym cislem, jinak false
     * @throws Exception
     */
    public static function existujeRC($database, $rodneCislo)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT RC FROM OsobniUdaj WHERE RodneCislo = " . $params->addParameter($rodneCislo);

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();
        if ($row) {
            if ($row[0] !== '')
                return $row[0];
        }

        return false;
    }

    /**
     * @throws Exception
     */
    protected function nactiZamestnance()
    {
        $sql =
            "SELECT Osoba.Jmeno, Osoba.Prijmeni, Osoba.RC, Osoba.Titul, Osoba.TitulZa, OsobniUdaj.DatumNarozeni, " . // 0 - 5
            "Osoba.eMail, Osoba.Telefon, " . $this->database->castWideMemo('Osoba.Poznamka') . ", " . // 6 - 8
            "Osoba.uzivatelska1, Osoba.uzivatelska2, Osoba.uzivatelska3, Osoba.uzivatelska4, Osoba.uzivatelska5, " . // 9 - 13
            "Osoba.uzivatelska6, Osoba.uzivatelska7, Osoba.uzivatelska8, Osoba.uzivatelska9, Osoba.uzivatelska10, " . // 14 - 18
            "Osoba.uzivatelska11, Osoba.uzivatelska12, Osoba.uzivatelska13, Osoba.uzivatelska14, Osoba.uzivatelska15, " . // 19 - 23
            "Osoba.uzivatelska16, Osoba.uzivatelska17, Osoba.uzivatelska18, Osoba.uzivatelska19, Osoba.uzivatelska20, " . // 24 - 28
            "OsobniUdaj.MistoNarozeni, OsobniUdaj.Narodnost, OsobniUdaj.StatniPrislusnost, OsobniUdaj.RodneJmeno, Osoba.IdFotografie, " . //29 - 33
            "OsobniUdaj.Pohlavi, (SELECT Popis FROM S_Pohlavi WHERE Kod = OsobniUdaj.Pohlavi), OsobniUdaj.RodneCislo " . //34, 35, 36
            "FROM Osoba LEFT JOIN OsobniUdaj ON (OsobniUdaj.RC = Osoba.RC)";

        $params = $this->database->createParameters();

        if ($this->idPP > 0) {
                $sql .=
                    " JOIN PracovniPomer ON (PracovniPomer.RC = Osoba.RC) " .
                    "WHERE PracovniPomer.ID = " . $params->addParameter($this->idPP);
            }
            else {
                $sql .=
                    "WHERE Osoba.RC = " . $params->addParameter($this->rc);
            }

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if (!$row) {
            throw new Exception("Zamestnanec s " . ($this->idPP ? ("pracovnim pomerem ID: " . $this->idPP) : "ID osoby: " . StringUtils::htmlTextConvert($this->rc)) . "nebyl nalezen!");
        }

        $this->nactiRadekDatabaze($row);
        $this->seznamPP = PracovniPomerOsoby::vratPP($this->database, $this->idPP, $this->rc, true);
    }

    private function nactiRadekDatabaze($row)
    {
        $this->jmeno = $row[0];
        $this->prijmeni = $row[1];
        $this->rc = $row[2];

        $this->uzivatelskaPole = [];

        $this->titul = $row[3];
        $this->titulZa = $row[4];
        $this->datumNarozeni = $this->database->stringToTimestamp($row[5]);
        $this->email = $row[6];
        $this->telefon = $row[7];
        $this->poznamka = $row[8];
        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelskaPole[$i] = $row[$i + 8];
        }

        $this->mistoNarozeni = $row[29];
        $this->narodnost = $row[30];
        $this->statniPrislusnost = $row[31];
        $this->rodnePrijmeni = $row[32];
        $this->idFotografie = $row[33];

        $this->pohlavi = $row[34];
        $this->pohlaviPopis = $row[35];
        $this->rodneCislo = $row[36];
    }

    /**
     * @throws Exception
     */
    protected function nactiMestoOkresZDB()
    {
        $params = $this->database->createParameters();

        $sql = 'SELECT Okres, Popis FROM Obec WHERE Kod = ' . $params->addParameter($this->mistoNarozeni);

        $cursor = $this->database->select($sql, $params);

        //pokud je zadane mesto z databaze, doplni okres narozeni a misto narozeni upravi na popis obce
        $row = $cursor->fetch();
        if ($row) {
            $this->okresNarozeni = $row[0];
            $this->mistoNarozeni = $row[1];
        }
    }

    /**
     * @param $povinnaPole
     * @return bool
     * @throws Exception
     */
    public static function overPovinnePole($povinnaPole)
    {
        foreach ($povinnaPole as $pole) {
            if ((!is_null($pole)) && ($_POST[$pole['pole']] == '')) {
                MessageManager::addErrorMessage(ronText($pole['chyba']), $pole['pole']);
            }
        }
        unset($pole);

        if (MessageManager::getInstance()->isError()) {
            return FALSE;
        }

        return TRUE;
    }
}

class OsobaRecord extends RonPortalDBRecord
{
    public $rc;
    public $jmeno;
    public $prijmeni;

    public $titul;
    public $titulZa;
    public $email;
    public $telefon;
    public $poznamka;
    /** @var DBRecordField[] */
    public $uzivatelskaPole;

    public function __construct($database)
    {
        parent::__construct($database, 'Osoba');

        $this->rc = $this->addField('RC',  '', false, 15);
        $this->jmeno = $this->addField('Jmeno', '', false, 30);
        $this->prijmeni = $this->addField('Prijmeni', '', false, 30);
        $this->titul = $this->addField('Titul', '', false, 20);
        $this->titulZa = $this->addField('TitulZa', '', false, 20);
        $this->email = $this->addField('eMail', '', false, 60);
        $this->telefon = $this->addField('Telefon', '', false, 60);
        $this->poznamka = $this->addField('Poznamka', 'memo');
        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelskaPole[$i] = $this->addField("uzivatelska{$i}", '', false, 100);
        }

        $this->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $this->addField('Insert_Timestamp', 'timestamp', true, 0, DateUtils::now());

        $this->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
            ->updateLog = true;
        $this->addField('Update_Timestamp', 'timestamp', false, 0, DateUtils::now())
            ->updateLog = true;
    }

    /**
     * @param string $rc ID osoby
     * @return bool
     * @throws DBException
     */
    public function nactiOsobu($rc)
    {
        $params = $this->database->createParameters();
        $this->setWhere("RC = " . $params->addParameter($rc), $params);

        return parent::loadRecord();
    }

    public function nactiPOST()
    {
        if (isset($_POST['RC']))
            $this->rc->setNewValue($_POST['RC']);
        else
            $this->rc->value = $this->rc->originalValue;

        $this->jmeno->setNewValue($_POST['Jmeno']);
        $this->prijmeni->setNewValue($_POST['Prijmeni']);
        $this->titul->setNewValue($_POST['Titul']);
        $this->titulZa->setNewValue($_POST['TitulZa']);
        $this->email->setNewValue($_POST['eMail']);
        $this->telefon->setNewValue(($_POST['telefon_predvolba'] ?: null) . $_POST['Telefon']);
        $this->poznamka->setNewValue($_POST['Poznamka']);
        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelskaPole[$i]->setNewValue($_POST["osoba_uzivatelska$i"]);
        }
    }

    /**
     * @throws Exception
     */
    private function nastavRC()
    {
        $nastaveniGenerovaniID = ZamestnanciUtils::getIDOsobySettings($this->database);

        //2 = mamualni zadani RC (ID) osoby
        if ($nastaveniGenerovaniID['typ'] !== 2) {
            $rc = SQLUtils::getNextRecordSerialCode($this->database, 'Osoba', 'RC', $nastaveniGenerovaniID['maska'], $nastaveniGenerovaniID['minimum'], $nastaveniGenerovaniID['maximum']);

            $this->rc->setNewValue($rc);
        }

    }

    /**
     * @throws Exception
     */
    function insertRecord(){
        $this->nastavRC();
        parent::insertRecord();
    }

}

class OsobniUdajRecord extends RonPortalDBRecord
{
    public $rc;
    public $rodneCislo;
    public $rodneJmeno;
    public $datumNarozeni;

    public $pohlavi;
    public $narodnost;
    public $mistoNarozeni;
    public $okresNarozeni;
    public $statniPrislusnost;

    public function __construct($database)
    {
        parent::__construct($database, 'OsobniUdaj');

        $this->rc = $this->addField('RC',  '', true, 15);
        $this->rodneCislo = $this->addField('RodneCislo', '', false, 30);
        $this->rodneJmeno = $this->addField('RodneJmeno', '', false, 30);
        $this->datumNarozeni = $this->addField('DatumNarozeni', 'timestamp', false);
        $this->pohlavi = $this->addField('Pohlavi', 'int', false);
        $this->narodnost = $this->addField('Narodnost', 'int', false);
        $this->mistoNarozeni = $this->addField('MistoNarozeni', '', false, 30);
        $this->okresNarozeni = $this->addField('OkresNarozeni', '', false, 10);
        $this->statniPrislusnost = $this->addField('StatniPrislusnost', '', false, 10);

        $this->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $this->addField('Insert_Timestamp', 'timestamp', true, 0, DateUtils::now());

        $this->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
            ->updateLog = true;
        $this->addField('Update_Timestamp', 'timestamp', false, 0, DateUtils::now())
            ->updateLog = true;
    }

    /**
     * @param string $rc ID osoby
     * @return bool
     * @throws DBException
     */
    public function nactiOsobniUdajeOsoby($rc)
    {
        $params = $this->database->createParameters();
        $this->setWhere("RC = " . $params->addParameter($rc), $params);

        return parent::loadRecord();
    }


    /**
     * @throws Exception
     */
    public function nactiPOST()
    {
        $this->rodneCislo->setNewValue($_POST['RodneCislo']);
        $this->rodneJmeno->setNewValue($_POST['RodnePrijmeni']);
        if ($_POST['DatumNarozeni'] != '')
            $this->datumNarozeni->setNewValue(DateUtils::dateStringToTimestamp($_POST['DatumNarozeni']));
        $this->pohlavi->setNewValue($_POST['Pohlavi']);
        $this->narodnost->setNewValue($_POST['Narodnost']);
        $this->mistoNarozeni->setNewValue($_POST['MistoNarozeni']);
        if ((strlen($this->mistoNarozeni->value) > 0) && (strlen($this->mistoNarozeni->value) < 11))
            $this->nactiMestoOkres();
        $this->statniPrislusnost->setNewValue($_POST['StatniPrislusnost']);
    }

    /**
     * @throws Exception
     */
    protected function nactiMestoOkres()
    {
        $params = $this->database->createParameters();

        $sql = 'SELECT Okres, Popis FROM Obec WHERE Kod = ' . $params->addParameter($this->mistoNarozeni->value);

        $cursor = $this->database->select($sql, $params);

        //pokud je zadane mesto z databaze, doplni okres narozeni a misto narozeni upravi na popis obce
        $row = $cursor->fetch();
        if ($row) {
            $this->okresNarozeni->setNewValue($row[0]);
            $this->mistoNarozeni->setNewValue($row[1]);
        }
    }
}

class PracovniPomerRecord extends RonPortalDBRecord
{
    public $idPP;
    public $rc;
    public $osobniCislo;
    public $datumNastupu;
    public $datumUkonceni;

    public $duvodVzniku;
    public $duvodUkonceni;
    public $poradoveCisloPP;
    public $zamestnanec;
    public $typPP;
    public $zpracovatDochazku;

    /** @var DBRecordField[] */
    public $uzivatelskaPole;

    public function __construct($database)
    {
        parent::__construct($database, 'PracovniPomer');

        $this->idPP = $this->addField('ID', 'int', true);
        $this->rc = $this->addField('RC',  '', true, 15);
        $this->osobniCislo = $this->addField('OsobniCislo', '', false, 15);
        $this->datumNastupu = $this->addField('DatumNastupu', 'date', false);
        $this->datumUkonceni = $this->addField('DatumUkonceni', 'date', false);

        $this->duvodVzniku = $this->addField('DuvodVznikuPP', '', false, 10);
        $this->duvodUkonceni = $this->addField('DuvodUkonceniPP', '', false, 10);
        $this->poradoveCisloPP = $this->addField('PoradoveCisloPP', 'int', false);
        $this->zamestnanec = $this->addField('Zamestnanec', 'int', false);
        $this->typPP = $this->addField('TypPracovnihoPomeru', '', false, 10);
        $this->zpracovatDochazku = $this->addField('ZpracovatDochazku', 'int', false);

        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelskaPole[$i] = $this->addField("uzivatelska$i", '', false, 100);
        }

        $this->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $this->addField('Insert_Timestamp', 'timestamp', true, 0, DateUtils::now());

        $this->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
            ->updateLog = true;
        $this->addField('Update_Timestamp', 'timestamp', false, 0, DateUtils::now())
            ->updateLog = true;
    }

    /**
     * @param string $id ID pracovniho pomeru
     * @return bool
     * @throws DBException
     */
    public function nactiPracovniPomer($id)
    {
        $params = $this->database->createParameters();
        $this->setWhere("ID = " . $params->addParameter($id), $params);

        return parent::loadRecord();
    }

    /**
     * @throws Exception
     */
    public function nactiPOST()
    {
        if (isset($_POST['idPP']))
            $this->idPP->setNewValue($_POST['idPP']);
        $this->osobniCislo->setNewValue($_POST['OsobniCislo']);
        if ($_POST['DatumNastupu'] != '')
            $this->datumNastupu->setNewValue(DateUtils::dateStringToTimestamp($_POST['DatumNastupu']));
        if ($_POST['DatumUkonceni'] != '')
            $this->datumUkonceni->setNewValue(DateUtils::dateStringToTimestamp($_POST['DatumUkonceni']));

        $this->duvodVzniku->setNewValue($_POST['DuvodVzniku']);
        $this->duvodUkonceni->setNewValue($_POST['DuvodUkonceni']);
        $this->poradoveCisloPP->setNewValue($_POST['PoradoveCisloPP']);
        $this->zamestnanec->setNewValue($_POST['TypOsoby']);
        $this->typPP->setNewValue($_POST['TypPracovnihoPomeru']);
        $this->zpracovatDochazku->setNewValue(($_POST['ZpracovatDochazku'] == 'on'));

        for ($i = 1; $i <= 20; $i++) {
            if (isset($_POST["pracovnipomer_uzivatelska{$i}_pocet_poli"])) {
                $pomStr = '';
                for ($n = 1; $n <= $_POST["pracovnipomer_uzivatelska{$i}_pocet_poli"]; $n++) {
                    $pomStr .= $_POST["pracovnipomer_uzivatelska{$i}_{$n}"];
                }
            }
            else {
                $pomStr = $_POST["pracovnipomer_uzivatelska$i"];
            }

            $this->uzivatelskaPole[$i]->setNewValue($pomStr);
        }
    }

    /**
     * @param string $rc RC osoby
     * @throws DBException
     */
    function insertPracovniPomer($rc)
    {
        $this->rc->setValue($rc);
        $this->idPP->setValue(SQLUtils::newSerialNumber($this->database, 'PracovniPomer', 'ID'));
        parent::insertRecord();
    }


}