<?php
/**
 * stanislav.elbl, 23.03.2021
 */

namespace Ron;

class CalendarItem
{
    var $dateTimeFrom;
    var $dateTimeTo;

    var $wholeDay;

    var $uid;

    var $summary;
    var $description;

    var $cancelled;

    var $busyStatus = null;


    const eol = "\r\n";

    function __construct()
    {

    }


    function dateTimeToCal($timestamp)
    {
        return date('Ymd\THis', $timestamp);
    }

    function dateToCal($timestamp)
    {
        return date('Ymd', $timestamp);
    }


    function convertText($text)
    {
        $pomText = str_replace("\r\n", '\n', $text);
        $pomText = str_replace("\n", '\n', $pomText);

        return htmlspecialchars(StringUtils::toUTF8($pomText));
    }

    function splitLines($text)
    {
        $result = '';

        $linecount = 0;
        while (strlen($text) > 0) {
            $linewidth = ($linecount == 0 ? 75 : 74);
            $linesize = (strlen($text) > $linewidth ? $linewidth : strlen($text));
            if($linecount > 0)
                $result .= " ";
            $result .= substr($text,0, $linesize) . self::eol;
            $linecount += 1;
            $text = substr($text, $linewidth);
        }

        return $result;
    }

    function getICalEvent()
    {
        $result =
            'BEGIN:VEVENT' . self::eol;

        if ($this->uid)
            $result .=
                "UID:{$this -> uid}" . self::eol;

        //18.10.2019 doplneno dle RFC 5545 https://tools.ietf.org/html/rfc5545#section-3.8.7.2
        $result .=
            "DTSTAMP:" . $this->dateTimeToCal(time()) . self::eol;

        if ($this->dateTimeFrom)
            $result .=
                $this->wholeDay
                    ? 'DTSTART;VALUE=DATE:' . $this->dateToCal($this->dateTimeFrom) . self::eol
                    : 'DTSTART:' . $this->dateTimeToCal($this->dateTimeFrom) . self::eol;

        if ($this->dateTimeTo  &&  $this ->dateTimeTo != $this ->dateTimeFrom)
            $result .=
                $this->wholeDay
                    ? 'DTEND;VALUE=DATE:' . $this->dateToCal(DateUtils::incDay($this->dateTimeTo, 1)) . self::eol
                    : "DTEND:" . $this->dateTimeToCal($this->dateTimeTo) . self::eol;

        if ($this->summary)
            $result .=
                $this->splitLines("SUMMARY:" . $this->convertText($this->summary));

        if ($this->description)
            $result .=
                $this->splitLines("DESCRIPTION:" . $this->convertText($this->description));

        if ($this ->cancelled)
            $result .=
                'METHOD:REQUEST' . self::eol .
                'STATUS:CANCELLED' . self::eol .
                'SEQUENCE:' . (time() - 1616541771) . self::eol;

        if ($this ->busyStatus  &&  !$this ->cancelled) {
            $result .=
                'X-MICROSOFT-CDO-BUSYSTATUS:' . $this ->busyStatus . self::eol;
        }

        /*
         "URL;VALUE=URI:" . htmlspecialchars($url) . $eol .
         CREATED:20140123T185745Z
         LAST-MODIFIED:20140520T211724Z
         LOCATION:Perkin Room
         SEQUENCE:0
         STATUS:CONFIRMED
         SUMMARY:Mary Blair \\ AMNH
         TRANSP:OPAQUE

        */

        $result .=
            "END:VEVENT" . self::eol;

        return $result;
    }


    static function iCalHeader()
    {
//        if (!$filename) $filename = 'calendar.ics';
//
//        SendHeaderForContent($filename, 0);

        //header("Content-Type: text/x-vCalendar");   -- TOTO JE OK
        //header("Content-Disposition: attachment; filename=calendar.vcs");

        //header('Content-type: text/calendar; charset=utf-8');
        //header('Content-Disposition: inline; filename='.$filename);
        //header('Content-type: text/x-vCalendar; charset=utf-8');
        //header('Content-Disposition: attachment; filename='.$filename);

        return
            "BEGIN:VCALENDAR" . self::eol .
            "VERSION:2.0" . self::eol .
            "PRODID:-//RON Software//ronportal//EN" . self::eol .
            "VERSION:2.0" . self::eol .
            "CALSCALE:GREGORIAN" . self::eol;
    }


    static function iCalFooter()
    {
        return
            "END:VCALENDAR" . self::eol;
    }


    function iCalEvent()
    {
        echo $this->iCalHeader();
        echo $this->getICalEvent();
        echo $this->iCalFooter();
    }


    /**
     * @param CalendarItem|CalendarItem[] $items
     * @return string
     */
    public static function getICalFileContent($items)
    {
        $result = self::iCalHeader();

        if (is_array($items)) {
            /** @var CalendarItem $item */
            foreach($items as $item) {
                $result .= $item ->getICalEvent();
            }
        } else {
            $result .= $items ->getICalEvent();
        }

        $result .= self::iCalFooter();
        return $result;
    }
}
