<?php

/**
 * Funkce pro praci s datem a casem
 */

namespace Ron;


use Exception;


/**
 * Trida obsahuje funkce pro praci s datumem a casem
 */
class DateUtils
{
    /**
     * Vraci aktualni cas
     * @return int
     */
    public static function now(): int
    {
        return time();
    }

    /**
     * Vraci aktualni datum (s casem 0:00:00)
     * @return int
     */
    public static function today(): int
    {
        return self::startOfTheDay(time());
    }


    /**
     * Pricte zadany pocet minut.
     * @param int $dateTime
     * @param int $count
     * @return int
     */
    public static function incMinute($dateTime, $count)
    {
        return $dateTime + $count * 60;
    }


    /**
     * Vraci timestamp zacatku dne pro zadany timestamp
     * @param int $date
     * @return int
     */
    public static function startOfTheDay($date): int
    {
        if ($date < 0) {
            $date = 0;
        }

        return mktime(0, 0, 0, date('n', $date), date('j', $date), date('Y', $date));
    }


    /**
     * Vraci timestamp konce dne pro zadany cas (tj. timestamp se stejnym datumem a casem 23:59:59)
     * @param int $date
     * @return int
     */
    public static function endOfTheDay($date):int
    {
        if (!($date > 0)) {
            $date = 0;
        }

        return mktime(23, 59, 59, date('n', $date), date('j', $date), date('Y', $date));
    }


    /**
     * Vraci cas (pocet sekund od zacatku dne) pro zadany timestamp.
     * @param int $date
     * @return int
     */
    public static function timeOf($date):int
    {
        if ($date >= 0  &&  $date < 86400) {
            return $date ?? 0;
        }

        return $date - self::startOfTheDay($date);
    }


    /**
     * Vraci timestamp pro zadany datum a cas ve dni
     * @param int $date Datum - timestamp
     * @param int $time Cas - pocet sekund od zacatku dne
     * @return int
     */
    public static function dateAndTimeToTimestamp($date, $time)
    {
        // TODO: vyresit prechodny den!
        return $date + $time;
    }

    /**
     * Vraci pocet dnu mezi zadanymi datumy - pracuje ale jen s datumem, cas ignoruje.
     * @param int $date1
     * @param int $date2
     * @return int
     */
    public static function daysBetween($date1, $date2): int
    {
        return (int)round((self::startOfTheDay($date2) - self::startOfTheDay($date1)) / 86400);
    }


    /**
     * Zvysi nebo snizi zadane datum o zadany pocet dnu
     * @param int $date
     * @param int $daysCount
     * @return int
     */
    public static function incDay($date, $daysCount = 1): int
    {
        if ($date >= 0  &&  $date <= 86400) {
            // finta. z nejakeho duvodu to nefunguje spravne pro nektere casy. bylo by dobre osetrit. zatim to resim jen pro casy 0-86400
            return $date + 86400;
        }

        $pomDate = self::startOfTheDay($date);
        $pomTime = self::timeOf($date);

        if ($daysCount > 0) {
            $pomDate = self::startOfTheDay($pomDate + $daysCount * 86400 + 3600);
        } else if ($daysCount < 0) {
            $pomDate = self::startOfTheDay($pomDate + $daysCount * 86400);
        }

        return $pomDate + $pomTime;
    }


    /**
     * Zvysi nebo snizi zadane datum - rychlejsi verze, ktera pracuje jen se zacatky dnu!
     * @param int $date
     * @param int $daysCount
     * @return int
     */
    public static function incDayStart($date, $daysCount = 1): int
    {
        return self::startOfTheDay($date + $daysCount * 86400 + 3600); // hodina navic kvuli zmene casu
    }


    /**
     * Zvysi nebo snizi zadane datum o zadany pocet mesicu
     * @param int $date
     * @param int $monthsCount
     * @return int
     */
    public static function incMonth($date, $monthsCount = 1): int
    {
        $pomTime = self::timeOf($date);

        $day = date('j', $date);
        $month = date('n', $date);
        $year = date('Y', $date);

        $year = $year + intval($monthsCount / 12);

        $monthsCount = $monthsCount % 12;
        $month = $month + $monthsCount;

        if ($month > 12) {
            $month = $month - 12;
            $year++;
        } else if ($month < 1) {
            $month = $month + 12;
            $year--;
        }

        return mktime(0, 0, 0, $month, $day, $year) + $pomTime;
    }


    /**
     * Vraci timestamp zacatku tydne (pondeli), ve kterem je zadane datum
     * @param int $date
     * @return int
     */
    public static function firstDayOfWeek($date): int
    {
        if (!($date > 0)) {
            $date = 0;
        }
        $date = self::startOfTheDay($date);

        $day = date('w', $date); // 0 - nedele, 6 - sobota
        $date = $day
            ? self::incDay($date, 1 - $day)
            : self::incDay($date, -6);

        return self::startOfTheDay($date);
    }


    /**
     * Vraci timestamp zacatku mesice, ve kterem je zadane datum
     * @param int $date
     * @return int
     */
    public static function firstDayOfMonth($date): int
    {
        if (!($date > 0)) {
            $date = 0;
        }

        $month = date('n', $date);
        $year = date('Y', $date);
        return mktime(0, 0, 0, $month, 1, $year);
    }


    /**
     * Vraci timestamp zacatku posledniho dne v mesici, ve kterem se nachazi zadane datum
     * @param int $date
     * @return int
     */
    public static function lastDayOfMonth($date)
    {
        if (!($date > 0)) {
            $date = 0;
        }

        $month = date('n', $date);
        $year = date('Y', $date);

        return self::incDayStart(mktime(0, 0, 0, $month + 1, 1, $year), -1);
//        return self::incDay(mktime(0, 0, 0, $month + 1, 1, $year), -1); - asi nefunguje spravne!

        //26.3.2019 zmenen zpusob zjistovani (puvodni nefungoval pri prechodu na letni cas)
//        $pomDate = mktime(0, 0, 0, $month + 1, 1, $year) - 3600;  - tato verze u zakaznika nefungovala spravne
//        return self::startOfTheDay($pomDate);
    }


    /**
     * Vraci timestamp zacatku prvniho dne roku, ve kterem je zadane datum
     * @param int $date
     * @return int
     */
    public static function firstDayOfYear($date)
    {
        if (!($date > 0)) {
            $date = 0;
        }

        $year = date('Y', $date);
        return mktime(0, 0, 0, 1, 1, $year);
    }


    /**
     * Vraci timestamp zacatku posledniho dne roku, ve kterem je zadane datum
     * @param int $date
     * @return int
     */
    public static function lastDayOfYear($date)
    {
        if (!($date > 0)) {
            $date = 0;
        }

        $year = date('Y', $date);
        return mktime(0, 0, 0, 12, 31, $year);
    }


    /**
     * Vraci datum zacatku prvniho tydne pro rok se zadanym datumem.
     * @param int $date
     * @return int
     */
    public static function firstWeekOfYear($date): int
    {
        $yearStart = self::firstDayOfYear($date);
        $day = date('N', $yearStart);

        $add = ($day <= 4)
            ? 1 - $day  // rok zacina pondelkem az ctvrtkem => prvni tyden v roce zacina 1.1. nebo v predchozim roce
            : 8 - $day; // rok zacina ctvrtkem az sobotou => prvni tyden v roce zacina pozdeji nez 1.1.

        return self::incDayStart($yearStart, $add);
    }

    /**
     * Vraci timestamp prvniho dne ctvrtleti, ve kterem se nachazi zadane datum
     * @param int $date
     * @return int
     */
    public static function firstDayOfQuarter($date): int
    {
        if (!($date > 0)) $date = 0;

        $month = (int)((date('n', $date) - 1) / 3) * 3 + 1;
        $year = date('Y', $date);
        return mktime(0, 0, 0, $month, 1, $year);
    }


    /**
     * Vraci timestamp prvniho dne pololeti, ve kterem se nachazi zadane datum
     * @param int $date
     * @return int
     */
    public static function firstDayOfHalfYear($date)
    {
        if (!($date > 0)) $date = 0;

        $month = (int)((date('n', $date) - 1) / 6) * 6 + 1;
        $year = date('Y', $date);
        return mktime(0, 0, 0, $month, 1, $year);
    }


    /**
     * Prevede zadany retezec na timestamp. Format retezce muze byt:
     *      1) den.mesic.       (doplni aktualni rok)
     *      2) den.mesic.rok
     *      3) ddmmyyyy
     *      Pokud je datum spatne, vyhodi vyjimku
     * @param string $dateStr
     * @param int|string $hourOrFieldName   pokud je zadan string a neni zadan zadny dalsi parametr, tak jej pouzije misto $fieldName
     * @param int $minute
     * @param int $second
     * @param string $fieldName             jmeno pole se pouzije pri generovani vyjimky
     * @return int
     * @throws Exception
     */
    public static function dateStringToTimestamp($dateStr, $hourOrFieldName = null, $minute = null, $second = null, $fieldName = null)
    {
        if (!isset($fieldName)) {
            if (!isset($minute) && !isset($second) && is_string($hourOrFieldName)) {
                $fieldName = $hourOrFieldName;
                $hourOrFieldName = 0;
            }
        }

        $hourOrFieldName = (int)$hourOrFieldName;
        $minute = (int)$minute;
        $second = (int)$second;

        if (preg_match('/^(\d+)\.(\d+)$/', $dateStr, $m) || preg_match('/^(\d+)\.(\d+).$/', $dateStr, $m)) {
            // kraty format bez roku d.m nebo d.m.
            $day = (int)$m[1]; // index 0 je puvodni string!
            $month = (int)$m[2];
            $year = (int)date('Y');
        } else if (preg_match('/^(\d+)\.(\d+)\.(\d+)$/', $dateStr, $m)) {
            // d.m.y
            $day = (int)$m[1];
            $month = (int)$m[2];
            $year = (int)$m[3];
        } else if (StringUtils::isInteger($dateStr) && strlen($dateStr) == 8) {
            // jen ciselny format ddmmyyyy
            $day = (int)substr($dateStr, 0, 2);
            $month = (int)substr($dateStr, 2, 2);
            $year = (int)substr($dateStr, 4, 4);
        } else {
            // spatny format
            throw new RonException( ronText('Bad_date_format'), $fieldName );
        }

        if ($day <= 0 || $month <= 0) {
            // neni zadan den nebo mesic
            throw new RonException( ronText('Bad_date_format'), $fieldName );
        }
        if ($day > 31 || $month > 12) {
            // spatne zadan den nebo mesic
            throw new RonException( ronText('Bad_date_format'), $fieldName );
        }

        if (!$year) {
            throw new RonException( ronText('Bad_date_format'), $fieldName );
        }

        // vypocte vysledek
        $result = mktime($hourOrFieldName, $minute, $second, $month, $day, $year);

        // dodatecne zkontroluje, jesli byly zadane spravne hodnoty,
        // protoze PHP ma tak uzasnou funkci mktime, ze se tam daji zadat totalni blaboly a ona vzdycky neco vymysli.
        if (date('d', $result) != $day) {
            throw new RonException( ronText('Bad_date_format'), $fieldName );
        }
        if (date('m', $result) != $month) {
            throw new RonException( ronText('Bad_date_format'), $fieldName );
        }

        return $result;
    }


    /**
     * Prevede zadany retezec na timestamp. Format retezce muze byt:
     *   1) hodina
     *   2) hodina:minuta (do sekund doplni 0)
     *   3) hodina:minuta:sekunda
     *   Pokud je cas spatny, vyhodi vyjimku
     *   V pripade, ze je zadan datum ($date), tak pouzije cas podle zadaneho datumu, jinak vraci pocet sekund
     *   POZOR! Pokud neni zadan datum, tak je potreba davat pozor na prechodny den a letni cas!
     *   Naopak, pokud ma funkce vracet "dobu", tak musi byt $date = 0!
     * @param string $timeStr
     * @param int    $date
     * @param bool   $enable_24_00 Urcuje, jestli je povoleno zadat 24:00 - pak vraci 0:00 nasledujiciho dne
     * @return int
     * @throws Exception
     */
    public static function timeStringToTimestamp($timeStr, $date = 0, $enable_24_00 = false)
    {
        if (!isset($timeStr)) {
            throw new Exception( ronText('Bad_time_format') );
        }

        // jen hodina 0 - 23/24, nebo hhmm (bez dvojtecky, napr. 830)
        if (preg_match('/^([0-9]+)$/', $timeStr, $m)) {
            if ($m[1] < 24 || ($m[1] == 24 && $enable_24_00)) {
                $hour = $m[1];
                $min = 0;
            } else if ($m[1] >= 100) {
                $hour = substr($m[1], 0, -2);
                $min = (int)substr($m[1], -2);  // posledni 2 znaky jsou minuty

                if ($hour > 24 || ($hour == 24 && (!$enable_24_00 || $min)) || $min > 59) {
                    throw new Exception( ronText('Bad_time_format') );
                }
            } else {
                throw new Exception( ronText('Bad_time_format') );
            }

            if ($date) {
                if ($hour == 24) {
                    $date = self::incDay($date, 1);

                    return mktime(0, 0, 0, date('n', $date), date('j', $date), date('Y', $date));
                }

                return mktime($hour, $min, 0, date('n', $date), date('j', $date), date('Y', $date));
            }

            return 3600 * $hour + 60 * $min;
        }

        // hodina:minuta 0:0 - 23:59 / 24:00
        if (preg_match('/^(\d+):(\d+)$/', $timeStr, $m) && ($m[1] <= 24) && ($m[2] < 60)) {
            if ($m[1] == 24) {
                // 24:00 nasledujiciho dne
                if (!$enable_24_00 || (int)$m[2]) {
                    throw new Exception( ronText('Bad_time_format') );
                }

                if ($date) {
                    $date = self::incDay($date, 1);

                    return mktime(0, 0, 0, date('n', $date), date('j', $date), date('Y', $date));
                }
            }

            if ($date) return mktime($m[1], $m[2], 0, date('n', $date), date('j', $date), date('Y', $date));

            return 3600 * $m[1] + 60 * $m[2];
        }

        // hodina:minuta 0:0:0 - 23:59:59 / 24:00:00
        if (preg_match('/^(\d+):(\d+):(\d+)$/', $timeStr, $m) && ($m[1] <= 24) && ($m[2] < 60) && ($m[3] < 60)) {
            if ($m[1] == 24) {
                if (!$enable_24_00 || (int)$m[2] || (int)$m[3]) {
                    throw new Exception( ronText('Bad_time_format') );
                }
                if ($date) {
                    // 24:00 nasledujiciho dne
                    $date = self::incDay($date, 1);

                    return mktime(0, 0, 0, date('n', $date), date('j', $date), date('Y', $date));
                }
            }

            if ($date) return mktime($m[1], $m[2], $m[3], date('n', $date), date('j', $date), date('Y', $date));

            return 3600 * $m[1] + 60 * $m[2] + $m[3];
        }

        throw new Exception( ronText('Bad_time_format') );
    }


    /**
     * Prevede zadany retezec na timestamp. Format retezce muze byt:
     *   1) datum   - doplni cas nulama
     *   2) datum cas   (nesmi byt vice mezer!!!!)
     *   Pokud je datum spatne, vyhodi vyjimku
     * @param string $dateStr
     * @return int
     * @throws Exception
     */
    public static function datetimeStringToTimestamp($dateStr)
    {
        if (!$dateStr) return 0;
        $pomarray = explode(' ', $dateStr);

        if (isset($pomarray[1])) {
            // pokud je zadany cas, tak nacte hodiny, minuty, sekundy

            // jen hodina 0 - 23/24, nebo hhmm (bez dvojtecky, napr. 830)
            if (StringUtils::isInteger($pomarray[1])) {
                $hour = (int)$pomarray[1];
                $minute = 0;
                $sec = 0;
                if ($hour >= 100) {
                    $hour = (int)substr($pomarray[1], 0, -2);
                    $minute = (int)substr($pomarray[1], -2);  // posledni 2 znaky jsou minuty
                }
            } else {
                $timearray = explode(':', $pomarray[1]);
                $hour = isset($timearray[0]) ? (int)$timearray[0] : 0;
                $minute = isset($timearray[1]) ? (int)$timearray[1] : 0;
                $sec = isset($timearray[2]) ? (int)$timearray[2] : 0;
            }

            if ($hour >= 24 || $minute >= 60 || $sec >= 60) {
                throw new Exception( ronText('Bad_time_format') );
            }
        } else {
            $hour = 0;
            $minute = 0;
            $sec = 0;
        }

        return self::dateStringToTimestamp($pomarray[0], $hour, $minute, $sec);
    }


    /**
     * @param string $timestr
     * @param bool $povolitZkraceny Pokud je true, tak povoli zkraceny zapis HHMM bez dvojtecky -
     *                               pokud ma text 3 a vice cislic, tak jsou posledni 2 cislice minuty, zbytek hodiny
     *                               Pozor! v tom pripade nelze zadat napriklad 130 hodin, protoze z toho udela 1:30 (musi se zapsat 130:00 nebo 13000)
     * @return float
     * @throws Exception
     */
    public static function timeStringToFloat($timestr, $povolitZkraceny = false)
    {
        if (!isset($timestr)) {
            // chyba
            throw new Exception( ronText('Bad_time_format') );
        }

        if (preg_match('/^(-)?(\d+)$/', $timestr, $pomarray))  // hodiny nebo hodinyminuty (bez dvojtecky)
        {
            $minus = $pomarray[1] == '-';
            $hour = $pomarray[2];

            $minute = 0;
            $sec = 0;

            if ($povolitZkraceny && strlen($hour) > 2) {
                // zkraceny zapis ve formatu hhmm (bez dvojtecky)
                $minute = (int)substr($hour, -2);  // posledni 2 znaky jsou minuty
                $hour = (int)substr($hour, 0, -2);
            }
        } else {
            if (!preg_match('/^(-)?(\d+):(\d+)$/', $timestr, $pomarray))  // hodiny:minuty
                if (!preg_match('/^(-)?(\d+):(\d+):(\d+)$/', $timestr, $pomarray)) { // hodiny:minuty:sekundy
                    throw new Exception( ronText('Bad_time_format') ); // chyba
                }

            $minus = $pomarray[1] == '-';
            $hour = (int)$pomarray[2];
            $minute = (int)$pomarray[3];
            $sec = isset($pomarray[4]) ? (int)$pomarray[4] : 0;
        }

        if ($minute >= 60 || $sec >= 60) {
            throw new Exception( ronText('Bad_time_format') );
        }

        $result = $hour + $minute / 60.0 + $sec / 3600.0;
        if ($minus) $result = -$result;

        return $result;
    }


    /**
     * Prevede pocet minut (cislo) na retezec. Napr. 30 => "0:30"
     * @param int $data
     * @return string
     */
    public static function minutesToString($data)
    {
        $pomstr = ($data < 0) ? '-' : '';
        $data = round(abs($data));

        $minut = $data % 60;
        $pomstr = $pomstr . floor($data / 60) . ':';
        if ($minut < 10) {
            $pomstr = $pomstr . '0';
        }
        $pomstr = $pomstr . $minut;

        return $pomstr;
    }


    /**
     * Prevede pocet sekund (int) na retezec ve formatu h:m:s
     * @param int $data
     * @param bool $includeSeconds
     * @return string
     */
    public static function secondsToString($data, $includeSeconds = true)
    {
        $sign = ($data < 0) ? '-' : '';
        $data = abs($data ?? 0);

        $sec = $data % 60;
        $data = floor($data / 60);
        $min = $data % 60;
        $data = floor($data / 60);

        return
            $sign .
            $data . ':' . ($min < 10 ? '0' : '') . $min .
            ($includeSeconds ? ':' . ($sec < 10 ? '0' : '') . $sec : '');
    }


    /**
     * Prevede zadany text na pocet sekund. Text muze byt ve formatu:
     *  s      - jen sekundy (int)
     *  m:s    - minuty:sekundy
     *  h:m:s  - hodiny:minuty:sekundy
     * @param $text
     * @return int
     * @throws Exception
     */
    public static function stringToSeconds($text)
    {
        $text = trim($text);

        if ($text == '') return null;


        if (preg_match('/^(\d+)$/', $text, $m)) {
            // jen sekundy
            return (int)$text;
        }

        if (preg_match('/^(\d+):(\d+)$/', $text, $m)) {
            // min:sec
            $min = $m[1]; // index 0 je puvodni string!
            $sec = $m[2];

            if ((int)$sec >= 60) return false; // spatny format - pocet sekund nemuze byt > 60

            return (int)$min * 60 + $sec;
        } else if (preg_match('/^(\d+).(\d+).(\d+)$/', $text, $m)) {
            // h:m:s
            $hours = $m[1]; // index 0 je puvodni string!
            $min = $m[2];
            $sec = $m[3];

            if ($sec >= 60  || $min >= 60) return false;
            return (int)$hours * 3600 + $min * 60 + $sec;
        } else {
            // spatny format
            throw new Exception( ronText('Bad_time_format') );
        }
    }


    /**
     * Vraci lepe citelny formatovany datum. Pokud je $showtime true, tak vraci i cas
     * @param int $date
     * @param bool $showTime
     * @return string
     */
    public static function fineDateString($date, $showTime)
    {
        if (self::startOfTheDay($date) == self::startOfTheDay(time())) {
            $result = ronText('Today');
        } else if (self::incDayStart($date, 1) == self::startOfTheDay(time())) {
            $result = ronText('Yesterday');
        } else if (self::incDayStart(time(), 1) == self::startOfTheDay($date)) {
            $result = ronText('Tomorrow');
        } else {
            $result = date('j.n.Y', $date);
        }

        if ($showTime) {
            $result .= date(' G:i', $date);
        }

        return $result;
    }


    /**
     * Vraci jmeno nebo zkratku dne (pondeli/po, utery/ut, ...)
     * @param int  $date
     * @param bool $short  Pokud je true, vraci zkratku
     * @return string
     */
    public static function dayName($date, $short = false)
    {
        return ronText( ($short ? 'Day_short_' : 'Day_') . date('w', $date));
    }


    /**
     * Vraci popis ctvrtleti pro zadany datum
     * @param int $date
     * @return string
     */
    public static function quarterDescription($date)
    {
        switch(date('n', $date)) {
            case 1 :
            case 2 :
            case 3 :
                $indexStrCtvrleti = 'I';
                break;

            case 4 :
            case 5 :
            case 6 :
                $indexStrCtvrleti = 'II';
                break;

            case 7 :
            case 8 :
            case 9 :
                $indexStrCtvrleti = 'III';
                break;

            case 10 :
            case 11 :
            case 12 :
                $indexStrCtvrleti = 'IV';
                break;

            default :
                $indexStrCtvrleti = '';
        }

        return sprintf(\Ron\ronText('Popis_ctvrtleti'), date('Y', $date), $indexStrCtvrleti);
    }

    /**
     * Zaokrouhli zadany cas na zadany pocet minut
     * @param int $datetime
     * @param int $minutes
     * @param int $direction  0 = normalne, <0 = dolu, >0 = nahoru
     * @return int
     */
    public static function roundTime($datetime, $minutes, $direction = 0)
    {
        if (!$minutes) return $datetime;

        $date = ($datetime >= 0  &&  $datetime <= 86400) ? 0 : self::startOfTheDay($datetime);
        $time = $datetime - $date;

        if ($direction > 0) {
            $time = 60 * $minutes * ceil($time / $minutes / 60);
        } else if ($direction < 0) {
            $time = 60 * $minutes * floor($time / $minutes / 60);
        } else {
            $time = 60 * $minutes * round($time / $minutes / 60);
        }

        return $date + $time;
    }


    /**
     * Prevede zadany timestamp na datum pro pouziti v URL
     * @param int $timestamp
     * @return string
     */
    public static function dateToUrlString($timestamp)
    {
        return date('j.n.Y', $timestamp);
    }


    /**
     * Prevede datumovy retezec na timestamp
     * @param $str
     * @param bool $enableTimestamp Urcuje, jestli muze byt misto retezce s datumem zadan primo timestamp
     * @return int
     * @throws Exception
     */
    public static function urlStringToDate($str, $enableTimestamp = false)
    {
        if ($enableTimestamp  &&  StringUtils::isInteger($str)) {
            return (int)$str;
        }

        return self::dateStringToTimestamp($str);
    }


}
