<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;


/**
 * Parametry se zastupnym znakem - do SQL dotazu se vklada znak "?" (pripadne jiny zastupny znak)
 */
class PlaceholderParameters extends Parameters
{
    var $data = [];

    public function addParameter($value)
    {
        if ($value instanceof BlobParameter) {
            $this ->data[] = $value ->data;
            return $value ->placeholder;
        }

        $this ->data[] = $value;
        return '?';
    }

    public function getSqlParameters()
    {
        return $this ->data;
    }


    /**
     * @param PlaceholderParameters $params
     * @return $this
     */
    public function appendParameters($params)
    {
        if ($params) {
            foreach ($params ->data as $param) {
                $this ->data[] = $param;
            }
        }
        return $this;
    }


    public function newInstance()
    {
        return new self($this ->database);
    }


    public function getLogText()
    {
        $result = '';
        foreach ($this ->data as $value) {
            $result .= $result ? ', ' : '';
            if (!isset($value)) {
                $result .= '(NULL)';
            } else {
                if ($value instanceof BlobParameter) {
                    $result .= '(BLOB)';
                } else if ($value instanceof \DateTime) {
                    $result .= $value ->format('d.m.Y H:i:s');
                } else {
                    $result .= $value;
                }
            }
        }

        return $result;
    }
}
