<?php
/**
 * UTF8
 */

namespace Ron;

use Ron\Http\HtmlHeaders;

class GuiUtils
{
    /**
     * Vraci string s HTML atributy ze zadaneho pole (napr. z ['class' => 'row', 'id' => 'moje_id']
     * @param string[]|string $attributes   Asociativni pole atributu, nebo primo text
     * @param bool $htmlConvert             Urcuje, jestli se maji konvertovat specialni znaky. Pouzije se jen v pripade ze je v $attributes zadano pole.
     * @return string
     */
    static function getAttributesHtml($attributes, $htmlConvert = true)
    {
        if (is_array($attributes)) {
            $result = '';
            foreach ($attributes as $attrName => $attrValue) {
                if ($attrName  &&  isset($attrValue)) { // 21.9.2023 - pokud je hodnota null, tak se atribut negeneruje
                    $result .=
                        ($result ? ' ' : '') .
                        $attrName . '="' . ($htmlConvert ? StringUtils::htmlTextConvert($attrValue) : $attrValue) . '"';
                }
            }
            return $result;
        } else if (is_string($attributes)) {
            return $attributes;
        }

        return '';
    }


    /**
     * Zobrazi checkbox. Pokud je zadan text, tak renderuje i <label>
     * @param string $varName jmeno html inputu
     * @param string $text zobrazeny nadpis
     * @param bool $checked
     * @param string[]|string $params   Asociativni pole parametru nebo string s parametry
     * @param string $className         css class
     * @param int $value
     * @param bool $textFirst
     */
    static function renderCheckbox($varName, $text, $checked, $params = [], $className = null, $value = 1, $textFirst = false )
    {
        if ($text) {
            echo '<label>';
        }

        //$className = $className ?: 'app_check';

        //$valueAttr = isset($value) ? "value='$value'" : '';

        if ($text  &&  $textFirst) {
            echo $checked
                ? "<span class='checkbox_text_checked'>$text</span>"
                : "<span class='checkbox_text_unchecked'>$text</span>";
        }

        $defaultAttrs = [
            'type' => 'checkbox',
            'name' => $varName
        ];

        if ($checked) {
            $defaultAttrs['checked'] = 'checked';
        }

        if (isset($value)) {
            $defaultAttrs['value'] = $value;
        }

        if (isset($className)) {
            $defaultAttrs['class'] = $className;
        }

        if (is_array($params)) {
//            if (isset($className)) {
//                $params['class'] = $className;
//            }

            $paramsStr = '';
//            foreach ($params as $pName => $pValue) {
//                $paramsStr .=
//                    ' ' .
//                    $pName . '="' . StringUtils::htmlTextConvert($pValue) . '"';
//            }

        } else {
            $paramsStr = ($params ?: '');
            $params = [];
            //    ($className ? " class='$className'" : '');
        }

        echo
            "<input " .
            GuiUtils::getAttributesHtml(array_merge($defaultAttrs, $params)) .
            $paramsStr .
            '>';

        if ($text  &&  !$textFirst) {
            echo $checked
                ? "<span class='checkbox_text_checked'>$text</span>"
                : "<span class='checkbox_text_unchecked'>$text</span>";
        }

        if ($text) {
            echo '</label>';
        }
        echo "\n";
    }


    /**
     * @param array $data
     * @param mixed $selectedValue
     */
    public static function renderOptions($data, $selectedValue = null)
    {
        foreach ($data as $key => $text) {
            echo
                "<option value='$key'" .
                ($selectedValue === $key ? " selected='selected'" : '') .
                '>' .
                StringUtils::htmlTextConvert($text) .
                "</option>\n";
        }
    }

    /**
     * Vraci url parametry pro nastaveni polohy (&x=...&y=...)
     * @return string
     */
    static function scrollPageUrlParams()
    {
        $result = '';

        $polohaY = null;
        if (isset($_POST['y'])) $polohaY = (int)$_POST['y'];
        else if (isset($_GET['y'])) $polohaY = (int)$_GET['y'];

        $result .= $polohaY ? '&y=' . $polohaY : '';

        $polohaX = null;
        if (isset($_POST['x'])) $polohaX = (int)$_POST['x'];
        else if (isset($_GET['x'])) $polohaX = (int)$_GET['x'];

        $result .= $polohaX ? '&x=' . $polohaX : '';

        return $result;
    }


    public static function scrollPageHeader()
    {
        $polohaX = null;
        if (isset($_POST['x'])) {
            $polohaX = (int)$_POST['x'];
        } else if (isset($_GET['x'])) {
            $polohaX = (int)$_GET['x'];
        }
        if ($polohaX) {
            HtmlHeaders::getInstance() ->addBodyAttribute('data-scroll-to-x', (int)$polohaX);
        }

        $polohaY = null;
        if (isset($_POST['y'])) {
            $polohaY = (int)$_POST['y'];
        } else if (isset($_GET['y'])) {
            $polohaY = (int)$_GET['y'];
        }
        if ($polohaY) {
            HtmlHeaders::getInstance() ->addBodyAttribute('data-scroll-to-y', (int)$polohaY);
        }
    }


    /**
     * Vraci html text pro zobrazeni potvrzovaci hlasky
     * @param string $text
     * @param string $question
     * @param string $url
     * @param string $htmlParams
     * @return string
     */
    static function changeLocationQuest($text, $question, $url, $htmlParams = null)
    {
        return
            '<span class="js confirmQuestion" data-ron-href="'. StringUtils::htmlTextConvert($url) . '" data-ron-confirm-text="' . StringUtils::htmlTextConvert($question) . '" ' . ($htmlParams ?: '') . '>' .
            $text .
            "</span>\n";
    }


    /**
     * Vraci HTML reprezentaci zadane barvy (#rrggbb)
     * @param int $color
     * @param float $darken    Ztmaveni (pokud je mensi nez 1) nebo zesvetleni (pokud je vetsi nez 1)
     * @return string
     */
    static function getHtmlColor($color, $darken = 1.0)
    {
        if ($darken == 1) {
            $result = $color % 256 * 0x10000 + ((int)($color / 0x100) % 0x100) * 0x100 + $color / 0x10000;
        } else {
            $r = (int)($color % 0x100 * $darken);
            $g = (int)(($color / 0x100) % 0x100 * $darken);
            $b = (int)($color / 0x10000 * $darken);

            $result = $r * 0x10000 + $g * 0x100 + $b;
        }

        return '#' . substr('00000' . dechex((int)$result), -6);
    }

}
