<?php
/**
 * UTF8
 * stanislav.elbl, 10.12.2015
 */

namespace Ron;

class HttpSocket
{
    const METHOD_GET = 'GET';
    const METHOD_POST = 'POST';

    const CONTENT_TYPE_XML = 'text/xml';
    const CONTENT_TYPE_FORM = 'application/x-www-form-urlencoded';

    var $host;
    var $port;
    var $sockOpenScheme = '';  // pridava se k sockopen - v pripade https se automaticky pridava ssl://

    var $method;
    var $contentType;

    var $socket;

    // odpoved
    var $responseCode; // napr. 200
    var $responseStatus;       // OK

    /**
     * HttpSocket constructor.
     * @param string $host  jmeno nebo IP hosta, nebo url. Pokud je zadano url, tak $port musi byt 0 a naopak
     * @param int    $port  pokud neni zadan, tak se urci automaticky ze zadane url
     */
    public function __construct($host, $port = null)
    {
        if ($port) {
            $this->host = $host;
            $this->port = $port;
        } else {
            $urlParts = parse_url($host);
            $this ->host = $urlParts['host'];
            $this ->port = 80;

            if (isset($urlParts['scheme'])) {
                switch ($urlParts['scheme']) {
                    case 'https' :
                    case 'ssl' :
                        $this ->sockOpenScheme = 'ssl://';
                        $this ->port = 443;
                        break;
                }
            }

            if (isset($urlParts['port'])) {
                $this ->port = $urlParts['port'];
            }
        }

        $this ->method = 'GET';
        $this ->contentType = self::CONTENT_TYPE_XML . '; charset=utf-8';
    }


    function open()
    {
        $errno = 0;
        $errstr = '';

        $level = error_reporting(E_ERROR); // vypne warningy
        $this->socket = fsockopen($this ->sockOpenScheme . $this->host, $this->port, $errno, $errstr);
        error_reporting($level);

        if (!$this ->socket) {
            throw new \Exception("HttpSocket: Nelze navazat spojeni se serverem! Chyba cislo $errno: $errstr");
        }
    }


    function sendRequest($content, $requestPath = '/')
    {
        if (!$this ->socket) {
            $this ->open();
        }

        $httphead =
            "{$this ->method} {$requestPath} HTTP/1.1\r\n".
            "Host: {$this ->host}:{$this ->port}\r\n".
            "Content-Type: {$this ->contentType}\r\n".
            'Content-Length: ' . strlen($content) . "\r\n".
            "Connection: Close\r\n" .
            "\r\n";

        $result = fwrite($this ->socket, $httphead . $content);
        if ($result === false) {
            throw new \Exception("HttpSocket: chyba pri odeslani dat!");
        }
    }


    function readAnswer()
    {
        $headerEnd = false;
        $header = '';

        while ( !$headerEnd  &&  !feof($this ->socket) )
        {
            $headerRow = fgets($this ->socket);
            $header .= $headerRow;

            if ( stripos($headerRow, 'HTTP/') !== FALSE ) {
                // HTTP/1.1 200 OK
                list(,$code, $status) = explode(' ', $headerRow);

                $this ->responseStatus = $status;
                $this ->responseCode = $code;
            }

            if ( trim($headerRow) == '' ) $headerEnd = true;
        }

        if ( strcasecmp('OK', trim($this ->responseStatus)) ) {
            throw new \Exception("Chyba pri komunikaci se serverem! Server nestihnul odpovedet nebo prisla spatna odpoved: {$this ->responseCode}, {$this ->responseStatus}, hlavicka:\n{$header}");
        }

        // OK - nacte obsah odpovedi
        $answer = '';
        while (!feof($this ->socket)) $answer .= fgets($this ->socket);

        return $answer;
    }


    public static function post($url, $content, $contentType = HttpSocket::CONTENT_TYPE_FORM)
    {
        $urlParts = parse_url($url);
        $requestPath =
            isset($urlParts['path']) ? $urlParts['path'] : '/';

        if (isset($urlParts['query'])) {
            $requestPath .= '?' . $urlParts['query'];
        }


        $httpSocket = new static($url);
        $httpSocket ->method = 'POST';
        $httpSocket ->contentType = $contentType;
        $httpSocket ->open();
        $httpSocket ->sendRequest($content, $requestPath);
        return $httpSocket ->readAnswer();
    }

}
