<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\MVC;


/**
 * Blokovy View object - obsahuje jine (vnorene) ViewObjecty a muze je vlozit do nejakeho HTML elementu (defaultne div)
 */
class BlockViewObject implements ViewObject
{
    /** @var string Obalujici html element (pokud neni zadan, tak se generuji jen vnorene objekty bez obalujiciho elementu) */
    protected $wrappingElement = 'div';

    /** @var string HTML atributy obalujiciho elementu */
    protected $wrappingElementParams = '';


    /** @var ViewObject[]   Seznam vnorenych ViewObjectu */
    private $childViewObjects = [];


    /**
     * Vraci obalujici HTML element ($wrappingElement)
     * @return string
     */
    public function getWrappingElement(): string
    {
        return $this->wrappingElement;
    }

    /**
     * Nastavi obalujici HTML element ($wrappingElement)
     * @param string $wrappingElement
     * @return BlockViewObject
     */
    public function setWrappingElement(string $wrappingElement): BlockViewObject
    {
        $this->wrappingElement = $wrappingElement;
        return $this;
    }

    /**
     * Vraci HTML atributy obalujiciho elementu
     * @return string
     */
    public function getWrappingElementParams(): string
    {
        return $this->wrappingElementParams;
    }

    /**
     * Nastavi HTML atributy obalujiciho elementu
     * @param string $wrappingElementParams
     * @return BlockViewObject
     */
    public function setWrappingElementParams(string $wrappingElementParams): BlockViewObject
    {
        $this->wrappingElementParams = $wrappingElementParams;
        return $this;
    }


    /**
     * Prida zadany $viewObject do seznamu vnorenych ViewObjectu
     * @param ViewObject $viewObject
     * @return BlockViewObject
     */
    public function addViewObject($viewObject): BlockViewObject
    {
        $this ->childViewObjects[] = $viewObject;
        return $this;
    }


    /**
     * Renderuje zacatek obalujiciho HTML elementu $wrappingElement
     * @return void
     */
    public function renderHeader()
    {
        if ($this ->wrappingElement) {
            echo "<{$this ->wrappingElement} {$this ->wrappingElementParams}>";
        }
    }


    /**
     * Renderuje konec obalujiciho HTML elementu $wrappingElement
     * @return void
     */
    public function renderFooter()
    {
        if ($this ->wrappingElement) {
            echo "</{$this ->wrappingElement}>";
        }
    }


    /**
     * Renderuje vnorene HTML elementy
     * @return void
     */
    public function renderChildren()
    {
        foreach ($this ->childViewObjects as $child) {
            $child ->render();
        }
    }


    /**
     * Renderuje obsah (vnorene HTML elementy, pripadne obalene HTML elementem $wrappingElement
     * @return void
     */
    public function render()
    {
        $this ->renderHeader();
        $this ->renderChildren();
        $this ->renderFooter();
    }

}
