<?php

namespace RonPortal\Ankety;

use Ron\Controls\BrowserField;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatum;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;

class AnketaBrowser extends RonPortalBrowser
{
    public function __construct($database)
    {
        parent::__construct($database, 'anketa');

        $this ->sqlFromPart = 'Anketa';
        $this ->title = TextManager::_('Ankety');
        $this ->code = 'browserankety';

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Id')
            ->setCode('col-id');

        $this ->primaryField = 'col-id';

        $this ->addField()
            ->setSqlFieldName('Nazev')
            ->setCode('col-nazev')
            ->setHeaderText('Nazev')
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('PlatnostOd')
            ->setFormat('date')
            ->setCode('col-platnostod')
            ->setHeaderText('Platnost_od')
            ->setOrdering(true);

        $this ->defaultSort = 'col-platnostod';

        $this ->addField()
            ->setSqlFieldName('PlatnostDo')
            ->setFormat('date')
            ->setCode('col-platnostdo')
            ->setHeaderText('Platnost_do')
            ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('Detail'))
            ->setLink('?menuid=anketa&action=zakladni');

        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('Vyhodnoceni_ankety'))
            ->setLink('?menuid=anketa&action=vyhodnoceni');
    }



    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);
        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setPovolitPrazdneDatumDo(true)
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('newrecord', FilterFormFilter::TYPE_TEXT))
                ->setCaption(false)
                ->setUrl('?menuid=anketa&formaction=new')
                ->setDefaultValue(TextManager::_('Novy_radek'))
                ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                ->setRowClass('margin-top-2')
        );

        $this ->filterForm ->addFilter(
            (new FilterFormFilter('backlink', FilterFormFilter::TYPE_TEXT))
                ->setUrl('?menuid=anketa')
                ->setDefaultValue(TextManager::_('Ankety_k_vyplneni'))
                ->setPosition(FilterFormFilter::POSITION_ABOVE_FORM)
                ->setCaption(false)
                ->setRowClass('margin-top-2 margin-bottom-2')
        );
    }



    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        if ($this ->filterForm ->getFilterValue('datumod', 'filtrovatDatum')  &&  $this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                '(PlatnostDo >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod')) . ' OR PlatnostDo IS NULL)' .
                ($this ->filterForm ->getFilterValue('datumod', 'datumDo')
                    ? ' AND (PlatnostOd <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo')) . ' OR PlatnostOd IS NULL)'
                    : ''
                );
        }

        return new SqlStatement($result, $params);
    }


}
