<?php

namespace RonPortal\Ankety;

use Ron\Controls\LinksList;
use Ron\Controls\LinksListItem;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\MVC\Renderer;
use Ron\MVC\TextViewObject;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RozdeleniList;
use RonPortal\Controls\RozdeleniShiftFormView;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\System\RecordNotFoundException;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\Testy\Dotaznik;
use RonPortal\Testy\DotaznikController;
use RonPortal\Testy\PripraveneTemaHodnoceniDBRecord;
use RonPortal\User;

class AnketaController extends DotaznikController
{
    public function __construct(Database $database)
    {
        parent::__construct($database);
        $this ->editovatBody = false;
    }


    /**
     * Overi, jestli ma uzivatel pravo editovat zadany test. Pokud ne, tak vyhodi vyjimku.
     * Metodu musi implementovat potomek (podle agendy)
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @return void
     */
    public function overPravoNaDotaznik(int $druhHodnoceni, int $temaCislo)
    {
        // overi, ze se jedna o elearningovy test
        if ($druhHodnoceni !== 12) {
            throw new NeniPravoException();
        }
    }



    private function hlavickaView($id, $action, $nazev = null)
    {
        Renderer::startRenderer();

        echo '<h1>' . TextManager::_('Anketa') . ($nazev ? ' ' . $nazev : '') .'</h1>';

        $links = [
            'zakladni' => TextManager::_('Zakladni_udaje'),
            'vyhodnoceni' => TextManager::_('Vyhodnoceni_ankety'),
            'otazky' => TextManager::_('Otazky')
        ];


        $links['seznam'] = (new LinksListItem('seznam', TextManager::_('Zpet_na_seznam')))
                ->setActive(false)
                ->setUrl('?menuid=anketa&action=seznam')
                ->setCssClass('margin-left-4 external-link');

        LinksList::renderLinks($links, $action, '?menuid=anketa' . ($id ? "&record={$id}" : '') . '&action=');

        return Renderer::renderToView();
    }


    private function anketaView()
    {
        Model::addViewObject($this ->hlavickaView($_GET['record'] ?? 0, 'zakladni'));

        $anketa = Anketa::loadAnketa($this ->database, $_GET['record']);
        Model::addData('anketa', $anketa);

        $rozdeleniList = new  RozdeleniList();
        $params = $this ->database ->createParameters();
        $rozdeleniList ->loadOptions(
            $this ->database,
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, TypRozdeleni.Popis' .
            ' FROM AnketaProRozdeleni, Rozdeleni, TypRozdeleni' .
            ' WHERE AnketaProRozdeleni.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
            '  AND AnketaProRozdeleni.Rozdeleni = Rozdeleni.Kod' .
            '  AND AnketaProRozdeleni.IdAnkety = ' . $params ->addParameter($anketa ->id) .
            '  AND Rozdeleni.KodTypuRozdeleni = TypRozdeleni.Kod' .
            ' ORDER BY TypRozdeleni.Popis, Rozdeleni.Popis',
            $params);

        Model::addData('rozdeleniList', $rozdeleniList);

        $shiftForm = new RozdeleniShiftFormView(null, '?menuid=anketa&action=vyberrozdeleni&anketa=' . $_GET['record'], 'rozdeleniankety');
        $shiftForm ->loadTypyRozdeleni($this ->database);
        Model::addViewObject($shiftForm);

        $reportList = Report::loadFormReports($this ->database, 'anketa');
        Model::addData('reportList', $reportList);

        return new BaseViewObject("ronportal/ankety/view_hlavicka_ankety.php");
    }

    /**
     * @return ViewObject|Redirect
     */
    private function showForm()
    {
        //Model::addViewObject($this ->hlavickaView($_GET['record'] ?? 0, 'form'));

        $f = new AnketaFormular($this ->database, ($_GET['record'] ?? 0) ? 'edit' : 'new');
        if (!$f ->handleAction()) {
            return $f;
        }

        return new Redirect(null, '?menuid=anketa&action=zakladni&record=' . $f ->idAnkety);
    }


    /**
     * Zajisti zobrazeni editace otazek ankety
     * @return BaseViewObject
     */
    private function editaceOtazek()
    {
        $anketa = Anketa::loadAnketa($this ->database, $_GET['record']);
        Model::addData('anketa', $anketa);

        Model::addViewObject($this ->hlavickaView($anketa ->id, 'otazky', $anketa ->nazev));

        return new BaseViewObject("ronportal/ankety/view_otazky_ankety.php");
        //throw new \Exception("Neni implementovano");
    }


    /**
     * @throws RecordNotFoundException
     */
    private function vyhodnoceni()
    {
        $anketa = Anketa::loadAnketa($this ->database, $_GET['record']);
        Model::addViewObject($this ->hlavickaView($anketa ->id, 'vyhodnoceni', $anketa ->nazev));

        $vyhodnoceniView = new VyhodnoceniAnketyView();
        $vyhodnoceniView ->nactiAnketu($this ->database, $anketa);

        return $vyhodnoceniView;
    }


    /**
     * @param Anketa $anketa
     * @return null|int
     */
    private function najdiVyplnenyDotaznik($anketa)
    {
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Id FROM PripraveneTemaHodnoceni' .
            ' WHERE DruhHodnoceni = ' . $params ->addParameter($anketa ->druhHodnoceni) .
            '  AND TemaCislo = ' . $params ->addParameter($anketa ->temaCislo) .
            '  AND IdAnkety = ' . $params ->addParameter($anketa ->id) .
            '  AND HodnotilRC = ' . $params ->addParameter(User::getRC()),
            $params
        );
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row[0] ?? null;

//        $params = $this ->database ->createParameters();
//        $sql =
//            'UPDATE PripraveneTemaHodnoceni ' .
//            '   SET HodnotilRC = ' . $params->addParameter(User::getRC()) . ', ' .
//            '       Hodnotici = ' . $params->addParameter(User::getLoginName()) . ', ' .
//            '       DatumHodnoceni = ' . $params ->addTimestampParameter(time()) . ', ' .
//            '       Test_Stav = ' . $params ->addParameter($novyStav) . ', ' . // 1 = probiha
//            '       Poznamka = ' . $params ->addParameter( $this ->poznamkaHodnoceni ) .
//            ' WHERE Id = ' . $params ->addParameter((int)$this ->fId);
    }


    /**
     * Vytvori novy zaznam v tabulce PripraveneTemaHodnoceni - hlavicku vyplnene ankety
     * @param Anketa $anketa
     * @return PripraveneTemaHodnoceniDBRecord  Vraci vytvoreny zaznam
     * @throws DBException
     */
    public function vygenerujDotaznik($anketa)
    {
        $rec = new PripraveneTemaHodnoceniDBRecord($this ->database, $anketa ->druhHodnoceni, $anketa ->temaCislo, $this);

        // pole pro ankety
        $rec ->addIntField('IdAnkety', true, $anketa ->id);
        $rec ->addStringField('HodnotilRC', true, 15, User::getRC());
        $rec ->addStringField('Hodnotici', true, 80, User::getLoginName());

        $rec ->addLogText('Anketa ' . $anketa ->id . ': ' . $anketa ->popis);

        $rec ->insertRecord();

        return $rec;
    }


    /**
     * Vytvori viewobjecty pro zobrazeni zadane ankety a jejiho dotazniku
     * @param $anketa
     * @param $dotaznik
     * @return AnketaViewObject
     */
    private function zobrazAnketu($anketa, $dotaznik)
    {
        Model::addViewObject(new TextViewObject(
            '<h1>' . StringUtils::htmlTextConvert($anketa ->nazev) . '</h1>' .
            '<div class="poznamka">' . $anketa ->popis . '</div>'
        ));

        Model::addViewObject(new MessageManagerViewObject());

        return new AnketaViewObject($dotaznik, !$anketa ->platnostDo || $anketa ->platnostDo >= DateUtils::today(),false);
    }


    /**
     * Zobrazi anketu pro vyplneni. Nejdrive zjisti, jestli uz je vyplnena a pokud ano, tak ji
     * @return ViewObject
     * @throws DBException|RecordNotFoundException
     */
    private function vyplnitAnketu()
    {
        $anketa = Anketa::loadAnketa($this ->database, $_GET['record']);
        $anketa ->zkontrolujPravoNaAnketu($this ->database);

        $idVyplneneAnkety = $this ->najdiVyplnenyDotaznik($anketa);

        $dotaznik = new Dotaznik($this ->database, $idVyplneneAnkety);
        if (!$idVyplneneAnkety) {
            $dotaznik->nactiSablonu($anketa->druhHodnoceni, $anketa->temaCislo);
        }

        if ($anketa ->platnostOd  &&  $anketa ->platnostOd > time()) {
            return new TextViewObject(TextManager::_('Anketa_jeste_neni_platna'), 'errormsg');
        }

        return $this ->zobrazAnketu($anketa, $dotaznik);
    }


    /**
     * Ulozi vyplnenou anketu do databaze
     * @return mixed
     * @throws RecordNotFoundException|NeniPravoException|DBException
     */
    private function ulozitVyplnenouAnketu()
    {
        $anketa = Anketa::loadAnketa($this ->database, $_GET['record']);
        $anketa ->zkontrolujPravoNaAnketu($this ->database);

        $idVyplneneAnkety = $this ->najdiVyplnenyDotaznik($anketa);

        $dotaznik = new Dotaznik($this ->database, $idVyplneneAnkety);
        if (!$idVyplneneAnkety) {
            $dotaznik->nactiSablonu($anketa->druhHodnoceni, $anketa->temaCislo);
        }

        if (!$dotaznik ->loadPost()) {
            return $this ->zobrazAnketu($anketa, $dotaznik);
        }

        if (!$idVyplneneAnkety) {
            $rec = $this ->vygenerujDotaznik($anketa);
            $dotaznik ->fId = $rec['Id'];
        }
        $dotaznik ->ulozOdpovedi();

        return new Redirect(null, '?menuid=anketa');
    }



    function shiftFormRozdeleniAction($output)
    {
        $typRozdeleni = $_POST['typRozdeleni'] ?? $_GET['typrozdeleni'] ?? null;
        if (!$typRozdeleni) {
            return;
        }

        $shiftForm = new RozdeleniAnketyShiftForm($this ->database, (int)$typRozdeleni, (int)$_GET['anketa'], '');
        $data = $shiftForm ->handleAction();

        $output ->addData('data', $data);
        $output ->compress = 'gzip';
    }



    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'vyberrozdeleni' :
                $this->shiftFormRozdeleniAction($output);
                break;

            default :
                parent::handleJsonAction($action, $output);
        }
    }


    function report()
    {
        if (!$_GET['record']) {
            throw new \Exception('Neni zadano id ankety pro tisk!');
        }

        $anketa = Anketa::loadAnketa($this ->database, $_GET['record']);

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, 'Anketa');

        $result->integerParams['ID'] = $anketa ->id;

        return $result;
    }



    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue('povolitankety', 0)) {
            throw new NeniPravoException('Modul Ankety neni povolen');
        }

        TextManager::getInstance()
            ->loadStrings('elearning.ini')
            ->loadStrings('dotaznik.ini');

        HtmlHeaders::getInstance()
            ->setAppTitle(TextManager::_( 'Ankety'))
            ->addCss('css/dotaznik.css');

        $this ->addTranslationsScript('anketa');

        // uzivatelska cast
        if ((int)($_POST['vyplnitanketu'] ?? 0)) {
            return $this ->ulozitVyplnenouAnketu();
        }

        switch ($_GET['action'] ?? '') {
            case 'vyplnit' :
                return $this->vyplnitAnketu();

            case 'translations' :
                return $this ->translations();
        }

        if (Prava::vratPrava('ankety') ->prava === Prava::PRAVA_VSE) {
            // administratorska cast
            HtmlHeaders::getInstance()
                ->addScript('script/testy.js');

            if (isset($_GET['formaction']) || isset ($_POST['formaction'])) {
                // formular pro editaci
                return $this->showForm();
            }

            switch ($_GET['action'] ?? '') {
                case '' :
                case 'moje' :
                    break;

                case 'zakladni' :
                    return $this->anketaView();

                case 'otazky' :
                    return $this->editaceOtazek();

                case 'vyhodnoceni' :
                    return $this->vyhodnoceni();

                case 'form' :
                    return $this->showForm();

                case 'seznam' :
                    $b = new AnketaBrowser($this->database);
                    $b->load();
                    return $b;

                case 'report' :
                    return $this ->report();

                default :
                    return parent::handleAction($action);
            }
        }

        // seznam anket k vyplneni
        $b = new AnketyKVyplneniBrowser($this ->database);
        $b ->load();

        return $b;
    }


}
