<?php

namespace RonPortal\Ankety;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\MVC\Model;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\RozdeleniShiftFormView;
use RonPortal\Controls\SeznamRozdeleniFormField;
use RonPortal\RonPortalDBRecord;


class AnketaFormular extends RonPortalForm
{
    const DRUH_HODNOCENI_ANKETA = 12;

    public $idAnkety;

    /**
     * @param Database $database
     * @param string $formAction
     */
    public function __construct($database, $formAction)
    {
        parent::__construct($database, $formAction);

        $this ->sqlTableName = 'Anketa';

        $this
            ->setCode('formanketa')
            ->setTitle('Anketa')
            ->setXmlEventsName('ANKETA');

        $this ->stornoAction = $this ->isInsert()
            ? '?menuid=anketa&action=seznam'
            : '?menuid=anketa&action=zakladni&record=' . (int)($_GET['record'] ?? 0);

        $this ->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('Id')
            ->setFormat('int')  // ->setSequence('Seq_Anketa') - seqence se resi v beforeInsert
            ->setPrimary(true)
            ->setCode('field-id');

        $this ->addField()
            ->setFieldName('Nazev')
            ->setMaxLength(160)
            ->setCaption('Nazev')
            ->setRequired(true)
            ->setCode('field-nazev');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Popis')
            ->setCaption('Popis')
            ->setCode('field-popis');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostOd')
            ->setCode('field-platnostod')
            ->setCaption('Platnost_od')
            ->setDefaultValue(StringUtils::format('date', DateUtils::today()));

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostDo')
            ->setCode('field-platnostdo')
            ->setCaption('Platnost_do');

//        if (!$this ->isInsert()) {   - presunuto do view
//            // pri editaci zobrazi i pole pro vyber rozdeleni
//            $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
//                ->setCaption(TextManager::_('Rozdeleni'));
//
//            $params = $database ->createParameters();
//            $this ->addField(
//                (new SeznamRozdeleniFormField())
//                    ->setAfterCloseAction("reload")
//                    ->setSelectSql(
//                        new SqlStatement(
//                            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, TypRozdeleni.Popis' .
//                            ' FROM AnketaProRozdeleni, Rozdeleni, TypRozdeleni' .
//                            ' WHERE AnketaProRozdeleni.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
//                            '  AND AnketaProRozdeleni.Rozdeleni = Rozdeleni.Kod' .
//                            '  AND AnketaProRozdeleni.IdAnkety = ' . $params ->addParameter($_GET['record'] ?? 0) .
//                            '  AND Rozdeleni.KodTypuRozdeleni = TypRozdeleni.Kod' .
//                            '  AND %FILTER%' .
//                            ' ORDER BY %ORDER%',
//                            $params),
//                        '1 = 1',
//                        'TypRozdeleni.Popis, Rozdeleni.Popis'
//                    )
//                    ->setItemsAttributes(['typ' => 2])
//            );
//
//            $shiftForm = new RozdeleniShiftFormView(null, '?menuid=anketa&action=vyberrozdeleni&anketa=' . $_GET['record'], 'rozdeleniankety');
//            $shiftForm ->loadTypyRozdeleni($database);
//            Model::addViewObject($shiftForm);
//        }

        $this ->addLogFields();
    }



    function beforeInsertRecord($rec)
    {
        $result = parent::beforeInsertRecord($rec);

        // Vyzvedne nove Id a nastavi ho v zaznamu
        $newId = SQLUtils::nextSequenceValue($this ->database, 'SEQ_Anketa');
        $rec['Id'] = $newId;

        // Vytvori nove TemaHodnocni (Protoze v tabulce Anketa je povinne)
        $recTemaHodnoceni = new RonPortalDBRecord($this ->database, 'TemaHodnoceni', $this);
        $recTemaHodnoceni ->addIntField('DruhHodnoceni', true, self::DRUH_HODNOCENI_ANKETA); // Anketa

        $cursor = $this ->database ->getSqlResult('SELECT MAX(TemaCislo) FROM TemaHodnoceni WHERE DruhHodnoceni = ' . self::DRUH_HODNOCENI_ANKETA);
        $row = $cursor ->fetch();
        $cursor ->close();

        $newTemaCislo = ($row && $row[0]) ? $row[0] + 1 : 1;
        if ($newTemaCislo < $newId) {
            // Pro prehlednost nastavi TemaCislo na stejnou hodnotu jako ID ankety (samozrejme jen v pripade, ze je to mozne)
            $newTemaCislo = $newId;
        }

        $recTemaHodnoceni ->addIntField('TemaCislo', true, $newTemaCislo);

        $recTemaHodnoceni ->addStringField('Kod', false, 20, 'Anketa_' . $newId);
        $recTemaHodnoceni ->addStringField('PopisTematu', false, 120, 'Anketa ' . $rec['Popis']);

        $recTemaHodnoceni ->addIntField('EditovatPoznamku', true, 0); // Vychozi hodnota je 1 - u anket IMHO nema smysl

        $recTemaHodnoceni ->insertRecord();

        $rec ->addIntField('DruhHodnoceni', false, self::DRUH_HODNOCENI_ANKETA);
        $rec ->addIntField('TemaCislo', false, $newTemaCislo);

        return $result;
    }

    public function insertRecord()
    {
        $this ->database ->beginTransaction();
        try {
            $result = parent::insertRecord();
            $this ->database ->commitTransaction();
            return $result;
        } catch (\Exception $e) {
            $this ->database ->rollbackTransaction();
            throw $e;
        }
    }

    function afterInsertRecord($rec)
    {
        $this ->idAnkety = $rec['Id'];
        return parent::afterInsertRecord($rec);
    }

    function afterUpdateRecord($rec)
    {
        $this ->idAnkety = $rec['Id'];
        return parent::afterUpdateRecord($rec);
    }
}
