<?php

namespace RonPortal\Ankety;

use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\User;


class StavAnketyBrowserField extends BrowserField
{

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-idvyplneneankety']) {
            $value = 1; // ok, vyplneno
        } else {
            $value = 0; // nevyplneno

            if ($rowData['col-platnostdo']) {
                $platnostDo = $this ->database ->stringToTimestamp($rowData['col-platnostdo']);

                if ($platnostDo < DateUtils::today()) {
                    // uz ukoncena
                    $value = 2;
                } else if ($platnostDo < time()) {
                    // platnost do "dnes"
                    $value = 4;
                } else if (DateUtils::incDayStart($platnostDo, -7) < time()) {
                    // k vyplneni ankety zbyva mene nez tyden
                    $value = 3;
                }
            }
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


class AnketyKVyplneniBrowser extends RonPortalBrowser
{
    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'anketa');

        $params = $database ->createParameters();
        $this->sqlFromPart = new SqlStatement(
            'Anketa LEFT OUTER JOIN PripraveneTemaHodnoceni' .
            ' ON PripraveneTemaHodnoceni.DruhHodnoceni = Anketa.DruhHodnoceni' .
            ' AND PripraveneTemaHodnoceni.TemaCislo = Anketa.TemaCislo' .
            ' AND PripraveneTemaHodnoceni.IdAnkety = Anketa.Id' .
            ' AND PripraveneTemaHodnoceni.HodnotilRC = ' . $params ->addParameter(User::getRC()),
            $params
        );

        $this
            ->setTitle('Ankety')
            ->setCode('browseranketykvyplneni')
            ->setMode(Browser::MODE_TILES)
            //->setRowCssClass($this ->rowCssClass . ' ' . 'col-12 col-sm-6 col-lg-4')
            ;


        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Anketa.Id')
            ->setCode('col-id');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PripraveneTemaHodnoceni.IdAnkety')
            ->setCode('col-idvyplneneankety');


        $this ->addField((new StavAnketyBrowserField($database))
            ->setCode('col-stav-ankety')
            ->setType(BrowserField::FIELD_TYPE_SELECT)
            ->setOptions([
                0 => '', // nevyplnena
                1 => '<i class="fa inline-glyph glyph-check"></i>', // vyplnena
                2 => '<i class="fa inline-glyph glyph-error"></i>', // ukoncena
                3 => '<i class="fa inline-glyph glyph-warning"></i>', // nevyplnena, brzy konci platnost
                4 => '<i class="fa inline-glyph glyph-warning glyph-background-red glyph-color-white"></i>'  // nevyplnena, dnes konci platnost
            ])
            ->setClassName('float-right fa-2x')
        );

        $this ->addField()
            ->setSqlFieldName('Anketa.Nazev')
            ->setCode('col-nazev')
            ->setOrdering(true)
            ->setClassName('font-weight-bold');

        $this ->addField()
            ->setSqlFieldName('Anketa.Popis')
            ->setCode('col-popis')
            ->setClassName('poznamka');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Anketa.PlatnostOd')
            ->setFormat('date')
            ->setCode('col-platnostod');

        $this ->defaultSort = '-col-platnostod';

        $this ->addField()
            ->setSqlFieldName('Anketa.PlatnostDo')
            ->setFormat('date')
            ->setCode('col-platnostdo')
            ->setHeaderText('Platnost_do')
            ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('Vyplnit_anketu'))
            ->setLink('?menuid=anketa&action=vyplnit');

    }


    function setFilterForm()
    {
        // parent::setFilterForm(); - nevolat, neni potreba
    }


    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $sql =
            'EXISTS (SELECT * FROM AnketaProRozdeleni, RozdeleniOsob, PracovniPomer' .
            ' WHERE AnketaProRozdeleni.IdAnkety = Anketa.Id' .
            ' AND AnketaProRozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni' .
            ' AND AnketaProRozdeleni.Rozdeleni = RozdeleniOsob.Rozdeleni' .
            ' AND RozdeleniOsob.RC = PracovniPomer.RC' .
            ' AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo' .
            ' AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter(time()) .
            ' AND (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            ' AND PracovniPomer.RC = ' . $params ->addParameter(User::getRC()) .
            ')';

        return new SqlStatement($sql, $params);
    }

    public function renderHeader()
    {
        parent::renderHeader();

        if (Prava::vratPrava('ankety') ->prava === Prava::PRAVA_VSE) {
            // uzivatel muze editovat ankety - zobrazi odkaz na seznam
            echo
                '<div class="row margin-top-2 margin-bottom-2">' .
                '<a href="?menuid=anketa&action=seznam">' . TextManager::_('Seznam_anket') . '</a>' .
                '</div>';
        }
    }


}
