<?php

namespace RonPortal\Ankety;

use Ron\DB\DBException;
use Ron\Exception\NotImplementedException;
use RonPortal\Controls\RozdeleniShiftForm;
use RonPortal\RonPortalDBRecord;


class RozdeleniAnketyShiftForm extends RozdeleniShiftForm
{
    protected $idAnkety;

    public function __construct($database, $typRozdeleni, $idAnkety)
    {
        parent::__construct($database, $typRozdeleni, 'rozdeleniankety');
        $this ->idAnkety = $idAnkety;
    }


    protected function getSqlPrirazenaRozdeleni($params, $filter)
    {
        // POZOR! Nefunguje na ODBC s driverem "SQL Server" - je potreba prepnout na jiny driver!
        // nacita vsechna rozdeleni vcetne neprirazenych
        return
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, AnketaProRozdeleni.IdAnkety, Rozdeleni.cssClass ' .
            ' FROM Rozdeleni, AnketaProRozdeleni ' .
            ' WHERE AnketaProRozdeleni.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
            '  AND AnketaProRozdeleni.Rozdeleni = Rozdeleni.Kod ' .
            '  AND AnketaProRozdeleni.IdAnkety = ' . $params->addParameter($this ->idAnkety) .
            '  AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
            '  AND (Rozdeleni.DatumZruseni > ' . $params ->addDateParameter(time()) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            "  AND (Rozdeleni.cssClass IS NULL  OR  Rozdeleni.cssClass <> 'hidden') " .

            (isset($filter)
                ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . ' OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                : '') .

            ' ORDER BY Rozdeleni.Popis ';
    }

    protected function getSqlVsechnaRozdeleni($params, $filter)
    {
        // POZOR! Nefunguje na ODBC s driverem "SQL Server" - je potreba prepnout na jiny driver!
        // nacita vsechna rozdeleni vcetne neprirazenych
        return
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, AnketaProRozdeleni.IdAnkety, Rozdeleni.cssClass ' .
            ' FROM Rozdeleni LEFT OUTER JOIN AnketaProRozdeleni ' .
            '  ON AnketaProRozdeleni.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
            '  AND AnketaProRozdeleni.Rozdeleni = Rozdeleni.Kod ' .
            '  AND AnketaProRozdeleni.IdAnkety = ' . $params->addParameter($this ->idAnkety) .
            ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
            ' AND (Rozdeleni.DatumZruseni > ' . $params ->addDateParameter(time()) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            " AND (Rozdeleni.cssClass IS NULL  OR  Rozdeleni.cssClass <> 'hidden') " .

            (isset($filter)
                ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . ' OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                : '') .

            ' ORDER BY Rozdeleni.Popis ';
    }



    protected function priradRozdeleni($kodRozdeleni)
    {
        $rec = new RonPortalDBRecord($this ->database, 'AnketaProRozdeleni', $this);
        $rec ->addIntField('IdAnkety', true, $this ->idAnkety);
        $rec ->addIntField('KodTypuRozdeleni', true, $this ->typRozdeleni);
        $rec ->addStringField('Rozdeleni', true, 30, $kodRozdeleni);

        try {
            $rec ->insertRecord();
        } catch (DBException $e) {
            if (!$e ->isPrimaryKeyViolation()) {
                // pokud uz zaznam existuje, neni potreba delat nic
                throw $e;
            }
        }
    }


    protected function odstranRozdeleni($kodRozdeleni)
    {
        $rec = new RonPortalDBRecord($this ->database, 'AnketaProRozdeleni', $this);
        $rec ->addIntField('IdAnkety', true);
        $rec ->addIntField('KodTypuRozdeleni', true);
        $rec ->addStringField('Rozdeleni', true);

        $params = $this ->database ->createParameters();
        $rec ->setWhere(
            'IdAnkety = ' . $params ->addParameter($this ->idAnkety) .
            ' AND KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
            ' AND Rozdeleni = ' . $params ->addParameter($kodRozdeleni),
            $params
        );

        $rec ->deleteRecord();
    }
}
