<?php
/**
 * Roman.Szablatura RON Software 6.11.2018
 */

namespace RonPortal\Benefity;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\User;
use Tracy\Debugger;


/**
 * Class SmazatSablonuField
 * @package Ron\RonPortal\Benefity
 */
class SmazatSablonuField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Momentalne na portale lze rusit pouze sve sablony
        if ($rowData[1] == User::getRC()) {
            $text = TextManager::_("odstranit_sablonu_info");

            return
                "<span class='confirmQuestion js' data-ron-confirm-text='$text'" .
                " data-ron-href='?menuid=benefity&action=odstranitsablonu&record=$rowData[0]" . Csrf::csrfUrlParameter("benefit_del_sablona_{$rowData[0]}") . "' >" .
                TextManager::_('odstranit_sablonu') .
                "</span>";
        }

        return '';
    }
}



/**
 * Class BrowserSablonyBenefitu
 * @package Ron\RonPortal\Benefity
 */
class SablonyBenefituBrowser extends RonPortalBrowser
{
    /**
     * BrowserSablonyBenefitu constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'benefity');

//        $this->title = TextManager::_('sablony_benefitu');
        $this->code = 'browsersablonybenefitu';
        $this->tableId = 'browsersablonybenefitu';
        $this->browserUrl = '?menuid=benefity&action=sablony';

        $this->sqlSelectPart =
            'SablonaRezervaceBenefitu.ID, SablonaRezervaceBenefitu.RC';   //0..1

        $this ->primaryField = 0;

        $this->sqlFromPart =
            'SablonaRezervaceBenefitu LEFT JOIN Benefit ON (Benefit.ID = SablonaRezervaceBenefitu.IdBenefitu)' .
            ' LEFT JOIN Osoba ON (SablonaRezervaceBenefitu.RC = Osoba.RC)';


        $this->addField()
            ->setSqlFieldName('Id', 0) ->setHeaderText('ID') ->setOrdering(true)
            ->setCode('col-id')
            ->setVisible(false);

        $this->addField()
            ->setSqlFieldName('Poradi') ->setHeaderText('Poradi') ->setOrdering(true)
            ->setCode('col-poradi');

        $this ->defaultSort = 'col-poradi';

        $this->addField()
            ->setSqlFieldName('Benefit.Popis') ->setHeaderText('benefit_popis') ->setOrdering(true)
            ->setCode('col-popis');

        // osoba
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Titul')
            ->setCode('col-titul');

        $pravo_sablony = Prava::vratPrava('benefit_sablona');
        $this ->addField(new JmenoPrijmeniBrowserField($this->database, 'col-jmeno', 'col-prijmeni', 'col-titul'))
            ->setCode('col-osoba')
            ->setHeaderText('Zamestnanec') ->setOrdering(true)
            ->setVisible(!$pravo_sablony->pouzeSebe()); // kdyz ma "pouze sebe" tak jmeno videt nepotrebuje

        $this->addField()
            ->setSqlFieldName('PocetBenefitu') ->setHeaderText('pocet_benefitu')
            ->setCode('col-pocetbenefitu')
            ->setOrdering(true) ->setAlign('right');

        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('detail_sablony'))
            ->setCode('col-detail')
            ->setLink('?menuid=benefity&action=sablony&record=');

        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('CasovaJednotka') ->setHeaderText('casova_jednotka')
            ->setCode('col-casovajednotka')
            ->setOrdering(true)
            ->setOptions([
                1 => TextManager::_('casova_jednotka_benefitu_1'),
                2 => TextManager::_('casova_jednotka_benefitu_2'),
                3 => TextManager::_('casova_jednotka_benefitu_3'),
                4 => TextManager::_('casova_jednotka_benefitu_4'),
                5 => TextManager::_('casova_jednotka_benefitu_5'),
                6 => TextManager::_('casova_jednotka_benefitu_6')
            ]);


        $this->addField()
            ->setSqlFieldName('PocetCasJednotek') ->setHeaderText('opakovani')
            ->setCode('col-pocetcasjednotek')
            ->setOrdering(true) ->setAlign('center');

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('TypKontrolySablony') ->setHeaderText('povolit_vlozit_mene')
            ->setCode('col-typkontrolysablony')
            ->setAlign('center') ->setOrdering(true)
            ->setOptions('yesno');

        $this->addField(new SmazatSablonuField($this->database))
            ->setHeaderText('odstraneni_sablony') ->setAlign('center')
            ->setCode('col-odstranit');

        // insert log
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SablonaRezervaceBenefitu.Insert_UserName')
            ->setCode('col-insert_username');

        $this ->addField()
            ->setSqlFieldName('SablonaRezervaceBenefitu.Insert_Timestamp') ->setSqlFieldIndex2('col-insert_username')
            ->setCode('col-insert_timestamp') ->setFormat('date')
            ->setHeaderText('datum_osoba_vlozeni')
            ->setOrdering(true);

        // update log
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SablonaRezervaceBenefitu.Update_UserName')
            ->setCode('col-update_username');

        $this ->addField()
            ->setSqlFieldName('SablonaRezervaceBenefitu.Update_Timestamp') ->setSqlFieldIndex2('col-update_username')
            ->setCode('col-update_timestamp') ->setFormat('date')
            ->setHeaderText('Posledni_aktualizace')
            ->setOrdering(true);
    }



    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this ->database, ['menuid' => 'benefity', 'action' => 'sablony'], true);

        $this->filterForm->addFilter(
            (new FilterFormFilter('typbenefitu', FilterFormFilter::TYPE_OPTIONS))
                ->setCaption(TextManager::_('typ_benefitu'))
                ->addItem('', TextManager::_('_vse_'))
                ->setOptionsSql('SELECT Kod, Popis FROM TypBenefitu WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
                ->setSqlFilter('Benefit.TypBenefitu = ')
        );


        $this->filterForm->addFilter(
            (new FilterFormFilter('jednotka', FilterFormFilter::TYPE_OPTIONS))
                ->setCaption(TextManager::_('pouze_casova_jednotka_opakovani'))
                ->setSessionName('filterform_jednotka_benefitu')
                ->addItem('', TextManager::_('_vse_'))
                ->addItem(1, TextManager::_('casova_jednotka_benefitu_1'))
                ->addItem(2, TextManager::_('casova_jednotka_benefitu_2'))
                ->addItem(3, TextManager::_('casova_jednotka_benefitu_3'))
                ->addItem(4, TextManager::_('casova_jednotka_benefitu_4'))
                ->addItem(5, TextManager::_('casova_jednotka_benefitu_5'))
                ->addItem(6, TextManager::_('casova_jednotka_benefitu_6'))
                ->setSqlFilter('SablonaRezervaceBenefitu.CasovaJednotka = ')
        );


        $this ->filterForm ->addFilter(
            (new FilterFormFilter('vytvoritsablonu', FilterFormFilter::TYPE_TEXT))
                ->setDefaultValue('<a href="?menuid=benefity&action=sablony&formaction=new">' . TextManager::_('vytvorit_sablonu') . '</a>')
                ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                ->setRowClass('margin-top-2')
                ->setCaption(false)
        );
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $pravo_sablony = Prava::vratPrava('benefit_sablona');
        $filtrPrava = $pravo_sablony ->sqlFiltr('PracovniPomer', time(), time(), $params);

        $result .=
            ($result ? ' AND ' : '') .
            'EXISTS (SELECT * FROM PracovniPomer WHERE RC = SablonaRezervaceBenefitu.RC AND (' . $filtrPrava . '))';

        return $result
            ? new SqlStatement($result, $params)
            : '';
    }
}
