<?php
/**
 * stanislav.elbl, 24.03.2021
 */

namespace RonPortal\Common;

use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\User;

class PotvrzeniPrecteniForm extends RonPortalForm
{

    protected $idDokumentu;
    protected $platnostDokumentuOd;
    protected $idVzkazu;

    protected $predmetVzkazu;
    protected $popisTypuDokumentu;

    /**
     * PotvrzeniPrecteniForm constructor.
     * @param Database $database
     * @param string $idDokumentu
     * @param int $platnostOd
     * @param int $idVzkazu
     * @throws DBException
     */
    function __construct($database, $idDokumentu, $platnostOd, $idVzkazu)
    {
        parent::__construct($database);

        $this->idDokumentu = $idDokumentu;
        $this->platnostDokumentuOd = $platnostOd;
        $this->idVzkazu = $idVzkazu;

        $this->nactiDetailyVzkazu();

        $this->title = TextManager::_('potvrzeni_precteni');
        $this->code = 'formpotvrzeniprecteni';

        $this->htmlAction = '?menuid=dokumenty&potvrzeniprecteniform=1&';
        $this->stornoAction = '?menuid=dokumenty';

        $this->sqlTableName = 'DokumentVzkazu';
        $this->sqlSelectPart =
            'IdVzkazu, idDokumentu, PlatnostOd, Precteno, ' . $this->database->castWideMemo('IdentifikacePodpisu') . ', ' . $this->database->castWideMemo('Vyjadreni');   // 0 - 5

        //Mozne hodnoty: VzkazId, VzkazPredmet, VzkazPopis, DokumentVzkazuidDokumentu, DokumentVzkazuPlatnostOd, TypDokumentuPopis, VzkazDatumCas

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setCaption('Nazev') ->setText(StringUtils::htmlTextConvert($this->predmetVzkazu))
            ->setCode('field-nazev');

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setCaption('popis_typu_dokumentu') ->setText(StringUtils::htmlTextConvert($this->popisTypuDokumentu))
            ->setCode('field-typdokumentu');

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setCaption('platnost_dokumentu_od') ->setText(StringUtils::format('date', $this->platnostDokumentuOd))
            ->setCode('field-platnostod');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Precteno') ->setSqlFieldIndex(3) ->setCaption('precteno')
            ->setCode('field-precteno')
            ->setSelectSql('SELECT Kod, Popis FROM S_AnoNe') ->setReadOnly(true);


        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Vyjadreni') ->setSqlFieldIndex(5) ->setCaption('vyjadreni')
            ->setCode('field-vyjadreni');

        /*$this->addFields([
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_NAME => 'dokumentid', FORM_FIELD_SQL_NAME => 'idDokumentu', FORM_FIELD_DEFAULT_VALUE => $this->idDokumentu],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_NAME => 'vzkazid', FORM_FIELD_SQL_NAME => 'idVzkazu', FORM_FIELD_DEFAULT_VALUE => $this->idVzkazu],
            [FORM_FIELD_TYPE => FORM_FIELD_TYPE_HIDDEN, FORM_FIELD_NAME => 'platnostod', FORM_FIELD_SQL_NAME => 'PlatnostOd', FORM_FIELD_DEFAULT_VALUE => StringUtils::format('date',$this->platnostDokumentuOd)]
        ]);*/
    }

    /**
     * @return string
     */
    function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        return new SqlStatement(
            ' IdVzkazu = ' . $params ->addParameter((int)$this->idVzkazu) .
            ' AND idDokumentu = ' . $params ->addParameter((int)$this->idDokumentu) .
            ' AND PlatnostOd = ' . $params ->addParameter($this->database->timestampToParam($this->platnostDokumentuOd)),
            $params);
    }


    function createDBRecordFields($rec, $insert)
    {
        parent::createDBRecordFields($rec, $insert);

        $rec->addField('Precteno', 'int', false, 0, 1);
        $rec->addField('IdentifikacePodpisu', 'memo', false, 0,
            sprintf(TextManager::_('dokumenty_identifikace_podpisu'), StringUtils::format('datetime',time()), User::getLoginName()));
    }


    /**
     * @param DBRecord $rec
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);
        $rec->setWhere($this->getWhereCondition());
    }


    function renderCommonFields()
    {
        parent::renderCommonFields();

        echo '<input type="hidden" name="dokumentid" value="' . $this->idDokumentu . '" />';
        echo '<input type="hidden" name="vzkazid" value="' . $this->idVzkazu . '" />';
        echo '<input type="hidden" name="platnostod" value="' . StringUtils::format('date', $this->platnostDokumentuOd) . '" />';
    }

    /**
     * @throws DBException
     */
    private function nactiDetailyVzkazu()
    {
        $params = $this->database ->createParameters();

        $sql =
            'SELECT Vzkaz.Id, Vzkaz.Predmet, ' . $this->database->castWideMemo('Vzkaz.Popis') . ', ' . // 0 - 2
            ' DokumentVzkazu.idDokumentu, DokumentVzkazu.PlatnostOd, TypDokumentu.Popis, Vzkaz.DatumCas' .  //3 - 6
            ' FROM Vzkaz JOIN DokumentVzkazu ON (Vzkaz.Id = DokumentVzkazu.IdVzkazu)' .
            '  JOIN Dokument ON (DokumentVzkazu.idDokumentu = Dokument.id)' .
            '  JOIN TypDokumentu ON (Dokument.typ = TypDokumentu.kod)' .
            ' WHERE Vzkaz.ZobrazitNaIntrawebu = 1' .
            '  AND TypDokumentu.Distribuovany = 1' .
            '  AND DokumentVzkazu.idVzkazu = ' .  $params->addParameter($this->idVzkazu) .
            ' ORDER BY TypDokumentu.Kod ';

        $cursor = $this->database ->getSqlResult($sql, $params);
        $row = $cursor->fetch();

        $this->predmetVzkazu = $row[1];
        $this->popisTypuDokumentu = $row[5];
    }
}
