<?php
/** roman.szablatura, 14.02.2016 */

namespace RonPortal\Common;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MVC\HttpController;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Kapacity\KapacitySettings;
use RonPortal\NeniPravoException;
use RonPortal\PravaNaOsoby;
use RonPortal\User;


class ProvozniDenikController extends HttpController
{
    /**
     * @var Database
     */
    private $database;
    /**
     * @var PravaNaOsoby
     */
    private $pravaProhlizet;
    /**
     * @var PravaNaOsoby
     */
    private $pravaEditovat;

    /**
     * UchazeciController constructor.
     * @param Database $database databaze
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        $this->database = $database;

        $this->pravaProhlizet = Prava::vratPrava('pr_denik_read');
        $this->pravaEditovat = Prava::vratPrava('pr_denik_write');
    }


    /**
     * @param Database $database
     * @param string $rc
     * @param int $datum
     * @return string
     * @throws Exception
     * @throws DBException
     */
    public static function vratRozdeleniPlanovaniKapacit($database, $rc, $datum)
    {
        $params = $database->createParameters();
        $sql = "SELECT Rozdeleni
                FROM PlanovaniKapacit
                WHERE PlanovaniKapacit.RC = " . $params->addParameter($rc) .
                " AND PlanovaniKapacit.KodTypuRozdeleni = " .  $params->addParameter(RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) .
                " AND PlanovaniKapacit.Datum = " .  $params->addDateParameter($datum);

        $cursor = $database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        $rozdeleni = $row[0] ?? null;

        if ($row = $cursor->fetch()) {
            $cursor ->close();

            throw new Exception(
                StringUtils::fromUTF8('Je povoleno použití rozdělení z plánování kapacit a uživatel má pro datum ' . StringUtils::format('date',$datum) . ' více přiřazených rozdělení.
                Nastavení: "Použít rozdělení plánování kapacit" je ignorováno.'));
        }

        $cursor ->close();

        return $rozdeleni;
    }

    /**
     * @return Redirect|ViewObject
     * @throws Exception
     */
    private function provozniDenikForm()
    {
        if (!($this->pravaEditovat->jsouNejakePrava())) {
            throw new NeniPravoException();
        }

        $formAction = $_POST['formaction'] ?? $_GET['formaction'] ?? null;

        $idZaznamu = isset($_POST['formaction'])
            ? $_POST['record']
            : ($_GET['record'] ?? null);

        switch ($formAction) {
            case 'new' :
            case 'insert' :
                break;

            case 'edit' :
            case 'save' :
                if  (!($this->pravaEditovat->priznak && $this->lzeZobrazitZaznam($idZaznamu))) {
                    throw new NeniPravoException();
                }
                break;

            default:
                throw new Exception('Chyba v ProvozniDenikController::provozniDenikForm!');
        }

        $f = new ProvozniDenikFormular($this->database);

        if ($f ->handleAction()) {
            return new Redirect(null, '?menuid=provoznidenik');
        }
        return $f;
    }

    /**
     * @param int $idZaznamu
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    private function lzeZobrazitZaznam($idZaznamu)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT ProvozniDenik.KodRozdeleni, ProvozniDenik.RC FROM ProvozniDenik" .
            " WHERE Id = " . $params->addParameter($idZaznamu);

        $cursor = $this->database->getSqlResult($sql,$params);

        if ($row = $cursor->fetch()) {
            if ($this->pravaEditovat->jePravoNaRozdeleni($this->pravaEditovat->typRozdeleni, $row[0], time(), time())) {
                return true;
            }
            elseif (RonPortalSettings::getIntValue('pd_pouzit_rozdeleni_kapacity')
                && ($row[0] == self::vratRozdeleniPlanovaniKapacit($this->database,User::getRC(),time()))
                && ($row[1] == User::getRC())
            ) {
                return true;
            }
        }
        return false;
    }

    /**
     * @return ViewObject
     * @throws DBException
     * @throws Exception
     */
    private function zobrazSeznam()
    {
        $b = new ProvozniDenikBrowser($this ->database);
        $b ->load();

        return $b;
    }

    /**
     * @param string $action
     * @return string|ViewObject|Redirect
     * @throws Exception
     */
    public function handleAction($action)
    {
    	if (!($this->pravaProhlizet->jsouNejakePrava() || $this->pravaEditovat->jsouNejakePrava())) {
            throw new NeniPravoException();
    	}
    	
        TextManager::getInstance() ->loadStrings('person.ini');

        switch ($_GET['action'] ?? null) {
            case 'denikform':
                return $this ->provozniDenikForm();

            default:
               return $this->zobrazSeznam();
        }
    }
}
