<?php
/**
 * stanislav.elbl, 18.03.2021
 */

namespace RonPortal\Common;

use Ron\Controls\BrowserField;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\User;

class Prijemce_OdesilatelField extends BrowserField
{
    function __construct()
    {
        parent::__construct();

        $this->align = 'left';
        $this->header = \Ron\ronText('Prijemce_odesilatel');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return $rowData[5]
            ? '<strong>' . \Ron\ronText('Pro') . ' </strong>' . StringUtils::htmlTextConvert($rowData[5])
            : '<strong>' . StringUtils::htmlTextConvert($rowData[6]) . '</strong>';
    }
}




class VzkazBrowser extends RonPortalBrowser
{

    function __construct($database)
    {
        parent::__construct($database, 'vzkazy');

        $this->title = TextManager::_('Seznam_vzkazu');

        $this->sqlSelectPart =
            'V.Id, {fn_CASTWIDEMEMO(V.Popis)}, {fn_CASTWIDEMEMO(V.Vyjadreni)}, V.Zpracovano, ' .
            ' V.Predmet, P1.Popis, P2.Popis, V.RCPrijemce, V.DatumCas';

        $this->sqlFromPart =
            '(Vzkaz V LEFT OUTER JOIN PrijemceVzkazu P1 ' .
            '   ON P1.Kod = V.KodPrijemce) ' .
            ' LEFT OUTER JOIN PrijemceVzkazu P2 ' .
            ' ON P2.Kod = V.KodOdesilatele ';

        $this->sqlWherePart =
            "( V.RCOdesilatele = '" . $database ->addSlashes(User::getRC()) . "' " .   // je to muj vzkaz
            ' OR ' .
            '( V.ZobrazitNaIntrawebu = 1  AND ' .        // ostatni vzkazy pouze pokud jsou povolene
            ' ( ' .
            " V.RCPrijemce = '" . $database ->addSlashes(User::getRC()) . "' " .  // vzkaz pro me
            ' OR ( V.RCOdesilatele IS NULL AND V.RCPrijemce IS NULL AND ' .  // vzkaz pro rozdeleni
            ' EXISTS ( ' .
            ' SELECT * FROM RozdeleniOsob R' .
            ' WHERE R.KodTypuRozdeleni = V.KodTypuRozdeleni ' .
            ' AND R.Rozdeleni = V.Rozdeleni ' .
            " AND R.RC = '" . $database ->addSlashes(User::getRC()) . "' " .
            ' AND R.PlatnostOd <= ' . $database ->dateToString(time()) .
            ' AND (R.PlatnostDo >= ' . $database ->dateToString(time()) . ' OR R.PlatnostDo IS NULL)' .
            ' ) ' .
            ' ) ' .
            ' OR ( V.RCOdesilatele IS NULL AND V.RCPrijemce IS NULL AND V.Rozdeleni IS NULL AND V.KodPrijemce IS NULL ) ' . // vzkaz pro vsechny
            ' ) ' .
            ')' .
            ')';

        $this->sqlOrderPart = 'DatumCas DESC';

        //$this->defaultSort = -4;
        $this->primaryField = 0;


        $this->addField()
            -> setSqlFieldIndex(8) ->setSqlFieldName('V.DatumCas')
            -> setHeaderText('Date') ->setFormat('finedatetime');

        $this->addField( new Prijemce_OdesilatelField() );
    }


    function getSQL($sqlFilter = '')
    {
        // TAKTO TO NEJDE, protoze v UNION nemuze byt memo. Mozna to pujde nacist bez nich a pripojit je dotatecne:
        // SELECT V.Id, V2.Popis ... FROM ( ... UNION ... UNION ... ) JOIN Vzkaz V2 ...
//        return
//            // moje vzkazy (= ode me)
//            'SELECT ' . $this->sqlSelectPart . ' FROM ' . $this ->sqlFromPart .
//            " WHERE V.RCOdesilatele = '" . $this ->database ->addSlashes(User::getRC()) . "' " .
//            // vzkazy pro me
//            ' UNION ' .
//            'SELECT ' . $this->sqlSelectPart . ' FROM ' . $this ->sqlFromPart .
//            ' WHERE V.ZobrazitNaIntrawebu = 1' .
//            "   AND V.RCPrijemce = '" . $this ->database ->addSlashes(User::getRC()) . "' ";
//            //' UNION ' .
//            // ...


        return parent::getSQL($sqlFilter);
    }


    function renderFilter()
    {
        echo '<div class="margin-bottom-2"><a class="text_link" href="?menuid=vzkazy&action=novyvzkaz">' . \Ron\ronText('Novy_vzkaz') . '</a></div>';
    }



    function zobrazPrilohy($values)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DokumentVzkazu.IdDokumentu, DokumentVzkazu.PlatnostOd, VerzeDokumentu.JmenoSouboru ' .
            ' FROM DokumentVzkazu, VerzeDokumentu ' .
            ' WHERE VerzeDokumentu.Id = DokumentVzkazu.IdDokumentu ' .
            '   AND VerzeDokumentu.PlatnostOd = DokumentVzkazu.PlatnostOd ' .
            '   AND DokumentVzkazu.IdVzkazu = ' . $params ->addParameter((int)$values[0]);

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            echo "<tr><td class='d-xs-none'></td><td class='prilohavzkazu' colspan='2'>" . \Ron\ronText('Priloha_vzkazu') .
                ": <a target='_blank' href='?menuid=vzkazy&action=download&idvzkazu={$values[0]}&iddokumentu={$row[0]}&platnostodtimestamp=" .
                $this ->database ->stringToTimestamp($row[1]) . "'>" . StringUtils::htmlTextConvert($row[2]) . "</a></td></tr>\n";
        }
        $cursor ->close();
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, 'browsercolor2', $rowNumber, $edit);

        // predmet
        echo "<tr><td class='d-xs-none'></td><td class='predmetvzkazu' colspan='2'>" . StringUtils::htmlTextConvert($values[4]) . "</td></tr>\n";

        // text vzkazu
        echo "<tr><td class='d-xs-none'></td><td class='textvzkazu' colspan='2'>" . StringUtils::longTextToHtml($values[1], true) . "</td></tr>\n";

        $this->zobrazPrilohy($values);

        // vyjadreni
        if ($values[2]) {
            echo "<tr><td class='d-xs-none'></td><td class='textvzkazu' colspan='2'><b>" . \Ron\ronText('Vyjadreni') . ":</b><br/>" . StringUtils::longTextToHtml($values[2], true) . "</td></tr>\n";
        }

        if (!$values[3] && $values[7] === User::getRC()) // zprava neni zpracovana a je pro me
        {
            // odkaz na vlozeni / upraveni vyjadreni
            $text = \Ron\ronText($values[2] ? 'Upravit_vyjadreni' : 'Napsat_vyjadreni');

            echo "<tr><td class='d-xs-none'></td><td><a href='?menuid=vzkazy&action=vyjadreni&record={$values[0]}'>$text</a></td></tr>\n";
        }

        $this->renderSeparator();
    }

}
