<?php

namespace RonPortal\Controller;


use \Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\RonPortal\Zamestnanci\PracovniPomerOsoby;
use Ron\StringUtils;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;


/**
 * Controller, ktery umoznuje nacist pritomnost zamestnancu.
 */
class PritomnostController extends JsonController
{
    /** @var Database $database */
    protected $database;

    /** @var PravaNaOsoby   Pravo, ucrujici, ktere zamestnance muze uzivatel monitorovat */
    protected $pravoNaPritomnost;

    public function __construct($database, $pravoNaPritomnost)
    {
        $this->database = $database;
        $this ->pravoNaPritomnost = $pravoNaPritomnost;
    }


    /** Vrati JSON objekt s informacemi o poslednim pruchodu zvolene osoby/konkretniho prac. pomeru.
     * @param JsonOutputObject $json
     * @param int $idPPosoby id libovolneho pracovniho pomeru osoby (budou se kontrolovat vsechny PP osoby)
     * @param int $idPP id konkretniho pracovniho pomeru
     * @throws Exception
     * @throws NeniPravoException
     */
    private function pritomnostZamestnance($json, $idPPosoby, $idPP)
    {
        //vyhledani posledniho pruchodu pro osobu, vyhledava se podle RC (RC se najde podle libovolneho ID prac. pomeru osoby)
        if ($idPPosoby > 0) {
            $osobaRC = DBUtils::getSqlValue($this->database, "SELECT RC FROM PracovniPomer WHERE ID = " . (int)$idPPosoby);
            if (!$this ->pravoNaPritomnost->lzeZobrazitRC($osobaRC, time(), time())) {
                throw new NeniPravoException();
            }

            $posledniOperace = DochazkaUtils::vratPosledniPruchodSQL($this->database, time(), $osobaRC, null, null, false);
        } //vyhledani posledniho pruchodu pro konkretni prac. pomer
        elseif ($idPP > 0) {
            if (!$this ->pravoNaPritomnost->lzeZobrazitPP($idPP, time(), time())) {
                throw new NeniPravoException();
            }

            $pracovniPomer = PracovniPomerOsoby::vratPP($this->database, $idPP);

            if (count($pracovniPomer) != 1) {
                throw new Exception('Zamestnanec nenalezen.');
            } elseif (count($pracovniPomer) == 1) {
                $pracovniPomer = current($pracovniPomer);
            }

            $posledniOperace = DochazkaUtils::vratPosledniPruchodSQL($this->database, time(), $pracovniPomer->rc, $pracovniPomer->osobniCislo, $pracovniPomer->datumNastupu);
        }

        if (!isset($posledniOperace)) {
            throw new Exception(StringUtils::fromUTF8('Nebyl nalezen posledni pruchod'));
        }

        //$json->addData('pritomnost', ['pritomen' => $posledniOperace[4], 'datumcasoperace' => $posledniOperace[0], 'popisoperace' => StringUtils::jsonText($posledniOperace[3])]);
        $data = [$idPP => [
            'idpp' => $idPP,
            'pritomnost' => (int)$posledniOperace[4],
            'datumcasoperace' => $posledniOperace[0],
            'popisoperace' => StringUtils::jsonText($posledniOperace[3])]
        ];

        $json->addData('data', $data);
    }



    /**
     * @param JsonOutputObject $json
     * @param string $kodRozdeleni1
     * @param int $vcetnePodrizenych1
     * @param string $kodRozdeleni2
     * @param int $vcetnePodrizenych2
     * @param bool $jenRC
     * @throws Exception
     * @throws \Ron\DB\DBException
     */
    private function pritomnostZamestnancu($json, $kodRozdeleni1, $vcetnePodrizenych1, $kodRozdeleni2, $vcetnePodrizenych2, $jenRC)
    {
        $sql =
            'SELECT PracovniPomer.Id, PuvDochProMonitorovani.Pritomnost, PuvDochProMonitorovani.DatumCasOperace, OperaceDochazky.Popis,' .
            '  Osoba.Prijmeni, Osoba.Jmeno, PracovniPomer.OsobniCislo, ' .
            '  (SELECT MAX(PP.ID) FROM PracovniPomer PP WHERE PP.RC = Osoba.RC)' . // idpp pro osoby
            ' FROM PracovniPomer INNER JOIN PuvDochProMonitorovani' .
            '  ON PuvDochProMonitorovani.RC = PracovniPomer.RC' .
            '  AND PuvDochProMonitorovani.OsobniCislo = PracovniPomer.OsobniCislo' .
            '  AND PuvDochProMonitorovani.DatumNastupu = PracovniPomer.DatumNastupu' .
            ' INNER JOIN Osoba' .
            '  ON Osoba.RC = PracovniPomer.RC' .
            ' LEFT OUTER JOIN OperaceDochazky' .
            '  ON OperaceDochazky.Kod = PuvDochProMonitorovani.KodOperace' .
            ' WHERE 1 = 1';

        // filtr rozdeleni 1
        $filtrRozdeleni1 = ($kodRozdeleni1 != '') && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)
            ? RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
                'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $kodRozdeleni1,
                time(), time(), $vcetnePodrizenych1)
            : '';

        if ($filtrRozdeleni1) {
            $sql .= "\nAND " . $filtrRozdeleni1;
        }

        // filtr rozdeleni 2
        $filtrRozdeleni2 = ($kodRozdeleni2 != '') && RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)
            ? RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
                'PracovniPomer', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $kodRozdeleni2,
                time(), time(), $vcetnePodrizenych2)
            : '';

        if ($filtrRozdeleni2) {
            $sql .= "\nAND " . $filtrRozdeleni2;
        }

        // filtr prava
        $filtrPrava = Prava::sqlFiltrPodlePrav($this ->pravoNaPritomnost, 'PracovniPomer');
        if ($filtrPrava) {
            $sql .= "\nAND " . $filtrPrava;
        }

        $sql .= ' ORDER BY PuvDochProMonitorovani.DatumCasOperace';


        $cursor = $this ->database ->getSqlResult($sql);

        $data = [];
        while ($row = $cursor ->fetch()) {
            $casOd = RonPortalSettings::getIntValue('monitorovani_dnu') > 0
                ? DateUtils::incDay(time(), -RonPortalSettings::getIntValue('monitorovani_dnu'))
                : DateUtils::startOfTheDay(time());

            $casOperace = $this ->database ->stringToTimestamp($row[2]);

            $data[$jenRC ? (int)$row[7] : (int)$row[0]] = [
                'idpp' => $row[0],
                'pritomnost' => ($casOperace >= $casOd) ? (int)$row[1] : 0,
                'datumcasoperace' => StringUtils::format('timestamp', $casOperace),
                'popisoperace' => StringUtils::jsonText($row[3]),
                'jmeno' => StringUtils::jsonText($row[4] . ' ' . $row[5]),
                'osobnicislo' => StringUtils::jsonText($row[6])
            ];
        }

        $json ->addData('data', $data);
    }


    /**
     * @param JsonOutputObject $output
     * @throws Exception
     */
    protected function pritomnost($output)
    {
        if (isset($_GET['osoba'])) {
            $this->pritomnostZamestnance($output, (int)$_GET['osoba'], (int)$_GET['osobapp']);
        } else {
            $this->pritomnostZamestnancu($output,
                $_GET['rozdeleni1'], (int)$_GET['vcetnepodrizenych1'], $_GET['rozdeleni2'], (int)$_GET['vcetnepodrizenych2'],
                isset($_GET['jenrc']) && (int)$_GET['jenrc']);
        }
    }



    function handleJsonAction($action, $output)
    {
        if (($_GET['action'] ?? '')  ===  'pritomnost') {
            $this ->pritomnost($output);
        }
    }

}
