<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;


use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ComboBoxes;
use Ron\DB\Database;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use RonPortal\PracovniPomer;
use RonPortal\PravaNaOsoby;
use RonPortal\RozdeleniUtils;
use RonPortal\TypRozdeleni;

class ComboBoxesRozdeleni extends ComboBoxes
{
    /** @var Database */
    protected $database;

    /** @var string */
    public $name;
    /** @var string */
    public $id;

    /** @var TypRozdeleni */
    public $typRozdeleni;

    /** @var PravaNaOsoby */
    public $prava = null;

    /** @var string     Vycet povolenych kodu rozdeleni oddelenych strednikem */
    public $vycetRozdeleni;

    /**
     * ComboBoxesRozdeleni constructor.
     * @param Database $database
     * @param string $name
     * @param string $id
     */
    public function __construct(Database $database, $name, $id)
    {
        $this->database = $database;

        $isError = MessageManager::getInstance() ->isError($name);

        $combo1 = new ComboBox($name);
        if ($id) {
            $combo1 ->htmlParams['id'] = "{$id}_popis";
        }
        $combo1 ->htmlParams['data-ron-select2'] = $name . '_kod';
        $combo1 ->htmlParams['data-max-visible-rows'] = '200';
        //$combo1 ->htmlParams = ($id ? "id='{$id}_popis' " : '') . "data-ron-select2='{$name}_kod'";
        $combo1 ->className .= ' form-control ronselect' . ($isError ? ' error' : '');


        $combo2 = new ComboBox($name.'_kod');
        //$combo2 ->htmlParams = ($id ? "id='{$id}_kod' " : '') . "data-ron-select2='$name'";
        if ($id) {
            $combo2 ->htmlParams['id'] = "{$id}_kod";
        }
        $combo2 ->htmlParams['data-ron-select2'] = $name;
        $combo2 ->htmlParams['data-drop-down-align'] = 'right';
        $combo2 ->htmlParams['data-max-visible-rows'] = '200';
        $combo2 ->className .= ' form-control ronselect' . ($isError ? ' error' : '');

        parent::__construct($combo1, $combo2);
    }


    /**
     * @param string $sql
     * @param Parameters $params
     * @param string $aktualniHodnota
     * @param bool $povinne
     */
    private function loadData($sql, $params, $aktualniHodnota, $povinne)
    {
        // combo 1 - popis
        $this ->combo1
            ->setAlwaysShowSelected(isset($aktualniHodnota) && $aktualniHodnota != '')
            ->addHtmlParam('data-columns', 'text;value')
            ->setEmptyRow((isset($aktualniHodnota) && $aktualniHodnota != ''  &&  $povinne) ? null : ' ')
            ->setAttrIndexes(['title' => 1, 'class' => 2]);

        if ($this ->vycetRozdeleni) {
            $this ->combo1 ->addHtmlParam('data-vycet-rozdeleni', $this ->vycetRozdeleni);
        }

        $this ->combo1
            ->loadData($this ->database, $sql.' ORDER BY Popis', $aktualniHodnota, $params);

        // combo 2 - kod
        $this ->combo2
            ->setAlwaysShowSelected(isset($aktualniHodnota)  &&  $aktualniHodnota != '')
            ->setCssClassIndex(2)
            ->setCaptionIndex(0)
            ->setTitleIndex(1)
            ->addHtmlParam('data-columns', 'value;popis')
            ->setAttrIndexes(['data-popis' => 1])
            ->setEmptyRow((isset($aktualniHodnota) && $aktualniHodnota != ''  &&  $povinne) ? null : ' ');

        if ($this ->vycetRozdeleni) {
            $this ->combo1 ->addHtmlParam('data-vycet-rozdeleni', $this ->vycetRozdeleni);
        }

        $this ->combo2
            ->loadData($this ->database, $sql.' ORDER BY Kod', $aktualniHodnota, $params);
    }


    /**
     * Prida zadane HTML parametry k obema comboboxum
     * @param string[] $params
     */
    public function addHtmlParams($params)
    {
        $this ->combo1 ->htmlParams = array_merge($this ->combo1 ->htmlParams, $params);
        $this ->combo2 ->htmlParams = array_merge($this ->combo2 ->htmlParams, $params);
    }


    /**
     * @param int $pouzeVlastni 0=vse, 1=pouze vlastni, 2=
     * @param PracovniPomer|string $pp PP, pro ktery se nacitaji rozdeleni. Pokud je zadan string, tak se filtruje jen podle RC
     * @param int $typRozdeleni
     * @param string $aktualniHodnota
     * @param bool $povinne
     * @param null $platnostOd
     * @param null $platnostDo
     */
    public function loadVlastniRozdeleni($pouzeVlastni, $pp, $typRozdeleni, $aktualniHodnota, $povinne, $platnostOd = null, $platnostDo = null)
    {
        if (!$platnostOd) {
            $platnostOd = time();
        }

        if (!$platnostDo) {
            $platnostDo = $platnostOd;
        }

        $sqlStatement = RozdeleniUtils::sqlSeznamRozdeleniOsoby($this ->database,
            $pp, $typRozdeleni, $platnostOd, $platnostDo, $pouzeVlastni,
            'Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass');

//        $params = $this ->database ->createParameters();

//        $sql =
//            'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
//            '  FROM RozdeleniOsob, Rozdeleni'.
//            ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
//            '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($this ->database ->timestampToParam($platnostOd)) . ' OR Rozdeleni.DatumZruseni IS NULL)'.
//            '   AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
//            '   AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
//            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->database ->timestampToParam($platnostDo)) . ' OR RozdeleniOsob.PlatnostDo IS NULL)'.
//            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->database ->timestampToParam($platnostOd)) .
//            (($pp  &&  is_string($pp))
//                ? '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp)
//                : '   AND RozdeleniOsob.RC = ' . $params->addParameter($pp ? $pp->rc : '') .
//                  '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($pp ? $pp->osobniCislo : '') .
//                  '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($pp ? $pp->datumNastupu : 0))
//            ) .
//            // ... OR ( Rozdeleni.Kod = '".db_addslashes($hodnota)."'" ... - nepouzivat - je moc pomale !
//            "   AND (cssClass IS NULL  OR  cssClass <> 'hidden')";

        $this ->loadData(
            $sqlStatement ->sqlText .
            " AND (cssClass IS NULL  OR  cssClass <> 'hidden')" .
            ($this ->vycetRozdeleni
                ? ' AND (' . SQLUtils::getINCondition($this ->database, 'Rozdeleni.Kod', explode(';', $this->vycetRozdeleni), $sqlStatement ->parameters) . ')'
                : ''),
            $sqlStatement ->parameters,
            $aktualniHodnota,
            $povinne);
    }


    public function loadVsechnaRozdeleni($typRozdeleni, $aktualniHodnota, $povinne, $platnostOd = null, $platnostDo = null)
    {
        if (!$platnostOd) {
            $platnostOd = time();
        }

        if (!$platnostDo) {
            $platnostDo = $platnostOd;
        }

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT Kod, Popis, cssClass FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND ( (' .
            ($aktualniHodnota != '' ? 'Rozdeleni.Kod = ' . $params->addParameter($aktualniHodnota)  : '0 = 1')   // zvolenou hodnotu zobrazi vzdy - pozor pri volbe "pouze vlastni" je to moc pomale
            . ')' .
            ' OR (' .
            '   (DatumZruseni >= ' . $params ->addParameter($this ->database ->timestampToParam($platnostOd)) . ' OR DatumZruseni IS NULL)' .
            '   AND DatumVytvoreni <= ' . $params ->addParameter($this ->database ->timestampToParam($platnostDo))  .
            ' )' .
            ')' .
            " AND (cssClass IS NULL  OR  cssClass <> 'hidden')" .
            ($this ->vycetRozdeleni
                ? ' AND (' . SQLUtils::getINCondition($this ->database, 'Rozdeleni.Kod', explode(';', $this->vycetRozdeleni), $params) . ')'
                : '');

        $this ->loadData($sql, $params, $aktualniHodnota, $povinne);
    }


    /**
     * @param int  $typRozdeleni
     * @param string $aktualniHodnota
     * @param bool $povinne
     * @param int  $platnostOd
     * @param int  $platnostDo
     * @throws Exception
     */
    public function loadRozdeleniPodlePrav($typRozdeleni, $aktualniHodnota, $povinne, $platnostOd = null, $platnostDo = null)
    {
        if (!$this ->prava) {
            throw new Exception('Chyba v ComboBoxesRozdeleni loadRozdeleniPodlePrav - neni nastaven typ prava!');
        }

        $sqlStatement = $this ->prava ->sqlRozdeleni($this ->database,
            'Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass',
            $typRozdeleni, $platnostOd, $platnostDo, false);

        $this ->loadData(
            $sqlStatement ->sqlText .
            " AND (cssClass IS NULL  OR  cssClass <> 'hidden')" .
            ($this ->vycetRozdeleni
                ? ' AND (' . SQLUtils::getINCondition($this ->database, 'Rozdeleni.Kod', explode(';', $this->vycetRozdeleni), $sqlStatement ->parameters) . ')'
                : ''),
            $sqlStatement ->parameters,
            $aktualniHodnota,
            $povinne);
    }


    /**
     * @param bool      $pouzeVlastni
     * @param PracovniPomer|string $pp
     * @param int       $typRozdeleni
     * @param string    $aktualniHodnota
     * @param bool      $povinne
     * @param int       $platnostOd
     * @param int       $platnostDo
     * @throws Exception
     */
    public function loadRozdeleni($pouzeVlastni, $pp, $typRozdeleni, $aktualniHodnota, $povinne, $platnostOd = null, $platnostDo = null)
    {
        $this ->typRozdeleni = TypRozdeleni::getTypRozdeleni($this ->database, $typRozdeleni);

        if ($this ->prava) {
            $this ->loadRozdeleniPodlePrav($typRozdeleni, $aktualniHodnota, $povinne, $platnostDo, $platnostDo);
        } else if ($pouzeVlastni) {
            $this ->loadVlastniRozdeleni($pouzeVlastni, $pp, $typRozdeleni, $aktualniHodnota, $povinne, $platnostOd, $platnostDo);
        } else {
            $this ->loadVsechnaRozdeleni($typRozdeleni, $aktualniHodnota, $povinne, $platnostOd, $platnostDo);
        }
    }

}
