<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\PravaNaOsoby;


/**
 * Class ComboBoxOsoba
 * Vyber osoby - jen RC, ne pracovni pomer.
 * Do HTML z bezpecnostnich duvodu nedava RC, ale prirazene (nektere) idPP
 * @package RonPortal\Controls
 */
class ComboBoxOsoba extends ComboBoxZamestnanec
{
    private $seznamOsob = [];

    function __construct($varName = 'osoba')
    {
        parent::__construct($varName);
        $this ->ronSelectColumns = 'jmeno;osobni-cislo';
        $this ->ronSelectCaptions = TextManager::_('Jmeno') . ';' . TextManager::_('Osobni_cislo');
    }


    /**
     * Prida zaznam pro osobu ze zadaneho DB radku
     * @param Database $database
     * @param array $row
     * @param bool  $rozlisitPP     Urcuje, jestli se maji rozlisit jednotlive PP
     * @param bool  $nastavit       Pokud je true, tak se udaje osoby nastavi do vlastnosti f_...
     */
    protected function pridejOsobu($database, $row, $rozlisitPP, $nastavit)
    {
        if (array_key_exists($row[0], $this ->seznamOsob)) {
            // osoba s RC=$row[0] uz byla pridana (jiny PP) => nedela nic
            return;
        }

        $this ->seznamOsob[$row[0]] = 1;

        parent::pridejOsobu($database, $row, false, $nastavit); // nerozlisovat PP
    }


    /**
     * Vraci hodnotu, ktera se ma pouzit pro zobrazeni
     * @param Database $database
     * @param string|int $defaultValue
     * @return string|int
     */
    function getPostValue($database, $defaultValue)
    {
        if (isset($_POST[$this->varName])) {
            $idPP = $_POST[$this->varName];
        } else if (isset($_GET[$this->varName])) {
            $idPP = $_GET[$this->varName];
        } else {
            return $defaultValue;
        }

        if ($idPP === '') {
            return '';
        }

        if ($this ->defaultRows && isset($this ->defaultRows[$idPP])) {
            return $idPP;
        }

        // Combobox do HTML generuje IdPP, ale je potreba RC => najdu RC pro zadane IdPP
        $params = $database ->createParameters();
        $cursor = $database ->getSqlResult('SELECT RC FROM PracovniPomer WHERE Id = ' . $params ->addParameter((int)$idPP), $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return null;
        }

        return $row[0];
    }


    /**
     * Pomocna funkce - vraci true, pokud je aktualni zaznam ($row) vybrany.
     * @param array  $row
     * @param string $selectedValue
     * @return bool
     */
    protected function compareSelectedValue($row, $selectedValue)
    {
        return $selectedValue == $row[0]; // zaznam ma stejne rc jako vybrana osoba
    }


    /**
     * Nacte seznam osob ze zadaneho dotazu
     * dale nacte promenne pro vybrany zaznam - tj. zaznam z $_POST nebo $selectedvalue (prednost ma hodnota v $_POST)
     * @param Database $database
     * @param SqlStatement|string   $sqlText
     * @param PravaNaOsoby $prava
     * @param bool     $emptyRow
     * @param int      $selectedValue
     * @param int      $datumOd
     * @param int      $datumDo
     * @return bool
     * @throws DBException
     */
    function loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo)
    {
        $this ->seznamOsob = [];
        return parent::loadSQLPP($database, $sqlText, $prava, $emptyRow, $selectedValue, $datumOd, $datumDo);
    }

}
