<?php
/**
 * stanislav.elbl, 12.04.2022 12:52
 */


namespace RonPortal\Controls;

use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\MVC\ViewObject;
use Ron\StringUtils;

class DokumentyView implements ViewObject
{
    /** @var string     URL pro stahnuti dokumentu (na konec URL se prida ID) */
    public $downloadUrl;

    /** @var string[]   Asociativni pole HTML parametru */
    public $htmlParams = [];

    public $cssClass = 'seznam-dokumentu margin-top-2';

    /** @var Database */
    protected $database;

    /** @var mixed[][] */
    protected $data = [];

    /**
     * DokumentyView constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
    }

    /**
     * @param string $downloadUrl
     * @return $this
     */
    public function setDownloadUrl($downloadUrl)
    {
        $this->downloadUrl = $downloadUrl;
        return $this;
    }

    /**
     * @param \string[] $htmlParams
     * @return $this
     */
    public function setHtmlParams($htmlParams)
    {
        $this->htmlParams = $htmlParams;
        return $this;
    }

    /**
     * @param string $cssClass
     * @return $this
     */
    public function setCssClass($cssClass)
    {
        $this->cssClass = $cssClass;
        return $this;
    }


    public static function getSqlSelectPart($dokumentTableName = 'Dokument', $typDokumentuTableName = 'TypDokumentu')
    {
        return
            "$dokumentTableName.Id, $dokumentTableName.Datum, $dokumentTableName.Popis, $dokumentTableName.JmenoSouboru," .
            ($typDokumentuTableName ? " $typDokumentuTableName.Popis" : ' NULL');
    }


    /**
     * Nacte seznam
     * @param SqlStatement|string $sql
     * @return $this
     */
    public function loadData($sql)
    {
        $cursor = $this ->database ->getSqlResult($sql);

        while ($row = $cursor ->fetch()) {
            $this ->data[] = [
                'id' => (int)$row[0],
                'datum' => $this ->database ->stringToTimestamp($row[1]),
                'popis' => $row[2],
                'jmenoSouboru' => $row[3],
                'typDokumentu' => $row[4]
            ];
        }

        $cursor ->close();

        return $this;
    }


    public function getHtmlContent($utf8 = false)
    {
        $result = '';

        foreach ($this ->data as $row) {
            $result .=
                '<tr>' .
                // datum
                '<td>' . StringUtils::format('date', $row['datum']) . '</td>' .
                // jmeno souboru
                '<td><a href="' . $this ->downloadUrl . $row['id'] . '" target="_blank">' .
                StringUtils::htmlTextConvert($utf8 ? StringUtils::toUTF8($row['jmenoSouboru']) : $row['jmenoSouboru']) .
                '</a></td>' .
                // popis
                '<td class="padding-left-2">' . StringUtils::htmlTextConvert($utf8 ? StringUtils::toUTF8($row['popis']) : $row['popis']) . '</div>' .
                '</td>';
//                '<div class="row">' .
//                // datum
//                '<div class="col-2 col-sm-1">' . StringUtils::format('date', $row['datum']) . '</div>' .
//                // jmeno souboru
//                '<div class="col-10 col-sm-5"><a href="' . $this ->downloadUrl . $row['id'] . '" target="_blank">' .
//                StringUtils::htmlTextConvert($utf8 ? StringUtils::toUTF8($row['jmenoSouboru']) : $row['jmenoSouboru']) .
//                '</a></div>' .
//                // popis
//                '<div class="col-12 col-sm-5 padding-left-2">' . StringUtils::htmlTextConvert($utf8 ? StringUtils::toUTF8($row['popis']) : $row['popis']) . '</div>' .
//                '</div>';
        }

        return '<table>' . $result . '</table>';
    }


    public function render()
    {
        echo
            '<div ' . GuiUtils::getAttributesHtml(array_merge(['class' => $this ->cssClass], $this ->htmlParams)) . '>' .
            $this ->getHtmlContent() .
            "</div>\n";
    }
}

