<?php

namespace RonPortal\Controls;


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\Kalendar;

/**
 * Filtr, ktery nabizi seznam dnu v obdobi jako linky
 */
class FilterFormFilterDny extends FilterFormFilter
{
    /** @var  string|null   SQL vyraz pro nacteni podrobnosti k jednotlivym dnu - pokud je zadan, musi v prvnim sloupci vracet datum */
    protected $sqlPodrobnostiDnu = null;

    /** @var string|null    Nadpis, ktery se zobrazi na zacatku radku s podrobnostmi (druhy radek pod seznamem dnu) */
    protected $nadpisPodrobnostiDnu = null;

    /** @var array          Seznam podrobnosti k jednotlivym dnum - standardne se nacita SQL dotazem sqlPodrobnostiDnu */
    protected $podrobnosti;

    /** @var array          Seznam CSS class pro jednotlive dny - standardne se nastavi podle typu dne (pracovni/vikend) */
    protected $dayCssClasses;


    public function __construct(RonPortalFilterForm $filterForm, string $name)
    {
        parent::__construct($name);

        $this ->filterForm = $filterForm;

        $this
            ->setRowId('filter_dnyobdobi')
            ->setRowClass('margin-top-2 margin-bottom')
            ->setPosition(self::POSITION_BELOW_FORM);
    }

    /**
     * @return string|null
     */
    public function getSqlPodrobnostiDnu(): ?string
    {
        return $this->sqlPodrobnostiDnu;
    }

    /**
     * @param string|null $sqlPodrobnostiDnu
     * @return FilterFormFilterDny
     */
    public function setSqlPodrobnostiDnu(?string $sqlPodrobnostiDnu): FilterFormFilterDny
    {
        $this->sqlPodrobnostiDnu = $sqlPodrobnostiDnu;
        return $this;
    }


    /**
     * @return string|null
     */
    public function getNadpisPodrobnostiDnu(): ?string
    {
        return $this->nadpisPodrobnostiDnu;
    }

    /**
     * @param string|null $nadpisPodrobnostiDnu
     * @return FilterFormFilterDny
     */
    public function setNadpisPodrobnostiDnu(?string $nadpisPodrobnostiDnu): FilterFormFilterDny
    {
        $this->nadpisPodrobnostiDnu = $nadpisPodrobnostiDnu;
        return $this;
    }




    protected function buildSqlPodrobnostiDnu()
    {
        return $this ->sqlPodrobnostiDnu;
    }


    public function getHtmlPodrobnostiKeDni($row)
    {
        return $row[1] ?? '';
    }



    public function loadCssClasses(Database $database)
    {
        $datum = $this ->filterForm ->getOmezitObdobiOd();

        // nacte kalendar prirazeny vybrane osobe
        $kodKalendare = null;
        $idPP = $this ->filterForm ->getFilterValue('osoba');
        if ($idPP) {
            $kalendare = PrirazeniKalendare::nactiKalendarProObdobi($database, $idPP, $datum, $datum);
            if ($kalendare) {
                $kodKalendare = $kalendare[0] ->kod;
            }
        }

        if ($kodKalendare === null  &&  $this ->filterForm ->getFilterValue('rozdeleni') !== null) {
            $kodKalendare = Kalendar::vratKalendarRozdeleni($database,
                $this ->filterForm ->getFilterValue('rozdeleni', 'typRozdeleni'),
                $this ->filterForm ->getFilterValue('rozdeleni')
            );
        }

        if ($kodKalendare !== null) {
            Kalendar::nactiSvatkyVObdobi($database,
                $this ->filterForm ->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo(),
                $kodKalendare);
        }

        // nacte css tridu pro jednotlive dny v obdobi
        while ($datum <= $this ->filterForm ->getOmezitObdobiDo()) {
            $className = Kalendar::getStyleForDate($database, $datum, $kodKalendare !== null, $kodKalendare);
            $this ->dayCssClasses[(int)date('j', $datum)] = $className;

            $datum = DateUtils::incDay($datum);
        }
    }


    public function loadPodrobnostiDnu(Database $database)
    {
        $this ->podrobnosti = [];

        $sqlPodrobnosti = $this ->buildSqlPodrobnostiDnu();
        if ($sqlPodrobnosti) {
            $cursor = $database ->getSqlResult($sqlPodrobnosti);
            while ($row = $cursor ->fetch()) {
                $date = $database ->stringToTimestamp($row[0]);
                $den = (int)date('j', $date);
                $this ->podrobnosti[$den] = $row;
            }
        }
    }


    public function load(Database $database)
    {
        if (!$this->enabled) {
            return;
        }

        $valueStr = trim($_GET[$this->name] ?? '');

        if ($valueStr !== '') {
            try {
                $datum = DateUtils::dateStringToTimestamp($valueStr);
                $this ->value = $datum;
            } catch (\Exception $e) {
                $this->loadDefaultValue($database);
            }

            $this->storeValue($database);
        } else {
            $this->loadDefaultValue($database);
        }

        if ($this ->value < $this ->filterForm ->getOmezitObdobiOd()  ||  $this ->value > $this ->filterForm ->getOmezitObdobiDo()) {
            $this ->value = ($this ->filterForm ->getOmezitObdobiOd() < time()  &&  $this ->filterForm ->getOmezitObdobiDo() >= DateUtils::today())
                ? DateUtils::today()
                : $this ->filterForm ->getOmezitObdobiOd();
        }

        $this ->loadPodrobnostiDnu($database);
        $this ->loadCssClasses($database);
    }


    public function render()
    {
//        $podrobnosti = [];
//
//        $sqlPodrobnosti = $this ->buildSqlPodrobnostiDnu();
//        if ($sqlPodrobnosti) {
//            $cursor = $this ->database ->getSqlResult($sqlPodrobnosti);
//            while ($row = $cursor ->fetch()) {
//                $date = $this ->database ->stringToTimestamp($row[0]);
//                $den = (int)date('j', $date);
//                $podrobnosti[$den] = $row;
//            }
//        }

        echo
            "<div id='{$this ->rowId}' class='{$this ->rowClass}'>" .
            '<input ' . GuiUtils::getAttributesHtml(
                array_merge(
                    [
                        'type' => 'hidden',
                        'name' => $this ->name,
                        'id' => 'filter_dny_obdobi_' . $this ->name,
                        'value' => DateUtils::dateToUrlString($this ->value)
                    ],
                    $this ->htmlAttributes ?: []
                )
            ) .
            '/>';

        $htmlElement = 'span';
        if ($this ->podrobnosti) {
            echo
                '<table><tr>' . ($this ->nadpisPodrobnostiDnu ? '<td></td>' : '');
            $htmlElement = 'td';
        }

        $rowPodrobnosti = $this ->nadpisPodrobnostiDnu
            ? '<td>' . $this ->nadpisPodrobnostiDnu . '</td>'
            : '';

        $datum = $this ->filterForm ->getOmezitObdobiOd();
        while ($datum <= $this ->filterForm ->getOmezitObdobiDo()) {
            $index = (int)date('j', $datum);

            $className =
                'js margin-right padding-all setValue ' . ($this ->dayCssClasses[$index] ?? '') .
                ($datum === $this ->value ? ' font-weight-bold' : '');

            $valueStr = DateUtils::dateToUrlString($datum);

            echo
                "<$htmlElement class='$className' data-selector='#filter_dny_obdobi_datum' data-value='$valueStr' data-submit='1'>" .
                date('j.n.', $datum) .
                "</$htmlElement>";

            $rowPodrobnosti .= '<td class="align-center">' . $this ->getHtmlPodrobnostiKeDni($this ->podrobnosti[$index] ?? null) . '</td>';

            $datum = DateUtils::incDay($datum);
        }

        if ($this ->podrobnosti) {
            echo
                '</tr>' .
                "<tr>$rowPodrobnosti</tr>" .
                '</table>';
        }

        echo '</div>';
    }
}

