<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\Browser;
use Ron\DB\Database;
use Ron\TextManager;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;

/**
 * Trida odvozena od Browser, navic pridava tuto funkcnost:
 * - nad seznamem umoznuje zobrazit filtrovaci formular (@see RonPortalFilterForm)
 * - nacita konfiguraci z XML formulare config_<browserCode>. Konfigurace umoznuje
 *   - pridavat, skryvat sloupce a menit jejich vlastnosti
 *   - menit poradi sloupcu
 *   - doplnit texty pro preklady (@see TextManager)
 *   - zobrazit XML formular nad nebo pod seznamem
 */
class RonPortalBrowser extends Browser
{
    public $action;

    /** @var RonPortalFilterForm  */
    public $filterForm;
    public $filterFormUseGet = false;
    public $filterFormAppCode; // kod aplikace pro vyber obdobi
    public $filterFormRights;  // typ prav pro vyber osob
    public $filterFormVcetneExternich = false;

//    /** @var  XmlFormsList  Seznam XML formularu, ktere se zobrazit nad/pod seznamem*/
//    public $xmlFormsList;

    /**
     * @param Database $database
     * @param string $action
     */
    function __construct($database, $action)
    {
        parent::__construct($database);

        $this ->action = $action;
        $this ->browserUrl = "?menuid=$action&";

        $this ->filterFormAppCode = 'ADS';
        $this ->filterFormRights = 'prohlizeni';

        $this ->maxPageRows = 200;

//        $this ->xmlFormsList = new XmlFormsList();
    }


    /**
     * @param string $action
     * @return RonPortalBrowser
     */
    public function setAction(string $action): RonPortalBrowser
    {
        $this->action = $action;
        return $this;
    }


    const OSOBY_BEZ_EXTERNICH = 0;
    const OSOBY_VCETNE_EXTERNICH = 1;
    /** @var int    K vyberu osoby prida filtr ZpracovavatDochazku = 1 */
    const OSOBY_JEN_S_DOCHAZKOU = 2;


    /**
     * @param string $obdobiAppCode Kod aplikace, ze ktere ma nacitat obdobi
     * @param string|PravaNaOsoby $rights
     * @param bool $povolitVsechnyOsoby
     * @param bool|int $vcetneExternich     Urcuje, jestli se maji nacist i externi zamestnanci. Pokud je 2, tak navic profiltruje jen osoby, kterym se zpracovava dochazka
     * @param bool $oldVersion  Pouze pro kompatibilitu
     * @return RonPortalFilterForm
     */
    function createFilterForm($obdobiAppCode = null, $rights = null, $povolitVsechnyOsoby = false, $vcetneExternich = false, $oldVersion = false)
    {
        $filterForm = new RonPortalFilterForm($this ->database, $this ->action, !$oldVersion  ||  $this ->filterFormUseGet);

        // prida filtr na obdobi
        if ($obdobiAppCode) {
//            if ($oldVersion) {
//                $filterForm->nastavObdobi(true, $obdobiAppCode, false); // jenotevrene = false
//            } else {
                $filterForm ->addFilter(
                    (new FilterFormFilterObdobi($filterForm, $obdobiAppCode))
                        ->setRequired(true),
                    true);
//            }
        }

        $prava = is_string($rights)
            ? Prava::vratPrava($rights)
            : $rights;

        // prida filtry na rozdeleni
        if (Prava::pravaNaViceOsob($rights)) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $filterForm ->addFilterRozdeleni1(null, $prava);
//            $filterForm ->nastavRozdeleni(true);
//            $filterForm ->f_pravarozdeleni = $prava;
//            $filterForm->vcetnePodrizenychRozdeleni = true;

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $filterForm ->addFilterRozdeleni2(null, $prava);
//                $filterForm ->nastavRozdeleni2(true);
//                $filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        // prida filtr na osobu
        if ($oldVersion) {
            $filterForm->nastavOsoby(true, $prava, 0, 0, $vcetneExternich); // datum od-do podle obdobi
        } else {
            $filterPP = $filterForm ->addFilterPP()
                ->setPovolitVsechnyOsoby($povolitVsechnyOsoby)
                ->setVcetneExternich($vcetneExternich === true  ||  $vcetneExternich === 1)
                ->setPrava($prava);

            if ($vcetneExternich === 2) {
                $filterPP ->setOptionsSqlFilter('PracovniPomer.ZpracovatDochazku = 1');
            }
        }

        return $filterForm;
    }


    function setFilterForm()
    {
        $this ->filterForm = $this->createFilterForm(
            $this->filterFormAppCode, $this ->filterFormRights,
            false, $this ->filterFormVcetneExternich,
            true);

//        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, $this ->filterFormUseGet);
//
//        if ($this ->filterFormAppCode) {
//            $this->filterForm->nastavObdobi(true, $this->filterFormAppCode, false); // jenotevrene = false
//        }
//
//        $prava = is_string($this ->filterFormRights)
//            ? Prava::vratPrava($this ->filterFormRights)
//            : $this ->filterFormRights;
//
//        if (Prava::pravaNaViceOsob($this ->filterFormRights)) { // napr. pro "pouze vlastni" se vyber nezobrazuje
//            $this ->filterForm ->addFilterRozdeleni1(null, $prava);
////            $this ->filterForm ->nastavRozdeleni(true);
////            $this ->filterForm ->f_pravarozdeleni = $prava;
////            $this->filterForm->vcetnePodrizenychRozdeleni = true;
//
//            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
//                $this ->filterForm ->addFilterRozdeleni2(null, $prava);
////                $this ->filterForm ->nastavRozdeleni2(true);
////                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
//            }
//        }
//
//        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich); // datum od-do podle obdobi
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'filterform') {
            if ($this ->filterForm  &&  $value instanceof \ArrayAccess) {
                foreach ($value as $item) {
                    $this ->filterForm ->loadConfigurationArray($item);
                }
            }
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }

    /**
     * @param \ArrayAccess $colSettings
     */
    public function loadColumnConfiguration($colSettings)
    {
        if (!$colSettings) {
            return;
        }

        //if (!$colSettings['profil']  ||  User::jeVProfilu($colSettings['profil'])) {
        if ($this ->objectConfiguration ->configurationItemEnabled($colSettings)) {
            parent::loadColumnConfiguration($colSettings);
        }
    }


//    public function loadConfigurationItem($key, $value, $valueText)
//    {
//        if ($key === 'xml') {
//            // xml formular, ktery se ma zobrazit
//            $this ->xmlFormsList ->loadSettings($value);
//        } else {
//            parent::loadConfigurationItem($key, $value, $valueText);
//        }
//    }


//    /**
//     * Nacte XML konfiguraci browseru se zadanym kodem
//     * @param Database $database
//     * @param string $browserCode
//     * @return string|null
//     * @deprecated
//     */
//    public static function loadBrowserConfiguration($database, $browserCode)
//    {
//        return RonPortalObjectConfiguration::loadConfigurationFromDatabase($database, $browserCode);
//    }


    /**
     * Nacte konfiguraci browseru z XML sablony
     * @throws \Exception
     */
    function loadConfiguration()
    {
        if (!$this ->code) {
            return;
        }

        $this ->objectConfiguration = new RonPortalObjectConfiguration($this ->database, $this);
        $this ->objectConfiguration ->loadConfiguration($this ->code);
    }


    /**
     * Nacte konfiguraci ze zadaneho xmlTextu
     * @param string $xmlText
     */
    public function loadConfigurationXml($xmlText)
    {
        $this ->objectConfiguration = new RonPortalObjectConfiguration($this ->database, $this);
        $this ->objectConfiguration ->loadConfigurationXml($xmlText);
    }


    function getXmlVariables()
    {
        return $this ->filterForm
            ? $this ->filterForm ->getVariables()
            : [];

//        if ($this ->filterForm) {
//            $filtrObdobi = $this ->filterForm ->getFilter('obdobi');
//            if ($filtrObdobi instanceof FilterFormFilterObdobi) {
//                $variables['obdobi'] = $filtrObdobi ->getValue();
//                $variables['obdobiDo'] = $filtrObdobi ->getValue('datumDo');
//            }

//            if ($this ->filterForm ->showOsoby) {
//                $variables['idPP'] = $this ->filterForm ->vybranaOsoba;
//                $variables['RC'] = $this ->filterForm ->vybranaOsobaRC;
//                //$variables['osobniCislo'] = $this ->filterForm ->vybranaOsobaOsobniCislo;
//                $variables['osobniCislo'] = $this ->filterForm ->vybranaOsobaRC;
//                $variables['datumNastupu'] = $this ->filterForm ->vybranaOsobaDatumNastupu;
//            }
//        }

//        return $variables;
    }


    function loadFilterData()
    {
        if ($this ->filterForm) {
            $this ->filterForm ->loadData();
        }
    }


    function load($sqlFilter = '')
    {
        $this ->setFilterForm();
        if ($this ->filterForm) {
            $this ->filterForm ->loadConfiguration();
        }

        $this ->loadConfiguration();

        $this ->loadFilterData();

        if ($this ->objectConfiguration instanceof RonPortalObjectConfiguration  &&  $this ->objectConfiguration ->xmlFormsList) {
            $this ->objectConfiguration ->xmlFormsList ->processXMLForms($this ->database, $this ->getXmlVariables());
        }

        parent::load($sqlFilter);
    }


    function renderFilter()
    {
        if ($this ->objectConfiguration instanceof RonPortalObjectConfiguration) {
            $this->objectConfiguration ->renderXmlForms('head');
        }

        if ($this ->filterForm) {
            $this->filterForm->render();
        } else {
            parent::renderFilter();
        }
    }

    public function renderContent()
    {
        if ($this ->objectConfiguration instanceof RonPortalObjectConfiguration) {
            $this->objectConfiguration ->renderXmlForms('top');
        }

        parent::renderContent();

        if ($this ->objectConfiguration instanceof RonPortalObjectConfiguration) {
            $this->objectConfiguration ->renderXmlForms('bottom');
        }
    }


    public function getSQLFilter()
    {
        return $this ->filterForm
            ? $this ->filterForm ->buildSqlFilter()
            : '';
    }


}
