<?php

namespace RonPortal\Controls;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\Http\Csrf;
use Ron\StringUtils;
use RonPortal\Common\RozdeleniOsobRecord;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;

class RozdeleniOsobyShiftForm extends RozdeleniShiftForm
{
    protected $idPP;

    /** @var PracovniPomer */
    protected $pp;

    /** @var PravaNaOsoby|PravaNaOsoby[]  */
    protected $prava;

    /**
     * RozdeleniShiftForm constructor.
     * @param Database $database
     * @param int    $typRozdeleni
     * @param PravaNaOsoby|PravaNaOsoby[] $prava
     */
    public function __construct($database, $typRozdeleni, $prava, $idPP = null)
    {
        parent::__construct($database, $typRozdeleni, 'rozdeleniosoby');

        $this ->prava = $prava;
        $this ->idPP = $idPP ?? (int)$_GET['idpp'];
    }


    /**
     * @param Parameters $params
     * @param ?string $filter
     * @return string
     */
    protected function getSqlPrirazenaRozdeleni($params, $filter): string
    {
        return
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.PlatnostOd, Rozdeleni.cssClass ' .
            ' FROM RozdeleniOsob, Rozdeleni ' .
            ' WHERE RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
            ' AND RozdeleniOsob.RC = ' . $params->addParameter($this ->pp->rc) .
            ' AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->pp->osobniCislo) .
            ' AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->pp->datumNastupu) .
            ' AND (RozdeleniOsob.PlatnostDo > ' . $params ->addDateParameter(time()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
            ' AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni ' .
            ' AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            ' AND (Rozdeleni.DatumZruseni > ' . $params ->addDateParameter(time()) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            " AND (cssClass IS NULL  OR  cssClass <> 'hidden') " .

            (isset($filter)
                ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . '  OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                : '') .

            ' ORDER BY Rozdeleni.Popis ';
    }


    /**
     * @param Parameters $params
     * @param string $filter
     * @return ?string
     */
    protected function getSqlVsechnaRozdeleni($params, $filter)
    {
        // POZOR! Nefunguje na ODBC s driverem "SQL Server" - je potreba prepnout na jiny driver!
        // nacita vsechna rozdeleni vcetne neprirazenych
        return
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.PlatnostOd, Rozdeleni.cssClass ' .
            ' FROM Rozdeleni LEFT OUTER JOIN RozdeleniOsob ' .
            '  ON RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
            '  AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
            '  AND RozdeleniOsob.RC = ' . $params->addParameter($this ->pp->rc) .
            '  AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->pp->osobniCislo) .
            '  AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->pp->datumNastupu) .
            '  AND (RozdeleniOsob.PlatnostDo > ' . $params ->addDateParameter(time()) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
            ' AND (Rozdeleni.DatumZruseni > ' . $params ->addDateParameter(time()) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            " AND (Rozdeleni.cssClass IS NULL  OR  Rozdeleni.cssClass <> 'hidden') " .

            (isset($filter)
                ? ' AND (Rozdeleni.Popis LIKE ' . $params ->addParameter("%{$filter}%") . ' OR  Rozdeleni.Kod LIKE ' . $params ->addParameter("%{$filter}%") . ')'
                : '') .

            ' ORDER BY Rozdeleni.Popis ';
    }


    public function loadData()
    {
        $this ->pp = PracovniPomer::nactiPP($this ->database, $this ->idPP);

        return parent::loadData();
    }


    /**
     * @param string $kodRozdeleni
     * @throws DBException
     */
    protected function odstranRozdeleni($kodRozdeleni)
    {
        if ($kodRozdeleni) {
            $rec = new RozdeleniOsobRecord($this->database, $this, $this ->pp, $this->typRozdeleni, $kodRozdeleni);
            $rec->checkMultiple = false;
            $rec->deleteRecords();
        }
    }


    /**
     * @throws DBException
     * @throws \Exception
     */
    public function handleMoveLeft()
    {
        $this ->pp = PracovniPomer::nactiPP($this ->database, $this ->idPP);

        // zkontrolovat prava
        if (!Prava::lzeZobrazitPP($this ->prava, $this ->pp ->idPP, time(), time())) {
            throw new NeniPravoException();
        }

        return parent::handleMoveLeft();
    }


    /**
     * @param string $kodRozdeleni
     * @throws DBException
     */
    protected function priradRozdeleni($kodRozdeleni)
    {
        $rec = new RozdeleniOsobRecord($this ->database, $this, $this ->pp, $this ->typRozdeleni, $kodRozdeleni);

        $params = $this ->database ->createParameters();
        $filter =
            'PlatnostOd <= ' . $params ->addParameter($this ->database ->timestampToParam(time())) .
            ' AND (PlatnostDo >=  ' . $params ->addParameter($this ->database ->timestampToParam(time())) . '  OR  PlatnostDo IS NULL )';

        $rec ->checkMultiple = false;
        if (!$rec ->loadRecord($filter, $params)) {
            $rec ->fPlatnostOd ->setNewValue($this ->pp ->datumNastupu);
            $rec ->insertRecord();
        }
    }


    /**
     * @throws NeniPravoException
     * @throws DBException|NeniPravoException
     */
    public function handleMoveRight()
    {
        // zkontrolovat prava!
        $this ->pp = PracovniPomer::nactiPP($this ->database, $this ->idPP);

        if (!Prava::lzeZobrazitPP($this ->prava, $this ->pp ->idPP, time(), time())) {
            throw new NeniPravoException();
        }

        return parent::handleMoveRight();
    }

}
