<?php

namespace RonPortal\Controls;

use Ron\Controls\ShiftForm;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\Http\Csrf;
use Ron\StringUtils;

/**
 * Dialog pro prirazovani rozdeleni zamestnancum
 * Class RozdeleniShiftForm
 */
abstract class RozdeleniShiftForm extends ShiftForm
{
    /** @var int */
    protected $typRozdeleni;

    /** @var string */
    protected $csrfRequestId;

    /**
     * RozdeleniShiftForm constructor.
     * @param Database $database
     * @param int    $typRozdeleni
     */
    public function __construct($database, $typRozdeleni, $csrfRequestId)
    {
        parent::__construct($database);

        $this ->typRozdeleni = (int)$typRozdeleni;
        $this ->csrfRequestId = $csrfRequestId;
    }


    /**
     * Funkce musi vratit SQL vyraz pro nacteni prirazenych rozdeleni - sloupce Kod, Popis, PlatnostOd, CssClass (misto Platnost od staci jakykoliv jiny priznak)
     * @param Parameters $params
     * @param string $filter
     * @return string
     */
    abstract protected function getSqlPrirazenaRozdeleni($params, $filter);


    /**
     * Funkce musi vratit SQL vyraz pro nacteni vsech rozdeleni (vcetne pripadneho datumu prirazeni) - sloupce Kod, Popis, PlatnostOd, CssClass (misto Platnost od staci jakykoliv jiny priznak)
     * @param Parameters $params
     * @param string $filter
     * @return string
     */
    abstract protected function getSqlVsechnaRozdeleni($params, $filter);


    /**
     * @throws DBException
     */
    public function loadData()
    {
        $result = [];

        $forceLoad = isset($_GET['force'])  &&  (int) $_GET['force'];
        $loadAll = !(isset($_GET['jenvlastni']) && (int)$_GET['jenvlastni']); // jen vlastni - pouziva se pro aktualizaci comboboxu

        $filter = isset($_GET['filter'])
            ? urldecode($_GET['filter'])        // iconv('utf-8', 'windows-1250', $_GET['filter'])
            : null;

        $result['filter'] = StringUtils::jsonText($filter);

        $count = null;

        session_write_close(); // aby fungoval .abort() v javascriptu

        // podle poctu existujicich zaznamu automaticky urci, jestli se nactou rovnou vsechna data
        // (pri velkem poctu zaznamu je to pomale - proto se nenacitaji zaznamy ihned)
        if ($loadAll) {
            $params = $this ->database ->createParameters();

            $sql =
                'SELECT COUNT(*) FROM Rozdeleni ' .
                ' WHERE KodTypuRozdeleni = ' . $params ->addParameter($this ->typRozdeleni) .
                ' AND (Rozdeleni.DatumZruseni > ' . $params ->addParameter($this ->database ->timestampToParam(time())) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ";

            if (isset($filter)  &&  $filter !== '') {
                $sql .=
                    '   AND (Popis LIKE ' . $params->addParameter('%' . $filter . '%') .
                    '  OR  Kod LIKE ' . $params->addParameter('%' . $filter . '%') . ')';
            }

            $cursor = $this ->database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();

            $count = $row ? $row[0] : 0;
            $loadAll = $count < 3000;
        }

        $params = $this ->database ->createParameters();
        if ($loadAll  ||  $forceLoad) {
            $sql = $this ->getSqlVsechnaRozdeleni($params, $filter);
            $result['nactenoVse'] = 1;
        } else {
            // nacita jen vlastni rozdeleni
            $sql = $this ->getSqlPrirazenaRozdeleni($params, $filter);
            $result['nactenoVse'] = 0;
        }

        if (isset($count)) {
            // pocet zaznamu
            $result['count'] = (int)$count;
        }

        $result['leftItems'] = [];
        $result['rightItems'] = [];

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            //$obj = ['kod' => jsonText($row[0]), 'popis' => jsonText($row[1])];
            $obj = [
                'kod' => StringUtils::jsonText($row[0], false),
                'popis' => StringUtils::jsonText($row[1], false)
            ];

            if ($row[3]) {
                $obj['cssClass'] = StringUtils::jsonText($row[3]);
            }

            if ($row[2]) {
                // prirazena
                $result['rightItems'][] = $obj;
            } else {
                $result['leftItems'][] = $obj;
            }
        }

        return $result;
    }


    /**
     * @param string $kodRozdeleni
     */
    abstract protected function odstranRozdeleni($kodRozdeleni);


    /**
     * @throws DBException
     * @throws \Exception
     */
    public function handleMoveLeft()
    {
        Csrf::validateCSRFPostParam($this->csrfRequestId);

        for ($i = 0; $i < $_POST['count']; $i++) {
            $kod = StringUtils::fromUTF8($_POST['kod#' . $i]);
            $this ->odstranRozdeleni($kod);
        }

        return [ 'rightAction' => 'remove', 'leftAction' => 'add' ];
    }


    /**
     * @param string $kodRozdeleni
     * @throws DBException
     */
    abstract protected function priradRozdeleni($kodRozdeleni);


    public function handleMoveRight()
    {
        Csrf::validateCSRFPostParam($this->csrfRequestId);

        for ($i = 0; $i < $_POST['count']; $i++) {
            $kod = StringUtils::fromUTF8($_POST['kod#' . $i]);
            $this ->priradRozdeleni($kod);
        }

        return [ 'leftAction' => 'remove', 'rightAction' => 'add' ];
    }

}
