<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal\Controls;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\StringUtils;

/**
 * Obali BrowserField do spanu se tridou podle data ukonceni. Zadane datum ukonceni vypise do elementu title.
 * Rozlisuje 3 stavy:
 * <ul>
 * <li> Platny pracovni pomer
 * <li> Pracovni pomer ukonceny nebo s datem ukonceni v aktualnim mesici (podle data).
 * <li> Pracovni pomer s datem ukonceni v budoucnu.
 * </ul>
 * @package Ron\RonPortal
 */
class UkonceniPracPomeruInfoField extends BrowserField
{
    /** @var  int index pole s datem ukonceni prac. pomeru */
    private $indexDatumUkonceniPP;

    /**
     * Obali BrowserField do spanu se tridou podle data ukonceni. Vice info v popisu tridy.
     * @param null|Database $database
     * @param array|null $options
     * @param int $indexDatumUkonceni index pole s datem ukonceni prac. pomeru
     */
    public function __construct($database, $indexDatumUkonceni, $options = null)
    {
        parent::__construct($database, $options);

        $this->indexDatumUkonceniPP = $indexDatumUkonceni;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $rowClass = 'browser_platny_pp';
        $title = '';

        if ($rowData[$this->indexDatumUkonceniPP]) {
            $datumUkonceni = $this->database->stringToTimestamp($rowData[$this->indexDatumUkonceniPP]);

            if ($datumUkonceni <= DateUtils::lastDayOfMonth(time())) {
                $rowClass = 'browser_ukonceny_pp';
            }
            else {
                $rowClass = 'browser_ukonceni_pp_budoucnost';
            }

            $title = \Ron\ronText('Browser_datum_ukonceni_pp') . ': ' . StringUtils::format('date', $datumUkonceni);
        }

        return '<span class="' . $rowClass . '" title="' . $title . '">' . parent::getViewText($value, $rowNumber, $recordId, $rowData) . ($rowClass ? '</span>' : '');
    }
}
