<?php
/**
 * stanislav.elbl, 11.06.2021 16:08
 */

namespace RonPortal\Dashboard;

use RonPortal\Prava;

/**
 * Seznam panelu, ktere muze uzivatel videt
 * Class DashboardFrames
 */
class DashboardFrames
{
    /** @var DashboardFrame[] */
    public $frames = [];

    public function addFrame($id, $dataUrl, $caption)
    {
        $frame = new DashboardFrame();
        $frame ->id = $id;
        $frame ->url = $dataUrl;
        $frame ->caption = $caption;

        $this ->frames[] = $frame;
    }


    public function addSQLFrame($id, $sql, $prava, $caption)
    {
        if ($prava) {
            if (!Prava::jsouNejakePrava($prava)) return;
        }

        $frame = new DashboardFrame();
        $frame ->id = $id;
        $frame ->sql = $sql;
        $frame ->caption = $caption;
        $frame ->prava = $prava;
        $frame ->url = '?menuid=dashboard&action=loaddata&id=' . $id;

        $this ->frames[] = $frame;
    }


    public function addXmlFrame($xmlId, $caption, $prava)
    {
        if ($prava  &&  !Prava::jsouNejakePrava($prava)) {
            return;
        }

        $frame = new DashboardFrame();
        $frame ->id = 'xml_' . $xmlId;
        $frame ->xmlId = $xmlId;
        $frame ->caption = $caption;
        $frame ->prava = $prava;
        $frame ->url = '?menuid=dashboard&action=loaddata&xmlid=' . $xmlId;

        $this ->frames[] = $frame;
    }



    public function findFrame($id)
    {
        foreach ($this ->frames as $frame) {
            if ($frame ->id == $id) {
                return $frame;
            }
        }

        return null;
    }


    public function findXmlFrame($xmlId)
    {
        if (!$xmlId) {
            return null;
        }

        foreach ($this ->frames as $frame) {
            if ($frame ->xmlId == $xmlId) {
                return $frame;
            }
        }

        return null;
    }


    public function sort($orderStr)
    {
        if (!$orderStr) {
            return;
        }

        // sestavi si asociativni pole  id=>poradi
        // nejdrive panely, u kterych ma explicitne nastavene poradi ...
        $orderArray = explode(';', $orderStr);
        $indexArray = [];
        $index = 1;
        foreach($orderArray as $frameId) {
            $indexArray[$frameId] = $index;
            $index++;
        }

        // ... a doplni ostatni panely (napr. nove pridane)
        foreach($this ->frames as $frame) {
            if (!isset($indexArray[$frame ->id])) {
                $indexArray[$frame ->id] = $index;
                $index ++;
            }
        }

        // a seradi panely
        usort($this ->frames, function($item1, $item2) use($indexArray) {
            if ($indexArray[$item1 ->id] == $indexArray[$item2 ->id]) {
                return 0;
            }

            if ($indexArray[$item1 ->id] < $indexArray[$item2 ->id]) {
                return -1;
            }

            return 1;
        });
    }


    public function loadSettings($settingsStr)
    {
        // zpracuje nastaveni panelu - v retezci oddelene strednikem, pricemz nastaveni kazdeho panelu je ve formatu
        // id:velikost    TODO:{color}
        // velikost muze byt def(=default), off(=vypnuty), nebo cols x height

        if (!$settingsStr) {
            return;
        }

        $pomA = explode(';', $settingsStr);

        foreach ($pomA as $frameSettingsStr) {
            $frameSettingsArray = explode(':', $frameSettingsStr);
            $frame = $this ->findFrame($frameSettingsArray[0]);
            if ($frame) {
                if (isset($frameSettingsArray[1])) {
                    // velikost/off
                    if ($frameSettingsArray[1] == 'off') {
                        $frame->hidden = true;
                    } else if ($frameSettingsArray[1] == 'def') {
                        // defaultni velikost - nedela nic
                    } else if ($frameSettingsArray[1]) {
                        list($cols, $height) = explode('x', $frameSettingsArray[1]);
                        if ((int)$cols && (int)$height) {
                            $frame->cols = (int)$cols;
                            $frame->height = (int)$height;

                            if ($frame ->height < 1) $frame ->height = 1;
                            if ($frame ->cols > 12) $frame ->cols =  (int)(12 * $frame ->cols / 1400);
                            if ($frame ->cols > 12) $frame ->cols = 12;
                            if ($frame ->cols < 1) $frame ->cols = 6;
                        }
                    }
                }
            }
        }
    }
}
