<?php
/**
 * stanislav.elbl, 22.10.2015
 */

namespace RonPortal\Dochazka;

use Exception;
use Ron\CalendarItem;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SQLUtils;
use Ron\DB\Database;
use Ron\EMail;
use Ron\MailManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Log;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\Rozdeleni;
use RonPortal\System\AgentManager;
use RonPortal\User;
use RonPortal\ZastupyUtils;


class AbsenceDBRecord extends RonPortalDBRecord
{
    const SETTINGS_POCET_DNU_ZPETNE = 'pocetdnuzpetneplanovani';

    const SCHVALENI_NEURCENO = 0;
    const SCHVALENI_SCHVALENO = 1;
    const SCHVALENI_ZAMITNUTO = 2;
    const SCHVALENI_ZADOSTOZRUSENI = 3;
    const SCHVALENI_ZRUSENO = 4;
    const SCHVALENI_NEZRUSENO = 5; // zruseni zamitnuto

    const PWD_URL_KEY = 'gh&7fr$ron';

    /** @var PracovniPomer */
    public $pp;

    /** @var string     Zdroj akce (kapacity/absence) */
    public $zdroj;

    /** @var DBRecordField */
    public $fDatumOd;
    /** @var DBRecordField */
    public $fDatumDo;
    /** @var DBRecordField */
    public $fPoradi;

    /** @var DBRecordField */
    public $fOperace;
    /** @var DBRecordField */
    public $fOperacePuvodni;

    /** @var DBRecordField */
    public $fSchvaleni;
    /** @var DBRecordField */
    public $fSchvalil;
    /** @var DBRecordField */
    public $fDatumSchvaleni;
    /** @var DBRecordField */
    public $fZamVzalNaVedomi;


    /** @var DBRecordField */
    public $fDoporuceni;
    /** @var DBRecordField */
    public $fDoporucil;
    /** @var DBRecordField */
    public $fDatumDoporuceni;

    /** @var DBRecordField */
    public $fPoznamka;

    /** @var DBRecordField */
    public $fDelka;
    /** @var DBRecordField */
    public $fCasOd;
    /** @var DBRecordField */
    public $fCasDo;

    /** @var DBRecordField */
    public $fKodTypuRozdeleni;
    /** @var DBRecordField */
    public $fRozdeleni;

    /** @var DBRecordField */
    public $fHistorie;

    /** @var DBRecordField */
    public $fUrlKey;

    /** @var DBRecordField */
    public $fUzivatelska1;
    /** @var DBRecordField */
    public $fUzivatelska2;
    /** @var DBRecordField */
    public $fUzivatelska3;
    /** @var DBRecordField */
    public $fUzivatelska4;
    /** @var DBRecordField */
    public $fUzivatelska5;

    /** @var string  Popis operace - pouziva se v emailu */
    public $popisOperace;


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param string $zdrojAkce
     * @param int $datumOd
     * @param int $poradi
     * @param string|object $logDescription
     */
    public function __construct($database, $pp, $zdrojAkce = 'kapacity', $datumOd = null, $poradi = null, $logDescription = null)
    {
        parent::__construct($database, 'Absence', $logDescription);

        $this->zdroj = $zdrojAkce;

        if (!$logDescription) {
            if (is_string($zdrojAkce)) {
                switch ($this->zdroj) {
                    case 'absence' :
                        $this->logDescription = 'Absence';
                        break;

                    case 'kapacity' :
                    default :
                        $this->logDescription = 'Planovani kapacit';

                }
            } else {
                $this->logDescription = $zdrojAkce;
            }
        }

        $this->pp = $pp;
        $this->addStringField('RC', true, 15, $pp->rc); // readonly
        $this->addStringField('OsobniCislo', true, 15, $pp->osobniCislo);
        $this->addDateField('DatumNastupu', true, $pp->datumNastupu);

        $this->addField(null, '', true, 0, '   ' . $pp->jmenoPrijmeni . ', idpp=' . $pp->idPP);

        $this->fDatumOd = $this->addDateField('DatumOd', true); // prozatim readonly
        $this->fDatumDo = $this->addDateField('DatumDo', true);
        $this->fPoradi = $this->addIntField('Poradi', true);

        $this->fDelka = $this->addIntField('Delka', true, 0); // default=0 (celodenni)
        $this->fCasOd = $this->addField('CasOd', 'time', true);
        $this->fCasDo = $this->addField('CasDo', 'time', true);

        $this->fOperace = $this->addStringField('OperaceDochazky', true, 10);
        $this->fOperacePuvodni = $this->addStringField('OperaceDochazkyPuvodni', true, 10);

        $this->fSchvaleni = $this->addIntField('Schvaleni', true);
        $this->fSchvalil = $this->addStringField('Schvalil', true, 25);
        $this->fDatumSchvaleni = $this->addTimestampField('DatumSchvaleni', true);

        $this->fDoporuceni = $this->addIntField('Doporuceni', true, 0);
        $this->fDoporucil = $this->addField('Doporucil', '', true, 25);
        $this->fDatumDoporuceni = $this->addField('DatumDoporuceni', 'timestamp', true);

        $this->fZamVzalNaVedomi = $this->addIntField('ZamVzalNaVedomi', true, 1);

        $this->fPoznamka = $this->addMemoField('Poznamka', true);

        $this->fKodTypuRozdeleni = $this->addIntField('KodTypuRozdeleni', true);
        $this->fRozdeleni = $this->addStringField('Rozdeleni', true, 30);

        $this->fUzivatelska1 = $this->addStringField('Uzivatelska1', true, 100);
        $this->fUzivatelska2 = $this->addStringField('Uzivatelska2', true, 100);
        $this->fUzivatelska3 = $this->addStringField('Uzivatelska3', true, 100);
        $this->fUzivatelska4 = $this->addStringField('Uzivatelska4', true, 100);
        $this->fUzivatelska5 = $this->addStringField('Uzivatelska5', true, 100);

        $this->fHistorie = $this->addField('HistorieSchvalovani', 'memo', true);
        $this->fHistorie->maxLogLength = 0; // defaultne nelogovat
        $this->fHistorie->updateLog = true;

        // logovaci pole ...
        $field_u = $this->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName());
        $field_u->updateLog = true;
        $field_t = $this->addTimestampField('Update_TimeStamp', false, time());
        $field_t->updateLog = true;

        $this->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName());
        $this->addTimestampField('Insert_TimeStamp', true, time());

        $this->fUrlKey = $this->addStringField('UrlKey', true, 40);

        $params = $database->createParameters();
        $where =
            ' RC = ' . $params->addParameter($this->pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($this->pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params->addParameter($database->dateToParam($this->pp->datumNastupu));

        if ($datumOd && $poradi) {
            $where .=
                ' AND DatumOd = ' . $params->addParameter($database->dateToParam($datumOd)) .
                ' AND Poradi = ' . $params->addParameter((int)$poradi);
        }

        $this->setWhere($where, $params);
    }


    /**
     * Vraci true, pokud ma uzivatel prislusne pravo na absenci se zadanou operaci
     * @param Database $database
     * @param int $idPP
     * @param int $datumOd
     * @param int $datumDo
     * @param string $kodOperace
     * @param string $typPrava
     * @return bool
     * @throws Exception
     */
    public static function zkontrolujPravoNaAbsenci($database, $idPP, $datumOd, $datumDo, $kodOperace, $typPrava = 'absence')
    {
        Prava::nactiPravaUzivatele($database);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        $pravo = Prava::vratPrava($typPrava, $typPrava . '_' . $kodOperace);
        return $pravo->lzeZobrazitPP($idPP, $datumOd, $datumDo);
    }


    /**
     * Vraci true, pokud ma uzivatel pravo smazat zadanou absenci
     * @param Database $database
     * @param int $idPP
     * @param int $datum
     * @param string $kodOperace
     * @param int $hodnotaSchvaleni
     * @return bool
     * @throws Exception
     */
    public static function jePravoNaSmazani($database, $idPP, $datum, $kodOperace, $hodnotaSchvaleni)
    {
        if (!self::zkontrolujPravoNaAbsenci($database, $idPP, $datum, $datum, $kodOperace)) {
            // uzivatel nema vubec pravo editovat tuto absenci
            return false;
        }

        if (!RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            // schvalovani neni zapnuto - vice netreba resit
            return true;
        }

        if ($hodnotaSchvaleni == self::SCHVALENI_SCHVALENO
            || $hodnotaSchvaleni == self::SCHVALENI_ZADOSTOZRUSENI
            || $hodnotaSchvaleni == self::SCHVALENI_NEZRUSENO
        ) {
            // absence je schvalena - nutno zkontrolovat, jestli ma uzivatel pravo i na schvalovani nebo smazani absence
            if (self::zkontrolujPravoNaAbsenci($database, $idPP, $datum, $datum, $kodOperace, 'absence_schval')) {
                // ma pravo schvalovat => muze ji zrusit
                return true;
            }

            if (self::zkontrolujPravoNaAbsenci($database, $idPP, $datum, $datum, $kodOperace, 'absence_delete')) {
                // ma pravo na mazani teto absence
                return true;
            }

            // absence je schvalena a uzivatel nema pravo ani mazat ani schvalovat
            return false;
        }

        return true;
    }


    /**
     * Zkontroluje, jestli je mozne editovat absenci v zadanem obdobi
     * @param $database
     * @param int $datumOd
     * @param int $datumDo
     * @param PracovniPomer $pp
     * @param bool $zkontrolovatZpetneVkladani
     * @throws Exception
     */
    public static function zkontrolujObdobiAbsence($database, $datumOd, $datumDo, $pp = null, $zkontrolovatZpetneVkladani = true)
    {
        if ($datumOd > $datumDo) {
            throw new Exception(TextManager::_('Pocatecni_datum_vetsi_nez_koncove'));
        }

        if ($zkontrolovatZpetneVkladani) {
            // zkontroluje, jestli se nevklada pred povolenym datem (podle nastaveni Portalu)
            if ($datumOd < DateUtils::incDay(DateUtils::startOfTheDay(time()), -RonPortalSettings::getIntValue(self::SETTINGS_POCET_DNU_ZPETNE))) {
                throw new Exception(TextManager::_('Nelze_vkladat_absence_zpetne'));
            }
        }

        if (Obdobi::jeObdobiUzavreno($database, 'ADS', $datumOd, $datumDo)) {
            throw new Exception(TextManager::_('Nelze_vkladat_absence_v_uzavrenem'));
        }

        if ($pp) {
            if ($maxUroven = RonPortalSettings::getIntValue('povolitabsenceprischvalenidoch')) {
                // je povoleno planovani absenci pri schvalene dochazce (ale maximalne urovni $maxUroven)
                $seznamSchvaleni = SchvaleniDochazky::nactiSchvaleniVObdobi($database, $pp, $datumOd, $datumDo);
                foreach ($seznamSchvaleni as $schvaleni) {
                    if ($schvaleni->nejvyssiUrovenSchvaleni($datumOd) > $maxUroven) {
                        throw new Exception(TextManager::_('Nelze_editovat_absenci_dochazka_schvalena'));
                    }
                }
            } else if (SchvaleniDochazky::jeSchvalenoObdobi($database, $pp, $datumOd, $datumDo)) {
                throw new Exception(TextManager::_('Nelze_editovat_absenci_dochazka_schvalena'));
            }
        }
    }

    /**
     * Overi, jestli je zadana delka absence u zadane operace povolena. (Pokud nema operace nastavene zadne delky, tak je povoleno vse)
     * @param Database $database
     * @param $operace
     * @param $delka
     * @throws Exception
     */
    public static function zkontrolujPovoleneDelkyOperace($database, $operace, $delka)
    {
        $result = true;

        $params = $database->createParameters();

        $sql = "SELECT Delka FROM PovolenaDelkaAbsence WHERE OperaceDochazky = " . $params->addParameter($operace);

        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            $result = false;

            if ($row[0] == $delka) {
                return;
            }
        }

        if (!$result)
            throw new Exception('Nepovolena_delka_absence');
    }

    /**
     * Zkontroluje uzavreni obdobi a schvaleni dochazky - ale jen v obdobi,
     * ve kterem se parametry absence, ktere mohou mit vliv na dochazku
     */
    public function zkontrolujZmeneneObdobi()
    {
        if ($this->fDelka->value != $this->fDelka->originalValue || $this->fOperace->value != $this->fOperace->originalValue ||
            $this->fCasOd->value != $this->fCasOd->originalValue || $this->fCasDo->value != $this->fCasDo->originalValue
        ) {
            // zmenil se nektery z dulezitych parametru absence => probehne kontrola v celem obdobi
            $kontrolaOd = $this->fDatumOd->value < $this->fDatumOd->originalValue
                ? $this->fDatumOd->value
                : $this->fDatumOd->originalValue;

            $kontrolaDo = $this->fDatumDo->value > $this->fDatumDo->originalValue
                ? $this->fDatumDo->value
                : $this->fDatumDo->originalValue;

            self::zkontrolujObdobiAbsence($this->database, $kontrolaOd, $kontrolaDo, $this->pp);
            // netreba dale pokracovat - bylo zkontrolovano kompletni obdobi absence
            return;
        }

        if ($this->fDatumOd->value != $this->fDatumOd->originalValue) {
            // zmenilo se pocatecni datum absence - kontrola schvaleni dochazky a uzavreni obdobi
            $kontrolaOd = $this->fDatumOd->value < $this->fDatumOd->originalValue
                ? $this->fDatumOd->value
                : $this->fDatumOd->originalValue;

            $kontrolaDo = $this->fDatumOd->value > $this->fDatumOd->originalValue
                ? $this->fDatumOd->value
                : $this->fDatumOd->originalValue;

            self::zkontrolujObdobiAbsence($this->database, $kontrolaOd, $kontrolaDo, $this->pp);
        }

        if ($this->fDatumDo->value != $this->fDatumDo->originalValue) {
            // zmenilo se koncove datum absence - kontrola schvaleni dochazky a uzavreni obdobi
            $kontrolaOd = $this->fDatumDo->value < $this->fDatumDo->originalValue
                ? $this->fDatumDo->value
                : $this->fDatumDo->originalValue;

            $kontrolaDo = $this->fDatumDo->value > $this->fDatumDo->originalValue
                ? $this->fDatumDo->value
                : $this->fDatumDo->originalValue;

            self::zkontrolujObdobiAbsence($this->database, $kontrolaOd, $kontrolaDo, $this->pp);
        }
    }


    /**
     * Zkontroluje prekryvani absenci, ale jen v pripade, ze se zmenilo obdobi nebo delka absence
     * @param bool $editace     Urcuje, jestli se jedna o editaci, nebo vkladani noveho zaznamu
     * @throws Exception
     */
    function zkontrolujPrekryvani($editace)
    {
        // kontrola prekryvani s jinou absenci
        $kontrolaPrekryvani = false;

        if (!$editace) {
            // pri vkladani kontrolu provede vzdy
            $kontrolaPrekryvani = true;
        }else if ($this->fDatumDo->originalValue < $this->fDatumDo->value || $this->fDatumOd->originalValue > $this->fDatumOd->value) {
            // Prodlouzilo se obdobi absence - probehne kontrola prekryvani
            $kontrolaPrekryvani = true;
        } else if ($this->fDelka->originalValue != $this->fDelka->value) {
            // Zmenila se delka absence
            $kontrolaPrekryvani = true;
        } else if ($this->fCasOd->originalValue != $this->fCasOd->value || $this->fCasDo->originalValue != $this->fCasDo->value) {
            $kontrolaPrekryvani = true;
        } else if ($this->fSchvaleni->value == 1 && $this->fSchvaleni->value != $this->fSchvaleni->originalValue) {
            // Doslo ke schvaleni absence
            $kontrolaPrekryvani = true;
        }

        if ($kontrolaPrekryvani) {
            self::zkontrolujPrekryvaniAbsenci($this->database, $this->pp, $this->fOperace->value,
                $this->fDatumOd->value, $this->fDatumDo->value,
                $this->fDelka->value, $this->fCasOd->value, $this->fCasDo->value,
                $editace ? $this->fDatumOd->originalValue : null, $editace ? $this->fPoradi->originalValue : null);
        }
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param string $kodOperace
     * @param int $datumOd
     * @param int $datumDo
     * @param int $delka
     * @param int $casOd
     * @param int $casDo
     * @param int $datumOdPuvodni
     * @param int $poradi
     * @throws Exception
     */
    public static function zkontrolujPrekryvaniAbsenci($database, $pp, $kodOperace, $datumOd, $datumDo, $delka, $casOd, $casDo, $datumOdPuvodni = null, $poradi = null)
    {
        $prekryvaneAbsence = Absence::nactiPrekryvaneAbsence($database, $pp, $kodOperace, $datumOd, $datumDo, $delka, $casOd, $casDo, $datumOdPuvodni, $poradi);

        $jePrekryvana = false;
        if (RonPortalSettings::getIntValue('prekryvaniabsenci')) {
            // neschvalene se mohou prekryvat - projde seznam a zjisti, jestli je nektera absence schvalena
            foreach ($prekryvaneAbsence as $absence) {
                if ($absence->schvaleni == 1 || $absence->schvaleni == 3) { // schvalena, nebo zadost o zruseni
                    $jePrekryvana = true;
                }
            }
        } else {
            $jePrekryvana = !!$prekryvaneAbsence;
        }

        if ($jePrekryvana) {
            throw new Exception(TextManager::_('Prekryvani_absenci'));
        }
    }


    /**
     * Zkontroluje povolene delky, obdobi atd. V pripade nalezeneho problemu vyhodi vyjimku
     * @throws Exception
     */
    public function zkontrolujDataPredZapisem()
    {
        self::zkontrolujPovoleneDelkyOperace($this->database, $this->fOperace->value, $this->fDelka->value);

        self::zkontrolujObdobiAbsence($this ->database, $this->fDatumOd->value, $this->fDatumDo->value, $this->pp);

        if (RonPortalSettings::getIntValue('zakazatabsencepoukonceni')) {
            if ($this->pp->datumUkonceni  &&  $this->fDatumDo->value > $this->pp->datumUkonceni) {
                throw new \Exception( sprintf(TextManager::_('Nelze_vkladat_absence_po_ukonceni_pp'), StringUtils::format('date', $this->pp ->datumUkonceni)) );
            }
        }
    }


    /**
     * Nacte vsechny zaznamy v zadanem obdobi
     * @param int $datumOd
     * @param int $datumDo
     * @param string $filtr
     * @param Parameters $filtrParams
     * @throws DBException
     */
    function loadAbsenceVObdobi($datumOd, $datumDo, $filtr = '', $filtrParams = null)
    {
        $this->checkMultiple = false;

        $params = $this->database->createParameters();
        $where =
            'DatumOd <= ' . $params->addParameter($this->database->dateToParam($datumDo)) .
            ' AND DatumDo >= ' . $params->addParameter($this->database->dateToParam($datumOd)) .
            ' AND Schvaleni <> 4  AND  Schvaleni <> 2 '; // zrusene a zamitnute ignoruje

        if ($filtr) {
            $where .= ' AND (' . $filtr . ')';
            if ($filtrParams) $params->appendParameters($filtrParams);
        }

        $this->loadRecords($where, $params);
    }


    /**
     * Vraci nove poradi absence pro zadane datum
     * @param int $datum
     * @return int
     */
    function novePoradi($datum)
    {
        $params = $this->database->createParameters();
        $filtr =
            'RC = ' . $params->addParameter($this->pp->rc) .
            ' AND OsobniCislo = ' . $params->addParameter($this->pp->osobniCislo) .
            ' AND DatumNastupu = ' . $params->addParameter($this->database->dateToParam($this->pp->datumNastupu)) .
            ' AND DatumOd = ' . $params->addParameter($this->database->dateToParam($datum));

        return SQLUtils::newSerialNumber($this->database, 'Absence', 'Poradi', $filtr, $params);
    }


    /**
     * @param string $filtr
     * @param Parameters $filtrParams
     * @param int $datumOd
     * @param int $datumDo
     * @return bool|int
     * @throws Exception
     * @throws DBException
     */
    protected function zmenObdobi($filtr, $filtrParams, $datumOd, $datumDo)
    {
        $this->fDatumOd->setNewValue($datumOd);
        $this->fDatumDo->setNewValue($datumDo);

        if ($this->fDatumOd->originalValue != $datumOd) {
            //pokud se posouva pocatek absence, je potreba aktualizovat i poradi, aby nedochazelo ke konfliktu v DB
            $this->fPoradi->setNewValue($this->novePoradi($datumOd));
        }

        return $this->updateRecord($filtr, $filtrParams);
    }


    function absenceScript($scriptName)
    {
        $scriptParam =
            'IDPP=' . $this->pp->idPP . ';' .
            'DatumOd=' . date('j.n.Y', $this->fDatumOd->value) . ';' .
            'Poradi=' . $this->fPoradi->value . ';' .
            'DatumDo=' . date('j.n.Y', $this->fDatumDo->value) . ';' .
            'Operace=' . $this->fOperace->value . ';' .
            'Delka=' . $this->fDelka->value . ';' .
            'CasOd=' . DateUtils::secondsToString($this->fCasOd->value) . ';' .
            'CasDo=' . DateUtils::secondsToString($this->fCasDo->value) . ';';

        AgentManager::executeScriptEvents($this->database, RonPortalScriptEvents::CATEGORY_ABSENCE, $scriptName, $scriptParam);
    }

    /**
     * @param string $eventName
     * @param string $context
     * @return bool
     * @throws Exception
     */
    protected function absenceXmlEvents($eventName, $context)
    {
        $variables = [];
        $variables['ABSENCE_ZDROJ'] = $this->zdroj;
        $variables['ABSENCE_IdPP'] = $this->pp->idPP;
        $variables['ABSENCE_Jmeno'] = $this->pp->jmenoPrijmeni;

        return $this->executeXmlEvents($eventName, $context, $variables);
    }


    function zkontrolujZaokrouhleni()
    {
        $zaokrouhleni = RonPortalSettings::getIntValue('zaokrouhleniabsence');
        if (!$zaokrouhleni) return;

        // pokud neni delka "cas od-do" pripadne "...kontinualne", neni co kontrolovat
        if ($this->fDelka->value != 3 && $this->fDelka->value != 4) return;

        if ($this->fCasOd->value != DateUtils::roundTime($this->fCasOd->value, $zaokrouhleni) || $this->fCasDo->value != DateUtils::roundTime($this->fCasDo->value, $zaokrouhleni)) {
            throw new Exception(sprintf(TextManager::_('Cas_absence_musi_byt_zaokrouhlen'), $zaokrouhleni));
        }
    }


    /**
     * @param string $filter
     * @param Parameters $params
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    function editujAbsenci($filter, $params)
    {
        // 6.3.2018 kontrola vypnuta - absence s casem od vetsim nez casem do je v poradku (je pres pulnoc)
        //if ($rec ->fDelka->value == '3' && $rec ->fCasOd->value > $rec ->fCasDo->value) {
        //    throw new \Exception( TextManager::_('Pocatecni_cas_vetsi_nez_koncovy') );
        //}
        if ($this->fDelka->value == '4' && $this->fDatumOd->value == $this->fDatumDo->value && $this->fCasOd->value > $this->fCasDo->value) {
            throw new Exception(TextManager::_('Pocatecni_cas_vetsi_nez_koncovy'));
        }

        if (RonPortalSettings::getIntValue('zakazatabsencepoukonceni')) {
            if ($this->pp->datumUkonceni && $this->fDatumDo->value > $this->pp->datumUkonceni) {
                throw new Exception(sprintf(TextManager::_('Nelze_vkladat_absence_po_ukonceni_pp'), StringUtils::format('date', $this->pp->datumUkonceni)));
            }
        }

        //TODO: kontrola povolene delky operace + (on change ve formu)
        self::zkontrolujPovoleneDelkyOperace($this->database, $this->fOperace->value, $this->fDelka->value);

        // zkontroluje uzavreni obdobi, nastaveni "vkladat zpetne", schvaleni dochazky
        $this->zkontrolujZmeneneObdobi();
        // v pripade zmeny obdobi zkontroluje prekryvani absenci
        $this->zkontrolujPrekryvani(true);
        $this->zkontrolujZaokrouhleni();

        $this->fHistorie->setNewValue(
            $this->fHistorie->originalValue .
            date('j.n.Y G:i', time()) . ', ' . RonPortalUtils::getLogUserName(false) . ': ' .
            TextManager::_('Log_editace_absence') . ' ' .
            date('j.n.Y', $this->fDatumOd->value) . ' - ' . date('j.n.Y', $this->fDatumDo->value) . "\n");

        // xml a skript pred
        if (!$this->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_BEFORE_POST, 'update')) {
            return false;
        }
        $this->absenceScript(RonPortalScriptEvents::EDITACE_ABSENCE_BEFORE);

        if (!$this->updateRecord($filter, $params)) {
            return false;
        }

        $this->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_AFTER_POST, 'update');
        $this->absenceScript(RonPortalScriptEvents::EDITACE_ABSENCE_AFTER);

        return true; // vraci true, protoze vlozeni absence uz probehlo
    }


    public static function getRandomText($length)
    {
        // pouzit bin2hex(random_bytes($length)) ?
        $chars = str_split(
            'abcdefghijklmnopqrstuvwxyz' .
            'ABCDEFGHIJKLMNOPQRSTUVWXYZ' .
            '0123456789-_');
        shuffle($chars);

        $result = '';
        for ($i = 0; $i < $length; $i++) {
            $result .= $chars[rand(0, count($chars) - 1)];
        }

        return $result;
    }


    public function calculateUrlKey($length = 40)
    {
        return self::getRandomText($length);
    }

    /**
     * Priradi nove poradi, provede XML udalosti a vlozi novy zaznam
     * @throws Exception
     */
    function provedVlozeni()
    {
        self::zkontrolujPovoleneDelkyOperace($this->database, $this->fOperace->value, $this->fDelka->value);

        $this->zkontrolujZaokrouhleni();

        $this->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_BEFORE_POST, 'insert');

        $this->fPoradi->setValue($this->novePoradi($this->fDatumOd->value));

        if (RonPortalSettings::getIntValue('schvalabs_bezprihlaseni')) {
            if (!isset($this->fUrlKey->value) || $this->fUrlKey->value == '') {
                $this->fUrlKey->setNewValue($this->calculateUrlKey());
            }
        }

        $this->insertRecord();

        $this->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_AFTER_POST, 'insert');
    }


    /**
     * @param OperaceDochazky $operace
     */
    function nastavSchvaleniPodleOperace($operace)
    {
        $schvalena = false;
        if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            if ($operace->autoSchvalovat) {
                $schvalena = true;
            } else {
                $schvalena = self::zkontrolujPravoNaAbsenci($this->database,
                    $this->pp->idPP,
                    $this->fDatumOd->value, $this->fDatumDo->value,
                    $this->fOperace->value, 'absence_schval');
            }
        }

        if ($schvalena) {
            // muze i schvalovat - vlozi se rovnou schvalena
            $this->fSchvaleni->setValue(1);
            $this->fSchvalil->setValue(User::getLoginName());
            $this->fDatumSchvaleni->setValue(time());
        } else {
            // nemuze schvalovat nebo se schvalovani nepouziva
            $this->fSchvaleni->setValue(0);
            $this->fSchvalil->setValue(null);
            $this->fDatumSchvaleni->setValue(null);
        }
    }


    /**
     * @param int $datumOd
     * @param int $datumDo
     * @param OperaceDochazky $operace
     * @param bool $rezimUprav
     * @throws Exception
     */
    function vlozNovouAbsenci($datumOd, $datumDo, $operace, $rezimUprav)
    {
        $this->fDatumOd->setValue($datumOd);
        $this->fDatumDo->setValue($datumDo);

        $this->fOperace->setValue($operace->kod);
        $this->fOperacePuvodni->setValue($rezimUprav ? null : $operace->kod);

        $this->nastavSchvaleniPodleOperace($operace);

        $this->provedVlozeni();
    }


    /**
     * Vlozi novy zaznam - stejny jako aktualni, ale s jinym obdobim
     * @param int $datumOd
     * @param int $datumDo
     * @param string|bool $operace Pokud neni false, tak nastavi na tuto hodnotu, jinak pouzije hodnotu z puvodniho zaznamu (pouziva se v rezimu uprav)
     * @param float|bool $casOd
     * @param float|bool $casDo
     * @param int|bool $delka
     */
    function vlozKopii($datumOd, $datumDo, $operace = false, $casOd = false, $casDo = false, $delka = false)
    {
        $rec = new AbsenceDBRecord($this->database, $this->pp, $this->zdroj, null, null, $this->logDescription);
        $rec->fDatumOd->setValue($datumOd);
        $rec->fDatumDo->setValue($datumDo);

        $rec->fDelka->setNewValue($delka === false ? $this->fDelka->originalValue : $delka);
        $rec->fCasOd->setNewValue($casOd === false ? $this->fCasOd->originalValue : $casOd);
        $rec->fCasDo->setNewValue($casDo === false ? $this->fCasDo->originalValue : $casDo);

        $rec->fOperace->setValue($operace === false ? $this->fOperace->originalValue : $operace);
        $rec->fOperacePuvodni->setValue($this->fOperacePuvodni->originalValue);

        $rec->fSchvaleni->setValue($this->fSchvaleni->originalValue);
        $rec->fSchvalil->setValue($this->fSchvalil->originalValue);
        $rec->fDatumSchvaleni->setValue($this->fDatumSchvaleni->originalValue);
        // HistorieSchvalovani ? ? ?
        $rec->fZamVzalNaVedomi->setValue($this->fZamVzalNaVedomi->originalValue);
        // ZamVzalNaVedomi_Zapsal, ZamVzalNaVedomi_Datum ?
        $rec->fPoznamka->setValue($this->fPoznamka->originalValue);

        $rec->fPoradi->setValue($this->novePoradi($datumOd));

        $rec->insertRecord();
    }


    /**
     * Vraci true, pokud ma uzivatel pravo editovat/mazat aktualni zaznam absence. Musi platit podminky:
     * - uzivatel ma pravo editovat absenci
     * - bud jeste neni schvalena, nebo ma uzivatel pravo i schvalovat, nebo se (u ni) schvalovani nepouziva
     * @param PracovniPomer $pp
     * @param OperaceDochazky $operace Pokud neni zadana, tak si ji nacte
     * @param bool $delete V pripade, ze se jedna o smazani tak kontroluje jeste pravo absence_delete
     * @return bool
     * @throws Exception
     */
    function jePravoEditovatAbsenci($pp, $operace = null, $delete = false)
    {
        $kodOperace = $operace ? $operace->kod : $this->fOperace->value;

        if (!self::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $this->fDatumOd->value, $this->fDatumDo->value, $kodOperace)) {
            // uzivatel nema pravo planovat tuto absenci - nemuze ji ani editovat
            return false;
        }

        if (!RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)) {
            // pokud se nepouziva schvalovani absenci, muze tuto absenci editovat
            return true;
        }

        if ($this->fSchvaleni->value == self::SCHVALENI_NEURCENO) {
            // absence jeste nebyla schvalena - muze ji editovat
            return true;
        }

        if (!$operace) {
            $operace = OperaceDochazky::nactiOperaci($this->database, $this->fOperace->value);
        }

        if ($operace->autoSchvalovat) {
            // u teto operace se schvalovani nepouziva - muze ji editovat
            if ($delete && Prava::existujePravo('absence_delete_' . $operace->kod)) {
                return Prava::lzeZobrazitPP('absence_delete_' . $operace->kod, $pp->idPP, $this->fDatumOd->value, $this->fDatumDo->value);
            }

            return true;
        }

//        if ( $this ->fOperace ->value !== $operace ->kod ) {     - kontrola vypnuta - pokud se pouziva rezim uprav, tak muze byt operace zapsana jen v "puvodni"
//            throw new \Exception('Chyba ve funkci jePravoEditovatAbsenci!');
//        }

        if (self::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $this->fDatumOd->value, $this->fDatumDo->value, $kodOperace, 'absence_schval')) return true;
        // muze editovat jen v pripade, ze ma pravo i schvalovat

        if ($delete) {
            return self::zkontrolujPravoNaAbsenci($this->database, $pp->idPP, $this->fDatumOd->value, $this->fDatumDo->value, $kodOperace, 'absence_delete');
        }

        return false;
    }


    /**
     * Upravi delku absence tak, aby nezasahovala do zadaneho intervalu. V pripade potreby ji rozdeli na 2 zaznamy
     * @param string $filtr
     * @param Parameters $params
     * @param int $novaDelkaAbsence
     * @param float $zrusitCasOd
     * @param float $zrusitCasDo
     */
    function upravDelkuAbsence($filtr, $params, $novaDelkaAbsence, $zrusitCasOd, $zrusitCasDo)
    {
        $kopieCasOd = false;
        $kopieCasDo = false;

        $this->fDelka->setNewValue($novaDelkaAbsence);

        if ($novaDelkaAbsence == 3) {
            // delka je nastavena na cas od-do - jeste zkrati cas
            if ($this->fCasOd->value < $zrusitCasOd) {
                // puvodni absence zacina pred novou
                if ($this->fCasDo->value > $zrusitCasDo) {
                    // ... a take za ni konci - je potreba rozdelit na 2
                    $kopieCasOd = $zrusitCasDo;
                    $kopieCasDo = $this->fCasDo->value;
                }

                // usekne konec
                $this->fCasDo->setNewValue($zrusitCasOd);
            } else if ($this->fCasDo->value > $zrusitCasDo) {
                // puvodni absence konci az za novou - useknout jeji zacatek, konec ponechat
                $this->fCasOd->setNewValue($zrusitCasDo);
            }
        }

        $this->updateRecord($filtr, $params);

        if ($kopieCasOd !== false && $kopieCasDo !== false) {
            $this->vlozKopii(
                $this->fDatumOd->value, $this->fDatumDo->value, // ve stejnem obdobi
                false, // stejna operace
                $kopieCasOd, $kopieCasDo); // ale jiny cas (za novou absenci)
        }
    }


    /**
     * Zkrati aktualni zaznam tak, aby nezasahoval do zadaneho obdobi ($datumOd - $datumDo).
     * Pokud je zadana delkaNoveAbsence a zaznam se prokazatelne neprekryva s novou absenci, tak ji necha, pripadne ji zkrati casove
     * @param int $datumOd
     * @param int $datumDo
     * @param int $delkaNoveAbsence
     * @param float $casOd
     * @param float $casDo
     * @return bool
     * @throws Exception
     */
    protected function provedZkraceniAbsence($datumOd, $datumDo, $delkaNoveAbsence = 0, $casOd = null, $casDo = null)
    {
        // Nejdrive zjisti, jestli je potreba upravit delku absence
        /** @var int|false $upravaDelka Urcuje, na jakou delku je potreba upravit puvodni absenci. False znamena, ze se ma v zadanem obdobi smazat */
        $upravaDelka = false;

        if ($delkaNoveAbsence == 0) {
            // nova absence je celodenni = starou musim smazat
        } else if ($this->fDelka->value == 0) {
            // puvodni absence je celodenni
            if ($delkaNoveAbsence == 1) {
                $upravaDelka = 2; // nova je prvni pulden, v puvodni nastavim druhy pulden
            } else if ($delkaNoveAbsence == 2) {
                $upravaDelka = 1;
            }
        } else if ($this->fDelka->value == 1 && $delkaNoveAbsence == 2) {
            return true;    // absence se neprekryvaji
        } else if ($this->fDelka->value == 2 && $delkaNoveAbsence == 1) {
            return true;    // absence se neprekryvaji
        } else if ($delkaNoveAbsence == 3 && $this->fDelka->value == 3) {
            // cas od - do
            // POZOR! muze se stat, ze puvodni absence se v zadanem intervalu muze rozdelit na 2 absence!
            // (takze celkem se muze rozpadnout az na 4 zaznamy (pred a za intervalem a v intervalu na 2 casy (pred a za)
            if ($this->fCasOd->value >= $casDo || $this->fCasDo->value <= $casOd) {
                // absence se vubec neprekryvaji - nemusi se mazat
                return true;
            }

            //if ($casOd <= $this ->fCasOd ->value  &&  $casDo >= $this ->fCasDo ->value) => nova absence zcela prekryva puvodni
            if ($casOd > $this->fCasOd->value || $casDo < $this->fCasDo->value) {
                // absence se castecne prekryvaji => puvodni bude casove zkracena
                // POZOR! Muze byt potreba ji rozdelit na 2, 3 nebo dokonce 4!
                $upravaDelka = 3;
            }
        }
        // TODO: vyresit kontinualni?

        $params = $this->database->createParameters();
        $filtr =
            'DatumOd = ' . $params->addParameter($this->database->dateToParam($this->fDatumOd->originalValue)) .
            ' AND Poradi = ' . $params->addParameter((int)$this->fPoradi->originalValue);

        if ($this->fDatumOd->originalValue >= $datumOd && $this->fDatumDo->originalValue <= $datumDo) {
            // Zaznam je cely v zadanem obdobi - uplne ho zrusi nebo upravi jeho delku
            if ($upravaDelka) {
                $this->upravDelkuAbsence($filtr, $params, $upravaDelka, $casOd, $casDo);
            } else {
                $this->doDelete($filtr, $params);
            }
            return true;
        }


        if ($this->fDatumOd->originalValue < $datumOd && $this->fDatumDo->originalValue > $datumDo) {
            // zaznam je potreba rozdelit na 2 - mazane obdobi je "uprostred"
            $old_datumod = $this->fDatumOd->originalValue;
            $old_datumdo = $this->fDatumDo->originalValue;

            if ($upravaDelka) {
                $this->database->beginTransaction();
                try {
                    // zkopiruje puvodni absenci za novou ...
                    $this->vlozKopii(DateUtils::incDay($datumDo, 1), $old_datumdo);

                    // ... zkopiruje puvodni absenci do intervalu nove absence, ale s upravenou delkou ...
                    $this->vlozKopii($datumOd, $datumDo, false, false, false, $upravaDelka);

                    // ... a upravi obdobi puvodni absence (ponecha interval pred novou absenci)
                    $this->fDatumDo->setNewValue(DateUtils::incDay($datumDo, -1));
                    $this->updateRecord($filtr, $params);

                    $this->database->commitTransaction();
                } catch (Exception $e) {
                    $this->database->rollbackTransaction();
                    throw $e;
                }

                return true;
            } else {
                // ze zadaneho obdobi zaznam uplne vymaze
                // puvodni zaznam necha na zacatku
                if (!$this->zmenObdobi($filtr, $params, $this->fDatumOd->originalValue, DateUtils::incDay($datumOd, -1))) {
                    return false;
                }

                // a zkopiruje zaznam na konec
                $this->vlozKopii(DateUtils::incDay($datumDo, 1), $old_datumdo);
                return true;
            }
        }

        if ($this->fDatumOd->originalValue > $datumDo) {
            return true; // absence je cela az za zadanym obdobim
        }
        if ($this->fDatumDo->originalValue < $datumOd) {
            return true; // .. pred ..
        }

        if ($this->fDatumOd->originalValue < $datumOd) {
            // konec absence zasahuje do zadaneho obdobi - usekne ho a zacatek absence ponecha
            return $this->zmenObdobi($filtr, $params, $this->fDatumOd->originalValue, DateUtils::incDay($datumOd, -1));
        }

        if ($this->fDatumDo->originalValue > $datumDo) {
            // ponecha konec absence, zacatek usekne
            return $this->zmenObdobi($filtr, $params, DateUtils::incDay($datumDo, 1), $this->fDatumDo->originalValue);
        }

        return true; // sem nedojde
    }


    /**
     * Zkraceni/smazani/preruseni absence v rezimu uprav
     * @param int $datumOd
     * @param int $datumDo
     * @param int $delkaNoveAbsence
     * @param float $casOd
     * @param float $casDo
     * @return bool|int
     * @throws Exception
     * @throws DBException
     */
    private function zkratAbsenciUprava($datumOd, $datumDo, $delkaNoveAbsence = 0, $casOd = null, $casDo = null)
    {
        if (!isset($this->fOperace->originalValue)) {
            return true; // operace je null => netreba mazat (uz je v rezimu uprav smazana)
        }

        if ($this->fDatumOd->originalValue > $datumDo) {
            return true; // absence je cela az za zadanym obdobim
        }
        if ($this->fDatumDo->originalValue < $datumOd) {
            return true; // .. pred ..
        }

        if (!isset($this->fOperacePuvodni->originalValue)) {
            // puvodni operace je null => neni potreba resit rezim uprav nejakym zvlastnim zpusobem
            return $this->provedZkraceniAbsence($datumOd, $datumDo, $delkaNoveAbsence, $casOd, $casDo);
        }

        $params = $this->database->createParameters();
        $filtr =
            'DatumOd = ' . $params->addParameter($this->database->dateToParam($this->fDatumOd->originalValue)) .
            ' AND Poradi = ' . $params->addParameter((int)$this->fPoradi->originalValue);

        if ($this->fDatumOd->originalValue >= $datumOd && $this->fDatumDo->originalValue <= $datumDo) {
            // Zaznam je cely v zadanem obdobi
            if (isset($this->fOperacePuvodni->value)) {
                // jenom smaze operaci, puvodni ponecha
                $this->fOperace->setNewValue(null);
                return $this->updateRecord($filtr, $params);
            }

            // Puvodni operace je null => uplne zrusi zaznam
            $this->doDelete($filtr, $params);
            return true;
        }

        if ($this->fDatumOd->originalValue < $datumOd && $this->fDatumDo->originalValue > $datumDo) {
            // zaznam je potreba rozdelit na 3 - mazane obdobi je "uprostred"
            // puvodni zaznam necha na zacatku, zkopiruje zaznam na konec a doprostred vlozi upraveny zaznam
            $old_datumod = $this->fDatumOd->originalValue;
            $old_datumdo = $this->fDatumDo->originalValue;

            // puvodni zaznam necha na zacatku
            if (!$this->zmenObdobi($filtr, $params, $old_datumod, DateUtils::incDay($datumOd, -1))) {
                return false;
            }

            // zkopiruje jej na konec (se vsim vsudy)
            $this->vlozKopii(DateUtils::incDay($datumDo, 1), $old_datumdo);

            // a jeste doprostred s vymazanou operaci
            $this->vlozKopii($datumOd, $datumDo, null);

            return true;
        }

        if ($this->fDatumOd->originalValue < $datumOd) {
            // konec absence zasahuje do zadaneho obdobi - usekne ho a zacatek absence ponecha
            $old_datumdo = $this->fDatumDo->originalValue;

            // puvodni zaznam necha na zacatku
            if (!$this->zmenObdobi($filtr, $params, $this->fDatumOd->originalValue, DateUtils::incDay($datumOd, -1))) {
                return false;
            }

            // a zkopiruje zaznam na konec s vymazanou operaci
            if (isset($this->fOperacePuvodni->originalValue)) {
                $this->vlozKopii($datumOd, $old_datumdo, null);
            }

            return true;
        }

        if ($this->fDatumDo->originalValue > $datumDo) {
            // zacatek absence zasahuje do zadaneho obdobi - usekne ho a konec absence ponecha
            $old_datumod = $this->fDatumOd->originalValue;

            // puvodni zaznam necha na konci
            if (!$this->zmenObdobi($filtr, $params, DateUtils::incDay($datumDo, 1), $this->fDatumDo->originalValue)) {
                return false;
            }

            // a zkopiruje zaznam na zacatek s vymazanou operaci
            if (isset($this->fOperacePuvodni->originalValue)) {
                $this->vlozKopii($old_datumod, $datumDo, null);
            }

            return true;
        }

        return true; // sem nedojde
    }


    /**
     * Zkrati aktualni zaznam tak, aby nezasahoval do zadaneho obdobi ($datumOd - $datumDo).
     * Pokud je zadana delkaNoveAbsence a zaznam se prokazatelne neprekryva s novou absenci, tak ji necha, pripadne ji zkrati casove
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $rezimUprav
     * @param int $delkaNoveAbsence
     * @param float $casOd
     * @param float $casDo
     * @return bool
     * @throws Exception
     */
    function zkratAbsenci($datumOd, $datumDo, $rezimUprav = false, $delkaNoveAbsence = 0, $casOd = null, $casDo = null)
    {
        if ($rezimUprav) {
            return $this->zkratAbsenciUprava($datumOd, $datumDo, $delkaNoveAbsence, $casOd, $casDo);
        }

        return $this->provedZkraceniAbsence($datumOd, $datumDo, $delkaNoveAbsence, $casOd, $casDo);
    }


    /**
     * Nacte seznam absenci, ktere se prekryvaji s aktualni absenci a uzivatel ma pravo je zrusit (nebo prerusit, zkratit)
     * Pokud najde absenci, kterou uzivatel zrusit nemuze, vyhodi vyjimku.
     * @return Absence[]
     * @throws Exception
     */
    public function nactiAbsenceKPreruseni($vsechny = false)
    {
        $prekryvaneAbsence = Absence::nactiPrekryvaneAbsence($this->database, $this->pp, $this->fOperace->value,
            $this->fDatumOd->value, $this->fDatumDo->value,
            $this->fDelka->value, $this->fCasOd->value, $this->fCasDo->value,
            $this->fDatumOd->originalValue, $this->fPoradi->originalValue);

        if ($vsechny) {
            return $prekryvaneAbsence;
        }

        // projde seznam a zjisti, jestli je nektera absence schvalena
        /** @var Absence[] $absenceKPreruseni */
        $absenceKPreruseni = [];

        foreach ($prekryvaneAbsence as $absence) {
            if ($absence->schvaleni == self::SCHVALENI_SCHVALENO || $absence->schvaleni == self::SCHVALENI_ZADOSTOZRUSENI || $absence->schvaleni == self::SCHVALENI_NEZRUSENO) {
                if (self::jePravoNaSmazani($this->database, $this->pp->idPP, $absence->datumOd, $absence->operaceDochazky, $absence->schvaleni)) {
                    if (!$absence ->f_PP) {
                        // doplni pracovni pomer
                        $absence ->f_PP = $this ->pp;
                    }
                    $absenceKPreruseni[] = $absence;
                } else {
                    // nasel prekryvanou absenci, kterou uzivatel nemuze prerusit
                    throw new Exception(TextManager::_('Prekryvani_absenci'));
                }
            }
        }

        return $absenceKPreruseni;
    }


    /**
     * Zkrati absence ze zadaneho seznamu, aby nezasahovaly do intervalu aktualni absence
     * @param Absence[] $absenceKPreruseni
     */
    public function zkratPrekryvaneAbsence($absenceKPreruseni)
    {
        if ($absenceKPreruseni) {
            foreach ($absenceKPreruseni as $absence) {
                $rec = new AbsenceDBRecord($this ->database, $this ->pp, $this ->zdroj, $absence ->datumOd, $absence ->poradi, $this ->logDescription);
                if ($rec ->loadRecord()) {
                    $rec ->zkratAbsenci($this ->fDatumOd ->value, $this ->fDatumDo ->value, false,
                        $this ->fDelka ->value, $this ->fCasOd ->value, $this ->fCasDo ->value);
                }
            }
        }
    }


    /**
     * @throws Exception
     */
    public function zkontrolujPrekryvaniPredSchvalenim()
    {
        if (!RonPortalSettings::getIntValue('prekryvaniabsenci')) {
            // prekryvani neschvalenych neni povoleno - v pripade prekryvani rovnou vyhodi vyjimku
            $this->zkontrolujPrekryvani(true);
            return;
        }

        $absenceKPreruseni = self::nactiAbsenceKPreruseni();
        $this ->zkratPrekryvaneAbsence($absenceKPreruseni);
    }


    /**
     * @param int $hodnota
     * @throws Exception
     */
    public function nastavDoporuceni($hodnota)
    {
        if ($hodnota === $this->fDoporuceni->value) {
            return;
        }

        // kontrola uzavreneho obdobi
        if (Obdobi::jeObdobiUzavreno($this->database, 'ADS', $this->fDatumOd->value, $this->fDatumDo->value)) {
            throw new Exception(TextManager::_('Nelze_menit_schvaleni_absence_v_uzavrenem'));
        }

        $desc = TextManager::_('Log_nastaveni_doporuceni'). ' ' . $hodnota;

        $this->fDoporuceni->setNewValue($hodnota);
        $this->fDoporucil->setNewValue(RonPortalUtils::getLogUserName(true));
        $this->fDatumDoporuceni->setNewValue(time());

        $this->fHistorie->setNewValue(
            $this->fHistorie->originalValue .
            date('j.n.Y G:i', time()) . ', ' . RonPortalUtils::getLogUserName(false) . ': ' . $desc . "\n");

        switch ($hodnota) {
            case 1 :
                $context = 'doporuceni';
                break;

            case 2 :
                $context = 'nedoporuceni';
                break;

            default : // nemelo by nastat
                $context = 'doporuceni' . $hodnota;
                break;
        }
        $this->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_SCHVALENI_BEFORE, $context);
        $this->updateRecord();
        $this->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_SCHVALENI_AFTER, $context);
    }

    /**
     * @param int $hodnota
     * @throws Exception
     */
    public function nastavSchvaleni($hodnota)
    {
        if ($hodnota === $this->fSchvaleni->value) {
            return;
        }

        // kontrola uzavreneho obdobi
        if (Obdobi::jeObdobiUzavreno($this->database, 'ADS', $this->fDatumOd->value, $this->fDatumDo->value)) {
            throw new Exception(TextManager::_('Nelze_menit_schvaleni_absence_v_uzavrenem'));
        }

        if (!$this ->popisOperace) {
            $operace = OperaceDochazky::nactiOperaci($this->database, $this->fOperace->value);
            $this->popisOperace = $operace->popis;
        }

        // nastaveni popisu pro zapis do logu a do HistorieSchvalovani
        switch ($hodnota) {
            case 1:
                $desc = TextManager::_('Log_schvaleni_absence');
                $xmlContext = 'schvaleni';
                break;
            case 2:
                $desc = TextManager::_('Log_zamitnuti_absence');
                $xmlContext = 'zamitnuti';
                break;
            case 3:
                $desc = TextManager::_('Log_zadost_o_zruseni');
                $xmlContext = 'zadostozruseni';
                break;
            case 4:
                $desc = TextManager::_('Log_zruseni_absence');
                $xmlContext = 'zruseni';
                break;
            case 5:
                $desc = TextManager::_('Log_zruseni_odmitnuto');
                $xmlContext = 'zruseniodmitnuto';
                break;
            default:
                $desc = TextManager::_('Log_nastaveni_schvaleni') . ' ' . $hodnota;
                $xmlContext = '';
                break;
        }

        // info text do logu
        $pomField = new DBRecordField('', '', true);
        $pomField->value = $desc;
        $this->addDBRecordField($pomField, 0);

        // schvaleni
        $this->fSchvaleni->setNewValue($hodnota);
        $this->fSchvalil->setNewValue(RonPortalUtils::getLogUserName(true));
        $this->fSchvalil->updateLog = true;

        $this->fDatumSchvaleni->setNewValue(time());
        $this->fDatumSchvaleni->updateLog = true;

        $params = $this->database->createParameters();
        $filtr =
            'DatumOd = ' . $params->addParameter($this->database->timestampToParam($this->fDatumOd->value)) .
            ' AND Poradi = ' . $params->addParameter((int)$this->fPoradi->value);

        if ($this->loadRecord($filtr, $params)) {
            if ($hodnota === 1) {
                // schvaleni - nesmi se prekryvat
                $this->zkontrolujPrekryvaniPredSchvalenim();
            }

            $this->fHistorie->setNewValue(
                $this->fHistorie->originalValue .
                date('j.n.Y G:i', time()) . ', ' . RonPortalUtils::getLogUserName(false) . ': ' . $desc . "\n");

            $this ->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_SCHVALENI_BEFORE, $xmlContext);
            $this->updateRecord($filtr, $params);
            $this ->absenceXmlEvents(RonPortalXMLEvents::ABSENCE_SCHVALENI_AFTER, $xmlContext);

            // spusteni skriptu
            $scriptName = null;

            switch ($hodnota) {
                case 1 :
                    $scriptName = RonPortalScriptEvents::SCHVALENI_ABSENCE;
                    break;
                case 2 :
                    $scriptName = RonPortalScriptEvents::ZAMITNUTI_ABSENCE;
                    break;
                case 4 :
                    $scriptName = RonPortalScriptEvents::ZRUSENI_ABSENCE;
                    break;
            }

            if ($scriptName) {
                $this->absenceScript($scriptName);
            }

            $this->absenceScript(RonPortalScriptEvents::STAV_ABSENCE);
        }
    }


    function sestavKlicProSchvaleni($akce, $linkId)
    {
        return sha1(strtolower($akce) . $linkId . self::PWD_URL_KEY . $this->fUrlKey->value);
    }


    function overKlicProSchvaleni($akce, $linkId, $klic)
    {
        $spravnyKlic = $this ->sestavKlicProSchvaleni($akce, $linkId);
        return !strcasecmp($klic, $spravnyKlic);
    }


    /**
     * @param string $text
     * @param bool $vsechny     Pokud je true, nahradi se i POZNAMKA, LINK a dalsi (pouzije se pro telo emailu)
     * @return string mixed
     */
    function nahradParametry($text, $vsechny = false)
    {
        switch ($this->fDelka->value) {
            case 0 :
                $delkastr = TextManager::_('Absence_cela_smena');
                break;

            case 1 :
                $delkastr = TextManager::_('Absence_pulka_smeny1');
                break;

            case 2 :
                $delkastr = TextManager::_('Absence_pulka_smeny2');
                break;

            case 3 :
            case 4 :
                $delkastr = // cas od - do
                    DateUtils::secondsToString($this->fCasOd->value) .
                    ' - ' .
                    DateUtils::secondsToString($this->fCasDo->value);
                break;

            default :
                $delkastr = '';
        }

        $text = str_replace(
            [
                '%DATUMOD%', '%DATUMDO%', '%DENVTYDNUOD%', '%DENVTYDNUDO%',
                '%ABSENCE%', '%DELKA%', '%ROZDELENI%',
                '%JMENO%', '%JMENO2%', '%OSOBNICISLO%', '%RC%',
            ],
            [
                // datumy
                date('j.n.Y', $this->fDatumOd->value),
                date('j.n.Y', $this->fDatumDo->value),
                DateUtils::dayName($this->fDatumOd->value),
                DateUtils::dayName($this->fDatumDo->value),
                // absence
                $this->popisOperace,
                $delkastr,
                $this->fRozdeleni->value,
                // zamestnanec
                $this->pp->jmenoPrijmeniKratke,
                $this->pp->jmenoPrijmeni,
                $this->pp->osobniCislo,
                $this->pp->rc
            ],
            $text);

        if ($vsechny) {
            $text = str_replace('%UZIVATEL%', User::getLoginName() . '(' . User::getUserName() .')', $text);
            $text = str_replace('%POZNAMKA%', $this ->fPoznamka ->value ?? '', $text);

            // link na zobrazeni absence
            $linkStr =
                '?menuid=absence&action=zobrazabsenci&idpp=' . $this->pp->idPP .
                '&datum=' . date('j.n.Y', $this ->fDatumOd ->value) .
                '&poradi=' . $this->fPoradi->value;
            $text = str_replace('%LINK%', $linkStr, $text);

            // link na schvaleni absence
            $linkStr =
                '&action=schvalitabsenci&idpp=' . $this->pp->idPP .
                '&datum=' . date('j.n.Y', $this ->fDatumOd ->value) .
                '&poradi=' . $this->fPoradi->value;
            $text = str_replace('%LINK_SCHVALIT%', '?menuid=absence' . $linkStr, $text);

            // schvaleni proklikem bez prihlaseni
            if (RonPortalSettings::getIntValue('schvalabs_bezprihlaseni')  &&  isset($this ->fUrlKey ->value)) {
                // nedava do linku primo key, ale hash s nejakym dalsim nahodnym textem
                // (aby to nemohl uzivatel jednoduse sestavit podle klice vycteneho z databaze)
                // "heslo" je prozatim natvrdo
                $linkId = self::getRandomText(20);
                $linkStr = '?menuid=absencelink' . $linkStr . '&linkid=' . $linkId . '&key=' . $this ->sestavKlicProSchvaleni('schvalitabsenci', $linkId);
            } else {
                $linkStr = '?menuid=absence' . $linkStr;
            }
            $text = str_replace('%LINK_SCHVALIT_KEY%', $linkStr, $text);

            // link na zamitnuti absence
            $linkStr =
                '&action=zamitnoutabsenci&idpp=' . $this->pp->idPP .
                '&datum=' . date('j.n.Y', $this ->fDatumOd ->value) .
                '&poradi=' . $this->fPoradi->value;
            $text = str_replace('%LINK_ZAMITNOUT%', '?menuid=absence' . $linkStr, $text);

            // zamitnuti proklikem bez prihlaseni
            if (RonPortalSettings::getIntValue('schvalabs_bezprihlaseni')  &&  isset($this ->fUrlKey ->value)) {
                // nedava do linku primo key, ale hash s nejakym dalsim nahodnym textem
                // (aby to nemohl uzivatel jednoduse sestavit podle klice vycteneho z databaze)
                // "heslo" je prozatim natvrdo
                $linkId = self::getRandomText(20);
                $linkStr = '?menuid=absencelink' . $linkStr . '&linkid=' . $linkId . '&key=' . $this ->sestavKlicProSchvaleni('zamitnoutabsenci', $linkId);
            } else {
                $linkStr = '?menuid=absence' . $linkStr;
            }
            $text = str_replace('%LINK_ZAMITNOUT_KEY%', $linkStr, $text);
        }


        return $text;
    }


    /**
     * @param Rozdeleni $rozdeleni
     * @return string
     */
    function vratEmailProRozdeleni($rozdeleni)
    {
        $kamPoslat = RonPortalSettings::getIntValue('planovaniabsence_nadrazene');
        // 0 - rozdeleni zamestnance
        // 1 - rozdeleni zamestnance nebo nadrazene
        // 2 - primo nadrazene
        // 3 - nektere nadrazene (prvni, ktere ma zadano email)
        // 10 - vedouci rozdeleni zamestnance

        if (!$kamPoslat  ||  ($kamPoslat == 1  &&  $rozdeleni ->eMail)) {
            return trim($rozdeleni->eMail);
        }

        if ($kamPoslat == 10) {
            if ($rozdeleni ->vedouciRC) {
                $params = $this->database->createParameters();
                $result = DBUtils::getSqlValue($this ->database, 'SELECT EMail FROM Osoba WHERE RC = ' . $params ->addParameter($rozdeleni ->vedouciRC), $params);

                // jeste najde pripadneho zastupce
                $emailZastupu = ZastupyUtils::vratEmaiZastupu($this ->database, $rozdeleni ->vedouciRC, 'absence_schval');

                if ($emailZastupu) {
                    $result = ($result ? $result . ';' : '') . $emailZastupu;
                }

                return $result;
            }

            return null;
        }

        // najde prvniho rodice se zadanym emailem
        while ($rozdeleni ->rodic) {
            $rozdeleni = Rozdeleni::loadRozdeleni($this ->database, $rozdeleni ->typ, $rozdeleni ->rodic);
            if ($rozdeleni ->eMail ||  $kamPoslat == 2) {
                return trim($rozdeleni ->eMail);
            }
        }

        return null;
    }



    /**
     * Volano pri vlozeni absence a pri zadosti o zruseni absence
     * @param \Ron\DB\Database $database
     * @param int $hodnotaSchvaleni
     * @throws Exception
     */
    function planovani_Email($database, $hodnotaSchvaleni)
    {
        if (!RonPortalSettings::getIntValue('planovaniabsence_posilatemail')) return;

        $logStr = "Odeslani emailu \n\n";

        $odesilatel = RonPortalSettings::getValue('planovaniabsence_odesilatel');

        // Pokud neni zadan odesilatel, tak nacte emailovou adresu zamestnance
        if (!$odesilatel) {
            $odesilatel = PracovniPomer::emailOsoby($database, $this->pp->rc);
        }

        // Sestavi predmet a text
        if ($hodnotaSchvaleni == 3) {
            $predmet = RonPortalSettings::getValue('zruseniabsence_zadostpredmet');
            $text = RonPortalSettings::getMultilineValue('zruseniabsence_zadosttext');
        } else {
            $predmet = RonPortalSettings::getValue('planovaniabsence_predmet');
            $text = RonPortalSettings::getMultilineValue('planovaniabsence_text');
        }

        $predmet = $this->nahradParametry($predmet);

        $text = $this->nahradParametry($text, true);

        $logStr .= StringUtils::fromUTF8('Odesílatel: ') . $odesilatel . "\n";
        $logStr .= StringUtils::fromUTF8('Předmět: ') . $predmet . "\n";
        $logStr .= "Text:\n" . $text . "\n\n";

        // priloha emailu
        $icalItems = AbsenceCalendarItem::loadAbsenceRec($this);

        $odeslanoPrijemci = []; // pole prijemcu, kterym jiz to bylo odeslano - umoznuje kontrolu, ze se neposila 2x na stejny email

        $prijemce = trim(RonPortalSettings::getValue('planovaniabsence_prijemce') ?? '');
        if ($prijemce) {
            $logStr .= sprintf(StringUtils::fromUTF8("Příjemce: %s (nastavení intrawebu)\n"), $prijemce);

            $eMail = new EMail();
            $eMail ->from = $odesilatel;
            $eMail ->to = $prijemce;
            $eMail ->subject = $predmet;
            $eMail ->text = $text;

            $eMail ->addAttachment('ronportalevent.ics', CalendarItem::getICalFileContent($icalItems));

            try {
                MailManager::sendMail($eMail);
                $odeslanoPrijemci[strtolower($prijemce)] = 1;
            } catch (Exception $e) {
                $logStr .= StringUtils::fromUTF8("   ... při odesílání emailu došlo k chybě!\n") . $e ->getMessage() . "\n\n";
            }
        }

        if (RonPortalSettings::getIntValue('planovaniabsence_rozdeleni')) {
            $seznamRozdeleni = Rozdeleni::loadRozdeleniOsoby($database, $this ->pp->idPP, RonPortalSettings::getIntValue('planovaniabsence_rozdeleni'));

            foreach ($seznamRozdeleni as $rozdeleni) {
                $adresa =  $this ->vratEmailProRozdeleni($rozdeleni);

                if ($adresa) {
                    if (!isset($odeslanoPrijemci[strtolower(trim($adresa))])) {

                        $logStr .= sprintf(StringUtils::fromUTF8("Příjemce: %s (rozdělení zaměstnance %s)\n"), $adresa, $rozdeleni ->kod);

                        $eMail = new EMail();
                        $eMail ->from = $odesilatel;
                        $eMail ->to = $adresa;
                        $eMail ->subject = $predmet;
                        $eMail ->text = $text;

                        $eMail ->addAttachment('ronportalevent.ics', CalendarItem::getICalFileContent($icalItems));

                        try {
                            MailManager::sendMail($eMail);
                            $odeslanoPrijemci[strtolower($adresa)] = 1;
                        } catch (Exception $e) {
                            $logStr .= StringUtils::fromUTF8("   ... při odesílání emailu došlo k chybě!\n") . $e ->getMessage() . "\n\n";
                        }
                    } else {
                        $logStr .= sprintf(StringUtils::fromUTF8("Příjemce: %s (rozdělení zaměstnance %s) - již odesláno\n"), $adresa, $rozdeleni ->kod) . "\n";
                    }
                } else {
                    $logStr .= sprintf(StringUtils::fromUTF8("Příjemce: rozdělení zaměstnance %s - nemá nastaveno email\n"), $rozdeleni ->kod) . "\n";
                }
            }
        }

        Log::addRecordToDBLog($database, 'Absence', 'INSERT', $this->logDescription, $logStr);
    }


    /**
     * @param int $hodnotaSchvaleni
     * @throws Exception
     */
    public function schvaleni_Email($hodnotaSchvaleni)
    {
        if (!RonPortalSettings::getIntValue('schvaleniabsence_posilatemail')) {
            return;
        }

        $prijemce = PracovniPomer::emailOsoby($this ->database, $this ->pp->rc);
        $odesilatel = RonPortalSettings::getValue('schvaleniabsence_odesilatel');

        // Pokud neni zadan odesilatel, tak nacte emailovou adresu aktualniho uzivatele
        if (!$odesilatel) {
            $odesilatel = PracovniPomer::emailOsoby($this ->database, User::getRC());
        }

        if ($hodnotaSchvaleni === 1) {
            $kopie = RonPortalSettings::getValue('schvaleniabsence_kopie');
        }
        else if ($hodnotaSchvaleni === 2) {
            $kopie = RonPortalSettings::getValue('zamitnutiabsence_kopie');
        }
        else {
            $kopie = null;
        }

        // pokud nema zamestnanec zadanou emailovou adresu, posle se zprava jen na adresu z kopie
        if (!$prijemce) {
            $prijemce = $kopie;
            $kopie = '';
        }

        if (!$prijemce) {
            // neni zadana zadna adresa pro odeslani
            Log::addRecordToDBLog($this ->database, 'Absence', 'UPDATE', $this->logDescription, StringUtils::fromUTF8('Odeslání emailu o schválení/zamítnutí - není zadána adresa příjemce'));
            return;
        }

        // Sestavi predmet a text
        switch ($hodnotaSchvaleni) {
            case 1 :
                $text = RonPortalSettings::getMultilineValue('schvaleniabsence_text');
                $predmet = RonPortalSettings::getValue('schvaleniabsence_predmet');
                break;

            case 2 :
                $text = RonPortalSettings::getMultilineValue('zamitnutiabsence_text');
                $predmet = RonPortalSettings::getValue('zamitnutiabsence_predmet');
                break;

            case 4 :
                $text = RonPortalSettings::getMultilineValue('zruseniabsence_text');
                $predmet = RonPortalSettings::getValue('zruseniabsence_predmet');
                break;

            case 5 :
                $text = RonPortalSettings::getMultilineValue('zamitnutizruseniabsence_text');
                $predmet = RonPortalSettings::getValue('zamitnutizruseniabsence_predmet');
                break;

            default :
                return; // zatim neimplementovano
        }

        if (!$text && !$predmet) {
            Log::addRecordToDBLog($this ->database, 'Absence', 'UPDATE', $this->logDescription, StringUtils::fromUTF8('Odeslání emailu o schválení/zamítnutí - není zadán předmět ani text emailu'));
            return; // neni zadan text ani predmet
        }

        $predmet = $this ->nahradParametry($predmet);

        $text = $this ->nahradParametry($text, true);

        $logstr = sprintf(StringUtils::fromUTF8(
            "Odeslání emailu \n\n" .
            "Příjemce: %s\n" .
            "Kopie: %s\n" .
            "Odesílatel: %s\n" .
            "Předmět: %s\n" .
            "Text: - - - - - - - -\n%s\n- - - - - - - - - - -\n"),
            $prijemce, $kopie, $odesilatel, $predmet, $text);

        // priloha emailu
        $icalItems = AbsenceCalendarItem::loadAbsenceRec($this);

        $eMail = new EMail();
        $eMail ->from = $odesilatel;
        $eMail ->to = $prijemce;
        $eMail ->subject = $predmet;
        $eMail ->text = $text;
        if ($kopie) {
            $eMail->copyTo = $kopie;
        }

        $eMail ->addAttachment('ronportalevent.ics', CalendarItem::getICalFileContent($icalItems));

        try {
            MailManager::sendMail($eMail);
            Log::addRecordToDBLog($this ->database, 'Absence', 'UPDATE', $this->logDescription, $logstr);
        } catch (Exception $e) {
            $logstr .= StringUtils::fromUTF8("   ... při odesílání emailu došlo k chybě!\n") . $e ->getMessage();
            Log::addRecordToDBLog($this ->database, 'Absence', 'UPDATE', $this->logDescription, $logstr);

            throw $e;
        }
    }

}
