<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DB\Database;
use RonPortal\PracovniPomer;

class DenniVysledekR extends DenniVysledek
{
    /** @var int */
    public $kodTypuRozdeleni;
    /** @var string */
    public $rozdeleni;
    /** @var string */
    public $popisRozdeleni;


    public static function nactiDenniVysledky($database, $osoba, $obdobiOd, $obdobiDo, $typCasoveSlozkyMzdy = null)
    {
        throw new \Exception('Chyba - nelze volat DenniVysledekR::nactiDenniVysledky!');
    }


    /**
     * @param Database $database
     * @param array $row
     */
    private function load($database, $row)
    {
        $this ->loadBasicData($database, $row);
        $this ->kodTypuRozdeleni = $row[8];
        $this ->rozdeleni = $row[9];
        $this ->popisRozdeleni = $row[10];
    }


    /**
     * @param Database $database
     * @param PracovniPomer $osoba
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @param string|string[] $typCasoveSlozkyMzdy
     * @return DenniVysledek[]
     * @throws \Ron\DB\DBException
     */
    public static function nactiDenniVysledkyR($database, $osoba, $obdobiOd, $obdobiDo, $typCasoveSlozkyMzdy = null)
    {
        $result = [];

        $params = $database ->createParameters();

        $sql =
            'SELECT DenniVysledek_R.RC, DenniVysledek_R.OsobniCislo, DenniVysledek_R.DatumNastupu, DenniVysledek_R.Den, DenniVysledek_R.TypCasoveSlozkyMzdy, ' .
            ' DenniVysledek_R.Hodiny, DenniVysledek_R.Dny, DenniVysledek_R.KalendarniDny, ' .
            ' DenniVysledek_R.KodTypuRozdeleni, DenniVysledek_R.Rozdeleni, Rozdeleni.Popis' .
            ' FROM DenniVysledek_R, Rozdeleni' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = DenniVysledek_R.KodTypuRozdeleni' .
            ' AND Rozdeleni.Kod = DenniVysledek_R.Rozdeleni';

        if ($osoba) {
            $sql .=
                ' AND RC = ' . $params ->addParameter($osoba ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($osoba ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($osoba ->datumNastupu));
        }

        if ($obdobiOd) {
            $sql .=
                ' AND Den >= ' . $params ->addParameter($database ->dateToParam($obdobiOd));
        }

        if ($obdobiDo) {
            $sql .=
                ' AND Den <= ' . $params ->addParameter($database ->dateToParam($obdobiDo));
        }

        if ($typCasoveSlozkyMzdy) {
            $sql .= ' AND';

            if (is_array($typCasoveSlozkyMzdy)) {
                $sql .= ' (';

                $first = true;
                foreach ($typCasoveSlozkyMzdy as $slozka) {
                    if (!$first) {
                        $sql .= ' OR';
                    }
                    $first = false;

                    $sql .= ' TypCasoveSlozkyMzdy = ' . $params ->addParameter($slozka);
                }

                $sql .= ')';
            } else {
                $sql .= ' TypCasoveSlozkyMzdy = ' . $params ->addParameter($typCasoveSlozkyMzdy);
            }
        }

        $sql .= ' ORDER BY RC, OsobniCislo, DatumNastupu, Den, DenniVysledek_R.Rozdeleni, TypCasoveSlozkyMzdy';

        $cursor = $database ->getSqlResult($sql, $params, 'Pri nacitani dennich vysledku R doslo k chybe!');
        while ($row = $cursor ->fetch()) {
            $vysledek = new static();
            $vysledek ->load($database, $row);

            $result[] = $vysledek;
        }
        $cursor ->close();

        return $result;
    }

}
