<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\MessageManager;
use Ron\StringUtils;
use RonPortal\Controls\FilterFormFilterObdobi;
use Ronportal\Controls\FilterFormFilterVariabilni;
use RonPortal\Controls\PriznakSchvaleniBrowserField;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Editace\DenniPoznamkaDochazkyRecord;
use RonPortal\Editace\DenniVysledekVypoctuRecord;
use RonPortal\Editace\MesicniVysledekVypoctuRecord;
use RonPortal\Kalendar;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;


class SloupecDennichVysledku
{
    /** @var string  vyraz pro vypocet */
    public $expression;
    /** @var string  ktera hodnota z DV se pouzije (H/D/K) */
    public $jednotka;
    /** @var string  casova slozka - v podstate puvodni vyraz bez podtrzitek a jednotky */
    public $casovaSlozka;
    public $soucet;

    public $obsahujeVyraz = false;
    
    public $editovat;

    /** @var bool priznak, ze je potreba prepocitat mesicni vysledek */
    public $prepocitat;
}



class DenniVysledkyBrowser extends VysledkyBrowser
{
    const SETTINGS_SLOUPCE = 'sloupcedennichvysledku';
    const SETTINGS_ZOBRAZIT_PUVODNI = 'zobrazitpuvodnivdennich';
    const SETTINGS_ZOBRAZIT_NULOVE = 'zobrazovatprazdnedv';

    /** @var SloupecDennichVysledku[] */
    var $sloupce;

    /** @var SchvaleniDochazky */
    var $schvaleni;

    var $pocetEditu; // pocet editovatelnych poli
    var $pocetPuvodni; // nejvetsi pocet zaznamu puvodni dochazky ve dni
    var $kalendar;

    /** @var Cursor */
    var $cursorPuvodni;

    var $lzeEditovat; // nastavi se podle prav a podle toho, jestli je obdobi uzavreno

    var $variantni; // jestli se maji zobrazovat variantni obdobi (true) nebo mesice (false)
    /** @var integer  zobrazovane obdobi nastavi podle vybraneho obdobi/variantniho obdobi */
    public $datumOd;
    /** @var integer  nastavi se v ShowFilter */
    var $datumDo;

    /** @var ?int   Nastavi se po nacteni dat na IdPP aktualni osoby */
    public $idPP;


    function __construct($database, $variantni)
    {
        parent::__construct($database, $variantni ? 'dennivysledkyvariantni' : 'dennivysledky');

        $this ->code = 'browserdenni';
        $this ->cssClass = 'browser grid dennivysledky keyInputMove';
        $this ->variantni = $variantni;

        $this ->title = $variantni
            ? \Ron\ronText('Prehled_variantnich_vysledku')
            : \Ron\ronText('Denni_vysledky');

        $this ->pocetPuvodni = 0;

        $this ->kalendar = ''; // kalendar, podle ktereho se nacitaji svatky

        $this ->parseFieldsString( RonPortalSettings::getValue(self::SETTINGS_SLOUPCE) );
    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        if ($this ->variantni) {
            $this ->filterForm ->addFilter(
                new FilterFormFilterVariabilni($this ->filterForm),
                true
            );
        } else {
            $this ->filterForm ->addFilter(
                (new FilterFormFilterObdobi($this ->filterForm))
                    ->setRequired(true),
                true
            );
            //$this ->filterForm ->nastavObdobi(true, 'ADS', false); // jenotevrene = false
        }

        $prava = Prava::vratPrava('prohlizeni');

        if ($prava ->zobrazitVyberRozdeleni()) {
            $this ->filterForm ->addFilterRozdeleni1(null, $prava);
            $this ->filterForm ->addFilterRozdeleni2(null, $prava);
        }

//        $this ->filterForm ->nastavOsoby(true, $prava, 0, 0); // datum od-do podle obdobi
//        $this ->filterForm ->osobyFiltr = 'PracovniPomer.ZpracovatDochazku = 1';

        ($this ->filterForm ->addFilterPP(false, $prava))
            ->setOptionsSqlFilter('PracovniPomer.ZpracovatDochazku = 1');

        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
    }


    function renderFilter()
    {
        if ($this ->variantni) {
            $this ->odkazyVariabilniVysledky('dennivysledkyvariantni');
        }

        parent::renderFilter();
    }


    /**
     * vytvori pole hlavicek sloupcu a pole vyrazu pro sloupce
     * @param string $str
     */
    function parseFieldsString($str)
    {
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(''); // pole pro ulozeni data

        $this ->addField(new PriznakSchvaleniBrowserField(1)) ->setCode('col-schvaleni1') ->setSqlFieldIndex(1);
        $this ->addField(new PriznakSchvaleniBrowserField(2)) ->setCode('col-schvaleni2') ->setSqlFieldIndex(2);
        $this ->addField(new PriznakSchvaleniBrowserField(3)) ->setCode('col-schvaleni3') ->setSqlFieldIndex(3);

        $this ->addField()
            ->setSqlFieldName('1', 4)
            ->setCode('col-datum')
            ->setHeaderText('Date') ->setFooterText('Celkem')
            ->setFormat('timestamp') ->setClassName('border-right') ->setAlign('center');

        $col_defs = explode(';', $str);
        $this ->sloupce = [];

        foreach ($col_defs as $index => $colstr) {
            $pomarray = explode(':', $colstr); // Nadpis:[*][#H|#D|#K]vyraz

            $sloupec = new SloupecDennichVysledku();

            $expression = $pomarray[1];
            $sloupec ->editovat = ($expression[0] == '*');
            if ($expression[0] == '*') {
                $expression = substr($expression, 1);
            }

            if (substr($expression, 0, 2) == '#H'
                || substr($expression, 0, 2) == '#D'
                || substr($expression, 0, 2) == '#K'
            ) {
                $sloupec ->jednotka = $expression[1];
                $expression = substr($expression, 2);
            } else {
                $sloupec ->jednotka = 'H';
            }

            $sloupec->casovaSlozka = str_replace('_', '', $expression);

            $sloupec ->obsahujeVyraz = self::obsahujeVzorec($expression);

            if ($sloupec ->obsahujeVyraz) {
                switch ($sloupec->jednotka) {
                    case 'H' :
                        $sloupec->expression = StringUtils::processIdentifiers($expression, '$H_'); // str_replace('_', '$H_', $expression);
                        break;
                    case 'D' :
                        $sloupec->expression = StringUtils::processIdentifiers($expression, '$D_'); // str_replace('_', '$D_', $expression);
                        break;
                    case 'K' :
                        $sloupec->expression = StringUtils::processIdentifiers($expression, '$K_'); // str_replace('_', '$K_', $expression);
                        break;
                }
            } else {
                $sloupec->expression =
                    (strcasecmp($expression, 'Poznamka') ? $sloupec->jednotka : '') .
                    $expression;
            }

            $this ->sloupce[] = $sloupec;

            $this ->addField()
                ->setSqlFieldName('1', $index + 5) ->setHeaderText($pomarray[0])
                ->setCode('col-dv-' . ($index + 1))
                ->setAlign(strcasecmp($sloupec ->expression, 'Poznamka') ? 'right' : 'left');
        }
    }


    function nactiDenniVysledky()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Den, TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny ' .
            ' FROM DenniVysledekVypoctu' .
            ' WHERE RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            ' AND Den >= ' . $params ->addDateParameter($this ->datumOd) .
            ' AND Den <= ' . $params ->addDateParameter($this ->datumDo) .
            ' ORDER BY Den';


        return $this ->database ->getSqlResult($sql, $params);
    }


    function nactiPuvodniDochazku()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT PuvodniDochazka.DatumCasOperace, OperaceDochazky.Popis' .
            ' FROM PuvodniDochazka, OperaceDochazky' .
            ' WHERE PuvodniDochazka.KodOperace = OperaceDochazky.Kod' .
            ' AND PuvodniDochazka.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND PuvodniDochazka.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND PuvodniDochazka.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            ' AND PuvodniDochazka.DatumCasOperace >= ' . $params ->addDateParameter($this ->datumOd) .
            ' AND PuvodniDochazka.DatumCasOperace < ' . $params ->addDateParameter(DateUtils::incDay($this ->datumDo, 1)) .
            ' ORDER BY PuvodniDochazka.DatumCasOperace';

        return $this ->database ->getSqlResult($sql, $params, 'Chyba pri nacitani puvodni dochazky!');
    }


    function nactiSchvaleni($rc, $osobniCislo, $datumNastupu, $datumOd, $datumDo)
    {
        $this ->schvaleni = new SchvaleniDochazky();
        $this ->schvaleni ->load($this ->database,
            $rc, $osobniCislo, $datumNastupu, $datumOd, $datumDo);
    }


    private static function obsahujeVzorec($text)
    {
        if (strpos($text, '+') !== false ||
            strpos($text, '-') !== false ||
            strpos($text, '*') !== false ||
            strpos($text, '/') !== false ||
            strpos($text, '(') !== false ||
            strpos($text, ')') !== false ||
            strpos($text, '?') !== false
            ) {
            return true;
        }

        return false;
    }


    private function vratVzorec($values)
    {
        //error_reporting(error_reporting() & ~E_NOTICE & ~E_WARNING); // vypne warningy undefined variable (zobrazuji se v pripade, ze nejaky sloupec obsahuje vzorec)

        $result = '';

        foreach ($values as $varName => $value) {
            $result .= '$' . $varName . '=' . $value . ';';
        }

        return $result;
    }

    /**
     * @param $datum
     * @param float[] $values   Asociativni pole s hodnotami pro jednotlive casove slozky (s prefixem H_, D_ a K_)
     * @param $jeToSoucet
     * @return array
     */
    function vytvorRadek($datum, $values, $jeToSoucet)
    {
        $browserRow = [];

        if (!$jeToSoucet) {
            // V nulte polozce radku ulozi typ dne (vikend/svatek/normalni)
            $browserRow[] = $datum;
            // V dalsich trech polozkach ulozi hodnotu true, pokud je dochazka
            // pro dany den zpracovana/zkontrolovana/schvalena
            $browserRow[] = $datum <= $this ->schvaleni ->schvaleni1;
            $browserRow[] = $datum <= $this ->schvaleni ->schvaleni2;
            $browserRow[] = $datum <= $this ->schvaleni ->schvaleni3;
            // Vytvori hodnotu v prvnim sloupci - datum
            $browserRow[] =
                '<span class="zkratkadne">' . \Ron\ronText('Day_short_' . date('w', $datum)) . '</span>' .
                date('j.n.Y', $datum);
        }

        // Vytvori jednotlive promenne pro vypocty a doplni jejich hodnoty
        $jeVzorecek = false;

        // Provede jednotlive vypocty
        foreach ($this ->sloupce as $sloupec) {
            $pomStr = '';
            $minus = false;

            if (strcasecmp($sloupec ->expression, 'Poznamka') == 0) {
                // Poznamka ke dni
                if (!$jeToSoucet) {
                    $pomStr = DenniPoznamkaDochazky::vratPoznamkuDochazky($this ->database, $datum, $this ->filterForm ->getFilterValue('osoba'));
                }
            } else if ($sloupec ->expression) {
                // Vyhodnoceni vzorecku
                $vysledek = null;

                if ($sloupec ->obsahujeVyraz) {
                    if (!$jeVzorecek) {
                        $valuesStr = $this->vratVzorec($values);
                        eval($valuesStr);
                        $jeVzorecek = true;
                    }

                    eval('$vysledek = ' . $sloupec->expression . ';');
                } else {
                    //var_dump($values);
                    //var_dump($sloupec ->expression);
                    $vysledek = $values[$sloupec->expression] ?? null;
                }

                if (!$jeToSoucet && !$vysledek && !RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZIT_NULOVE)) {
                    $pomStr = '';
                } else {
                    switch ($sloupec->jednotka) {
                        case 'H' :
                            $vysledek = round($vysledek * 60);
                            $pomStr = StringUtils::format('minuty', $vysledek);
                            break;

                        case 'D' :
                        case 'K' :
                            $pomStr = StringUtils::format('number_2', $vysledek);
                            break;
                    }
                }

                if ($vysledek < 0) $minus = true;

                if (!$jeToSoucet) $sloupec ->soucet += $vysledek;
            }

            $pomStr = str_replace("'", '&#39;', $pomStr ?? '');

            // Pokud se pole muze editovat, vlozi edit, jinak jenom text
            if ($sloupec ->editovat && !$jeToSoucet && $this ->lzeEditovat &&
                $this ->schvaleni ->lzeEditovatPP($this ->filterForm ->getFilterValue('osoba'), $datum)
            ) {
                $inputName = 'hodnota_' . $datum . '_' . $sloupec ->jednotka . '_' . $sloupec ->casovaSlozka;

                if (strcasecmp($sloupec ->expression, 'Poznamka') == 0) {
                    $col_str =
                        "<input type='text' name='$inputName' class='poznamkaedit' size='30' value='$pomStr' />";
                    // pozor, musi byt nahrazene '
                } else {
                    $col_str = '';

                    if ($minus) $col_str .= '<span class="minus">';

                    $className =
                        'vysledekedit ' .
                        ( ($sloupec ->jednotka == 'D' || $sloupec ->jednotka == 'K')
                            ? 'inputFloat'
                            : 'inputTime'
                        );

                    $col_str .=
                        "<input type='text' name='$inputName' class='$className' maxlength='5' size='5' value='$pomStr' />";
                    // pozor, musi byt nahrazene '

                    if ($minus) $col_str .= '</span>';
                }

                $browserRow[] = $col_str;

                $this ->pocetEditu++;
            } else {
                if ($minus) $browserRow[] = '<span class="minus">' . $pomStr . '</span>';
                else $browserRow[] = $pomStr;
            }

        }

        return $browserRow;
    }


    /**
     * @param Cursor $cursor
     * @param Cursor $puvodni
     * @return array
     */
    function prevedDataDoPole($cursor, $puvodni)
    {
        $this ->pocetEditu = 0;
        //$this -> f_soucet = array();

        $datum = DateUtils::startOfTheDay($this ->datumOd);
        $posledniDatum = DateUtils::startOfTheDay($this ->datumDo);
        $result = [];

        $zaznam = $cursor ->fetch();
        $zaznamPuvodni = $puvodni ? $puvodni ->fetch() : null;

        // Pro kazdy den vytvori radek
        while ($datum <= $posledniDatum) {
            $dalsiDatum = DateUtils::incDay($datum, 1);

            // Prida do vypoctu vsechny zaznamy z daneho dne
//            $valuesStr = '';
            $values = [];

            while ($zaznam && $this ->database ->stringToTimestamp($zaznam[0]) < $dalsiDatum) {
                // Nekdy se cislo prevede na retezec s desetinnou carkou misto
                // tecky, takze potom nefunguje eval. Proto se tady prevede s teckou
                $values['H_' . $zaznam[1]] = is_string($zaznam[2])
                    ? (float)str_replace(',', '.', $zaznam[2])
                    : (float)$zaznam[2];

                $values['D_' . $zaznam[1]] = is_string($zaznam[3])
                    ? (float)str_replace(',', '.', $zaznam[3])
                    : (float)$zaznam[3];

                $values['K_' . $zaznam[1]] = is_string($zaznam[4])
                    ? (float)str_replace(',', '.', $zaznam[4])
                    : (float)$zaznam[4];

//                $pom = str_replace(',', '.', $zaznam[2]);
//                $valuesStr = $valuesStr . '$H_' . $zaznam[1] . ' += ' . $pom . ';';
//
//                $pom = str_replace(',', '.', $zaznam[3]);
//                $valuesStr = $valuesStr . '$D_' . $zaznam[1] . ' += ' . $pom . ';';
//
//                $pom = str_replace(',', '.', $zaznam[4]);
//                $valuesStr = $valuesStr . '$K_' . $zaznam[1] . ' += ' . $pom . ';';

                $zaznam = $cursor ->fetch();
            }

            $radek = $this ->vytvorRadek($datum, $values, false);

            $pocetPuvodni = 0;
            // pokud se ma zobrazit puvodni dochazka, tak do radku doplni zaznamy puvodni dochazky
            while ($zaznamPuvodni  &&  $this ->database ->stringToTimestamp($zaznamPuvodni[0]) < $dalsiDatum) {
                $radek[] = date("H:i", $this ->database ->stringToTimestamp($zaznamPuvodni[0])) . " &nbsp;" . StringUtils::htmlTextConvert($zaznamPuvodni[1]);
                $pocetPuvodni++;
                $zaznamPuvodni = $puvodni ->fetch();
            }

            if ($pocetPuvodni > $this ->pocetPuvodni) {
                $this ->pocetPuvodni = $pocetPuvodni;
            }

            $result[] = $radek;

            // while ($zaznam && $zaznam[0]
            $datum = $dalsiDatum; // incday($datum, 1);
        }

        return $result;
    }


//    function vlozPoleSBarvou($vyplneno, $classname)
//    {
//        if ($vyplneno) echo "<td class='schvaleni $classname'>";
//        else echo '<td class="schvaleni">';
//    }



    function renderDenniVysledky($poleVysledku)
    {
        $rowNumber = 0;
        foreach ($poleVysledku as $index => $row) {
            $rowNumber++;

            // Podle toho jestli je to vikend nebo ne zvoli barvu
            $cssclass = Kalendar::getStyleForDate($this->database, $row[0], 1, $this->kalendar);
            echo $cssclass ? "<tr class='$cssclass'>" : '<tr>';

            // Vykresli barvicky podle schvalovani
//            $this->vlozPoleSBarvou($row[1], 'schvaleni1'); // zelena
//            $this->vlozPoleSBarvou($row[2], 'schvaleni2'); // zluta
//            $this->vlozPoleSBarvou($row[3], 'schvaleni3'); // cervena

            // Vypise obsah radku
            //$prvniDatovySloupec = 4;

            //for ($i = $prvniDatovySloupec; $i < count($this ->fields); $i++) {
            //for ($i = 0; $i < count($this ->fields); $i++) {
            foreach ($this->fields as $i => $field) {
                if ($field ->visible  &&  $field ->type != 'hidden') {
                    $value = $row[$field->sqlFieldIndex] ?? '';

                    $class = $field ->getDataCellClass($value, $rowNumber, null, $row);

//                    if ($i < 4) {
//
//                    } else if ($i == $prvniDatovySloupec) {
//                        // Prvni radek je datum
//                        echo '<td class="browser_rowheader align-center">';
//                    } else {
//                        echo '<td align="' . $this->fields[$i]->align . '">';
//                    }
                    echo "<td class='$class'>";

                    if ($field ->type != 'priznakschvaleni') {
                        echo $value;
                    }
                    echo "</td>\n";
                }
            }

            echo '</tr>';
        }
    }



    function nactiSoucet($variantni)
    {
        $params = $this ->database ->createParameters();

        if ($variantni) {
            $sql =
                'SELECT VariantniObdobi, TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny ' .
                '  FROM VariantniVysledek ' .
                ' WHERE RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
                '   AND OsobniCislo = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                '   AND DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
                '   AND VariantniObdobi = ' . $params ->addDateParameter($this ->datumOd);
        } else {
            $sql =
                'SELECT Mesic, TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny ' .
                '  FROM MesicniVysledekVypoctu ' .
                ' WHERE RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
                '   AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                '   AND DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
                '   AND Mesic = ' . $params ->addDateParameter($this ->datumOd);
        }

        $cursor = $this ->database ->getSqlResult($sql, $params);
        //$valuesStr = '';
        $values = [];
        while ($zaznam = $cursor ->fetch()) {
            $values['H_' . $zaznam[1]] = is_string($zaznam[2])
                ? (float)str_replace(',', '.', $zaznam[2])
                : (float)$zaznam[2];

            $values['D_' . $zaznam[1]] = is_string($zaznam[3])
                ? (float)str_replace(',', '.', $zaznam[3])
                : (float)$zaznam[3];

            $values['K_' . $zaznam[1]] = is_string($zaznam[4])
                ? (float)str_replace(',', '.', $zaznam[4])
                : (float)$zaznam[4];

//            $pom = str_replace(',', '.', $zaznam[1]);
//            $valuesStr = $valuesStr . '$H_' . $zaznam[0] . ' += ' . $pom . ';';
//
//            $pom = str_replace(',', '.', $zaznam[2]);
//            $valuesStr = $valuesStr . '$D_' . $zaznam[0] . ' += ' . $pom . ';';
//
//            $pom = str_replace(',', '.', $zaznam[3]);
//            $valuesStr = $valuesStr . '$K_' . $zaznam[0] . ' += ' . $pom . ';';
        }

        return $this ->vytvorRadek(0, $values, true);
    }


    function renderSoucet()
    {
        $soucet = $this ->nactiSoucet($this ->variantni);
        if ($soucet) {
            echo '<tr class="browserfooter">';

            foreach ($this ->fields as $i => $field) {
                if ($field ->visible  &&  $field ->type != 'hidden') {
                    $class = $field->getFooterClass();
                    echo "<td class='$class'>";

                    if ($field ->footer) {
                        echo $field ->footer;
                    } else if ($field ->sqlFieldIndex > 4) {
                        echo ((isset($soucet[$i - 5]) && $soucet[$i - 5]) ? $soucet[$i - 5] : '');
                    }

                    echo '</td>';
                }
            }

//            echo '<td class="schvaleni"></td><td class="schvaleni"></td><td class="schvaleni"></td>'; // barvicky
//            echo '<td>' . \Ron\ronText('Celkem') . '</td>';
//
//            foreach ($soucet as $value) {
//                echo "<td class='align-right'> $value </td>";
//            }
//
//            for ($i = 0; $i < $this ->pocetPuvodni; $i++) {
//                echo '<td></td>';
//            }

            echo "</tr>\n";
        }
    }


    function tlacitkoSchvaleni()
    {
        if ( SchvaleniDochazky::lzeSchvalovatOsobu($this ->filterForm ->getFilterValue('osoba'), $this ->datumOd, $this ->datumDo) ) {
            echo "<input class='submitbutton' type='submit' name='dialogschvaleni' value='" . \Ron\ronText('Schvalit_dochazku') . "' />";
        }
    }


    function load($sqlFilter = '')
    {
        $this ->loadConfiguration();

        $this ->setFilterForm();
        $this ->filterForm ->loadData();

        $this ->datumOd = $this ->filterForm ->getOmezitObdobiOd();
        $this ->datumDo = $this ->filterForm ->getOmezitObdobiDo();

        $this ->idPP = $this ->filterForm ->getFilterValue('osoba');

        $prava = Prava::vratPrava('editace');

        $this ->lzeEditovat = !$this ->variantni &&
            !Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $this ->datumOd, $this ->datumDo) &&
            $prava ->lzeZobrazitPP($this ->idPP, $this ->datumOd, $this ->datumDo);

        $this ->kalendar = Kalendar::vratKalendarOsoby($this ->database,
            $this ->filterForm ->getFilterValue('osoba', 'rc'),
            $this ->filterForm ->getFilterValue('osoba', 'osobniCislo'),
            $this ->filterForm ->getFilterValue('osoba', 'datumNastupu'),
            $this ->datumOd, $this ->datumDo);

        $this ->nactiSchvaleni(
            $this ->filterForm ->getFilterValue('osoba', 'rc'),
            $this ->filterForm ->getFilterValue('osoba', 'osobniCislo'),
            $this ->filterForm ->getFilterValue('osoba', 'datumNastupu'),
            $this ->datumOd, $this ->datumDo);

        $this ->cursor = $this ->nactiDenniVysledky();

        $this ->cursorPuvodni = RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZIT_PUVODNI)
            ? $this ->nactiPuvodniDochazku()
            : null;
    }


    function render()
    {
        echo '<div class="dennivysledkybrowser">';
        $this ->renderHeader();

        $pomarray = $this ->prevedDataDoPole($this ->cursor, $this ->cursorPuvodni);

        // doplni sloupce pro puvodni dochazku
        $pocetPoliPuvodni = count($this ->fields);
        for ($i = 0; $i < $this ->pocetPuvodni; $i++) {
            $this ->addField()
                ->setSqlFieldName('1', $pocetPoliPuvodni + $i) ->setWrap(1) ->setClassName('zaznampuvodni');
        }

        //if ($this ->f_poceteditu) {
        //echo '<FORM method="post" action="?menuid=editacedennich">';
        echo '<form method="post" action="?menuid=dennivysledky">';
        //echo '<INPUT type="hidden" name="osobaeditace" value="'.S_GetVybranaOsoba().'">';
        //echo '<INPUT type="hidden" name="obdobieditace" value="'.$this -> f_datumod.'">';

        echo '<INPUT type="hidden" name="editacedennich" value="1" />';
        echo '<INPUT type="hidden" name="obdobi" value="' . $this ->datumOd . '" />';
        echo '<INPUT type="hidden" name="osoba" value="' . $this ->filterForm ->getFilterValue('osoba') . '" />';
        //}

        $this ->renderTableStart();
        echo "<tbody class='border-bottom'>\n";
        $this ->renderDenniVysledky($pomarray);
        echo "</tbody>\n";
        echo "<tbody>\n";
        $this ->renderSoucet();
        echo "</tbody>\n";
        $this ->renderTableEnd();

        echo '<div class="buttons">';
        if ($this ->pocetEditu) {
            //echo '<INPUT type="hidden" name="poceteditu" value="'.$this -> f_poceteditu.'">';
            echo '<INPUT type="submit" class="submitbutton" name="ulozit" value="' . \Ron\ronText('form.Store') . '">';

            if ( RonPortalSettings::getSpousteniADS() ) {
                echo '<INPUT class="submitbutton" type="submit" name="prepocet" value="' . \Ron\ronText('Prepocet_DV') . '" >';
            }

        }

        if ( !$this ->variantni ) {
            $this ->tlacitkoSchvaleni();
        }

        echo '</div>';
        echo "</form>\n";

        echo "</div>\n"; // class=dennivysledkybrowser
    }



    // EDITACE DAT ------------------------------------------------------------
    /**
     * @param PracovniPomer $pp
     * @param int $datum
     * @param string $hodnota
     */
    function ulozPoznamku($pp, $datum, $hodnota)
    {
        $rec = new DenniPoznamkaDochazkyRecord($this ->database, $pp, $datum, $this);
        $rec ->zapisPoznamku($hodnota);
    }


    /**
     * @param PracovniPomer $pp
     * @param int $datum
     * @param SloupecDennichVysledku $sloupec
     * @param string $hodnotaStr
     * @return bool
     */
    function ulozDenniVysledek($pp, $datum, $sloupec, $hodnotaStr)
    {
        try {
            if (trim($hodnotaStr) != '') {
                switch ($sloupec->jednotka) {
                    case 'D' :
                    case 'K' :
                        $hodnota = StringUtils::stringToFloat($hodnotaStr);
                        break;

                    default :
                        $hodnota = DateUtils::timeStringToFloat($hodnotaStr, true); // povolitZkraceny = true
                }
            } else {
                $hodnota = 0;
            }
        } catch (\Exception $e) {
            MessageManager::addErrorMessage(date('j.n.Y', $datum) . ': ' . \Ron\ronText('Spatna_hodnota'));
            //MessageManager::addErrorMessage($e ->getMessage());
            return false;
        }

        $rec = new DenniVysledekVypoctuRecord($this ->database, $pp, $datum, $sloupec ->casovaSlozka, $this);
        $rec ->zapisHodnotu($sloupec ->jednotka, $hodnota);

        if ($rec ->recordUpdated) {
            $sloupec ->prepocitat = true;
        }

        return true;
    }


    /**
     * @param PracovniPomer $pp
     * @param int $datum
     * @return bool
     */
    function ulozDen($pp, $datum)
    {
        $result = true;

        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->editovat) {

                // hodnota_<den>_H/D/K_<slozka>
                $inputname = 'hodnota_' . $datum . '_' . $sloupec ->jednotka . '_' . $sloupec ->casovaSlozka;

                if (isset($_POST[$inputname])) {
                    $hodnota = $_POST[$inputname];

                    if (strcasecmp($sloupec ->expression, 'Poznamka') == 0) {
                        $this ->ulozPoznamku($pp, $datum, $hodnota);
                    } else {
                        $result = $result && $this ->ulozDenniVysledek($pp, $datum, $sloupec, $hodnota);
                    }
                }
            }
        }

        return $result;
    }


    /**
     * Prepocita mesicni vysledky sloupcu, ktere maji nastaven priznak prepocitat
     * @param PracovniPomer $pp
     * @param Obdobi $obdobi
     * @throws \Ron\DB\DBException
     */
    function prepocitejMesicniVysledky($pp, $obdobi)
    {
        foreach ($this ->sloupce as $sloupec) {
            if ($sloupec ->prepocitat) {
                // zjisti mesicni soucty pro casovou slozku
                $params = $this ->database ->createParameters();
                $sql =
                    'SELECT SUM(Hodiny), SUM(Dny), SUM(KalendarniDny) FROM DenniVysledekVypoctu' .
                    ' WHERE RC = ' . $params->addParameter($pp ->rc) .
                    ' AND OsobniCislo = ' . $params->addParameter($pp ->osobniCislo) .
                    ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu) .
                    ' AND Den >= ' . $params ->addDateParameter($obdobi ->zacatek) .
                    ' AND Den <= ' . $params ->addDateParameter($obdobi ->konec) .
                    ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($sloupec ->casovaSlozka);

                $cursor = $this ->database ->getSqlResult($sql, $params);
                $soucty = $cursor ->fetch();
                $cursor ->close();

                // nastavi zaznam
                $rec = new MesicniVysledekVypoctuRecord($this ->database, $pp, $obdobi ->zacatek, $sloupec ->casovaSlozka, $this);

                $rec ->zapisHodnotu($soucty[0], $soucty[1], $soucty[2], null); // poznamku nenastavuje
            }
        }
    }


    function ulozDenniVysledky()
    {
        if (!isset($_POST['editacedennich'])) {
            // zadna data, nedela nic
            return true;
        }

        // probehla editace, je potreba nacist udaje editovane osoby
        $pp = PracovniPomer::nactiPP($this ->database, $_POST['osoba']);
        $pp ->vyberNactenyPP($this ->database, null);

        $this ->idPP = $pp ->idPP;

        // Pozor! Nefunguje pro variabilni obdobi!
        $obdobi = new Obdobi();
        if (!$obdobi ->nactiObdobi($this ->database, 'ADS', $_POST['obdobi'])) {
            throw new \Exception('Chyba pri editaci dennich vysledku - zadane obdobi nebylo nalezeno!'); // nemelo by nastat
        }

        $obdobi ->nastavSession();

        $this ->datumOd = $obdobi ->zacatek;
        $this ->datumDo = $obdobi ->konec;

        Prava::nastavObdobiProPrava($obdobi ->zacatek, $obdobi ->konec);

        if ($obdobi ->uzavreno) {
            throw new \Exception( \Ron\ronText('Obdobi_uzavreno') );
        }

        $this ->nactiSchvaleni(
            $pp ->rc, $pp ->osobniCislo, $pp ->datumNastupu,
            $obdobi ->zacatek, $obdobi ->konec
        );

        $result = true;

        $prava = Prava::vratPrava('editace');
        if ($prava ->lzeZobrazitPP($pp ->idPP, $obdobi ->zacatek, $obdobi ->konec)) {
            // pokud nema prava, tak se tu nic nedela, ale vyjimku nevyhodi, protoze muze mit pravo na schvalovani
            $datum = $obdobi->zacatek;

            while ($datum <= $obdobi->konec) {
                if ($this->schvaleni->lzeEditovatPP($pp->idPP, $datum)) {
                    $result &= $this->ulozDen($pp, $datum);
                }

                $datum = DateUtils::incDay($datum, 1);
            }

            $this->prepocitejMesicniVysledky($pp, $obdobi);

            if (RonPortalSettings::getSpousteniADS() && isset($_POST['prepocet'])) if ($_POST['prepocet']) {
                AgentManager::prepocetDochazky(
                    $pp->idPP,
                    $obdobi->zacatek, $obdobi->konec);
            }
        }

        return $result;
    }
}
