<?php
/**
 * UTF8
 * stanislav.elbl, 28.08.2015
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use RonPortal\RonPortalSettings;


class DochazkaUtils
{
    /**
     * Vraci datum a cas posledniho zpracovani dochazky
     * @param Database $database
     * @return int
     * @throws \Exception
     */
    static function datumCasZpracovaniDochazky($database)
    {
        $sql = "SELECT val FROM SYS_VAR WHERE (applicationID = 'ADS') AND (Name = 'DatumZpracovaniDochazky')";
        $cursor = $database ->getSqlResult($sql, null, 'Chyba pri nacitani zpracovani dochazky!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row  ||  !$row[0]) return 0;

        try {
            $result = DateUtils::datetimeStringToTimestamp($row[0]); // Pozor! Hodnota v db ulozena jako text!
        } catch (\Exception $e) {
            throw new \Exception("Chyba pri zjistovani posledniho zpracovnani dochazky: {$row[0]}");
        }

        return $result;
    }


    /**
     * Vybere zaznamy z tabulky PuvodniDochazka a HistoriePruchodu, ktere nejsou
     * starsi nez 1 den (vzhledem k zadanemu datu) pro zadaneho cloveka (pracovni pomer). Z nich vybere ten posledni.
     * @param Database $database
     * @param int $datum
     * @param string $rc
     * @param string|null $osobnicislo
     * @param int|null $datumnastupu
     * @param bool $pouzitPP slouzi pro urceni, zda se vyhledava operace pro konkretni PP, nebo osobu
     * @return array|bool|null
     * @throws DBException
     */
    public static function vratPosledniPruchodSQL($database, $datum, $rc, $osobnicislo, $datumnastupu, $pouzitPP = true)
    {
        $dnu = RonPortalSettings::getIntValue('monitorovani_dnu');
        if ($dnu < 1) {
            $dnu = 1;
        }

        $params = $database ->createParameters();

        $sql =
            'SELECT PuvodniDochazka.DatumCasOperace as DatumCas, PuvodniDochazka.Terminal, PuvodniDochazka.Poznamka,' . // 0..2
            ' OperaceDochazky.popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani,' . // 3,4
            ' Umisteni.popis as popisUmisteni, PuvodniDochazka.Rozdeleni, Rozdeleni.Popis, OperaceDochazky.Kod' . // 5..8
            "\nFROM" .
            ' ( ( (PuvodniDochazka INNER JOIN OperaceDochazky' .
            '        ON OperaceDochazky.kod = PuvodniDochazka.KodOperace' .
            '     ) LEFT OUTER JOIN Terminal' .
            '       ON Terminal.kod = PuvodniDochazka.terminal' .
            '   ) LEFT OUTER JOIN Umisteni' .
            '     ON Umisteni.kod = Terminal.KodUmisteni' .
            ' ) LEFT OUTER JOIN Rozdeleni ' .
            '   ON Rozdeleni.Kod = PuvodniDochazka.Rozdeleni ' .
            '   AND Rozdeleni.KodTypuRozdeleni = PuvodniDochazka.KodTypuRozdeleni ' .
            "\nWHERE (PuvodniDochazka.datumCasOperace <= " . $params ->addTimestampParameter($datum) . ')' .
            ' AND (PuvodniDochazka.datumCasOperace > ' . $params ->addTimestampParameter(DateUtils::incDay($datum, -$dnu)) . ')' .
            ' AND ((Terminal.bezMonitorovani <> 1) OR (Terminal.bezMonitorovani IS NULL))' .
            ' AND OperaceDochazky.PritomnostProMonitorovani <= 2 ' . // nastaveni 3=nemonitorovat uplne ignoruje
            ' AND PuvodniDochazka.RC = ' . $params ->addParameter($rc);

        if ($pouzitPP) {
            $sql .=
                ' AND PuvodniDochazka.OsobniCislo = ' . $params ->addParameter($osobnicislo) .
                ' AND PuvodniDochazka.DatumNastupu = ' . $params ->addDateParameter($datumnastupu);
        }

        $sql .=
            "\nUNION";

        $sql = $sql .
            "\nSELECT HistoriePruchodu.DatumCasPruchodu as DatumCas, HistoriePruchodu.Terminal," .
            ' HistoriePruchodu.Poznamka,' .
            ' OperaceDochazky.Popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani,' .
            ' Umisteni.Popis as PopisUmisteni, HistoriePruchodu.Rozdeleni, Rozdeleni.Popis, OperaceDochazky.Kod' .
            "\nFROM " .
            ' ( ( (HistoriePruchodu INNER JOIN OperaceDochazky' .
            '        ON OperaceDochazky.kod = HistoriePruchodu.KodOperace' .
            '     ) INNER JOIN Terminal' .
            '       ON Terminal.kod = HistoriePruchodu.terminal' .
            '   ) LEFT OUTER JOIN Umisteni' .
            '     ON Umisteni.kod = Terminal.KodUmisteni' .
            ' ) LEFT OUTER JOIN Rozdeleni ' .
            '   ON Rozdeleni.Kod = HistoriePruchodu.Rozdeleni ' .
            '   AND Rozdeleni.KodTypuRozdeleni = HistoriePruchodu.KodTypuRozdeleni ' .
            "\nWHERE (HistoriePruchodu.uspesny = 1)" .
            ' AND (HistoriePruchodu.DatumCasPruchodu <= ' . $params ->addTimestampParameter($datum) . ')' .
            ' AND (HistoriePruchodu.DatumCasPruchodu > ' . $params ->addTimestampParameter(DateUtils::incDay($datum, -1)) . ')' .
            ' AND ((Terminal.bezMonitorovani <> 1) OR (Terminal.bezMonitorovani IS NULL))' .
            ' AND OperaceDochazky.PritomnostProMonitorovani <= 2 ' . // nastaveni 3=nemonitorovat uplne ignoruje
            ' AND HistoriePruchodu.RC = ' . $params ->addParameter($rc);

        if ($pouzitPP) {
            $sql .=
                ' AND HistoriePruchodu.OsobniCislo = ' . $params ->addParameter($osobnicislo) .
                ' AND HistoriePruchodu.DatumNastupu = ' . $params ->addDateParameter($datumnastupu);
        }


        if ($database ->getDbType() == 'interbase') {
            $sql = $sql . ' ORDER BY 1 DESC';
        } else {
            $sql = $sql . ' ORDER BY DatumCas DESC';
        }

        $cursor = $database ->select($sql, $params);

        $dochazka_row = $cursor ->fetch();

        if ($dochazka_row) {
            $OK = $dochazka_row[4] != 2;
            while (!$OK) {
                $pomRow = $cursor->fetch();
                if ($pomRow) {
                    $dochazka_row[4] = $pomRow[4];
                    $OK = $dochazka_row[4] != 2;
                } else {
                    $dochazka_row[4] = 0;
                    $OK = true;
                }
            }
        } else {
            $dochazka_row = [null, null, null, null, null, null, null, null, null];
        }

        $cursor ->close();

        // Nacte absenci
        $params = $database ->createParameters();
        $sql =
            'SELECT Absence.DatumOd, Absence.DatumDo, ' .
            ' OperaceDochazky.Popis as PopisOperace, OperaceDochazky.PritomnostProMonitorovani, OperaceDochazky.Kod, ' . $database->castWideMemo('Absence.Poznamka') .
            ' FROM Absence, OperaceDochazky' .
            ' WHERE (OperaceDochazky.Kod = Absence.OperaceDochazky)' .
            '  AND (Absence.DatumOd <= ' . $params ->addParameter($database ->timestampToParam($datum)) . ')' .
            '  AND (Absence.DatumDo >= ' . $params ->addParameter($database ->timestampToParam(DateUtils::incDay($datum, -1))) . ')' .
            '  AND Absence.Schvaleni <> 2  AND  Absence.Schvaleni <> 4 ' . // nezobrazuje zamitnute a zrusene
            ' AND Absence.RC = ' . $params ->addParameter($rc);

        if ($pouzitPP) {
            $sql .=
                ' AND Absence.OsobniCislo = ' . $params->addParameter($osobnicislo) .
                ' AND Absence.DatumNastupu = ' . $params->addParameter($database ->dateToParam($datumnastupu));
        }

        $sql .=
            ' ORDER BY DatumOd DESC';


        $cursor = $database ->getSqlResult($sql, $params, 'Spatny SQL vyraz pro vyber planovanych absenci');
        $absence_row = $cursor ->fetch();
        $cursor ->close();

        $datumDochazky = $database ->stringToTimestamp($dochazka_row[0]);

        if ($absence_row) {
            $datumAbsence = $database ->stringToTimestamp($absence_row[0]);
            if ($datumAbsence > $datumDochazky) {
                return [
                    'DatumCas' => $absence_row[0],
                    'DatumDo' => $absence_row[1],
                    'PopisOperace' => $absence_row[2],
                    'Pritomnost' => $absence_row[3],
                    'Operace' => $absence_row[4],
                    'Poznamka' => $absence_row[5],
                    // prida jeste integer indexy - jen pro kompatibilitu stareho kodu!
                    0 => date('j.n.Y', $datumAbsence) . ' - ' . DBUtils::getFormatedValue($database, 'date', $absence_row[1]), // cas jako text
                    3 => $absence_row[2], // popis operace
                    4 => $absence_row[3] // pritomnost
                ];
//                // Vraci absenci
//                $result = array();
//                $result[] = date('j.n.Y', $datumAbsence) . ' - ' . DBUtils::getFormatedValue($database, 'date', $absence_row[1]);
//                $result[] = ''; // terminal
//                $result[] = $absence_row[5]; // poznamka
//                $result[] = $absence_row[2]; // popis operace
//                $result[] = $absence_row[3]; // pritomnost
//                $result[] = ''; // umisteni
//                $result[] = ''; // kod rozdeleni
//                $result[] = ''; // popis rozdeleni
//                $result[] = $absence_row[4]; // kod operace
//
//                return $result;
            }
        }

        return $dochazka_row
            ? [
                'DatumCas' => $dochazka_row[0],
                'Terminal' => $dochazka_row[1],
                'PopisOperace' => $dochazka_row[3],
                'Pritomnost' => $dochazka_row[4],
                'Operace' => $dochazka_row[8],
                'Poznamka' => $dochazka_row[2],
                'PopisUmisteni' => $dochazka_row[5],
                'Rozdeleni' => $dochazka_row[6],
                'PopisRozdeleni' => $dochazka_row[7],
                // prida jeste integer indexy - jen pro kompatibilitu stareho kodu!
                0 => DBUtils::getFormatedValue($database, 'datetime', $dochazka_row[0]), // cas jako text
                3 => $dochazka_row[3], // popis operace
                4 => $dochazka_row[4] // pritomnost
            ]
            : [ // jen pro kompatibilitu stareho kodu
                0 => null,
                3 => null,
                4 => null
            ];
    }
}
