<?php
/**
 * pavel.rudol, 04.06.2024 16:51
 */

namespace RonPortal\Dochazka;

use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DebugException;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;

class ENeschopenkaDialog extends RonPortalForm
{
    private $cisloDokladu;

    /**
     * @param Database $database
     * @throws DBException
     * @throws DebugException
     */
    public function __construct($database, $cisloDokladu)
    {
        parent::__construct($database, 'new', true);

        $this ->cssClass = 'large-form rondialog ronform form-25-75';
        $this ->code = 'formeneschopenka';

        $this ->stornoAction = "#";

        $this ->htmlParams = array_merge($this ->htmlParams, [
            'id' => 'eneschopenka_dialog',
            'data-ronform-submit-url' => '?menuid=eneschopenka&action=edit',
            'data-ronform-after-submit' => 'reload',
            'title' => TextManager::_('E_neschopenka')
        ]);

        $this->cisloDokladu = $cisloDokladu;

        $this ->sqlTableName = 'ENeschopenka';

        $this ->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('CisloDokladu') ->setPrimary(true)->setCaption('E_neschopenka_cislo_dokladu')->setVisible(true)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-cislo-dokladu')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'CisloDokladu']);

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setSqlFieldName("Osoba.Jmeno + ' ' + Osoba.Prijmeni")->setFieldName('Osoba')
            ->setCaption('Osoba')->setCaptionColumnClass('caption font-weight-bold')
            ->addFieldCssClass('ronform-field')
            ->setCode('field-osoba')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Osoba']);

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setSqlFieldName('DruhDavkyNZ.Popis')->setFieldName('DruhDavky')
            ->setCaption('E_neschopenka_druh')->setCaptionColumnClass('caption font-weight-bold')
            ->addFieldCssClass('ronform-field')
            ->setCode('field-druhdavky')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'DruhDavky']);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumPocatkuDavky') ->setCaption('E_neschopenka_datum_pocatku')->setRequired(true)
            ->setReadOnly(2)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-datum-pocatku')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'DatumPocatkuDavky']);

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('UkoncenaDavka') ->setCaption('E_neschopenka_ukoncena_davka')->setRequired(true)
            ->addFieldCssClass('ronform-field')
            ->setRowId('ukoncena-davka')
            ->setCode('field-ukoncena-davka')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'UkoncenaDavka']);

        $this ->loadConfiguration();
    }

    public function renderMessages()
    {
        //neni nutne opet vypisovat
    }

    function buildSelectSQL()
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT ' . $this ->getSqlSelectPart() .
            "  FROM ENeschopenka JOIN DruhDavkyNZ ON DruhDavkyNZ.Kod = ENeschopenka.DruhDavky\n" .
            "       JOIN Osoba ON Osoba.RC = ENeschopenka.RC\n" .
            ' WHERE ENeschopenka.CisloDokladu = ' . $params ->addParameter($this->cisloDokladu);

        return new SqlStatement($sql, $params);
    }

}