<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\Controls\BrowserField;
use Ron\DB\ArrayCursor;
use Ron\DB\Cursor;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use Ronportal\Controls\FilterFormFilterVariabilni;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;

/**
 * Zobrazi nastavene mesicni vysledky
 */
class MesicniVysledkyBrowser extends VysledkyBrowser
{

    /** @var string[]   pole vyrazu pro vypocet jednotlivych polozek */
    private $expressions;
    /** @var string[]   pole souctu jednotlivych polozek */
    private $rowCaptions;
    /** @var bool       jestli se maji zobrazovat variantni obdobi (true) nebo mesice (false) */
    private $variantni;
    /** @var bool       ve zpracovane dochazce se filtr nezobrazi */
    public $showFilter;

    private $slozky;



    function __construct($database, $variantni)
    {
        $action = $variantni ? 'variantnivysledky' : 'mesicnivysledky';
        parent::__construct($database, $action);

        $this->showFilter = true;
        $this->variantni = $variantni;

        $this ->code = 'browsermesicni';

        $tableName = $variantni ? 'VariantniVysledek' : 'MesicniVysledekVypoctu';
        $this ->sqlFromPart =
            "$tableName INNER JOIN TypCasoveSlozkyMzdy ON TypCasoveSlozkyMzdy.Kod = $tableName.TypCasoveSlozkyMzdy";

        $this ->sqlSelectPart = 'TypCasoveSlozkyMzdy, Hodiny, Dny, KalendarniDny';

        $this->cssClass .= ' ' . $action;

        $this ->title = \Ron\ronText($variantni ? 'Prehled_variantnich_vysledku' : 'Mesicni_vysledky');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('1', 0) ->setHeaderText('Casova_slozka')
            ->setCode('col-kod');

        $this->addField()
            ->setSqlFieldName('1', 1) ->setHeaderText('Casova_slozka')
            ->setCode('col-casovaslozka');

        $this->addField()
            ->setSqlFieldName('2', 2) ->setHeaderText('Vysledek_Hodiny')
            ->setFormat('hodiny') ->setAlign('right') ->setClassName('hodiny')
            ->setCode('col-hodiny');

        $this->addField()
            ->setSqlFieldName('3', 3) ->setHeaderText('Vysledek_Pracovni_dny')
            ->setFormat('number_2') ->setAlign('right') ->setClassName('dny')
            ->setCode('col-dny');

        $this->addField()
            ->setSqlFieldName('4', 4) ->setHeaderText('Vysledek_Kalendarni_dny')
            ->setFormat('number_2') ->setAlign('right') ->setClassName('kalendarnidny')
            ->setCode('col-kalendarni-dny');
    }


    function setFilterForm()
    {
        if (!$this ->showFilter) {
            return;
        }

        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        if ($this->variantni) {
            $this ->filterForm ->addFilter(
                new FilterFormFilterVariabilni($this ->filterForm),
                true
            );
        } else {
            $this->filterForm->addFilterObdobi(); // jenOtevrene = false
        }

        $prava = Prava::vratPrava('prohlizeni');

        if ($prava->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this->filterForm->addFilterRozdeleni1(null, $prava);
            $this->filterForm->addFilterRozdeleni2(null, $prava);
        }

        $f = $this->filterForm->addFilterPP(false, $prava); // datum od-do podle obdobi
        $f ->setOptionsSqlFilter('PracovniPomer.ZpracovatDochazku = 1');

        $this ->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
        //$this ->filterForm ->nastavAkciGet($this ->variantni ? 'variantnivysledky' : 'mesicnivysledky');
    }


    function renderFilter()
    {
        if ($this->showFilter) {
            if ($this->variantni) {
                $this->odkazyVariabilniVysledky('variantnivysledky');
            } else {
                $this->odkazyMesicniVysledky('mesicnivysledky');
            }

            parent::renderFilter();
        }
    }


    /**
     * Vytvori pole hodnot z databaze ve tvaru <kod_casove_slozky> => [ 'h' => hodiny, 'd' => dny, 'k' => kal.dny ]
     * @param Cursor $cursor
     * @return array
     */
    function nactiDataDoPole($cursor)
    {
        $result = [];

        while ($zaznam = $cursor ->fetch()) {
            $result['_' . $zaznam[0]] = [ 'h' => $zaznam[1], 'd' => $zaznam[2], 'k' => $zaznam[3] ]; // nastaveni pridava _!
        }

        return $result;
    }


    /**
     * Vytvori pole PHP promennych z hodnot v databazi (pouziva se, pokud jsou povoleny vzorce)
     * @param Cursor $cursor
     * @return string
     */
    function nactiDataDoPromennych($cursor)
    {
        $result = '';

        while ($zaznam = $cursor ->fetch()) {
            $result = $result .
                '$h_' . "$zaznam[0]=" . str_replace(',', '.', $zaznam[1]) . ';' .
                '$d_' . "$zaznam[0]=" . str_replace(',', '.', $zaznam[2]) . ';' .
                '$k_' . "$zaznam[0]=" . str_replace(',', '.', $zaznam[3]) . ';';
        }

        return $result;
    }



    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $sql =
            'RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            ' AND ' .  ($this ->variantni ? 'VariantniObdobi' : 'Mesic') . ' = ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiOd());

        return new SqlStatement($sql, $params);
    }


    /**
     * Vypocte hodnotu podle zadaneho vzorce.
     * @param array $slozky     Slozky jsou hodnoty nactene z databaze
     * @param string $vzorec
     * @param string $pismeno   Urcuje jestli se pocitaji hodiny (h), dny (d) nebo kalendarni dny (k)
     * @return int
     */
    function vypoctiHodnotu($slozky, $vzorec, $pismeno)
    {
        $vzorec = StringUtils::processIdentifiers($vzorec, '$' . $pismeno . '_');
        //$vzorec = str_replace('_', '$' . $pismeno . '_', $vzorec); - puvodi verze nezvlada podtrzitka v nazvu casove slozky!

        error_reporting(error_reporting() & ~E_NOTICE); // vypne warningy undefined variable
        eval($slozky);

        $result = 0;
        eval('$result = ' . $vzorec . ';');

        return $result;
    }


//    function vypisMesicniVysledky($slozky)
//    {
//        foreach ($this->rowCaptions as $index => $rowname) {
//            $emptyrow = false;
//
//            $h = null;
//            $d = null;
//            $k = null;
//
//            if ($this->expressions[$index]) {
//                if ( RonPortalSettings::getIntValue(RonPortalSettings::MESICNI_VYSLEDKY_ZOBRAZIT) == 2 ) {
//                    // vzorce povoleny
//                    $h = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'h');
//                    $d = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'd');
//                    $k = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'k');
//                } else {
//                    // vzorce nepovoleny
//                    $data = $slozky[$this->expressions[$index]] ?? null;
//
//                    if ($data) {
//                        $h = $data['h'];
//                        $d = $data['d'];
//                        $k = $data['k'];
//                    }
//                }
//            } else {
//                $emptyrow = true;
//            }
//
//            if (!RonPortalSettings::getIntValue('skrytnulovemesicni') || $emptyrow || $h || $d || $k) {
//                $classname =
//                    ($index % 2) ? 'browsercolor1' : 'browsercolor2';
//
//                $classname .=
//                    ' mv' . StringUtils::htmlTextConvert(str_replace(' ', '_', $this->expressions[$index]));
//
//                $rowdata = [TextManager::_($rowname), $h, $d, $k];
//
//                $this->renderLine($rowdata, $classname, $index);
//            }
//        }
//    }


    function loadMesicniVysledky($slozky)
    {
        $result = [];

        foreach ($this->rowCaptions as $index => $rowname) {
            $emptyRow = false;

            $h = null;
            $d = null;
            $k = null;

            if ($this->expressions[$index]) {
                if ( RonPortalSettings::getIntValue(RonPortalSettings::MESICNI_VYSLEDKY_ZOBRAZIT) == 2 ) {
                    // vzorce povoleny
                    $h = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'h');
                    $d = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'd');
                    $k = $this->vypoctiHodnotu($slozky, $this->expressions[$index], 'k');
                } else {
                    // vzorce nepovoleny
                    $data = $slozky[$this->expressions[$index]] ?? null;

                    if ($data) {
                        $h = $data['h'];
                        $d = $data['d'];
                        $k = $data['k'];
                    }
                }
            } else {
                $emptyRow = true;
            }

            if (!RonPortalSettings::getIntValue('skrytnulovemesicni') || $emptyRow || $h || $d || $k) {
                $classname =
                    ($index % 2) ? 'browsercolor1' : 'browsercolor2';

//                $classname .=
//                    ' mv' . StringUtils::htmlTextConvert(str_replace(' ', '_', $this->expressions[$index]));

                $result[] = [
                    $this->expressions[$index], // kod
                    TextManager::_($rowname),   // nadpis
                    $h, $d, $k]; // hodnoty
            }
        }

        return $result;
    }


    public function loadSQL($sql)
    {
        parent::loadSQL($sql);

        StringUtils::parseStringToArrays(
            RonPortalSettings::getValue(RonPortalSettings::MESICNI_VYSLEDKY_RADKY),
            $this->rowCaptions, $this->expressions, true);

        //$cursor = $this->nactiMesicniVysledky();

        if ( RonPortalSettings::getIntValue(RonPortalSettings::MESICNI_VYSLEDKY_ZOBRAZIT) == 2 ) {
            // povoleny vyrazy => nacita do retezce PHP promennych
            $this->slozky = $this->nactiDataDoPromennych($this ->cursor);
        } else {
            // nejsou povoleny vyrazy => nacita do pole
            $this ->slozky = $this ->nactiDataDoPole($this ->cursor);
        }

        $this ->cursor = new ArrayCursor($this ->loadMesicniVysledky($this ->slozky));
    }


//    function load($sqlFilter = '')
//    {
//        if ($this->showFilter) {
//            $this->setFilterForm();
//            $this->filterForm->loadData();
//        }
//
//        StringUtils::parseStringToArrays(
//            RonPortalSettings::getValue(RonPortalSettings::MESICNI_VYSLEDKY_RADKY),
//            $this->rowCaptions, $this->expressions, true);
//
//        $cursor = $this->nactiMesicniVysledky();
//
//        if ( RonPortalSettings::getIntValue(RonPortalSettings::MESICNI_VYSLEDKY_ZOBRAZIT) == 2 ) {
//            // povoleny vyrazy => nacita do retezce PHP promennych
//            $this->slozky = $this->nactiDataDoPromennych($cursor);
//        } else {
//            // nejsou povoleny vyrazy => nacita do pole
//            $this ->slozky = $this ->nactiDataDoPole($cursor);
//        }
//
//        $this ->cursor = new ArrayCursor($this ->loadMesicniVysledky($this ->slozky));
//    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        // prida class mv_<kod_casove_slozky>
        $mvClass = 'mv' . StringUtils::htmlTextConvert(str_replace(' ', '_', $values['col-kod']));
        $className = ($className ? $className . ' ' : '') . $mvClass;

        return parent::getRowAttributes($values, $className, $rowNumber);
    }

//    function render()
//    {
//        $this->renderHeader();
//
//        $this->renderTableStart();
//        echo '<tbody>';
//        $this->vypisMesicniVysledky($this ->slozky);
//        echo "</tbody>\n";
//        $this->renderTableEnd();
//    }

}
