<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;

/**
 * Seznam mesicnich vysledku zamestnance. Zobrazuje vsechny mesicni vysledky nactene SQL dotazem,
 * takze pokud to neni omezeno nejakym uzivatelskym filtrem, tak zobrazi vsechny zaznamy v mesici.
 */
class MesicniVysledkyBrowser1 extends VysledkyBrowser
{
    /** @var bool    Ve zpracovane dochazce se filtr nezobrazi */
    var $showFilter = true;

    /** @var bool  */
    private $skrytNuloveVysledky;

    function __construct($database)
    {
        parent::__construct($database, 'mesicnivysledky');

        $this->code = 'browsermesicni';

        $this ->title = \Ron\ronText('Mesicni_vysledky');

        $this ->skrytNuloveVysledky = RonPortalSettings::getIntValue('skrytnulovemesicni');

        $this->sqlFromPart =
            'MesicniVysledekVypoctu, TypCasoveSlozkyMzdy';

        $this->sqlWherePart =
            'TypCasoveSlozkyMzdy.Kod = MesicniVysledekVypoctu.TypCasoveSlozkyMzdy';

        $this->cssClass .= ' mesicnivysledky';

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.TypCasoveSlozkyMzdy')
            ->setCode('col-kod')
            ->setHeaderText('Kod_casove_slozky') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('TypCasoveSlozkyMzdy.Popis')
            ->setCode('col-popis')
            ->setHeaderText('Casova_slozka') ->setOrdering(true)
            ->setTranslationPrefix('db_typcasoveslozkymzdy_popis_', 'col-kod');

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.Hodiny')
            ->setCode('col-hodiny')
            ->setHeaderText('Vysledek_Hodiny') ->setFormat('hodiny') ->setAlign('right')
            ->setOrdering(true)
            ->setClassName('hodiny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.Dny')
            ->setCode('col-dny')
            ->setHeaderText('Vysledek_Pracovni_dny') ->setFormat('number_2') ->setAlign('right')
            ->setOrdering(true)
            ->setClassName('dny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.KalendarniDny')
            ->setCode('col-kalendarni-dny')
            ->setHeaderText('Vysledek_Kalendarni_dny') ->setFormat('number_2') ->setAlign('right')
            ->setOrdering(true)
            ->setClassName('kalendarnidny');

        $this->addField()
            ->setSqlFieldName('MesicniVysledekVypoctu.Poznamka')
            ->setCode('col-poznamka') ->setVisible(false) ->setHeaderText('Poznamka');

        $this->defaultSort = 'col-kod';
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'skrytNuloveVysledky') {
            $this ->skrytNuloveVysledky = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }

    public function getTableAttributes()
    {
        $result = parent::getTableAttributes();
        $result['data-skrytNuloveVysledky'] = $this ->skrytNuloveVysledky ? '1' : '0';
        return $result;
    }


    function setFilterForm()
    {
        if ($this->showFilter) {
            $this ->filterForm = $this->createFilterForm('ADS', 'prohlizeni');

            $this->filterForm->getFilter('osoba') ->setOptionsSqlFilter('PracovniPomer.ZpracovatDochazku = 1');
            $this->filterForm ->downloadFotoOsobyUrl = '?menuid=editace&action=foto&idpp=';
        }
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        $this ->browserUrl =
            '?menuid=mesicnivysledky&idpp=' . $this ->filterForm ->getFilterValue('osoba') .
            '&obdobi=' . DateUtils::dateToUrlString($this ->filterForm ->getFilterValue('obdobi'));
    }



    function renderFilter()
    {
        if ($this->showFilter) {
            $this->odkazyMesicniVysledky('mesicnivysledky');
            parent::renderFilter();
        }
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result = ($this ->showFilter)
            ? $this ->filterForm ->buildSqlFilter($params)
            : '';

        return new SqlStatement(
            $result .
            ($result ? ' AND ' : '') .
            'MesicniVysledekVypoctu.Mesic = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ' AND MesicniVysledekVypoctu.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND MesicniVysledekVypoctu.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND MesicniVysledekVypoctu.DatumNastupu = ' . $params->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }


    function enableLine($values)
    {
        if (!$this ->skrytNuloveVysledky) {
            return true;
        }

        // pokud jsou vsechny hodnoty vysledku nulove, tak radek skryje
        return (float)$values['col-hodiny'] || (float)$values['col-dny'] || (float)$values['col-kalendarni-dny'];
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        // prida class mv_<kod_casove_slozky>
        $mvClass = 'mv_' . StringUtils::htmlTextConvert(str_replace(' ', '_', $values['col-kod']));
        $className = ($className ? $className . ' ' : '') . $mvClass;

        return parent::getRowAttributes($values, $className, $rowNumber);
    }

}
