<?php
/**
 * stanislav.elbl, 23.03.2021
 */

namespace RonPortal\Dochazka;

use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Kalendar;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;


class PoctyLidiBrowser extends RonPortalBrowser
{
    const SETTINGS_ZOBRAZOVAT_NULOVY_POCET = 'zobrazovatnulovypocet';
    const SETTINGS_SLOUPCE = 'sloupcepoctulidi';

    protected $expressions; // pole vyrazu pro vypocet jednotlivych polozek
    protected $soucet;      // pole souctu jednotlivych polozek
    protected $zpracovanodo;
    protected $zkontrolovanodo;
    protected $schvalenodo;

    protected $data;


    function __construct($database)
    {
        parent::__construct($database, 'pocty');

        $this->cssClass = 'browser grid';

        $this->title = TextManager::_('Pocty_zamestnancu');
        $this->code = 'browserpoctylidi';

        $this ->parseFieldsString(RonPortalSettings::getValue(self::SETTINGS_SLOUPCE));
    }


    /**
     * Vytvori pole hlavicek sloupcu a pole vyrazu pro sloupce
     * @param string $str
     */
    function parseFieldsString($str)
    {
        $this->expressions = [];

        $this ->addField()
            ->setSqlFieldName('1') ->setHeaderText('Date') ->setAlign('center')
            ->setCode('col-datum');

        $captions = [];
        StringUtils::parseStringToArrays($str, $captions, $this->expressions, false);

        $index = 1;
        foreach ($captions as $caption) {
            $this ->addField()
                ->setSqlFieldName('1') ->setHeaderText($caption) ->setAlign('right')
                ->setCode('col-' . $index);
            $index++;
        }

        foreach ($this->expressions as $i => $expression)
            $this->expressions[$i] = str_replace('_', '$_', $expression);
    }


    function nactiPoctyLidi()
    {
        $params = $this ->database ->createParameters();

        if ($this ->filterForm ->getFilterValue('rozdeleni') !== null) {
            $sql =
                'SELECT DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy, SUM(DenniVysledekVypoctu.Dny)' .
                ' FROM DenniVysledekVypoctu, RozdeleniOsob' .
                ' WHERE DenniVysledekVypoctu.RC = RozdeleniOsob.RC' .
                ' AND DenniVysledekVypoctu.OsobniCislo = RozdeleniOsob.OsobniCislo' .
                ' AND DenniVysledekVypoctu.DatumNastupu = RozdeleniOsob.DatumNastupu' .
                ' AND RozdeleniOsob.KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->filterForm ->getFilterValue('rozdeleni', 'typRozdeleni')) .
                ' AND RozdeleniOsob.Rozdeleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('rozdeleni')) .
                ' AND RozdeleniOsob.PlatnostOd <= DenniVysledekVypoctu.Den' .
                ' AND (RozdeleniOsob.PlatnostDo >= DenniVysledekVypoctu.Den OR RozdeleniOsob.PlatnostDo IS NULL)' .
                ' AND DenniVysledekVypoctu.Den >= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiOd()) .
                ' AND DenniVysledekVypoctu.Den <= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiDo()) .
                ' GROUP BY DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy' .
                ' ORDER BY DenniVysledekVypoctu.Den';
        } else {
            $sql =
                'SELECT DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy, SUM(DenniVysledekVypoctu.Dny)' .
                ' FROM DenniVysledekVypoctu' .
                ' WHERE DenniVysledekVypoctu.Den >= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiOd()) .
                ' AND DenniVysledekVypoctu.Den <= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiDo()) .
                ' GROUP BY DenniVysledekVypoctu.Den, DenniVysledekVypoctu.TypCasoveSlozkyMzdy' .
                ' ORDER BY DenniVysledekVypoctu.Den';
        }

        return $this ->database ->select($sql, $params);
    }


    function vytvorRadek($datum, $valuesStr, $jeToSoucet)
    {
        $browserRow = [];

        if (!$jeToSoucet) {
            // V nulte polozce radku ulozi typ dne 0=pracovni, 1=vikend, 2=svatek
            $browserRow[] = Kalendar::vratTypDne($this ->database, $datum);

            // Vytvori hodnotu v prvnim sloupci - datum
            $browserRow[] = date('j.n.Y', $datum);
        }

        // Vytvori jednotlive promenne pro vypocty a doplni jejich hodnoty
        eval($valuesStr);

        $vysledek = null;

        // Provede jednotlive vypocty
        foreach ($this->expressions as $index => $expression) {
            if ($expression) {
                eval('$vysledek=' . $expression . ';');
            } else {
                $vysledek = null;
            }

            if (!$jeToSoucet && !$vysledek && !RonPortalSettings::getIntValue(self::SETTINGS_ZOBRAZOVAT_NULOVY_POCET)) {
                $browserRow[] = '';
            } else {
                $browserRow[] = StringUtils::format('number2', $vysledek);
            }

            if (!$jeToSoucet) {
                $this->soucet[$index] += $vysledek;
            }
        }

        return $browserRow;
    }


    /**
     * @param Cursor $cursor
     * @return array
     */
    function prevedDataDoPole($cursor)
    {
        // vypne warningy/noticy "undefined variable"
        ini_set('error_reporting', ini_get('error_reporting') & ~E_WARNING  &  ~E_NOTICE);

        $this->soucet = [];
        $datum = $this ->filterForm ->getOmezitObdobiOd();
        $posledniDatum = $this ->filterForm ->getOmezitObdobiDo();
        $result = [];

        $zaznam = $cursor ->fetch();
        // prevede data na zacatek, resp. konec dne
        $datum = DateUtils::startOfTheDay($datum);
        $posledniDatum = DateUtils::endOfTheDay($posledniDatum);

        // Pro kazdy den vytvori radek
        while ($datum <= $posledniDatum) {
            // Prida do vypoctu vsechny zaznamy z daneho dne
            $valuesStr = '';
            $dalsiDatum = DateUtils::incDayStart($datum, 1);

            while ($zaznam && $this ->database ->stringToTimestamp($zaznam[0]) < $dalsiDatum) {
                $valuesStr = $valuesStr . '$_' . $zaznam[1] . '+=' . $zaznam[2] . ';';
                $zaznam = $cursor ->fetch();
            }

            $result[] = $this->vytvorRadek($datum, $valuesStr, false);
            $datum = DateUtils::startOfTheDay($datum + 86400 + 3600); // takhle je to kvuli prechodu letni/zimni cas
        }

        $cursor ->close();

        return $result;
    }

    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        $this ->filterForm ->addFilterObdobi();

        $f = $this ->filterForm ->addFilterRozdeleni1();
        $f ->setShowVcetnePodrizenych(false);
    }


    function load($sqlFilter = '')
    {
        $this ->loadConfiguration();

        $this ->setFilterForm();

        if ($this ->filterForm) {
            $this ->filterForm ->loadData();
        }

        Prava::nastavObdobiProPrava($this ->filterForm ->getOmezitObdobiOd(), $this ->filterForm ->getOmezitObdobiDo());

        $cursor = $this ->nactiPoctyLidi();
        $this ->data = $this ->prevedDataDoPole($cursor);
    }


    function renderFilter()
    {
        $this ->filterForm ->render();
    }


    function vypisPoctyLidi()
    {
        foreach ($this ->data as $index => $row) {
            echo $row[0]
                ? "<tr class='browser_row_high'>"
                : '<tr>';

            // Vypise obsah radku
            $prvniDatovySloupec = 1;

            for ($i = $prvniDatovySloupec; $i < count($row); $i++) {
                echo ($i == $prvniDatovySloupec)
                    ? '<td class="browser_rowheader">' // datum
                    : '<td class="align-right">';

                echo $row[$i];
                echo '</td>';
            }

            echo "</tr>\n";
        }
    }


    function vypisSoucet()
    {
        echo '<tr class="browserfooter">';
        echo '<td>' . \Ron\ronText('Celkem') . '</td>';
        foreach ($this->soucet as $index => $value) {
            echo '<td class="align-right"> ' . StringUtils::format('number2', $value) . '</td>';
        }
        echo "</tr>\n";
    }


    function render()
    {
        $this->renderHeader();
        $this->renderTableStart();
        echo "<tbody class='border-bottom'>\n";
        $this->vypisPoctyLidi();
        echo "</tbody>\n";
        echo "<tbody>\n";
        $this->vypisSoucet();
        echo "</tbody>\n";
        $this->renderTableEnd();
    }

}
