<?php
/**
 * pavel.rudol, 01.11.2022 18:05
 */

namespace RonPortal\Dovednosti;

use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\RozdeleniOsob;

/**
 * Osoba a jeji ziskane, pozadovane dovednosti.
 */
class OsobaSDovednosti
{
    protected $rc;

    public $jmeno;
    public $prijmeni;

    public $pocetZiskanychDovednosti;
    public $pocetPozadovanychDovednosti;

    /** @var RozdeleniOsob[][] */
    public $prirazenaRozdeleniOsoby;
    /** @var int */
    public $idOsoby;
    /** @var string[] */
    public $osobniCislo;
    /** @var string[] */
    public $typPP;
    /** @var PrirazeniKalendare[] */
    public $kalendar;

    /** @var DovednostOsoby[] */
    public $pozadovaneDovednosti;

    /** @var DovednostOsoby[] */
    public $ziskaneDovednosti;

    public $lzeEditovat;
    public $lzeZobrazit;
    public $lzeZobrazitOsobniUdaje;

    /** Nastavi udaje osoby a inicializuje pole.
     * @param $rc
     * @param $jmeno
     * @param $prijmeni
     */
    public function __construct($rc, $jmeno, $prijmeni)
    {
        $this -> rc = $rc;
        $this -> jmeno = $jmeno;
        $this -> prijmeni = $prijmeni;

        $this->osobniCislo = [];
        $this->typPP = [];
        $this->kalendar = [];

        $this -> ziskaneDovednosti = [];
        $this -> pozadovaneDovednosti = [];
        $this -> prirazenaRozdeleniOsoby = [];

        $this -> pocetZiskanychDovednosti = 0;
        $this -> pocetPozadovanychDovednosti = 0;
    }

    /** Vraci ID osoby.
     * @return string
     */
    public function getRC()
    {
        return $this->rc;
    }

    /** Nacte udaje z PP do pole osobnich cisel, typu PP.
     * @param $row
     * @return void
     */
    public function nactiUdajePP($row)
    {
        if (!in_array($row['col-pp-osobnicislo'], $this->osobniCislo)) {
            $this->osobniCislo[] = $row['col-pp-osobnicislo'];
        }
        if (!in_array($row['col-pp-typ'], $this->typPP)) {
            $this->typPP[] = $row['col-pp-typ'];
        }
    }

    /** Vrati osobni cisla oddelena carkou. Pouziva se ve vystupu do CSV.
     * @return string
     */
    public function getOsobniCislo()
    {
        $result = '';
        foreach ($this->osobniCislo as $item) {
            $result .= ($result ? ', ' : '') . $item;
        }
        return $result;
    }

    /** Vrati typy PP oddelene carkou. Pouziva se ve vystupu do CSV.
     * @return string
     */
    public function getTypPP()
    {
        $result = '';
        foreach ($this->typPP as $item) {
            $result .= ($result ? ', ' : '') . $item;
        }
        return $result;
    }

    /** Vrati kody/popisy prirazenych kalendaru osoby.
     * @param int $typZobrazeni 1 = kody, 2 = popisy
     * @return string
     */
    public function getKalendar($typZobrazeni)
    {
        $result = '';
        foreach ($this->kalendar as $kalendar) {
            if ($typZobrazeni == 1) {
                $result .= ($result ? ', ' : '') . $kalendar->kod;
            } elseif ($typZobrazeni == 2) {
                $result .= ($result ? ', ' : '') . $kalendar->popis;
            }
        }
        return $result;
    }

    /** Nastavi pozadovanou dovednost k osobe a upravi pocty pozadovanych dovednosti.
     * @param DovednostOsoby $dovednost
     */
    public function priradPozadovanouDovednost($dovednost)
    {
        //pokud zamestnanec jeste nema pozadavek na dovednost, doplni se do seznamu pozadavku
        if (!array_key_exists($dovednost->kod, $this->pozadovaneDovednosti))
        {
            $this ->pozadovaneDovednosti[$dovednost->kod] = $dovednost;
            $this ->pocetPozadovanychDovednosti += $dovednost->uroven;
        }
        else
        {
            //pokud je jiny pozadavek dovednosti na vyssi uroven, zvysi se pozadovana uroven dovednosti a celkovy pocet pozadovanych urovni
            if ($this ->pozadovaneDovednosti[$dovednost->kod]->uroven < $dovednost->uroven)
            {
                $rozdil = $dovednost->uroven - $this ->pozadovaneDovednosti[$dovednost->kod]->uroven;
                $this ->pozadovaneDovednosti[$dovednost->kod]->uroven = $dovednost->uroven;
                $this ->pozadovaneDovednosti[$dovednost->kod]->popisUrovne = $dovednost->popisUrovne;
                $this ->pocetPozadovanychDovednosti += $rozdil;
            }
        }
    }

    /** Nastavi ziskanou dovednost k osobe a upravi pocty ziskanych dovednosti.
     *
     * @param DovednostOsoby $dovednost
     * @param int $specifikaceUrovneDovednosti
     */
    public function priradZiskanouDovednost($dovednost, $specifikaceUrovneDovednosti)
    {
        //pokud zamestnanec jeste nema prirazenou dovednost, doplni se do seznamu ziskanych dovednosti
        if (!array_key_exists($dovednost->kod, $this->ziskaneDovednosti))
        {
            $this ->ziskaneDovednosti[$dovednost->kod] = $dovednost;
            $this ->pocetZiskanychDovednosti += $dovednost->uroven;
        }
        else
        {
            //pokud je jina ziskana dovednost na vyssi uroven, zvysi se ziskana uroven dovednosti a celkovy pocet ziskanych urovni
            if ($this ->ziskaneDovednosti[$dovednost->kod]->uroven < $dovednost->uroven)
            {
                $rozdil = $dovednost->uroven - $this ->ziskaneDovednosti[$dovednost->kod]->uroven;
                $this ->ziskaneDovednosti[$dovednost->kod]->uroven = $dovednost->uroven;
                $this ->ziskaneDovednosti[$dovednost->kod]->popisUrovne = $dovednost->popisUrovne;
                $this ->pocetZiskanychDovednosti += $rozdil;
            }
        }

        $this ->ziskaneDovednosti[$dovednost->kod]->specifikaceUrovne[$dovednost->uroven] = $specifikaceUrovneDovednosti;
    }

    /** Prida prirazene rozdeleni k osobe, pokud jeste nebylo nacteno.
     * @param RozdeleniOsob $rozdeleni
     * @param int $typRozdeleni
     */
    public function priradRozdeleniOsoby($rozdeleni, $typRozdeleni)
    {
        if (!isset($this->prirazenaRozdeleniOsoby[$typRozdeleni])) {
            $this->prirazenaRozdeleniOsoby[$typRozdeleni] = [];
        }
        if (!array_key_exists($rozdeleni->kod, $this->prirazenaRozdeleniOsoby[$typRozdeleni])) {
            $this->prirazenaRozdeleniOsoby[$typRozdeleni][$rozdeleni->kod] = $rozdeleni;
        }
    }

    /** Prida kalendar k osobe, pokud jeste nebyl nacten.
     * @param PrirazeniKalendare $kalendar
     */
    public function priradKalendar($kalendar)
    {
        if (!array_key_exists($kalendar->kod, $this->kalendar)) {
            $this->kalendar[$kalendar->kod] = $kalendar;
        }
    }

    /** Vrati uroven ziskane dovednosti osoby. Pokud tuto dovednost nema vraci prazdny retezec, nebo 0.
     * @param DovednostOsoby $dovednost
     * @param bool $zobrazitNulu
     * @return int|string
     */
    public function urovenZiskaneDovednosti($dovednost, $zobrazitNulu = false)
    {
        if (isset($this->ziskaneDovednosti[$dovednost->kod])) {
            return $this->ziskaneDovednosti[$dovednost->kod]->uroven;
        }

        return $zobrazitNulu ? 0 : '';
    }

    /** Vrati uroven pozadovane dovednosti osoby. Pokud tato dovednost neni pozadovana vraci prazdny retezec, nebo 0.
     * @param DovednostOsoby $dovednost
     * @param bool $zobrazitNulu
     * @return int|string
     */
    public function urovenPozadovaneDovednosti($dovednost, $zobrazitNulu = false)
    {
        if (isset($this->pozadovaneDovednosti[$dovednost->kod])) {
            return $this->pozadovaneDovednosti[$dovednost->kod]->uroven;
        }

        return $zobrazitNulu ? 0 : '';
    }

}
