<?php
/**
 * pavel.rudol, 01.11.2022 17:55
 */

namespace RonPortal\Dovednosti;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\DateUtils;
use Ron\DB\ArrayCursor;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\DebugException;
use Ron\MVC\DataOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FotoOsobyBrowserField;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\PritomnostBrowserField;
use RonPortal\Dochazka\PrirazeniKalendare;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniOsob;
use RonPortal\RozdeleniUtils;
use RonPortal\TypRozdeleni;
use RonPortal\Zamestnanci\SeznamOsobBrowser;

class DovednostBrowserField extends BrowserField
{
   /** @var OsobaSDovednosti[] */
    protected $osoby;

    /**
     * @param OsobaSDovednosti[] $osoby
     * @return void
     */
    public function setData($osoby)
    {
        $this->osoby = $osoby;
    }

    /**
     * @param string $rc
     * @return OsobaSDovednosti
     */
    public function getDataOsoby($rc)
    {
        return $this->osoby[$rc];
    }
}

class DovednostOsobyBrowserField extends DovednostBrowserField
{
    private $zobrazeniUrovneDovednosti;
    private $ziskaneDovednosti;

    /** @var DovednostOsoby */
    protected $dovednost;

    protected $pocetDovednostiTypu;
    protected $zobrazitPlatnost;

    /**
     * @param DovednostOsoby $dovednost
     * @param int $zobrazeniUrovneDovednosti 0 = zobrazí kód, 1 = zobrazí popis
     * @param bool $zobrazitPlatnost
     * @param bool $ziskaneDovednosti
     */
    public function __construct($dovednost, $zobrazeniUrovneDovednosti, $zobrazitPlatnost = false, $ziskaneDovednosti = true)
    {
        parent::__construct();

        $this->dovednost = $dovednost;
        $this->zobrazeniUrovneDovednosti = $zobrazeniUrovneDovednosti;
        $this->header = $this->dovednost->popis;

        $this->zobrazitPlatnost = $zobrazitPlatnost;
        $this->ziskaneDovednosti = $ziskaneDovednosti;
    }

    /** Nastavi kolik je nacteno dovednosti stejneho typu.
     * @param array $pocetDovednostiTypu
     * @return void
     */
    public function setPocty($pocetDovednostiTypu)
    {
        $this->pocetDovednostiTypu = $pocetDovednostiTypu;
    }

    /** Vrati kod typu dovednosti.
     * @return string
     */
    public function getTypDovednosti()
    {
        return $this->dovednost->typ;
    }

    /** Zobrazi hlavicku s popisem dovednosti. Pokud je text dlouhy, zalomi ho na vice radku.
     * @param $fieldIndex
     * @param $sortField
     * @param $url
     * @return string
     */
    function getHeader($fieldIndex, $sortField, $url = null)
    {
        $class = (strlen($this->header) > 30 ? 'dovednost_longtext' : 'dovednost');

        $result =
            '<th ' . $this->getHeaderClass() . '>';

        $result .=
            "<div class='$class' title='" . StringUtils::htmlTextConvert($this->header) . "'>" .
            wordwrap(StringUtils::htmlTextConvert($this->header), 30, '<br/>', true) .
            "</div>";

        $result .= '</th>';

        if ($this->zobrazitPlatnost) {
            $result .= '<th><div class="dovednost" >' . TextManager::_('Platnost_do') . '</div></th>';
        }

        return $result;
    }

    /** Zobrazi hlavicku s popisem typu dovednosti.
     * @return string
     */
    function getHeaderTypDovednosti()
    {
        $this->setHeaderClass('align-center');

        $result =
            '<th class="' . $this->getHeaderClass() . '" colspan="' . $this->pocetDovednostiTypu[$this->dovednost->typ] . '">' .
            StringUtils::htmlTextConvert($this->dovednost->typPopis) .
            '</th>';

        if ($this->zobrazitPlatnost) {
            $result .= '<th><div class="dovednost" >' . TextManager::_('Platnost_do') . '</div></th>';
        }

        return $result;
    }

    /** Vypise bunku se ziskanou urovni dovednosti, kde hrany bunky jsou vybarveny dle dosazene urovne 1 - x (Od urovne 4 jsou vybarveny 4 strany ctverce)
     * @param OsobaSDovednosti $osoba
     * @return string
     */
    protected function getZiskanaDovednost($osoba)
    {
        $result = '<div class="' . ($osoba->urovenZiskaneDovednosti($this->dovednost, true) > 0 ? 'dovednosti_dovednost_osoby ' : '') .
            'ziskana_'.$osoba->urovenZiskaneDovednosti($this->dovednost) . ($osoba->urovenZiskaneDovednosti($this->dovednost, true) > 4 ? ' ziskana_4' : '');
        if (isset($osoba->ziskaneDovednosti[$this->dovednost->kod])  &&  is_array($osoba->ziskaneDovednosti[$this->dovednost->kod]->specifikaceUrovne)) {
            foreach ($osoba->ziskaneDovednosti[$this->dovednost->kod]->specifikaceUrovne as $kodUrovne => $urovenDovednosti) {
                $result .= $urovenDovednosti == 1 ? " samokontrola_$kodUrovne" : '';
            }
        }
        $result .= '" title="' . ($this->zobrazeniUrovneDovednosti == 0
                ? (isset($osoba->ziskaneDovednosti[$this->dovednost->kod]->uroven) && $this->dovednost ->htmlPopisUrovni[$osoba->ziskaneDovednosti[$this->dovednost->kod]->uroven] ?: '')
                : ($osoba->urovenZiskaneDovednosti($this->dovednost) ?: '')) . '">';

        if ($osoba->lzeEditovat || $osoba->lzeZobrazit) {
            if ($osoba->lzeEditovat) {
                $dovednostAction = 'novadovednost';
            } else {
                $dovednostAction = 'dovednostiosoby';
            }

            $result .= '<a href="?menuid=maticedovednosti&amp;action=' . $dovednostAction . '&amp;osoba=' . $osoba->idOsoby . '&amp;dovednost=' . $this->dovednost->kod . '">';
        }
        $result .= ($this->zobrazeniUrovneDovednosti == 0 ? $osoba->urovenZiskaneDovednosti($this->dovednost) : $this->dovednost ->htmlPopisUrovni[$osoba->ziskaneDovednosti[$this->dovednost->kod]->uroven]);
        if ($osoba->lzeEditovat || $osoba->lzeZobrazit) {
            $result .= '</a>';
        }

        $result .= '</div>';

        if ($this->zobrazitPlatnost) {
            if (isset($osoba->ziskaneDovednosti[$this->dovednost->kod]->platnostDo)) {
                $result .= '<div title="' . StringUtils::format('date', $osoba->ziskaneDovednosti[$this->dovednost->kod]->platnostDo) . '">'
                    . date('n/y', $osoba->ziskaneDovednosti[$this->dovednost->kod]->platnostDo) . '</div>';
            }
        }

        return $result;
    }

    /** Vypise bunku s pozadovanou urovni dovednosti, kde hrany bunky jsou vybarveny dle pozadovane (cervene)/dosazene (modre) urovne 1 - x (Od urovne 4 jsou vybarveny 4 strany ctverce).
     * V pripade, ze osoba ma pozadovanou nebo vyssi uroven je bunka podbarvena zelene, jinak cervene.
     * @param OsobaSDovednosti $osoba
     * @return string
     */
    protected function getPozadovanaDovednost($osoba)
    {
        if ($osoba->urovenPozadovaneDovednosti($this->dovednost, true) == 0) {
            return '';
        }

        $result = '<div class="' .
            (($osoba->urovenZiskaneDovednosti($this->dovednost, true) > 0) || ($osoba->urovenPozadovaneDovednosti($this->dovednost, true) > 0)
                ? 'dovednosti_dovednost_osoby ' : '') .
            'pozadovana_'.$osoba->urovenPozadovaneDovednosti($this->dovednost).' ziskana_'.$osoba->urovenZiskaneDovednosti($this->dovednost) .
            ($osoba->urovenZiskaneDovednosti($this->dovednost, true) > 4 ? ' ziskana_4' : '') .
            (($osoba->urovenZiskaneDovednosti($this->dovednost, true) >= $osoba->urovenPozadovaneDovednosti($this->dovednost, true)) ? ' dovednosti_dovednost_splnena' : ' dovednosti_dovednost_nesplnena');
        if (isset($osoba->ziskaneDovednosti[$this->dovednost->kod])) {
            foreach ($osoba->ziskaneDovednosti[$this->dovednost->kod]->specifikaceUrovne as $kodUrovne => $urovenDovednosti) {
                $result .= $urovenDovednosti == 1 ? " samokontrola_$kodUrovne" : '';
            }
        }

        $result .= '" title="' . ($this->dovednost->htmlPopisUrovni[$osoba->pozadovaneDovednosti[$this->dovednost->kod]->uroven] ?: '') . '">';


        //$result = $osoba->urovenZiskaneDovednosti($dovednost->kod);
        if ($osoba->urovenPozadovaneDovednosti($this->dovednost, true))
        {
            if ($osoba->lzeEditovat || $osoba->lzeZobrazit)
            {
                if ($osoba->lzeEditovat) {
                    $dovednostAction = 'novadovednost';
                } else {
                    $dovednostAction = 'dovednostiosoby';
                }

                $result .= '<a href="?menuid=maticedovednosti&amp;action=' . $dovednostAction . '&amp;osoba=' . $osoba->idOsoby . '&amp;dovednost=' . $this->dovednost->kod . '">';
            }
            $result .= $osoba->urovenZiskaneDovednosti($this->dovednost, true) . '/' . $osoba->urovenPozadovaneDovednosti($this->dovednost, true);
            if ($osoba->lzeEditovat || $osoba->lzeZobrazit)
            {
                $result .= '</a>';
            }
        }
        $result .= '</div>';

        return $result;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $osoba = $this->getDataOsoby($rowData['col-rc']);

        // Vypise ziskanou/pozadovanou uroven dovednosti dle aktualniho browseru.
        if ($this->ziskaneDovednosti) {
            return $this->getZiskanaDovednost($osoba);
        } else {
            return $this->getPozadovanaDovednost($osoba);
        }
    }


//    /** Vypise ziskanou/pozadovanou uroven dovednosti dle aktualniho browseru.
//     * @param $value
//     * @param $rowNumber
//     * @param $recordId
//     * @param $rowData
//     * @return string
//     */
//    function getFieldCellText($value, $rowNumber, $recordId, $rowData)
//    {
//        $osoba = $this->getDataOsoby($rowData['col-rc']);
//
//        if ($this->ziskaneDovednosti) {
//            return $this->getZiskanaDovednost($osoba);
//        } else {
//            return $this->getPozadovanaDovednost($osoba);
//        }
//    }
}
/** Sloupec pro zobrazeni sloupce databaze ze vsech aktivnich pracovnich pomeru osoby. */
class PracovniPomerBrowserField extends BrowserField
{
    protected $pracPomerData;

    /** Sloupec pro zobrazeni sloupce databaze ze vsech aktivnich pracovnich pomeru osoby.
     * @param $type
     * @param Database $database
     */
    public function __construct($type = BrowserField::FIELD_TYPE_DEFAULT, $database = null)
    {
        parent::__construct($database);

        $this->setType($type);
    }

    /**
     * @param array $pracPomerData
     * @return void
     */
    public function setData($pracPomerData)
    {
        $this->pracPomerData = $pracPomerData;
    }

    /** Pro vsechny pracovni pomery osoby zobrazi udaje z pozadovaneho sloupce.
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        foreach ($this->pracPomerData[$rowData['col-rc']] as $rowData) {
            $value = $rowData[$this->code];
            $result .= ($result ? ', ' : '') .
                parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }
        return $result;
    }
}

/** Sloupec pro zobrazeni kodu/popisu kalendare ze vsech aktivnich pracovnich pomeru osoby. */
class KalendarBrowserField extends DovednostBrowserField
{
    /**  @param int $typZobrazeni 1 = kod, 2 = popis */
    private $typZobrazeni;

    /**
     * @param int $typZobrazeni 1 = kod, 2 = popis
     */
    public function __construct($typZobrazeni)
    {
        parent::__construct();

        $this->typZobrazeni = $typZobrazeni;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $kalendare = $this->getDataOsoby($rowData['col-rc'])->kalendar;

        $result = '';
        if (is_array($kalendare)) {
            foreach ($kalendare as $kalendar) {
                if ($this->typZobrazeni == 1) {
                    $result .= '<div title="' . StringUtils::htmlTextConvert($kalendar->popis) . '">' . $kalendar->kod . '</div>';
                } elseif ($this->typZobrazeni == 2) {
                    $result .= StringUtils::htmlTextConvert($kalendar->popis);
                }
            }
        }

        return $result;
    }


//    /**
//     * @param $value
//     * @param $rowNumber
//     * @param $recordId
//     * @param $rowData
//     * @return string
//     */
//    function getFieldCellText($value, $rowNumber, $recordId, $rowData)
//    {
//        $result = '<td class="dovednosti_kalendar">';
//
//        $kalendare = $this->getDataOsoby($rowData['col-rc'])->kalendar;
//
//        if (is_array($kalendare)) {
//            foreach ($kalendare as $kalendar) {
//                if ($this->typZobrazeni == 1) {
//                    $result .= '<div title="' . StringUtils::htmlTextConvert($kalendar->popis) . '">' . $kalendar->kod . '</div>';
//                } elseif ($this->typZobrazeni == 2) {
//                    $result .= StringUtils::htmlTextConvert($kalendar->popis);
//                }
//            }
//        }
//
//        return $result . '</td>';
//    }
}

/** Sloupec pro zobrazeni kodu/popisu prirazeneho rozdeleni ze vsech aktivnich pracovnich pomeru osoby. */
class RozdeleniBrowserField extends DovednostBrowserField
{
    public $typRozdeleni;
    /** @var int|mixed 1 = kody, 2 = popisy rozdeleni */
    private $typZobrazeni;

    /**
     * @param int $typRozdeleni
     * @param int|null $typZobrazeni 1 = kody, 2 = popisy rozdeleni
     */
    public function __construct($typRozdeleni, $typZobrazeni = null)
    {
        parent::__construct();

        $this->typRozdeleni = $typRozdeleni;

        if (isset($typZobrazeni)) {
            $this->typZobrazeni = $typZobrazeni;
        } else {
            $this->typZobrazeni = 1;
        }
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $rozdeleniOsoby = $this->getDataOsoby($rowData['col-rc'])->prirazenaRozdeleniOsoby;

        if (!isset($rozdeleniOsoby[$this->typRozdeleni])) {
            $result .= '';
        } else {
            $seznamRozdeleni = '';
            $seznamPopisuRozdeleni = '';
            foreach ($rozdeleniOsoby[$this->typRozdeleni] as $rozdeleni) {
                if ($seznamRozdeleni) {
                    $seznamRozdeleni .= ', ';
                }
                if ($seznamPopisuRozdeleni) {
                    $seznamPopisuRozdeleni .= ', ';
                }
                $seznamPopisuRozdeleni .= StringUtils::htmlTextConvert($rozdeleni->popis);
                $seznamRozdeleni .= '<div style="display:inline;" title="' . StringUtils::htmlTextConvert($rozdeleni->popis) . '">' . StringUtils::htmlTextConvert($rozdeleni->kod) . '</div>';
            }
            if ($this->typZobrazeni == 1) {
                $result .= $seznamRozdeleni;
            } elseif ($this->typZobrazeni == 2) {
                $result .= $seznamPopisuRozdeleni;
            }
        }

        return $result;
    }


//    /**
//     * @param $value
//     * @param $rowNumber
//     * @param $recordId
//     * @param $rowData
//     * @return string
//     */
//    function getFieldCellText($value, $rowNumber, $recordId, $rowData)
//    {
//        $class =  $this->getFieldClass();
//        $result = "<td class='$class''>";
//
//        $rozdeleniOsoby = $this->getDataOsoby($rowData['col-rc'])->prirazenaRozdeleniOsoby;
//
//        if (!isset($rozdeleniOsoby[$this->typRozdeleni])) {
//            $result .= '';
//        } else {
//            $seznamRozdeleni = '';
//            $seznamPopisuRozdeleni = '';
//            foreach ($rozdeleniOsoby[$this->typRozdeleni] as $rozdeleni) {
//                if ($seznamRozdeleni) {
//                    $seznamRozdeleni .= ', ';
//                }
//                if ($seznamPopisuRozdeleni) {
//                    $seznamPopisuRozdeleni .= ', ';
//                }
//                $seznamPopisuRozdeleni .= StringUtils::htmlTextConvert($rozdeleni->popis);
//                $seznamRozdeleni .= '<div style="display:inline;" title="' . StringUtils::htmlTextConvert($rozdeleni->popis) . '">' . StringUtils::htmlTextConvert($rozdeleni->kod) . '</div>';
//            }
//            if ($this->typZobrazeni == 1) {
//                $result .= $seznamRozdeleni;
//            } elseif ($this->typZobrazeni == 2) {
//                $result .= $seznamPopisuRozdeleni;
//            }
//        }
//
//        return $result . '</td>';
//    }
}

/** Sloupec pro zobrazeni poctu ziskanych dovednosti, nebo poctu ziskanych z poctu pozadovanych dovednosti. */
class StavDovednostiBrowserField extends DovednostBrowserField
{
    protected $zobrazitPocetZiskanych;
    protected $ziskaneDovednosti;

    /**
     * @param bool $zobrazitPocetZiskanych
     * @param bool $ziskaneDovednosti
     */
    public function __construct($zobrazitPocetZiskanych, $ziskaneDovednosti = true)
    {
        parent::__construct();

        $this->zobrazitPocetZiskanych = $zobrazitPocetZiskanych;
        $this->ziskaneDovednosti = $ziskaneDovednosti;
    }

    /** Dle prehledu zobrazi pocet ziskanych pripadne i pocet pozadovanych dovednosti.
     * @param $value
     * @param $rowNumber
     * @param $recordId
     * @param $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $osoba = $this->getDataOsoby($rowData['col-rc']);

        if ($this->ziskaneDovednosti) {
            if ($this->zobrazitPocetZiskanych) {
                $result .= count($osoba->ziskaneDovednosti) . '/';
            }
            $result .= $osoba->pocetZiskanychDovednosti;
        } else {
            $result .= $osoba->pocetZiskanychDovednosti .  '/' . $osoba->pocetPozadovanychDovednosti;
        }

        return $result;
    }

}

/**
 * Nacita do pameti (a nasledne zobrazi) matici dovednosti zamestnancu
 *
 */
class SeznamDovednostiOsobBrowser extends SeznamOsobBrowser
{
    const CSV_SEPARATOR = ';';

    /** @var OsobaSDovednosti[] */
    public $osoby;

    protected $udajePP;

    public $seznamIDPP;
    public $rcPracPomeru;

    /** @var DovednostOsoby[] */
    public $seznamDovednosti;

    /** @var PravaNaOsoby */
    public $pravaEditace;
    /** @var PravaNaOsoby */
    public $pravaZobrazeni;
    /** @var PravaNaOsoby[] */
    public $pravaZobrazeniOsoby;

    /** @var TypRozdeleni */
    public $typRozdeleniFiltr;
    /** @var TypRozdeleni  */
    public $typRozdeleniPracMisto;

    protected $zobrazitFoto;
    protected $zobrazitTypPP;
    protected $zobrazitPracovniMisto;
    protected $zobrazitRozdeleni;
    protected $zobrazitKalendar;
    protected $zobrazitPritomnost;
    protected $zobrazitPocetDosazenychDovednosti;
    /** @var int typ zobrazeni urovne: 0 = kod, 1 = popis */
    protected $zobrazeniUrovneDovednosti;

    /** @var string urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti */
    protected $typZobrazeni;

    /** @var PracovniPomerBrowserField[] */
    protected $fieldsPP = [];

    protected $typyRozdeleni = [];

    /** @var SqlStatement */
    protected $sqlFilterDovednosti;

    /** @var bool nastaveni zda radit dle typu a dovednosti, nebo jen dle dovednosti */
    protected $raditDleTypu;

    /**
     *
     * @param Database $database
     * @param string $typZobrazeni urcuje zda se jedna o 'pozadovane', nebo 'ziskane' dovednosti
     * @throws Exception
     * @noinspection PhpMissingParentConstructorInspection
     */
    function __construct($database, $typZobrazeni)
    {
        $this->database = $database;
        $this->action = 'maticedovednosti';
        $this->maxPageRows = 50;
        $this ->cssClass = 'browser rowOver pritomnost-container fixedCols';
        $this ->showMessages = true;

        $this->pouzitPracovniPomer = false;

        $this ->osoby = [];
        $this ->seznamDovednosti = [];
        $this ->typZobrazeni = $typZobrazeni;
        $this->seznamIDPP = [];
        $this->rcPracPomeru = [];

        Prava::nastavObdobiProPrava(time(), time());

        $this ->pravaEditace = Prava::vratPrava('doved_edit');
        $this ->pravaZobrazeni = Prava::vratPrava('doved_ziskane');

        $prava = Prava::vratPrava('prohl_osobni');
        $pravadoch = Prava::vratPrava('prohlizeni'); // osoby u kterych muze zobrazit zakladni udaje

        $this ->pravaZobrazeniOsoby = [$prava, $pravadoch];

        $this->sqlFromPart = "Osoba";

        $this->typRozdeleniFiltr = TypRozdeleni::getTypRozdeleni($this->database,
            (RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_filtr1') ?: RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP)));
        //pokud je v nastavení jiný typ rozdělení použije se místo pracovního místa
        $this->typRozdeleniPracMisto = TypRozdeleni::getTypRozdeleni($this->database, (RonPortalSettings::getIntValue('dovednosti_typ_rozdeleni_tpm') ?: -1));

        $this->code = 'browsematicedovednosti';
        $this->tableId = 'maticeDovednosti';

        $this->browserUrl = '?menuid=maticedovednosti&action=ziskane&';

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)->setSqlFieldName('Osoba.RC')->setCode('col-rc');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)->setCode('col-idpp')
            ->setSqlFieldName('(SELECT MAX(PracovniPomer.ID) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC AND (PracovniPomer.DatumUkonceni >= {fn_GETDATE()} OR PracovniPomer.DatumUkonceni IS NULL))');


        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)->setSqlFieldName('Osoba.Jmeno')->setCode('col-jmeno');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)->setSqlFieldName('Osoba.Prijmeni')->setCode('col-prijmeni');

        $this->addField(new PracovniPomerBrowserField(BrowserField::FIELD_TYPE_HIDDEN))->setSqlFieldName('PracovniPomer.ID')->setCode('col-pp-id');
        $this->addField(new PracovniPomerBrowserField(BrowserField::FIELD_TYPE_HIDDEN))->setSqlFieldName('PracovniPomer.RC')->setCode('col-pp-rc');

        $this->filterForm = new DovednostiFilterForm($this->database,
            $this->typZobrazeni,
            $this->typRozdeleniFiltr->kod, $this->typRozdeleniPracMisto->kod, $this->zobrazitKalendar);

        $str = RonPortalSettings::getBvarSettings($this ->database, 'RonPortal_dovednosti_sloupce');
        if ($str) {
            parent::loadConfigurationXml($str);
        } else {
            $this->loadDefaultFields();
        }

        $this->addField(new StavDovednostiBrowserField($this->zobrazitPocetDosazenychDovednosti, $this->typZobrazeni == 'ziskane'))
            ->setHeaderText('Dovednosti_stav')
            ->setCode('col-stav')
            ->setClassName('dovednosti_stav');
    }

    /** Prida sloupec z konfigurace dle zvoleneho typusloupce.
     * @param $colSettings
     * @return int|null
     * @throws DBException
     */
    function createNewField($colSettings)
    {
        $newField = null;
        $zobrazitKody = null;

        switch ($colSettings['typsloupce']) {
            case 'prijmeni' :
                $newField = $this ->addField()
                    ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni')
                    ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno')
                    ->setCode('col-prijmeni');
                $this->defaultSort = 'col-prijmeni';
                break;

//            case 'jmeno' :
//                $newField = $this->addField()
//                    ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno')
//                    ->setSortSQL('Osoba.Jmeno, Osoba.Prijmeni')
//                    ->setCode('col-jmeno');
//                break;

//            case 'titul' :
//                $newField = $this->addField()
//                    ->setSqlFieldName('Osoba.Titul') ->setHeaderText('Titul')
//                    ->setSortSQL('Osoba.Titul, Osoba.Prijmeni, Osoba.Jmeno')
//                    ->setCode('col-titul');
//                break;

//            case 'titulza' :
//                $newField = $this->addField()
//                    ->setSqlFieldName('Osoba.TitulZa') ->setHeaderText('Titul_za')
//                    ->setSortSQL('Osoba.TitulZa, Osoba.Prijmeni, Osoba.Jmeno')
//                    ->setCode('col-titulza');
//                break;

            case 'rodneprijmeni' :
            case 'datumnarozeni' :
            case 'rczahranicni' :
                // udaje z OsobniUdaj nejsou dostupne
                break;

//            case 'email' :
//                $newField = $this->addField(BrowserField::FIELD_TYPE_EMAIL)
//                    ->setSqlFieldName('Osoba.eMail') ->setHeaderText('e_mail') ->setOrdering(true)
//                    ->setCode('col-email');
//                break;

//            case 'telefon' :
//                $newField = $this->addField()
//                    ->setSqlFieldName('Osoba.Telefon') ->setHeaderText('Telefon') ->setOrdering(true)
//                    ->setCode('col-telefon');
//                break;

//            case 'mobil' :
//                $newField = $this->addField()
//                    ->setSqlFieldName('Osoba.Mobil') ->setHeaderText('Mobil') ->setOrdering(true)
//                    ->setCode('col-mobil');
//                break;

//            case 'osobauzivatelska' :
//                $index = (int)$colSettings['index'];
//                $newField = $this->addField()
//                    ->setSqlFieldName("Osoba.uzivatelska$index") ->setHeaderText("osoba_uzivatelska_{$index}") ->setOrdering(true)
//                    ->setCode("col-uzivatelska$index");
//                break;

            case 'ppomeruzivatelska' :
                $index = (int)$colSettings['index'];
                $newField =
                    $this->addField(new PracovniPomerBrowserField())
                        ->setSqlFieldName("PracovniPomer.uzivatelska$index")
                        ->setHeaderText("ppomer_uzivatelska_{$index}")
                        ->setCode("col-pp-uzivatelska$index");
                break;

            case 'idosoby' :
                $newField = $this->addField()
                    ->setSqlFieldName('Osoba.RC', 0) ->setHeaderText('Id_osoby') ->setOrdering(true)
                    ->setCode('col-rc');
                break;

            case 'foto' :
                $newField = $this->addField(new FotoOsobyBrowserField(self::INDEX_ID_PP, '?menuid=dovednosti&action=foto&osoba=', (int)$colSettings['zobrazitfoto'] == 1));
                $newField
                    ->setHeaderText('Foto')
                    ->setSqlFieldName('Osoba.idFotografie')
                    ->setCode('col-foto')
                    ->setClassName('dovednosti_foto');
                break;

            case 'osobnicislo' :
                $newField = $this->addField(new PracovniPomerBrowserField())
                    ->setSqlFieldName('PracovniPomer.OsobniCislo')->setHeaderText('Osobni_cislo')
                    ->setCode('col-pp-osobnicislo')->setClassName('dovednosti_osobnicislo');
                break;

            case 'datumnastupu' :
                $newField = $this->addField(new PracovniPomerBrowserField(BrowserField::FIELD_TYPE_DEFAULT, $this->database))
                    ->setSqlFieldName('PracovniPomer.DatumNastupu')->setHeaderText('Datum_nastupu')
                    ->setCode('col-pp-datumnastupu')->setFormat('date') ->setAlign('center');
                break;

            case 'datumukonceni' :
                $newField = $this->addField(new PracovniPomerBrowserField(BrowserField::FIELD_TYPE_DEFAULT, $this->database))
                    ->setSqlFieldName('PracovniPomer.DatumUkonceni')->setHeaderText('Datum_ukonceni')
                    ->setCode('col-pp-datumukonceni')->setFormat('date') ->setAlign('center');
                break;

            case 'zkusebnidobadovypoctena' :
                $newField = $this->addField(new PracovniPomerBrowserField(BrowserField::FIELD_TYPE_DEFAULT, $this->database))
                    ->setSqlFieldName('PracovniPomer.ZkusebniDobaDoVypoctena')->setHeaderText('Zkusebni_doba_do_vypoctena')
                    ->setCode('col-pp-zkusebnidobadovypoctena')->setFormat('date') ->setAlign('center');
                break;

            case 'typpp' :
                $newField = $this->addField(new PracovniPomerBrowserField())
                    ->setSqlFieldName('PracovniPomer.TypPracovnihoPomeru')
                    ->setHeaderText('Typ_PP')
                    ->setCode('col-pp-typ')->setClassName('dovednosti_typ_pp');
                break;

            case 'kodkalendare' :
                $newField = $this->addField(new KalendarBrowserField(1))
                    ->setCode('col-kalendar-kod')
                    ->setHeaderText('Kalendar')
                    ->setClassName('dovednosti_kalendar');
                break;
            case 'popiskalendare' :
                $newField = $this->addField(new KalendarBrowserField(2))
                    ->setCode('col-kalendar-popis')
                    ->setHeaderText('Kalendar')
                    ->setClassName('dovednosti_kalendar');
                break;

            case 'pritomnost' :
                $newField = $this->addField(new PritomnostBrowserField('col-idpp'))
                        ->setCode('col-pritomnost')->setHeaderText('Pritomnost')
                        ->setClassName('dovednosti_pritomnost pritomnost');
                break;

            case 'kodrozdeleni' :
                $zobrazitKody = true;
            case 'popisrozdeleni' :
                $typRozdeleni = (int)$colSettings['index'];
                $newField = $this->addField(new RozdeleniBrowserField($typRozdeleni, $zobrazitKody ? 1 : 2));

                if (!isset($colSettings['header'])) {
                    $colSettings['header'] = StringUtils::htmlTextConvert(DBUtils::getSqlValue($this->database, 'SELECT Popis FROM TypRozdeleni WHERE Kod = ' . (int)$typRozdeleni));
                }

                if (!isset($colSettings['code'])) {
                    $colSettings['code'] = "col-rozdeleni{$typRozdeleni}-kod";
                }

                break;

            default :
                return parent::createNewField($colSettings);
        }

        if ($newField) {
            $newField ->loadConfigurationArray($colSettings);
            if ($newField->ordering  &&  $newField instanceof PracovniPomerBrowserField) {
                $newField->setOrdering(false);
            }
        }

        return $this->fields ? count($this->fields) - 1 : -1;
    }

    /** Nacte konfiguraci. U tagu config lze atributem raditdletypu zapnout seskupeni ziskanych dovednosti dle typu a atributem filtrkalendar zobrazeni filtru na kalendar zamestnance.
     * @param $key
     * @param $value
     * @param $valueText
     * @return void
     */
    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'raditdletypu') {
            $this->raditDleTypu = (int)$value;
        } elseif ($key === 'filtrkalendar') {
            $this->zobrazitKalendar = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }

    /** Nastavi vychozi sloupce prehledu (Pritomnost, Zamestnanec, foto, Typ PP, Rozdeleni 1, Rozdeleni 2, kalendar).
     * Zda sloupce zobrazit nacita z puvodniho nastaveni (nelze jiz menit), nove se konfiguruje pomoci XML.
     * @return void
     */
    public function loadDefaultFields()
    {
        $this->zobrazitFoto = RonPortalSettings::getIntValue('dovednosti_sloupce_foto', 1);
        $this->zobrazitTypPP = RonPortalSettings::getIntValue('dovednosti_sloupce_typ_pp', 1);
        $this->zobrazitPracovniMisto = RonPortalSettings::getIntValue('dovednosti_sloupce_prac_misto', 1);
        $this->zobrazitRozdeleni = RonPortalSettings::getIntValue('dovednosti_sloupce_rozdeleni', 1);
        $this->zobrazitKalendar = RonPortalSettings::getIntValue('dovednosti_sloupce_kalendar', 1);
        $this->zobrazitPritomnost = RonPortalSettings::getIntValue('dovednosti_sloupce_pritomnost', 0);
        $this->zobrazitPocetDosazenychDovednosti = false;
        $this->zobrazeniUrovneDovednosti = RonPortalSettings::getIntValue('dovednosti_typ_zobrazeni_urovne', 0);

        if ($this->zobrazitPritomnost) {
            $this->addField(new PritomnostBrowserField('col-idpp'))
                ->setCode('col-pritomnost')->setHeaderText('Pritomnost')
                ->setClassName('dovednosti_pritomnost pritomnost');
        }

        $this->addField(new JmenoPrijmeniBrowserField($this->database, 'col-jmeno', 'col-prijmeni'))
            ->setOrdering(true)->setCode('col-osoba')
            ->setHeaderText('Zamestnanec')->setClassName('dovednosti_zamestnanec');

        $this->addField(new PracovniPomerBrowserField())
            ->setSqlFieldName('PracovniPomer.OsobniCislo')->setHeaderText('Osobni_cislo')
            ->setCode('col-pp-osobnicislo')->setClassName('dovednosti_osobnicislo');

        if ($this->zobrazitFoto) {
            $this->addField(new FotoOsobyBrowserField(1, '?menuid=dovednosti&action=foto&osoba=', $this->zobrazitFoto == 2))
                ->setHeaderText('Foto')->setCode('col-foto')
                ->setClassName('dovednosti_foto')->setSqlFieldName('Osoba.idFotografie');
        }

        if ($this->zobrazitTypPP) {
            $this->addField(new PracovniPomerBrowserField())
                ->setSqlFieldName('PracovniPomer.TypPracovnihoPomeru')
                ->setHeaderText('Typ_PP')
                ->setCode('col-pp-typ')->setClassName('dovednosti_typ_pp');
        }
        if ($this->zobrazitPracovniMisto) {
            $this->addField(new RozdeleniBrowserField($this->typRozdeleniPracMisto->kod, $this->zobrazitPracovniMisto))
                ->setCode('col-rozdeleni1')
                ->setHeaderText($this->typRozdeleniPracMisto->popis)
                ->setClassName('dovednosti_pracovni_mista');
        }
        if ($this->zobrazitRozdeleni) {
            $this->addField(new RozdeleniBrowserField($this->typRozdeleniFiltr->kod, $this->zobrazitRozdeleni))
                ->setCode('col-rozdeleni2')
                ->setHeaderText($this->typRozdeleniFiltr->popis)
                ->setClassName('dovednosti_rozdeleni');
        }
        if ($this->zobrazitKalendar) {
            $this->addField(new KalendarBrowserField($this->zobrazitKalendar))
                ->setCode('col-kalendar')
                ->setHeaderText('Kalendar')
                ->setClassName('dovednosti_kalendar');
        }

        $this->defaultSort = 'col-osoba';
    }


    /**
     * Doplni do SQL dotazu filtr na rozdeleni, PM podle zvoleneho typu zobrazeni.
     */
    function getSQLFilter()
    {
        if (!$this->pravaZobrazeni->jsouNejakePrava()) {
            return '1 = 0';
        }

        $params = $this ->database ->createParameters();

        $result = $this ->filterForm ->buildSqlFilter($params);

        $datumOd = time();
        $datumDo = time();

        if (isset($this->sqlFilterDovednosti)) {
            $params->appendParameters($this->sqlFilterDovednosti->parameters);

            $filterDovednosti = '';

            if ($this->filterForm->getFilterValue('typdovednosti')) {
                $filterDovednosti = " Dovednost.TypDovednosti = " . $params->addParameter($this->filterForm->getFilterValue('typdovednosti'));
            }

            if ($this->filterForm->getFilterValue('dovednost')) {
                $filterDovednosti .=
                    ($filterDovednosti ? ' AND ' : '') .
                    " Dovednost.Kod = " . $params->addParameter($this->filterForm->getFilterValue('dovednost'));
            }

            if (($this->typZobrazeni == 'pozadovane') && ($this ->filterForm ->getFilterValue('rozdeleni2') !== null)) {
                $filterDovednosti .=
                    ($filterDovednosti ? ' AND ' : '') .
                    "EXISTS(SELECT H.kodPodrizene FROM HierarchieRozdeleni H" .
                    " WHERE H.typPodrizene = PozadavekNaDovednost.KodTypuRozdeleni" .
                    "  AND H.kodPodrizene = PozadavekNaDovednost.KodRozdeleni" .
                    "  AND H.typNadrazene = " . $params->addParameter($this ->filterForm ->getFilterValue('rozdeleni2', 'typRozdeleni')) .
                    "  AND H.kodNadrazene = " . $params->addParameter($this ->filterForm ->getFilterValue('rozdeleni2')) .
                    ($this ->filterForm ->getFilterValue('rozdeleni2', 'vcetnePodrizenych')
                        ? ''
                        : ' AND H.kodNadrazene = H.kodPodrizene') .
                    ")";
            }

            if (empty($filterDovednosti)) {
                $filterDovednosti = '1 = 1';
            }

            $result .=
                ($result ? ' AND ' : '') .
                str_replace('%FILTER%', $filterDovednosti,$this->sqlFilterDovednosti->sqlText);
        }

        $result .=
            ($result ? ' AND ' : '') .
            'EXISTS (SELECT P.RC FROM PracovniPomer P' .
            ' WHERE (P.RC = Osoba.RC)' .
            '   AND (' . $this->pravaZobrazeni->sqlFiltr('P', time(), time(), $params) . ')' .
            "   AND (P.DatumNastupu <= " . $params ->addDateParameter($datumDo) . ")" .
            "   AND ((P.DatumUkonceni >= " . $params->addDateParameter($datumOd) . ") OR (P.DatumUkonceni IS NULL))" .
            "\n";

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $result .=
                    ($result ? '   AND ' : '') .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database,
                        'P', $rozdeleniFilter->getTypRozdeleni(), $rozdeleniFilter->getValue(),
                        time(), time(),
                        $rozdeleniFilter->getValue('vcetnePodrizenych'),
                        $params) .
                    "\n";
            }
        }

        $result .= ")\n";

//            (($this ->filterForm ->kodTypuRozdeleni && isset($this ->filterForm ->kodRozdeleni))
//                ? ' AND ' .
//                RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                    'P', $this->filterForm->kodTypuRozdeleni, $this->filterForm->kodRozdeleni,
//                    time(), time(),
//                    $this->filterForm->vcetnePodrizenychRozdeleni,
//                    $params)
//                : '') .
//            (($this ->filterForm ->kodTypuRozdeleni2 && isset($this ->filterForm->kodRozdeleni2))
//                ? ' AND ' .
//                RozdeleniUtils::sqlFilterRozdeleniPP($this ->database,
//                    'P', $this->filterForm->kodTypuRozdeleni2, $this->filterForm->kodRozdeleni2,
//                    time(), time(),
//                    $this->filterForm->vcetnePodrizenychRozdeleni2,
//                    $params)
//                : '') .
//            ")\n";

        if ($this->filterForm->getFilterValue('kalendar')) {
            $result .=
                ($result ? ' AND ' : '') .
                "EXISTS (" .
                "SELECT PK.RC FROM PrirazeniKalendare PK JOIN PracovniPomer PP ON (PP.RC = PK.RC AND PP.OsobniCislo = PK.OsobniCislo AND PP.DatumNastupu = PK.DatumNastupu)".
                " WHERE PK.RC = Osoba.RC " .
                "  AND PP.DatumNastupu <= " . $params->addDateParameter(time()) .
                "  AND (PP.DatumUkonceni >= " . $params->addDateParameter(time()) . " OR PP.DatumUkonceni IS NULL) " .
                "  AND PK.Kalendar = " . $params->addParameter($this->filterForm->getFilterValue('kalendar')) .
                "  AND PK.PlatnostOd <= " . $params->addDateParameter(time()) .
                "  AND (PK.PlatnostDo >= " . $params->addDateParameter(time()) . " OR PK.PlatnostDo IS NULL)" .
                ")\n";
        }

        if ($this->filterForm->getFilterValue('urovendovednosti')) {
            $result .=
                ($result ? ' AND ' : '') .
                "EXISTS (SELECT RC FROM DovednostOsoby DO 
                             WHERE DO.KodUrovne = " . $params->addParameter($this->filterForm->getFilterValue('urovendovednosti')) .
                "            AND DO.RC = Osoba.RC " .
                ($this->filterForm->getFilterValue('dovednost') ? " AND DO.KodDovednosti = " . $params->addParameter($this->filterForm->getFilterValue('dovednost')) : '') .
                "            AND DO.DatumZiskani <= " . $params->addDateParameter(time()) .
                "            AND (DO.PlatnostDo >= " . $params->addDateParameter(time()) . " OR DO.PlatnostDo IS NULL)" .
                ")\n";
        }

        return $result
            ? new SqlStatement($result, $params)
            : '';

    }

    /** Prida sloupec do seznamu poli. U sloupce pracovniho pomeru pak navic i do seznamu poli PP a u sloupce rozdeleni pak prida typ rozdeleni do seznamu typu pro nacteni z DB.
     * @param $field
     * @return array|BrowserField|PracovniPomerBrowserField|RozdeleniBrowserField|string|null
     */
    public function addField($field = null)
    {
        if ($field instanceof PracovniPomerBrowserField) {
            $this->fieldsPP[] = $field;
            $field->sqlFieldIndex = 999;
            $this->fields[] = $field;
            return $field;
        }
        if ($field instanceof RozdeleniBrowserField) {
            $field->sqlFieldIndex = 999;
            $this->fields[] = $field;
            if (!in_array($field->typRozdeleni, $this->typyRozdeleni)) {
                $this->typyRozdeleni[] = $field->typRozdeleni;
            }
            return $field;
        }
        return parent::addField($field);
    }

    /** Vrati seznam poli pro SQL dotaz k nacteni udaju pracovnich pomeru osob.
     * @return string
     */
    private function getPPSelectSQLPart()
    {
        $sqlFieldIndex = 0;

        $result = '';

        foreach ($this ->fieldsPP as $i => $field) {
            if ($field ->sqlFieldName  &&  $field ->visible  &&  (!isset($field ->sqlFieldIndex) || ($field->sqlFieldIndex == 999))) {
                // prida nazev pole do SQL dotazu
                $result .=
                    ($result ? ', ' : '') .
                    ($field ->castWideMemo ? $this ->database ->castWideMemo($field ->sqlFieldName) : $field ->sqlFieldName);


                // seznam nema nastaven select - muzu cislovat normalne od nuly
                $this ->fieldsPP[$i] ->sqlFieldIndex = $sqlFieldIndex;

                $sqlFieldIndex++;
            }
        }

        return $result;
    }

    /** Nacte udaje aktivnich pracovnich pomeru nactenych osob. Dale nacte prirazena rozdeleni a kalendare.
     * @return void
     * @throws DBException
     * @throws DebugException
     */
    protected function loadPP()
    {
        $seznamRC = [];
        while ($row = $this->cursor->fetch()) {
            $seznamRC[] = $row[0];
            $osoba = new OsobaSDovednosti($row[0], $row[2], $row[3]);
            $osoba->idOsoby = $row[1];
            $osoba->lzeEditovat = $this->pravaEditace->lzeZobrazitPP($osoba->idOsoby);
            $osoba->lzeZobrazit = $this->pravaZobrazeni->lzeZobrazitPP($osoba->idOsoby);
            $osoba->lzeZobrazitOsobniUdaje = Prava::lzeZobrazitPP($this->pravaZobrazeniOsoby, $osoba->idOsoby);

            $this->osoby[$row[0]] = $osoba;
        }

        if (!empty($seznamRC)) {
            $params = $this->database->createParameters();

            $sql =
                "SELECT " . $this->getPPSelectSQLPart() .
                " FROM PracovniPomer " .
                " WHERE " . SQLUtils::getINCondition($this->database, 'RC', $seznamRC, $params) .
                "   AND PracovniPomer.DatumNastupu <= " . $params->addDateParameter(time()) .
                "   AND (PracovniPomer.DatumUkonceni >= " . $params->addDateParameter(time()) . " OR PracovniPomer.DatumUkonceni IS NULL) " .
                "ORDER BY RC, DatumNastupu";

            $cursor = $this->database->select($sql, $params);

            $this->udajePP = [];
            while ($row = $cursor->fetch()) {
                $rowArr = new BrowserRowArray($this->fieldsPP);
                $rowArr->setData($row);

                $this->udajePP[$rowArr['col-pp-rc']][] = $rowArr;
                $this->osoby[$rowArr['col-pp-rc']]->nactiUdajePP($rowArr);

                if (!in_array($rowArr['col-pp-id'], $this->seznamIDPP)) {
                    $this->seznamIDPP[] = $rowArr['col-pp-id'];
                }

                $this->rcPracPomeru[$rowArr['col-pp-id']] = $rowArr['col-pp-rc'];
            }

            foreach ($this->fieldsPP as $field) {
                $field->setData($this->udajePP);
            }

            $this->doplnRozdeleniKalendar();
        }

        $this->cursor->seek(0);
    }

    /** Nacte platne ziskane dovednosti osob. Dovednosti je mozne odfiltrovat (vyuzije se u pozadovanych, kde se hledaji jen ziskane k pozadavkum).
     * @param SqlStatement $sqlFilter
     * @return void
     * @throws DBException
     */
    protected function loadZiskaneDovednostiOsoby($sqlFilter = null)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT DovednostOsoby.RC, Dovednost.Popis AS PopisDovednosti, Dovednost.Kod AS KodDovednosti, UrovenDovednosti.KodUrovne, DovednostOsoby.PlatnostDo, UrovenDovednosti.Popis, DovednostOsoby.SpecifikaceUrovne,
            Dovednost.TypDovednosti, TypDovednosti.Popis              
            FROM DovednostOsoby  
            INNER JOIN UrovenDovednosti ON (DovednostOsoby.KodDovednosti = UrovenDovednosti.KodDovednosti AND DovednostOsoby.KodUrovne = UrovenDovednosti.KodUrovne)  
            INNER JOIN Dovednost ON (UrovenDovednosti.KodDovednosti = Dovednost.Kod)
            INNER JOIN TypDovednosti ON TypDovednosti.Kod = Dovednost.TypDovednosti
            WHERE Dovednost.Aktivni = 1    
            AND DovednostOsoby.DatumZiskani <= " . $params->addDateParameter(time()) . "  
            AND (DovednostOsoby.PlatnostDo >= " . $params->addDateParameter(time()) . " OR DovednostOsoby.PlatnostDo IS NULL)
            AND NOT EXISTS(SELECT RC FROM DovednostOsoby AS DO WHERE DO.RC = DovednostOsoby.RC AND DO.KodDovednosti = DovednostOsoby.KodDovednosti
                            AND DO.DatumZiskani <= " . $params->addDateParameter(time()) . "  
                            AND (DO.PlatnostDo >= " . $params->addDateParameter(time()) . " OR DO.PlatnostDo IS NULL) 
                            AND DO.KodUrovne > DovednostOsoby.KodUrovne)";

        if ($this->filterForm->getFilterValue('typdovednosti')) {
            $sql .= " AND Dovednost.TypDovednosti = " . $params->addParameter($this->filterForm->getFilterValue('typdovednosti'));
        }

        if ($this->filterForm->getFilterValue('dovednost')) {
            $sql .= " AND Dovednost.Kod = " . $params->addParameter($this->filterForm->getFilterValue('dovednost'));
        }

        $sql .=
            " AND " . SQLUtils::getINCondition($this->database, 'DovednostOsoby.RC', $this->rcPracPomeru, $params);

        if (!is_null($sqlFilter)) {
            $sql .= " AND " . $sqlFilter->sqlText;
            $params->appendParameters($sqlFilter->parameters);
        }

        $cursor = $this->database->select($sql, $params);

        while ($row = $cursor->fetch()) {
            //pokud dovednost jeste neni v seznamu dovednosti pro hlavicku, doplni ji
            $dovednost = $this->seznamDovednosti[$row[2]] ?? null;
            if (!$dovednost) {
                $dovednost = new DovednostOsoby($row[2], $row[1]);
                $dovednost->typ = $row[7];
                $dovednost->typPopis = $row[8];
                $this->seznamDovednosti[$row[2]] = $dovednost;
            }

            if (($this->typZobrazeni == 'ziskane') || ($this->osoby[$row[0]]->urovenPozadovaneDovednosti($dovednost, true) !== 0)) {
                $ziskanaDovednost = new DovednostOsoby($row[2], $row[1], $row[3]);
                $ziskanaDovednost->popisUrovne = $row[5];
                $ziskanaDovednost->platnostDo = $this->database->stringToTimestamp($row[4]);
                $this->osoby[$row[0]]->priradZiskanouDovednost($ziskanaDovednost, $row[6]);
            }
            if (!isset($dovednost->htmlPopisUrovni[$row[3]])) {
                $dovednost->htmlPopisUrovni[$row[3]] = StringUtils::htmlTextConvert($row[5]);
            }

        }

        //ve vychozim stavu se radi dle popisu dovednosti
        //toto chovani je mozno prenastavit na razeni dle typu dovednosti a nasledne dovednosti
        if ($this->raditDleTypu == 1) {
            uasort($this->seznamDovednosti, array($this, "seradDovednostiDleTypu"));
        } else {
            uasort($this->seznamDovednosti, array($this, "seradDovednostiDlePopisu"));
        }

        $typ = '';
        $pocetDovednostiTypu = [];
        foreach ($this->seznamDovednosti as $dovednost) {
            if (!isset($pocetDovednostiTypu[$dovednost->typ])) {
                $pocetDovednostiTypu[$dovednost->typ] = 1;
            }
            if ($typ == $dovednost->typ) {
                $pocetDovednostiTypu[$dovednost->typ]++;
            }

            $this->addField(new DovednostOsobyBrowserField($dovednost, $this->zobrazeniUrovneDovednosti, $this->filterForm->getFilterValue('expirace'), $this->typZobrazeni == 'ziskane'))->setData($this->osoby);
            $typ = $dovednost->typ;
        }
        foreach ($this->fields as $field) {
            if ($field instanceof DovednostOsobyBrowserField)
                $field->setPocty($pocetDovednostiTypu);
        }
    }

    /** Nacte data pro browser ziskanych dovednosti.
     * @return void
     * @throws DBException
     */
    public function loadZiskaneDovednosti()
    {
        $params = $this->database->createParameters();

        $filter = new SqlStatement(
            " EXISTS(SELECT DovednostOsoby.RC FROM DovednostOsoby JOIN Dovednost ON Dovednost.Kod = DovednostOsoby.KodDovednosti WHERE DovednostOsoby.RC = Osoba.RC " .
            "AND DovednostOsoby.DatumZiskani <= " . $params->addDateParameter(time()) .
            "  AND (DovednostOsoby.PlatnostDo >= " . $params->addDateParameter(time()) . " OR DovednostOsoby.PlatnostDo IS NULL) " .
            "  AND %FILTER% " .
            //17.3.2017 RON-PR doplnena podminka jen na aktivni dovednosti
            "  AND Dovednost.Aktivni = 1)", $params);

        $this->sqlFilterDovednosti = $filter;

        $this->load();

        $this->loadPP();

        if (!empty($this->rcPracPomeru)) {
            $this->loadZiskaneDovednostiOsoby();
        }
    }

    /** Nacte data pro browser pozadovanych dovednosti.
     * @return void
     * @throws DBException
     */
    public function loadPozadovaneDovednosti()
    {
        $params = $this->database->createParameters();

        $filter = new SqlStatement(
            " EXISTS(SELECT Dovednost.Kod FROM PozadavekNaDovednost JOIN Dovednost ON Dovednost.Kod = PozadavekNaDovednost.KodDovednosti " .
            " JOIN RozdeleniOsob ON RozdeleniOsob.KodTypuRozdeleni = PozadavekNaDovednost.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadavekNaDovednost.KodRozdeleni " .
            " JOIN PracovniPomer ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu " .
            " WHERE PracovniPomer.RC = Osoba.RC " .
            "   AND PracovniPomer.DatumNastupu <= " . $params->addDateParameter(time()) .
            "   AND (PracovniPomer.DatumUkonceni >= " . $params->addDateParameter(time()) . " OR PracovniPomer.DatumUkonceni IS NULL) " .
            "   AND RozdeleniOsob.PlatnostOd <= " . $params->addDateParameter(time()) .
            "   AND (RozdeleniOsob.PlatnostDo >= " . $params->addDateParameter(time()) . " OR RozdeleniOsob.PlatnostDo IS NULL) " .
            "   AND PozadavekNaDovednost.PlatnostOd <= " . $params->addDateParameter(time()) .
            "   AND (PozadavekNaDovednost.PlatnostDo >= " . $params->addDateParameter(time()) . " OR PozadavekNaDovednost.PlatnostDo IS NULL) " .
            "   AND %FILTER% " .
            "   AND Dovednost.Aktivni = 1)", $params);

        $this->sqlFilterDovednosti = $filter;

        $this->load();

        $this->loadPP();

        if (!empty($this->rcPracPomeru)) {
            $params = $this->database->createParameters();

            $sql =
                "SELECT PracovniPomer.RC, Dovednost.Popis AS PopisDovednosti, Dovednost.Kod AS KodDovednosti, UrovenDovednosti.KodUrovne, UrovenDovednosti.Popis, Dovednost.TypDovednosti, TypDovednosti.Popis              
            FROM PozadavekNaDovednost 
                INNER JOIN Dovednost ON Dovednost.Kod = PozadavekNaDovednost.KodDovednosti
                INNER JOIN RozdeleniOsob ON RozdeleniOsob.KodTypuRozdeleni = PozadavekNaDovednost.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadavekNaDovednost.KodRozdeleni
                INNER JOIN PracovniPomer ON RozdeleniOsob.RC = PracovniPomer.RC AND RozdeleniOsob.OsobniCislo = PracovniPomer.OsobniCislo AND RozdeleniOsob.DatumNastupu = PracovniPomer.DatumNastupu
                INNER JOIN TypDovednosti ON TypDovednosti.Kod = Dovednost.TypDovednosti
                INNER JOIN UrovenDovednosti ON UrovenDovednosti.KodDovednosti = Dovednost.Kod AND UrovenDovednosti.KodUrovne = PozadavekNaDovednost.PozadovanaUroven
            WHERE Dovednost.Aktivni = 1    
            AND PracovniPomer.DatumNastupu <= " . $params->addDateParameter(time()) . "  
            AND (PracovniPomer.DatumUkonceni >= " . $params->addDateParameter(time()) . " OR PracovniPomer.DatumUkonceni IS NULL) 
            AND RozdeleniOsob.PlatnostOd <= " . $params->addDateParameter(time()) . "  
            AND (RozdeleniOsob.PlatnostDo >= " . $params->addDateParameter(time()) . " OR RozdeleniOsob.PlatnostDo IS NULL)  
            AND PozadavekNaDovednost.PlatnostOd <= " . $params->addDateParameter(time()) . "  
            AND (PozadavekNaDovednost.PlatnostDo >= " . $params->addDateParameter(time()) . " OR PozadavekNaDovednost.PlatnostDo IS NULL) 
            AND NOT EXISTS(SELECT VyssiUroven.PozadovanaUroven FROM PozadavekNaDovednost AS VyssiUroven 
                        WHERE VyssiUroven.KodTypuRozdeleni = PozadavekNaDovednost.KodTypuRozdeleni AND VyssiUroven.KodRozdeleni = PozadavekNaDovednost.KodRozdeleni 
                          AND VyssiUroven.KodDovednosti = PozadavekNaDovednost.KodDovednosti
                          AND VyssiUroven.PlatnostOd <= " . $params->addDateParameter(time()) . "  
                          AND (VyssiUroven.PlatnostDo >= " . $params->addDateParameter(time()) . " OR VyssiUroven.PlatnostDo IS NULL)  
                          AND VyssiUroven.PozadovanaUroven > PozadavekNaDovednost.PozadovanaUroven)";

            if ($this->filterForm->getFilterValue('typdovednosti')) {
                $sql .= " AND Dovednost.TypDovednosti = " . $params->addParameter($this->filterForm->getFilterValue('typdovednosti'));
            }

            if ($this->filterForm->getFilterValue('dovednost')) {
                $sql .= " AND Dovednost.Kod = " . $params->addParameter($this->filterForm->getFilterValue('dovednost'));
            }

            $sql .=
                " AND " . SQLUtils::getINCondition($this->database, 'PracovniPomer.RC', $this->rcPracPomeru, $params);

            $cursor = $this->database->select($sql, $params);

            $pozadovaneDovednosti = [];

            while ($row = $cursor->fetch()) {
                //pokud dovednost jeste neni v seznamu dovednosti pro hlavicku, doplni ji
                $dovednost = $this->seznamDovednosti[$row[2]] ?? null;
                if (!$dovednost) {
                    $dovednost = new DovednostOsoby($row[2], $row[1]);
                    $dovednost->typ = $row[5];
                    $dovednost->typPopis = $row[6];
                    $this->seznamDovednosti[$row[2]] = $dovednost;
                    $pozadovaneDovednosti[] = $row[2];
                }

                $pozadovanaDovednost = new DovednostOsoby($row[2], $row[1], $row[3]);
                $pozadovanaDovednost->popisUrovne = $row[4];
                $this->osoby[$row[0]]->priradPozadovanouDovednost($pozadovanaDovednost);

                if (!isset($dovednost->htmlPopisUrovni[$row[3]])) {
                    $dovednost->htmlPopisUrovni[$row[3]] = StringUtils::htmlTextConvert($row[4]);
                }
            }

            $paramsFilter = $this->database->createParameters();
            $dovednostiFilter = SQLUtils::getINCondition($this->database, 'Dovednost.Kod', $pozadovaneDovednosti, $paramsFilter);

            $this->loadZiskaneDovednostiOsoby(new SqlStatement($dovednostiFilter, $paramsFilter));
        }
    }

    /** Doplni do hlavicky tabulky nactene udaje z filtru. Pouzije se pri nacitani pritomnosti.
     * @param $sqlFilter
     * @return void
     * @throws Exception
     */
    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);

        $this->tableHtmlParams['data-ron-pritomnost-url'] =
            '?menuid=dovednosti&action=pritomnost' .
            '&rozdeleni1=' . StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni')) .
            '&vcetnepodrizenych1=' . (int)$this ->filterForm ->getFilterValue('rozdeleni2', 'vcetnePodrizenych') .
            '&rozdeleni2=' . StringUtils::htmlTextConvert($this ->filterForm ->getFilterValue('rozdeleni2')) .
            '&vcetnepodrizenych2=' . (int)$this ->filterForm ->getFilterValue('rozdeleni2', 'vcetnePodrizenych') .
            '&jenrc=' . 1;

        $this->tableHtmlParams['data-ron-fixed-cols'] = 3;
    }

    /** Pouziti Array cursoru pro spravnou funkcnost posunu na zacatek cursoru (dat). (ODBC cursor a Array cursor mají odlišný index prvního prvku, použití ArrayCursor umožní indexovat vždy stejně)
     * @param $sql
     * @return void
     * @throws Exception
     */
    public function loadSQL($sql)
    {
        parent::loadSQL($sql);

        $this->cursor = new ArrayCursor($this->cursor);
    }

    /** Vypise hlavicku tabulky. V pripade zapnuteho seskupovani dovednosti dle typu je hlavicka pres 2 radky,
     * kdy zakladni sloupce zabiraji 2 radky a u dovednosti je v prvnim radku zobrazeny popis typu (zabirajici odpovidajici pocet sloupcu) a ve druhem pak popis dovednosti.
     * @return void
     */
    public function renderTableHeader()
    {
        if ($this->raditDleTypu) {

            echo '<thead class="floating-table-header">';
            echo '<tr>';

            $this->columnCount = 0;
            $colSpan = 0;
            $typDovednosti = '';

            foreach ($this->fields as $i => $field) {
                if ($field->visible) {
                    if ($colSpan > 1) {
                        // tento header nezobrazuje, protoze predchozi byl pres vice sloupcu a zasahuje i do tohoto sloupce
                        $colSpan--;
                    } else {
                        if (!$field instanceof DovednostOsobyBrowserField) {
                            $field->setHeaderAttributes(['rowSpan' => 2]);
                            $poms = $field->getHeader($i, $this->orderFieldIndex, $this->browserUrl);
                        } else {
                            if ($typDovednosti !== $field->getTypDovednosti()) {
                                $poms = $field->getHeaderTypDovednosti();
                            } else {
                                $poms = '';
                            }
                            $typDovednosti = $field->getTypDovednosti();
                        }
                        if ($poms) {
                            echo $poms;

                            $colSpan = (int)$field->headerColSpan;
                            $this->columnCount += ($colSpan > 1) ? $colSpan : 1;
                        }
                    }
                }
            }
            echo "</tr>\n";
            echo '<tr>';

            foreach ($this->fields as $i => $field) {
                if ($field instanceof DovednostOsobyBrowserField) {
                    echo $field->getHeader($i, $this->orderFieldIndex, $this->browserUrl);
                }
            }
            echo "</tr>\n";
            echo "</thead>\n";
        } else {
            parent::renderTableHeader();
        }
    }

    /** Zobrazi tabulku. Nad tabulkou pak odkaz ke stazeni CSV a pod doplnujici info z prekladu.
     * @return void
     * @throws DBException
     */
    public function renderContent()
    {
        $this->zobrazFiltrProTisk();
        echo '<a class="no-print" href="?menuid=maticedovednosti&action=ziskane&format=csv">' . TextManager::_('Dovednosti_csv') . '</a>';

        parent::renderContent();

        echo TextManager::_('Dovednosti_prehled_doplnujici_info');
    }

    /** Nacte prirazena rozdeleni a kalendare osob.
     * @throws DBException
     */
    public function doplnRozdeleniKalendar()
    {
        $setData = false;

        if (isset($this->typyRozdeleni)) {
            $seznamRozdeleni = RozdeleniOsob::nactiRozdeleniOsoby($this->database, $this->seznamIDPP, $this->typyRozdeleni, DateUtils::today());

            if (!empty($seznamRozdeleni)) {
                foreach ($seznamRozdeleni as $rozdeleni) {
                    $rc = $this->rcPracPomeru[$rozdeleni->idPP];
                    $this->osoby[$rc]->priradRozdeleniOsoby($rozdeleni, $rozdeleni->kodTypuRozdeleni);
                }
            }

            $setData = true;
        }

        if ($this->zobrazitKalendar) {
            $seznamKalendaru = PrirazeniKalendare::nactiKalendarProObdobi($this->database, $this->seznamIDPP, DateUtils::today());
            $seznamKalendaru = !empty($seznamKalendaru) ? $seznamKalendaru : null;

            if (!empty($seznamKalendaru)) {
                foreach ($seznamKalendaru as $kalendar) {
                    $this->osoby[$kalendar->rc]->priradKalendar($kalendar);
                }
            }

            $setData = true;
        }

        if ($setData) {
            foreach ($this->fields as $field) {
                if ($field instanceof DovednostBrowserField) {
                    $field->setData($this->osoby);
                }
            }
        }
    }

    /** Zobrazeni zvolenych filtru pri tisku stranky z prohlizece
     * @throws Exception
     * @throws DBException
     */
    public function zobrazFiltrProTisk()
    {
        $rozdeleni = new Rozdeleni();
        if ($this->filterForm->getFilterValue('rozdeleni') !== null) {
            $rozdeleni->load($this->database,
                $this->filterForm ->getFilterValue('rozdeleni', 'typRozdeleni'),
                $this->filterForm ->getFilterValue('rozdeleni'));
        }

        $params = $this->database->createParameters();
        $typDovednosti = DBUtils::getSqlValue($this->database,
            'SELECT Popis FROM TypDovednosti WHERE Kod = ' . $params->addParameter($this->filterForm->getFilterValue('typdovednosti')), $params);

        echo "<div class='dovednostifilterprint'>";
        if ($rozdeleni->kod) {
            echo "<strong>" . $this->typRozdeleniFiltr->popis . "</strong>: <span class='$rozdeleni->cssClass'> " . StringUtils::htmlTextConvert($rozdeleni->popis . ' (' . $rozdeleni->kod . ')') . "</span> <br>";
        }
        if ($this->filterForm->getFilterValue('typdovednosti')) {
            echo "<strong>" . TextManager::_('Dovednosti_typ_dovednosti') . "</strong>: <span>" . StringUtils::htmlTextConvert($typDovednosti) . "</span> <br>";
        }
        if ($this->filterForm->vedouciJmenoPrijmeni) {
            echo "<strong>" . TextManager::_('Dovednosti_vedouci') . "</strong>: <span>" . StringUtils::htmlTextConvert($this->filterForm->vedouciJmenoPrijmeni) . "</span> <br>";
        }
        if ($this->filterForm->zastupceJmenoPrijmeni) {
            echo "<strong>" . TextManager::_('Dovednosti_zastupce') . "</strong>: <span>" . StringUtils::htmlTextConvert($this->filterForm->zastupceJmenoPrijmeni) . "</span> <br>";
        }
        echo "</div>";
    }

    /**
     * Doplni k radku id pracovniho pomeru pro javascript.
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @return string
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        return 'data-ron-idpp="' . $values['col-idpp'] . '" ' . parent::getRowAttributes($values, $className, $rowNumber);
    }

    /** Sestavi text z nadpisy sloupcu pro export do CSV. Sloupce jsou napevno: Zamestnanec, Osobni cislo, typ PP, prac. mist, rozdeleni, kalendar, stav, dovednosti
     * @return string
     */
    private function hlavickaCSV()
    {
        $data =
            (TextManager::getInstance() ->isUTF8()
                ? "\xEF\xBB\xBF"    // v UTF8 prida BOM
                : ''
            ) .
            TextManager::_('Zamestnanec') . static::CSV_SEPARATOR .
            TextManager::_('Osobni_cislo') . static::CSV_SEPARATOR .
            ($this->zobrazitTypPP
                ? TextManager::_('Dovednosti_typ_PP') . static::CSV_SEPARATOR
                : ''
            ) .
            ($this->zobrazitPracovniMisto
                ? $this->typRozdeleniPracMisto->popis . static::CSV_SEPARATOR
                : ''
            ) .
            ($this->zobrazitRozdeleni
                ? $this->typRozdeleniFiltr->popis . static::CSV_SEPARATOR
                : ''
            ) .
            ($this->zobrazitKalendar
                ? TextManager::_('Kalendar') . static::CSV_SEPARATOR
                : ''
            ) .
            TextManager::_('Dovednosti_stav') . static::CSV_SEPARATOR;

        foreach($this->seznamDovednosti as $dovednost) {
            $data .= $dovednost->popis . static::CSV_SEPARATOR;
            //zobrazit platnost
            if ($this->filterForm->getFilterValue('expirace')) {
                $data .=  TextManager::_('Platnost_do') . static::CSV_SEPARATOR;
            }

        }

        $data .= "\n";

        return $data;
    }

    /** Sestavi retezec s nactenymi daty do sloupcu pro CSV, sloupce viz hlavickaCSV.
     * @return string
     */
    private function dataDovednostiDoCSV()
    {
        $data = $this->hlavickaCSV();

        foreach($this->osoby as $osoba)
        {
            $data .= $osoba->prijmeni . ' ' . $osoba->jmeno . static::CSV_SEPARATOR;

            $data .= $osoba->getOsobniCislo() . static::CSV_SEPARATOR;

            if ($this->zobrazitTypPP == 1) {
                $data .= $osoba->getTypPP() . static::CSV_SEPARATOR;
            }

            if ($this->zobrazitPracovniMisto) {
                //vypise seznam pracovnich mist osoby
                $seznamPM = '';
                $seznamPopisuPM = '';
                foreach ($osoba->prirazenaRozdeleniOsoby[$this->typRozdeleniPracMisto->kod] as $pracovniMisto) {
                    if ($seznamPM) {
                        $seznamPM .= ', ';
                    }
                    if ($seznamPopisuPM) {
                        $seznamPopisuPM .= ', ';
                    }
                    $seznamPopisuPM .= $pracovniMisto->popis;
                    $seznamPM .= $pracovniMisto->kod;
                }
                if ($this->zobrazitPracovniMisto == 1) {
                    $data .= $seznamPM . static::CSV_SEPARATOR;
                } elseif ($this->zobrazitPracovniMisto == 2) {
                    $data .= $seznamPopisuPM . static::CSV_SEPARATOR;
                }
            }

            if ($this->zobrazitRozdeleni) {
                //vypise seznam kodu rozdeleni osoby
                $seznamRozdeleni = '';
                $seznamPopisuRozdeleni = '';
                foreach ($osoba->prirazenaRozdeleniOsoby[$this->typRozdeleniFiltr->kod] as $rozdeleni) {
                    if ($seznamRozdeleni) {
                        $seznamRozdeleni .= ', ';
                    }
                    if ($seznamPopisuRozdeleni) {
                        $seznamPopisuRozdeleni .= ', ';
                    }
                    $seznamPopisuRozdeleni .= $rozdeleni->popis;
                    $seznamRozdeleni .= $rozdeleni->kod;
                }
                if ($this->zobrazitRozdeleni == 1) {
                    $data .= $seznamRozdeleni . static::CSV_SEPARATOR;
                } elseif ($this->zobrazitRozdeleni == 2) {
                    $data .= $seznamPopisuRozdeleni . static::CSV_SEPARATOR;
                }
            }

            if ($this->zobrazitKalendar) {
                $data .= $osoba->getKalendar($this->zobrazitKalendar) . static::CSV_SEPARATOR;
            }

            $data .= $osoba->pocetZiskanychDovednosti . ($this->typZobrazeni == 'pozadovane' ? '/' . $osoba->pocetPozadovanychDovednosti : '') . static::CSV_SEPARATOR;

            /** @var DovednostOsoby $dovednost */
            foreach($this->seznamDovednosti as $dovednost)
            {
                if ($this->typZobrazeni == 'pozadovane') {
                    $data .= $osoba->urovenZiskaneDovednosti($dovednost, true) . '/' . $osoba->urovenPozadovaneDovednosti($dovednost, true) . static::CSV_SEPARATOR;
                }
                else {
                    $data .=
                        ($this->zobrazeniUrovneDovednosti == 0 ? $osoba->urovenZiskaneDovednosti($dovednost) : $dovednost ->htmlPopisUrovni[$osoba->ziskaneDovednosti[$dovednost->kod]->uroven]) . static::CSV_SEPARATOR;

                    //zobrazit platnost do
                    if ($this->filterForm->getFilterValue('expirace')) {
                        if (isset($osoba->ziskaneDovednosti[$dovednost->kod]->platnostDo)) {
                            $data .= StringUtils::format('date', $osoba->ziskaneDovednosti[$dovednost->kod]->platnostDo);
                        }
                        $data .= static::CSV_SEPARATOR;
                    }
                }
            }

            $data .= "\n";
        }

        return $data;
    }

    /** Sestavi obsah CSV souboru a posle jej na vystup.
     * @return DataOutputObject
     */
    public function outputCSV()
    {
        $data = $this->dataDovednostiDoCSV();
        switch ($this->typZobrazeni)
        {
            case 'pozadovane':
                $fileName = 'pozadovaneDovednosti.csv';
                break;
            case 'ziskane':
            default :
                $fileName = 'ziskaneDovednosti.csv';
                break;
        }
        return new DataOutputObject($data, $fileName, TextManager::getInstance() ->getCharSet());
    }

    /** Seradi seznam dovednosti dle popisu.
     * @param DovednostOsoby $dovednost1
     * @param DovednostOsoby $dovednost2
     * @return int
     */
    public function seradDovednostiDlePopisu($dovednost1, $dovednost2)
    {
        return StringUtils::compareStrings($dovednost1->popis, $dovednost2->popis);
    }

    /** Seradi seznam dovednosti dle popisu typu a pripadne dle popisu dovednosti.
     * @param DovednostOsoby $dovednost1
     * @param DovednostOsoby $dovednost2
     * @return int
     */
    public function seradDovednostiDleTypu($dovednost1, $dovednost2)
    {
        $poradi = StringUtils::compareStrings($dovednost1->typPopis, $dovednost2->typPopis);
        if ($poradi == 0) {
            return $this->seradDovednostiDlePopisu($dovednost1, $dovednost2);
        }
        return $poradi;
    }
}