<?php
/**
 * pavel.rudol, 02.11.2022 13:25
 */

namespace RonPortal\Dovednosti;

use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\TextManager;

class UkonceniTypovehoMistaField extends BrowserField
{

    /** Zobrazi odkaz pro ukonceni prirazeneho typoveho mista.
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return "<a href='#' class='ukonceniTPM' data-tpm-kod='" . $rowData['col-kod'] . "'
                    data-tpm-platnost-od='" . DBUtils::getFormatedValue($this->database, 'date', $rowData['col-platnostod']) . "'>"
            . TextManager::_('Dovednosti_ukoncit_tpm') . "</a>";
    }
}

class TypovePracovniMistoOsobyBrowser extends Browser
{
    /**
     * TypovePracovniMistoOsobyBrowser constructor.
     * @param Database $database
     * @param string $url url na ktere je browser zobrazen
     * @param int $idOsoby id PP osoby
     * @param int $typRozdeleniTypovePracovniMisto
     * @throws DBException
     */
    public function __construct($database, $url, $idOsoby, $typRozdeleniTypovePracovniMisto)
    {
        parent::__construct($database);

        $this->showMessages = false;

        $this->tableId = 'tpm';
        $this->code = 'browsertpm';

        $osobaRC = DBUtils::getSqlValue($this->database, 'SELECT RC FROM PracovniPomer WHERE ID = ' . (int)$idOsoby);

        $this->title = TextManager::_('Dovednosti_typova_mista_zamestnance');

        $this->browserUrl = $url . '&';

        $this->sqlFromPart =
            "RozdeleniOsob 
                JOIN PracovniPomer ON ((PracovniPomer.RC = RozdeleniOsob.RC) AND (PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo) 
                    AND (PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu)) 
                JOIN Rozdeleni ON (Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni AND Rozdeleni.Kod = RozdeleniOsob.Rozdeleni)";

        $params = $this->database->createParameters();
        $this->sqlWherePart = new SqlStatement(
            " RozdeleniOsob.KodTypuRozdeleni = " . $params->addParameter($typRozdeleniTypovePracovniMisto) .
            " AND RozdeleniOsob.RC = " . $params->addParameter($osobaRC) .
            " AND PracovniPomer.DatumNastupu <= " . $params->addDateParameter(DateUtils::today()) .
            " AND (PracovniPomer.DatumUkonceni >= " . $params->addDateParameter(DateUtils::today()) . " OR PracovniPomer.DatumUkonceni IS NULL)",
            $params);

        $this ->addField()
            ->setSqlFieldName('RozdeleniOsob.Rozdeleni') ->setAlign('left') ->setHeaderText('Kod')
            ->setCode('col-kod')
            ->setOrdering(true);
        $this ->addField()
            ->setSqlFieldName('Rozdeleni.Popis') ->setAlign('left') ->setHeaderText('Popis')
            ->setCode('col-popis')
            ->setOrdering(true);
        $this ->addField()
            ->setSqlFieldName('RozdeleniOsob.PlatnostOd') ->setAlign('right') ->setHeaderText('Platnost_od')
            ->setCode('col-platnostod')->setFormat('date')
            ->setOrdering(true);
        $this ->addField()
            ->setSqlFieldName('RozdeleniOsob.PlatnostDo') ->setAlign('right') ->setHeaderText('Kod')
            ->setCode('col-platnostdo')->setFormat('date')
            ->setOrdering(true);
        $this ->addField()
            ->setSqlFieldName('RozdeleniOsob.Poznamka') ->setAlign('left') ->setHeaderText('Kod')
            ->setCode('col-poznamka');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.ID')->setCode('col-idpp');

        $this->defaultSort = 'col-platnostod';

        $this->addField(new UkonceniTypovehoMistaField($this->database));
    }

}