<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use Ron\DB\DBRecordField;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;


class DenniPoznamkaDochazkyRecord extends RonPortalDBRecord
{
    /** @var DBRecordField  */
    var $fPoznamka;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int    $den
     * @param string|object $logDescription    Vychozi hodnota je 'Zpracovana dochazka'
     */
    function __construct($database, $pp, $den, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'DenniPoznamkaDochazky', $logDescription);

        $this->addField('RC', '', true, 15, $pp ->rc);
        $this->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $this->addField('Den', 'date', true, 0, $den);

        $this->fPoznamka = $this->addField('Poznamka', 'memo', false, 0, null);

        $pomfield = $this->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName());
        $pomfield->updateLog = true;
        $pomfield->noInsert = true;

        $pomfield = $this->addField('Update_Timestamp', 'datetime', false, 25, time());
        $pomfield->updateLog = true;
        $pomfield->noInsert = true;

        $this->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $this->addField('Insert_Timestamp', 'datetime', true, 25, time());

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND Den = ' . $params ->addParameter($database ->dateToParam($den)),
            $params);
    }


    /**
     * @param string $text
     */
    function zapisPoznamku($text)
    {
        $this->fPoznamka->setNewValue($text);

        if ($this->loadRecord()) {
            if (trim($text) == '') {
                $this->deleteRecord();
            } else {
                $this->updateRecord();
            }

        } else {
            if (trim($text) == '') return; // neni co vkladat

            $this->insertRecord();
        }
    }
}
