<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;

class DenniVysledekVypoctuRecord extends RonPortalDBRecord
{
    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $den
     * @param string $typCasoveSlozkyMzdy
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $den, $typCasoveSlozkyMzdy, $logDescription)
    {
        parent::__construct($database, 'DenniVysledekVypoctu', $logDescription);

        $this->addStringField('RC', true, 15, $pp ->rc);
        $this->addStringField('OsobniCislo', true, 15, $pp ->osobniCislo);
        $this->addDateField('DatumNastupu', true, $pp ->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $this->addDateField('Den', true, $den);
        $this->addStringField('TypCasoveSlozkyMzdy', true, 10, $typCasoveSlozkyMzdy);

        $this->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName());
        $this->addTimestampField('Insert_Timestamp', true, time());

        $f = $this->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName());
        $f ->setUpdateLog(true) ->setNoInsert(true);

        $f = $this->addTimestampField('Update_Timestamp', false, time());
        $f ->setUpdateLog(true) ->setNoInsert(true);

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND Den = ' . $params ->addParameter($database ->dateToParam($den)) .
            ' AND TypCasoveSlozkyMzdy = ' . $params ->addParameter($typCasoveSlozkyMzdy),
            $params);
    }


    /**
     * @param string $jednotka
     * @param mixed $hodnota
     * @param bool $vlozitNulovouHodnotu    Urcuje, jestli se ma vlozit novy zaznam, pokud je zadana hodnota nulova
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    function zapisHodnotu($jednotka, $hodnota, $vlozitNulovouHodnotu = false)
    {
        // nastaveni pole s hodnotou
        $hodiny = 0;
        $dny = 0;
        $kalendarnidny = 0;
        $poznamka = null;

        switch ($jednotka) {
            case 'D' :
                $dny = $hodnota;
                break;
            case 'K' :
                $kalendarnidny = $hodnota;
                break;
            case 'P' :
                $poznamka = $hodnota;
                break;
            default :
                $hodiny = $hodnota;
                break;
        }

        $this ->addField('Hodiny', 'float', $jednotka == 'D' || $jednotka == 'K' || $jednotka == 'P', 0, $hodiny); // readonly pokud se nejedna o hodiny
        $this ->addField('Dny', 'float', $jednotka != 'D', 0, $dny);
        $this ->addField('KalendarniDny', 'float', $jednotka != 'K', 0, $kalendarnidny);
        $this ->addStringField('Poznamka', $jednotka != 'P', 250, $poznamka);

        if ($this ->loadRecord()) {
            $this ->updateRecord();
        } else if ($vlozitNulovouHodnotu  ||  $hodnota || ($jednotka == 'P' && $hodnota != '')) {
            $this ->insertRecord();
        }
    }
}
