<?php
/**
 * stanislav.elbl, 03.02.2022 12:57
 */

namespace RonPortal\Editace;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\TextManager;
use RonPortal\Dochazka\SmenaProDen;
use RonPortal\Dochazka\ZaznamZpracovaneDochazky;
use RonPortal\Kalendar;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;


/**
 * Class EditaceDneUtils
 * @package RonPortal\Editace
 * Pomocne funkce pro editaci dochazky
 */
class EditaceDneUtils
{
    /**
     * Zkontroluje, jestli ma uzivatel pravo editovat zadany den (nebo vsechny dny v zdanem rozsahu). Kontroluje
     *   - jestli ma uzivatel pravo editovat zadany PP
     *   - jestli ma uzivatel pravo editovat vsechny existujici zaznamy
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $datum
     * @param int $datumDo      Pokud neni zadan, pouzije se $datum
     * @return bool
     * @throws \Exception
     */
    public static function jePravoEditovatDen($database, $pp, $datum, $datumDo = null)
    {
        $prava = Prava::vratPrava('editace');

        if (!$prava ->lzeZobrazitPP($pp ->idPP, $datum)) {
            // uzivatel vubec nemuze editovat zadany PP
            return false;
        }

        if ($prava ->priznak3 == '') {
            // uzivatel nema omezene operace, ktere muze editovat
            return true;
        }

        $datumDo = $datumDo ?: $datum;

        // Nacte zaznamy zadaneho dne a overi, jestli je muze vsechny editovat
        $zaznamy = ZaznamZpracovaneDochazky::nactiOperaceDochazkyZDB($database, $pp, $datum, $datumDo, false);

        foreach ($zaznamy as $zaznam) {
            if (!EditaceUtils::jePravoEditovatOperaci($zaznam ->kodOperace)) {
                return false;
            }
        }

        return true;
    }


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param ZaznamZpracovaneDochazky[] $zdrojoveZaznamy
     * @param int $cilovyDatum
     * @param object|string $logDescription
     */
    private static function zkopirujZaznamy($database, $pp, $zdrojoveZaznamy, $cilovyDatum, $logDescription)
    {
        if (!$zdrojoveZaznamy) {
            // neni co kopirovat
            return;
        }

        foreach ($zdrojoveZaznamy as $zaznam) {
            $rec = new DochazkaRecord($database, $pp, $logDescription);

            $rec ->fDen ->setNewValue($cilovyDatum);
            $rec ->fDatumCasPocatku ->setNewValue($zaznam ->datumCasPocatku - $zaznam ->den + $cilovyDatum);
            $rec ->fDatumCasUkonceni ->setNewValue($zaznam ->datumCasUkonceni - $zaznam ->den + $cilovyDatum);
            $rec ->fDatumCasOperacePuvDoch ->setNewValue($rec ->fDatumCasPocatku ->value);

            $rec ->fOperace ->setNewValue($zaznam ->kodOperace);

            $rec ->fDeltaPred ->setNewValue($zaznam ->deltaPred);
            $rec ->fDeltaZa ->setNewValue($zaznam ->deltaZa);

            $rec ->fSmena ->setNewValue($zaznam ->smena);
            $rec ->fZpracoval ->setNewValue(RonPortalUtils::getLogUserName());

            $rec ->insertRecord();
            // TODO: zkopirovat rozdeleni dochazky - ale asi jen editovatelne?
        }

    }


    /**
     * @param Database $database
     * @param PracovniPomer|int $pp Pracovni pomer nebo id pracovniho pomeru
     * @param int $zdrojovyDatum Den dochazky, ktery se ma zkopirovat
     * @param int $datumOd Pocatek rozsahu dnu, do kterych se ma kopirovat
     * @param int $datumDo Konec rozsahu dnu, do kterych se ma kopirovat
     * @param object|string $logDescription
     * @throws NeniPravoException
     * @throws \Exception
     */
    public static function zkopirujDenDochazky($database, $pp, $zdrojovyDatum, $datumOd, $datumDo, $logDescription, $options)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) {
            throw new \Exception('Editace dochazky neni povolena!');
        }

        if (!$pp instanceof PracovniPomer) {
            $pp = PracovniPomer::nactiPP($database, $pp);
        }

        $prava = Prava::vratPrava('editace');
        if (!$prava ->lzeZobrazitPP($pp ->idPP)) {
            throw new NeniPravoException();
        }

        $zdrojoveZaznamy = ZaznamZpracovaneDochazky::nactiOperaceDochazkyZDB($database, $pp, $zdrojovyDatum, $zdrojovyDatum, false);

        // zkontroluje prava uzivatele na editaci zdrojovych zaznamu
        foreach ($zdrojoveZaznamy as $zaznam) {
            if (!EditaceUtils::jePravoEditovatOperaci($zaznam ->kodOperace)) {
                // nelze zkopirovat den, ktery obsahuje operaci, na kterou uzivatel nema pravo
                throw new \Exception(TextManager::_('Neni_pravo_na_operaci'));
            }
        }

        // pokud je to potreba, tak nacte smeny zamestnance serazene podle dne
        $smeny = ($options['pouzeDnySeSmenou'] ?? false)
            ? SmenaProDen::nactiSmenyOsoby($database, $pp, $datumOd, $datumDo)
            : null;

        // nacte puvodni zaznamy v zadanem intervalu
        $zaznamyKeZruseni = ZaznamZpracovaneDochazky::nactiOperaceDochazkyZDB($database, $pp, $datumOd, $datumDo, false);

        $cilovyDatum = $datumOd;
        $index = 0;
        $indexSmeny = 0;

        while ($cilovyDatum <= $datumDo) {
            if ($cilovyDatum != $zdrojovyDatum) { // stejny den, jako zdrojovy - nekopiruje jej
                $prepsatDen = true;
                $existujeZaznam = false;

                while ($index < count($zaznamyKeZruseni)  &&  $zaznamyKeZruseni[$index] ->den < $cilovyDatum) {
                    // preskoci predchozi dny
                    $index++;
                }

                // overi, jestli existuje smena pro zadany den
                if ($options['pouzeDnySeSmenou'] ?? false) {
                    while ($indexSmeny < count($smeny)  &&  $smeny[$indexSmeny] ->datum < $cilovyDatum) {
                        // preskoci predchozi dny
                        $indexSmeny++;
                    }

                    if ($indexSmeny >= count($smeny) ||  $smeny[$indexSmeny] ->datum != $cilovyDatum) {
                        // pro dany den neexistuje smena
                        $prepsatDen = false;
                    }
                }

                // zkontroluje, jestli v cilovem dni neexistuje zaznam, na ktery nema uzivatel pravo
                while ($index <count($zaznamyKeZruseni)  &&  $zaznamyKeZruseni[$index] ->den == $cilovyDatum) {
                    $existujeZaznam = true;
                    if ($prepsatDen  &&  !EditaceUtils::jePravoEditovatOperaci($zaznamyKeZruseni[$index] ->kodOperace)) {
                        $prepsatDen = false;
                    }
                    $index++;
                }

                if ($existujeZaznam  &&  $options['pouzePrazdneDny'] ?? false) {
                    // dny, ve kterych existuje zaznam se nemaji prepisovat - nedela nic
                    $prepsatDen = false;
                }

                if ($prepsatDen) {
                    if ($existujeZaznam) {
                        $rec = new DochazkaRecord($database, $pp, $logDescription);
                        $rec->logInfoMessage = 'Smazani dochazky pro den ' . date('j.n.Y', $cilovyDatum);

                        $params = $database->createParameters();
                        $rec->smazZaznamyDochazky(
                            ' Den = ' . $params->addParameter($database->dateToParam($cilovyDatum)),
                            $params
                        );
                    }

                    self::zkopirujZaznamy($database, $pp, $zdrojoveZaznamy, $cilovyDatum, $logDescription);
                }
            }

            $cilovyDatum = DateUtils::incDayStart($cilovyDatum);
        }
    }
}
