<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\Dochazka\SchvaleniDochazky;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\System\AgentManager;

class EditaceDochazkyFormular implements ViewObject
{
    /** @var  Database */
    public $database;
    //var $f_formaction; // edit / insert / nic (readonly)

    public $action;

    /** @var PracovniPomer */
    var $pp;

    /** @var ZaznamDochazky */
    var $zaznam;

    var $editacePovolena;
    var $pravaEditace;
    var $readOnly;

    var $obdobiUzavreno;

    /** @var SchvaleniDochazky */
    var $schvaleni;


    function __construct($database, $action = 'ulozzaznam')
    {
        $this->zaznam = null;
        $this->database = $database;

        $this ->action = $action;
    }


    /**
     * @param string $operace
     * @param string $varSuffix  Pripona pro jmeno promenne - slouzi pro rozliseni zaznamu v editaci dne
     * @param bool $zobrazitKod
     */
    function showVyberOperace($operace, $varSuffix, $zobrazitKod)
    {
//        $readonlyStr = $this->readOnly
//            ? 'disabled="disabled" '
//            : '';


        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM OperaceDochazky ' .
            ' WHERE (VkladatVEditaciDochazky = 1';

        $prava = Prava::vratPrava('editace');
        if (trim($prava ->priznak3 ?? '') !== '') {
            $sql .= ' AND ' . SQLUtils::getINCondition($this ->database, 'Kod', explode(';', $prava ->priznak3), $params);
        }

        $sql .= ')';

        if (isset($operace)) {
            $sql .= '  OR  Kod = ' . $params->addParameter($operace);
        }


        $combo = new ComboBox(($zobrazitKod ? 'kodoperace' : 'operace') . $varSuffix);
        $combo ->emptyRow = ' ';
        $combo ->alwaysShowSelected = true;
        $combo ->className = 'form-control';

        if ($this ->readOnly) {
            $combo ->htmlParams['disabled'] = 'disabled';
        }

        if ($zobrazitKod) {
            $combo ->captionIndex = 0; // jen kod
            $combo ->loadData($this ->database, $sql . ' ORDER BY Kod', $operace, $params);
            $combo ->htmlParams = ['data-ron-select2' => "operace{$varSuffix}"];
        } else {
            $combo ->setTranslationPrefix('db_operacedochazky_popis_');
            $combo ->loadData($this ->database, $sql . ' ORDER BY Popis', $operace, $params);
            $combo ->htmlParams = ['data-ron-select2' => "kodoperace{$varSuffix}"];
        }

        $combo ->render();
    }


    /**
     * @param ZaznamDochazky $zaznam
     * @param string $varName
     * @throws DBException
     */
    function showVyberSmeny($zaznam, $varName)
    {
        // pri vkladani prednastavi smenu podle kalendare
        if ($zaznam ->fSmena ->value) {
            $smena = ($zaznam->fSmena->value);
        } else {
            $s = EditaceUtils::vychoziSmenaProOsobu($this->database, $zaznam->f_pp, $zaznam->fDen->value);
            $smena = $s ? $s->kod : null;
        }

        $params = $this ->database ->createParameters();

        // pres odbc nefunguji parametry v EXISTS, takze nejdrive nacitam smeny z kalendare a az pak sestavi sql pro vyber smen
        $sql =
            'SELECT DISTINCT KodSmeny FROM PouzivaneSmeny, PrirazeniKalendare  ' .
            ' WHERE PrirazeniKalendare.Kalendar = PouzivaneSmeny.KodKalendare' .
            ' AND PrirazeniKalendare.RC = ' . $params ->addParameter($zaznam ->f_pp ->rc) .
            ' AND PrirazeniKalendare.OsobniCislo = ' . $params ->addParameter($zaznam ->f_pp ->osobniCislo) .
            ' AND (PrirazeniKalendare.DatumNastupu = ' . $params ->addParameter($this ->database ->dateToParam($zaznam ->f_pp ->datumNastupu)) . ')' .
            ' AND (PrirazeniKalendare.PlatnostOd <= ' . $params ->addParameter($this ->database ->dateToParam($zaznam ->fDen ->value)) . ')' .
            ' AND ((PrirazeniKalendare.PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam($zaznam ->fDen ->value)) . ') OR (PrirazeniKalendare.PlatnostDo IS NULL))';
        $cursor = $this ->database ->getSqlResult($sql, $params);
        $seznamSmen = [];
        while ($row = $cursor ->fetch()) {
            $seznamSmen[] = $row[0];
        }
        $cursor ->close();

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT DISTINCT Smena.Kod, Smena.Popis' .
            ' FROM Smena' .
            ' WHERE (Smena.Aktivni = 1' .
            '   AND (Smena.Kod IS NULL ';

        foreach ($seznamSmen as $kodSmeny) {
            $sql .= ' OR Smena.Kod = ' . $params ->addParameter($kodSmeny);
        }

        $sql .=
            '))';

        // smenu zapsanou v existujicim zaznamu zobrazi vzdy (muze byt jina - napr. z planovani kapacit)
        if ($zaznam ->fSmena ->value) {
            $sql .= ' OR Smena.Kod = ' . $params ->addParameter($zaznam ->fSmena ->value);
        }

        $sql .= ' ORDER BY Popis';

        $combo = new ComboBox($varName);
        $combo
            ->setAlwaysShowSelected(true)
            ->setShowCode(true)
            ->setTranslationPrefix('db_smena_popis_');

        if ($this ->readOnly) {
            $combo->addHtmlParam('disabled', 'disabled');
        }
        $combo->loadData($this ->database, $sql, $smena, $params);
        $combo->render();
    }


    /**
     * @param ZaznamDochazky $zaznam
     * @param string $varname
     * @param bool   $zobrazitKod
     */
    function showVyberRozdeleni($zaznam, $varname, $zobrazitKod)
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Kod, Popis FROM Rozdeleni ' .
            ' WHERE KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI) .
            '   AND (DatumZruseni >= ' . $params ->addParameter($this ->database ->dateToParam($zaznam ->fDen ->value)) . ' OR DatumZruseni IS NULL) ' .
            '   AND DatumVytvoreni <= ' . $params ->addParameter($this ->database ->dateToParam($zaznam ->fDen ->value));

        if ($zobrazitKod) {
            // vyber kodu rozdeleni
            $combo = new ComboBox($varname . '_kod');
            $combo->alwaysShowSelected = true;
            $combo->captionIndex = 0; // jen kod
            $combo->emptyRow = ' ';
            $combo->loadData($this ->database, $sql . ' ORDER BY Kod', $zaznam->rozdeleni, $params);
            $combo->htmlParams = ($this->readOnly ? ' disabled="disabled" ' : '') . " data-ron-select2='$varname' style='width:100%'";
            $combo->render();
        } else {
            // vyber rozdeleni podle popisu
            $combo = new ComboBox($varname);
            $combo->alwaysShowSelected = true;
            $combo->emptyRow = ' ';
            $combo->loadData($this ->database, $sql . ' ORDER BY Popis', $zaznam->rozdeleni, $params); // $keynum, $captionnum, $emptyrow, $selectedvalue);
            $combo->htmlParams = ($this->readOnly ? ' disabled="disabled" ' : '') . " data-ron-select2='{$varname}_kod' style='width:100%'";
            $combo->render();
        }
    }



    function zobrazSchvaleni($den)
    {
        if (!$this->schvaleni) return; // pri vkladani nemusi byt schvaleni nacteno

        $this->schvaleni->renderDiv($den);
    }


    /**
     * Nacte data pro Javascripty
     * @return bool
     */
    function getJsonData()
    {
        $jsonData = [ 'smeny' => [] ];

        $sql =
            'SELECT Kod, Popis, DenDochPodleKonceSmeny, PocatekSmeny, KonecSmeny '.
            ' FROM Smena' .
            ' WHERE Aktivni = 1' .
            ' ORDER BY Kod';

        $cursor = $this ->database ->getSqlResult($sql);
        while ( $row = $cursor ->fetch() ) {
            $jsonData['smeny'][] = [
                'kod' => StringUtils::jsonText($row[0], false),
                'popis' => StringUtils::jsonText($row[1], false),
                'denDochPodleKonceSmeny' => $row[2] ? 1 : 0
                // TODO: doplnit pocatek a konec smeny
            ];
        }

        return $jsonData;
    }


    /**
     * Zobrazi data pro Javascripty
     * @return bool
     */
    function showJsonData()
    {
        echo '<script type="text/javascript">';
        echo 'ronPortal.dochazka.editaceDochazky(' . json_encode($this ->getJsonData()) . ');';
        echo "</script>\n";

        return true;
    }


    public function render()
    {
        require('ronportal/editace/view_editace.php');
    }


    /**
     * @param string $action
     */
    function zobrazFormular($action)
    {
        $this ->action = $action;
        $this ->render();
    }


    // -----     Nacteni dat -------------------------------------------------
    /**
     * @param int $idpp
     * @param int $datumOd
     * @param int $datumDo
     * @param string $operace // kod operace, ktera se edituje
     * @return bool
     * @throws \Exception
     */
    function nastavReadOnly($idpp, $datumOd, $datumDo, $operace = null)
    {
        $this ->editacePovolena = false;
        $this ->pravaEditace = false;

        // overi prava a uzavreni obdobi
        $this->readOnly = true;

        // overi uzavreni obdobi atd.
        if (!RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_POVOLENA)) return true; // editace neni vubec zapnuta

        $this->obdobiUzavreno = Obdobi::jeObdobiUzavreno($this ->database, 'ADS', $datumOd, $datumDo);
        if (!$this->obdobiUzavreno) $this ->editacePovolena = true;

        // overi prava
        $prava = Prava::vratPrava('editace');
        if ( $prava ->lzeZobrazitPP($idpp, $datumOd, $datumDo) ) {
            // ma pravo na editaci osoby
            if (!isset($operace)  ||  EditaceUtils::jePravoEditovatOperaci($operace)) {
                // ma pravo na editaci teto operace
                if ($prava->priznak2 > 0) {
                    // kontrola maximalniho poctu dnu, ktere je mozne zpetne editovat
                    if (DateUtils::incDay($datumOd, $prava->priznak2) >= DateUtils::startOfTheDay(time())) {
                        $this->pravaEditace = true;
                    }
                } else {
                    // pocet dnu pro zpetnou editaci neni omezen
                    $this->pravaEditace = true;
                }
            }
        }


        $this ->readOnly = !($this ->editacePovolena  &&  $this ->pravaEditace);

        return $this ->readOnly;
    }


    /**
     * @param int $idpp
     * @param int $datumCasPocatku
     * @throws \Exception
     */
    function nactiData($idpp, $datumCasPocatku)
    {
        $this ->pp = PracovniPomer::nactiPP($this ->database, $idpp);

        $this ->zaznam = new ZaznamDochazky($this ->database, $this ->pp, $this);
        $params = $this ->database ->createParameters();
        if ( !$this ->zaznam ->loadRecord('Dochazka.DatumCasPocatku = ' . $params ->addParameter($this ->database ->timestampToParam($datumCasPocatku)), $params) ) {
            throw new \Exception('Chyba pri nacitani zaznamu dochazky -  zadany zaznam neexistuje!');
        }

        Prava::nastavObdobiProPrava($this ->zaznam ->fDatumCasPocatku ->value, $this ->zaznam ->fDatumCasUkonceni ->value);

        $this ->nastavReadOnly($idpp,
            $this->zaznam->fDatumCasPocatku ->value, $this->zaznam->fDatumCasUkonceni ->value, $this ->zaznam ->fOperace ->value);

        if ($this->readOnly) {
            // pokud nemuze editovat, tak jeste zkontroluje, jestli ma vubec pravo prohlizet
            $prava = Prava::vratPrava('prohlizeni');
            if ( !$prava ->lzeZobrazitPP($idpp,
                $this->zaznam->fDatumCasPocatku ->value, $this->zaznam->fDatumCasUkonceni ->value)
            ) {
                throw new NeniPravoException();
            }
        }

        $this ->zaznam ->nactiPoznamku();
        $this ->zaznam ->nactiRozdeleni();

        $this ->schvaleni = new SchvaleniDochazky();
        $this ->schvaleni ->load($this ->database,
            $this ->pp ->rc, $this ->pp ->osobniCislo, $this ->pp ->datumNastupu,
            $this ->zaznam ->fDatumCasPocatku ->value);

        if (!$this->schvaleni->lzeEditovatPP($idpp, $this ->zaznam ->fDatumCasPocatku ->value)) {
            $this ->readOnly = true;
        }
    }



    /**
     * @throws \Exception
     */
    function nactiDataProVlozeni()
    {
        $idpp = (int)$_GET['idpp'];
        $datum = $_GET['datum'];

        Prava::nastavObdobiProPrava($datum, $datum);

        $pp = PracovniPomer::nactiPP($this ->database, $idpp);

        $prava = Prava::vratPrava('editace');
        if (!$prava ->lzeZobrazitPP($pp ->idPP, $datum, $datum)) {
            throw new NeniPravoException();
        }

        $this ->zaznam = new ZaznamDochazky($this ->database, $pp, $this);

        $this ->schvaleni = new SchvaleniDochazky();
        $this ->schvaleni ->load($this ->database,
            $pp ->rc, $pp ->osobniCislo, $pp ->datumNastupu, $datum);

        if (!$this ->schvaleni ->lzeEditovatPP($idpp, $datum)) {
            throw new \Exception('Nelze vkladat zaznamy! Dochazka je jiz schvalena.');
        }

        $this ->zaznam ->fDatumCasPocatku ->setNewValue( $datum );
        $this ->zaznam ->fDen ->setNewValue( DateUtils::startOfTheDay($this ->zaznam ->fDatumCasPocatku ->value) );
        $this ->zaznam ->denStr = date('j.n.Y', $this ->zaznam ->fDen ->value);
    }


    /**
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $vlozitPrestavky
     */
    function prepocet($datumOd, $datumDo, $vlozitPrestavky)
    {
        if (!$datumDo) $datumDo = $datumOd;
        AgentManager::prepocetDochazky($this ->zaznam ->f_pp ->idPP, $datumOd, $datumDo, $vlozitPrestavky);
    }


    /**
     * @param bool $insert Urcuje, jestli se jedna o vlozeni noveho zaznamu
     * @return bool
     * @throws \Exception
     */
    function nactiDataPodlePost($insert)
    {
        $result = $this ->zaznam ->nactiDataPodlePost($insert, '', !RonPortalSettings::getIntValue(ZaznamDochazky::SETTINGS_SKRYT_ZAPOCTENY)); // $nacistZapoctenyCas = true

        if ($result && $insert && $this->zaznam->f_ViceDnu) {
            // zkontroluje, jestli neni datum do vetsi, nez zpracovani dochazky
            // nebo vetsi nez aktualni den (se zapocitanim pripadneho posunu nastaveneho uzivatelem)
            $zpracovanoDo = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);
            $maxDatumVkladani = DateUtils::startOfTheDay(time());

            if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADANI_DNU)) {
                $maxDatumVkladani = DateUtils::incDayStart($maxDatumVkladani, RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADANI_DNU));
            }

            if ($zpracovanoDo < $maxDatumVkladani) $maxDatumVkladani = $zpracovanoDo;

            if ($this ->zaznam ->f_DatumDo > $maxDatumVkladani) {
                $this ->zaznam ->chybaDatumDo = true;

                throw new \Exception( sprintf( TextManager::_('Datum_do_maximalne'), date('j.n.Y', $maxDatumVkladani)) );
            }
        }

        return $result;
    }


    /**
     * @return bool  Pokud doslo ke zmene, tak ulozi zaznam. V pripade chyby vraci false.
     * @throws \Exception
     */
    function save()
    {
        $datumCasPocatku = (int)$_POST['datumcaspocatku'];
        $idPP = (int)$_POST['idpp'];

        // nejdrive nacte puvodni data
        $this->nactiData($idPP, $datumCasPocatku);

        // nacte data zadana uzivatelem
        $result = $this ->nactiDataPodlePost(false);

        Csrf::validateCSRFPostParam('editacedochazky_' . $this->zaznam->f_pp->idPP . '_' . DateUtils::dateToUrlString($this->zaznam->fDatumCasPocatku->originalValue));

        if ($result) {
            $result = $this ->zaznam ->kontrolaPrekrytiZaznamuDochazky(false);  // insert = false
        }

        if ( RonPortalSettings::getIntValue(ZaznamDochazky::SETTINGS_PREPISOVAT) ) {
            $this ->zaznam ->zkratPrekryvaneZaznamy(false);
        }

        if ($result) {
            if ($this ->readOnly) {
                // prakticky by nemelo nastat
                throw new \Exception('Chyba! Data nelze editovat (obdobi je uzavreno, nebo nemate pravo pro editaci).');
            }

            $jeZmena = $this ->zaznam ->isChanged();

            if ($jeZmena) {
                EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $this ->zaznam['Den'], 'before');
            }

            $jeZmenaPoznamky = $this ->zaznam ->ulozPoznamku(false);  // $insert = false

            if ($this ->zaznam ->ulozRozdeleni(false)) {    // $insert = false
                $jeZmenaPoznamky = true;
            }

            if ($jeZmenaPoznamky) {
                // pokud doslo ke zmene poznamky nebo rozdeleni, tak vzdy nastavi "zpracoval"
                $this ->zaznam ->fZpracoval ->setUpdateLog(false); //
                $jeZmena = true;
            }

            $this ->zaznam ->loadedFilter = ''; // aby se predeslo pripadnym chybam, pokud se upravi filtr pro nacitani
            if ( $this ->zaznam ->ulozZmenenyZaznam() ) {
                $result = true;

                if ($jeZmena) {
                    if ( !$this ->zaznam ->executeXmlEvents(RonPortalXMLEvents::EDITACE_DOCHAZKY_AFTER, 'update') ) $result = false;

                    if (!$this ->zaznam ->executeScripts(RonPortalScriptEvents::EDITACE_DOCHAZKY_IW_AFTER, false, false)) $result = false;

                    MessageManager::addDebugLog('Provedu prepocet od ' . date('j.n.Y', $this ->zaznam ->fDatumCasPocatku ->value));
                    $this ->prepocet($this ->zaznam ->fDatumCasPocatku ->value, null,
                        isset($_POST['vlozitPrestavkyProDen']) && $_POST['vlozitPrestavkyProDen']);

                    EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $this ->zaznam['Den'], 'after');
                }
            } else {
                $result = false;
            }
        }

        if (!$result) {
            $this ->zaznam ->fDatumCasPocatku ->value = $this ->zaznam ->fDatumCasPocatku ->originalValue; // musi pouzit puvodni datum cas pocatku (je to primarni klic)
        }

        return $result;
    }


    /**
     * Nacte udaje zadane uzivatelem a vlozi novy zaznam
     * @return bool
     * @throws \Exception
     */
    function insert()
    {
        $result = false;

        $this ->pp = PracovniPomer::nactiPP($this ->database, (int)$_POST['idpp']);

        $this ->zaznam = new ZaznamDochazky($this ->database, $this ->pp, $this);

        $this-> zaznam-> fDatumCasPocatku ->setNewValue( (int)$_POST['datumcaspocatku'] );
        $this-> zaznam-> fDen ->setNewValue( $this ->zaznam->fDatumCasPocatku ->value ); // aby se spravne zobrazil seznam smen

        Csrf::validateCSRFPostParam('editacedochazky_' . $this->pp->idPP . '_' . DateUtils::dateToUrlString($this->zaznam->fDatumCasPocatku->value));

        if ($this->nactiDataPodlePost(true)) // $insert = true
        {
            if ($this->readOnly) {
                // prakticky by nemelo nastat
                throw new \Exception('Chyba! Data nelze editovat (obdobi je uzavreno, nebo nemate pravo pro editaci).');
            }

            if ($this ->zaznam ->f_ViceDnu) {
                //var $f_DatumDo; // pouze pri vkladani
                $datumod = $this ->zaznam->fDatumCasPocatku ->value;
                $datumdo = DateUtils::incDayStart($this ->zaznam ->f_DatumDo, 1);
                $index = 0;
                $result = true;

                while ($result  &&  DateUtils::incDay($this->zaznam->fDatumCasPocatku ->value, $index)  <  $datumdo) {
                    // znovu nacte POST aby se jednotlive zaznamy neovlivnovaly (doplneno 31.5.2018)
                    // (XML formular muze menit zaznam - napr. automaticka uprava deltaPred a deltaZa)
                    $novyZaznam = new ZaznamDochazky($this ->database, $this ->pp, $this);

                    $novyZaznam->fDatumCasPocatku ->setNewValue( (int)$_POST['datumcaspocatku'] );
                    $novyZaznam->fDen ->setNewValue( $this ->zaznam->fDatumCasPocatku ->value ); // aby se spravne zobrazil seznam smen
                    $novyZaznam->nactiDataPodlePost(true, '', !RonPortalSettings::getIntValue(ZaznamDochazky::SETTINGS_SKRYT_ZAPOCTENY)); // $nacistZapoctenyCas = true

                    if ($index) {
                        // cas od - do
                        $novyZaznam->fDatumCasPocatku ->setNewValue( DateUtils::incDay($this ->zaznam->fDatumCasPocatku ->value, $index) );
                        $novyZaznam->fDatumCasUkonceni ->setNewValue( DateUtils::incDay($this->zaznam->fDatumCasUkonceni ->value, $index) );
                        // den prirazeni
                        $novyZaznam->fDen ->setNewValue( DateUtils::incDay($this->zaznam->fDen ->value, $index) );
                    }

                    $result = $novyZaznam->vlozNovyZaznam();

                    $index++;

                    // znovu nacte POST (XML formular mohl zmenit zaznam - napr. automaticka uprava deltaPred a deltaZa)
//                    // pred nactenim musi nastavit puvodni datumod
//                    $this ->zaznam ->fDatumCasPocatku ->setNewValue( $datumod );
//                    $this ->zaznam ->nactiDataPodlePost(true, '', !RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_SKRYT_ZAPOCTENY)); // $nacistZapoctenyCas = true
//
//                    // cas od - do
//                    $this ->zaznam ->fDatumCasPocatku ->setNewValue( DateUtils::incDay($datumod, $index) );
//                    $this ->zaznam ->fDatumCasUkonceni ->setNewValue( DateUtils::incDay($this->zaznam->fDatumCasUkonceni ->value, $index) );
//                    // den prirazeni
//                    $this ->zaznam ->f_den ->setNewValue( DateUtils::incDay($this->zaznam->f_den ->value, $index) );
                }

                // prepocet provede i kdyz doslo k chybe - nektere zaznamy mohly byt vlozene
                $this ->prepocet($datumod, $datumdo,
                    RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADAT_PRESTAVKY)
                    || (isset($_POST['vlozitPrestavkyProDen']) && $_POST['vlozitPrestavkyProDen'])
                );

            } else {
                EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $this ->zaznam['Den'], 'before');

                // TODO: Predelat na vyjimku?
                if ( $this ->zaznam ->vlozNovyZaznam() ) {
                    $this ->prepocet($this ->zaznam ->fDatumCasPocatku ->value, null,
                        RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADAT_PRESTAVKY)
                        || (isset($_POST['vlozitPrestavkyProDen']) && $_POST['vlozitPrestavkyProDen']));

                    EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $this ->zaznam['Den'], 'after');

                    return true;
                }
            }
        }

        return $result;
    }


    /**
     * @param int $idPP
     * @param int $datumCasPocatku
     * @throws \Exception
     */
    function delete($idPP, $datumCasPocatku)
    {
        $this->nactiData($idPP, $datumCasPocatku);

        if ($this->readOnly) {
            throw new \Exception('Zaznam nelze smazat! (obdobi je uzavrene, nemate prava nebo je dochazka schvalena)');
        }

        $pp = PracovniPomer::nactiPP($this ->database, $idPP);

        EditaceUtils::editaceDochazkyEvent($this ->database, $pp, $this ->zaznam['Den'], 'before');

        $params = $this ->database ->createParameters();
        $this->zaznam->doDelete('DatumCasPocatku = ' . $params ->addParameter($this->database->timestampToParam($datumCasPocatku)), $params);
        $this->prepocet($this->zaznam->fDatumCasPocatku->value, null, false);

        EditaceUtils::editaceDochazkyEvent($this ->database, $pp, $this ->zaznam['Den'], 'after');
    }

}
