<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Editace;

use Ron\Controls\Dialog;
use Ron\Controls\DialogButton;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\MVC\Renderer;
use Ron\Profiler;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Log;
use RonPortal\Mzdy\MzdyUtils;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\RozdeleniUtils;
use RonPortal\System\AgentManager;
use RonPortal\User;
use Tracy\Debugger;

class EditaceMesicnichHromadna extends MesicniData // EditaceMesicnich
{
    public $zobrazitRozdeleni;
    public $raditPodleRozdeleni;

    /** @var int        jestli se ma zobrazit odkaz na zpracovanou dochazku (0=ne, 1=ve stejnem okne, 2=v novem pojmenovanem okne, 3=v novem okne) */
    public $odkazNaZpracovanou;

    /** @var int        kod typu rozdeleni, podle ktereho se (pripadne) seradi - lze nastavit sloupcem 'R' nebo 'RP' */
    public $hlavniTypRozdeleni;

    /** @var int        defaultne se v XML prava nekontroluji, v editaci podle nastaveni aplikace se edituji */
    public $kontrolovatPrava;

    /** @var int        jestli je zaskrtnuto "souhlasim s prekrocenim limitu"
     *                  (pouzije se jen v pripade ze formular obsahuje sloupec, ktery ma povolen prekroceni limit)
     */
    public $souhlasPrekrocitLimit;

    /** @var OsobaProEditaci[]  seznam osob - nacte se podle prav, nebo z XML */
    public $osoby;

    public $logMP; // zprava pro zapsani do logu - mzdove polozky
    public $logCS; //  -//-  - casove slozky

    /** @var bool   priznak, jestli je v datech chyba */
    public $jeChyba;
    /** @var bool   priznak, jestli formular obsahuje nejaky edit */
    public $jeEdit;

    /** @var bool       priznak, jestli je prekrocen neprekrocitelny limit */
    public $jeChybaNeprekrocitelnyLimit;
    /** @var string     text chybove hlasky pro neprekrocitelny limit; pokud neni nastaven, zobrazi se standardni */
    public $textChybyNeprekrocitelnyLimit;

    /** @var string     format pro mzdove polozky */
    public $formatMP;

    /** @var string     dodatecny filtr pro vyber osob */
    public $filtrOsoby;
    /** @var int        filtrovani podle rozdeleni 2 */
    public $typRozdeleni2;
    public $rozdeleni2;

    /** @var bool   priznak, jestli se maji zobrazit jen zamestnanci se zpracovavanou dochazkou, nebo vsichni */
    public $filtrovatZamestnance = true;
    /** @var bool   priznak, jestli se maji skryt duplicitni radky pro p. pomery, ktere jsou zobraze vicekrat */
    public $skrytDuplicitni = false;
    /** @var bool   jestli se ma po editaci hodnot provest prepocet dochazky (0=ne, 1=v pripade zmeny, 2=vzdy) */
    public $provestPrepocet;
    /** @var int    maximalni pocet osob pro volani prepoctu */
    public $maxPocetOsobProPrepocet;
    /** @var string */
    public $parametryAgentaPrepoctu;

    public $fixedCols = 2;

    /** @var int        Maximalni pocet zaznamu na strance - pokud je zadan, pokusi se strankovat (nemusi fungovat na vsech databazich!) */
    public $maxPageRows = null;

    public $exportRowDelimiter = "\r\n";
    public $exportColDelimiter = ';';


    /**
     * @param Database $database
     * @param int $typRozdeleni
     * @param string $rozdeleni
     */
    function __construct($database, $typRozdeleni, $rozdeleni)
    {
        //EditaceMesicnich::EditaceMesicnich();
        parent::__construct($database);

        $this ->osoby = [];
        $this ->f_TypRozdeleni = $typRozdeleni;
        $this ->f_Rozdeleni = $rozdeleni !== '' ? $rozdeleni : null;

        $this ->vyberPPVObdobi = 0;

        $this ->hlavniTypRozdeleni = 0;

        $this ->jeChyba = false;
        $this ->jeChybaNeprekrocitelnyLimit = false;

        $this ->formatMP = 'number2'; // 2 desetinna mista

        $this ->souhlasPrekrocitLimit = 0;

        $this ->provestPrepocet = false;
        $this ->maxPocetOsobProPrepocet = 20;
        $this ->parametryAgentaPrepoctu = '';

        $this ->odkazNaZpracovanou = 2; // 2 = v novem pojmenovanem okne

        $this ->kontrolovatPrava = null;
        $this ->raditPodleRozdeleni = null;
    }


    /**
     * ZASTARALE!  Nacte a zpracuje textovy retezec nastaveni - popis formatu - viz nahore
     * @param $nastaveni
     * @deprecated
     */
    function vytvorSloupcePodleNastaveni($nastaveni)
    {
        $this ->polozky = [];

        // zpracuje nastaveni ve formatu sloupec1;sloupec2;...
        $nastaveniarray = explode(';', $nastaveni);

        // projde vsechny sloupce
        foreach ($nastaveniarray as $nastavenisloupce) {
            if ($nastavenisloupce) {
                $sloupec = new PolozkaEditace();
                $sloupec ->nactiNastaveniZRetezce($nastavenisloupce);
                $this ->pridejPolozku($sloupec);
            }
        }
    }


    function processXMLAttribute($name, $value)
    {
        switch (strtolower($name)) {
            case 'datumpp' :
                $this->vyberPPVObdobi = (int)$value;
                break;

            case 'sloupce' : // stary zpusob nastaveni sloupcu
                $this->vytvorSloupcePodleNastaveni($value);
                break;

            case 'vcetnepodrizenych' :
                $this->vcetnePodrizenych = ($value === '') ? null : (int)$value;
                break;

            case 'prepocetdochazky' :
                $this->provestPrepocet = (int)$value;
                break;

            case 'proklikzpracovana' :
                $this->odkazNaZpracovanou = (int)$value;
                break;

            case 'kontrolovatprava' :
                $this ->kontrolovatPrava = (int)$value;
                break;

            case 'raditpodlerozdeleni' :
                $this ->raditPodleRozdeleni = (int)$value;
                break;

            case 'maxpocetosobproprepocet' :
                $this ->maxPocetOsobProPrepocet = (int)$value;
                break;

            case 'parametryagentaprepoctu' :
                $this ->parametryAgentaPrepoctu = $value;
                break;

            case 'filtrovatzamestnance' :
                $this ->filtrovatZamestnance = (int)$value;
                break;

            case 'skrytduplicitni' :
                $this ->skrytDuplicitni = (int)$value;
                break;

            case 'fixedcols' :
                $this ->fixedCols = (int)$value;
                break;

            case 'kontrolaobdobi' :
                $this ->zpusobKontrolyObdobi = strtolower($value);
                break;

            case 'kodaplikace' :
                $this ->kodAplikace = $value;
                break;

            case 'maxpagerows' :
                $this ->maxPageRows = (int)$value;
                break;

            default :
                parent::processXMLAttribute($name, $value);
                break;
        }
    }


    /**
     * @param string $name
     * @param array  $settingsArray
     */
    function loadChildNode($name, $settingsArray)
    {
        // TODO: umoznit konfiguraci filterformu?
        switch ($name) {
            case 'filtreditacemesicnich' :
                $filtr = $settingsArray['#text'];
                $this ->filtrOsoby = $filtr;
                break;

            case 'editacemesicnichlimit' :
                if (isset($settingsArray['#attr']['povolit'])) {
                    $this->souhlasPrekrocitLimit = (int)$settingsArray['#attr']['povolit'];
                }
                break;

            case 'textchybyneprekrocitelnylimit' :
                $this ->textChybyNeprekrocitelnyLimit = $settingsArray['#text'];
                break;

            default :
                parent::loadChildNode($name, $settingsArray);
        }
    }


    function doplnSloupceOsoby()
    {
        $this ->pridejPolozkuPodleNastaveni(
            ['typ' => 'JMENOPRIJMENI', 'nadpis' => TextManager::_('Jmeno'), 'class' => 'no-wrap'],
            true); // na zacatek

        $this ->pridejPolozkuPodleNastaveni(
            ['typ' => 'OSOBNICISLO', 'nadpis' => TextManager::_('Osobni_cislo'), 'class' => 'no-wrap'],
            true); // na zacatek
    }


    function loadSettings($settingsArray)
    {
        parent::loadSettings($settingsArray);

        $jeSloupecOsoby = false;

        // zjisti, jestli se ma radit podle rozdeleni a jestli existuje sloupec s osobou
        foreach ($this ->polozky as $sloupec) {
            if ($sloupec->typ == 'R') {
                $this->zobrazitRozdeleni = true;
                if (!isset($this ->raditPodleRozdeleni)) {
                    $this->raditPodleRozdeleni = true;
                }
            }

            if ($sloupec ->typ == 'OSOBA' || $sloupec ->typ == 'JMENO' || $sloupec ->typ == 'PRIJMENI' || $sloupec ->typ == 'JMENOPRIJMENI') {
                $jeSloupecOsoby = true;
            }
        }

        if (!$jeSloupecOsoby) {
            $this ->doplnSloupceOsoby();
        }
    }


    function nactiNastaveni($nastaveni)
    {
        if (!$nastaveni) {
            return false;
        }

        if (strpos($nastaveni, 'xml') === false) {
            // neobsahuje podretezec xml -> puvodni radkovy format
            $this ->vytvorSloupcePodleNastaveni($nastaveni);
            $this ->vcetnePodrizenych = null; // podle zadani uzivatele
        } else {
            // format xml
            $this ->loadXML($nastaveni);
        }

        return true;
    }


    function nastavViditelnostSloupcu()
    {
        foreach ($this ->polozky as $sloupec) {
            // skryje sloupce, na ktere uzivatel nema nastavene pravo
            if ($sloupec ->zobrazit && $sloupec->profil) {
                // vypne sloupec, pokud je pro jiny profil uzivatele
                $sloupec->zobrazit = User::jeVProfilu($sloupec ->profil);
            }

            if ($sloupec ->zobrazit) {
                // vypne sloupec, pokud uzivatel nema prislusna prava
                if ($sloupec ->prava  &&  !$sloupec ->prava ->jsouNejakePrava()) {
                    // uzivatel nema zadna prava na tento sloupec => vypne ho
                    $sloupec ->zobrazit = false;
                }
            }

            if ($sloupec ->zobrazit  &&  $sloupec ->condition) {
                $sloupec ->zobrazit = $sloupec ->evaluateCondition($this ->obdobi ->zacatek);
            }

            if ($sloupec ->typ == 'JMENOPRIJMENI'  &&  !isset($sloupec ->proklikZpracovana)) {
                $sloupec ->proklikZpracovana = $this ->odkazNaZpracovanou;
            }
        }
    }

    /**
     * Nacte limity jednotlivych sloupcu pro zadane rozdeleni z tabulky MzdPolSumaNaRozdeleni
     */
    function nactiLimityRozdeleni()
    {
        if (!$this ->f_TypRozdeleni  ||  !$this ->f_Rozdeleni) {
            return;
        }

        $typRozdeleni = (int)$this ->f_TypRozdeleni;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT MzdovaPolozka, Castka FROM MzdPolSumaNaRozdeleni ' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter((int)$typRozdeleni) .
            '   AND Rozdeleni = ' . $params->addParameter($this ->f_Rozdeleni) .
            '   AND Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek));

        $seznamOmezeni = [];
        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $seznamOmezeni[$row[0]] = StringUtils::stringToFloat($row[1]);
        }
        $cursor ->close();

        foreach ($this ->polozky as $sloupec) {
            if ($sloupec ->typ == 'MP') {
                //8.10.2019 oprava načítání limitu mzdové položky rozdělení (nenačítala se 0 - chovalo se to pak jako by nebyl limit)
                if (isset($seznamOmezeni[$sloupec ->kod])) {
                    $sloupec ->limitRozdeleni = $seznamOmezeni[$sloupec ->kod];
                }
            }
        }
    }


    /**
     * Nastavi hodnotu sloupce typu R a RP
     * @param OsobaProEditaci $osoba
     * @param integer $typRozdeleni
     * @param string $kod
     * @param string $popis
     */
    function nastavHlavniRozdeleni($osoba, $typRozdeleni, $kod, $popis)
    {
        foreach ($this ->polozky as $i => $polozka) {
            if ($polozka ->kodTypuRozdeleni == $typRozdeleni ||
                ($polozka ->kodTypuRozdeleni == 0 && $typRozdeleni == $this ->hlavniTypRozdeleni)
            ) {
                switch ($polozka ->typ) {
                    case 'R' :
                        $osoba ->f_data[$i] ->f_text = $kod;
                        break;

                    case 'RP' :
                        $osoba ->f_data[$i] ->f_text = $popis;
                        break;
                }
            }
        }
    }


    /**
     * @param int    $typRozdeleni  Pokud je zadan typ rozdeleni, pripojuje se k hlavnimu dotazu
     * @param string $kodRozdeleni  Rozdeleni, pro ktere se data odfiltruji. Nemusi byt zadane - v tom pripade rozdeleni nefiltruje, ale jen nacte aktualne prirazene
     * @param int    $idPP          Pro nacteni jednoho konkretniho PP
     * @param bool   $kontrolovatPrava
     * @param SqlStatement|string $filtr  Dodatecny filtr, napr. pro druhy typ rozdeleni
     * @return string
     */
    function vratSQL($typRozdeleni, $kodRozdeleni, $idPP = null, $kontrolovatPrava = false, $filtr = '')
    {
        $typRozdeleni = (int)$typRozdeleni;
        if ($typRozdeleni) {
            $this->hlavniTypRozdeleni = $typRozdeleni;
        } else {
            $this ->raditPodleRozdeleni = false; // nelze radit podle rozdeleni
        }

        if ( isset($idPP) ) {
            $paramsPP = $this ->database ->createParameters();
            $filtr = SqlStatement::concatenate(
                [
                    $filtr,
                    new SqlStatement('PracovniPomer.Id = ' . $paramsPP ->addParameter((int)$idPP), $paramsPP)
                ],
                SqlStatement::SEPARATOR_AND
            );

            $filtr_rozdeleni = null; // jen konkretni PP - nefiltruje rozdeleni
        } else {
            $filtr_rozdeleni = $kodRozdeleni; // filtruje zadane rozdeleni
        }

        if ($this ->filtrOsoby) {
            $filtr = SqlStatement::concatenate(
                [$filtr, $this ->filtrOsoby],
                SqlStatement::SEPARATOR_AND
            );
        }

        $params = $this ->database ->createParameters();

        $zakladniSql =  new SqlStatement(
            OsobaProEditaci::vratZakladniSQL($this ->database,
                $this ->obdobi, $this ->vyberPPVObdobi,
                $typRozdeleni, $filtr_rozdeleni, $this ->vcetnePodrizenych,
                !$this ->filtrovatZamestnance, $params),
            $params
        );

        $pripojitVlastni = false;

        $filtrPrava = '';

        if (isset($this->pravaZobrazeni) || $kontrolovatPrava) {
            $prava = $this ->pravaZobrazeni ?: [$this ->f_pravaMP, $this ->f_pravaMV];

            $pripojitVlastni = Prava::pravaVcetneSebe($prava);

            // seznam osob defaultne nacte podle prav pro zobrazeni MP a MV, ale muze to byt predefinovano nastavenim jineho (napr. uzivatelskeho) prava
            $paramsPrava = $this ->database ->createParameters();
            $filtrPrava = new SqlStatement(
                Prava::sqlFiltrPodlePrav( $prava, 'PracovniPomer',
                    $this ->obdobi ->zacatek, $this ->obdobi ->konec,
                    true, $paramsPrava
                ),
                $paramsPrava);
        }

        $sql = SqlStatement::concatenate(
            [$zakladniSql, $filtr, $filtrPrava],
            SqlStatement::SEPARATOR_AND);

//        $sql =
//            $zakladniSql .
//            ($filtr ? "\nAND ($filtr)" : '') .
//            ($filtrPrava ? "\nAND ($filtrPrava)" : '');

        if ($pripojitVlastni) {
            $paramsRC = $this ->database ->createParameters();
            $filtrRC = 'PracovniPomer.RC = ' . $paramsRC ->addParameter(User::getRC());

            $sql2 = SqlStatement::concatenate(
                [$zakladniSql, $filtr, new SqlStatement($filtrRC, $paramsRC)],
                SqlStatement::SEPARATOR_AND
            );

            $sql = SqlStatement::concatenate(
                [$sql, "UNION", $sql2]
            );

//            $sql .=
//                "\nUNION\n" .
//                $zakladniSql .
//                ($filtr ? "\nAND ($filtr)" : '') .
//                "\nAND PracovniPomer.RC = '" . $this ->database ->addSlashes(User::getRC()) . "'\n";
        }

        // order by
        if ($this ->database ->getDbType() == 'interbase') {
            $sql = SqlStatement::concatenate([
                $sql,
                'ORDER BY ' . ($this->raditPodleRozdeleni ? ' 7, ' : '') . ' 5, 4'
            ]);
        } else {
            $sql = SqlStatement::concatenate([
                $sql,
                'ORDER BY ' . ($this->raditPodleRozdeleni ? ' RozdeleniOsob.Rozdeleni, ' : '') . ' Prijmeni, Jmeno '
            ]);
        }

        return $sql;
    }


    /**
     * Spocita soucty pro vsechny sloupce
     * @return void
     */
    function spocitejSoucty()
    {
        foreach ($this ->polozky as $i => $polozka) {
            $polozka ->soucet = 0;

            foreach ($this ->osoby as $osoba) {
                $polozka ->soucet += (float)$osoba ->f_data[$i] ->f_hodnota;
            }
        }
    }


    /**
     * Porovna data zadanych osob v zadanem sloupci.
     * @param OsobaProEditaci $osoba1
     * @param OsobaProEditaci $osoba2
     * @param int $indexSloupce
     * @return int
     */
    function compareData($osoba1, $osoba2, $indexSloupce)
    {
        switch ($this ->polozky[$indexSloupce] ->typ) {
            case 'NASTUP' :
                if ($osoba1 ->datumNastupu < $osoba2 ->datumNastupu) {
                    return -1;
                }
                if ($osoba1 ->datumNastupu > $osoba2 ->datumNastupu) {
                    return 1;
                }
                return 0;

            case 'UKONCENI' :
                if ($osoba1 ->datumUkonceni < $osoba2 ->datumUkonceni) {
                    return -1;
                }
                if ($osoba1 ->datumUkonceni > $osoba2 ->datumUkonceni) {
                    return 1;
                }
                return 0;

            case 'MV' :
            case 'MP' :
                if ($osoba1 ->f_data[$indexSloupce] ->f_hodnota < $osoba2 ->f_data[$indexSloupce] ->f_hodnota) {
                    return -1;
                }
                if ($osoba1 ->f_data[$indexSloupce] ->f_hodnota > $osoba2 ->f_data[$indexSloupce] ->f_hodnota) {
                    return 1;
                }
                return 0;

            case 'R' :
            case 'RP' :
            case 'ROZD' :
            case 'ROZDP' :
            default :
                return StringUtils::compareStrings($osoba1 ->f_data[$indexSloupce] ->f_text, $osoba2 ->f_data[$indexSloupce] ->f_text);
        }
    }

    /**
     * Zjisti, jestli se ma podle nekterych sloupcu seznam seradit a pokud ano, tak to provede.
     */
    function seradData()
    {
        // vytvori seznam indexu sloupcu, podle kterych se radi
        /** @var int[] $sloupce */
        $sloupce = [];
        foreach ($this ->polozky as $index => $sloupec) {
            if ($sloupec ->razeni) {
                $sloupce[$sloupec ->razeni] = $index;
            }
        }

        if (!$sloupce) {
            return;
        }


        usort($this ->osoby, function($item1, $item2) use ($sloupce) {
            /** @var OsobaProEditaci $item1 */
            /** @var OsobaProEditaci $item2 */
            foreach ($sloupce as $indexSloupce) {
                $result = $this ->compareData($item1, $item2, $indexSloupce);
                if ($result) {
                    return $result;
                }
            }

            // podle prijmeni a jmena
            $result = StringUtils::compareStrings($item1 ->prijmeni, $item2 ->prijmeni);
            if ($result) {
                return $result;
            }

            return StringUtils::compareStrings($item1 ->jmeno, $item2 ->jmeno);
        });
    }


    /**
     * @param SqlStatement|string $sql
     * @param bool $kontrolovatPrava
     */
    function nactiSeznamOsobSQL($sql, $kontrolovatPrava)
    {
        // nastaveni prav
        switch ($this ->vyberPPVObdobi) {
            case 1 : // zacatek obdobi
                $datumOd = $this ->obdobi ->zacatek;
                $datumDo = $this ->obdobi ->zacatek;
                break;

            case 2 : // konec obdobi
                $datumOd = $this ->obdobi ->konec;
                $datumDo = $this ->obdobi ->konec;
                break;

            default : // kdykoliv behem obdobi
                if ($this ->vyberPPVObdobi < -1) {
                    // kdykoliv behem obdobi a zadaneho poctu predchozich mesicu (napr. -3 je ctvrtleti koncici zadanym obdobim)
                    $datumOd = DateUtils::incMonth($this ->obdobi->zacatek, $this ->vyberPPVObdobi + 1);
                    $datumDo = $this ->obdobi->konec;
                } else {
                    // kdykoliv behem obdobi
                    $datumOd = $this ->obdobi ->zacatek;
                    $datumDo = $this ->obdobi ->konec;
                }

                break;
        }

        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        $nacteneOsoby = []; // seznam pro kontrolu duplicitnich PP

        /**
         * @param OsobaProEditaci $osoba
         * @return bool
         * @throws \Exception
         */
        $filterFunc = function($osoba) use ($datumOd, $datumDo, $kontrolovatPrava, &$nacteneOsoby) { // $nacteneOsoby by reference!
            if ($this ->skrytDuplicitni) {
                if (isset($nacteneOsoby[$osoba ->idPP]) &&  $nacteneOsoby[$osoba ->idPP]) {
                    // osoba je uz zobrazena
                    return false;
                } else {
                    $nacteneOsoby[$osoba ->idPP] = true;
                }

            }

            $osoba ->f_PravoEditovatMP = $this ->f_pravaMPEdit ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
            $osoba ->f_PravoEditovatMV = $this ->f_pravaMVEdit ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);

            $osoba ->f_PravoZobrazitMV = $osoba ->f_PravoEditovatMV || $this ->f_pravaMV ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
            $osoba ->f_PravoZobrazitMP = $osoba ->f_PravoEditovatMP || $this ->f_pravaMP ->lzeZobrazitPP($osoba ->idPP, $datumOd, $datumDo);
            ///return true;
            /// POZOR, pouziva se i v XML formularich, pokud je v XML nastaveno kontrolovat prava a prirazene vlastni pravo pro nacteni seznamu osob
            /// muze dojit k situaci, kdy zamestnanec je dle prav nacten, ale nasledne odfiltrovan a nezobrazen, protoze uzivatel nema pravo zobrazit jeho MV nebo MP
            return !$kontrolovatPrava || $osoba ->f_PravoZobrazitMV || $osoba ->f_PravoZobrazitMP;
        };

        $this->osoby = OsobaProEditaci::nactiSeznamOsobSQL($this ->database, $sql, $filterFunc);

        if ($this ->maxPageRows  &&  count($this ->osoby) > $this ->maxPageRows) {
            array_splice($this ->osoby, $this ->maxPageRows);

            MessageManager::addWarning(TextManager::_('Seznam_prehledu_zpracovane_omezen'));
        }

        $nactenePP = [];
        foreach ($this ->osoby as $osoba) {
            $this ->nactiDataOsoby($osoba);

            if (in_array($osoba ->idPP, $nactenePP)) {
                $osoba ->duplicitniZaznam = true;
            } else {
                $nactenePP[] = $osoba ->idPP;
            }

            if ($this ->zobrazitRozdeleni) { // musi byt az po nactiDataOsoby!
                $this ->nastavHlavniRozdeleni($osoba, $this ->hlavniTypRozdeleni, $osoba ->kodRozdeleni, $osoba ->popisRozdeleni);
            }
        }

        // spocita soucty za vsechny osoby
        $this ->spocitejSoucty();

        // provede dodatecne serazeni (pokud je u nektereho sloupce razeni zapnuto)
        $this ->seradData();
    }


    function nactiSeznamOsobPodleRozdeleni($typRozdeleni, $kodRozdeleni)
    {
        $kontrolovatPrava = isset($this ->kontrolovatPrava) ? $this ->kontrolovatPrava : false; // v XML defaultne prava nekontroluje
        $this ->nastavPrava(!$kontrolovatPrava); // povolitvse = true

        $this ->nactiLimityRozdeleni();

        if ( $this ->typRozdeleni2  &&  $this ->rozdeleni2 ) {
            // filtr podle rozdeleni 2
            $filtr2 = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
                $this ->typRozdeleni2,  $this ->rozdeleni2,
                $this ->obdobi ->zacatek, $this ->obdobi ->konec, true); // vcetnepodrizenych = true
        } else {
            $filtr2 = '';
        }

        $sql = $this ->vratSQL($typRozdeleni, $kodRozdeleni, null, $kontrolovatPrava, $filtr2);
        $this ->nactiSeznamOsobSQL($sql, $kontrolovatPrava); // kontrolovatprava = false - zobrazi vsechny nactene osoby
    }


    /**
     * @param RonPortalFilterForm $filterForm
     */
    function nactiSeznamOsobPodlePrav(RonPortalFilterForm $filterForm)
    {
        $this ->nastavObdobi($filterForm ->getFilterValue('obdobi'));
        $this ->nastavViditelnostSloupcu();

        $kontrolovatPrava = $this->kontrolovatPrava ?? true;
        $this ->nastavPrava(!$kontrolovatPrava);

        $params2 = $this ->database ->createParameters();
        $filtr2 = $filterForm ->buildSqlFilter($params2);

        foreach ($filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->name != 'rozdeleni'  &&  $rozdeleniFilter ->getValue() !== null) { // rozdeleni1 vynecha - resi to samostatne, protoze se pridava primo do dotazu
                $rozdeleniSql = $rozdeleniFilter ->sestavSQLFiltr($this ->database,'PracovniPomer', $params2);

                if ($rozdeleniSql) {
                    $filtr2 .=
                        ($filtr2 ? "\nAND " : '') .
                        '(' . $rozdeleniSql . ')';
                }
            }
        }

        if ($filtr2) {
            $filtr2 = new SqlStatement($filtr2, $params2);
        }

//        if (RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)  &&  RonPortalSession::rozdeleni2()) {
//            // filtr podle rozdeleni 2
//            $filtr2 = RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'PracovniPomer',
//                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP),  RonPortalSession::rozdeleni2(),
//                $this ->obdobi ->zacatek, $this ->obdobi ->konec, RonPortalSession::rozdeleni2VcetnePodrizenych()); // vcetnePodrizenych = true
//        } else {
//            $filtr2 = '';
//        }

        // TODO: OPRAVIT PRO ROZDELENI S KODEM 0
        if ($this ->f_TypRozdeleni && (isset($this ->f_Rozdeleni) || $this ->zobrazitRozdeleni)) {
            // je zadan filtr na rozdeleni
            $sql = $this ->vratSQL($this ->f_TypRozdeleni, $this ->f_Rozdeleni, null, $kontrolovatPrava, $filtr2); // kontrolovat prava = true

            if (isset($this ->f_Rozdeleni)) {
                $this ->nactiLimityRozdeleni();
            }
        } else {
            $sql = $this ->vratSQL(0, '', null, $kontrolovatPrava, $filtr2); // vsechny osoby bez rozdeleni
        }

        $this ->nactiSeznamOsobSQL($sql, true); // kontrolovatprava = true - zobrazi jen osoby, ktere muze uzivatel editovat
    }



    /**
     * Nacte vsechny osoby, ktere jsou v danem obdobi na zadanem rozdeleni
     * Standardne nekontroluje prava, ale da se to zapnout parametrem kontrolovatprava="1"
     * @param int    $typRozdeleni
     * @param string $kodRozdeleni
     * @param int $obdobi
     */
    function nactiSeznamOsobXML($typRozdeleni, $kodRozdeleni, $obdobi)
    {
        $this ->nastavObdobi($obdobi);
        $this ->nastavPrava(false); // povolitvse = false - urci se podle prav uzivatele

        $this ->nastavViditelnostSloupcu();

        if (!$typRozdeleni) {
            $typRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP);
        }

        $this ->nactiSeznamOsobPodleRozdeleni($typRozdeleni, $kodRozdeleni);
    }


    /**
     * @param $osoba OsobaProEditaci
     * @return bool
     */
    function nactiDataOsobyPodlePost($osoba)
    {
        $result = true;

        // Dale doplni uzivatelem zadane hodnoty
        foreach ($this ->polozky as $i => $sloupec) {
            $readonly = ($this ->f_readonly || $this ->uzamknoutPolozku($sloupec, $osoba)); // uzamykani podle schvaleni

            $rozdeleniName = ($sloupec ->rozlisitRozdeleni == 2) ? '_' . $osoba ->kodRozdeleni : '';

            // pokud neni sloupec zobrazeny (napr. v pripade, ze je pro jiny profil), tak nelze ani editovat!
            if (!$readonly  &&  $sloupec ->zobrazit  &&  ($sloupec ->lzeEditovat == 2  ||  isset($_POST["{$this ->prefixHTML}{$osoba ->idPP}{$rozdeleniName}_$i"]))  &&  $this ->lzeEditovatSloupec($sloupec, $osoba)) {
                $chyba = false;
                $hodnota = null;

                if ($sloupec ->lzeEditovat == 2) {
                    $hodnota = (isset($_POST["{$this ->prefixHTML}{$osoba ->idPP}{$rozdeleniName}_$i"]) && $_POST["{$this ->prefixHTML}{$osoba ->idPP}{$rozdeleniName}_$i"])
                        ? 1
                        : 0;
                    $text = $hodnota;
                } else {
                    $text = trim($_POST["{$this ->prefixHTML}{$osoba ->idPP}{$rozdeleniName}_$i"]);


                    if ($text == '') {
                        // prazdny retezec => hodnota = 0
                        $hodnota = null;
                    } else if ($sloupec->isTimeFormat()) {
                        try {
                            $hodnota = DateUtils::timeStringToFloat($text);
                        } catch (\Exception $e) {
                            $chyba = true;
                        }
                    } else if ($sloupec->isNumberFormat()) {
                        if (StringUtils::isFloat($text)) {
                            $hodnota = StringUtils::stringToFloat($text);
                        } else {
                            $chyba = true;
                        }
                    } else {
                        $hodnota = $text;
                    }
                }

                if ($chyba) {
                    // chyba - ponecha puvodni hodnotu (nactenou z databaze), a text zadany uzivatelem
                    // a oznaci pole jako chybne
                    $osoba ->f_data[$i] ->f_chyba = true;
                    $osoba ->f_data[$i] ->f_text = $text;
                    $result = false;
                } else {
                    // OK - ulozi uzivatelem zaznamenanou hodnotu a k ni odpovidajici text
                    $osoba ->f_data[$i] ->f_hodnota = $hodnota;
                    $osoba ->f_data[$i] ->f_text = $sloupec ->vratTextProHodnotu($hodnota);
                }
            }
        }

        return $result;
    }


    /**
     * @param int $index
     * @return bool
     * @deprecated
     */
    private function nactiOsobuPodlePost($index)
    {
        // TODO: vyresit moznost nacist vice rozdeleni jedne osoby !
        $idpp = (int)$_POST[$this ->prefixHTML . "IdPP_$index"];

        $typRozdeleni = ($this ->f_TypRozdeleni && $this ->zobrazitRozdeleni)
            ? (int)$this ->f_TypRozdeleni
            : null;

        $sql = $this ->vratSQL($typRozdeleni, null, $idpp);
        //MessageManager::addDebugLog("Nacitam osobu $idpp: ".$sql);

        $cursor = $this ->database ->getSqlResult($sql);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return MessageManager::addErrorMessage('Chyba - pracovni pomer nebyl nalezen! (pracovnipomer.id = ' . $idpp . ')');
        }

        $osoba = $this ->vytvorOsobuPodleRadku($row);
        $this ->osoby[] = $osoba;

        // Nejdrive nacte puvodni data z databaze
        // - jiny uzivatel mohl provest zmenu, takze aby byl stav konzistentni,
        //   tak puvodni hodnoty urci podle aktualnich hodnot v databazi
        // - zaroven nacita pripadne limity jednotlivych polozek
        $this ->nactiDataOsoby($osoba);
        $result = $this ->nactiDataOsobyPodlePost($osoba);

        if ($this ->zobrazitRozdeleni) { // musi byt az po nactidataosoby
            $this ->nastavHlavniRozdeleni($osoba, $this ->hlavniTypRozdeleni, $row[7], $row[8]);
        }

        return $result;
    }


    /**
     * @param int $obdobi
     * @param bool  $povolitVse  urcuje, jestli se maji nastavit prava podle uzivatele (false)
     *              nebo muze editovat vsechny nactene osoby (true - pouzije se v XML)
     * @return bool
     * @throws \Exception
     * @deprecated      Pouziva se jeste ve starych XML
     */
    function nactiSeznamOsobPodlePost($obdobi, $povolitVse)
    {
        // POZOR! POUZIVA SE POUZE VE STARYCH XML
        $this ->nastavObdobi($obdobi);
        $this ->nastavPrava($povolitVse);

        $this ->nastavViditelnostSloupcu();

        foreach ($this ->polozky as $sloupec) {
            if ($sloupec ->rozlisitRozdeleni == 2) {
                throw new \Exception('Ve starych XML formularich neni rozliseni rozdeleni podporovano!');
            }
        }


        // zaskrtavatko "souhlasim s prekrocenim limitu"
        if (isset($_POST[$this ->prefixHTML . 'povolitprekrocenilimitu'])) {
            $this ->souhlasPrekrocitLimit = $_POST[$this ->prefixHTML . 'povolitprekrocenilimitu'];
        } else {
            $this ->souhlasPrekrocitLimit = 0;
        }

        // nacte postnute data osob
        $cnt = $_POST[$this ->prefixHTML . 'POCET_RADKU'];

        if (!$cnt) {
            throw new \Exception('Chyba - nebyla odeslana zadna data!');
        }

        for ($i = 0; $i < $cnt; $i++) {
            $OK = $this ->nactiOsobuPodlePost($i);

            if (!$OK) {
                $this ->jeChyba = true;
            }
        }

        $this ->spocitejSoucty();

        return true;
    }


    function nactiPost()
    {
        //var_dump($_POST);
        $this ->souhlasPrekrocitLimit = isset($_POST[$this ->prefixHTML . 'povolitprekrocenilimitu'])
            ? $_POST[$this ->prefixHTML . 'povolitprekrocenilimitu']
            : 0;

        // nacte postnute data osob
        $cnt = $_POST[$this ->prefixHTML . 'POCET_RADKU'];

        if (!$cnt) {
            throw new \Exception('Chyba - nebyla odeslana zadna data!');
        }

        foreach ($this ->osoby as $osoba) {
            $OK = $this ->nactiDataOsobyPodlePost($osoba);

            if (!$OK) {
                $this ->jeChyba = true;
            }
        }

        $this ->spocitejSoucty();
    }


    /**
     * vraci textovy seznam sloupcu (nadpisy), ktere maji povoleno prekrocit limit
     * @return string
     */
    function zobrazitPovolitPrekroceni()
    {
        $result = '';

        foreach ($this ->polozky as $sloupec) {
            //if ( $sloupec -> f_Limit > 0 )
            if ($sloupec ->kodLimitMaximum || isset($sloupec ->maximum)) {
                if ($sloupec ->povolitPrekroceni) {
                    $result .=
                        ($result ? ', ' : '') .
                        $sloupec ->nadpis;
                }
            }
        }

        return $result;
    }


    /**
     * @param PolozkaEditace $sloupec
     * @param OsobaProEditaci $osoba
     * @return bool
     */
    function lzeEditovatSloupec($sloupec, $osoba)
    {
        if (!$sloupec ->lzeEditovat  ||  !$sloupec ->zobrazit) {
            return false;
        }

        return $this ->jePravoNaPolozku($sloupec, $osoba);
    }


    /**
     * @param PolozkaEditace $sloupec
     * @param string $textValue
     * @return string
     */
    protected function vratHtmlOptions($sloupec, $textValue)
    {
        $result = '';

        foreach ($sloupec ->options as $option) {
            $result .=
                "<option value='$option'" .
                ($option === $textValue ? ' selected="selected"' : '') .
                '>' .
                ($option === '' ? ' ' : $option) .
                '</option>';
        }

        return $result;
    }


    protected function getColumnClass($sloupec, $defaultClass)
    {
        $result = '';
        switch (strtolower($sloupec ->align ?: '')) {
            case 'left' :
                $result = 'align-left';
                break;

            case 'right' :
                $result = 'align-right';
                break;

            case 'center' :
                $result = 'align-center';
                break;
        }

        $result .= ($result ? ' ' : '') . ($defaultClass ?: '');

        if ($sloupec ->typ == 'MP' || $sloupec ->typ == 'MV') {
            $result = $result ?: 'align-right';
        }

        return $result;
    }

    /**
     * @param $sloupec PolozkaEditace
     * @param $i int
     * @param $osoba OsobaProEditaci
     * @return string
     */
    function vratHTMLTextSloupce($sloupec, $i, $osoba)
    {
        if (!$sloupec ->zobrazit) {
            return '';
        }

        if ($sloupec ->export && $sloupec ->export != 'html') {
            return '';
        }

        $result = '';

        $text = $osoba ->f_data[$i] ->f_text;

        $class = $this ->getColumnClass($sloupec, $sloupec ->cssClass);

        if ($sloupec ->typ == 'MP' || $sloupec ->typ == 'MV') {
            if ($osoba ->f_data[$i] ->f_hodnota < 0) {
                $class .= ' minus';
            }
        }

        $td_class = $class ? 'class="' . $class . '"' : '';

        $hintstr = ($osoba ->f_data[$i] ->f_hint)
            ? "title='" . StringUtils::htmlTextConvert($osoba ->f_data[$i] ->f_hint) . "'"
            : '';

        $td_styl = $sloupec ->styl
            ? "style='{$sloupec ->styl}'"
            : '';

        $result .= "<td $td_styl $td_class $hintstr>";

        $lzeEditovat = $this ->lzeEditovatSloupec($sloupec, $osoba); // kontrola prav na editaci

        if (!$lzeEditovat) {
            if ($sloupec ->prava) {
                $lzeZobrazit = $sloupec ->prava ->lzeZobrazitPP($osoba ->idPP);
            } else {
                $lzeZobrazit = true;
                if ($sloupec->typ == 'MP' || $sloupec->typ == 'PMP') {
                    $lzeZobrazit = $osoba->f_PravoZobrazitMP;
                }
                if ($sloupec->typ == 'MV' || $sloupec->typ == 'PMV') {
                    $lzeZobrazit = $osoba->f_PravoZobrazitMV;
                }
            }
        } else {
            $lzeZobrazit = true;
        }


        if ($lzeEditovat) {
            $classname = '';
            $maxlenstr = "maxlength='10'";

            $data = '';

            if (!$osoba ->duplicitniZaznam  ||  $sloupec ->rozlisitRozdeleni == 2) {
                if ($sloupec ->lzeEditovat == 2) {
                    // checkbox
                    $classname .= ' editaceCheck';
                } else if ($sloupec->typ == 'PMP') {
                    // poznamka mzdove polozky
                    $classname .= " poznamkaMP editacePoznamky";
                    $maxlenstr = '';
                } else if ($sloupec->typ == 'PMV') {
                    // poznamka mesicniho vysledku
                    $classname .= " poznamkaMV editacePoznamky";
                    $maxlenstr = '';
                } else if ($sloupec->typ == 'MP') {
                    $classname .= " inputFloat editaceMP";
                    $data = "data-ron-index='{$i}'";
                } else if ($sloupec->isTimeFormat()) {
                    $classname .= " inputTime editaceMV";
                    $data = "data-ron-index='{$i}'";
                } else {
                    $classname .= " inputFloat editaceMV";
                    $data = "data-ron-index='{$i}'";
                }

                if ($sloupec->isTimeFormat() || $sloupec->isNumberFormat()) {
                    if ($osoba->f_data[$i]->f_hodnota < 0) {
                        $classname .= ' minus';
                    }
                }

                // chybu zvyrazni cervene
                if ($osoba->f_data[$i]->f_chyba) {
                    $classname .= ' error';
                }

                // pokud je hodnota editovana (puvodni hodnota z DB je jina nez hodnota zadana uzivatelem), zobrazi ji tucne
                if ($osoba->f_data[$i]->f_puvodniText != $osoba->f_data[$i]->f_text) {
                    $classname .= ' editovanyzaznam';
                }

                // kontrola readonly a schvaleni dochazky
                $readonlystr = ($this->f_readonly || $this->uzamknoutPolozku($sloupec, $osoba))
                    ? "readonly='readonly'"
                    : '';

                $rozdeleniName = ($sloupec->rozlisitRozdeleni == 2)
                    ? '_' . StringUtils::htmlTextConvert($osoba->kodRozdeleni)
                    : '';

                if ($sloupec ->lzeEditovat == 2) {
                    // checkbox
                    $checkedStr = $osoba->f_data[$i]->f_hodnota ? ' checked="checked"' : '';
                    $result .= "<input type='checkbox' class='$classname' name='{$this ->prefixHTML}{$osoba ->idPP}{$rozdeleniName}_$i' $checkedStr value='1' $readonlystr $data/>";
                } else if ($sloupec ->options) {
                    $result .=
                        "<select class='$classname' name='{$this ->prefixHTML}{$osoba ->idPP}{$rozdeleniName}_$i' $readonlystr $maxlenstr $data data-value-text='$text'/>" .
                        $this ->vratHtmlOptions($sloupec, $text) .
                        '</select>';
                } else {
                    $result .= "<input type='text' class='$classname' name='{$this ->prefixHTML}{$osoba ->idPP}{$rozdeleniName}_$i' value='" . StringUtils::htmlTextConvert($text) . "' $readonlystr $maxlenstr $data/>";
                }

                $this->jeEdit = true;
            }

        } else if ($lzeZobrazit) {
            if ( $sloupec ->lzeEditovat &&  ($sloupec ->isTimeFormat() || $sloupec ->isNumberFormat()) ) {
                // vlozi hidden pole, aby fungovaly javascripty pro prepocet souctu
                //   (protoze uzivatel sice nema prava na tuto osobu, ale na jine muze mit prava editace)
                // (od 29.11.2014 - pokud neni sloupec editovatelny, tak input nevklada, aby zbytecne nedoslo k prekroceni maximalniho poctu inputu)
                $result .= "<input type='hidden' name='{$this ->prefixHTML}{$osoba ->idPP}_$i' value='" . $text . "' />";
            }

            $result .= $this ->vratHTMLText($osoba, $i);
        }

        $result .= '</td>';

        return $result;
    }


    /**
     * @param $sloupec PolozkaEditace
     * @param $i int
     * @param $osoba OsobaProEditaci
     * @return string
     */
    function vratTextSloupce($sloupec, $i, $osoba)
    {
        if (!$sloupec ->zobrazit) {
            return '';
        }

        if ($sloupec ->export  &&  $sloupec ->export !== 'export') {
            return '';
        }

        $result = '';

        $lzeeditovat = $this ->lzeEditovatSloupec($sloupec, $osoba); // kontrola prav na editaci

        if (!$lzeeditovat) {
            $lzezobrazit = true;
            if ($sloupec ->typ == 'MP'  ||  $sloupec ->typ == 'PMP') {
                $lzezobrazit = $osoba ->f_PravoZobrazitMP;
            }
            if ($sloupec ->typ == 'MV'  ||  $sloupec ->typ == 'PMV') {
                $lzezobrazit = $osoba ->f_PravoZobrazitMV;
            }
        } else {
            $lzezobrazit = true;
        }


        if ($lzezobrazit) {
            $result .= $this ->vratHTMLText($osoba, $i, false);
        }

        $result .= $this ->exportColDelimiter;

        return $result;
    }


    /**
     * @param int $index
     * @param OsobaProEditaci $osoba
     * @return string
     */
    function vratRadekOsoby($index, $osoba)
    {
        $classStr = $osoba ->rowClass;
        if ($index == count($this ->osoby) - 1) {
            $classStr .= ($classStr ? ' ' : '') . 'border-bottom';
        }

        //$classStr = $classStr ? "class='$classStr'" : '';

        $result = "<tr class='focusToggle $classStr' data-focus-toggle-class='active-row' data-ron-idpp='{$osoba ->idPP}'>";

        // hodnoty sloupcu
        foreach ($this ->polozky as $i => $sloupec) {
            $result .= $this ->vratHTMLTextSloupce($sloupec, $i, $osoba);
        }

        $result .= "</tr>\n";

        return $result;
    }


    /**
     * Vraci souctovy radek pro zobrazeni v HTML nebo pro export
     * @param bool $html    Urcuje, jestli se jedna o html zobrazeni
     * @return string
     */
    private function vratRadekSoucet($html)
    {
        $result = '';

        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec ->zobrazit) {
                $exportovat = $sloupec ->export // zobrazeni sloupce je omezeno pouze pro html/export
                    ? ($html
                        ? $sloupec ->export === 'html'
                        : $sloupec ->export === 'export')
                    : true;

                if ($exportovat) {
                    if ($sloupec->zobrazitSoucet) {
                        switch ($sloupec->typ) {
                            case 'MP' :
                            case 'MV' :
                                $text = $sloupec->vratTextProHodnotu($sloupec->soucet);
                                break;

                            case 'SQL' :
                                $text = DBUtils::getFormatedValue($this->database, $sloupec->format, $sloupec->soucet);
                                break;

                            default :
                                $text = '';
                                break;
                        }

                        switch ($sloupec->format) {
                            case 'number0' :
                                $digits = 0;
                                break;
                            case 'number1' :
                            case 'number_1' :
                                $digits = 1;
                                break;
                            case 'number2' :
                            case 'number_2' :
                            default :
                                $digits = 2;
                                break;
                        }

                        $cssClass = $this ->getColumnClass($sloupec, $sloupec ->footerClass);
                        $result .= $html
                            ? "<td class='$cssClass'><div id='{$this ->prefixHTML}soucet_$i' data-digits='$digits'>$text</div></td>"
                            : $text . $this->exportColDelimiter;
                    } else {
                        $class = $sloupec->footerClass ? "class='$sloupec->footerClass'" : '';
                        $result .= $html
                            ? "<td $class></td>"
                            : $this->exportColDelimiter;
                    }
                }
            }
        }

        return $result;
    }


    /**
     * @param bool $html
     * @return string
     */
    private function vratRadkyLimit($html)
    {
        $jeLimit = false; // priznak, jestli je u nektere polozky limit

        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec->zobrazit && $sloupec ->limitRozdeleni != -1) {
                $jeLimit = true;
            }
        }

        if (!$jeLimit) {
            return '';
        }

        // zobrazeni limitu
        $result = $html
            ? "<tr class='browserfooter'>"
            : '';

        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec ->zobrazit) {
                $exportovat = $sloupec ->export // zobrazeni sloupce je omezeno pouze pro html/export
                    ? ($html
                        ? $sloupec ->export === 'html'
                        : $sloupec ->export === 'export')
                    : true;

                if ($exportovat) {
                    if ($sloupec->typ == 'JMENOPRIJMENI' || $sloupec->typ == 'PRIJMENI') {
                        $result .= $html
                            ? '<td>' . TextManager::_('Limit') . '</td>'
                            : TextManager::_('Limit') . $this->exportColDelimiter;
                    } else {
                        $result .= $html
                            ? "<td class='align-right {$sloupec->footerClass}'>"
                            : '';

                        if ($sloupec->limitRozdeleni != -1) {
                            $text = $sloupec->vratTextProHodnotu($sloupec->limitRozdeleni);
                            $class = '';
                            if (($sloupec->limitRozdeleni - $sloupec->soucet) < 0) {
                                $class = 'class="errorcolor"';
                            }

                            $result .= $html
                                ? "<div id='{$this ->prefixHTML}limit_$i' $class>$text</div>"
                                : $text;
                        }

                        $result .= $html
                            ? '</td>'
                            : $this->exportColDelimiter;
                    }
                }
            }
        }

        $result .= $html
            ? "</tr>\n"
            : $this ->exportRowDelimiter;

        // kolik jeste zbyva
        $result .= $html
            ? "<tr class='browserfooter'>"
            : '';

        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec ->zobrazit) {
                if ($sloupec ->typ == 'JMENOPRIJMENI'  ||  $sloupec ->typ == 'PRIJMENI') {
                    $result .= $html
                        ? '<td>' . TextManager::_('Zbyva') . '</td>'
                        : TextManager::_('Zbyva') . $this ->exportColDelimiter;
                } else {
                    $result .= $html
                        ? "<td class='align-right {$sloupec->footerClass}'>"
                        : '';

                    if ($sloupec->limitRozdeleni != -1) {
                        $hodnota = $sloupec->limitRozdeleni - $sloupec->soucet;
                        if ($hodnota < 0) {
                            $hodnota = 0;
                        }

                        $text = $sloupec->vratTextProHodnotu($hodnota);
                        $result .= $html
                            ? "<div id='{$this->prefixHTML}zbyvadolimitu_$i'>$text</div>"
                            : $text;
                    }

                    $result .= $html
                        ? '</td>'
                        : $this ->exportColDelimiter;
                }
            }
        }

        $result .= $html
            ? "</tr>\n"
            : $this ->exportRowDelimiter;

        return $result;
    }


    private static $initialized = false;

    /**
     * Vraci obsah formulare. Je pouzito i v XML!
     * @param bool $tlacitkoUlozit
     * @return string
     */
    function vratObsahFormulareH($tlacitkoUlozit)
    {
        $this ->jeEdit = false; // priznak bude nastaven na true, pokud formular obsahuje alespon jeden edit

        $result = '';
        if ($this ->style  ||  $this ->sirkaPole) {
            $result .=
                '<style>' .
                ($this ->sirkaPole ? '.editaceMesicnichHromadna input.editaceMP, .editaceMesicnichHromadna input.editaceMV { width: ' . (int)$this ->sirkaPole . "px; }\n" : '') .
                ($this ->style ? $this ->style : '') .
                "</style>\n";
        }

        $hlp_cnt = count($this ->osoby);
        $result .=
            "<input type='hidden' name='{$this ->prefixHTML}POCET_RADKU' value='$hlp_cnt' />\n" .
            "<input type='hidden' name='{$this ->prefixHTML}ROZDELENI' value='" . StringUtils::htmlTextConvert($this ->f_Rozdeleni). "' />\n" .
            "<input type='hidden' name='{$this ->prefixHTML}VCETNE_PODRIZENYCH' value='" . ($this ->vcetnePodrizenych === true ? 1 : (int)$this ->vcetnePodrizenych) . "' />\n";


        if ($this ->osoby)
            foreach ($this ->osoby as $index => $osoba) {
                $result .= "<input type='hidden' name='{$this ->prefixHTML}IdPP_$index' value='{$osoba ->idPP}' />\n";
            }

        // TODO: doplnit input s obdobim!

        $result .=
            "<table class='editaceMesicnichHromadna browser middle rowOver rowSelect oddEvenRows keyInputMove fixedCols'" .
            " data-ron-fixed-cols='{$this ->fixedCols}' style='{$this ->cssStyl}' data-ron-prefix='{$this ->prefixHTML}'>" .
            '<thead class="floating-table-header"><tr>';

        // prvni 2 sloupce pro jmeno a osobni cislo - zruseno, nyni je mozne tyto 2 sloupce nakonfigurovat

        //$result .= "<tr><th class='align-left'>" . TextManager::_('Osobni_cislo') . "</th><th class='align-left'>" . TextManager::_('Jmeno') . "</th>";

        // hlavicka - nadpisy sloupcu
        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec ->zobrazit  &&  (!$sloupec ->export || $sloupec ->export == 'html')) {
                // sloupec se zobrazi jen kdyz nema prirazene zadne pravo, nebo uzivatel ma prislusne pravo

                $css = $sloupec->headerClass;

                $params = ($sloupec ->lzeEditovat == 1)
                    ? "class='menuHromadne $css' data-index='$i'"
                    : ($css ? "class='$css'" : '');

                $params .= $sloupec ->title
                    ? ' title="' . StringUtils::htmlTextConvert($sloupec ->title) . '"'
                    : '';

                $result .=
                    "<th $params>" . $sloupec->nadpis .
                    "<input type='hidden' name='{$this ->prefixHTML}limit_$i' value='" . $sloupec->limitRozdeleni . "' />".
                    "</th>";
            }
        }

        // konec hlavicky
        $result .= "</tr></thead>\n";

        // Data osob
        if ($this ->osoby) {
            foreach ($this->osoby as $i => $osoba) {
                $result .= $this->vratRadekOsoby($i, $osoba);
            }
        }

        // paticka - soucty

        $result .=
            '<tr class="browserfooter">' .
            $this ->vratRadekSoucet(true) .
            "</tr>";

        // limity
        $result .=
            $this ->vratRadkyLimit(true);

        $result .= "</table>\n";

        if ($tlacitkoUlozit  &&  $this ->jeEdit) {
            // tlacitko ulozit
            $result .= "<div class='buttons'><input type='submit' class='submitbutton' value='" . TextManager::_('form.Store') . "' /></div>";
        }

        // pokud formular obsahuje sloupce, ktere maji povoleno prekroceni limitu,
        // zobrazi zaskrtavatko "souhlasim s ..."
        $sloupceprekroceni = $this ->zobrazitPovolitPrekroceni();
        if ($sloupceprekroceni) {
            // nektery ze sloupcu ma povoleno prekroceni limitu -> zobrazi zaskrtavatko
            $checkedstr = $this ->souhlasPrekrocitLimit ? 'checked="checked"' : '';

            $result .=
                '<div><label><input name="' . $this ->prefixHTML . 'povolitprekrocenilimitu" type="checkbox" value="1" ' . $checkedstr . '/><strong>' .
                TextManager::_('Povolit_prekroceni_limitu') . '</strong> (' . $sloupceprekroceni . ')</label></div>';
        }

        if (!self::$initialized) {
            // menu pro hromadne smazani/nastaveni/obnoveni hodnot ve sloupci
            // POZOR! Nepouzivat &nbsp; - v XML nefunguje spravne!
            $result .=
                "\n" .
                '<ul class="popupmenu" id="editaceHromadnaMenu">' .
                //"<li data-ron-action='hromadne'>" . TextManager::_('Nastavit_hromadne') . "</li>".
                "<li data-ron-action='smazat' data-ron-question='" . TextManager::_('Opravdu_smazat_hodnoty_ve_sloupci') . "'>" . TextManager::_('Smazat_vse_ve_sloupci') . "</li>".
                "<li data-ron-action='original' data-ron-question='" . TextManager::_('Opravdu_nastavit_puvodni_hodnoty_ve_sloupci'). "'>" . TextManager::_('form.Original') . "</li>".
                "<li data-ron-action='nastavit'>" . TextManager::_('Nastavit_sloupec_hromadne') . "</li>".
                "</ul>\n" .
                // dialog pro hromadne nastaveni nove hodnoty
                Renderer::renderViewToString(
                    (new Dialog())
                        ->setView('ronportal/editace/view_editace_hromadna_dialog_sloupec.php')
                        ->setId('editaceHromadnaDialogSloupec')
                        ->setTitle('Hromadne_nastaveni_hodnoty_sloupce')
                        ->addButton(
                            (new DialogButton())
                                ->setId('editaceHromadnaDialogOK')
                                ->setValue('OK')
                        )
                        ->addButton(DialogButton::cancelButton('editaceHromadnaDialogCancel'))
                ) .
                "\n";

            ;
            self::$initialized = true;
        }

        return $result;
    }


    /**
     * Vraci obsah formulare jako text - pouziva se v XML formularich pro generovani dat pro report
     * @return string
     */
    function vratObsahFormulareT()
    {
        $result = '';

        // hlavicka - nadpisy sloupcu
        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec ->zobrazit) {
                $result .=
                    ($result ? $this ->exportColDelimiter : '') .
                    $sloupec->typ . ':' . $sloupec->nadpis;
            }
        }

        $result .= $this ->exportRowDelimiter;

        // Data osob
        if ($this ->osoby) {
            foreach ($this->osoby as $i => $osoba) {
                // hodnoty sloupcu
                foreach ($this ->polozky as $s => $sloupec) {
                    $result .=
                        $this ->vratTextSloupce($sloupec, $s, $osoba);
                }

                $result .=
                    $this ->exportRowDelimiter;
            }
        }

        // paticka - soucty
        $jeLimit = false; // priznak, jestli je u nektere polozky limit

        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec ->zobrazit) {
                if ($sloupec->zobrazitSoucet) {
                    switch ($sloupec ->typ) {
                        case 'MP' :
                        case 'MV' :
                            $text = $sloupec->vratTextProHodnotu($sloupec->soucet);
                            break;

                        case 'SQL' :
                            $text = DBUtils::getFormatedValue($this ->database, $sloupec ->format, $sloupec ->soucet);
                            break;

                        default :
                            $text = '';
                            break;
                    }

                    switch ($sloupec ->format) {
                        case 'number0' :
                            $digits = 0;
                            break;
                        case 'number1' :
                        case 'number_1' :
                            $digits = 1;
                            break;
                        case 'number2' :
                        case 'number_2' :
                        default :
                            $digits = 2;
                            break;
                    }

                    $result .= $text;
                }

                $result .= $this ->exportColDelimiter;

                if ($sloupec ->limitRozdeleni != -1) {
                    $jeLimit = true;
                }
            }
        }
        $result .= $this ->exportRowDelimiter;

        // limity
        if ($jeLimit) {
            // zobrazeni limitu
            foreach ($this ->polozky as $i => $sloupec) {
                if ($sloupec ->zobrazit) {
                    if ($sloupec ->typ == 'JMENOPRIJMENI'  ||  $sloupec ->typ == 'PRIJMENI') {
                        $result .= TextManager::_('Limit');
                    } else {
                        if ($sloupec->limitRozdeleni != -1) {
                            $result .= $sloupec->vratTextProHodnotu($sloupec->limitRozdeleni);
                        }
                    }

                    $result .= $this ->exportColDelimiter;
                }
            }

            $result .= $this ->exportRowDelimiter;

            // kolik jeste zbyva
            foreach ($this ->polozky as $i => $sloupec) {
                if ($sloupec ->zobrazit) {
                    if ($sloupec ->typ == 'JMENOPRIJMENI'  ||  $sloupec ->typ == 'PRIJMENI') {
                        $result .= TextManager::_('Zbyva');
                    } else {
                        if ($sloupec->limitRozdeleni != -1) {
                            $hodnota = $sloupec->limitRozdeleni - $sloupec->soucet;
                            if ($hodnota < 0) {
                                $hodnota = 0;
                            }

                            $result .= $sloupec->vratTextProHodnotu($hodnota);
                        }
                    }

                    $result .= $this ->exportColDelimiter;
                }
            }

            $result .= $this ->exportRowDelimiter;
        }

        return $result;
    }


    function zobrazFormular($formaction)
    {
        if (!count($this ->osoby)) {
            return MessageManager::addErrorMessage( TextManager::_('Zadna_data') );
        }

        if ($this ->style) {
            echo "\n<style>" . $this ->style . "</style>\n";
        }

        echo
            '<div id="editacehromadna" class="position-relative">' .
            "<form method='post' class='submitWorkingInfo' name='editacevysledkuformular' action='$formaction'>\n" .
            "<input type='hidden' name='zapiseditacimesicnich' value='1' />\n";

        echo $this ->vratObsahFormulareH(!$this ->f_readonly); // tlacitkoulozit

        echo
            "</form>\n" .
            "</div>\n";

        return true;
    }


    /**
     * zkontroluje jesli nejsou prekroceny limity - vraci true, pokud je to OK, jinak false
     * @return bool
     */
    function zkontrolujLimity()
    {
        $result = true;

        // kontrola prekroceni limitu jednotlivych osob
        /**
         * @var integer $indexosoby
         * @var OsobaProEditaci $osoba
         */
        foreach ($this ->osoby as $indexosoby => $osoba) {
            foreach ($this ->polozky as $i => $sloupec) {
                // zkontroluje minimum
                if (isset($osoba ->f_data[$i] ->f_minHodnota) && isset($osoba ->f_data[$i] ->f_hodnota) && $osoba ->f_data[$i] ->f_hodnota < $osoba ->f_data[$i] ->f_minHodnota) {
                    $osoba ->f_data[$i] ->f_chyba = true;
                    $result = false;
                }

                // zkontroluje maximum a neprekrocitelny limit
                if ((isset($osoba ->f_data[$i] ->f_maxHodnota) && $osoba ->f_data[$i] ->f_hodnota > $osoba ->f_data[$i] ->f_maxHodnota)
                    || ($sloupec ->neprekrocitelnyLimit > 0 && $osoba ->f_data[$i] ->f_hodnota > $sloupec ->neprekrocitelnyLimit)
                ) {
                    $osoba ->f_data[$i] ->f_chyba = true;

                    if ($sloupec ->povolitPrekroceni && $this ->souhlasPrekrocitLimit) {
                        // polozka ma povoleno prekroceni limitu a je zaskrtnuto "souhlasim ..."
                        if ($sloupec ->neprekrocitelnyLimit > 0 &&
                            $osoba ->f_data[$i] ->f_hodnota > $sloupec ->neprekrocitelnyLimit
                        ) {
                            $this ->jeChybaNeprekrocitelnyLimit = true;
                            $result = false;
                        }
                    } else {
                        $result = false;
                    }
                }
            }
        }

        if (!$result) {
            return false;
        }


        // kontrola prekroceni limitu celkovych souctu (za vsechny osoby)
        foreach ($this ->polozky as $i => $sloupec) {
            if ($sloupec ->limitRozdeleni != -1 &&
                round($sloupec ->soucet, 2) > round($sloupec ->limitRozdeleni, 2)
            ) {
                return false;
            }
        }

        return true;
    }


    function getLogDescription() {
        return $this;
    }


    /**
     * @param $osoba OsobaProEditaci
     * @param $indexSloupce integer
     * @return bool
     */
    function zapisMzdovouPolozku($osoba, $indexSloupce)
    {
        $polozka = $this ->polozky[$indexSloupce];
        $castka = $polozka ->vratHodnotuProText($osoba ->f_data[$indexSloupce] ->f_text);

        // zapise zmenenou hodnotu
        $this ->zapisMzdovouPolozkuDoDB($osoba, $polozka, $castka);

        $this ->logMP .=
            "{$osoba ->jmenoPrijmeni},  os.c. {$osoba ->osobniCislo} (id {$osoba ->idPP})" .
            ", KodMP: {$polozka ->kod}, Castka: " . $osoba ->f_data[$indexSloupce] ->f_text .
            ' <old: ' . $osoba ->f_data[$indexSloupce] ->f_puvodniText . ">\n";

        // naformatuje zadanou hodnotu (uzivatel mohl pouzit jiny format)
        $osoba ->f_data[$indexSloupce] ->f_hodnota = $polozka ->vratTextProHodnotu($castka);
        $osoba ->f_data[$indexSloupce] ->f_puvodniText = $osoba ->f_data[$indexSloupce] ->f_text;

        return true;
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param int $indexSloupce
     * @return bool
     */
    function zapisPoznamkuMP($osoba, $indexSloupce)
    {
        $polozka = $this ->polozky[$indexSloupce];

        if ($this ->zapisPoznamkuMzdovePolozkyDoDB($osoba, $polozka, $osoba ->f_data[$indexSloupce] ->f_text)) {
            $this ->logMP .=
                "{$osoba ->jmenoPrijmeni},  os.c. {$osoba ->osobniCislo} (id {$osoba ->idPP})" .
                ", KodMP: {$polozka ->kod},\n   Poznamka: " . $osoba ->f_data[$indexSloupce] ->f_text .
                "\n   <old: {$osoba ->f_data[$indexSloupce] ->f_puvodniText}>\n";

            $osoba ->f_data[$indexSloupce] ->f_puvodniText = $osoba ->f_data[$indexSloupce] ->f_text;

            return true;
        }
        else {
            return false;
        }
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param PolozkaEditace $sloupec
     * @param string $pole H/D/K/P (P = poznamka)
     * @param mixed $hodnota
     * @throws \Exception
     */
    protected function zapisMesicniVysledekHromadny($osoba, $sloupec, $pole, $hodnota)
    {
        if ($sloupec ->rozlisitRozdeleni == 3) {
            // nemelo by nastat - pokud je rozlisitRozdeleni=3, tak je editace zakazana
            return;
        } else if ($sloupec ->rozlisitRozdeleni == 2) {
            // rozlisuje se rozdeleni - podle rozdeleni zamestnance
            if (!isset($osoba ->kodRozdeleni)) {
                throw new \Exception('Chyba - ' . $osoba ->jmenoPrijmeni . ', ' . ' os. c. ' . $osoba ->osobniCislo . ' nema prirazeno rozdeleni!');
            }
            $rec = new MesicniVysledek_R_ERecord($this ->database, $osoba, $this ->obdobi ->zacatek, $sloupec ->kod, $this ->f_TypRozdeleni, $osoba ->kodRozdeleni, $this);
        } else if ($sloupec ->rozlisitRozdeleni && $this ->f_TypRozdeleni && $this ->f_Rozdeleni) {
            // rozlisuje se rozdeleni - podle uzivatelem vybraneho rozdeleni ve formulari
            if (!isset($this ->f_Rozdeleni)) {
                throw new \Exception('Chyba - neni vybrano rozdeleni - nelze editovat sloupec s casovou slozkou ' . $sloupec ->kod);
            }
            $rec = new MesicniVysledek_R_ERecord($this ->database, $osoba, $this ->obdobi ->zacatek, $sloupec ->kod, $this->f_TypRozdeleni, $this->f_Rozdeleni, $this);
        } else {
            $rec = new MesicniVysledekVypoctuRecord($this ->database, $osoba, $this ->obdobi ->zacatek, $sloupec ->kod, $this);
        }

        switch ($pole) {
            case 'P' : // poznamka
                $rec ->fPoznamka ->setNewValue( $hodnota === '' ? null : $hodnota );
                break;

            case 'D' :
                $rec ->fDny ->setNewValue($hodnota);
                break;

            case 'K' :
                $rec ->fKalendarniDny ->setNewValue($hodnota);
                break;

            default :
                $rec ->fHodiny ->setNewValue($hodnota);
                break;
        }

        $rec ->updateOrInsert();
    }


    /**
     * @param OsobaProEditaci $osoba
     * @param int $indexSloupce
     */
    function zapisMesicniVysledek($osoba, $indexSloupce)
    {
        /** @var PolozkaEditace $polozka */
        $polozka = $this ->polozky[$indexSloupce];

        $hodnota = $osoba ->f_data[$indexSloupce] ->f_text;

        switch ($polozka ->typ) {
            case 'PMV' :
                $jednotka = 'P';
                break;

            default :
                $jednotka = $polozka ->jednotka;
                $hodnota = $polozka ->vratHodnotuProText($hodnota);
                break;
        }

        // zapise zmenenou hodnotu
        $this ->zapisMesicniVysledekHromadny($osoba, $polozka, $jednotka, $hodnota);

        $this ->logCS .=
            "{$osoba ->jmenoPrijmeni},  os.c. {$osoba ->osobniCislo} (id {$osoba ->idPP})" .
            ", KodCS: {$polozka ->kod}, Jednotka: {$jednotka}, Hodnota: " . $osoba ->f_data[$indexSloupce] ->f_text .
            ' <old: ' . $osoba ->f_data[$indexSloupce] ->f_puvodniText . ">\n";

        // naformatuje zadanou hodnotu (uzivatel mohl pouzit jiny format)
        $osoba ->f_data[$indexSloupce] ->f_text = $polozka ->vratTextProHodnotu($hodnota);
        $osoba ->f_data[$indexSloupce] ->f_puvodniText = $osoba ->f_data[$indexSloupce] ->f_text;
    }


    /**
     * @param $osoba OsobaProEditaci
     * @param $indexSloupce integer
     * @return bool
     */
    function zkontrolujPovinnouPoznamku($osoba, $indexSloupce)
    {
        // poznamka MP neni zadana => kontrola, jestli je prislusna mzdova polozka nulova
        $polozka = $this ->polozky[$indexSloupce];

        if (!$this ->lzeEditovatSloupec($polozka, $osoba)) return true;
        // needitovatelny sloupec nekontroluje

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Castka FROM MzdovaPolozkaVeMzde ' .
            ' WHERE RC = ' . $params->addParameter($osoba ->rc) .
            '   AND OsobniCislo = ' . $params->addParameter($osoba ->osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($this ->database ->timestampToParam($osoba ->datumNastupu)) .
            '   AND Obdobi = ' . $params ->addParameter($this ->database ->timestampToParam($this ->obdobi ->zacatek)) .
            '   AND DruhZdrojeMzdovePolozky = 4' . // jen MP z dochazky
            '   AND MzdovaPolozka = ' . $params->addParameter($polozka ->kod);

        if ($polozka ->rozlisitRozdeleni && $this ->f_TypRozdeleni && isset($this ->f_Rozdeleni)) {
            // jen mzdove polozky daneho rozdeleni
            $sql .=
                ' AND KodTypuRozdeleni = ' . $params ->addParameter((int)$this ->f_TypRozdeleni) .
                ' AND Rozdeleni = ' . $params->addParameter($this ->f_Rozdeleni);
        }

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $row = $cursor ->fetch();
        if ($row) if ((float)$row[0]) {
            MessageManager::addErrorMessage(
                $osoba ->jmenoPrijmeni . ': ' .
                TextManager::_('Chyba_neni_poznamka_MP') . ' (' . $polozka ->nadpis . ')' .
                (App::getDebugMode() ? 'idpp: ' . $osoba ->idPP : '') );

            $this ->jeChyba = true;

            // mzdova polozka je zadana a neni nulova
            $osoba ->f_data[$indexSloupce] ->f_chyba = true;
        }
        $cursor ->close();

        return true;
    }



    /**
     * Zapise zmenena data do databaze
     * @param OsobaProEditaci $osoba
     * @return bool
     */
    function zapisDataOsoby($osoba)
    {
        $zaznamMzdyZkontrolovan = false;
        $jezmena = false;

        foreach ($this ->polozky as $indexpolozky => $polozka) {
            // zkontroluje jestli je zadana povinna poznamka (pokud je vyplnena mzdova polozka)
            // TODO: toto by se melo zkontrolovat jeste pred zapisem hodnot!
            if ($polozka ->typ == 'PMP' && $polozka ->povinna && !$osoba ->f_data[$indexpolozky] ->f_text) {
                $this ->zkontrolujPovinnouPoznamku($osoba, $indexpolozky);
            }

            if ($osoba ->f_data[$indexpolozky] ->f_text != $osoba ->f_data[$indexpolozky] ->f_puvodniText  &&  $this ->lzeEditovatSloupec($polozka, $osoba)) {
                // doslo ke zmene editovatelne hodnoty
                if ($polozka ->typ == 'MP' || $polozka ->typ == 'PMP') {
                    // je to mzdova polozka - nejdrive zkontroluje, jestli existuje zaznam mzdy
                    if (!$zaznamMzdyZkontrolovan) {
                        if (!MzdyUtils::zkontrolujZaznamMzdy($this ->database, $osoba, $this ->obdobi ->zacatek, $this)
                        ) {
                            return MessageManager::addErrorMessage($osoba ->jmenoPrijmeni . ' (' . $osoba ->osobniCislo . ') ' . TextManager::_('Mzda_uzamcena_ve_mzdach'));
                        } else {
                            $zaznamMzdyZkontrolovan = true;
                        }
                    }

                    // zapise hodnotu do DB
                    if ($polozka ->typ == 'PMP') {
                        // poznamka mzdove polozky
                        $this ->zapisPoznamkuMP($osoba, $indexpolozky);
                    } else {
                        $this ->zapisMzdovouPolozku($osoba, $indexpolozky);
                    }

                    $jezmena = true;
                } else if ($polozka ->typ == 'MV'  ||  $polozka ->typ == 'PMV') {
                    // je to mesicni vysledek vypoctu
                    $this ->zapisMesicniVysledek($osoba, $indexpolozky);
                    $jezmena = true;
                }
            }
        }

        $this ->executeXmlEvent($osoba, RonPortalXMLEvents::EDITACE_MESICNICH_AFTER, $jezmena);
        // v pripade chyby zapisuje text chyby do MessageManageru

        // nakonec provede prepocet dochazky
        return ($this ->provestPrepocet == 2) || ($jezmena && $this ->provestPrepocet);
    }


    function prepocet($seznamPP)
    {
        $seznamStr = '';
        $pocet = 0;
        for ($i = 0;  $i < count($seznamPP); $i++) {
            $seznamStr .= ($seznamStr ? ';' : '') . $seznamPP[$i];
            $pocet++;

            if ($pocet >= $this ->maxPocetOsobProPrepocet  ||  $i == count($seznamPP) - 1) {
                // nacetl maximalni pocet osob, ktere se maji prepocitat, nebo dosel na konec seznamu
                try {
                    AgentManager::prepocetDochazky(
                        $seznamStr,
                        $this->obdobi->zacatek, DateUtils::lastDayOfMonth($this->obdobi->konec), false, false,
                        $this ->parametryAgentaPrepoctu ?: '-threadCount:4');
                } catch (\Exception $e) {
                    MessageManager::addErrorMessage($e->getMessage());
                }

                $seznamStr = '';
                $pocet = 0;
            }
        }
    }

    /**
     * zapise aktualni data do databaze
     */
    function zapisData()
    {
        $this ->logMP = '';
        $this ->logCS = '';

        // seznam IdPP, ktere se maji prepocitat
        $prepocitatPP = [];

        Profiler::addItem('');

        $profilerItem = Profiler::addItem('EditaceMesicnichHromadna::zapisData (' . $this ->f_Rozdeleni . ')');
        foreach ($this ->osoby as $osoba) {
            if ($this ->zapisDataOsoby($osoba)) {
                // osoba se ma prepocitat - ulozi IdPP do seznamu
                $prepocitatPP[] = $osoba ->idPP;
            }
        }
        Profiler::endItem($profilerItem);

        if ($prepocitatPP) {
            $this ->prepocet($prepocitatPP);
        }

        if ($this ->logMP) {
            Log::addRecordToDBLog($this ->database, 'MzdovaPolozkaVeMzde', 'UPDATE', $this,
                "Hromadna editace mesicnich vysledku \n" . $this ->logMP);
        }

        if ($this ->logCS) {
            Log::addRecordToDBLog($this ->database, 'MesicniVysledekVypoctu', 'UPDATE', $this,
                "Hromadna editace mesicnich vysledku \n" . $this ->logCS);
        }
    }
}
