<?php
/**
 * stanislav.elbl, 24.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;

class MesicniVysledekVypoctuRecord extends RonPortalDBRecord
{
    public $fHodiny;
    public $fDny;
    public $fKalendarniDny;
    public $fPoznamka;

    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int $mesic
     * @param string $typCasoveSlozkyMzdy
     * @param string|object $logDescription
     * @param ?string $tableName
     */
    public function __construct(Database $database, $pp, $mesic, $typCasoveSlozkyMzdy, $logDescription, $tableName = null)
    {
        parent::__construct($database, $tableName ?? 'MesicniVysledekVypoctu', $logDescription);

        $this->addField('RC', '', true, 0, $pp->rc); // readonly = true
        $this->addField('OsobniCislo', '', true, 0, $pp->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp->datumNastupu);
        $this->addField('', '', true, 0, '   ' . $pp ->jmenoPrijmeni);

        $this->addField('Mesic', 'date', true, 0, $mesic); // readonly = true
        $this->addField('TypCasoveSlozkyMzdy', '', true, 0, $typCasoveSlozkyMzdy);

        $this ->fHodiny = $this ->addField('Hodiny', 'float', true, 0, 0); // prozatim readonly
        $this ->fDny = $this ->addField('Dny', 'float', true, 0, 0);
        $this ->fKalendarniDny = $this ->addField('KalendarniDny', 'float', true, 0, 0);

        $this ->fPoznamka = $this ->addField('Poznamka', '', true, 250, null);

        if (!$tableName ||  $tableName === 'MesicniVysledekVypoctu') {
            $this->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName(true));
            $this->addTimestampField('Insert_Timestamp', true, time());

            $this->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName(true))
                ->setUpdateLog(true);
            $this->addTimestampField('Update_Timestamp', false, time())
                ->setUpdateLog(true);

            $params = $database->createParameters();
            $this->setWhere(
                'RC = ' . $params->addParameter($pp->rc) .
                ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                ' AND DatumNastupu = ' . $params->addParameter($database->dateToParam($pp->datumNastupu)) .
                ' AND Mesic = ' . $params->addParameter($database->dateToParam($mesic)) .
                ' AND TypCasoveSlozkyMzdy = ' . $params->addParameter($typCasoveSlozkyMzdy),
                $params);
        }
    }


    function zapisHodnotu($hodiny, $dny, $kalendarniDny, $poznamka)
    {
        if (isset($hodiny)) {
            $this ->fHodiny ->setNewValue($hodiny);
        }
        if (isset($dny)) {
            $this ->fDny ->setNewValue($dny);
        }
        if (isset($kalendarniDny)) {
            $this ->fKalendarniDny ->setNewValue($kalendarniDny);
        }

        if (isset($poznamka)) {
            $this ->fPoznamka ->setNewValue($poznamka === '' ? null : $poznamka);
        }

        $this ->updateOrInsert();
    }

}
