<?php
/**
 * stanislav.elbl, 26.02.2021
 * Formular pro zadani hodnot pro vlozeni dochazky podle kalendare. Zaznamy se vkladaji volanim agenta.
 */

namespace RonPortal\Editace;

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\RonException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\Dochazka\DochazkaUtils;
use RonPortal\NeniPravoException;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\System\AgentManager;
use Tracy\Debugger;

class PodleKalendareFormular extends RonPortalForm
{
    /** @var PracovniPomer */
    public $pp;

    /** @var Obdobi */
    public $obdobi;

    /** @var int */
    protected $datumZpracovaniDochazky;

    public function __construct($database)
    {
        parent::__construct($database);

        $this ->code = 'formpodlekalendare';

        $this ->pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);
        $this ->datumZpracovaniDochazky = DochazkaUtils::datumCasZpracovaniDochazky($this ->database);

        $this ->defaultRowClass = 'row';
        $this ->defaultCaptionColClass = 'col-12 col-sm-3 caption';
        $this ->defaultDataColClass = 'col-12 col-sm-9';

        $this ->title = TextManager::_('Generovani_podle_kalendare');

        $datum = isset($_GET['datum'])
            ? DateUtils::urlStringToDate($_GET['datum'])
            : DateUtils::today();

        $datumDo = isset($_GET['datumdo'])
            ? DateUtils::urlStringToDate($_GET['datumdo'])
            : $datum;

        $this ->obdobi = Obdobi::vratObdobi($database, 'ADS', (int)($_POST['Obdobi'] ?? $datum));
        $this ->stornoAction = "?menuid=zpracovanadochazka&osoba={$this ->pp ->idPP}&obdobi=" . DateUtils::dateToUrlString($this ->obdobi ->zacatek);

        Prava::nastavObdobiProPrava($this ->obdobi ->zacatek, $this ->obdobi ->konec);
        $prava = Prava::vratPrava('editace');
        if (!$prava ->lzeZobrazitPP($this ->pp ->idPP)) {
            throw new NeniPravoException();
        }

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('Obdobi')
            ->setDefaultValue($this ->obdobi ->zacatek);

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setText($this ->obdobi ->popisTranslation)
            ->setCode('field-obdobi')
            ->addFieldCssClass('font-weight-bold')
            ->setDataColumnClass('col-10 col-sm-7');

        if ($this ->pp ->idFotografie) {
            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setCode('field-foto')
                ->setNewRow(false)
                ->setCaption(false)
                ->setDataColumnClass('col-2 formFotoContainer')
                ->setFieldCssClass('formFoto')
                ->setText("<img src='?menuid=editace&action=foto&idpp={$this ->pp ->idPP}' alt='foto'/>");
        }

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setText($this ->pp ->jmenoPrijmeni)
            ->setCode('field-osoba')
            ->addFieldCssClass('font-weight-bold')
            ->setDataColumnClass('col-10 col-sm-7');

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('Datum_posledniho_zpracovani') . ' ' . StringUtils::format('datetime', $this ->datumZpracovaniDochazky))
            ->setCode('field-datum-posledniho-zpracovani')
            ->setDataColumnClass('col-10 col-sm-7');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumOd')
            ->setDefaultValue(StringUtils::format('date', $datum))
            ->setCaption('Datum_od')
            ->setRowId('podlekalendare_row_datum')
            ->setFieldId('podle_kalendare_DatumOd')
            ->setCode('field-datum-od')
            ->setRequired(true)
            ->setOtherHtmlParams(['data-roncalendar-next' => 'DatumDo']);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumDo')
            ->setDefaultValue(StringUtils::format('date', $datumDo))
            ->setCaption('Do')
            ->setRowId('podlekalendare_row_datum_do')
            ->setFieldId('podle_kalendare_DatumDo')
            ->setCode('field-datum-do')
            ->setRequired(true)
            ->setOtherHtmlParams(['data-roncalendar-prev' => 'DatumOd']);

        $prava = Prava::vratPrava('editace');

        $params = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('operace')
            ->setCaption('Operace')
            ->setCode('field-operace')
            ->setRowId('podlekalendare_row_operace')
            ->setSelectSql(
                new SqlStatement(
                    'SELECT Kod, Popis FROM OperaceDochazky' .
                    ' WHERE VkladatVEditaciDochazky = 1' .
                    ((trim($prava ->priznak3 ?? '') !== '')
                        ? ' AND ' . SQLUtils::getINCondition($this ->database, 'Kod', explode(';', $prava ->priznak3), $params)
                        : ''
                    ) .
                    ' AND (%FILTER%)' .
                    ' ORDER BY %ORDER%', $params),
                '1 = 1', 'Popis'
            )
            ->setTranslationPrefix('db_operacedochazky_popis_')
            ->addFieldCssClass('ronselect')
            ->setRequired(true)
            ;

        if (RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI)) {
            $fRozdeleni = new RozdeleniFormField(RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI));
            $fRozdeleni
                ->setFieldName('Rozdeleni') ->setTypRozdeleniFieldName('KodTypuRozdeleni')
                ->setCode('field-rozdeleni')
                ->setCaption(StringUtils::htmlTextConvert(RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI) ?: TextManager::_('Rozdeleni')))
                ->setRowId('podlekalendare_row_rozdeleni')
                ->setRequired(RonPortalSettings::getIntValue(EditaceUtils::SETTINGS_ROZDELENI_POVINNE));
            $this ->addField($fRozdeleni);

//            $this ->addField(FormField::FIELD_TYPE_SELECT)
//                ->setSelectSql(
//                    new SqlStatement(
//                        'SELECT Kod, Popis FROM Rozdeleni' .
//                        ' WHERE KodTypuRozdeleni = ' . RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI) .
//                        '  AND (DatumZruseni >= ' . $params ->addDateParameter($datumDo) . ' OR DatumZruseni IS NULL) ' .
//                        '  AND DatumVytvoreni <= ' . $params ->addDateParameter($datum) .
//                        '  AND (%FILTER%)' .
//                        ' ORDER BY (%ORDER%)',
//                        $params),
//                        '1 = 1', 'Popis')
        }

        $this ->addField(FormField::FIELD_TYPE_RADIO)
            ->setFieldName('delka')
            ->setCaption('Delka')
            ->setCode('field-delka')
            ->setOptions([
                0 => TextManager::_('Absence_cela_smena'),
                1 => TextManager::_('Absence_pulka_smeny1'),
                2 => TextManager::_('Absence_pulka_smeny2')
            ])
            ->setRequired(true)
            ->setDefaultValue(0)
            ->setRowId('podlekalendare_row_delka');

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('pocatekkonec')
            ->setCode('field-pocatek-konec')
            ->setCaption2('Pouzit_pocatek_konec_smeny')
            ->setRowId('podlekalendare_row_pocatekkonec');

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('pevnejadro')
            ->setCode('field-pevne-jadro')
            ->setCaption2('Pouzit_pevne_jadro')
            ->setRowId('podlekalendare_row_pevnejadro');

        $this ->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('autoopakovat')
            ->setCode('field-auto-opakovat')
            ->setCaption2('Automaticky_opakovat')
            ->setRowId('podlekalendare_row_autoopakovat');

        if ( !RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADAT_PRESTAVKY) ) {
            $this ->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('vlozitprestavky')
                ->setCode('field-vlozit-prestavky')
                ->setCaption2('Vlozit_prestavky')
                ->setRowId('podlekalendare_row_vlozitprestavky');
        }
    }


    public function validate($edit)
    {
        if (!parent::validate($edit)) {
            return false;
        }

        // kontrola datumu
        $datumOd = $this ->findFieldByName('DatumOd') ->getPostValue($this ->utf8);
        $datumDo = $this ->findFieldByName('DatumDo') ->getPostValue($this ->utf8) ?? $datumOd;

        if ($datumDo < $datumOd) {
            throw new RonException( TextManager::_('Pocatecni_datum_vetsi_nez_koncove'), 'DatumDo' );
        }

        $maxDatum = $this ->obdobi ->konec;
        if ($maxDatum > $this ->datumZpracovaniDochazky) {
            $maxDatum = $this ->datumZpracovaniDochazky;
        }
        if ($maxDatum < $this ->obdobi ->zacatek) {
            throw new Exception(TextManager::_('Dochazka_jeste_neni_zpracovana'));
        }

        if ($datumOd < $this ->obdobi ->zacatek || $datumDo > $maxDatum) {
            throw new Exception(sprintf(
                TextManager::_('Datum_musi_byt_v_rozsahu'),
                StringUtils::format('date', $this ->obdobi ->zacatek),
                StringUtils::format('date', $maxDatum)
            ));
        }

        // kontrola prav na operaci
        $operace = $this ->findFieldByName('operace') ->getPostValue($this ->utf8);

        if (!EditaceUtils::jePravoEditovatOperaci($operace)) {
            throw new NeniPravoException();
        }

        // kontrola prav na den
        if (!EditaceDneUtils::jePravoEditovatDen($this ->database, $this ->pp, $datumOd, $datumDo)) {
            throw new NeniPravoException();
        }

        // kontrola rozdeleni
        $fRozdeleni = $this ->findFieldByName('Rozdeleni');
        if ($fRozdeleni  &&  !$fRozdeleni ->getPostValue($this ->utf8)) {
            if (EditaceUtils::jeRozdeleniDochazkyPovinne($this->database, $operace)) {
                throw new RonException(
                    TextManager::_('Neni_zadano_rozdeleni_dochazky') .
                    (RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI)
                        ? ' (' . RonPortalSettings::getValue(RonPortalSettings::EDITACE_NADPIS_ROZDELENI) . ')'
                        : ''),
                    'Rozdeleni'
                );
            }
        }

        return true;
    }


    /**
     * @return bool
     * @throws DBException
     * @throws NeniPravoException
     * @throws Exception
     */
    public function insertRecord()
    {
        if (!$this->validate(false)) {
            return false;
        }

        $datumOd = $this ->findFieldByName('DatumOd') ->getPostValue($this ->utf8);
        $datumDo = $this ->findFieldByName('DatumDo') ->getPostValue($this ->utf8) ?? $datumOd;

        EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $datumOd, 'before');

        if ( RonPortalSettings::getSpousteniADS() ) {
            $params = '9 ' .
                ' -interval:' . date('Y-n-j', $datumOd) . ';' . date('Y-n-j', $datumDo) .
                ' -IDPP:' . $this ->pp ->idPP .
                ' -operace:' . $this ->findFieldByName('operace') ->getPostValue($this ->utf8) .
                ' -delka:' . $this ->findFieldByName('delka') ->getPostValue($this ->utf8);

            if ( RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI) ) {
                $rozdeleni = $this ->findFieldByName('Rozdeleni') ->getPostValue($this ->utf8);
                if ($rozdeleni !== null  &&  $rozdeleni !== '') {
                    $params .= ' -attSubDiv:"' . RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_TYP_ROZDELENI) . '/' . $rozdeleni . '"';
                }
            }

            $vlozitPrestavky =
                RonPortalSettings::getIntValue(RonPortalSettings::EDITACE_VKLADAT_PRESTAVKY)
                || $this ->findFieldByName('vlozitprestavky') ->getPostValue($this ->utf8);

            $params .=
                ($this ->findFieldByName('pocatekkonec') ->getPostValue($this ->utf8) ? ' -pouzitKonecSmeny' : '') .
                ($this ->findFieldByName('pevnejadro') ->getPostValue($this ->utf8) ? ' -pevneJadro' : '') .
                ($this ->findFieldByName('autoopakovat') ->getPostValue($this ->utf8) ? ' -autoOpakovat' : '') .
                ($vlozitPrestavky ? ' -vkladatPrestavky' : '');

            AgentManager::executeADS('ADS_Agent_Processing.exe', $params);
        }

        EditaceUtils::editaceDochazkyEvent($this ->database, $this ->pp, $datumOd, 'after');

        return true;
    }

}
