<?php
/**
 * stanislav.elbl, 26.02.2021
 */

namespace RonPortal\Editace;

use Ron\DB\Database;
use Ron\DB\DBRecordField;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;

class PoznamkaDochazkyRecord extends RonPortalDBRecord
{
    /** @var DBRecordField  */
    var $fPoznamka;


    /**
     * @param Database $database
     * @param PracovniPomer $pp
     * @param int           $datumCasPocatku
     * @param string|object $logDescription
     */
    function __construct($database, $pp, $datumCasPocatku, $logDescription = 'Zpracovana dochazka')
    {
        parent::__construct($database, 'PoznamkaDochazky', $logDescription);

        $this->addField('RC', '', true, 15, $pp ->rc);
        $this->addField('OsobniCislo', '', true, 15, $pp ->osobniCislo);
        $this->addField('DatumNastupu', 'date', true, 0, $pp ->datumNastupu);

        $this->addField('DatumCasPocatku', 'datetime', true, 0, $datumCasPocatku);

        $this->fPoznamka = $this->addField('Poznamka', '', true, 160, null);

        $params = $database ->createParameters();
        $this->setWhere(
            'RC = ' . $params ->addParameter($pp ->rc) .
            ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
            ' AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($pp ->datumNastupu)) .
            ' AND DatumCasPocatku = ' . $params ->addParameter($database ->timestampToParam($datumCasPocatku)),
            $params);
    }


    /**
     * @param string $text
     * @param bool   $onlyInsert  pokud je true, tak nekontroluje, jestli zaznam existuje (pouzije se pri vlozeni zaznamu dochazky)
     * @return bool     Vraci true, pokud doslo ke zmene (editace, vlozeni nebo smazani poznamky)
     */
    function zapisPoznamku($text, $onlyInsert = false)
    {
        $this ->fPoznamka ->setNewValue( $text );

        if ($onlyInsert) {
            $exists = false;
        } else {
            $exists = $this ->loadRecord();
        }

        if ($exists) {
            if (trim($text) == '') {
                $this ->doDelete(null);
                return true;
            } else {
                $result = $this ->isChanged();
                $this->updateRecord();
                return $result;
            }
        } else {
            if (trim($text) != '') {
                $this->insertRecord();
                return true;
            }
        }

        return false;   // zadna zmena
    }
}
