<?php

namespace RonPortal\Editace;

use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\Controls\OptionLoader;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\NastaveniUzivatele;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSettings;
use RonPortal\TypRozdeleni;


class RozdeleniVysledkuFormular
{
    /** @var Database */
    var $database;

    /* @var PracovniPomer pp */
    var $pp;
    /* @var int den */
    var $den;

    /* @var ZaznamRozdeleniVysledku[]    Seznam zaznamu nacteny z DB nebo z POST */
    var $dataList;

    var $readOnly;

    /** @var bool       Priznak, ze se ma pokracovat stejnym formularem */
    var $pokracovat;

    var $kodTypuRozdeleni;

    /** @var TypRozdeleni */
    var $typRozdeleni;

    /** @var string     Pro zapamatovani posledni zadane uzivatelske slozky v uzivatelskem nastaveni */
    var $casovaSlozkaNovyZaznam;

    var $jeZmena;

    /** @var Option[] */
    var $optionsKod;
    /** @var Option[] */
    var $optionsPopis;


    function __construct($database, $pp, $den)
    {
        $this ->database = $database;

        $this->pp = $pp;
        $this->den = $den;

        $this->kodTypuRozdeleni = RonPortalSettings::getIntValue(RonPortalSettings::DV_TYP_ROZDELENI);
    }


    function nactiSeznamRozdeleni()
    {
        $sql = RozdeleniVysledkuUtils::getSQLVyberRozdeleniDV($this ->database, $this->pp, $this->kodTypuRozdeleni, null, $this->den, $this->den, true);
        //$this ->optionsKod = Option::loadSQL($this ->database, $sql, null, 0, 0, null, 1, 2);
        $this ->optionsKod = (new OptionLoader($this ->database))
            ->setSql($sql, null)
            ->setCaptionIndex(0) ->setAttrIndexes(['data-popis' => 1]) ->setCssClassIndex(2)
            ->loadData();

        $sql = RozdeleniVysledkuUtils::getSQLVyberRozdeleniDV($this ->database, $this->pp, $this->kodTypuRozdeleni, null, $this->den, $this->den);
        //$this ->optionsPopis = Option::loadSQL($this ->database, $sql, null, 0, 1, null, 0, 2);
        $this ->optionsPopis = (new OptionLoader($this ->database))
            ->setSql($sql, null)
            ->setTitleIndex(0) ->setCssClassIndex(2)
            ->loadData();
    }

    /**
     * @param int    $index
     * @param string $hodnota
     * @param bool $chyba
     */
    function zobrazVyberRozdeleniVysledku($index, $hodnota, $chyba, $kod)
    {
        $htmlParams = [
                'id=' => "DenniVysledekR{$index}_kod" . ($kod ? '1' : ''),
                'class' => 'form-control ronselect',
                'data-columns' => $kod ? 'value;popis' : 'text;value',
                'data-max-visible-rows' => '200',
                'data-ron-select2' => "DenniVysledekR{$index}_kod"  . ($kod ? '' : '1')
        ];

        if ($chyba  &&  is_string($chyba)) {
            $htmlParams['title'] = $chyba;
        }

        if ($this ->readOnly) {
            $htmlParams['disabled'] = 'disabled';
        }

        $comboOptions = [
            'alwaysShowSelected' => true,
            'titleFieldName' => 'title',
            'cssClassFieldName' => 'cssClass',
            'emptyRow' => (isset($hodnota) && $hodnota != '') ? null : true,
            'htmlParams' => $htmlParams
        ];
        if ($chyba) {
            $comboOptions['className'] = 'form-input error';
        }
        ComboBox::renderComboBox('DenniVysledekR' . $index . '_kod' . ($kod ? '1' : ''), $kod ? $this->optionsKod : $this ->optionsPopis, $hodnota, $comboOptions);
    }


    function zobrazCasoveSlozkyVysledku($index, $hodnota, $chyba)
    {
        $sql = RozdeleniVysledkuUtils::getSQLVyberCasSlozkyDV($this ->database);

        $value = null;

        if (isset($hodnota)) {
            $value = $hodnota;
        } else {
            // neni zadana hodnota = novy radek
            $slozky = explode(';', RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY));
            if (trim($slozky[0]) != '') {
                // natvrdo zadano v nastaveni intrawebu
                $value = trim($slozky[0]);
            }
            else {
                // nacte z nastaveni uzivatele
                $value = NastaveniUzivatele::getUserSettingsItem($this ->database, NastaveniUzivatele::CASOVA_SLOZKA_ROZDELENI_DV);
            }
        }

        $combo = new ComboBox('DenniVysledekR' . $index . '_cs');
        $titleStr = '';
        if ($chyba) {
            $combo->className = 'form-input error';
            $titleStr = is_string($chyba)
                ? ' title="' . StringUtils::htmlTextConvert($chyba) . '"'
                : '';
        }
        $combo ->emptyRow = isset($value) ? false : ' ';
        $combo->loadData($this ->database, $sql, $value);
        $combo ->htmlParams = 'class="form-input"' . ($this->readOnly ? ' disabled="disabled"' : '') . $titleStr;
        $combo->render();
    }



    /**
     * @param $index
     * @param ZaznamRozdeleniVysledku $item     pokud je null, pouzije se pro hodnotu $zbyvaStr
     * @param string $zbyvaStr
     */
    function zobrazRadekRozdeleniVysledku($index, $item, $zbyvaStr = '')
    {
        echo '<div class="row">';

        if ($item && $item ->rozdeleniSchvaleno) {
            echo
                '<div class="col col-8">' .
                $item ->rozdeleni . ' - ' . $item ->popisRozdeleni .
                "<input type='hidden' name='DenniVysledekR{$index}_kod' value='" . StringUtils::htmlTextConvert($item ->rozdeleni) . "'/>" .
                '</div>';
        } else {
            if ($item) {
                $value = $item->rozdeleni;
            } else if (isset($_POST['rozdeleniosobypost']) && $_POST['selectedvalue']) {
                // dvojklikem vybrany zaznam ve "vyhledavaci"
                $value = $_POST['selectedvalue'];
            }  else {
                $value = null;
            }

            echo '<div class="col col-4 col-sm-2">';
            $this->zobrazVyberRozdeleniVysledku($index, $value, $item ? $item->error_rozdeleni : false, true);
            echo '</div>';
            echo '<div class="col col-8 col-sm-6">';
            $this->zobrazVyberRozdeleniVysledku($index, $value, $item ? $item->error_rozdeleni : false, false);
            echo '</div>';

            if (!$item || $item->novy_radek) {
                // pri zadani nove polozky si pamatuje posledni nastaveni casove slozky
                echo "<input id='DenniVysledekR{$index}_novyradek' type='hidden' name='DenniVysledekR{$index}_novyradek' value='1' />";
            }
        }

        echo '<div class="col col-6 col-sm-2">';

        if ($item  &&  $item ->rozdeleniSchvaleno) {
            echo
                $item ->popisCasoveSlozky .
                "<input type='hidden' name='DenniVysledekR{$index}_cs' value='" . StringUtils::htmlTextConvert($item ->casovaSlozka) . "'/>";
        } else {
            $this->zobrazCasoveSlozkyVysledku($index, $item ? $item->casovaSlozka : null, $item ? $item->error_casovaslozka : false);
        }

        echo '</div><div class="col col-3 col-sm-1 align-right">';
        $hodnota = $item ? $item->hodinyStr : $zbyvaStr;

        if ($this->readOnly) {
            echo $hodnota;
        } else {
            $class = $item && $item->error_hodiny ? 'error' : '';
            $readOnlyStr = $item && $item->rozdeleniSchvaleno ? 'readonly="readonly"' : '';
            $titleStr = ($item && $item->error_hodiny && is_string($item ->error_hodiny))
                ? 'title="' . StringUtils::htmlTextConvert($item ->error_hodiny) . '"'
                : '';

            echo "<input class='form-input $class align-right inputTime rozdeleniDV_hodnota' type='text' name='DenniVysledekR{$index}_hodnota' value='$hodnota' $titleStr $readOnlyStr data-ron-rozdeleni-input='DenniVysledekR{$index}_kod'/>";
        }

        echo '</div><div class="col col-3 col-sm-1">h';

        if (!$this->readOnly && $item  &&  !$item ->rozdeleniSchvaleno) {
            echo
                '<i class="inline-glyph glyph-button fa glyph-delete rozdeleni_dv_delete" title="' . TextManager::_('Smazat_cely_radek')  . '" data-ron-confirm-id="Opravdu_smazat_cely_radek" data-index="' . $index . '"></i>' .
                "<input type='hidden'  id='DenniVysledekR{$index}_DELETE' ' name='DenniVysledekR{$index}_DELETE' value='' />";
        } else if ($item  &&  $item ->rozdeleniSchvaleno) {
            echo '<span class="inline-glyph fa glyph-lock"></span>';
        }

        echo '</div>';
        echo "</div>\n";
    }


    function zobrazRozdeleniVysledku()
    {
        $this ->typRozdeleni = new TypRozdeleni();
        $this ->typRozdeleni->load($this ->database, $this->kodTypuRozdeleni);

        require 'ronportal/editace/view_rozdeleni_dv.php';
    }


    function nactiData_DB()
    {
        $slozky = explode(';', strtolower(RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY)));
        $this->dataList = ZaznamRozdeleniVysledku::nactiZaznamyOsoby($this ->database, $this->kodTypuRozdeleni, $this ->pp, $slozky, $this->den);
    }


    function zkontrolujData()
    {
        $chyba_duplicita = false;
        $chyba_povinnahodnota = false;
        $chyba_hodiny = false;

        $soucet = 0;

        for ($i = 0; $i < count($this->dataList); $i++) {
            if (!$this->dataList[$i] ->smazat) {
                if ($this->dataList[$i]->rozdeleni == '') {
                    $this->dataList[$i]->error_rozdeleni = true;
                    if (!$chyba_povinnahodnota) {
                        MessageManager::addErrorMessage(TextManager::_('Chybi_povinna_hodnota'));
                    }
                    $chyba_povinnahodnota = true;
                }

                if ($this->dataList[$i]->casovaSlozka == '') {
                    $this->dataList[$i]->error_casovaslozka = TextManager::_('Chybi_povinna_hodnota');
                    if (!$chyba_povinnahodnota) {
                        MessageManager::addErrorMessage(TextManager::_('Chybi_povinna_hodnota'));
                    }
                    $chyba_povinnahodnota = true;
                }

                // zkontroluje duplicity
                for ($j = 0; $j < $i; $j++) {
                    if ($this->dataList[$i]->rozdeleni == $this->dataList[$j]->rozdeleni
                        && $this->dataList[$i]->casovaSlozka == $this->dataList[$j]->casovaSlozka
                    ) {
                        $this->dataList[$i]->error_rozdeleni =
                        $this->dataList[$i]->error_casovaslozka = TextManager::_('Zaznam_s_kodem_uz_existuje');

                        $chyba_duplicita = true;
                    }
                }

                if (RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)) {
                    if (60 * $this->dataList[$i]->hodiny < RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS)) {
                        $errorMsg = sprintf(TextManager::_('Nelze_vykazat_mene'), RonPortalSettings::getIntValue(RonPortalSettings::DV_MINIMALNI_CAS));
                        $this->dataList[$i]->error_hodiny = $errorMsg;

                        if (!$chyba_hodiny) {
                            MessageManager::addErrorMessage($errorMsg);
                        }

                        $chyba_hodiny = true;
                    }
                }

                $soucet += $this->dataList[$i]->hodiny;
            }
        }

        if ($chyba_duplicita) {
            return MessageManager::addErrorMessage( TextManager::_('Zaznam_s_kodem_uz_existuje') );
        }

        if ($chyba_hodiny) {
            return false;
        }

        // zkontroluje, jestli neni rozdeleno vice nez odpracovano
        if (RonPortalSettings::getValue(RonPortalSettings::DV_CASOVA_SLOZKA_SOUCET)) {
            $odpracovano = DenniVysledek::nactiHodnotu($this ->database,
                    $this->pp, $this->den, RonPortalSettings::getValue(RonPortalSettings::DV_CASOVA_SLOZKA_SOUCET));

            if (round($odpracovano - $soucet, 5) < 0) {
                return MessageManager::addErrorMessage( TextManager::_('Nelze_rozdelit_vice_dv') );
            }
        }

        return !$chyba_povinnahodnota;
    }


    /**
     * zkontroluje zadane hodnoty pro rozdeleni dochazky
     * @return bool
     * @throws \Exception
     */
    function nactiData_Post()
    {
        $slozky = explode(';', strtolower(RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY)));
        $originalData = ZaznamRozdeleniVysledku::nactiZaznamyOsoby($this ->database, $this->kodTypuRozdeleni, $this ->pp, $slozky, $this->den);

        $this->dataList = [];
        $this->casovaSlozkaNovyZaznam = null;

        $cnt = (int)$_POST['DenniVysledekR_COUNT'];
        if (!$cnt) {
            // neni co ukladat
            return true;
        }

        $result = true;

        for ($i = 0; $i < $cnt; $i++) {
            $hodnotastr = $_POST["DenniVysledekR{$i}_hodnota"];
            $rozdeleni = $_POST["DenniVysledekR{$i}_kod"];
            $casovaslozka = $_POST["DenniVysledekR{$i}_cs"];

            if (!in_array(strtolower($casovaslozka), $slozky)) {
                throw new \Exception('Chyba - zadana casova slozka neni povolena!');
            }

            //if ( trim($rozdeleni) == ''  &&  trim($hodnotastr) == '' )
            if (trim($rozdeleni) == '') {
                // radek ignoruje (bude smazan)
            } else {
                // zkusi najit existujici zaznam
                $zaznam = null;
                for ($orig = 0;  $orig < count($originalData) && !$zaznam;  $orig++) {
                    if (!strcasecmp($originalData[$orig] ->rozdeleni, $rozdeleni)  &&  !strcasecmp($originalData[$orig] ->casovaSlozka, $casovaslozka)) {
                        // nasel puvodni zaznam
                        $zaznam = $originalData[$orig];
                    }
                }

                if (!$zaznam) {
                    $zaznam = new ZaznamRozdeleniVysledku();
                    $zaznam->rozdeleni = $rozdeleni;
                    $zaznam->casovaSlozka = $casovaslozka;
                    $zaznam->novy_radek = true;
                }

                $zaznam->hodinyStr = $hodnotastr;

                if (isset($_POST["DenniVysledekR{$i}_DELETE"]) && $_POST["DenniVysledekR{$i}_DELETE"]) {
                    // uzivatel kliknul na "smazat"
                    $zaznam->smazat = true;
                    $this->pokracovat = true; // nezavira formular
                } else {
                    $zaznam->novy_radek = isset($_POST["DenniVysledekR{$i}_novyradek"]);

                    try {
                        $hodnota = DateUtils::timeStringToFloat($hodnotastr, true); // povolitzkraceny = true
                        if ($hodnota == 0) {
                            $zaznam->smazat = true; // = null; // zaznam bude smazan
                        } else {
                            $zaznam->hodiny = $hodnota;

                            // zapamatuje si casovou slozku nove zadaneho radku - pro pouziti v uzivatelskem nastaveni
                            if ($zaznam->novy_radek) {
                                $this->casovaSlozkaNovyZaznam = $zaznam->casovaSlozka;
                            }
                        }
                    } catch (\Exception $e) {
                        // chyba v zadane hodnote (format casu)
                        MessageManager::addErrorMessage($e ->getMessage());

                        $result = false;
                        $zaznam->error_hodiny = $e ->getMessage();
                    }
                }

                if ($zaznam) $this->dataList[] = $zaznam;
            }
        }

        if ($result) return $this->zkontrolujData();

        return $result;
    }



    function show($readonly)
    {
        if (!RonPortalSettings::getPouzivatRozdeleniDV()) {
            // dochazka se nerozdeluje
            return true;
        }

        $this->readOnly = $readonly;
        if (!isset($this->dataList)) {
            return MessageManager::addErrorMessage('Chyba ve funkci RozdeleniVysledkuFormular::Show - nejsou nactena data!');
        }

//        if (S_Get_DV_PovolitPrirazeniR() && isset($_POST['rozdeleniosobypost']) && $_POST['selectedvalue']) {
//            // prida zaznam, na ktery uzivatel dvojkliknul do jeho oblibenych
//            // TODO: odstranit?
//            PridejRozdeleniOsoby($this ->database, $this->f_pp, $this->f_typrozdeleni, $_POST['selectedvalue'], $this->f_pp->f_DatumNastupu);
//        }

        $this ->nactiSeznamRozdeleni();

        $this->zobrazRozdeleniVysledku();

        return true;
    }


    /**
     * Najde v seznamu polozku pro aktualni zaznam
     * @param RozdeleniDennichVysledkuRecord $rec
     * @return ZaznamRozdeleniVysledku
     */
    function findItemRozdeleni($rec)
    {
        foreach ($this ->dataList as $polozka) {
            if ($polozka->rozdeleni === $rec->fRozdeleni->value &&
                $polozka->casovaSlozka === $rec->fSlozka->value
            ) {
                return $polozka;
            }
        }

        return null;
    }


    /**
     * Zpracuje (edit/delete) vsechny zaznamy podle zadaneho seznamu
     */
    function saveRecords()
    {
        $slozky = explode(';', strtolower(RonPortalSettings::getValue(RonPortalSettings::DV_CASOVE_SLOZKY)));

        $rec = new RozdeleniDennichVysledkuRecord($this ->database, $this->kodTypuRozdeleni, $this->pp->rc, $this->pp->osobniCislo, $this->pp->datumNastupu, $this->den, $this);
        $rec->checkMultiple = false;

        $rec ->loadRecords();
        while ($rec ->next()) {
            if (in_array( strtolower($rec['TypCasoveSlozkyMzdy']), $slozky, true)) { // zaznamy s jinou casovou slozkou ignoruje
                $item = $this->findItemRozdeleni($rec);
                $params = $this->database->createParameters();
                $filtr =
                    'TypCasoveSlozkyMzdy = ' . $params->addParameter($rec->fSlozka->value) .
                    ' AND Rozdeleni = ' . $params->addParameter($rec->fRozdeleni->value);

                if ($item && !$item->smazat) {
                    // v pripade potreby provede update
                    $rec->fHodnota->value = $item->hodiny;

                    if ($rec->isChanged()) {
                        RozdeleniVysledkuUtils::overSchvaleni($this->database, $rec->fRozdeleni->originalValue, $this->den);
                        $rec->updateRecord($filtr, $params);

                        $this->jeZmena = $this->jeZmena || $rec->recordUpdated;
                    }

                    $item->zapsano = true;
                } else {
                    // smaze zaznam - neni v POST nebo se ma smazat
                    RozdeleniVysledkuUtils::overSchvaleni($this->database, $rec->fRozdeleni->originalValue, $this->den);
                    $rec->doDelete($filtr, $params);
                    $this->jeZmena = true;
                }
            }
        }

//        $params = $this ->database ->createParameters();
//        $sql = $rec->getSelectSQL(null, $params);
//
//        $cursor = $this ->database ->getSqlResult($sql, $params);
//
//        while ($row = $cursor ->fetch()) {
//            $this ->loadRecordValues($row);
//
//            $item = $this ->findItemRozdeleni($list);
//            $params = $this ->database ->createParameters();
//            $filtr =
//                'TypCasoveSlozkyMzdy = ' . $params->addParameter($this ->fSlozka ->value) .
//                ' AND Rozdeleni = ' . $params->addParameter($this ->fRozdeleni ->value);
//
//            if ($item && !$item ->smazat) {
//                // v pripade potreby provede update
//                $this ->fHodnota ->value = $item ->hodiny;
//                if (!$this ->updateRecord($filtr, $params)) return false;
//
//                $this ->jeZmena = $this ->jeZmena  ||  $this ->recordUpdated;
//
//                $item ->zapsano = true;
//            } else {
//                // smaze zaznam - neni v POST
//                $this ->doDelete($filtr, $params);
//                $this ->jeZmena = true;
//            }
//        }

        return true;
    }


    /**
     * Doplni zaznamy, ktere nebyly ulozeny v SaveRecords
     * @return bool
     * @throws \Exception
     */
    function insertRecords()
    {
        $rec = new RozdeleniDennichVysledkuRecord($this ->database, $this->kodTypuRozdeleni, $this->pp->rc, $this->pp->osobniCislo, $this->pp->datumNastupu, $this->den, $this);

        $error = null;

        foreach ($this ->dataList as $item) {
            if (!$item ->zapsano && !$item ->smazat) {
                try {
                    RozdeleniVysledkuUtils::overSchvaleni($this->database, $item->rozdeleni, $this->den);

                    $rec->fSlozka->value = $item->casovaSlozka;
                    $rec->fRozdeleni->value = $item->rozdeleni;
                    $rec->fHodnota->value = $item->hodiny;

                    $rec->insertRecord();

                    $this->jeZmena = true;
                } catch (\Exception $e) {
                    $item ->error_rozdeleni = $e ->getMessage();
                    $error = $e;
                }
            }
        }

        if ($error) throw $error;
    }


//    function save_NEW()
//    {
//        $this->jeZmena = false;
//        $this->pokracovat = isset($_POST['novyradekrozdeleni'])  &&  (int)$_POST['novyradekrozdeleni']; // pokracuje dale stejnym formularem
//
//        // nacte puvodni data do $this->dataList
//        $this ->nactiData_DB();
//
//        if (!isset($_POST['DenniVysledekR_COUNT'])) return false;
//
//
//    }

    /**
     * zkontroluje zadane hodnoty pro rozdeleni dochazky
     * @return bool
     */
    function save()
    {
        $this->jeZmena = false;
        $this->pokracovat = false;

        if (isset($_POST['novyradekrozdeleni'])) {
            // pokracuje dale stejnym formularem
            $this->pokracovat = true;
        }

        if (!isset($_POST['DenniVysledekR_COUNT'])) {
            return true;
        }

        //if ( !$this -> ZkontrolujRozdeleniVysledku() ) return false;
        // nejaka chyba - musi nejdrive opravit

        // po kontrole provede smazani zaznamu, ktere uzivatel zrusil (nebo zmenil rozdeleni)
        // pak updatne a doplni nove zaznamy

        if (!$this->nactiData_Post()) {
            return false;
        }

        $cnt = (int)$_POST['DenniVysledekR_COUNT']; // az za NactiData_Post!
        if (!$cnt) {
            // neni co ukladat
            return true;
        }
        //$rec = new RozdeleniDennichVysledkuRecord($this ->database, $this->typRozdeleni, $this->pp->rc, $this->pp->osobniCislo, $this->pp->datumNastupu, $this->den);

        $this->saveRecords();

        // doplni nove zaznamy
        $this->insertRecords();

        // pri vkladani si zapamatuje posledni casovou slozku zadanou uzivatelem
        if ($this->casovaSlozkaNovyZaznam) {
            NastaveniUzivatele::setUserSettingsItem($this ->database, NastaveniUzivatele::CASOVA_SLOZKA_ROZDELENI_DV, $this ->casovaSlozkaNovyZaznam);
        }

        $this->nactiData_DB();

        return true;
    }
}
