<?php
/**
 * Formular pro editaci zakladnich udaju elearningoveho kurzu
 */

namespace RonPortal\ELearning;

use Ron\Controls\FormField;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\SQLUtils;
use RonPortal\Controls\RonPortalForm;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;


class KurzFormular extends RonPortalForm
{
    public $recId = null; // nastavi se pri insertu


    function __construct($database, $formAction = 'new')
    {
        parent::__construct($database, $formAction);

        if (isset($_POST['Id'])) {
            $record = (int)$_POST['Id'];
        } else if (isset($_GET['record'])) {
            $record = (int)$_GET['record'];
        } else {
            $record = 0;
        }

        $this->htmlAction = '?menuid=kurzy';
        $this->stornoAction = '?menuid=kurzy' . ($record ? '&action=show&record=' . $record : '');

        $this ->code = 'formekurz';

        $this->title = \Ron\ronText('Kurz');
        $this->sqlTableName = 'EKurz';

        $this->sqlSelectPart =
            'Id, Nazev, Stav, Popis, Poznamka, PlatnostOd, PlatnostDo,' . // 0..6
            ' NULL, NULL'; // 7,8 - Revize.SestavaProCertifikat, Revize.UlozitCertifikatDoDok

        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('Id') ->setSqlFieldIndex(0) ->setPrimary(1)
            ->setCode('field-id');

        $this->addField()
            ->setFieldName('Nazev') ->setSqlFieldIndex(1)
            ->setCode('field-nazev')
            ->setCaption('Nazev') ->setMaxLength(160) ->setRequired(1);

        $this->addField(FormField::FIELD_TYPE_COMBO)
            ->setFieldName('Stav') ->setSqlFieldIndex(2)
            ->setCode('field-stav')
            ->setCaption('Stav_kurzu') ->setRequired(1)
            ->setOptions([0 => \Ron\ronText('Kurz_Stav_0'), 1 => \Ron\ronText('Kurz_Stav_1'), 2 => \Ron\ronText('Kurz_Stav_2')])
            ->setSelectDataTypeTest(false) ->setDefaultValue(0);

        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Popis') ->setSqlFieldIndex(3)
            ->setCode('field-popis')
            ->setCaption('Popis');

        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setSqlFieldIndex(4)
            ->setCode('field-poznamka')
            ->setCaption('Poznamka');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostOd') ->setSqlFieldIndex(5) ->setDefaultValue(date('j.n.Y'))
            ->setCode('field-platnostod')
            ->setCaption('Platnost_od') ->setRequired(1);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostDo') ->setSqlFieldIndex(6)
            ->setCode('field-platnostdo')
            ->setCaption('Platnost_do');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('SestavaProCertifikat') ->setSqlFieldIndex(7) ->setSqlFieldName(null) // POZOR! Pole je v tabulce RevizeEKurzu
            ->setCode('field-sestavaprocertifikat')
            ->setSelectSql('SELECT Kod, Nazev FROM SestavyProIntraweb ORDER BY Nazev')
            ->setCaption('Sestava_pro_certifikat') ->setShowEmptyValue(true);

        $this->addField(FormField::FIELD_TYPE_CHECK)
            ->setFieldName('UlozitCertifikatDoDok') ->setSqlFieldIndex(8) ->setSqlFieldName(null)  // POZOR! Pole je v tabulce RevizeEKurzu
            ->setCode('field-ulozitcertifikatdodok')
            ->setCaption('Ulozit_certifikat_do_dokumentu');
    }


    /**
     * @param int $idKurzu
     * @return array|null
     * @throws DBException
     */
    protected function getRevize($idKurzu)
    {
        // nacte ID platne revize
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Id, SestavaProCertifikat, UlozitCertifikatDoDok FROM RevizeEKurzu' .
            ' WHERE IdKurzu = ' . $params ->addParameter($idKurzu) .
            '  AND Stav < 2' . // jen aktivni, pripadne pripravovane
            ' ORDER BY PlatnostOd DESC', // prvni je nejnovejsi
            $params);

        $row = $cursor ->fetch();
        $cursor ->close();

        return $row;
    }


    function loadRecord()
    {
        $result = parent::loadRecord();

        if ($result) {
            // nacte podrobnosti z posledni revize
            $rowRevize = $this->getRevize($result[0]);
            $result[7] = $rowRevize[1]; // Revize.SestavaProCertifikat
            $result[8] = $rowRevize[2]; // Revize.UlozitCertifikatDoDok
        }

        return $result;
    }


    /**
     * @param int $idKurzu
     * @param int $platnostOd
     * @return RonPortalDBRecord
     */
    protected function createRevizeRec($idKurzu, $platnostOd)
    {
        $revizeRec = new RonPortalDBRecord($this ->database, 'RevizeEKurzu', $this);
        $revizeRec ->addIntField('ID', true);
        $revizeRec ->addIntField('IDKurzu', true, $idKurzu);
        $revizeRec ->addDateField('PlatnostOd', true, $platnostOd);
        $revizeRec ->addStringField('Popis', true, 160, \Ron\ronText('Revize') . ' 1');
        $revizeRec ->addStringField('Autor', true, 160, User::getUserName());
        $revizeRec ->addIntField('Stav', true, 1); // aktivni

        $revizeRec ->addStringField('SestavaProCertifikat', false, 40, $_POST['SestavaProCertifikat'] ?: null);
        $revizeRec ->addIntField('UlozitCertifikatDoDok', false, (isset($_POST['UlozitCertifikatDoDok'])  &&  $_POST['UlozitCertifikatDoDok']) ? 1 : 0);

        return $revizeRec;
    }


    protected  function insertRevize($idKurzu, $platnostOd)
    {
        $revizeRec = $this ->createRevizeRec($idKurzu, $platnostOd);
        $idField = $revizeRec ->findFieldByName('ID');

        $idField ->setNewValue( SQLUtils::newSerialNumber($this ->database, 'RevizeEKurzu', 'ID') );

        $revizeRec ->insertRecord();
    }

    /**
     * @param DBRecord $rec
     * @return bool
     */
    function afterInsertRecord($rec)
    {
        $this -> recId = $rec['Id'];

        $this ->insertRevize($this ->recId, $rec['PlatnostOd']);

        return parent::afterInsertRecord($rec);
    }


    function afterUpdateRecord($rec)
    {
        $rowRevize = $this ->getRevize($rec['Id']);
        $idRevize = $rowRevize ? $rowRevize[0] : null;

        if (isset($idRevize)) {
            $revizeRec = $this ->createRevizeRec($rec['ID'], $rec['PlatnostOd']);

            $params = $this->database->createParameters();
            $revizeRec->setWhere('Id = ' . $params ->addParameter($idRevize), $params);

            $revizeRec ->loadRecord();
            $revizeRec ->updateRecord();
        } else {
            $this ->insertRevize($rec['ID'], $rec['PlatnostOd']);
        }

        return parent::afterUpdateRecord($rec);
    }
}
