<?php

namespace RonPortal\ELearning;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\FileUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\Testy\TestDBRecord;


// TODO: volba pozadi
// TODO: zpusob prezentace - slidy / html prezentace
// TODO: preddefinovana hlavicka


class KurzyController extends JsonController
{
    const URL_DOWNLOAD = '?menuid=kurzy&%DOWNLOAD%';

    /** @var Database*/
    var $database;

    /**
     * @param Database $database
     */
    function __construct($database) {
        $this ->database = $database;
    }


    function viewElearning()
    {
        HtmlHeaders::getInstance()
            ->addScript('tinymce/tinymce.min.js')
            ->addCss('css/elearning.css')
            ->addScript('script/elearning.js');

        return 'ronportal/elearning/view_elearning.php';
    }


    /**
     * @param int    $id
     * @param string $action
     * @return string|ViewObject
     */
    function zobrazKurz($id, $action = '') {
        $kurz = new Kurz();

        $kurz->load($this ->database, $id);

//        switch ($action) {
//            case 'content' :
//                $kurz ->loadRevize($this ->database);
//                $kurz ->loadContent($this ->database);
//                break;
//
//            default :
//                $kurz ->loadRevize($this ->database);
//                break;
//        }
        $kurz ->loadRevize($this ->database);
        $kurz ->loadContent($this ->database);

        Model::addData('kurz', $kurz);
        Model::addData('revize', $kurz ->getRevize()[0]);
        Model::addData('action', $action);
        Model::addData('editace', true); // rezim editace - priznak pro layout

//        return new BaseViewObject('ronportal/elearning/view_kurz.php');
        Model::addViewObject( new BaseViewObject('ronportal/elearning/view_kurz.php') );

        return $this ->viewElearning();
    }


    /**
     * @return ViewObject
     * @throws \Exception
     */
    function zobrazKurzy()
    {
        $browser = new KurzBrowser($this ->database);
        $browser ->load();

        return $browser;
    }


    /**
     * @param string $formAction
     * @return Redirect|string
     * @throws NeniPravoException
     */
    function editKurz($formAction) {
        $id = isset($_POST['Id']) ? (int)$_POST['Id'] : 0;

        if ($formAction) {
            $pravo = Prava::vratHodnotuPrava('editacekurzu');
            if (!$pravo) {
                throw new NeniPravoException();
            }
            $f = new KurzFormular($this ->database, $formAction, MessageManager::getInstance());

            if ($f ->handleAction()) {
                // insert/update OK => show record
                if ($f ->recId) {
                    $id = $f ->recId;
                }
            } else {
                // zobrazi formular
                Model::addViewObject($f);
                return $this ->viewElearning();
            }
        }

        return new Redirect(null, '?menuid=kurzy' . ($id ? '&action=show&record=' . $id : ''));
    }


    private function pridejStranu($idRevize, $content = null)
    {
        $strana = new ObsahKurzu();
        $strana ->insertRecord($this ->database, $idRevize, $this, $content);

        return $strana;
    }


    /**
     * Vytvori novou stranku kurzu
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    private function newContent($output)
    {
        $idRevize = (int)$_POST['idRevize'];
        Csrf::validateCSRFPostParam('nova_strnka_' . $idRevize);

        $strana = $this ->pridejStranu($idRevize);
        $output ->addData('obsah', $strana ->getJson());
    }


    /**
     * @param int $id
     * @param RonPortalDBRecord $rec
     * @throws \Exception
     */
    private function updateKurz($id, $rec, $csrfPrefix = null)
    {
        $params = $this ->database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter($id), $params);

        if (!$rec ->loadRecord()) {
            throw new \Exception('Chyba - zadana stranka kurzu nebyla nalezena!');
        }

        if ($csrfPrefix) {
            Csrf::validateCSRFPostParam($csrfPrefix . $rec['IdRevizeKurzu']);
        }

        if (!$rec ->updateRecord()) {
            throw new \Exception('Chyba - pri zapisu stranky kurzu doslo k chybe!');
        }
    }


    private function odstranId($text, $searchStr, $replacement)
    {
        if ($text == '') return $text; // bez tohoto vyhazuje strpos warning!

        $pos = 1; // zamerne vynechava 1. znak - tam to stejne nema smysl
        while ( $pos = strpos($text, $searchStr, $pos) ) {
            $length = strlen($searchStr);
            while ($text[$pos + $length] >= '0'  &&  $text[$pos + $length] <= '9') {
                $length++;
            }

            $text = substr_replace($text, $replacement, $pos, $length);
            $pos += strlen($replacement);
        }

        return $text;
    }


    private function upravTextProUlozeni($text)
    {
        // upravi odkazy %DOWNLOAD%&revize=<id> - odstrani revize=<id>
        // link ve formatu %DOWNLOAD%&revize=<idrevize>&priloha=<id> se pouziva v rezimu editace
        // v rezimu prohlizeni se link zmeni
        // tohle harakiri se musi udelat, aby se daly kurzy exportovat (id revize se muze lisit)
        $text = $this ->odstranId($text, '%DOWNLOAD%&revize=', '%DOWNLOAD%');
        $text = $this ->odstranId($text, '%DOWNLOAD%&amp;revize=', '%DOWNLOAD%');
        return $text;
    }

    /**
     * JS - Ulozi obsah stranky kurzu
     * @param JsonOutputObject $output
     */
    private function saveContent($output)
    {
        $text = StringUtils::fromUTF8($_POST['obsah']);
        $text = $this ->upravTextProUlozeni($text);

        $rec = new RonPortalDBRecord($this ->database, 'ObsahEKurzu');

        $fieldObsah = $rec ->addMemoField('Obsah', false, $text);

        $rec ->addIntField('IdRevizeKurzu', true); // readonly pro zalogovani
        $rec ->addStringField('Nazev', true);
        $rec ->addStringField('KodStrany', true);

        $rec ->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName()) ->setUpdateLog(true);
        $rec ->addTimestampField('Update_Timestamp', false, time()) ->setUpdateLog(true);

        $this ->updateKurz((int)$_POST['id'], $rec, 'save_content_');

        $output ->addData('obsah', StringUtils::jsonText($fieldObsah ->value));
    }


    private function deleteContent()
    {
        $rec = new RonPortalDBRecord($this ->database, 'ObsahEKurzu');
        $rec ->addIntField('Id', true);
        $rec ->addIntField('IdRevizeKurzu', true);
        $rec ->addIntField('Stav', false, 2); // odstranena
        $rec ->addIntField('Poradi', false, -1);
        $rec ->addStringField('Nazev', true);
        $fObsah = $rec ->addMemoField('Obsah', true);      // readonly

        $rec ->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName()) ->setUpdateLog(true);
        $rec ->addTimestampField('Update_Timestamp', false, time()) ->setUpdateLog(true);

        $params = $this ->database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter((int)$_POST['id']), $params);

        if (!$rec ->loadRecord()) {
            throw new \Exception('Chyba - zadana stranka kurzu nebyla nalezena!');
        }

        Csrf::validateCSRFPostParam('seznam_stranek_' . $rec['IdRevizeKurzu']);

        if ($fObsah ->value == '') {
            // prazdny obsah => stranku uplne smaze
            $rec ->deleteRecord();
        } else {
            $rec ->updateRecord();
        }
    }


    /**
     * Ulozi vlastnosti stranky (nazev, kod atd.)
     */
    private function saveProperties()
    {
        $rec = new RonPortalDBRecord($this ->database, 'ObsahEKurzu');
        $rec ->addStringField('Nazev', false, 60, StringUtils::fromUTF8($_POST['nazev']));

        $kod = (isset($_POST['kodStrany'])  &&  trim($_POST['kodStrany']) != '')
            ? StringUtils::fromUTF8(trim($_POST['kodStrany']))
            : null;
        $rec ->addStringField('KodStrany', false, 30, $kod);

        if ( isset($_POST['zobrazitHlavicku']) ) {
            $rec ->addIntField('ZobrazitHlavicku', false, (int)$_POST['zobrazitHlavicku']);
        }

        if ( isset($_POST['zobrazitPaticku']) ) {
            $rec ->addIntField('ZobrazitPaticku', false, (int)$_POST['zobrazitPaticku']);
        }

        if ( isset($_POST['povolitPokracovatBezTestu']) ) {
            $rec ->addIntField('PovolitPokracovatBezTestu', false, (int)$_POST['povolitPokracovatBezTestu']);
        }

        if ( isset($_POST['temaCislo']) ) {
            $rec ->addIntField('DruhHodnoceni', false, (int)$_POST['temaCislo'] ? 6 : null);
            $rec ->addIntField('TemaCislo', false, (int)$_POST['temaCislo'] ? (int)$_POST['temaCislo'] : null);
        }

        $rec ->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName()) ->setUpdateLog(true);
        $rec ->addTimestampField('Update_Timestamp', false, time()) ->setUpdateLog(true);

        $this ->updateKurz((int)$_POST['id'], $rec, 'vlastnosti_strany_');
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function changeOrder($output)
    {
        if (!isset($_POST['newindex'])) {
            throw new \Exception('Chyba pri nastavovani poradi stranek - neni zadano nove poradi!');
        }

        $rec = new RonPortalDBRecord($this ->database, 'ObsahEKurzu');
        $fieldIdKurzu = $rec ->addField('IDRevizeKurzu', 'int', true);
        $fieldPoradi = $rec ->addField('Poradi', 'int', false, 0, (int)$_POST['newindex']);

        $id = (int)$_POST['id'];
        $this ->updateKurz($id, $rec, 'seznam_stranek_');

        $sql = '';
        $params = $this ->database ->createParameters();

        // jeste posune ostatni
        if ($fieldPoradi ->value > $fieldPoradi ->originalValue) {
            // posun polozky dale => ostatni posunu zpet
            $sql =
                'UPDATE ObsahEKurzu SET Poradi = Poradi - 1 ' .
                ' WHERE IDRevizeKurzu = ' . $params ->addParameter((int)$fieldIdKurzu ->originalValue) .
                '   AND Poradi > ' . $params ->addParameter((int)$fieldPoradi ->originalValue) .
                '   AND Poradi <= ' . $params ->addParameter((int)$fieldPoradi ->value) .
                '   AND ID <> ' . $params ->addParameter($id);
        } else if ($fieldPoradi ->value < $fieldPoradi ->originalValue) {
            // posun polozky zpet => ostatni posunu dale
            $sql =
                'UPDATE ObsahEKurzu SET Poradi = Poradi + 1 ' .
                ' WHERE IDRevizeKurzu = ' . $params ->addParameter((int)$fieldIdKurzu ->originalValue) .
                '   AND Poradi < ' . $params ->addParameter((int)$fieldPoradi ->originalValue) .
                '   AND Poradi >= ' . $params ->addParameter((int)$fieldPoradi ->value) .
                '   AND ID <> ' . $params ->addParameter($id);
        }

        if ($sql) {
            $this->database->sql($sql, $params);

            $params = $this ->database ->createParameters();
            $sql =
                'SELECT Id, Poradi' .
                ' FROM ObsahEKurzu' .
                ' WHERE IdRevizeKurzu = ' . $params ->addParameter((int)$fieldIdKurzu ->originalValue) .
                ' AND Stav = 1' .
                ' ORDER BY Poradi, Id';

            $items = [];
            $cursor = $this ->database ->getSqlResult($sql, $params);
            while ($row = $cursor ->fetch()) {
                $items[] = [ 'id' => $row[0], 'poradi' => $row[1] ];
            }
            $cursor ->close();

            $output ->addData('poradiStranek', $items);
        }
    }


    /**
     * @return array
     * @throws DBException
     * @throws \Exception
     */
    private function insertPriloha()
    {
        if (!$_FILES['upload_file']) {
            throw new \Exception( \Ron\ronText('Err_chyba_pri_nahravani_souboru') );
        }

        if ($_FILES['upload_file']['error'] == UPLOAD_ERR_INI_SIZE ||
            $_FILES['upload_file']['error'] == UPLOAD_ERR_FORM_SIZE
        ) {
            throw new \Exception( \Ron\ronText('Err_velky_soubor') );
        }
//var_dump($_FILES['upload_file']);
//        throw new \Exception('jsme tu -' . $_FILES['upload_file']['error'] . '-');

        if (!is_uploaded_file($_FILES['upload_file']['tmp_name'])) {
            throw new \Exception('Chyba! Neni zadan soubor!');
        }

        $fileName = (isset($_POST['filename'])  &&  $_POST['filename'])
            ? StringUtils::fromUTF8($_POST['filename'])
            : StringUtils::fromUTF8($_FILES['upload_file']['name']); // jmeno puvodniho souboru
        $tmpFileName = $_FILES['upload_file']['tmp_name']; // jmeno docasneho souboru (data pro upload do databaze)

        //$filename = ZkratJmenoSouboru($filename, 160);

        //$fp = fopen($tmpfilename, 'rb');
        $data = file_get_contents($tmpFileName);
        //$data = gzcompress($data);

        $newId = SQLUtils::newSerialNumber($this ->database, 'Priloha', 'Id');

        $rec = new RonPortalDBRecord($this ->database, 'Priloha', $this);
        $rec ->addIntField('Id', true, $newId);
        $rec ->addStringField('Nazev', true, 160, $fileName);
        $rec ->addField('Obsah', 'blob', true, 0, $data);

        $rec ->insertRecord();

        return ['id' => $newId, 'fileName' => $fileName];
    }


    /**
     * @param int $idPrilohy
     * @param int $idRevize
     * @return int
     * @throws DBException
     * @throws \Exception
     */
    private function pripojPrilohu($idPrilohy, $idRevize)
    {
        if (!$idRevize) {
            throw new \Exception('Chyba - neni zadana revize!');
        }

        $params = $this ->database ->createParameters();
        $newId = SQLUtils::newSerialNumber($this ->database,
            'PrilohaRevizeEKurzu', 'ID', 'IdRevizeKurzu = ' . $params ->addParameter((int)$idRevize),
            $params);

        $rec = new RonPortalDBRecord($this ->database, 'PrilohaRevizeEKurzu', $this);

        $rec ->addField('IDRevizeKurzu', 'int', false, 0, $idRevize);
        $rec ->addField('IDPrilohy', 'int', false, 0, $idPrilohy);
        $rec ->addField('ID', 'int', false, 0, $newId);
        $rec ->insertRecord();

        return $newId;
    }


    public static function getPrilohaUrl($revize, $id)
    {
        return self::URL_DOWNLOAD . '&revize=' . $revize . '&priloha=' . $id;
    }


    /**
     * Upload prilohy pomoci d&d
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function upload(JsonOutputObject $output)
    {
        $idRevize = (int)$_GET['revize'];
        Csrf::validateCSRFGetParam('upload_priloha_' . $idRevize);
        $prilohaInfo = $this->insertPriloha();

        $newId = $this ->pripojPrilohu($prilohaInfo['id'], $idRevize);
        $output ->addData('location', self::getPrilohaUrl($idRevize, $newId) );
        $output ->addData('revize', $idRevize);

        $prilohy = PrilohaKurzu::loadPrilohy($this ->database, $idRevize);

        $prilohyJson = [];
        foreach ($prilohy as $priloha) {
            $prilohyJson[] = $priloha ->getJson();
        }

        $output ->addData('prilohy', $prilohyJson);

        if ($_GET['createpage'] ?? false) {
            $type = FileUtils::getFileType($prilohaInfo['fileName']);

            switch ($type) {
                case 'image' :
                    $content = '<img src="?menuid=kurzy&%DOWNLOAD%&priloha=' . $newId . '" class="uploaded-page-image" />';
                    break;

                case 'video' :
                    $content =
                        '<p style="text-align: center;">' .
                        '<video controls="controls" width="640" height="480">' .
                        '<source src="?menuid=kurzy&%DOWNLOAD%&priloha=' . $newId . '" />' .
                        '</video>' .
                        '</p>';

                    break;

                default :
                    $content =
                        '<p>' .
                        '<a href="?menuid=kurzy&%DOWNLOAD%&priloha=' . $newId . '" target="_blank" >' . \Ron\ronText('Stahnout_prilohu') . '</a>' .
                        '</p>';
                    break;
//                    <p><a href="?menuid=kurzy&amp;%DOWNLOAD%&amp;revize=6&amp;priloha=17" target="_blank">Stahnout_prilohu</a></p>
            }

            $strana = $this ->pridejStranu($idRevize, $content);
            $output ->addData('strana', $strana ->getJson());
        }
    }


    /**
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    private function deletePriloha(JsonOutputObject $output)
    {
        $id = (int)$_POST['id']; // id v ramci kurzu
        $idRevize = (int)$_POST['revize'];
        Csrf::validateCSRFPostParam('delete_priloha_' . $idRevize);

        $rec = new RonPortalDBRecord($this ->database, 'PrilohaRevizeEKurzu', $this);
        $rec ->addField('IDRevizeKurzu', 'int', true);
        $fIdPrilohy = $rec ->addField('IDPrilohy', 'int', true);
        $rec ->addField('ID', 'int', true);

        $params = $this ->database ->createParameters();
        $rec ->setWhere(
            'IdRevizeKurzu = ' . $params ->addParameter($idRevize) .
            '  AND Id = ' . $params ->addParameter($id),
            $params);

        if (!$rec ->loadRecord()) {
            throw new \Exception('Pozadovany zaznam nebyl nalezen!');
        }

        $rec ->doDelete('');

        // Zjisti, jestli je priloha pripojena k jine revizi. Pokud ne, tak ji smaze z databaze
        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->select(
            'SELECT IdRevizeKurzu FROM PrilohaRevizeEKurzu' .
            ' WHERE IDPrilohy = ' . $params ->addParameter((int)$fIdPrilohy ->originalValue),
            $params);

        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            $rec = new RonPortalDBRecord($this ->database, 'Priloha', $this);
            $rec ->addField('ID', 'int', true);
            $rec ->addField('Nazev', '', true);

            $params = $this ->database ->createParameters();
            $rec ->setWhere('ID = ' . $params ->addParameter((int)$fIdPrilohy ->originalValue), $params);

            $rec ->deleteRecord();
        }
    }


    /**
     * @return OutputObject
     * @throws \Exception
     */
    private function download()
    {
        // url ve tvaru ...kurzosoby=<id_kurzu_osoby>&priloha=<id_prilohy>
        $idKurzuOsoby = (int)$_GET['kurzosoby'];
        $id = ELearningUtils::overPravoNaPrilohuKurzu($this ->database, $idKurzuOsoby, (int)$_GET['priloha']);

        return new PrilohaOutputObject($this ->database, $id);
    }


    /**
     * @return OutputObject
     * @throws \Exception
     */
    private function downloadPrilohaRevize()
    {
        return new PrilohaRevizeKurzuOutputObject($this ->database, (int)$_GET['revize'], (int)$_GET['priloha']);
    }


    /**
     * Vytvoreni noveho testu
     * @param JsonOutputObject $result
     * @throws DBException
     * @throws \Exception
     */
    function newTest($result)
    {
        $idRevize = (int)$_POST['idRevize'];

        Csrf::validateCSRFPostParam('novy_test_' . $idRevize);

        $kod = StringUtils::fromUTF8($_POST['kod']);
        $popis = StringUtils::fromUTF8($_POST['popis']);

        if ($kod === '') {
            throw new \Exception(\Ron\ronText('Neni_zadan_kod_testu'));
        }
        if ($popis === '') {
            throw new \Exception(\Ron\ronText('Neni_zadan_popis_testu'));
        }

        // vlozi zaznam do tabulky TemaHodnoceni
        $rec = new TestDBRecord($this->database, TestDBRecord::DRUH_HODNOCENI_TEST, null, $this);
        $rec->fKod->setNewValue($kod);
        $rec->fPopisTematu->setNewValue($popis);
        $rec->insertRecord();

        $rec2 = new RonPortalDBRecord($this->database, 'TestEKurzu', $this);
        $rec2->addField('IDRevize', 'int', false, 0, $idRevize);
        $rec2->addField('DruhHodnoceni', 'int', false, 0, TestDBRecord::DRUH_HODNOCENI_TEST);
        $rec2->addField('TemaCislo', 'int', false, 0, $rec->fTemaCislo->value);
        $rec2->insertRecord();

        // vrati aktualni seznam testu
        $testy = TestEKurzu::loadTesty($this->database, $idRevize);
        $json = [];
        foreach ($testy as $test) {
            $json[] = $test->getJson();
        }
        $result->addData('testy', $json);
    }


    /**
     * Nacte seznam priloh pro zadany kurz
     * @param JsonOutputObject $output
     */
    function prilohy(JsonOutputObject $output)
    {
        $revize = (int)($_GET['revize'] ?? 0);
        $prilohy = PrilohaKurzu::loadPrilohy($this ->database, $revize);

        $output ->addData('prilohy', PrilohaKurzu::getJsonPrilohy($prilohy));
    }


    function export()
    {
//        $kurz = new Kurz();
//
//        try {
//            $kurz->load($this ->database, $_GET['id']);
//        } catch (\Exception $e) {
//            return \Ron\RonPortal\errorMessageView($e ->getMessage());
//        }
//
//        $kurz ->loadRevize($this ->database);
//        $kurz ->loadContent($this ->database);

        throw new \Exception('Neni implementovano!');
    }


    function handleJsonAction($action, $output)
    {
        switch ($action) {
            case 'newcontent' :     // pridani nove stranky kurzu
                $this->newContent($output);
                break;

            case 'upload' :         // upload prilohy
                $this ->upload($output);
                break;

            case 'savecontent' :    // ulozeni stranky kurzu
                $this ->saveContent($output);
                break;

            case 'changeorder' :    // zmena poradi stranek
                $this ->changeOrder($output);
                break;

            case 'newtest' : // vytvoreni noveho testu
                $this ->newTest($output);
                break;

            case 'deletepriloha' :
                $this ->deletePriloha($output);
                break;

            case 'deletecontent' :  // smazani stranky kurzu
                $this ->deleteContent();
                break;

            case 'saveproperties' : // ulozeni vlastnosti stranky
                $this ->saveProperties();
                break;

            case 'prilohy' :
                $this ->prilohy($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return string|OutputObject|ViewObject
     * @throws \Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ELEARNING_POVOLEN)) {
            throw new \Exception('Elarning neni povolen!');
        }

        HtmlHeaders::getInstance() ->setAppTitle(\Ron\ronText('E_Learning'));

        TextManager::getInstance() ->loadStrings('elearning.ini');

        $this ->addTranslationsScript('kurzy');

        $action = '';
        if (isset($_GET['action'])) {
            $action = $_GET['action'];
        }

        if ($action == 'download') {
            return $this ->download();
        }

        if (!Prava::vratHodnotuPrava('editacekurzu')) {
            throw new NeniPravoException();
        }

        if (isset($_POST['formaction'])) {
            $formaction = $_POST['formaction'];
        } else if (isset($_GET['formaction'])) {
            $formaction = $_GET['formaction'];
        } else {
            $formaction = '';
        }

        if ($formaction) {
            // formular (editace, insert)
            return $this ->editKurz($formaction);
        }

        switch ($action) {
            case 'show' :
            case 'content' :
                return $this ->zobrazKurz($_GET['record'], $action);

            // javascripty
            case 'newcontent' :
            case 'upload' :
            case 'changeorder' :
            case 'savecontent' :
            case 'newtest' :
            case 'deletepriloha' :
            case 'deletecontent' :  // smazani stranky kurzu
            case 'saveproperties' : // ulozeni vlastnosti stranky
            case 'prilohy' :
                return parent::handleAction($action);


//            case 'uploadform' : // upload pomoci <form>
//                return $this ->uploadForm();

            case 'translations' :   // texty pro javascripty
                return $this ->translations();

            case 'download' :       // download prilohy
                return $this ->download();

            case 'export' :
                return $this ->export();
        }

        if (isset($_GET['%DOWNLOAD%'])) {
            // stahnuti prilohy v rezimu editace
            return $this ->downloadPrilohaRevize();
        }

        return $this ->zobrazKurzy();
    }

}
