<?php
/**
 * stanislav.elbl, 25.11.2015
 */

namespace RonPortal\ELearning;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\StringUtils;

class TestEKurzu
{
    /** @var Test */
    public $test;

    /** @var int */
    public $idRevize;
    /** @var int */
    public $druhHodnoceni;
    /** @var int */
    public $temaCislo;

    // Hodnoty z tabulky TestEKurzu
    /** @var int        TestEKurzu.PocetPokusu */
    public $pocetPokusu;
    /** @var int        TestEKurzu.CasovyLimit */
    public $casovyLimit;

    /** @var string     TemaHodnoceni.Kod */
    public $kod;
    /** @var string     TemaHodnoceni.PopisTematu */
    public $popis;

    // Hodnoty uzivatele
    public $vycerpanoPokusu;


    /**
     * Nacte data z radku z databaze
     * @param array $rowData
     */
    private function loadRow($rowData)
    {
        $this ->druhHodnoceni = (int)$rowData[0];
        $this ->temaCislo = (int)$rowData[1];

        $this ->kod = $rowData[2];
        $this ->popis = $rowData[3];

        $this ->pocetPokusu = $rowData[4];
        $this ->casovyLimit = $rowData[5];
    }


    /**
     * @param Database $database
     * @param int  $idRevize
     * @return TestEKurzu[]
     * @throws DBException
     */
    public static function loadTesty($database, $idRevize)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT TemaHodnoceni.DruhHodnoceni, TemaHodnoceni.TemaCislo, TemaHodnoceni.Kod, TemaHodnoceni.PopisTematu,' .
            '   TestEKurzu.PocetPokusu, TestEKurzu.CasovyLimit' .
            ' FROM TestEKurzu, TemaHodnoceni ' .
            ' WHERE TemaHodnoceni.DruhHodnoceni = TestEKurzu.DruhHodnoceni '.
            '   AND TemaHodnoceni.TemaCislo = TestEKurzu.TemaCislo '.
            '   AND TestEKurzu.IdRevize = ' . $params ->addParameter((int)$idRevize) .
            ' ORDER BY TemaHodnoceni.Kod';

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani pripojenych testu!');

        $result = [];
        while ($rowData = $cursor ->fetch()) {
            $test = new static();

            $test ->idRevize = (int)$idRevize;
            $test ->loadRow($rowData);

            $result[] = $test;
        }

        $cursor ->close();
        return $result;
    }



    /**
     * @param Database $database
     * @param int $idRevize
     * @param int $druhHodnoceni
     * @param int $temaCislo
     * @return TestEKurzu
     * @throws \Exception
     */
    public static function nactiTestEKurzu($database, $idRevize, $druhHodnoceni, $temaCislo)
    {
        $params = $database ->createParameters();

        $sql =
            'SELECT TemaHodnoceni.DruhHodnoceni, TemaHodnoceni.TemaCislo, TemaHodnoceni.Kod, TemaHodnoceni.PopisTematu,' .
            '   TestEKurzu.PocetPokusu, TestEKurzu.CasovyLimit' .
            ' FROM TestEKurzu, TemaHodnoceni ' .
            ' WHERE TestEKurzu.DruhHodnoceni = TemaHodnoceni.DruhHodnoceni' .
            '   AND TestEKurzu.TemaCislo = TemaHodnoceni.TemaCislo' .
            '   AND TestEKurzu.IdRevize = ' . $params ->addParameter($idRevize) .
            '   AND TestEKurzu.DruhHodnoceni = ' . $params ->addParameter($druhHodnoceni) .
            '   AND TestEKurzu.TemaCislo = ' . $params ->addParameter($temaCislo);

        $cursor = $database ->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new \Exception('TestEKurzu: zadany zaznam nebyl nalezen!');
        }

        $result = new static();

        $result ->idRevize = (int)$idRevize;
        $result ->loadRow($row);

        return $result;
    }


    /**
     * Doplni hodnoty podle vyplnenych testu
     * @param Database $database
     * @param KurzOsoby $kurzOsoby
     */
    public function loadPocetAbsolvovanych($database, $kurzOsoby)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT COUNT(*) FROM PripraveneTemaHodnoceni ' .
            ' WHERE EKurzOsoby = ' . $kurzOsoby ->id;

        $cursor = $database ->select($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        $this ->vycerpanoPokusu = $row ? $row[0] : 0;
    }


    /**
     * Vraci true, pokud je mozne spustit (dalsi) test
     * @return bool
     */
    public function lzeSpustitTest()
    {
        return !$this ->pocetPokusu || $this ->vycerpanoPokusu < $this ->pocetPokusu;
    }


    function getJson()
    {
        $data = $this ->test
            ? $this ->test ->getJsonData()
            : [
                'druhHodnoceni' => $this->druhHodnoceni,
                'temaCislo' => $this->temaCislo,
            ];

        $data['idRevize'] = $this ->idRevize;
        $data['kod'] = StringUtils::jsonText($this->kod);
        $data['popis'] = StringUtils::jsonText($this ->popis);

        if ($this ->pocetPokusu) {
            $data['pocetPokusu'] = $this ->pocetPokusu;
        }
        if ($this ->casovyLimit) {
            $data['casovyLimit'] = $this ->casovyLimit;
        }

        return $data;
    }
}
