<?php

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;


class PravidlaProTypJidla
{
    /** @var string     Kod typu denniho jidla */
    public $typDennihoJidla;

    /** @var bool       Priznak, jestli je mozne toto jidla pro aktualni cenovou kategorii objednavat */
    public $povolitObjednavky = true;

    public $omezit_PocetObjDen = 1;
    public $maxPocetObjNaDen = 0;

    public $pravidloVypoctuDotace = 0;
    public $pevnaCenaJidla = 0;
    public $dotace_Castka = 0;
    public $dotace_Procentni = 0;
    public $typZobrazeniCenObjednavek = 0;

    /** @var string     Seznam typu jidel, ktere musi byt objednane, aby slo objednat toto jidlo */
    public $objednatJenSTypemJidla;

    /** @var string     Seznam typu jidel, ktere musi byt objednane, aby slo objednat toto jidlo */
    public $neobjednatSTypemJidla;      // TODO: dodelat!

    /** @var string     Doplni se po nacteni - "inverzni" udaj k $objednatJenSTypemJidla */
    public $podminujeObjednavkuTypuJidla;


    /**
     * @param $typDennihoJidla
     */
    function __construct($typDennihoJidla)
    {
        $this->typDennihoJidla = $typDennihoJidla;
    }


    /**
     * Nacte hodnoty z radku
     * @param array $row
     */
    private function loadRow($row)
    {
        $this->omezit_PocetObjDen = (int)$row[1];
        $this->maxPocetObjNaDen = $row[2];

        $this->pravidloVypoctuDotace = (int)$row[3];
        $this->pevnaCenaJidla = $row[4];
        $this->dotace_Castka = $row[5];
        $this->dotace_Procentni = $row[6];
        $this->typZobrazeniCenObjednavek = (int)$row[7];

        $this->povolitObjednavky = (int)$row[8];
        $this->objednatJenSTypemJidla = $row[9];
        $this->neobjednatSTypemJidla = $row[10];
    }

    /**
     * Nacte zadany zaznam z databaze
     * @param Database $database
     * @param string $kodCenoveKategorie
     * @param int $platnostOd
     * @return bool
     * @throws DBException
     */
    function nactiZDatabaze($database, $kodCenoveKategorie, $platnostOd)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT PravidlaCenoveKategorie.TypDennihoJidla, PravidlaCenoveKategorie.Omezit_PocetObjDen, PravidlaCenoveKategorie.MaxPocetObjNaDen,' .
            '   PravidlaCenoveKategorie.PravidloVypoctuDotace, PravidlaCenoveKategorie.PevnaCenaJidla,' .
            '   PravidlaCenoveKategorie.Dotace_Castka, PravidlaCenoveKategorie.Dotace_Procentni,' .
            '   PravidlaCenoveKategorie.TypZobrazeniCenObjednavek, PravidlaCenoveKategorie.PovolitObjednavky,' .
            '   PravidlaCenoveKategorie.ObjednatJenSTypemJidla, PravidlaCenoveKategorie.NeobjednatSTypemJidla' .
            ' FROM PravidlaCenoveKategorie ' .
            ' WHERE PravidlaCenoveKategorie.CenovaKategorie = ' . $params->addParameter($kodCenoveKategorie) .
            '   AND PravidlaCenoveKategorie.PlatnostOd = ' . $params ->addDateParameter($platnostOd) .
            '   AND PravidlaCenoveKategorie.TypDennihoJidla = ' . $params->addParameter($this->typDennihoJidla);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return false;
        }

        $this ->loadRow($row);

        return true;
    }


    /**
     * Nacte pravidla pro vsechny typy dennich jidel pro zadanou CK
     * @param Database $database
     * @param string $kodCenoveKategorie
     * @param int   $platnostOd
     * @return PravidlaProTypJidla[]    Vraci asociativni pole kod => PravidlaProTypJidla
     */
    public static function nactiPravidlaProCK($database, $kodCenoveKategorie, $platnostOd)
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT PravidlaCenoveKategorie.TypDennihoJidla, PravidlaCenoveKategorie.Omezit_PocetObjDen, PravidlaCenoveKategorie.MaxPocetObjNaDen,' .
            '   PravidlaCenoveKategorie.PravidloVypoctuDotace, PravidlaCenoveKategorie.PevnaCenaJidla,' .
            '   PravidlaCenoveKategorie.Dotace_Castka, PravidlaCenoveKategorie.Dotace_Procentni,' .
            '   PravidlaCenoveKategorie.TypZobrazeniCenObjednavek, PravidlaCenoveKategorie.PovolitObjednavky,' .
            '   PravidlaCenoveKategorie.ObjednatJenSTypemJidla, PravidlaCenoveKategorie.NeobjednatSTypemJidla' .
            ' FROM PravidlaCenoveKategorie ' .
            ' WHERE PravidlaCenoveKategorie.CenovaKategorie = ' . $params->addParameter($kodCenoveKategorie) .
            '   AND PravidlaCenoveKategorie.PlatnostOd = ' . $params ->addDateParameter($platnostOd);

        /** @var PravidlaProTypJidla[] $result */
        $result = [];

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $pravidla = new PravidlaProTypJidla($row[0]);
            $pravidla ->loadRow($row);
            $result[$row[0]] = $pravidla;
        }
        $cursor ->close();

    foreach ($result as $typJidla => $pravidla) {
        if ($pravidla->objednatJenSTypemJidla) {
            $typy = explode(';', $pravidla->objednatJenSTypemJidla);
            foreach ($typy as $jenSTypemJidla) {
                if ($result[$jenSTypemJidla] ?? null) {
                    $result[$jenSTypemJidla]->podminujeObjednavkuTypuJidla =
                        ($result[$jenSTypemJidla]->podminujeObjednavkuTypuJidla
                            ? $result[$jenSTypemJidla]->podminujeObjednavkuTypuJidla . ';'
                            : '') . $typJidla;
                }
            }
        }
    }


        return $result;
    }


    /**
     * @param float $cenaJidla
     * @return float
     */
    function vratDotovanouCenu($cenaJidla)
    {
        switch ($this->pravidloVypoctuDotace) {

            case 1 :
                return $this->pevnaCenaJidla;
            case 2 :
                return $cenaJidla > $this->dotace_Castka ? $cenaJidla - $this->dotace_Castka : 0;
            case 3 :
                return $cenaJidla * (100 - $this->dotace_Procentni) / 100;

            case 0 :
            default :
                return $cenaJidla;
        }
    }
}




class CenovaKategorie
{
    public $rc;
    public $osobniCislo;
    public $datumNastupu;

    public $f_kod;

    public $f_platnostod;
    public $f_platnostdo;

    public $f_prirazeniod;
    public $f_prirazenido;

    public $f_ZpusobUctovani;
    public $f_MinimalniZustatek;

    public $f_Omezit_PocetObjDenCelkem;
    public $f_MaxPocetObjNaDenCelkem;

    /** @var PravidlaProTypJidla[]  $pravidlaProTypy */
    public $pravidlaProTypy;


    function __construct($rc, $osobniCislo, $datumNastupu)
    {
        $this ->rc = $rc;
        $this ->osobniCislo = $osobniCislo;
        $this ->datumNastupu = $datumNastupu;

        $this->clear();
    }



    function clear()
    {
        $this->f_kod = '';

        $this->f_platnostod = 0;
        $this->f_platnostdo = 0;
        $this->f_prirazeniod = 0;
        $this->f_prirazenido = 0;

        $this->f_Omezit_PocetObjDenCelkem = 0;
        $this->f_MaxPocetObjNaDenCelkem = 0;

        $this->pravidlaProTypy = [];
    }


    /**
     * Vraci true, pokud jsou nactene hodnoty platne k zadanemu datu
     * @param $datum
     * @return bool
     */
    function jePlatneKDatu($datum)
    {
        if (!$this->f_platnostod) {
            // jeste neni nacteno
            return false;
        }

        if (!$this->f_prirazeniod) {
            return false;
        }

        if ($this->f_platnostod > $datum) {
            return false;
        }
        if ($this->f_platnostdo && $this->f_platnostdo < $datum) {
            return false;
        }

        if ($this->f_prirazeniod > $datum) {
            return false;
        }
        if ($this->f_prirazenido && $this->f_prirazenido < $datum) {
            return false;
        }

        return true;
    }


    /**
     * @param Database $database
     * @param int $datum
     * @return bool
     * @throws DBException
     */
    function nactiVlastnostiKDatu($database, $datum)
    {
        $this->clear();
        $params = $database ->createParameters();
        $sql =
            'SELECT VlastnostCenoveKategorie.CenovaKategorie, ' .
            '   VlastnostCenoveKategorie.PlatnostOd, VlastnostCenoveKategorie.PlatnostDo, ' .
            '   RozdeleniOsob.PlatnostOd, RozdeleniOsob.PlatnostDo, ' .
            '   VlastnostCenoveKategorie.Omezit_PocetObjDenCelkem, VlastnostCenoveKategorie.MaxPocetObjNaDenCelkem, ' .
            '   VlastnostCenoveKategorie.ZpusobUctovani, VlastnostCenoveKategorie.MinimalniZustatek ' .
            ' FROM VlastnostCenoveKategorie, RozdeleniOsob ' .
            ' WHERE VlastnostCenoveKategorie.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND VlastnostCenoveKategorie.CenovaKategorie = RozdeleniOsob.Rozdeleni ' .
            '   AND VlastnostCenoveKategorie.PlatnostOd <= ' . $params ->addDateParameter($datum) .
            '   AND (VlastnostCenoveKategorie.PlatnostDo >= ' . $params ->addDateParameter($datum) . ' OR VlastnostCenoveKategorie.PlatnostDo IS NULL) ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->rc) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($this ->osobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->datumNastupu) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($datum) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($datum) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ';

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return false;
        }

        $this->f_kod = $row[0];

        $this->f_platnostod = $database ->stringToTimestamp($row[1]);
        $this->f_platnostdo = $row[2] ? $database ->stringToTimestamp($row[2]) : 0;
        $this->f_prirazeniod = $database ->stringToTimestamp($row[3]);
        $this->f_prirazenido = $row[4] ? $database ->stringToTimestamp($row[4]) : 0;

        $this->f_ZpusobUctovani = $row[7];
        $this->f_MinimalniZustatek = $row[8]; // pro kontrolu pri zalohovem uctovani

        $this->f_Omezit_PocetObjDenCelkem = $row[5];
        $this->f_MaxPocetObjNaDenCelkem = $row[6];

        $this->pravidlaProTypy = PravidlaProTypJidla::nactiPravidlaProCK($database, $this ->f_kod, $this ->f_platnostod);
        return true;
    }


    /**
     * @param $database
     * @param $datum
     * @return bool
     * @throws DBException
     */
    function overNacteniVlastnostiKDatu($database, $datum)
    {
        if ($this->jePlatneKDatu($datum)) {
            return true;
        }

        return $this->nactiVlastnostiKDatu($database, $datum);
    }


    /**
     * @param $database
     * @param $typDennihoJidla
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    function nactiPravidlaProTyp($database, $typDennihoJidla)
    {
        if (!$this->f_platnostod) {
            throw new Exception("Chyba pri nacitani pravidel pro typ jidla - nejsou nacteny vlasnosti cenove kategorie");
        }

        $pravidla = new PravidlaProTypJidla($typDennihoJidla);
        $result = $pravidla->nactiZDatabaze($database, $this->f_kod, $this->f_platnostod);
        $this->pravidlaProTypy[$typDennihoJidla] = $pravidla;

        return $result;
    }


//    /**
//     * POZNAMKA: uz by nemelo byt potreba - doplnil jsem nacteni vsech pravidel hned pri nacteni vlastnosti
//     * @param $database
//     * @param $typDennihoJidla
//     * @return bool
//     * @throws Exception
//     * @throws DBException
//     */
//    function overNacteniPravidelProTyp($database, $typDennihoJidla)
//    {
//        if ($this->pravidlaProTypy[$typDennihoJidla]) return true;
//
//        return $this->nactiPravidlaProTyp($database, $typDennihoJidla);
//    }


    /**
     * @param Database $database
     * @param float $pozadovanaCastka
     * @param int $den
     * @return bool
     * @throws DBException
     */
    function zkontrolujKreditStravnika($database, $pozadovanaCastka, $den)
    {
        if ($pozadovanaCastka <= 0) return true; // neni co kontrolovat

        if ($this->f_ZpusobUctovani == 1) {
            // zalohove uctovani => nasleduje kontrola kreditu
            $params = $database ->createParameters();
            $sql =
                'SELECT castka_AktualniStav FROM UcetStravnikaZaObdobi ' .
                ' WHERE RC = ' . $params->addParameter($this ->rc) .
                '   AND OsobniCislo = ' . $params->addParameter($this ->osobniCislo) .
                '   AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($this ->datumNastupu)) .
                '   AND Obdobi = ' . $params ->addParameter($database ->dateToParam(DateUtils::firstDayOfMonth($den)));

            $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri zjistovani aktualniho stavu uctu!');
            $row = $cursor ->fetch();
            $cursor ->close();

            if (!$row) {
                // neni vyuctovano aktualni obdobi - nelze zjistit stav uctu
                return false;
            }

            if ($row[0] - $pozadovanaCastka < $this->f_MinimalniZustatek) {
                // neni dost penez na uctu
                return false;
            }

            return true;
        } else {
            // srazkove uctovani - nic se nekontroluje
            return true;
        }
    }


    /**
     * @param string $typDennihoJidla
     * @return int
     * @throws DBException
     */
    function typZobrazeneCenyProTypJidla($typDennihoJidla)
    {
//        try {
//            if (!$this->overNacteniPravidelProTyp($database, $typDennihoJidla)) return 1;
//        } catch (DBException $e) {
//            throw $e;
//        } catch (Exception $e) {
//            // pripadnou vyjimku spolkne
//            MessageManager::addDebugMessage($e ->getMessage());
//            return 1;
//        }

        $pravidla = isset($this->pravidlaProTypy[$typDennihoJidla]) ? $this->pravidlaProTypy[$typDennihoJidla] : null;

        return $pravidla ? $pravidla->typZobrazeniCenObjednavek : 0;
    }


    /**
     * @param string $typDennihoJidla
     * @param float $cenaJidla
     * @return float
     * @throws DBException
     */
    function vratDotovanouCenuProTypJidla($typDennihoJidla, $cenaJidla)
    {
//        if (!$this->overNacteniPravidelProTyp($database, $typDennihoJidla)) {
//            return $cenaJidla;
//        }

        $pravidla = $this->pravidlaProTypy[$typDennihoJidla];

        return $pravidla ? $pravidla->vratDotovanouCenu($cenaJidla) : $cenaJidla;
    }


}
