<?php
/**
 * stanislav.elbl, 1.6.2016
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\TextManager;


/**
 * Class DetailObjednavky
 */
class DetailObjednavky
{
    /** @var int    Id objednavky */
    public $idObjednavky;
    /** @var int    Id polozky jidelnicku */
    public $idPolozkyJidelnicku;

    public $datumCasObjednani;
    public $datumCasZruseni;
    public $datumCasVydani;
    public $datumCasVlozeniNaBurzu;

    public $uzivatel;

    /** @var int    Id pracovniho pomeru, kteremu patri objednavka */
    public $idPP;
    /** @var string Jmeno stravnika */
    public $stravnikJmeno;

    /** @var string Popis jidla */
    public $popis;
    public $den;
    public $typDennihoJidla;
    public $cisloNabidky;
    public $popisNabidky;
    public $popisNabidky_Jazyk2;
    public $rozsirujiciPopis;
    public $kodSerialFoto;
    public $cenaJidla_A;
    public $jidelna;
    public $jidelnaKod;

    public $hodnoceniJidla;
    public $hodnoceniPoznamka;


    /**
     * @param Database $database
     * @param int $id
     * @return DetailObjednavky
     * @throws DBException
     * @throws Exception
     */
    public static function nactiDetail($database, $id)
    {
        $params = $database->createParameters();

        $sql =
            ' SELECT OJ.ID, OJ.IDPolozkyJidelnicku, ' .                       // 0-1
            '  OJ.DatumCasObjednani, OJ.DatumCasZruseni, OJ.DatumCasVydani, OJ.uzivatel, ' .    // 2-5
            '  Rozdeleni.Popis, PolozkaJidelnicku.den, ' .                                      // 6-7
            '  PolozkaJidelnicku.typDennihoJidla, PolozkaJidelnicku.cisloNabidky, ' .           // 8-9
            '  PolozkaJidelnicku.popisNabidky, PolozkaJidelnicku.popisNabidky_Jazyk2, ' .       // 10-11
            '  {fn_CASTWIDEMEMO(PolozkaJidelnicku.rozsirujiciPopis)}, ' .                       // 12
            '  PolozkaJidelnicku.kodSerialFoto, PolozkaJidelnicku.cenaJidla_A, ' .              // 13-14
            '  Rozdeleni.popis, HodnoceniJidla.Hodnoceni, {fn_CASTWIDEMEMO(HodnoceniJidla.Poznamka)}, ' . // 15-17
            '  TypDennihoJidla.Popis, PP.ID, Osoba.Prijmeni, Osoba.Jmeno, ' .                   // 18-21
            '  PolozkaJidelnicku.Jidelna, OJ.DatumCasVlozeniNaBurzu ' .                         // 22,23
            ' FROM  ObjednaneJidlo OJ INNER JOIN PolozkaJidelnicku ON (PolozkaJidelnicku.ID = OJ.IDPolozkyJidelnicku) ' .
            ' INNER JOIN PracovniPomer PP ON (PP.RC = OJ.RC AND PP.OsobniCislo = OJ.OsobniCislo AND PP.DatumNastupu = OJ.DatumNastupu)' .
            ' INNER JOIN Rozdeleni ON (PolozkaJidelnicku.jidelna = Rozdeleni.Kod  AND  PolozkaJidelnicku.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni)' .
            ' INNER JOIN TypDennihoJidla ON (TypDennihoJidla.Kod = PolozkaJidelnicku.typDennihoJidla)' .
            ' INNER JOIN Osoba ON (Osoba.RC = OJ.RC)' .
            ' LEFT OUTER JOIN HodnoceniJidla ON (HodnoceniJidla.RC = PP.RC)' .
            ' WHERE OJ.ID = ' . $params->addParameter($id);

        $cursor = $database->getSqlResult($sql, $params);

        if(!$row = $cursor->fetch()){
            throw new Exception(TextManager::_('Jidelna_objednavka_neexistuje'));
        }

        $cursor->close();

        $detailObjednavky = new DetailObjednavky();
        $detailObjednavky->nactiHodnotyZRadku($database, $row);

        return $detailObjednavky;
    }

    /**
     * @param Database $database
     * @param $row
     */
    private function nactiHodnotyZRadku($database, $row)
    {
        $this->idObjednavky = (int)$row[0];
        $this->idPolozkyJidelnicku = (int)$row[1];
        $this->datumCasObjednani = $database->stringToTimestamp($row[2]);
        $this->datumCasZruseni = $database->stringToTimestamp($row[3]);
        $this->datumCasVydani = $database->stringToTimestamp($row[4]);
        $this->uzivatel = $row[5];
        $this->popis = $row[6];
        $this->den = $database->stringToTimestamp($row[7]);
        $this->typDennihoJidla = $row[18];
        $this->cisloNabidky = (int)$row[9];
        $this->popisNabidky = $row[10];
        $this->popisNabidky_Jazyk2 = $row[11];
        $this->rozsirujiciPopis = $row[12];
        $this->kodSerialFoto = $row[13];
        $this->cenaJidla_A = (float)$row[14];
        $this->jidelna = $row[15];
        $this->hodnoceniJidla = isset($row[16]) ? (int)$row[16] : null;
        $this->hodnoceniPoznamka = $row[17];
        $this->idPP = (int)$row[19];
        $this->stravnikJmeno = $row[20] . ($row[21] ? ' ' . $row[21] : '');
        $this->jidelnaKod = $row[22];
        $this->datumCasVlozeniNaBurzu = $database ->stringToTimestamp($row[23]);
    }
}
