<?php
/**
 * UTF8
 * stanislav.elbl, 25.11.2015
 * roman.szablatura, 15.2.2017
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Log;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\System\FotoOutputObject;
use RonPortal\User;


class JidelnaController extends HttpController
{
    /** @var Database */
    var $database;


//    public static function getDatumJidelnicku()
//    {
//        return RonPortalSession::getValue('datumJidelnicku');
//    }
//
//    public static function setDatumJidelnicku($value)
//    {
//        RonPortalSession::setValue('datumJidelnicku', $value);
//    }

    /**
     * JidelnaController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this ->enableAccessWithoutLogin = true;
    }


    /**
     * @param $typPrava
     * @param $idPP
     * @throws Exception
     */
    function zkontrolujPrava($typPrava, $idPP)
    {
        $prava = Prava::vratPrava($typPrava);
        if (!$prava->lzeZobrazitPP($idPP)) {
            throw new NeniPravoException();
        }
    }


    /**
     * @return ViewObject
     * @throws Exception
     */
    function stravenky()
    {
        $b = new StravenkyBrowser($this->database);
        $b->load();

        $idPP = $b ->filterForm ->getFilterValue('osoba');
        $this->zkontrolujPrava('prohl_jidelna', $idPP);

        Model::addViewObject($b);

        $b = new VyuctovaniStravenekBrowser($this->database, $idPP);
        $b->load();

        return $b;
    }


    /**
     * @param PracovniPomer $pp P. pomer, pro ktery se zobrazi objednavky. Pokud je null, nacte se z filtru
     * @param bool $zobrazitObjednavky Priznak, ze se maji zobrazit pocty jidel a objednavani
     * @return BaseViewObject|string
     * @throws DBException
     */
    function zobrazJidelnicek($pp = null, $zobrazitObjednavky = true)
    {
        $view = new JidelnicekView($this ->database, $zobrazitObjednavky);
        $view ->load($pp);


//        $filterForm = new JidelnicekFilterForm($this ->database, $zobrazitObjednavky);
//        $filterForm ->loadData();
//        Model::addData('filterForm', $filterForm);
//
//        $cenovaKategorie = ($filterForm ->showOsoby  &&  $filterForm ->vybranaOsoba)
//            ? new CenovaKategorie($filterForm ->vybranaOsobaRC, $filterForm ->vybranaOsobaOsobniCislo, $filterForm ->vybranaOsobaDatumNastupu)
//            : null;
//
//        if ($zobrazitObjednavky) {
//            if (!$pp  ||  $pp ->idPP != $filterForm ->vybranaOsoba) {
//                $pp = PracovniPomer::nactiPP($this->database, $filterForm->vybranaOsoba);
//            }
//        }
//
//        $jidelnicek = new Jidelnicek($cenovaKategorie);
//        $jidelnicek ->load($this ->database, $filterForm ->getFilterValue('vyberjidelny'), $filterForm ->datumJidelnickuOd, $filterForm ->datumJidelnickuDo, $pp);
//        Model::addData('jidelnicek', $jidelnicek);

        GuiUtils::scrollPageHeader();

        if ($zobrazitObjednavky) {
            return $view; // BaseViewObject('ronportal/jidelna/view_jidelnicek.php');
        } else {
            // frame
            Model::addViewObject($view); // new BaseViewObject('ronportal/jidelna/view_jidelnicek.php'));
            return 'view/appview_frame.php';
        }
    }


    private function redirect($idPolozky) {
        return new Redirect(null, "?menuid=jidelnicek&polozka={$idPolozky}" . ($_POST['x'] ? '&x=' . (int)$_POST['x'] : '') . ($_POST['y'] ? '&y=' . (int)$_POST['y'] : ''));
    }


    /**
     * @return BaseViewObject|Redirect|string
     * @throws Exception
     */
    function objednejJidlo()
    {

        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze objednat jidlo, neni zadana polozka jidelnícku!');
        }

        // nacte p. pomer
        $ppomer = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        // nacte polozku
        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id, $ppomer);

        // zkontroluje prava
        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $prava = Prava::vratPrava('objednani');
        if (!$prava ->lzeZobrazitPP($ppomer ->idPP)) {
            throw new NeniPravoException();
        }


        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        try {
            $manager->objednejJidlo($polozkaJidelnicku);

            if (!MessageManager::getInstance()->isError()) { //RON-PR vyjimka mohla byt zachycena drive a jiz pridana jako chybova hlaska
                MessageManager::addSuccessMessage(sprintf(TextManager::_('Jidelna_uspesna_objednavka'), $polozkaJidelnicku->f_popisnabidky));
            }
            // OK => redirect na jidelnicek
            //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
            return $this ->redirect($polozkaJidelnicku ->f_id);
        } catch (Exception $e) {
            MessageManager::addErrorMessage($e);
        }

        // pri chybe nedela redirect
        return $this->zobrazJidelnicek($ppomer);
    }


    /**
     * @return BaseViewObject|Redirect|string
     * @throws Exception
     */
    function zrusObjednavku()
    {
        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze zrusit objednavku, neni zadana polozka jidelnicku!');
        }

        $ppomer = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id, $ppomer);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);
        $prava = Prava::vratPrava('objednani');
        if (!$prava ->lzeZobrazitPP($ppomer ->idPP)) {
            throw new NeniPravoException();
        }

        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        try {
            $manager ->zrusObjednavku($polozkaJidelnicku);

            if (!MessageManager::getInstance()->isError()) {
                MessageManager::addSuccessMessage(sprintf(TextManager::_('Jidelna_zrusena_objednavka'), $polozkaJidelnicku->f_popisnabidky));
            }

            //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
            return $this ->redirect($polozkaJidelnicku ->f_id);
        } catch (DBException $e) {
            MessageManager::addErrorMessage($e->getMessage(), null, 'alertMessage', $e->getErrorDescription());
            Log::addRecordToDBLog($this->database,
                'ObjednaneJidlo', 'ERROR', $this,
                StringUtils::fromUTF8('Chyba pri ruseni objednavky!') . "\n{$e ->getMessage()}\n{$e ->getErrorDescription()}");
            return $this->zobrazJidelnicek($ppomer);
        } catch (Exception $e) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba pri objednavani jidla! ') . $e->getMessage(), null, 'alertMessage');
            return $this->zobrazJidelnicek($ppomer);
        }
    }


    /**
     * @return BaseViewObject|Redirect
     * @throws Exception
     */
    function vlozObjednavkuDoBurzy()
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA)) {
            return new Redirect(null, '?menuid=jidelnicek');
        }

        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze vlozit objednavku do burzy! Neni zadana polozka jidelnicku.');
        }

        $ppomer = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id, $ppomer);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $prava = Prava::vratPrava('objednani');
        if (!$prava ->lzeZobrazitPP($ppomer ->idPP, $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den)) {
            throw new NeniPravoException();
        }

        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $manager ->nactiStavObjednavani($polozkaJidelnicku);
        $polozkaJidelnicku ->f_pocetobjednavekosoby = $manager ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);

        if (!$manager->lzePredatObjednavkuDoBurzy($polozkaJidelnicku)) {
            MessageManager::addErrorMessage(TextManager::_('Objednavka_nebyla_nalezena'), null, 'alertMessage');
            return new Redirect(null, "?menuid=jidelnicek#{$polozkaJidelnicku ->f_den}");
        }

        try {
            Burza::vlozPolozkuDoBurzy($this->database, $ppomer, $polozkaJidelnicku);
            if (!MessageManager::getInstance()->isError()) {
                MessageManager::addSuccessMessage(sprintf(TextManager::_('Jidelna_uspesne_vlozeni_burzy'), $polozkaJidelnicku->f_popisnabidky));
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba pri vkladani jidla do burzy! ') . $e->getMessage(), null, 'alertMessage');
            return $this->zobrazJidelnicek($ppomer);
        }

        //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
        return $this ->redirect($polozkaJidelnicku ->f_id);
    }


    /**
     * @return BaseViewObject|Redirect|string
     * @throws DBException
     * @throws Exception
     */
    function vyzvedniObjednavkuZBurzy()
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::BURZA_POVOLENA)) {
            return new Redirect(null, '?menuid=jidelnicek');
        }

        $id = (int)$_POST['polozka'];
        if (!$id) {
            throw new Exception('Chyba: Nelze vlozit objednavku do burzy! Neni zadana polozka jidelnicku.');
        }

        $ppomer = PracovniPomer::nactiPP($this ->database, (int)$_POST['osoba']);

        $polozkaJidelnicku = new PolozkaJidelnicku();
        $polozkaJidelnicku->nactiPolozkuACelkovyPocetObjednavek($this->database, $id, $ppomer);

        Prava::nastavObdobiProPrava($polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $prava = Prava::vratPrava('objednani');
        if (!$prava ->lzeZobrazitPP($ppomer ->idPP, $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den)) {
            throw new NeniPravoException();
        }

        $manager = new ObjednavkyManager($this ->database);
        $manager ->nactiData($ppomer,
            //$ppomer ->rc, $ppomer ->osobniCislo, $ppomer ->datumNastupu, $ppomer->idPP,
            $polozkaJidelnicku->f_den, $polozkaJidelnicku->f_den);

        $manager ->nactiStavObjednavani($polozkaJidelnicku);
        $polozkaJidelnicku ->f_pocetobjednavekosoby = $manager ->pocetObjednavekOsobyProPolozku($polozkaJidelnicku ->f_id);

        if (!$manager->lzeVyzvednoutPolozkuZBurzy($polozkaJidelnicku)) {
            MessageManager::addErrorMessage(TextManager::_('Objednavka_nebyla_nalezena'), null, 'alertMessage');
            return new Redirect(null, "?menuid=jidelnicek#{$polozkaJidelnicku ->f_den}");
        }

        try {
            Burza::vyzvedniZBurzy($this->database, $ppomer, $polozkaJidelnicku);
            if (!MessageManager::getInstance()->isError()) {
                MessageManager::addSuccessMessage(sprintf(TextManager::_('Jidelna_uspesne_vyzvednuti_burzy'), $polozkaJidelnicku->f_popisnabidky));
            }
        } catch (Exception $e) {
            MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba při vyzvednutí jídla z burzy! ') . $e->getMessage(), null, 'alertMessage');
            return $this->zobrazJidelnicek($ppomer);
        }

        //return new Redirect(null, "?menuid=jidelnicek&polozka={$polozkaJidelnicku ->f_id}#{$polozkaJidelnicku ->f_den}");
        return $this ->redirect($polozkaJidelnicku ->f_id);
    }


    /**
     * @param int $idpolozky
     * @return null|FotoOutputObject
     * @throws DBException
     * @throws Exception
     */
    private function fotoPolozkyJidelnicku($idpolozky)
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT KodSerialFoto FROM PolozkaJidelnicku WHERE ID = " . $params->addParameter($idpolozky);

        $cursor = $this->database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            return new FotoOutputObject($this->database, $row[0]);
        }
        $cursor->close();

        return null;
    }


    /**
     * @return InternalRedirect|ViewObject|FotoOutputObject|string
     * @throws DBException
     * @throws Exception
     */
    public function jidelnicek()
    {
        if (isset($_GET['action']) && $_GET['action'] == 'zobrazfoto') {
            return $this->fotoPolozkyJidelnicku((int)$_GET['idpolozky']);
        }

        HtmlHeaders::getInstance() ->setTitle( TextManager::_('RON_Portal') . ' - ' . TextManager::_('Jidelnicek'));

        if (!isset($_GET['frame'])) {
            if (!User::userAuthenticated()) {
                return new InternalRedirect('login');
            }

            if (isset($_POST['action'])) {
                switch ($_POST['action']) {
                    case 'objednatjidlo' :
                        return $this->objednejJidlo();

                    case 'zrusitobjednavku' :
                        return $this->zrusObjednavku();

                    case 'objednavkadoburzy' :
                        return $this->vlozObjednavkuDoBurzy();

                    case 'objednavkazburzy' :
                        return $this->vyzvedniObjednavkuZBurzy();

                    default :
                        MessageManager::addErrorMessage("Neznama akce! ({$_POST['action']})");
                }
            }

            return $this->zobrazJidelnicek();
        } else {
            return $this->zobrazJidelnicek(null, false);
        }
    }

    /**
     * @param string $action
     * @return ViewObject|OutputObject|Redirect|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('jidelna.ini');

        HtmlHeaders::getInstance()->addScript('script/jidelnascripts.js');
        HtmlHeaders::getInstance()->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_jidelna.png');
        HtmlHeaders::getInstance()->addCss('css/jidelna.css');

        // akce povolene bez prihlaseni
        switch ($action) {
            case 'jidelnicek' :
                return $this->jidelnicek();
        }

        if (!User::userAuthenticated()) {
            return new Redirect(null, '?menuid=logout');
        }

        switch ($action) {
            case 'stravenky' :
                return $this->stravenky();
        }

        throw new Exception('Chyba v metode JidelnaController::handleAction - neznama akce!');
    }
}
