<?php
/**
 * UTF8
 * stanislav.elbl, 24.12.2015
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\User;

class JidelnaJSController extends JsonController
{
    /** @var Database */
    var $database;

    /**
     * JidelnaJSController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;
    }


    /**
     * Nacte polozku jidelnicku pro javascript
     * @param JsonOutputObject $result
     * @throws Exception
     */
    public function nactiPolozkuJidelnicku($result)
    {
        $polozka = new PolozkaJidelnicku();
        $polozka->nactiPolozku($this ->database, (int)$_GET['id']);
        $polozka->nactiAlergeny($this ->database);

        $polozka->nactiHodnoceni($this ->database);

        if ($polozka->f_id) {
            $result->addData('idpolozky', $polozka->f_id);
            $result->addText('den', date('j.n.Y', $polozka->f_den));
            $result->addText('popis', $polozka->f_popisnabidky);
            $result->addData('hodnoceni', $polozka->hodnoceni);
            $result->addText('hodnocenipoznamka', $polozka->hodnoceniPoznamka);
            $result->addData('idfotografie', $polozka->kodFotografie);

            $alergeny = [];

            if (is_array($polozka->alergeny)) {
                /** @var Alergen $alergen */
                foreach ($polozka->alergeny as $alergen) {
//                        $alergen->nactiPolozku($this ->database);

                    $alergen_info = [
                        'kod' => (int)$alergen->kod,
                        'popis' => StringUtils::jsonText($alergen->popis)
                        // 'poznamka' => StringUtils::jsonText($alergen->poznamka)  nepouziva se, neni nacteno!
                    ];
                    $alergeny[] = $alergen_info;
                }
            }

            $result->addData('alergeny', $alergeny);

        } else {
            $result->addError('Chyba - nelze nacist polozku jidelnicku!');
        }
    }


    /**
     * Zapise hodnoceni jidla z POSTu
     * @throws Exception
     */
    function zapisHodnoceniJidla()
    {
        $polozka = new PolozkaJidelnicku();
        $polozka->nactiPolozku($this ->database, (int)$_POST['idpolozky']);
        $polozka->nactiHodnoceni($this ->database);
        $polozka->nactiHodnoceniPost();

        $polozka->zapisHodnoceni($this ->database, $this);
    }


    /**
     * Nacte objednavky uzivatele pro dashboard
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function mojeObjednavky($output)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_JIDELNICEK)) {
            $output ->addText('html', 'Jidelnicek neni povolen');
            return;
        }

        $html = '<table class="rowOver clickHref clickLink" data-ron-href="?menuid=jidelnicek">';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT ObjednaneJidlo.Id,' .
            ' PolozkaJidelnicku.Id, PolozkaJidelnicku.Den, PolozkaJidelnicku.TypDennihoJidla, PolozkaJidelnicku.PopisNabidky,' .
            ' ObjednaneJidlo.DatumCasVydani, TypDennihoJidla.Popis, TypDennihoJidla.Poradi' .
            ' FROM ObjednaneJidlo, PolozkaJidelnicku, TypDennihoJidla' .
            ' WHERE PolozkaJidelnicku.Id = ObjednaneJidlo.IdPolozkyJidelnicku' .
            '  AND PolozkaJidelnicku.TypDennihoJidla = TypDennihoJidla.Kod' .
            '  AND ObjednaneJidlo.DatumCasZruseni IS NULL' .
            '  AND ObjednaneJidlo.RC = ' . $params ->addParameter(User::getRC()) .
            '  AND PolozkaJidelnicku.Den >= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
            '  AND PolozkaJidelnicku.Den <= ' . $params ->addParameter($this ->database ->dateToParam(DateUtils::incDay(time(), 7))) .
            ' ORDER BY PolozkaJidelnicku.Den, TypDennihoJidla.Poradi, PolozkaJidelnicku.TypDennihoJidla, PolozkaJidelnicku.Id';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $lastDate = null;
        while ($row = $cursor ->fetch()) {
            $datum = $cursor ->stringToTimestamp($row[2]);
            $datumStr = ($datum != $lastDate) ? StringUtils::format('finedate', $datum) : '';

            $lastDate = $datum;

            $html .=
                '<tr class="rowOver">' .
                '<td>' . $datumStr . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[4]) . '</td>' .
                '</tr>';
        }
        $cursor ->close();

        $html .= '</table>';

        $output ->addText('html', $html);
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
            case 'loaddata' :
                $this ->nactiPolozkuJidelnicku($output);
                break;

            case 'hodnotit' :
                $this ->zapisHodnoceniJidla();
                break;

            case 'mojeobjednavky' :
                $this ->mojeObjednavky($output);
                break;

            default :
                throw new Exception('Neznama akce!');
        }
    }


}
