<?php
/**
 * stanislav.elbl, 26.12.2015
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\Http\HttpResponse;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;



class DatumJidelnickuFilter extends FilterFormFilter
{
    public function __construct()
    {
        parent::__construct('datum');
        $this ->sessionName = 'datumJidelnicku';
        $this ->defaultValue = DateUtils::firstDayOfWeek(time());
    }

    public function render()
    {
        $prevDateStr = DateUtils::dateToUrlString(DateUtils::firstDayOfWeek(DateUtils::incDay($this->value, -1)));
        $nextDateStr = DateUtils::dateToUrlString(DateUtils::firstDayOfWeek(DateUtils::incDay($this->value, 8)));

        $urlParams =
            (isset($_GET['frame']) ? '&frame=1' : '') .
            (isset($_GET['vyberjidelny']) ? '&vyberjidelny=' . $_GET['vyberjidelny'] : '');

        $aktualniTyden = DateUtils::firstDayOfWeek(time());

        echo "<div class='padding-all-2'>";

        // o tyden zpet
        echo "<span class='margin-right-2'><a href='?menuid=jidelnicek&datum=$prevDateStr$urlParams'><i class='fa glyph-left inline-glyph glyph-button'></i></a></span>";

        // aktualne zvolene datum
        echo ($this->value === $aktualniTyden)
            ? TextManager::_('Aktualni_tyden')
            : date('j.n.Y', $this->value);

        // o tyden dal
        echo "<span class='margin-left-2'><a href='?menuid=jidelnicek&datum=$nextDateStr$urlParams'><i class='fa glyph-right inline-glyph glyph-button'></i></a></span>";

        if ($this->value !== $aktualniTyden) {
            $aktualniTydenStr = DateUtils::dateToUrlString($aktualniTyden);

            echo
                '<span class="margin-left-2">' .
                "<a href='?menuid=jidelnicek&datum=$aktualniTydenStr$urlParams'>" . TextManager::_('Aktualni_tyden') . '</a>' .
                '</span>';
        }

        echo "</div>\n";
    }


    public function load(Database $database)
    {
        if (isset($_GET['datum'])) {
            try {
                $this ->value = DateUtils::firstDayOfWeek(DateUtils::dateStringToTimestamp($_GET['datum']));
                $this ->storeValue($database);
            } catch (Exception $e) {
                $this ->loadDefaultValue($database);
            }
        } else {
            $this ->loadDefaultValue($database);
        }
    }
}


class JidelnaFilter extends FilterFormFilter
{
    public function __construct()
    {
        parent::__construct('vyberjidelny');

        // sql dotaz se nastavi az dodatecne
        $this
            ->setOptionsSqlOrder('Popis')
            ->setOptionsSqlFilter('1 = 1');
    }

    /**
     * Zjisti, zda ma zamestnanec nastavenou jednu hlavni jidelnu. Pokud ano, tak vraci jeji kod.
     * Pokud nema zadnou hlavni nebo ma vice hlavnich, vraci null.
     * POZOR! Uz musi byt nastaven zamestnanec!
     * @return string|null
     * @throws DBException
     */
    private function vratHlavniJidelnuZamestnance($database)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT Rozdeleni.kod, Rozdeleni.popis, RozdeleniOsob.Hlavni, RozdeleniOsob.PlatnostOd ' .
            ' FROM Rozdeleni, RozdeleniOsob ' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($this ->filterForm ->datumJidelnickuDo) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($this ->filterForm ->datumJidelnickuOd) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($this ->filterForm ->datumJidelnickuOd) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            '   AND RozdeleniOsob.Hlavni = 1' .
            '   AND (' . $this ->optionsSqlFilter  . ')' .
            ' ORDER BY RozdeleniOsob.Hlavni DESC, RozdeleniOsob.PlatnostOd DESC';

        $cursor = $database ->getSqlResult($sql, $params);

        $pouzitHlavni = true;
        $hlavniJidelna = null;

        while ($pouzitHlavni  &&  $row = $cursor->fetch()) {
            if (!isset($hlavniJidelna)) {
                // prvni hlavni jidelna
                $hlavniJidelna = $row[0];
            } else {
                // je vice hlavnich jidelen
                $pouzitHlavni = false;
                $hlavniJidelna = null;
            }
        }

        $cursor ->close();

        return $hlavniJidelna;
    }


    public function load(Database $database)
    {
        parent::load($database);

        if (!isset($this ->value)  &&  $this ->filterForm ->getFilterValue('osoba')) {
            $this ->value = $this ->vratHlavniJidelnuZamestnance($database) ?? $_COOKIE['jidelna_jidelna'] ?? null;
        }

        if (($_GET[$this ->name] ?? null) === $this ->value) {
            // cookie nastavi jen v pripade, ze lze vybrat jidelnu, kterou zvolil uzivatel
            HttpResponse::setCookie('jidelna_jidelna', $this ->value, time() + 30000000);
        }
    }


    public function loadOptions(Database $database): FilterFormFilter
    {
        if (!$this ->filterForm ->getFilterValue('osoba')) {
            // neni vybrana osoba => napriklad ve frame
            $params = $database->createParameters();
            $this->optionsSql = new SqlStatement(
                'SELECT Rozdeleni.Kod, Rozdeleni.Popis, 0 ' .
                ' FROM Rozdeleni ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($this->filterForm ->datumJidelnickuOd) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                '   AND (%FILTER%)' .
                ' ORDER BY %ORDER%',
                $params
            );
        } else if (!$this ->optionsSql) {
            $params = $database->createParameters();
            $this->optionsSql = new SqlStatement(
                'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.Hlavni ' .
                ' FROM Rozdeleni, RozdeleniOsob ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
                '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params->addDateParameter($this->filterForm ->getFilterValue('osoba', 'datumNastupu')) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($this->filterForm ->datumJidelnickuDo) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($this->filterForm ->datumJidelnickuOd) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($this->filterForm ->datumJidelnickuOd) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
                '   AND (%FILTER%)' .
                ' ORDER BY %ORDER%',
                $params
            );
        }

        parent::loadOptions($database);

        return $this;
    }
}


class JidelnicekFilterForm extends RonPortalFilterForm
{
    /** @var DatumJidelnickuFilter */
    public $datumJidelnickuFilter;

    public $jidelny = [];

    /** @var int */
    public $datumJidelnickuOd;
    /** @var int */
    public $datumJidelnickuDo;

    /** @var array Seznam typu jidel: key = kod typu jidla, value = popis typu jidla */
    private $typyJidel = [];
    /** @var array vybrane typy, jen kody */
    private $vybraneTypy = [];

//    public $vybranaJidelna = null;

    function __construct($database, $zobrazitObjednavky)
    {
        parent::__construct($database, 'jidelnicek', true);

        $this ->getAction = $_SERVER['SCRIPT_NAME']; // kvuli odstraneni #..., jinak by nebylo potreba

        if ($zobrazitObjednavky) {
            $prava = Prava::vratPrava('objednani');

//            if ($prava->jsouNejakePrava()) {
//                $this->nastavOsoby(true, $prava, time(), time(), true); // vcetne externich = true
//            }
            $this ->addFilterPP(false, $prava)
                ->setVcetneExternich(true)
                ->setSessionName('jidelnicek_vybranaosoba');
                //->setSessionName('');

            $this ->addFilter(new StavUctuFilter());
        } else {
            $this ->skrytTlacitkoNastavit = true;
        }

        $this ->addFilter(new JidelnaFilter());

        if ($_GET['frame'] ?? false) {
            $this ->addFilter(
                (new FilterFormFilter('frame', FilterFormFilter::TYPE_HIDDEN))
                    -> setDefaultValue('1')
            );
        }

        $this ->datumJidelnickuFilter = (new DatumJidelnickuFilter())
            ->setPosition(FilterFormFilter::POSITION_BELOW_FORM);
        $this ->addFilter($this ->datumJidelnickuFilter);
    }

    protected function loadFilters()
    {
        $this ->datumJidelnickuFilter ->load($this ->database);
        $this ->datumJidelnickuOd = $this ->datumJidelnickuFilter ->getValue();
        $this ->datumJidelnickuDo = DateUtils::incDay($this ->datumJidelnickuFilter ->getValue(), 6);

        parent::loadFilters();
    }


    /**
     * @throws DBException
     */
    private function loadTypyJidel()
    {
        $params = $this ->database ->createParameters();

        if ($this ->getFilterValue('osoba')) {
            // nacte jen typy jidel povolene v cenove kategorii
            $sql =
                'SELECT DISTINCT TypDennihoJidla.Kod, TypDennihoJidla.Popis, TypDennihoJidla.Poradi' .
                ' FROM TypDennihoJidla, PravidlaCenoveKategorie, RozdeleniOsob, VlastnostCenoveKategorie' .
                " WHERE TypDennihoJidla.Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
                // odfiltruje typy jidel, ktere se namaji zobrazovat
                '  AND TypDennihoJidla.Kod = PravidlaCenoveKategorie.TypDennihoJidla' .
                '  AND PravidlaCenoveKategorie.PovolitObjednavky = 1' .
                // jen pravidla platna v zadanem rozsahu dnu ...
                '  AND PravidlaCenoveKategorie.KodTypuRozdeleni = VlastnostCenoveKategorie.KodTypuRozdeleni' .
                '  AND PravidlaCenoveKategorie.CenovaKategorie = VlastnostCenoveKategorie.CenovaKategorie' .
                '  AND PravidlaCenoveKategorie.PlatnostOd = VlastnostCenoveKategorie.PlatnostOd' .
                '   AND VlastnostCenoveKategorie.PlatnostOd <= ' . $params ->addDateParameter($this ->datumJidelnickuDo) .
                '   AND (VlastnostCenoveKategorie.PlatnostDo >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR VlastnostCenoveKategorie.PlatnostDo IS NULL) ' .
                // ... prirazena aktualne vybrane osobe
                '   AND VlastnostCenoveKategorie.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '   AND VlastnostCenoveKategorie.CenovaKategorie = RozdeleniOsob.Rozdeleni ' .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->getFilterValue('osoba', 'rc')) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($this ->getFilterValue('osoba', 'osobniCislo')) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->getFilterValue('osoba', 'datumNastupu')) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($this ->datumJidelnickuDo) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                // seradit podle poradi
                ' ORDER BY TypDennihoJidla.Poradi';
        } else {
            $sql =
                'SELECT Kod, Popis FROM TypDennihoJidla' .
                " WHERE Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
                ' ORDER BY Poradi';
        }

        $cursor = $this->database->getSqlResult($sql, $params);

        //nutne pro porovnani pri ukladani uz. nastaveni, v poli musi byt kody
        $vsechnyTypy = [];

        while ($row = $cursor->fetch()) {
            if (isset($_GET['jidelnicek_typ_' . $row[0]])) {
                $this->vybraneTypy[] = $row[0];
            }
            $vsechnyTypy[] = $row[0];

            $this->typyJidel[$row[0]] = $row[1];
        }

        //nastaveni se ulozi jen pri odeslani filtru (je nastavena osoba)
        if (isset($_GET['osoba'])) {
            NastaveniUzivatele::setUserSettingsItem($this->database, 'jidelnicek_vybrane_typy', implode(';', $this->vybraneTypy), implode(';', $vsechnyTypy));
        }
        //jinak se vycte z uzivatelskeho nastaveni
        else {
            $nastaveniTypu = NastaveniUzivatele::getUserSettingsItem($this->database, 'jidelnicek_vybrane_typy');
            if ($nastaveniTypu != '') {
                $this->vybraneTypy = explode(';', $nastaveniTypu);
            }
        }
    }

//    /**
//     * Zjisti, zda ma zamestnanec nastavenou jednu hlavni jidelnu. Pokud ano, tak vraci jeji kod.
//     * Pokud nema zadnou hlavni nebo ma vice hlavnich, vraci null.
//     * POZOR! Uz musi byt nastaven zamestnanec!
//     * @return string|null
//     * @throws DBException
//     */
//    private function vratHlavniJidelnuZamestnance()
//    {
//        $params = $this->database->createParameters();
//
//        $sql =
//            'SELECT Rozdeleni.kod, Rozdeleni.popis, RozdeleniOsob.Hlavni, RozdeleniOsob.PlatnostOd ' .
//            ' FROM Rozdeleni, RozdeleniOsob ' .
//            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
//            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
//            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
//            '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->vybranaOsobaRC) .
//            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->vybranaOsobaOsobniCislo) .
//            '   AND RozdeleniOsob.DatumNastupu = ' . $params->addDateParameter($this ->vybranaOsobaDatumNastupu) .
//            '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter($this ->datumJidelnickuDo) .
//            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter($this ->datumJidelnickuOd) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
//            '   AND (Rozdeleni.DatumZruseni >= ' . $params->addDateParameter($this ->datumJidelnickuOd) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
//            '   AND RozdeleniOsob.Hlavni = 1' .
//            ' ORDER BY RozdeleniOsob.Hlavni DESC, RozdeleniOsob.PlatnostOd DESC';
//
//        $cursor = $this ->database ->getSqlResult($sql, $params);
//
//        $pouzitHlavni = true;
//        $hlavniJidelna = null;
//
//        while ($pouzitHlavni  &&  $row = $cursor->fetch()) {
//            if (!isset($hlavniJidelna)) {
//                // prvni hlavni jidelna
//                $hlavniJidelna = $row[0];
//            } else {
//                // je vice hlavnich jidelen
//                $pouzitHlavni = false;
//                $hlavniJidelna = null;
//            }
//        }
//
//        $cursor ->close();
//
//        return $hlavniJidelna;
//    }


//    /**
//     * Nacte seznam jidelen a vybere jidelnu podle GET/SESSION/COOKIE
//     * @throws DBException
//     */
//    private function nactiVybranouJidelnu()
//    {
//        if (isset($_GET['vyberjidelny'])  &&  $_GET['vyberjidelny'] != '') {
//            $this ->vybranaJidelna = $_GET['vyberjidelny'];
//        } else if (isset($_SESSION['jidelna_jidelna'])) {
//            $this ->vybranaJidelna = $_SESSION['jidelna_jidelna'];
//        } else {
//            // po prihlaseni nastavi prednostne hlavni jidelnu zamestnance
//            // a pokud nema prave jednu hlavni jidelnu, tak prednastavi jidelnu podle cookie
//            $this ->vybranaJidelna = $this ->vratHlavniJidelnuZamestnance() ?? $_COOKIE['jidelna_jidelna'] ?? null;
//        }
//
//        // sestavi seznam jidelen
//        $params = $this ->database ->createParameters();
//        $sql =
//            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, RozdeleniOsob.Hlavni ' .
//            ' FROM Rozdeleni, RozdeleniOsob ' .
//            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
//            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
//            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
//            '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->vybranaOsobaRC) .
//            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->vybranaOsobaOsobniCislo) .
//            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->vybranaOsobaDatumNastupu) .
//            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($this ->datumJidelnickuDo) .
//            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
//            '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
//            ' ORDER BY Popis';
//
//        $cursor = $this ->database ->getSqlResult($sql, $params);
//
//        $prvniJidelna = null;
//        $jeVybranaJidelna = false;
//
//        while ($row = $cursor ->fetch()) {
//            if (!isset($prvniJidelna)  ||  $row[2]) {
//                // hlavni ma prednost, takze pokud je $row[2] nenulove, tak tuto jidelnu pouzije
//                // (pozor - v tomto pripade vybere posledni hlavni)
//                $prvniJidelna = $row[0];
//            }
//
//            $this ->jidelny[$row[0]] = $row[1];
//            if ($row[0] === $this ->vybranaJidelna) {
//                $jeVybranaJidelna = true;
//                $_SESSION['jidelna_jidelna'] = $this ->vybranaJidelna;
//                if (($_GET['vyberjidelny'] ?? null) === $this ->vybranaJidelna) {
//                    // cookie nastavi jen v pripade, ze lze vybrat jidelnu, kterou zvolil uzivatel
//                    HttpResponse::setCookie('jidelna_jidelna', $this ->vybranaJidelna, time() + 30000000);
//                }
//            }
//        }
//
//        $cursor ->close();
//
//        if (!$jeVybranaJidelna  &&  isset($prvniJidelna) ) {
//            $this ->vybranaJidelna = $prvniJidelna;
//        }
//
//        $_SESSION['jidelna_jidelna'] = $this ->vybranaJidelna;
//    }


    function loadData()
    {
        $this ->jidelny = [];

        parent::loadData();

//        $this ->nactiVybranouJidelnu();

        $this->loadTypyJidel();
    }


//    function renderJidelny()
//    {
//        echo
//            '<div class="filterrow">' .
//            '<span class="filtercaption"></span>' .
//            '<span class="filterformtext">';
//
//        LinksList::renderLinks($this ->jidelny, $this ->vybranaJidelna, '?menuid=jidelnicek&vyberjidelny=', '', false);
//
//        echo
//            '</span>' .
//            "</div>\n";
//
//    }

    private function renderTypyJidel()
    {
        echo '<div class="filterrow">';
        echo '<SPAN class="filtercaption"></SPAN>';
        echo '<SPAN class="filterformtext">';

        foreach ($this->typyJidel as $kodTypu => $popisTypu) {
            //pokud je uz. nastaveni prazdne jsou vybrane vsechny typy
            GuiUtils::renderCheckbox('jidelnicek_typ_' . $kodTypu, $popisTypu, empty($this->vybraneTypy) ? true : in_array($kodTypu, $this->vybraneTypy), true);
        }

        echo '</SPAN>';
        echo "</DIV>\n";
    }

    public function renderFilters()
    {
        parent::renderFilters();
//        $this ->renderJidelny();

        if (RonPortalSettings::getIntValue('jidelnicek_filtrovat_typy', 0)) {
            $this->renderTypyJidel();
        }
    }

}
