<?php

namespace RonPortal\Jidelna;

use Ron\Controls\ConfigurableObject;
use Ron\Controls\ObjectConfiguration;
use Ron\DB\Database;
use Ron\MVC\BaseViewObject;
use RonPortal\Controls\RonPortalObjectConfiguration;
use RonPortal\PracovniPomer;

class JidelnicekView extends BaseViewObject implements ConfigurableObject
{
    /** @var Database */
    private $database;

    /** @var JidelnicekFilterForm */
    public $filterForm;

    private $zobrazitObjednavky;

    /** @var Jidelnicek */
    public $jidelnicek;

    /** @var ObjectConfiguration    Konfiguracni objekt - umoznuje nacist konfiguraci z XML */
    public $objectConfiguration;

    public function __construct(Database $database, $zobrazitObjednavky)
    {
        parent::__construct('ronportal/jidelna/view_jidelnicek.php');

        $this ->database = $database;
        $this ->zobrazitObjednavky = $zobrazitObjednavky;

        $this ->filterForm = new JidelnicekFilterForm($database, $zobrazitObjednavky);
    }


    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'filterform' :
                if ($this ->filterForm  &&  $value instanceof \ArrayAccess) {
                    foreach ($value as $item) {
                        $this ->filterForm ->loadConfigurationArray($item);
                    }
                }
                break;
        }
    }


    public function load(?PracovniPomer $pp)
    {
        $this ->objectConfiguration = new RonPortalObjectConfiguration($this ->database, $this);
        $this ->objectConfiguration ->loadConfiguration('jidelnicek');

        $this ->filterForm ->loadData();

        $cenovaKategorie = $this ->filterForm ->getFilterValue('osoba') // ($this ->filterForm ->showOsoby  &&  $this ->filterForm ->vybranaOsoba)
            ? new CenovaKategorie(
                $this ->filterForm ->getFilterValue('osoba', 'rc'),
                $this ->filterForm ->getFilterValue('osoba', 'osobniCislo'),
                $this ->filterForm ->getFilterValue('osoba', 'datumNastupu'))
            : null;

        if ($this ->zobrazitObjednavky) {
            if (!$pp  ||  $pp ->idPP != $this ->filterForm ->getFilterValue('osoba')) {
                $pp = PracovniPomer::nactiPP($this->database, $this ->filterForm ->getFilterValue('osoba'));
            }
        }

        $this ->jidelnicek = new Jidelnicek($cenovaKategorie);
        $this ->jidelnicek ->load($this ->database,
            $this ->filterForm ->getFilterValue('vyberjidelny'),
            $this ->filterForm ->datumJidelnickuOd, $this ->filterForm ->datumJidelnickuDo,
            $pp
        );

        if ($this ->objectConfiguration instanceof RonPortalObjectConfiguration  &&  $this ->objectConfiguration ->xmlFormsList) {
            $this ->objectConfiguration ->xmlFormsList ->processXMLForms(
                $this ->database,
                ['datumOd' => $this ->filterForm ->datumJidelnickuOd,
                    'datumDo' => $this ->filterForm ->datumJidelnickuDo,
                    'idPP' => $this ->filterForm ->getFilterValue('osoba')
                ]);
        }
    }
}
