<?php
/**
 * pavel.rudol, 1.8.2022 14:40
 */

namespace RonPortal\Jidelna;

use Ron\DateUtils;
use Ron\DB\Database;

class UcetStravnika
{
    public $stavUctu;
    public $casVyuctovani;

    /**
     * Nacte aktualni stav uctu stravnika k nejvyssimu existujicimu pocatku obdobi dle zadaneho dne.
     * u srazkove aktualni stav v obdobi dle data
     * u zalohove nacitat stav z posledniho vypocteneho obdobi
     * @param Database $database
     * @param string $rc
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @param int $den
     * @return UcetStravnika|null
     * @throws \Ron\DB\DBException
     */
    public static function nactiAktualniStav($database, $rc, $osobniCislo, $datumNastupu, $den, $srazkoveUctovani)
    {
        $order = $srazkoveUctovani ? 'ASC' : 'DESC';

        $params = $database ->createParameters();
        $sql =
            'SELECT castka_AktualniStav, casVyuctovani FROM UcetStravnikaZaObdobi ' .
            ' WHERE RC = ' . $params->addParameter($rc) .
            '   AND OsobniCislo = ' . $params->addParameter($osobniCislo) .
            '   AND DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)) .
            '   AND Obdobi >= ' . $params ->addParameter($database ->dateToParam(DateUtils::firstDayOfMonth($den))) .
            " ORDER BY Obdobi {$order}";

        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri zjistovani aktualniho stavu uctu!');
        $row = $cursor ->fetch();
        $cursor ->close();

        $stav = null;

        if ($row) {
            $stav = new static();
            $stav->stavUctu = $row[0];
            $stav->casVyuctovani = $database->stringToTimestamp($row[1]);
        }

        return $stav;
    }
}