<?php
/**
 * stanislav.elbl, 04.03.2017
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\StringUtils;

class DenHarmonogramu
{
    public $cisloDne;
    public $smena;
    public $pracovniDobaOd;
    public $pracovniDobaDo;
    public $normoHodinyMin;
    public $normoHodinyMax;
}


class Harmonogram
{
    public $kod;
    public $popis;

    public $pocetDnuCyklu;
    public $zkratkaProPlanovani;

    /** @var bool       Priznak, jestli se ma planovat podle dne v tydnu (tj. harmonogram zacina v pondeli) */
    public $podleDneVTydnu = false;

    /** @var DenHarmonogramu[] */
    public $dny;

    public function getJson()
    {
        return [
            'kod' => StringUtils::jsonText($this ->kod),
            'popis' => StringUtils::jsonText($this ->popis),
            'pocetDnuCyklu' => $this ->pocetDnuCyklu,
            'zkratka' => StringUtils::jsonText($this ->zkratkaProPlanovani),
            'podleDneVTydnu' => $this ->podleDneVTydnu ? 1 : 0
        ];
    }

    /**
     * @param Database $database
     * @throws DBException
     */
    public function nactiDny($database)
    {
        $this ->dny = [];

        $params = $database ->createParameters();
        $sql =
            'SELECT DenHarmonogramu.CisloDne, DenHarmonogramu.Smena' .
            ' FROM DenHarmonogramu' .
            ' WHERE Harmonogram = ' . $params ->addParameter($this ->kod);

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $den = new DenHarmonogramu();
            $den ->cisloDne = $row[0];
            $den ->smena = $row[1];

            $this ->dny[$den ->cisloDne] = $den;
        }
    }


    /**
     * @param Database $database
     * @param string $filter
     * @param Parameters $filterParams
     * @return Harmonogram[]
     * @throws DBException
     */
    public static function nactiHarmonogramy($database, $filter = null, $filterParams = null)
    {
        $result = [];

        $sql =
            'SELECT Harmonogram.Kod, Harmonogram.Popis, Harmonogram.PocetDnuCyklu, Harmonogram.ZkratkaProPlanovani, Harmonogram.PodleDneVTydnu' .
            ' FROM Harmonogram ';

        if ($filter) {
            $sql .= ' WHERE ' . $filter;
        }

        $sql .= ' ORDER BY Popis';

        $cursor = $database ->getSqlResult($sql, $filterParams);

        while ($row = $cursor ->fetch()) {
            $harmonogram = new self();
            $harmonogram ->kod = $row[0];
            $harmonogram ->popis = $row[1];
            $harmonogram ->pocetDnuCyklu = (int)$row[2];
            $harmonogram ->zkratkaProPlanovani = $row[3];
            $harmonogram ->podleDneVTydnu = $row[4] ? 1 : 0;

            $result[] = $harmonogram;
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param string $kod
     * @return Harmonogram
     * @throws DBException
     */
    public static function nactiHarmonogram($database, $kod)
    {
        $params = $database ->createParameters();
        $filter = 'Kod = ' . $params ->addParameter($kod);
        $data = static::nactiHarmonogramy($database, $filter, $params);

        return count($data) ? $data[0] : null;
    }
}
