<?php
/**
 * stanislav.elbl, 28.02.2021
 */

namespace RonPortal\Kapacity;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\DenniZaznamyBrowser;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\RonPortalSettings;
use RonPortal\TypRozdeleni;

class PlanovaneSmenyBrowserField extends BrowserField
{
    var $fieldIndex;
    var $fieldIndexVariabilni;

    /**
     * @param Database $database
     * @param int    $fieldIndex
     * @param int    $fieldIndexVariabilni
     */
    function __construct($database, $fieldIndex, $fieldIndexVariabilni)
    {
        parent::__construct($database);

        $this->align = 'right';

        $this->fieldIndex = $fieldIndex;
        $this->fieldIndexVariabilni = $fieldIndexVariabilni;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData  &&  count($rowData) > 1) {
            $index = $rowData[3]
                ? $this->fieldIndexVariabilni // variabilni smena
                : $this->fieldIndex;

            if ($rowData[$index]) {
                return DateUtils::minutesToString($this->database->stringToTimeOfDay($rowData[$index]) * 60);
            }
        }

        return '';
    }
}


class PreruseniBrowserField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->header = TextManager::_('Preruseni_smeny');
        $this->align = 'center';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData  &&  count($rowData) > 1  &&  $rowData[16]) {
            return
                DateUtils::minutesToString($this->database->stringToTimeOfDay($rowData[17]) * 60) .
                ' - ' .
                DateUtils::minutesToString($this->database->stringToTimeOfDay($rowData[18]) * 60);
        }

        return '';
    }
}


class PlanovaneSmenyBrowser extends DenniZaznamyBrowser
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'prehledsmen');

        $this
            ->setTitle('Prehled_planovanych_smen')
            ->setCode('planovanesmeny')
            ->setAction('prehledsmen')
            ->setCssClass('browser grid');

        $this->sqlSelectPart =
            'PlanovaniKapacit.Datum, Smena.Kod, Smena.Popis, Smena.Variabilni,' . // 0..3
            ' Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny,' . // 4..6
            ' PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka,' . // 7..9
            ' KodTypuRozdeleni2, Rozdeleni2, Rozdeleni2.Popis,' . // 10..12
            ' Rozdeleni.KodTypuRozdeleni, Rozdeleni, Rozdeleni.Popis,' . // 13..15
            ' Smena.Delena, PlanovaniKapacit.Smena_PreruseniOd, PlanovaniKapacit.Smena_PreruseniDo,' . // 16..18
            ' PlanovaniKapacit.Harmonogram,' . // 19
            ' Smena.PocatekPevneCasti, Smena.KonecPevneCasti, Smena.DelkaPevnecasti, Smena.ZkratkaProPlanovani'; // 20..23

        $this->sqlFromPart =
            '(PlanovaniKapacit LEFT OUTER JOIN Smena ON PlanovaniKapacit.Smena = Smena.Kod)' .
            ' LEFT OUTER JOIN Rozdeleni ' .
            '   ON Rozdeleni.KodTypuRozdeleni = PlanovaniKapacit.KodTypuRozdeleni' .
            '  AND Rozdeleni.Kod = PlanovaniKapacit.Rozdeleni' .
            ' LEFT OUTER JOIN Rozdeleni Rozdeleni2 ' .
            '   ON Rozdeleni2.KodTypuRozdeleni = PlanovaniKapacit.KodTypuRozdeleni2' .
            '  AND Rozdeleni2.Kod = PlanovaniKapacit.Rozdeleni2' .
            ' INNER JOIN PracovniPomer' .
            '   ON PracovniPomer.RC = PlanovaniKapacit.RC' .
            '  AND PracovniPomer.OsobniCislo = PlanovaniKapacit.OsobniCislo' .
            '  AND PracovniPomer.DatumNastupu = PlanovaniKapacit.DatumNastupu' .
            ' INNER JOIN Osoba' .
            '   ON Osoba.RC = PlanovaniKapacit.RC';

        $this->sqlWherePart = '';

        $this ->addField()
            ->setSqlFieldName('PlanovaniKapacit.Datum', 0)
            ->setCode('col-datum')
            ->setHeaderText('Date') ->setFormat('timestamp') ->setClassName('browser_rowheader');

        $this->addField()
            ->setSqlFieldName('Smena.Kod', 1)
            ->setCode('col-kod');

        $this->addField()
            ->setSqlFieldName('Smena.Popis', 2)
            ->setCode('col-popis')
            ->setHeaderText('Smena');

        $this->addField(new PlanovaneSmenyBrowserField($database, 4, 7))  // pocatek smeny
            ->setHeaderText('Pocatek') ->setCode('col-pocatek');

        $this->addField(new PlanovaneSmenyBrowserField($database, 5, 8)) // konec smeny
            ->setHeaderText('Konec') ->setCode('col-konec');

        $this->addField(new PlanovaneSmenyBrowserField($database, 6, 9)) // delka smeny
            ->setHeaderText('Delka') ->setCode('col-delka');

        $this->addField(new PreruseniBrowserField($database));

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2)) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI2));

            $this->addField()
                ->setSqlFieldName('PlanovaniKapacit.Rozdeleni2', 11)
                ->setCode('col-rozdeleni2')
                ->setHeader($typRozdeleni ? StringUtils::htmlTextConvert($typRozdeleni ->popis) : '', 2);

            $this->addField()
                ->setSqlFieldName('Rozdeleni2.Popis', 12)
                ->setCode('col-popis-rozdeleni2');
        }

        if (RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI)) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue(KapacitySettings::KAPACITY_TYP_ROZDELENI));

            $this->addField()
                ->setSqlFieldName('PlanovaniKapacit.Rozdeleni', 14)
                ->setCode('col-rozdeleni')
                ->setHeader($typRozdeleni ? StringUtils::htmlTextConvert($typRozdeleni ->popis) : '', 2);

            $this->addField()
                ->setSqlFieldName('Rozdeleni.Popis', 15)
                ->setCode('col-popis-rozdeleni');
        }

        $this->addField()
            ->setSqlFieldName('PlanovaniKapacit.Harmonogram', 19)
            ->setCode('col-harmonogram') ->setVisible(false)
            ->setHeaderText('Harmonogram');

        $this->addField()
            ->setSqlFieldName('Smena.PocatekPevneCasti', 20) ->setFormat('timeofday')
            ->setCode('col-pocatek-pevne-casti') ->setVisible(false)
            ->setHeaderText('Pocatek_pevne_casti');

        $this->addField()
            ->setSqlFieldName('Smena.KonecPevneCasti', 21) ->setFormat('timeofday')
            ->setCode('col-konec-pevne-casti') ->setVisible(false)
            ->setHeaderText('Konec_pevne_casti');

        $this->addField()
            ->setSqlFieldName('Smena.DelkaPevneCasti', 22) ->setFormat('timeofday')
            ->setCode('col-delka-pevne-casti') ->setVisible(false)
            ->setHeaderText('Delka_pevne_casti');

        $this->addField()
            ->setSqlFieldName('Smena.ZkratkaProPlanovani', 23)
            ->setCode('col-zkratka') ->setVisible(false)
            ->setHeaderText('Zkratka');

        $this->defaultSort = 0;
        $this->primaryField = 0;
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $sql = $this ->filterForm ->buildSqlFilter($params);

        return new SqlStatement(
            $sql . ($sql ? ' AND ' : '') .
            'PlanovaniKapacit.Datum >= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiOd()) .
            ' AND PlanovaniKapacit.Datum <= ' . $params ->addDateParameter($this ->filterForm ->getOmezitObdobiDo()) .
            ' AND PlanovaniKapacit.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND PlanovaniKapacit.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND PlanovaniKapacit.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }
}
