<?php
/**
 * stanislav.elbl, 12.03.2019
 */

namespace RonPortal\Kapacity;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\StringUtils;
use RonPortal\Kalendar;
use RonPortal\PoznamkaKeDni;

/**
 * Class PlanovaniKapacit_Den
 */
class PlanovaniKapacit_Den
{
    public $datum;
    public $typ; // 0=pracovni,1=vikend,2=svatek,3=placeny svatek

    // pro pasivni rezim (poznamky ke dni)
    public $className;
    public $barva;
    public $hint;

    /** @var VysledekKontrolyPlanovani[]  Seznam kontrol pro pasivni rezim */
    public $kontroly;

    /** @var PoznamkaKeDni[] */
    public $poznamky = [];

    /** @var bool   Pouziva se v editaci podminek planovani */
    public $lzeEditovat;

    /** @var PodminkaPlanovaniKapacit[]    Seznam slotu pro zobrazeni v pasivnim rezimu */
    public $sloty = [];


    /**
     * @param Database $database
     * @param int $datum
     * @param string $kalendar
     */
    public function nactiData($database, $datum, $kalendar)
    {
        $this->datum = $datum;

        $this->typ = Kalendar::vratTypDne($database, $datum, true, $kalendar); // nacist svatky = true
    }


    public static function nactiDnyProObdobi($database, $datumOd, $datumDo, $kalendarProSvatky)
    {
        $result = [];

        // naplni seznam dnu
        for ($datum = $datumOd; $datum <= $datumDo; $datum = DateUtils::incDayStart($datum, 1)) {
            $den = new static();
            $den ->nactiData($database, $datum, $kalendarProSvatky);
            $result[] = $den;
        }

        return $result;
    }


    /**
     * @param Database $database
     * @param PlanovaniKapacit_Den[] $dny
     * @param int    $kodTypuRozdeleni
     * @param string $rozdeleni
     * @return PoznamkaKeDni[]
     */
    public static function nactiPoznamky($database, $dny, $kodTypuRozdeleni, $rozdeleni)
    {
        $datumOd = $dny[0]->datum;
        $datumDo = $dny[count($dny)-1]->datum;

        $poznamky = PoznamkaKeDni::nactiPoznamkyKeDni($database, $datumOd, $datumDo, $kodTypuRozdeleni, $rozdeleni);

        // priradi poznamky ke dnum a to tak, aby se pri zobrazeni neprekryvaly
        foreach ($poznamky as $poznamka) {
            $dayIndexFrom = ($poznamka ->datumOd > $datumOd)
                ? DateUtils::daysBetween($datumOd, $poznamka ->datumOd)
                : 0;

            $dayIndexTo = ($poznamka ->datumDo < $datumDo)
                ? DateUtils::daysBetween($datumOd, $poznamka ->datumDo)
                : count($dny) - 1;

            // najde pro poznamku vhodny radek (tj. prvni radek, do ktereho se vleze ve vsech dnech) ...
            $index = 0;
            $found = false;
            while (!$found) {
                $full = false;
                for ($dayIndex = $dayIndexFrom; !$full  &&  $dayIndex < $dayIndexTo; $dayIndex++) {
                    if (count($dny[$dayIndex] ->poznamky) > $index  &&  isset($dny[$dayIndex] ->poznamky[$index])) {
                        $full = true;
                    }
                }

                if (!$full) {
                    $found = true;
                } else {
                    $index++;
                }
            }

            // ... a doplni ji ke vsem dnum
            for ($dayIndex = $dayIndexFrom; $dayIndex <= $dayIndexTo; $dayIndex++) {
                $dny[$dayIndex] ->poznamky[$index] = $poznamka;
            }
        }

        return $poznamky;
    }


    /**
     * @param PlanovaniKapacit_Den[] $dny
     * @return int
     */
    public static function pocetRadkuPoznamek($dny)
    {
        $result = 0;
        foreach ($dny as $den) {
            $pocet = $den ->poznamky ? max(array_keys($den ->poznamky)) + 1 : 0;
            if ($result < $pocet) $result = $pocet;
        }

        return $result;
    }


    /**
     * Vraci css tridu pro zobrazeni zadaneho dne
     * @param int $datum
     * @param int $typDne   Typ dne nacteny z databaze (z DenniUdajKalendare)
     * @return string
     */
    public static function getHtmlClassDenKalendare($datum, $typDne)
    {
        $className = '';

        switch (date('w', $datum)) {
            case 6 : // sobota
            case 0 : // nedele
                $className .= ($className ? ' ' : '') . 'planovani_vikend';
        }

        if ($typDne === 2 || $typDne === 3) {
            $className .= ($className ? ' ' : '') . 'planovani_svatek';
        }
        if ($datum === DateUtils::today()) {
            $className .= ($className ? ' ' : '') . 'today';
        }

        return $className;
    }


    function getHtmlClass()
    {
        $className = $this ->className ?: '';

        if ($this ->typ & Kalendar::VIKEND) {
            $className .= ($className ? ' ' : '') . 'planovani_vikend';
        }
        if ($this ->typ & Kalendar::SVATEK) {
            $className .= ($className ? ' ' : '') . 'planovani_svatek';
        }
        if ($this ->datum === DateUtils::today()) {
            $className .= ($className ? ' ' : '') . 'today';
        }

        return $className;
    }


    function getHeaderHtml()
    {
        $result = \Ron\ronText('Day_short_' . date("w", $this->datum)) . '<br/>' . date("j", $this->datum);

        // kontroly
        if ($this->kontroly) {
            $result .= '<div class="planovani_kontroly">';

            foreach ($this ->kontroly as $kontrola) {
                $result .= "<span class='inline-glyph fa planovani_kontrola_priznak kontrola_{$kontrola ->kategorie}' title='" .
                    StringUtils::htmlTextConvert($kontrola ->popis) . ($kontrola ->poznamka ? "\n" . StringUtils::htmlTextConvert($kontrola ->poznamka) : '') .
                    "'></span>";
            }

            $result .= '</div>';
        }

        return $result;
    }


    /**
     * Vraci HTML pro zobrazeni radku poznamek ke dni.
     * @param PlanovaniKapacit_Den[] $dny
     * @param int $radek
     * @param string $cellTag
     * @return string
     */
    public static function getPoznamkyHtml($dny, $radek, $cellTag = "td")
    {
        $result = "";

        $datumDo = $dny[count($dny) - 1] ->datum;

        for ($i = 0;  $i < count($dny);  $i++) {
            if (isset($dny[$i] ->poznamky[$radek])) {
                $poznamka = $dny[$i] ->poznamky[$radek];

                //$indexDo = DateUtils::daysBetween()
                $datumOd = $dny[0] ->datum > $poznamka ->datumOd ? $dny[0] ->datum : $poznamka ->datumOd;
                $count = DateUtils::daysBetween($datumOd, $poznamka ->datumDo) + 1;
                if ($i + $count - 1 >= count($dny)) {
                    $count = count($dny) - $i;
                }

                $result .=
                    "<$cellTag class='poznamka-ke-dni'" .
                    ($count > 1 ? " colspan='$count'" : '') .
                    ($poznamka ->barva ? " style='background-color: " . GuiUtils::getHtmlColor($poznamka ->barva) . "'" : '') .
                    ' title="' . StringUtils::htmlTextConvert($poznamka ->text) . "\n" . StringUtils::htmlTextConvert($poznamka ->poznamka) . '"' .
                    '>' .
                    StringUtils::htmlTextConvert($poznamka ->text) .
                    "</$cellTag>";

                $i += $count - 1;
            } else {
                $result .= "<$cellTag></$cellTag>";
            }
        }

        return $result;
    }


    public function getJSON()
    {
        return [
            'datumStr' => StringUtils::jsonDate($this->datum),
            'typ' => $this->typ
        ];
    }
}
