<?php
/**
 * stanislav.elbl, 23.05.2022 12:29
 */

namespace RonPortal\Kapacity;
use Ron\GuiUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;

/**
 * Class PlanovaniKapacitLegenda
 * @package Ron\Kapacity
 */
class PlanovaniKapacitLegenda extends RonPortalBrowser
{
    protected $absence;


    public function __construct($database, $absence)
    {
        parent::__construct($database, null);

        $this ->absence = $absence;
        $this ->showMessages = false;

        $this ->code = 'browserpklegenda_' . ($absence ? 'absence' : 'smeny');

        $this ->addField()
            ->setSqlFieldName('ZkratkaProPlanovani', 0) ->setAlign('center')
            ->setCode('col-zkratka');

        $this ->addField()
            ->setSqlFieldName('Popis', 1)
            ->setCode('col-popis')
            ->setHeaderText(TextManager::_($absence ? 'Absence_udalost' : 'Smena'))
            ->setTranslationPrefix($absence ? 'db_operacedochazky_popis_' : 'db_smena_popis_', 4);
        //$this ->AddFieldDef(array("normal", "ZkratkaProPlanovani", "", "", "center", "", 0, 0), 0);
        //$this ->AddFieldDef(array("normal", "Popis", $popis, "", "left", "", 0, 0), 1);

        $this ->tableHtmlParams['data-typseznamu'] = $this ->absence ? 'absence' : 'smeny';
    }

    function setFilterForm()
    {
        // Nedela nic - filtr se nezobrazuje
    }


    public function loadColumnsConfiguration($columnsConfArray)
    {
        if (isset($columnsConfArray['typseznamu'])) {
            if ($this ->absence  &&  $columnsConfArray['typseznamu'] !== 'absence') {
                return;
            }

            if (!$this ->absence  &&  $columnsConfArray['typseznamu'] !== 'smeny') {
                return;
            }
        }

        parent::loadColumnsConfiguration($columnsConfArray); // TODO: Change the autogenerated stub
    }


    public function getSQL($sqlFilter = '')
    {
        if ($this ->absence) {
            $result =
                'SELECT OperaceDochazky.ZkratkaProPlanovani, OperaceDochazky.Popis, OperaceDochazky.BarvaProVykresleni, 1, OperaceDochazky.Kod AS absence ' .
                ' FROM OperaceDochazky ' .
                ' WHERE OperaceDochazky.ZkratkaProPlanovani IS NOT NULL ' .
                "   AND  OperaceDochazky.ZkratkaProPlanovani <> '' ";
        } else {
            $result =
                'SELECT Smena.ZkratkaProPlanovani, Smena.Popis, Smena.BarvaProVykresleni, 0 AS absence, Smena.Kod ' .
                ' FROM Smena ' .
                " WHERE Smena.ZkratkaProPlanovani IS NOT NULL  AND  Smena.ZkratkaProPlanovani <> '' ";
        }


        if ($sqlFilter) {
            $result .= ' AND ' . $sqlFilter;
        }

        $result .= ' ORDER BY Popis';

        return $result;
    }


    public function renderField($index, $field, $value, $rowNumber, $recordId, $values)
    {
        if (!$index) {
            echo '<td>';

            //
            $class = ($values[3]) ? "planovani_absence" : "planovani_smena";

            if ( isset($values[2]) ) {
                $color = GuiUtils::getHtmlColor($values[2]);
                echo "<div style='background-color: $color' class='$class'>";
            } else {
                echo "<div class='$class'>";
            }

            echo '</div>';

            echo "<div>$value</div>";

            echo '</td>';
        } else {
            parent::renderField($index, $field, $value, $rowNumber, $recordId, $values);
        }
    }
}
