<?php
/**
 * stanislav.elbl, 10.03.2021
 */

namespace RonPortal\Kapacity;

use Ron\ArrayUtils;
use Ron\DateUtils;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\MVC\BaseViewObject;
use RonPortal\Obdobi;
use RonPortal\Rozdeleni;

class ZaznamSmenKalendare
{
    public $datum;

    /** @var int    0=pracovni, 1=volno, 2=neplaceny svatek, 3=placeny svatek */
    public $typDne;

    /** @var bool */
    public $jeSmena = false;

    /** @var int */
    public $idSablonyPodminek;
    /** @var string */
    public $zkratkaSablonyPodminek;
    /** @var string */
    public $popisSablonyPodminek;
}


class RadekSmenKalendare
{
    // bud se muze editovat kalendar k rozdeleni, nebo primo konkretni kalendar
    /** @var string */
    public $kodRozdeleni;
    /** @var string */
    public $popisRozdeleni;

    /** @var string */
    public $kodKalendare;

    /** @var string */
    public $kodSmeny;
    /** @var string */
    public $popisSmeny;

    /** @var ZaznamSmenKalendare[] */
    public $data = [];

    /** @var SchvaleniPodminekPlanovani     Nacita se pro kazde rozdeleni zvlast, proto je u radku */
    public $schvaleni;

    /**
     * @param $datum
     * @return ZaznamSmenKalendare
     */
    function vratZaznamProDen($datum) {
        $index = ArrayUtils::findInSortedArray( function($dataIndex) use ($datum) {
            $zaznam = $this ->data[$dataIndex];
            if ($datum < $zaznam ->datum) return -1;
            if ($datum > $zaznam ->datum) return 1;
            return 0;
        }, $this ->data);

        if (isset($index)) {
            return $this ->data[$index];
        }

        return null;
    }
}


class SmenyKalendareFormular extends BaseViewObject
{
    /** @var Database */
    private $database;

    /** @var Obdobi */
    public $obdobi;

    /** @var Rozdeleni */
    public $rozdeleni;

    /** @var RadekSmenKalendare[] */
    public $radky;


    /**
     * SmenyKalendareFormular constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct('ronportal/kapacity/view_smeny_kalendare.php');
        $this->database = $database;
    }


    /**
     * @param array $row
     * @return RadekSmenKalendare
     */
    private function pridejRadek($row)
    {
        $result = new RadekSmenKalendare();

        $result ->kodRozdeleni = $row[0];
        $result ->popisRozdeleni = $row[1];
        $result ->kodKalendare = $row[2];
        $result ->kodSmeny = $row[3];
        $result ->popisSmeny = $row[4];

        $result ->data = [];

        $this ->radky[] = $result;

        $datum = $this ->obdobi ->zacatek;
        while ($datum <= $this ->obdobi ->konec) {
            $zaznam = new ZaznamSmenKalendare();
            $zaznam ->datum = $datum;
            $result ->data[] = $zaznam;

            $datum = DateUtils::incDayStart($datum);
        }

        return $result;
    }


    /**
     * @param Cursor $cursor
     */
    private function loadData($cursor)
    {
        $this->radky = [];

        // schvaleni podminek rozdeleni nacita pro kazde zobrazene rozdeleni samostatne
        $schvaleni = null;
        $posledniRozdeleni = null;

        /** @var RadekSmenKalendare $posledniRadek */
        $posledniRadek = null;
        while ($row = $cursor ->fetch()) {
            if (!$posledniRadek || $posledniRadek ->kodRozdeleni != $row[0]) {
                // novy kod rozdeleni => nacte schvaleni
                $schvaleni = SchvaleniPodminekPlanovani::nactiSchvaleni($this ->database, $this ->obdobi ->zacatek, $this ->rozdeleni ->typ, $row[0]);
            }

            if (!$posledniRadek  ||  $posledniRadek ->kodRozdeleni != $row[0]  ||  $posledniRadek ->kodKalendare != $row[2]  ||  $posledniRadek ->kodSmeny != $row[3]) {
                // nasel dalsi rozdeleni a/nebo smenu
                $posledniRadek = $this ->pridejRadek($row);
                // priradi zaznam schvaleni (vsechny radky jednoho rozdeleni maji prirazen stejny zaznam)
                $posledniRadek->schvaleni = $schvaleni;
            }

            $datum = $this->database ->stringToTimestamp($row[6]);
            $zaznam = $posledniRadek ->vratZaznamProDen($datum);

            if ($zaznam) {
                $zaznam ->typDne = $row[7];

                if ($row[8]) {
                    // doplni data
                    $zaznam->jeSmena = true;
                    $zaznam->idSablonyPodminek = $row[9];
                    $zaznam->zkratkaSablonyPodminek = $row[10];
                    $zaznam->popisSablonyPodminek = $row[11];
                }
            }
        }
    }


    /**
     * vraci css tridu pro zobrazeni zadaneho dne z kalendare
     * @param int $datum
     * @param int $typDneVKalendari     Typ dne nacteny z databaze (0=pracovni, 1=volno, 2=svatek, 3=placeny svatek)
     * @return string
     */
    public static function cssClassProDen($datum, $typDneVKalendari)
    {
        return PlanovaniKapacit_Den::getHtmlClassDenKalendare($datum, $typDneVKalendari);
    }


    /**
     * @param Rozdeleni $rozdeleni
     * @param Obdobi $obdobi
     * @param bool $vcetnePodrizenych
     */
    public function load($rozdeleni, $obdobi, $vcetnePodrizenych)
    {
        $this ->obdobi = $obdobi;
        $this ->rozdeleni = $rozdeleni;

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.Kalendar,' .
            ' Smena.Kod, Smena.Popis, PouzivaneSmeny.Poradi,' . // 3..5
            ' DenniUdajKalendare.Den, DenniUdajKalendare.TypDne, SmenaProDenKalendare.Smena,' . // 6..8
            ' SmenaProDenKalendare.SadaPodminekPlanovani, SadaPodminekPlanovani.Zkratka, SadaPodminekPlanovani.Popis'; // 9..11

        if ($vcetnePodrizenych) {
            $sql .=
                ' FROM Rozdeleni INNER JOIN HierarchieRozdeleni' .
                ' ON HierarchieRozdeleni.TypPodrizene = Rozdeleni.KodTypuRozdeleni' .
                ' AND HierarchieRozdeleni.KodPodrizene = Rozdeleni.Kod';
        } else {
            $sql .=
                ' FROM Rozdeleni';
        }

        $sql .=
            "\n" .
            ' INNER JOIN PouzivaneSmeny' .
            '  ON PouzivaneSmeny.KodKalendare = Rozdeleni.Kalendar' .
            ' INNER JOIN Smena' .
            '  ON Smena.Kod = PouzivaneSmeny.KodSmeny' .
            ' INNER JOIN DenniUdajKalendare' .
            '  ON DenniUdajKalendare.Kalendar = Rozdeleni.Kalendar' .
            ' LEFT OUTER JOIN SmenaProDenKalendare' .
            '  ON SmenaProDenKalendare.Kalendar = Rozdeleni.Kalendar' .
            '  AND SmenaProDenKalendare.Smena = PouzivaneSmeny.KodSmeny' .
            '  AND SmenaProDenKalendare.Den = DenniUdajKalendare.Den' .
            ' LEFT OUTER JOIN SadaPodminekPlanovani' .
            '  ON SadaPodminekPlanovani.Id = SmenaProDenKalendare.SadaPodminekPlanovani' .
            "\n" .
            ' WHERE DenniUdajKalendare.Den >= ' . $params ->addParameter($this ->database ->dateToParam($obdobi ->zacatek)) .
            '  AND DenniUdajKalendare.Den <= ' . $params ->addParameter($this ->database ->dateToParam($obdobi ->konec)) .
            "\n";



        if ($vcetnePodrizenych) {
            $sql .=
                ' AND HierarchieRozdeleni.TypNadrazene = ' . $params ->addParameter($rozdeleni ->typ) .
                ' AND HierarchieRozdeleni.KodNadrazene = ' . $params ->addParameter($rozdeleni ->kod);
        } else {
            $sql .=
                ' AND Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($rozdeleni ->typ) .
                ' AND Rozdeleni.Kod = ' . $params ->addParameter($rozdeleni ->kod);
        }

        $sql .= ' ORDER BY Rozdeleni.Popis, PouzivaneSmeny.Poradi, SmenaProDenKalendare.Den';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $this ->loadData($cursor);
    }


}
