<?php
/**
 * User: pavel.rudol
 * 13.6.2016 13:10
 */

namespace RonPortal\Kapacity;

use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;

class ZastupRecord extends RonPortalDBRecord
{
    /** @var int */
    public $id;

    /** @var int  */
    public $typRozdeleni;

    /** @var  string */
    public $rozdeleni;

    /** @var DBRecordField */
    public $datumpZastupu;
    /** @var DBRecordField */
    public $rcfield;
    /** @var DBRecordField */
    public $osobniCislofield;
    /** @var DBRecordField */
    public $datumNastupufield;

    /** @var DBRecordField */
    public $schvaleni;

    /** @var  DBRecordField */
    public $typSchvaleni;

    public function __construct($database, $id, $logDescription)
    {
        parent::__construct($database, 'ZadostOZastup', $logDescription);

        $this->id = (int)$id;

        $params = $database ->createParameters();
        $this->setWhere(
            'ZadostOZastup.Id = ' . $params ->addParameter((int)$id),
            $params);

        $this->addIntField('ZadostOZastup.Id', true); // readonly

        $this->typRozdeleni = $this->addIntField('ZadostOZastup.KodTypuRozdeleni', true); //readonly
        $this->rozdeleni = $this->addStringField('ZadostOZastup.Rozdeleni', true);

        $this->rcfield = $this->addStringField('ZadostOZastup.RC', true); // readonly
        $this->osobniCislofield = $this->addStringField('ZadostOZastup.OsobniCislo', true);
        $this->datumNastupufield = $this->addDateField('ZadostOZastup.DatumNastupu', true);

        $this->datumpZastupu = $this->addDateField('ZadostOZastup.Datum', true);

        $this->addField('ZadostOZastup.Poznamka', '', true);

        $this->schvaleni = $this->addTimestampField('ZadostOZastup.Schvaleni', true);
        $this->typSchvaleni = $this->addIntField('ZadostOZastup.TypSchvaleni', true);
    }

    /**
     * @param array $row
     * @return bool
     * @throws \Exception
     */
    public function loadRecordValues($row)
    {
        if (!$this ->id) {
            throw new \Exception('Chyba - neni zadano ID zaznamu zadosti o zastup!');
        }

        parent::loadRecordValues($row);

        return true;
    }

    /**
     * @param PravaNaOsoby $prava
     * @throws \Exception
     */
    private function zkontrolujPrava($prava)
    {
        Prava::nastavObdobiProPrava($this ->datumpZastupu ->value, $this ->datumpZastupu ->value);

        if (!$prava ->jePravoNaRozdeleni($this->typRozdeleni->value,$this->rozdeleni->value)) {
            throw new NeniPravoException();
        }
    }


    /**
     * @param string $eventName
     * @param string $xmlContext
     * @return bool
     */
    private function xmlEventsZastup($eventName, $xmlContext)
    {
        return $this ->executeXmlEvents($eventName, $xmlContext, ['zadostozastup' => $this ->id]);
    }

    /**
     * @param PravaNaOsoby $prava
     * @param int $hodnota 0 = nerozhodnuto, 1 = schvaleno, 2 = zamitnuto, 3 = zrusena zadost (jiz je prirazen zamestnanec)
     * @return bool
     * @throws DBException
     * @throws \Exception
     */
    public function schval($prava, $hodnota)
    {
        $this->typSchvaleni->setNewValue($hodnota);

        $pomf = $this ->addField('Schvalil', '', false, 0, RonPortalUtils::getLogUserName());
        $pomf ->updateLog = true;

        $this->schvaleni->setNewValue(time());
        $this->schvaleni->updateLog = true;

        switch ($hodnota) {
            case 0 :
                $logstr = TextManager::_("Zastup_zruseno_schvaleni_log");
                $scriptContext = "zrusitschvaleni_";
                break;

            case 1 :
                $logstr = TextManager::_("Zastup_schvaleni_log");
                $scriptContext = "schvaleni_";
                break;

            case 2 :
                $logstr = TextManager::_("Zastup_zamitnuti_log");
                $scriptContext = "zamitnuti_";
                break;

            case 3 :
                $logstr = TextManager::_("Zastup_zruseni_log");
                $scriptContext = "zruseniplanu_";
                break;

            default :
                $scriptContext = null;
                $logstr = '';
        }

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana zadost o zastup nebyla nalezena!');
        }

        if ($this->typSchvaleni->originalValue == 3) {
            throw new \Exception( StringUtils::fromUTF8('Chyba - nelze měnit schválení u zamítnuté žádosti!') );
        }

        if (!$this ->isChanged()) {
            // neni co menit
            return true;
        }

        $this ->zkontrolujPrava($prava);

        if ($hodnota != 3) {
            if ($this->rcfield->value != '') {
                throw new \Exception( StringUtils::fromUTF8('Chyba - nelze měnit schválení, je přiřazen zaměstnanec!') );
            }
        }
        else if ($this->rcfield->value == '') {
            throw new \Exception( StringUtils::fromUTF8('Chyba - nelze zrušit žádost, která nemá přiřazeného zaměstnance!') );
        }

        $this ->logText = $logstr;

        if (!$this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, $scriptContext.'before')) {
            return false;
        }

        if (!$this ->updateRecord()) {
            return false;
        }

        return $this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, $scriptContext.'after');
    }

    /**
     * @return bool
     * @throws \Exception
     */
    function zrusSchvaleni()
    {

        $this ->schvaleni ->setNewValue(null);

        $f = $this ->addField('Schvalil', '', false, 0, null);
        $f ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana zadost o zastup nebyla nalezena!');
        }

        if (!$this ->isChanged()) {
            // neni co menit
            return true;
        }

        $this ->zkontrolujPrava(Prava::vratPrava('zastupy_schvaleni'));

        $this ->logText = StringUtils::fromUTF8('Zrušeno schválení zástupu');

        if (!$this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, 'zrusitschvaleni_before')) {
            return false;
        }

        if (!$this ->updateRecord()) {
            return false;
        }

        return $this ->xmlEventsZastup(RonPortalXMLEvents::ZASTUP_SCHVALENI, 'zrusitschvaleni_after');
    }

    /** Zrusi prirazeneho zamestnance
     * @return bool
     * @throws \Exception
     */
    function zrusZamestnance()
    {
        $this->rcfield ->setNewValue(null);
        $this->osobniCislofield->setNewValue(null);
        $this->datumNastupufield->setNewValue(null);

        if (!$this ->loadRecord()) {
            throw new \Exception('Chyba - zadana zadost o zastup nebyla nalezena!');
        }

        if (!$this ->isChanged()) {
            // neni co menit
            return true;
        }

        $this ->zkontrolujPrava(Prava::vratPrava('zastupy_prideleni'));

        $this ->logText = StringUtils::fromUTF8('Zrušen přiřazený zaměstnanec');

        return $this ->updateRecord();
    }

    /**
     * @param Database $database
     * @param $logDescription
     * @return bool
     * @throws \Exception
     */
    public static function hromadneSchvaleniZastupu($database, $logDescription)
    {
        // uzivatel odeslal hromadne schvaleni Zastupu
        $pocet = (int)$_POST['pocet'];

        $result = true;

        for ($i = 0;  $i < $pocet;  $i++) {
            $id = $_POST["idZastup_$i"];
            $rec = new self($database, $id, $logDescription);

            try {
                if (isset($_POST["NastavitSchvaleni_{$id}"]) && $_POST["NastavitSchvaleni_{$id}"]) {
                    if ($_POST["Schvaleni_{$id}"] ?? false) {
                        if (!$rec->schval(Prava::vratPrava('zastupy_schvaleni'),1)) {
                            $result = false;
                        }
                    } else {
                        if (!$rec->schval(Prava::vratPrava('zastupy_schvaleni'),0)) {
                            $result = false;
                        }
                    }
                }

            } catch (\Exception $e) {
                $result = false;
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        if (!$result) {
            MessageManager::addErrorMessage( StringUtils::fromUTF8('Při hromadném schvalování zástupů došlo k chybě!') );
        }

        return $result;
    }
}
