<?php

namespace RonPortal\Mzdy;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\App;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\Redirect;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\User;
use function Ron\ronText;

//require_once ('mzdy/danecontroller.php');
//require_once ('mzdy/zadostozuctovanizaloh.php');
//require_once ('mzdy/prohlasenipoplatnikadane.php');

class DaneController extends JsonController
{
    //prohlaseni
    const DOKUMENT_DETI = 'deti';
    const DOKUMENT_PARTNER_NEUPLATNUJE = 'partner';
    const DOKUMENT_OBECNY = 'odecet';

    //vyuctovani
    const DOKUMENT_ZAMESTNAVATEL = 'predchozizamestnavatel';
    const DOKUMENT_SLEVA_PARTNER = 'slevanapartnera';
    const DOKUMENT_SKOLKOVNE = 'skolkovne';
    const DOKUMENT_SLEVA_ODECET = 'odecitatelnepolozky';
    const DOKUMENT_SLEVA_EXEKUCE = 'slevaexekuce';

    /** @var Database */
    public $database;

    /** @var PracovniPomer $pracovniPomer */
    public $pracovniPomer;


    /** @var PravaNaOsoby $pravo_vyplnit */
    public $pravo_prohlaseni;

    /** @var PravaNaOsoby $pravo_vyuctovani */
    public $pravo_vyuctovani;

    /**
     * DaneController constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        $this->database = $database;

        $this->pravo_prohlaseni = Prava::vratPrava('dane_prohlaseni');
        $this->pravo_vyuctovani = Prava::vratPrava('dane_vyuctovani');
    }

    /**
     * @return string
     */
    public static function podpisUzivatele(): string
    {
        $ip = $_SERVER['REMOTE_ADDR'];
        $ipAdresa = "IP: " . $ip;
        $login = " Login: " . User::getLoginName() . "(" . User::getUserName() . ")";
        $datumCas = "Datum: " . StringUtils::format("datetime", time());

        return $datumCas . ", " . $ipAdresa. ", " . $login;
    }

    /**
     * @param $datum
     * @param $datumNastupu
     * @return int
     */
    public static function vratDatumPocatkuNavrhuOdpoctu($datum, $datumNastupu)
    {
        //pri podepsani do 15.2. se datum pocatku odpoctu nastavi na 1.1.
        if ($datum <= mktime(0,0,0,2,15, (int)date('Y', $datum))) {
            $result = mktime(0, 0, 0, 1, 1, (int)date('Y', $datum));
        } elseif (date('Y', $datum) == date('Y', $datumNastupu) &&
        //pri podpisu 30 dnu od data nastupu, bude pocatek datum od 1. den mesice data nastupu, $datum muze 1. den v mesici nastupu (pripad vlozeni zmeny v mesici nastupu)
         ((($datum > $datumNastupu) && (DateUtils::daysBetween($datumNastupu, $datum) <= 30)) || (date('n', $datum) == date('n', $datumNastupu))) ) {
            $result = DateUtils::firstDayOfMonth($datumNastupu);
        } elseif ($datum > time() && date('n', $datum) > date('n', time())) {
            //pokud je zadane datum pocatku platnosti v mesici nasledujicim po aktualnim, nastavi se primo zvolene datum
            $result = DateUtils::firstDayOfMonth($datum);
        } else {
            //od 15.2. se nastavi na 1. den nasledujiciho mesice
            $result = DateUtils::firstDayOfMonth(DateUtils::incMonth($datum));
        }

        return $result;
    }

    /**
     * @param int $rok Rok vyuctovani dane
     * @return bool
     * @throws Exception
     */
    public static function lzeEditovatVyuctovani($rok)
    {
        if (RonPortalSettings::getIntValue('zadostizuctovanidanepoterminu')) {
            return true;
        }

        //editace vyuctovani za zvoleny rok je mozna od 1.1. nasledujiciho roku
        $editovatOd = DateUtils::incMonth($rok, 12);
        //do zvoleneho terminu v danem (nasledujicim) roce (napr. 28.2.) - pokud neni zadano bere se 31.3.
        $editovatDo = DateUtils::dateStringToTimestamp(RonPortalSettings::getValue('daneterminvyplneni', '31.3.') . date('Y', $editovatOd));

        if ((DateUtils::today() >= $editovatOd) && (DateUtils::today() <= $editovatDo)) {
            return true;
        }

        return false;
    }

    /** Vrati typ dokumentu pro zvolenou sekci
     * @param $sekce
     * @return int|mixed
     */
    public static function vratTypDokumentuProSekci($sekce)
    {
        switch ($sekce) {
            //vyuctovani
            case static::DOKUMENT_ZAMESTNAVATEL :
                return RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_typ', -5);
            case static::DOKUMENT_SLEVA_PARTNER :
                return RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_typ', -6);
            case static::DOKUMENT_SKOLKOVNE :
                return RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_typ', -7);
            case static::DOKUMENT_SLEVA_ODECET :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_typ', -8);
            case static::DOKUMENT_SLEVA_EXEKUCE :
                return RonPortalSettings::getIntValue('dane_dokumenty_exekuce_typ', -11);
            //prohlaseni
            case static::DOKUMENT_DETI :
                return RonPortalSettings::getIntValue('dane_dokumenty_deti_typ', -2);
            case static::DOKUMENT_PARTNER_NEUPLATNUJE :
                return RonPortalSettings::getIntValue('dane_dokumenty_partner_typ', -4);
            case static::DOKUMENT_OBECNY :
            default :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ', -3);
        }
    }

    /** Vrati true pokud je prilozeni dokumentu pro danou sekci povinne
     * @param string $sekce - casti formulare: deti, partner, odecet (slevy na dani)
     * @return int|mixed
     */
    public static function povinnyDokumentProSekci($sekce)
    {
        switch ($sekce) {
            //vyuctovani
            case static::DOKUMENT_ZAMESTNAVATEL :
                return RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_povinne');
            case static::DOKUMENT_SLEVA_PARTNER :
                return RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_povinne');
            case static::DOKUMENT_SKOLKOVNE :
                return RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_povinne');
            case static::DOKUMENT_SLEVA_ODECET :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_povinne');
            case static::DOKUMENT_SLEVA_EXEKUCE :
                return RonPortalSettings::getIntValue('dane_dokumenty_exekuce_povinne');
            //prohlaseni
            case static::DOKUMENT_DETI :
                return RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne');
            case static::DOKUMENT_PARTNER_NEUPLATNUJE :
                return RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne');
            case static::DOKUMENT_OBECNY :
            default :
                return RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne');
        }
    }

    /** Vrati typ formulare (akci pro url) dle zvolene sekce.
     * @param string $sekce - casti formulare: deti, partner, odecet (slevy na dani)
     * @return string
     */
    public static function vratTypFormulareDleSekce($sekce)
    {
        switch ($sekce) {
            case static::DOKUMENT_ZAMESTNAVATEL :
            case static::DOKUMENT_SLEVA_PARTNER :
            case static::DOKUMENT_SKOLKOVNE :
            case static::DOKUMENT_SLEVA_ODECET :
            case static::DOKUMENT_SLEVA_EXEKUCE :
                return 'formularzadosti';
            case static::DOKUMENT_DETI :
            case static::DOKUMENT_PARTNER_NEUPLATNUJE :
            case static::DOKUMENT_OBECNY :
            default :
                return 'formularprohlaseni';
        }
    }

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param int $rok
     * @param int $typDokumentu
     * @param string $popisDokumentu
     * @return int
     * @throws DBException
     */
    public static function vratNovePoradoveCisloDokumentu($database, $pracovniPomer, $rok, $typDokumentu, $popisDokumentu = null)
    {
        $params = $database->createParameters();
        if (!is_null($popisDokumentu)) {
            $sql =
                "SELECT Popis FROM Dokument JOIN DokumentyOsoby ON Dokument.id = DokumentyOsoby.id 
                WHERE DokumentyOsoby.VyuctovaniDane_RC = " . $params->addParameter($pracovniPomer->rc) .
                "  AND DokumentyOsoby.VyuctovaniDane_Rok = " . $params->addDateParameter($rok) .
                "  AND Dokument.typ = " . $params->addParameter($typDokumentu) .
                "  AND Dokument.Popis LIKE " . $params->addParameter("%{$popisDokumentu}%") .
                " ORDER BY Dokument.id DESC";
            $posledniPopis = DBUtils::getSqlValue($database, $sql, $params);

            $posledniCislo = substr($posledniPopis, -1);
        } else {
            $sql =
                "SELECT COUNT(Dokument.id) FROM Dokument JOIN DokumentyOsoby ON Dokument.id = DokumentyOsoby.id 
                WHERE DokumentyOsoby.VyuctovaniDane_RC = " . $params->addParameter($pracovniPomer->rc) .
                "  AND DokumentyOsoby.VyuctovaniDane_Rok = " . $params->addDateParameter($rok) .
                "  AND Dokument.typ = " . $params->addParameter($typDokumentu);
            $posledniCislo = DBUtils::getSqlValue($database, $sql, $params);
        }

        return (int)$posledniCislo + 1;
    }

    /** Zobrazi seznam pripojenych dokumentu k sekci s moznosti vlozeni noveho.
     * @param Database $database
     * @param $rok
     * @param $pracovniPomer
     * @param string $sekceFormulare ke ktere casti jsou dokumenty mozne hodnoty: deti, partner, odecet
     * @param $povolitEditaci
     * @param $kontrolovatPovinnost
     * @throws DBException
     */
    public static function zobrazDokumenty($database, $rok, $pracovniPomer, $sekceFormulare, $povolitEditaci, $kontrolovatPovinnost = true, $castiSekce = null)
    {
        echo "<A name='dokumenty'></A>\n";

        $typDokumentu = self::vratTypDokumentuProSekci($sekceFormulare);
        $povinny = false;
        if ($kontrolovatPovinnost) {
            $povinny = self::povinnyDokumentProSekci($sekceFormulare);
        }

        if ($typDokumentu) {
            // Seznam dokumentu
            $b = new DokumentyVyuctovaniBrowser($database, $rok, $pracovniPomer, $typDokumentu, $povolitEditaci, $povinny, self::vratTypFormulareDleSekce($sekceFormulare));
            $b->load();
            $b->render();

            if ($povolitEditaci) {
                echo "<br/>";

                if (is_array($castiSekce)) {
                    foreach ($castiSekce as $cast => $popis) {
                        echo "<label for='{$sekceFormulare}_{$cast}'>$popis</label>
                            <span id='{$sekceFormulare}_{$cast}'><input type='file' name='{$sekceFormulare}_{$cast}' " . DokumentyUtils::fileInputAcceptAttribute() . "></span><br/>";
                    }
                } else {
                    echo "<label for='{$sekceFormulare}'>" . ronText('Vlozit_novy_dokument') . "</label><br/>
                            <span id='{$sekceFormulare}'><input type='hidden' name='{$sekceFormulare}_pocet' value='1'><input type='file' name='{$sekceFormulare}'" .  DokumentyUtils::fileInputAcceptAttribute() . "></span>";

                    echo "<br/> <a href='#' title='" . TextManager::_('Vlozit_novy_dokument') . "'>" .
                        "<i class='inline-glyph fa glyph-upload add-dokument-input' data-input-name='{$sekceFormulare}' ></i> " .
                        "</a>";

                    echo "<br/>";
                }

                echo "<div class='font-italic padding-bottom-2'>" . sprintf(TextManager::_('dokument_max_velikost_soboru'), ini_get('upload_max_filesize'), ini_get('post_max_size')) . "</div>";
            }
        }
    }

    /** Zobrazi odkaz na pripojeni dokumentu k radku nezdanitelnych polozek (slev na dani). Umoznuje cislovani popisu dokumentu.
     * @param Database $database
     * @param int $rok
     * @param \RonPortal\PracovniPomer $pracovniPomer
     * @param string $castProhlaseni
     * @param string $popisDokumentu
     * @throws DBException
     */
    public static function zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, $castProhlaseni, $popisDokumentu)
    {
        echo
            "<a href='#' title='" . TextManager::_('Vlozit_novy_dokument') . "'>" .
            "<i class='inline-glyph fa glyph-upload show-dokument-input-dialog' data-input-name='{$castProhlaseni}_{$popisDokumentu}' ></i> " .
            "</a>";
    }

//    /**
//     * @param ProhlaseniPoplatnikaDane $hodnotyProhlaseni
//     * @param array $historieOdpoctuNaDeti
//     * @return bool
//     */
//    public function jePovinnyDokument($hodnotyProhlaseni, $historieOdpoctuNaDeti)
//    {
//        $povinnyDokumentDeti = false;
//        if (self::povinnyDokumentProSekci('deti')) {
//            $povinnyDokumentDeti = true;
//
//            $odpocetNaDetiZmeny = $historieOdpoctuNaDeti['zmeny'];
//
//            if (empty($odpocetNaDetiZmeny) || count($odpocetNaDetiZmeny) == 0)
//                $povinnyDokumentDeti = false;
//
//            $uplatnujeOdpocetNaDite = false;
//            foreach ($odpocetNaDetiZmeny as $dite) {
//                if (($dite['datumOdNavrh'] != '' && $dite['poradiDitete_navrh'] != '') || ($dite['datumOd'] != '' && $dite['poradiDitete'] != '')) {
//                    $uplatnujeOdpocetNaDite = true;
//                    break;
//                }
//            }
//            if (!$uplatnujeOdpocetNaDite)
//                $povinnyDokumentDeti = false;
//        }
//
//        if ($povinnyDokumentDeti
//            || self::povinnyDokumentProSekci('partner')
//            || (self::povinnyDokumentProSekci('odecet') && (($hodnotyProhlaseni->naZdravotniStav != $hodnotyProhlaseni->naZdravotniStavNavrh)
//                    || ($hodnotyProhlaseni->odpocetNaStudenta != $hodnotyProhlaseni->naZdravotniStavNavrh))))
//            return true;
//
//        return false;
//    }
//

    /**
     * @param int $rok
     * @param array $hodnoty
     * @param VyuctovaniDaneForm $vyuctovaniForm
     * @return void
     * @throws DBException
     */
    public function zkontrolujPovinneDokumentyProhlaseni($rok, $hodnoty, $vyuctovaniForm)
    {
        $pocetDeti = is_null($vyuctovaniForm->deti) ? 0 : count($vyuctovaniForm->deti);

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_deti_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne') && ($pocetDeti > 0)) {
            $diteStr = isset($hodnoty['pozaduji_upravu_poradi_ditete']) ? 'poradi_ditete_navrh_' : 'poradi_ditete_old_';
            $uplatnujeOdecet = false;

            for ($i = 0; $i < $pocetDeti; $i++) {
                $idDitete = $_POST["dite_" . $i];

                $dite = $vyuctovaniForm->deti[$idDitete];

                //27.9.2023 pridana podminka na vek od 18 let (od 18 je potreba dokladat studium, u mladsich se rodny list doklada pri zalozeni v osobnich udajich)
                if (isset($hodnoty["$diteStr{$i}"]) && ($hodnoty["$diteStr{$i}"] > 0) && ($dite->vekNaZacatkuRoku->y >= 18)) {
                    $uplatnujeOdecet = true;
                }
            }
            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
            if ($uplatnujeOdecet) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        //TODO: kontrolovat povinnost jen pokud je zadana zmena na invaliditu nebo studenta
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne') && $this->jeZmenaSlev($hodnoty, $vyuctovaniForm))
            $this->jePripojenyDokument($typ, $rok);

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_partner_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne') && isset($hodnoty['vyzivujeJinyPoplatnik']) && $hodnoty['vyzivujeJinyPoplatnik'] == 1)
            $this->jePripojenyDokument($typ, $rok);
    }

    /**
     * @param int $rok
     * @param array $hodnoty
     * @param VyuctovaniDaneForm $vyuctovaniForm
     * @return void
     * @throws DBException
     */
    public function zkontrolujPovinneDokumentyZmenyProhlaseni($rok, $hodnoty, $vyuctovaniForm)
    {
        $pocetDeti = is_null($vyuctovaniForm->deti) ? 0 : count($vyuctovaniForm->deti);

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_deti_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_deti_povinne') && ($pocetDeti > 0) && (isset($hodnoty['pozaduji_upravu_poradi_ditete']))) {

            $uplatnujeOdecet = false;
            for ($i = 0; $i < $pocetDeti; $i++) {
                $idDitete = $_POST["dite_" . $i];

                $dite = $vyuctovaniForm->deti[$idDitete];
                if (($hodnoty["dite_poradi_navrh_$i"] > 0) && ($dite->vekNaZacatkuRoku->y >= 18)) {
                    $uplatnujeOdecet = true;
                }
            }
            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
            if ($uplatnujeOdecet) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        //TODO: kontrolovat povinnost jen pokud je zadana zmena na invaliditu nebo studenta
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecet_typ');
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecet_povinne') && ($this->jeZmenaSlev($hodnoty, $vyuctovaniForm)))
            $this->jePripojenyDokument($typ, $rok);
    }


    public static function zkontrolujNastaveni()
    {
        $chybaZamestnavatele = false;
        $chybaPartner = false;

        //pokud je vyzadovan dokument o predchozim zamestnavateli a nejde zadat zadneho, vypise chybu
        if (RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_povinne') && RonPortalSettings::getIntValue('danepocetzamestnavatelu', 3) == 0) {
            $chybaZamestnavatele = true;
        }

        //pokud se vyzaduje potvrzeni, ze druhy poplatnik neuplatnuje slevu na dite a udaje druheho poplatnika neni mozne zadavat pak vypise chybu
        if (RonPortalSettings::getIntValue('dane_dokumenty_partner_povinne') && RonPortalSettings::getIntValue('dane_skryt_dalsiho_poplatnika') == 1) {
            $chybaPartner = true;
        }

        if ($chybaZamestnavatele || $chybaPartner) {
            MessageManager::addWarning(TextManager::_('Mzdy_dane_chyba_nastaveni'));
        }

        if (Prava::vratPrava('admin')->jsouNejakePrava()) {
            if ($chybaZamestnavatele) {
                MessageManager::addErrorMessage(TextManager::_('Mzdy_dane_chyba_nastaveni_zamestnavatel'));
            }
            if ($chybaPartner) {
                MessageManager::addErrorMessage(TextManager::_('Mzdy_dane_chyba_nastaveni_partner'));
            }
        }
    }

    /**
     * @param int $rok
     * @param VyuctovaniDaneForm $vyuctovaniForm
     * @return void
     * @throws DBException
     */
    public function zkontrolujPovinneDokumentyVyuctovani($rok, $vyuctovaniForm)
    {
        $typ = RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_typ', -7);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_skolkovne_povinne') && !is_null($vyuctovaniForm->deti) && (count($vyuctovaniForm->deti) > 0)) {
            $uplatnujeSkolkovne = false;
            for ($i = 0; $i < count($vyuctovaniForm->deti); $i++) {
                $skolkovneNaDite = StringUtils::stringToFloat($_POST['skolkovnenadite_' . $i] ?? 0);
                if ($skolkovneNaDite > 0)
                    $uplatnujeSkolkovne = true;
            }
            //kontrola pripojeneho dokumentu, jen pokud u nektereho z deti uplatnuje odecet
            if ($uplatnujeSkolkovne) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_typ', -8);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_odecitatelne_povinne')) {
            if (($_POST['zadostdaryvyse'] != "" && $_POST['zadostdaryvyse'] > 0) ||
                ($_POST['zadosturokyvyse'] != "" && $_POST['zadosturokyvyse'] > 0) ||
                ($_POST['zadostpenzijnivyse'] != "" && $_POST['zadostpenzijnivyse'] > 0) ||
                ($_POST['zadostpojistnevyse'] != "" && $_POST['zadostpojistnevyse'] > 0) ||
                ($_POST['zadostclenskevyse'] != "" && $_POST['zadostclenskevyse'] > 0) ||
                ($_POST['zadostuhradazazkouskyvyse'] != "" && $_POST['zadostuhradazazkouskyvyse'] > 0) )
            $this->jePripojenyDokument($typ, $rok);
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_typ', -6);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_slevapartner_povinne')) {
            if (isset($_POST['slevanapartnera']) && ($_POST['slevanapartnera'] !== '')) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_typ', -5);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_predchozizam_povinne')) {
            if (RonPortalSettings::getIntValue('danepocetzamestnavatelu', 3) && 
                (($_POST['predchoziplatcedaneadresa_0'] !== '') || ($_POST['predchoziplatcedaneadresa_1'] !== '') || ($_POST['predchoziplatcedaneadresa_2'] !== ''))) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }

        $typ = RonPortalSettings::getIntValue('dane_dokumenty_exekuce_typ', -11);
        if ($typ && RonPortalSettings::getIntValue('dane_dokumenty_exekuce_povinne')) {
            if (isset($_POST['nahrada_exekuce']) && ((float)$_POST['nahrada_exekuce'] > 0)) {
                $this->jePripojenyDokument($typ, $rok);
            }
        }
    }

    /** Zkontroluje zda je k vyctovani zvoleneho roku pripojen dokument daneho typu.
     * @param $typDokumentu
     * @param $rok
     * @return void
     * @throws DBException
     */
    public function jePripojenyDokument($typDokumentu, $rok)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentyOsoby.Id ' .
            '  FROM DokumentyOsoby JOIN Dokument ON (Dokument.Id = DokumentyOsoby.Id) ' .
            ' WHERE DokumentyOsoby.RC =  ' . $params->addParameter($this->pracovniPomer->rc) .
            '   AND DokumentyOsoby.VyuctovaniDane_Rok = ' . $params->addParameter($this->database->dateToParam($rok)) .
            '   AND DokumentyOsoby.VyuctovaniDane_RC = ' . $params->addParameter($this->pracovniPomer->rc) .
            '   AND Dokument.typ = ' . $params->addParameter($typDokumentu);

        if (!$cursor = $this->database->select($sql, $params)) {
            throw new DBException($this->database, $sql);
        }

        $row = $cursor ->fetch();
        if (!$row) {
            $params = $this->database->createParameters();
            $typDokumentuPopis = DBUtils::getSqlValue($this->database, "SELECT Popis FROM TypDokumentu WHERE kod = " . $params->addParameter($typDokumentu), $params);
            throw new Exception(sprintf(ronText('Prohlaseni_neexistuje_povinny_dokument'), $typDokumentuPopis));
        }
    }

    /**
     * @param array $hodnoty
     * @param VyuctovaniDaneForm $vyuctovaniForm
     * @return boolean
     */
    public function jeZmenaSlev($hodnoty, $vyuctovaniForm)
    {
        if (isset($hodnoty['pozaduji_upravu_slev']) && (((isset($hodnoty['sleva_na_zdravotni_stav1'])) && ($hodnoty['sleva_na_zdravotni_stav1'] == 1) && ($vyuctovaniForm->odpocetZdravotniStav != 1))
            || (($hodnoty['sleva_na_zdravotni_stav2'] == 1) && ($vyuctovaniForm->odpocetZdravotniStav != 2))
            || (($hodnoty['sleva_ztpp'] == 1) && ($hodnoty['sleva_ztpp'] != $vyuctovaniForm->odpocetZTPP))
            || (($hodnoty['sleva_na_studenta_nova'] == 1) && ($hodnoty['sleva_na_studenta_nova'] != $vyuctovaniForm->odpocetZTPP))))
            return true;

        return false;
    }

    /** Kontrola zdali je string ve tvaru mm-mm
     * @param $rozsah
     * @return string
     */
    public function kontrolaRozsahuMesicu($rozsah)
    {
        $odDo = "";
        $datum = explode("-", filter_var($rozsah, FILTER_SANITIZE_SPECIAL_CHARS));
        $datumOd = str_pad($datum[0], 2, "0", STR_PAD_LEFT);
        $datumDo = str_pad($datum[1], 2, "0", STR_PAD_LEFT);
        $pattern = "/^(0?[1-9]|1[012])$/";

        if (preg_match($pattern, $datum[0]) && preg_match($pattern, $datum[1]) && $datumDo >= $datumOd) {
            $odDo .= $datumOd . "-" . $datumDo;
        }

        return $odDo;
    }

    /**
     * @return Redirect|RocniVyuctovaniDaneBrowser
     * @throws DBException
     */
    public function browserProhlaseniPoplatnikaDane()
    {
        $osoba = isset($_GET['osoba']) ? $_GET['osoba'] : '';
        //Pokud se jedna o dny kdy jeste nebylo vytvoreno obdobi pak nemusi jeste existovat zaznam o rocnim vyuctovani dane, proto zkontrolovat
        $obdobi = DateUtils::firstDayOfYear(DateUtils::today());         //1.1.aktu.roku

        //14.2.2022 zrusena kontrola na PP se zdanenim po dohode s Radovanem Kulou, nove se nacitaji PP platne v minulem a aktualnim roce
        //Kontrola na PP se zdanenim, musí existovat alespon jeden
        /*$ppSeZdanenim = $this->kontrolaNaVicePracovnichPomeruSeZdanenim($this->database, $obdobi);
        if (empty($ppSeZdanenim)){
            //Pokud existuje pak neni jiste zda je tento PP prirazen k RVD, tedy pokusi se najit PP v RVD, pokud tam nebude pak se pravdepodobne jedna o 1. pristup
            //a nutno vyzvat uzivatele k vyberu PP, tedy v pripade ze se nejedná o zamestnance s prave jednim PP
            throw new Exception(StringUtils::fromUTF8("Neexistuje pracovní poměr pro zdanění! Informujte prosím mzdového pracovníka."));
        }*/

        if ($osoba != '') {
            $pracPomer = PracovniPomer::nactiPP($this->database, $osoba);
        } else {
            $seznamPPosoby = PracovniPomer::nactiPPOsoby($this->database, User::getRC());
            //pokud ma uzivatel jen jeden PP, pak vytvorit zaznam vyuctovani z kopie predchoziho
            if (count($seznamPPosoby) == 1) {
                $pracPomer = $seznamPPosoby[0];
            } else {
                foreach ($seznamPPosoby as $pracovniPomer) {
                    if ($pracovniPomer->typPP == 'HPP') {
                        $pracPomer = $pracovniPomer;
                    }
                }
            }
        }

        /*
         * Pred zobrazenim se zkontroluje zda existuje zaznam na aktualni rok.
         * Pokud neexistuje vytvori se dle lonskeho roku.
         * Pokud neexistuje pro lonsky rok, vytvori a nastavi se prohlaseni elektronicky dle nastaveni v portalu
         * */
        $rec = new RocniVyuctovaniDaneRecord($this->database, StringUtils::fromUTF8('Vytvoření vyúčtování ze seznamu daní'));
        //pokud neexistuje je potreba vytvorit
        if (!$rec->nactiVyuctovani($obdobi, User::getRC())) {
            $recLoni = new RocniVyuctovaniDaneRecord($this->database, $this);
            //lonsky zaznam neni, vytvori se uplne novy
            if (!$recLoni->nactiVyuctovani(DateUtils::incMonth($obdobi, -12), User::getRC())) {
                $recLoni = null;
            }
            //vytvori se jen pokud najde vhodny PP
            if (isset($pracPomer)) {
                $rec->vytvorZaznam($recLoni, $obdobi, $pracPomer);

                try {
                    $rec->insertRecord();
                } catch (Exception $exception) {
                    throw new Exception(StringUtils::fromUTF8('Chyba při vytváření záznamu ročního vyúčtování.'));
                }
            }
        }


//        //Pokud je v URL osoba pak jiz nekontrolovat
//        if ($osoba != "") {
//            //Kontroly zda je vytvoreno, zda ma nastaveno IDPP, zda existuje vice PP a zaroven neni nastaveno IDPP v RVD
//            //Pred zobrazenim browseru overi zda existuje zaznam na aktualni rok, pokud neni pak ho doplni a pouzije hodnoty z lonska pokud jsou k dispozici
//            $this->kontrolaZaznamuRocnihoVyuctovaniDane($osoba, $obdobi);
//        } else {
//            $osoba = $this->nactiPracovniPomerZRocnihoVyuctovaniDane($obdobi);
//            if (!is_null($osoba)) {
//                $pp = PracovniPomer::nactiPP($this->database, $osoba);
//                $pp->vyberNactenyPP();
//            } else {
//                $seznamPPosoby = PracovniPomer::nactiPPOsoby($this->database, User::getRC());
//                //pokud ma uzivatel jen jeden PP, pak vytvorit zaznam vyuctovani z kopie predchoziho
//                if (count($seznamPPosoby) == 1) {
//                    $osoba = $seznamPPosoby[0];
//                    $this->kontrolaZaznamuRocnihoVyuctovaniDane($osoba->idPP, $obdobi);
//                }
//            }
//        }

        //Pokud je vic PP a neni v RVD urcen konkretni pak se vypise hlaska a uzivatel se vyzve aby jeden zvolil.
//        require_once ('mzdy/b_prohlasenipoplatnikadane.php');
//        $b = new BrowserProhlaseniPoplatnikaDane($this->database, null, $obdobi);
        $b = new RocniVyuctovaniDaneBrowser($this->database, $obdobi);
        $b->load();

        return $b;
    }

    /**
     * @param $osoba
     * @param bool $zmenaProhlaseni
     * @return BaseViewObject
     * @throws DBException
     */
    public function zobrazitFormularProhlaseni($osoba, $zmenaProhlaseni = false, $rok = null)
    {
        if ($osoba == "") {
            throw new Exception(StringUtils::fromUTF8("Neexistuje pracovní poměr pro zdanění! Informujte prosím mzdového pracovníka."));
        }

        if (!isset($rok)) {
            if (isset($_GET['obdobi'])) {
                $rok = (int)$_GET['obdobi'];
            } elseif (isset($_GET['rok'])) {
                $rok = mktime(0,0,0,1,1,(int)$_GET['rok']);
            }

            if (!isset($rok)) {
                throw new Exception(StringUtils::fromUTF8('Není zadán rok pro prohlášení k dani.'));
            }
        }

        $this->pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        //Kontrola práv - nejdříve se hledá pp pro zdanění, pokud je nalezen je dle jeho ID načten příslušný pp
        if (!($this->pravo_prohlaseni->lzeZobrazitRC($this->pracovniPomer->rc))) {
            throw new NeniPravoException();
        }

        $form = new VyuctovaniDaneForm($this->database);

        $params = $this->database->createParameters();
        $form->setWhere(
            'RocniVyuctovaniDane.Rok = ' . $params ->addParameter($this->database->timestampToParam($rok)) .
            ' AND RocniVyuctovaniDane.RC = ' . $params ->addParameter($this->pracovniPomer->rc), $params);

        $form->setPracovniPomer($this->pracovniPomer);
        $form->loadRecord('', null, $zmenaProhlaseni);

        if ((RonPortalSettings::getMultilineValue('RonPortal_dane_nerezident_text_souhlasu') != '') && $form['DanovyNerezident']) {
            if (!isset($form['ElektronickyPodpisCestneP'])) {
                Model::addData('prohlaseni_nerezident', RonPortalSettings::getMultilineValue('RonPortal_dane_nerezident_text_souhlasu'));
                return new BaseViewObject('ronportal/mzdy/view_prohlaseninerezidenta.php');
            }
            if ($form['MaPrijemVZahranici']) {
                throw new Exception(ronText('Prohlaseni_nerezident_ma_jine_prijmy'));
            }
        }

        Model::addData('rocnivyuctovaniform', $form);
        Model::addData('zmenaProhlaseni', $zmenaProhlaseni);
        Model::addData('jednoduchyVzhled', RonPortalSettings::getIntValue('dane_zjednoduseny_vzhled'));
        Model::addData('skrytDalsihoPoplatnika', RonPortalSettings::getIntValue('dane_skryt_dalsiho_poplatnika'));
        Model::addData('reportlist', Report::loadFormReports($this->database, 'DaneProhlaseni'));
        if ($zmenaProhlaseni) {
            return new BaseViewObject('ronportal/mzdy/view_zmenyprohlaseni.php');
        }
        return new BaseViewObject('ronportal/mzdy/view_prohlaseni.php');
    }

    /**
     * @param $idPP
     * @param null|int $rok
     * @return BaseViewObject
     * @throws DBException
     * @throws NeniPravoException
     */
    public function zobrazitFormularZadostiOZuctovaniDane($idPP, $rok = null)
    {
        if (!isset($rok)) {
            if (isset($_GET['obdobi'])) {
                $rok = (int)$_GET['obdobi'];
            } elseif (isset($_GET['rok'])) {
                $rok = mktime(0,0,0,1,1,(int)$_GET['rok']);
            }

            if (!isset($rok)) {
                throw new Exception(StringUtils::fromUTF8('Není zadán rok pro prohlášení k dani.'));
            }
        }

        //Načte PP
        $this->pracovniPomer = new PracovniPomer();
        $this->pracovniPomer->nactiPPPodleId($this->database, $idPP);

        //Kontrola práv - nejdříve se hledá pp pro zdanění, pokud je nalezen je dle jeho ID načten příslušný pp
        if (!($this->pravo_vyuctovani->lzeZobrazitRC($this->pracovniPomer->rc))) {
            throw new NeniPravoException();
        }

        $form = new VyuctovaniDaneForm($this->database);
        $params = $this->database->createParameters();
        $form->setWhere(
            'RocniVyuctovaniDane.Rok = ' . $params ->addParameter($this->database->timestampToParam($rok)) .
            ' AND RocniVyuctovaniDane.RC = ' . $params ->addParameter($this->pracovniPomer->rc), $params);

        $form->setPracovniPomer($this->pracovniPomer);
        $form->loadRecord('', null, false, false);

        Model::addData('rocnivyuctovaniform', $form);
        Model::addData('jednoduchyVzhled', RonPortalSettings::getIntValue('dane_zjednoduseny_vzhled'));
        Model::addData('nastaveniOdpoctu', NastaveniOdpoctu::nactiNastaveni($this->database, $rok));

        Model::addData('reportlist', Report::loadFormReports($this->database, 'DaneVyuctovani'));

        switch (date('Y', $rok)) {
            case 2022:
            case 2023:
                return new BaseViewObject("ronportal/mzdy/view_zadostozuctovanizaloh_vz3.php");
            case 2021:
                return new BaseViewObject("ronportal/mzdy/view_zadostozuctovanizaloh_vz2.php");
            case 2020:
            default:
                return new BaseViewObject("ronportal/mzdy/view_zadostozuctovanizaloh.php");
        }

    }

    public function sestaveniOsobUplatnujicichUverZHodnot($hodnoty)
    {
        $result = "";

        for ($i = 0; $i < $hodnoty['pocetjinychosobuplatnujicichodpocet']; $i++)
        {
            if ($hodnoty['jinaosobauplatnujeodpocetjmeno_' . $i] != ""){
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetjmeno_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."$-$";
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetprijmeni_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."$-$";
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetrc_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."$-$";
                $result .= filter_var($hodnoty['jinaosobauplatnujeodpocetadresa_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) ."\n";
            }
        }

        return $result;
    }


    /**
     * @param $hodnoty
     * @return string
     */
    public function sestaveniPredchozichPrijmuKdyOdKohoZHodnot($hodnoty)
    {
        $result = "";

        for ($i = 0; $i < $hodnoty['pocetpredchozichplatcudane']; $i++)
        {
            if ($hodnoty['predchoziplatcedanemesice_' . $i] != ""){

                $datum = explode("-", filter_var($hodnoty['predchoziplatcedanemesice_' . $i], FILTER_SANITIZE_SPECIAL_CHARS));
                $datumOd = str_pad($datum[0], 2, "0", STR_PAD_LEFT);
                $datumDo = str_pad($datum[1], 2, "0", STR_PAD_LEFT);
                $pattern = "/^(0?[1-9]|1[012])$/";

                if (preg_match($pattern, $datum[0]) && preg_match($pattern, $datum[1]) && $datumDo >= $datumOd) {
                    $result .= $datumOd . "-" . $datumDo . "$-$";
                    $result .= filter_var($hodnoty['predchoziplatcedaneadresa_' . $i], FILTER_SANITIZE_SPECIAL_CHARS) . "\n";
                }
            }
        }

        return $result;
    }

    /**
     * @return \Ron\MVC\BaseViewObject|Redirect
     * @throws DBException
     * @throws NeniPravoException
     */
    public function zapisZadosti()
    {
        $rok = mktime(0,0,0,1,1, $_POST['rok']);
        //Kontrola zda nedošlo k submitu po termínu
        if (!static::lzeEditovatVyuctovani($rok)) {
            throw new NeniPravoException();
        }

        $this->pracovniPomer = PracovniPomer::nactiPP($this->database, (int)$_POST["osoba"]);

        $recVyuctovani = new VyuctovaniDaneForm($this->database);
        $recVyuctovani->setLogDescription('Rocni vyuctovani dane poplatnika - portal');

        $recVyuctovani->nactiVyuctovani($rok, $this->pracovniPomer->rc);

        //Kontrola zdali žádost nebyla schválena mzdovým pracovníkem v průběhu editace formuláře
        if ($recVyuctovani->jeZadostSchvalena()) {
            throw new Exception(TextManager::_('zadost_schvalena'));
        }

        $recVyuctovani->setPracovniPomer($this->pracovniPomer);
        $recVyuctovani->nactiDetiPoplatnika($rok, DateUtils::lastDayOfYear($rok));

        try {
            $this->dokumentUpload(static::DOKUMENT_ZAMESTNAVATEL, $rok, $this->pracovniPomer, $this->pravo_vyuctovani);
            $this->dokumentUpload(static::DOKUMENT_SLEVA_PARTNER, $rok, $this->pracovniPomer, $this->pravo_vyuctovani);
            $this->dokumentUpload(static::DOKUMENT_SKOLKOVNE, $rok, $this->pracovniPomer, $this->pravo_vyuctovani);
            $this->dokumentUpload(static::DOKUMENT_SLEVA_EXEKUCE, $rok, $this->pracovniPomer, $this->pravo_vyuctovani);
            $this->dokumentUpload(static::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_dary');
            $this->dokumentUpload(static::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_uroky');
            $this->dokumentUpload(static::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_penzijni');
            $this->dokumentUpload(static::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_zivotni');
            $this->dokumentUpload(static::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_clenske_prispevky');
            $this->dokumentUpload(static::DOKUMENT_SLEVA_ODECET, $rok, $this->pracovniPomer, $this->pravo_vyuctovani, '_zkousky');

            $this->zkontrolujPovinneDokumentyVyuctovani($rok, $recVyuctovani);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->zobrazitFormularZadostiOZuctovaniDane($this->pracovniPomer->idPP, $rok);
        }

        //Pokud ještě není žádost schválená pak spočítat školkovné - a zapsat sumu
        $sumaSkolkovnehoProVyuctovaniDane = 0;

        if (!is_null($recVyuctovani->deti) && (count($recVyuctovani->deti) > 0)) {
            $nastaveni = NastaveniOdpoctu::nactiNastaveni($this->database, $rok);
            for ($i = 0; $i < count($recVyuctovani->deti); $i++) {
                $idDitete = $_POST['ditepoplatnika_' . $i] ?? 0;
                if ($idDitete !== 0) {
                    $dite = $recVyuctovani->deti[$idDitete];

                    if ($dite->vekNaZacatkuRoku->y <= 6) {
                        $idDitete = $_POST['ditepoplatnika_' . $i];
                        //Pokud pres kontrolu klienta projde skolkovne vetsi nez max. pak nastavit na max
                        $skolkovneNaDite = min(StringUtils::stringToFloat($_POST['skolkovnenadite_' . $i]), $nastaveni->maxSkolkovne ?? 0);

                        $params = $this->database->createParameters();

                        $rec = new RonPortalDBRecord($this->database, 'SkolkovneNaDite');
                        $rec->logDescription = 'Skolkovne na dite poplatnika - portal';

                        $rec->addDateField('Rok', true, $rok);
                        $rec->addStringField('RC', true, 15, $this->pracovniPomer->rc);
                        $rec->addStringField('RC_RodinnehoPrislusnika', true, 15, $idDitete);
                        $rec->addFloatField('Skolkovne', false, (int)$skolkovneNaDite);

                        $rec->setWhere(
                            'Rok = ' . $params->addDateParameter($rok) .
                            ' AND RC = ' . $params->addParameter($this->pracovniPomer->rc) .
                            ' AND RC_RodinnehoPrislusnika = ' . $params->addParameter($idDitete), $params);

                        try {
                            $rec->updateOrInsert();
                        } catch (Exception $exception) {
                            throw new Exception(StringUtils::fromUTF8('Chyba při zápisu/aktualizaci školkovného: ' . $exception->getMessage()));
                        }

                        if (is_numeric($skolkovneNaDite)) {
                            $sumaSkolkovnehoProVyuctovaniDane += $skolkovneNaDite;
                        }
                    }
                }
            }
        }

        $termin = mktime(0,0,0,2,15, (int)date("Y", time()));
        $aktualniDatum = time();
        $datumPortal = ($termin < $aktualniDatum) ? $termin : $aktualniDatum;
        $stav = isset($_POST['podepsatzadost']) ? 1 : 0;

        $partnerZtpMesicu = 0;
        $partnerMesicu = 0;
        $slevaNaPartneratext = '';

        //Zápis ve formě mm-mm;mm-mm
        if (isset($_POST['slevanapartnera']) && ($_POST['slevanapartnera'] !== "" )) {
            $slevaNaPartneratext = $this->kontrolaRozsahuMesicu($_POST['slevanapartnera']) . ";" . (isset($_POST['slevanapartneraproztp']) ? $this->kontrolaRozsahuMesicu($_POST['slevanapartneraproztp']) : '');

            $partnerMesiceOd = (int)(explode("-", $_POST['slevanapartnera'])[0]);
            $partnerMesiceDo = (int)(explode("-", $_POST['slevanapartnera'])[1]);

            $partnerMesicu = $partnerMesiceDo - $partnerMesiceOd;
            $partnerMesicu += 1;
        }

        //Zápis ve formě mm-mm;mm-mm
        if (isset($_POST['slevanapartneraproztp']) && ($_POST['slevanapartneraproztp'] !== "")) {
            $partnerZtpMesiceOd = (int)(explode("-", $_POST['slevanapartneraproztp'])[0]);
            $partnerZtpMesiceDo = (int)(explode("-", $_POST['slevanapartneraproztp'])[1]);

            $partnerZtpMesicu = $partnerZtpMesiceDo - $partnerZtpMesiceOd;
            $partnerZtpMesicu += 1;
        }

        if ($partnerZtpMesicu > $partnerMesicu) {
            throw new Exception(ronText('zadost_chyba_pocet_mesicu_ztp_partnera'));
        }

        $recVyuctovani->nastavProvedeniVyuctovani(1);

        $recVyuctovani->findFieldByName('AdresaBytovePotrebyUveru')->setNewValue($_POST['zadostbytovepotrebyuveruadresa'] ?? null);
        $recVyuctovani->findFieldByName('OpravaHodnotDaru')->setNewValue(isset($_POST['zadostdaryvyse']) && StringUtils::stringToFloat($_POST['zadostdaryvyse']) > 0 ? $_POST['zadostdaryvyse'] : 0);

        //do vyuctovani 2020 se zadavala jedna castka uveru, od roku 2021 je ve formulari zatrzitko na vyber zda uver byl pred rokem 2021 nebo az po
        if (date('Y', $rok) <= 2020) {
            $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue(isset($_POST['zadosturokyvyse']) && StringUtils::stringToFloat($_POST['zadosturokyvyse']) > 0 ? $_POST['zadosturokyvyse'] : 0);
        } elseif ((int)$_POST['zadosturokyvyse'] == 0) {
            $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue(0);
            $recVyuctovani->findFieldByName('UrokZeStavebSporPo2021')->setNewValue(0);
        } elseif ($_POST['zadosturokyvyse'] != "") {
                //13.1.2023 Teams RaKu: takže pokud zaškrtnou obě varianty, zapiš do každé položky max. 150. Nejprve použij tu nižší (do 150 000) a zbytek dej do té druhé.
            if (!empty($_POST['zadostbytovepotrebypred2021']) and !empty($_POST['zadostbytovepotrebypo2021'])) {
                $recVyuctovani->findFieldByName('UrokZeStavebSporPo2021')->setNewValue(min((float)$_POST['zadosturokyvyse'], RocniVyuctovaniDaneRecord::LIMIT_UVERU_BYDLENI_PO_2021));
                if ((float)$_POST['zadosturokyvyse'] > RocniVyuctovaniDaneRecord::LIMIT_UVERU_BYDLENI_PO_2021) {
                    $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue((float)$_POST['zadosturokyvyse'] - RocniVyuctovaniDaneRecord::LIMIT_UVERU_BYDLENI_PO_2021);
                } else {
                    $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue(0);
                }
            } elseif (!empty($_POST['zadostbytovepotrebypred2021'])) {
                $recVyuctovani->findFieldByName('OpravaUrokuZeStavebSpor')->setNewValue((float)$_POST['zadosturokyvyse']);
            } elseif (!empty($_POST['zadostbytovepotrebypo2021'])) {
                $recVyuctovani->findFieldByName('UrokZeStavebSporPo2021')->setNewValue((float)$_POST['zadosturokyvyse']);
            }
        }

        $recVyuctovani->findFieldByName('OpravaPenzijnihoPrip')->setNewValue(isset($_POST['zadostpenzijnivyse']) && StringUtils::stringToFloat($_POST['zadostpenzijnivyse']) > 0 ? $_POST['zadostpenzijnivyse'] : 0);
        $recVyuctovani->findFieldByName('ZivotniPojisteni')->setNewValue(isset($_POST['zadostpojistnevyse']) && StringUtils::stringToFloat($_POST['zadostpojistnevyse']) > 0 ? $_POST['zadostpojistnevyse'] : 0);
        $recVyuctovani->findFieldByName('OpravaClenskychPrispevku')->setNewValue(isset($_POST['zadostclenskevyse']) && StringUtils::stringToFloat($_POST['zadostclenskevyse']) > 0 ? $_POST['zadostclenskevyse'] : 0);
        $recVyuctovani->findFieldByName('OpravaUhradZaZkousky')->setNewValue(isset($_POST['zadostuhradazazkouskyvyse']) && StringUtils::stringToFloat($_POST['zadostuhradazazkouskyvyse']) > 0 ? $_POST['zadostuhradazazkouskyvyse'] : 0);
        $recVyuctovani->findFieldByName('OsobyUplatnujiciUver')->setNewValue($this->sestaveniOsobUplatnujicichUverZHodnot($_POST));
        $recVyuctovani->findFieldByName('PredchoziPrijmyKdyOdKoho')->setNewValue($this->sestaveniPredchozichPrijmuKdyOdKohoZHodnot($_POST));
        $recVyuctovani->findFieldByName('Skolkovne')->setNewValue((float)$sumaSkolkovnehoProVyuctovaniDane);

//        $recVyuctovani->addField('DatumZadostiPortal', 'timestamp', false, 0, $datumPortal); TODO overit

        //pri kliknuti na schvalit se zapise i elektronicky podpis
        if ($stav == 1) {
            $recVyuctovani->podepsatZadost();
        }

        $recVyuctovani->findFieldByName('UplatnujeOdpocetPartnera')->setNewValue(isset($_POST['slevanapartnera']) && ($_POST['slevanapartnera'] !== ""));

        $recVyuctovani->findFieldByName('SlevaPartnerObdobi')->setNewValue($slevaNaPartneratext);
        $recVyuctovani->findFieldByName('PocetMesicuOdpoctu')->setNewValue($partnerMesicu);
        $recVyuctovani->findFieldByName('PartnerZTPP')->setNewValue($partnerZtpMesicu);

        $recVyuctovani->findFieldByName('SlevaZaZastavenouExekuci')->setNewValue((float)$_POST['nahrada_exekuce']);

        $recVyuctovani->aktualizaceVypoctuVyuctovani();

        if (!$recVyuctovani->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_rocniho_vyuctovani"));
        }

        //jen uložení, znovu zobrazí formulář
        if ($stav == 0) {
            $rokStr = date('Y', $rok);
            return new Redirect(null, "?menuid=dane&action=formularzadosti&rok=$rokStr&osoba={$this->pracovniPomer->idPP}");
        }

        return new Redirect(null, "?menuid=dane");
    }

    public function zapisProhlaseniNerezidenta()
    {
        $rok = (mktime(0,0,0,1,1, (int)$_POST["obdobi"]));
        $osoba = (int)$_POST['osoba'];
        $rokStr = date('Y', $rok);

        if (!$this->pravo_prohlaseni->lzeZobrazitPP($osoba)) {
            throw new NeniPravoException();
        }

        $pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        $rec = new RocniVyuctovaniDaneRecord($this->database, 'Zapis prohlaseni nerezidenta - portal');

//        TODO: zapsat podpis i u tlacitka jineprijmy?
//        if (isset($_POST['souhlas'])) {
        $podpis = static::podpisUzivatele() . "\n" . ronText('Prohlaseni_nerezident_cestne_prohlaseni') . ":\n" . RonPortalSettings::getMultilineValue('RonPortal_dane_nerezident_text_souhlasu');
        $rec->findFieldByName('ElektronickyPodpisCestneP')->setNewValue($podpis);
//        }
        if (isset($_POST['jineprijmy'])) {
            $rec->findFieldByName('MaPrijemVZahranici')->setNewValue(1);
        }

        $rec->nactiVyuctovani($rok, $pracovniPomer->rc);

        if (!$rec->updateRecord()) {
            throw new Exception(ronText("chyba_aktualizace_prohlaseni"));
        }

        return new Redirect(null, "?menuid=dane&action=formularprohlaseni&osoba=$osoba&rok=$rokStr");
    }

    public function zapisProhlaseni()
    {
//        $hodnoty = $this->nactiPOSTProhlaseni();

        $rok = (mktime(0,0,0,1,1, $_POST["obdobi"]));
        $konecRoku = DateUtils::lastDayOfYear($rok);

        $pracovniPomer = PracovniPomer::nactiPP($this->database, $_POST["osoba"]);
        $this->pracovniPomer = $pracovniPomer;

        if (!($this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc))){
            throw new NeniPravoException();
        }

        $rec = new VyuctovaniDaneForm($this->database);
        $rec->setLogDescription('Zapis prohlaseni poplatnika dane (podepsani) - portal');

        $rec->nactiVyuctovani($rok, $this->pracovniPomer->rc);

        //Kontrola zdali prohlášení nebylo schváleno mzdovým pracovníkem v průběhu editace formuláře prohlášení
        if ($rec->jeProhlaseniSchvaleno()) {
            throw new Exception(TextManager::_('prohlaseni_schvaleno'));
        }
        //kontrola "ProhlaseniElektronicky"
        if (!$rec->lzePodepsatProhlaseniElektronicky()) {
            throw new Exception(TextManager::_('Mzdy_elektronicky_podpis_zrusen'));
        }

        $rec->setPracovniPomer($this->pracovniPomer);
        $rec->nactiDetiPoplatnika($rok, $konecRoku);
        $rec->nactiOdpocetNaDite($rok, $konecRoku);
        $rec->nactiOdpocetNaDiteNavrh(0, $rok, $konecRoku);
        $rec->nactiOdpocet($rok, $konecRoku);

        try {
            $this->dokumentUpload(static::DOKUMENT_OBECNY, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);
            $this->dokumentUpload(static::DOKUMENT_DETI, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);
            $this->dokumentUpload(static::DOKUMENT_PARTNER_NEUPLATNUJE, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);

            $this->zkontrolujPovinneDokumentyProhlaseni($rok, $_POST, $rec);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->zobrazitFormularProhlaseni($this->pracovniPomer->idPP, isset($_POST['zmenovacast']), $rok);
        }

        $podepsani = isset($_POST['podepsatprohlaseni']) ? 1 : 0;

        //RocniVyuctovaniDane ---------------------------------------------------

        //zapise se jen pri podepsani, ne pri ulozeni
        if ($podepsani == 1) {
            $rec->podepsatProhlaseni($_POST['prohlasenikdanicinim']);
//            $rec->findFieldByName('DatumProhlaseniPortal')->setNewValue(time());
        }

        //pokud uzivatel ve formulari zmeni volbu z prohlaseni k dani cinim na necinim, je potreba smazat drive vlozene navrhy odpoctu
        if (($rec['ProhlaseniPortal'] == 0) && $rec->findFieldByName('ProhlaseniPortal')->isChanged()) {
            $recOdpocetNavrh = new OdpocetNavrhRecord($this->database, 'Odstraneni navrhu odpoctu - po zmene prohlaseni Portal');

            $paramsOdp = $this->database->createParameters();
            $recOdpocetNavrh->setWhere( " DatumVlozeniNavrhu >= " . $paramsOdp->addDateParameter($rok) .
                " AND DatumVlozeniNavrhu < " . $paramsOdp->addDateParameter(DateUtils::lastDayOfYear($rok)) .
                ' AND RC  = ' . $paramsOdp->addParameter($this->pracovniPomer->rc) .
                ' AND OsobniCislo  = ' . $paramsOdp->addParameter($this->pracovniPomer->osobniCislo) .
                ' AND DatumNastupu  = ' . $paramsOdp->addDateParameter($this->pracovniPomer->datumNastupu) .
                ' AND Stav = ' . $paramsOdp->addParameter(0)
                , $paramsOdp);

            $recOdpocetNavrh->checkMultiple = false;
            try {
                $recOdpocetNavrh->deleteRecords();
            } catch (Exception $exception) {
                MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba pri odstraneni navrhu odpoctu.'));
            }

            $recOdpocetDiteNavrh = new OdpocetNaDiteNavrhRecord($this->database, 'Odstraneni navrhu odpoctu na dite - po zmene prohlaseni Portal');

            $paramsOdp = $this->database->createParameters();
            $recOdpocetDiteNavrh->setWhere(" DatumVlozeniNavrhu >= " . $paramsOdp->addDateParameter($rok) .
                " AND DatumVlozeniNavrhu < " . $paramsOdp->addDateParameter(DateUtils::lastDayOfYear($rok)) .
                ' AND RC  = ' . $paramsOdp->addParameter($this->pracovniPomer->rc) .
                ' AND OsobniCislo  = ' . $paramsOdp->addParameter($this->pracovniPomer->osobniCislo) .
                ' AND DatumNastupu  = ' . $paramsOdp->addDateParameter($this->pracovniPomer->datumNastupu) .
                ' AND Stav = ' . $paramsOdp->addParameter(0)
                , $paramsOdp);

            $recOdpocetDiteNavrh->checkMultiple = false;
            try {
                $recOdpocetNavrh->deleteRecords();
            } catch (Exception $exception) {
                MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba pri odstraneni navrhu odpoctu na dite.'));
            }
        }

        $jinyPoplatnik = false;
        $rec->findFieldByName('VyzivujeJinyPoplatnik')->setNewValue(isset($_POST['vyzivujeJinyPoplatnik']) && (int)$_POST['vyzivujeJinyPoplatnik'] == 1);
        if (isset($_POST['vyzivujeJinyPoplatnik']) && (int)$_POST['vyzivujeJinyPoplatnik'] == 1) {
            $jinyPoplatnik = true;
        }

        $rec->findFieldByName('JinyPoplatnikUplatnujeZv')->setNewValue($jinyPoplatnik ? (int)$_POST['jinyPoplatnikUplatnujeZv'] : 0);
        $rec->findFieldByName('JinyPoplatnikZamestnani')->setNewValue($jinyPoplatnik ? (int)$_POST['jinyPoplatnikZamestnani'] : 0);
        $rec->findFieldByName('JinyPoplatnikJmeno')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikJmeno'] : null);
        $rec->findFieldByName('JinyPoplatnikPrijmeni')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikPrijmeni'] : null);
        $rec->findFieldByName('JinyPoplatnikRC')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikRC'] : null);

        $rec->findFieldByName('JinyPoplatnikObec')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikObec'] : null);
        $rec->findFieldByName('JinyPoplatnikPSC')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikPSC'] : null);
        $rec->findFieldByName('JinyPoplatnikUlice')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikUlice'] : null);

        $rec->findFieldByName('JinyPoplatnikNazevPlatce')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikNazevPlatce'] : null);
        $rec->findFieldByName('JinyPoplatnikObecPlatce')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikObecPlatce'] : null);
        $rec->findFieldByName('JinyPoplatnikUlicePlatce')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikUlicePlatce'] : null);
        $rec->findFieldByName('JinyPoplatnikPSCPlatce')->setNewValue($jinyPoplatnik ? $_POST['jinyPoplatnikPSCPlatce'] : null);

        //Nastavi se az pri podepsani a pouze tehdy kdyz NENI Prohlaseni schvaleno mzdovym pracovnikem
        //Pole muze menit dokud nebude schvaleno mzdovym pracovnikem
        if ($rec->findFieldByName('ProhlaseniPortalStav')->value != 1) {
            $rec->nastavPracovniPomer($pracovniPomer);
        }

        $rec->doplnHistoriiProhlaseni($podepsani == 1 ? ronText('prohlaseni_podepsani') : ronText('prohlaseni_ulozeni'));

        if (!$rec->updateRecord()) {
            throw new Exception(TextManager::_("chyba_aktualizace_prohlaseni"));
        }
        //RocniVyuctovaniDane ---------------------------------------------------



        //OdpocetNavrh - pokud došlo ke změně požadovaných slev -----------------
        //datum od - bude datum hlášené změny a poté již bude použit tento datum (při opětovném otevření bude hledán datum v rámci 1.1. - 31.12.)

        if ($_POST['prohlasenikdanicinim']) {
            //Pokud ano, pak zapsat zmeny
            if (isset($_POST["pozaduji_upravu_slev"])) {
                $rec->nactiOdpocetNavrh(0, $rok, $konecRoku);
                $recOdpocetNavrh = new OdpocetNavrhRecord($this->database, 'Prohlaseni poplatnika dane a navrh odpoctu - portal');

                $recOdpocetNavrh->loadPost($this->pracovniPomer, $rec->odpocetNavrhDatumOd, $rec->odpocetNavrhDatumVlozeni);

                try {
                    $recOdpocetNavrh->updateOrInsert();
                } catch (Exception $exception) {
                    MessageManager::addErrorMessage(TextManager::_('chyba_vkladani_aktualizaci_odpoctu'));
                }
            }

            //OdpocetNaDiteNavrh - pokud došlo ke změně poradi slev u ditete --------
            if (isset($_POST['pozaduji_upravu_poradi_ditete']) && !is_null($rec->deti) && (count($rec->deti) > 0)) {

                for ($i = 0; $i < count($rec->deti); $i++) {
                    $idDitete = $_POST["dite_" . $i];

                    $dite = $rec->deti[$idDitete];

                    $puvodniPoradi = $dite->poradDitete;
                    $novePoradi = $_POST["dite_poradi_navrh_" . $i];

                    if (($novePoradi != "") && ($puvodniPoradi != $novePoradi)) {
                        $recOdpocetNavrh = new OdpocetNaDiteNavrhRecord($this->database, 'Prohlaseni poplatnika dane a zmena odpoctu na deti - portal');

                        $recOdpocetNavrh->loadPost($this->pracovniPomer, $i, $dite);

                        try {
                            $recOdpocetNavrh->updateOrInsert();
                        } catch (Exception $exception) {
                            MessageManager::addErrorMessage(TextManager::_('chyba_vkladani_aktualizaci_navrhu_odpoctu_ditete'));
                        }
                    }
                }
            }
        }

        $osobaDoURL = $this->pracovniPomer->idPP;

        //jen uložení, znovu zobrazí formulář
        if ($podepsani == 0) {
            return new Redirect(null, "?menuid=dane&action=formularprohlaseni&osoba=$osobaDoURL&rok=" . date('Y', $rok));
        }

        return new Redirect(null, "?menuid=dane&osoba=$osobaDoURL");
    }

    /** Zapis zmenove casti, prevzaty z puvodniho kodu, castecne upraven.
     * @return BaseViewObject|Redirect
     * @throws DBException
     * @throws NeniPravoException
     */
    public function zapisZmenyProhlaseni()
    {
        $rok = (mktime(0,0,0,1,1, $_POST["obdobi"]));
        $konecRoku = DateUtils::lastDayOfYear($rok);

        $pracovniPomer = PracovniPomer::nactiPP($this->database, $_POST["osoba"]);
        $this->pracovniPomer = $pracovniPomer;

        $datumOd = $rok;
        $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        if (!$this->pravo_prohlaseni->lzeZobrazitRC($this->pracovniPomer->rc)) {
            throw new NeniPravoException();
        }

        $rec = new VyuctovaniDaneForm($this->database);
        $rec->setLogDescription('Rocni vyuctovani dane poplatnika - aktualizace zmenove casti, portal');

        $rec->nactiVyuctovani($rok, $this->pracovniPomer->rc);

        if (!$rec->lzePodepsatProhlaseniElektronicky()) {
            throw new Exception(TextManager::_('Mzdy_elektronicky_podpis_zrusen'));
        }

//        $aktualniStavOdpoctuOsoby = $this->nactiAktualniStavOdpoctu($this->pracovniPomer);
//
//        //zmeny porovnat s aktu. hodnotami
//        $hodnotyPOSTu = $this->nactiPOSTZmenoveCastiAVyhledejZmenu($aktualniStavOdpoctuOsoby);
//        $zmenyOdpoctuNavrh = $this->nactiOdpocetNavrh($this->pracovniPomer, $hodnotyPOSTu->obdobi_timestamp, false);
        $rec->setPracovniPomer($this->pracovniPomer);
        $rec->nactiDetiPoplatnika($rok, $konecRoku);
        $rec->nactiOdpocetNaDite(DateUtils::today(), DateUtils::today());
        $rec->nactiOdpocet($rok, $konecRoku);

        try {
            $this->dokumentUpload(static::DOKUMENT_OBECNY, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);
            $this->dokumentUpload(static::DOKUMENT_DETI, $rok, $this->pracovniPomer, $this->pravo_prohlaseni);

            $this->zkontrolujPovinneDokumentyZmenyProhlaseni($rok, $_POST, $rec);
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
            return $this->zobrazitFormularProhlaseni($this->pracovniPomer->idPP, true, $rok);
        }

        $aktualizovatRVDSlevy = false;
        $aktualizovatRVDDeti = false;
        if (isset($_POST["pozaduji_upravu_slev"])) {
            $rec->nactiOdpocetNavrh(2, $rok, $konecRoku);
            $recOdpocetNavrh = new OdpocetNavrhRecord($this->database, 'Prohlaseni poplatnika dane a navrh odpoctu - portal');

            $recOdpocetNavrh->loadPost($this->pracovniPomer, $rec->odpocetNavrhDatumOd, $rec->odpocetNavrhDatumVlozeni, true);

            try {
                if ($recOdpocetNavrh->loadRecord()) {
                    if ($recOdpocetNavrh->updateRecord() > 0) {
                        $aktualizovatRVDSlevy = true;
                    }
                } else {
                    $recOdpocetNavrh->insertRecord();
                    $aktualizovatRVDSlevy = true;
                }
            } catch (Exception $exception) {
                MessageManager::addErrorMessage(TextManager::_('chyba_vkladani_aktualizaci_odpoctu'));
            }
        }

        if (isset($_POST['pozaduji_upravu_poradi_ditete']) && !is_null($rec->deti) && (count($rec->deti) > 0)) {
            $rec->nactiOdpocetNaDiteNavrh(2, $rok, $konecRoku);
            for ($i = 0; $i < count($rec->deti); $i++) {
                $idDitete = $_POST["dite_" . $i];

                $dite = $rec->deti[$idDitete];

                $puvodniPoradi = $dite->poradDitete;
                $novePoradi = $_POST["dite_poradi_navrh_" . $i];

                if (($novePoradi != "") && ($puvodniPoradi != $novePoradi)) {
                    $recOdpocetNavrh = new OdpocetNaDiteNavrhRecord($this->database, 'Prohlaseni poplatnika dane a zmena odpoctu na deti - portal');

                    $recOdpocetNavrh->loadPost($this->pracovniPomer, $i, $dite, true);

                    try {
                        if ($recOdpocetNavrh->loadRecord()) {
                            if ($recOdpocetNavrh->updateRecord() > 0) {
                                $aktualizovatRVDDeti = true;
                            }
                        } else {
                            $recOdpocetNavrh->insertRecord();
                            $aktualizovatRVDDeti = true;
                        }
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage(TextManager::_('chyba_vkladani_aktualizaci_navrhu_odpoctu_ditete'));
                    }
                }
            }
        }

        //aktualizuje pokud doslo k zapisu zmeny
        if ($aktualizovatRVDDeti || $aktualizovatRVDSlevy){
            //Pokud probehlo OK pak aktualizovat RVD na "ProhlaseniPortalZmenaStav"
            //Pokud probehlo OK pak aktualizovat RVD na "ProhlaseniPortalZmenaStav"

            $rec->zmenaProhlaseni();

            $rec->doplnHistoriiProhlaseni(ronText('prohlaseni_ulozeni_zmen'));

            MessageManager::addInfoMessage(ronText('Prohlaseni_zmenova_cast_zapsana'));
        }
/*
    TODO: kdy se ma zde provadet mazani? - provadi se pri update rocniho vyuctovani, kdy se vyhledaji a smazou navrhy odpoctu pro jine PP
        if (!isset($hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev)  && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev_old == 1 && $zmenyOdpoctuNavrh->stav == 2 ){
            //Pokud uzivatel odvola zmenu jeste pred schvalenim, pak zmenu smazat

            $this->ukliditNavrhyRocnihoVyuctovaniDane($this->pracovniPomer, (int)$hodnotyPOSTu->zmenova_cast_datumOd_old,'Odstraneni navrhu odpoctu/odpoctu na dite - uzivatel neuplatnuje slevu  Portal', true);
            MessageManager::addInfoMessage(ronText('Prohlaseni_zmenova_cast_odstranena'));
        } elseif (!isset($hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev) && $hodnotyPOSTu->zmenova_cast_pozadujiUpravuSlev_old == 1 && !isset($zmenyOdpoctuNavrh->stav) ){

            //Pokud v prubehu otevreni formulare mzdovy pracovnik zmenu jiz schvali a uzivatel chce zmenu odvolat
            MessageManager::addErrorMessage(ronText('Prohlaseni_zmenova_cast_jiz_schvalena'));
        }*/

        try {
            $rec->updateRecord();
        } catch (Exception $exception) {
            throw new Exception('Chyba pri aktualizaci prohlaseni ' . $exception->getMessage());
        }

        return new Redirect(null,"?menuid=dane&osoba={$this->pracovniPomer->idPP}");
    }

    function dokumentUpload($castProhlaseni, $rok, $pracovniPomer, $prava, $suffix = '')
    {
        switch ($_FILES[$castProhlaseni . $suffix]['error']) {
            case UPLOAD_ERR_OK :
            case UPLOAD_ERR_NO_FILE: //soubor nemusi byt ve formulari pripojen, kontrola se provadi az po nacteni
                break;

            case UPLOAD_ERR_INI_SIZE :
            case UPLOAD_ERR_FORM_SIZE :
                throw new \Exception( TextManager::_('Err_velky_soubor') );

            case UPLOAD_ERR_PARTIAL :
                throw new \Exception( TextManager::_('Err_nekompletni_soubor') );

            case UPLOAD_ERR_NO_TMP_DIR :
                throw new \Exception( TextManager::_('Err_chybi_slozka') );

            case UPLOAD_ERR_CANT_WRITE :
                throw new \Exception( TextManager::_('Err_chyba_zapisu') );

            case UPLOAD_ERR_EXTENSION :
            default :
                throw new \Exception( TextManager::_(sprintf('Err_chyba_nahrani_souboru', $_FILES[$castProhlaseni . $suffix]['error'])));
        }

        $pocetDokumentu = $_POST[$castProhlaseni . $suffix . "_pocet"] ?? 1;

        for ($i = 1; $i <= $pocetDokumentu; $i++) {
            $index = $i > 1 ? "-$i" : "";

            if (($_FILES[$castProhlaseni . $suffix . $index]['error'] == UPLOAD_ERR_INI_SIZE) || ($_FILES[$castProhlaseni . $suffix . $index]['error'] == UPLOAD_ERR_FORM_SIZE)) {
                throw new Exception(TextManager::_('Err_velky_soubor') . ' (' . $_FILES[$castProhlaseni . $suffix . $index]['name'] . ')');
            }

            if ($_FILES[$castProhlaseni . $suffix . $index]['error'] !== UPLOAD_ERR_OK) {
                return;
            }

            $datumOd = $rok;
            $datumDo = DateUtils::incDay(DateUtils::incMonth($datumOd, 12), -1);
            Prava::nastavObdobiProPrava($datumOd, $datumDo);

            if (!$this->pracovniPomer->lzeZobrazitPP($prava)) {
                throw new NeniPravoException();
            }

            $typDokumentu = static::vratTypDokumentuProSekci($castProhlaseni);
            $params = $this->database->createParameters();
            $popisTypu = DBUtils::getSqlValue($this->database, "SELECT Popis FROM TypDokumentu WHERE kod = " . $params->addParameter($typDokumentu), $params);
            $popisTypu = ronText($popisTypu);
            $popisDokumentu = null;

            $popis = RonPortalSettings::getValue('dane_dokumenty_maska', '%TYPDOKUMENTU% %PORADOVECISLO%');

            if ($suffix !== '') {
                $popisDokumentu = substr($suffix, 1);

                switch ($popisDokumentu) {
                    case 'dary':
                        $popisDokumentu = ronText('zadost_dary_pripojeni_dokumentu');
                        break;
                    case 'uroky':
                        $popisDokumentu = ronText('zadost_uroky_pripojeni_dokumentu');
                        break;
                    case 'penzijni':
                        $popisDokumentu = ronText('zadost_penzijni_pripojeni_dokumentu');
                        break;
                    case 'zivotni':
                        $popisDokumentu = ronText('zadost_zivotni_pripojeni_dokumentu');
                        break;
                    case 'clenske_prispevky':
                        $popisDokumentu = ronText('zadost_clenske_pripojeni_dokumentu');
                        break;
                    case 'zkousky':
                        $popisDokumentu = ronText('zadost_zkousky_pripojeni_dokumentu');
                        break;
                }

                $popisTypu = "{$popisTypu} - {$popisDokumentu}";
            }

            $noveCislo = static::vratNovePoradoveCisloDokumentu($this->database, $pracovniPomer, $rok, $typDokumentu, $popisDokumentu);

            $popis = str_replace(
                ['%JMENO%', '%PRIJMENI%', '%OSOBNICISLO%', '%IDOSOBY%', '%IDPP%', '%DATUM%', '%TYPDOKUMENTU%', '%PORADOVECISLO%'],
                [$pracovniPomer->jmeno, $pracovniPomer->prijmeni, $pracovniPomer->osobniCislo, $pracovniPomer->rc, $pracovniPomer->idPP, date('j.n.Y', time()), $popisTypu, $noveCislo],
                $popis);

            // vlozi do databaze obsah dokumentu
            $id = DokumentyUtils::vlozDokument($this->database, $castProhlaseni . $suffix . $index, $typDokumentu, $popis, $this);

            $rec = new RonPortalDBRecord($this->database, 'DokumentyOsoby', $this);
            $rec->addField('Id', 'int', false, 0, $id);
            $rec->addField('RC', '', false, 15, $pracovniPomer->rc);
            $rec->addField('VyuctovaniDane_Rok', 'timestamp', false, 0, $datumOd);
            $rec->addField('VyuctovaniDane_RC', '', false, 15, $pracovniPomer->rc);

            $rec->insertRecord();
        }
    }

    /**
     * Nastavi pole provest rocni vyuctovani na zadam a zobrazi formular, nebo nastavi nezadam a zadost podepise. Provadi se odkazem z browseru.
     * @param int $rok
     * @param int $zadam
     * @param $idPP
     * @return BaseViewObject|Redirect
     * @throws DBException
     * @throws NeniPravoException
     */
    public function nastavProvestRocniVyuctovaniDane($rok, $zadam, $idPP)
    {
        $this->pracovniPomer = PracovniPomer::nactiPP($this->database, $idPP);

        if (!($this->pravo_vyuctovani->lzeZobrazitRC($this->pracovniPomer->rc))) {
            throw new NeniPravoException();
        }

        $rec = new RocniVyuctovaniDaneRecord($this->database, 'Rocni vyuctovani dane zadam/nezadam pres odkaz - portal');

        $rok = mktime(0,0,0,1,1,$rok);

        $rec->nactiVyuctovani($rok, $this->pracovniPomer->rc);
        if (!$rec->lzePodepsatZadostElektronicky()) {
            throw new Exception(TextManager::_('Mzdy_elektronicky_podpis_zrusen'));
        }

        $rec->nastavProvedeniVyuctovani($zadam);

        //po kliknuti na nezadam se zmeni stav ProvestRocniVyuctovani a podepise
        if ($zadam == 0) {
            $rec->podepsatZadost();
        }

        $rec->doplnHistoriiZadosti($zadam ? ronText('zadost_stav_zadam') : ronText('zadost_stav_nezadam'));

        try {
            $rec->updateRecord();
        } catch (Exception $exception) {
            throw new Exception(StringUtils::fromUTF8('Chyba při zápisu rocniho zuctovani dane.'));
        }

        //pri stisku zadam se pokracuje primo na formular zadosti
        if ($zadam == 1) {
            return $this->zobrazitFormularZadostiOZuctovaniDane($idPP, $rok);
        }

        return new Redirect(null,'?menuid=dane&');
    }

    public function report()
    {
        if (!$_GET['idpp'] || !$_GET['rok']) {
            throw new Exception('Neni zadano rocni vyuctovani pro tisk!');
        }

        Prava::nastavObdobiProPrava(time(), time());

        $idPP = (int)$_GET['idpp'];
        $rok  = mktime(0,0,0,1,1, (int)$_GET['rok']);
        $konecRoku = DateUtils::incDay(DateUtils::incMonth($rok, 12), -1);

        if (($_GET['typ'] == 'prohlaseni')) {
            $reportName = 'DaneProhlaseni';
            if (!$this->pravo_prohlaseni->lzeZobrazitPP($idPP)) {
                throw new NeniPravoException();
            }
        }
        if ($_GET['typ'] == 'vyuctovani') {
            $reportName = 'DaneVyuctovani';
            if (!$this->pravo_vyuctovani->lzeZobrazitPP($idPP)) {
                throw new NeniPravoException();
            }
        }

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, $reportName, false);
        $result->nastavPP(PracovniPomer::nactiPP($this->database, $_GET['idpp']));
        $result->nastavObdobi(3, $rok, $konecRoku);

        $result->dateParams['rok'] = date('j.n.Y', $rok);

        return $result;
    }

    public function overPravoNaDokument($idDokumentu, $pracovniPomer, $rok)
    {
        $params = $this->database->createParameters();
        $sql =
            'SELECT DokumentyOsoby.Id ' .
            '  FROM DokumentyOsoby ' .
            ' WHERE DokumentyOsoby.Id = ' . $params->addParameter($idDokumentu) .
            '   AND DokumentyOsoby.RC =  ' . $params->addParameter($pracovniPomer->rc) .
            '   AND DokumentyOsoby.VyuctovaniDane_Rok = ' . $params->addDateParameter($rok) .
            '   AND DokumentyOsoby.VyuctovaniDane_RC = ' . $params->addParameter($pracovniPomer->rc);

        if (!($cursor = $this->database->select($sql, $params))) {
            throw new DBException($this->database, $sql);
        }

        if (!$cursor ->fetch()) {
            throw new Exception('Pozadovany dokument vyuctovani dane nebyl nalezen!');
        }
    }

    public function downloadDokument($idDokumentu, $osoba, $rok)
    {
        $datumOd = mktime(0,0,0,1,1, $rok);
        $datumDo = DateUtils::lastDayOfYear($datumOd);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        $pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        if (!$this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc)) {
            throw new NeniPravoException();
        }

        $this->overPravoNaDokument($idDokumentu, $pracovniPomer, $datumOd);

        return new DokumentOutputObject($this->database, $idDokumentu);
    }

    public function smazDokument($idDokumentu, $osoba, $rok)
    {
        $datumOd = mktime(0,0,0,1,1, $rok);
        $datumDo = DateUtils::lastDayOfYear($datumOd);
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        $pracovniPomer = PracovniPomer::nactiPP($this->database, $osoba);

        if (!$this->pravo_prohlaseni->lzeZobrazitRC($pracovniPomer->rc)) {
            throw new NeniPravoException();
        }

        $this->overPravoNaDokument($idDokumentu, $pracovniPomer, $datumOd);

        $rec = new RonPortalDBRecord($this->database, 'DokumentyOsoby');

        $rec ->addIntField('Id', true);
        $rec ->addStringField('RC', true);

        $params = $this->database->createParameters();
        $rec ->deleteRecord(
            'Id = ' . $params ->addParameter($idDokumentu) .
            ' AND RC = ' . $params ->addParameter($pracovniPomer->rc),
            $params);

        try {
            DokumentyUtils::smazDokument($this->database, $idDokumentu, $this);
        } catch (Exception $e) {
            // chybu ignoruje - zobrazi ji jen v debug rezimu - dokument totiz muze byt pripojen jeste v jine agende
            if (App::getDebugMode()) {
                MessageManager::addErrorMessage($e);
            }
        }

        $action = $_GET['backurl'] ?? 'formularprohlaseni';

        return new Redirect(null, "?menuid=dane&action=$action&osoba={$pracovniPomer ->idPP}&rok=$rok");
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    public function mojeProhlaseniDashboard($output)
    {
        require_once 'ron/httputils.php';

        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=dane">' .
            '<tr>' .
//            '<th>' . \Ron\ronText('zadost') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
            '</th><th>' . ronText('rok') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
            '</th><th>' . ronText('Mzdy_stav_prohlaseni') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
            '</th><th>' . ronText('Mzdy_stav_zadosti') . "&nbsp;&nbsp;&nbsp;&nbsp;" .
//            '</th><th>' . \Ron\ronText('Mzdy_pozaduji_rocni_vyuctovani') . '</th>' .
            '</tr>';

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT DatumProhlaseniPortal, ProhlaseniPortalStav, S_AnoNe.popis, Rok, ProvestRocniVyuctovani, ZadostPortalStav, DatumZadostiPortal, ' .  // 0..6
            '{fn_CASTWIDEMEMO(RVD.PoznamkaProZamestnance)}, {fn_CASTWIDEMEMO(RVD.PoznamkaProZamZadost)}, ProhlaseniPortalZmenaStav ' . // 7 - 9

            ' FROM RocniVyuctovaniDane RVD LEFT JOIN S_AnoNe ON (RVD.ProvestRocniVyuctovani = S_AnoNe.Kod)' .
            ' Where RC = ' . $params ->addParameter(User::getRC()) .
            '  AND {fn_YEAR(RVD.Rok)} >= ' . $params->addParameter(RonPortalSettings::getIntValue('dane_pocatecni_rok', 2020));

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $rok = $this->database->stringToTimestamp($row[3]);

            $rokStr = date('Y', $rok);

            $stavProhlaseni = ronText("prohlaseni_stav_{$row[1]}");

            if ($row[1] == 1) {
                $stavProhlaseni .= ' / '  . ronText("prohlaseni_zmena_stav_$row[9]");
            }


            $stavZadosti = ronText("zadost_stav_{$row[5]}");

            $seznamPP = PracovniPomer::nactiPPOsoby($this->database, User::getRC(), $rok, DateUtils::lastDayOfYear($rok));
            $pracPomer = $seznamPP[0];

            $html .=
                '<tr>' .

                "<td>" . date("Y", $rok) . '</td>' .

                "<td><a href='?menuid=dane&action=formularprohlaseni&osoba=" . $pracPomer->idPP . "&rok=" . $rokStr . "'>" . $stavProhlaseni . "</a>&nbsp;" .
                ($row[0] != "" ? sprintf(ronText('zadost_podepsano'), StringUtils::format('date', $cursor ->stringToTimestamp($row[0]))) : ronText('nepodepsano')) . '</td>' .

                "<td><a href='?menuid=dane&action=formularzadosti&osoba=" . $pracPomer->idPP . "&rok=" . $rokStr . "'>" . $stavZadosti . "</a>&nbsp;" .
                ($row[6] != "" ? sprintf(ronText('zadost_podepsano'), StringUtils::format('date', $cursor ->stringToTimestamp($row[6]))) : ronText('nepodepsano')) . '</td>' .

                '</tr>';

            //je zadana poznamka pro zamestnance k prohlaseni nebo zadosti
            if ($row[7] || $row[8]) {
                $html .=
                    '<tr>' .

                    '<td>&nbsp;</td>' .

                    '<td colspan="2">' . ($row[8] !== '' ? $row[8] : $row[7]) . '</td>' .

                    '</tr>';
            }
        }
        $cursor->close();

        $html .= '</table>';

        $output->addText('html', $html);
    }

    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']){
            case 'mojeprohlaseni':
                $this->mojeProhlaseniDashboard($output);
                break;
        }
    }

    public function handleAction($action)
    {
        if (!($this->pravo_prohlaseni->jsouNejakePrava() || $this->pravo_vyuctovani->jsouNejakePrava())){
            throw new NeniPravoException();
        }

        TextManager::getInstance()->loadStrings('mzdy.ini');
        TextManager::getInstance()->loadStrings('person.ini');
        HtmlHeaders::getInstance()->addCss('css/mzdy.css');
        HtmlHeaders::getInstance()->addScript('script/mzdy.js');
        HtmlHeaders::getInstance()->addScript('', '?menuid=dane&action=translations&ver=' . HtmlHeaders::getInstance() ->versionId);

        static::zkontrolujNastaveni();

        HtmlHeaders::getInstance()->setTitle(ronText('RON_Portal') . ' - ' . ronText('MI_Dane'));

        $daneaction = isset($_GET['action']) ? $_GET['action'] : '';
        switch ($daneaction) {
            case 'mojeprohlaseni':
                return parent::handleAction($action);
            // dokumenty
            case 'dokumentdelete' :
                Csrf::validateCSRFGetParam('dokumentdane_' . (int)$_GET['osoba']);
                return $this ->smazDokument((int)$_GET['record'], (int)$_GET['osoba'], (int)$_GET['rok']);

            case 'dokumentdownload' : // download dokumentu
                Csrf::validateCSRFGetParam('dokumentdane_' . (int)$_GET['osoba']);
                return $this->downloadDokument((int)$_GET['record'], (int)$_GET['osoba'], (int)$_GET['rok']);

            case 'formularzadosti':
                HtmlHeaders::getInstance()->setTitle(ronText('RON_Portal') . ' - ' . ronText('Mzdy_formular_zadosti'));
                return $this->zobrazitFormularZadostiOZuctovaniDane((int)$_GET['osoba']);
            case 'formularprohlaseni':
                HtmlHeaders::getInstance()->setTitle(ronText('RON_Portal') . ' - ' . ronText('prohlaseni_nadpis'));
                return $this->zobrazitFormularProhlaseni((int)$_GET['osoba'], isset($_GET['zmenovacast']));

            case 'prohlaseninerezidenta':
                Csrf::validateCSRFPostParam('prohlaseni_'. (int)$_POST['osoba']);
                return $this->zapisProhlaseniNerezidenta();

            case 'ulozitprohlaseni':
                Csrf::validateCSRFPostParam('prohlaseni_'. (int)$_POST['osoba']);
                if (isset($_POST['zmenovacast'])) {
                    return $this->zapisZmenyProhlaseni();
                }
                return $this->zapisProhlaseni();
            case 'ulozitzadost':
                Csrf::validateCSRFPostParam('vyuctovani_'. (int)$_POST['osoba']);
                return $this->zapisZadosti();
            case 'rocnizuctovani':
                Csrf::validateCSRFGetParam('provestvyuctovani_' . (int)$_GET['osoba']);
                return $this->nastavProvestRocniVyuctovaniDane((int)$_GET['rok'], (int)$_GET['zadamzuctovani'], (int)$_GET['osoba']);
            case 'report':
                return $this->report();
            case 'translations':
                return $this->translations();

            default:
                return $this->browserProhlaseniPoplatnikaDane();
        }
    }

}
