<?php

namespace RonPortal\Mzdy;

use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;


class MzdovePolozkyBrowser extends RonPortalBrowser
{
    function __construct($database)
    {
        parent::__construct($database, 'mzdovepolozky');

        $this->code = 'browsermzdovepolozky';
        $this->title = TextManager::_('Prehled_mzdovych_polozek');

        $this->sqlFromPart =
            '( MzdovaPolozkaVeMzde INNER JOIN TypMzdovePolozky ' .
            '  ON TypMzdovePolozky.Kod = MzdovaPolozkaVeMzde.MzdovaPolozka ' .
            ') ' .
            'LEFT OUTER JOIN VlastnostMzdovePolozky ' .
            'ON VlastnostMzdovePolozky.Kod = TypMzdovePolozky.Kod ' .
            '   AND VlastnostMzdovePolozky.PlatnostOd <= MzdovaPolozkaVeMzde.Obdobi' .
            '   AND (VlastnostMzdovePolozky.PlatnostDo >= MzdovaPolozkaVeMzde.Obdobi OR  VlastnostMzdovePolozky.PlatnostDo IS NULL)';

        $this->sqlWherePart = 'MzdovaPolozkaVeMzde.DruhZdrojeMzdovePolozky = 4';   // jen dochazkove MP

        $this->addField()
            ->setSqlFieldName('MzdovaPolozkaVeMzde.MzdovaPolozka')
            ->setCode('col-kod') ->setHeaderText('Kod_mzdove_polozky');

        $this->addField()
            ->setSqlFieldName('TypMzdovePolozky.Popis')
            ->setCode('col-popis') ->setHeaderText('Mzdova_polozka')
            ->setTranslationPrefix('db_typmzdovepolozky_popis_', 'col-kod');

        $this->addField()
            ->setSqlFieldName('MzdovaPolozkaVeMzde.PocetJednotek')
            ->setCode('col-pocet')
            ->setHeaderText('Pocet_jednotek') ->setFormat('number_2') ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('MzdovaPolozkaVeMzde.CenaZaJednotku')
            ->setCode('col-cenazajednotku')
            ->setHeaderText('Cena_za_jednotku') ->setFormat('number2') ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('MzdovaPolozkaVeMzde.Castka')
            ->setCode('col-castka')
            ->setHeaderText('Castka') ->setFormat('number2') ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('VlastnostMzdovePolozky.NazevJednotky')
            ->setCode('col-jednotka');

        $this->addField()
            ->setSqlFieldName('MzdovaPolozkaVeMzde.Rozdeleni')
            ->setCode('col-rozdeleni') ->setHeaderText('Rozdeleni');

        $this->defaultSort = 'col-kod';
        $this->primaryField = 0;
    }


    function setFilterForm()
    {
        $this ->filterForm = $this ->createFilterForm('ADS', 'zobrazit_mzdpol');
//
//        parent::setFilterForm();
//
//        // jina prava pro mzdove polozky
//        $prava = Prava::vratPrava('zobrazit_mzdpol');
//        $this->filterForm->nastavOsoby(true, $prava, 0, 0); // datum od-do podle obdobi
//
//        $this->filterForm->f_pravarozdeleni = $prava;
//        $this->filterForm->nastavAkciGet('mzdovepolozky');
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        return new SqlStatement(
            $result .
            ($result ? ' AND ' : '') .
            'MzdovaPolozkaVeMzde.Obdobi = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('obdobi')) .
            ' AND MzdovaPolozkaVeMzde.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc')) .
            ' AND MzdovaPolozkaVeMzde.OsobniCislo = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'osobniCislo')) .
            ' AND MzdovaPolozkaVeMzde.DatumNastupu = ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('osoba', 'datumNastupu')),
            $params);
    }

}
