<?php
/**
 * pavel.rudol, 22.09.2023 14:20
 */

namespace ronportal\mzdy;

use Ron\DateUtils;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;

class OdpocetNaDiteNavrhRecord extends RonPortalDBRecord
{
    public function __construct($database, $logDescription)
    {
        parent::__construct($database, 'OdpocetNaDiteNavrh', $logDescription);

        $this->addTimestampField('DatumVlozeniNavrhu', true);
        $this->addStringField('RC',true,15);
        $this->addStringField('OsobniCislo',true,15);
        $this->addDateField('DatumNastupu', true);
        $this->addStringField('RC_Ditete', true, 15);
        $this->addDateField('DatumOd',true);

        $this->addIntField('PoradiDitete',true);
        $this->addIntField('Stav',true);

        $this->addMemoField('ElektronickyPodpis', true)->setUpdateLog(true);
        $this->addMemoField('ElektronickyPodpisUcetni', true);
    }

    /**
     * @param PracovniPomer $pracovniPomer
     * @param int $i
     * @param DitePoplatnika $dite
     * @param bool $zmena
     * @return void
     */
    public function loadPost($pracovniPomer, $i, $dite, $zmena = false)
    {
        $stav = 0;
        $datumOd = $dite->zmenaOd ?? DateUtils::today();
        $novePoradi = $_POST["dite_poradi_navrh_" . $i];

        $datumVlozeniNavrhu =  $dite->datumVlozeniNavrhu ?? time();

        //pri nastupu v prubehu roku, pri podpisu 30 dnu od data nastupu, bude pocatek datum od 1. den mesice nasledujiciho po datu nastupu, jinak se nastavi 1. den mesice nasledujiciho po datu podpisu
        $datumOdOdpocetNavrh = DateUtils::today();
        if ($zmena) {
            $datumOdOdpocetNavrh = mktime(0,0,0,(int)$_POST['mesic_zmeny_deti'], 1, (int)$_POST['obdobi']);
            $stav = 2;
        } else {
            $datumOdOdpocetNavrh = $dite->zmenaOd ?? DaneController::vratDatumPocatkuNavrhuOdpoctu($datumOdOdpocetNavrh, $pracovniPomer->datumNastupu);
        }

        $this->findFieldByName('DatumVlozeniNavrhu')->setNewValue($datumVlozeniNavrhu);
        $this->findFieldByName('DatumOd')->setNewValue($datumOdOdpocetNavrh);

        $this->findFieldByName('RC')->setNewValue($pracovniPomer->rc);
        $this->findFieldByName('OsobniCislo')->setNewValue($pracovniPomer->osobniCislo);
        $this->findFieldByName('DatumNastupu')->setNewValue($pracovniPomer->datumNastupu);

        $this->findFieldByName('RC_Ditete')->setNewValue($dite->id);
        $this->findFieldByName('PoradiDitete')->setNewValue($novePoradi);
        $this->findFieldByName('Stav')->setNewValue($stav);

        $this->findFieldByName('ElektronickyPodpis')->setNewValue(DaneController::podpisUzivatele());

        $paramsOdp = $this->database->createParameters();
        $sqlWhere =
            " DatumVlozeniNavrhu = " . $paramsOdp->addTimestampParameter($dite->datumVlozeniNavrhu) .
            " AND Stav = " . $paramsOdp->addParameter($stav) .
            ' AND RC = ' . $paramsOdp->addParameter($pracovniPomer->rc) .
            ' AND OsobniCislo = ' . $paramsOdp->addParameter($pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $paramsOdp->addDateParameter($pracovniPomer->datumNastupu) .
            " AND RC_Ditete = " .  $paramsOdp->addParameter($dite->id) .
            " AND DatumOd = " . $paramsOdp->addDateParameter((int)$datumOd);

        $this->setWhere($sqlWhere, $paramsOdp);
    }
}