<?php
/**
 * pavel.rudol, 22.09.2023 14:20
 */

namespace RonPortal\Mzdy;

use Ron\DateUtils;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;

class OdpocetNavrhRecord extends RonPortalDBRecord
{
    public function __construct($database, $logDescription)
    {
        parent::__construct($database, 'OdpocetNavrh', $logDescription);

        $this->addTimestampField('DatumVlozeniNavrhu', true);
        $this->addStringField('RC',true,15);
        $this->addStringField('OsobniCislo',true,15);
        $this->addDateField('DatumNastupu', true);
        $this->addDateField('DatumOd',true);

        $this->addIntField('ZakladniOdpocet',true);
        $this->addIntField('OdpocetNaStudenta',true);

        $this->addIntField('NaZdravotniStav',true);
        $this->addIntField('ZTPP', true);

        $this->addIntField('Stav',true);
        $this->addMemoField('ElektronickyPodpis', true)->setUpdateLog(true);
        $this->addMemoField('ElektronickyPodpisUcetni', true);
    }

    /**
     * @param PracovniPomer $pracovniPomer
     * @param int $datumOd puvodni datum od v navrhu odpoctu, slouzi pro nalezeni zaznamu k editaci
     * @param int $datumVlozeni puvodni datum vlozeni navrhu odpoctu, slouzi pro nalezeni zaznamu k editaci
     * @param bool $zmena
     * @return void
     */
    public function loadPost($pracovniPomer, $datumOd, $datumVlozeni, $zmena = false)
    {
        $stav = 0;
        //pri nastupu v prubehu roku, pri podpisu 30 dnu od data nastupu, bude pocatek datum od 1. den mesice nasledujiciho po datu nastupu, jinak se nastavi 1. den mesice nasledujiciho po datu podpisu
        $datumOdOdpocetNavrh = DateUtils::today();
        if ($zmena) {
            $datumOdOdpocetNavrh = mktime(0,0,0,(int)$_POST['mesic_zmeny'], 1, (int)$_POST['obdobi']);
            $stav = 2;
        } else {
            $datumOdOdpocetNavrh = $datumOd ?? DaneController::vratDatumPocatkuNavrhuOdpoctu($datumOdOdpocetNavrh, $pracovniPomer->datumNastupu);
        }

        $datumVlozeniNavrhu = $datumVlozeni ?? time();

        $this->findFieldByName('DatumVlozeniNavrhu')->setNewValue($datumVlozeniNavrhu);
        $this->findFieldByName('DatumOd')->setNewValue($datumOdOdpocetNavrh);

        $this->findFieldByName('RC')->setNewValue($pracovniPomer->rc);
        $this->findFieldByName('OsobniCislo')->setNewValue($pracovniPomer->osobniCislo);
        $this->findFieldByName('DatumNastupu')->setNewValue($pracovniPomer->datumNastupu);

        $this->findFieldByName('ZakladniOdpocet')->setNewValue($_POST['sleva_zakladni_nova'] ?? 0);
        $this->findFieldByName('OdpocetNaStudenta')->setNewValue($_POST['sleva_na_studenta_nova'] ?? 0);

        $this->findFieldByName('NaZdravotniStav')->setNewValue((isset($_POST['sleva_na_zdravotni_stav2']) && $_POST['sleva_na_zdravotni_stav2'] == 1) ? 2 :
            ((isset($_POST['sleva_na_zdravotni_stav1']) && $_POST['sleva_na_zdravotni_stav1'] == 1) ? 1 : 0));
        $this->findFieldByName('ZTPP')->setNewValue($_POST['sleva_ztpp'] ?? 0);
        $this->findFieldByName('Stav')->setNewValue($stav);

        $this->findFieldByName('ElektronickyPodpis')->setNewValue(DaneController::podpisUzivatele());

        $paramsOdp = $this->database->createParameters();
        $sqlWhere =
            " DatumVlozeniNavrhu = " . $paramsOdp->addTimestampParameter((int)$datumVlozeni) .
            " AND Stav = " . $paramsOdp->addParameter($stav) .
            ' AND RC = ' . $paramsOdp->addParameter($pracovniPomer->rc) .
            ' AND OsobniCislo = ' . $paramsOdp->addParameter($pracovniPomer->osobniCislo) .
            " AND DatumNastupu = " . $paramsOdp->addDateParameter($pracovniPomer->datumNastupu) .
            " AND DatumOd = " . $paramsOdp->addDateParameter((int)$datumOd);

        $this->setWhere($sqlWhere, $paramsOdp);
    }
}