<?php
/**
 * pavel.rudol, 28.12.2022 10:56
 */

namespace RonPortal\Mzdy;

use Exception;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\User;

class RocniVyuctovaniDaneRecord extends RonPortalDBRecord
{
    const PROHLASENI_NAVRH = 0;
    const PROHLASENI_SCHVALENO = 1;
    const PROHLASENI_VRACENO = 2;
    const PROHLASENI_VICE_PP = 3;
    const PROHLASENI_PODEPSANO = 4;

    const ZMENA_NEZADANA = -1;
    const ZMENA_NAVRH = 0;
    const ZMENA_SCHVALENA = 1;

    const ZADOST_NAVRH = 0;
    const ZADOST_SCHVALENO = 1;
    const ZADOST_VRACENO = 2;
    const ZADOST_VICE_PP = 3;
    const ZADOST_SCHVALENO_UCETNI = 4;

    const LIMIT_UVERU_BYDLENI_PO_2021 = 150000;

    public $rok;
    public $rc;
    public $historieProhlaseniField;
    public $historieZadostiField;

    public function __construct($database, $logDescription)
    {
        parent::__construct($database, 'RocniVyuctovaniDane', $logDescription);

        $this->rok = $this->addTimestampField('RocniVyuctovaniDane.Rok', true);
        $this->rc = $this->addStringField('RocniVyuctovaniDane.RC', true, 15);

        $this->addStringField('ProhlaseniPortalRC', true, 15);
        $this->addStringField('ProhlaseniPortalOsc', true, 15);
        $this->addTimestampField('ProhlaseniPortalDatNastup', true);

        $this->addIntField('ProhlaseniElektronicky', true);
        $this->addIntField('ZadostElektronicky', true);
        $this->addIntField('Schvaleno', true, 0);

        $this->addTimestampField('DatumZmenyZadosti', true);

        $this->historieProhlaseniField = $this->addMemoField('HistorieProhlaseni', true)->setUpdateLog(false);
        $this->historieZadostiField = $this->addMemoField('HistorieZadost', true)->setUpdateLog(false);

        $this->addIntField('ProhlaseniPortal', true); //Prohlášení k dani činím: 0 - prohlášení k dani ne, 1 - prohlášení k dani ano
        $this->addIntField('ProhlaseniPortalStav', true, 0);
        $this->addMemoField('ElektronickyPodpis', true);
        $this->addTimestampField('DatumProhlaseniPortal', true);
        $this->addMemoField('ElektronickyPodpisUcetni', true);

        $this->addMemoField('ElektronickyPodpisCestneP', true);
        $this->addIntField('MaPrijemVZahranici', true, 0);

        $this->addIntField('ProvestRocniVyuctovani', true);
        $this->addIntField('ZadostPortalStav', true, 0);
        $this->addMemoField('ElektronickyPodpisZadost', true);
        $this->addTimestampField('DatumZadostiPortal', true);

        //udaje pro prohlaseni
        $this->addIntField('VyzivujeJinyPoplatnik', true, 0);
        $this->addIntField('JinyPoplatnikUplatnujeZv', true, 0);
        $this->addIntField('JinyPoplatnikZamestnani', true, 0);
        $this->addStringField('JinyPoplatnikJmeno', true, 30);
        $this->addStringField('JinyPoplatnikPrijmeni', true, 40);
        $this->addStringField('JinyPoplatnikRC', true, 15);

        $this->addStringField('JinyPoplatnikObec', true, 80);
        $this->addStringField('JinyPoplatnikPSC', true, 10);
        $this->addStringField('JinyPoplatnikUlice', true, 80);

        $this->addStringField('JinyPoplatnikNazevPlatce', true, 120);
        $this->addStringField('JinyPoplatnikObecPlatce', true, 80);
        $this->addStringField('JinyPoplatnikUlicePlatce', true, 80);
        $this->addStringField('JinyPoplatnikPSCPlatce', true, 10);

        $this->addIntField('DanovyNerezident', true, 0);

        $this->addMemoField('PoznamkaProZamestnance', true);
        $this->addMemoField('PoznamkaProZamZadost', true);

        //zmenova cast prohlaseni
        $this->addIntField('ProhlaseniPortalZmenaStav', true, -1);
        $this->addDateField('ProhlaseniDatumZmeny', true);

        //udaje pro rocni vyuctovani
        $this->addMemoField('AdresaBytovePotrebyUveru', true);
        $this->addFloatField('OpravaHodnotDaru', true);
        $this->addFloatField('OpravaUrokuZeStavebSpor', true);
        $this->addFloatField('OpravaPenzijnihoPrip', true);
        $this->addFloatField('ZivotniPojisteni', true);
        $this->addFloatField('OpravaClenskychPrispevku', true);
        $this->addFloatField('OpravaUhradZaZkousky', true);
        $this->addMemoField('OsobyUplatnujiciUver', true);
        $this->addMemoField('PredchoziPrijmyKdyOdKoho', true);
        $this->addFloatField('Skolkovne', true);

        $this->addFloatField('UrokZeStavebSporPo2021', true);
        $this->addFloatField('SlevaZaZastavenouExekuci', true);

        $this->addTimestampField('MesicVyuctovani', true);

        $this->addIntField('UplatnujeOdpocetPartnera', true, 0);

        $this->addStringField('SlevaPartnerObdobi', true, 15);
        $this->addIntField('PocetMesicuOdpoctu', true, 0);
        $this->addIntField('PartnerZTPP', true, 0);

        $this->addIntField('AktualizovatVypocet', true, 0);


    }

    /**
     * @return bool
     */
    public function lzePodepsatProhlaseniElektronicky()
    {
        if ($this->findFieldByName('ProhlaseniElektronicky')->value == 0) {
            return false;
        }
        return true;
    }

    /**
     * @return bool
     */
    public function jeProhlaseniSchvaleno()
    {
        if (!is_null($this->findFieldByName('ProhlaseniPortalStav')->value) && $this->findFieldByName('ProhlaseniPortalStav')->value == self::PROHLASENI_SCHVALENO) {
            return true;
        }
        return false;
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function lzePodepsatZadostElektronicky()
    {
        if ($this->findFieldByName('ZadostElektronicky')->value == 0 || $this->findFieldByName('Schvaleno')->value == 1) {
            return false;
        }
        return true;
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function jeZadostSchvalena()
    {
        if ($this->findFieldByName('ZadostPortalStav')->value == self::ZADOST_SCHVALENO || $this->findFieldByName('ZadostPortalStav')->value == self::ZADOST_SCHVALENO_UCETNI ) {
            return true;
        }
        return false;
    }

    /**
     * @param int $rok
     * @param string $rc
     * @return bool
     * @throws DBException
     */
    public function nactiVyuctovani(int $rok, string $rc): bool
    {
        $params = $this->database->createParameters();

        $this->setWhere(
            'RocniVyuctovaniDane.Rok = ' . $params ->addParameter($this->database->timestampToParam($rok)) .
            ' AND RocniVyuctovaniDane.RC = ' . $params ->addParameter($rc), $params);

        $loaded = parent::loadRecord();

        if ($loaded && $this->findFieldByName('ProhlaseniPortalRC')->value != '') {
            $params = $this->database->createParameters();
            $pp = DBUtils::getSqlValue($this->database, "SELECT ID FROM PracovniPomer WHERE RC = " . $params->addParameter($this->findFieldByName('ProhlaseniPortalRC')->value) .
                " AND OsobniCislo = " . $params->addParameter($this->findFieldByName('ProhlaseniPortalOsc')->value) . " AND DatumNastupu = " . $params->addDateParameter($this->findFieldByName('ProhlaseniPortalDatNastup')->value)
                ,$params);
            $this->addIntField('idpp', true, $pp);
        }

        return $loaded;
    }

    /**
     * @param PracovniPomer $pracovniPomer
     * @return void
     */
    public function nastavPracovniPomer(PracovniPomer $pracovniPomer)
    {
        $this->findFieldByName('ProhlaseniPortalRC')->setNewValue($pracovniPomer->rc);
        $this->findFieldByName('ProhlaseniPortalOsc')->setNewValue($pracovniPomer->osobniCislo);
        $this->findFieldByName('ProhlaseniPortalDatNastup')->setNewValue($pracovniPomer->datumNastupu);
    }
    
    /**
     * Nastavi pole ProvestRocniVyctovani na hodnotu Zadam/Nezadam
     * @param int $zadam
     * @return void
     */
    public function nastavProvedeniVyuctovani(int $zadam)
    {
        $this->findFieldByName('ProvestRocniVyuctovani')->setNewValue($zadam);
        $this->findFieldByName('DatumZmenyZadosti')->setNewValue(time());
    }

    /**
     * Podepise prohlaseni dane, nastavi pole ZadostPortalStav, ElektronickyPodpisZadost, DatumZadostiPortal
     * @param int $prohlaseniCini 0/1 - prohlaseni k dani ne/ano
     * @return void
     */
    public function podepsatProhlaseni(int $prohlaseniCini)
    {
        $this->findFieldByName('ProhlaseniPortal')->setNewValue($prohlaseniCini);
        $this->findFieldByName('ProhlaseniPortalStav')->setNewValue(self::PROHLASENI_PODEPSANO);

        $this->findFieldByName('DatumProhlaseniPortal')->setNewValue(time());

        //23.2.2023 pri opakovanem podpisu po vraceni mzdovou ucetni, zustane elektronicky podpis na puvodni hodnote
        if (empty($this->findFieldByName('ElektronickyPodpis')->originalValue)) {
            $this->findFieldByName('ElektronickyPodpis')->setNewValue(DaneController::podpisUzivatele());
        }
    }

    /**
     * Podepise zadost o rocni vyuctovani dane, nastavi pole ZadostPortalStav, ElektronickyPodpisZadost, DatumZadostiPortal
     * @return void
     */
    public function podepsatZadost()
    {
        $this->findFieldByName('ZadostPortalStav')->setNewValue(self::ZADOST_SCHVALENO);
        $this->findFieldByName('ElektronickyPodpisZadost')->setNewValue(DaneController::podpisUzivatele());
        $this->findFieldByName('DatumZadostiPortal')->setNewValue(time()); //TODO: v puvodnim kodu bylo omezeni na 15.2. aktualniho roku, ma se zachovat, nebo upravit?
    }

    /**
     * Doplni text aktualne provadene cinnosti do pole s historii zadosti o vyuctovani
     * @param string $udalost
     * @return void
     */
    public function doplnHistoriiZadosti(string $udalost)
    {
        $this->historieZadostiField->value =  date('j.n.Y G:i') .
            ', ' . StringUtils::htmlTextConvert(User::getUserName()) . ': ' . $udalost
            . "\n" . $this->historieZadostiField->originalValue;
    }

    /**
     * Doplni text aktualne provadene cinnosti do pole s historii prohlaseni
     * @param string $udalost
     * @return void
     */
    public function doplnHistoriiProhlaseni(string $udalost)
    {
        $this->historieProhlaseniField->value =  date('j.n.Y G:i') .
            ', ' . StringUtils::htmlTextConvert(User::getUserName()) . ': ' . $udalost
            . "\n" . $this->historieProhlaseniField->originalValue;
    }

    /**
     * @return void
     */
    public function aktualizaceVypoctuVyuctovani()
    {
        if ($this->findFieldByName('OpravaHodnotDaru')->isChanged() ||
            $this->findFieldByName('OpravaUrokuZeStavebSpor')->isChanged() ||
            $this->findFieldByName('OpravaPenzijnihoPrip')->isChanged() ||
            $this->findFieldByName('ZivotniPojisteni')->isChanged() ||
            $this->findFieldByName('OpravaClenskychPrispevku')->isChanged() ||
            $this->findFieldByName('OpravaUhradZaZkousky')->isChanged() ||
            $this->findFieldByName('PocetMesicuOdpoctu')->isChanged() ||
            $this->findFieldByName('Skolkovne')->isChanged()
        ) {
            $this->findFieldByName('AktualizovatVypocet')->setNewValue(1);
        }
    }

    public function zmenaProhlaseni()
    {
        $this->findFieldByName('ProhlaseniPortalZmenaStav')->setNewValue(self::PROHLASENI_NAVRH);
        $f = $this->findFieldByName('ProhlaseniDatumZmeny');
        if (!is_null($f)) {
            $f->setNewValue(time());
        }
    }

    public function vytvorZaznam($rec, $obdobi, $pracovniPomer)
    {
        $daneElektronicky = RonPortalSettings::getIntValue('povolitprohlaseninaportale');

        $this->findFieldByName('Rok')->setNewValue($obdobi);
        $this->findFieldByName('RC')->setNewValue($pracovniPomer->rc);
        $this->findFieldByName('ProhlaseniElektronicky')->setNewValue($rec['ProhlaseniElektronicky'] ?? $daneElektronicky);
        $this->findFieldByName('ZadostElektronicky')->setNewValue($rec['ZadostElektronicky'] ?? $daneElektronicky);
        //nastavi se dle lonskeho zaznamu nebo vychozi jako provest
        $this->findFieldByName('ProvestRocniVyuctovani')->setNewValue($rec['ProvestRocniVyuctovani'] ?? 1);

        $this->findFieldByName('MesicVyuctovani')->setNewValue(DateUtils::incMonth($obdobi, 14)); //1.3.nasledujici rok

        $this->nastavPracovniPomer($pracovniPomer);

        //nastavi udaje danoveho nerezidenta z predchoziho prohlaseni
        if (isset($rec['CisloDanoveIdentifikace'])) {
            $this->findFieldByName('CisloDanoveIdentifikace')->setNewValue($rec['CisloDanoveIdentifikace']);
            $this->findFieldByName('TypDanovehoIdentifikatoru')->setNewValue($rec['TypDanovehoIdentifikatoru']);
            $this->findFieldByName('DanovyNerezident')->setNewValue($rec['DanovyNerezident']);
            $this->findFieldByName('StatDanoveRezidence')->setNewValue($rec['StatDanoveRezidence']);
        }
    }

    public function updateRecord($filtr = '', $filtrParams = null)
    {
        //pokud doslo ke zmene PP v zaznamu RocniVyuctovaniDane, pak zkontrolovat a smazat navrhy odpoctu pro jine PP
        $recOdpocet = new OdpocetNavrhRecord($this->database, 'Smazani navrhu pri zmene PP');
        $this->smazNavrhyOdpoctu($recOdpocet);

        $recOdpocet = new OdpocetNaDiteNavrhRecord($this->database, 'Smazani navrhu pri zmene PP');
        $this->smazNavrhyOdpoctu($recOdpocet);

        return parent::updateRecord($filtr, $filtrParams);
    }

    /**
     * @param OdpocetNavrhRecord|OdpocetNaDiteNavrhRecord $recOdpocet
     * @throws DBException
     */
    private function smazNavrhyOdpoctu($recOdpocet)
    {
        $paramsOdp = $this->database->createParameters();
        $sqlWhere =
            " Stav IN (" . $paramsOdp->addParameter(0) . ', ' . $paramsOdp->addParameter(2) . ")" .
            ' AND RC = ' . $paramsOdp->addParameter($this->rc->value) .
            " AND DatumOd >= " . $paramsOdp->addDateParameter($this->rok->value);

        $recOdpocet->setWhere($sqlWhere, $paramsOdp);
        $recOdpocet->checkMultiple = false;
        $recOdpocet->loadRecords();

        while ($recOdpocet->next()) {
            //pokud existuje navrh pro jiny PP, nez je aktualne v rocnim vyuctovani, tak ho smaze
            if (($recOdpocet['OsobniCislo'] != $this['ProhlaseniPortalOsc']) || ($recOdpocet['DatumNastupu'] != $this['ProhlaseniPortalDatNastup'])) {
                $recOdpocet->deleteRecord();
            }
        }

    }
}