<?php
/**
 * UTF8
 * Roman.Szablatura RON Software 12.02.2019
 * Pavel.Rudol RON Software 01/2023
 */
namespace RonPortal\Mzdy;

use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\Model;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FormReportsListView;
use RonPortal\RonPortalSettings;

$model = Model::getInstance();

/** @var VyuctovaniDaneForm $rocniVyuctovaniForm */
$rocniVyuctovaniForm = $model->getData('rocnivyuctovaniform');
$rok = (int)date("Y", $rocniVyuctovaniForm['rok']);
$disabled = $rocniVyuctovaniForm->lzeEditovatVyuctovani() ? "" : "disabled";

/** @var NastaveniOdpoctu $nastaveniOdpoctu */
$nastaveniOdpoctu = $model->getData('nastaveniOdpoctu');
$reportlist = $model->getData('reportlist');
$jednoduchyVzhled = $model->getData('jednoduchyVzhled');

if ($jednoduchyVzhled) {
    $classStr = 'class="dane_info_skryte"';
} else {
    $classStr = '';
}

?>

<?php MessageManager::getInstance() ->showMessages(); ?>

<?php if ($rocniVyuctovaniForm['ZadostPortalStav'] == RocniVyuctovaniDaneRecord::ZADOST_VICE_PP)  { ?>

    <div class="form">
        <div class="form-content">
            <?=   TextManager::_('formular_nelze_vyplnit') ?>
        </div>
        <div class="buttons">
            <input type="button" class="button clickHref" data-ron-href="?menuid=dane&action=zadost" value="<?= TextManager::_('zpet')?>">
        </div>
    </div>

<?php } else { ?>

    <?php
        if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
            echo
            "<div id='novydokument' title='" . TextManager::_('Vlozit_novy_dokument') . "' class='rondialog'>
            <div>
                <div class='row'>
                    <div class='col col-12 col-sm-4 caption'>" . TextManager::_('Nahrat_soubor') . "</div>
                    <div class='col col-12 col-sm-8' id='input-file'><input type='file' name='soubor'></div>
                </div>
            </div>
            <div class='buttons'>
                <div>
                    <input id='vlozitDokumentBtn' type='button' class='button' value='" . TextManager::_('form.Insert') . "' />
                    <input id='stornoDokumentBtn' type='button' class='button cancel' value='" . TextManager::_('Cancel') . "' />
                </div>
            </div>
            
            </div>";
        }
    ?>

    <form method="post" name="formularzadosti" enctype="multipart/form-data" action="?menuid=dane&action=ulozitzadost">
        <input type="hidden" name="rok" value="<?= $rok ?>">
        <input type="hidden" name="osoba" value="<?= $rocniVyuctovaniForm->pracovniPomer->idPP ?>">
        <input type="hidden" name="skolkovne_max" value="<?= $nastaveniOdpoctu->maxSkolkovne ?>">
        <?= Csrf::csrfHiddenInput('vyuctovani_' . $rocniVyuctovaniForm->pracovniPomer->idPP)?>

        <div class='form large-form zadost-zuctovani'>
            <div class="form-content">

                <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h1> <?= $rocniVyuctovaniForm->nadpisVyuctovani() ?> </h1>
                    </div>
                </div>

                 <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h3><?= TextManager::_("zadost_za_zdanovaci_obdobi")?>
                            <label>
                                <?php //"Žádost a o roční zúčtování záloh" je za předchozí rok ?>
                                <?= $rok; ?>
                            </label>
                        </h3>
                    </div>
                </div>

                <?php if ($rocniVyuctovaniForm['ZadostPortalStav'] == 2) { ?>
                    <div class="row">
                        <div class="col col-12 col-md-12 warning">
                            <?= TextManager::_('zadost_poznamka_ucetni') . StringUtils::htmlTextConvert($rocniVyuctovaniForm['PoznamkaProZamZadost'] ?? TextManager::_('Zadost_vychozi_poznamka_vraceni')) ?>
                        </div>
                    </div>
                <?php } ?>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-12 font-weight-bold">
                            <?= TextManager::_("identifikace_poplatnika")?>
                        </div>
                    </div>

                    <div class="row">
                        <div class="col col-12 col-md-4 ">
                            <?= TextManager::_('prijmeni'). ": " ?><span class="font-weight-bold"><?=  StringUtils::htmlTextConvert($rocniVyuctovaniForm->pracovniPomer->prijmeni) ?></span>
                        </div>

                        <div class="col col-12 col-md-4">
                            <?= TextManager::_('Prohlaseni_jmeno_ditete'). ": " ?><span class="font-weight-bold"><?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->pracovniPomer->jmeno) ?></span>
                        </div>

                        <div class="col col-12 col-md-4">
                            <?= TextManager::_('rodne_cislo'). ": " ?><span class="font-weight-bold"><?=  StringUtils::htmlTextConvert($rocniVyuctovaniForm['RodneCislo']) ?></span>
                        </div>
                    </div>

                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte1"></div>
                <?php } ?>

                <div id="dane_info_skryte1" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("nejsem_povinen_podat")?></div>
                </div>

                <?php if (!$rocniVyuctovaniForm['DanovyNerezident']) { ?>

                <div class="row">
                    <div class="col col-12 col-md-12 font-weight-bold">
                        <?= TextManager::_("zadost_uvadim_prijmy_za_obdobi")?>
                    </div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-3 center">
                            <?= TextManager::_("v_kalendarnich_mesicich")?>
                        </div>
                        <div class="col col-12 col-md-9 center">
                            <?= TextManager::_("zadost_identifikace_predchozich_platcu")?>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <?php $predchoziPlatceDane = RonPortalSettings::getIntValue('danepocetzamestnavatelu', 3);
                    if (!is_null($rocniVyuctovaniForm['PredchoziPrijmyKdyOdKoho'])) {
                        $predchoziPrijmy = $rocniVyuctovaniForm->parseMemoToArray($rocniVyuctovaniForm['PredchoziPrijmyKdyOdKoho']);
                    } else {
                        $predchoziPrijmy = [];
                    }
                    for($i = 0; $i < $predchoziPlatceDane; $i++) { ?>
                        <div class="row">
                             <div class="col col-12 col-md-3">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti filter-zadosti-od-do"
                                            name="predchoziplatcedanemesice_<?= $i ?>"
                                            placeholder="mm-mm" <?= $disabled ?>
                                            value="<?= $_POST["predchoziplatcedanemesice_$i"] ?? $predchoziPrijmy[$i][0] ?? '' ?>">
                                </label>
                            </div>

                            <div class="col col-12 col-md-9">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti"
                                           name="predchoziplatcedaneadresa_<?= $i ?>"
                                           placeholder="<?php echo TextManager::_('zadost_nazev_platce_dane') . ",    " .
                                              TextManager::_('zadost_ico_platce_dane') . ",    " .
                                              TextManager::_('adresa') . ",     "
                                              ?>" <?= $disabled ?>
                                            value="<?= $_POST["predchoziplatcedaneadresa_$i"] ?? $predchoziPrijmy[$i][1] ?? '' ?>">
                                </label>
                            </div>
                        </div>

                    <?php }?>
                    <input type="hidden" name="pocetpredchozichplatcudane" value="<?= $i?>">

                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte2"></div>
                <?php } ?>

                <div id="dane_info_skryte2" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("zadost_za_prve")?></div>
                </div>

                <?php
                DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'predchozizamestnavatel', $rocniVyuctovaniForm->lzeEditovatVyuctovani());
                ?>

                <div class="row">
                    <div class="col col-12 col-md-12 font-weight-bold info">
                        <?= TextManager::_("zadost_uplatnuji_narok_na")?>
                    </div>
                </div>

                <br>
                <br>

                <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h2><?= TextManager::_("zadost_slevy_na_dani")?></h2>
                    </div>
                </div>

                 <div class="row">
                    <div class="col col-12 col-md-12 info">
                        <?= TextManager::_("zadost_sleva_na_manzelku")?>
                    </div>
                </div>


                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("jmeno")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("prijmeni_partnera")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("rc")?>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("zadost_narok_splnen_za")?><sup>2)</sup>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("ZTP/P")?>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("zadost_narok_splnen_za_ztp")?><sup>3a)</sup>
                        </div>

                    </div>

                    <hr class="oddelovac">

                    <div class="row">
                        <?php if (isset($rocniVyuctovaniForm->partnerPoplatnika)) {
                            if (!is_null($rocniVyuctovaniForm['SlevaPartnerObdobi'])) {
                                $partnerObdobi = explode(";", $rocniVyuctovaniForm['SlevaPartnerObdobi']);
                            } else {
                                $partnerObdobi = [];
                            }
                            ?>
                        <div class="col col-12 col-md-2 ">
                            <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->partnerPoplatnika->jmeno) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->partnerPoplatnika->prijmeni) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= StringUtils::htmlTextConvert($rocniVyuctovaniForm->partnerPoplatnika->rodneCislo) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="text"
                                       class="pole-zadosti filter-zadosti-od-do"
                                       name="slevanapartnera"
                                       id="slevanapartnera"
                                       placeholder="mm-mm"
                                       value="<?= StringUtils::htmlTextConvert($_POST['slevanapartnera'] ?? $partnerObdobi[0] ?? '') ?>" <?= $disabled ?>>
                            </label>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= $rocniVyuctovaniForm->partnerPoplatnika->ztpp ? "X" : "" ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label title="<?= TextManager::_('zadost_rozsah_mesicu_naroku')?>">
                                <input type="text"
                                       class="pole-zadosti filter-zadosti-od-do"
                                       name="slevanapartneraproztp"
                                       id="slevanapartneraproztp"
                                       placeholder="mm-mm"
                                       value="<?= StringUtils::htmlTextConvert($_POST['slevanapartneraproztp'] ?? $partnerObdobi[1] ?? '') ?>" <?= (!$rocniVyuctovaniForm->partnerPoplatnika->ztpp) ? "disabled" : $disabled ?>>
                            </label>
                        </div>

                        <?php } ?>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte3"></div>
                <?php } ?>

                <div id="dane_info_skryte3" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("zadost_rozsah_mesicu")?></div>
                    <div class="info"><?= TextManager::_("zadost_za_druhe")?></div>
                    <div class="info"><?= TextManager::_("zadost_za_treti_a")?></div>
                </div>

                <?php
                    DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'slevanapartnera', $rocniVyuctovaniForm->lzeEditovatVyuctovani(),
                        isset($rocniVyuctovaniForm->partnerPoplatnika) && (!empty($partnerObdobi[0])));
                ?>

                <br>
                <br>

                  <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h2><?= TextManager::_("zadost_skolkovne")?></h2>
                    </div>
                </div>

                 <div class="row">
                    <div class="col col-12 col-md-12 info">
                        <?= TextManager::_("zadost_sleva_na_dite")?>
                    </div>
                </div>


                <div class="form-content ram">

                    <div class="row">
                        <div class="col col-12 col-md-3 ">
                            <?= TextManager::_("jmeno")?>
                        </div>

                        <div class="col col-12 col-md-4 ">
                            <?= TextManager::_("prijmeni_ditete")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("rc")?>
                        </div>


                        <div class="col col-12 col-md-3 center">
                            <?= sprintf(TextManager::_('zadost_vynalozena_castka'), $nastaveniOdpoctu->maxSkolkovne) ?>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <?php $i=0; ?>
                    <?php foreach ($rocniVyuctovaniForm->deti as $dite) { ?>
                        <?php if ($dite->vekNaZacatkuRoku->y <= 6) { ?>
                            <input type="hidden" name="ditepoplatnika_<?= $i ?>" value="<?= $dite->id ?>">

                            <div class="row">
                                <div class="col col-12 col-md-3 ">
                                    <?= StringUtils::htmlTextConvert($dite->jmeno) ?>
                                </div>

                                <div class="col col-12 col-md-4 ">
                                    <?= StringUtils::htmlTextConvert($dite->prijmeni) ?>
                                </div>

                                <div class="col col-12 col-md-3 ">
                                    <?= StringUtils::htmlTextConvert($dite->rodneCislo) ?>
                                </div>

                                <div class="col col-12 col-md-2 ">
                                    <label>
                                        <input type="number" class="pole-zadosti"
                                               name="skolkovnenadite_<?= $i ?>"
                                               min="0"
                                               max="<?= $nastaveniOdpoctu->maxSkolkovne ?>"
                                               value="<?= StringUtils::format('number_2', $_POST["skolkovnenadite_{$i}"] ?? $dite->skolkovne) ?>" <?= $disabled ?>>
                                    </label>
                                </div>
                            </div>
                            <?php $i++; ?>
                        <?php } ?>
                    <?php } ?>
                    <input type="hidden" name="pocetdetipoplatnika" value="<?= $i ?>">
                </div>

                 <div class="row">
                    <div class="col col-12 col-md-12 info">
                        <?= TextManager::_("zadost_prohlaseni_na_dite_uplatnuji_pouze_ja")?>
                    </div>
                </div>

                <?php
                    DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'skolkovne', $rocniVyuctovaniForm->lzeEditovatVyuctovani(), $i);
                ?>

                <br>
                <br>

                <div class="row">
                    <div class="col col-12 col-md-12 center">
                        <h2><?= TextManager::_("zadost_nezdanitelne_casti")?></h2>
                    </div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("zadost_casti_zakladu_dane_uplatnuji")?><sup>4)</sup>
                        </div>

                        <div class="col col-12 col-md-6 ">
                            <?= TextManager::_("zadost_uplatnuji_nezdanitelne_casti_zakladu")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("zadost_nezdanitelne_casti_napoctena_castka") ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <?= TextManager::_("zadost_nezdanitelne_casti_uplatnovana_castka")?>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostdaryzadam', $rocniVyuctovaniForm['OpravaHodnotDaru'] > 0);?>
                        </div>

                        <div class="col col-12 col-md-7 ">
                            <?= TextManager::_("zadost_nezdanitelne_bezuplatne")?>
                        </div>

                        <div class="col col-12 col-md-1 ">
                            <?= StringUtils::format('number0', $rocniVyuctovaniForm['HodnotaDaru']) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" name="zadostdaryvyse" min="0" value="<?= $_POST['zadostdaryvyse'] ?? $rocniVyuctovaniForm['OpravaHodnotDaru'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
//                                    DaneController::zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, 'odecitatelnepolozky', TextManager::_('zadost_dary_pripojeni_dokumentu'));
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'dary');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadosturokyzadam', ($rocniVyuctovaniForm['OpravaUrokuZeStavebSpor'] + $rocniVyuctovaniForm['UrokZeStavebSporPo2021']) > 0); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_uroky")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" name="zadosturokyvyse" min="0" value="<?= $_POST['zadosturokyvyse'] ?? $rocniVyuctovaniForm['OpravaUrokuZeStavebSpor'] + $rocniVyuctovaniForm['UrokZeStavebSporPo2021'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
//                                    DaneController::zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, 'odecitatelnepolozky', TextManager::_('zadost_uroky_pripojeni_dokumentu'));
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'uroky');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostpenzijnizadam', $rocniVyuctovaniForm['OpravaPenzijnihoPrip'] > 0); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_penzijni")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" name="zadostpenzijnivyse" min="0" value="<?= $_POST['zadostpenzijnivyse'] ?? $rocniVyuctovaniForm['OpravaPenzijnihoPrip'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
//                                    DaneController::zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, 'odecitatelnepolozky', TextManager::_('zadost_penzijni_pripojeni_dokumentu'));
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'penzijni');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostpojistnezadam', $rocniVyuctovaniForm['ZivotniPojisteni'] > 0);?>
                        </div>

                        <div class="col col-12 col-md-7 ">
                            <?= TextManager::_("zadost_nezdanitelne_pojistne")?>
                        </div>

                        <div class="col col-12 col-md-1 ">
                            <?= StringUtils::format('number0', $rocniVyuctovaniForm['UplatneneZivotniPojisteni']) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" name="zadostpojistnevyse" min="0" value="<?=$_POST['zadostpojistnevyse'] ?? $rocniVyuctovaniForm['ZivotniPojisteni'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
//                                    DaneController::zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, 'odecitatelnepolozky', TextManager::_('zadost_zivotni_pripojeni_dokumentu'));
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'zivotni');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div>

                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostclenskezadam', $rocniVyuctovaniForm['OpravaClenskychPrispevku'] > 0); ?>
                        </div>

                        <div class="col col-12 col-md-7 ">
                            <?= TextManager::_("zadost_nezdanitelne_clenske")?>
                        </div>

                        <div class="col col-12 col-md-1 ">
                            <?= StringUtils::format('number0', $rocniVyuctovaniForm['ClenskePrispevkyOdborum']) ?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" name="zadostclenskevyse" min="0" value="<?= $_POST['zadostclenskevyse'] ?? $rocniVyuctovaniForm['OpravaClenskychPrispevku'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
//                                    DaneController::zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, 'odecitatelnepolozky', TextManager::_('zadost_clenske_pripojeni_dokumentu'));
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'clenske_prispevky');
                                } ?>
                            </label>
                        </div>
                    </div>

                    <div class="mobile-only"><br></div>


                    <div class="row">
                        <div class="col col-12 col-md-2 ">
                            <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostuhradazazkouskyzadam', $rocniVyuctovaniForm['OpravaUhradZaZkousky'] > 0); ?>
                        </div>

                        <div class="col col-12 col-md-8 ">
                            <?= TextManager::_("zadost_nezdanitelne_uhrada")?>
                        </div>

                        <div class="col col-12 col-md-2 ">
                            <label>
                                <input type="number" class="odecitatelne-polozky" name="zadostuhradazazkouskyvyse" min="0" value="<?= $_POST['zadostuhradazazkouskyvyse'] ?? $rocniVyuctovaniForm['OpravaUhradZaZkousky'] ?>" <?= $disabled ?>>

                                <?php
                                if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) {
//                                    DaneController::zobrazOdkazPripojeniDokumentu($database, $rok, $pracovniPomer, 'odecitatelnepolozky', TextManager::_('zadost_zkousky_pripojeni_dokumentu'));
                                    DaneController::zobrazOdkazPripojeniDokumentu($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', 'zkousky');
                                } ?>
                            </label>
                        </div>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte4"></div>
                <?php } ?>

                <div id="dane_info_skryte4" <?= $classStr ?>>
                    <div class="info"><sup>2)</sup>
                        <?= TextManager::_("zadost_za_ctvrte")?></div>
                    <div class="info"><sup>2a)</sup>
                        <?= TextManager::_("zadost_za_ctvrte_a")?></div>
                </div>

                <?php
                DaneController::zobrazDokumenty($rocniVyuctovaniForm->database, $rok, $rocniVyuctovaniForm->pracovniPomer, 'odecitatelnepolozky', $rocniVyuctovaniForm->lzeEditovatVyuctovani(),true,
                    ['dary' => TextManager::_('zadost_dary_pripojeni_dokumentu'),
                        'uroky' => TextManager::_('zadost_uroky_pripojeni_dokumentu'),
                        'penzijni' => TextManager::_('zadost_penzijni_pripojeni_dokumentu'),
                        'zivotni' => TextManager::_('zadost_zivotni_pripojeni_dokumentu'),
                        'clenske_prispevky' => TextManager::_('zadost_clenske_pripojeni_dokumentu'),
                        'zkousky' => TextManager::_('zadost_zkousky_pripojeni_dokumentu')]);
                ?>


                <?php if ($jednoduchyVzhled) { ?>
                    <div class="row">
                        <div class="col col-12 col-md-12 center">
                            <h3><?= TextManager::_("zadost_prohlaseni_k_nezdanitelne_casti_nadpis")?></h3>
                        </div>
                    </div>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte5"></div>
                <?php } ?>

                <div id="dane_info_skryte5" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("zadost_prohlaseni_k_nezdanitelne_casti")?></div>
                    <div class="info"><?= TextManager::_("zadost_prohlaseni_k_nezdanitelne_casti_info")?><sup>4b)</sup></div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("Prohlaseni_jmeno_ditete")?>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("prijmeni")?>
                        </div>

                        <div class="col col-12 col-md-2 center">
                            <?= TextManager::_("rc")?>
                        </div>

                        <div class="col col-12 col-md-6 center">
                            <?= TextManager::_("zadost_adresa_trvaleho_bydliste")?>
                        </div>
                    </div>

                    <hr class="oddelovac">

                    <?php $pocetJinychOsobUplatnujicichOdpocet = 3;

                    if (!is_null($rocniVyuctovaniForm['OsobyUplatnujiciUver'])) {
                        $osobyUveru = $rocniVyuctovaniForm->parseMemoToArray($rocniVyuctovaniForm['OsobyUplatnujiciUver']);
                    } else {
                        $osobyUveru = [];
                    }
                    for($i = 0; $i < $pocetJinychOsobUplatnujicichOdpocet; $i++) { ?>
                        <div class="mobile-only"><?= TextManager::_('zadost_osoba') ?></div>
                        <div class="row">
                            <div class="col col-12 col-md-2">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti bytove_potreby"
                                           name="jinaosobauplatnujeodpocetjmeno_<?= $i ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetjmeno_$i"] ?? $osobyUveru[$i][0] ?? '' ?>" <?= $disabled ?>>
                                </label>
                            </div>

                            <div class="col col-12 col-md-2">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti bytove_potreby"
                                           name="jinaosobauplatnujeodpocetprijmeni_<?= $i ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetprijmeni_$i"] ?? $osobyUveru[$i][1] ?? '' ?>" <?= $disabled ?>>
                                </label>
                            </div>


                            <div class="col col-12 col-md-2">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti ronRC bytove_potreby"
                                           name="jinaosobauplatnujeodpocetrc_<?= $i ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetrc_$i"] ?? $osobyUveru[$i][2] ?? '' ?>" <?= $disabled ?>>
                                </label>
                            </div>

                            <div class="col col-12 col-md-6">
                                <label title="<?= TextManager::_('')?>">
                                    <input type="text" class="pole-zadosti bytove_potreby"
                                           name="jinaosobauplatnujeodpocetadresa_<?= $i ?>"
                                           placeholder="<?= TextManager::_('obec') . ": " .
                                           TextManager::_('mesto') . ": " .
                                           TextManager::_('ulice') . ":  " .
                                           TextManager::_('psc') . ":" ?>"
                                           value="<?= $_POST["jinaosobauplatnujeodpocetadresa_$i"] ?? $osobyUveru[$i][3] ?? '' ?>" <?= $disabled ?>>
                                </label>
                            </div>
                        </div>

                    <?php } ?>
                    <input type="hidden" name="pocetjinychosobuplatnujicichodpocet" value="<?= $pocetJinychOsobUplatnujicichOdpocet?>">

                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte6"></div>
                <?php } ?>

                <div id="dane_info_skryte6" <?= $classStr ?>>
                    <div class="info"><sup>4b)&nbsp;</sup><?= TextManager::_("zadost_za_ctvrte_b")?></div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-2 ">
                        <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostbytovepotrebypred2021', $rocniVyuctovaniForm['OpravaUrokuZeStavebSpor'] > 0, ['class' => 'bytove_potreby']); ?><sup>3b)&nbsp;</sup>
                    </div>

                    <div class="col col-12 col-md-10 font-weight-bold">
                        <?= TextManager::_('zadost_bytove_potreby_pred_2021') ?>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte8"></div>
                <?php } ?>

                <div id="dane_info_skryte8" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("zadost_bytove_potreby_pred_2021_info")?></div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-2 ">
                        <?php $rocniVyuctovaniForm->renderComboOdpocet('zadostbytovepotrebypo2021', $rocniVyuctovaniForm['UrokZeStavebSporPo2021'] > 0, ['class' => 'bytove_potreby']); ?><sup>3b)&nbsp;</sup>
                    </div>

                    <div class="col col-12 col-md-10 font-weight-bold">
                        <?= TextManager::_('zadost_bytove_potreby_po_2021') ?>
                    </div>
                </div>

                <?php if ($jednoduchyVzhled) { ?>
                    <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte9"></div>
                <?php } ?>

                <div id="dane_info_skryte9" <?= $classStr ?>>
                    <div class="info"><?= TextManager::_("zadost_bytove_potreby_po_2021_info")?></div>
                    <div class="info"><sup>3b)&nbsp;</sup><?= TextManager::_("zadost_za_ctvrte_c")?></div>
                    <div class="info"><?= TextManager::_("zadost_bytove_potreby_upozorneni")?></div>
                </div>

                <div class="row">
                    <div class="col col-12 col-md-12 font-weight-bold">
                        <?= TextManager::_("zadost_misto_bytove_potreby_2023")?>
                    </div>
                </div>

                <div class="form-content ram">
                    <div class="row">
                        <div class="col col-12 col-md-12 ">
                            <input type="hidden" id="adresabytovepotrebyuveru" value="<?= TextManager::_('obec') . ":" .  $rocniVyuctovaniForm->bydliste->obecPopis . ", \n" .
                            TextManager::_('mesto') . ":" . $rocniVyuctovaniForm->bydliste->mesto . ", \n" .
                            TextManager::_('ulice') . ":" . $rocniVyuctovaniForm->bydliste->ulice . ", \n" .
                            TextManager::_('psc') . ":" . $rocniVyuctovaniForm->bydliste->psc ?>"  >

                            <label title="<?= TextManager::_('zadost_pouzit_aktualni_adresu')?>">
                                <textarea name="zadostbytovepotrebyuveruadresa" class="bytove_potreby"
                                          id="zadostbytovepotrebyuveruadresa" rows="3" cols="115"
                                          placeholder="<?php echo TextManager::_('obec') . ":     \n" .
                                              TextManager::_('mesto') . ":     \n" .
                                              TextManager::_('ulice') . ":      \n" .
                                              TextManager::_('psc') . ":    " ?>" <?= $disabled ?>><?= StringUtils::htmlTextConvert($_POST["zadostbytovepotrebyuveruadresa"] ?? $rocniVyuctovaniForm['AdresaBytovePotrebyUveru']) ?></textarea>
                            </label>
                        </div>
                    </div>
                </div>

                <?php } //if ! $danovyNerezident ?>

            </div>


            <?php if ($jednoduchyVzhled) { ?>
                <div class="fa inline-glyph glyph-info dane_info" data-selector="#dane_info_skryte7"></div>
            <?php } ?>

            <div id="dane_info_skryte7" <?= $classStr ?>>
                <div class="info"><?= TextManager::_("zadost_neopravnene_vyplaceni")?></div>
            </div>
            <div class="buttons">
                <input type="button" class="button clickHref" data-ron-href="?menuid=dane" value="<?= TextManager::_('zpet')?>">
                <?php if ($rocniVyuctovaniForm->lzeEditovatVyuctovani()) { ?>
                    <button type="submit" class="button" name="ulozit" value="1"><?= TextManager::_('zadost_ulozit')?></button>
                    <button type="submit" class="button confirmQuestion"  data-ron-confirm-text="<?= TextManager::_('zadost_opravdu_podepsat'); ?>" name="podepsatzadost" value="1"><?= TextManager::_('zadost_potvrzuji_pravdivost_zadosti')?></button>
                <?php } ?>
            </div>
        </div>

        <br>

    </form>

<?php } ?>

<?php
    FormReportsListView::renderReportList($reportlist, '?menuid=dane&action=report&typ=vyuctovani&idpp=' . $rocniVyuctovaniForm->pracovniPomer->idPP . '&rok=' . $rok);
?>