<?php
/**
 * pavel.rudol, 25.2.2022
 */

namespace RonPortal\Personalistika;

use Exception;
use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controller\DokumentyController;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Dokumenty\DokumentUploadForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\User;
use function Ron\ronText;


/**
 * Dialog pro upload obecnych dokumentu
 * Class NovyDokumentOsobyForm
 * @package RonPortal\Personalistika
 */
class NovyDokumentOsobyForm extends DokumentUploadForm
{
    function __construct($database, $idPP)
    {
        parent::__construct($database, '?menuid=dokumenty&action=dokumentupload');
        $this ->target = null;

        $this ->hiddenFields = [
            'dokumentosoby' => '1',
            'idosoby' => $idPP,
        ];
    }
}



//class DokumentyOsobyFilterForm extends RonPortalFilterForm
//{
//    function __construct($database, $action)
//    {
//        parent::__construct($database, $action);
//
//        $this->f_formaction = '?menuid=dokumenty&';
//        $this->f_classname = 'filtercontainer leftcaptions';
//
//        $this->nastavPersonalistiku([Prava::vratPrava('prohl_osobni'), Prava::vratPrava('dokumenty_prohl'), Prava::vratPrava('dokumenty_seznam')]);
//    }
//
//    public function loadData()
//    {
//        parent::loadData();
////        TODO: overit
//        RonPortalSession::setValue('typ_dokumentu', isset($_GET['typ_dokumentu']) ? $_GET['typ_dokumentu'] : null );
//    }
//
//
//    public function renderTypDokumentu()
//    {
//        echo '<DIV class="filterrow">';
//
//        echo '<SPAN class="filtercaption">';
//        echo ronText('Typ_dokumentu');
//        echo '</SPAN>';
//
//        // combobox pro vyber
//        $sql =
//            "SELECT Kod, Popis FROM TypDokumentu WHERE kod >= 0 AND KategorieDokumentu IN (" . DokumentyOsobyBrowser::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ")";
//
//        $sql .=
//            ' ORDER BY Popis';
//
//        $zvolenyTyp = (int)RonPortalSession::getValue('typ_dokumentu');
//        // vyber umisteni podle popisu
//        $combo = new ComboBox('typ_dokumentu');
//        $combo->className = 'form-input adsfilterforminput_full ronselect';
//        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
//        $combo->emptyRow = ronText('_vse_');
//        $combo->loadData($this ->database, $sql, $zvolenyTyp);
//        $combo->setSelectedOption($zvolenyTyp);
//        $combo->render();
//
//        echo "</DIV>\n";
//    }
//
//    public function renderFilters()
//    {
//        //TODO: predelat na nove filtry
//        parent::renderFilters();
//
//        $this->renderTypDokumentu();
//    }
//}

class DokumentOsobyLinkField extends BrowserField
{
    /** @var DokumentyOsobyBrowser */
    private $dokumentyBrowser;

    /**
     * @param Database $database
     * @param DokumentyOsobyBrowser $dokumentyBrowser
     */
    public function __construct($database, $dokumentyBrowser)
    {
        parent::__construct($database);
        $this ->dokumentyBrowser = $dokumentyBrowser;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $href = $this->link;

        $distribuovany = '';
        $osobaID = '';

        if (isset($rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX]) && $rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX] != 0) {
            //dokument je distribuovany
            $distribuovany = '&vzkazid=' . $rowData[6] .
                '&platnostod=' . StringUtils::format('date', $this->database->stringToTimestamp($rowData[7]));
        } else {
            //je vybrana jina osoba nez prihlaseny uzivatel, do odkazu na stazeni se doplni id vybraneho PP (kontroluje se pravo podle RC)
            if ($this ->dokumentyBrowser ->getRC() != User::getRC()) {
                $osobaID = '&osoba=' . $this ->dokumentyBrowser ->getVybranaOsoba();
            }
        }

        if ($recordId != '') {
            $href .= '&dokumentid=' . $recordId . $osobaID . $distribuovany;
        }

        return
            '<a href="' . $href . '" target="_blank">' .
            ($value ? StringUtils::htmlTextConvert($value) : StringUtils::htmlTextConvert($rowData[DokumentyOsobyBrowser::DOKUMENT_NAZEV_INDEX])) .
            '</a>';
    }
}


class DistribuovanyDokumentField extends BrowserField
{
    /** @var DokumentyOsobyBrowser */
    private $dokumentyBrowser;

    /** @var  PravaNaOsoby */
    var $pravaProhlizet;

    function __construct($database, $dokumentyBrowser)
    {
        parent::__construct($database);
        $this ->dokumentyBrowser = $dokumentyBrowser;

        $this->header = ronText('distribuovany');
        $this->align = 'center';
        $this->className = 'dokument_distribuovany';
        $this->code = 'col-distribuovany';

        $this->pravaProhlizet = Prava::vratPrava('dokumenty_prohl');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $distribuovanyTyp = $rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX] ?? 0;

        //typ dokumentu neni distribuovany
        if ($distribuovanyTyp == 0) {
            return ronText('Str_No');
        }

        if ($distribuovanyTyp == 1 || $distribuovanyTyp == 2) {
            //pokud ma uzivatel prava na vybranou osobu
            // TODO: neuvidim i osoby na ktere nemam pravo?
            if ($this->pravaProhlizet->lzeZobrazitRC($this ->dokumentyBrowser ->getRC())) {
                return
                    '<a href="?menuid=dokumenty&distribuovanedetail=1&dokumentid=' . $rowData[0] .
                    '&vzkazid=' . $rowData[6] .
                    '&platnostod=' . DateUtils::dateToUrlString($this->database->stringToTimestamp($rowData[7])) . '"' .
                    ' title="' . ronText('odkaz_na_detail') .
                    '">' .
                    StringUtils::format('date', $this->database->stringToTimestamp($rowData[10])) .
                    '</a>';
            }

            //pokud neni pravo, zobrazi se jen datum distribuce bez odkazu
            return StringUtils::format('date', $this->database->stringToTimestamp($rowData[10]));
        }

        return '';
    }
}

class PrectenyDokumentField extends BrowserField
{
    function __construct($database)
    {
        parent::__construct($database);

        $this->header = ronText('identifikace_podpisu');
        $this->className = 'dokument_identifikace_podpisu';
        $this->code = 'col-prectenidokumentu';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

        $distribuovanyTyp = $rowData[DokumentyOsobyBrowser::DISTRIBUOVANY_TYP_INDEX] ?? 0;

        //typ dokumentu neni distribuovany, nebo se nevyzaduje precteni
        if (($distribuovanyTyp == 0) || ($distribuovanyTyp == 2)) {
            return $result;
        }

//TODO: u dokumentu ktere maji zadane potvrdit do zobrazit datum a neumoznit potvrzeni po tomto datu (doplnit kontrolu do controlleru!)
        //pokud je dokument precteny, zobrazi identifikaci podpisu
        if ($rowData[9] == 1) {
            $result .= $rowData[11];
        }
        //pokud je dokument distribuovany = je nastaven datumCas u vzkazu, zobrazi odkaz na potrvrzeni precteni
        //Potvrdit je mozno jen dokument distribuovany prihlasene osobe.
        elseif ($rowData[10] && ($rowData[13] == User::getRC())) {

            $result .=
                '<a href="?menuid=dokumenty&potvrzeniprecteniform=1&formaction=edit&vzkazid=' . $rowData[6] .
                '&dokumentid=' . $rowData[0] .
                '&platnostod=' . StringUtils::format('date', $this->database->stringToTimestamp($rowData[7])) .
                '">';
            $result .= ronText('potvrzeni_precteni');
            $result .= '</a>';
        }

        if ($rowData[14]) {
            $class = '';
            if ($rowData[14] && (DateUtils::startOfTheDay(time()) >= $this->database->stringToTimestamp($rowData[14]))) {
                $class = 'class="distribuovany_po_terminu"';
            }
            if ($result) {
                $result .= ', ';
            }
            $result .= '<span ' . ($class ?: '') . '>' . ronText('dokumenty_potvrdit_do') . ': ' .
                DBUtils::getFormatedValue($this->database, 'finedate', $rowData[14])  .'</span>';
        }
        return $result;
    }
}


class DokumentyOsobyBrowser extends RonPortalBrowser
{
    const ZOBRAZOVAT_KATEGORIE_DOKUMENTU = "'1','2','50','51'";
    const DISTRIBUOVANY_TYP_INDEX = 8;
    const DOKUMENT_PRECTENO_INDEX = 9;
    const DOKUMENT_NAZEV_INDEX = 3;


    var $downloadLink = '';

    var $prvniDokument;  // pomocne promenne pro zobrazeni sekci
    var $nadpisSekce;

    var $downloadField;

    var $defaultSelectPart;
    var $distribuovanePrvni = false;

    /**
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'dokumenty'); // odkaz pro download se nastavuje az u radku, protoze se zobrazuje vice typu dokumentu (osoby, prac.mista, rozdeleni)

        $this->maxPageRows = null;
        $this ->browserUrl = '?menuid=dokumenty&';

        $this->code = 'browserdokumenty';

        $this->title = ronText('Dokumenty');
        $this->tableId = 'Dokumenty';
        $this->sqlFromPart = 'Dokument, TypDokumentu';
        $this->sqlWherePart = 'Dokument.Typ = TypDokumentu.Kod';
        $this->sqlSelectPart =  //U distribuovanych se doplnuji dalsi sloupce!
            'Dokument.Id, Dokument.Datum, TypDokumentu.Popis, Dokument.Popis, Dokument.JmenoSouboru, TypDokumentu.DefinicePrava';
        $this->defaultSelectPart = $this->sqlSelectPart;

        $this ->primaryField = 0;
        $this->distinct = true; //U distribuovanych se distinct nepouziva!

        $this->cssClass = 'browser minwidth';

        $this->prvniDokument = true;

        $this ->addField()
            ->setSqlFieldName('Dokument.Datum', 1) ->setHeaderText('Date')
            ->setFormat('date')->setOrdering(true)->setClassName('dokument_datum')
            ->setCode('col-datum');
        $this->defaultSort = 'col-datum'; // podle datumu

        $this ->addField()
            ->setSqlFieldName('TypDokumentu.Popis', 2) ->setHeaderText('Typ_dokumentu')
            ->setOrdering(1)->setClassName('dokument_typ')
            ->setCode('col-typ');

        $this ->addField()
            ->setSqlFieldName('Dokument.Popis', 3) ->setHeaderText('Popis')
            ->setOrdering(1)->setClassName('dokument_popis')
            ->setCode('col-popis');

        // odkaz na stahnuti souboru
        $this->downloadField = new DokumentOsobyLinkField($this ->database, $this);
        $this->addField($this->downloadField)
            ->setSqlFieldName('Dokument.JmenoSouboru', 4)
            ->setLink($this->downloadLink)->setHeaderText('Dokument_nazev')
            ->setOrdering(true)
            ->setCode('col-nazev');

        // datum distribuce s odkazem na prehled distribuce podle prav
        $this->addField(new DistribuovanyDokumentField($database, $this));

        // u prectenych identifikace podpisu, u neprectenych odkaz na potvrzeni precteni
        $this->addField(new PrectenyDokumentField($database));

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldIndex(12) ->setHeaderText(TextManager::_('vyjadreni') . ': ')
            ->setHeaderColSpan(-1) ->setClassName('poznamka')
            ->setFormat('longtext');

    }

    /**
     * @throws Exception
     */
    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, 'dokumenty', true);
//        $this->filterForm->f_formaction = '?menuid=dokumenty&';
//        $this->filterForm->f_classname = 'filtercontainer leftcaptions';

        $this->filterForm->nastavPersonalistiku([
            Prava::vratPrava('prohl_osobni'),
            Prava::vratPrava('dokumenty_prohl'),
            Prava::vratPrava('dokumenty_seznam')
        ]);

        // filtr typ dokumentu
        $filtrTyp = new FilterFormFilter('typdokumentu', FilterFormFilter::TYPE_OPTIONS);
        $filtrTyp
            ->setCaption(TextManager::_('Typ_dokumentu'))
            ->setRowId('filterrow_typdokumentu')
            ->setSessionName('filterform_typ_dokumentu')
            ->setOptionsSql(
                'SELECT Kod, Popis FROM TypDokumentu WHERE kod >= 0 AND KategorieDokumentu IN (' . DokumentyOsobyBrowser::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ') AND %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setAddClass('ronselect')
            ->setHtmlAttributes( ['data-ronselect-class' => 'ronselect adsfilterforminput_full','data-ron-autosubmit' => 1, 'data-columns' => 'text'])
            ->setSqlFilter('TypDokumentu.Kod =');
        $this ->filterForm ->addFilter($filtrTyp);

        $filtrPosledniVerze = new FilterFormFilter('posledniverze', FilterFormFilter::TYPE_CHECKBOX);
        $filtrPosledniVerze
            ->setCaption2(TextManager::_('jen_posledni_verze_dokumentu'))
            ->setSessionName('filterform_posledni_verze_dokumentu')
            ->setDefaultValue(false);

        $this ->filterForm ->addFilter($filtrPosledniVerze);

//        $this ->filterForm = new DokumentyOsobyFilterForm($this->database, null);
//        $this ->filterForm ->nastavAkciGet('dokumenty');
//        $this ->filterForm ->loadData();
    }


    public function getVybranaOsoba()
    {
        return $this ->filterForm ->getFilterValue('osoba');
    }

    public function getRC()
    {
        return $this ->filterForm ->getFilterValue('osoba', 'rc');
    }


    function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key == 'distribuovaneprvni') {
            $this ->distribuovanePrvni = (int)$valueText;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }

    /**
     * @param Parameters $params
     * @return string
     */
//    public function setFilterTypDokumentu($params)
//    {
//        if (RonPortalSession::getValue('typ_dokumentu')) {
//            return ' AND TypDokumentu.Kod = ' . $params->addParameter(RonPortalSession::getValue('typ_dokumentu'));
//        }
//
//        return '';
//    }

    /**
     * Dokumenty prirazene osobe nebo pracovnimu pomeru
     * @throws Exception
     */
    function showDokumentyOsoby()
    {
        $this->downloadField->setLink('?menuid=dokumenty&dokumentosoby=1');

        $params = $this->database->createParameters();

        $this->sqlFromPart =
            'Dokument INNER JOIN TypDokumentu ON (Dokument.Typ  =  TypDokumentu.Kod)' .
            ' INNER JOIN DokumentyOsoby ON (Dokument.id  =  DokumentyOsoby.id)';
        $this->sqlWherePart = new SqlStatement(
            '(Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND (DokumentyOsoby.RC = ' . $params->addParameter($this->getRC()) . ')' .
            ' AND TypDokumentu.KategorieDokumentu IN (' . self::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ')'.
            ' AND TypDokumentu.Kod >= 0' . //-1 = vyplatni pasky - jsou ve vlastnim browseru
            ' AND NOT ((TypDokumentu.RizenyDokument = 1) AND ((TypDokumentu.Distribuovany = 1) OR (TypDokumentu.Distribuovany = 2)) )' ,
//            $this->setFilterTypDokumentu($params),
            $params);


        $sqlText = $this ->getSQL();
        $this ->showDokumentySQL($sqlText, ronText('Dokumenty_osobni'));

        if (RonPortalSettings::getIntValue('povolituploaddokumentu', 1)) {
            echo
                '<tr><td colspan="' . $this->columnCount . '">' .
                '<div class="normaltext"><a href="#" class="show-dokument-dialog">' . ronText('Vlozit_novy_dokument') . '</a></div>' .
                "</td></tr>\n";
        }
    }


    /**
     * Dokumenty k rozdeleni osoby
     * @throws Exception
     */
    function showDokumentyRozdeleni()
    {
        $this ->downloadField ->setLink('?menuid=dokumenty&dokumentrozdeleni=1');

        $params = $this->database->createParameters();

        $this ->sqlFromPart =
            'Dokument INNER JOIN TypDokumentu ON (Dokument.Typ  =  TypDokumentu.Kod)' .
            ' INNER JOIN DokumentyRozdeleni ON (Dokument.Id = DokumentyRozdeleni.Id)' .
            ' INNER JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod)';

        $this ->sqlWherePart = new SqlStatement(
            '(Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND RozdeleniOsob.KodTypuRozdeleni <> -1 ' .    // vynecha pracovni mista
            ' AND TypDokumentu.KategorieDokumentu IN (' . self::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ')'.
            ' AND (RozdeleniOsob.RC = ' . $params->addParameter($this->getRC()) . ')' .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter(time()) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND NOT ((TypDokumentu.RizenyDokument = 1) AND ((TypDokumentu.Distribuovany = 1) OR (TypDokumentu.Distribuovany = 2)) )' ,
//            $this->setFilterTypDokumentu($params),
            $params);

        $sqlText = $this ->getSQL();

        $this ->showDokumentySQL($sqlText, ronText('Dokumenty_rozdeleni'));
    }


    /**
     * Dokumenty pripojene k pracovnimu mistu
     * @throws Exception
     */
    function showDokumentyPracMista()
    {
        $this ->downloadField->setLink('?menuid=dokumenty&dokumentpmista=1');

        $params = $this->database->createParameters();

        $this ->sqlFromPart = 'Dokument INNER JOIN TypDokumentu ON (Dokument.typ  =  TypDokumentu.kod)
            INNER JOIN DokumentyRozdeleni ON (Dokument.Id = DokumentyRozdeleni.Id)
            INNER JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = DokumentyRozdeleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = DokumentyRozdeleni.Kod)  ';
        $this ->sqlWherePart = new SqlStatement(
            'Dokument.Id = DokumentyRozdeleni.Id ' .
            ' AND (Dokument.ZobrazitNaIntrawebu = 1  OR  (Dokument.ZobrazitNaIntrawebu = 2  AND  TypDokumentu.ZobrazitNaIntrawebu = 1))' .
            ' AND RozdeleniOsob.KodTypuRozdeleni = -1 ' .    // jen pracovni mista
            " AND TypDokumentu.KategorieDokumentu IN (" . self::ZOBRAZOVAT_KATEGORIE_DOKUMENTU . ") ".
            " AND (RozdeleniOsob.RC = " . $params->addParameter($this->getRC()) . ")" .
            ' AND RozdeleniOsob.PlatnostOd <= ' . $params->addDateParameter(time()) .
            ' AND (RozdeleniOsob.PlatnostDo >= ' . $params->addDateParameter(time()) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)' .
            ' AND NOT ((TypDokumentu.RizenyDokument = 1) AND ((TypDokumentu.Distribuovany = 1) OR (TypDokumentu.Distribuovany = 2)) )' ,
//            $this->setFilterTypDokumentu($params),
            $params);

        $sqltext = $this ->getSQL();
        $this ->showDokumentySQL($sqltext, ronText('Dokumenty_pracovni_misto'));
    }

    /**
     * @throws Exception
     */
    function showDistribuovaneDokumenty()
    {
        $this ->downloadField->setLink('?menuid=dokumenty&distribuovanedokumenty=1');

        $this->distinct = false;

        $params = $this->database->createParameters();

        $this->sqlSelectPart .=
            ', DokumentVzkazu.IdVzkazu, DokumentVzkazu.PlatnostOd, TypDokumentu.Distribuovany, DokumentVzkazu.Precteno, Vzkaz.DatumCas, ' . //6 - 10
            $this->database->castWideMemo('DokumentVzkazu.IdentifikacePodpisu') . ', ' . $this->database->castWideMemo('DokumentVzkazu.Vyjadreni') . ', ' . // 11 - 12
            'Vzkaz.RCPrijemce, VerzeDokumentu.PotvrditDo'; //13 - 14

        $this->sqlFromPart =
            'Vzkaz JOIN DokumentVzkazu ON (Vzkaz.Id = DokumentVzkazu.IdVzkazu)' .
            ' JOIN VerzeDokumentu ON (VerzeDokumentu.Id = DokumentVzkazu.IdDokumentu AND VerzeDokumentu.PlatnostOd = DokumentVzkazu.PlatnostOd)' .
            ' JOIN Dokument ON (Dokument.Id = VerzeDokumentu.Id) JOIN TypDokumentu ON (Dokument.Typ  =  TypDokumentu.Kod)';

        $this->sqlWherePart = new SqlStatement(
            ' RCPrijemce = ' . $params->addParameter($this->getRC()) .
            ($this->filterForm->getFilterValue('posledniverze')
                ? ' AND NOT EXISTS(SELECT VDN.ID FROM VerzeDokumentu VDN WHERE VDN.ID = VerzeDokumentu.Id AND VDN.PlatnostOd > VerzeDokumentu.PlatnostOd)'
                : ''
            ),
//            $this->setFilterTypDokumentu($params),
            $params);

        $sqlText = $this->getSQL();
        $this->showDokumentySQL($sqlText, ronText('distribuovane_dokumenty'));
    }



    /**
     * @throws Exception
     */
    function renderData()
    {
        if ($this->distribuovanePrvni) {
            //metoda rusi pouziti distinctu a meni SELECT
            $this ->showDistribuovaneDokumenty();
            $this->distinct = true;
            $this->sqlSelectPart = $this->defaultSelectPart;
        }

        // Osobni dokumenty
        $this ->showDokumentyOsoby();
        $this ->showDokumentyPracMista();
        $this ->showDokumentyRozdeleni();

        if (!$this->distribuovanePrvni) {
            //metoda rusi pouziti distinctu
            $this->showDistribuovaneDokumenty();
        }
    }

    public function render()
    {
        parent::render();

        if (RonPortalSettings::getIntValue('povolituploaddokumentu', 1)) {
            $form = new NovyDokumentOsobyForm($this->database, $this ->getVybranaOsoba());
            $form->render();
        }
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //pokud je u typu dokumentu prirazen typ prava
        if ($values[5]) {
            $prava = Prava::vratPrava($values[5]);
            if (!$prava->lzeZobrazitRC($this->getRC(), DateUtils::today(), DokumentyController::vratObdobiDoProPrava())) return false;
        }
        return true;
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        if ($this->prvniDokument && $this->nadpisSekce) {
            echo '<tr><td class="padding-top-2" colspan="' . $this->columnCount . '"><b>' . $this->nadpisSekce . '</b></td></tr>';
            $this ->prvniDokument = false;
        }

        parent::renderLine($values, $className, $rowNumber, $edit);
    }


    /**
     * @param SqlStatement $sql
     * @param string $nadpisSekce
     * @throws Exception
     */
    function showDokumentySQL($sql, $nadpisSekce)
    {
        $this->prvniDokument = true;
        $this->nadpisSekce = $nadpisSekce;

        MessageManager::addDebugLog('Browser sekce ' . $nadpisSekce . $sql->sqlText);

        $cursor = $this ->database ->select($sql);
        if ($cursor) {
            $this ->renderCursor($cursor);
        } else {
            echo '<tr><td colspan="' . $this ->columnCount . '">';
            MessageManager::addDbErrorMessage($this->database,$sql);
            echo '</td></tr>';
        }

        if (!$this ->prvniDokument) {
            $this ->renderSeparator();
        }
    }

}
