<?php
/** Roman.Szablatura 15.3.2016 */

namespace RonPortal\Personalistika;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\Dokument;
use RonPortal\PracovniPomer;

/**
 * Class LekarskaProhlidka
 */
class LekarskaProhlidka
{
    public $rc;
    public $jmeno;
    public $prijmeni;
    public $kodLekarskeProhlidky;
    public $popisLekarskeProhlidky;
    public $datumVznikuPovinnosti;
    public $duvodProhlidky;
    public $datumAbsolvovani;
    public $nazevDokumentu;
    public $vysledek;
    /** @var string     Lookup z tabulky S_VyhodnoceniProhlidky.Popis */
    public $popisVysledku;
    /** @var string */
    public $vysledekProhlidky;
    public $platnostDo;
    public $datumPosledniProhlidky;
    public $povinna;
    public $datumProhlidkyRezervovany;
    public $casProhlidkyRezervovany;
    public $doplnujiciTextPozvanky;
    public $historieKorespondence;
    public $osetrujiciLekar;
    public $poznamka;
    public $idOsoby;

    /* @var Dokument[] $dokumenty */
    public $dokumenty;

    /**
     * @param Database $database
     * @param PracovniPomer $pracovniPomer
     * @param string $kodLP
     * @param int $datumVzniku
     * @return LekarskaProhlidka
     * @throws Exception
     * @throws DBException
     */
    public static function nactiProhlidku($database, $pracovniPomer, $kodLP, $datumVzniku)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT KodLekarskeProhlidky, DatumVznikuPovinnosti, S_DuvodProhlidky.Popis, DatumAbsolvovani, S_VyhodnoceniProhlidky.Popis, PlatnostDo,' . //0 - 5
            ' DatumPosledniProhlidky, Povinna, Vysledek, DatumProhlidkyRezervovany, CasProhlidkyRezervovany, ' . $database->castWideMemo('DoplnujiciTextPozvanky') . ', ' . //6 - 11
            $database->castWideMemo('HistorieKorespondence') . ",  COALESCE(OsetrujiciLekar.JmenoLekare,'') {fn_concat()} ' ' {fn_concat()} COALESCE(OsetrujiciLekar.Prijmeni,''), " . // 12,13
            $database->castWideMemo('GenerovaneLekProhlidky.Poznamka') . ',' . //14
            ' Osoba.Jmeno, Osoba.Prijmeni, LekarskaProhlidka.Popis, ' . //15 - 17
            $database->castWideMemo('GenerovaneLekProhlidky.VysledekProhlidky') . //18
            "\nFROM LekarskaProhlidka" .
            '  JOIN GenerovaneLekProhlidky ON (GenerovaneLekProhlidky.KodLekarskeProhlidky = LekarskaProhlidka.Kod)' .
            '  JOIN Osoba ON (GenerovaneLekProhlidky.RC = Osoba.RC)' .
            '  LEFT OUTER JOIN OsetrujiciLekar ON (GenerovaneLekProhlidky.OsetrujiciLekar = OsetrujiciLekar.Kod)' .
            '  LEFT OUTER JOIN S_DuvodProhlidky ON (GenerovaneLekProhlidky.DuvodProhlidky = S_DuvodProhlidky.Kod)' .
            '  LEFT OUTER JOIN S_VyhodnoceniProhlidky ON (GenerovaneLekProhlidky.Vysledek = S_VyhodnoceniProhlidky.Kod)' .
            "\nWHERE GenerovaneLekProhlidky.RC = ". $params->addParameter($pracovniPomer->rc)  .
            " AND LekarskaProhlidka.Kod = " . $params->addParameter($kodLP)  .
            ' AND GenerovaneLekProhlidky.DatumVznikuPovinnosti = ' . $params->addTimestampParameter($datumVzniku) ;

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if(!$row){
            throw new Exception(TextManager::_('prohlidka_neexistuje'));
        }

        $prohlidka = new LekarskaProhlidka();
        $prohlidka->rc = $pracovniPomer->rc;
        $prohlidka->idOsoby = $pracovniPomer->idPP;
        $prohlidka->nactiProhlidkuZRadku($database, $row);

        $prohlidka->dokumenty = self::nactiDokumentyProhlidky($database, $pracovniPomer->rc, $kodLP, $datumVzniku);

        return $prohlidka;
    }

    /**
     * @param Database $database
     * @param $row
     */
    public function nactiProhlidkuZRadku($database ,$row)
    {
        $this->kodLekarskeProhlidky = $row[0];
        $this->datumVznikuPovinnosti = $database->stringToTimestamp($row[1]);
        $this->duvodProhlidky = $row[2];
        $this->datumAbsolvovani = $database->stringToTimestamp($row[3]);
        $this->popisVysledku = $row[4];
        $this->platnostDo = $database->stringToTimestamp($row[5]);
        $this->datumPosledniProhlidky = $database->stringToTimestamp($row[6]);
        $this->povinna = $row[7];
        $this->vysledek = $row[8];
        $this->datumProhlidkyRezervovany = $database->stringToTimestamp($row[9]);
        $this->casProhlidkyRezervovany = $database->stringToTimestamp($row[10]);
        $this->doplnujiciTextPozvanky = $row[11];
        $this->historieKorespondence = $row[12];
        $this->osetrujiciLekar = $row[13];
        $this->poznamka = $row[14];
        $this->jmeno = $row[15];
        $this->prijmeni = $row[16];
        $this->popisLekarskeProhlidky = $row[17];
        $this->vysledekProhlidky = $row[18];
    }


    /**
     * @param Database $database
     * @param string $rc
     * @param string $kodLP
     * @param int $datumVzniku
     * @return Dokument[]
     * @throws DBException
     */
    public static function nactiDokumentyProhlidky($database, $rc, $kodLP, $datumVzniku)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT ' . Dokument::getSelectPart($database) .
            ' FROM Dokument, DokumentyOsoby, TypDokumentu' .
            ' WHERE Dokument.Id = DokumentyOsoby.Id' .
            '  AND TypDokumentu.Kod = Dokument.Typ' .
            '  AND DokumentyOsoby.GLP_RC = ' . $params->addParameter($rc) .
            '  AND DokumentyOsoby.GLP_KodLP = ' . $params->addParameter($kodLP) .
            '  AND DokumentyOsoby.GLP_Datum = ' . $params->addParameter($database->timestampToParam($datumVzniku)) .
            '  AND TypDokumentu.KategorieDokumentu = 3' .
            ' ORDER BY Dokument.Datum DESC, Dokument.Id DESC';

        return Dokument::nactiDokumenty($database, $sql, $params);
    }
}
