<?php
/**
 * stanislav.elbl, 09.03.2021
 */

namespace RonPortal\Personalistika;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\DokumentyFormField;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\ReportsFormField;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;


/**
 * Class RezervovanyCasLekarskeProhlidkyField
 * @package RonPortal\Personalistika
 */
class RezervovanyCasLekarskeProhlidkyField extends FormField
{
    /**
     * RezervovanyCasLekarskeProhlidkyField constructor.
     */
    function __construct()
    {
        parent::__construct(FormField::FIELD_TYPE_DATEANDTIME);
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $strDate = $rowData
            ? StringUtils::format('date', $database ->stringToTimestamp($rowData[7]))
            : '';

        $strTime = $rowData
            ? StringUtils::format('time', $database ->stringToTimestamp($rowData[8]))
            : '';

        if ($canEdit) {
            echo
            "<input type='text' class='inputDate' size='7' name='{$this->fieldName}' value='$strDate' />";
            $this->showCalendarButton();

            echo
            "<input type='text' class='inputTime' size='3' name='{$this->fieldName}_time' value='$strTime' />";
        } else {
            echo $strDate . ' ' . $strTime;
        }
    }
}



///**
// * Class DokumentyLekarskeProhlidkyField
// * @package Ron\Personalistika
// */
//class DokumentyLekarskeProhlidkyField extends FormField
//{
//    /** @var Dokument[] */
//    var $dokumenty;
//    /** @var int */
//    var $idOsoby;
//
//
//    function __construct($dokumenty, $idOsoby)
//    {
//        parent::__construct();
//
//        $this->dokumenty = $dokumenty;
//        $this->idOsoby = $idOsoby;
//    }
//
//    /**
//     * @param Database $database
//     * @param mixed $value
//     * @param bool $canEdit
//     * @param bool $insert
//     * @param array $rowData
//     * @throws Exception
//     */
//    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
//    {
//        if ($this->dokumenty) {
//            Dokument::renderTable($this ->dokumenty,
//                "?menuid=lekarskeprohlidky&action=download&osoba={$this->idOsoby}&dokument=");
//        }
//    }
//}


/**
 * Class LekarskaProhlidkaEditaceAbsolvovaniForm
 * @package RonPortal\Personalistika
 */
class LekarskaProhlidkaForm extends RonPortalForm
{
    /** @var PravaNaOsoby $pravaZapis */
    var $pravaZapis;
    /** @var bool */
    var $lzeEditovatZaznam;

    /** @var PravaNaOsoby $pravaAbsolvovat */
    var $pravaAbsolvovat;
    /** @var bool */
    var $lzeAbsolvovatProhlidku;

    /** @var string */
    private $osobaRC;

    private $muzeAbsolvovat;

    /**
     * LekarskaProhlidkaEditaceAbsolvovaniForm constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravoNaZapis
     * @param PravaNaOsoby $pravoAbsolvovat
     * @param bool $absolvovat
     * @throws NeniPravoException
     * @throws \Exception
     */
    function __construct($database, $pravoNaZapis, $pravoAbsolvovat, $absolvovat)
    {
        parent::__construct($database);

        $this->muzeAbsolvovat = $absolvovat;
        $this->pravaZapis = $pravoNaZapis;
        $this->pravaAbsolvovat = $pravoAbsolvovat;

        $pracovniPomer = null;

        // kontrola prav na zaznam
        if ($this ->formAction != 'new') {
            $pracovniPomer = PracovniPomer::nactiPP($this->database);

            if (!(Prava::vratPrava('lekarske_prohl')->lzeZobrazitRC($pracovniPomer->rc)
                || $this->pravaAbsolvovat->lzeZobrazitRC($pracovniPomer->rc)
                || $this->pravaZapis->lzeZobrazitRC($pracovniPomer->rc))
            ) {
                throw new NeniPravoException();
            }
        } else if (!$this ->pravaZapis ->jsouNejakePrava()) {
            // nema pravo zapisovat => nemuze vytvorit novou prohlidku
            throw new NeniPravoException();
        }

        $this->osobaRC = $pracovniPomer ? $pracovniPomer->rc : null;

        $this->lzeEditovatZaznam = isset($this->osobaRC)
            ? $this->pravaZapis->lzeZobrazitRC($this->osobaRC)
            : $this->pravaZapis->jsouNejakePrava();

        $this->lzeAbsolvovatProhlidku = isset($this->osobaRC)
            ? $this->pravaAbsolvovat->lzeZobrazitRC($this->osobaRC)
            : $this->pravaAbsolvovat->jsouNejakePrava();

        $this->code = 'formlekarskaprohlidka';

        $this->title = TextManager::_('Zdravotni_prohlidka');

        //$this->htmlAction = '?menuid=lekarskeprohlidky&action=' . ( $this->muzeAbsolvovat ? 'absolvovat' : 'lekarskaprohlidkaform');
        $this->stornoAction = '?menuid=lekarskeprohlidky';

        $this->sqlTableName = 'GenerovaneLekProhlidky';

        $this->sqlSelectPart =
            'GenerovaneLekProhlidky.DatumVznikuPovinnosti, GenerovaneLekProhlidky.DatumAbsolvovani, GenerovaneLekProhlidky.KodLekarskeProhlidky, ' .   // 0 - 2
            $database->castWideMemo('GenerovaneLekProhlidky.Poznamka') . ' as Poznamka, ' .     // 3
            'GenerovaneLekProhlidky.Vysledek, GenerovaneLekProhlidky.OsetrujiciLekar, ' .       // 4, 5
            $database->castWideMemo('GenerovaneLekProhlidky.DoplnujiciTextPozvanky') . ' as DoplnujiciTextPozvanky, ' . // 6
            'GenerovaneLekProhlidky.DatumProhlidkyRezervovany, GenerovaneLekProhlidky.CasProhlidkyRezervovany, ' .      // 7,8
            'S_DuvodProhlidky.Popis,' .   // 9
            $database->castWideMemo('GenerovaneLekProhlidky.VysledekProhlidky') . ', ' . // 10
            'GenerovaneLekProhlidky.Generovana';  //11


        //Osoba
//        $this->addField(new OsobaField(\Ron\ronText('Osoba'),'lekarske_edit',time(),$vlozeni));
        //$this->addField(new OsobaProhlidkyField($pracovniPomer));
        $this->addField(new OsobaFormField('lekarske_edit', DateUtils::today(), $pracovniPomer))
            ->setFieldId('prohlidka_osoba')
            ->setReadOnly(true)
            ->otherHtmlParams = [
                'class' => 'form-input ronselect',
                'data-columns' => 'text;osobni-cislo',
                'data-captions' => \Ron\ronText('Jmeno') . ';' . \Ron\ronText('Osobni_cislo')
            ];

        //Zobrazeni sekce Detaily
        $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCaption('Detail');

        //Prohlidka
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('KodLekarskeProhlidky') ->setCaption('Zdravotni_prohlidka') ->setRequired($this ->isInsert())
            ->setCode('field-kodprohlidky')
            ->setSqlFieldName('GenerovaneLekProhlidky.KodLekarskeProhlidky') ->setSqlFieldIndex(2)
            ->setSelectSql('SELECT LekarskaProhlidka.Kod, LekarskaProhlidka.Popis  FROM LekarskaProhlidka')
            ->setReadOnly(true);

        //Datum vzniku povinnosti
        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumVznikuPovinnosti') ->setCaption('datum_vzniku_povinnosti') ->setRequired(1)
            ->setCode('field-datumvznikupovinnosti')
            ->setSqlFieldName('DatumVznikuPovinnosti') ->setSqlFieldIndex(0)
            ->setReadOnly(true) ->setPrimary(true)
            ->setDefaultValue(StringUtils::format('date',DateUtils::startOfTheDay(time())));

        if (!$this ->isInsert()) {
            $this->addField()   // popis duvodu
                ->setSqlFieldIndex(9)->setReadOnly(true)
                ->setCaption('duvod_zdravotni_prohlidky')
                ->setCode('field-duvodprohlidky');
        }

        //Rezervovany cas
        $this->addField(new RezervovanyCasLekarskeProhlidkyField())
            ->setCaption('rezervovany_cas_prohlidky')
            ->setFieldName('DatumProhlidkyRezervovany') ->setSqlFieldName(null)     // uklada se datum a cas samostatne (viz createDBRecordFields)
            ->setReadOnly($this->muzeAbsolvovat ? true : !$this->lzeEditovatZaznam);

        //Osetrujici lekar
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('OsetrujiciLekar') ->setCaption('osetrujici_lekar') ->setRequired(0)
            ->setCode('field-osetrujicilekar')
            ->setSqlFieldIndex(5) ->setReadOnly($this->muzeAbsolvovat ? true : !$this->lzeEditovatZaznam)
            ->setSelectSql(
                "SELECT Kod, COALESCE(Prijmeni,'') {fn_concat()} ' ' {fn_concat()} COALESCE(JmenoLekare,'')" .
                ' FROM OsetrujiciLekar WHERE %FILTER% ORDER BY %ORDER%',
                '1 =  1', 'Prijmeni, JmenoLekare');

        //Doplnujici poznamka
        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('DoplnujiciTextPozvanky') ->setCaption('doplnujici_text')
            ->setCode('field-doplnujicitextpozvanky')
            ->setSqlFieldIndex(6)
            ->setReadOnly($this->muzeAbsolvovat ? true : !$this->lzeEditovatZaznam);

        //Poznamka
        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setCaption('Poznamka')
            ->setCode('field-poznamka')
            ->setSqlFieldIndex(3)
            ->setReadOnly($this->muzeAbsolvovat ? true : !$this->lzeEditovatZaznam);

        //Vysledek
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Vysledek') ->setCaption('vysledek') ->setRequired($this->muzeAbsolvovat) // !$vlozeni
            ->setCode('field-vysledek')
            ->setSqlFieldIndex(4) ->setReadOnly(!($this->muzeAbsolvovat && $this->lzeAbsolvovatProhlidku) && !$this->lzeEditovatZaznam)
            ->setTranslationPrefix('db_vyhodnoceniprohlidky_')
            ->setSelectSql(
                'SELECT Kod, Popis FROM S_VyhodnoceniProhlidky WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Kod');

        // Doplnujici popis (pole VysledekProhlidky)
        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('VysledekProhlidky') ->setCaption('Doplnujici_popis_prohlidky')
            ->setCode('field-vysledekprohlidky')
            ->setSqlFieldIndex(10)
            ->setReadOnly($this->muzeAbsolvovat ? true : !$this->lzeEditovatZaznam);

        //31.1.2018 po porada s RaUr datum absolvovani skryji (31.1.2018), doplni se automaticky po spusteni personalniho agenta
        //Datum absolvovani
        //$this->addField([FORM_FIELD_TYPE => FORM_FIELD_TEXT, FORM_FIELD_CAPTION_TEXT => 'datum_absolvovani', FORM_FIELD_TEXT => \Ron\ronText('datum_absolvovani_agentem')]);

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumAbsolvovani') ->setSqlFieldIndex(1) ->setCaption('datum_absolvovani')
            ->setRequired($this->muzeAbsolvovat)
            ->setReadOnly(!($this->muzeAbsolvovat && $this->lzeAbsolvovatProhlidku) && !$this->lzeEditovatZaznam)
            ->setUseDefaultValueToEditNull(true)
            ->setDefaultValue($this->muzeAbsolvovat ? ($this ->formAction == 'edit' ? StringUtils::format('date', time()) : '') : '');


        if ($pracovniPomer  &&  !$this ->isInsert()  &&  isset($_GET['kod'])) {
            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER) ->setCaption('tiskove_sestavy');

            //Zobrazeni tiskove sestavy
            $f = new ReportsFormField(
                '?menuid=lekarskeprohlidky&action=report&osoba=' . $pracovniPomer->idPP . '&kod=' . ($_GET['kod'] ?: '') .
                '&vznik=' . (StringUtils::format('date', $_GET['vznik']) ?: ''),
                'LekarskeProhlidky'
            );
            $f ->nameStart = 'LekarskeProhlidky'; // deprecated - nepouzivat - jen pro kompatibilitu
            $f ->setFieldId('prohlidky_reports');
            $this->addField($f);
        }

        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('Generovana')->setSqlFieldIndex(11)->setDefaultValue(0);

        $dokumenty = null;
        if ($pracovniPomer  &&  $this ->formAction == 'edit') {
            $dokumenty = LekarskaProhlidka::nactiDokumentyProhlidky($database, $this ->osobaRC, $_GET['kod'], $_GET['vznik']);
        }

        if($dokumenty  ||  $this->lzeEditovatZaznam) {
            $this->addField(FormField::FIELD_TYPE_SECTION_HEADER)->setCaption('Dokumenty');
        }

        //Zobrazeni sekce Dokumenty
        if ($dokumenty) {
            //$this->addField(new DokumentyLekarskeProhlidkyField($dokumenty, $pracovniPomer ->idPP));
            $this->addField(new DokumentyFormField($dokumenty, "?menuid=lekarskeprohlidky&action=download&osoba={$pracovniPomer ->idPP}&dokument="));
        }


        if($this->lzeEditovatZaznam) {
            $this->addField(FormField::FIELD_TYPE_FILE)
                ->setCaption('') ->setFieldName('*dokument')    // nenastavuje sqlFieldName
                ->setCode('field-dokument');
        }
    }


    function getSqlFromPart()
    {
        return
            'GenerovaneLekProhlidky INNER JOIN S_DuvodProhlidky' .
            ' ON S_DuvodProhlidky.Kod = GenerovaneLekProhlidky.DuvodProhlidky';
    }


    function getUrlParam($name)
    {
        if(isset($_POST[$name])) {
            return $_POST[$name];
        }

        if (isset($_GET[$name])) {
            return $_GET[$name];
        }

        return null;
    }

    /**
     * @return SqlStatement
     */
    function getWhereCondition()
    {
        $kod = $this ->getUrlParam('kod');
        $vznik = $this ->getUrlParam('vznik');

        if ($this->osobaRC != '' && $kod && $vznik) {
            $params = $this->database->createParameters();
            return new SqlStatement(
                ' RC = ' . $params->addParameter($this->osobaRC) .
                ' AND KodLekarskeProhlidky = ' . $params->addParameter($kod) .
                ' AND DatumVznikuPovinnosti = ' . $params->addParameter($this->database->timestampToParam($vznik)),
                $params);
        }

        return null;
    }


    function renderCommonFields()
    {
        parent::renderCommonFields();

        /*echo '<input type="hidden" name="absolvovat" value="' . $_GET['absolvovat'] . '" />';*/
        echo '<input type="hidden" name="kod" value="' . ($this ->getUrlParam('kod') ?: '') . '" />';
        echo '<input type="hidden" name="vznik" value="' . ($this ->getUrlParam('vznik') ?: '') . '" />';
    }

    /**
     * @param DBRecord $rec
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);

        $where = $this->getWhereCondition();
        if ($where instanceof SqlStatement) {
            $rec->setWhere($where->sqlText, $where->parameters);
        } else {
            $rec->where = $where;
        }
    }


    function createDBRecordFields($rec, $insert)
    {
        $rec->addStringField('RC', true, 15, $this->osobaRC);

        parent::createDBRecordFields($rec, $insert);

        //Datum
        if (isset($_POST['DatumProhlidkyRezervovany']) && ($_POST['DatumProhlidkyRezervovany'] != '')) {
            $rec->addDateField('DatumProhlidkyRezervovany', false, DateUtils::dateStringToTimestamp($_POST['DatumProhlidkyRezervovany']));
        }

        //cas
        if (isset($_POST['DatumProhlidkyRezervovany_time'])  && ($_POST['DatumProhlidkyRezervovany_time'] != '')) {
            $rec->addField('CasProhlidkyRezervovany', 'time', false, 0, DateUtils::timeStringToTimestamp($_POST['DatumProhlidkyRezervovany_time']));
        }

        if ($insert) {
            $rec ->addIntField('DuvodProhlidky', true, 1); // individualni

            $rec ->addStringField('Insert_UserName', true, 25, RonPortalUtils::getLogUserName());
            $rec ->addTimestampField('Insert_Timestamp', true, time());
        } else {
            $f = $rec ->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName());
            $f ->setUpdateLog(true);

            $f = $rec ->addTimestampField('Update_Timestamp', false, time());
            $f ->setUpdateLog(true);
        }
    }


    /**
     * @param DBRecord $rec
     * @return bool
     * @throws DBException
     */
    function beforeInsertRecord($rec)
    {
        return $this->testovaniVlozeniProhlidky($rec);
    }

    function afterInsertRecord($rec)
    {
        $this->vlozeniDokumentu($rec);
        return parent::afterInsertRecord($rec);
    }


    /**
     * @param DBRecord $rec
     * @return bool
     * @throws DBException
     */
    function beforeUpdateRecord($rec)
    {
        $rec->findFieldByName('Generovana')->setNewValue(0); // 9.9.2022 pri jakekoliv editaci zrusit priznak generovana

        $res = $this->testovaniVlozeniProhlidky($rec);

        $this->vlozeniDokumentu($rec);

        return $res;
    }

    function vlozeniDokumentu($rec)
    {
        if (isset($_FILES['dokument']['tmp_name']) &&  is_uploaded_file($_FILES['dokument']['tmp_name'])) {
            $typDokumentu = RonPortalSettings::getValue('lekprohl_typdokumentu');
            if (isset($typDokumentu) && $typDokumentu != '') {
                if (3 != DBUtils::getSqlValue($this->database,
                        'SELECT KategorieDokumentu FROM TypDokumentu' .
                        ' WHERE kod = ' . (int)$typDokumentu)
                ) {
                    throw new \Exception(TextManager::_('nekompatibilni_typ_dokumentu'));
                }
            }

            $idDokumentu = DokumentyUtils::vlozDokument($this->database,
                'dokument', (int)$typDokumentu,
                sprintf(TextManager::_('lekprohl_dokument'), $rec['KodLekarskeProhlidky']), $this);

            $recDokumentyOsoby = new RonPortalDBRecord($this->database, 'DokumentyOsoby', $this);

            $recDokumentyOsoby ->addField('id', 'int', false, 0, $idDokumentu);
            $recDokumentyOsoby ->addField('RC', '', false, 15, $this->osobaRC);
            $recDokumentyOsoby ->addField('GLP_RC', '', false, 15, $this->osobaRC);
            $recDokumentyOsoby ->addField('GLP_KodLP', '', false, 10, $rec['KodLekarskeProhlidky']);
            $recDokumentyOsoby ->addField('GLP_Datum', 'datetime', false, 0, $rec['DatumVznikuPovinnosti']);

            $recDokumentyOsoby ->insertRecord();
        }
    }

    /**
     * @param $rec
     * @return bool
     * @throws DBException
     * @throws \Exception
     */
    function testovaniVlozeniProhlidky($rec)
    {
        // kontrola prav na zapis/absolvovani
        $prava = $this->muzeAbsolvovat ? $this->pravaAbsolvovat : $this->pravaZapis;

        if (!$prava->lzeZobrazitRC($this ->osobaRC, time(), time())) {
            MessageManager::addErrorMessage(TextManager::_('Nemate_pravo'),'osoba');
            return false;
        }

        if ($rec['DatumAbsolvovani']) {
            // kontrola, jestli neexistuje jiny zaznam se stejnym datem absolvovani
            $params = $this ->database ->createParameters();
            $cursor = $this ->database ->getSqlResult(
                'SELECT DatumVznikuPovinnosti FROM GenerovaneLekProhlidky' .
                ' WHERE RC = ' . $params ->addParameter($rec['RC']) .
                ' AND KodLekarskeProhlidky = ' . $params ->addParameter($rec['KodLekarskeProhlidky']) .
                ' AND DatumVznikuPovinnosti <> ' . $params ->addTimestampParameter($rec['DatumVznikuPovinnosti']) .
                ' AND DatumAbsolvovani = ' . $params ->addDateParameter($rec['DatumAbsolvovani']),
                $params
            );
            $row = $cursor ->fetch();
            $cursor ->close();

            if ($row) {
                throw new \Exception(TextManager::_('Existuje_jiny_zaznam_se_stejnym_datem_absolvovani_prohlidky'));
            }
        }

        return true;
    }
}
