<?php
/**
 * Roman.Szablatura, 18.3.-016
 */

namespace RonPortal\Personalistika;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\HttpUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\OsobniCisloBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;


/**
 * Class DatumAbsolvovaniProhlidkyField
 */
class DatumAbsolvovaniProhlidkyField extends BrowserField
{

    private $prava;

    /**
     * DatumAbsolvovaniProhlidkyField constructor.
     * @param null|Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this->prava = Prava::vratPrava('lekarske_absolv');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Pokud je prohlidka absolvovana pak nezobrazit odkaz ale pouze "absolvovano"
        if ($rowData['col-vysledek']) {
            return ($value)
                ? parent::getViewText($value, $rowNumber, $recordId, $rowData) // StringUtils::format('date', $this->database->stringToTimestamp($rowData[LekarskeProhlidkyBrowser::DATUM_ABSOLVOVANI_INDEX]))
                : "<span title='" . TextManager::_('lekarska_prohlidka_absolvovano_info') . "'>" . TextManager::_('lekarska_prohlidka_absolvovano') . "</span>";
        }

        if (!$value  &&  $this->prava->lzeZobrazitRC($rowData['col-rc'])) {
            return
                '<a href="?menuid=lekarskeprohlidky&formaction=edit&osoba=' . $rowData[1] .
                '&kod=' . HttpUtils::urlEncode($rowData['col-kod']) . '&absolvovat=1' .
                '&vznik=' . $this->database->stringToTimestamp($rowData[0]) . '">' .
                TextManager::_('absolvovat') .
                '</a>';
        }

        return  '';
    }
}

/**
 * Class DetailProhlidkyField
 */
class DetailProhlidkyField extends BrowserField
{
    private $prava;

    /**
     * DetailProhlidkyField constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $prava
     */
    public function __construct($database, $prava)
    {
        parent::__construct($database);
        $this->header = TextManager::_('detail');
        $this->align = "center";
        $this ->code = "col-detail";

        $this->prava = $prava;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $lzeEditovat = $this->prava->lzeZobrazitRC($rowData['col-rc']);

        return
            '<a href="?menuid=lekarskeprohlidky&' . ($lzeEditovat ? 'formaction=edit' : 'action=detail') .
            '&osoba=' . $rowData[1] .
            '&kod=' . HttpUtils::urlEncode($rowData['col-kod']) .
            '&vznik=' . $this->database->stringToTimestamp($rowData[0]) .
            '"' .
            (RonPortalSettings::getIntValue('otevreni_detailu') ? ' target="_blank"' : '') .
            '>' .
            TextManager::_('detail') .
            '</a>';
    }
}


class PlatnostProhlidkyBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datum-absolvovani'] == '') {
            return '';
        }
        if ($this->database->stringToTimestamp($rowData['col-platnostdo']) < DateUtils::today()) {
            return TextManager::_('lekarska_prohlidka_propadla');
        }

        return DBUtils::getFormatedValue($this->database, 'date', $rowData['col-platnostdo']);
    }
}

/**
 * Class LekarskeProhlidkyBrowser
 */
class LekarskeProhlidkyBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby  */
    var $pravaProhlizet;
    /** @var PravaNaOsoby  */
    var $pravaZapis;
    /** @var PravaNaOsoby  */
    var $pravaAbsolvovat;

    /**
     * LekarskeProhlidkyBrowser constructor.
     * @param Database $database
     * @throws \Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'lekarskeprohlidky');

        $this->pravaProhlizet = Prava::vratPrava('lekarske_prohl');
        $this->pravaZapis = Prava::vratPrava('lekarske_edit');
        $this->pravaAbsolvovat = Prava::vratPrava('lekarske_absolv');

        $this->title = TextManager::_('zdravotni_prohlidky');
        $this->code = 'browserlekarskeprohlidky';
        $this->tableId = 'browserlekarskeprohlidky';

        $this->sqlSelectPart =
            'GenerovaneLekProhlidky.DatumVznikuPovinnosti, (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC )';    //0 - 1

        $this->sqlFromPart =
            'Osoba JOIN GenerovaneLekProhlidky ON (GenerovaneLekProhlidky.RC = Osoba.RC)' .
            ' LEFT OUTER JOIN OsetrujiciLekar ON (GenerovaneLekProhlidky.OsetrujiciLekar = OsetrujiciLekar.Kod)' .
            ' LEFT OUTER JOIN  LekarskaProhlidka ON (GenerovaneLekProhlidky.KodLekarskeProhlidky = LekarskaProhlidka.Kod)' .
            ' LEFT OUTER JOIN  S_DuvodProhlidky ON (GenerovaneLekProhlidky.DuvodProhlidky = S_DuvodProhlidky.Kod )' .
            ' LEFT OUTER JOIN  S_TypLekarskeProhlidky ON  (LekarskaProhlidka.TypProhlidky = S_TypLekarskeProhlidky.Kod )' .
            ' LEFT OUTER JOIN  S_VyhodnoceniProhlidky ON  (GenerovaneLekProhlidky.Vysledek = S_VyhodnoceniProhlidky.Kod )';

        $this->browserUrl = '?menuid=lekarskeprohlidky&';

//        $this ->filterForm = new LekarskeProhlidkyFilterForm($this ->database, $this->pravaProhlizet, $this->pravaAbsolvovat, $this->pravaZapis);
//        $this ->filterForm ->loadData();

        $this->addField(BrowserField::FIELD_TYPE_LINE)
            ->setCode('col-line');

        $this->addField()
            ->setSqlFieldName('Osoba.Prijmeni')
            ->setCode('col-prijmeni') //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno')
            ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC');

        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno')
            ->setCode('col-jmeno') //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setHeaderText('Jmeno') ->setOrdering(true);


        $this->addField()
            ->setSqlFieldName('GenerovaneLekProhlidky.DatumVznikuPovinnosti', 0)
            ->setCode('col-datum-vzniku')
            ->setHeaderText('datum_povinnosti') ->setFormat('date') ->setAlign('center') ->setOrdering(true);

        //Urci podle ktereho fieldu se bude defaultne radit
        $this->defaultSort = 'col-datum-vzniku';

        // rezervovany cas a datum prohlidky
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('GenerovaneLekProhlidky.CasProhlidkyRezervovany')
            ->setCode('col-cas-rezervace')
            ->setHeaderText('rezervovany_cas') ->setFormat('time') ->setAlign('center') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('GenerovaneLekProhlidky.DatumProhlidkyRezervovany')
            ->setCode('col-datum-rezervace')
            ->setHeaderText('rezervovany_datum') ->setFormat('date') ->setAlign('center') ->setOrdering(true)
            ->setSqlFieldIndex2('@col-cas-rezervace', ' ')
            ->setSortSQL('GenerovaneLekProhlidky.DatumProhlidkyRezervovany, GenerovaneLekProhlidky.CasProhlidkyRezervovany')
            ->setSortSQLDesc('GenerovaneLekProhlidky.DatumProhlidkyRezervovany DESC, GenerovaneLekProhlidky.CasProhlidkyRezervovany DESC')
            ->setVisible(RonPortalSettings::getIntValue('lekprohl_rezervovany'));

        //Zobrazeni odkazu Absolvovani nebo pouze datumu
        $this->addField(new DatumAbsolvovaniProhlidkyField($this->database))
            ->setSqlFieldName('GenerovaneLekProhlidky.DatumAbsolvovani')
            ->setHeaderText('datum_absolvovani')
            ->setFormat('date') ->setAlign('center') ->setOrdering(true)
            ->setCode('col-datum-absolvovani');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('GenerovaneLekProhlidky.PlatnostDo')
            ->setCode('col-platnostdo');

        $this->addField(new PlatnostProhlidkyBrowserField($this->database))
            ->setCode('col-platnost-prohlidky')
            ->setHeaderText('lekarska_prolidka_platnost');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('GenerovaneLekProhlidky.Vysledek')    // smallint
            ->setCode('col-vysledek');

        //filtr na Vse/Absolvovana/Povinne
        //if ($this ->filterForm ->absolvovaneNeabsolvovane != 2) {
            $this->addField()
                ->setSqlFieldName('S_VyhodnoceniProhlidky.Popis')
                ->setCode('col-vyhodnoceni')
                ->setHeaderText('vysledek') ->setAlign('center') ->setOrdering(true)
                ->setTranslationPrefix('db_vyhodnoceniprohlidky_', 'col-vysledek');
        //}

        $this->addField(new DetailProhlidkyField($this->database, $this->pravaZapis));

        $this->addField()
            ->setSqlFieldName('GenerovaneLekProhlidky.KodLekarskeProhlidky')
            ->setCode('col-kod')
            ->setHeaderText('Kod') ->setAlign('center') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('LekarskaProhlidka.Popis')
            ->setCode('col-popis')
            ->setHeaderText('popis_zdravotni_prohlidky') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('S_DuvodProhlidky.Popis')
            ->setCode('col-duvod')
            ->setHeaderText('duvod_zdravotni_prohlidky') ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('GenerovaneLekProhlidky.Generovana')
            ->setCode('col-generovana') ->setFormat('anone')
            ->setHeaderText('generovana_zdravotni_prohlidka') ->setAlign('center') ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsetrujiciLekar.JmenoLekare')
            ->setCode('col-jmeno-lekare');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsetrujiciLekar.Prijmeni')
            ->setCode('col-prijmeni-lekare');

        $f = new JmenoPrijmeniBrowserField($this->database, 'col-jmeno-lekare', 'col-prijmeni-lekare');
        $f ->setNejdriveJmeno(true);
        $this->addField($f)
            ->setCode('col-lekar')
            ->setHeaderText('osetrujici_lekar') ->setOrdering(true)
            ->setSortSQL('OsetrujiciLekar.JmenoLekare, OsetrujiciLekar.Prijmeni')
            ->setSortSQLDesc('OsetrujiciLekar.JmenoLekare DESC, OsetrujiciLekar.Prijmeni DESC');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.RC')
            ->setCode('col-rc');

        $this->addField(new  OsobniCisloBrowserField($this->database, 'col-rc'))
            ->setCode('col-osobni-cislo')
            //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setAlign('center');

        // poznamky
        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(GenerovaneLekProhlidky.DoplnujiciTextPozvanky)}')
            ->setCode('col-doplnujicitextpozvanky')
            ->setHeader(TextManager::_('doplnujici_text') . ': ', -1)
            ->setClassName('poznamka doplnujiciText')
            ->setVisible(RonPortalSettings::getIntValue('lekprohl_doplnujici_text'));

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(GenerovaneLekProhlidky.Poznamka)}')
            ->setCode('col-poznamka')
            ->setHeader(TextManager::_('Poznamka') . ': ', -1)
            ->setClassName('poznamka poznamkaLekarskeProhlidky')
            ->setVisible(RonPortalSettings::getIntValue('lekprohl_poznamka'));

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(GenerovaneLekProhlidky.VysledekProhlidky)}')
            ->setCode('col-vysledekprohlidky')
            ->setHeader(TextManager::_('vysledek_prohlidky') . ': ', -1)
            ->setClassName('poznamka vysledekProhlidky')
            ->setVisible(RonPortalSettings::getIntValue('lekprohl_vysledek_prohlidky'));
    }


    function setFilterForm()
    {
        // parent::setFilterForm(); - nevolat
        $this ->filterForm = new RonPortalFilterForm($this ->database, 'lekarskeprohlidky', true);

        $this ->filterForm ->addFilterRozdeleni1();
        $this ->filterForm ->addFilterRozdeleni2();

//        $this ->filterForm ->nastavOsoby(true, [$this ->pravaProhlizet, $this ->pravaAbsolvovat, $this ->pravaZapis], time(), time());
//        $this ->filterForm ->povolitVsechyOsoby = true;
        $this ->filterForm ->addFilterPP(true, [$this ->pravaProhlizet, $this ->pravaAbsolvovat, $this ->pravaZapis]);

        // filtr typ prohlidky
        $filtrTyp = new FilterFormFilter('typprohlidky', FilterFormFilter::TYPE_OPTIONS);
        $filtrTyp
            ->setCaption(TextManager::_('typ_prohlidky'))
            ->setRowId('filterrow_typprohlidky')
            ->setSessionName('filterform_typ_lekarske_prohlidky')
            ->setOptionsSql(
                'SELECT Kod, Popis FROM S_TypLekarskeProhlidky WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('S_TypLekarskeProhlidky.Kod = ');
        $this ->filterForm ->addFilter($filtrTyp);

        // filtr prohlidka
        $filtrProhlidka = new FilterFormFilter('vyberprohlidka', FilterFormFilter::TYPE_OPTIONS);
        $filtrProhlidka
            ->setCaption(TextManager::_('Zdravotni_prohlidka'))
            ->setRowId('filterrow_prohlidka')
            ->setSessionName('filterform_lekarska_prohlidka')
            ->setOptionsSql(
                'SELECT Kod, Popis FROM LekarskaProhlidka WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1',
                'Popis'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('LekarskaProhlidka.Kod = ');
        $this ->filterForm ->addFilter($filtrProhlidka);

        // filtr vysledek prohlidky
        $filtrVysledek = new FilterFormFilter('vysledekprohlidky', FilterFormFilter::TYPE_OPTIONS);
        $filtrVysledek
            ->setCaption(TextManager::_('vysledek'))
            ->setRowId('filterrow_vysledekprohlidky')
            ->setSessionName('filterform_vysledek_prohlikdy')
            ->setOptionsSql(
                'SELECT Kod, Popis FROM S_VyhodnoceniProhlidky  WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1',
                'Kod'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('S_VyhodnoceniProhlidky.Kod = ');
        $this ->filterForm ->addFilter($filtrVysledek);

        $this->filterForm->addFilter((new FilterFormFilter('jenPosledniZaznam', FilterFormFilter::TYPE_CHECKBOX))
            ->setSessionName('filterform_prohlidky_poslednizaznam')
            ->setRowId('filterrow_jen_posledni_prohlidka')
            ->setCaption2(TextManager::_('prohlidky_jen_posledni_zaznam'))
        );

        $params = $this->database->createParameters();

        // vse/absolvovane/neabsolvovane
        $filtrAbsolvovana = new FilterFormFilter('absolvovane');
        $filtrAbsolvovana
            ->setCaption(TextManager::_('zobraz_zdravotnicke_prohlidky'))
            ->setSessionName('filterform_prohlidky_absolvovane')
            ->addItem('0', TextManager::_('Vse'))
            ->addItem( (new FilterFormFilterItem('1', TextManager::_('absolvovana')))
                ->setSqlFilter('GenerovaneLekProhlidky.DatumAbsolvovani IS NOT NULL')
            )
            ->addItem((new FilterFormFilterItem('2', TextManager::_('neabsolvovana')))
                ->setSqlFilter('GenerovaneLekProhlidky.DatumAbsolvovani IS NULL')
            )
            ->addItem((new FilterFormFilterItem('3', TextManager::_('lekarska_prohlidka_propadla')))
                ->setSqlFilter(new SqlStatement('GenerovaneLekProhlidky.PlatnostDo < ' . $params->addDateParameter(DateUtils::today()), $params))
            )
            ->setDefaultValue('0');

        $this ->filterForm ->addFilter($filtrAbsolvovana);
    }


    function renderFilter()
    {
        parent::renderFilter(); // TODO: Change the autogenerated stub

        if (Prava::vratPrava('lekarske_edit')->jsouNejakePrava()) {
            echo
                '<DIV class="margin-top-2">' .
                '<a href="?menuid=lekarskeprohlidky&formaction=new">' .
                TextManager::_('zadat_zdravotni_prohlidku') .
                '</a>' .
                "</DIV>\n";
        }
    }


    public function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $datum = DateUtils::today();

        if ($this ->filterForm ->getFilterValue('jenPosledniZaznam')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' GenerovaneLekProhlidky.DatumVznikuPovinnosti = (Select Max(DatumVznikuPovinnosti) FROM GenerovaneLekProhlidky GLP WHERE GLP.KodLekarskeProhlidky = LekarskaProhlidka.Kod AND GLP.RC = GenerovaneLekProhlidky.RC' .
                (($this->filterForm->getFilterValue('absolvovane') == 1)
                    ? ' AND GLP.DatumAbsolvovani IS NOT NULL'
                    : ''
                ) .
                (($this->filterForm->getFilterValue('absolvovane') == 2)
                    ? ' AND GLP.DatumAbsolvovani IS NULL'
                    : ''
                ) .
                (($this->filterForm->getFilterValue('absolvovane') == 3)
                    ? ' AND GLP.PlatnostDo < ' . $params->addDateParameter(time())
                    : ''
                ) .
                ")\n";
        }

        //Overeni prav
        $result .=
            ($result ? ' AND ' : '') .
            "EXISTS(SELECT PP.RC FROM PracovniPomer PP " .
            "WHERE (PP.RC = Osoba.RC) " .
            ' AND ((' . $this->pravaProhlizet->sqlFiltr('PP', $datum, $datum, $params) . ')' .
            '       OR (' . $this->pravaZapis->sqlFiltr('PP', $datum, $datum, $params) . ')' .
            '       OR (' . $this->pravaAbsolvovat->sqlFiltr('PP', $datum, $datum, $params) . ')' .
            '     )' .
            " AND (PP.DatumNastupu <= " . $params->addDateParameter($datum) . ")" .
            " AND ((PP.DatumUkonceni >= " . $params->addDateParameter($datum) . ") OR (PP.DatumUkonceni IS NULL))";

        if (null !== $this->filterForm->getFilterValue('osoba')) {
            $result .= ' AND PP.RC = ' . $params->addParameter($this->filterForm->getFilterValue('osoba', 'rc'));
        } else {
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                $sqlRozdeleni = $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'PP', $params);

                if ($sqlRozdeleni) {
                    $result .= ' AND ' . $sqlRozdeleni;
                }
            }
        }

        $result .=
            ')';

        return new SqlStatement($result, $params);
    }
}
