<?php

namespace RonPortal\Personalistika;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\Http\HtmlHeaders;
use Ron\MVC\BaseViewObject;
use Ron\MVC\HttpController;
use Ron\MVC\Redirect;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\User;
use RonPortal\Prava;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalSettings;
use RonPortal\XML\RonPortalXMLForm;
use RonPortal\XML\XMLFormViewObject;


class PersonalniProcesyController extends HttpController
{
    private $database;

    private $procesID;
    /** @var PracovniPomer */
    private $pracovniPomer;

    private $pravaProhlizeni;
    private $pravaZapisu;

    public function __construct(Database $database)
    {
        $this->database = $database;
        $this->pravaProhlizeni = Prava::vratPrava('pers_procesy_prohl');
        $this->pravaZapisu = Prava::vratPrava('pers_procesy_vlozeni');
        TextManager::getInstance()->loadStrings('person.ini');
    }

    /**
     * @param Parameters $params
     * @return string
     */
    public static function getSQLFilterProcesy(Parameters $params)
    {//TODO: viditelnost pro zaměstnance u řádku pers. proceus???
        return " (EXISTS(SELECT IDprocesu 
                  FROM RadekPersProcesu 
                    JOIN RadekSablonyPersProcesu ON (RadekSablonyPersProcesu.KodTypuProcesu = RadekPersProcesu.KodTypuProcesu AND RadekSablonyPersProcesu.Kod = RadekPersProcesu.KodRadkuSablony)
                  WHERE RadekPersProcesu.IDprocesu = PersProces.ID
                    AND ((RadekPersProcesu.VyrizujeRC = " . $params->addParameter(User::getRC()) . ")" .
        //primy nadrizeny
        "            OR (RadekSablonyPersProcesu.TypVyhodnoceni = 1 AND (EXISTS(SELECT RozdeleniOsob.RC FROM RozdeleniOsob 
                                                                                JOIN Rozdeleni ON (Rozdeleni.RozdeleniRodice = RozdeleniOsob.KodTypuRozdeleni AND Rozdeleni.Rodic = RozdeleniOsob.Rozdeleni)
                                                                                JOIN RozdeleniOsob RO_Zam ON (RO_Zam.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni AND RO_Zam.Rozdeleni = Rozdeleni.Kod) 
                                                                                WHERE RO_Zam.KodTypuRozdeleni = TypPersProcesu.KodTypuRozdeleni
                                                                                  AND RO_Zam.RC = PersProces.RC
                                                                                  AND RO_Zam.OsobniCislo = PersProces.OsobniCislo
                                                                                  AND RO_Zam.DatumNastupu = PersProces.DatumNastupu
                                                                                  AND RO_Zam.PlatnostOd <= PersProces.DatumUkonceniPozadovany
                                                                                  AND (RO_Zam.PlatnostDo >= PersProces.DatumPocatku OR RO_Zam.PlatnostDo IS NULL)
                                                                                  AND RozdeleniOsob.RC = " . $params->addParameter(User::getRC()) .
        "                                                                         AND RozdeleniOsob.PlatnostOd <= PersProces.DatumUkonceniPozadovany 
                                                                                  AND (RozdeleniOsob.PlatnostDo >= PersProces.DatumPocatku OR RozdeleniOsob.PlatnostDo IS NULL)" .
        ")))" .
        //uzivatel na vybranem rozdeleni
        "            OR (RadekSablonyPersProcesu.TypVyhodnoceni = 2 AND (EXISTS(SELECT RC FROM RozdeleniOsob
                                                                                WHERE RozdeleniOsob.KodTypuRozdeleni = RadekSablonyPersProcesu.ZodpTypRozdeleni
                                                                                  AND RozdeleniOsob.Rozdeleni = RadekSablonyPersProcesu.ZodpRozdeleni
                                                                                  AND RozdeleniOsob.RC = " . $params->addParameter(User::getRC()) .
        "                                                                         AND RozdeleniOsob.PlatnostOd <= PersProces.DatumUkonceniPozadovany 
                                                                                  AND (RozdeleniOsob.PlatnostDo >= PersProces.DatumPocatku OR RozdeleniOsob.PlatnostDo IS NULL)" .
        ")))" .
        //vybrana osoba
        "            OR (RadekSablonyPersProcesu.TypVyhodnoceni = 3 AND (RadekSablonyPersProcesu.ZodpRC LIKE " . $params->addParameter("%" . User::getRC() . "%") . ")))" .
        " ))";
    }

    /**
     * Na proces ma pravo, pokud ma u prava nastaven priznak na zobrazeni vsech procesu, nebo se ma vyjadrit k nekteremu z radku procesu.
     * @return bool
     * @throws DBException
     */
    public function jePravoNaProces()
    {
        if ($this->pravaProhlizeni->priznak || $this->pravaZapisu->priznak) {
            return true;
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT ID FROM PersProces JOIN TypPersProcesu ON PersProces.KodTypuProcesu = TypPersProcesu.Kod WHERE PersProces.ID = " . $params->addParameter($this->procesID) .
            " AND " . self::getSQLFilterProcesy($params);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if (!$row) {
            return false;
        }

        return true;
    }

    /**
     * @return RonPortalXMLForm
     * @throws Exception
     */
    public function createXMLForm()
    {
        $xmlform = new RonPortalXMLForm($this ->database);
        $xmlform->showHeader = false;

        $xmlform->setVariable('Proces_Id', (int)$this->procesID);

        $xmlform->pp_RC = $this->pracovniPomer->rc;
        $xmlform->osobniCislo = $this->pracovniPomer->osobniCislo;
        $xmlform->datumNastupu = $this->pracovniPomer->datumNastupu;

        return $xmlform;
    }

    /**
     * @return int ID XML sablony
     * @throws DBException
     */
    public function nactiProces()
    {
        $params = $this->database->createParameters();
        $sql =
            "SELECT ID, RC, OsobniCislo, DatumNastupu, TypPersProcesu.IDxmlSablony 
            FROM PersProces JOIN TypPersProcesu ON PersProces.KodTypuProcesu = TypPersProcesu.Kod 
            WHERE ID = " . $params->addParameter($this->procesID);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            $this->pracovniPomer = new PracovniPomer();
            $this->pracovniPomer->nactiPPPodleRC($this->database, $row[1], $row[2], $this->database->stringToTimestamp($row[3]));
        }

        return $row[4];
    }

    /**
     * @param int $idProcesu
     * @return Redirect|XMLFormViewObject|string
     * @throws NeniPravoException
     * @throws Exception
     */
    public function processDetail($idProcesu)
    {
        $this->procesID = $idProcesu;

        if (!$this->jePravoNaProces()) {
            throw new NeniPravoException();
        }

        $idSablony = $_GET['record'];

        $idXMLsablony = $this->nactiProces();

        if (!$idXMLsablony) {
            throw new Exception(TextManager::_('Pers_procesy_chybi_sablona'));
        }

        if ($idXMLsablony != $idSablony) {
            throw new NeniPravoException();
        }

        $xmlForm = $this->createXMLForm();
        $xmlForm ->nactiOdeslanyFormular(); // nacte xml text
        $xmlForm ->loadXML(); // vytvori elementy
        $xmlForm ->process(); // zpracuje formular

        if ($xmlForm->isError()) {
            return XMLFormViewObject::showXmlForm($xmlForm);
        }

        if ($xmlForm->redirect) {
            return $xmlForm ->redirect;
        }

        return new Redirect(null, '?menuid=personalniprocesy&record=' . $this->procesID);
    }

    /**
     * @param int $idProcesu
     * @return BaseViewObject|XMLFormViewObject|string
     * @throws NeniPravoException
     * @throws Exception
     */
    public function showDetail($idProcesu)
    {
        $this->procesID = $idProcesu;

        if (!$this->jePravoNaProces()) {
            throw new NeniPravoException();
        }

        $idXMLsablony = $this->nactiProces();

        if (!$idXMLsablony) {
            throw new Exception(TextManager::_('Pers_procesy_chybi_sablona'));
        }

        $xml = $this->createXMLForm();

        $xml->nactiXMLFormular($idXMLsablony, true);

        $xml->formAction = 'personalniprocesy';
        $xml->hiddenFields['idprocesu'] = $idProcesu;

        return XMLFormViewObject::showXmlForm($xml);
    }

    /**
     * @param string $action
     * @return PersonalniProcesyBrowser|XMLFormViewObject|string
     * @throws DBException
     * @throws NeniPravoException
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue('povolitpersonalniprocesy')) {
            throw new NeniPravoException();
        }

        if (!($this->pravaProhlizeni->jsouNejakePrava() || $this->pravaZapisu->jsouNejakePrava())) {
            throw new NeniPravoException();
        }

        HtmlHeaders::getInstance() ->setAppTitle(TextManager::_('Pers_procesy'));

        if (isset($_GET['record'])) {
            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                return $this->processDetail($_POST['idprocesu']);
            } else {
                return $this->showDetail($_GET['record']);
            }
        }

        $b = new PersonalniProcesyBrowser($this->database, $this->pravaProhlizeni, $this->pravaZapisu);
        $b->load();
        return $b;
    }

}