<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 13:21
 */
namespace RonPortal\PokladniKniha;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\Controls\FormField;
use RonPortal\Controls\OsobaFormField;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\Controls\RonPortalForm;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\Controls\UserLoginFormField;
use Ron\StringUtils;
use RonPortal\TypRozdeleni;
use function Ron\ronText;


class CastkaFormField extends FormField
{
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo '<span id="castka_pokladni_knihy">' . $value . '</span>';
    }
}

class PokladniKnihaForm extends RonPortalForm
{
    public $id;

    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;
//    private $idPP;

    /**
     * PokladniKnihaForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param SeznamPolozekPokladniKnihy $browser
     * @param bool $existujiRadky
     * @throws Exception
     */
    public function __construct($database, $formAction, $browser, $existujiRadky = false)
    {
        parent::__construct($database, $formAction);

        $this->seznamPolozekPokladniKnihy = $browser;

        $this->code = 'pokl_kniha_' . $browser->cislo;

        $this ->htmlAction = '?menuid=pokladnikniha';
        $this ->stornoAction = '?menuid=pokladnikniha&browser='.$this->seznamPolozekPokladniKnihy->cislo;

        $this ->title = $formAction == 'new' ? ronText('New_record') //. ' ' . \Ron\ronText('pokladni_kniha' . ($browser->cislo != 1 ? "_$browser->cislo" : ''))
            : ronText('PoklK_editace_zaznamu' . ($browser->cislo != 1 ? "_$browser->cislo" : ''));

        $this ->sqlTableName = 'PokladniKniha';
        $this ->dbLogSrcDescription = StringUtils::fromUTF8('Pokladní kniha');

//        $this->idPP = $idPP;

        $this->sqlSelectPart =
            'PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, PokladniKniha.Castka, ' . //0 - 5
            'PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, ' . //6 - 9
            'PokladniKniha.IDDokumentu, PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, ' . //10 - 14
            'PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, ' . //15 - 18
            'PokladniKniha.ObdobiVyuctovani, PokladniKniha.Insert_UserName, PokladniKniha.Insert_Timestamp, PokladniKniha.Update_UserName, PokladniKniha.Update_Timestamp, ' .  //19 - 23
            'PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH, PokladniKniha.KodTypuRozdeleni, PokladniKniha.Rozdeleni, PokladniKniha.KodTypuRozdeleni2, PokladniKniha.Rozdeleni2, ' . //24 - 29
            'PokladniKniha.KodTypuRozdeleni3, PokladniKniha.Rozdeleni3, PokladniKniha.ZpusobPlatby, PokladniKniha.KodTypuRozdeleni4, PokladniKniha.Rozdeleni4, ' . //30 - 34
            'T1, T2, T3, T4, T5, I1, I2, I3, F1, F2, F3'; //35 - 45

        $this->addField(FormField::FIELD_TYPE_ID)
            ->setTableAlias('PokladniKniha')
            ->setFieldName('ID')->setPrimary(true)->setSqlFieldIndex(0)->setCode('field-id');

        $this->addField(new OsobaFormField('pkniha_vloz', time()))
            ->setCode('field-osoba')
            ->setSqlFieldName('Id', 'PracovniPomer')->setCaption('Osoba')
            ->setRowId('poklk_row_osoba')->setFieldId('Osoba')
            ->setRequired(true);;

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setCaption('datum_pohybu')->setRequired(true)->setSqlFieldIndex(1)->setSqlFieldName('DatumPohybu')
            ->setFieldName('DatumPohybu')->setMaxLength(10)->setDefaultValue(StringUtils::format('date',DateUtils::startOfTheDay(time())))
            ->setCode('field-datum');

        $defaultTypPolozky = DBUtils::getSqlValue($this->database, 'SELECT TypPolozkyPokladniKnihy.Kod FROM TypPolozkyPokladniKnihy ' .
            ' WHERE ('. SQLUtils::getINCondition($this->database, 'TypPolozkyPokladniKnihy.Kod', $this->seznamPolozekPokladniKnihy->seznamTypuSQL) . ')' .
            ' ORDER BY Popis');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setCaption('typ_polozky')->setRequired(true)->setRowId('typ_polozky')->setSqlFieldName('TypPolozky')
            ->setFieldName('TypPolozky')->setSqlFieldIndex(7)->setDefaultValue($defaultTypPolozky)
            ->setSelectSql('SELECT TypPolozkyPokladniKnihy.Kod, TypPolozkyPokladniKnihy.Popis FROM TypPolozkyPokladniKnihy ' .
                ' WHERE ('. SQLUtils::getINCondition($this->database, 'TypPolozkyPokladniKnihy.Kod', $this->seznamPolozekPokladniKnihy->seznamTypuSQL) . ' AND %FILTER% )' .
                ' ORDER BY %ORDER%',
                "1 = 1",
                'Popis')
            ->setCode('field-typ-polozky')->setTranslationPrefix('db_typpolozkypokladniknihy_popis_');

        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, RonPortalSettings::getIntValue('typrozdelenipoklk'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 27);
            }
        }

        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk2'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 29, '2');
            }
        }
        
        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk3'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 31, '3');
            }
        }
        
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk4'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 31, '4');
            }
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption('castkaBezDPH')->setRequired(false)->setRowId('row_castka_bez_dph')->setSqlFieldName('CastkaBezDPH')
                ->setFieldName('CastkaBezDPH')->setSqlFieldIndex(24)->setCode('field-castka-bez-dph')->setReadOnly($existujiRadky);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption('castkaDPH')->setRequired(false)->setRowId('row_castka_dph')->setSqlFieldName('CastkaDPH')
                ->setFieldName('CastkaDPH')->setSqlFieldIndex(25)->setCode('field-castka-dph')->setReadOnly($existujiRadky);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastku()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption('castka')->setRequired(true)->setSqlFieldName('Castka')->setSqlFieldIndex(5)
                ->setFieldName('castka')->setCode('field-castka')->setReadOnly($existujiRadky);
        }
        
        //needitovatelne pole s celkovu castkou - defaultne skryte, zobrazi se misto editovatelne castky (pokud nelze editovat)
        $this->addField(new CastkaFormField(FormField::FIELD_TYPE_FLOAT))
            ->setCaption('castka')->setRowId('poklkniha_castka_noedit')->setRequired(false)->setReadOnly()->setRowClass('d-none ' . $this->defaultRowClass ?: '')
            ->setCode('field-castka-text');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setCaption('mena')->setRequired(true)->setSqlFieldName('Mena')->setFieldName('Mena')->setSqlFieldIndex(6)
            ->setSelectSql('SELECT Mena.KodMeny_ISO, Mena.KodISO FROM Mena WHERE (Mena.Aktivni = 1 AND %FILTER% ) ORDER BY %ORDER% ',
                "1 = 1",
                'KodISO')
            ->setDefaultValue(RonPortalSettings::getValue('poklk_vychozi_mena'))
            ->setCode('field-mena');

        $params = $this->database->createParameters();
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setCaption('PoklK_zpusobplatby')->setRequired(true)->setSqlFieldName('ZpusobPlatby')->setFieldName('ZpusobPlatby')->setSqlFieldIndex(32)
            ->setSelectSql(new SqlStatement('SELECT ZpusobPlatbyPK.Kod, ZpusobPlatbyPK.Popis FROM ZpusobPlatbyPK WHERE RonPortalBrowser = ' . $params->addParameter($browser->cislo) .
                ($this->seznamPolozekPokladniKnihy->cislo == 1 ? ' OR RonPortalBrowser IS NULL ' : '') . ' AND (%FILTER%) ' .
                ' ORDER BY %ORDER%', $params), '1 = 1', 'Popis')
            ->setCode('field-zpusobplatby')->setTranslationPrefix('db_zpusobplatbypk_popis_');

        $this->addField()
            ->setCaption('popis' . ($browser->cislo != 1 ? "_$browser->cislo" : ''))->setRequired(true)->setSqlFieldName('Popis')->setFieldName('Popis')->setSqlFieldIndex(8)
            ->setCode('field-popis');

        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t1Popis != '') {
            $this->addField()
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t1Popis))->setSqlFieldName('T1')->setFieldName('T1')->setSqlFieldIndex(35)
                ->setCode('field-t1');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t2Popis != '') {
            $this->addField()
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t2Popis))->setSqlFieldName('T2')->setFieldName('T2')->setSqlFieldIndex(36)
                ->setCode('field-t2');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t3Popis != '') {
            $this->addField()
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t3Popis))->setSqlFieldName('T3')->setFieldName('T3')->setSqlFieldIndex(37)
                ->setCode('field-t3');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t4Popis != '') {
            $this->addField()
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t4Popis))->setSqlFieldName('T4')->setFieldName('T4')->setSqlFieldIndex(38)
                ->setCode('field-t4');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t5Popis != '') {
            $this->addField()
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->t5Popis))->setSqlFieldName('T5')->setFieldName('T5')->setSqlFieldIndex(39)
                ->setCode('field-t5');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->i1Popis != '') {
            $this->addField(FormField::FIELD_TYPE_INT)
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->i1Popis))->setSqlFieldName('I1')->setFieldName('I1')->setSqlFieldIndex(40)
                ->setCode('field-i1');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->i2Popis != '') {
            $this->addField(FormField::FIELD_TYPE_INT)
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->i2Popis))->setSqlFieldName('I2')->setFieldName('I2')->setSqlFieldIndex(41)
                ->setCode('field-i2');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->i3Popis != '') {
            $this->addField(FormField::FIELD_TYPE_INT)
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->i3Popis))->setSqlFieldName('I3')->setFieldName('I3')->setSqlFieldIndex(42)
                ->setCode('field-i3');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->f1Popis != '') {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->f1Popis))->setSqlFieldName('F1')->setFieldName('F1')->setSqlFieldIndex(43)
                ->setCode('field-f1');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->f2Popis != '') {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->f2Popis))->setSqlFieldName('F2')->setFieldName('F2')->setSqlFieldIndex(44)
                ->setCode('field-f2');
        }
        if ($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->f3Popis != '') {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption(StringUtils::htmlTextConvert($this->seznamPolozekPokladniKnihy->vratTypPolozky($defaultTypPolozky)->f3Popis))->setSqlFieldName('F3')->setFieldName('F3')->setSqlFieldIndex(45)
                ->setCode('field-f3');
        }


        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka')->setSqlFieldName('Poznamka')->setSqlFieldIndex(9) ->setCaption('Poznamka')
            ->setCode('field-poznamka');

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('PokladniKniha.Update_Timestamp')->setFieldName('Update_Timestamp');
        $this->addField(new UserLoginFormField(true, 'PokladniKniha.Update_UserName'));

        $this->addField( FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('PokladniKniha.Insert_Timestamp')->setFieldName('Insert_Timestamp')->setReadOnly(true);
        $this->addField(new UserLoginFormField(false,'PokladniKniha.Insert_UserName'));
    }

    /**
     * @param TypRozdeleni $typRozdeleni
     * @param int $indexRozdeleni
     * @param string $poradiRozdeleni
     * @throws DBException
     */
    private function pridejRozdeleni($typRozdeleni, $indexRozdeleni, $poradiRozdeleni = '')
    {
        $this->addField(
            new RozdeleniFormField($typRozdeleni->kod, RonPortalSettings::getIntValue("rozdelenipoklk{$poradiRozdeleni}_pouzevlastni")))
            ->setFieldName("Rozdeleni{$poradiRozdeleni}")->setSqlFieldIndex($indexRozdeleni)->setTypRozdeleniFieldName("KodTypuRozdeleni{$poradiRozdeleni}")
            ->setCode("field-rozdeleni{$poradiRozdeleni}")
            ->setCaption(StringUtils::htmlTextConvert($typRozdeleni ->popis))->setShowEmptyValue(true)
            ->setFieldId("poklk_rozdeleni{$poradiRozdeleni}");
    }

    /**
     * @return string
     */
    public function getSqlFromPart()
    {
        return
            'PokladniKniha JOIN PracovniPomer' .
            ' ON PracovniPomer.RC = PokladniKniha.RC  AND  PracovniPomer.OsobniCislo = PokladniKniha.OsobniCislo  AND  PracovniPomer.DatumNastupu = PokladniKniha.DatumNastupu';
    }

    function createDBRecordFields($rec, $insert)
    {
        $pracPomer = PracovniPomer::nactiPP($this->database, (int)$_POST['osoba']);

        $rec->addField('RC','',true,15, $pracPomer->rc);
        $rec->addField('OsobniCislo','',true,15, $pracPomer->osobniCislo);
        $rec->addField('DatumNastupu','timestamp',true,0, $pracPomer->datumNastupu);

        parent::createDBRecordFields($rec, $insert);
    }


    /** Pokud je typ pohybu vydej, nastavi castku na zapornou.
     * @param RonPortalDBRecord $rec
     * @return RonPortalDBRecord
     * @throws DBException
     */
    private function nastavPodleTypuPohybu($rec)
    {
        $params = $this->database->createParameters();
        $jeVydej = (int)DBUtils::getSqlValue($this->database, "SELECT DruhPohybu FROM TypPolozkyPokladniKnihy WHERE Kod = " . $params->addParameter($rec['TypPolozky']), $params) == 2;
        //pro vydej bude castka zaporna
        if ($jeVydej) {
            $rec['Castka'] = - abs($rec['Castka']);
        }
        else {
            $rec['Castka'] = abs($rec['Castka']);
        }

        return $rec;
    }

    /** Overi, zda jsou zadana pole, ktera jsou u typu polozky povolena a pokud je povolena castka bez DPH a castka DPH secte je do pole castka.
     * @param RonPortalDBRecord $rec
     * @return RonPortalDBRecord
     * @throws Exception
     */
    private function overVypoctiPodleTypuPohybu($rec)
    {
        $typPolozky = $this->seznamPolozekPokladniKnihy->vratTypPolozky($rec['TypPolozky']);
        
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && ($rec['CastkaBezDPH'] == '')) {
            throw new Exception(ronText('Nezadana_castka_bez_dph'));
        }

        if ($typPolozky->lzeZobrazitCastkuDPH() && ($rec['CastkaDPH'] == '')) { 
            throw new Exception(ronText('Nezadana_castka_dph'));
        }

        //zadava se castka bez dph a celkova castka, castka dph se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && !$typPolozky->lzeZobrazitCastkuDPH()) {
            if ($rec['CastkaBezDPH'] > $rec['Castka']) {
                throw new Exception(ronText('Castka_bez_dph_vyssi'));
            }
            $rec['CastkaDPH'] = $rec['Castka'] - $rec['CastkaBezDPH'];
        }

        //zadava se castka dph a celkova castka, castka bez dph se dopocte
        if (!$typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            if ($rec['CastkaDPH'] > $rec['Castka']) {
                throw new Exception(ronText('Castka_dph_vyssi'));
            }
            $rec['CastkaBezDPH'] = $rec['Castka'] - $rec['CastkaDPH'];
        }

        //zadava se castka bez dph a castka dph, celkova castka se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            $rec['Castka'] = $rec['CastkaBezDPH'] + $rec['CastkaDPH'];
        }
        
        return $rec;
    }


    /**
     * @param RonPortalDBRecord $rec
     */
    private function zkontrolujPravoNaTyp($rec)
    {
        $prava = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $rec->findFieldByName('TypPolozky')->value);

        if (!$prava->jsouNejakePrava())
            throw new NeniPravoException();
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function beforeInsertRecord($rec)
    {
        $this->zkontrolujPravoNaTyp($rec);

        $rec = $this->overVypoctiPodleTypuPohybu($rec);
        
        $rec = $this->nastavPodleTypuPohybu($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_BEFORE_POST,'insert',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function beforeUpdateRecord($rec)
    {
        $this->zkontrolujPravoNaTyp($rec);

        $rec = $this->overVypoctiPodleTypuPohybu($rec);
        
        $rec = $this->nastavPodleTypuPohybu($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_BEFORE_POST,'update',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    public function afterInsertRecord($rec)
    {
        $this->id = $rec['ID'];
        
        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_AFTER_POST,'insert',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    public function afterUpdateRecord($rec)
    {
        $this->id = $rec['ID'];

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_AFTER_POST,'update',null,false);
    }

    public function renderCommonFields()
    {
        parent::renderCommonFields();
        
        echo '<input type="hidden" name="browser" value="' . $this->seznamPolozekPokladniKnihy->cislo . '" />';
        echo "<input type='hidden' name='typy' id='seznamtypu' value='" . json_encode($this->seznamPolozekPokladniKnihy->jsonObject) . "' />";
    }

}