<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 8.6.2016
 * Time: 12:43
 */

namespace RonPortal\PokladniKniha;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use RonPortal\Security;
use RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

class PokladniKnihaRecord extends RonPortalDBRecord
{
    /** @var int */
    public $id;

    /** @var  int */
    public $idpp; // id PP pro kontrolu prav

    /** @var DBRecordField */
    public $datumpohybufield;
    /** @var DBRecordField */
    public $rcfield;

    /** @var DBRecordField */
    public $typPolozkyfield;

    /** @var DBRecordField */
    public $schvaleni1;
    /** @var DBRecordField */
    public $schvaleni2;
    /** @var DBRecordField */
    public $schvaleni3;
    /** @var DBRecordField */
    public $schvaleni4;

    /** @var DBRecordField */
    public $historiefield; // pole historie

    public $castka;
    public $castkaBezDPH;
    public $castkaDPH;

    /** @var int cislo browseru pro kontrolu prav */
    public $cisloBrowseru;


    public function __construct($database, $id)
    {
        parent::__construct($database, 'PokladniKniha');
        $this->id = (int)$id;

        $this->logDescription = StringUtils::fromUTF8('Pokladní kniha');
        $this->where = 'PokladniKniha.Id = ' . (int)$id;

        // je nutne pridat jmeno tabulky PokladniKniha., protoze se pripojuje jeste PracovniPomer
        $this->addField('PokladniKniha.Id', 'int', true); // readonly

        $this->rcfield = $this->addField('PokladniKniha.RC', '', true); // readonly
        $this->addField('PokladniKniha.OsobniCislo', '', true);
        $this->addField('PokladniKniha.DatumNastupu', 'date', true);

        $this->datumpohybufield = $this->addField('PokladniKniha.DatumPohybu', 'date', true);

        $this->typPolozkyfield = $this->addField('PokladniKniha.TypPolozky','string',true);

        $this->addField('PokladniKniha.Popis', '', true);

        $this->castka = $this->addField('PokladniKniha.Castka', 'float', true);
        $this->castkaBezDPH = $this->addField('PokladniKniha.CastkaBezDPH', 'float', true);
        $this->castkaDPH = $this->addField('PokladniKniha.CastkaDPH', 'float', true);

        $this->schvaleni1 = $this->addField('PokladniKniha.Schvaleni1', 'datetime', true);
        $this->schvaleni2 = $this->addField('PokladniKniha.Schvaleni2', 'datetime', true);
        $this->schvaleni3 = $this->addField('PokladniKniha.Schvaleni3', 'datetime', true);
        $this->schvaleni4 = $this->addField('PokladniKniha.Schvaleni4', 'datetime', true);

        $this->addField('PokladniKniha.Poznamka', 'memo', true);

        $this->historiefield = $this->addField('PokladniKniha.HistorieSchvalovani', 'memo', false); // readonly = false
        $this->historiefield->updateLog = true;
    }

    /**
     * @param $row
     * @return bool
     * @throws Exception
     */
    public function loadRecordValues($row)
    {
        if (!$this ->id) {
            throw new Exception(StringUtils::fromUTF8('Chyba - není zadáno ID záznamu pokladní knihy!'));
        }

        parent::loadRecordValues($row);

        $this ->idpp = $row[count($this ->fields)];
        $this ->cisloBrowseru = $row[count($this->fields)+1];

        return true;
    }



    public function getSelectSQL($filtr = '', $targetParams = null, $filtrParams = null)
    {
        $sql =
            'SELECT ' . $this ->getSelectPartSQL() . ', PracovniPomer.Id, TypPolozkyPokladniKnihy.RonPortalBrowser ' . // PP.Id predposledni!
            '  FROM PokladniKniha, PracovniPomer, TypPolozkyPokladniKnihy ' .
            ' WHERE PokladniKniha.RC = PracovniPomer.RC ' .
            '   AND PokladniKniha.OsobniCislo = PracovniPomer.OsobniCislo ' .
            '   AND PokladniKniha.DatumNastupu = PracovniPomer.DatumNastupu ' .
            '   AND TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky' .
            '   AND (' . $this ->where . ')';

        if ($filtr) $sql .= ' AND (' . $filtr . ')';

        return $sql;
    }

    /**
     * @param $pravastr
     * @param bool $zruseni
     * @throws Exception
     */
    private function zkontrolujPrava($pravastr, $zruseni = false)
    {
        $prava = Prava::vratPrava($pravastr, $pravastr . '_' . $this->typPolozkyfield->value);
        if ($zruseni  &&  $prava ->priznak) {
            // je zaskrtnuto "zakazat zrusit schvaleni"
            throw new NeniPravoException();
        }

        Prava::nastavObdobiProPrava($this ->datumpohybufield ->value, $this ->datumpohybufield ->value);

        if (!$prava ->lzeZobrazitPP($this ->idpp)) {
            throw new NeniPravoException();
        }
    }


    /**
     * @param $xmlContext
     * @return bool
     * @throws Exception
     */
    private function xmlEvents($xmlContext)
    {
        return $this ->executeXmlEvents(RonPortalXMLEvents::POKLK_SCHVALENI, $xmlContext, ['pokladnikniha' => $this ->id]);
    }

    /**
     * @param $uroven
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function schval($uroven)
    {
        $pomf = $this ->addField('Schvalil' . $uroven, '');
        $pomf ->updateLog = true;

        for ($i = $uroven; $i <= 4; $i++) {
            $this ->addField('DatumVraceni' . $i, 'timestamp')->setNewValue(null);
        }
        $this ->addField('PoznamkaKVraceni', '')->setNewValue(null);

        if (!$this ->loadRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba - zadaný záznam pokladní knihy nebyl nalezen!'));
        }

        // schvaleni nastavuje az po LoadRecord, aby se zbytecne nevolal update, pokud uz bylo schvaleno
        // (neni to totiz priznak, ale datum)
        switch ($uroven) {
            case '1' :
                if ( !$this ->schvaleni1 ->value) {
                    $this->schvaleni1->setNewValue(time());
                    $pomf->setNewValue(RonPortalUtils::getLogUserName());
                }
                break;

            case '2' :
                if ( !$this ->schvaleni2 ->value) {
                    $this->schvaleni2->setNewValue(time());
                    $pomf->setNewValue(RonPortalUtils::getLogUserName());
                }
                break;

            case '3' :
                if ( !$this ->schvaleni3 ->value) {
                    $this ->schvaleni3 ->setNewValue(time());
                    $pomf->setNewValue(RonPortalUtils::getLogUserName());
                }
                break;

            case '4' :
                if ( !$this ->schvaleni4 ->value) {
                    $this ->schvaleni4 ->setNewValue(time());
                    $pomf->setNewValue(RonPortalUtils::getLogUserName());
                }
                break;

        }

        if (!$this ->isChanged()) return true; // neni co menit

        $this ->zkontrolujPrava("pkniha_schval$uroven");

        $this ->historiefield ->value =  "<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . sprintf(ronText('Schvaleni_poklk_log'), '' . $uroven)
            . "\n" . $this ->historiefield ->originalValue;

        $this ->logText = StringUtils::fromUTF8('Schválení záznamu pokladní knihy');

        if (!$this ->xmlEvents('schvaleni' . $uroven . '_before')) return false;

        if (!$this ->updateRecord()) return false;

        return $this ->xmlEvents('schvaleni' . $uroven . '_after');
    }

    /**
     * @param $uroven
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function zrusSchvaleni($uroven)
    {
        switch ($uroven) {
            case '1' :
                $this ->schvaleni1 ->setNewValue(null);
                break;

            case '2' :
                $this ->schvaleni2 ->setNewValue(null);
                break;

            case '3' :
                $this ->schvaleni3 ->setNewValue(null);
                break;

            case '4' :
                $this ->schvaleni4 ->setNewValue(null);
                break;

        }

        $f = $this ->addField('Schvalil1', '', $uroven != 1);
        $f ->updateLog = true;
        $f = $this ->addField('Schvalil2', '', $uroven != 2);
        $f ->updateLog = true;
        $f = $this ->addField('Schvalil3', '', $uroven != 3);
        $f ->updateLog = true;
        $f = $this ->addField('Schvalil4', '', $uroven != 4);
        $f ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba - zadaný záznam pokladní knihy nebyl nalezen!'));
        }

        if (!$this ->isChanged()) return true; // neni co menit

        $this ->zkontrolujPrava("pkniha_schval$uroven", true);

        if ( ( $uroven <= 3 && $this ->schvaleni4 ->value )
            ||( $uroven <= 2 && $this ->schvaleni3 ->value )
            || ( $uroven == 1 && $this ->schvaleni2 ->value )
        ) {
            throw new Exception(StringUtils::fromUTF8('Záznam pokladní knihy je již schválen vyšší úrovní - schválení nelze zrušit!'));
        }

        $this ->historiefield ->value = "<I>" . date('j.n.Y G:i') .
            ', <B>' . StringUtils::htmlTextConvert(User::getUserName()) . '</B></I>: ' . sprintf(ronText('Odschvaleni_poklk_log'), '' . $uroven)
            . "\n" . $this ->historiefield ->originalValue;

        $this ->logText = StringUtils::fromUTF8('Zrušeno schválení záznamu pokladní knihy');

        if (!$this ->xmlEvents('zrusitschvaleni' . $uroven . '_before')) return false;

        if (!$this ->updateRecord()) return false;

        return $this ->xmlEvents('zrusitschvaleni' . $uroven . '_after');
    }

    /**
     * Zkontroluje, jestli uzivatel muze vratit schvaleni zadane urovne
     * @param int $uroven
     * @return bool
     * @throws Exception
     */
    function lzeVratitZaznam($uroven)
    {
        // vetsina chyb by nemela vubec nastat
        // - nektere pouze v pripade, ze by 2 uzivatele soubezne pracovali se stejnym zaznamem

        if ($this ->schvaleni4 ->value) {
            // jiz je schvalena uroven 4 - nelze vratit
            throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - schvalování záznamu pokladní knihy již bylo dokončeno!'));
        }

        switch ($uroven) {
            case 3 :
                if (!$this ->schvaleni3 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 3 není schválena!'));
                }
                break;
            case 2 :
                if ($this ->schvaleni3 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 3 je již schválena!'));
                }
                if (!$this ->schvaleni2 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 2 není schválena!'));
                }
                break;

            case 1 :
                if ($this ->schvaleni2 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 2 je již schválena!'));
                }

                if (!$this ->schvaleni1 ->value) {
                    throw new Exception(StringUtils::fromUTF8('Chyba - nelze vrátit - úroveň 1 není schválena!'));
                }
                break;

            default:
                throw new Exception(StringUtils::fromUTF8('Chyba - špatně zadaná úroveň schválení při vrácení záznamu pokladní knihy! (' . $uroven . ')'));
        }


        $this ->zkontrolujPrava('pkniha_schval' . ($uroven + 1));

        return true;
    }

    /**
     * @param int    $uroven
     * @param string $poznamka
     * @return bool
     * @throws Exception
     */
    function vratSchvaleni($uroven, $poznamka)
    {
        $this ->addField('DatumVraceni' . $uroven, 'timestamp', false, 0, time());
        $this ->addField('PoznamkaKVraceni', '', false, 100, $poznamka);

        if (!$this ->loadRecord()) {
            throw new Exception(StringUtils::fromUTF8('Chyba při vrácení schválení - zadaný záznam pokladní knihy nebyl nalezen!'));
        }

        if (!$this ->lzeVratitZaznam($uroven)) return false;

        switch ($uroven) {
            case 1 :
                $this ->schvaleni1 ->setNewValue(null);
                break;

            case 2 :
                $this ->schvaleni2 ->setNewValue(null);
                break;
                
            case 3 :
                $this ->schvaleni3 ->setNewValue(null);
                break;

            default :
                throw new Exception(StringUtils::fromUTF8('Chyba při vrácení schválení pokladní knihy - špatně zadaná úroveň schválení!'));
        }

        $this ->historiefield ->setNewValue(
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " . sprintf(ronText('Vraceni_poklk_log'), '' . $uroven) .
            "\n> " . $poznamka . "\n" . $this ->historiefield ->originalValue
        );

        $this ->logText = StringUtils::fromUTF8('Vráceno schválení záznamu pokladní knihy');

        if (!$this ->xmlEvents('vratitschvaleni' . $uroven . '_before')) return false;
        if (!$this ->updateRecord()) return false;
        return $this ->xmlEvents('vratitschvaleni' . $uroven . '_after');
    }

    /**
     * @param $zauctovano
     * @return void
     * @throws Exception
     */
    function nastavZauctovani($zauctovano)
    {
        $this ->addField('Zauctovano', 'int', false, 0, (int)$zauctovano);

        $pomfield = $this ->addField('Zauctovano_Uzivatel', '', false, 0, RonPortalUtils::getLogUserName());
        $pomfield ->updateLog = true;

        $pomfield = $this ->addField('Zauctovano_DatumCas', 'datetime', false, 0, time());
        $pomfield ->updateLog = true;

        if (!$this ->loadRecord()) {
            throw new Exception('Chyba - zadany zaznam pokladni knihy nebyl nalezen!');
        }

        $this ->zkontrolujPrava('pkniha_zauct');

        // zauctovani
        $logstr = ronText($zauctovano ? 'Zauctovani_poklk_log' : 'Zruseno_zauctovani_poklk_log');

        $this->historiefield->value =
            "<I>" . date('j.n.Y G:i') . ", <B>" . StringUtils::htmlTextConvert(User::getUserName()) . "</B></I>: " .
            $logstr . "\n" . $this->historiefield->originalValue;

        $this ->logText = TextManager::_('LOG_nastaveno_zauctovani_poklk');

        if (!$this ->xmlEvents('zauctovani_before')) {
            throw new Exception('Pri nastaveni zauctovani doslo k chybe!');
        }

        if (!$this ->updateRecord()) {
            throw new Exception('Chyba - pri nastaveni zauctovani pokladni knihy doslo k chybe!');
        }

        $this ->xmlEvents('zauctovani_after');
    }

    /**
     * @param Database $database
     * @return bool
     */
    public static function hromadneSchvaleni($database)
    {
        Security::validateCSRFPostParam();

        // uzivatel odeslal hromadne schvaleni zaznamu Pokladni knihy
        $pocet = (int)$_POST['pocet'];

        $result = true;

        for ($i = 0;  $i < $pocet;  $i++) {
            $id = $_POST["idPoklK_$i"];
            $rec = new PokladniKnihaRecord($database, $id);

            try {
                if (isset($_POST["NastavitSchvaleni_{$id}_1"]) && $_POST["NastavitSchvaleni_{$id}_1"]) {
                    if (isset($_POST["Schvaleni_{$id}_1"])) {
                        if (!$rec->schval(1)) $result = false;
                    } else {
                        if (!$rec->zrusSchvaleni(1)) $result = false;
                    }
                }

                if (isset($_POST["NastavitSchvaleni_{$id}_2"]) && $_POST["NastavitSchvaleni_{$id}_2"]) {
                    if (isset($_POST["Schvaleni_{$id}_2"])) {
                        if (!$rec->schval(2)) $result = false;
                    } else {
                        if (!$rec->zrusSchvaleni(2)) $result = false;
                    }
                }

                if (isset($_POST["NastavitSchvaleni_{$id}_3"]) && $_POST["NastavitSchvaleni_{$id}_3"]) {
                    if (isset($_POST["Schvaleni_{$id}_3"])) {
                        if (!$rec->schval(3)) $result = false;
                    } else {
                        if (!$rec->zrusSchvaleni(3)) $result = false;
                    }
                }

                if (isset($_POST["NastavitSchvaleni_{$id}_4"]) && $_POST["NastavitSchvaleni_{$id}_4"]) {
                    if (isset($_POST["Schvaleni_{$id}_4"])) {
                        if (!$rec->schval(4)) $result = false;
                    } else {
                        if (!$rec->zrusSchvaleni(4)) $result = false;
                    }
                }
            } catch (Exception $e) {
                $result = false;
                MessageManager::addErrorMessage($e ->getMessage());
            }
        }

        if (!$result) {
            MessageManager::addErrorMessage(ronText('PoklK_chyba_hromadneho_schvalovani'));
        }

        return $result;
    }

}