<?php

namespace RonPortal\PokladniKniha;

use Ron\Controls\FormField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\DebugException;
use Ron\StringUtils;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Controls\RozdeleniFormField;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RozdeleniOsob;
use RonPortal\TypRozdeleni;
use function Ron\ronText;

class RadekPokladniknihyDialog extends RonPortalForm
{
    /** @var PokladniKniha */
    protected $pokladniKniha;

    /** @var SeznamPolozekPokladniKnihy */
    protected $seznamPolozekPokladniKnihy;

    /**
     * @param Database $database
     * @param PokladniKniha $pokladniKniha
     * @param SeznamPolozekPokladniKnihy $seznamPolozekPokladniKnihy
     * @throws DBException
     * @throws DebugException
     */
    public function __construct($database, $pokladniKniha, $seznamPolozekPokladniKnihy)
    {
        parent::__construct($database, 'new', true);

        $this->pokladniKniha = $pokladniKniha;
        $this->seznamPolozekPokladniKnihy = $seznamPolozekPokladniKnihy;

        $this ->cssClass = 'large-form rondialog ronform form-25-75';
        $this ->code = 'formradekpoklk' . ($seznamPolozekPokladniKnihy->cislo != 1 ? "_$seznamPolozekPokladniKnihy->cislo" : '');

        $this ->stornoAction = "#";

        $this ->htmlParams = array_merge($this ->htmlParams, [
            'id' => 'radek_poklk_dialog',
            'data-ronform-submit-url' => '?menuid=pokladnikniha&action=zapisradek',
            'data-ronform-after-submit' => 'reload',
            'title' => ronText('Radek_pokladni_knihy')
        ]);

        $this ->sqlTableName = 'RadekPokladniKnihy';

//        $this ->setXmlEventsName('SC_RADEK');

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('IDPokladniKnihy') ->setDefaultValue($this->pokladniKniha->vratID()) ->setPrimary(true)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-pokladnikniha_id')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'IDPokladniKnihy']);

        $idParams = $this ->database ->createParameters();
        $this ->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('CisloRadku') ->setPrimary(true)
            ->setIdSqlFilter(new SqlStatement('IDPokladniKnihy = ' . $idParams ->addParameter($this->pokladniKniha->vratID()), $idParams))
            ->addFieldCssClass('ronform-field')
            ->setCode('field-cisloradku')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'CisloRadku']);

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumPohybu') ->setCaption('datum_pohybu')->setRequired(true)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-datum-pohybu')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'DatumPohybu']);


        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('TypPolozky') ->setCaption('typ_polozky')->setRequired(true)
            ->addFieldCssClass('ronform-field ronselect')
            ->setRowId('typ_polozky')
            ->setCode('field-typ-polozky')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'TypPolozky'])
            ->setTranslationPrefix('db_typpolozkypokladniknihy_popis_')
            ->setSelectSql(
                'SELECT Kod, Popis FROM TypPolozkyPokladniKnihy' .
                ' WHERE (' . SQLUtils::getINCondition($this->database, 'TypPolozkyPokladniKnihy.Kod', $this->seznamPolozekPokladniKnihy->seznamTypuSQL) . ' AND %FILTER%)' .
                ' ORDER BY %ORDER%',
                "1 = 1",
                'Popis'
            );

        $this ->addField()
            ->setFieldName('Popis') ->setCaption('popis') ->setMaxLength(250)
            ->addFieldCssClass('ronform-field')
            ->setCode('field-popis')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Popis']);


        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, RonPortalSettings::getIntValue('typrozdelenipoklk'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni);
            }
        }

        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk2'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, '2');
            }
        }

        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk3'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, '3');
            }
        }
        
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk4'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, '4');
            }
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setFieldName('castkaBezDPH')->setCaption('castkaBezDPH')
                ->setFieldCssClass('ronform-field')
                ->setRowId('row_castka_bez_dph')
                ->setCode('field-castka-bez-dph')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'castkaBezDPH']);
        }
        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setFieldName('castkaDPH')->setCaption('castkaDPH')
                ->setFieldCssClass('ronform-field')
                ->setRowId('row_castka_dph')
                ->setCode('field-castka-dph')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'castkaDPH']);
        }
        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastku()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setFieldName('castka')->setCaption('castka')
                ->setFieldCssClass('ronform-field')
                ->setCode('field-castka')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'castka']);
        }

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setCaption('Poznamka')
            ->addFieldCssClass('ronform-field')
            ->setCode('field-poznamka')
            ->setMaxLength(250)
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Poznamka']);

        if (Prava::vratPrava('pkniha_k_ucet') ->lzeZobrazitPP($this->pokladniKniha->idPP)) {
            $this ->addField(FormField::FIELD_TYPE_CHECK)
                ->setFieldName('Zkontrolovano') ->setCaption('PoklK_radek_zkontrolovan')
                ->addFieldCssClass('ronform-field')
                ->setCode('field-zkontrolovano')
                ->setCaptionColumnClass('col-6 col-sm-3 caption') ->setDataColumnClass('col-6 col-sm-2')
                ->setOtherHtmlParams(['data-ronform-field-name' => 'Zkontrolovano']);

            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setFieldName('Zkontroloval')
                ->addFieldCssClass('ronform-field') ->setCaptionColumnClass('d-none') ->setDataColumnClass('col-12 col-sm-7')
                ->setNewRow(false)
                ->setOtherHtmlParams(['data-ronform-field-name' => 'Zkontroloval']);

        }

        $this ->loadConfiguration();
    }

    public function getCsrfRequestId()
    {
        // csrf podle id pokladni knihy
        return $this ->code . '_' . $this ->pokladniKniha ->vratID();
    }

    public function renderMessages()
    {
       //neni nutne opet vypisovat
    }

    /**
     * @param TypRozdeleni $typRozdeleni
     * @param string $poradiRozdeleni
     * @throws DBException
     */
    private function pridejRozdeleni($typRozdeleni, $poradiRozdeleni = '')
    {
        $rozdeleni = null;
        // u noveho zaznamu doplni vychozi rozdeleni
        if (RonPortalSettings::getIntValue("rozdelenipoklk{$poradiRozdeleni}_vychozihodnota")) {
            $r = RozdeleniOsob::nactiRozdeleniOsoby($this ->database,
                $this->pokladniKniha->idPP, $typRozdeleni->kod,
                time(), time(), (RonPortalSettings::getIntValue("rozdelenipoklk{$poradiRozdeleni}_vychozihodnota") == 2) ? 'Hlavni = 1' : null );

            if ($r) {
                $rozdeleni = $r[0]->kod;
            }
        }

        $this->addField(
            new RozdeleniFormField($typRozdeleni->kod, RonPortalSettings::getIntValue("rozdelenipoklk{$poradiRozdeleni}_pouzevlastni")))
            ->setFieldName("Rozdeleni{$poradiRozdeleni}")->setTypRozdeleniFieldName("KodTypuRozdeleni{$poradiRozdeleni}")
            ->setCode("field-rozdeleni{$poradiRozdeleni}")
            ->setCaption(StringUtils::htmlTextConvert($typRozdeleni ->popis))->setShowEmptyValue(true)
            ->setFieldId("poklk_rozdeleni{$poradiRozdeleni}")
            ->setDefaultValue($rozdeleni)
            ->addFieldCssClass('ronform-field')
            ->setOtherHtmlParams(['data-ronform-field-name' => "Rozdeleni{$poradiRozdeleni}"]);
    }

    public function renderCommonFields()
    {
        parent::renderCommonFields();

        echo '<input type="hidden" name="browser" value="' . $this->seznamPolozekPokladniKnihy->cislo . '" />';
        echo "<input type='hidden' name='typy' id='seznamtypu' value='" . json_encode($this->seznamPolozekPokladniKnihy->jsonObject) . "' />";
    }

    function createDBRecord($tableName = null)
    {
        return new RadekPokladniKnihyRecord($this->database, $this->pokladniKniha->vratID(), $_POST['CisloRadku'] ?? null);
    }

    public function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        $cisloRadku = isset($_POST['CisloRadku'])
            ? (int)$_POST['CisloRadku']
            : (int)$_GET['record'];

        return new SqlStatement(
            'IDPokladniKnihy = ' . $params ->addParameter($this->pokladniKniha->vratID()) .
            ' AND CisloRadku = ' . $params ->addParameter($cisloRadku),
            $params);
    }

    public function createDBRecordFields($rec,$insert) {
        parent::createDBRecordFields($rec,$insert);

        if (Prava::vratPrava('pkniha_k_ucet') ->lzeZobrazitPP($this->pokladniKniha->idPP)) {
            $fZkontrolovano = $rec ->findFieldByName('Zkontrolovano');
            if ($fZkontrolovano  &&  $fZkontrolovano ->value  &&  $fZkontrolovano ->isChanged()) {
                // uzivatel zaskrtnul "zkontrolovano" - doplni pole "Zkontroloval" a "DatumCasKontroly"
                $rec ->addTimestampField('DatumCasKontroly', false, time());
                $rec ->addStringField('Zkontroloval', false, 25, RonPortalUtils::getLogUserName());
            }
        }
    }

}