<?php

namespace RonPortal\PokladniKniha;

use Exception;
use Ron\DB\DBException;
use Ron\DB\DBRecordField;
use Ron\DB\DBUtils;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\RonPortalXMLEvents;
use function Ron\ronText;

class RadekPokladniKnihyRecord extends RonPortalDBRecord
{
    /** @var int */
    public $idPokladniKnihy;
    /** @var DBRecordField */
    public $cisloRadku;
    /** @var DBRecordField */
    public $datum;
    /** @var DBRecordField */
    public $popis;
    /** @var DBRecordField */
    public $typ;
    /** @var DBRecordField */
    public $castka;
    /** @var DBRecordField */
    public $castkaBezDPH;
    /** @var DBRecordField */
    public $castkaDPH;
    /** @var DBRecordField */
    public $zpusobPlatby;
    /** @var DBRecordField */
    public $poznamka;
    /** @var DBRecordField */
    public $kodTypuRozdeleni;
    /** @var DBRecordField */
    public $rozdeleni;
    /** @var DBRecordField */
    public $kodTypuRozdeleni2
        /** @var DBRecordField */;
    public $rozdeleni2;
    /** @var DBRecordField */
    public $kodTypuRozdeleni3;
    /** @var DBRecordField */
    public $rozdeleni3;
    /** @var DBRecordField */
    public $kodTypuRozdeleni4;
    /** @var DBRecordField */
    public $rozdeleni4;

    public function __construct($database, $idPokladniKnihy, $cisloRadku = null, $logDescription = 'Pokladní kniha')
    {
        parent::__construct($database, 'RadekPokladniKnihy', $logDescription);

        $this->beforePostEvent = RonPortalXMLEvents::POKLK_RADEK_BEFORE_POST;
        $this->afterPostEvent = RonPortalXMLEvents::POKLK_RADEK_AFTER_POST;

        $this->idPokladniKnihy = $idPokladniKnihy;

        $this->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())->setUpdateLog(true)->setNoInsert(true);
        $this->addField('Update_Timestamp', 'datetime', false, 25, time())->setUpdateLog(true)->setNoInsert(true);

        $this->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
        $this->addField('Insert_Timestamp', 'datetime', true, 25, time());

        $params = $database ->createParameters();
        $where =
            ' IDPokladniKnihy = ' . $params->addParameter((int)$idPokladniKnihy);

        if ($cisloRadku) {
            $where .= ' AND CisloRadku = ' . $params->addParameter($cisloRadku);
        }

        $this->setWhere($where, $params);
    }

    public function createFields($cisloRadku)
    {
        $this->cisloRadku = $this->addField('CisloRadku', 'int', true, 0, $cisloRadku);

        $this->datum = $this->addField('DatumPohybu', 'timestamp', true);
        $this->popis = $this->addField('Popis', '', true, 250);
        $this->typ = $this->addField('TypPolozky', '', true, 15);
        $this->castka = $this->addField('Castka', 'float', true);
        $this->castkaBezDPH = $this->addField('CastkaBezDPH', 'float', true);
        $this->castkaDPH = $this->addField('CastkaDPH', 'float', true);
        $this->zpusobPlatby = $this->addField('ZpusobPlatby', 'int');
        $this->poznamka = $this->addField('Poznamka', '', true, 250);

        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            $this->kodTypuRozdeleni = $this->addField('KodTypuRozdeleni', 'int', true);
            $this->rozdeleni = $this->addField('Rozdeleni', '', true, 30);
        }

        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            $this->kodTypuRozdeleni2 = $this->addField('KodTypuRozdeleni2', 'int', true);
            $this->rozdeleni2 = $this->addField('Rozdeleni2', '', true, 30);
        }

        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            $this->kodTypuRozdeleni3 = $this->addField('KodTypuRozdeleni3', 'int', true);
            $this->rozdeleni3 = $this->addField('Rozdeleni3', '', true, 30);
        }
        
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            $this->kodTypuRozdeleni4 = $this->addField('KodTypuRozdeleni4', 'int', true);
            $this->rozdeleni4 = $this->addField('Rozdeleni4', '', true, 30);
        }
    }

    /**
     * @throws Exception
     */
    public function upravCastku()
    {
        $seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowserDleID($this->database, $this->idPokladniKnihy);
        $typPolozky = $seznamPolozekPokladniKnihy->vratTypPolozky($this['TypPolozky']);

        if ($typPolozky->lzeZobrazitCastkuBezDPH() && ($this['CastkaBezDPH'] == '')) {
            throw new Exception(ronText('Nezadana_castka_bez_dph'));
        }

        if ($typPolozky->lzeZobrazitCastkuDPH() && ($this['CastkaDPH'] == '')) {
            throw new Exception(ronText('Nezadana_castka_dph'));
        }

        //zadava se castka bez dph a celkova castka, castka dph se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && !$typPolozky->lzeZobrazitCastkuDPH()) {
            if ($this['CastkaBezDPH'] > $this['Castka']) {
                throw new Exception(ronText('Castka_bez_dph_vyssi'));
            }
            $this->findFieldByName('CastkaDPH')->setNewValue($this['Castka'] - $this['CastkaBezDPH']);
        }

        //zadava se castka dph a celkova castka, castka bez dph se dopocte
        if (!$typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            if ($this['CastkaDPH'] > $this['Castka']) {
                throw new Exception(ronText('Castka_dph_vyssi'));
            }
            $this->findFieldByName('CastkaBezDPH')->setNewValue($this['Castka'] - $this['CastkaDPH']);
        }

        //zadava se castka bez dph a castka dph, celkova castka se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            $this->findFieldByName('Castka')->setNewValue($this['CastkaBezDPH'] + $this['CastkaDPH']);
        }
    }

    /**
     * @throws DBException
     */
    public function nastavCastkuPodleTypu()
    {
        $params = $this->database->createParameters();
        $jeVydej = (int)DBUtils::getSqlValue($this->database, "SELECT DruhPohybu FROM TypPolozkyPokladniKnihy WHERE Kod = " . $params->addParameter($this['TypPolozky']), $params) == 2;
        //pro vydej bude castka zaporna
        if ($jeVydej) {
            $this->findFieldByName('Castka')->setNewValue(- abs($this['Castka']));
        }
        else {
            $this->findFieldByName('Castka')->setNewValue(abs($this['Castka']));
        }
    }


    /**
     * @throws DBException
     */
    public function prepocitejSoucet()
    {
        $params = $this->database->createParameters();

        $sql = "SELECT COALESCE(SUM(RadekPokladniKnihy.Castka), 0), COALESCE(SUM(RadekPokladniKnihy.CastkaBezDPH), 0), COALESCE(SUM(RadekPokladniKnihy.CastkaDPH), 0)
                FROM RadekPokladniKnihy WHERE IDPokladniKnihy = " . $params->addParameter($this->idPokladniKnihy);

        $cursor = $this->database->getSqlResult($sql, $params);

        $row = $cursor->fetch();

        if ($row) {
            $rec = new PokladniKnihaRecord($this->database, $this->idPokladniKnihy);
            $rec->castka->setNewValue($row[0]);
            $rec->castkaBezDPH->setNewValue($row[1]);
            $rec->castkaDPH->setNewValue($row[2]);

            $rec->loadAndUpdateRecord();
        }
    }

    public function updateRecord($filtr = '', $filtrParams = null)
    {
        $this->upravCastku();

        $this->nastavCastkuPodleTypu();

        $result = parent::updateRecord($filtr, $filtrParams);

        if ($this->isChanged()) {
            try {
                $this->prepocitejSoucet();
            } catch (Exception $exception) {
                throw new Exception('Chyba pri prepoctu radku pokladni knihy');
            }
        }

        return $result;
    }

    public function insertRecord()
    {
        $this->upravCastku();

        $this->nastavCastkuPodleTypu();

        parent::insertRecord();

        try {
            $this->prepocitejSoucet();
        } catch (Exception $exception) {
            throw new Exception('Chyba pri prepoctu radku pokladni knihy');
        }
    }

    /**
     * @param string $filtr
     * @param null $filtrParams
     * @throws Exception
     */
    public function doDelete($filtr, $filtrParams = null)
    {
        parent::doDelete($filtr, $filtrParams);

        try {
            $this->prepocitejSoucet();
        } catch (Exception $exception) {
            throw new Exception('Chyba pri prepoctu radku pokladni knihy');
        }
    }


}
