<?php
/** UTF-8
 * Roman.Szablatura, 8.6.2016
 */

namespace RonPortal\Pomucky;

use Exception;
use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBRecord;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalUtils;


class EditaceExpiracePomuckyForm extends RonPortalForm
{
    /** @var PracovniPomer */
    private $pPomer;

    /** @var string */
    private $kodPomucky;
    /** @var int */
    private $datumNaroku;
    /** @var int */
    private $poradi;
//    private $vydanipomucky;
//    private $editexpirace;


    /**
     * EditaceExpiracePomuckyForm constructor.
     * @param Database $database
     * @param PracovniPomer $pPomer
     */
    function __construct($database, $pPomer)
    {
        parent::__construct($database);

        $this ->pPomer = $pPomer;

        $this->title = TextManager::_('editace_expirace_pomucky');
        $this->htmlAction = '?menuid=pracovnipomucky&action=editaceexpirace';
        $this->stornoAction = '?menuid=pracovnipomucky';

        $this->code = 'formexpiracepomucky';

        $this->sqlTableName = 'PrirazenaPracPomucka';

        $this ->setXmlEventsName('POMUCKY_EXPIRACE');

        $this ->kodPomucky = $_POST['pomucka'] ?? $_GET['pomucka'] ?? null;

        $datumNaroku = $_POST['datumnaroku'] ?? $_GET['datumnaroku'] ?? null;
        $this ->datumNaroku = isset($datumNaroku)
            ? (int)$datumNaroku
            : null;

        $poradi = $_POST['poradi'] ?? $_GET['poradi'] ?? null;
        $this ->poradi = isset($poradi)
            ? (int)$poradi
            : null;

        $prava = Prava::vratPrava('pomucky_vydej');
        if (!$prava ->lzeZobrazitRC($this ->pPomer ->rc)  ) {
            throw new NeniPravoException();
        }

        if (!isset($this ->kodPomucky)) {
            throw new Exception('Neni zadan kod pomucky!');
        }

        $pomucka = PracovniPomucka::nactiPomucku($this ->database, $this ->kodPomucky);
        if (!$prava ->priznak3ObsahujeKod($pomucka ->typPomucky)) {
            throw new NeniPravoException();
        }

        $this->addField(FormField::FIELD_TYPE_TEXT)
            ->setText(StringUtils::htmlTextConvert($pomucka ->popis)) ->setCaption('pracovni_pomucka')
            ->setCode('field-pomucka');


        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumVydani') ->setCaption('datum_vydani') ->setReadOnly(true)
            ->setCode('field-datumvydani');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumExpiraceVypoctene') ->setCaption('datum_expirace_vypoctene') ->setReadOnly(true)
            ->setCode('field-datumexpiracevypoctene');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumExpirace') ->setCaption('datum_expirace')
            ->setCode('field-datumexpirace');


        $this->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setCaption('Poznamka')
            ->setCode('field-poznamka');
    }


//    private function getPomuckaDetail()
//    {
//        if (isset($_POST['pomucka'])) {
//            $this ->kodPomucky = $_POST['pomucka'];
//        }
//        else if (isset($_GET['pomucka'])) {
//            $this ->kodPomucky = $_GET['pomucka'];
//        }
//        else {
//            $this ->kodPomucky = null;
//        }
//
//        if (isset($_POST['datumnaroku'])) {
//            $this ->datumNaroku = (int)$_POST['datumnaroku'];
//        }
//        else if(isset($_GET['datumnaroku'])) {
//            $this ->datumNaroku = (int)$_GET['datumnaroku'];
//        }
//        else {
//            $this ->datumNaroku = null;
//        }
//
//        if (isset($_POST['poradi'])) {
//            $this ->poradi = (int)$_POST['poradi'];
//        }
//        else if(isset($_GET['poradi'])) {
//            $this ->poradi = (int)$_GET['poradi'];
//        }
//        else {
//            $this ->poradi = '';
//        }
//    }

    /**
     * @return string
     * @throws DBException
     */
    function getWhereCondition()
    {
//        $this->getPomuckaDetail();

        if ($this->pPomer  &&  isset($this->kodPomucky)  &&  isset($this->datumNaroku)  &&  isset($this->poradi)) {
            $params = $this->database->createParameters();
            return new SqlStatement(
                ' RC = ' . $params->addParameter($this ->pPomer ->rc) .
                ' AND PracovniPomucka = ' . $params->addParameter($this->kodPomucky) .
                ' AND DatumNaroku = ' .  $params->addTimestampParameter($this->datumNaroku) .
                ' AND Poradi = ' .  $params->addParameter($this->poradi),
                $params);
        }

        return '0 = 1';
    }


    function renderCommonFields()
    {
        parent::renderCommonFields();

        echo '<input type="hidden" name="osoba" value="' . $this->pPomer ->idPP . '" />';
        echo '<input type="hidden" name="pomucka" value="' . StringUtils::htmlTextConvert($this->kodPomucky) . '" />';
        echo '<input type="hidden" name="datumnaroku" value="' . $this->datumNaroku . '" />';
        echo '<input type="hidden" name="poradi" value="' . $this->poradi . '" />';
    }


    function createDBRecordFields($rec, $insert)
    {
        $rec->addStringField('RC', true, 15);
//        $result->addStringField('OsobniCislo', true, 15);
//        $result->addDateField('DatumNastupu', true);

        $rec->addStringField('PracovniPomucka', true, 10);
        $rec->addDateField('DatumNaroku', true);
        $rec->addIntField('Poradi', true);

        parent::createDBRecordFields($rec, $insert);

        $rec->addStringField('Update_UserName', false, 25, RonPortalUtils::getLogUserName())
            ->setUpdateLog(true);
        $rec->addTimestampField('Update_Timestamp', false, time())
            ->setUpdateLog(true);
    }


    /**
     * @param DBRecord $rec
     * @throws DBException
     */
    function setDBRecord($rec)
    {
        parent::setDBRecord($rec);

        $rec ->setWhere($this->getWhereCondition());
    }


    /**
     * @param DBRecord $rec
     * @return bool
     * @throws Exception
     */
    function beforeUpdateRecord($rec)
    {
        if($rec['DatumExpirace'] == '') {
            throw new Exception(TextManager::_('form.Missing_value') . ' ' . TextManager::_('datum_expirace') . '!');
        }
        if($rec['DatumExpirace'] < $rec['DatumVydani']){
            throw new Exception(TextManager::_('spatne_datum_expirace'));
        }

        return parent::beforeUpdateRecord($rec);
    }

}
