<?php
/**
 * Roman.Szablatura, 18.2.2016
 * TODO: predelat filtr "stav"
 */

namespace RonPortal\Pomucky;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\Controls\Dialog;
use Ron\Controls\DialogButton;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\HttpUtils;
use Ron\MVC\BaseViewObject;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatum;
use RonPortal\Controls\FilterFormFilterSwitchLinks;
use RonPortal\Controls\FormReportsListView;
use RonPortal\Controls\OsobniCisloBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use Ron\StringUtils;
use RonPortal\System\Report;
use RonPortal\User;


/**
 * Filtr na vyber pomucky - pokud je zadan typ pomucky, tak zobrazuje jen pomucky daneho typu
 */
class PomuckaFilter extends FilterFormFilter
{
    private $parentFilterForm;

    public function __construct(string $name, RonPortalFilterForm $parentFilterForm)
    {
        parent::__construct($name, self::TYPE_OPTIONS);
        $this ->parentFilterForm = $parentFilterForm;
        $this
            ->setOptionsSqlFilter('1 = 1')
            ->setOptionsSqlOrder('Popis')
            ->setOptionsSqlOrder2('Kod');
    }

    public function loadOptions(Database $database): FilterFormFilter
    {
        // pokud je zapnut filtr na umisteni, tak zobrazi jen terminaly z daneho umisteni
        $typ = $this ->parentFilterForm ->getFilterValue('typPomucky');
        if (isset($typ)) {
            $params = $database ->createParameters();
            $sql =
                'SELECT Kod, Popis FROM PracovniPomucka' .
                ' WHERE PracovniPomucka.TypPomucky = ' . $params->addParameter($typ) .
                ' AND (%FILTER%)' .
                ' ORDER BY %ORDER%';

            $this ->optionsSql = new SqlStatement($sql, $params);
        } else {
            $this ->optionsSql =
                'SELECT Kod, Popis FROM PracovniPomucka WHERE %FILTER% ORDER BY %ORDER%';
        }

        return parent::loadOptions($database);
    }
}


/**
 * Class PracovniPomuckyFilterForm
 * @package Ron\Personalistika
 */
class PracovniPomuckyFilterForm extends RonPortalFilterForm
{
    public $timestamp;

    /**
     * @param Database $database
     * @param PravaNaOsoby $pravaProhlizet
     * @param PravaNaOsoby $pravaPotvrdit
     * @param PravaNaOsoby $pravaVydatVratit
     */
    public function __construct($database, $pravaProhlizet, $pravaPotvrdit, $pravaVydatVratit)
    {
        parent::__construct($database, 'pracovnipomucky', true);

        $this ->addFilterRozdeleni1();
        $this ->addFilterRozdeleni2();

        $this ->addFilterPP(true, [$pravaProhlizet, $pravaPotvrdit, $pravaVydatVratit]);
//        $this->nastavOsoby(true,[$pravaProhlizet, $pravaPotvrdit, $pravaVydatVratit]);
//        $this->povolitVsechyOsoby = true;

        //$this->nastavDatumOdDo(true, true, TextManager::_('PracovniPomucky_obdobi') . ' ' . TextManager::_('Od'));
        //$this->nastavDatum(true);

        $this ->addFilter(
            (new FilterFormFilterDatum('datum')),
                //->setSessionName(null)  // pokud neni zadano v URL, tak se zobrazi aktualni stav
                //->setCaption(TextManager::_('Datum_a_cas'));
            true
        );


        $filtrTyp = new FilterFormFilter('typPomucky', FilterFormFilter::TYPE_OPTIONS);
        $filtrTyp
            ->setSessionName('filterform_typ_pomucky')
            ->setCaption(TextManager::_('typ_pomucky'))
            ->addItem('', TextManager::_('_vse_'))
            ->setOptionsSql('SELECT Kod,TypPomucky.Popis FROM TypPomucky WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis')
            ->setSqlFilter('PracovniPomucka.TypPomucky = ');
        $this ->addFilter($filtrTyp);

        $filtrPomucka = new PomuckaFilter('pomucka', $this);
        $filtrPomucka
            ->setSessionName('filterform_pomucka')
            ->setCaption(TextManager::_('pomucka'))
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('PracovniPomucka.Kod = ')
            ->setShowCode(true);
        $this ->addFilter($filtrPomucka);

        $this ->addFilter(
            (new FilterFormFilterSwitchLinks('stavpomucky'))
                ->addItem('', TextManager::_('pomucky_stav_k_datu'))
                ->addItem('nevydane', TextManager::_('nevydane_pomucky'))
                ->addItem('potvrzene', TextManager::_('vydane_potvrzene_pomucky'))
                ->addItem('nepotvrzene', TextManager::_('vydane_nepotvrzene_pomucky'))
                ->addItem('expirovane', TextManager::_('vydane_expirovane_pomucky'))
                ->addItem('vracene', TextManager::_('vracene_pomucky'))
        );
    }





//    public function renderLinkVydani($name, $caption, $value = null, $selected = null)
//    {
//        if (!isset($value)) {
//            $value = (is_array(RonPortalSession::getValue('filtrVydanePomucky')) && isset($_SESSION['filtrVydanePomucky'][$name]))
//                ? $_SESSION['filtrVydanePomucky'][$name]
//                : 0;
//        }
//
//        $newValue = $value ? 0 : 1;
//
//        $className =
//            'setValue' .
//            ( ($selected ||$value) ? ' font-weight-bold' : '');
//
//        echo
//            '<span class="margin-right-2 padding-right-2">' .
//            "<input type='hidden' id='stav_{$name}' name='$name' value='$value'/>" .
//            "<a href='#' id='filtrpomucky_{$name}' class='$className' data-submit='1' data-selector='#stav_{$name}' data-value='$newValue'>" .
//            $caption .
//            '</a>' .
//            '</span>';
//    }

//    public function renderFiltrNevydanePracovniPomucky()
//    {
//        echo '<div>';
//        echo '<span class="filtercaption padding-top-2">';
//
//        $jeNejakyFiltr = !!RonPortalSession::getValue('filtrVydanePomucky');
//        // aktualni stav - vzdy nastavi hodnotu "0" a vybrany je, kdyz neni vybrany zadny jiny
//        // (zakliknuti tohoto filtru vypina vsechny ostatni a zakliknuti kterehokoliv jineho filtru vypina tento)
//        $this ->renderLinkVydani('aktualniStav', TextManager::_('pomucky_stav_k_datu'), 0, !$jeNejakyFiltr);
//
//        $this ->renderLinkVydani('jenNevydane', TextManager::_('nevydane_pomucky'));
//        $this ->renderLinkVydani('jenPotvrzene', TextManager::_('vydane_potvrzene_pomucky'));
//        $this ->renderLinkVydani('jenNepotvrzene', TextManager::_('vydane_nepotvrzene_pomucky'));
//        $this ->renderLinkVydani('jenExpirovane', TextManager::_('vydane_expirovane_pomucky'));
//        $this ->renderLinkVydani('jenVracene', TextManager::_('vracene_pomucky'));
//
//        echo "</span>";
//        echo '</div>';
//    }

    private function renderReportList()
    {
        $reportList = Report::loadFormReports($this->database, 'PracovniPomucky');
        FormReportsListView::renderReportList($reportList,
            '?menuid=pracovnipomucky&action=report&osoba=' . ($this->getFilterValue('osoba') ?? '') .
            '&datumod=' . StringUtils::format('date', $this->getFilterValue('datum')) .
            '&datumdo=' . StringUtils::format('date', $this->getFilterValue('datum')) .
            '&rozdeleni=' . $this->getFilterValue('rozdeleni') .
            '&rozdelenipodrizene=' . $this->getFilterValue('rozdeleni', 'vcetnePodrizenych') .
            '&rozdeleni2=' . $this->getFilterValue('rozdeleni2') .
            '&rozdelenipodrizene2=' . $this->getFilterValue('rozdeleni2', 'vcetnePodrizenych') .
            '&vsechnyosoby=' . ($this->getFilterValue('osoba') ? 0 : 1) .
            '&jentyp=' . (($this ->getFilterValue('typPomucky') !== null) ? 1 : 0) .
            '&typpomucky=' . ($this ->getFilterValue('typPomucky') ?? '') .
            '&jenpomucka=' . (($this ->getFilterValue('pomucka') !== null) ? 1 : 0) .
            '&pomucka=' . ($this ->getFilterValue('pomucka') ?? '') .
            '&nevydane=' . (($this ->getFilterValue('stavpomucky', 'nevydane')) ? 1 : 0) .
            '&vracene=' . (($this ->getFilterValue('stavpomucky', 'vracene')) ? 1 : 0) .
            '&potvrzene=' . (($this ->getFilterValue('stavpomucky', 'potvrzene')) ? 1 : 0) .
            '&expirovane=' . (($this ->getFilterValue('stavpomucky', 'expirovane')) ? 1 : 0) .
            '&nepotvrzene=' . (($this ->getFilterValue('stavpomucky', 'nepotvrzene')) ? 1 : 0)
        );
    }

//    private function loadVydaniCheckbox($name)
//    {
//        if (isset($_GET[$name])) {
//            //$value = (is_array($_SESSION['filtrVydanePomucky']) && isset($_SESSION['filtrVydanePomucky'][$name])) ? 1 : 0;
//            if (!is_array(RonPortalSession::getValue('filtrVydanePomucky'))) {
//                RonPortalSession::setValue('filtrVydanePomucky', []);
//            }
//
//            if ((int)$_GET[$name]) {
//                $_SESSION['filtrVydanePomucky'][$name] = 1;
//            } else {
//                unset($_SESSION['filtrVydanePomucky'][$name]);
//            }
//        }
//    }

//    private function loadJenNevracene()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $_SESSION['jenNevracene'] = isset($_POST['jenNevracene']);
//        }
//    }

//    private function loadTypPomucky()
//    {
//        if (isset($_GET['typPomucky'])) {
//            $_SESSION['jenTypPomucky'] = isset($_GET['jenTypPomucky'])  &&  (int)$_GET['jenTypPomucky'];
//            $_SESSION['typPomucky'] = $_GET['typPomucky'];
//        }
//    }


//    /**
//     * @throws Exception
//     */
//    private function loadPomucka()
//    {
//        if (isset($_GET['pomucka'])) {
//            $_SESSION['jenPomucka'] = isset($_GET['jenPomucka']) && (int)$_GET['jenPomucka'];
//            $_SESSION['pomucka'] = $_GET['pomucka'];
//        }
//
//        if (RonPortalSession::getValue('pomucka') != '' && RonPortalSession::getValue('jenPomucka')) {
//            $params = $this->database->createParameters();
//            $_SESSION['typPomucky'] = DBUtils::getSqlValue($this->database,
//                'SELECT TypPomucky FROM PracovniPomucka' .
//                ' WHERE Kod = ' . $params->addParameter(RonPortalSession::getValue('pomucka')), $params);
//        }
//    }

//    private function setVsechnyOsoby()
//    {
//        RonPortalSession::setVsechnyOsoby(true);
//    }

//    function loadData()
//    {
//        parent::loadData();
//
//        //finta, po prihlaseni se nastavuje na aktu.den
//        $datumOd = RonPortalSession::getDatumOd();
//        if (!isset($datumOd)){
//            $this ->datumOd = DateUtils::today();
//            $this ->datumDo = DateUtils::today();
//
//            RonPortalSession::setFiltrovatDatum($this ->filtrovatDatum,
//                $this ->datumOd,
//                $this ->datumDo );
//        }
        //$this->loadDatumOdDo();
//        if (isset($_GET['aktualniStav'])  &&  $_GET['aktualniStav']) {
//            // uzivatel vybral filtr "aktualni stav" => vypne vsechny ostatni filtry
//            $_SESSION['filtrVydanePomucky'] = [];
//        } else {
//            $this->loadVydaniCheckbox('jenNevydane');
//            $this->loadVydaniCheckbox('jenNepotvrzene');
//            $this->loadVydaniCheckbox('jenPotvrzene');
//            $this->loadVydaniCheckbox('jenExpirovane');
//            $this->loadVydaniCheckbox('jenVracene');
//        }

//        $this->loadJenNevracene();
        //$this->loadJenVracene();
        //$this->loadJenNevydane();
        //$this->loadJenNepotvrzene();
        //$this->loadJenPotvrzene();
        //$this->loadJenExpirovane();

//        $this->loadTypPomucky();
//        $this->loadPomucka();
        //parent::loadData();
//    }

    /**
     * @throws Exception
     */
    function renderFilters()
    {
        parent::renderFilters();

//        $this->renderFiltrNevydanePracovniPomucky();
        $this->renderReportList();
    }
}

/**
 * Class PotvrzeniPrijetiField
 * @package Ron\Personalistika
 */
class PotvrzeniPrevzetiField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $idosoby = $rowData['col-idpp'];
//        $params = $this->database->createParameters();
//        $prihlasenaOsoba = DBUtils::getSqlValue($this->database,
//            "SELECT MAX(ID) FROM PracovniPomer WHERE RC = " . $params->addParameter(User::getRC()), $params);

        if ($rowData['col-potvrzeniprevzetidatum']) {
            // jiz potvrzeno - zobrazi cas potvrzeni
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        if ($rowData['col-datumvydani'] && !$rowData['col-datumvraceni'] && $rowData['col-vyzadovatpotvrzeni'] && Prava::vratPrava('pomucky_potvrzeni')->lzeZobrazitRC($rowData['col-rc'])) {
            // je vydana, neni vracena a je vyzadovano potvrzeni
            return
                '<a class="confirmQuestion"' .
                ' href="?menuid=pracovnipomucky&action=potvrzeniprevzeti&osoba=' . $idosoby .
                '&pomucka=' . HttpUtils::urlEncode($rowData['col-pracovnipomucka']) .
                '&datumnaroku=' . $this->database->stringToTimestamp($rowData['col-datumnaroku']) .
                '&poradi=' . $rowData['col-poradi'] . '"' .
                (User::hasRC($rowData['col-rc'])
                    ? ''    // potvrzeni uzivatele - neni potreba dotaz
                    : ' data-ron-confirm-text="' . TextManager::_('dotaz_potvrdit_prevzeti_za_jinou_osobu') . '"' // potvrzeni za jineho zamestnance
                ) .
                '>' .
                TextManager::_('potvrdit') .
                '</a>';
        }

        return '';
    }
}


/**
 * Class DatumVraceniField
 * @package Ron\Personalistika
 */
class DatumVraceniField extends BrowserField
{
    private $pravoVydej;

    /**
     * DatumVraceniField constructor.
     * @param null|Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this->pravoVydej = Prava::vratPrava('pomucky_vydej');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$value  &&  $rowData['col-datumvydani']) {
            // jeste zkontroluje pravo (na RC a taky na typ pomucky)
            if ((!$this->pravoVydej->priznak3 || $this->pravoVydej->priznak3ObsahujeKod($rowData['col-typ']))
                && $this->pravoVydej->lzeZobrazitRC($rowData['col-rc'])) {
                // OK, uzivatel ma pravo vydat/vratit
                return
                    '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=edit&osoba=' . $rowData['col-idpp'] .
                    '&pomucka=' . HttpUtils::urlEncode($rowData['col-pracovnipomucka']) .
                    '&datumnaroku=' . $this->database->stringToTimestamp($rowData['col-datumnaroku']) .
                    '&poradi=' . $rowData['col-poradi'] .
                    '&vydanipomucky=0">' .
                    TextManager::_('vratit') .
                    '</a>';
            }
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


/**
 * Class DatumVydaniField
 * @package Ron\Personalistika
 */
class DatumVydaniField extends BrowserField
{
    private $pravoVydej;

    /**
     * DatumVydaniField constructor.
     * @param null|Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);
        $this->pravoVydej = Prava::vratPrava('pomucky_vydej');
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumvydani']) {
            return StringUtils::format('date', $this->database->stringToTimestamp($rowData['col-datumvydani']));
        }

        // jeste zkontroluje pravo (na RC a taky na typ pomucky)
        if ((!$this->pravoVydej->priznak3 || $this->pravoVydej->priznak3ObsahujeKod($rowData['col-typ']))
            && $this->pravoVydej->lzeZobrazitRC($rowData['col-rc'])) {
            // OK, uzivatel ma pravo vydat/vratit
            return
                '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=edit&osoba=' . $rowData['col-idpp'] .
                '&pomucka=' . HttpUtils::urlEncode($rowData['col-pracovnipomucka']) .
                '&datumnaroku=' . $this->database->stringToTimestamp($rowData['col-datumnaroku']) .
                '&poradi=' . $rowData['col-poradi'] . '&vydanipomucky=1">' .
                TextManager::_('vydat') .
                '</a>';
        }

        return '';
    }
}

/**
 * Odkaz na editaci data expirace
 * Class EditaceExpiraceField
 * @package Ron\Personalistika
 */
class EditaceExpiraceField extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $pravoVydej;

    function __construct($database)
    {
        parent::__construct($database);
        $this->pravoVydej = Prava::vratPrava('pomucky_vydej');
    }


//    function getHtmlTitle($rowNumber, $recordId, $rowData)
//    {
//        $result = parent::getHtmlTitle($rowNumber, $recordId, $rowData);
//        if ($result) {
//            return $result;
//        }
//
//        if ($rowData['col-datumvydani']  &&  !$rowData['col-datumvraceni']) {
//            if ($this ->pravoVydej ->priznak3ObsahujeKod($rowData['col-typ'])  &&  $this->pravoVydej->lzeZobrazitRC($rowData['col-rc'])) {
//                return $rowData['col-datumexpirace']
//                    ? TextManager::_('info_editace_expirace_pomucky')
//                    : TextManager::_('uprava_expirace_pomucky');
//            }
//        }
//
//        return '';
//    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumvydani']  &&  !$rowData['col-datumvraceni']) { // je vydana, jeste neni vracena
            if ($this ->pravoVydej ->priznak3ObsahujeKod($rowData['col-typ'])  &&  $this->pravoVydej->lzeZobrazitRC($rowData['col-rc'])) { // je pravo na editaci
                return
                    '<a href="?menuid=pracovnipomucky&action=editaceexpirace&formaction=edit&osoba=' . $rowData['col-idpp'] .
                    '&pomucka=' . HttpUtils::urlEncode($rowData['col-pracovnipomucka']) .
                    '&datumnaroku=' . $this->database->stringToTimestamp($rowData['col-datumnaroku']) .
                    '&poradi=' . $rowData['col-poradi'] . '&editexpirace=1">' .
                    ($value
                        ? parent::getViewText($value, $rowNumber, $recordId, $rowData)
                        : TextManager::_('expirace_neuvedena')
                    ) .
                    '</a>';
            }
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}

class ExpiraceVypoctenaField extends BrowserField
{
    function getHtmlTitle($rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-datumvydani']  &&  !$rowData[$this ->sqlFieldIndex]) { // pomucka byla vydana a nema expiraci
            return TextManager::_('uprava_expirace_pomucky');
        }

        return parent::getHtmlTitle($rowNumber, $recordId, $rowData);
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$rowData['col-datumvydani']) {
            // pomucka nebyla vydana
            return '';
        }

        if (!$rowData[$this ->sqlFieldIndex]) {
            return TextManager::_('expirace_neuvedena');
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



/**
 * Class PracovniPomuckyBrowser
 * @package Ron\Personalistika
 */
class PracovniPomuckyBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby  */
    var $pravaProhlizet;
    /** @var PravaNaOsoby  */
    var $pravaZapisPrevzetiVydej;
    /** @var PravaNaOsoby  */
    var $pomuckyPotvrzeni;

    var $typRozdeleni;
    var $rozdeleni;

    /**
     * PracovniPomuckyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct($database)
    {
        parent::__construct($database, 'pracovnipomucky');

        $this->pravaProhlizet = Prava::vratPrava('pomucky_prohl');
        $this->pravaZapisPrevzetiVydej = Prava::vratPrava('pomucky_vydej');
        $this->pomuckyPotvrzeni = Prava::vratPrava('pomucky_potvrzeni');

        Prava::nastavObdobiProPrava(time(),time());

        $this->title = TextManager::_('pracovni_pomucky');
        $this->code = 'browserpracovnipomucky';
        $this->tableId = 'browserpracovnipomucky';

        $this->sqlFromPart =
            'PrirazenaPracPomucka, PracovniPomucka, StavPomucky, TypPomucky, Osoba';

        $this->sqlWherePart =
            'PrirazenaPracPomucka.PracovniPomucka=PracovniPomucka.Kod
             AND PrirazenaPracPomucka.RC = Osoba.RC
             AND PracovniPomucka.Stav = StavPomucky.Kod
             AND PracovniPomucka.TypPomucky = TypPomucky.Kod';

        $this->defaultSort = 'col-popis-pomucky';
        $this->browserUrl = '?menuid=pracovnipomucky&';

        $this->maxPageRows = 100;

//        $this ->filterForm = new PracovniPomuckyFilterForm($this ->database, $this->pravaProhlizet, $this->pomuckyPotvrzeni, $this->pravaZapisPrevzetiVydej);
//        $this ->filterForm ->loadData();

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PrirazenaPracPomucka.RC')
            ->setCode('col-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PrirazenaPracPomucka.Poradi')
            ->setCode('col-poradi');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomucka.Kod')
            ->setCode('col-pracovnipomucka');

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni');

        $this->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno');

        $this->addField(new OsobniCisloBrowserField($database, 'col-rc'))
            ->setCode('col-osobni-cislo');

//        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
//            ->setSqlFieldName('TypPomucky.Kod')
//            ->setCode('col-typpomucky');

        $this->addField()   // new TypPomuckyField($database, null));
            ->setSqlFieldName('TypPomucky.Popis') ->setHeaderText('typ_pomucky') ->setOrdering(true)
            ->setCode('col-popis-typu-pomucky');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TypPomucky.Kod')
            ->setCode('col-typ');

        $this->addField()   // new PopisPomuckyField($database, null));
            ->setSqlFieldName('PracovniPomucka.Popis') ->setHeaderText('Popis') ->setOrdering(true)
            ->setCode('col-popis-pomucky');

        $this->addField(BrowserField::FIELD_TYPE_TEXT)   // new DetailPomuckyField($database,null));
            ->setText(TextManager::_('detail')) //->setLink('?menuid=pracovnipomucky&action=detail&record=')
            ->setCode('col-detail')
            ->setClassName('js pracovni-pomucky-detail-link')
            ->setAttributeIndexes(['data-kod' => 'col-pracovnipomucka']);

        $this->addField(new DatumVydaniField($database))
            ->setSqlFieldName('PrirazenaPracPomucka.DatumVydani') ->setHeaderText('datum_vydani')
            ->setCode('col-datumvydani')
            ->setFormat('date')
            ->setAlign('center') ->setOrdering(true);

        $this->addField(new DatumVraceniField($database))
            ->setSqlFieldName('PrirazenaPracPomucka.DatumVraceni') ->setHeaderText('datum_vraceni')
            ->setCode('col-datumvraceni')
            ->setFormat('date')
            ->setAlign('center') ->setOrdering(true);

        $this->addField(new EditaceExpiraceField($database))
            ->setSqlFieldName('PrirazenaPracPomucka.DatumExpirace') ->setHeaderText('datum_expirace')
            ->setCode('col-datumexpirace')
            ->setFormat('date')
            ->setAlign('center') ->setOrdering(true);

        $this->addField(new ExpiraceVypoctenaField($database))
            ->setSqlFieldName('PrirazenaPracPomucka.DatumExpiraceVypoctene') ->setHeaderText('datum_expirace_vypoctene')
            ->setCode('col-expirace-vypoctena')
            ->setFormat('date')
            ->setAlign('center') ->setOrdering(true);

        /*        $this->addField([BROWSER_FIELD_SQL_INDEX => 4, BROWSER_FIELD_SQL_NAME => 'PrirazenaPracPomucka.DatumExpirace',
                    BROWSER_FIELD_HEADER_TEXT => 'datum_expirace', 'format' => 'date' , 'align' => 'center', 'ordering' => 1]);*/

        $this->addField(new PotvrzeniPrevzetiField($database))
            ->setSqlFieldName('PotvrzeniPrevzetiDatum') ->setHeaderText('potvrzeni_prevzeti')
            ->setCode('col-potvrzeniprevzetidatum')
            ->setFormat('datetime') ->setOrdering(true)
            ->setAlign('center')
            ->setTitleFieldIndex('col-potvrzeniprevzetipopis');


        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('VyzadovatPotvrzeni')
            ->setCode('col-vyzadovatpotvrzeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('(SELECT MAX(ID) FROM PracovniPomer WHERE RC = Osoba.RC) AS IdPP')
            ->setCode('col-idpp');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PotvrzeniPrevzetiPopis')
            ->setCode('col-potvrzeniprevzetipopis');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PrirazenaPracPomucka.DatumNaroku')
            ->setCode('col-datumnaroku');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(PrirazenaPracPomucka.Poznamka)}')
            ->setCode('col-poznamka')
            ->setClassName('poznamka browserpoznamka')
            ->setHeaderColSpan(-1);
    }


    function setFilterForm()
    {
        $this ->filterForm = new PracovniPomuckyFilterForm($this ->database,
            $this->pravaProhlizet, $this->pomuckyPotvrzeni, $this->pravaZapisPrevzetiVydej);
    }


    /**
     * @return SqlStatement|string
     */
    public function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            '(' .
            $this->pravaProhlizet->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
            ' OR ' .
            $this->pravaZapisPrevzetiVydej->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
            ' OR ' .
            $this->pomuckyPotvrzeni->sqlFiltr('PrirazenaPracPomucka',time(),time(), $params) .
            ')';

        if ($this ->pravaProhlizet ->priznak3) {
            // uzivatel ma omezeny seznam typu pracovnich pomucek
            $result .=
                ($result ? ' AND ' : '') .
                SQLUtils::getINCondition($this ->database,
                'PracovniPomucka.TypPomucky',
                explode(';', $this ->pravaProhlizet ->priznak3),
                $params
            );
        }

        $today = DateUtils::today();

        if ($this->filterForm->getFilterValue('osoba')) {
            // filtr pro vybranou osobu
            $result .=
                ' AND PrirazenaPracPomucka.RC = ' . $params->addParameter($this->filterForm->getFilterValue('osoba', 'rc'));
        } else {
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                $sqlRozdeleni = $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'PrirazenaPracPomucka', $params);
                if ($sqlRozdeleni) {
                    $result .= ' AND ' . $sqlRozdeleni;
                }
            }
        }

        $datum = $this->filterForm->getFilterValue('datum');

        $resultVydani = '';

        //if (!isset($_SESSION['filtrVydanePomucky'])  ||  !$_SESSION['filtrVydanePomucky']  ||  !is_array($_SESSION['filtrVydanePomucky'])) {
        //if ($this ->filterForm ->getFilterValue('stavpomucky', '')) {
        if (!$this ->filterForm ->getFilterValue('stavpomucky')) {
            // filtr "stav k datu"
            $resultVydani .=
                '(' .
                '(PrirazenaPracPomucka.DatumVraceni IS NULL  OR  PrirazenaPracPomucka.DatumVraceni >=' . $params ->addDateParameter($datum) . ')';

            if ($datum !== $today) {
                // jiny datum nez dnes (stav v minulosti)
                $resultVydani .=
                    'AND (' .
                    'PrirazenaPracPomucka.DatumVydani <= ' . $params->addDateParameter($datum) . // k zadanemu datu vydana
                    ' OR PrirazenaPracPomucka.DatumNaroku <= ' . $params->addDateParameter($datum) . // k zadanemu datu je na ni narok
                    ')';
            }

            $resultVydani .=
                ')';
        } else {
            if ($this ->filterForm ->getFilterValue('stavpomucky', 'nevydane')) { // isset($_SESSION['filtrVydanePomucky']['jenNevydane'])  &&  $_SESSION['filtrVydanePomucky']['jenNevydane']) {
                $resultVydani .=
                    ($resultVydani ? ' OR ' : '') .
                    'PrirazenaPracPomucka.DatumVydani IS NULL';

                if ($datum != $today) {
                    // je zvolen jiny nez aktualni datum
                    $resultVydani .= ' OR PrirazenaPracPomucka.DatumVydani > ' . $params->addDateParameter($datum);
                }
            }

            if ($this ->filterForm ->getFilterValue('stavpomucky', 'potvrzene')) { // isset($_SESSION['filtrVydanePomucky']['jenPotvrzene'])  &&  $_SESSION['filtrVydanePomucky']['jenPotvrzene']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                if ($datum == $today) {
                    // k aktualnimu datu
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani IS NOT NULL AND PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NOT NULL AND PrirazenaPracPomucka.DatumVraceni IS NULL)';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani <= ' . $params->addDateParameter($datum) . // k zadanemu datu vydana
                        ' AND PrirazenaPracPomucka.PotvrzeniPrevzetiDatum < ' . $params->addDateParameter(DateUtils::incDay($datum)) . // k zadanemu datu potvrzena
                        ' AND (PrirazenaPracPomucka.DatumVraceni > ' . $params->addDateParameter($datum) . // k zadanemu datu nevracena
                        '  OR PrirazenaPracPomucka.DatumVraceni IS NULL)' .
                        ')';
                }
            }

            if ($this ->filterForm ->getFilterValue('stavpomucky', 'nepotvrzene')) { // isset($_SESSION['filtrVydanePomucky']['jenNepotvrzene'])  &&  $_SESSION['filtrVydanePomucky']['jenNepotvrzene']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                if ($datum == $today) {
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani IS NOT NULL AND  PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NULL AND PrirazenaPracPomucka.DatumVraceni IS NULL)';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        '(PrirazenaPracPomucka.DatumVydani <= ' . $params->addDateParameter($datum) . // k zadanemu datu vydana
                        ' AND (PrirazenaPracPomucka.PotvrzeniPrevzetiDatum >= ' . $params->addDateParameter(DateUtils::incDay($datum)) . // k zadanemu datu nepotvrzena
                        '  OR PrirazenaPracPomucka.PotvrzeniPrevzetiDatum IS NULL)' .
                        ' AND (PrirazenaPracPomucka.DatumVraceni > ' . $params->addDateParameter($datum) . // k zadanemu datu nevracena
                        '  OR PrirazenaPracPomucka.DatumVraceni IS NULL)' .
                        ')';
                }
            }

            if ($this ->filterForm ->getFilterValue('stavpomucky', 'vracene')) { // isset($_SESSION['filtrVydanePomucky']['jenVracene']) && $_SESSION['filtrVydanePomucky']['jenVracene']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                if ($datum == $today) {
                    $resultVydani .=
                        'PrirazenaPracPomucka.DatumVraceni IS NOT NULL';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        'PrirazenaPracPomucka.DatumVraceni <= ' . $params->addDateParameter($datum); // k zadanemu datu vracena
                }
            }

            if ($this ->filterForm ->getFilterValue('stavpomucky', 'expirovane')) { // isset($_SESSION['filtrVydanePomucky']['jenExpirovane']) && $_SESSION['filtrVydanePomucky']['jenExpirovane']) {
                $resultVydani .= ($resultVydani ? ' OR ' : '');

                $resultVydani .=
                    '(PrirazenaPracPomucka.DatumExpirace <= ' . $params->addDateParameter($datum);

                if ($datum == $today) {
                    $resultVydani .=
                        ' AND PrirazenaPracPomucka.DatumVraceni IS NULL';
                } else {
                    // jiny datum (stav v minulosti)
                    $resultVydani .=
                        ' AND (PrirazenaPracPomucka.DatumVraceni IS NULL OR PrirazenaPracPomucka.DatumVraceni > ' . $params->addDateParameter($datum) . ')';
                }


                $resultVydani .= ')';
            }
        }

        if ($resultVydani != '') {
            $result .= ' AND (' . $resultVydani . ')';
        }


        return new SqlStatement($result, $params);
    }

    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        $this ->browserUrl = '?menuid=pracovnipomucky&osoba=' . $this ->filterForm ->getFilterValue('osoba') . '&';
    }


    function renderFilter()
    {
        parent::renderFilter();

        // zobrazi odkazy na zapis vydeje pomucky
        if ($this->pravaZapisPrevzetiVydej->jsouNejakePrava()) {
            echo
                '<div class="margin-top-2">' .

                // vydej jedne pomucky
                '<span class="margin-right-2 padding-right-2">' .
                '<a href="?menuid=pracovnipomucky&action=pomuckyform&formaction=new&vydanipomucky=2&individualni=1">' .
                TextManager::_('vydej_pomucky_zamestnanci') .
                '</a>' .
                '</span>' .
                // hromadny vydej naroku
                '<span class="margin-right-2 padding-right-2">' .
                '<a href="?menuid=pracovnipomucky&action=hromadnyvydej">' .
                TextManager::_('Hromadny_vydej_naroku') .
                '</a>' .
                '</span>' .

                "</div>\n";
        }
    }


    function render()
    {
        parent::render();

        (new Dialog())
            ->setView(new BaseViewObject('ronportal/pomucky/view_detail_pomucky.php'))
            ->setId('detail_pracovni_pomucky')
            ->setTitle('pracovni_pomucka')
            ->addButton(DialogButton::cancelButton('btn_cancel', 'OK'))
            ->render();

//        echo
//            "\n" .
//            '<div id="detail_pracovni_pomucky" class="rondialog" title="' . TextManager::_('pracovni_pomucka') . '">';
//
//        $pracovniPomucka = null;
//        require 'ronportal/pomucky/view_detail_pomucky.php';
//
//        echo
//            '<div class="buttons">' .
//            '<button type="button" class="button cancel">' . TextManager::_('OK') . '</button>' .
//            '</div>';
//
//        echo
//        '</div>';
    }
}
