<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;
use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\StringUtils;

/**
 * Class PracovniPomer
 */
class PracovniPomer
{
    /** @var int */
    public $idPP;
    /** @var string */
    public $rc;
    /** @var string */
    public $osobniCislo;
    /** @var int */
    public $datumNastupu;
    /** @var int|null */
    public $datumUkonceni;
    /** @var string */
    public $jmeno;
    /** @var string */
    public $prijmeni;
    /** @var string */
    public $jmenoPrijmeni;
    /** @var string     bez doplnujicich poli nastavenych v Nastaveni Intrawebu */
    public $jmenoPrijmeniKratke;
    /** @var string */
    public $titul;

    /** @var int        0=externi, 1=zamestnanec, ...  */
    public $zamestnanec;
    /** @var string     Kod typu pracovniho pomeru */
    public $typPP;

    /** @var string */
    public $eMail;
    /** @var int */
    public $idFotografie;

    /** @var string */
    public $telefon;
    /** @var string */
    public $mobil;

    /** @deprecated     Prejmenovano na idPP */
    var $f_IdPP;
    /** @deprecated     Prejmenovano */
    var $f_RC;


    public function __construct()
    {
    }


    public function getJsonData()
    {
        $result = [
            'idPP' => $this ->idPP,
            'osobniCislo' => StringUtils::jsonText($this ->osobniCislo),
            'datumNastupu' => StringUtils::jsonDate($this ->datumNastupu),
            'jmeno' => StringUtils::jsonText($this ->jmeno),
            'prijmeni' => StringUtils::jsonText($this ->prijmeni),
            'jmenoPrijmeni' => StringUtils::jsonText($this ->jmenoPrijmeniKratke)
        ];

        if ($this ->datumUkonceni) {
            $result['datumUkonceni'] = StringUtils::jsonDate($this ->datumUkonceni);
        }

        return $result;
    }


    private static function getSqlSelectPart()
    {
        $sql =
            'SELECT PracovniPomer.Id, PracovniPomer.RC, PracovniPomer.OsobniCislo, ' . // 0..2
            '  PracovniPomer.DatumNastupu, PracovniPomer.DatumUkonceni, ' . // 3..4
            '  Osoba.Jmeno, Osoba.Prijmeni, Osoba.Titul, Osoba.IdFotografie, Osoba.eMail, ' . // 5..9
            '  PracovniPomer.Zamestnanec, PracovniPomer.TypPracovnihoPomeru, ' . // 10..11
            '  Osoba.Telefon, Osoba.Mobil '; // 12-13

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $sql .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1);  // (12)
        }

        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $sql .= ', ' . RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2);  // (12 nebo 13)
        }

        return $sql;
    }


    /**
     * Vraci SQL vyraz pro razeni pracovnich pomeru zamestnance
     * @param string $tableName
     * @return array|mixed|string|string[]
     */
    public static function getRazeniPP(string $tableName = 'PracovniPomer')
    {
        //  nahradi %TABLENAME% skutecnym jmenem tabulky
        $razeni = str_replace('%TABLENAME%', $tableName, RonPortalSettings::getValue('razeni_pp', '%TABLENAME%.OsobniCislo'));

        // odstrani specialni znaky
        $razeni = str_replace(
            [';', '"', "'", '(', ')'],
            ' ',
            $razeni);

        return $razeni ?: $tableName . '.OsobniCislo';
    }


    /**
     * @param Database $database
     * @param array $row
     */
    private function loadRow($database, $row)
    {
        $this->idPP = $this->f_IdPP = (int)$row[0];
        $this->rc = $this->f_RC = $row[1];
        $this->osobniCislo = $row[2];
        $this->datumNastupu = $database ->stringToTimestamp($row[3]);       // $this->f_DatumNastupu =

        $this->datumUkonceni = $row[4] ? $database ->stringToTimestamp($row[4]) : null;

        $this->jmeno = $row[5];
        $this->prijmeni = $row[6];
        $this->titul = $row[7];

        $popis = $row[6] . ' ' . $row[5]; // prijmeni a jmeno
        if ($row[7]) {
            $popis .= ', ' . $row[7]; // prida titul
        }

        $popis = StringUtils::htmlTextConvert($popis);

        $this->jmenoPrijmeniKratke = $popis;

        $this ->idFotografie = $row[8];
        $this ->eMail = $row[9];

        $this ->zamestnanec = $row[10];
        $this ->typPP = $row[11];

        $this->telefon = $row[12];
        $this->mobil = $row[13];

        // pripadne jeste uzivatelsky vybrane pole - nepouziva htmlTextConvert!
        $pomval = 0;
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE1)) {
            $popis = $row[14] . ' ' . $popis;
            $pomval = 1;
        }
        if (RonPortalSettings::getValue(RonPortalSettings::VYBER_OSOB_POLE2)) {
            $popis .= ' ' . $row[14 + $pomval];
        }

        $this->jmenoPrijmeni = $popis;
    }


    /* @param Database $database
     * @param string $filter
     * @param Parameters $filterParams
     * @return bool
     * @throws DBException
     */
    protected function load($database, $filter, $filterParams = null)
    {
        $sql =
            static::getSqlSelectPart() .
            ' FROM PracovniPomer, Osoba' .
            ' WHERE Osoba.RC = PracovniPomer.RC ';

        if ($filter) {
            $sql .= ' AND (' . $filter . ')';
        }

        //$sql .= ' ORDER BY DatumNastupu DESC';
        $sql .= ' ORDER BY ' . self::getRazeniPP();

        $cursor = $database ->getSqlResult($sql, $filterParams);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            return false;
        }

        $this ->loadRow($database, $row);

        return true;
    }



    /**
     * @param Database $database
     * @param string $RC
     * @param string $osobniCislo
     * @param int $datumNastupu
     * @return bool
     * @throws DBException
     */
    public function nactiPPPodleRC($database, $RC, $osobniCislo, $datumNastupu)
    {
        $params = $database ->createParameters();
        return $this->load($database,
            'PracovniPomer.RC = ' . $params->addParameter($RC) .
            '  AND  PracovniPomer.OsobniCislo = ' . $params->addParameter($osobniCislo) .
            '  AND  PracovniPomer.DatumNastupu = ' . $params ->addParameter($database ->dateToParam($datumNastupu)),
            $params);
    }


    /**
     * @param Database $database
     * @param int $id
     * @return bool
     * @throws DBException
     */
    public function nactiPPPodleId($database, $id)
    {
        $params = $database ->createParameters();
        return $this->load($database, 'PracovniPomer.Id = ' . $params ->addParameter((int)$id), $params);
    }


    /**
     * Nacte existujici pracovni pomer pro zadane rc. Pokud je jenAktivni true, tak vyhledava jen aktivni PP v zadanem obdobi.
     * Vraci posledni PP (podle data nastupu), ktery vyhovuje zadanym podminkam, ale prednostne zkusi najit HPP - kdyz nenajde, tak jiny.
     * @param Database $database
     * @param string $rc
     * @param bool $jenAktivni Pokud je true, tak najde jen PP, ktere jsou aktivni v zadanem obdobi
     * @param int $obdobiOd
     * @param int $obdobiDo
     * @return PracovniPomer
     * @throws DBException
     */
    public static function najdiPPOsoby($database, $rc, $jenAktivni = true, $obdobiOd = null, $obdobiDo = null)
    {
        if (!$obdobiOd) {
            $obdobiOd = time();
        }

        if (!$obdobiDo) {
            $obdobiDo = $obdobiOd;
        }

        $result = new static();

        $params = $database ->createParameters();
        $sql =
            'PracovniPomer.RC = ' . $params->addParameter($rc);

        if ($jenAktivni) {
            $sql .=
                ' AND PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter($obdobiDo) .
                ' AND (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter($obdobiOd) . ' OR PracovniPomer.DatumUkonceni IS NULL)';
        }

        if ($result->load($database,
            $sql . " AND PracovniPomer.TypPracovnihoPomeru = 'HPP'",
            $params)) {
            // nasel HPP
            return $result;
        }

        // nenasel HPP - zkusi jine
        if ($result->load($database,
            $sql,
            $params)) {
            return $result;
        }

        return null;
    }


    /**
     * @param Database $database
     * @param int $id Pokud neni zadano vezme se z POST[osoba], GET[osoba] nebo SESSION (v
     * @return PracovniPomer
     * @throws Exception
     * @throws DBException
     */
    public static function nactiPP($database, $id = null)
    {
        if (!isset($id)) {
            $id = isset($_POST['osoba']) ?
                (int)$_POST['osoba'] :
                (isset($_GET['osoba']) ? (int)$_GET['osoba'] : RonPortalSession::vybranaOsoba());
        }

        $result = new static();
        if (!$result ->nactiPPPodleId($database, $id)) {
            throw new Exception('Chyba! Zadany pracovni pomer nebyl nalezen!');
        }

        return $result;
    }


    /**
     * @param PravaNaOsoby $prava
     * @param int $datumOd
     * @param int $datumDo
     * @return bool
     * @throws Exception
     */
    public function lzeZobrazitPP($prava, $datumOd = 0, $datumDo = 0)
    {
        return $prava->lzeZobrazitPP($this->idPP, $datumOd, $datumDo);
    }


    /**
     * @param $database
     * @param $prava
     * @return bool
     * @throws Exception
     */
    public function vyberNactenyPP($database = null, $prava = null)
    {
        // TODO: odstranit parametr $database
        if ($prava) {
            if (!$this->lzeZobrazitPP($prava)) {
                return false;
            }
        }

        RonPortalSession::setVybranaOsoba($this->idPP, $this->rc, $this->osobniCislo, $this->datumNastupu);

        return true;
    }


//    /* @param Database $database
//     * @param string $filter
//     * @param Parameters $filterParams
//     * @return bool
//     * @throws \Ron\DB\DBException
//     */
//    public static function loadList($database, $filter, $filterParams = null)
//    {
//        $sql =
//            self::getSqlSelectPart() .
//            ' FROM PracovniPomer, Osoba' .
//            ' WHERE Osoba.RC = PracovniPomer.RC ';
//
//        if ($filter) $sql .= ' AND (' . $filter . ')';
//
//        $cursor = $database ->getSqlResult($sql, $filterParams);
//
//        $result = [];
//        while ($row = $cursor ->fetch()) {
//            $pp = new static();
//            $pp ->loadRow($database, $row);
//            $result[] = $pp;
//        }
//
//        return $result;
//    }


    /**
     * Vraci seznam pracovnich pomeru, na ktere ma uzivatel $pravo (pokud je zadano) a odpovidaji zadanemu filtru
     * @param Database $database
     * @param string|SqlStatement $filter
     * @param PravaNaOsoby $prava
     * @param int $platnostOd
     * @param int $platnostDo
     * @return PracovniPomer[]
     * @throws DBException
     */
    public static function nactiSeznamPP($database, $filter = null, $prava = null, $platnostOd = null, $platnostDo = null)
    {
        $result = [];

        $sql =
            static::getSqlSelectPart() .
            ' FROM PracovniPomer, Osoba' .
            ' WHERE Osoba.RC = PracovniPomer.RC ';

        $params = null;

        if ($prava) {
            if ($filter) {
                if ($filter instanceof SqlStatement) {
                    $sql .= ' AND (' . $filter ->sqlText . ')';
                    $params = $filter ->parameters;
                } else {
                    $sql .= ' AND (' . $filter . ')';
                }
            }

            $sqlFilter = $prava ->sqlFiltr('PracovniPomer', $platnostOd, $platnostDo);
            if ($sqlFilter) {
                $sql .= ' AND (' . $sqlFilter . ')';
            }
        } else if ($platnostOd) {
            if (!$platnostDo) {
                $platnostDo = $platnostOd;
            }
            // jen PP platne v zadanem obdobi

            $params = $database ->createParameters();

            $sql .=
                ' AND PracovniPomer.DatumNastupu <= ' . $params ->addParameter($database ->dateToParam($platnostDo)) .
                ' AND (PracovniPomer.DatumUkonceni >= ' . $params ->addParameter($database ->dateToParam($platnostOd)) . ' OR PracovniPomer.DatumUkonceni IS NULL)';

            if ($filter) {
                if ($filter instanceof SqlStatement) {
                    $sql .= ' AND (' . $filter ->sqlText . ')';
                    $params ->appendParameters($filter ->parameters);
                } else {
                    $sql .= ' AND (' . $filter . ')';
                }
            }
        } else if ($filter) {
            if ($filter instanceof SqlStatement) {
                $sql .= ' AND (' . $filter ->sqlText . ')';
                $params = $filter ->parameters;
            } else {
                $sql .= ' AND (' . $filter . ')';
            }
        }

        $sql .= ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno, ' . self::getRazeniPP('PracovniPomer');

        $cursor = $database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $pp = new PracovniPomer();
            $pp-> loadRow($database, $row);

            $result[] = $pp;
        }
        $cursor ->close();

        return $result;
    }


    /**
     * Nacte seznam vsech pracovnich pomeru zadane osoby platnych v zadanem obdobi. Pokud neni obdobi zadano, bere aktualne platne
     * @param Database $database
     * @param string $rc
     * @param int $platnostOd
     * @param int $platnostDo
     * @return PracovniPomer[]
     * @throws DBException
     */
    public static function nactiPPOsoby($database, $rc, $platnostOd = null, $platnostDo = null)
    {
        if (!$platnostOd) {
            $platnostOd = time();
        }
        if (!$platnostDo) {
            $platnostDo = $platnostOd;
        }

        $params = $database ->createParameters();
        $filter = new SqlStatement('Osoba.RC = ' . $params ->addParameter($rc), $params);

        return static::nactiSeznamPP($database, $filter, null, $platnostOd, $platnostDo);
    }


    /**
     * Vraci emailovou adresu zadane osoby
     * @param Database $database
     * @param string $RC
     * @return string
     * TODO: Vracet pole (tj. rozebrat podle stredniku)?
     * @throws DBException
     */
    public static function emailOsoby($database, $RC)
    {
        $params = $database ->createParameters();
        $sql = 'SELECT eMail FROM Osoba WHERE RC =' . $params ->addParameter($RC);
        $cursor = $database ->getSqlResult($sql, $params, 'Chyba pri nacitani emailove adresy zamestnance!');
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * Vraci RC osoby pro zadany pracovni pomer
     * @param Database $database
     * @param int $idPP
     * @return string
     */
    public static function rcOsoby($database, $idPP)
    {
        $params = $database ->createParameters();
        $sql = 'SELECT RC FROM PracovniPomer WHERE Id =' . $params ->addParameter($idPP);
        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }


    /**
     * Porovna 2 osoby podle prijmeni, jmena a osobniho cisla (pro razeni)
     * @param PracovniPomer $osoba1
     * @param PracovniPomer $osoba2
     * @return int
     */
    public static function compare($osoba1, $osoba2)
    {
        $result = StringUtils::compareStrings($osoba1->prijmeni, $osoba2->prijmeni);
        if ($result) {
            return $result;
        }

        $result = StringUtils::compareStrings($osoba1->jmeno, $osoba2->jmeno);
        if ($result) {
            return $result;
        }

        return StringUtils::compareStrings($osoba1->osobniCislo, $osoba2->osobniCislo);
    }
}
