<?php

/**
 * UTF8
 */

namespace RonPortal;

require_once('appconstants.php');

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\Http\App;
use Ron\Http\HtmlHeaders;
use Ron\Http\HttpHeaders;
use Ron\Http\HttpResponse;
use Ron\MessageManager;
use Ron\MVC;
use Ron\MVC\Controller;
use Ron\MVC\Model;
use Ron\RonException;
use Ron\RonPortal\XML\XMLFormController;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Admin\NovinkaPortalu;
use RonPortal\Controller\LoginController;
use RonPortal\Controller\PasswordProtectionController;
use RonPortal\Controller\UserPageController;


class RonPortalApp extends App
{

    public function __construct()
    {
        parent::__construct();

        global $php_session_name;
        $this ->sessionName = isset($php_session_name) ? $php_session_name : 'RonPortal';
        $this ->originalLang = 'cz';

        $this ->defaultView = 'view/appview.php';

        global $appVersion;
        HtmlHeaders::getInstance() ->versionId = $appVersion;
        //HtmlHeaders::getInstance() ->originalTitle = TextManager::_('RON_Portal');
    }


    function initStrings()
    {
        // nastavi charset a langdir TextManagera (v _appdbset.php lze prepnout pomoci $charset = 'utf-8';)
        global $charset;
        $chSet = $charset ?? 'windows-1250';
        TextManager::getInstance() ->charSet = $chSet;
        if ($chSet != 'windows-1250') {
            TextManager::getInstance() ->langDir = TextManager::MAIN_LANG_DIR . $chSet . '/';
        }

        parent::initStrings();

        if ($GLOBALS['development']) {
            TextManager::getInstance()->loadStrings('doc_events.ini');
        }

        HtmlHeaders::getInstance() ->originalTitle = TextManager::_('RON_Portal');
    }


    /**
     * @param Database $database
     * @return bool
     */
    function afterDBConnect($database)
    {
        parent::afterDBConnect($database);

//        if (User::userAuthenticated()) { // jmeno a heslo se jiz overovalo
        Prava::$database = $database;
//        }

        RonPortalSettings::getInstance() ->load($database);

        // TODO: uplne odstranit global $connection ze stareho kodu a pak zrusit toto prirazeni
        if ($GLOBALS['very_old_compat_connection'] ?? null) {
            global $connection;
            $connection = $database; // pouze pro kompatibilitu stareho kodu!
        }

        return true;
    }


    /** Volano po inicializaci, pripojeni databaze atd. */
    public function afterInit()
    {
        HtmlHeaders::getInstance() ->setTitle(TextManager::_('RON_Portal'));

        if (isset($_GET['xmlmenuid'])) {
            $_GET['xmlmenuid'] = StringUtils::secureString($_GET['xmlmenuid']);
        }

        global $secure_get_params;
        if ($secure_get_params  &&  is_array($_GET)) {
            foreach ($_GET as $key => $value) {
                $_GET[$key] = StringUtils::secureString($value, $secure_get_params);
            }
        }

        HtmlHeaders::getInstance() ->addHeader('<meta name="apple-mobile-web-app-capable" content="yes">');

        // ikony
        HtmlHeaders::getInstance() ->addHeader("<link rel='shortcut icon' type='image/png' sizes='128x128' href='images/icon_128.png'>", 'shortcut_icon_128');
        $iconFile = RonPortalSettings::getValue('shortcut_icon');
        if ($iconFile) {
            // ikona je nastavena v nastaveni Portalu
            HtmlHeaders::getInstance() ->setIcon($iconFile);
        } else if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_JIDELNU) && !RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_DOCHAZKU)) {
            HtmlHeaders::getInstance() ->setIcon('images/icon_jidelna.png');
        } else {
            HtmlHeaders::getInstance()->setIcon('images/icon_ads.png');
        }

        // css a skripty ronportal
        HtmlHeaders::getInstance()
            ->addCss('css/s_app.css', 'ronportal-css')
            ->addScript('script/ronportal.js')
            ->addScript('script/xmlform.js');

        $lang = TextManager::getInstance() ->translationLang;
        HtmlHeaders::getInstance() ->addScript('', 'translations',
            "?menuid=translations&lang=$lang&ver=" . HtmlHeaders::getInstance() ->versionId);

        if (self::getDebugMode()) {
            HtmlHeaders::getInstance() ->addScript('script/debug.js');
        }

        HtmlHeaders::getInstance()
            ->addBodyAttribute('data-profil-uzivatele', User::getSkupinaUzivatele())
            ->addBodyAttribute('data-login-uzivatele', User::getLoginName());

        // zapinani/vypinani jednotlivych modulu menu
        if (isset($_GET['session@hidesection'])) {
            $sections = RonPortalSession::switchMenuSection($_GET['session@hidesection'], false);
            HttpResponse::setCookie('hidden_menu_sections', $sections, time() + 30000000);
        } else if (isset($_GET['session@showsection'])) {
            $sections = RonPortalSession::switchMenuSection($_GET['session@showsection'], true);
            HttpResponse::setCookie('hidden_menu_sections', $sections, time() + 30000000);
        } else {
            RonPortalSession::hiddenMenuSections(isset($_COOKIE['hidden_menu_sections']) ? $_COOKIE['hidden_menu_sections'] : null);
        }

        // prepinani sekci v hornim menu (uzivatelske menu nebo druhy zpusob prepinani aplikaci)
        if (isset($_GET['session@menusection'])) {
            HttpResponse::setCookie('menusection', $_GET['session@menusection'], time() + 30000000);
            $_SESSION['menusection'] = $_GET['session@menusection'];
        } else {
            if (!isset($_SESSION['menusection'])) $_SESSION['menusection'] = isset($_COOKIE['menusection']) ? $_COOKIE['menusection'] : null;
        }

        if (isset($_GET['terminalvkladani'])) {
            $_SESSION['terminalprovkladani'] = $_GET['terminalvkladani']; // VkladaniDochazkyController
        }
    }

    /**
     * @param string $itemName
     * @param string $itemValue
     * @param string $defaultValue
     */
    public function storeUserSettings($itemName, $itemValue, $defaultValue = null)
    {
        NastaveniUzivatele::setUserSettingsItem($this ->database, strtolower($itemName), $itemValue, $defaultValue);
    }


    public function getUserSettings($itemName, $defaultValue = null)
    {
        return NastaveniUzivatele::getUserSettingsItem($this ->database, strtolower($itemName), $defaultValue);
    }


    public static function getDefaultPortalAction()
    {
        // neni zadana akce - pouzije vychozi
        if (RonPortalSettings::getValue(RonPortalSettings::VYCHOZI_OBRAZOVKA)) {
            return RonPortalSettings::getValue(RonPortalSettings::VYCHOZI_OBRAZOVKA);
        }

        // Dochazka ma prednost, takze predchozi nastaveni pripadne "preplacne"
        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_DOCHAZKU)) {
            if (Prava::jsouNejakePrava('prohlizeni')) {
                if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_PUVODNI)) {
                    return 'dochazkazamestnance';
                } else if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_HISTORII_PRUCHODU)) {
                    return 'historiepruchodu';
                } else if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_DENNI_VYSLEDKY)) {
                    return 'dennivysledky';
                } else if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_MESICNI_VYSLEKDY)) {
                    return 'mesicnivysledky';
                } else if (RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_OSOBNI)) {
                    return 'osobniudaje';
                }
            } else if (Prava::jsouNejakePrava('vkladani')) {
                return 'vlozeni';
            }
        }

        // automaticky
        if (RonPortalSettings::getIntValue(RonPortalSettings::POVOLIT_JIDELNU)) {
            return 'jidelnicek';
        }

        return null;
    }


    public function getAction()
    {
        $action = parent::getAction();

        if ($action) return $action;

        if (User::isAuthenticated()) {
            return self::getDefaultPortalAction();
        }

        return null;
    }



    /**
     * Vraci true, pokud je zadana akce chranena heslem a heslo jeste nebylo zadano, nebo skoncila platnost
     * @param string $action
     * @return bool
     */
    public static function isPasswordProtected($action)
    {
        $settingsStr = RonPortalSettings::getValue('protected_menuid');
        if (!$settingsStr) return false;

        // heslo jeste nebylo zadano, nebo uz neni platne
        $actions = explode(';', $settingsStr);
        if (!in_array($action, $actions)) {
            // akce neni chranena
            return false;
        }

        return !RonPortalSession::enablePasswordProtected();
//        if (RonPortalSession::getValue(RonPortalSession::SESSION_PASSWORD_PROTECTION_TIMESTAMP)) {
//            // heslo uz bylo zadano
//            if (!RonPortalSettings::getIntValue('protected_timeout')) {
//                // heslo je platne stale (neni nastavena platnost)
//                return false;
//            }
//
//            if (time() < RonPortalSession::getValue(RonPortalSession::SESSION_PASSWORD_PROTECTION_TIMESTAMP) + 60 * RonPortalSettings::getIntValue('protected_timeout')) {
//                // heslo uz bylo zadano a je jeste platne
//                RonPortalSession::setValue(RonPortalSession::SESSION_PASSWORD_PROTECTION_TIMESTAMP, time()); // prodlouzi platnost hesla
//                return false;
//            }
//        }
//
//        return true;

    }


    public function createRonPortalController($action)
    {
        if (self::isPasswordProtected($action)) {
            return new PasswordProtectionController($this ->database);
        }

        // zkontroluje, jestli existuje uzivatelsky predefinovany controller (usercontroller/<action>.php)
        if ($result = $this ->createControllerForAction($action, 'usercontrollers')) {
            return $result;
        }

        // defaultni akce - soubor controllers/<action>.php
        return parent::createController($action);
    }


    public function createLoginController()
    {
        if ($result = $this ->createControllerForAction('login', 'usercontrollers')) {
            // existuje uzivatelsky predefinovany controller usercontrollers/login.php
            return $result;
        }

        // SE 17.3.2017 - uzivatelske kontrolery prehozeny do usercontrollers
        // TODO: v budoucnu odstranit!
        if ($result = $this ->createControllerForAction('login', 'user')) {
            // existuje uzivatelsky predefinovany controller user/login.php  - jen pro kompatibilitu! ! ! Nepouzivat
            MessageManager::addInfoMessage('Pozor! Umisteni uzivatelskeho controlleru ve slozce user je zastarale - premistete jej do usercontrollers!');
            return $result;
        }

        return new LoginController($this ->database);
    }


    /**
     * @param string $action
     * @return Controller|UserPageController
     * @throws \Exception
     */
    public function createController($action)
    {
        if (User::isAuthenticated()  &&  RonPortalSettings::getLogoutTimer()) {
            if (!RonPortalSession::isValid(RonPortalSettings::getLogoutTimer())) {
                LoginController::logoutUser();
                MessageManager::addErrorMessage(TextManager::_('Probehlo_automaticke_odhlaseni'));
            }
        }

        if (User::isLoggedAs()  &&  User::getPlatnostZastupuDo() !== null  &&  User::getPlatnostZastupuDo() < DateUtils::today()) {
            LoginController::logoutUser();
            MessageManager::addErrorMessage(TextManager::_('Probehlo odhlaseni_platnost_zastupu_skoncila'));
        }

        global $tokenLogin;
        if ($tokenLogin  &&  isset($_GET['token'])) {
            // prihlaseni pomoci tokenu (integrace s jinym systemem, napr. softip
            return $this ->createLoginController();
        }

        global $enableurlidlogin;
        if ($enableurlidlogin  &&  isset($_GET['idlogin'])) {
            // prihlaseni pomoci ID v URL
            return $this ->createLoginController();
        }

        // login
        if ($action === 'logout'  ||  $action === 'changepassword' /* ||  !User::userAuthenticated() */
            || (User::getVynutitZmenuHesla()  &&  $action !== 'download')    // povoli download, aby slo stahnout css
            ||  isset($_GET['loginname']) ) {
            return $this ->createLoginController();
        }

        if (isset($_SESSION['AfterLogin'])  &&  $_SESSION['AfterLogin']) {
            // tento kod se provede jen po prihlaseni (az po redirect)
            unset($_SESSION['AfterLogin']);
            // automaticke zobrazeni novinek - jen admin
            if (Prava::vratHodnotuPrava('admin') == Prava::PRAVA_VSE) {
                if (NovinkaPortalu::jsouNovinky($this->database)) {
                    Model::addData('zobrazitNovinky', 1);
                }

                if (TextManager::getInstance() ->getCharSet() == 'utf-8'  &&  !function_exists('mb_substr')) {
                    MessageManager::addWarning('V PHP neni zapnuto rozsireni mbstring! Pro spavnou funkcnost RonPortalu v UTF8 je toto rozsireni vyzadovano!');
                }
            }
        }

        if ($result = $this ->createRonPortalController($action)) {
            if (!User::userAuthenticated()) {
                // uzivatel neni prihlasen - overi, jestli tento controller umoznuje pouziti bez prihlaseni
                //if (!isset($result ->enableAccessWithoutLogin)  ||  !$result ->enableAccessWithoutLogin  ||  !isset($_GET['menuid'])  ||  !$_GET['menuid']) {  // default action nemuze byt bez prihlaseni, jinak to blbne!
                if (!isset($result ->enableAccessWithoutLogin)  ||  !$result ->enableAccessWithoutLogin) {  // do getAction doplnena kontrola, ze je uzivatel prihlasen, jinak nevraci defaultaction
                    return $this ->createLoginController();
                }
            }

            return $result;
        }

        if (!User::userAuthenticated())  {
            return $this ->createLoginController();
        }

        // mapovani XML = zobrazeni konkretniho xml formulare
        $xmlformid = RonPortalSettings::getMapovaniXML($action);
        if ($xmlformid) {
            require_once 'xml/xmlcontroller.php';
            return new XMLFormController($this ->database);
        }

        // nakonec zkusi uzivatelske stranky nakonfigurovane v nastaveni Portalu
        return new UserPageController($this ->database);
    }


    public function processControllerResult($controller, $action)
    {
//        if (isset($_POST)) {
//            throw new \Exception(count($_POST) . ' / ' . ini_get('max_input_vars'));
//        }
        if (isset($_POST)  &&  count($_POST) >= ini_get('max_input_vars')) {
            if (Prava::jsouNejakePrava('admin')) {
                MessageManager::addWarning('Pocet promennych v POST pravdepodobobne presahl maximalni povoleny pocet! Zvazte upravu nastaveni max_input_vars v php.ini!');
            }
        }
        parent::processControllerResult($controller, $action);

        $setLastSessionAccess = ($controller instanceof MVC\HttpController)
            ? $controller ->enlargeSessionValidity
            : true;

        RonPortalSession::lastSessionAccess($setLastSessionAccess);
    }


    /**
     * @param \Exception $e
     */
    public function catchUnhandledException($e)
    {
        HttpResponse::renderHeaders();

        if (User::userAuthenticated()) {
            Model::addViewObject(new MVC\TextViewObject($e->getMessage(), MVC\TextViewObject::ERROR_CLASS));

            global $development;

            if ($e instanceof RonException  &&  $e ->getResponseCode()) {
                http_response_code($e ->responseCode);
            }

            if ($e instanceof DBException) {
                // chybovou hlasku zobrazi jen v debug rezimu - aby uzivateli nezobrazoval hodnoty z SQL dotazu, napr. RC
                if ($development || self::getDebugMode()) {
                    Model::addViewObject(new MVC\TextViewObject($e->dbMessage, MVC\TextViewObject::ERROR_CLASS));
                    Model::addViewObject(new MVC\TextViewObject('<pre>' . $e->sql . '</pre>'));
                }

                try {
                    Log::addRecordToDBLog($this ->database,
                        SQLUtils::getTableNameFromSQL($e ->sql), 'ERROR', '',
                        $e ->getMessage() . "\n" . $e ->dbMessage . "\n\n" . $e ->sql);

                    error_log($e ->dbMessage . "\n" . $e ->getTraceAsString());
                } catch (\Exception $e) {
                    // ted uz radsi nedela nic
                }
            } else if ($e instanceof RonException) {
                Model::addViewObject(new MVC\TextViewObject($e->getErrorDescription()));
            }

            if ($development || self::getDebugMode()) {
                Model::addViewObject(new MVC\TextViewObject(str_replace("\n", '<br/>', $e->getTraceAsString())));
            }

            Model::addViewObject(new MVC\MessageManagerViewObject());

            require($this ->defaultView);
        } else {
            // uzivatel neni prihlasen
            MessageManager::addErrorMessage($e ->getMessage());

            require('view/login.php');
        }
    }



    public function renderView($action, $view)
    {
        if ($GLOBALS['development'] ?? false) {
            HtmlHeaders::getInstance() ->addScript('script/development.js', 'development_js', '?ver=' . time());
        }


        if (file_exists('s_user.css')) {
            // uzivatelske predefinovani stylu
            HtmlHeaders::getInstance() ->addCss('s_user.css', 's_user.css');
        }

        // css z nastaveni portalu
        HtmlHeaders::getInstance() ->addCss('?menuid=download', 'user_settings_css', '&css&timestamp=' . RonPortalSession::sessionTimestamp() );

        if (User::getUserCss()) {
            // css podle profilu uzivatele
            HtmlHeaders::getInstance() ->addCss(User::getUserCss(), 'user_profile_css', '?timestamp=' . RonPortalSession::sessionTimestamp());
        }

        if (file_exists('user_scripts.js')) {
            // uzivatelske skripty
            HtmlHeaders::getInstance() ->addScript('user_scripts.js', 'user_scripts');
        }

        parent::renderView($action, $view);
    }
}
