<?php
/**
 * stanislav.elbl, 21.02.2021
 */

namespace RonPortal;

class RonPortalSession
{
    const SESSION_DOCHAZKA = 'skryt_dochazku';
    const SESSION_JIDELNA = 'skryt_jidelnu';
    const SESSION_PERSON = 'skryt_personalistiku';

    /** Cas (timestamp), ve kterem bylo zadano heslo pro pristup k chranenym sekcim */
    const SESSION_PASSWORD_PROTECTION_TIMESTAMP = 'pwd_protection_time';

    public static function getValue($name)
    {
        return isset($_SESSION[$name]) ? $_SESSION[$name] : null;
    }


    public static function setValue($name, $value)
    {
        $_SESSION[$name] = $value;
    }


    public static function initSessionTimestamp()
    {
        $_SESSION['sessionTimestamp'] = time();
    }

    /**
     * SessionTimestamp se nastavuje pri prihlaseni
     * @return int
     */
    public static function sessionTimestamp()
    {
        return isset($_SESSION['sessionTimestamp']) ? $_SESSION['sessionTimestamp'] : null;
    }


    public static function getLanguage()
    {
        return self::getValue('language');
    }


    /**
     * Vraci posledni zaregistrovany cas pristupu. Pokud je $setNewValue = true, tak zaroven nastavi aktualni cas.
     * @param bool $setNewValue
     * @return mixed
     */
    public static function lastSessionAccess($setNewValue = false)
    {
        $result = isset($_SESSION['lastSessionAccess']) ? $_SESSION['lastSessionAccess'] : null;
        if($setNewValue) $_SESSION['lastSessionAccess'] = time();

        return $result;
    }

    /**
     * Vraci true, pokud je session jeste platna
     * @param int $logoutTimer  Urcuje maximalni dobu platnosti session v sekundach
     * @return bool
     */
    public static function isValid($logoutTimer)
    {
        if (!$logoutTimer) {
            return true;
        }

        return (self::lastSessionAccess() + $logoutTimer > time());
    }


    /**
     * Vraci true, pokud je umoznen pristup k chranenym sekcim (tj. uzivatel zadal heslo)
     * Zaroven prodlouzi platnost zadaneho hesla.
     */
    public static function enablePasswordProtected()
    {
        if (!self::getValue(self::SESSION_PASSWORD_PROTECTION_TIMESTAMP)) {
            // heslo jeste nebylo zadano
            return false;
        }

        if (!RonPortalSettings::getIntValue('protected_timeout')) {
            // heslo je platne stale (neni nastavena platnost)
            return true;
        }

        if (time() < self::getValue(self::SESSION_PASSWORD_PROTECTION_TIMESTAMP) + 60 * RonPortalSettings::getIntValue('protected_timeout')) {
            // heslo uz bylo zadano a je jeste platne
            self::setValue(self::SESSION_PASSWORD_PROTECTION_TIMESTAMP, time()); // prodlouzi platnost hesla
            return true;
        }

        // heslo uz vyprselo
        return false;
    }


    // menu
    /**
     * Vraci nebo nastavi vypnute sekce menu (seznam kodu oddelenych strednikem)
     * @param string $newValue
     * @return mixed
     */
    public static function hiddenMenuSections($newValue = null)
    {
        $result = $_SESSION['hidden_menu_sections'] ?? null;

        if (isset($newValue)) {
            $_SESSION['hidden_menu_sections'] = $newValue;
        }

        return $result;
    }

    public static function menuSectionIsVisible($section)
    {
        $hiddenSections = self::hiddenMenuSections();
        if (!$hiddenSections) {
            return true;
        }

        $sections = explode(';', $hiddenSections);
        return !in_array($section, $sections);
    }

    /**
     * Zapne nebo vypne (podle parametru $on) zadanou sekci menu. Vraci novy seznam vypnutych sekci
     * @param $section
     * @param $on
     * @return string
     */
    public static function switchMenuSection($section, $on)
    {
        $hiddenSections = self::hiddenMenuSections();

        $sections = explode(';', $hiddenSections ?? '');
        if ($on) {
            // zapne sekci = odstrani ji ze seznamu vypnutych
            $index = array_search($section, $sections);
            if ($index !== false) {
                unset($sections[$index]);
            }
        } else {
            // vypne sekci = prida ji do seznamu vypnutych
            if (!in_array($section, $sections)) {
                $sections[] = $section;
            }
        }

        $result = implode(';', $sections);
        self::hiddenMenuSections($result);
        return $result;
    }


    // filtry
    /**
     * @return int
     * @deprecated
     */
    public static function vybranaOsoba()
    {
        return isset($_SESSION['vybranaosoba']) ? $_SESSION['vybranaosoba'] : null;
    }

    public static function setVybranaOsoba($idPP, $rc, $osobniCislo, $datumNastupu)
    {
        $_SESSION['vybranaosoba'] = $idPP;
//        $_SESSION['osobajmenoprijmeni'] = $jmenoPrijmeni;
        $_SESSION['osobarc'] = $rc;
        $_SESSION['osobaosobnicislo'] = $osobniCislo;
        $_SESSION['osobanastup'] = $datumNastupu;
//        $_SESSION['osobaukonceni'] = $datumUkonceni;
    }

    /**
     * smaze ze session udaje zamestnance
     */
    public static function smazudajezamestnance()
    {
        unset($_SESSION['vybranaosoba']);
//        unset($_SESSION['osobajmenoprijmeni']);
        unset($_SESSION['osobarc']);
        unset($_SESSION['osobaosobnicislo']); // osobni cislo zobrazene osoby
        unset($_SESSION['osobanastup']);      // datum nastupu zobrazene osoby
    }


    /**
     * @return mixed
     * @deprecated  Pouzit ze session jen idPP a zbytek nacist z databaze
     */
//    public static function osobaJmenoPrijmeni()
//    {
//        return $_SESSION['osobajmenoprijmeni'];
//    }

    public static function osobaRC()
    {
        return isset($_SESSION['osobarc']) ?$_SESSION['osobarc'] : null;
    }

    /**
     * @deprecated
     * @return mixed
     */
    public static function osobaOsobniCislo()
    {
        return isset($_SESSION['osobaosobnicislo']) ? $_SESSION['osobaosobnicislo'] : null;
    }

    /**
     * @deprecated
     * @return mixed
     */
    public static function osobaNastup()
    {
        return isset($_SESSION['osobanastup']) ? $_SESSION['osobanastup'] : null;
    }

//    public static function osobaUkonceni()
//    {
//        return $_SESSION['osobaukonceni'];
//    }


//    /**
//     * Priznak, jestli ma uzivatel ve filtru vybrany "vsichni"
//     * @return bool
//     * @deprecated
//     */
//    public static function vsechnyOsoby()
//    {
//        return isset($_SESSION['vsechnyOsoby']) && $_SESSION['vsechnyOsoby'] == true;
//    }

    /**
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @return string
     * @deprecated
     */
    public static function rozdeleni($typRozdeleni = null)
    {
        $typRozdeleniSuffix = ($typRozdeleni ? '_' . $typRozdeleni : '');
        $result = isset($_SESSION["ads_rozdeleni$typRozdeleniSuffix"]) ? $_SESSION["ads_rozdeleni$typRozdeleniSuffix"] : null;
        return ($result === '') ? null : $result;
    }

    /**
     * @param $value
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @deprecated
     */
    public static function setRozdeleni($value, $typRozdeleni = null)
    {
        $_SESSION['ads_rozdeleni' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
    }

    /**
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @return int  0=jen vlastni, 1=vcetne podrizenych, 2=jen podrizene, 3=jen primo podrizene
     * @deprecated
     */
    public static function rozdeleniVcetnePodrizenych($typRozdeleni = null)
    {
        $typRozdeleniSuffix = ($typRozdeleni ? '_' . $typRozdeleni : '');
        return isset($_SESSION["ads_rozdeleni_vcetne_podrizenych$typRozdeleniSuffix"]) ? $_SESSION["ads_rozdeleni_vcetne_podrizenych$typRozdeleniSuffix"] : 0;
    }

    /**
     * @param int $value
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @deprecated
     */
    public static function setRozdeleniVcetnePodrizenych($value, $typRozdeleni = null)
    {
        $_SESSION['ads_rozdeleni_vcetne_podrizenych' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
    }

    /**
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @return string
     * @deprecated
     */
    public static function rozdeleni2($typRozdeleni = null)
    {
        $typRozdeleniSuffix = ($typRozdeleni ? '_' . $typRozdeleni : '');
        $result = isset($_SESSION["ads_rozdeleni2$typRozdeleniSuffix"]) ? $_SESSION["ads_rozdeleni2$typRozdeleniSuffix"] : null;
        return ($result === '') ? null : $result;
    }

    /**
     * @param $value
     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
     * @deprecated
     */
    public static function setRozdeleni2($value, $typRozdeleni = null)
    {
        $_SESSION['ads_rozdeleni2' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
    }

    /**
     * @param $typRozdeleni
     * @return int|mixed
     * @deprecated
     */
    public static function rozdeleni2VcetnePodrizenych($typRozdeleni = null)
    {
        $typRozdeleniSuffix = ($typRozdeleni ? '_' . $typRozdeleni : '');
        return isset($_SESSION["ads_rozdeleni2_vcetne_podrizenych$typRozdeleniSuffix"]) ? $_SESSION["ads_rozdeleni2_vcetne_podrizenych$typRozdeleniSuffix"] : 0;
    }

//    /**
//     * @param int $value
//     * @param int $typRozdeleni nepovinny parametr pro browsery, ktere ve filtru pouzivaji jiny typ rozdeleni
//     * @deprecated
//     */
//    public static function setRozdeleni2VcetnePodrizenych($value, $typRozdeleni = null)
//    {
//        $_SESSION['ads_rozdeleni2_vcetne_podrizenych' . ($typRozdeleni ? '_' . $typRozdeleni : '')] = $value;
//    }


//    /**
//     * @return mixed|null
//     * @deprecated
//     */
//    public static function vybranaJidelna()
//    {
//        return isset($_SESSION['jidelna_jidelna']) ? $_SESSION['jidelna_jidelna'] : null;
//    }

//    /**
//     * @param $value
//     * @return void
//     * @deprecated
//     */
//    public static function setVybranaJidelna($value)
//    {
//        $_SESSION['jidelna_jidelna'] = $value;
//    }



    /**
     * @return int
     * @deprecated
     */
    public static function obdobi()
    {
        return isset($_SESSION['vybraneobdobi']) ? $_SESSION['vybraneobdobi'] : null;
    }

    public static function setObdobi($pocatek, $konec)
    {
        $_SESSION['vybraneobdobi'] = $pocatek;
        $_SESSION['konecvybranehoobdobi'] = $konec;
        //$_SESSION['popisvybranehoobdobi'] = $popis;
        //$_SESSION['vybraneobdobiuzavreno'] = $uzavreno;
    }

//    public static function popisObdobi()
//    {
//        return $_SESSION['popisvybranehoobdobi'];
//    }

//    /**
//     * @return bool
//     * @deprecated
//     */
//    public static function jenObdobi()
//    {
//        // pouze pokud je vyber obdobi nepovinny
//        return isset($_SESSION['filtrjenobdobi']) && $_SESSION['filtrjenobdobi'];
//    }

//    /**
//     * @return mixed|null
//     * @deprecated
//     */
//    public static function konecObdobi()
//    {
//        return isset($_SESSION['konecvybranehoobdobi']) ? $_SESSION['konecvybranehoobdobi'] : null;
//    }

//    public static function obdobiUzavreno()
//    {
//        return $_SESSION['vybraneobdobiuzavreno'];
//    }


//    /**
//     * @param $value
//     * @return void
//     * @deprecated
//     */
//    public static function setJenObdobi($value)
//    {
//        $_SESSION['filtrjenobdobi'] = $value;
//    }


//    /**
//     * @return mixed|null
//     * @deprecated
//     */
//    public static function variantniObdobi()
//    {
//        return isset($_SESSION['ads_variantni']) ? $_SESSION['ads_variantni'] : null;
//    }
//
//    /**
//     * @return mixed|null
//     * @deprecated
//     */
//    public static function variantniObdobiKonec()
//    {
//        return isset($_SESSION['ads_konecvariantnihoobdobi']) ? $_SESSION['ads_konecvariantnihoobdobi'] : null;
//    }

//    /**
//     * @param $zacatek
//     * @param $konec
//     * @return void
//     * @deprecated
//     */
//    public static function setVariantniObdobi($zacatek, $konec)
//    {
//        $_SESSION['ads_variantni'] = $zacatek;
//        $_SESSION['ads_konecvariantnihoobdobi'] = $konec;
//    }


    /**
     * @return int|null
     * @deprecated
     */
    public static function getFiltrovatDatum()
    {
        return isset($_SESSION['filtrjendatumoddo']) ? $_SESSION['filtrjendatumoddo'] : 0;
    }

    /**
     * @return int|null
     * @deprecated
     */
    public static function getDatumOd()
    {
        return isset($_SESSION['datumod']) ? $_SESSION['datumod'] : null;
    }

    /**
     * @return int|null
     * @deprecated
     */
    public static function getDatumDo()
    {
        return isset($_SESSION['datumdo']) ? $_SESSION['datumdo'] : null;
    }

    /**
     * @param $filtrovat
     * @param $datumOd
     * @param $datumDo
     * @return void
     * @deprecated
     */
    public static function setFiltrovatDatum($filtrovat, $datumOd = null, $datumDo = null)
    {
        $_SESSION['filtrjendatumoddo'] = $filtrovat;
        $_SESSION['datumod'] = $datumOd;
        $_SESSION['datumdo'] = $datumDo;
    }
}
