<?php
/** pavel.rudol 9.4.2020 */

namespace RonPortal\Sklad;


use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\MessageManager;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;
use Ron\StringUtils;
use function Ron\ronText;

class FakturaPrijata
{
    public $cisloFaktury;
    public $cisloObjednavky;

    public $datumPrijeti;
    public $datumVystaveni;
    public $datumSplatnosti;

    public $stredisko;
    public $strediskoPopis;
    public $dodavatel;
    public $dodavatelPopis;

    public $cisloFakturyDodavatele;
    public $castka;
    public $mena;
    public $castka_DM;
    public $mena_DM;

    public $datumZaplaceni;

    public $popis;
    public $poznamka;
    public $vyrizujeRC;
    public $vyrizuje;

    public $zapsat = false;

    /** @var array 10 uzivatelskych poli v DB */
    public $uzivatelskaPole;

    /**
     * @param Database $database
     * @param $row
     */
    protected function nactiRadek($database, $row)
    {
        $this->cisloFaktury = $row[0];
        $this->stredisko = $row[1];
        $this->strediskoPopis = $row[23];
        $this->datumPrijeti = $database->stringToTimestamp($row[2]);
        $this->datumVystaveni = $database->stringToTimestamp($row[3]);
        $this->datumSplatnosti = $database->stringToTimestamp($row[4]);
        $this->dodavatel = $row[5];
        $this->dodavatelPopis = $row[24];
        $this->cisloFakturyDodavatele = $row[6];
        $this->castka = $row[7];
        $this->mena = $row[27];
        $this->castka_DM = $row[28];
        $this->mena_DM = $row[29];
        $this->datumZaplaceni = $database->stringToTimestamp($row[8]);
        $this->vyrizujeRC = $row[9];
        $this->popis = $row[10];
        $this->poznamka = $row[11];
        $this->cisloObjednavky = $row[12];
        $this->vyrizuje = $row[26] . ' ' . $row[25];

        $this->uzivatelskaPole = [];
        for ($i = 0; $i < 10; $i++) {
            $this->uzivatelskaPole[$i+1] = $row[13+$i];
        }
    }

    /**
     * @param $index
     * @throws Exception
     */
    public function nactiPost($index)
    {
        $this->stredisko = $_POST['stredisko'];
        $this->dodavatel = $_POST['dodavatel'];

        if (isset($_POST["faktura_cislo_$index"]))
            $this->cisloFaktury = $_POST["faktura_cislo_$index"];

        if (isset($_POST["faktura_cislofakturydodavatele_$index"]))
            $this->cisloFakturyDodavatele = $_POST["faktura_cislofakturydodavatele_$index"];

        if ($_POST["faktura_datum_vystaveni_$index"] !== '')
            $this->datumVystaveni = DateUtils::dateStringToTimestamp($_POST["faktura_datum_vystaveni_$index"]);
        else
            MessageManager::addErrorMessage(ronText('Chybi_datum_vystaveni'), "faktura_datum_vystaveni_$index");
        if ($_POST["faktura_datum_splatnosti_$index"] !== '')
            $this->datumSplatnosti = DateUtils::dateStringToTimestamp($_POST["faktura_datum_splatnosti_$index"]);
        else
            MessageManager::addErrorMessage(ronText('Chybi_datum_splatnosti'), "faktura_datum_splatnosti_$index");
        if ($_POST["faktura_datum_prijeti_$index"] !== '')
            $this->datumPrijeti = DateUtils::dateStringToTimestamp($_POST["faktura_datum_prijeti_$index"]);
        else
            MessageManager::addErrorMessage(ronText('Chybi_datum_prijeti'), "faktura_datum_prijeti_$index");
        if ($_POST["faktura_datum_zaplaceni_$index"] !== '')
            $this->datumZaplaceni = DateUtils::dateStringToTimestamp($_POST["faktura_datum_zaplaceni_$index"]);

        if (isset($_POST["faktura_popis_$index"]))
            $this->popis = $_POST["faktura_popis_$index"];
        if (isset($_POST["faktura_poznamka_$index"]))
            $this->poznamka = $_POST["faktura_poznamka_$index"];
        if (isset($_POST["faktura_castka_$index"]))
            $this->castka = StringUtils::stringToFloat($_POST["faktura_castka_$index"]);

        if ($this->castka)
            $this->zapsat = true;
    }


    /**
     * @param Database $database
     * @param string $cisloObjednavky
     * @throws DBException
     * @throws Exception
     */
    public function zapisFakturu($database, $cisloObjednavky)
    {
        $rec = new RonPortalDBRecord($database, 'FakturaPrijata');
        $rec->logDescription = StringUtils::fromUTF8('Přijatá faktura');

        $cisloFaktury = $rec->addField('cisloFakturyPrijate', '', true, 20);

        if (empty($this->cisloFaktury)) {
            $obdobi = SkladUtils::getObdobiProDatum($database, $this->datumVystaveni);
            $nastaveniMasky = SkladUtils::getNastaveniCiselneRady($database, 'CR_FakturaPrijata', $this->stredisko, $obdobi);

            $cisloFaktury->setNewValue(SQLUtils::getNextRecordSerialCode($database, 'FakturaPrijata', 'cisloFakturyPrijate', $nastaveniMasky['maska'], $nastaveniMasky['min'], $nastaveniMasky['max']));
        }

        $rec->addField('stredisko', '', true, 30)->setNewValue($this->stredisko);

        $rec->addField('datumPrijeti', 'timestamp', true)->setNewValue($this->datumPrijeti);
        $rec->addField('datumVystaveni', 'timestamp', true)->setNewValue($this->datumVystaveni);
        $rec->addField('datumSplatnosti', 'timestamp', true)->setNewValue($this->datumSplatnosti);
        $rec->addField('datumZaplaceni', 'timestamp', true)->setNewValue($this->datumZaplaceni);

        $rec->addField('castka', 'float', true)->setNewValue($this->castka);
        $rec->addField('mena', '', true, 10)->setNewValue($this->mena);
        $rec->addField('vyrizuje', '', true, 15)->setNewValue(User::getRC());
        $rec->addField('popis', '', true, 60)->setNewValue($this->popis);
        $rec->addField('cisloFakturyDodavatele', '', true, 20)->setNewValue($this->cisloFakturyDodavatele);
        $rec->addField('poznamka', 'memo', true)->setNewValue($this->poznamka);
        $rec->addField('dodavatel', '', true, 10)->setNewValue($this->dodavatel);
        $rec->addField('cisloVydaneObjednavky', '', true, 20)->setNewValue($cisloObjednavky);

        if (!empty($this->cisloFaktury)) {
            $params = $database->createParameters();
            $rec->setWhere('cisloFakturyPrijate = ' . $params->addParameter($this->cisloFaktury), $params);
            $rec->loadAndUpdateRecord();
        } else {
            $rec->insertRecord();
        }
    }

    /**
     * @param Database $database
     * @param string $cisloObjednavky
     * @return FakturaPrijata[]|null
     * @throws DBException
     */
    public static function nactiFaktury($database, $cisloObjednavky)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT cisloFakturyPrijate, FakturaPrijata.stredisko, datumPrijeti, datumVystaveni, datumSplatnosti, dodavatel, cisloFakturyDodavatele, " . // 0 - 6
            "   castka, datumZaplaceni, vyrizuje, FakturaPrijata.Popis, {fn_CASTWIDEMEMO(FakturaPrijata.poznamka)}, cisloVydaneObjednavky, " . // 7 - 12
            "   FakturaPrijata.UserText1, FakturaPrijata.UserText2, FakturaPrijata.UserText3, FakturaPrijata.UserText4, FakturaPrijata.UserText5, " . // 13 - 17
            "   FakturaPrijata.UserText6, FakturaPrijata.UserText7, FakturaPrijata.UserText8, FakturaPrijata.UserText9, FakturaPrijata.UserText10, " . // 18 - 22
            "   Stredisko.popis, Firma.nazev1, Osoba.Prijmeni, Osoba.Jmeno, FakturaPrijata.mena, FakturaPrijata.DM_Castka, FakturaPrijata.DM_KodMeny ". // 23 - 29
            "FROM FakturaPrijata JOIN Stredisko ON Stredisko.kod = FakturaPrijata.stredisko
                LEFT OUTER JOIN Firma ON Firma.kodFirmy = FakturaPrijata.dodavatel
                LEFT JOIN Osoba ON Osoba.RC =  FakturaPrijata.Vyrizuje                
            WHERE cisloVydaneObjednavky = " . $params->addParameter($cisloObjednavky);

        $cursor = $database->getSqlResult($sql, $params);

        $seznamFaktur = null;

        while ($row = $cursor->fetch()) {
            $faktura = new static();
            $faktura->nactiRadek($database, $row);
            $seznamFaktur[] = $faktura;
        }

        return $seznamFaktur;
    }

}