<?php
/** pavel.rudol 9.4.2020 */

namespace RonPortal\Sklad;


use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\App;
use RonPortal\Log;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\TypRozdeleni;
use Ron\StringUtils;

class Firma
{
    public $kodFirmy;
    public $nazev1;
    public $nazev2;
    public $mesto;
    public $PSC;
    public $stat;
    public $statPopis;
    public $ulice;
    public $web;
    public $ico;
    public $dic;
    public $odberatel;
    public $dodavatel;
    public $poznamka;
    public $klicovaSlova;
    public $aktivni;

    public $ucet;
    public $kodBanky;
    public $popisBanky;

    /** @var array */
    public $kontaktniOsoba;
    /** @var array */
    public $telefon;
    /** @var array */
    public $e_mail;

    public $uzivatelskaData;

    public $uzivatelskyFiltr;
    /** @var Rozdeleni[] */
    public $rozdeleni;

    /**
     * @param $row
     */
    protected function nactiRadek($row)
    {
        $this->kodFirmy = $row[0];
        $this->nazev1 = $row[1];
        $this->nazev2 = $row[2];
        $this->mesto = $row[3];
        $this->PSC = $row[4];
        $this->stat = $row[5];
        $this->statPopis = $row[28];
        $this->ulice = $row[6];
        $this->web = $row[7];
        $this->ico = $row[8];
        $this->dic = $row[9];
        $this->odberatel = (bool)$row[10];
        $this->dodavatel = (bool)$row[11];
        $this->poznamka = $row[12];
        $this->klicovaSlova = $row[13];
        $this->aktivni = (bool)$row[14];

        $this->ucet = $row[29];
        $this->kodBanky = $row[30];
        $this->popisBanky = $row[31];

        $n = 0;
        for ($i = 1; $i <= 4; $i++) {
            $this->kontaktniOsoba[$i] = $row[15 + $n];
            $this->telefon[$i] = $row[16 + $n];
            $this->e_mail[$i] = $row[17 + $n];
            $n += 3;
        }

        for ($i = 1; $i <= 9; $i++) {
            $this->uzivatelskaData[$i] = $row[32 + ($i - 1)];
        }

        $index = RonPortalSettings::getIntValue('dodavatele_filtrovat_dle_uzivatelska');
        if ($index)
            $this->uzivatelskyFiltr = $row[32 + ($index - 1)];
    }

    /**
     * @param Database $database
     * @param string $kodFirmy
     * @param null $typRozdeleni
     * @return Firma|null
     * @throws DBException
     */
    public static function nactiFirmu($database, $kodFirmy = null, $typRozdeleni = null)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT kodFirmy, nazev1, nazev2, Firma.mesto, Firma.PSC, Firma.stat, Firma.ulice, web, ICO, DIC, jeOdberatel, jeDodavatel, " . // 0 - 11
            "   {fn_CASTWIDEMEMO(poznamka)}, {fn_CASTWIDEMEMO(KlicovaSlova)}, Aktivni, ". // 12 - 14
            "   kontaktniOsoba1, telefon1, e_mail, kontaktniOsoba2, telefon2, e_mail2, kontaktniOsoba3, telefon3, e_mail3, kontaktniOsoba4, platceDPH, telefon4, e_mail4, Stat.Popis, " . // 15 - 28
            "   Ucet, Firma.kodBanky, Banka.nazev, " . // 29 - 31
            "   userData1, userData2, userData3, userData4, userData5, userData6, userData7, userData8, userData9 " . // 32 - 40
            "FROM Firma LEFT OUTER JOIN Stat ON Stat.Kod = Firma.Stat LEFT OUTER JOIN Banka ON Banka.kod = Firma.kodBanky
            WHERE 1 = 1 " .
            "  AND kodFirmy = " . $params->addParameter($kodFirmy);

        $cursor = $database->getSqlResult($sql, $params);

        $firma = null;

        while ($row = $cursor->fetch()) {
            $firma = new static();
            $firma->nactiRadek($row);

            $firma->rozdeleni = self::nactiRozdeleniFirmy($database, $row[0], $typRozdeleni);
        }

        return $firma;
    }

    /**
     * @param Database $database
     * @param string $kodFirmy
     * @param int $typRozdeleni
     * @return Rozdeleni[]|null
     * @throws DBException
     */
    public static function nactiRozdeleniFirmy($database, $kodFirmy, $typRozdeleni)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT Rozdeleni.Kod, Rozdeleni.Popis, TypRozdeleni.Kod, TypRozdeleni.Popis " .
            "FROM Rozdeleni JOIN RozdeleniFirmy RF ON (Rozdeleni.KodTypuRozdeleni = RF.KodTypuRozdeleni AND Rozdeleni.Kod = RF.Rozdeleni) " .
            " JOIN TypRozdeleni ON (TypRozdeleni.Kod = RF.KodTypuRozdeleni) " .
            "WHERE kodFirmy = " . $params->addParameter($kodFirmy) . (!is_null($typRozdeleni) ? (" AND RF.KodTypuRozdeleni = " . $params->addParameter($typRozdeleni)) : "") .
            "  AND TypRozdeleni.PouzitProRozdeleniFirmy = 1 AND RF.PlatnostOd <= " . $params->addParameter($database->dateToParam(DateUtils::today())) .
            "  AND (RF.PlatnostDo >= " . $params->addParameter($database->dateToParam(DateUtils::today())) . " OR RF.PlatnostDo IS NULL) " .
            "ORDER BY TypRozdeleni.Kod";

        $cursor = $database->getSqlResult($sql, $params);

        $seznamRozdeleni = null;

        while ($row = $cursor->fetch()) {
            $typRozdeleniPom = new TypRozdeleni();
            $typRozdeleniPom->kod = $row[2];
            $typRozdeleniPom->popis = $row[3];
            $rozdeleni = new Rozdeleni();
            $rozdeleni->typ = $typRozdeleniPom;
            $rozdeleni->kod = $row[0];
            $rozdeleni->popis = $row[1];
            $seznamRozdeleni[] = $rozdeleni;
        }

        return $seznamRozdeleni;
    }

    /**
     * @param Database $database
     * @param string $kodFirmy
     * @param string $rozdeleni
     * @return mixed|string
     * @throws DBException
     */
    public static function vratEmailDodavatele($database, $kodFirmy, $rozdeleni)
    {
        $params = $database->createParameters();

        $indexUzivatelskehoPole = RonPortalSettings::getIntValue('vydanaobjednavka_prijemce_uzivatelske_pole');
        $indexUzivatelskehoPoleFirmy = RonPortalSettings::getValue('vydanaobjednavka_prijemce_pole_firmy');
        $typRozdeleni = RonPortalSettings::getIntValue('vydanaobjednavka_prijemce_typ_rozdeleni');

        if (($indexUzivatelskehoPole ==  0))
            $indexUzivatelskehoPole = 1;

        $sql =
            "SELECT RF.Uzivatelska{$indexUzivatelskehoPole}, Firma.e_mail, Firma.e_mail2, Firma.e_mail3, Firma.e_mail4 " .
            "FROM Firma LEFT OUTER JOIN RozdeleniFirmy RF ON (Firma.kodFirmy = RF.kodFirmy" . " AND RF.KodTypuRozdeleni = " . $params->addParameter($typRozdeleni) .
            "  AND RF.PlatnostOd <= " . $params->addParameter($database->dateToParam(DateUtils::today())) .
            "  AND (RF.PlatnostDo >= " . $params->addParameter($database->dateToParam(DateUtils::today())) . " OR RF.PlatnostDo IS NULL) " .
            "  AND RF.Rozdeleni = " . $params->addParameter($rozdeleni) . ") " .
            "WHERE Firma.kodFirmy = " . $params->addParameter($kodFirmy);

        if (App::getDebugMode()) {
            Log::addRecordToDBLog($database, 'ObjednavkaVydana', 'UPDATE', StringUtils::fromUTF8('Vydaná objednávka'), StringUtils::fromUTF8('Vyhledani adresy dodavatele:') .
                $sql . "\n" .
                "typ rozdeleni: " . $typRozdeleni . "\n" .
                "rozdeleni: " . $rozdeleni . "\n" .
                "firma: " . $kodFirmy
            );
        }

        $cursor = $database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            if (!is_null($row[0]))
                return $row[0];
            else {
                $indexy = explode(";", $indexUzivatelskehoPoleFirmy);
                $result = '';
                foreach ($indexy as $index) {
                    if ($row[$index] != '') {
                        if ($result != '')
                            $result .= ';';
                        $result .= $row[$index];
                    }
                }
                return $result;
            }
        }

        return '';
    }
}
