<?php
/** pavel.rudol 9.4.2020 */

namespace RonPortal\Sklad;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\FormReportsListView;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\RonPortalSettings;
use RonPortal\System\Report;
use RonPortal\TypRozdeleni;

class FirmaFilterForm extends RonPortalFilterForm
{

    public $stav;

    /**
     * FirmaFilterForm constructor.
     * @param $database
     * @param $action
     * @param $useGet
     * @param $indexUzivatelskehoFiltru
     * @param TypRozdeleni $typRozdeleni
     */
    public function __construct($database, $action, $useGet, $indexUzivatelskehoFiltru, $typRozdeleni)
    {
        parent::__construct($database, $action, $useGet);

        if ($typRozdeleni) {
            ($this->addFilterRozdeleni('rozdeleni_filtr', $typRozdeleni->kod))
                ->setSessionName('rozdeleni_filtr')
                ->setShowVcetnePodrizenych(false)
                ->setPouzitPopisTypuRozdeleni(true);
        }

        $filtrMesto = new FilterFormFilter('mesto', FilterFormFilter::TYPE_OPTIONS);
        $filtrMesto
            ->setCaption(TextManager::_('Mesto'))
            ->setRowId('filterrow_mesto')
            ->setSessionName('firma_mesto')
            ->setOptionsSql(
                'SELECT DISTINCT Mesto, Mesto AS MestoPopis FROM Firma WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Mesto'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('Firma.Mesto = ');
        $this ->addFilter($filtrMesto);

        if ($indexUzivatelskehoFiltru) {
            $filtrUser = new FilterFormFilter('uzivatelskyfiltr', FilterFormFilter::TYPE_OPTIONS);
            $filtrUser
                ->setCaption(RonPortalSettings::getValue('firma_uzivatelska'. $indexUzivatelskehoFiltru))
                ->setRowId('filterrow_userfiltr')
                ->setSessionName('firma_filtr')
                ->setOptionsSql(
                    "SELECT DISTINCT userData{$indexUzivatelskehoFiltru}, userData{$indexUzivatelskehoFiltru} AS Popis 
                                FROM Firma WHERE userData{$indexUzivatelskehoFiltru} IS NOT NULL AND %FILTER% ORDER BY %ORDER%",
                    '1 = 1', '1'
                )
                ->addItem('', TextManager::_('_vse_'))
                ->setSqlFilter("Firma.UserData{$indexUzivatelskehoFiltru} = ");
            $this->addFilter($filtrUser);
        }

        $textFiltr1 = new FilterFormFilter('klicova_slova', FilterFormFilter::TYPE_EDIT);
        $textFiltr1
            ->setCaption(TextManager::_('Klicova_slova'))
            ->setSessionName('firma_klicova_slova');
        $this->addFilter($textFiltr1);

        $textFiltr2 = new FilterFormFilter('nazev', FilterFormFilter::TYPE_EDIT);
        $textFiltr2
            ->setCaption(TextManager::_('Nazev1'))
            ->setSessionName('firma_nazev');
        $this->addFilter($textFiltr2);

        $filterStav = (new FilterFormFilter('stav'))
            ->setSessionName('firma_stav')
            -> addItem( '-1', TextManager::_('Vse'))
            -> addItem( (new FilterFormFilterItem('1', TextManager::_('Firma_aktivni'))) ->setSqlFilter('(Firma.Aktivni = 1)'))
            -> addItem( (new FilterFormFilterItem('0', TextManager::_('Firma_neaktivni'))) ->setSqlFilter('(Firma.Aktivni = 0)'))
            ->setDefaultValue('-1');
        $this ->addFilter($filterStav);
    }

    private function resetFilter()
    {
        if (isset($_GET['resetfilter'])) {
            $_SESSION['firma_mesto'] = null;
            $_SESSION['firma_filtr'] = null;
            $_SESSION['firma_klicova_slova'] = null;
            $_SESSION['firma_nazev'] = null;
            $_SESSION['rozdeleni_filtr'] = null;
        }
    }

    private function renderReports()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';

        echo '</SPAN>';

        $reportList = Report::loadFormReports($this->database, 'Firma');

        $params =
            ['rozdeleni' => isset($_SESSION['rozdeleni_filtr']) ? $_SESSION['rozdeleni_filtr'] : '',
                'mesto' => isset($_SESSION['firma_mesto']) ? $_SESSION['firma_mesto'] : '',
                'klicova-slova' => isset($_SESSION['firma_klicova_slova']) ? $_SESSION['firma_klicova_slova'] : '',
                'nazev' => isset($_SESSION['firma_nazev']) ? $_SESSION['firma_nazev'] : '',
                'aktivni' => $this->stav];

        FormReportsListView::renderReportList($reportList, '?menuid=firma&action=report&' . http_build_query($params));

        echo "</DIV>";
    }

    /**
     * @throws DBException
     */
    private function renderXML()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';

        echo '</SPAN>';

        $params = $this->database->createParameters();
        $sql =
            "SELECT ID, Popis FROM SablonaXMLFormulare WHERE Popis LIKE " . $params->addParameter('NOVA_FIRMA%');

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            $popis = substr($row[1], strpos($row[1], '_', 5)+1);
            echo "<a href=?menuid=xml&record={$row[0]}>{$popis}</a><br />";
        }

        echo "</DIV>";
    }

    function renderFoto()
    {
        echo "<a class='submitbutton margin-left-2' href='?menuid=firma&resetfilter=1'>" . TextManager::_('Objednavka_resetovat_filtr') . "</a>";
    }


    function loadData()
    {
        $this->resetFilter();
        parent::loadData();
    }


    /**
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters();

        $this->renderReports();
        $this->renderXML();
    }


}

class RozdeleniFirmyBrowserField extends BrowserField
{
    private $typRozdeleni;

    /**
     * RozdeleniFirmyBrowserField constructor.
     * @param Database $database
     * @param TypRozdeleni $typRozdeleni
     */
    public function __construct($database, $typRozdeleni)
    {
        parent::__construct($database);

        $this->typRozdeleni = $typRozdeleni->kod;
        $this->header = TextManager::_($typRozdeleni->popis);
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws DBException
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';
        $params = $this->database->createParameters();

        $sql =
            "SELECT Kod, Popis " .
            "FROM Rozdeleni JOIN RozdeleniFirmy RF ON (Rozdeleni.KodTypuRozdeleni = RF.KodTypuRozdeleni AND Rozdeleni.Kod = RF.Rozdeleni) " .
            "WHERE kodFirmy = " . $params->addParameter($rowData['col-kod']) . " AND RF.KodTypuRozdeleni = " . $params->addParameter($this->typRozdeleni) .
            "  AND RF.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
            "  AND (RF.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RF.PlatnostDo IS NULL)";

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            if ($result != '')
                $result .= ', ';

            $result .= "<span title='{$row[1]}'>{$row[0]}</span>";
        }

        return $result;
    }

}

class FirmaBrowser extends RonPortalBrowser
{
    private $indexUzivatelskehoFiltru;
    /** @var TypRozdeleni  */
    private $typRozdeleniKategorie;

    /** @var FirmaFilterForm */
    public $filterForm;

    /**
     * FirmaBrowser constructor.
     * @param $database
     * @param $action
     * @param $indexFiltr
     * @param TypRozdeleni $typRozdeleniKategorie
     */
    public function __construct($database, $action, $indexFiltr, $typRozdeleniKategorie)
    {
        parent::__construct($database, $action);

        $this->indexUzivatelskehoFiltru = $indexFiltr;
        $this->typRozdeleniKategorie = $typRozdeleniKategorie;

        $this->title = TextManager::_('Seznam_firem');
        $this->code = "browserseznamfirem";

        $this->sqlFromPart = 'Firma LEFT OUTER JOIN Stat ON (Stat.kod = Firma.stat)';

        $this->primaryField = 0;

        $this->addField()
            ->setHeaderText('Kod_firmy')
            ->setCode('col-kod')
            ->setLink('?menuid=firma&action=detailfirmy')
            ->setSqlFieldName('kodFirmy')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Nazev1')
            ->setCode('col-nazev1')
            ->setLink('?menuid=firma&action=detailfirmy')
            ->setSqlFieldName('Nazev1')
            ->setOrdering(true);

        $this->addField()
            ->setHeaderText('Nazev2')
            ->setCode('col-nazev2')
            ->setSqlFieldName('Nazev2')
            ->setOrdering(true);

        if ($this->indexUzivatelskehoFiltru) {
            $this->addField()
                ->setHeader(RonPortalSettings::getValue('firma_uzivatelska' . $this->indexUzivatelskehoFiltru))
                ->setCode('col-uzivatelsky-filtr')
                ->setSqlFieldName('userData' . $this->indexUzivatelskehoFiltru)
                ->setOrdering(true);
        }

        $this->addField()
            ->setHeaderText('Mesto')
            ->setCode('col-mesto')
            ->setSqlFieldName('mesto')
            ->setOrdering(true);
        $this->addField()
            ->setHeaderText('Ulice')
            ->setCode('col-ulice')
            ->setSqlFieldName('ulice')
            ->setOrdering(true);
        $this->addField()
            ->setHeaderText('Stat')
            ->setCode('col-stat')
            ->setSqlFieldName('Stat.Popis')
            ->setOrdering(true);
        $this->addField()
            ->setHeaderText('Klicova_slova')
            ->setCode('col-klicova-slova')
            ->setSqlFieldName('KlicovaSlova');
        $this->addField()
            ->setHeaderText('Poznamka')
            ->setCode('col-poznamka')
            ->setSqlFieldName('{fn_CASTWIDEMEMO(poznamka)}');

        if (!is_null($this->typRozdeleniKategorie)) {
            $this->addField(new RozdeleniFirmyBrowserField($database, $this->typRozdeleniKategorie));
        }

        $this->defaultSort = 'col-nazev1';
    }

    function setFilterForm()
    {
        $this->filterForm = new FirmaFilterForm($this->database, $this->action, true, $this->indexUzivatelskehoFiltru, $this->typRozdeleniKategorie);
    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $sql = $this->filterForm->buildSqlFilter($params);


        if (isset($_SESSION['firma_klicova_slova']) && !empty($_SESSION['firma_klicova_slova'])) {
            $sqlKlicovaSlova = '';

            $klicovaSlovaArr = explode(' ', $_SESSION['firma_klicova_slova']);

            foreach ($klicovaSlovaArr as $klicoveSlovo) {
                if (!empty($klicoveSlovo)) {
                    if ($sqlKlicovaSlova != '') {
                        $sqlKlicovaSlova .= ' OR ';
                    }

                    $sqlKlicovaSlova .= " Firma.KlicovaSlova LIKE " . $params->addParameter('%' . $klicoveSlovo . '%');
                }
            }

            if ($sql != '') {
                $sql .= ' AND ';
            }
            $sql .= "(" . $sqlKlicovaSlova . ")";
        }

        if (isset($_SESSION['firma_nazev']) && !empty($_SESSION['firma_nazev'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sql .= "(Firma.nazev1 LIKE " . $params->addParameter('%' . $_SESSION['firma_nazev'] . '%') . ")";
        }

        if (isset($_SESSION['rozdeleni_filtr']) && !empty($_SESSION['rozdeleni_filtr'])) {
            if ($sql != '') {
                $sql .= ' AND ';
            }

            $sql .=
                " EXISTS(SELECT Rozdeleni FROM RozdeleniFirmy WHERE RozdeleniFirmy.kodFirmy = Firma.kodFirmy AND RozdeleniFirmy.KodTypuRozdeleni = " . $params->addParameter($this->typRozdeleniKategorie->kod) .
                "       AND RozdeleniFirmy.Rozdeleni = " . $params->addParameter($_SESSION['rozdeleni_filtr']) . " AND RozdeleniFirmy.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
                "       AND (RozdeleniFirmy.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RozdeleniFirmy.PlatnostDo IS NULL)" .
                ")";
        }

        return new SqlStatement($sql, $params);
    }


}