<?php
/** pavel.rudol 9.4.2020 */

namespace RonPortal\Sklad;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\HttpController;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\Dokumenty\DokumentUploadOutput;
use RonPortal\Dokumenty\DokumentyUtils;
use RonPortal\NeniPravoException;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\System\ReportOutputObject;
use function Ron\ronText;

class ObjednavkyController extends HttpController
{
    /** @var Database */
    private $database;
    private $pravaProhlizeni;
    private $pravaVlozeni;
    private $pravaSchvaleni1;
    private $pravaSchvaleni2;
    private $pravaSchvaleni3;
    private $pravaFakturace;
    private $vydanaObjednavkaFiltrDodavatelTR;

    /* @var int[]|null */
    public static $typRozdeleniVydaneObjednavky = null;

    public static $indexUzivatelskehoFiltruDodavatele;
    public static $nadpisUzivatelskehoFiltruDodavatele;

    /**
     * ObjednavkyController constructor.
     * @param $database
     * @throws Exception
     */
    public function __construct($database)
    {
        $this->database = $database;

        Prava::nactiPravaUzivatele($this->database);

        $this->pravaProhlizeni = Prava::vratPrava('objednavka_vydana_prohl');
        $this->pravaVlozeni = Prava::vratPrava('objednavka_vydana_vlozeni');
        $this->pravaSchvaleni1 = Prava::vratPrava('objednavka_vydana_schvaleni1');
        $this->pravaSchvaleni2 = Prava::vratPrava('objednavka_vydana_schvaleni2');
        $this->pravaSchvaleni3 = Prava::vratPrava('objednavka_vydana_schvaleni3');
        $this->pravaFakturace = Prava::vratPrava('objednavka_vydana_faktura');

        $this->nactiNastaveni();

        switch (RonPortalSettings::getIntValue('vydanaobjednavka_dodavatel_dle_rozdeleni')) {
            case 1 :
                $this->vydanaObjednavkaFiltrDodavatelTR = static::$typRozdeleniVydaneObjednavky[1];
                break;
            case 2 :
                $this->vydanaObjednavkaFiltrDodavatelTR = static::$typRozdeleniVydaneObjednavky[2];
                break;
            case 3 :
                $this->vydanaObjednavkaFiltrDodavatelTR = static::$typRozdeleniVydaneObjednavky[3];
                break;
            case 4 :
                $this->vydanaObjednavkaFiltrDodavatelTR = static::$typRozdeleniVydaneObjednavky[4];
                break;
            case 5 :
                $this->vydanaObjednavkaFiltrDodavatelTR = static::$typRozdeleniVydaneObjednavky[5];
                break;
            default :
                $this->vydanaObjednavkaFiltrDodavatelTR = 0;
        }

        static::$indexUzivatelskehoFiltruDodavatele = RonPortalSettings::getIntValue('dodavatele_filtrovat_dle_uzivatelska');
        static::$nadpisUzivatelskehoFiltruDodavatele = RonPortalSettings::getValue('firma_uzivatelska' . static::$indexUzivatelskehoFiltruDodavatele);
    }

    private function nactiNastaveni()
    {
        $params = $this->database->createParameters();

        $typRozdeleni1Name = 'ObjednavkaVydana_TypRozdeleni1';
        $typRozdeleni2Name = 'ObjednavkaVydana_TypRozdeleni2';
        $typRozdeleni3Name = 'ObjednavkaVydana_TypRozdeleni3';
        $typRozdeleni4Name = 'ObjednavkaVydana_TypRozdeleni4';
        $typRozdeleni5Name = 'ObjednavkaVydana_TypRozdeleni5';

        $sql =
            "SELECT Name, Val FROM SYS_VAR_GLOBAL WHERE " .
            SQLUtils::getINCondition($this->database, 'Name', [$typRozdeleni1Name, $typRozdeleni2Name, $typRozdeleni3Name, $typRozdeleni4Name, $typRozdeleni5Name],$params);

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()) {
            switch ($row[0]) {
                case $typRozdeleni1Name :
                    static::$typRozdeleniVydaneObjednavky[1] = $row[1];
                    break;
                case $typRozdeleni2Name :
                    static::$typRozdeleniVydaneObjednavky[2] = $row[1];
                    break;
                case $typRozdeleni3Name :
                    static::$typRozdeleniVydaneObjednavky[3] = $row[1];
                    break;
                case $typRozdeleni4Name :
                    static::$typRozdeleniVydaneObjednavky[4] = $row[1];
                    break;
                case $typRozdeleni5Name :
                    static::$typRozdeleniVydaneObjednavky[5] = $row[1];
                    break;
            }
        }

    }

    /**
     * @return bool
     */
    protected function jePravo()
    {
        if ($this->pravaProhlizeni->jsouNejakePrava())
            return true;
        if ($this->pravaVlozeni->jsouNejakePrava())
            return true;
        if ($this->pravaSchvaleni1->jsouNejakePrava())
            return true;
        if ($this->pravaSchvaleni2->jsouNejakePrava())
            return true;
        if ($this->pravaSchvaleni3->jsouNejakePrava())
            return true;
        if ($this->pravaFakturace->jsouNejakePrava())
            return true;

        return  false;
    }

    /*private function redirectToForm($cisloObjednavky)
    {
        return new Redirect(null, '?menuid=objednavky&formaction=edit&record=' . $cisloObjednavky);
    }*/

    private function redirectToDetail($cisloObjednavky)
    {
        return new Redirect(null, '?menuid=objednavky&action=detail&record=' . $cisloObjednavky);
    }

    private function redirectToBrowser()
    {
        return new Redirect(null, '?menuid=objednavky');
    }

    /**
     * @return ReportOutputObject
     * @throws DBException
     * @throws Exception
     */
    private function report()
    {
        if (isset($_GET['objednavka'])) {
            $result = new ReportOutputObject();
            $result->loadReportFromGet($this->database, 'VydanaObjednavka', false);

            $result->nastavObdobiOdDo(DateUtils::today(), DateUtils::today());
            $result->stringParams =
                ['CisloObjednavky' => $_GET['objednavka']];

            return $result;
        }

        $vystavilRC = '';
        $schvalovatel1RC = '';
        $schvalovatel2RC = '';
        $schvalovatel3RC = '';

        if (isset($_GET['objednavka_vystavil']) && ($_GET['objednavka_vystavil'] !== '')) {
            $pPomer = PracovniPomer::nactiPP($this->database, $_GET['objednavka_vystavil']);
            $vystavilRC = $pPomer->rc;
        }
        if (isset($_GET['objednavka_schvalovatel1']) && ($_GET['objednavka_schvalovatel1'] !== '')) {
            $pPomer = PracovniPomer::nactiPP($this->database, $_GET['objednavka_schvalovatel1']);
            $schvalovatel1RC = $pPomer->rc;
        }
        if (isset($_GET['objednavka_schvalovatel2']) && ($_GET['objednavka_schvalovatel2'] !== '')) {
            $pPomer = PracovniPomer::nactiPP($this->database, $_GET['objednavka_schvalovatel2']);
            $schvalovatel2RC = $pPomer->rc;
        }
        if (isset($_GET['objednavka_schvalovatel3']) && ($_GET['objednavka_schvalovatel3'] !== '')) {
            $pPomer = PracovniPomer::nactiPP($this->database, $_GET['objednavka_schvalovatel3']);
            $schvalovatel3RC = $pPomer->rc;
        }

        $result = new ReportOutputObject();
        $result->loadReportFromGet($this->database, 'VydaneObjednavky', false);

        $result->nastavObdobiOdDo($_GET['datumod'], $_GET['datumdo']);

        $result->integerParams['TypRozdeleni1'] = self::$typRozdeleniVydaneObjednavky[1];
        $result->integerParams['TypRozdeleni2'] = self::$typRozdeleniVydaneObjednavky[2];
        $result->integerParams['TypRozdeleni3'] = self::$typRozdeleniVydaneObjednavky[3];
        $result->integerParams['TypRozdeleni4'] = self::$typRozdeleniVydaneObjednavky[4];
        $result->integerParams['TypRozdeleni5'] = self::$typRozdeleniVydaneObjednavky[5];
        $result->integerParams['CenaOd'] = $_GET['objednavka_cenaod'];
        $result->integerParams['CenaDo'] = $_GET['objednavka_cenado'];

        $result->integerParams['TypPoleText'] = $_GET['objednavka_typ_pole'];

        $result->integerParams['vse'] = !isset($_GET['objednavka_stav']) ? 1 : 0;
        $result->integerParams['schvalene'] = 0;
        $result->integerParams['schvalene1'] = 0;
        $result->integerParams['schvalene2'] = 0;
        $result->integerParams['schvalene3'] = 0;
        $result->integerParams['neschvalene'] = 0;
        $result->integerParams['vracene'] = 0;
        $result->integerParams['zamitnute'] = 0;
        $result->integerParams['odeslane'] = 0;
        $result->integerParams['dodane'] = 0;
        $result->integerParams['nedodane'] = 0;
        $result->integerParams['vyrizene'] = 0;
        $result->integerParams['vyfakturovane'] = 0;
        $result->integerParams['nevyfakturovane'] = 0;

        if (isset($_GET['objednavka_stav'])) {
            $result->integerParams['schvalene'] = in_array('schvalene', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['schvalene1'] = in_array('schvalene1', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['schvalene2'] = in_array('schvalene2', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['schvalene3'] = in_array('schvalene3', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['neschvalene'] = in_array('neschvalene', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['vracene'] = in_array('vracene', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['zamitnute'] = in_array('zamitnute', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['odeslane'] = in_array('odeslane', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['dodane'] = in_array('dodane', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['nedodane'] = in_array('nedodane', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['vyrizene'] = in_array('vyrizene', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['vyfakturovane'] = in_array('vyfakturovane', $_GET['objednavka_stav']) ? 1 : 0;
            $result->integerParams['nevyfakturovane'] = in_array('nevyfakturovane', $_GET['objednavka_stav']) ? 1 : 0;
        }

        $result->stringParams = [
            'Rozdeleni1' => $_GET['objednavka_rozdeleni'],
            'Rozdeleni2' => $_GET['objednavka_rozdeleni2'],
            'Rozdeleni3' => $_GET['objednavka_rozdeleni3'],
            'Rozdeleni4' => $_GET['objednavka_rozdeleni4'],
            'Rozdeleni5' => $_GET['objednavka_rozdeleni5'],
            'Stredisko' => $_GET['objednavka_stredisko'],
            'Dodavatel' => $_GET['objednavka_dodavatel'],
            'Mena' => $_GET['objednavka_mena'],
            'Text' => $_GET['objednavka_text'],
            'VystavilRC' => $vystavilRC,
            'Schvalovatel1RC' => $schvalovatel1RC,
            'Schvalovatel2RC' => $schvalovatel2RC,
            'Schvalovatel3RC' => $schvalovatel3RC
        ];



        return $result;
    }

    /**
     * @param $cisloObjednavky
     * @return VydanaObjednavkaFormular
     * @throws DBException
     * @throws Exception
     */
    private function detailObjednavky($cisloObjednavky)
    {
        $objednavka = VydanaObjednavka::nactiObjednavku($this->database, $cisloObjednavky);
        $objednavka->readOnly = true;

        if (!$objednavka->cisloObjednavky)
            throw new Exception(StringUtils::fromUTF8('Zadaná objednávka nebyla nalezena'));

        if (!$objednavka->jePravo([$this->pravaProhlizeni, $this->pravaVlozeni, $this->pravaSchvaleni1, $this->pravaSchvaleni2, $this->pravaSchvaleni3, $this->pravaFakturace])) {
            throw new NeniPravoException();
        }

        $f = new VydanaObjednavkaFormular($this->database, $objednavka, '', $this->pravaVlozeni);

        $f->load();

        return $f;
    }

    /**
     * @param int $uroven
     * @param string $cisloObjednavky
     * @param bool $vyzadovatSchvaleni
     * @return Redirect|VydanaObjednavkaFormular
     * @throws DBException
     * @throws Exception
     */
    private function schvaleni($uroven, $cisloObjednavky, $vyzadovatSchvaleni)
    {
        $rec = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);

        $pravo = $this->pravaSchvaleni1;

        switch ($uroven) {
            case 1 :
                $pravo = $this->pravaSchvaleni1;
                break;
            case 2 :
                $pravo = $this->pravaSchvaleni2;
                break;
            case 3 :
                $pravo = $this->pravaSchvaleni3;
                break;
        }

        if (!$rec->jePravo($pravo, false, $uroven))
            RonPortalUtils::neniPravoException();

        if ($rec->zapisSchvaleni($uroven, $vyzadovatSchvaleni, User::getRC()))
            return $this->redirectToDetail($cisloObjednavky);

        return $this->detailObjednavky($cisloObjednavky);
    }

    /**
     * @param $cisloObjednavky
     * @return Redirect|VydanaObjednavkaFormular
     * @throws Exception
     */
    private function odesliSchvaleni($cisloObjednavky)
    {
        $rec = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);

        $pravo = $this->pravaVlozeni;

        if (!$rec->jePravo($pravo))
            RonPortalUtils::neniPravoException();

        if ($rec->predejKeSchvaleni())
            return $this->redirectToDetail($cisloObjednavky);

        return $this->detailObjednavky($cisloObjednavky);
    }

    /**
     * @param $cisloObjednavky
     * @return Redirect|VydanaObjednavkaFormular
     * @throws DBException
     * @throws Exception
     */
    private function vraceni($cisloObjednavky)
    {
        if (!$cisloObjednavky) throw new Exception('Chyba - neni zadano cislo objednavky!');

        if (!isset($_POST['poznamkakschvaleni']) || !$_POST['poznamkakschvaleni']) {
            Model::addData('vraceni', true);
            return $this->detailObjednavky($cisloObjednavky);
        }

        $uroven = (int)$_POST['urovenvraceni'];
        if (!$uroven) {
            // nemelo by nastat
            MessageManager::addErrorMessage('Chyba pri vraceni - neni zadana uroven schvaleni!');
            Model::addData('vraceni', true);
            return $this->detailObjednavky($cisloObjednavky);
        }

        $rec = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);
        if ($rec->zapisVraceni($uroven, $_POST['poznamkakschvaleni'])) {
            return $this->redirectToDetail($cisloObjednavky);
        } else {
            return $this->detailObjednavky($cisloObjednavky);
        }
    }

    /**
     * @param $cisloObjednavky
     * @return Redirect|VydanaObjednavkaFormular
     * @throws DBException
     * @throws Exception
     */
    private function zamitnuti($cisloObjednavky)
    {
        if (!$cisloObjednavky) throw new Exception('Chyba - neni zadano cislo objednavky!');

        if (!isset($_POST['poznamkakschvaleni']) || !$_POST['poznamkakschvaleni']) {
            Model::addData('zamitnuti', true);
            return $this->detailObjednavky($cisloObjednavky);
        }

        $uroven = (int)$_POST['urovenvraceni'];
        if (!$uroven) {
            // nemelo by nastat
            MessageManager::addErrorMessage('Chyba pri zamitnuti - neni zadana uroven schvaleni!');
            Model::addData('zamitnuti', true);
            return $this->detailObjednavky($cisloObjednavky);
        }

        $rec = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);
        if ($rec->zapisZamitnuti($uroven, $_POST['poznamkakschvaleni'])) {
            return $this->redirectToDetail($cisloObjednavky);
        } else {
            return $this->detailObjednavky($cisloObjednavky);
        }
    }

    /**
     * @param $cisloObjednavky
     * @param $uroven
     * @return Redirect|VydanaObjednavkaFormular
     * @throws DBException
     * @throws Exception
     */
    private function nastavSchvalovatele($cisloObjednavky, $uroven)
    {
        if (!$cisloObjednavky) throw new Exception('Chyba - neni zadano cislo objednavky!');

        $varName = "schvalovatel$uroven";

        if (!isset($_POST[$varName]) || !$_POST[$varName]) {
            return $this->detailObjednavky($cisloObjednavky);
        }

        if (!$uroven) {
            // nemelo by nastat
            MessageManager::addErrorMessage('Chyba pri nastaveni schvalovatele - neni zadana uroven schvaleni!');
            return $this->detailObjednavky($cisloObjednavky);
        }

        $osobaIDPP = $_POST[$varName];
        $pp = PracovniPomer::nactiPP($this->database, $osobaIDPP);

        $rec = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);
        if ($rec->nastavSchvalovatele($uroven, $pp->rc)) {
            return $this->redirectToDetail($cisloObjednavky);
        } else {
            return $this->detailObjednavky($cisloObjednavky);
        }
    }

    /**
     * @param $cisloObjednavky
     * @param $dotaz
     * @param bool $redirect
     * @return Redirect|VydanaObjednavkaFormular|void
     * @throws DBException
     * @throws Exception
     */
    private function odesliDotazZadavateli($cisloObjednavky, $dotaz, $redirect = true)
    {
        if (!$cisloObjednavky) throw new Exception('Chyba - neni zadano cislo objednavky!');

        $rec = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);

        $result = $rec->odesliDotazZadavateli($this->pravaFakturace, $dotaz);

        if (is_string($result)) {
            if ($redirect) {
                Model::addData('dotaz', $dotaz);
                return $this->detailObjednavky($cisloObjednavky);
            }
            else
                return;
        }

        if ($result) {
            if ($rec->zrusSchvaleni() && $redirect)
                return $this->redirectToDetail($cisloObjednavky);
            return $this->detailObjednavky($cisloObjednavky);
        }
    }

    /**
     * @param int $cisloObjednavky
     * @param int $idDokumentu
     * @return Redirect
     * @throws Exception
     */
    function smazDokument($cisloObjednavky, $idDokumentu)
    {
        $objednavka = VydanaObjednavka::nactiObjednavku($this->database, $cisloObjednavky);
        if (!$objednavka) {
            throw new Exception('Chyba pri mazani dokumentu objednavky - nelze nacist pozadovanou objednavku!');
        }

        DokumentyVydaneObjednavkyBrowser::zrusDokumentVydaneObjednavky($this->database, $cisloObjednavky, $idDokumentu);

        return $this->redirectToDetail($cisloObjednavky);
    }

    /**
     * @param string $cisloObjednavky
     * @return DokumentUploadOutput
     */
    private function dokumentUpload($cisloObjednavky)
    {
        $result = new DokumentUploadOutput();

        try {
            $objednavka = VydanaObjednavka::nactiObjednavku($this->database, $cisloObjednavky);

            if (!$objednavka->jePravo([$this->pravaProhlizeni, $this->pravaVlozeni, $this->pravaSchvaleni1, $this->pravaSchvaleni2, $this->pravaSchvaleni3])) {
                RonPortalUtils::neniPravoException();
            }

            $id = DokumentyUtils::vlozDokument($this ->database, 'soubor', $_POST['typ'], $_POST['popis'], StringUtils::fromUTF8('Vydaná objednávka'));

            $rec = new RonPortalDBRecord($this ->database, 'DokumentObjednavkyVydane');
            $rec ->logDescription = StringUtils::fromUTF8('Vydaná objednávka');
            $rec ->addField('Id', 'int', false, 0, (int)$id);
            $rec ->addField('cisloVydaneObjednavky', '', false, 20, $cisloObjednavky);
            $rec ->insertRecord();
        } catch (Exception $e) {
            $result ->setError($e ->getMessage());
        }

        return $result;
    }

    /**
     * @param string $cisloObjednavky
     * @return Redirect|VydaneObjednavkyBrowser
     * @throws Exception
     */
    private function zrusObjednavku($cisloObjednavky)
    {
        $objednavka = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);
        if (!$objednavka->jePravo($this->pravaVlozeni, true))
            RonPortalUtils::neniPravoException();

        if ($objednavka->jeSchvalenaObjednavka())
            RonPortalUtils::neniPravoException();

        try {
            $objednavka->deleteRecord();
            return $this->redirectToBrowser();
        } catch (Exception $exception) {
            MessageManager::addErrorMessage($exception);
        }

        return $this->browserObjednavky();
    }

    /**
     * @return VydaneObjednavkyBrowser
     * @throws DBException
     */
    private function browserObjednavky()
    {
        $b = new VydaneObjednavkyBrowser($this->database, 'objednavky', $this->pravaProhlizeni, $this->pravaVlozeni, $this->pravaSchvaleni1, $this->pravaSchvaleni2, $this->pravaSchvaleni3, $this->pravaFakturace);
        $b->load();
        return $b;
    }

    private function odesliObjednavkuDodavateli($cisloObjednavky)
    {
        $rec = new VydanaObjednavkaRecord($this->database, $cisloObjednavky);

        if ($rec->odesliObjednavkuMailem($this->pravaVlozeni)) {
            MessageManager::addInfoMessage(ronText('Objednavka_odeslana'));
            return $this->redirectToDetail($cisloObjednavky);
        }

        return $this->detailObjednavky($cisloObjednavky);
    }

    /**
     * @param string $formaction
     * @return Redirect|VydanaObjednavkaFormular
     * @throws DBException
     * @throws Exception
     */
    public function formularObjednavky($formaction)
    {
        $cisloObjednavky = isset($_GET['record']) ? $_GET['record'] : null;

        if (isset($_POST['dotaz_zadavateli']) && !(empty($_POST['dotaz_zadavateli'])))
            $this->odesliDotazZadavateli($cisloObjednavky, $_POST['dotaz_zadavateli'], false);

        $objednavka = new VydanaObjednavka();
        $action = $formaction;
        switch ($formaction) {
            case 'createtemplatefrom':
            case 'newfromcopy' :
            case 'newfromtemplate' :
                $objednavka = VydanaObjednavka::nactiObjednavku($this->database, $cisloObjednavky);
                if (!$objednavka->jePravo([$this->pravaVlozeni, $this->pravaSchvaleni1, $this->pravaSchvaleni2, $this->pravaSchvaleni3]))
                    RonPortalUtils::neniPravoException();

                $nastaveniKopie = 0;
                if ($formaction == 'createtemplatefrom')
                    $nastaveniKopie = 1;
                if ($formaction == 'newfromtemplate')
                    $nastaveniKopie = 2;

                $objednavka->nastavKopii($nastaveniKopie);
                $action = 'insert';
                break;
            case 'new' :
            case 'newtemplate' :
                if (!$this->pravaVlozeni->jsouNejakePrava())
                    RonPortalUtils::neniPravoException();

                if ($formaction == 'newtemplate')
                    $objednavka->jeSablona = true;

                $objednavka->novyRadekObjednavky();
                $action = 'insert';
                break;
            case 'edit' :
                $objednavka = VydanaObjednavka::nactiObjednavku($this->database, $cisloObjednavky);
                if (!$objednavka->jePravo([$this->pravaVlozeni, $this->pravaSchvaleni1, $this->pravaSchvaleni2, $this->pravaSchvaleni3]))
                    RonPortalUtils::neniPravoException();
                if (!($objednavka->lzeEditovat() || $objednavka->lzeEditovatDodano() || $objednavka->lzeEditovatDatumDodani()))
                    RonPortalUtils::neniPravoException();

                if (empty($objednavka->radky))
                    $objednavka->novyRadekObjednavky();

                if ($objednavka->lzeEditovatDodano()) {
                    $objednavka->poleProEditaci['radek_dodano'] = 2;
                    $objednavka->povolitZmenuDodavatele = false;
                }

                if ($objednavka->lzeEditovatDatumDodani()) {
                    $objednavka->poleProEditaci['termin_dodani'] = 1;
                    $objednavka->poleProEditaci['radek_datum_dodani'] = 1;
                    $objednavka->povolitZmenuDodavatele = false;
                }

                $action = 'save';
                break;
            case 'fakturaedit' :
                $objednavka = VydanaObjednavka::nactiObjednavku($this->database, $cisloObjednavky);
                if (!$objednavka->jePravo($this->pravaFakturace))
                    RonPortalUtils::neniPravoException();
                if (!$objednavka->lzeFakturovat())
                    RonPortalUtils::neniPravoException();
                $objednavka->poleProEditaci = ['vyrizeno'];
                $objednavka->poleProEditaci['radek_dodano'] = 1;
                if ($objednavka->lzeEditovatDodano()) {
                    $objednavka->poleProEditaci['radek_dodano'] = 2;
                }

                if ($objednavka->lzeEditovatDatumDodani()) {
                    $objednavka->poleProEditaci['termin_dodani'] = 1;
                    $objednavka->poleProEditaci['radek_datum_dodani'] = 1;
                }

                $objednavka->povolitZmenuDodavatele = false;
                $action = 'save';
                Model::addData('editacefaktury', true);
                break;
            case 'save' :
                $objednavka = VydanaObjednavka::nactiObjednavku($this->database, $cisloObjednavky);

                if (isset($_POST['editacefaktury'])) {
                    if (!$objednavka->jePravo($this->pravaFakturace))
                        RonPortalUtils::neniPravoException();
                    if (!$objednavka->lzeFakturovat())
                        RonPortalUtils::neniPravoException();
                    Model::addData('editacefaktury', true);
                } else {
                    if (!$objednavka->jePravo([$this->pravaVlozeni, $this->pravaSchvaleni1, $this->pravaSchvaleni2, $this->pravaSchvaleni3]))
                        RonPortalUtils::neniPravoException();
                    if (!($objednavka->lzeEditovat() || $objednavka->lzeEditovatDodano()))
                        RonPortalUtils::neniPravoException();
                }

                $objednavka->nactiPost();

                //22.3.2021 pokud je chyba v poli cena (nesouhlasi fakturovana castka), pak umozni ulozeni, aby bylo mozno schvalit s upravenou cenou
                if (!MessageManager::getInstance()->isError() || MessageManager::getInstance()->isError('cena')) {
                    try {
                        if ($objednavka->aktualizujObjednavku($this->database, $cisloObjednavky)) {
                            MessageManager::getInstance()->clear();
                            if (isset($_POST['editacefaktury'])) {
                                MessageManager::addInfoMessage(ronText('Faktura_vlozeni_ok'));
                            } elseif ($objednavka->jeSablona) {
                                MessageManager::addInfoMessage('Sablona_aktualizace_ok');
                            } else {
                                MessageManager::addInfoMessage(ronText('Objednavka_aktualizace_ok'));
                            }
                            return $this->redirectToDetail($cisloObjednavky);
                        }
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage($exception);
                    }
                }

                break;
            case 'insert' :
                if (!$objednavka->jePravo($this->pravaVlozeni))
                    RonPortalUtils::neniPravoException();

                $objednavka->nactiPost();

                if (!MessageManager::getInstance()->isError()) {
                    try {
                        $cisloObjednavky = $objednavka->vlozObjednavku($this->database);
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage($exception);
                        $cisloObjednavky = false;
                    }
                    if ($cisloObjednavky !== false) {
                        if ($objednavka->jeSablona)
                            MessageManager::addInfoMessage(ronText('Sablona_vlozeni_ok'));
                        else
                            MessageManager::addInfoMessage(ronText('Objednavka_vlozeni_ok'));
                        return $this->redirectToDetail($cisloObjednavky);
                    }
                }
        }

        $f = new VydanaObjednavkaFormular($this->database, $objednavka, $action, $this->pravaVlozeni);

        $f->load();

        return $f;
    }

    /**
     * @param string|null $dodavatel
     * @return JsonOutputObject
     * @throws DBException
     */
    private function seznamZboziDodavatele($dodavatel = null)
    {
        $output = new JsonOutputObject();

        $params = $this->database->createParameters();

        $sql =
            "SELECT Zbozi.Kod, COALESCE(Zbozi.Popis, PopisZboziDodavatel), KodZboziDodavatel, COALESCE(CenaBezDane, Zbozi.JednotkovaCena), CenaSDani, {fn_CASTWIDEMEMO(COALESCE(ZboziFirmy.poznamka, Zbozi.Poznamka))} " .
            "FROM Zbozi LEFT OUTER JOIN ZboziFirmy ON (ZboziFirmy.KodZbozi = Zbozi.Kod AND ZboziFirmy.Dodavatel = 1 ) ";

        if (!empty($dodavatel)) {
            $sql .= "WHERE ZboziFirmy.KodFirmy = " . $params->addParameter($dodavatel);
        }

        $sql .= " ORDER BY Zbozi.Popis";

        $cursor = $this->database->getSqlResult($sql, $params);

        $seznamZbozi = [];

        while ($row = $cursor->fetch()) {
            $zbozi = [];
            $zbozi['kod'] = StringUtils::jsonText($row[0]);
            $zbozi['popis'] = StringUtils::jsonText($row[1]);
            $zbozi['koddodavatele'] = StringUtils::jsonText($row[2]);
            $zbozi['cenabezdane'] = round($row[3] ?? 0, 2);
            $zbozi['cenasdani'] = round($row[4] ?? 0, 2);
            $zbozi['poznamka'] = StringUtils::jsonText($row[5]);
            $seznamZbozi[] = $zbozi;
        }

        $output->addData('items', $seznamZbozi);

        return $output;
    }

    private function seznamDodavatelu($rozdeleni = null)
    {
        $output = new JsonOutputObject();

        $params = $this->database->createParameters();

        $index = self::$indexUzivatelskehoFiltruDodavatele;

        $userField = '';

        if ($index)
            $userField = ", userData{$index} ";

        $sql =
            "SELECT kodFirmy, nazev1, nazev2, {fn_CASTWIDEMEMO(KlicovaSlova)}, {fn_CASTWIDEMEMO(poznamka)} {$userField} FROM Firma WHERE jeDodavatel = 1 AND Aktivni = 1";

        if (!empty($rozdeleni)) {
            $sql .=
                " AND EXISTS(SELECT Rozdeleni FROM RozdeleniFirmy RF WHERE RF.kodFirmy = Firma.kodFirmy AND RF.KodTypuRozdeleni = " . $params->addParameter($this->vydanaObjednavkaFiltrDodavatelTR) .
                "       AND RF.Rozdeleni = " . $params->addParameter($rozdeleni) . " AND RF.PlatnostOd <= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) .
                "       AND (RF.PlatnostDo >= " . $params->addParameter($this->database->dateToParam(DateUtils::today())) . " OR RF.PlatnostDo IS NULL))";
        }

        $sql .= " ORDER BY nazev1";

        $cursor = $this->database->getSqlResult($sql, $params);

        $seznamDodavatelu = [];

        while ($row = $cursor->fetch()) {
            $dodavatel = [
                'kod' => StringUtils::jsonText($row[0]),
                'nazev1' => StringUtils::jsonText($row[1]),
                'nazev2' => StringUtils::jsonText($row[2]),
                'klicova-slova' => StringUtils::jsonText($row[3]),
                'poznamka' => StringUtils::jsonText($row[4])
            ];

            if (self::$indexUzivatelskehoFiltruDodavatele)
                $dodavatel['filtr'] = StringUtils::htmlTextConvert($row[5]);
            $seznamDodavatelu[] = $dodavatel;
        }

        $output->addData('items', $seznamDodavatelu);

        return $output;
    }

    /**
     * @param int $typRozdeleni
     * @param int $index
     * @param string|null $stredisko
     * @return JsonOutputObject
     * @throws DBException
     */
    private function seznamRozdeleni($typRozdeleni, $index, $stredisko = null)
    {
        $output = new JsonOutputObject();

        $cursor = $this->database->getSqlResult(VydanaObjednavkaFormular::getSQLRozdeleni($typRozdeleni, $index, $this->database, $stredisko));

        $seznamRozdeleni = [];

        while ($row = $cursor->fetch()) {
            $rozdeleni = [
                'kod' => StringUtils::jsonText($row[0]),
                'popis' => StringUtils::jsonText($row[1]),
            ];

            $seznamRozdeleni[] = $rozdeleni;
        }

        $output->addData('items', $seznamRozdeleni);

        return $output;
    }

    /**
     * @param string $action
     * @return InternalRedirect|JsonOutputObject|OutputObject|Redirect|ViewObject|DokumentOutputObject|DokumentUploadOutput|VydanaObjednavkaFormular|VydaneObjednavkyBrowser|string|void
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        TextManager::getInstance()->loadStrings('sklad.ini');
        HtmlHeaders::getInstance()
            ->addCss('css/sklad.css')
            ->addScript('script/sklad.js');

        if (!$this->jePravo())
            RonPortalUtils::neniPravoException();

        if (!RonPortalSettings::getIntValue('vydanaobjednavka_filtrovat_stredisko_tr'))
            throw new Exception(StringUtils::fromUTF8('Není nastaven typ rozdělení pro filtrování středisek objednávky!'));

        if (isset($_POST['formaction'])) $formaction = $_POST['formaction'];
        else if (isset($_GET['formaction'])) $formaction = $_GET['formaction'];
        else $formaction = '';

        if ($formaction) {
            return $this->formularObjednavky($formaction);
        }

        $objednavkyAction = isset($_GET['action']) ? $_GET['action'] : '';

        switch ($objednavkyAction) {
            case 'report' :
                return $this->report();
            case 'detail' :
                return $this->detailObjednavky($_GET['record']);
            case 'zbozidodavatele' :
                return $this->seznamZboziDodavatele($_GET['dodavatel']);
            case 'seznamdodavatelu' :
                return $this->seznamDodavatelu($_GET['rozdeleni']);
            case 'seznamrozdeleni' :
                return $this->seznamRozdeleni($_GET['typrozdeleni'], $_GET['index'], $_GET['stredisko']);
            case 'schvalobjednavku1' :
                return $this->schvaleni(1, $_GET['record'], isset($_GET['vyzadovatschvaleni']));
            case 'schvalobjednavku2' :
                return $this->schvaleni(2, $_GET['record'], isset($_GET['vyzadovatschvaleni']));
            case 'schvalobjednavku3' :
                return $this->schvaleni(3, $_GET['record'], isset($_GET['vyzadovatschvaleni']));
            case 'odeslikeschvaleni' :
                return $this->odesliSchvaleni($_GET['record']);

            case 'zrusitobjednavku' :
                return $this->zrusObjednavku($_GET['record']);

            case 'vraceni' :
                return $this->vraceni($_GET['record']);
            case 'zamitnuti' :
                return $this->zamitnuti($_GET['record']);
            case 'nastavschvalovatele' :
                return $this->nastavSchvalovatele($_GET['record'], (int)$_POST['uroven']);
            case 'vyjadrenifaktura' :
                return $this->odesliDotazZadavateli($_GET['record'], $_POST['dotaz_zadavateli']);

            case 'odesliobjednavku' :
                return $this->odesliObjednavkuDodavateli($_GET['record']);

            // dokumenty
            case 'zrusitdokument' :
                return $this ->smazDokument($_GET['cisloobjednavky'], (int)$_GET['record']);

            case 'download' : // download dokumentu
                return DokumentyVydaneObjednavkyBrowser::downloadDokument($this ->database, $_GET['cisloobjednavky'], (int)$_GET['record']);
            case 'dokumentupload' :
                return $this->dokumentUpload($_GET['cisloobjednavky']);

            default :
                return $this->browserObjednavky();
        }
    }

}
