<?php
/**
 * stanislav.elbl, 07.09.2022 11:53
 * Dialog pro zobrazeni detailu skoleni v seznamu skoleni (skolenibrowser.php)
 */

namespace RonPortal\Skoleni;

use Ron\Controls\FormField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;


class DatumSkoleniFormField extends FormField
{
    public function loadJsonValue($database, $rowData)
    {
        if ($rowData['col-potvrzeni'] == 3  ||  $rowData['col-potvrzeni'] == 4  ||  $rowData['col-potvrzeni'] >= 7) {
            //absolvovano uspesne   absolvovano neuspesne   neucast, prominuto
            return parent::loadJsonValue($database, $rowData);
        }

        return '';
    }
}


class CasSkoleniFormField extends FormField
{
    public function loadJsonValue($database, $rowData)
    {
        if ($rowData['col-caspocatkuskoleni']  ||  $rowData['col-casukonceniskoleni']) {
            return
                date('G:i', $database->stringToTimestamp($rowData['col-caspocatkuskoleni'])) .
                ' - ' .
                date('G:i', $database->stringToTimestamp($rowData['col-casukonceniskoleni']));
        }

        return '';
    }
}

class DownloadICALField extends FormField
{
    public function loadJsonValue($database, $rowData)
    {
        if ($rowData['col-cisloskoleni']) {
            return "<a href='?menuid=skoleni&action=ical&cisloskoleni={$rowData['col-cisloskoleni']}'>" . TextManager::_('Stahnout_do_kalendare') . "</a>";
        }

        return '';
    }
}


class DetailSkoleniDialog extends RonPortalForm
{

    /**
     * DetailSkoleniDialog constructor.
     * @param \Ron\DB\Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'new', true); // dialogMode = true

        $this ->cssClass = 'large-form rondialog ronform form-30-70';
        $this ->code = 'dialogdetailskoleni';
        $this ->readOnly = true;

        $this ->stornoAction = "#";

        $this ->htmlParams = array_merge($this ->htmlParams, [
            'id' => 'detail_skoleni_dialog',
            //'data-ronform-url' => '?menuid=skolenijs&action=loaddata&record=',
            'title' => TextManager::_('podrobnosti_skoleni')
        ]);


        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-skoleni') ->setCaption(TextManager::_('Skoleni_absolvovane_caption'));

        $this ->addField()
            ->setFieldName('DruhSkoleni.Popis') ->setReadOnly(true) ->setCaption('typ_skoleni')
            ->setCode('col-druhskoleni')
            ->setFieldCssClass('form-input ronform-field font-weight-bold');

        $this ->addField(new DatumSkoleniFormField(FormField::FIELD_TYPE_DATE))
            ->setFieldName('GenerovaneSkoleni.DatumSkoleni') ->setReadOnly(true) ->setCaption('Skoleni_datum')
            ->setCode('col-datumskoleni')
            ->setFieldCssClass('form-input ronform-field font-weight-bold');

        $this ->addField()
            ->setFieldName('S_DuvodSkoleni.Popis') ->setReadOnly(true) ->setCaption('Skoleni_duvod')
            ->setCode('col-duvodskoleni')
            ->setFieldCssClass('form-input ronform-field');

//        $platnost = $this->database->stringToTimestamp($row[47]); // GS.PozadovanaPlatnostDo
//        $platnostText = ($platnost == '2147483647' ? ronText('Skoleni_platnost_neomezene') : "");
//        $hodnoty['Skoleni_delka_platnosti'] = StringUtils::jsonText($platnostText);

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('GenerovaneSkoleni.Potvrzeni') ->setReadOnly(true)
            ->setCode('col-potvrzeni');

        $this ->addField()
            ->setFieldName('S_StavPotvrzeni.Popis') ->setReadOnly(true) ->setCaption('skoleni_vysledek')
            ->setCode('col-potvrzeni-popis')
            ->setFieldCssClass('form-input ronform-field font-weight-bold');

        $this ->addField()
            ->setFieldName('Skoleni.Kredity') ->setReadOnly(true) ->setCaption('Skoleni_Kredity')
            ->setCode('col-kredity')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('GenerovaneSkoleni.Poznamka') ->setReadOnly(true) ->setCaption('Skoleni_poznamka_skoleni')
            ->setCode('col-poznamka')
            ->setFieldCssClass('form-input ronform-field poznamka');

        $this ->addField(FormField::FIELD_TYPE_COMBO)
            ->setFieldName('DruhSkoleni.Elearning') ->setReadOnly(true) ->setCaption('Skoleni_elearning')
            ->setCode('col-elearning')
            ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')])
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField()
            ->setFieldName('KategorieSkoleni.Popis') ->setReadOnly(true) ->setCaption('Skoleni_kategorie')
            ->setCode('col-kategorie')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('DruhSkoleni.Poznamka') ->setReadOnly(true) ->setCaption('Skoleni_poznamka_typu_skoleni')
            ->setCode('col-druhskoleni-poznamka')
            ->setFieldCssClass('form-input ronform-field poznamka');

        // informace o kurzu
        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCode('header-kurz') ->setCaption(TextManager::_('Skoleni_kurz_caption'));

//        $hodnoty['Skoleni_ICAL'] = StringUtils::jsonText("?menuid=skoleni&action=ical&cisloskoleni=$row[48]");

        $this ->addField()
            ->setFieldName('S_StavSkoleni.Popis') ->setReadOnly(true) ->setCaption('Skoleni_stav_kurzu')
            ->setCode('col-stavskoleni')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField()
            ->setFieldName('Skoleni.CisloSkoleni') ->setReadOnly(true) ->setCaption('Skoleni_CisloSkoleni')
            ->setCode('col-cisloskoleni')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField()
            ->setFieldName('Skoleni.Popis') ->setReadOnly(true) ->setCaption('Skoleni_popis_kurzu')
            ->setCode('col-skoleni-popis')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Skoleni.HodnoceniSkoleni') ->setReadOnly(true) ->setCaption('Skoleni_poznamka')
            ->setCode('col-skoleni-hodnoceniskoleni')
            ->setFieldCssClass('form-input ronform-field poznamka');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Skoleni.ObsahSkoleni') ->setReadOnly(true) ->setCaption('Skoleni_obsah_kurzu')
            ->setCode('col-skoleni-obsahskoleni')
            ->setFieldCssClass('form-input ronform-field poznamka');

        $this ->addField()
            ->setFieldName('S_StavSchvalPrihlNaKurz.Popis') ->setReadOnly(true) ->setCaption('Skoleni_schvaleni_prihlasky')
            ->setCode('col-schvaleniprihlasky')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('Skoleni.DatumPocatkuSkoleni') ->setReadOnly(true) ->setCaption('datum_pocatku_skoleni')
            ->setCode('col-datumpocatkuskoleni')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('Skoleni.DatumUkonceniSkoleni') ->setReadOnly(true) ->setCaption('datum_ukonceni_skoleni')
            ->setCode('col-datumukonceniskoleni')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Skoleni.CasPocatkuSkoleni')
            ->setCode('col-caspocatkuskoleni');
        $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Skoleni.CasUkonceniSkoleni')
            ->setCode('col-casukonceniskoleni');

        $this ->addField(new CasSkoleniFormField())
            ->setFieldName('Cas_skoleni') ->setSqlFieldName(null) ->setReadOnly(true) ->setCaption('Skoleni_cas_skoleni')
            ->setCode('col-cas-skoleni')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField()
            ->setFieldName('Umisteni.Popis') ->setReadOnly(true) ->setCaption('misto_skoleni')
            ->setCode('col-mistoskoleni')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('Skoleni.DelkaSkoleniDnu') ->setReadOnly(true) ->setCaption('delka_skoleni_dnu')
            ->setCode('col-delkaskolenidnu')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('Skoleni.DelkaSkoleniHodin') ->setReadOnly(true) ->setCaption('delka_skoleni_hodin') ->setFormat('timeofday')
            ->setCode('col-delkaskolenihodin')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_INT)
            ->setFieldName('Skoleni.Kapacita') ->setReadOnly(true) ->setCaption('Skoleni_kapacita')
            ->setCode('col-kapacita')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('GenerovaneSkoleni.PoznamkaSchvalovani') ->setReadOnly(true) ->setCaption('Skoleni_poznamka_schvalovani')
            ->setCode('col-poznamkaschvalovani')
            ->setFieldCssClass('form-input ronform-field poznamka');

        $this ->addField()
            ->setFieldName('SpecifikaceKurzu.Popis') ->setReadOnly(true) ->setCaption('Skoleni_specifikace')
            ->setCode('col-specifikace')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField()
            ->setFieldName('Skoleni.Skolitel') ->setReadOnly(true) ->setCaption('skolitel')
            ->setCode('col-skolitel')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField()
            ->setFieldName('Skoleni.Hodnotici') ->setReadOnly(true) ->setCaption('hodnotici')
            ->setCode('col-hodnotici')
            ->setFieldCssClass('form-input ronform-field');

        $this ->addField()
            ->setFieldName('Firma.nazev1') ->setReadOnly(true) ->setCaption('skolici_firma') // TODO: doplnit nazev2?
            ->setCode('col-skolicifirma')
            ->setFieldCssClass('form-input ronform-field');

        $this->addField(new DownloadICALField())
            ->setFieldName('*ICAL') ->setReadOnly(true)
            ->setCode('col-ical')
            ->setFieldCssClass('form-input ronform-field')->setOtherHtmlParams(['data-raw-html' => '1']);

    }


    public function getSqlFromPart()
    {
        return
            'GenerovaneSkoleni' .
            ' INNER JOIN DruhSkoleni ON DruhSkoleni.Kod = GenerovaneSkoleni.DruhSkoleni'.
            ' LEFT JOIN Skoleni ON Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni' .
            ' LEFT JOIN KategorieSkoleni ON KategorieSkoleni.Kod = DruhSkoleni.Kategorie' .
            ' LEFT JOIN Firma ON Skoleni.SkoliciFirma = Firma.kodFirmy' .
            // LEFT JOIN PracovniPomer PP ON (GS.RC = PP.RC)
            ' LEFT JOIN SpecifikaceKurzu ON (SpecifikaceKurzu.Kod = Skoleni.Specifikace)' .
            ' LEFT JOIN S_StavSchvalPrihlNaKurz ON (S_StavSchvalPrihlNaKurz.Kod = GenerovaneSkoleni.SchvaleniPrihlasky)' .
            ' LEFT JOIN S_DuvodSkoleni ON (S_DuvodSkoleni.Kod = GenerovaneSkoleni.DuvodSkoleni)' .
            //LEFT JOIN S_AnoNe ON (D.Elearning = S_AnoNe.Kod)
            ' LEFT JOIN S_StavSkoleni ON (S_StavSkoleni.Kod = Skoleni.StavSkoleni)' .
            ' LEFT JOIN Umisteni ON (Skoleni.MistoSkoleni = Umisteni.Kod)' .
            ' LEFT JOIN S_StavPotvrzeni ON  (S_StavPotvrzeni.Kod = GenerovaneSkoleni.Potvrzeni)';

//                WHERE (GS.CisloSkoleni = Skoleni.CisloSkoleni OR GS.CisloSkoleni IS NULL ) AND GS.DruhSkoleni = " . $params->addParameter($druh) .
//            " AND PP.ID = " . $params->addParameter($idPP) .
//            " AND GS.DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($datum));
    }


    public function getWhereCondition()
    {
        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['idpp']);
        if (!Prava::lzeZobrazitPP('skoleni_prohlizeni', $pp ->idPP)) {
            throw new NeniPravoException();
        }

        //return parent::getWhereCondition();
        $params = $this ->database ->createParameters();

        return new SqlStatement(
            'GenerovaneSkoleni.RC = ' . $params ->addParameter($pp ->rc) .
            ' AND GenerovaneSkoleni.DruhSkoleni = ' . $params ->addParameter(StringUtils::fromUTF8($_GET['druhskoleni'])) .
            ' AND GenerovaneSkoleni.DatumVznikuPovinnosti = ' . $params ->addTimestampParameter((int)$_GET['datum']),
            $params
        );
    }


}
