<?php
/**
 * stanislav.elbl, 19.11.2015, pavel.rudol, 8.11.2017
 * Rozsireni obecneho dotazniku o funkce specificke pro hodnoceni skoleni
 */

namespace RonPortal\Skoleni;

use Exception;
use Ron\DB\DBException;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Hodnoceni\Hodnoceni;
use RonPortal\User;
use function Ron\ronText;


class HodnoceniSkoleni extends Hodnoceni
{
    /** @var string     Skoleni.DruhSkoleni */
    public $typSkoleni;
    /** @var string     Skoleni.Popis */
    public $nazevSkoleni;
    /** @var int        Skoleni.DatumPocatkuSkoleni, Skoleni.CasPocatkuSkoleni */
    public $zacatekSkoleni;
    /** @var int        Skoleni.DatumUkonceniSkoleni, Skoleni.CasUkonceniSkoleni */
    public $konecSkoleni;
    /** @var  int       HodnoceniSkoleni.TypGenerovani: 1 Hodnoceni ucastnika dle typu rozdeleni, 2 Hodnoceni ucastnika zvolenou osobou, 3 Hodnoceni skoleni ucastnikem */
    public $typGenerovaniHodnoceni;

    /**
     * HodnoceniSkoleni constructor.
     * @param $database
     * @param null $id
     * @throws Exception
     */
    public function __construct($database, $id = null)
    {
        parent::__construct($database, $id);

        $this->nactiDetailySkoleni();
    }

    /**
     * @throws Exception
     */
    private function nactiDetailySkoleni()
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT Skoleni.Popis, Skoleni.DruhSkoleni, Skoleni.DatumPocatkuSkoleni, Skoleni.CasPocatkuSkoleni," . // 0..3
            "   Skoleni.DatumUkonceniSkoleni, Skoleni.CasUkonceniSkoleni, HodnoceniSkoleni.TypGenerovani" . // 4, 5, 6
            " FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni" .
            "   ON ((PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni) AND (PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo))" .
            " JOIN Skoleni ON (HodnoceniSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)" .
            " WHERE PripraveneTemaHodnoceni.ID = " . $params->addParameter($this->fId);

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        $this->nazevSkoleni = $row[0];
        $this->typSkoleni = $row[1];
        $this->zacatekSkoleni = $this->database->stringToTimestamp($row[2]) + 3600 * $this->database->stringToTimeOfDay($row[3]);
        $this->konecSkoleni = $this->database->stringToTimestamp($row[4]) + 3600 * $this->database->stringToTimeOfDay($row[5]);
        $this->typGenerovaniHodnoceni = $row[6];
    }

    /**
     * Vyplni u zadaneho dotazniku pole HodnotilRC a Hodnotil (uzivatelske jmeno)
     * Pokud uz je to vyplneno a je tam nekdo jiny nez aktualni uzivatel, tak zobrazi dotaz, jestli chce uzivatel tento dotaznik prevzit
     * @param bool $prevzit
     * @return bool
     * @throws DBException
     * @throws Exception
     */
    function nastavHodnotitele($prevzit = false)
    {
        // TODO: upravit ! ! !
        if (!$this->fId) {
            throw new Exception('Chyba pri nastavovani hodnotitele - neni nacten dotaznik!');
        }

        if ( $this->fXMLSablona) {
            // v XML sablone je mozne zabranit vyplneni (napr. uzivatelska kontrola prav)
            if (!$this->zobrazDotaznikXML(false, false, 'nastavhodnotitele')) {
                return false;
            }
        }

        $params = $this->database->createParameters();

        $sql =
            "UPDATE PripraveneTemaHodnoceni " .
            "  SET HodnotilRC = " . $params->addParameter(User::getRC()) . ", " .
            "   Hodnotici = " . $params->addParameter(User::getLoginName()) . ", " .
            '   DatumHodnoceni = ' . $params->addTimestampParameter(time()) . ', ' .
            '   Stav = 1 ' . // = probiha
            '  WHERE Id = ' . $params->addParameter($this ->fId) .
            '   AND (HodnotilRC IS NULL)';

        if (!$this ->database ->sql($sql, $params)) {
            throw new DBException($this ->database, $sql, 'Chyba pri nastaveni hodnotitele!');
        }

        $params = $this->database->createParameters();

        // Overi, jestli je aktualni uzivatel zapsany jako resitel
        $sql =
            'SELECT PripraveneTemaHodnoceni.HodnotilRC, Osoba.Prijmeni, Osoba.Jmeno, ' .
            '       PripraveneTemaHodnoceni.Hodnotici, PripraveneTemaHodnoceni.DatumHodnoceni, ' .
            '       PripraveneTemaHodnoceni.Stav ' .
            '  FROM PripraveneTemaHodnoceni, Osoba ' .
            ' WHERE Id = ' . $params->addParameter($this->fId) .
            '   AND Osoba.RC = PripraveneTemaHodnoceni.HodnotilRC ';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row) {
            throw new Exception("Chyba pri nastavovani hodnotitele - zadany zaznam neexistuje!");
        }

        // updatne hodnoty
        $this->hodnoticiPrijmeni = $row[1];
        $this->hodnoticiJmeno = $row[2];
        $this->hodnotici = $row[3];
        $this->fDatumHodnoceni = $this->database->stringToTimestamp($row[4]);
        $this->fStav = $row[5];

        if (User::hasRC($row[0])) {
            // OK, uzivatel je nastaveny jako hodnotitel
            return true;
        }

        throw new Exception(TextManager::_('Hodnoceni_ma_jineho_hodnotitele'));
    }


    function zobrazHlavicku()
    {
        require('ronportal/skoleni/view_hlavicka_hodnoceni_skoleni.php');
    }


    function vratHTMLTlacitka($editace, $ulozitstr = null, $odeslatstr = null, $vratitstr = null, $uzavritstr = null)
    {
        if ($this->fPravo != 2) return ''; // pokud nemuze editovat, nic nezobrazi

        $result = '<DIV class="tlacitkahodnoceni">';

        // Tlacitka Odeslat a Upravit
        if ($editace) { //  || $postvalues) {
            if ($ulozitstr != 'null') {
                // tlacitko "odeslat" (ulozi odpovedi, ale nezmeni stav)
                if (!$ulozitstr) $ulozitstr = ronText('form.Store');

                if ($this->fId) $hlpstr = '';
                else $hlpstr = 'disabled';

                $result .= "<INPUT type='submit' class='submitbutton' $hlpstr value='$ulozitstr' />\n";
            }
        } else {
            if (($this->fPravo == 2) && // ma pravo editovat
                ($this->jeEditovatelnyStav())
            ) // neuzavreno
            {
                // moznost upravit ? ? ? - jen pokud mam pravo a neni uzavren
                //echo '<SPAN style="float:right"> ble </SPAN>';
                $result .=
                    '<a href="?menuid=skoleni&#38;action=hodnoceni&#38;hodnoceniaction=editujhodnoceni&#38;record=' . $this->fId . '">' .
                    ronText('Upravit_hodnoceni') .
                    "</a>\n";
            }
        }


        // Tlacitka Uzavrit a Vratit
        if ($this->jeEditovatelnyStav()) // v reseni   // 0=pripraveno, 1=probiha, 2=ukonceno, 3=ukonceno-vraceno, 4=pripraveno (vraceno)
        {
            $result .= '<SPAN style="float:right">';

            $editace = (int)$editace; // prevede praznou hodnotu na 0 - jinak je prazdny parametr javascriptu a nefachci to

            // pokud neni vyplneno vse, nepovoli uzavrit
            // pokud se ale jedna o editaci, tak uzavreni povoli, protoze uzivatel mohl chybejici odpovedi doplnit
            if ($this->fVyplnenoVse || $editace) {
                $btntext = $uzavritstr ?: ronText('Uzavrit_hodnoceni');
                $msgId = 'Opravdu_uzavrit_hodnoceni';

                if ($msgId != 'null') {
                    $result .= '<INPUT type="submit" name="uzavrit" data-ron-confirm-id="' . $msgId . '" class="submitbutton" value="' . $btntext . '" />';
                }
            } else {
                $result .= ronText('Chybi_odpovedi');
            }

            $result .= '</SPAN>';
        }

        $result .= "</DIV>\n";

        return $result;
    }


    /**
     * @param bool $editace
     */
    function zobrazTlacitka($editace)
    {
        echo $this->vratHTMLTlacitka($editace);
    }

//    function vratAkciDotazniku()
//    {
//        return "?menuid=skoleni&action=hodnoceni&record={$this->fId}";
//    }

}
