<?php
/**
 * stanislav.elbl, 13.09.2022 10:41
 */

namespace RonPortal\Skoleni;

use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;


class AktualniStavHodnoceniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        switch ((int)$value) {
            case 3 :
                return TextManager::_('uspesne');

            case 4 :
                return TextManager::_('neuspesne');

            case 7 :
                return TextManager::_('ucast_omluvena');

            case 8 :
                return TextManager::_('ucast_neomluvena');

            default :
                return TextManager::_('bez_nastavene_uspesnosti');
        }
    }
}



class PotvrzeniSkoleniEditField extends AktualniStavHodnoceniField
{
    public $readOnly = false;

    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'readOnly' :
                $this ->{$key} = (int)$value;
                return $this;

            default :
                return parent::loadConfigurationItem($key, $value);
        }
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $hodnota = (int)$rowData['col-potvrzeni'];

        $prava = Prava::vratPrava('skoleni_absolvovani');

        if ($this ->readOnly  ||  !$prava ->lzeZobrazitRC($rowData['col-rc'])) {
            return parent::getViewText($hodnota, $rowNumber, $recordId, $rowData);
        }

        return
            "<select class='hodnoceni-ucastnika-skoleni' name='hodnoceni_{$rowData['col-rc']}'>" .
            '<option value="0" ' . '>' . \Ron\ronText('nepotvrzeno') . '</option>' .
            '<option value="3" ' . ($hodnota == 3 ? 'selected="selected" ':'') . '>' . TextManager::_('absolvovani_uspesne') . '</option>' .
            '<option value="4" ' . ($hodnota == 4 ? 'selected="selected" ':'') . '>' . TextManager::_('absolvovani_neuspesne') . '</option>' .
            '<option value="7" ' . ($hodnota == 7 ? 'selected="selected" ':'') . '>' . TextManager::_('ucast_omluvena') . '</option>' .
            '<option value="8" ' . ($hodnota == 8 ? 'selected="selected" ':'') . '>' . TextManager::_('ucast_neomluvena') . '</option>' .
            '</select>';
    }

}


class SlovniHodnoceniUcastnikaField extends BrowserField
{
    public $readOnly = false;

    public $cols = 40;
    public $rows = 2;


    function loadConfigurationItem($key, $value)
    {
        switch ($key) {
            case 'readOnly' :
            case 'rows' :
            case 'cols' :
                $this ->{$key} = (int)$value;
                return $this;

            default :
                return parent::loadConfigurationItem($key, $value);
        }
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $prava = Prava::vratPrava('skoleni_absolvovani');

        if (!$this ->readOnly  &&  $prava ->lzeZobrazitRC($rowData['col-rc'])) {
            return
                "<textarea name='poznamka_{$rowData['col-rc']}' cols='{$this ->cols}' rows='{$this ->rows}'>" .
                StringUtils::htmlTextConvert($value) .
                '</textarea>';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }


}

class HodnoceniUcastnikuBrowser extends RonPortalBrowser
{
    /** @var string */
    private $cisloSkoleni;

    /** @var Skoleni */
    private $skoleni;


    /**
     * @param Database $database
     * @param string $cisloSkoleni
     */
    function __construct($database, $cisloSkoleni)
    {
        parent::__construct($database, 'skoleni&action=skolenihodnoceni&record=' . StringUtils::htmlTextConvert($cisloSkoleni));

        $this ->cisloSkoleni = $cisloSkoleni;

        $this->code = 'browserhodnoceniucastnikuskoleni';
        $this->title = TextManager::_('hodnoceni_skoleni');

        $this ->skoleni = Skoleni::nactiSkoleni($this ->database, $cisloSkoleni);

        $this ->sqlFromPart =
            'GenerovaneSkoleni' .
            ' INNER JOIN Osoba ON (Osoba.RC = GenerovaneSkoleni.RC)' .
            ' INNER JOIN S_StavPotvrzeni ON S_StavPotvrzeni.Kod = GenerovaneSkoleni.Potvrzeni';

        $this ->sqlWherePart =
            '(GenerovaneSkoleni.Potvrzeni <> 2  AND  GenerovaneSkoleni.Potvrzeni <> 5)'; // vynecha "Zamitnuto" a "Nepotvrzeny nahradnik"
            //'((GenerovaneSkoleni.Potvrzeni < 5) OR (GenerovaneSkoleni.Potvrzeni > 6))';  -  puvodni filtr

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.RC') ->setCode('col-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)   // primarni klic (pro editaci)
            ->setSqlFieldName('GenerovaneSkoleni.DruhSkoleni') ->setCode('col-druhskoleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)   // primarni klic (pro editaci)
            ->setSqlFieldName('GenerovaneSkoleni.DatumVznikuPovinnosti') ->setCode('col-datumvznikupovinnosti') ->setFormat('datetime');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni') ->setCode('col-prijmeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno') ->setCode('col-jmeno');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Titul') ->setCode('col-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-jmeno', 'col-prijmeni', 'col-titul'))
            ->setCode('col-jmeno-prijmeni')
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno') ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC')
            ->setOrdering(true);
        $this ->defaultSort = 'col-jmeno-prijmeni';

        $this ->addField(new AktualniStavHodnoceniField())
            ->setSqlFieldName('GenerovaneSkoleni.Potvrzeni') ->setHeaderText('Stav_skoleni')
            ->setCode('col-potvrzeni')
            ->setOrdering(true);

        $this ->addField(new PotvrzeniSkoleniEditField())
            ->setHeaderText('hodnoceni')
            ->setCode('col-hodnoceni');

        $this ->addField(new SlovniHodnoceniUcastnikaField())
            ->setSqlFieldName('GenerovaneSkoleni.Poznamka') ->setHeaderText('slovni_hodnoceni') ->setFormat('longtext')
            ->setCode('col-poznamka')
            ->setClassName('poznamka') ->setHeaderClass('');
    }


    function setFilterForm()
    {
        // nedela nic
    }


    public function getRowAttributes($values, $className, $rowNumber)
    {
        $className = $className ? $className . ' ' : '';

        switch ((int)$values['col-potvrzeni']) {
            case 3 :
                $className .= 'uspesny';
                break;

            case 4 :    // neuspesne
            case 7 :    // omluven
            case 8 :    // neomluven
                $className .= 'neuspesne';
                break;

            default :
                $className .= 'bez_hodnoceni';
        }

        return
            parent::getRowAttributes($values, $className, $rowNumber) .
            " data-potvrzeni-value='{$values['col-potvrzeni']}'";
    }


    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        return new SqlStatement(
            'GenerovaneSkoleni.CisloSkoleni = ' . $params ->addParameter($this ->cisloSkoleni),
            $params
        );
    }



    function renderFilter()
    {
        MessageManager::getInstance() ->showMessages();

        echo
            '<div class="form middle-form">' .
            '<div class="form-content">';

        echo
            // cislo
            '<div class="row">' .
            ' <div class="col-4 caption">' . TextManager::_('Skoleni_CisloSkoleni') . '</div>' .
            ' <div class="col-8">' . StringUtils::htmlTextConvert($this ->skoleni ->cisloSkoleni) . '</div>' .
            '</div>' .
            // nazev
            '<div class="row">' .
            ' <div class="col-4 caption">' . TextManager::_('Nazev') . '</div>' .
            ' <div class="col-8 font-weight-bold">' . StringUtils::htmlTextConvert($this ->skoleni ->popis) . '</div>' .
            '</div>' .
            // termin
            '<div class="row">' .
            ' <div class="col-4 caption">' . TextManager::_('termin') . '</div>' .
            ' <div class="col-8 font-weight-bold">' .
                StringUtils::format('finedate', $this ->skoleni ->datumPocatkuSkoleni) .
                ($this ->skoleni ->datumUkonceniSkoleni ? ' - ' . StringUtils::format('finedate', $this ->skoleni ->datumUkonceniSkoleni) : '') .
                (($this ->skoleni ->casPocatkuSkoleni || $this ->skoleni ->casUkonceniSkoleni) ? ' ' . StringUtils::format('timeofday', $this ->skoleni ->casPocatkuSkoleni) : '') .
                ($this ->skoleni ->casUkonceniSkoleni ? ' - ' . StringUtils::format('timeofday', $this ->skoleni ->casUkonceniSkoleni) : '') .
            ' </div>' .
            '</div>' .
            // skolitel
            ($this ->skoleni ->skolitel
                ? '<div class="row">' .
                  ' <div class="col-4 caption">' . TextManager::_('skolitel') . '</div>' .
                  ' <div class="col-8">' . StringUtils::htmlTextConvert($this ->skoleni ->skolitel) . '</div>' .
                  '</div>'
                : '') .
            // hodnotitel
            ($this ->skoleni ->hodnotici
                ? '<div class="row">' .
                  ' <div class="col-4 caption">' . TextManager::_('hodnotici') . '</div>' .
                  ' <div class="col-8">' . StringUtils::htmlTextConvert($this ->skoleni ->hodnotici) . '</div>' .
                  '</div>'
                : '') .
            // skolici firma
            ($this ->skoleni ->nazevSkoliciFirmy
                ? '<div class="row">' .
                  ' <div class="col-4 caption">' . TextManager::_('skolici_firma') . '</div>' .
                  ' <div class="col-8">' . StringUtils::htmlTextConvert($this ->skoleni ->nazevSkoliciFirmy) . '</div>' .
                  '</div>'
                : '')
        ;

        // TODO: stary formular zobrazoval jeste
        // Pocet ucastniku   Vsichni / Bez nastavene uspesnosti:   x/y
        // Uspesne / Neuspesne      a/b
        // CSS Filtr  Zobrazit: Vsechny/Uspesne/Neuspesne/Bez nastavene uspesnosti

        echo
            '</div>' .
            "</div>\n";
    }


    public function renderTable()
    {
        echo
            '<form method="post">' .
            '<input type="hidden" name="HodnoceniUcastniku" value="1" />' .
            Csrf::csrfHiddenInput() .
            "\n";

        parent::renderTable();

        echo
            // tlacitka
            '<div>' .
            "<input type='submit' class='submitbutton' value='" . TextManager::_('form.Store') . "' >" .
            "<input type='submit' name='BackButton' class='button' value='" . TextManager::_('ulozit_na_seznam') . "' >" .
            '<input type="button" class="button clickHref" data-ron-href="?menuid=skoleni&action=skolenikurzy" value="' . TextManager::_("Zpet_na_seznam") . '"/>' .
            "</div>\n";

        echo
            "</form>\n";

    }


    public function ulozZmenyHodnoceni()
    {
        $this ->load();

        $prava = Prava::vratPrava('skoleni_absolvovani');

        Csrf::validateCSRFPostParam();

        try {
            $browserRowArray = new BrowserRowArray($this ->fields);

            while ($row = $this->cursor->fetch()) {
                $browserRowArray ->setData($row);

                $rc = $browserRowArray['col-rc'];

                if ($prava ->lzeZobrazitRC($rc)) {
                    switch ((int)$browserRowArray['col-hodnoceni']) {
                        case 3 :
                        case 4 :
                        case 7 :
                        case 8 :
                            $puvodniHodnota = (int)$browserRowArray['col-hodnoceni'];
                            break;

                        default :
                            $puvodniHodnota = 0;
                    }

                    $rec = new GenerovaneSkoleniRecord($this->database, $this);
                    $rec->setWhereSkoleni(
                        $browserRowArray['col-rc'],
                        $browserRowArray['col-druhskoleni'],
                        $this->database->stringToTimestamp($browserRowArray['col-datumvznikupovinnosti'])
                    );

                    /** @var string $rc */
                    $jeZmena = false;
                    if (isset($_POST["hodnoceni_$rc"])) {
                        switch ((int)$_POST["hodnoceni_$rc"]) {
                            case 3 :
                            case 4 :
                            case 7 :
                            case 8 :
                                $novaHodnota = (int)$_POST["hodnoceni_$rc"];
                                break;

                            default :
                                $novaHodnota = 0;
                        }

                        if ($novaHodnota != $puvodniHodnota) {
                            $jeZmena = true;
                            $rec->fPotvrzeni->setNewValue($novaHodnota);
                        }
                    }

                    if (isset($_POST["poznamka_$rc"])) {
                        if ($_POST["poznamka_$rc"] != $browserRowArray['col-poznamka']) {
                            $jeZmena = true;
                            $rec->addMemoField('Poznamka', false, $_POST["poznamka_$rc"]);
                        }
                    }

                    if ($jeZmena && $rec->loadRecord()) {
                        $rec->updateRecord();
                    }
                }
            }
        } catch (\Exception $e) {
            $this ->cursor ->seek(0);
            throw $e;
        }
    }

}
