<?php
/**
 * stanislav.elbl, 08.09.2022 11:06
 */

namespace RonPortal\Skoleni;

use Ron\Controls\FormField;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalUtils;


class PlanovanaCenaSkoleniField extends FormField
{
    /** @var string */
    private $typPolozky;

    /** @var float */
    private $sazbaDPH;

    /**
     * PlanovanaCenaSkoleniField constructor.
     * @param string $typPolozky
     * @param float $sazbaDPH
     */
    public function __construct($typPolozky, $sazbaDPH)
    {
        parent::__construct(self::FIELD_TYPE_FLOAT);

        $this ->typPolozky = $typPolozky;
        $this ->sazbaDPH = $sazbaDPH;

        $this ->fieldCssClass = 'form-control inputFloat prepocet-dph';
        $this ->otherHtmlParams = [
            'data-selector-bez-dph' => "[name='cena_bez_dph_{$this ->typPolozky}']",
            'data-selector-dph' => "#castka_dph_{$this ->typPolozky}",
            'data-sazba-dph' => $this ->sazbaDPH
        ];
    }


    /**
     * @return string
     */
    public function getTypPolozky()
    {
        return $this->typPolozky;
    }


    /**
     * @return float
     */
    public function getSazbaDPH()
    {
        return $this->sazbaDPH;
    }


    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $cenaBezDPH = isset($this ->defaultValue)  &&  $this ->sazbaDPH >= 0
            ? ($this ->defaultValue / (100 + $this ->sazbaDPH) * 100)
            : null;

        $cenaBezDPHStr = isset($cenaBezDPH)
            ? StringUtils::format($this ->format, $cenaBezDPH)
            : '';

        $dphStr = isset($cenaBezDPH)
            ? StringUtils::format($this ->format, $this ->defaultValue - $cenaBezDPH)
            : '';

        echo
            '<div class="row">' .
            '<div class="col-3">' .
            "<input type='text' name='cena_bez_dph_{$this ->typPolozky}' class='{$this ->fieldCssClass}' value='$cenaBezDPHStr' data-selector-s-dph='[name={$this->fieldName}]' data-sazba-dph='{$this ->sazbaDPH}' data-selector-dph='#castka_dph_{$this ->typPolozky}'>" .
            '</div>' .
            "<div class='col-3 align-center'>" . StringUtils::format('number_2', $this ->sazbaDPH) . '%</div>' .
            "<div class='col-3' id='castka_dph_{$this ->typPolozky}'>" . $dphStr . '</div>' .
            '<div class="col-3">';

        parent::showFieldContent($database, StringUtils::format($this ->format, $this ->defaultValue), $canEdit, $insert, $rowData);

        echo
            '</div>' .
            '</div>';
        //return parent::showFieldContent($database, $value, $canEdit, $insert, $rowData); // TODO: Change the autogenerated stub
    }


}


class PlanovaneSkoleniForm extends RonPortalForm
{
    /**
     * @var PlanovanaCenaSkoleniField[] Seznam poli pro editaci predpokladanych cen
     */
    private $fieldsNaklady = [];

    function __construct($database)
    {
        parent::__construct($database);

        $this ->sqlTableName = 'PlanovaneSkoleni';

        $this ->code = 'formplanovaneskoleni';
        $this ->stornoAction = '?menuid=skoleni&action=planovaneskolenikurzy';
        $this ->setXmlEventsName('PLANOVANE_SKOLENI');

        $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
            ->setCaption($this ->isInsert() ? 'Nove_planovane_skoleni' : 'Planovane_skoleni')
            ->setCode('header-skoleni');

        $this ->addField()
            ->setFieldName('CisloSkoleni') ->setCaption('cislo_skoleni') ->setPrimary(true) ->setReadOnly(true)
            ->setCode('field-cisloskoleni')->setRequired(true)
            ->setMaxLength(40);

        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DruhSkoleni') ->setCaption('typ_skoleni')
            ->setCode('field-druhskoleni')
            ->setSelectSql('SELECT Kod, Popis FROM DruhSkoleni WHERE %FILTER% ORDER BY %ORDER%', 'Aktivni = 1', 'Popis')
            ->setRequired(true)
            ->setShowEmptyValue(true) ->setSelectShowCode(true)
            ->setItemsAttributes(['data-popis' => 1])
            ->setOtherHtmlParams(
                $this->isInsert()
                    ?   [
                            'data-copy-target' => '[name="Popis"]',    // pri vkladani kopiruje do popisu popis vybraneho typu skoleni
                            'data-copy-source' => 'popis'
                        ]
                    : []
                );

        $this ->addField()
            ->setFieldName('Popis') ->setCaption('Popis')
            ->setCode('field-popis')->setRequired(true)
            ->setMaxLength(160);


        $muzeSchvalovat = Prava::jsouNejakePrava('skoleni_schvaleni_planu');
        $stavFilter = $muzeSchvalovat
            ? ''
            : 'Kod = -1 AND ';
        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Stav') ->setCaption('stav_skoleni') ->setReadOnly(!$muzeSchvalovat) // pokud nemuze schvalovat, tak readonly
            ->setCode('field-stav') ->setDefaultValue($muzeSchvalovat ? 0 : -1) ->setSelectDataTypeTest(false)
            ->setSelectSql(
                "SELECT Kod, Popis FROM S_StavPlanovanehoSkoleni WHERE ($stavFilter %FILTER%) ORDER BY %ORDER%",
                '1 = 1', 'Kod')
            ->setRequired(true)
            ->setShowEmptyValue(false);

        // pocatek a konec
        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumPocatkuSkoleni') ->setCaption('realizace_od')
            ->setCode('field-datumpocatkuskoleni') ->setRequired(true)
            ->setRowClass('form-row margin-top-3');

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('DatumUkonceniSkoleni') ->setCaption('realizace_do')
            ->setCode('field-datumukonceniskoleni');

        $this ->addField(FormField::FIELD_TYPE_TIME)
            ->setFieldName('CasPocatkuSkoleni') ->setCaption('cas_realizace_od')
            ->setCode('field-caspocatkuskoleni');

        $this ->addField(FormField::FIELD_TYPE_TIME)
            ->setFieldName('CasUkonceniSkoleni') ->setCaption('cas_realizace_do')
            ->setCode('field-casukonceniskoleni');

        // dalsi podrobnosti
        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('DelkaSkoleniDnu') ->setCaption('delka_skoleni_dnu')
            ->setCode('field-delkaskolenidnu')
            ->setRowClass('form-row margin-top-3');

        $this ->addField(FormField::FIELD_TYPE_FLOAT)
            ->setFieldName('DelkaSkoleniHodin') ->setCaption('delka_skoleni_hodin')
            ->setCode('field-delkaskolenihodin');

        $this ->addField(FormField::FIELD_TYPE_INT)
            ->setFieldName('PocetOsob') ->setCaption('pocet_ucastniku')->setRequired(true)->setDefaultValue(0)
            ->setCode('field-pocetosob');

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka') ->setCaption('Poznamka')
            ->setCode('field-poznamka');


        $this ->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('SkoliciFirma') ->setCaption('realizuje_firma')
            ->setCode('field-skolicifirma')
            ->setSelectSql(
                'SELECT KodFirmy, Nazev1, Mesto FROM FIRMA WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Nazev1')
            ->addFieldCssClass('ronselect')
            ->setItemsAttributes(['data-mesto' => 2])
            ->setOtherHtmlParams(['data-columns' => 'text;value;mesto']);


        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Insert_UserName')->setReadOnly(true)
            ->setCode('field-insert_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Insert_Timestamp')->setReadOnly(true)
            ->setCode('field-insert_timestamp');

        $this->addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('Update_UserName')->setUpdateLog(true)
            ->setCode('field-update_username')
            ->setDefaultValue(RonPortalUtils::getLogUserName());

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setFieldName('Update_Timestamp')->setUpdateLog(true)
            ->setCode('field-update_timestamp');

        // Naklady skoleni
        $params = $database ->createParameters();
        if ($this ->isInsert()) {
            $cursor = $database->getSqlResult(
                'SELECT TypPolozky.Kod, TypPolozky.Popis, TypPolozky.SazbaDPH,' .
                '  NULL, NULL, NULL' .
                ' FROM TypPolozky' .
                ' ORDER BY TypPolozky.Popis',
                $params);
        } else {
            $cursor = $database->getSqlResult(
                'SELECT TypPolozky.Kod, TypPolozky.Popis, TypPolozky.SazbaDPH,' .
                '  PlanovanaCenaSkolCelkova.SazbaDPH, PlanovanaCenaSkolCelkova.DPH, PlanovanaCenaSkolCelkova.CenaSDPH' .
                ' FROM TypPolozky LEFT OUTER JOIN PlanovanaCenaSkolCelkova' .
                '  ON PlanovanaCenaSkolCelkova.TypPolozky = TypPolozky.Kod' .
                '  AND PlanovanaCenaSkolCelkova.CisloSkoleni = ' . $params ->addParameter($_GET['record']) .
                ' ORDER BY TypPolozky.Popis',
                $params);
        }

        $first = true;
        while ($row = $cursor ->fetch()) {
            if ($first) {
                $first = false;

                $this ->addField(FormField::FIELD_TYPE_GROUP)
                    ->setCode('group-naklady');

                $this ->addField(FormField::FIELD_TYPE_SECTION_HEADER)
                    ->setCode('header-naklady') ->setCaption('Naklady_planovaneho_kurzu');

                $this ->addField(FormField::FIELD_TYPE_TEXT)
                    ->setCode('field-naklady-nadpis')
                    ->setCaption('typ_polozky')
                    ->setText(
                        '<div class="row">' .
                        '<div class="col-3">' . TextManager::_('cena_bez_dph') . '</div>' .
                        '<div class="col-3 align-center">' . TextManager::_('sazba_dph') . '</div>' .
                        '<div class="col-3">' . TextManager::_('dph') . '</div>' .
                        '<div class="col-3">' . TextManager::_('cena_s_dph') . '</div>' .
                        '</div>'
                    );
            }

            $field = new PlanovanaCenaSkoleniField($row[0], $row[3] ?: $row[2]);
            $this ->fieldsNaklady[] = $field;
            $this ->addField($field)
                ->setFieldName('cena_s_dph_' . $row[0]) ->setSqlFieldName(null)
                ->setCaption($row[1]) ->setDefaultValue(isset($row[5]) ? (double)$row[5] : null)
                ->setCode('field-naklady-' . $row[0])
                ->setFormat('number2');
        }
    }


    function beforeInsertRecord($rec)
    {
        $params = $this->database->createParameters();
        $cursor = $this->database->getSqlResult(
            'SELECT CisloSkoleni FROM PlanovaneSkoleni WHERE CisloSkoleni = ' . $params->addParameter($rec['CisloSkoleni']),
            $params);

        $row = $cursor->fetch();
        $cursor ->close();

        if ($row) {
            throw new \Exception(TextManager::_('cislo_skoleni_existuje'));
        }

        return parent::beforeInsertRecord($rec);
    }


    // TODO: pri editaci zkontrolovat, jestli nesnizil pocet lidi pod pocet jiz prihlasenych?
//    private function pocetOsobNaKurzu()
//    {
//        $params = $this->database->createParameters();
//        $sql = "SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO Where PSO.Potvrzeni = 1 AND PSO.CisloSkoleni = " . $params->addParameter($cisloSkoleni);
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        $row = $cursor->fetch();
//
//        return $row[0];
//    }

    /**
     * Zapise naklady do tabulky PlanovanaCenaSkolCelkova
     * @param $rec
     */
    function zapisNaklady($rec)
    {
        foreach ($this ->fieldsNaklady as $field) {
            $delete = !isset($_POST[$field ->fieldName])  ||  trim($_POST[$field ->fieldName]) == ''; // neni zadano => smazat

            $cena = $delete
                ? null
                : StringUtils::stringToFloat($_POST[$field ->fieldName]);

            $recPlanovanaCena = new RonPortalDBRecord($this ->database, 'PlanovanaCenaSkolCelkova', $this);
            $recPlanovanaCena ->addStringField('CisloSkoleni', true, 40, $rec['CisloSkoleni']);
            $recPlanovanaCena ->addStringField('TypPolozky', true, 10, $field ->getTypPolozky());
            $recPlanovanaCena ->addField('CenaSDPH', 'double', false, 0, $cena);
            $recPlanovanaCena ->addField('SazbaDPH', 'double', true, 0, $field ->getSazbaDPH());
            $recPlanovanaCena ->addField('DPH', 'double', false, 0, $cena / (100 + $field ->getSazbaDPH()) * $field ->getSazbaDPH());

            $params = $this ->database ->createParameters();
            $recPlanovanaCena ->setWhere(
                'CisloSkoleni = ' . $params ->addParameter($rec['CisloSkoleni']) .
                ' AND TypPolozky = ' . $params ->addParameter($field ->getTypPolozky()),
                $params
            );

            if ($recPlanovanaCena->loadRecord()) {
                if ($delete) {
                    $recPlanovanaCena ->doDelete(null);
                } else {
                    $recPlanovanaCena ->updateRecord();
                }
            } else {
                if (isset($cena)) {
                    $recPlanovanaCena ->insertRecord();
                }
            }
        }
    }


    // TODO: prihlasit uzivatele na nove vytvorene skoleni? (zapis do PlanovaneSkoleniOsob)
    /*
        $pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');

        $potvrzeni = 0;
        if ($pravoPrihlasit->lzeZobrazitRC(User::getRC())){
            $potvrzeni = 1;
        }

        $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob', $this);

        $recZapis->addField('CisloSkoleni', '', false, 10, $hodnoty->cisloSkoleni);
        $recZapis->addField('RC', '', false, 15, User::getRC());
        $recZapis->addField('Potvrzeni', 'int', false, 0, $potvrzeni);
        $recZapis->addField('Vlozil', '', false, 25, User::getRC());
        $recZapis->addField('DatumVlozeni', 'timestamp', false, 0, time());
        $recZapis->addField('Vyhodnotil', '', false, 25, '');
        $recZapis->addField('DatumVyhodnoceni', 'timestamp', false, 0, '');
        $recZapis->addField('PrihlasenoZAplikace', 'int', false, 0, 1);

        $params = $this->database->createParameters();
        $recZapis->setWhere(
            'RC  = ' . $params->addParameter(User::getRC()) .
            ' AND CisloSkoleni = ' . $params->addParameter($hodnoty->cisloSkoleni), $params);

        if (!$recZapis->loadRecord()) {
            $recZapis->insertRecord();
            MessageManager::addInfoMessage(StringUtils::fromUTF8('byl jste prihlasen ...'));
        } else {
            MessageManager::addInfoMessage(StringUtils::fromUTF8('uz jste prihlasen ...'));
        }

     */

    function afterInsertRecord($rec)
    {
        $this ->zapisNaklady($rec);
        return parent::afterInsertRecord($rec);
    }


    function afterUpdateRecord($rec)
    {
        $this ->zapisNaklady($rec);
        return parent::afterUpdateRecord($rec);
    }
}
