<?php

namespace RonPortal\Skoleni;

use Ron\Controls\BrowserField;
use Ron\Controls\Dialog;
use Ron\Controls\DialogButton;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;


/**
 * Class PocetOsobNaKurzuField
 */
class PocetOsobNaKurzuField extends BrowserField
{

    /**
     * PocetOsobNaKurzuField constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this
            ->setHeaderText('kapacita_obsazenost_volne')
            //->setSqlFieldName('Kapacita')
            ->setAlign('center')
            ->setOrdering(true);
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $vycerpano = ((int)$rowData['col-pocetosob']-(int)$rowData['col-celkem-prihlasenych']) <= 0 // pocetosob = kapacita
            ? 'minus'
            : '';

        $kapacita = (int)$rowData['col-pocetosob'];
        $volno = ((int)$rowData['col-pocetosob']-(int)$rowData['col-celkem-prihlasenych']);

        return
            "<span>" . $kapacita . "</span>/<span class='$vycerpano'>" . $volno . "</span>";
    }
}



/**
 * Class PotvrzeniSkoleniField
 */
class PotvrzeniSkoleniField extends BrowserField
{
    public function __construct($database)
    {
        parent::__construct($database);
        $this
            ->setType(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('PlanovaneSkoleniOsob.Potvrzeni')
            ->setCode('col-potvrzeni')
            ->setHeaderText('Stav')
            ->setOrdering(true)
            ->setOptions(
                [
                    0 => TextManager::_('osoba_plan_kurzu_prihlasen_neschvaleno'),
                    1 => TextManager::_('osoba_plan_kurzu_prihlasen_schvaleno'),
                    2 => TextManager::_('osoba_plan_kurzu_zamitnuto')
                ]
            );
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!isset($rowData['col-potvrzeni'])) {
            return '-';
        }
        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}





/**
 * Class SchvaleniPrihlaseniOsobyNaKurzuField
 */
class SchvaleniPrihlaseniOsobyNaKurzuField extends BrowserField
{
    /** @var PravaNaOsoby $pravoPrihlasit */
    public $pravoPrihlasit;
    /** @var PravaNaOsoby $pravoSchvalit */
    public $pravoSchvalit;

    /** @var int */
    protected $hromadne;

    /**
     * SchvaleniPrihlaseniOsobyNaKurzuField constructor.
     * @param Database $database
     * @param int $hromadne     Urcuje, jestli se zobrazuje z hromadneho seznamu nebo ze seznamu pro jednu osobu
     */
    function __construct(Database $database, $hromadne)
    {
        parent::__construct($database);

        $this->hromadne = $hromadne;

        $this->header = TextManager::_('Schvaleni');

        $this->pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $this->pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');
    }


    /**
     * @param array $rowData
     * @param string $akce  schvalit nebo zamitnout
     * @param string $textOznameniPoznamky
     * @return string
     */
    private function schvalitPrihlaseniLink($rowData, $akce, $textOznameniPoznamky)
    {
        return
            "<span class='pripojit-poznamku-schvaleni js margin-left-4'" .
            " data-ron-akce='$akce'" .
            " data-ron-osoba='" . $rowData['col-idpp'] . "'" .
            " data-ron-cislokurzu='" . StringUtils::htmlTextConvert($rowData['col-cisloskoleni']) . "'" .
            " data-ron-poznamkaschvaleni='" . StringUtils::htmlTextConvert($rowData['col-poznamkaschvalovani']) . "'" .
            ">$textOznameniPoznamky</span>";
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $datumPocatkuKurzu = $this->database->stringToTimestamp($rowData['col-datumpocatkuskoleni']);

        if ($datumPocatkuKurzu < DateUtils::today()) {
            // kurz uz probiha nebo probehl
            return TextManager::_('expirace_kurzu');
        }

        //Prava
        $muzePrihlasit = $this->pravoPrihlasit->lzeZobrazitRC($rowData['col-rc']);
        $muzeSchvalit = $this->pravoSchvalit->lzeZobrazitRC($rowData['col-rc']);

        //Url
        $url = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&cisloskoleni={$rowData['col-cisloskoleni']}&osoba={$rowData['col-idpp']}&hromadne={$this ->hromadne}";
        $urlSchvaleni = $url . "&plankurzuzamestnance=schvalit";
        $urlOdhlaseni = $url . "&plankurzuzamestnance=zrusit";
        $urlZamitnuti = $url . "&plankurzuzamestnance=zamitnout";

        //Stav
        $jePrihlasen = isset($rowData['col-potvrzeni']) && (int)$rowData['col-potvrzeni'] === 0;
        $jeSchvalen = (int)$rowData['col-potvrzeni'] === 1;
        $jeZamitnut = (int)$rowData['col-potvrzeni'] === 2;

        if (!$jeSchvalen  &&  !$jePrihlasen  &&  (int)$rowData['col-pocetosob'] <= (int)$rowData['col-celkem-prihlasenych']) {
            // neni prihlasen ani schvalen a kapacita kurzu je naplnena
            return TextManager::_('kapacita_planu_naplnena');
        }

        //Pokud ma pravo na schvaleni ale neni zapsan na planu pak nabidnout prihlaseni a schvaleni jinak pouze prihlaseni
        //Pokud je prihlasen a nema pravo schvalit pak zobrazit pouze text uzivatel je prihlasen
        if ($muzePrihlasit || $muzeSchvalit) {
            if ($jeSchvalen) {
                //Je schvalen -> je i prihlasen
                if ($muzeSchvalit) {
                    $textOznameni = TextManager::_('odhlasit_z_planu_skoleni');
                    return "<a href='$urlOdhlaseni'>$textOznameni</a>";
                }

                return '';
            }

            if ($jePrihlasen) {
                //Je prihlasen, ale neschvalen ->  moznost schvalit nebo ignorovat
                if ($muzeSchvalit) {
                    $textOznameni = TextManager::_('Schvalit');

                    $result =
                        "<div>" .
                        "<a href='$urlSchvaleni'>$textOznameni</a>" .
                        $this->schvalitPrihlaseniLink($rowData, 'schvalit', TextManager::_('schvalit_plan_kurzu_poznamka')) .
                        "</div>";

                    if (!$jeZamitnut) {
                        $textOznameni = TextManager::_('Zamitnout');

                        $result .=
                            "<div>" .
                            "<span class='margin-right-2'><a href='$urlZamitnuti'>$textOznameni</a></span>" .
                            $this->schvalitPrihlaseniLink($rowData, 'zamitnout', TextManager::_('zamitnout_plan_kurzu_poznamka')) .
                            "</div>";
                    }

                    return $result;
                }

                if ($muzePrihlasit){
                    $textOznameni = TextManager::_('odhlasit_z_planu_skoleni');
                    return "<a href='$urlOdhlaseni'>$textOznameni</a>";
                }

                return '';
            }

            //Neni prihlasen, kapacita je -> moznost prihlasit i schvalit
            if ($muzeSchvalit) {
                $textOznameni = TextManager::_('prihlasit_schvalit_plan_skoleni_uzivatel_neprojevil_zajem');
                $textOznameniPoznamky = TextManager::_('prihlasit_schvalit_plan_kurzu_poznamka');
            } elseif ($muzePrihlasit) {
                $textOznameni = TextManager::_('Skoleni_prihlasit');
                $textOznameniPoznamky = TextManager::_('prihlasit_plan_kurzu_poznamka');
            } else {
                $textOznameni = $textOznameniPoznamky = null;
            }

            if ($textOznameni) {
                return
                    '<span class="margin-right-2">' .
                    "<a href='$urlSchvaleni'>$textOznameni</a>" .
                    $this->schvalitPrihlaseniLink($rowData, 'schvalit', $textOznameniPoznamky) .
                    '</span>';
            }
        }

        return '';
    }
}



class PrihlaseniNaPlanovanaBrowser extends RonPortalBrowser
{
    public function render()
    {
        parent::render();

        // dialog pro zapsani poznamky pri prihlasovani
        (new Dialog())
            ->setView('ronportal/skoleni/view_poznamka_schvaleni_planu.php')
            ->setId('dialog-pripojeni-poznamky-schvaleni')
            ->setTitle('dialog_pripojeni_poznamky_schvaleni')
            ->setFormTag(
                [
                    'method' => 'post',
                    'id' => 'formularpoznamkyplanukurzu',
                    'name' => 'formularpoznamkyplanukurzu',
                    'action' => '?menuid=skoleni&action=prihlaseninaplanovaneskoleni'
                ]
            )
            ->addButton(
                (new DialogButton())
                    ->setClass('submitbutton')
                    ->setValue('form.Store')
            )
            ->addButton(DialogButton::cancelButton())
            ->render();
    }
}
