<?php

/* 
 * Pavel Rudol, RON Software
 *  10.12.2015
 */

namespace RonPortal\Skoleni;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use function Ron\ronText;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;

class UmisteniSkoleniFilter extends FilterFormFilter
{
    public function createOption(Database $database, $row)
    {
        $item = parent::createOption($database, $row);

        $item ->setCaption(
            (trim($row[1] ?? '') != ''
                ? $row[1] . ' - '
                : '') .
            $row[2]
        );

        return $item;
    }
}


//class UmisteniSkoleniCombo extends ComboBox implements ICreateItem
//{
//    function createItem($row, $keyIndex = 0, $captionIndex = 1, $titleIndex = null, $cssClassIndex = null)
//    {
//        if (!$row) {
//            return new Option('', ' ');
//        }
//
//        $key = $row[$keyIndex];
//        $caption = '';
//        if (trim($row[1] ?? '') != '') {
//            $caption = $row[1] . ' - ';
//        }
//
//        $caption .= $row[2];
//
//        return new Option($key, $caption);
//    }
//}


/**
 * Class PrihlaseniSkoleniFilterForm
 */
class PrihlaseniSkoleniFilterForm extends SkoleniFilterForm
{
//    public $hromadnePrihlaseni;

    public $popisSkoleni = null;

//    public $umisteniSkoleni = null;

    public $datumCasSkoleniOd = null;

    /**
     * PrihlaseniSkoleniFilterForm constructor.
     * @param Database $database
     * @param $action
     */
    public function __construct($database, $action) // , $hromadnePrihlaseni
    {
        parent::__construct($database, ['menuid' => 'skoleni', 'action' => 'zobrazprihlaseni', 'new' => ''], true);
        
        $prava = Prava::vratPrava('skoleni_prihlasit');

        $this ->addFilterPP(true, $prava)
            ->setDatumOd(time())
            ->setDatumDo(time());
//        $this->hromadnePrihlaseni = $hromadnePrihlaseni;

        $this ->addFilter(
            (new FilterFormFilterDatumOdDo($this)),
            true
        );

        $this ->addFilterRozdeleni1();

        //$this->f_classname = 'filtercontainer';

        $this ->addSpecifikaceFilter();
        $this ->addKategorieFilter();
        $this ->addDruhSkoleniFilter();

        $this ->addFilter((new UmisteniSkoleniFilter('umisteni', FilterFormFilter::TYPE_OPTIONS))
            ->setCaption(TextManager::_('Skoleni_Umisteni'))
            ->addItem('', TextManager::_('_vse_'))
            ->setAddClass('ronselect')
            ->setOptionsSql(
                'SELECT Kod, Mesto, Popis FROM Umisteni WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1',
                'Popis')
        );

        $this ->addFilter((new FilterFormFilter('typPovinnosti'))
            ->setCaption(TextManager::_('Skoleni_jenPovinne'))
            ->addItem('', TextManager::_('Vse'))
            ->addItem(1, TextManager::_('Skoleni_povinne'))
            ->addItem(2, TextManager::_('Skoleni_doporucene'))
        );
    }


    /**
     * @throws DBException
     */
    public function renderPopisSkoleni()
    {
        echo '<DIV class="filterrow">';

        echo
            '<SPAN class="filtercaption">' .
            ronText('Skoleni_Popis') .
            '</SPAN>';

        // combobox pro vyber
        $params = $this->database->createParameters();

        //Puvodne byl filtr pro vybranou osobu, momentalne upraven na vyber vsech
        $sql =
            "SELECT DISTINCT Skoleni.Popis, Skoleni.Popis" .
            " FROM Skoleni JOIN DruhSkoleni ON (DruhSkoleni.Kod = Skoleni.DruhSkoleni)
             WHERE DruhSkoleni.Aktivni = 1 AND ( Skoleni.StavSkoleni IN (0, 1) AND ( Skoleni.DatumPocatkuSkoleni >= " . $params->addDateParameter(time()) .
            " AND (Skoleni.DatumPotvrzeniUcasti >= " . $params->addDateParameter(time()) .
            " OR Skoleni.DatumPotvrzeniUcasti IS NULL) OR DruhSkoleni.Elearning = 1 ))" .
            ' AND (Skoleni.DatumUkonceniSkoleni >= ' .$params->addDateParameter(time()) . ' OR Skoleni.DatumUkonceniSkoleni IS NULL)' .
            " AND ((" .
            " EXISTS(SELECT * FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni)
				WHERE Skoleni.DruhSkoleni=PozadovanaSkoleni.DruhSkoleni   
				  AND PozadovanaSkoleni.Povinne = 0 " .
                ($this ->getFilterValue('osoba') ? '  AND RozdeleniOsob.RC = ' . $params->addParameter($this ->getFilterValue('osoba', 'rc')) : '') .
                " AND RozdeleniOsob.PlatnostOd <= Skoleni.DatumPocatkuSkoleni
				  AND (RozdeleniOsob.PlatnostDo >= Skoleni.DatumPocatkuSkoleni OR RozdeleniOsob.PlatnostDo IS NULL)) " .
            " OR " .
            " EXISTS(SELECT * FROM GenerovaneSkoleni
				WHERE Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni 
				  AND GenerovaneSkoleni.Potvrzeni = 1 " .
                ($this ->getFilterValue('osoba') ? ' AND GenerovaneSkoleni.RC = ' . $params->addParameter($this ->getFilterValue('osoba', 'rc')) : '') .
				" AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)) ))" .
            ' ORDER BY Skoleni.Popis';

        $combo = new ComboBox('popisSkoleni');
        $combo->emptyRow = ronText('_vse_');
        $combo->className = 'form-input adsfilterforminput_full ronselect';
        $combo->htmlParams = [
            'data-ronselect-class' => 'ronselect adsfilterforminput_full',
            'data-ron-autosubmit' => '1',
            'data-columns' => 'text'
        ];

        $combo->loadData($this ->database, $sql, RonPortalSession::getValue('popisSkoleni'), $params);
        $combo->render(); 

        echo "</DIV>\n";
    }


//    /**
//     * @throws Exception
//     */
//    public function renderUmisteni()
//    {
//        echo '<DIV class="filterrow">';
//
//        // checkbox "Jen umisteni"
//        echo '<SPAN class="filtercaption">';
//        echo ronText('Skoleni_Umisteni');
//        echo '</SPAN>';
//
//        // combobox pro vyber
//        $sql =
//            'SELECT Kod, Mesto, Popis FROM Umisteni' .
//            ' ORDER BY Popis';
//
//        // vyber umisteni podle popisu
//        $combo = new UmisteniSkoleniCombo('umisteni');
//        $combo->className = 'form-input adsfilterforminput_full ronselect';
//        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
//        $combo->emptyRow = ronText('_vse_');
//        $combo->loadData($this ->database, $sql, $this ->umisteniSkoleni);
//        $combo->setSelectedOption($this ->umisteniSkoleni);
//        $combo->render();
//
//        echo "</DIV>\n";
//
//    }
    
//    public function renderPovinnostSkoleni()
//    {
//        echo '<DIV class="filterrow">';
//
//        // checkbox "Jen typ skoleni"
//        echo '<SPAN class="filtercaption">';
//        echo ronText('Skoleni_jenPovinne');
//        echo '</SPAN>';
//
//        // vyber povinne/doporucene
//        $povinneVse = new Option(0, ronText('_vse_'));
//        $povinne = new Option(1, ronText('Skoleni_povinne'));
//        $doporucene = new Option(2, ronText('Skoleni_doporucene'));
//
//        $combo = new ComboBox('typPovinnosti',[$povinneVse,$povinne,$doporucene]);
//        $combo->className = 'form-input adsfilterforminput_full ronselect';
//        $combo->htmlParams = "data-ronselect-class='ronselect adsfilterforminput_full' data-ron-autosubmit='1' data-columns='text'";
//        $combo->selectedOption = $combo->findOption((int)RonPortalSession::getValue('typPovinnosti'));
//        $combo->render();
//
//        echo "</DIV>\n";
//    }


//    public function renderHromadnePrihlaseni()
//    {
//        echo "<br>";
//        echo '<div class="filterrow">';
//        $popis = ronText('Skoleni_prihlasit_hromadne');
//        echo "<button class='button clickHref' type='button' name='hromadne_prihlaseni' data-ron-href='?menuid=skoleni&action=zobrazprihlaseni&hromadneprihlaseni=1' >$popis</button>";
//        echo "</div>";
//    }

    public function renderDetailUmisteni()
    {
        echo '<div class="filterrow">';

        echo
            '<span class="filtercaption">' .
            ronText('Skoleni_Umisteni') .
            '</span>';

        $params = $this->database->createParameters();
        $sql =
            'SELECT Kod, Popis, KapacitaMistnosti, Mesto, Ulice, Stat, PSC, Patro, CisloMistnosti' .
            ' FROM Umisteni WHERE Kod = ' . $params->addParameter($this ->getFilterValue('umisteni'));

        $cursor = $this->database->getSqlResult($sql, $params);

        if ($row = $cursor->fetch()) {
            echo
                "<span class='font-weight-bold'>$row[1]</span>" .
                ($row[2] ? ' ' . ronText('Skoleni_kapacita_mistnosti') . ": $row[2]" : '') .
                ($row[3] ? ' ' . ronText('Skoleni_Adresa') . ": $row[4] $row[3] $row[6]" : '') .
                ($row[7] ? ', ' . ronText('Skoleni_Patro') . ": $row[7]" : '') .
                ($row[8] ? ', ' . ronText('Skoleni_Cislo_mistnosti') . ": $row[8]" : '');

        }

        if ($this ->datumCasSkoleniOd && ($row[2] > 0))  {
            // je zadana kapacita - zjisti volne misto
            $params = $this->database->createParameters();

            $sql =
                "SELECT COUNT(DISTINCT RC)" .
                " FROM GenerovaneSkoleni JOIN Skoleni ON (Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni)" .
                " WHERE Skoleni.MistoSkoleni = " . $params->addParameter($this ->getFilterValue('umisteni')) .
                '  AND Skoleni.CasPocatkuSkoleni IS NOT NULL' .
                '  AND Skoleni.DatumUkonceniSkoleni IS NOT NULL' .
                '  AND Skoleni.CasUkonceniSkoleni IS NOT NULL' .
                "  AND Skoleni.DatumPocatkuSkoleni <= " . $params->addParameter($this->database->dateToParam($this ->datumCasSkoleniOd)) .
                "  AND Skoleni.DatumUkonceniSkoleni >= " . $params->addParameter($this->database->dateToParam($this ->datumCasSkoleniOd)) .
                "  AND Skoleni.CasPocatkuSkoleni <= " . $params->addParameter($this->database->timeToParam($this ->datumCasSkoleniOd)) .
                "  AND Skoleni.CasUkonceniSkoleni >= " . $params->addParameter($this->database->timeToParam($this ->datumCasSkoleniOd));

            $cursor = $this->database->select($sql, $params);

            if ($row = $cursor->fetch()) {
                echo "<span> " . sprintf(ronText('Skoleni_PocetPrihlasenych_Cas'), StringUtils::format('datetime', $this ->datumCasSkoleniOd)) . ": $row[0]</span>";
            }
            $cursor ->close();
        }

        echo "</div>";
    }


//    private function loadUmisteni()
//    {
//        if (isset($_GET['umisteni'])) {
//            $this ->umisteniSkoleni = $_GET['umisteni'];
//            if (trim($this ->umisteniSkoleni) === '') {
//                $this ->umisteniSkoleni = null;
//            }
//
//            $_SESSION['umisteniSkoleni'] = $this ->umisteniSkoleni;
//        } else {
//            $this ->umisteniSkoleni = $_SESSION['umisteniSkoleni'] ?? null;
//        }
//    }
    
//    private function loadPovinnost()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $_SESSION['jenPovinne'] = $_POST['typPovinnosti'] !== '';
//            $_SESSION['typPovinnosti'] = $_POST['typPovinnosti'];
//        }
//    }

    private function loadPopisSkoleni()
    {
        if (isset($_GET['popisSkoleni'])) {
            $this ->popisSkoleni = $_GET['popisSkoleni'];
            if (trim($this ->popisSkoleni === '')) {
                $this ->popisSkoleni = null;
            }
            $_SESSION['popisSkoleni'] = $this ->popisSkoleni;
        } else {
            $this ->popisSkoleni = $_SESSION['popisSkoleni'] ?? null;
        }
    }


    public function loadData()
    {
        //$this->loadDruhSkoleni();
        
        //$this->loadUmisteni();
        
        //$this->loadPovinnost();

        //$this->loadSpecifikaceKurzu();
        //$this->loadKategorieSkoleni();

        $this->loadPopisSkoleni();

        if (isset($_GET['datumcasod'])) {
            // pouziva se pro odkazy pro vyhledani skoleni zacinajici ve stejnem case a stejnem umisteni
            $this ->datumCasSkoleniOd = (int)$_GET['datumcasod'];
        }

        parent::loadData();        
    }

    /**
     * @throws Exception
     */
    public function renderFilters()
    {
        parent::renderFilters();

        //$this->renderSpecifikaceKurzu();
        //$this->renderKategorieSkoleni();
        //$this->renderDruhSkoleni();

        //$this->renderUmisteni();

        //$this->renderPovinnostSkoleni();

        $this->renderPopisSkoleni();

        if ($this ->getFilterValue('umisteni')) {
            $this->renderDetailUmisteni();
        }

//        if (!$this->hromadnePrihlaseni){ $this->renderHromadnePrihlaseni();}
    }
}


/**
 * Class PrihlaseniSkoleniBrowserField
 */
class PrihlaseniSkoleniBrowserField extends BrowserField
{
    private $hromadnePrihlaseni;

    /**
     * @param Database $database
     * @param bool $hromadnePrihlaseni
     */
    function __construct($database, $hromadnePrihlaseni)
    {
        parent::__construct($database);

        $this->align = 'center';
        $this->code = 'col-prihlaseni';

        if ($hromadnePrihlaseni){
            $this->hromadnePrihlaseni = $hromadnePrihlaseni;
            $this->header = ronText('Skoleni_prihlasit_odhlasit');
        }
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $prihlasen = (bool)$rowData['col-prihlasen'];

        $datumPovinnosti = null;
        $datumPovinnostiStr = '';
        if ($rowData['col-datumpovinnosti']) {
            $datumPovinnosti = $this->database->stringToTimestamp($rowData['col-datumpovinnosti']);
            $datumPovinnostiStr = '&datumpovinnosti=' . $datumPovinnosti;
        }

        $pravaPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        $prihlasit = false;
        $odhlasit = false;
        $schvalitPrihlasku = false;

        $prihlasitStr = null;

        if ($pravaPrihlasit->lzeZobrazitRC($rowData['col-rc'])) {
            $prihlasitStr = ronText('Skoleni_prihlasit');
        }

        $result = '';

        if ($rowData['col-vyzadovano-schvaleni']) {
            //pokud je vyzadovano potvrzeni schvaleni prihlasky a uzivatel nema pravo ke schvaleni prihlasky, zobrazi se text registrovat a prihlaska bude cekat na schvaleni
            if (($pravaPrihlasit->lzeZobrazitRC($rowData['col-rc'])) && (!($pravaPotvrditPrihlaseni->lzeZobrazitRC($rowData['col-rc'])))) {
                $prihlasitStr = ronText('Skoleni_registrovat_zajem');
            }

            //existuje vygenerovany pozadavek, nebo jiz probehlo prihlaseni
            if (isset($datumPovinnosti)) {
                if ($pravaPotvrditPrihlaseni->lzeZobrazitRC($rowData['col-rc'])) {
                    //pokud zatim nebylo schvaleno a uzivatel ma pravo schvalit zobrazi odkaz na schvaleni/zamitnuti
                    if (($rowData['col-schvaleniprihlasky'] == 0) && $prihlasen) {
                        $schvalitPrihlasku = true;
                    }
                    //u jiz schvalene registrace, umozni uzivateli s pravem schvalovani prihlasky odhlaseni
                    elseif ($prihlasen && ($rowData['col-schvaleniprihlasky'] == 1)) {
                        $odhlasit = true;
                    }
                }
                elseif ($pravaPrihlasit->lzeZobrazitRC($rowData['col-rc'])) {
                    if ($prihlasen && ($rowData['col-schvaleniprihlasky'] == 0)) {
                        $odhlasit = true;
                    }
                }
            }
            if (!$prihlasen) {
                $prihlasit = true;
            }
        }


        if ($rowData['col-elearning']) {
            if (((!(int)$rowData['col-vyzadovano-schvaleni']) || ((int)$rowData['col-schvaleniprihlasky'] != 0))
                && ($rowData['col-pocet-schvalenych-prihlasek-rc'] <> $rowData['col-absolvovanych-elearningu-rc']))
                return '';

            if ($pravaPrihlasit->lzeZobrazitRC($rowData['col-rc'])) {
                if (!$prihlasen) {
                    $prihlasit = true;
                }
            }
        }
        else {
            //nejedna se o elearningovy kurz
            if ($pravaPrihlasit->lzeZobrazitRC($rowData['col-rc'])) {
                if (!($rowData['col-vyzadovano-schvaleni'] == 1)) {
                    if ($prihlasen) {
                        $odhlasit = true;
                    } elseif ((int)$rowData['col-pocet-prihlasenych'] < (int)$rowData['col-kapacita-kurzu']) {
                        $prihlasit = true;
                    }
                }
            }
        }

        $osobaIDPP = $rowData['col-idpp'];
        $cisloSkoleni = $rowData['col-cislo-skoleni'];
        $mistoSkoleni = $rowData['col-mistoskoleni'];
        $datumCasPocatkuSkoleni = $this->database->stringToTimestamp($rowData['col-datum-pocatku']);
        if ($rowData['col-cas-pocatku']) {
            $datumCasPocatkuSkoleni = $this->database->stringToSecondsOfDay($rowData['col-cas-pocatku']) + $datumCasPocatkuSkoleni;
        }

        if ($prihlasit){

            //TODO: pokud existuje vice kurzu pro termin, zobrazit js dotaz na vyhledani dalsich, po potvrzeni odfiltrovat browser
            //u hromadne zobrazovat checkboxy - v js zkontrolovat zda je vybrana jedna mistnost a cas, pokud ne upozornit, jinak odfiltrovat
            //zobrazit info o kapacite kurzu a prihlasenych osobach

            if (!$this->hromadnePrihlaseni) {
                $csrfParam = Csrf::csrfUrlParameter();
                if ($rowData['col-pocet-kurzu-ve-stejnem-case'] > 1) {
                    $result =
                        "<span
                            class='js skoleniVyhledatUmisteni'
                            data-skoleni-provest='?menuid=skoleni&action=prihlaseni&osoba={$osobaIDPP}{$datumPovinnostiStr}&cisloskoleni={$cisloSkoleni}{$csrfParam}'
                            data-skoleni-vyhledat='?menuid=skoleni&action=zobrazprihlaseni&umisteni=$mistoSkoleni&datumcasod=$datumCasPocatkuSkoleni'>" .
                            $prihlasitStr .
                        "</span>";
                } else {
                    $result =
                        '<a href="?menuid=skoleni&action=prihlaseni&osoba=' . $osobaIDPP . $datumPovinnostiStr . '&cisloskoleni=' . $cisloSkoleni . $csrfParam . '">' .
                        $prihlasitStr .
                        '</a>';
                }

            } else {
                //Hromadne prihlaseni
                $index = $rowNumber;
                $result .=
                    "<input type='hidden'   name='prihlasitNaSkoleni[$index][osoba]' value='{$osobaIDPP}'/>" .
                    "<input type='hidden'   name='prihlasitNaSkoleni[$index][cislo]' value='{$cisloSkoleni}'/>" .
                    "<input type='hidden'   name='prihlasitNaSkoleni[$index][datumpovinnosti]' value='{$datumPovinnosti}'/>" .
                    "<input type='hidden'   name='prihlasitNaSkoleni[$index][prihlaseno]' value='0'/>" .
                    "<input type='checkbox' name='prihlasitNaSkoleni[$index][prihlasit]' id='prihlasitNaSkoleni[$index][prihlasit]' value='1'/>";
            }
        }

        if ($odhlasit) {
            $csrfParam = Csrf::csrfUrlParameter();
            if (!$this->hromadnePrihlaseni) {
                if ($rowData['col-pocet-kurzu-ve-stejnem-case'] > 1) {
                    $result =
                        '<span' .
                            " class='js skoleniVyhledatUmisteni' data-skoleni-provest='?menuid=skoleni&action=odhlaseni&osoba={$osobaIDPP}{$datumPovinnostiStr}&cisloskoleni={$cisloSkoleni}{$csrfParam}'" .
                            " data-skoleni-vyhledat='?menuid=skoleni&action=zobrazprihlaseni&umisteni=$mistoSkoleni&datumcasod=$datumCasPocatkuSkoleni'>" .
                        ronText('Skoleni_odhlasit') .
                        '</span>';
                } else {
                    $result =
                        '<a data-prihlaseny-terminy="' . (SkoleniUtils::jeOsobaNaTerminuSkoleni($this ->database, $cisloSkoleni, $rowData['col-rc']) ? 1 : 0) . '"' .       // TODO: lepsi by bylo vyresit to ajaxem
                        ' data-skoleni-kod="' . $cisloSkoleni . '" data-ponechat-navrh="' . RonPortalSettings::getIntValue(SkoleniController::NASTAVENI_PONECHAT_NAVRH) . '"' .
                        " href='?menuid=skoleni&action=odhlaseni&osoba={$osobaIDPP}{$datumPovinnostiStr}&cisloskoleni={$cisloSkoleni}{$csrfParam}'>" .
                        ronText('Skoleni_odhlasit') .
                        '</a>';
                }

            } else {
                //Hromadne odhlaseni
                $index = $rowNumber;
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][osoba]' value='{$osobaIDPP}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][cislo]' value='{$cisloSkoleni}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][datumpovinnosti]' value='{$datumPovinnosti}'  />";
                $result .= "<input type='hidden'   name='prihlasitNaSkoleni[$index][prihlaseno]' value='1'  />";
                $result .= "<input type='checkbox' name='prihlasitNaSkoleni[$index][prihlasit]'
                                    id='prihlasitNaSkoleni[$index][prihlasit]'                                      
                                    value='1' checked='checked' />";
            }
        }

        if ($schvalitPrihlasku) {
            $csrfParam = Csrf::csrfUrlParameter();

            $result =
                '<a href="?menuid=skoleni&action=schvaleniprihlaseni&osoba=' . $rowData['col-idpp'] . $datumPovinnostiStr . '&cisloskoleni=' . $rowData['col-cislo-skoleni'] . $csrfParam . '">' .
                    ronText('Skoleni_schvaleni_registrace') .
                '</a> &nbsp ' .
                '<a href="?menuid=skoleni&action=zamitnutiprihlaseni&osoba=' . $rowData['col-idpp'] . $datumPovinnostiStr . '&cisloskoleni=' . $rowData['col-cislo-skoleni'] . $csrfParam . '">' .
                    ronText('Skoleni_zamitnuti_registrace') .
                '</a>';
        }

        return $result;
    }

}


/**
 * Class PovinneSkoleniBrowserField
 */
class PovinneSkoleniBrowserField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $params = $this->database->createParameters();

        $sql = 
            "SELECT RC, GenerovaneSkoleni.Povinne FROM GenerovaneSkoleni " .
            "WHERE GenerovaneSkoleni.RC = " . $params->addParameter($rowData['col-rc']) .
            "  AND GenerovaneSkoleni.DruhSkoleni = " . $params->addParameter($rowData['col-druhskoleni']) .
            //"  AND GenerovaneSkoleni.DuvodSkoleni = 0 " .
            "  AND (((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) AND GenerovaneSkoleni.DatumSkoleni IS NULL)" .
            "      OR GenerovaneSkoleni.CisloSkoleni = " . $params->addParameter($rowData['col-cislo-skoleni']) . ")";
    
        $cursor = $this->database->select($sql, $params);
        
        $row = $cursor->fetch();

        $cursor->close();

        if ($row[1]) {

            $params = $this->database->createParameters();

            $sql = 
                "SELECT RC FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
                "WHERE PozadovanaSkoleni.DruhSkoleni = " . $params->addParameter($rowData['col-druhskoleni']) .
                "  AND PozadovanaSkoleni.Povinne = 0 ".
                "  AND RozdeleniOsob.RC = " . $params->addParameter($rowData['col-rc']) .
                "  AND RozdeleniOsob.PlatnostOd <= " . $params->addDateParameter($this->database->stringToTimestamp($rowData['col-datum-pocatku'])) .
                "  AND (RozdeleniOsob.PlatnostDo >= " . $params->addDateParameter($this->database->stringToTimestamp($rowData['col-datum-pocatku'])) . " OR RozdeleniOsob.PlatnostDo IS NULL)";
            
            $cursor = $this->database->select($sql, $params);
        
            $row = $cursor->fetch();

            $cursor->close();

            if ($row) {
               return ronText('Skoleni_povinne') . ', ' . ronText('Skoleni_doporucene');
            }
            else {
                return ronText('Skoleni_povinne');
            }
        }

        return ronText('Skoleni_doporucene');
    }
}

/**
 * Class SkoleniElearningField
 */
class SkoleniElearningField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData['col-schvaleniprihlasky'] == 2) { // zamitnuto
            $value = null;
        }

        //pro bezne kurzy zobrazi kapacit a pocet prihlasenych osob, u e-learningoveho se nezobrazuje
        return $rowData['col-elearning']
            ? ''
            : ($this ->getLink(($this ->htmlConvert ||  $this ->format)
                ? DBUtils::getFormatedValue($this ->database, $this ->format, $value)
                : $value,
                $recordId,
                $rowData));
    }
}


/**
 * Class SkoleniPrihlasenBrowserField
 */
class SkoleniPrihlasenBrowserField extends BrowserField
{

    /**
     * SkoleniPrihlasenBrowserField constructor.
     * @param string $header
     */
    public function __construct($header) {
        parent::__construct();

        $this->align = 'center';
        $this->header = $header;
        $this->code = 'col-prihlasen-zamestnanec';
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $stav = $rowData['col-popis-stavu-prihlaseni']
            ? " ({$rowData['col-popis-stavu-prihlaseni']})"
            : '';

        return $rowData['col-prihlasen'] >= 1
            ? ("<span class='font-weight-bold'>".ronText('Str_Yes') . "</span>". $stav)
            : ronText('Str_No');
    }
}


/**
 * Nacte a zobrazi podrobnosti umisteni (adresu)
 * Class UmisteniSkoleniCommentField
 */
class UmisteniSkoleniCommentField extends BrowserField
{
    /** @var array      Cache - aby nemusel opakovane nacitat stejne umisteni */
    private static $rowsUmisteni = [];

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$value) {
            return '';
        }

        $umisteniRow = self::$rowsUmisteni[$value] ?? null;
        if (!$umisteniRow) {
            $params = $this->database->createParameters();

            $sql =
                "SELECT Popis, Ulice, Mesto, Patro" .
                " FROM Umisteni WHERE Kod = " . $params->addParameter($value);

            $cursor = $this->database->select($sql, $params);
            $umisteniRow = $cursor ->fetch();
            // ulozi do cache
            self::$rowsUmisteni[$value] = $umisteniRow;
        }

        $value = $umisteniRow
            ?
               $umisteniRow[0] .
              ($umisteniRow[1] ? ', ' . $umisteniRow[1] : '') .
              ($umisteniRow[2] ? ', ' . $umisteniRow[2] : '') .
              ($umisteniRow[3] ? ', ' . ronText('Skoleni_Patro') . ': ' . $umisteniRow[3] : '')
            : null;

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



/**
 * Nacte a zobrazi dokumenty skoleni
 * Class DokumentySkoleniCommentField
 */
class DokumentySkoleniCommentField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.ID, Dokument.Popis, Dokument.JmenoSouboru" . //0..2
            " FROM DokumentSkoleni JOIN Dokument ON (DokumentSkoleni.id = Dokument.id) " .
            " JOIN TypDokumentu ON (TypDokumentu.kod = Dokument.typ) " .
            " WHERE DokumentSkoleni.cisloSkoleni = " . $params->addParameter($rowData['col-cislo-skoleni']) .
            "  AND (Dokument.zobrazitNaIntrawebu = 1 OR TypDokumentu.zobrazitNaIntrawebu = 1)";

        if (!$rowData['col-prihlasen']) {
            // pokud neni prihlasen, zobrazi jen verejne dokumenty - SE: nemela by tu byt jeste podminka, ze je to zaznam aktualniho uzivatele?
            $sql .= " AND TypDokumentu.Verejny = 1";
        }

        $cursor = $this->database->select($sql, $params);

        $result = '';
        while ($rowDokument = $cursor->fetch()) {
            $linkStr = "?menuid=skoleni&action=download&cisloskoleni={$rowData['col-cislo-skoleni']}&dokument=$rowDokument[0]";

            $result .=
                '<div class="margin-top-2">' .
                "<a target='_blank' href='$linkStr'>" .
                StringUtils::htmlTextConvert($rowDokument[1]) . ' - ' . StringUtils::htmlTextConvert($rowDokument[2]) .
                "</a>" .
                '</div>';
        }

        return $result;
    }
}



/**
 * Class PrihlaseniNaSkoleniBrowser
 */
class PrihlaseniNaSkoleniBrowser extends RonPortalBrowser
{
    public $hromadnePrihlaseni;


    /**
     * PrihlaseniNaSkoleniBrowser constructor.
     * @param $database
     * @throws Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'skoleni&action=zobrazprihlaseni');

        $this->code = 'browserprihlaseniskoleni';
//        $urlHromadnePrihlaseni = $hromadnePrihlaseni != "" ? "&hromadneprihlaseni=$hromadnePrihlaseni&" : "";

            //"(SELECT MIN(Potvrzeni) FROM GenerovaneSkoleni WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni AND GenerovaneSkoleni.RC = SkoleniOsoby.RC AND GenerovaneSkoleni.DatumVznikuPovinnosti = SkoleniOsoby.DatumPovinnosti)";  //22

          //cast(hodnoceniSkoleni as char(300))+(select ' *** Adresa : '+coalesce(umisteni.popis,'') + ', '+coalesce(umisteni.ulice,'') + ', '+coalesce(umisteni.mesto,'')+''+ coalesce(', patro : '+CAST(umisteni.patro as char(2)),'') from umisteni where umisteni.kod=Skoleni.MistoSkoleni)

        /*$this->sqlFromPart =
            'Skoleni JOIN DruhSkoleni ON (DruhSkoleni.Kod = Skoleni.DruhSkoleni) ' .
            'LEFT OUTER JOIN GenerovaneSkoleni ON (GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni) ' .
            'LEFT OUTER JOIN Osoba ON (GenerovaneSkoleni.RC = Osoba.RC) ';
        */

        $this->sqlFromPart =
            '(SELECT DruhSkoleni.Elearning, Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, DruhSkoleni.Kod AS DruhSkoleni, DruhSkoleni.Kategorie AS KategorieSkoleni, GenerovaneSkoleni.DatumVznikuPovinnosti AS DatumPovinnosti, ' .
            ' GenerovaneSkoleni.CisloSkoleni, GenerovaneSkoleni.SchvaleniPrihlasky ' .
            'FROM DruhSkoleni ' .
            'JOIN GenerovaneSkoleni ON (GenerovaneSkoleni.DruhSkoleni = DruhSkoleni.Kod) ' .
            'JOIN Osoba ON (GenerovaneSkoleni.RC = Osoba.RC) ' .
//            "WHERE ((GenerovaneSkoleni.Potvrzeni = 1 " .
//	        "   AND (DruhSkoleni.Elearning = 1 OR GenerovaneSkoleni.DatumSkoleni IS NULL)) OR GenerovaneSkoleni.SchvaleniPrihlasky = 0)" .
            "WHERE (GenerovaneSkoleni.Potvrzeni = 1 " .
	        "    OR  GenerovaneSkoleni.SchvaleniPrihlasky IN (0,2))" .
            'UNION  ' .
            'SELECT DruhSkoleni.Elearning, Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, DruhSkoleni.Kod AS DruhSkoleni, DruhSkoleni.Kategorie AS KategorieSkoleni, NULL AS DatumPovinnosti, NULL AS CisloSkoleni, NULL AS SchvaleniPrihlasky ' .
            'FROM DruhSkoleni ' .
            "JOIN PozadovanaSkoleni ON PozadovanaSkoleni.DruhSkoleni = DruhSkoleni.Kod " .
            "JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
            'JOIN Osoba ON (RozdeleniOsob.RC = Osoba.RC) ' .
            "       WHERE PozadovanaSkoleni.Povinne = 0 ".
            "         AND ( ((RozdeleniOsob.PlatnostOd <= {fn_GETDATE()}" .
            "                AND (RozdeleniOsob.PlatnostDo >= {fn_GETDATE()} OR RozdeleniOsob.PlatnostDo IS NULL))  " .
            "                AND (PozadovanaSkoleni.PlatnostOd <= {fn_GETDATE()}" .
            "                AND (PozadovanaSkoleni.PlatnostDo >= {fn_GETDATE()} OR PozadovanaSkoleni.PlatnostDo IS NULL)))  " .
            "           OR (DruhSkoleni.Elearning = 1)) " .
            "         AND NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GS WHERE GS.RC = Osoba.RC AND GS.DruhSkoleni = DruhSkoleni.Kod AND GS.Potvrzeni = 1 AND GS.DatumSkoleni IS NULL)" .
            ") AS SkoleniOsoby ".
            "JOIN Skoleni ON Skoleni.DruhSkoleni = SkoleniOsoby.DruhSkoleni " .
            "LEFT OUTER JOIN S_StavSchvalPrihlNaKurz ON (S_StavSchvalPrihlNaKurz.Kod = SkoleniOsoby.SchvaleniPrihlasky) " .
            "LEFT OUTER JOIN Umisteni ON Umisteni.Kod = Skoleni.MistoSkoleni ";

        $dnes = DateUtils::today();
        $whereParams = $this->database->createParameters();
        $whereSQL =
            "  ((SkoleniOsoby.CisloSkoleni = '' OR SkoleniOsoby.CisloSkoleni IS NULL) OR SkoleniOsoby.CisloSkoleni = Skoleni.CisloSkoleni) " .

            //u doporucenych nezobrazi, pokud jiz existuje prihlaska, ktera nebyla schvalena, nebo existuje navrh bez prirazeneho skoleni
            // (29.8.2019 zrusena podminka na prihlasene skoleni, pri odhlaseni se skoleni rusi: AND GS.CisloSkoleni = Skoleni.CisloSkoleni)
            'AND ((DatumPovinnosti IS NULL AND NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GS WHERE GS.RC = SkoleniOsoby.RC AND GS.DruhSkoleni = SkoleniOsoby.DruhSkoleni AND (GS.CisloSkoleni = Skoleni.CisloSkoleni OR GS.CisloSkoleni IS NULL)
                AND (GS.Potvrzeni = 1 OR (GS.Potvrzeni = 0 AND GS.SchvaleniPrihlasky <> 2))) ) ' .
//	        '   OR DatumPovinnosti IS NOT NULL)' .
            '   OR (DatumPovinnosti IS NOT NULL AND (SkoleniOsoby.SchvaleniPrihlasky = 2 OR NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GS WHERE GS.RC = SkoleniOsoby.RC AND GS.DruhSkoleni = SkoleniOsoby.DruhSkoleni AND GS.CisloSkoleni = Skoleni.CisloSkoleni
                AND GS.SchvaleniPrihlasky < 2 AND GS.DatumVznikuPovinnosti <> SkoleniOsoby.DatumPovinnosti)))) ';

        //2.10.2019 prozatim bez filtru na konkretni skoleni, neni osestrene prihlasovani/odhlasovani
//        if (isset($_GET['cisloskoleni']))
//            $this->sqlWherePart .= " AND (Skoleni.CisloSkoleni = '" . $this->database->addSlashes($_GET['cisloskoleni']) . "')";
//        else {
            $whereSQL .=
                ' AND Skoleni.StavSkoleni IN (0, 1) ' . //skoleni ve stavu probiha, pripravuje se - stejne jako v aplikaci

                //8.8.2019 doplnena podminka na ukonceni skoleni, lze se prihlasit jen na neukoncene (plati i pro e-learning)
                ' AND ((Skoleni.DatumUkonceniSkoleni > ' . $whereParams->addDateParameter($dnes) . ') OR  (Skoleni.DatumUkonceniSkoleni = ' . $whereParams->addDateParameter($dnes) . ' AND Skoleni.CasUkonceniSkoleni >= ' . $whereParams->addParameter($this->database->timeToParam(time())) . ') OR Skoleni.DatumUkonceniSkoleni IS NULL)' .

                //19.7.2016 RON-PR - narozdil od aplikace je mozne se prihlasit jen na kurz, ktery zacina v budoucnosti a ma datum potvrzeni v budoucnu
                ' AND ( ((Skoleni.DatumPocatkuSkoleni > ' . $whereParams->addDateParameter($dnes) . ') OR (Skoleni.DatumPocatkuSkoleni = ' . $whereParams->addDateParameter($dnes) . ' AND (Skoleni.CasPocatkuSkoleni >= ' . $whereParams->addParameter($this->database->timeToParam(time())) . ' OR Skoleni.CasPocatkuSkoleni IS NULL)) ) ' .
                ' AND   (Skoleni.DatumPotvrzeniUcasti >= ' . $whereParams->addDateParameter($dnes) . ' OR Skoleni.DatumPotvrzeniUcasti IS NULL)' .

                //19.7.2016 RON-PR na e-learningovy je mozne prihlasit vzdy
                '      OR SkoleniOsoby.Elearning = 1' .
                '      )';
//        }

        $this->sqlWherePart = new SqlStatement($whereSQL, $whereParams);

        //v get SQL filter se jeste filtruje na skoleni, ktera jsou pro osobu povinna, nebo doporucena a pokud bylo skoleni vytvorene z planovaneho a prihlaseni je jen pro zajemce o plan, zobrazi se jen potvrzenym zajemcum

        $this->filterForm = new PrihlaseniSkoleniFilterForm($this ->database, $this ->action);  // , $this->hromadnePrihlaseni
        $this->filterForm->loadData();

        $this->hromadnePrihlaseni = (bool)$this ->filterForm ->datumCasSkoleniOd;
        if (!$this->hromadnePrihlaseni) {
            $this->maxPageRows = 100;
        }

        //$this->primaryField = 0;

        $this->cssClass .= ' prihlaseniSkoleniBrowser';

        // Osobni udaje - viditelne jen kdyz jsou zobrazeny vsechny osoby
        $this ->addField()
            ->setSqlFieldName(
                '(SELECT OsobniCislo From PracovniPomer PP' .
                '  WHERE PP.ID = (SELECT MAX(ID) FROM PracovniPomer WHERE PracovniPomer.RC = SkoleniOsoby.RC AND PracovniPomer.DatumNastupu <= {fn_GETDATE()}' .
                '    AND (PracovniPomer.DatumUkonceni >= {fn_GETDATE()} OR PracovniPomer.DatumUkonceni IS NULL))' .
                ') AS OsobniCislo')
            ->setSortSQL('OsobniCislo') ->setHeaderText('Osobni_cislo')
            ->setCode('col-osobni-cislo')
            //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('SkoleniOsoby.Prijmeni') ->setHeaderText('Prijmeni')
            ->setCode('col-prijmeni')
            //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setOrdering(true)
            ->setWrap(true);

//        $this ->defaultSort = $this ->filterForm ->vsechnyOsoby
//            ? 'col-prijmeni'
//            : 'col-datum-pocatku';

        $this ->defaultSort = 'col-datum-pocatku';

        $this ->addField()
            ->setSqlFieldName('SkoleniOsoby.Jmeno') ->setHeaderText('Jmeno')
            ->setCode('col-jmeno')
            //->setVisible($this ->filterForm ->vsechnyOsoby)
            ->setOrdering(true)
            ->setWrap(true);

        // Udaje o skoleni
        $this->addField()
            ->setSqlFieldName('Skoleni.CisloSkoleni') ->setHeaderText('Skoleni_CisloSkoleni')
            ->setCode('col-cislo-skoleni')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('Skoleni.Popis') ->setHeaderText('Skoleni_Popis')
            ->setCode('col-popis-skoleni')
            ->setOrdering(true) ->setWrap(true) ->setClassName('font-weight-bold');

        $this->addField()
            ->setSqlFieldName('Umisteni.Popis AS Umisteni') ->setHeaderText('Skoleni_Umisteni')
            ->setSortSQL('Umisteni')
            ->setCode('col-umisteni')
            ->setOrdering(true) ->setWrap(true);

        $this ->addField(new SkoleniElearningField($this->database))
            ->setSqlFieldName('Skoleni.DatumPocatkuSkoleni') ->setHeaderText('Skoleni_DatumZacatku') ->setFormat('finedate')
            ->setCode('col-datum-pocatku')
            ->setOrdering(true) ->setAlign('center');

        $this->addField()
            ->setSqlFieldName('Skoleni.DatumUkonceniSkoleni') ->setHeaderText('Skoleni_DatumUkonceni') ->setFormat('finedate')
            ->setCode('col-datum-ukonceni')
            ->setOrdering(true) ->setAlign('center');

        $this->addField()
            ->setSqlFieldName('Skoleni.CasPocatkuSkoleni') ->setHeaderText('Skoleni_CasZacatku') ->setFormat('time')
            ->setCode('col-cas-pocatku')
            ->setOrdering(true) ->setAlign('center');

        $this->addField()
            ->setSqlFieldName('Skoleni.CasUkonceniSkoleni') ->setHeaderText('Skoleni_CasUkonceni') ->setFormat('time')
            ->setCode('col-cas-ukonceni')
            ->setOrdering(true) ->setAlign('center');

        $this->addField()
            ->setSqlFieldName('Skoleni.Kredity') ->setHeaderText('Skoleni_Kredity')
            ->setCode('col-kredity')
            ->setOrdering(true);

        $this->addField(new PovinneSkoleniBrowserField($this->database))
            ->setCode('col-povinne')
            ->setHeaderText('Skoleni_Povinnost') ->setAlign('center');

        $this->addField(new SkoleniElearningField($this->database))
            ->setSqlFieldName('Umisteni.KapacitaMistnosti') ->setHeaderText('Skoleni_kapacita_mistnosti')
            ->setCode('col-kapacita-mistnosti')
            ->setOrdering(true) ->setAlign('center');

        $this->addField(new SkoleniElearningField($this->database))
            ->setSqlFieldName('Skoleni.Kapacita') ->setHeaderText('Skoleni_kapacita')
            ->setCode('col-kapacita-kurzu')
            ->setOrdering(true) ->setAlign('center');

        $this->addField(new SkoleniElearningField($this->database))
            ->setSqlFieldName(
                '(SELECT count(rc) FROM GenerovaneSkoleni' .
                ' WHERE CisloSkoleni = Skoleni.CisloSkoleni) AS PocetPrihlasenych')
            ->setSortSQL('PocetPrihlasenych')
            ->setCode('col-pocet-prihlasenych')
            ->setHeaderText('Skoleni_PocetPrihlasenych') ->setOrdering(true) ->setAlign('center');

        $this->addField(new SkoleniPrihlasenBrowserField(ronText('Skoleni_Prihlasen_stav')));

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('SkoleniOsoby.Elearning') ->setHeaderText('Skoleni_elearning') ->setAlign('center')
            ->setCode('col-elearning')
            ->setOptions([0 => ronText('Str_No'), 1 => ronText('Str_Yes')]);

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('Skoleni.PozadovatSchvaleniPrihl') ->setHeaderText('Skoleni_vyzadovat_schvaleni') ->setAlign('center')
            ->setCode('col-vyzadovano-schvaleni')
            ->setOptions([0 => ronText('Str_No'), 1 => ronText('Str_Yes')]);


        $this->addField(new PrihlaseniSkoleniBrowserField($this->database, $this->hromadnePrihlaseni));

        // poznamky atd
        $this ->addField(new UmisteniSkoleniCommentField($this ->database)) ->setType(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('Skoleni.MistoSkoleni')
            ->setCode('col-mistoskoleni')
            ->setHeader(ronText('Skoleni_Adresa') . ': ') ->setClassName('poznamka')
            ->setVisible(!$this ->hromadnePrihlaseni)
            ->setHeaderColSpan(-1);

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(Skoleni.HodnoceniSkoleni)}')
            ->setCode('col-poznamka')
            ->setVisible(RonPortalSettings::getIntValue('skoleni_zobrazit_poznamku')  &&  !$this ->hromadnePrihlaseni)
            ->setHeader(ronText('Skoleni_poznamka') . ': ') ->setClassName('poznamka')
            ->setHeaderColSpan(-1);

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(Skoleni.ObsahSkoleni)}')
            ->setCode('col-obsahskoleni')
            ->setVisible(RonPortalSettings::getIntValue('skoleni_zobrazit_obsah')  &&  !$this ->hromadnePrihlaseni)
            ->setHeader(ronText('Skoleni_obsah_kurzu') . ': ') ->setClassName('poznamka')
            ->setHeaderColSpan(-1);

        $this ->addField(new DokumentySkoleniCommentField($this ->database)) ->setType(BrowserField::FIELD_TYPE_COMMENT)
            ->setCode('col-dokumenty')
            ->setVisible(!$this ->hromadnePrihlaseni)
            ->setHeaderColSpan(-1);


        // Hidden pole
        // ... RC
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SkoleniOsoby.RC')
            ->setCode('col-rc');

        // ... Id PP
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                '(SELECT MAX(Id) FROM PracovniPomer' .
                ' WHERE PracovniPomer.RC = SkoleniOsoby.RC' . // AND PracovniPomer.DatumNastupu <= {fn_GETDATE()}' .
                ' AND (PracovniPomer.DatumUkonceni >= {fn_GETDATE()} OR PracovniPomer.DatumUkonceni IS NULL)' .
                ') AS IdPP')
            ->setSortSQL('IdPP')
            ->setCode('col-idpp');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SkoleniOsoby.DatumPovinnosti')
            ->setCode('col-datumpovinnosti') ->setFormat('date');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('SkoleniOsoby.SchvaleniPrihlasky')
            ->setCode('col-schvaleniprihlasky');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('S_StavSchvalPrihlNaKurz.Popis')
            ->setCode('col-popis-stavu-prihlaseni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Skoleni.DruhSkoleni')
            ->setCode('col-druhskoleni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)  // prihlaseni
            ->setSqlFieldName(
                '(SELECT COUNT(RC) FROM GenerovaneSkoleni' .
                ' WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni AND GenerovaneSkoleni.RC = SkoleniOsoby.RC' .
                ' AND GenerovaneSkoleni.DatumVznikuPovinnosti = SkoleniOsoby.DatumPovinnosti)')
            ->setCode('col-prihlasen');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)  // pocet kurzu na stejnem miste ve stejnem case
            ->setSqlFieldName(
                '(SELECT COUNT(S2.CisloSkoleni) FROM Skoleni S2' .
                ' WHERE EXISTS(SELECT CisloSkoleni FROM Skoleni S' .
                '   WHERE S.CisloSkoleni = Skoleni.CisloSkoleni' .
                '   AND S2.DatumPocatkuSkoleni = S.DatumPocatkuSkoleni' .
                '   AND S2.CasPocatkuSkoleni = S.CasPocatkuSkoleni' .
                '   AND S2.MistoSkoleni = S.MistoSkoleni))')
            ->setCode('col-pocet-kurzu-ve-stejnem-case');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)  // pocet schvalenych prihlasek osoby
            ->setSqlFieldName(
                '(SELECT COUNT(RC) FROM GenerovaneSkoleni' .
                ' WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni' .
                ' AND GenerovaneSkoleni.RC = SkoleniOsoby.RC' .
                ' AND GenerovaneSkoleni.SchvaleniPrihlasky < 2)')
            ->setCode('col-pocet-schvalenych-prihlasek-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)  // pocet absolvovanych elearningu
            ->setSqlFieldName(
                '(SELECT COUNT(ID) FROM GenerovaneSkoleni JOIN EKurzOsoby' .
                ' ON ((GenerovaneSkoleni.RC = EKurzOsoby.RC_Skoleni) AND (GenerovaneSkoleni.DruhSkoleni = EKurzOsoby.DruhSkoleni)' .
                '  AND (GenerovaneSkoleni.DatumVznikuPovinnosti = EKurzOsoby.DatumVznikuPovinnosti))' .
                ' WHERE GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni AND GenerovaneSkoleni.RC = SkoleniOsoby.RC' .
                '  AND EKurzOsoby.Stav IN (3, 4))')
            ->setCode('col-absolvovanych-elearningu-rc');
    }



    function setFilterForm()
    {
        // nedela nic - filter form uz je vytvoren v konstruktoru
    }


    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $filtr = $this ->filterForm ->buildSqlFilter($params);

        $pravaPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        $filtr .=
            ($filtr ? "\n AND " : '') .
            'EXISTS(SELECT RC FROM PracovniPomer' .
            ' WHERE PracovniPomer.RC = SkoleniOsoby.RC AND PracovniPomer.DatumNastupu <= ' . $params ->addDateParameter(time()) .
            '  AND (PracovniPomer.DatumUkonceni >= ' . $params ->addDateParameter(time()) . ' OR PracovniPomer.DatumUkonceni IS NULL) ' .
            '  AND (' . $pravaPrihlasit->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '       OR ' . $pravaPotvrditPrihlaseni->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today(), $params) .
            '   )' .
            ")\n";



        /*$filtr =
            "(" . //doporucene skoleni
            "EXISTS(SELECT * FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
            "       WHERE Skoleni.DruhSkoleni=PozadovanaSkoleni.DruhSkoleni ".
            "         AND PozadovanaSkoleni.Povinne = 0 ".
            "         AND RozdeleniOsob.RC = Osoba.RC " .
            "         AND ((RozdeleniOsob.PlatnostOd <= Skoleni.DatumPocatkuSkoleni " .
            "               AND (RozdeleniOsob.PlatnostDo >= Skoleni.DatumPocatkuSkoleni OR RozdeleniOsob.PlatnostDo IS NULL))  " .
            "           OR (DruhSkoleni.Elearning = 1))" .
            " )" .
            " OR " . //vygenerovane skoleni ve stavu potrvrzeny navrh
            "EXISTS(SELECT * FROM GenerovaneSkoleni " .
            "       WHERE GenerovaneSkoleni.RC =  Osoba.RC " .
            "         AND Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni " .
            //"         AND GenerovaneSkoleni.DuvodSkoleni = 0 " .
            "         AND GenerovaneSkoleni.Potvrzeni = 1" .
            "         AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)) " .
            ")";*/

        //pokud bylo skoleni vytvorene z planovaneho a prihlaseni je jen pro zajemce o plan, zobrazi se jen potvrzenym zajemcum
        $filtr .=
            ' AND (Skoleni.UcastniciDlePlanSkoleni = 0 ' .
            '  OR EXISTS(SELECT RC FROM PlanovaneSkoleniOsob ' .
            '  WHERE PlanovaneSkoleniOsob.CisloSkoleni = Skoleni.CisloPlanovanehoSkoleni ' .
            '  AND PlanovaneSkoleniOsob.Potvrzeni = 1 ' .
            '  AND PlanovaneSkoleniOsob.RC = SkoleniOsoby.RC ))' .
            "\n";

        if ($this ->filterForm ->getFilterValue('osoba')) {
            $filtr .=
                ' AND SkoleniOsoby.RC = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc'));
        }

        foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
            if ($rozdeleniFilter ->getValue() !== null) {
                $sqlRozdeleni = RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'SkoleniOsoby',
                    $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                    DateUtils::today(), DateUtils::today(),
                    $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                    $params,
                    true);

                if ($sqlRozdeleni) {
                    $filtr .=
                        ($filtr ? ' AND ' : '') .
                        $sqlRozdeleni;
                }
            }
        }

//        if ($this ->filterForm ->kodRozdeleni) {
//            $filtrRozdeleni1 = RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'SkoleniOsoby',
//                RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this ->filterForm ->kodRozdeleni,
//                DateUtils::today(), DateUtils::today(),
//                $this ->filterForm ->vcetnePodrizenychRozdeleni,
//                $params,
//                true);
//
//            if ($filtrRozdeleni1) {
//                $filtr .=
//                    ($filtr ? ' AND ' : '') .
//                    $filtrRozdeleni1;
//            }
//        }

        if ($this ->filterForm ->getFilterValue('druhskoleni')) {
            $filtr .=
                ' AND Skoleni.DruhSkoleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('druhskoleni')) .
                "\n";
        }


        if ($this ->filterForm ->getFilterValue('datumod')) {
            $filtr .=
                ' AND Skoleni.DatumPocatkuSkoleni >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
                "\n";
        }

        if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
            $filtr .=
                ' AND Skoleni.DatumPocatkuSkoleni <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo')) .
                "\n";
        }


        if ($this ->filterForm ->getFilterValue('umisteni')) {
            $filtr .=
                ' AND Skoleni.MistoSkoleni =  ' . $params->addParameter($this ->filterForm ->getFilterValue('umisteni')) .
                "\n";
        }

        if ($this ->filterForm ->datumCasSkoleniOd) {
            $filtr .=
                ' AND Skoleni.DatumPocatkuSkoleni = ' . $params->addDateParameter((int)$this ->filterForm ->datumCasSkoleniOd) .
                ' AND Skoleni.CasPocatkuSkoleni =  ' . $params ->addParameter($this->database->timeToParam((int)$this ->filterForm ->datumCasSkoleniOd)) .
                "\n";
        }
        
        if ($this ->filterForm ->getFilterValue('typPovinnosti') !== null) {
            switch ($this ->filterForm ->getFilterValue('typPovinnosti')) {
                case 1: //povinne
                    $filtr .= 
                        " AND EXISTS(SELECT * FROM GenerovaneSkoleni " .
                        "        WHERE GenerovaneSkoleni.RC =  SkoleniOsoby.RC" .
                        "          AND Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni " .
                        //"          AND GenerovaneSkoleni.DuvodSkoleni = 0 " .
                        "          AND GenerovaneSkoleni.Povinne = 1" .
                        "          AND GenerovaneSkoleni.Potvrzeni = 1" .
                        "          AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni))" .
                        "\n";
                    break;
                case 2: //doporucene
                    $filtr .= 
                        " AND ( " .
                        "   EXISTS(SELECT * FROM PozadovanaSkoleni JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni) " .
                        "       WHERE Skoleni.DruhSkoleni=PozadovanaSkoleni.DruhSkoleni ".
                        "         AND PozadovanaSkoleni.Povinne = 0 ".
                        "         AND RozdeleniOsob.RC =  SkoleniOsoby.RC " .
                        "         AND RozdeleniOsob.PlatnostOd <= Skoleni.DatumPocatkuSkoleni " .
                        "         AND (RozdeleniOsob.PlatnostDo >= Skoleni.DatumPocatkuSkoleni OR RozdeleniOsob.PlatnostDo IS NULL))  " .
                        "   OR EXISTS(SELECT * FROM GenerovaneSkoleni " .
                        "        WHERE GenerovaneSkoleni.RC =  SkoleniOsoby.RC" .
                        "          AND Skoleni.DruhSkoleni=GenerovaneSkoleni.DruhSkoleni " .
                        "          AND (GenerovaneSkoleni.Povinne = 0 OR GenerovaneSkoleni.Povinne IS NULL)" .
                        "          AND (GenerovaneSkoleni.Potvrzeni = 1)" .
                        "          AND ((GenerovaneSkoleni.CisloSkoleni = '' OR GenerovaneSkoleni.CisloSkoleni IS NULL) OR GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni)) ".
                        ")\n";
                    break;
            }
        }

        if ($this ->filterForm ->getFilterValue('specifikace')) {
            $filtr .=
                ' AND Skoleni.Specifikace = ' . $params ->addParameter($this ->filterForm ->getFilterValue('specifikace'));
        }


        if ($this ->filterForm ->getFilterValue('kategorie')) {
            $filtr .=
                ' AND SkoleniOsoby.KategorieSkoleni = ' . $params ->addParameter($this ->filterForm ->getFilterValue('kategorie'));
        }

        if ($this ->filterForm ->popisSkoleni) {
            $filtr .=
                ' AND Skoleni.Popis = ' . $params ->addParameter($this ->filterForm ->popisSkoleni);
        }

        return new SqlStatement($filtr, $params);
    }

    /**
     * @param $values
     * @param $className
     * @throws Exception
     */
    private function zobrazElearning($values, $className)
    {
        $params = $this ->database ->createParameters();

        $sql =
            'SELECT EKurz.ID, EKurz.Nazev, EKurz.PlatnostOd, EKurz.PlatnostDo, ' . // 0..3
            '  EKurzOsoby.Id, EKurzOsoby.Stav, EKurzOsoby.DatumCasZahajeni, EKurzOsoby.PosledniPristup, ' . // 4..7
            '  RevizeEKurzu.SestavaProCertifikat, RevizeEKurzu.UlozitCertifikatDoDok, SestavyProIntraweb.Nazev, ' . // 8..10
            '  GenerovaneSkoleni.DatumVznikuPovinnosti' . //11
            ' FROM (EKurzDruhuSkoleni INNER JOIN EKurz  ON EKurzDruhuSkoleni.IDKurzu = EKurz.ID ' .
            '   INNER JOIN Skoleni ON Skoleni.DruhSkoleni = EKurzDruhuSkoleni.KodDruhuSkoleni' .
            '   INNER JOIN GenerovaneSkoleni ON GenerovaneSkoleni.CisloSkoleni = Skoleni.CisloSkoleni )' .
            ' LEFT OUTER JOIN (EKurzOsoby INNER JOIN RevizeEKurzu ON EKurzOsoby.IdRevize = RevizeEKurzu.Id)' .
            '   ON EKurzOsoby.RC_Skoleni = GenerovaneSkoleni.RC AND EKurzOsoby.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni AND EKurzOsoby.DatumVznikuPovinnosti = GenerovaneSkoleni.DatumVznikuPovinnosti AND RevizeEKurzu.IDKurzu = EKurz.ID ' .
            ' LEFT OUTER JOIN SestavyProIntraweb ON RevizeEKurzu.SestavaProCertifikat = SestavyProIntraweb.Kod' .
            " WHERE EKurzDruhuSkoleni.KodDruhuSkoleni = " . $params->addParameter($values['col-druhskoleni']) .
            '   AND EKurz.Stav = 1' . // jen aktivni
            "   AND Skoleni.CisloSkoleni = " . $params->addParameter($values['col-cislo-skoleni']) .
            "   AND GenerovaneSkoleni.RC = " . $params->addParameter($values['col-rc']) .
            "   AND GenerovaneSkoleni.SchvaleniPrihlasky <> 2" .
            ' ORDER BY EKurzDruhuSkoleni.Poradi, EKurzOsoby.DatumCasZahajeni DESC';

        $cursor = $this->database->getSqlResult($sql, $params);

        while ($rowElearning = $cursor->fetch()) {
            //pokud je u kurzu vyzadovano schvaleni a neni aktualne schvalena prihlaska, zobrazi jen absolvovane kurzy
            if ((!$values['col-vyzadovano-schvaleni']) || ($values['col-vyzadovano-schvaleni'] && (((($values['col-schvaleniprihlasky'] == -1) || ($values['col-schvaleniprihlasky'] == 1))) || (($rowElearning[5] == 3) || ($rowElearning[5] == 4))))) {
                echo "<tr class='$className'><td></td><td colspan='" . ($this ->columnCount - 1) . "'>";

                if ($rowElearning[3] && $this->database->stringToTimestamp($rowElearning[3]) < DateUtils::today()) {
                    echo ronText('Platnost_kurzu_skoncila');
                } else {
                    if ($rowElearning[4]) {
                        // existuje zaznam EKurzOsoby
                        $linkStr = '?menuid=prezentace&kurzosoby=' . $rowElearning[4];
                    } else {
                        $linkStr =
                            '?menuid=prezentace&zahajitkurz=' . $rowElearning[0] .
                            '&druhskoleni=' . StringUtils::htmlTextConvert($values['col-druhskoleni']) .
                            '&datumvznikupovinnosti=' . $this->database->stringToTimestamp($rowElearning[11]); // date('j.n.Y', $this ->database ->stringToTimestamp($values[5]) ); - je tam i cas!
                    }

                    if ($linkStr) echo "<a target='_blank' href='$linkStr'>";
                    echo StringUtils::htmlTextConvert($rowElearning[1]);
                    if ($linkStr) echo "</a>";

                    switch ($rowElearning[5]) {
                        case 1 :
                            echo ' - <span class="ekurz_probiha">' . ronText('EKurz_probiha') . '</span>';
                            break;

                        case 3 :
                            echo ' - <span class="ekurz_absolvovan_uspesne">' . ronText('EKurz_absolvovan_uspesne') . '</span>';
                            if ($rowElearning[8]) {
                                // download certifikatu
                                // TODO: presunout do kontroleru skoleni?
                                echo ", <a target='_blank' href='?menuid=prezentace&action=certifikat&id={$rowElearning[4]}'>" . StringUtils::htmlTextConvert($rowElearning[10]) . '</a>';
                            }
                            break;

                        case 4 :
                            echo ' - <span class="ekurz_absolvovan_neuspesne">' . ronText('EKurz_absolvovan_neuspesne') . '</span>';
                            break;
                    }
                }

                echo "</td></tr>\n";
            }
        }
    }

    /**
     * @param array $values
     * @param string $className
     * @param int $rowNumber
     * @param bool $edit
     * @throws Exception
     */
    public function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if (!$this->hromadnePrihlaseni) {
            //druh skoleni je e-learning
            if ($values['col-elearning']) {
                if (!$values['col-vyzadovano-schvaleni']  ||  $values['col-schvaleniprihlasky'] == -1  ||  $values['col-schvaleniprihlasky'] == 1) {
                    //u kurzu se nevyzaduje schvaleni, nebo je schvalen
                    $this->zobrazElearning($values, $className);
                }
            }
        }

        $this ->renderSeparator();
    }


    public function renderDialogs()
    {
        //dialog Existuji terminy, smazat?
        echo
            "<div id='dialogSkoleniTerminy' class='rondialog' title='" . ronText('Skoleni_odhlasit') . "' >" .

            "<div id='skoleniTerminy' class='dialogcontent'>" . ronText('Skoleni_SmazatTerminy') . "</div>" .

            "<div class='buttons align-center'>" .
            "<input type='submit' id='btnTerminySmazat' class='submitbutton' value='" . ronText('Str_Yes') . "' />" .
            "<input type='button' id='btnTerminyNechat' class='button cancel' value='" . ronText('Str_No') . "' />" .
            "</div>" .

            "</div>";

        //dialog Ponechat potvrzeny navrh
        echo
            "<div id='dialogSkoleniNavrh' class='rondialog' title='" . ronText('Skoleni_odhlasit') . "' >" .

            "<div id='skoleniNavrh' class='dialogcontent'>" . ronText('Skoleni_NechatNavrh') . "</div>" .

            "<div class='buttons'>" .
            "<input type='submit' id='btnPonechatNavrh' class='submitbutton' value='" . ronText('Str_Yes') . "' />" .
            "<input type='button' id='btnSmazatNavrh' class='button' value='" . ronText('Str_No') . "' />" .
            "<input type='button' id='btnNavrhZavrit' class='button cancel' value='" . ronText('Cancel') . "' />" .
            "</div>" .

            "</div>";

        //dialog Vyhledat dalsi terminy/prihlasit na jeden kurz
        echo
            "<div id='dialogSkoleniVyhledatUmisteni' class='rondialog' title='' >" .

            "<div id='skoleniVyhledatUmisteni' class='dialogcontent'>" . ronText('Vyhledat_kurzy') . "</div>" .

            "<div class='buttons align-center'>" .
            "<input type='submit' id='btnVyhledatUmisteni' class='submitbutton' value='" . ronText('Str_Yes') . "' />" .
            "<input type='button' id='btnPrihlasitKurz' class='button' value='" . ronText('Str_No') . "' />" .
            "<input type='button' id='btnUmisteniZavrit' class='button cancel' value='" . ronText('Cancel') . "' />" .
            "</div>" .

            "</div>";
    }

    /**
     * @throws Exception
     */
    function renderContent()
    {
        if ($this->hromadnePrihlaseni) {
            echo
                "<form method='POST' name='hromadneSchvaleniSK' action='?menuid=skoleni&action=zobrazprihlaseni'>
                 <input type='hidden' id='hromadnePrihlasitSkoleni' name='hromadnePrihlasitSkoleni' value='1'>";
        }

        parent::renderContent();

        if ($this->hromadnePrihlaseni) {
            echo
                "<button type='button' class='button clickHref' data-ron-href='?menuid=skoleni&action=zobrazprihlaseni' name='zpet' >" . ronText('zpet') . "</button>" .
                "<button type='submit' class='button' data-ron-working-info='" . ronText('Probiha_zpracovani_dat') . "'>" . ronText('Skoleni_prihlasit_odhlasit_vybrane') . "</button>" .
                "</form>\n";
        }

        $this->renderDialogs();
    }
}
