<?php
/**
 * Roman.Szablatura RON Software 17.07.2019
 */

namespace RonPortal\Skoleni;


use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Prava;


/**
 * Class SchvalovaniPlanuKurzuOsobyFilter
 */
class SchvalovaniPlanuKurzuOsobyFilter extends SkoleniFilterForm
{
    /**
     * SchvalovaniPlanuKurzuOsobyFilter constructor.
     * @param $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'skoleni', true);

        $pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');
        $pravoSchvalit = Prava::vratPrava('skoleni_schvaleni_planu');

//        $this ->nastavOsoby(true,
//            [$pravoPrihlasit, $pravoSchvalit],
//            time(), time()); // prava podle aktualniho casu

        $this ->addFilterPP(false, [$pravoPrihlasit, $pravoSchvalit])
            ->setDatumOd(time())
            ->setDatumDo(time());

        Prava::nastavObdobiProPrava(time(),time());

        if ($pravoPrihlasit->jsouNejakePrava() || $pravoSchvalit->jsouNejakePrava()) {
            $this->addFilter((new FilterFormFilter('filtrprihlaseninaplanovaneskoleni'))
                ->addItem('planypodleosoby', TextManager::_('plany_podle_osoby'))
                ->addItem('planypodlekurzu', TextManager::_('plany_podle_kurzu'))
                ->setSessionName('filtrprihlaseninaplanovaneskoleni'),
                true // top - na prvnim radku
            );
        }

        $this->addFilter(
            (new FilterFormFilterDatumOdDo($this)),
            true
        );

        $this ->addDruhSkoleniFilter();

        $this ->addFilter((new FilterFormFilter('action', FilterFormFilter::TYPE_HIDDEN))
            ->setValue('prihlaseninaplanovaneskoleni')
        );

        $this ->addFilter((new FilterFormFilter('aktualniplanykurzu', FilterFormFilter::TYPE_CHECKBOX))
            ->setCaption2(TextManager::_('filtr_pouze_aktualni_plany'))
        );

        $this ->addFilter((new FilterFormFilter('neschvaleneplanykurzu', FilterFormFilter::TYPE_CHECKBOX))
            ->setCaption2(TextManager::_('filtr_pouze_neschvalene'))
            ->setSqlFilter('PlanovaneSkoleniOsob.Potvrzeni = 0')
        );

        $this ->addFilter((new FilterFormFilter('pouzenaplanukurzu', FilterFormFilter::TYPE_CHECKBOX))
            ->setCaption2(TextManager::_('filtr_pouze_na_planu_kurzu'))
            ->setSqlFilter('PlanovaneSkoleniOsob.RC IS NOT NULL')   // TODO: nemelo by tu byt AND PlanovaneSkoleniOsob.Potvrzeni < 2 ? (aby nezobrazoval zamitnute)
        );

    }
}


/**
 * Class EditacePlanovanehoSkoleniField
 */
class EditacePlanovanehoSkoleniField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // Prava na editaci planovanych skoleni se urci podle pole "Zadatel" (tj. kdo pozadal o kurz)
        if (Prava::vratPrava('skoleni_plan') ->lzeZobrazitRC($rowData['col-zadatel'], time())) {
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        return '';
    }
}



/**
 * Class SchvalovaniPlanuOsobyBrowser
 */
class SchvalovaniPlanuOsobyBrowser extends PrihlaseniNaPlanovanaBrowser
{
    /**
     * SchvalovaniPlanuKurzuPodrizenehoBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'skoleni');

        $this->code = 'browserschvalovaniplanukurzuosoby';
        $this->tableId = 'schvalovaniplanukurzuosoby';
        $this->browserUrl = "?menuid=skoleni&action=prihlaseninaplanovaneskoleni&filtrprihlaseninaplanovaneskoleni=planypodleosoby";

        Prava::nastavObdobiProPrava(time(), time());

        $fromParameters = $this ->database ->createParameters();
        $this->sqlFromPart = new SqlStatement(
            'PracovniPomer JOIN PlanovaneSkoleni' .
            ' ON 1 = 1' .  // odfiltruje se ve WHERE
            ' LEFT OUTER JOIN PlanovaneSkoleniOsob' .
            '  ON PlanovaneSkoleniOsob.CisloSkoleni = PlanovaneSkoleni.CisloSkoleni' .
            '  AND PlanovaneSkoleniOsob.RC = PracovniPomer.RC',
            $fromParameters
        );

        $commentColSpan = 1;

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleni.CisloSkoleni')
            ->setCode('col-cisloskoleni');
        $this ->primaryField = 0;

        if (Prava::jsouNejakePrava('skoleni_plan')) {
            $commentColSpan++;

            // odkaz na editaci planovaneho skoleni
            $this->addField(new EditacePlanovanehoSkoleniField($this ->database))
                ->setType(BrowserField::FIELD_TYPE_TEXT)
                ->setText('<i class="inline-glyph fa glyph-edit"></i>')
                ->setLink('?menuid=skoleni&action=planovaneskolenikurzy&formaction=edit&record=')
                ->setLinkAttributes(['target' => '_blank'])
                ->setCode('col-detail')
                ->setAlign('center');
        }

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.Id')
            ->setCode('col-idpp');
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PracovniPomer.RC')
            ->setCode('col-rc');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DruhSkoleni') ->setHeaderText('druh_skoleni') ->setOrdering(true)
            ->setCode('col-druhskoleni');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.Popis') ->setHeaderText('Popis') ->setClassName('font-weight-bold') ->setOrdering(true)
            ->setCode('col-popis');

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DatumPocatkuSkoleni') ->setHeaderText('datum_pocatku_skoleni')
            ->setFormat('date') ->setOrdering(true) ->setAlign('center')
            ->setCode('col-datumpocatkuskoleni');

        $this ->defaultSort = '-col-datumpocatkuskoleni';

        $this->addField()
            ->setSqlFieldName('PlanovaneSkoleni.DatumUkonceniSkoleni') ->setHeaderText('datum_ukonceni_skoleni')
            ->setFormat('date') ->setOrdering(true) ->setAlign('center')
            ->setCode('col-datumukonceni');

        $this->addField(new PocetOsobNaKurzuField($database));

        $this->addField(new PotvrzeniSkoleniField($database));

        $this->addField(new SchvaleniPrihlaseniOsobyNaKurzuField($database, 0))
            ->setCode('col-schvaleni');

        // hidden pole - pouzivaji se v kodu
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleniOsob.RC')
            ->setCode('col-prihlaseni-rc');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleniOsob.Potvrzeni')
            ->setCode('col-potvrzeni');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN) // pro kontrolu prav na editaci planovaneho skoleni
            ->setSqlFieldName('PlanovaneSkoleni.Zadatel')
            ->setCode('col-zadatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('PlanovaneSkoleni.PocetOsob AS Kapacita')
            ->setCode('col-pocetosob');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName(
                '(SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO' .
                ' WHERE PSO.CisloSkoleni = PlanovaneSkoleni.CisloSkoleni AND PSO.Potvrzeni < 2 ) AS CelkemPrihlasenych')
            ->setCode('col-celkem-prihlasenych');

        // poznamky (zobrazeny na dalsich radcich)
        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(PlanovaneSkoleniOsob.PoznamkaSchvalovani)}')
            ->setCode('col-poznamkaschvalovani')
            ->setHeader(TextManager::_('Skoleni_poznamka_schvalovani') . ': ', -$commentColSpan)
            ->setClassName('poznamka')
            ->setHeaderClass('font-weight-bold');


        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(PlanovaneSkoleni.Poznamka)}')
            ->setCode('col-poznamka')
            ->setHeader(TextManager::_('Skoleni_poznamka') . ': ', -$commentColSpan)
            ->setClassName('poznamka')
            ->setHeaderClass('font-weight-bold');
            //->setFooterColSpan(1);
    }


    function setFilterForm()
    {
        $this->filterForm = new SchvalovaniPlanuKurzuOsobyFilter($this ->database);
    }


    /**
     * @return SqlStatement
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            'PracovniPomer.Id = ' . $params ->addParameter((int)$this ->filterForm ->getFilterValue('osoba'));

        $result .=
            ($result ? ' AND ' : '') .
            'PlanovaneSkoleni.Stav < 2';

        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                'PlanovaneSkoleni.DatumPocatkuSkoleni >= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod'));
        }

        if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
            $result .=
                ($result ? ' AND ' : '') .
                'PlanovaneSkoleni.DatumPocatkuSkoleni <= ' . $params->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
        }

        if ($this ->filterForm ->getFilterValue('druhskoleni')) {
            $result .=
                ($result ? ' AND ' : '') .
                'PlanovaneSkoleni.DruhSkoleni = ' . $params ->addParameter($this ->filterForm ->getFilterValue('druhskoleni'));
        }

//        if ($this ->filterForm ->getFilterValue('neschvaleneplanykurzu')) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                'PlanovaneSkoleniOsob.Potvrzeni = 0';
//        }

        if ($this ->filterForm ->getFilterValue('aktualniplanykurzu')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' PlanovaneSkoleni.DatumPocatkuSkoleni >= ' . $params->addDateParameter(time());
        }

//        if ($this ->filterForm ->getFilterValue('pouzenaplanukurzu')) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                'PlanovaneSkoleniOsob.RC IS NOT NULL';
//        }


        return new SqlStatement($result, $params);
    }
}
