<?php
/**
 * Seznam dlouhodobych skoleni s moznosti editace terminu a zaznamenani ucasti
 */

namespace RonPortal\Skoleni;

use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\RonPortalBrowser;


/**
 * Class UcastNaSkoleniFiltr
 * @package Ron\RonPortal
 */
class UcastNaSkoleniFiltr extends SkoleniFilterForm
{
    /**
     * UcastNaSkoleni constructor.
     * @param Database $database
     * @internal param string $action
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, ['menuid' => 'skoleni', 'action' => 'skoleniucast'], true);

        $this ->addFilter(
            (new FilterFormFilterDatumOdDo($this))
                ->setDatumOdDoLinks(['currentMonth', 'currentQuarter', 'currentYear'])
                ->setRequired(true)
                ->setCaption(TextManager::_('zobrazit_terminy_za'))
                ->setSessionName('filterform_ucastskoleni_datum')
        );

        $this ->addCisloSkoleniFilter(
            'SELECT DISTINCT CisloSkoleni, Popis FROM Skoleni WHERE %FILTER% ORDER BY %ORDER%',
            TextManager::_('nevybrano'),
            'DlouhodobyKurz = 1'
        );

        $this ->addFilter((new FilterFormFilter('jennevyplnene'))
            ->setSessionName('jen_nevyplnene_formulare_skoleni')
            ->addItem(0, TextManager::_('Vse'))
            ->addItem(1, TextManager::_('jen_nevyplnene_formulare_skoleni'))
            ->setDefaultValue(0)
        );
    }


    function renderDetailDlouhodobehoKurzu()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Skolitel, DatumPocatkuSkoleni, CasPocatkuSkoleni, DatumUkonceniSkoleni, CasUkonceniSkoleni' .        //0-4
            ' FROM Skoleni' .
            ' WHERE Skoleni.CisloSkoleni = ' . $params ->addParameter($this ->getFilterValue('cisloskoleni'));
        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        /**
         * Popis zacatku - konce casem od - do skoleni napr.        1.7.2017 - 30.9.2017     8:00 - 16:00
         **/
        echo '<div class="filterrow">';
        echo  '<span class="filtercaption margin-top-2">' . \Ron\ronText('zacatek_konec_skoleni').  '</span>';
        echo  '<div>';

        if ($row) {
            echo
                '<strong>' .
                StringUtils::format('date', $this->database->stringToTimestamp($row[1])) .
                ($row[3]
                    ? " - " .
                    StringUtils::format('date', $this->database->stringToTimestamp($row[3]))
                    : ''
                ) .
                '</strong>';

            if ($row[2] || $row[4]) {
                echo
                    ' <span class="margin-left-2">' .
                    StringUtils::format('time', $this->database->stringToTimestamp($row[2])) .
                    ' - ' .
                    StringUtils::format('time', $this->database->stringToTimestamp($row[4])) .
                    '</span>';
            }
        }
        echo '</div>';
        echo '</div>';

        // jmeno skolitele
        if ($row  &&  $row[0]) {
            echo '<div class="filterrow" id="filter_row_jmeno_skolitele">';
            echo '<span class="filtercaption">' . \Ron\ronText('skolitel') . '</span>';
            echo '<strong><span class="filtercaption">' . StringUtils::htmlTextConvert($row[0]) . '</span></strong>';
            echo '</div>';
        }
    }


    function renderFilters()
    {
        parent::renderFilters();

        $this->renderDetailDlouhodobehoKurzu();

        echo
            '<div class="filterrow">' .
            '<div class="col margin-top-2"><a href="?menuid=skoleni&action=skolenikurzy">' . TextManager::_('Zpet_na_seznam') . '</a></div>' .
            '</div>';
    }
}



class EditaceTerminuSkoleniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $datum = $this ->database ->stringToTimestamp($rowData['col-datum']);

        return
            '<i class="inline-glyph glyph-button fa glyph-edit open-dialog"' .
            ' data-dialog="#termin_skoleni_dialog"' .
            ' data-ronform-record="' . StringUtils::htmlTextConvert($rowData['col-cisloskoleni']) . '"' .
            //' data-ronform-url="?menuid=skolenijs&action=terminskoleni&cisloskoleni=' . StringUtils::htmlTextConvert($rowData[0]) . '&datum=' . StringUtils::format('date', $datum)  . '"' .
            ' data-ronform-url="?menuid=skolenijs&action=terminskoleni&datum=' . StringUtils::format('date', $datum)  . '&cisloskoleni="' .
            ' title="' . TextManager::_('Editovat_zaznam') . '">' .
            '</i>';
    }
}


class EditaceUcastnikuSkoleniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $datum = $this ->database ->stringToTimestamp($rowData['col-datum']);

        return
            '<span class="js open-dialog"' .
            ' data-dialog="#ucast_na_skoleni_dialog"' .
            ' data-ronform-record="' . StringUtils::htmlTextConvert($rowData['col-cisloskoleni']) . '"' .
            //' data-ronform-url="?menuid=skolenijs&action=terminskoleni&cisloskoleni=' . StringUtils::htmlTextConvert($rowData[0]) . '&datum=' . StringUtils::format('date', $datum)  . '"' .
            ' data-ronform-url="?menuid=skolenijs&action=ucastnaskoleni&datum=' . StringUtils::format('date', $datum)  . '&cisloskoleni="' .
            '>' . TextManager::_('zaznamenat_ucast') .
            '</span>';
    }
}


class UcastNaSkoleniEditovanaField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return TextManager::_(($rowData['col-ucast']  ||   $rowData['col-neucast'])
            ? 'Str_Yes'
            : 'Str_No');
    }
}



class UcastNaSkoleniPercentField extends BrowserField
{
    private $pocetPrihlasenych;

    /**
     * UcastNaSkoleniPercentField constructor.
     * @param Database $database
     * @param int $pocetPrihlasenych
     */
    public function __construct($database, $pocetPrihlasenych)
    {
        parent::__construct($database);
        $this->pocetPrihlasenych = $pocetPrihlasenych;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        if ($this ->pocetPrihlasenych) {
            $result .= ' (' . (int)($value / $this ->pocetPrihlasenych * 100) . '%)';
        }

        return $result;
    }
}



/**
 * Class SkoleniUcastBrowser
 * @package Ron\RonPortal
 */
class UcastNaSkoleniBrowser extends RonPortalBrowser
{
    private $pocetPrihlasenych = null;

    /** @var bool */
    private $jePravoEditovatUcast;

    /**
     * SkoleniUcastBrowser constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'skoleni');

        //$this->prava = Prava::vratPrava('skoleni_prihlasit');

        $this
            ->setTitle('ucast_na_skoleni')
            ->setCode('browserskoleniucast')
            ->setTableId('browserskoleniucast')
            ->setBrowserUrl('?menuid=skoleni&action=skoleniucast');

        $this->sqlFromPart = 'TerminSkoleni';

        $this->filterForm = new UcastNaSkoleniFiltr($database);
        $this->filterForm->loadData();

        $this ->jePravoEditovatUcast = SkoleniJsController::jePravoNaZapisUcasti($this ->filterForm ->getFilterValue('cisloskoleni'));

        $this ->pocetPrihlasenych = $this ->getPocetPrihlasenych();

        $this->defaultSort = '-col-datum';

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('CisloSkoleni')
            ->setCode('col-cisloskoleni');

        $this->addField()
            ->setSqlFieldName('Datum') ->setHeaderText('Datum') ->setFormat('finedate') ->setOrdering(true)
            ->setCode('col-datum')
            ->setAlign('center');

        $this->addField()
            ->setSqlFieldName('Cas') ->setHeaderText('Cas') ->setFormat('time') ->setAlign('center')
            ->setCode('col-cas');

        //$this->addField(new UcastZaznamenanaEditovan($database, null));

        //$this->addField(new ProcentualniUcastNaSkoleniField($database, null));

        //$this->addField(new ProcentualniNeucastNaSkoleniField($database, null));

        $this ->addField(new UcastNaSkoleniEditovanaField())
            ->setCode('col-formular-vyplnen')
            ->setAlign('center')
            ->setHeaderText('ucast_zaznamenana');

        $this ->addField(new UcastNaSkoleniPercentField($this ->database, $this ->pocetPrihlasenych))
            ->setSqlFieldName(
                '(SELECT COUNT (*) FROM UcastNaTerminuSkoleni' .
                ' WHERE UcastNaTerminuSkoleni.CisloSkoleni = TerminSkoleni.CisloSkoleni' .
                ' AND UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum' .
                ' AND Pritomen = 1) AS PocetPritomnych')
            ->setCode('col-ucast')
            ->setHeaderText('Ucast')
            ->setAlign('center');

        $this ->addField(new UcastNaSkoleniPercentField($this ->database, $this ->pocetPrihlasenych))
            ->setSqlFieldName(
                '(SELECT COUNT (*) FROM UcastNaTerminuSkoleni' .
                ' WHERE UcastNaTerminuSkoleni.CisloSkoleni = TerminSkoleni.CisloSkoleni' .
                ' AND UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum' .
                ' AND Pritomen = 0  AND  DuvodNepritomnosti = 1) AS PocetOmluvenych')
            ->setCode('col-neucast')
            ->setHeaderText('neucast')
            ->setAlign('center');

        //$this->addField(new UcastNaSkoleniField($database, null));

        if ($this ->jePravoEditovatUcast) {
            $this->addField(new EditaceUcastnikuSkoleniField($database))
                ->setCode('col-ucastnici');

            $this->addField(new EditaceTerminuSkoleniField($database))
                ->setCode('col-edit');
        }

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_castwidememo(Poznamka)}') ->setHeaderColSpan(1)
            ->setClassName('poznamka');

    }


    function setFilterForm()
    {
        //parent::setFilterForm();      Nedela nic, byl nastaven v konstruktoru
    }


    private function getPocetPrihlasenych()
    {
        $params = $this ->database ->createParameters();
        return DBUtils::getSqlValue($this ->database,
            'SELECT COUNT(*)' .
            ' FROM GenerovaneSkoleni GS' .
            ' WHERE GS.CisloSkoleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('cisloskoleni')) .  // TODO: predelat
            '  AND (GS.Potvrzeni<5)',
            $params);
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result =
            ($result ? ' AND ' : '') .
            'TerminSkoleni.CisloSkoleni = ' . $params->addParameter($this ->filterForm ->getFilterValue('cisloskoleni')) .
            ' AND TerminSkoleni.Datum >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod')) .
            ' AND TerminSkoleni.Datum < ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));

        if ($this ->filterForm ->getFilterValue('jennevyplnene')) {
            $result .=
                ' AND NOT EXISTS (' .
                '  SELECT * FROM UcastNaTerminuSkoleni' .
                '  WHERE UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum' .
                '  AND UcastNaTerminuSkoleni.CisloSkoleni = TerminSkoleni.CisloSkoleni' .
                '  AND (Pritomen = 1  OR  DuvodNepritomnosti = 1)' .
                ' )';
        }

        return new SqlStatement($result, $params);
    }

    function renderFilter()
    {
        parent::renderFilter();

        if ($this ->filterForm ->getFilterValue('cisloskoleni')  &&  $this ->jePravoEditovatUcast) {
            echo
                '<div class="col margin-top-2">' .
                // odkaz pro zobrazeni dialogu
                '<span class="js open-dialog"' .
                ' data-dialog="#termin_skoleni_dialog">' .
                TextManager::_('vytvorit_termin') .
                '</span>' .
                '</div>';
        }
    }


    public function render()
    {
        parent::render();

        if ($this ->jePravoEditovatUcast) {
            // dialog pro editaci terminu
            $f = new TerminSkoleniFormular($this->database, $this->filterForm->getFilterValue('cisloskokleni'));
            $f->render();

            // dialog pro editaci ucasti
            $f = new UcastNaSkoleniFormular($this->database, $this->filterForm->getFilterValue('cisloskoleni'));
            $f->render();
        }
    }
}
