<?php
/**
 * stanislav.elbl, 08.07.2022 14:16
 */

namespace RonPortal\Skoleni;

use Ron\Controls\FormField;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\RonPortalDBRecord;


class UcastNaSkoleniDataField extends FormField
{
    public function loadJsonValue($database, $rowData)
    {
        // aktualni hodnota pole je nactena uz v konstruktoru formulare
        return $this ->defaultValue;
    }
}



class UcastNaSkoleniFormular extends RonPortalForm
{
    /** @var string */
    private $cisloSkoleni;

    /** @var int */
    private $termin;

    /** @var string[]   Seznam ID osob, ktere se zpracovavaji */
    private $osoby = [];

    /**
     * TerminSkoleniFormular constructor.
     * @param \Ron\DB\Database $database
     * @param string $cisloSkoleni
     * @param int $termin
     */
    public function __construct($database, $cisloSkoleni = null, $termin = null)
    {
        parent::__construct($database, 'new', true); // dialogmode = true

        $this ->cisloSkoleni = $cisloSkoleni;
        $this ->termin = $termin;

        $this ->code = 'Skoleni';
        $this ->stornoAction = "#";

        $this ->htmlParams = array_merge($this ->htmlParams, [
            'id' => 'ucast_na_skoleni_dialog',
            'data-ronform-submit-url' => '?menuid=skolenijs&action=zapisucast',
            'data-ronform-after-submit' => 'reload',
            'data-table-name' => 'UcastNaTerminuSkoleni',
            'title' => \Ron\ronText('ucast_na_skoleni')
        ]);

        $this ->sqlTableName = 'Skoleni';

        //$this ->setXmlEventsName('TERMIN_SKOLENI');

        $this ->fCisloSkoleni = $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('CisloSkoleni') ->setTableName('Skoleni') ->setPrimary(true) ->setReadOnly(true)
            ->addFieldCssClass('ronform-field') ->setDefaultValue($this ->cisloSkoleni)
            ->setCode('field-cisloskoleni')
            ->setOtherHtmlParams([
                'data-ronform-field-name' => 'CisloSkoleni'
            ]);

        $this ->fCisloSkoleni = $this ->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName('Datum') ->setTableName('TerminSkoleni') ->setPrimary(true) ->setReadOnly(true)
            ->addFieldCssClass('ronform-field') ->setDefaultValue($this ->termin) ->setDataType('date') ->setFormat('date')
            ->setOtherHtmlParams([
                'data-ronform-field-name' => 'Datum'
            ]);

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setCaption('Nazev') ->setReadOnly(true)
            ->setFieldName('Popis') ->setSqlFieldName('Popis') ->addFieldCssClass('ronform-field font-weight-bold ronform-field')
            ->setCode('field-popis')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Popis']);

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setCaption('termin') ->setReadOnly(true)
            ->addFieldCssClass('ronform-field font-weight-bold ronform-field')
            ->setCode('field-termin')
            ->setOtherHtmlParams(['data-ronform-field-name' => 'Datum']);

        // nadpisy tabulky
        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setCode('header-jmeno') ->setNewRow(true) ->setRowClass('row form-section')->setCaption(false) //->setShowColumns(false)
            ->setDataColumnClass('col-8 col-sm-5')
            ->setText(TextManager::_('Jmeno'));

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setCode('header-pritomen') ->setNewRow(false) ->setCaption(false)
            ->setDataColumnClass('col-2 col-sm-1')
            ->setText(TextManager::_('pritomen'));

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setCode('header-omluven') ->setNewRow(false) ->setCaption(false)
            ->setText(TextManager::_('omluven'))
            ->setDataColumnClass('col-2 col-sm-1');

        $this ->addField(FormField::FIELD_TYPE_TEXT)
            ->setCode('header-poznamka') ->setNewRow(false) ->setCaption(false)
            ->setText(TextManager::_('Poznamka'))
            ->setDataColumnClass('col-12 col-sm-5');

        // seznam osob - vytvori editovatelna pole pro kazdou osobu
        // a v pripade, ze je zadan termin, tak rovnou nacita i aktualni hodnoty v databazi
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Osoba.RC, Osoba.Prijmeni, Osoba.Jmeno, U.Pritomen, U.DuvodNepritomnosti, U.Poznamka' .
            ' FROM GenerovaneSkoleni GS' .
            '  INNER JOIN Osoba ON (Osoba.RC = GS.RC)' .
            '  LEFT OUTER JOIN UcastNaTerminuSkoleni U' .
            ($termin
                ? ' ON U.CisloSkoleni = GS.CisloSkoleni  AND  U.RC = Osoba.RC  AND  U.Datum = ' . $params ->addDateParameter($termin)
                : ' ON 0 = 1'
            ) .
            ' WHERE GS.CisloSkoleni = ' . $params->addParameter($this ->cisloSkoleni) .
            '  AND (GS.Potvrzeni<5)' .
            ' ORDER BY Osoba.Prijmeni, Osoba.Jmeno';

        $this ->addField(FormField::FIELD_TYPE_GROUP)
            ->setCode('group-osoby')
            ->setFieldCssClass('oddEvenChild overflow-auto');

        /** @var  $cursor */
        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $this ->osoby[] = $row[0];

            $this ->addField(FormField::FIELD_TYPE_TEXT)
                ->setRowClass('row padding-top padding-bottom')
                ->setCode('osoba-jmeno-' . $row[0]) ->setCaption(false)
                ->setText(StringUtils::htmlTextConvert($row[1] . ' ' . $row[2]))
                ->setDataColumnClass('col-8 col-sm-5');

            $this ->addField(new UcastNaSkoleniDataField(FormField::FIELD_TYPE_CHECK))
                ->setCode('osoba-pritomen-' . $row[0]) ->setNewRow(false) ->setCaption(false)
                ->setFieldName('osoba-pritomen-' . $row[0]) ->setSqlFieldName(null)
                ->setDefaultValue((int)$row[3])
                ->setDataColumnClass('col-2 col-sm-1 align-center')
                ->addFieldCssClass('ronform-field')
                ->setOtherHtmlParams([
                    'data-ronform-field-name' => 'osoba-pritomen-' . $row[0]
                ]);

            $this ->addField(FormField::FIELD_TYPE_CHECK)
                ->setCode('osoba-omluven-' . $row[0]) ->setNewRow(false) ->setCaption(false)
                ->setFieldName('osoba-omluven-' . $row[0]) ->setSqlFieldName(null)
                ->setDefaultValue((int)$row[4])
                ->setDataColumnClass('col-2 col-sm-1 align-center')
                ->addFieldCssClass('ronform-field')
                ->setOtherHtmlParams([
                    'data-ronform-field-name' => 'osoba-omluven-' . $row[0]
                ]);

            $this ->addField(FormField::FIELD_TYPE_MEMO)
                ->setCode('osoba-poznamka-' . $row[0]) ->setNewRow(false) ->setCaption(false)
                ->setFieldName('osoba-poznamka-' . $row[0]) ->setSqlFieldName(null)
                ->setDefaultValue($row[5])
                ->setDataColumnClass('col-12 col-sm-5')
                ->addFieldCssClass('ronform-field')
                ->setOtherHtmlParams([
                    'data-ronform-field-name' => 'osoba-poznamka-' . $row[0]
                ])
                ->setRows(1);

//            $this ->addField(FormField::FIELD_TYPE_TEXT)
//                ->setCode('header-poznamka') ->setNewRow(false) ->setShowColumns(false)
//                ->setText(TextManager::_('Poznamka'))
//                ->setFieldCssClass('col-12 col-sm-5');
        }
        $cursor ->close();

        $this ->loadConfiguration();
    }



    public function getSqlFromPart()
    {
        return
            'Skoleni INNER JOIN TerminSkoleni' .
            ' ON TerminSkoleni.CisloSkoleni = Skoleni.CisloSkoleni';
    }


    public function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        return new SqlStatement(
            'TerminSkoleni.CisloSkoleni = ' . $params ->addParameter($this ->cisloSkoleni) .
            ' AND TerminSkoleni.Datum = ' . $params ->addDateParameter($this ->termin),
            $params);
    }



    public function editRecord()
    {
        foreach ($this ->osoby as $osobaRC) {
            $rec = new RonPortalDBRecord($this ->database, 'UcastNaTerminuSkoleni', $this);

            $rec ->addStringField('CisloSkoleni', true, 10, $this ->cisloSkoleni);
            $rec ->addDateField('Datum', true, $this ->termin);
            $rec ->addStringField('RC', true, 15, $osobaRC);

            $rec ->addIntField('Pritomen', false, (int)($_POST['osoba-pritomen-' . $osobaRC] ?? 0));
            $rec ->addIntField('DuvodNepritomnosti', false, (int)($_POST['osoba-omluven-' . $osobaRC] ?? 0));
            $rec ->addMemoField('Poznamka', false, $_POST['osoba-poznamka-' . $osobaRC] ?? null);

            $params = $this ->database ->createParameters();
            $rec ->setWhere(
                'CisloSkoleni = ' . $params ->addParameter($this ->cisloSkoleni) .
                ' AND Datum = ' . $params ->addDateParameter($this ->termin) .
                ' AND RC = ' . $params ->addParameter($osobaRC),
                $params
            );

            $rec ->updateOrInsert();
        }
    }

    // TODO: stary dialog mel jeste filtr na vsechny/pritomne/nepritomne
    // TODO: stary dialog mel jeste v hlavicce zobrazeno   pocet ucastniku/pritomni/ucast:  7/5/71%
    // TODO: stary dialog mel jeste moznost hromadne kopirovat poznamku do ostatnich prazdnych poznamek
    // TODO: stary dialog mel jeste v hlavicce moznost hromadne za/od-skrtnout vsechny pritomnosti/omluvenky
}
