<?php
/**
 * stanislav.elbl, 29.09.2021 9:14
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\ComboBox;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SQLUtils;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\RonPortalSettings;

class RadekUctovaniSluzebniCestyFormular implements ViewObject
{
    /** @var  Database */
    var $database;

    /** @var SluzebniCesta */
    var $sluzebniCesta;
    /** @var int */
    var $datum;

    /** @var RadekUctovaniSluzebniCesty[] */
    var $radky;

    var $readOnly;


    /**
     * @param Database $database
     * @param SluzebniCesta $sluzebniCesta
     * @param int $datum
     */
    function __construct($database, $sluzebniCesta, $datum)
    {
        $this ->database = $database;

        $this ->sluzebniCesta = $sluzebniCesta;
        $this ->datum = $datum;
    }


    /**
     * @return bool
     */
    function loadRadky_DB()
    {
        $this ->readOnly = !$this ->sluzebniCesta ->lzeEditovatSluzebniCestu($this ->database, false);

        $vychoziMena = null;
        if (!$this ->readOnly) {
            $vychoziMena = SluzebniCestyUtils::vratVychoziMenu($this ->database, $this ->sluzebniCesta ->id);
        }

        $this ->radky = [];

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT TypRUctovaniSlCesty.Kod, TypRUctovaniSlCesty.Popis, TypRUctovaniSlCesty.NahradaZaVlastniVozidlo, TypRUctovaniSlCesty.DruhPohybu,' . // 0..3
            ' RadekUctovaniSCesty.CisloRadku, RadekUctovaniSCesty.Den, RadekUctovaniSCesty.Popis,' . // 4..6 POZOR! muze byt null!
            ' RadekUctovaniSCesty.Castka, RadekUctovaniSCesty.KodMeny, RadekUctovaniSCesty.ZpusobPlatby, RadekUctovaniSCesty.CastkaDPH,' . // 7..10
            ' RadekUctovaniSCesty.NVV_SpotrebaVozidla, RadekUctovaniSCesty.NVV_PocetUjetychKm, RadekUctovaniSCesty.NVV_NepocitatZaklSazbu,' . // 11..13
            ' RadekUctovaniSCesty.NVV_NeprokazanaCenaPHM, RadekUctovaniSCesty.NVV_CenaPHM, RadekUctovaniSCesty.NVV_DruhPHM, RadekUctovaniSCesty.NVV_Vozidlo' . // 14..17
            '  FROM TypRUctovaniSlCesty ' . ($this ->readOnly ? 'INNER JOIN' : 'LEFT OUTER JOIN') . ' RadekUctovaniSCesty' .
            '  ON TypRUctovaniSlCesty.Kod = RadekUctovaniSCesty.Typ ' .
            ' AND RadekUctovaniSCesty.SluzebniCesta_Id = ' . $params ->addParameter((int)$this -> sluzebniCesta -> id) .
            ' AND RadekUctovaniSCesty.Den = ' . $params ->addTimestampParameter($this ->datum) .
            ' WHERE (TypRUctovaniSlCesty.Aktivni = 1) AND (NOT (TypRUctovaniSlCesty.Kod IN (1,2,3,4))) '.
            ' ORDER BY TypRUctovaniSlCesty.Popis';

        $cursor = $this ->database ->select($sql, $params);

        while ($row  = $cursor ->fetch()) {
            $radek = new RadekUctovaniSluzebniCesty($this -> sluzebniCesta -> id);

            $radek ->den = $this ->datum;

            $radek ->typ = $row[0];
            $radek ->popisTypu = TextManager::_('db_typructovanislcesty_popis_' . $row[0], $row[1]);
            $radek ->druhPohybu = (int)$row[3];

            $radek ->cisloRadku = $row[4];
            $radek ->den = $this ->datum;
            $radek ->popis = $row[6];

            $radek ->castka = $row[7];
            $radek ->castkaStr = $radek -> getCastkaView() ? StringUtils::format('number2', $radek -> getCastkaView()) : ''; // podle druhu puhybu a castky

            $radek ->kodMeny = $row[8];
            if (!$radek ->kodMeny) {
                $radek ->kodMeny = $vychoziMena;
            }
            $radek ->zpusobPlatby = $row[9];
            $radek ->castkaDPH = $row[10];
            $radek ->castkaDPHStr = StringUtils::format('number2', $row[10]);

            $radek ->nahrada = $row[2] == '1';
            $radek ->NVVSpotrebaVozidla = $row[11];
            $radek ->NVVPocetUjetychKm = $row[12];
            $radek ->NVVNepocitatZakladniSazbu = $row[13];

            $radek ->NVVNeprokazanaCenaPHM = $row[14];
            $radek ->NVVCenaPHM = $row[15];
            $radek ->NVVDruhPHM = $row[16];
            $radek ->NVVVozidlo = $row[17];

            if (!$radek ->cisloRadku  &&  $radek ->nahrada) {
                // zatim nevyplneny radek - nastavi spotrebu podle vozidla
                if ($this ->sluzebniCesta ->spotrebaVozidla) {
                    $radek->NVVSpotrebaVozidla = $this->sluzebniCesta->spotrebaVozidla;
                }

                if ($this ->sluzebniCesta ->phmVozidla) {
                    $radek ->NVVDruhPHM = $this ->sluzebniCesta ->phmVozidla;
                }
            }

            $this ->radky[] = $radek;
        }

        $cursor ->close();

        return true;
    }


    /**
     * @param RadekUctovaniSluzebniCesty $radek
     * @param boolean  $delete
     * @return bool
     * @throws \Exception
     */
    function updateRadekDB($radek, $delete)
    {
        $rec = new RadekUctovaniSCDBRecord($this ->database, $radek->idSluzebniCesty, $radek->cisloRadku, $this);
        $rec ->fDen ->setNewValue($radek ->den);
        $rec ->fPopis ->setNewValue($radek ->popis);
        $rec ->fTyp ->setNewValue($radek->typ);
        $rec ->fKodMeny ->setNewValue($radek->kodMeny);
        $rec ->fZpusobPlatby ->setNewValue($radek->zpusobPlatby);

        $jeNahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $radek ->typ);

        if ($jeNahrada) {
            $rec ->fCastka ->setNewValue(0); // nastavi na 0, musi se preuctovat!
            $rec ->fNVV_SpotrebaVozidla ->setNewValue($radek->NVVSpotrebaVozidla);
            $rec ->fNVV_PocetUjetychKM ->setNewValue($radek->NVVPocetUjetychKm);
            if (!RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT)) {
                $rec ->fNVV_NepocitatZaklSazbu ->setNewValue($radek->NVVNepocitatZakladniSazbu ? 1 : 0);
            }
            $rec ->fNVV_NeprokazanaCenaPHM ->setNewValue($radek->NVVNeprokazanaCenaPHM ? 1 : 0);

            if ($radek->NVVNeprokazanaCenaPHM) {
                $rec ->fNVV_DruhPHM ->setNewValue($radek->NVVDruhPHM);
            } else {
                $rec ->fNVV_CenaPHM ->setNewValue($radek->NVVCenaPHM);
            }

            $rec ->fNVV_Vozidlo ->setNewValue($radek->NVVVozidlo);
        } else {
            // znamenko nastavi automaticky podle druhu pohybu (+/-)
            if ($radek->druhPohybu == 1) {
                $radek->castka = abs($radek->castka);
                $radek->castkaDPH = abs($radek->castkaDPH);
            }
            else if ($radek->druhPohybu == 2) {
                $radek->castka = -abs($radek->castka);
                $radek->castkaDPH = -abs($radek->castkaDPH);
            }

            $rec ->fCastka ->setNewValue($radek->castka);
            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH)) {
                $rec->fCastkaDPH->setNewValue((float)$radek->castkaDPH);
            }
        }

        if ($radek->cisloRadku) {
            if ($delete) {
                return $rec->deleteRecord();
            } else {
                if (!$rec->loadRecord()) {
                    throw new \Exception('Chyba! Editovany radek nebyl nalezen!');
                }

                return $rec->updateRecord();
            }
        } else {
            // radek nema pridelene cislo => vlozi novy radek
            $noveCislo = SQLUtils::newSerialNumber($this ->database,
                'RadekUctovaniSCesty', 'CisloRadku', 'SluzebniCesta_ID = ' . (int)$this->sluzebniCesta->id);

            $rec ->fCisloRadku ->setNewValue($noveCislo);

            $rec->insertRecord();
            $radek->cisloRadku = $noveCislo;

            return true;
        }
    }


    function deleteRadek($cisloRadku)
    {
        foreach ($this->radky as $i => $radek) {
            if ($radek->cisloRadku == $cisloRadku) {
                // nasel radek - smaze ho
                if (!$this->updateRadekDB($radek, true)) {
                    return false;
                }

                array_splice($this->radky, $i, 1);

                return true;
            }
        }

        return true; // zadany radek neexistuje - neni co mazat
    }



    /**
     * @param $radek RadekUctovaniSluzebniCesty
     * @param $indexhtml int
     * @return bool Zkontroluje zadana data. V pripade chyby vypise chybove hlaseni a vraci false
     */
    function loadRadekFromPost($radek, $indexhtml)
    {
        $radek->den = DateUtils::dateStringToTimestamp($_POST["datum_$indexhtml"]);

        $radek->typ = trim($_POST["typ_$indexhtml"]) != '' ? (int)$_POST["typ_$indexhtml"] : null;
        $radek->nahrada = TypRadkuUctovaniSluzebniCesty::typUctovaniJeNVV($this ->database, $radek ->typ);

        $radek->kodMeny = $_POST["kodMeny_$indexhtml"];
        $radek->zpusobPlatby = $_POST["zpusobPlatby_$indexhtml"] == '' ? null : (int)$_POST["zpusobPlatby_$indexhtml"];
        $radek->popis = $_POST["popis_$indexhtml"];

        if ($radek->nahrada) {
            if (isset($_POST["NVV_PocetUjetychKM_$indexhtml"])) {
                $radek->NVVPocetUjetychKm = (int)$_POST["NVV_PocetUjetychKM_$indexhtml"];
            }
            $radek->NVVNepocitatZakladniSazbu = isset($_POST["NVV_NepocitatZaklSazbu_$indexhtml"]) && $_POST["NVV_NepocitatZaklSazbu_$indexhtml"];
            $radek->NVVNeprokazanaCenaPHM = isset($_POST["NVV_NeprokazanaCenaPHM_$indexhtml"]) && $_POST["NVV_NeprokazanaCenaPHM_$indexhtml"] == 1;

            if ($radek->NVVNeprokazanaCenaPHM && isset($_POST["NVV_DruhPHM_$indexhtml"])) {
                $radek->NVVDruhPHM = $_POST["NVV_DruhPHM_$indexhtml"];
            }

            if (isset($_POST["NVV_SpotrebaVozidla_$indexhtml"])) {
                $radek->NVVSpotrebaVozidla = StringUtils::stringToFloat($_POST["NVV_SpotrebaVozidla_$indexhtml"]);
            }
            if (!$radek->NVVNeprokazanaCenaPHM && isset($_POST["NVV_CenaPHM_$indexhtml"])) {
                $radek->NVVCenaPHM = StringUtils::stringToFloat($_POST["NVV_CenaPHM_$indexhtml"]);
            }

            if (isset($_POST["NVV_Vozidlo_$indexhtml"])) {
                $radek->NVVVozidlo = $_POST["NVV_Vozidlo_$indexhtml"];

                if ($radek ->NVVVozidlo) {
                    // pokud je zadano vozidlo, tak spotrebu a druh PHM nacte z databaze (tj. prebije to, co zadal uzivatel)
                    $params = $this ->database ->createParameters();
                    $cursor = $this ->database ->getSqlResult(
                        'SELECT Vozidlo.RegistracniZnacka, Vozidlo.DruhPHM, Vozidlo.Spotreba '.
                        '  FROM Vozidlo '.
                        ' WHERE Vozidlo.RegistracniZnacka = ' . $params->addParameter($radek->NVVVozidlo), $params);

                    if ($cursor  &&  $row = $cursor ->fetch()) {
                        if ($row[1]) {
                            $radek ->NVVDruhPHM = $row[1];
                        }
                        if ($row[2]) {
                            $radek ->NVVSpotrebaVozidla = $row[2];
                        }
                    }
                }
            }


        } else {
            $radek->castkaStr = trim($_POST["castka_$indexhtml"]);

            if ($radek->castkaStr != '') {
                if (StringUtils::isFloat($radek->castkaStr)) {
                    $radek->castka = StringUtils::stringToFloat($radek->castkaStr);
                } else {
                    MessageManager::addErrorMessage( \Ron\ronText('Bad_number_format') );
                    $radek->error = true;
                }
            }

            if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_ZADAVAT_DPH)) {
                $radek->castkaDPHStr = trim($_POST["castkaDPH_$indexhtml"]);

                if ($radek->castkaDPHStr != '') {
                    if (StringUtils::isFloat($radek->castkaDPHStr)) {
                        $radek->castkaDPH = StringUtils::stringToFloat($radek->castkaDPHStr);
                    } else {
                        MessageManager::addErrorMessage( \Ron\ronText('Bad_number_format') );
                        $radek->error = true;
                    }
                }
            }
        }

        return !$radek->error;
    }


    /**
     * @param RadekUctovaniSluzebniCesty $radek
     * @param string $indexHtml
     * @return bool
     * @throws \Exception
     */
    function updateRadek($radek, $indexHtml)
    {
        // TODO: zkontrolovat prava na typ radku uctovani
        if (TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($this ->sluzebniCesta ->f_PP ->idPP, $radek ->typ)) {
            if (!$this->loadRadekFromPost($radek, $indexHtml)) {
                return false;
            }

            return $this->updateRadekDB($radek, false); // delete = false
        }

        return true;
    }


    /**
     * @param $radek RadekUctovaniSluzebniCesty
     * @param $indexhtml int
     * @return bool
     * @throws \Exception
     */
    function insertRadek($radek, $indexhtml)
    {
        $vlozit = false;

        if (!$radek) {
            $radek = new RadekUctovaniSluzebniCesty($this ->sluzebniCesta ->id);
            $radek ->novyRadek = true;

            if ($this ->sluzebniCesta ->spotrebaVozidla) {
                $radek ->NVVSpotrebaVozidla = $this ->sluzebniCesta ->spotrebaVozidla;
            }

            if ($this ->sluzebniCesta ->phmVozidla) {
                $radek ->NVVDruhPHM = $this ->sluzebniCesta ->phmVozidla;
            }
        }

        if ( !$this -> loadRadekFromPost($radek, $indexhtml) ) {
            // v radku je chyba - nezapisuje ho
            return false;
        }

        if (!TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($this ->sluzebniCesta ->f_PP ->idPP, $radek ->typ)) {
            throw new NeniPravoException();
        }

        if ($radek ->castka  ||  $radek ->popis  ||  $radek ->NVVPocetUjetychKm) {
            $vlozit = true;
        } else if ($radek ->novyRadek  &&  $radek ->typ  &&  $radek ->typ != RonPortalSettings::getIntValue('typradkuuctovanisc')) {
            $vlozit = true;
        }

        if ($vlozit) {
            if ($radek ->novyRadek) {
                // prida nove vlozeny radek do seznamu
                $this ->radky[] = $radek;
            }

            if ($radek ->nahrada) {

            }
            else {
                if (!isset($radek ->castka)) {
                    $radek ->error = true;
                    throw new \Exception(\Ron\ronText('form.Missing_value') . \Ron\ronText('Castka') );
                }
            }

            if (!$radek ->kodMeny) {
                $radek ->error = true;
                throw new \Exception(\Ron\ronText('form.Missing_value') . ' (' . \Ron\ronText('Mena') . ')');
            }

            if (!isset($radek ->zpusobPlatby)) {
                $radek ->error = true;
                throw new \Exception(\Ron\ronText('form.Missing_value') . ' (' . \Ron\ronText('SC_Zpusob_platby') .')');
            }

            if ($radek ->novyRadek) {
                // pro nove vlozeny radek doplni popis typu
                $typPohybu = TypRadkuUctovaniSluzebniCesty::vratTypUctovani($this->database, (int)$radek->typ);
                $radek->popisTypu = TextManager::_('db_typructovanislcesty_popis_' . (int)$radek->typ, $typPohybu->popis);
                $radek->druhPohybu = $typPohybu ->druhPohybu;
            }

            $radek -> error = !$this -> updateRadekDB($radek, false);

            if ($radek ->error) {
                return false;
            }

            if ($radek ->novyRadek) {
//                $params = $this ->database ->createParameters();
//                $radek ->popisTypu = TextManager::_('db_typructovanislcesty_popis_' . (int)$radek ->typ, false);
//                if (!$radek ->popisTypu) {
//                    $radek->popisTypu = DBUtils::getSqlValue($this->database,
//                        'SELECT Popis FROM TypRUctovaniSlCesty WHERE Kod = ' . $params->addParameter((int)$radek->typ),
//                        $params);
//                }
                $radek ->novyRadek = false;
            }

            return true;
        }

        return true; // zadny novy radek nepridavam
    }



    /**
     * @param $cisloRadku int   cislo vyhledavaneho radku
     * @param $typ int  typ hledaneho radku - pouzije se pouze pokud neni zadano cisloRadku
     * @return null|RadekUctovaniSluzebniCesty   vraci radek se zadanym cislem, nebo "prazdny" radek se zadanym typem (pokud neni cislo zadano)
     */
    function vratRadek($cisloRadku, $typ)
    {
        foreach($this ->radky as $radek) {
            if ( $cisloRadku  &&  $radek ->cisloRadku == $cisloRadku ) {
                return $radek;
            }

            if ( !$cisloRadku  &&  !$radek ->cisloRadku  &&  $radek ->typ == $typ) {
                return $radek;
            }
        }

        return null;
    }


    function loadPost()
    {
        if ( !isset($_POST['editace_radku_uctovani_sluzebky'])  ||  !$_POST['editace_radku_uctovani_sluzebky']  ||
            !isset($_POST['pocetradku'])  ||  !$_POST['pocetradku'] ) {
            return true;
        }

        Csrf::validateCSRFPostParam('radekuctovanisluzebnicestyform_' . $this ->sluzebniCesta ->id);

        $result = true;

        $radekDelete = isset($_POST['smazat_radek_uctovani_sluzebky']) ? (int)$_POST['smazat_radek_uctovani_sluzebky'] : null;

        for ($i = 1;  $i <= $_POST['pocetradku'];  $i++) {
            $radek = null;

            try {
                $cisloRadku = isset($_POST['cisloradku_' . $i]) ? (int)$_POST['cisloradku_' . $i] : null;

                if ($cisloRadku && $radekDelete == $cisloRadku) {
                    // tento radek ma byt smazan
                    if (!$this->deleteRadek((int)$_POST['smazat_radek_uctovani_sluzebky'])) {
                        $result = false;
                    }
                } else {
                    $typ = isset($_POST['typ_' . $i]) ? (int)($_POST['typ_' . $i]) : null;
                    /** @var $radek RadekUctovaniSluzebniCesty */
                    $radek = isset($_POST['novyradek_' . $i])
                        ? null
                        : $this->vratRadek($cisloRadku, $typ);

                    if ($radek && $radek->cisloRadku) {
                        if (!$this->updateRadek($radek, $i)) {
                            $result = false;
                        }
                    } else {
                        if (!$this->insertRadek($radek, $i)) {
                            $result = false;
                        }
                    }
                }
            } catch (\Exception $e) {
                MessageManager::addErrorMessage($e);
                $result = false;
                if ($radek) {
                    $radek ->error = true;
                }
            }
        }

        return $result;
    }


    function loadRadky() {
        if ( !$this -> loadRadky_DB() ) {
            return false;
        }

        if ( !$this -> loadPost() ) {
            return false;
        }

        if ( isset($_POST['novyRadekUctovani'])  &&  $_POST['novyRadekUctovani'] ) {
            // predvyplni novy radek
            $radek = new RadekUctovaniSluzebniCesty($this ->sluzebniCesta ->id);
            $radek ->novyRadek = true;
            $radek ->den = $this ->datum;
            if (RonPortalSettings::getIntValue('typradkuuctovanisc')) {
                $radek ->typ = RonPortalSettings::getIntValue('typradkuuctovanisc');
            }

            $radek ->kodMeny = SluzebniCestyUtils::vratVychoziMenu($this ->database, $this ->sluzebniCesta ->id);

            $this ->radky[] = $radek;
        }

        return true;
    }


    /**
     * @param $radek RadekUctovaniSluzebniCesty
     * @param $index integer
     */
    function zobrazRadekNVV($radek, $index) {
        if ($this ->readOnly  &&  !$radek ->cisloRadku) {
            return;
        }

        $readOnly = $this ->readOnly;
        if (!$readOnly) {
            $readOnly = !TypRadkuUctovaniSluzebniCesty::lzeEditovatRadekUctovani($this ->sluzebniCesta ->f_PP ->idPP, $radek ->typ);
        }

        // spotreba - litru
        if ($readOnly) {
            echo '<div>';
            if ($radek ->NVVVozidlo) {
                echo \Ron\ronText('Vozidlo') . ": {$radek ->NVVVozidlo}, ";
            }
            echo \Ron\ronText('Spotreba_TP') . ': ' . StringUtils::format('number_2', $radek ->NVVSpotrebaVozidla) . ' ' . \Ron\ronText('Litru_100');
            echo '</div>';

            if ($radek ->NVVNeprokazanaCenaPHM) {
                $params = $this ->database ->createParameters();
                $druhPHM = TextManager::_('db_druhphm_popis_' . $radek ->NVVDruhPHM, false); // zkusi najit preklad
                if (!$druhPHM) {
                    // nenasel - nacte text z databaze
                    $druhPHM = DBUtils::getSqlValue($this->database,
                        'SELECT Popis FROM DruhPHM WHERE Kod = ' . $params->addParameter($radek->NVVDruhPHM),
                        $params);
                }
                echo '<div>';
                echo StringUtils::htmlTextConvert($druhPHM);
                echo '</div>';
            }

            echo '<div>';
            echo StringUtils::format('number_2', $radek ->NVVPocetUjetychKm) . ' ' . \Ron\ronText('km');
            if (!$radek ->NVVNeprokazanaCenaPHM) {
                echo ', ' . \Ron\ronText('Cena_PHM') . ': ' . StringUtils::format('number_2', $radek ->NVVCenaPHM) . ' ' . \Ron\ronText('Kc_litr');
            }
            echo '</div>';

            if ($radek ->NVVNepocitatZakladniSazbu) {
                echo '<div>' . \Ron\ronText('Nepocitat_zakladni_sazbu') . '</div>';
            }

            return;
        }

        echo '<table class="middle">';

        $value = isset($radek ->NVVSpotrebaVozidla)
            ? StringUtils::format('number_2', $radek ->NVVSpotrebaVozidla)  // number_format($radek ->NVVSpotrebaVozidla, 1, ',', '')
            : '';

        echo '<tr>';


        if ($readOnly) {
            echo "<td class='align-right'>" . \Ron\ronText('Vozidlo') . ":</td><td></td>";
        } else {
            $params = $this ->database ->createParameters();

            $combo = new ComboBox("NVV_Vozidlo_$index");
            $combo ->captionIndex = 0;
            $combo ->emptyRow = ' ';
            $combo ->attrIndexes = ['data-spotreba:number_2' => 1, 'data-druhphm' => 2];
            $combo->loadData($this ->database,
                'SELECT Vozidlo.RegistracniZnacka, Vozidlo.Spotreba, Vozidlo.DruhPHM' .
                ' FROM VozidloOsoby, Vozidlo' .
                ' WHERE VozidloOsoby.RegistracniZnacka = Vozidlo.RegistracniZnacka' .
                ' AND VozidloOsoby.RC = ' . $params->addParameter($this->sluzebniCesta->f_PP->rc),
                $radek->NVVVozidlo, $params);

            if (count($combo->data) > 1) {  // prazdny radek nepocita
                // pokud ma zamestnanec prirazene vozidlo, zobrazi vyber
                echo "<td class='align-right'>" . \Ron\ronText('Vozidlo') . "</td>";
                echo '<td>';

                $combo ->htmlParams = [
                    'data-edit-spotreba' => "#NVV_SpotrebaVozidla_$index",
                    'data-edit-druhphm' => "#NVV_DruhPHM_$index"
                ];
                $combo ->className = "vozidlo-uctovani-sc";
                $combo->render();

                echo \Ron\ronText('Spotreba_TP') . ": <span class='NVV_SpotrebaVozidla'><input type='text' id='NVV_SpotrebaVozidla_$index' class='appinput inputFloat' name='NVV_SpotrebaVozidla_$index' value='" . StringUtils::htmlTextConvert($value) . "' size='8' readonly='readonly' /></span> " . \Ron\ronText('Litru_100');
                echo " <span id='NVV_PopisVozidla_$index'></span>";
            } else {
                // jinak jen zadani spotreby
                echo "<td class='align-right'>" . \Ron\ronText('Spotreba_TP') . '</td>';
                echo "<td><input type='text' name='NVV_SpotrebaVozidla_$index' class='appinput inputFloat' value='" . StringUtils::htmlTextConvert($value) . "' size='8' maxlength='8' />";
                echo ' ' . \Ron\ronText('Litru_100'). "</td>";
            }
        }
        echo "</tr>\n";

        $value = isset($radek ->NVVPocetUjetychKm)
            ? $radek ->NVVPocetUjetychKm
            : '';

        echo
            "<tr>" .
            "<td class='align-right'>" . \Ron\ronText('Pocet_km') . "</td>" .
            "<td>" .
            "<input type='text' name='NVV_PocetUjetychKM_$index' class='appinput inputInt' value='". StringUtils::htmlTextConvert($value) . "' size='8' maxlength='8' />";

        echo
            " <label>" .
            \Ron\ronText('Nepocitat_zakladni_sazbu') .
            " <input name='NVV_NepocitatZaklSazbu_$index' type='checkbox'";
        if ($radek ->NVVNepocitatZakladniSazbu) {
            echo ' checked="checked"';
        }
        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_NEPOCITAT_ZS_NEMENIT)) {
            echo ' disabled="disabled"';
        }
        echo '></label>';

//        VYPNUTO - JIZ NENI DOSTUPNE ZDARMA
//        if (RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_VYPOCET_VZDALENOSTI) && $mista = $this ->sluzebniCesta ->GetMistaProVzdalenost()) {
//            // vypocet vzdalenosti:
//            echo '<script src="https://maps.googleapis.com/maps/api/js?sensor=false"></script>';
//            echo '<span style="display:inline-block; width:50px;"> </span>';
//            $vzdalenostscript = "GetDistance(this, '" . StringUtils::htmlTextConvert($mista[0]) . "', '" . StringUtils::htmlTextConvert($mista[1]) . "', '" . StringUtils::htmlTextConvert($mista[2]) . "', $index)";
//            echo '<INPUT type="button" class="button" name="vypocetvzdalenosti" value="' . \Ron\ronText('Vzdalenost') . '" onclick="' . $vzdalenostscript . '">';
//
//            echo "<INPUT type='checkbox' name='NVV_DvojnasobekVzdalenosti_$index'>" . \Ron\ronText('Zpatecni_cesta');
//            if ($mista[2]) echo '<INPUT type="checkbox" checked="checked" name="NVV_PocitatPres">' . \Ron\ronText('Pres');
//        }


        echo "</td></tr>\n";

        // Neprokaze cenu PHM => vyber druhu PHM
        echo '<tr><td>';
        echo "<label><input class='NVV_ProkazaniCenyPHM' data-index='$index' name='NVV_NeprokazanaCenaPHM_$index' type='radio' value='1'";
        if ($radek ->NVVNeprokazanaCenaPHM) {
            echo ' checked';
        }
        echo '/>';
        echo \Ron\ronText('Neprokaze_cenu_PHM') . "</label></td>";

        echo '<td>';

        $params = ['id' => "NVV_DruhPHM_$index"];

        if (!$radek ->NVVNeprokazanaCenaPHM) {
            $params['style'] = "display:none";
        }

        if ($radek ->NVVVozidlo) {
            // TODO: jeste by se melo zkontrolovat, jestli ma vozidlo zadany druh PHM
            $params['disabled'] = "disabled";
        }

        $combo = new ComboBox('NVV_DruhPHM_'.$index);
        $combo ->emptyRow = ' ';
        $combo ->translationPrefix = 'db_druhphm_popis_';
        $combo ->translationValueIndex = 0;
        $combo ->loadData($this ->database, 'SELECT Kod, Popis FROM DruhPHM ORDER BY Popis', $radek ->NVVDruhPHM);
        $combo ->htmlParams = $params;
        $combo ->render();
        echo "</td></tr>\n";


        // Prokaze cenu PHM => zadani ceny
        echo '<tr>';
        echo "<td><label><input class='NVV_ProkazaniCenyPHM' data-index='$index' name='NVV_NeprokazanaCenaPHM_$index' type='radio' value='0'";
        if (!$radek ->NVVNeprokazanaCenaPHM) {
            echo ' checked';
        }
        echo '/>';
        echo \Ron\ronText('Prokaze_cenu_PHM') . "</label></td>";

        if ($radek ->NVVNeprokazanaCenaPHM) $stylestr = 'style="display:none"';
        else $stylestr = '';

        echo "<td><input type='text' name='NVV_CenaPHM_$index' class='appinput inputFloat' value='" . StringUtils::format('number_2', $radek ->NVVCenaPHM) . "' $stylestr size='8' maxlength='8' />";
        echo ' ' . \Ron\ronText('Kc_litr') . '</td>';
        echo "</tr>\n";

        echo '</table>';
    }


    /**
     */
    public function render()
    {
        echo '<h1>' . \Ron\ronText('Radky_uctovani_sc') . " <span id='cisloSC'>{$this ->sluzebniCesta ->id}</span> - " . date('j.n.Y', $this ->datum) . "</h1>";

        $this ->sluzebniCesta -> zobrazZkracenouHlavicku();

        require('ronportal/sluzebnicesty/view_uctovaniformular.php');
    }
}
