<?php

namespace RonPortal\SluzebniCesty;


use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\DB\SqlStatement;


class RegistracniZnackaField extends FormField
{
    /** @var int    Id pracovniho pomeru, pro ktery se nacitaji vozidla. Pokud neni zadan, tak se pouzije $rowData['field-osoba'] */
    private $idPP = null;

    public function __construct($fieldType = self::FIELD_TYPE_EDIT, $idPP = null)
    {
        parent::__construct($fieldType);
        $this ->idPP = $idPP;
    }


    /**
     * @param Database $database
     * @param int $idPP
     * @return SqlStatement|string
     */
    public static function getSqlVozidlaOsoby($database, $idPP = null)
    {
        if ($idPP) {
            $params = $database->createParameters();
            return new SqlStatement(
                'SELECT Vozidlo.RegistracniZnacka, ' .
                ' Vozidlo.RegistracniZnacka ' . $database->concatenateOperator . " ' ' " . $database->concatenateOperator . ' Vozidlo.Popis, ' .
                ' Vozidlo.Soukrome, VozidloOsoby.RC' .
                ' FROM PracovniPomer JOIN Vozidlo ON 1 = 1 LEFT OUTER JOIN VozidloOsoby' . // v ON podmince nezvlada parametry, proto pripojuji tabulku PracovniPomer a podminka je ve where
                '   ON VozidloOsoby.RegistracniZnacka = Vozidlo.RegistracniZnacka' .
                '  AND VozidloOsoby.RC = PracovniPomer.RC' .
                ' WHERE PracovniPomer.Id = ' . $params->addParameter((int)$idPP) .
                ' AND (Vozidlo.Soukrome = 0  OR  VozidloOsoby.RC IS NOT NULL)' .
                ' AND (Vozidlo.Aktivni = 1)' .
                ' AND (VozidloOsoby.PlatnostOd <= ' . $params ->addDateParameter(time()) . ' OR VozidloOsoby.PlatnostOd IS NULL)' . // kvuli neprirazenym firemnim vozidlum
                ' AND (VozidloOsoby.PlatnostDo >= ' . $params ->addDateParameter(time()) . ' OR VozidloOsoby.PlatnostDo IS NULL)' .
                ' ORDER BY Vozidlo.RegistracniZnacka',
                $params
            );
        }

        return
            'SELECT Vozidlo.RegistracniZnacka, ' .
            ' Vozidlo.RegistracniZnacka ' . $database->concatenateOperator . " ' ' " . $database->concatenateOperator . ' Vozidlo.Popis ' .
            ' FROM Vozidlo WHERE Aktivni = 1  AND  Soukrome = 0';
    }

    protected function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        $this->setSelectSql(self::getSqlVozidlaOsoby($database, $this ->idPP ?? $rowData['field-osoba'] ?? null)); // pozor! ?? nefunguje
        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }
}
