<?php

namespace RonPortal\SluzebniCesty;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\Log;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalSettings;
use RonPortal\Rozdeleni;
use RonPortal\User;
use Exception;


class SluzebniCestyUtils
{
    const SETTINGS_KONTROLOVAT_PREKRYVANI = 'kontrolovatprekryvanisc';
    const SETTINGS_POVINNY_UCEL = 'povinnyucelslcesty';
    const SETTINGS_CASY_AUTOMATICKY = 'casyslcestyautomaticky';
    const SETTINGS_EDITACE_V_UZAVRENEM = 'editacesluzebnichvuzavrenem';
    const SETTINGS_EDITACE_SCHVALENE_NADRIZENYM = 'editacesc_schvalnad';    // jestli je mozne editovat SC schvalenou nadrizenym

    /**
     * Zjisti, jestli ma uzivatel prislusne pravo (typ $kodTypuPrava) na sluzebni cestu zadaneho zamestnance ($idPP) v zadanem terminu
     * @param string $kodTypuPrava  Kod typu prava, ktere se ma zkontrolovat
     * @param int   $idPP       Id pracovniho pomeru, kteremu patri SC
     * @param bool  $zahranicni Urcuje, jestli je cesta zahranicni
     * @param int   $datumOd    Pocatek SC
     * @param int   $datumDo    Konec SC
     * @return bool
     * @throws \Exception
     */
    static function jePravoNaSluzebniCestu($kodTypuPrava, $idPP, $zahranicni, $datumOd, $datumDo)
    {
        $prava = Prava::vratPrava($kodTypuPrava, $zahranicni ? $kodTypuPrava . '_Z' : null); // pro zahranicni SC je mozne nadefinovat uzivatelske pravo s priponou _Z
        return $prava ->lzeZobrazitPP($idPP, $datumOd, $datumDo);
    }


    /**
     * Vraci true, pokud muze uzivatel zobrazit sluzebni cestu zadaneho zamestnance ($idPP)
     * @param int   $idPP       Id pracovniho pomeru, kteremu patri SC
     * @param bool  $zahranicni Urcuje, jestli je cesta zahranicni
     * @param int   $datumOd    Pocatek SC
     * @param int   $datumDo    Konec SC
     * @return bool
     * @throws \Exception
     */
    static function jePravoNaSlCestuZamestnance($idPP, $zahranicni, $datumOd, $datumDo)
    {
        Prava::nastavObdobiProPrava($datumOd, $datumDo);

        foreach ([
                     'sluzebnicesty', 'slcesty_prohl', 'slcesty_ucet', 'slcesty_nadriz',
                     'slcesty_schval1', 'slcesty_schval2', 'slcesty_schval3', 'slcesty_schval4',
                     'slcesty_zauct'] as $kodPrava) {
            if (self::jePravoNaSluzebniCestu($kodPrava, $idPP, $zahranicni, $datumOd, $datumDo)) {
                return true;
            }
        }

        return false;
    }


//    /**
//     * Vraci false, pokud jiz neni mozne editovat hlavicku (z duvodu schvaleni nadrizenym)
//     * @param int   $idPP
//     * @param bool  $zahranicni
//     * @param int   $datumOd
//     * @param int   $datumDo
//     * @param bool  $schvalenoNadrizenym
//     * @return bool
//     * @throws Exception
//     */
//    public static function lzeEditovatHlavicku($idPP, $zahranicni, $datumOd, $datumDo, $schvalenoNadrizenym)
//    {
//        if (!$schvalenoNadrizenym) {
//            // jeste neni schvalena nadrizenym
//            return true;
//        }
//
//        if (RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_SCHVALENE_NADRIZENYM, 1)) {
//            // editace sluzebni cesty schvalene nadrizenym je povolena
//            return true;
//        }
//
//        // sc schvalena nadrizenym - muzu editovat jen kdyz jsem nadrizeny
//        return self::jePravoNaSluzebniCestu('slcesty_nadriz', $idPP, $zahranicni, $datumOd, $datumDo);
//    }


//    /**
//     * Vraci true, pokud je sluzebni cesta schvalena nejvyssi urovni, na kterou ma uzivatel pravo, nebo vyssi urovni
//     * @param int   $idPP
//     * @param bool  $zahranicni
//     * @param int   $datumOd
//     * @param int   $datumDo
//     * @param array $schvaleni  Pole hodnot schvaleni SC (jednotlive urovne indexovane od 1)
//     * @return bool
//     * @throws Exception
//     */
//    public static function sluzebniCestsaJeSchvalenaProUzivatele($idPP, $zahranicni, $datumOd, $datumDo, array $schvaleni)
//    {
//        if ( !RonPortalSettings::getIntValue(self::SETTINGS_EDITACE_SCHVALENYCH, 1) ) {
//            // nelze editovat schvalene - jakakoliv uroven zamyka editaci
//            return $schvaleni[1] || $schvaleni[2] || $schvaleni[3] || $schvaleni[4];
//        }
//
//        if ($schvaleni[4]) {
//            // nejvyssi uroven zamyka vzdy
//            return true;
//        }
//
//        if (self::jePravoNaSluzebniCestu('slcesty_schval4', $idPP, $zahranicni, $datumOd, $datumDo)) {
//            // ma pravo na uroven 4
//            return false;
//        }
//
//        if ($schvaleni[3]) {
//            return true;
//        }
//
//        //if ($this ->f_PP ->lzeZobrazitPP(Prava::vratPrava('slcesty_schval3'), $this ->datumCasOd, $this ->datumCasDo)) {
//        if (self::jePravoNaSluzebniCestu('slcesty_schval3', $idPP, $zahranicni, $datumOd, $datumDo)) {
//            // ma pravo na uroven 3
//            return false;
//        }
//
//        if ($schvaleni[2]) {
//            return true;
//        }
//
//        //if ($this ->f_PP ->lzeZobrazitPP(Prava::vratPrava('slcesty_schval2'), $this ->datumCasOd, $this ->datumCasDo)) {
//        if (self::jePravoNaSluzebniCestu('slcesty_schval2', $idPP, $zahranicni, $datumOd, $datumDo)) {
//            // ma pravo na uroven 2
//            return false;
//        }
//
//        return $schvaleni[1];
//    }


//    /**
//     * Overi, jestli muze uzivatel zadanou sluzebni cestu editovat.
//     * @param Database $database
//     * @param int   $idPP
//     * @param bool  $zahranicni
//     * @param int   $datumCasOd
//     * @param int   $datumCasDo
//     * @param bool  $zauctovano
//     * @param array $schvaleni
//     * @param bool  $raise Pokud je true, tak v pripade nemoznosti editovat vyhodi vyjimku, jinak vraci false
//     * @return bool
//     * @throws Exception
//     */
//    public static function lzeEditovatSC(Database $database, $idPP, $zahranicni, $datumCasOd, $datumCasDo, $zauctovano, array $schvaleni, $raise)
//    {
//        // kontrola zauctovani - nelze editovat zauctovane
//        if ($zauctovano) {
//            if ($raise) {
//                throw new Exception( TextManager::_('Sluzebka_je_jiz_zauctovana') );
//            }
//
//            return false;
//        }
//
//        // kontrola schvaleni (nelze editovat schvalene, ale pokud mam pravo na vyssi uroven, tak muzu)
//        $schvalena = self::sluzebniCestsaJeSchvalenaProUzivatele($idPP, $zahranicni, $datumCasOd, $datumCasDo, $schvaleni);
//
//        if ($schvalena) {
//            if ($raise) {
//                throw new Exception( TextManager::_('Sluzebka_je_jiz_schvalena') );
//            }
//
//            return false;
//        }
//
//        // kontrola uzavreneho obdobi
//        if (!RonPortalSettings::getIntValue(SluzebniCestyUtils::SETTINGS_EDITACE_V_UZAVRENEM) && Obdobi::jeObdobiUzavreno($database, 'ADS', $datumCasOd, $datumCasDo)) {
//            if ($raise) {
//                throw new Exception( TextManager::_('Obdobi_uzavreno') );
//            }
//
//            return false;
//        }
//
//        // kontrola prav
//        if (!self::jePravoNaSluzebniCestu('sluzebnicesty', $idPP, $zahranicni, $datumCasOd, $datumCasDo)) {
//            if ($raise) {
//                throw new Exception( TextManager::_('Neni_pravo_editovat_sc') );
//            }
//
//            return false;
//        }
//
//        return true;
//    }


    /**
     * Vraci seznam typu uctovani SC, ktere jsou NVV (kody oddelene strednikem)
     * @param Database $database
     * @return string
     */
    public static function typyUctovaniNVV($database)
    {
        $sql =
            'SELECT Kod, DruhPohybu, NahradaZaVlastniVozidlo FROM TypRUctovaniSlCesty' .
            ' WHERE Aktivni = 1 AND NahradaZaVlastniVozidlo = 1';

        $cursor = $database ->select($sql);
        $result = '';

        while ($row = $cursor ->fetch()) {
            $result .= ($result ? ';' : '') . $row[0];
        }

        $cursor ->close();

        return $result;
    }


    /**
     * @param Database $database
     * @param int $idSluzebniCesty
     * @return null|string
     */
    public static function vratVychoziMenu($database, $idSluzebniCesty)
    {
        $params = $database ->createParameters();

        // podle ciloveho statu
        $sql =
            'SELECT Stat.MenaStatu ' .
            ' FROM SluzebniCesta, Stat ' .
            ' WHERE SluzebniCesta.ID = ' . $params ->addParameter((int)$idSluzebniCesty) .
            '   AND Stat.Kod = SluzebniCesta.CilovyStat ';

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) {
            return $row[0];
        }

        // podle cile
        $params = $database ->createParameters();
        $sql =
            'SELECT Stat.MenaStatu ' .
            ' FROM SluzebniCesta, MistoSluzebniCesty, Stat ' .
            ' WHERE SluzebniCesta.ID = ' . $params ->addParameter((int)$idSluzebniCesty) .
            '   AND MistoSluzebniCesty.Kod = SluzebniCesta.Cil ' .
            '   AND Stat.Kod = MistoSluzebniCesty.Stat ';

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();
        if ($row) {
            return $row[0];
        }

        // prednastavena vychozi mena
        if ( RonPortalSettings::getValue(RonPortalSettings::VYCHOZI_MENA) ) {
            return RonPortalSettings::getValue(RonPortalSettings::VYCHOZI_MENA);
        }

        return '';
    }


    /**
     * Zjisti maximalni denni sazbu stravneho pro dany stat
     * @param Database $database
     * @param int    $datum
     * @param string $cilovyStat
     * @param string $vychoziStat
     * @return array
     * @throws \Exception
     */
    static function vratMaximalniDenniStravneProStat($database, $datum, $cilovyStat, $vychoziStat)
    {
        // kontrola doplnena kvuli firebirdu - tam to hazelo chybu "Incorrect values within SQLDA structure empty pointer to data at SQLVAR index 1"
        if (!$cilovyStat) {
            throw new \Exception('Chyba pri nacitani maximalniho stravneho - neni zadan cilovy stat!');
        }

        if (!$vychoziStat) {
            throw new \Exception('Chyba pri nacitani maximalniho stravneho - neni zadan vychozi stat!');
        }

        if (!$datum) {
            throw new \Exception('Chyba pri nacitani maximalniho stravneho - neni zadan vychozi datum!');
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT Stat.MenaStatu, Sazba1, Sazba2, Sazba3, Sazba4, Sazba5 ' .
            '  FROM NastaveniVypoctuCN, HistorieNVypoctuCN, Stat ' .
            ' WHERE HistorieNVypoctuCN.Stat = NastaveniVypoctuCN.Stat ' .
            '   AND HistorieNVypoctuCN.PlatnostOd = NastaveniVypoctuCN.PlatnostOd ' .
            '   AND Stat.Kod = NastaveniVypoctuCN.CilovyStat ' .
            '   AND NastaveniVypoctuCN.Stat = ' . $params ->addParameter($vychoziStat) .
            '   AND NastaveniVypoctuCN.CilovyStat = ' . $params ->addParameter($cilovyStat) .
            '   AND NastaveniVypoctuCN.PlatnostOd <= ' . $params ->addParameter($database ->dateToParam($datum)) .
            '   AND (HistorieNVypoctuCN.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam($datum)) . '  OR HistorieNVypoctuCN.PlatnostDo IS NULL) ';

        $cursor = $database ->getSqlResult($sql, $params, 'Pri zjistovani sazby doslo k chybe!');
        $row = $cursor ->fetch();
        $cursor ->close();

        if ( !$row ) {
            throw new \Exception(TextManager::_('Nebyla_nalezena_sazba_stravneho'));
        }

        $sazba = (float)$row[1];
        if ((float)$row[2] > $sazba) {
            $sazba = (float)$row[2];
        }
        if ((float)$row[3] > $sazba) {
            $sazba = (float)$row[3];
        }
        if ((float)$row[4] > $sazba) {
            $sazba = (float)$row[4];
        }
        if ((float)$row[5] > $sazba) {
            $sazba = (float)$row[5];
        }

        return [
            'sazba' => $sazba,
            'mena' => $row[0]
        ];
    }


    /**
     * Z tabulky Stat zjisti sazbu kapesneho pro zadany stat.
     * @param Database $database
     * @param string $stat
     * @return float|null
     * @throws DBException
     */
    public static function sazbaKapesnehoProStat(Database $database, $stat)
    {
        if (!$stat) {
            return null;
        }

        $params = $database ->createParameters();
        $sql = 'SELECT ImplicitniSazbaKapesneho FROM Stat WHERE Kod = ' . $params ->addParameter($stat);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        return $row ? $row[0] : null;
    }

    /**
     * Podle nastaveni portalu a pripadne zadane osoby zjisti vychozi stat
     * @param Database $database
     * @param int|PracovniPomer $pp
     * @param int      $kDatu
     * @return string|null
     */
    public static function getVychoziStatOsoby($database, $pp, $kDatu)
    {
        $idPP = ($pp instanceof PracovniPomer) ? $pp ->idPP : $pp;

        if ($typRozdeleni = RonPortalSettings::getIntValue(SluzebniCesta::SETTINGS_TYP_ROZDELENI_PRO_STAT)) {
            $rozdeleni = Rozdeleni::loadRozdeleniOsoby($database, $idPP, $typRozdeleni, $kDatu);
            if (count($rozdeleni) && $rozdeleni[0] ->stat) {
                return $rozdeleni[0] ->stat;
            }
        }

        return RonPortalSettings::getValue(SluzebniCesta::SETTINGS_STAT) ?: null;
    }


    /**
     * @param Database $database
     * @param string $ucel
     * @return bool
     * @throws DBException
     */
    public static function jePovinnyCilProUcel($database, $ucel)
    {
        if (!$ucel) {
            return false;
        }

        $params = $database ->createParameters();
        return DBUtils::getSqlValue($database,
            'SELECT PovinnyCil FROM UcelSluzebniCesty WHERE Kod = ' . $params ->addParameter($ucel),
            $params);
    }


    public static function lzeSchvalitZamestnancem($rc, $idPP, $zahranicni, $datumOd, $datumDo)
    {
        Prava::nastavObdobiProPrava($datumOd, $datumDo);
        $pravaSchvaleniZam = Prava::vratPrava('slcesty_zam', $zahranicni ? 'slcesty_zam_Z' : null);

        return $pravaSchvaleniZam ->priznak
            ? $pravaSchvaleniZam ->lzeZobrazitPP($idPP, $datumOd, $datumDo)
            : $rc == User::getRC();
    }

    /**
     * Zkontroluje, jestli se sluzebni cesta neprekryva s jinou SC.
     * @param Database $database
     * @param string $rc    RC osoby, ktere SC patri
     * @param int $casOd    Pocatecni cas SC
     * @param int $casDo    Koncovy cas SC
     * @param int|null $idSC    ID SC, ktera se edituje (pri editaci musi byt zadano, pri vkladani je potreba ponechat null)
     * @param int|null $zahrnoutCas     Cas, ktery se ma do kontroly zahrnout (tj. pokud je potreba, tak se upravi $casOd nebo $casDo tak aby interval zahrnoval i zadany cas)
     * @return void
     * @throws \Exception
     */
    public static function kontrolaPrekryvaniSC(Database $database, string $rc, int $casOd, int $casDo, ?int $idSC = null, ?int $zahrnoutCas = null)
    {
        if (!RonPortalSettings::getIntValue(self::SETTINGS_KONTROLOVAT_PREKRYVANI)) {
            return;
        }

        $params = $database ->createParameters();
        $sql =
            'SELECT DatumCasOd, DatumCasDo' .
            ' FROM SluzebniCesta' .
            ' WHERE RC = ' . $params ->addParameter($rc) .
            ($idSC
                ? ' AND Id <> ' . $params ->addParameter((int)$idSC)
                : '');

        if ($zahrnoutCas) {
            if ($zahrnoutCas < $casOd) {
                $casOd = $zahrnoutCas;
            }

            if ($zahrnoutCas > $casDo) {
                $casDo = $zahrnoutCas;
            }
        }

        if (RonPortalSettings::getIntValue('casyslcestyautomaticky')  &&  $casOd == $casDo) {
            // casy se nastavuji automaticky a sluzebni cesta jeste nema nastaven cas
            // => nekontroluje se prekryvani s SC, ktere zacinaji nebo konci ve stejny den
            //    protoze po nastaveni casu se prekryvat nemusi
            $sql .=
                ' AND DatumCasOd < ' . $params->addParameter($database ->dateToParam($casDo)) .
                ' AND DatumCasDo >= ' . $params->addParameter($database ->dateToParam(DateUtils::incDay($casOd, 1)));

        } else {
            $sql .=
                ' AND DatumCasOd < ' . $params->addParameter($database ->timestampToParam($casDo)) .
                ' AND DatumCasDo > ' . $params->addParameter($database ->timestampToParam($casOd));
        }

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if ($row) {
            throw new \Exception( TextManager::_('Chyba_prekryvani_SC') );
        }
    }


    /**
     * @param Database $database
     * @param int $idOriginal
     * @param int $idNew
     * @param int $dateAdd
     * @param object|string $logSrcDescription
     * @throws DBException
     */
    public static function kopirujRadky($database, $idOriginal, $idNew, $dateAdd, $logSrcDescription)
    {
        MessageManager::addDebugMessage('Kopirovani radku SC ... ');

        $params = $database ->createParameters();
        $sql =
            'SELECT CisloRadku, Popis, DatumCasOd, KodOperace, Stat, PocetPJidel_S, PocetPJidel_O, PocetPJidel_V, Misto, Stav, DopravniProstredek, Poznamka' .
            ' FROM RadekSluzebniCesty ' .
            ' WHERE SluzebniCesta_ID = ' . $params ->addParameter($idOriginal);

        try {
            $cursor = $database->select($sql, $params);
        } catch (DBException $e) {
            $errorText = 'Chyba pri kopirovani - nelze nacist radky puvodni sluzebni cesty';
            Log::addRecordToDBLog($database, 'RadekSluzebniCesty', 'ERROR', 'Sluzebni cesty', "$errorText\n{$e ->getMessage()}\n$sql");
            throw $e;
        }

        $logMessage = $sql . "\n";

        while ($row = $cursor ->fetch()) {
            $insertParams = $database ->createParameters();
            $time = $dateAdd + $database ->stringToTimestamp($row[2]);
            $sql =
                'INSERT INTO RadekSluzebniCesty (' .
                'SluzebniCesta_ID, CisloRadku, Popis, DatumCasOd, KodOperace, Stat, PocetPJidel_S, PocetPJidel_O, PocetPJidel_V, Misto, Stav, DopravniProstredek, Poznamka' .
                ') VALUES (' .
                $insertParams ->addParameter($idNew) . ', ' . $insertParams ->addParameter($row[0]) . ', ' . // id, cisloRadku
                $insertParams ->addParameter($row[1]) . ', ' . // popis
                $insertParams ->addParameter($database ->timestampToParam($time)) . ', ' . // datumCasOd
                $insertParams ->addParameter($row[3]) . ', ' . // kodOperace
                $insertParams ->addParameter($row[4]) . ', ' . // stat
                $insertParams ->addParameter((int)$row[5]) . ', ' . $insertParams ->addParameter((int)$row[6]) . ', ' . $insertParams ->addParameter((int)$row[7]) . ', ' . // S,O,V
                $insertParams ->addParameter($row[8]) . ', ' . // misto
                $insertParams ->addParameter((int)$row[9]) . ', ' . // stav
                $insertParams ->addParameter($row[10]) . ', ' . // DopravniProstredek
                $insertParams ->addParameter($row[11]) . // Poznamka
                ')';

            $logMessage .= $sql."\n";
            $logMessage .=
                $idNew . ', ' . $row[0] . ', ' . ($row[1] ?? 'NULL') . ', ' . date('j.n.Y G:i', $time) . ', ' .
                $row[3] . ', ' . ($row[4] ?? 'NULL') . ", ...\n";

//            try {
                $database->sql($sql, $insertParams);
//            } catch (DBException $e) {
//                $errorText = 'Chyba pri kopirovani - pri vkladani zaznamu doslo k chybe!';
//
//                Log::addRecordToDBLog($database, 'RadekSluzebniCesty', 'ERROR', 'SluzebniCestyUtils', "$logMessage$errorText\n{$e ->getMessage()}");
//
//                throw $e;
//            }
        }
        $cursor ->close();

        Log::addRecordToDBLog($database, 'RadekSluzebniCesty', 'INSERT', $logSrcDescription, "Kopie sluzebni cesty\n" . $logMessage);
    }


    /**
     * @param Database $database
     * @param int $idOriginal
     * @param int $idNew
     * @param int $dateAdd
     * @param object|string $logSrcDescription
     * @throws DBException
     */
    public static function kopirujRadkyUctovani($database, $idOriginal, $idNew, $dateAdd, $logSrcDescription)
    {
        MessageManager::addDebugMessage('Kopirovani uctovani SC ... ');

        $params = $database ->createParameters();
        $sql =
            'SELECT CisloRadku, Typ, Castka, KodMeny, ZpusobPlatby, Den, Popis, NVV_SpotrebaVozidla,' .
            '    NVV_PocetUjetychKM, NVV_NepocitatZaklSazbu, NVV_NeprokazanaCenaPHM, NVV_CenaPHM, NVV_DruhPHM ' .
            '  FROM RadekUctovaniSCesty ' .
            ' WHERE SluzebniCesta_ID = ' . $params ->addParameter((int)$idOriginal) .
            ' AND Typ <> 1  AND Typ <> 2  AND  Typ <> 3 '; // vynecha nahrady a soucty

        $cursor = $database ->select($sql, $params);

        $logMessage = $sql . "\n";

        while ($row = $cursor ->fetch()) {
            $time = $row[5] ? $dateAdd + $database ->stringToTimestamp($row[5]) : null;

            $insertParams = $database ->createParameters();
            $sql =
                'INSERT INTO RadekUctovaniSCesty (' .
                ' SluzebniCesta_ID, CisloRadku, Typ, Castka, KodMeny, ZpusobPlatby, Den, Popis, NVV_SpotrebaVozidla,' .
                '    NVV_PocetUjetychKM, NVV_NepocitatZaklSazbu, NVV_NeprokazanaCenaPHM, NVV_CenaPHM, NVV_DruhPHM ' .
                ') VALUES (' .
                $insertParams ->addParameter($idNew) . ', ' . $insertParams ->addParameter($row[0]) . ', ' .
                $insertParams ->addParameter($row[1]) . ', ' . $insertParams ->addParameter($row[2]) . ', ' .
                $insertParams ->addParameter($row[3]) . ', '. $insertParams ->addParameter($row[4]) . ', ' . // kodMeny, zpusobPlatby
                // den - opravit (nepouzivat cas!)
                $insertParams ->addParameter( $time ? $database ->timestampToParam($time) : null) . ', ' .
                $insertParams ->addParameter($row[6]) . ', ' .
                // NVV
                $insertParams ->addParameter($row[7]) . ', ' . // spotreba
                $insertParams ->addParameter($row[8]) . ', ' . // pocet km
                $insertParams ->addParameter($row[9]) . ', ' . $insertParams ->addParameter($row[10]) . ', ' . // nepocitat z.sazbu, neprokazana cena
                $insertParams ->addParameter($row[11]) . ', ' . // cena PHM
                $insertParams ->addParameter($row[12]) .
                ')';

            $logMessage .= $sql."\n";
            $logMessage .=
                $idNew . ', ' . $row[0] . ', ' . $row[1] . ', ' . $row[2] . ', ' . $row[3] . ', ' . $row[4] . ', ' .
                date('j.n.Y G:i', $time) . ', ' . ($row[6] ?: 'NULL') . ", ...\n";

            try {
                $database ->sql($sql, $insertParams);
            } catch (DBException $e) {
                $errortext = 'Chyba pri kopirovani - pri vkladani zaznamu doslo k chybe!';
                Log::addRecordToDBLog($database, 'RadekUctovaniSCesty', 'ERROR', 'Sluzebni cesty', "$logMessage$errortext\n{$e ->getMessage()}");

                throw $e;
            }
        }
        $cursor ->close();

        Log::addRecordToDBLog($database, 'RadekUctovaniSCesty', 'INSERT', $logSrcDescription, "Kopie sluzebni cesty\n" . $logMessage);
    }


    /**
     * Vraci cas prvniho a posledniho radku zadane sluzebni cesty.
     * @param Database $database
     * @param int $idSC
     * @return array|null
     * @throws DBException
     */
    public static function vratCasyPodleRadkuSC(Database $database, int $idSC): ?array
    {
        $params = $database ->createParameters();
        $sql =
            'SELECT MIN(RadekSluzebniCesty.DatumCasOd), MAX(RadekSluzebniCesty.DatumCasOd)' .
            ' FROM RadekSluzebniCesty'.
            ' WHERE RadekSluzebniCesty.SluzebniCesta_ID = ' . $params ->addParameter($idSC);

        $cursor = $database ->getSqlResult($sql, $params);
        $row = $cursor ->fetch();
        $cursor ->close();

        if (!$row  ||  !$row[0]) {
            // neni zadny radek SC
            return null;
        }

        return [
            'DatumCasOd' => $database ->stringToTimestamp($row[0]),
            'DatumCasDo' => $database ->stringToTimestamp($row[1])
        ];
    }

//    /**
//     * Pokud je zapnuto "Nastavit casy automaticky podle radku", tak vraci cas pocatku a konce sluzebni cesty
//     * upraveny podle prvniho a posledniho radku.
//     * @param Database $database
//     * @throws DBException
//     */
//    public function vratCasyPodleRadkuSC(Database $database, int $idSC)
//    {
//        if ( !RonPortalSettings::getIntValue(self::SETTINGS_CASY_AUTOMATICKY) ) {
//            return null;
//        }
//
//        $params = $database ->createParameters();
//        $sql =
//            'SELECT MIN(RadekSluzebniCesty.DatumCasOd), MAX(RadekSluzebniCesty.DatumCasOd), SluzebniCesta.DatumCasOd, SluzebniCesta.DatumCasDo' .
//            ' FROM RadekSluzebniCesty, SluzebniCesta '.
//            ' WHERE SluzebniCesta.Id = ' . $params ->addParameter($idSC) .
//            ' AND RadekSluzebniCesty.SluzebniCesta_ID = SluzebniCesta.Id';
//
//        $cursor = $database ->getSqlResult($sql, $params);
//        $row = $cursor ->fetch();
//        $cursor ->close();
//
//        if (!$row  ||  !$row[0]) {
//            // neni zadny radek SC
//            return null;
//        }
//
//        // aktualni casy pocatku a konce v SC
//        $casOd = $database ->stringToTimestamp($row[2]);
//        $casDo = $database ->stringToTimestamp($row[3]);
//
//        $casOdR = $database ->stringToTimestamp($row[0]);   // cas podle prvniho radku
//        if (DateUtils::startOfTheDay($casOdR) === DateUtils::startOfTheDay($casOd)) {
//            // prvni radek ma stejny datum jako SC => pouzije cas prvniho radku
//            $casOd = $casOdR;
//        }
//
//        $casDoR = $database ->stringToTimestamp($row[1]);
//        if (DateUtils::startOfTheDay($casDoR) === DateUtils::startOfTheDay($casDo)) {
//            // posledni radek ma stejny datum jako konec SC => pouzije cas posledniho radku
//            $casDo = $casDoR;
//        }
//
//        return ['DatumCasOd' => $casOd, 'DatumCasDo' => $casDo];
//    }
//
}
