<?php

/**
 * Seznam vozidel
 */

namespace RonPortal\SluzebniCesty;

use Ron\Controls\BrowserField;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;

class VozidloBrowser extends RonPortalBrowser
{
    /**
     * konstruktor ve kterem vytvarime pozadovane objekty
     * @param $database
     * @param $action
     */
    public function __construct($database, $action)
    {
        parent::__construct($database, $action);

        $this->title = TextManager::_('Seznam_vozidel');

        $this->sqlFromPart = 'Vozidlo';                         // tabulka pro zobrazeni

        $this ->sqlSelectPart = 'RegistracniZnacka';
        $this ->primaryField = 0;

        $this ->code = 'browservozidla';

        // musi byt zadane polozky nebo error
        $this->addField()->setSqlFieldName('RegistracniZnacka', 0)
            ->setHeaderText('Registracni_znacka')
            ->setLink("?menuid=" . $action . "&formaction=edit")
            ->setOrdering(true)
            ->setCode('col-registracniznacka');

        $this ->defaultSort = 'col-registracniznacka';

        $this->addField() ->SetSqlFieldName('Popis') ->setOrdering(true)
            ->setHeaderText('Popis')
            ->setCode('col-popis');

        $this->addField()->setSqlFieldName('Aktivni') ->setFormat('anone') ->setOrdering(true)
            ->setHeaderText('Vozidlo_aktivni')
            ->setCode('col-aktivni')
            ->setAlign('center');

        $this->addField()->setSqlFieldName('DruhPHM') ->setOrdering(true)
            ->setHeaderText('Druh_PHM')
            ->setCode('col-druhphm');

        $this->addField()->setSqlFieldName('Spotreba') ->setFormat('number2') ->setOrdering(true)
            ->setHeaderText('Spotreba_TP') ->setAlign('right')
            ->setCode('col-spotreba');

        $this->addField()->setSqlFieldName('Soukrome') ->setFormat('anone') ->setOrdering(true)
            ->setHeaderText('Soukrome_vozidlo')
            ->setCode('col-soukrome');

        $this->addField()->setSqlFieldName('Jednostope') ->setFormat('anone') ->setOrdering(true)
            ->setHeaderText('Jednostope')
            ->setCode('col-jednostope')
            ->setAlign('center');

        $this->addField()->setSqlFieldName('PovolenoSC') ->setFormat('anone') ->setOrdering(true)
            ->setHeaderText('Povoleno_pro_SC')
            ->setCode('col-povolenosc')
            ->setAlign('center');

        $this->addField()->setSqlFieldName('PovolenoSPZ') ->setFormat('anone') ->setOrdering(true)
            ->setHeaderText('Povoleno_SPZ')
            ->setCode('col-povolenospz')
            ->setAlign('center');

        for ($i = 1;  $i <= 5;  $i++) {
            $this->addField()
                ->setSqlFieldName("Uzivatelska$i") ->setOrdering(true)
                ->setHeaderText("Vozidlo_uzivatelska_$i")
                ->setCode("col-uzivatelska$i")
                ->setVisible(false);
        }

//        $this->addField(new CDeleteBrowserField());
        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText('<i class="fa inline-glyph glyph-delete"></i>')
            ->setLink('?menuid=vozidla&action=delete&csrfkey=%CSRF%&record=')
            ->setCsrfKeyIndex('col-registracniznacka', 'vozidlo_delete_')
            ->setLinkAttributes([
                'title' => TextManager::_('Zrusit_zaznam'),
                'class' => 'confirmQuestion',
                'data-ron-confirm-id' => 'Opravdu_zrusit_zaznam'
            ]);

        $this->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('Poznamka')
            ->setCode('col-poznamka')
            ->setClassName('poznamka')
            ->setHeaderColSpan(-1);

    }


    function setFilterForm()
    {
        $this ->filterForm = new RonPortalFilterForm($this ->database, $this ->action, true);

        $pravo = Prava::vratPrava("vozidla_prohl");
        $this ->filterForm ->addFilterRozdeleni1(null, $pravo);
        $this ->filterForm ->addFilterRozdeleni2(null, $pravo);

//        $this ->filterForm ->nastavOsoby(true, $pravo, 0, 0, true); // vcetne externich
//        $this ->filterForm ->povolitVsechyOsoby = true;
        $filterPP = $this ->filterForm ->addFilterPP(true, $pravo);
        $filterPP ->setVcetneExternich(true);

        if (Prava::vratPrava('vozidla') ->jsouNejakePrava()) {
            $this ->filterForm ->addFilter(
                (new FilterFormFilter('insert_link', FilterFormFilter::TYPE_TEXT))
                    ->setDefaultValue("<a href='?menuid={$this ->action}&formaction=new'>" . TextManager::_('Nove_vozidlo') . '</a>')
                    ->setPosition(FilterFormFilter::POSITION_BELOW_FORM)
                    ->setRowClass('margin-top-2')
                    ->setCaption(false)
            );
        }
    }


    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $pravo = Prava::vratPrava("vozidla_prohl");

        $filtrOsoba = null;
        $filtrPrava = null;
        $filtrRozdeleni = '';

        if ($this ->filterForm ->getFilterValue('osoba')) {
            // jen jedna vybrana osoba => staci filtr na RC
            if ($pravo ->lzeZobrazitRC($this ->filterForm ->getFilterValue('osoba', 'rc'), time(), time())) {
                $filtrOsoba = 'PracovniPomer.RC = ' . $params ->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc'));
            } else {
                // nemam pravo na vybranou osobu
                return '0 = 1';
            }
        } else {
            // vsechny osoby => prida filtry podle prav ...
            if ($pravo->prava !== Prava::PRAVA_VSE) {
                $filtrPrava = $pravo->sqlFiltr('PracovniPomer', time(), time(), $params);
            }

            // ... a podle rozdeleni
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                $sqlRozdeleni = $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'PracovniPomer', $params);
                if ($sqlRozdeleni) {
                    $filtrRozdeleni .=
                        ($filtrRozdeleni ? ' AND ' : '') .
                        $sqlRozdeleni;
                }
            }
        }

        if ($filtrOsoba  ||  $filtrPrava  ||  $filtrRozdeleni) {
            $result .=
                ($result ? ' AND ' : '') .
                'EXISTS' .
                ' (SELECT * FROM PracovniPomer, VozidloOsoby' .
                '  WHERE VozidloOsoby.RC = PracovniPomer.RC' .
                '    AND VozidloOsoby.RegistracniZnacka = Vozidlo.RegistracniZnacka' .
                ($filtrOsoba ? ' AND (' . $filtrOsoba . ')' : '') .
                ($filtrPrava ? ' AND (' . $filtrPrava . ')' : '') .
                ($filtrRozdeleni ? ' AND (' . $filtrRozdeleni . ')' : '') .
                '    AND VozidloOsoby.PlatnostOd <= ' . $params->addDateParameter(time()) .
                '    AND (VozidloOsoby.PlatnostDo >= ' . $params->addDateParameter(time()) . ' OR VozidloOsoby.PlatnostDo IS NULL)' .
                " )\n";
        }

        return $result
            ? new SqlStatement($result, $params)
            : '';
    }
}

