<?php
/**
 * stanislav.elbl, 23.02.2021
 */

namespace RonPortal\System;

use Ron\DB\DBException;
use Ron\Http\App;
use Ron\MessageManager;
use Ron\DB\Database;
use Ron\Profiler;
use Ron\StringUtils;
use Ron\TextManager;
use Ron\XmlLoader;
use RonPortal\Log;
use RonPortal\RonPortalSettings;
use RonPortal\User;

class AgentManagerResult
{
    public $resultCode = null;
    public $agentResult = null;
    public $finalMessage = null;
    public $errorMessage = null;
    public $logText = null;
    public $resultParams = null;

    public $abortAvailable = null;
    public $currStatusWindowCaption = null;
    public $endTime;
    public $isRunning;
    public $progressCurr;
    public $progressMax;
    public $progressMessage;
    public $startTime;
    public $statusType;

    public $handle;
    /** @var string     Id ulohy v taskManageru */
    public $taskId;
}


class AgentManager implements TaskExecutor
{
    /** @deprecated */
    var $resultCode = null;
    /** @deprecated */
    var $agentResult = null;
    /** @deprecated */
    var $finalMessage = null;
    /** @deprecated */
    var $errorMessage = null;
    /** @deprecated */
    var $logText = null;

    /**
     * Rozebere a zpracuje odpoved remote executoru
     * @param string $xmlText
     * @return AgentManagerResult
     * @throws \Exception
     */
    private function processRemoteResult($xmlText)
    {
        if (!$xmlText) {
            throw new \Exception('Chyba pri komunikaci s agentem Agent_RemoteExecutor! Prazdna odpoved!');
        }

        $resultArray = XmlLoader::loadXmlToArray($xmlText, [XmlLoader::PARAM_CONVERT_FROM_UTF8 => true], 'utf-8');
        if (!$resultArray) {
            throw new \Exception('Chyba pri komunikaci s agentem Agent_RemoteExecutor! Spatny format odpovedi!');
        }

        $result = new AgentManagerResult();

        if ( isset($resultArray['#children']) ) {
            foreach ($resultArray['#children'] as $child) {
                $element = $child['element'];
                switch (strtolower($element['#attr']['Name'])) {
                    case 'agentresult' :
                        $result->agentResult = (int)$element['#text'];
                        break;

                    case 'resultcode' :
                        $result->resultCode = (int)$element['#text'];
                        break;

                    case 'handle' :
                        $result ->handle = (int)$element['#text'];
                        break;

                    case 'finalmessage' :
                        $result->finalMessage = $element['#text'];
                        break;

                    case 'logtext' :
                        $result->logText = $element['#text'];
                        break;

                    case 'abortavailable' :
                        $result ->abortAvailable = strtolower($element['#text']) == 'true' || $element['#text'] == '1';
                        break;

                    case 'currstatuswindowcaption' :
                        $result ->currStatusWindowCaption = $element['#text'];
                        break;

                    case 'endtime' :
                        $result ->endTime = $element['#text'];
                        break;

                    case 'isrunning' :
                        $result ->isRunning = strtolower($element['#text']) == 'true' || $element['#text'] == '1';
                        break;

                    case 'progresscurr' :
                        $result ->progressCurr = (int)$element['#text'];
                        break;

                    case 'progressmax' :
                        $result ->progressMax = (int)$element['#text'];
                        break;

                    case 'progressmessage' :
                        $result ->progressMessage = $element['#text'];
                        break;

                    case 'starttime' :
                        $result ->startTime = $element['#text'];
                        break;

                    case 'statustype' :
                        $result ->statusType = (int)$element['#text'];
                        break;

                    case 'errormessage' :
                        $result->errorMessage = $element['#text'];
                        break;
                    case 'resultparams' :
                        $resultParams = null;
                        if (isset($element['#children']) ) {
                            $resultParams = [];
                            foreach ($element['#children'] as $childE) {
                                $elementChild = $childE['element'];
                                $resultValue = $elementChild['#text'];

                                switch (strtolower($childE['name'])) {
                                    case 'integer' :
                                        $resultValue = (int)$resultValue;
                                        break;

                                    case 'blob' :
                                        if ($elementChild['blobdata']) {
                                            $resultValue = '';
                                            foreach ($elementChild['blobdata'] as $blobDataElement) {
                                                $resultValue .= hex2bin($blobDataElement['#text']);
                                            }
                                        } else {
                                            $resultValue = hex2bin($resultValue);
                                        }
                                        break;
                                }

                                $resultParams[strtolower($elementChild['#attr']['Name'])] = $resultValue;
                            }
                        }
                        $result->resultParams = $resultParams;
                        break;
                }
            }
        }

        if (!isset($result ->resultCode) && !isset($result ->agentResult) && !isset($result ->finalMessage)) {
            throw new \Exception('Chyba - AgentManager: nepodarilo se zjistit vysledek spusteni agenta!');
        }

        if ($result ->resultCode) {
            throw new \Exception('Nelze spustit agenta. Kód chyby: ' . $result ->resultCode . ' - ' . $result->errorMessage);
        }

        return $result;
        //return !$this ->agentResult; // 0 = OK
    }


    /**
     * @param $host
     * @param $port
     * @param $xml
     * @return string
     * @throws \Exception
     */
    private function executeRemoteProcedure($host, $port, $xml)
    {
        $errno = 0;
        $errstr = '';
        $level = error_reporting(E_ERROR); // vypne warningy

        try {
            $socket = fsockopen($host, $port, $errno, $errstr);
            error_reporting($level);

            if ($errstr) {
                // chyba - nepriklad nedostupny remote executor
                // pokusi se prevest zpravu z UTF8. Na nekterych verzich je totiz v UTF8, na jinych neni
                $errstr2 = StringUtils::fromUTF8($errstr);
                if ($errstr2) {
                    $errstr = $errstr2;
                }
            }

            //  stream_set_timeout ( $socket, 5, 0 ); - 5s nestaci!

            if (!$socket) {
                throw new \Exception("Nelze navazat spojeni s agentem Agent_RemoteExecutor! Chyba cislo $errno: $errstr");
            }

            $httphead =
                "POST / HTTP/1.1\r\n" .
                "Host: $host:$port\r\n" .
                'Content-Length: ' . strlen($xml) . "\r\n" .
                "Connection: Close\r\n\r\n";

            fputs($socket, $httphead . $xml);

            $statusOK = false;
            $header_end = false;
            $code = '';
            $status = '';

            while (!$header_end && !feof($socket)) {
                $headerrow = fgets($socket);

                if (stripos($headerrow, 'HTTP/') !== FALSE) {
                    // HTTP/1.1 200 OK
                    list(, $code, $status) = explode(' ', $headerrow);
                    $statusOK = !strcasecmp('OK', trim($status));
                }

                if (trim($headerrow) == '') $header_end = true;
            }

            if (!$statusOK) {
                throw new \Exception("Chyba pri komunikaci s agentem Agent_RemoteExecutor! Server nestihnul odpovedet nebo prisla spatna odpoved: $code, $status");
            }

            $answer = '';
            while (!feof($socket)) {
                $answer .= fgets($socket);
            }
            fclose($socket);

            if ($GLOBALS['agentmanager_debugdir']) {
                file_put_contents($GLOBALS['agentmanager_debugdir'] . '/agentmanager_' . date('Ymd-His', time()) . '.xml', $answer);
            }
        } catch (\Exception $e) {
            // chyba - napr. nedostupny server
            throw new \Exception($e ->getMessage()); // nepouzivat fromUTF8!
        }

        return $answer;
    }


    /**
     * @param TaskManagerTask $task
     * @param AgentManagerResult $result
     */
    private function updateTaskData($task, $result)
    {
        $task ->progressMessage = $result ->progressMessage;
        $task ->progressMax = $result ->progressMax;
        $task ->progressCurrent = $result ->progressCurr;

        if ($result ->resultCode) {
            // chyba pri spusteni programu agenta
            $task ->errorMessage = $result ->errorMessage;
            $task ->isRunning = false;
        } else {
            if ($result ->agentResult) {
                $task ->errorMessage = $result ->logText;
            }
            $task->isRunning = $result->isRunning;
        }
    }


    /**
     * Spusti agenta pomoci RemoteExecutor
     * @param string $host
     * @param string $port
     * @param string $agent
     * @param string $params
     * @param bool $defaultParams
     * @param bool $background
     * @param string $agentData
     * @return AgentManagerResult
     * @throws \Exception
     */
    public function executeAgentRemote($host, $port, $agent, $params, $defaultParams = true, $background = false, $agentData = null)
    {
        $profilerItem = Profiler::addItem("Vzdalene spusteni agenta $agent", 'AGENT');

        if ($defaultParams) {
            $params .=
                ' ' .
                str_replace(
                    '%LANG%',
                    TextManager::getInstance() ->translationLang,
                    RonPortalSettings::getValue('agent_params', '-lang:%LANG% -alwaysLog')
                ) .
                ' -user:' . User::getLoginName();
        }

        $functionName = $background ? 'StartAgentTask' : 'ExecuteAgentTask';

        $xml =
            '<?xml version="1.0" encoding="windows-1250"?'.">\n".
            "<Container>\n".
            "<String Name='FunctionName'>$functionName</String>\n".
            "<String Name='AgentFileName'>$agent</String>\n".
            '<String Name="AgentParameters">' . StringUtils::xmlTextConvert($params) . "</String>\n".
            ($agentData
                ? '<Container Name="AgentData"><String Name="Text">' . StringUtils::xmlTextConvert($agentData) . "</String></Container>\n"
                : '') .
            '</Container>';

        if ($profilerItem) $profilerItem ->texts[] = $xml;
        MessageManager::addDebugLog("Odesilam xml na $host:$port \n $xml");

        try {
            $xml = StringUtils::toUTF8($xml);
            $answer = $this ->executeRemoteProcedure($host, $port, $xml);

            Profiler::endItem($profilerItem, $answer);
        } catch (\Exception $e) {
            Profiler::endItem($profilerItem, $e);
            throw $e;
        }

        MessageManager::addDebugLog( 'Remote executor answer: ' . StringUtils::fromUTF8($answer) );
        $result = $this ->processRemoteResult($answer);

        if ($background) {
            // proces byl spusten na pozadi - zaznamena informaci do task managera
            $task = new TaskManagerTask();

            $task ->executorParams= [
                'handle' => $result ->handle,
                'agentHost' => $host,
                'agentPort' => $port
            ];

            $task ->caption = $agent;

            if ($result ->resultCode) {
                $task ->isRunning = false;
                $task ->errorMessage = $result ->errorMessage;
            } else {
                $task ->isRunning = true;
            }

            TaskManager::getInststance() ->addTask($task);
            $result ->taskId = $task ->id;
        }

        return $result;
    }


    /**
     * @param TaskManagerTask $task
     * @return AgentManagerResult
     * @throws \Exception
     */
    function getTaskStatus(TaskManagerTask $task)
    {
        $profilerItem = Profiler::addItem('Zjisteni stavu beziciho agenta', 'AGENT');

        $xml =
            '<?xml version="1.0" encoding="windows-1250"?'.">\n".
            "<Container>\n".
            "<String Name='FunctionName'>GetAgentTaskStatus</String>\n".
            "<Integer Name='Handle'>{$task ->executorParams['handle']}</Integer>\n" .
            '</Container>';

        MessageManager::addDebugLog("Odesilam xml na {$task ->executorParams['agentHost']}:{$task ->executorParams['agentPort']} \n $xml");

        try {
            $xml = StringUtils::toUTF8($xml);
            $answer = $this ->executeRemoteProcedure($task ->executorParams['agentHost'], $task ->executorParams['agentPort'], $xml);

            Profiler::endItem($profilerItem, $answer);
        } catch (\Exception $e) {
            Profiler::endItem($profilerItem, $e);
            throw $e;
        }

        MessageManager::addDebugLog( 'Remote executor answer: ' . StringUtils::fromUTF8($answer) );
        $result = $this ->processRemoteResult($answer);

        // proces byl spusten na pozadi - zaznamena informaci do task managera
        if ($result) {
            $this->updateTaskData($task, $result);
        }

        return $result;
    }




    /**
     * Spusti zadaneho agenta ($agentFullPath) se zadanymi parametry ($params)
     * @param string $agentFullPath
     * @param string $params
     * @param bool $defaultParams  Pokud je true, tak navic prida jeste standardni parametry (-sendMessagesToStdOut -user)
     * @return AgentManagerResult
     * @throws \Exception
     */
    private function executeAgentExe($agentFullPath, $params, $defaultParams = true)
    {
        if ($defaultParams) $params .= ' ' . RonPortalSettings::getValue('agent_params', '-lang:%LANG% -alwaysLog') . ' -sendMessagesToStdOut -user:' . User::getLoginName();

        $cmd = ($agentFullPath[0] != '"')
            ? '"' . $agentFullPath . '" ' . $params
            : $agentFullPath . ' ' . $params;

        MessageManager::addDebugMessage('Spoustim prikaz: ' . $cmd);
        $profilerItem = Profiler::addItem($cmd, 'AGENT');

        $result = new AgentManagerResult();

        try {

            $execoutput = [];
            exec($cmd, $execoutput, $execreturnvar);

            if ($execreturnvar) {
                $result->agentResult = $execreturnvar;

                // nepovedlo se spustit agenta
                MessageManager::addDebugMessage("ExecuteAgentExe - RESULT=$execreturnvar");

                if ($execoutput) {
                    $msg = '';
                    foreach ($execoutput as $outputRow) {
                        $msg .= ($msg ? "\n" : '') . $outputRow;  // htmlTextConvert($outputRow);
                        if ($profilerItem) $profilerItem ->texts[] = $outputRow;
                    }
                } else {
                    if (!file_exists($agentFullPath)) {
                        throw new \Exception("Chyba pri spusteni agenta $agentFullPath! Soubor s agentem neexistuje.");
                    }

                    throw new \Exception("Chyba pri spusteni agenta $agentFullPath");
                }

                $result->finalMessage = $msg;
                //MessageManager::addErrorMessage( htmlTextConvert($msg) );

                //return false;
            }

            Profiler::endItem($profilerItem);
        } catch (\Exception $e) {
            Profiler::endItem($profilerItem, $e);
            throw $e;
        }

        return $result;
    }


    /**
     * Spusti zadaneho agenta dochazky ($agent) se zadanymi parametry ($params)
     * @param string $agent
     * @param string $params
     * @param bool $defaultParams   Pokud je true, tak navic prida jeste standardni parametry (-sendMessagesToStdOut -user)
     * @param bool $background      Priznak, ze se ma proces spustit na pozadi - tj. neceka na vysledek
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public function executeAgent($agent, $params, $defaultParams = true, $background = false)
    {
//        $this ->resultCode = null;
//        $this ->agentResult = null;
//        $this ->finalMessage = null;
//        $this ->errorMessage = null;
//        $this ->logText = null;

//        $result = false;

        if ( RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_HOST) ) {
            // spousteni pres remote executor
            $result = $this->executeAgentRemote(
                RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_HOST),
                RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_PORT),
                $agent, $params, $defaultParams, $background);

            //$result = !$this ->agentResult; // 0 = OK
        } else if ( RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_CESTA) ) {
            $agentFullPath = RonPortalSettings::getValue(RonPortalSettings::ADS_AGENT_CESTA) . "\\" . $agent;
            $result = $this ->executeAgentExe($agentFullPath, $params, $defaultParams);
        } else {
            $result = null;
        }

        if ($result && $result ->finalMessage) MessageManager::addDebugMessage($result ->finalMessage);

        return $result;
    }


    /**
     * Vraci true, pokud je povoleno spousteni dochazkovych agentu
     * @return bool
     */
    public static function spousteniADSPovoleno()
    {
        return RonPortalSettings::getSpousteniADS();
    }


    /**
     * Spusti agenta se zadanymi parametry ($params) a nastavi vysledek (agentResult atd.).
     * V pripade chyby pri spousteni vyhodi vyjimku. Neresi chybu ktera nastane uvnitr agenta!
     * @param string $agent
     * @param string $params
     * @param bool $defaultParams Pokud je true, tak navic prida jeste standardni parametry (-sendMessagesToStdOut -user)
     * @param bool $background Priznak, jestli se ma spustit na pozadi (= necekat na vysledek)
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public function executeAgentPerson($agent, $params, $defaultParams = true, $background = false)
    {
        if ( RonPortalSettings::getValue('person_agenthost') ) {
            // spousteni pres remote executor
            $result = $this->executeAgentRemote(RonPortalSettings::getValue('person_agenthost'),
                RonPortalSettings::getValue('person_agentport'), $agent, $params, $defaultParams, $background);
        } else {
            $result = null;
        }

        if ($result && $result ->finalMessage) MessageManager::addDebugMessage($result ->finalMessage);

        return $result;
    }

    /**
     * Spusti zadaneho agenta ($agent) se zadanymi parametry ($params) a nastavi vysledek (agentResult atd.).
     * V pripade chyby pri spousteni vyhodi vyjimku. Neresi chybu ktera nastane uvnitr agenta!
     * @param string $agent
     * @param string $params
     * @param bool $defaultParams Pokud je true, tak navic prida jeste standardni parametry (-sendMessagesToStdOut -user)
     * @return AgentManagerResult|null
     * @throws \Exception
     */
    public function executeAgentJidelna($agent, $params, $defaultParams = true)
    {
        if ( RonPortalSettings::getValue('jidelna_agenthost') ) {
            // spousteni pres remote executor
            $result = $this->executeAgentRemote(RonPortalSettings::getValue('jidelna_agenthost'),
                RonPortalSettings::getValue('jidelna_agentport'),
                $agent, $params, $defaultParams);
        } else if ( RonPortalSettings::getValue('jidelna_cesta') ) {
            // spousteni pres exe
            $agentFullPath = RonPortalSettings::getValue('jidelna_cesta') . "\\" . $agent;
            $result = $this ->executeAgentExe($agentFullPath, $params, $defaultParams);
        } else {
            $result = null;
        }

        if ($result && $result ->finalMessage) MessageManager::addDebugMessage($result ->finalMessage);

        return $result;
    }

    /**
     * Spusti zadaneho agenta ($agent) se zadanymi parametry ($params) a nastavi vysledek (agentResult atd.).
     * V pripade chyby pri spousteni vyhodi vyjimku. Neresi chybu ktera nastane uvnitr agenta!
     * @param string $agent
     * @param string $params
     * @param bool $defaultParams Pokud je true, tak navic prida jeste standardni parametry (-sendMessagesToStdOut -user)
     * @param bool $background Priznak, jestli se ma spustit na pozadi (= necekat na vysledek)
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public function executeAgentVyroba($agent, $params, $defaultParams = true, $background = false)
    {
        if ( RonPortalSettings::getValue('vyroba_agenthost') ) {
            // spousteni pres remote executor
            $result = $this->executeAgentRemote(RonPortalSettings::getValue('vyroba_agenthost'),
                RonPortalSettings::getValue('vyroba_agentport'), $agent, $params, $defaultParams, $background);
        } else {
            $result = null;
        }

        if ($result && $result ->finalMessage) MessageManager::addDebugMessage($result ->finalMessage);

        return $result;
    }


    /**
     * Spusti agenta - viz executeAgent - a vraci vysledek. V pripadne chyby vyhodi vyjimku.
     * @param string $agent
     * @param string $params
     * @param bool $defaultParams
     * @param bool $background      Priznak, ze se ma uloha spustit na pozadi
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public static function executeADS($agent, $params, $defaultParams = true, $background = false)
    {
        $agentManager = new AgentManager();
        $result = $agentManager ->executeAgent($agent, $params, $defaultParams, $background);

        if ($result && $result ->agentResult) {
            if ($result ->logText) {
                //  . ($finalmessage ? ' ' . $finalmessage : '') ); - finalmessage nepridavat, aby se nezobrazovaly hlasky dvojmo!
                throw new \Exception($result ->logText);
            }

            throw new \Exception( 'Agent byl ukoncen chybou: kod ' . $result ->agentResult . ($result ->finalMessage ? ' ' . $result ->finalMessage : '') );
        }

        return $result;
    }


//    /**
//     * Spusti agenta - viz executeAgent - a vraci vysledek. Pripadne chyby vlozi do MessageManagera
//     * @param string $agent
//     * @param string $params
//     * @param bool   $defaultParams
//     * @return bool
//     * @deprecated - pouzijte executeADS, ktera nevraci bool, ale vyhazuje vyjimku!
//     */
//    public static function execute($agent, $params, $defaultParams = true)
//    {
//        try {
//            self::executeADS($agent, $params, $defaultParams);
//            return true;
//        } catch (\Exception $e) {
//            return MessageManager::addErrorMessage($e ->getMessage());
//        }
//    }

    /**
     * Spusti agenta - viz executeAgentPerson. V pripadne chyby vyhodi vyjimku.
     * @param string $agent
     * @param string $params
     * @param bool $defaultParams
     * @param bool $background Priznak, jestli se ma uloha spustit na pozadi
     * @return AgentManagerResult|null
     * @throws \Exception
     */
    public static function executePerson($agent, $params, $defaultParams = true, $background = false)
    {
        $agentManager = new AgentManager();
        $result = $agentManager ->executeAgentPerson($agent, $params, $defaultParams, $background);

        if ($result  &&  $result ->agentResult) {
            if ($result ->logText) {
                throw new \Exception($result ->logText);
            }

            throw new \Exception( 'Agent byl ukoncen chybou: kod ' . $result ->agentResult . ($result ->finalMessage ? ' ' . $result ->finalMessage : '') );
        }

        return $result;
    }

    /**
     * Spusti agenta - viz executeAgentPerson. V pripadne chyby vyhodi vyjimku.
     * @param string $params
     * @param bool $defaultParams
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public static function executePersonAgent($params, $defaultParams = true)
    {
        $agentName = RonPortalSettings::getValue('person_agentapp', 'PERSON') == 'PERSON' ? 'Person_Agent_Utils' : 'MZDY_Agent_Person';

        $agentManager = new AgentManager();

        if ( RonPortalSettings::getValue('person_agenthost') ) {
            $result = $agentManager->executeAgentPerson($agentName, $params, $defaultParams);
        } else if ( RonPortalSettings::getValue(RonPortalSettings::AGENT_HODNOCENI) ) {
            $result = $agentManager->executeAgentExe(RonPortalSettings::getValue(RonPortalSettings::AGENT_HODNOCENI), $params, $defaultParams);
        } else {
            $result = null;
        }


        if ($result && $result ->agentResult) {
            if ($result ->logText) {
                //  . ($finalmessage ? ' ' . $finalmessage : '') ); - finalmessage nepridavat, aby se nezobrazovaly hlasky dvojmo!
                throw new \Exception($result ->logText);
            }

            throw new \Exception( 'Agent byl ukoncen chybou: kod ' . $result ->agentResult . ($result ->finalMessage ? ' ' . $result ->finalMessage : '') );
        }

        return $result;
    }

    /**
     * Spusti agenta - viz executeAgent - a vraci vysledek. V pripadne chyby vyhodi vyjimku.
     * @param  string $agent
     * @param string $params
     * @param bool $defaultParams
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public static function executeJidelna($agent, $params, $defaultParams = true)
    {
        $agentManager = new AgentManager();
        $result = $agentManager ->executeAgentJidelna($agent, $params, $defaultParams);

        if ($result  &&  $result ->agentResult) {
            if ($result ->logText) {
                //  . ($finalmessage ? ' ' . $finalmessage : '') ); - finalmessage nepridavat, aby se nezobrazovaly hlasky dvojmo!
                throw new \Exception($result ->logText);
            }

            throw new \Exception( 'Agent byl ukoncen chybou: kod ' . $result ->agentResult . ($result ->finalMessage ? ' ' . $result ->finalMessage : '') );
        }

        return $result;
    }

    /**
     * Spusti agenta - viz executeAgentVyroba. V pripadne chyby vyhodi vyjimku.
     * @param string $agent     Jmeno agenta je  mozne zadat bez prefixu s kodem aplikace (tj. zacinajici podtrzitkem) - prefix se automaticky doplni podle nastaveni
     * @param string $params
     * @param bool $defaultParams
     * @param bool $background Priznak, jestli se ma uloha spustit na pozadi
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public static function executeVyroba($agent, $params, $defaultParams = true, $background = false)
    {
        if ($agent[0] == '_') {
            // doplni kod aplikace podle nastaveni
            $agent = (RonPortalSettings::getValue('vyroba_agentapp', 'ADS') == 'ADS' ? 'ADS' : 'Zakazky') . $agent;

        }
        $agentManager = new AgentManager();
        $result = $agentManager ->executeAgentVyroba($agent, $params, $defaultParams, $background);

        if ($result && $result ->agentResult) {
            if ($result ->logText) {
                throw new \Exception($result ->logText);
            }

            throw new \Exception( 'Agent byl ukoncen chybou: kod ' . $result ->agentResult . ($result ->finalMessage ? ' ' . $result ->finalMessage : '') );
        }

        return $result;
    }

    /**
     * @param int|string $idPP IdPP nebo seznam IdPP (oddelene strednikem), ktere se maji prepocitat
     * @param int $datumOd
     * @param int $datumDo
     * @param bool $vlozitPrestavky
     * @param bool $kontrola
     * @param string $agentParams
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public static function prepocetDochazky($idPP, $datumOd, $datumDo, $vlozitPrestavky = false, $kontrola = false, $agentParams = '')
    {
        // provede prepocet dochazky
        if ( !RonPortalSettings::getSpousteniADS() ) return null; // neni nastaveno spousteni agenta - prepocet se neprovadi

        $params = ($vlozitPrestavky ? '7' : '3') .  // uloha
            ($kontrola ? ' -checkup' : '') .
            ' -interval:' . date('Y-n-j', $datumOd) . ';' . date('Y-n-j', $datumDo) .
            ' -IDPP:' . $idPP .
            ($agentParams ? ' ' : '') . $agentParams;

        return self::executeADS('ADS_Agent_Processing.exe', $params );
    }

    /**
     * @param int $idPP
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public static function vyuctovaniJidelny($idPP)
    {
        // provede vyuctovani jidelny
        if ( !((RonPortalSettings::getValue('jidelna_agenthost') && RonPortalSettings::getValue('jidelna_agentport')) ||
            (RonPortalSettings::getValue('jidelna_cesta'))
        ) ) return null; // neni nastaveno spousteni agenta - prepocet se neprovadi

        $params = 0 .  // uloha - kompletni vyuctovani jidelny
            ' -IDPP:' . $idPP;

        return self::executeJidelna('Jidelna_Agent_Processing.exe', $params );
    }


    /**
     * Pomoci agenta nebo report serveru spusti skript zadaneho jmena. V pripade chyby vyhodi vyjimku.
     * @param string $category
     * @param string $name
     * @param string $scriptParam
     * @param string $app
     * @return null|AgentManagerResult
     * @throws \Exception
     */
    public static function executeScript($category, $name, $scriptParam, $app = 'ADS')
    {
        global $reportserver_scripts;

        if ($reportserver_scripts) {
            // zapnuto spousteni skriptu pres report server
            $server = ReportServer::createInstance();

            try {
                $server->executeScript($category, $name, $scriptParam);
            } catch (\Exception $e) {
                $server ->close();
                throw $e;
            }

            return null;
        } else {
            // spousteni pres remote executor nebo ADS_Agent_Script.exe
            $params = '-script:"' . $category . '/' . $name . '"';
            if (isset($scriptParam) && $scriptParam !== '') {
                $params .=
                    ' -parameter:' .
                    ($scriptParam[0] === '"' ? '' : '"') .
                    $scriptParam .
                    ($scriptParam[0] === '"' ? '' : '"');
            }

            //return self::execute('ADS_Agent_Script.exe', $params, true); // $defaultparams = true

            $agentManager = new AgentManager();
            switch (strtolower($app)) {
                case 'jidelna' :
                    $result = $agentManager ->executeAgentJidelna('Jidelna_Agent_Script.exe', $params, true);
                    break;

                case 'person' :
                    $result = $agentManager ->executeAgentPerson('Person_Agent_Script.exe', $params, true);
                    break;

                case 'ads' :
                default :
                    $result = $agentManager ->executeAgent('ADS_Agent_Script.exe', $params, true);
                    break;
            }


            if ($result  &&  $result ->agentResult) {
                if ($result ->logText) {
                    //  . ($finalmessage ? ' ' . $finalmessage : '') ); - finalmessage nepridavat, aby se nezobrazovaly hlasky dvojmo!
                    throw new \Exception($result ->logText);
                }

                throw new \Exception( 'Agent byl ukoncen chybou: kod ' . $result ->agentResult . ($result ->finalMessage ? "\n" . $result ->finalMessage : '') );
            }

            return $result;
        }
    }


    static $scriptCache = [];

    /**
     * Pomoci agenta nebo printserveru spusti udalosti zadaneho jmena (nebo jmen). V pripade chyby vyhodi vyjimku.
     * @param Database $database
     * @param string   $category
     * @param string|string[]   $name   Spousti skripty, jejichz nazev zacina zadanym jmenem
     * @param string   $scriptParam
     * @throws \Exception
     */
    public static function executeScriptEvents($database, $category, $name, $scriptParam)
    {
        global $reportserver_scripts;

        if ( !RonPortalSettings::getSpousteniADS()  &&  !$reportserver_scripts ) return; // nic se nespousti

        $cacheName = $category . '/' . (is_array($name) ? implode(', ', $name) : $name);

        MessageManager::addDebugLog('ExecuteScriptEvents: ' . $cacheName);
        $time = microtime(true);

        $scripts = [];

        if (isset(self::$scriptCache[$cacheName])) {
            $scripts = self::$scriptCache[$cacheName];
        } else {
            $params = $database->createParameters();
            $sql =
                'SELECT ScriptName FROM SYS_SCR ' .
                " WHERE ApplicationID = 'ADS'" . // jen ADS (prozatim?)
                ' AND ScriptCategoryName = ' . $params->addParameter($category);

            if (is_array($name)) {
                $first = true;
                $sql .= ' AND (';
                foreach ($name as $scriptName) {
                    if ($first) {
                        $first = false;
                    } else {
                        $sql .= ' OR ';
                    }

                    $sql .= 'ScriptName LIKE ' . $params->addParameter($scriptName . '%');
                }

                $sql .= ')';
            } else {
                $sql .=
                    ' AND ScriptName LIKE ' . $params->addParameter($name . '%');
            }

            $sql .=
                ' ORDER BY ScriptName';

            try {
                $cursor = $database->select($sql, $params);
            } catch (DBException $e) {
                Log::addRecordToDBLog($database, 'SYS_SCR', 'Error', 'ScriptEvents', StringUtils::fromUTF8("Chyba pri nacitani udalosti: \n") . $e->getMessage() . "\n" . $e->dbMessage . " \n\n" . $sql);
                throw new \Exception('Pri nacitani udalosti doslo k chybe!');
            } catch (\Exception $e) {
                Log::addRecordToDBLog($database, 'SYS_SCR', 'Error', 'ScriptEvents', StringUtils::fromUTF8("Chyba pri nacitani udalosti: \n\n") . $e->getMessage() . " \n\n" . $sql);
                throw new \Exception('Pri nacitani udalosti doslo k chybe!');
            }

            while ( $row = $cursor ->fetch() ) {
                $scripts[] = $row[0];
            }
            $cursor ->close();

            self::$scriptCache[$cacheName] = $scripts;
        }

        $cnt = 0;
        $logNames = '';

        if ($scripts) {
            foreach ($scripts as $script) {
                $logNames .= ($logNames ? ', ' : '') . $script;
                // pokud pri provedeni skriptu dojde k chybe, tak vyhodi vyjimku
                self::executeScript($category, $script, $scriptParam);
                $cnt ++;
            }
        }

        $time = microtime(true) - $time;
        if (App::getDebugMode()) {
            Log::addRecordToLog($database, $reportserver_scripts ? "Reportserver skript events ($cnt): $time" : "Agent skript events ($cnt): $time", 'Skript', $category . '/' . $logNames . "... \n" . $scriptParam);
        }
    }
}
