<?php
/**
 * Roman.Szablatura 17.1.2017
 */

namespace RonPortal\Terminaly;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\BrowserRowArray;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;


/**
 * Filtr na vyber terminalu - pokud je zadano umisteni, tak zobrazuje jen terminaly z daneho umisteni
 */
class TerminalFilter extends FilterFormFilter
{
    private $parentFilterForm;

    public function __construct(string $name, RonPortalFilterForm $parentFilterForm)
    {
        parent::__construct($name, self::TYPE_COMBOBOXES);
        $this ->parentFilterForm = $parentFilterForm;
        $this
            ->setOptionsSqlFilter('1 = 1')
            ->setOptionsSqlOrder('Popis')
            ->setOptionsSqlOrder2('Kod');
    }

    public function loadOptions(Database $database): FilterFormFilter
    {
        // pokud je zapnut filtr na umisteni, tak zobrazi jen terminaly z daneho umisteni
        $umisteni = $this ->parentFilterForm ->getFilterValue('umisteni');
        if (isset($umisteni)) {
            $params = $database ->createParameters();
            $sql =
                'SELECT Terminal.Kod, Terminal.Popis' .
                ' FROM Terminal, Umisteni' .
                ' WHERE (Terminal.KodUmisteni = Umisteni.Kod)' .
                ' AND Terminal.KodUmisteni = ' . $params->addParameter($umisteni) .
                ' AND (%FILTER%)' .
                ' ORDER BY %ORDER%';

            $this ->optionsSql = new SqlStatement($sql, $params);
        } else {
            $this ->optionsSql =
                'SELECT Kod, Popis FROM Terminal WHERE %FILTER% ORDER BY %ORDER%';
        }

        return parent::loadOptions($database);
    }
}



/**
 * Class FieldFoto
 * @package Ron\RonPortal\Dochazka
 */
class FieldFoto extends BrowserField
{
    private $browserAction;

    /** @var string */
    private $tabulka;
    /** @var bool */
    public $zobrazitFotky = false;

    public function __construct($database, $action, $tabulka = 'foto', $zobrazitFotky = false)
    {
        parent::__construct($database);

        $this->browserAction = $action;
        $this->tabulka = $tabulka;
        $this->zobrazitFotky = $zobrazitFotky;
    }


    function getHeaderText($fieldIndex, $sortField, $url = null)
    {
        return $this ->zobrazitFotky
            ? parent::getHeaderText($fieldIndex, $sortField, $url)
            : '';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this ->zobrazitFotky) {
            if ($value) {
                return
                    '<div class="position-relative">' .
                    '<div class="browserfoto ronFoto">' .
                    '<img src="?menuid=' . $this->browserAction . '&' . $this->tabulka . '=' . (int)$value . '" alt="' . $this->tabulka . '"/>' .
                    '</div>' .
                    '</div>';
            }
        } else {
            if ($value) {
                return
                    '<span class="ronInfoFoto" data-foto-url="?menuid=' . $this->browserAction . '&' . $this->tabulka . '=' . (int)$value . '"' .
                    ($rowData['col-prijmeni'] ? ' data-foto-alt="' . StringUtils::htmlTextConvert($rowData['col-prijmeni']) . '"' : '') .
                    '>' .
                    '<i class="inline-glyph fa glyph-image"></i>' .
                    //'<img src="images/16_info.png"  alt="' . $this->tabulka . '"/>' .
                    '</span>';
            }
        }

        return '';
    }
}


/**
 * Zobrazuje fotky z terminalu prislusejici k jednotlivym zaznamum.
 * Cas fotky z terminalu nemusi byt stejny jako cas zaznamu, proto si fotky nejdrive nacte a pak vyhledava nejblizsi.
 */
class FotoZTerminaluField extends FieldFoto
{
    /** @var int    Maximalni casovy rozdil mezi zaznamem z terminalu a casem ulozeni fotografie */
    public $maxTimeDiff = 2;

    /** @var array  Seznamy nactenych fotek pro kazdy terminal */
    protected $data;


    function loadConfigurationItem($key, $value)
    {
        if ($key === 'maxTimeDiff') {
            $this ->maxTimeDiff = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value);
        }
    }


    /**
     * Nacte seznam fotografii z terminalu v zadanem casovem rozsahu
     * @param int $datumCasOd
     * @param int $datumCasDo
     * @param string $terminal
     * @return void
     * @throws DBException
     */
    public function loadFotoZTerminalu($datumCasOd, $datumCasDo, $terminal = null)
    {
        $this ->data[] = [];

        $params = $this ->database ->createParameters();
        $cursor = $this ->database ->getSqlResult(
            'SELECT Id, DatumCasOperace, Terminal FROM FotoZTerminalu' .
            ' WHERE DatumCasOperace >= ' . $params ->addDateParameter($datumCasOd) .
            '  AND DatumCasOperace <= ' . $params ->addDateParameter($datumCasDo) .
            (isset($terminal) ? ' AND Terminal = ' . $params ->addParameter($terminal) : '') .
            ' ORDER BY Terminal, DatumCasOperace',
            $params
        );

        while ($row = $cursor ->fetch()) {
            if (!isset($this ->data[$row[2]])) {
                $this ->data[$row[2]] = [];
            }

            $this ->data[$row[2]][] = [
                'id' => $row[0],
                'cas' => $this ->database ->stringToTimestamp($row[1])
            ];
        }
        $cursor ->close();
    }

    function getHeaderAttributes()
    {
        $result = parent::getHeaderAttributes() ?? [];
        $result['data-maxTimeDiff'] = $this ->maxTimeDiff;
        return $result;
    }


    private function getFotoId($rowData)
    {
        $terminal = $rowData['col-kod-terminalu'];

        $list = $this ->data[$terminal] ?? null;
        if (!$list) {
            return null;
        }

        // najde foto metodou puleni intervalu
        $min = 0; $max = count($list) - 1;
        $cas = $this ->database ->stringToTimestamp($rowData['col-cas']);

        while ($min <= $max) {
            $i = intdiv($min + $max, 2);

            if ($this ->data[$terminal][$i]['cas'] === $cas) {
                // nasel fotku se presne stejnym casem
                return $this ->data[$terminal][$i]['id'];
            }

            if ($this ->data[$terminal][$i]['cas'] < $cas) {
                $min = $i + 1;
            } else {
                $max = $i - 1;
            }
        }

        if ($min < count($list)  &&  $this ->data[$terminal][$min]['cas'] - $cas <= $this ->maxTimeDiff) {
            return $this ->data[$terminal][$min]['id'];
        }

        return null;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $value = $this ->getFotoId($rowData);
        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


/**
 * Class FieldDatumCasDen
 */
class FieldDatumCasDen extends BrowserField
{
    var $datumPoslednihoRadku = null;


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param BrowserRowArray $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $date = $this->database->stringToTimestamp($value);

        $datum = StringUtils::format('date', $date);
        $den = TextManager::_('Day_short_' . date("w", $date));

        $result =
            "<span class='padding-right-2' title='$datum'>$den</span>" .
            ($datum != $this ->datumPoslednihoRadku
                ? ' ' . parent::getViewText($value, $rowNumber, $recordId, $rowData)
                : '');

        $this ->datumPoslednihoRadku = $datum;

        return $result;
    }
}



/**
 * Class BrowserZaznamySnimacu
 */
class ZaznamySnimacuBrowser extends RonPortalBrowser
{
    public $action;

    /** @var PravaNaOsoby  */
    var $pravaProhlizet;

    /** @var FieldFoto */
    protected $fFotoOsoby;

    /** @var FieldFoto */
    protected $fFotoZTerminalu;

    /** @var int    Urcuje jak casto (pocet sekund) se ma stranka automaticky aktualizovat */
    private $dobaAktualizace;

    /**
     * BrowserZaznamySnimacu constructor.
     * @param Database $database
     * @param $action
     * @throws Exception
     */
    public function __construct(Database $database, $action)
    {
        parent::__construct($database, 'zaznamysnimacu');

        $this->title = TextManager::_('zaznamy_snimacu');
        $this->code = 'browserzaznamysnimacu';
        $this->tableId = 'browserzaznamysnimacu';
        $this->browserUrl = '?menuid=zaznamysnimacu&';
        $this->action = $action;

        $this->pravaProhlizet = Prava::vratPrava('prohl_terminaly');

        $this ->maxPageRows = 500;
        $this ->dobaAktualizace = RonPortalSettings::getIntValue('zaznamysnimacu_dobaaktualizacestranky', 60);

        $this->defaultSort = "-col-datum";

        $this->addField(new FieldDatumCasDen($this->database))
            ->setSqlFieldName('ZaznamySnimacu.DatumCas') ->setHeaderText('Datum') ->setFormat('date')
            ->setCode('col-datum') ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('ZaznamySnimacu.DatumCas') ->setFormat('fulltime')
            ->setCode('col-cas')
            ->setHeaderText('Cas') ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.RC')
            ->setCode('col-rc');

        $this ->fFotoOsoby = new FieldFoto($this->database, $this->action, 'foto');
        $this->addField($this ->fFotoOsoby)
            ->setSqlFieldName('Osoba.IdFotografie')
            ->setCode('col-foto') ->setHeaderText('foto_z_karty');

//        $this ->fFotoZTerminalu = new FieldFoto($this->database, $this->action, 'fotozterminalu');
//        $this->addField($this ->fFotoZTerminalu)
//            ->setSqlFieldName('FotoZTerminalu.ID')
//            ->setCode('col-fotozterminalu') ->setHeaderText('foto_z_terminalu');

        $this ->fFotoZTerminalu = new FotoZTerminaluField($this->database, $this->action, 'fotozterminalu');
        $this->addField($this ->fFotoZTerminalu)
            ->setCode('col-fotozterminalu') ->setHeaderText('foto_z_terminalu');

        // jmeno a prijmeni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni') ->setCode('col-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno') ->setCode('col-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Titul') ->setCode('col-titul');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.TitulZa') ->setCode('col-titulza');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-jmeno', 'col-prijmeni', 'col-titul', 'col-titulza'))
            ->setCode('col-osoba') ->setOrdering(true)
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC, ZaznamySnimacu.DatumCas DESC');

        $this->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo') ->setHeaderText('Osobni_cislo')
            ->setCode('col-osobnicislo')
            ->setOrdering(true)
            ->setSortSQL('PracovniPomer.OsobniCislo, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('PracovniPomer.OsobniCislo DESC, ZaznamySnimacu.DatumCas DESC');

        $this ->addField()
            ->setSqlFieldName('PracovniPomer.DatumNastupu') ->setHeaderText('Datum_nastupu') ->setFormat('date')
            ->setCode('col-datumnastupu')
            ->setOrdering(true) ->setAlign('left');
            //->setVisible(RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitdatumnastupu')); - zruseno - lze vypnout v XML konfiguraci


        // kod a popis operace
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OperaceDochazky.Kod')
            ->setCode('col-kod-operace');

        $this ->addField()
            ->setSqlFieldName('OperaceDochazky.Popis') ->setHeaderText('kod_operace') ->setOrdering(true)
            ->setCode('col-operace')
            ->setSortSQL('OperaceDochazky.Popis, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('OperaceDochazky.Popis DESC, ZaznamySnimacu.DatumCas DESC')
            ->setSqlFieldIndex2('col-kod-operace');

        // kod a popis terminalu
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Terminal.Kod')
            ->setCode('col-kod-terminalu');

        $this ->addField()
            ->setSqlFieldName('Terminal.Popis') ->setHeaderText('terminal') ->setOrdering(true)
            ->setCode('col-terminal')
            ->setSqlFieldIndex2('col-kod-terminalu')
            ->setSortSQL('Terminal.Popis, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('Terminal.Popis DESC, ZaznamySnimacu.DatumCas DESC');


        // karta
        $this->addField()
            ->setSqlFieldName('ZaznamySnimacu.Karta') ->setHeaderText('Karta')
            ->setCode('col-karta')
            ->setOrdering(true);

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('VlozenoManualne') ->setHeaderText('vlozeno_manualne') ->setAlign('center') ->setOrdering(true)
            ->setCode('col-vlozenomanualne')
            ->setOptions("yesno")
            ->setSqlFieldIndex2('col-uzivatel');

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('VlozenoOnline') ->setHeaderText('vlozeno_online') ->setAlign('center') ->setOrdering(true)
            ->setCode('col-vlozenoonline')
            ->setOptions("yesno");

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('ZaznamySnimacu.Uzivatel') ->setHeaderText('Uzivatel') ->setOrdering(true)
            ->setCode('col-uzivatel');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('TypRozdeleni.Popis')
            ->setCode('col-typrozdeleni');

        $this ->addField()
            ->setsqlFieldName('Rozdeleni.Popis') ->setHeaderText('Rozdeleni')
            ->setCode('col-rozdeleni')
            ->setTitleFieldIndex('col-typrozdeleni')
            ->setOrdering(true);

        $this->addField()
            ->setSqlFieldName('ZaznamySnimacu.Poznamka') ->setHeaderText('Poznamka') ->setOrdering(true)
            ->setCode('col-poznamka')
            ->setClassName('poznamka');

        $this ->addField(BrowserField::FIELD_TYPE_LINE)
            ->setCode('col-line-number');  // ->setLink("#top")???
    }


    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this->database, 'zaznamysnimacu', true);

//        $this ->filterForm ->nastavDatumOdDo(true, TextManager::_('zobrazit_data_za_pocet_dni'));
//        $this ->filterForm ->datumOdDoSessionPrefix = 'zaznamysnimacu_';
//        $this ->filterForm ->defaultDatumOdDo = DateUtils::today();
//        $this ->filterForm ->datumOdDoLinks = [
//            [
//                'datumOd' => DateUtils::today(),
//                'datumDo' => DateUtils::incDayStart(DateUtils::today()) - 60,
//                'text' => TextManager::_('Dnes')
//            ]
//        ];
//        $this ->filterForm ->showCasOdDo = true;

        // puvodnidochazka /  historiepruchodu / vse
        $filterSrc = new FilterFormFilter('src');
        $filterSrc
            ->setSessionName('filterform_zaznamysnimacu_src')
            ->setRowId('filterrow_zaznamy_zdroj')
            ->addItem('all', TextManager::_('Vse'))
            ->addItem('puvodnidochazka',  TextManager::_('puvodnidochazka'))
            ->addItem('historiepruchodu',  TextManager::_('historiepruchodu'))
            ->setDefaultValue('all');
        $this ->filterForm ->addFilter($filterSrc, true);

        $this->filterForm ->addFilterRozdeleni1();
        $this->filterForm ->addFilterRozdeleni2();

        ($this ->filterForm ->addFilterPP(true, 'prohl_terminaly'))
            ->setVcetneExternich(true);
//        $this->filterForm ->nastavOsoby(true, Prava::vratPrava('prohl_terminaly'), time(), time(),true);
//        $this->filterForm ->povolitVsechyOsoby = true;

        $this ->filterForm ->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setShowTime(true)
                ->addLink([
                    'datumOd' => DateUtils::today(),
                    'datumDo' => DateUtils::incDayStart(DateUtils::today()) - 60,
                    'text' => TextManager::_('Dnes')
                ])
                ->setRequired(true)
                ->setSessionName('zaznamysnimacu_datum'),
            true
        );

        // umisteni
        $filterUmisteni = new FilterFormFilter('umisteni', FilterFormFilter::TYPE_COMBOBOXES);
        $filterUmisteni
            ->setSessionName('filterform_umisteniterminalu')
            ->setRowId('filterrow_umisteniterminalu')
            ->setCaption(TextManager::_('umisteni'))
            ->addItem('', TextManager::_('_vse_'))
            ->addItem2('', TextManager::_('_vse_'))
            ->setHtmlAttributes(['id' => 'filtr_umisteni', 'data-ron-select2' => '#filtr_umisteni_kod', 'data-ron-autosubmit' => '1'])
            ->setHtmlAttributes2(['id' => 'filtr_umisteni_kod', 'data-ron-select2' => '#filtr_umisteni', 'data-ron-autosubmit' => '1'])
            ->setOptionsSql('SELECT Kod, Popis FROM Umisteni WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setOptionsSqlOrder2('Kod')
            ->setSqlFilter('Umisteni.Kod = ');
        $this ->filterForm ->addFilter($filterUmisteni);

        // terminal
        $filterTerminal = new TerminalFilter('terminal', $this ->filterForm);
        $filterTerminal
            ->setSessionName('filterform_terminal')
            ->setRowId('filterrow_terminal')
            ->setCaption(TextManager::_('terminal'))
            ->addItem('', TextManager::_('_vse_'))
            ->addItem2('', TextManager::_('_vse_'))
            ->setHtmlAttributes(['id' => 'zaznamy_filtr_terminal', 'data-ron-select2' => '#zaznamy_filtr_kod_terminalu', 'data-ron-autosubmit' => '1'])
            ->setHtmlAttributes2(['id' => 'zaznamy_filtr_kod_terminalu', 'data-ron-select2' => '#zaznamy_filtr_terminal', 'data-ron-autosubmit' => '1'])
            ->setSqlFilter('ZaznamySnimacu.Terminal = ');
        $this ->filterForm ->addFilter($filterTerminal);

        // kod operace
        $filterOperace = new FilterFormFilter('operace', FilterFormFilter::TYPE_COMBOBOXES);
        $filterOperace
            ->setSessionName('filterform_operaceterminalu')
            ->setCaption(TextManager::_('kod_operace'))
            ->addItem('', TextManager::_('_vse_'))
            ->addItem2('', TextManager::_('_vse_'))
            ->setOptionsSql('SELECT Kod, Popis FROM OperaceDochazky WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setOptionsSqlOrder2('Kod')
            ->setHtmlAttributes(['id' => 'zaznamy_filtr_operace', 'data-ron-select2' => '#zaznamy_filtr_operace_kod', 'data-ron-autosubmit' => '1'])
            ->setHtmlAttributes2(['id' => 'zaznamy_filtr_operace_kod', 'data-ron-select2' => '#zaznamy_filtr_operace', 'data-ron-autosubmit' => '1'])
            ->setSqlFilter('ZaznamySnimacu.KodOperace = ');
        $this ->filterForm ->addFilter($filterOperace);

        // zobrazit foto ve strance
        $filtrZobrazitFotky = new FilterFormFilter('zobrazitfotky', FilterFormFilter::TYPE_CHECKBOX);
        $filtrZobrazitFotky
            ->setSessionName('zaznamysnimacku_zobrazitfotky')
            ->setCaption(TextManager::_('foto_v_radku'))
            ->setRowId('ilterrow_zaznamy_volby');
        $this ->filterForm ->addFilter($filtrZobrazitFotky);
    }

    public function loadConfigurationItem($key, $value, $valueText)
    {
        if ($key === 'dobaAktualizace') {
            $this ->dobaAktualizace = (int)$value;
        } else {
            parent::loadConfigurationItem($key, $value, $valueText);
        }
    }

    function loadConfiguration()
    {
        parent::loadConfiguration();

        $this ->tableHtmlParams['data-dobaAktualizace'] = $this ->dobaAktualizace;
    }


    public function getSqlFromPart()
    {
        $fieldsZaznamy =
            'RC, OsobniCislo, DatumNastupu, KodOperace, Terminal, Karta,' .
            ' VlozenoManualne, VlozenoOnLine,' .
            ' ZemSirka, ZemDelka, PopisPolohy,' .
            ' KodTypuRozdeleni, Rozdeleni, Uzivatel, Poznamka';

        //Volba tabulky pro sqlFromPart
        if ($this->filterForm ->getFilterValue('src') === 'historiepruchodu') {
            $fieldsZaznamySQL =
                "SELECT DatumCasPruchodu DatumCas, $fieldsZaznamy FROM HistoriePruchodu";
        } else if ($this ->filterForm ->getFilterValue('src') === 'puvodnidochazka') {
            $fieldsZaznamySQL =
                "SELECT DatumCasOperace DatumCas, $fieldsZaznamy FROM PuvodniDochazka";
        } else {
            $fieldsZaznamySQL =
                "SELECT DatumCasOperace DatumCas, $fieldsZaznamy FROM PuvodniDochazka" .
                ' UNION ALL' .
                " SELECT DatumCasPruchodu DatumCas, $fieldsZaznamy FROM HistoriePruchodu";
        }

        return
            "($fieldsZaznamySQL) ZaznamySnimacu" .
            "\nLEFT JOIN PracovniPomer ON (PracovniPomer.RC = ZaznamySnimacu.RC AND PracovniPomer.OsobniCislo = ZaznamySnimacu.OsobniCislo AND PracovniPomer.DatumNastupu = ZaznamySnimacu.DatumNastupu)" .
            "\nLEFT JOIN Osoba ON (Osoba.RC = ZaznamySnimacu.RC)" .
            "\nLEFT JOIN OperaceDochazky ON (OperaceDochazky.Kod = ZaznamySnimacu.KodOperace)" .
            "\nLEFT JOIN Terminal ON (Terminal.Kod = ZaznamySnimacu.Terminal)" .
            "\nLEFT JOIN Umisteni ON (Terminal.KodUmisteni = Umisteni.Kod)" .
            //"\nLEFT JOIN FotoZTerminalu ON (ZaznamySnimacu.DatumCas = FotoZTerminalu.DatumCasOperace AND ZaznamySnimacu.Terminal = FotoZTerminalu.Terminal)" .
            "\nLEFT JOIN TypRozdeleni ON (ZaznamySnimacu.KodTypuRozdeleni = TypRozdeleni.Kod)" .
            "\nLEFT JOIN Rozdeleni ON (ZaznamySnimacu.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni  AND  ZaznamySnimacu.Rozdeleni = Rozdeleni.Kod)";
    }

    function loadFilterData()
    {
        parent::loadFilterData();
        $this ->fFotoZTerminalu ->zobrazitFotky = $this ->filterForm ->getFilterValue('zobrazitfotky');
        $this ->fFotoOsoby ->zobrazitFotky = $this ->filterForm ->getFilterValue('zobrazitfotky');
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        if ($this ->fFotoZTerminalu ->visible) {
            $this ->fFotoZTerminalu ->loadFotoZTerminalu(
                $this->filterForm ->getFilterValue('datumod'), $this->filterForm ->getFilterValue('datumod', 'datumDo') + 60,
                $this ->filterForm ->getFilterValue('terminal'));
        }
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        // Zobrazi pokud ma na osobu pravo, nebo neni osoba prirazena
        return !isset($values['col-rc'])  ||  $this->pravaProhlizet->lzeZobrazitRC($values['col-rc']);
    }




    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $result .=
            ($result ? ' AND ' : '') .
            'Zaznamysnimacu.DatumCas >= ' . $params->addTimestampParameter($this->filterForm ->getFilterValue('datumod')) .
            ' AND Zaznamysnimacu.DatumCas < ' . $params->addTimestampParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo') + 60);

        if ($this ->filterForm ->getFilterValue('osoba')) {
            $result .=
                ' AND Zaznamysnimacu.RC = ' . $params->addParameter($this ->filterForm ->getFilterValue('osoba', 'rc'));
        } else {
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                if ($rozdeleniFilter ->getValue() !== null) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'ZaznamySnimacu',
                            $rozdeleniFilter ->getTypRozdeleni(), $rozdeleniFilter ->getValue(),
                            time(), time(),
                            $rozdeleniFilter ->getValue('vcetnePodrizenych'),
                            $params);
                }
            }
        }

        return new SqlStatement($result, $params);
    }



    public function render()
    {
        if ($this ->dobaAktualizace) {
            echo "<div class='auto-refresh' data-timeout='{$this ->dobaAktualizace}' data-url='?menuid=zaznamysnimacu'>";
        }

        parent::render();

        if ($this ->dobaAktualizace) {
            echo '</div>';
        }
    }

}