<?php
/**
 * stanislav.elbl, 25.03.2021
 */

namespace RonPortal\Uchazeci;

use Ron\Controls\FormField;
use Ron\DB\Database;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\User;

class KomunikaceUchazeceForm extends RonPortalForm
{
    /**
     * @var Uchazec $uchazec
     */
    private $uchazec;

    /**
     * @param Database $database
     * @param Uchazec $uchazec
     */
    function __construct($database, $uchazec)
    {
        parent::__construct($database);

        $this ->uchazec = $uchazec;

        $this -> htmlAction = '?menuid=uchazeci&action=komunikace';
        $this -> stornoAction = '?menuid=uchazeci&action=zobrazuchazece&record=' . $uchazec->rc;
        $this -> title = TextManager::_('Formular_zapisu_komunikace_s_uchazecem');
        $this -> sqlTableName = 'Komunikace';

        $this ->code = 'formkomunikaceuchazece';

        $this ->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('IdKomunikace') ->setPrimary(true)
            ->setCode('field-id');

        $this -> addField(FormField::FIELD_TYPE_CONSTANT)
            ->setFieldName('RCOsoby') ->setReadOnly(true) ->setDefaultValue(User::getRC())
            ->setCode('field-rcosoby');

        $this -> addField(FormField::FIELD_TYPE_TEXT)
            ->setCaption('Uchazec')
            ->setCode('field-uchazec')
            ->setText('<strong>' . StringUtils::htmlTextConvert($uchazec->jmeno) . ' ' .  StringUtils::htmlTextConvert($uchazec->prijmeni) . '</strong>');

        if ($this ->isInsert()) {
            $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
                ->setFieldName('DatumCas')->setReadOnly(true)
                ->setCode('field-datumcas-novy');
        } else {
            $this->addField(FormField::FIELD_TYPE_DATETIME)
                ->setFieldName('DatumCas')->setReadOnly(true) ->setCaption('Datum_a_cas')
                ->setCode('field-datumcas');
        }

        $this -> addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('DruhKomunikace') ->setCaption('Druh_Komunikace')
            ->setCode('field-druhkomunikace')
            ->setSelectSql(
                'SELECT Kod, Popis FROM DruhKomunikace WHERE %FILTER% ORDER BY %ORDER%',
                '1 = 1',
                'Kod');

        $this -> addField()
            ->setFieldName('DruhSdeleni') ->setCaption('Druh_Sdeleni') ->setMaxLength(50)
            ->setCode('field-druhsdeleni');

        $this -> addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('ObsahSdeleni') ->setCaption('Vyjadreni_personalisty')
            ->setCode('field-obsahsdeleni');

        $this -> addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('OdpovedUchazece') ->setCaption('Odpoved_uchazece');
    }


    function renderCommonFields()
    {
        parent::renderCommonFields();
        echo '<input type="hidden" name="RCUchazece" value="' . StringUtils::htmlTextConvert($this ->uchazec ->rc) . '"/>';
    }


    function createDBRecordFields($rec, $insert)
    {
        parent::createDBRecordFields($rec, $insert);

        $rec ->addField('RCUchazece', '', true, 15, $this ->uchazec ->rc);
        $rec ->addField(null, '', true, 0, '   ' . $this ->uchazec ->jmeno . ' ' . $this ->uchazec ->prijmeni);
    }
}
