<?php

namespace RonPortal\Uchazeci;

use Ron\DB\Database;
use Ron\MessageManager;
use RonPortal\Zamestnanci\VzdelaniOsoby;

/**
 * Trida reprezentuje zaznam z tabulky P_Uchazec
 * Class Uchazec
 */
class Uchazec
{
    public $rc;
    public $jmeno;
    public $prijmeni;
    public $titul;
    public $titulza;
    public $email;
    public $mobil;
    public $telefon1;
    public $telefon2;
    public $uzivatelskaPole;
    public $idFotografie;
    public $poznamka;
    public $zdrojVzniku;

    /* @var RozdeleniUchazece[] $rozdeleni */
    public $rozdeleni;
    /* @var VzdelaniOsoby[] $vzdelani */
    public $vzdelani;
    /* @var DovednostUchazece[] $dovednosti */
    public $dovednosti;
    /* @var DokumentUchazece[] $dokumenty */
    public $dokumenty;
    /* @var KomunikaceUchazece[] $komunikace */
    public $komunikace;

    /**
     * @param Database $database
     * @param string $rc
     * @return Uchazec
     * @throws \Exception
     * @throws \Ron\DB\DBException
     */
    public static function nactiUchazece($database, $rc)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT Titul, Jmeno, Prijmeni, TitulZa, E_mail, Mobil, Telefon1, Telefon2,' . //0..7
            '  uzivatelska1, uzivatelska2, uzivatelska3, uzivatelska4, uzivatelska5, uzivatelska6,' . //8..13
            '  uzivatelska7, uzivatelska8, uzivatelska9, uzivatelska10, uzivatelska11, uzivatelska12,' . //14..19
            '  uzivatelska13, uzivatelska14, uzivatelska15, uzivatelska16, uzivatelska17,' . //20..24
            '  uzivatelska18, uzivatelska19, uzivatelska20, idFotografie, Poznamka,' . // 25..26
            '  RC,' . // 30
            '  ZdrojVznikuPP.Popis' . // 31
            ' FROM  P_Uchazec LEFT OUTER JOIN ZdrojVznikuPP ON P_Uchazec.ZdrojVzniku = ZdrojVznikuPP.Kod ' .
            ' WHERE RC = ' . $params->addParameter($rc);

        $cursor = $database ->getSqlResult($sql,$params);
        if (!$row = $cursor ->fetch()) {
            throw new \Exception('Uchazec neexistuje');
        }
        $cursor ->close();

        $uchazec = new Uchazec();
        $uchazec->nactiUchazeceZRadku($row);
        $uchazec->nactiDetailyUchazece($database, $rc);

        return $uchazec;
    }


    public function nactiUchazeceZRadku($row)
    {
        $this->rc = $row[30];

        $this->titul = $row[0];
        $this->jmeno = $row[1];
        $this->prijmeni = $row[2];
        $this->titulza = $row[3];
        $this->email = $row[4];
        $this->mobil = $row[5];
        $this->telefon1 = $row[6];
        $this->telefon2 = $row[7];
        $this->uzivatelskaPole = [];
        for ($i = 1; $i <= 20; $i++) {
            $this->uzivatelskaPole[$i] = $row[$i+7];
        }
        $this->idFotografie = $row[28];
        $this->poznamka = $row[29];
        $this->zdrojVzniku = $row[31];
    }


    public function nactiDetailyUchazece($database, $rc)
    {
        $this->vzdelani = VzdelaniOsoby::nactiVzdelani($database, $rc, 'VzdelaniUchazece');
        $this->dovednosti = DovednostUchazece::nactiDovednostiUchazece($database, $rc);
        $this->dokumenty = DokumentUchazece::nactiDokumentyUchazece($database,$rc);
        $this->rozdeleni = RozdeleniUchazece::nactiRozdeleniUchazece($database, $rc);
        $this->komunikace = KomunikaceUchazece::nactiKomunikaciUchazece($database, $rc);
    }
}