<?php
/**
 * User: pavel.rudol
 * Date: 15.5.2018
 * Time: 13:15
 */

namespace RonPortal\Vyroba;

use Ron\DateUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\MessageManager;
use Ron\MVC\Model;
use Ron\TextManager;
use RonPortal\Dochazka\DenniVysledek;
use RonPortal\Dochazka\MesicniVysledek;
use Ron\StringUtils;

$model = Model::getInstance();

$vyroba = $model->getData('vyroba');
$den = $model->getData('den');
$nacistPost = $model->getData('zobrazitpost');

$pocetChybnychZaznamu = $model->getData('pocet_chyb');

$nastaveniFormulare = $model->getData('nastaveni_formulare');
/** @var VyrobaFilterForm $filterForm */
$filterForm = $model->getData('filter');
$vychoziCinnost = VyrobaController::$vychoziCinnost;
$vychoziVyrobek = VyrobaController::$vychoziVyrobek;
$vychoziZakazka = VyrobaController::$vychoziZakazka;
$datumZpracovani = $model->getData('datum_zpracovani');

?>

<h1><?= TextManager::_('Vyroba'); ?></h1>

<?php
if ($filterForm instanceof VyrobaFilterForm)
    $filterForm->render();
else
    MessageManager::getInstance()->showMessages();

echo "<h3>" . TextManager::_('Vyroba_zpracovano_do') . StringUtils::format('datetime', $datumZpracovani) . "</h3>";

if (is_array($vyroba)) {
    ?>
    <div class="vyroba_form">
    <form method="post" name="vyroba" action="?menuid=vyroba" class="submitWorkingInfo">
        <input type="hidden" name="den" value="<?= StringUtils::format('date', $den ?: DateUtils::today()) ?>">
        <?= Csrf::csrfHiddenInput('editace_vyroby_den_' . $den ?: DateUtils::today()) ?>

        <?php
            if (!VyrobaController::$existujeSloupecCinnost) {
                echo "<input type='hidden' name='vyroba_vychozi_cinnost' value='{$vychoziCinnost}'>";
            }
        ?>

        <table class="browser rowOver oddEvenRows vyroba">
            <thead>
                <tr>
                    <?php if (!$filterForm ->getFilterValue('osoba')) { ?>
                    <th><?= TextManager::_('Osobni_cislo'); ?></th><th><?= TextManager::_('Zamestnanec'); ?></th>
                    <?php }
                        if (isset($nastaveniFormulare['sloupeceditace'])  &&  $nastaveniFormulare['sloupeceditace'] instanceof \ArrayAccess) {
                            foreach ($nastaveniFormulare['sloupeceditace'] as $sloupecEditace) {
                                if ($sloupecEditace['nadpis']) {
                                    echo "<th>{$sloupecEditace['nadpis']}</th>";
                                }
                                else {
                                    switch ($sloupecEditace['typ']) {
                                        case 'VYROBEK' :
                                            echo "<th>" . TextManager::_('Vyroba_vyrobek') . "</th>";
                                            break;
                                        case 'CINNOST' :
                                            echo "<th>" . TextManager::_('Vyroba_cinnost') . "</th>";
                                            break;
                                        case 'ZAKAZKA' :
                                            echo "<th>" . TextManager::_('Vyroba_zakazka') . "</th>";
                                            break;
                                        case 'MNOZSTVI' :
                                            echo "<th>" . TextManager::_('Vyroba_mnozstvi_dobre') . "</th>";
                                            break;
                                        case 'MNOZSTVISPATNE' :
                                            echo "<th>" . TextManager::_('Vyroba_mnozstvi_spatne') . "</th>";
                                            break;
                                        case 'HODINY' :
                                            echo "<th>" . TextManager::_('Vyroba_hodiny') . "</th>";
                                            break;
                                        case 'CASOD' :
                                            echo "<th>" . TextManager::_('Vyroba_casod') . "</th>";
                                            break;
                                        case 'CASDO' :
                                            echo "<th>" . TextManager::_('Vyroba_casdo') . "</th>";
                                            break;
                                        case 'VYSLEDEKHODINY' :
                                            echo "<th>" . TextManager::_('Vyroba_vysledek_hodiny') . "</th>";
                                            break;
                                    }
                                }
                            }
                        }
                    ?>
                </tr>
            </thead>
    <?php
    $n = 1;

    if ($pocetChybnychZaznamu > 0)
        echo "<input type='hidden' name='pocet_chybnych_zaznamu' value='{$pocetChybnychZaznamu}' />";

    $lastPP = null;

    /** @var ZaznamEditaceVyroby $zaznamVyroby */
    foreach ($vyroba as $zaznamVyroby) {
        $nacistRadekPost = $nacistPost;
        
        $idZaznamu = $zaznamVyroby->serialVyroba ?: $zaznamVyroby->idPP;
        $maxIndexRadku = $nacistPost ? StringUtils::htmlTextConvert($_POST["{$idZaznamu}_max_index"]) : 1;

        if ($zaznamVyroby->jeChyba) {
            $idZaznamu = "vyroba_chyba_{$n}";
            $n++;
        }

        for ($i = 1; $i <= $maxIndexRadku; $i++) {
            $idRadku = "{$idZaznamu}_{$i}";
            $maxIndexRadkuHidden = $maxIndexRadku;

        ?>
        <tr class="vyroba_radek" data-vyroba-idpp="<?= $zaznamVyroby->idPP ?>" data-vyroba-serial="<?= $zaznamVyroby->serialVyroba ?: 0 ?>" <?= ($i == $maxIndexRadku) ? "data-vyroba-max-index='$maxIndexRadku'" : '' ?> data-vyroba-idradku="<?= $idZaznamu ?>">
            <?php if (!$filterForm ->getFilterValue('osoba')) { ?>
            <td class="vyroba_osc">
                <?php if ($maxIndexRadkuHidden > 0) { ?>
                    <input type="hidden" name="<?= $idZaznamu ?>_max_index" value="<?= $maxIndexRadkuHidden ?>" />
                <?php } ?>

                <?php if ($zaznamVyroby->jeChyba) { ?>
                    <input type="hidden" name="vyroba_idpp_<?= $idZaznamu ?>" value="<?= $zaznamVyroby->idPP ?>" />
                <?php } ?>

                <?= ($lastPP != $zaznamVyroby ->idPP) ? StringUtils::htmlTextConvert($zaznamVyroby->osobniCislo) : '' ?>
            </td>
            <td class="vyroba_jmeno">
                <?= ($lastPP != $zaznamVyroby ->idPP) ? StringUtils::htmlTextConvert($zaznamVyroby->jmenoOsoby) : '' ?>
            </td>

            <?php } else {
                if ($maxIndexRadkuHidden > 0) {
                    echo "<input type='hidden' name='{$idZaznamu}_max_index' value='{$maxIndexRadkuHidden}' />";
                }
            }
                if (isset($nastaveniFormulare['sloupeceditace'])  &&  $nastaveniFormulare['sloupeceditace'] instanceof \ArrayAccess) {
                    foreach ($nastaveniFormulare['sloupeceditace'] as $sloupecEditace) {
                        switch ($sloupecEditace['typ']) {
                            case 'VYROBEK' :
                                echo "<td class='vyroba_vyrobek {$sloupecEditace['class']} '>";

                                $vyrobek_txt = TextManager::_('Vyroba_vyrobek');
                                $vyrobek = $nacistRadekPost ? StringUtils::htmlTextConvert($_POST["vyroba_vyrobek_{$idRadku}"]) : ($zaznamVyroby->vyrobek ?: $vychoziVyrobek);

                                if (isset($sloupecEditace['zobrazitnaseptavac']) && ($sloupecEditace['zobrazitnaseptavac'] == 0)) {
                                    $class = $zaznamVyroby->vyrobekError ? 'error' : '';
                                    echo "<input type='text' name='vyroba_vyrobek_{$idRadku}' value='{$vyrobek}' class='$class'>";
                                } else {
                                    $comboFields = $sloupecEditace['combo_pole'] ?: 'kod;popis';
                                    $comboTextField = $sloupecEditace['combo_pole_text'] ?: 'popis';

                                    $reload = '';
                                    if (isset($sloupecEditace['nacistvyrobkyzakazky']) && ($sloupecEditace['nacistvyrobkyzakazky'] == 1)) {
                                        $reload =  "data-reload-item='#vyroba_zakazka_{$idZaznamu}_view_{$i}'";
                                    }

                                    /*echo "<input name='vyroba_vyrobek_{$idZaznamu}_view' id='vyroba_vyrobek_{$idZaznamu}_view_{$i}'
                                                    data-ron-combo-url='?menuid=vyroba&action=seznamvyrobku'
                                                    data-ron-combo-code='vyroba_vyrobek_{$idRadku}'
                                                    data-ron-combo-group='vyroba_vyrobek'
                                                    data-ron-combo-fields='{$comboFields}'
                                                    data-ron-combo-text-field='{$comboTextField}'
                                                    data-ron-combo-default-value='$vychoziVyrobek'
                                                    title='{$vyrobek_txt}'
                                                    class='vyroba_vyrobek_t ron_select_combo form-select' value=''/>";*/

                                    echo "<input name='vyroba_vyrobek_{$idZaznamu}_view' id='vyroba_vyrobek_{$idZaznamu}_view_{$i}' 
                                                    data-max-visible-rows='30'
                                                    data-ronselect-class='form-control'
                                                    data-auto-load-items='1'
                                                    data-items-url='?menuid=vyroba&action=seznamvyrobku'
                                                    data-url-add='zakazka'
                                                    $reload
                                                    data-enable-url-cache='1'
                                                    data-columns='{$comboFields}'
                                                    data-value-column='kod'
                                                    data-text-column='{$comboTextField}'
                                                    data-code-input='vyroba_vyrobek_{$idRadku}'
                                                    data-default-value='$vychoziVyrobek'
                                                    title='{$vyrobek_txt}'
                                                    class='vyroba_vyrobek_t ronselect form-control vyroba_combo' value='$vyrobek'/>";

                                    echo "<input type='hidden' name='vyroba_vyrobek_{$idRadku}' value='{$vyrobek}'>";
                                }
                                echo "</td>";
                                break;
                            case 'CINNOST' :
                                echo "<td class='vyroba_cinnost {$sloupecEditace['class']} '>";

                                $cinnost_txt = TextManager::_('Vyroba_cinnost');
                                $cinnost = $nacistRadekPost ? StringUtils::htmlTextConvert($_POST["vyroba_cinnost_{$idRadku}"]) : ($zaznamVyroby->cinnost ?: $vychoziCinnost);

                                if (isset($sloupecEditace['zobrazitnaseptavac']) && ($sloupecEditace['zobrazitnaseptavac'] == 0)) {
                                    $class = $zaznamVyroby->cinnostError ? 'error' : '';
                                    echo "<input type='text' name='vyroba_cinnost_{$idRadku}' value='$cinnost' class='$class'>";
                                } else {
                                    $comboFields = $sloupecEditace['combo_pole'] ?: 'kod;popis';
                                    $comboTextField = $sloupecEditace['combo_pole_text'] ?: 'popis';

//                                    echo "<input name='vyroba_cinnost_{$idZaznamu}_view' id='vyroba_cinnost_{$idZaznamu}_view_{$i}'
//                                                    data-ron-combo-url='?menuid=vyroba&action=seznamcinnosti'
//                                                    data-ron-combo-code='vyroba_cinnost_{$idRadku}'
//                                                    data-ron-combo-group='vyroba_cinnost'
//                                                    data-ron-combo-fields='{$comboFields}'
//                                                    data-ron-combo-text-field='{$comboTextField}'
//                                                    data-ron-combo-default-value='$vychoziCinnost'
//                                                    title='{$cinnost_txt}'
//                                                    class='vyroba_cinnost_t ron_select_combo form-select' value=''/>";

                                    echo "<input name='vyroba_cinnost_{$idZaznamu}_view' id='vyroba_cinnost_{$idZaznamu}_view_{$i}' 
                                                    data-max-visible-rows='30'
                                                    data-ronselect-class='form-control'
                                                    data-auto-load-items='1'
                                                    data-items-url='?menuid=vyroba&action=seznamcinnosti'
                                                    data-enable-url-cache='1'
                                                    data-columns='{$comboFields}'
                                                    data-value-column='kod'
                                                    data-text-column='{$comboTextField}'
                                                    data-code-input='vyroba_cinnost_{$idRadku}'
                                                    data-default-value='$vychoziCinnost'
                                                    title='{$cinnost_txt}'
                                                    class='vyroba_cinnost_t ronselect form-control vyroba_combo' value='$cinnost'/>";

                                    echo "<input type='hidden' name='vyroba_cinnost_{$idRadku}' value='$cinnost'>";
                                }
                                echo "</td>";
                                break;

                            case 'ZAKAZKA' :
                                echo "<td class='vyroba_zakazka {$sloupecEditace['class']} '>";

                                $zakazka_txt = TextManager::_('Vyroba_zakazka');
                                $zakazka = $nacistRadekPost ? StringUtils::htmlTextConvert($_POST["vyroba_zakazka_{$idRadku}"]) : ($zaznamVyroby->zakazka ?: $vychoziZakazka);

                                if (isset($sloupecEditace['zobrazitnaseptavac']) && ($sloupecEditace['zobrazitnaseptavac'] == 0)) {
                                    $class = $zaznamVyroby->zakazkaError ? 'error' : '';
                                    echo "<input type='text' name='vyroba_zakazka_{$idRadku}' value='$zakazka' class='$class'>";
                                } else {
                                    $comboFields = $sloupecEditace['combo_pole'] ?: 'kod;popis;datumprijeti';
                                    $comboTextField = $sloupecEditace['combo_pole_text'] ?: 'popis';

                                    $reload = '';
                                    if (isset($sloupecEditace['nacistvyrobkyzakazky']) && ($sloupecEditace['nacistvyrobkyzakazky'] == 1)) {
                                        $reload =  "data-reload-item='#vyroba_vyrobek_{$idZaznamu}_view_{$i}'";
                                    }

                                    /*echo "<input name='vyroba_zakazka_{$idZaznamu}_view' id='vyroba_zakazka_{$idZaznamu}_view_{$i}'
                                                    data-ron-combo-url='?menuid=vyroba&action=seznamzakazek'
                                                    data-ron-combo-code='vyroba_zakazka_{$idRadku}'
                                                    data-ron-combo-group='vyroba_zakazka'                                                    
                                                    data-ron-combo-fields='{$comboFields}'
                                                    data-ron-combo-text-field='{$comboTextField}'
                                                    data-ron-combo-default-value='$vychoziZakazka'
                                                    title='{$zakazka_txt}'
                                                    class='vyroba_zakazka_t ron_select_combo form-select' value=''/>";*/

                                    echo "<input name='vyroba_zakazka_{$idZaznamu}_view' id='vyroba_zakazka_{$idZaznamu}_view_{$i}' 
                                                    data-max-visible-rows='30'
                                                    data-ronselect-class='form-control'
                                                    data-auto-load-items='1'
                                                    data-items-url='?menuid=vyroba&action=seznamzakazek'
                                                    data-url-add='vyrobek'
                                                    $reload
                                                    data-enable-url-cache='1'
                                                    data-columns='{$comboFields}'
                                                    data-value-column='kod'
                                                    data-text-column='{$comboTextField}'
                                                    data-code-input='vyroba_zakazka_{$idRadku}'
                                                    data-default-value='$vychoziZakazka'
                                                    title='{$zakazka_txt}'
                                                    class='vyroba_zakazka_t ronselect form-control vyroba_combo' value='{$zakazka}'/>";

                                    echo "<input type='hidden' name='vyroba_zakazka_{$idRadku}' value='$zakazka'>";
                                }
                                echo "</td>";
                                break;

                            case 'MNOZSTVI' :
                                $pocet = $nacistRadekPost ? StringUtils::htmlTextConvert($_POST["vyroba_pocetks_{$idRadku}"]) : $zaznamVyroby->mnozstviDobre;

                                $readOnlyStr = '';
                                if (isset($sloupecEditace['lzeeditovat']) && ($sloupecEditace['lzeeditovat'] == 0))
                                    $readOnlyStr = 'readonly';

                                echo "<td class='vyroba_pocetks {$sloupecEditace['class']}'> 
                                        <input type='text' class='app_input inputFloat' size='10' name='vyroba_pocetks_{$idRadku}' value='$pocet' $readOnlyStr>
                                      </td>";
                                break;

                            case 'MNOZSTVISPATNE' :
                                $pocet = $nacistRadekPost ? StringUtils::htmlTextConvert($_POST["vyroba_pocetks_spatne_{$idRadku}"]) : $zaznamVyroby->mnozstviSpatne;

                                $readOnlyStr = '';
                                if (isset($sloupecEditace['lzeeditovat']) && ($sloupecEditace['lzeeditovat'] == 0))
                                    $readOnlyStr = 'readonly';

                                echo "<td class='vyroba_pocetks_spatne {$sloupecEditace['class']}'> 
                                        <input type='text' class='app_input inputFloat' size='10' name='vyroba_pocetks_spatne_{$idRadku}' value='$pocet' $readOnlyStr>
                                      </td>";
                                break;

                            case 'HODINY' :
                                $hodiny = $nacistRadekPost ? StringUtils::htmlTextConvert($_POST["vyroba_hodiny_{$idRadku}"]) : $zaznamVyroby->hodiny;
                                if (isset($sloupecEditace['vkladatcas']) && $sloupecEditace['vkladatcas'] == 0) {
                                    $classCas = '';
                                    $hodiny = StringUtils::format('number_2', $hodiny);
                                } else {
                                    $classCas = 'inputTime';
                                    $hodiny = StringUtils::format('hodiny', $hodiny);
                                }

                                $readOnlyStr = '';
                                if (isset($sloupecEditace['lzeeditovat']) && ($sloupecEditace['lzeeditovat'] == 0))
                                    $readOnlyStr = 'readonly';

                                echo "<td class='vyroba_hodiny {$sloupecEditace['class']}'>
                                    <input type='text' class='app_input $classCas' 
                                        data-ron-hodiny-casod='vyroba_casod_{$idRadku}' 
                                        data-ron-hodiny-datumod='vyroba_datumod_{$idRadku}'  
                                        data-ron-hodiny-casdo='vyroba_casdo_{$idRadku}' 
                                        data-ron-hodiny-datumdo='vyroba_datumdo_{$idRadku}' 
                                        data-ron-hodiny-hodiny='vyroba_hodiny_{$idRadku}' 
                                        name='vyroba_hodiny_{$idRadku}' value='$hodiny' $readOnlyStr>
                                </td>";
                                break;

                            case 'CASOD' :
                                $casOd = $nacistRadekPost ? $_POST["vyroba_casod_{$idRadku}"] : $zaznamVyroby->casOd;
                                $casOd = StringUtils::format('time', $casOd);
                                $datumOd = $nacistRadekPost ? $_POST["vyroba_datumod_{$idRadku}"] : $zaznamVyroby->casOd;
                                $datumOd = StringUtils::format('date', $datumOd);

                                $readOnlyStr = '';
                                if (isset($sloupecEditace['lzeeditovat']) && ($sloupecEditace['lzeeditovat'] == 0))
                                    $readOnlyStr = 'readonly';

                                $classStr = $zaznamVyroby->casOdError ? 'error' : '';

                                echo
                                "<td class='vyroba_casod {$sloupecEditace['class']}'>
                                    <input type='text' class='app_input inputTime ronInputHodiny $classStr' 
                                        data-ron-hodiny-casod='vyroba_casod_{$idRadku}' 
                                        data-ron-hodiny-datumod='vyroba_datumod_{$idRadku}'  
                                        data-ron-hodiny-casdo='vyroba_casdo_{$idRadku}' 
                                        data-ron-hodiny-datumdo='vyroba_datumdo_{$idRadku}' 
                                        data-ron-hodiny-hodiny='vyroba_hodiny_{$idRadku}' 
                                        size='5' maxlength='5'                                        
                                        name='vyroba_casod_{$idRadku}' value='$casOd' $readOnlyStr>";

                                if (isset($sloupecEditace['zobrazitdatum']) && ((int)$sloupecEditace['zobrazitdatum'] == 1))
                                    echo "<input type='text' class='app_input ronInputHodiny inputDate $classStr' 
                                            size='10' maxlength='10'
                                            data-ron-hodiny-casod='vyroba_casod_{$idRadku}' 
                                            data-ron-hodiny-datumod='vyroba_datumod_{$idRadku}'  
                                            data-ron-hodiny-casdo='vyroba_casdo_{$idRadku}' 
                                            data-ron-hodiny-datumdo='vyroba_datumdo_{$idRadku}' 
                                            data-ron-hodiny-hodiny='vyroba_hodiny_{$idRadku}'
                                            name='vyroba_datumod_{$idRadku}' value='$datumOd' $readOnlyStr>
                                          <i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='vyroba_datumod_{$idRadku}'></i>";
                                echo
                                "</td>";
                                break;

                            case 'CASDO' :
                                $casDo = $nacistRadekPost ? $_POST["vyroba_casdo_{$idRadku}"] : $zaznamVyroby->casDo;
                                $casDo = StringUtils::format('time', $casDo);
                                $datumDo = $nacistRadekPost ? $_POST["vyroba_datumdo_{$idRadku}"] : $zaznamVyroby->casDo;
                                $datumDo = StringUtils::format('date', $datumDo);

                                $readOnlyStr = '';
                                if (isset($sloupecEditace['lzeeditovat']) && ($sloupecEditace['lzeeditovat'] == 0))
                                    $readOnlyStr = 'readonly';

                                $classStr = $zaznamVyroby->casDoError ? 'error' : '';

                                echo "<td class='vyroba_casdo {$sloupecEditace['class']}'>
                                    <input type='text' class='app_input inputTime ronInputHodiny $classStr' 
                                        data-ron-hodiny-casod='vyroba_casod_{$idRadku}' 
                                        data-ron-hodiny-datumod='vyroba_datumod_{$idRadku}'  
                                        data-ron-hodiny-casdo='vyroba_casdo_{$idRadku}' 
                                        data-ron-hodiny-datumdo='vyroba_datumdo_{$idRadku}' 
                                        data-ron-hodiny-hodiny='vyroba_hodiny_{$idRadku}' 
                                        size='5' maxlength='5'
                                        name='vyroba_casdo_{$idRadku}' value='$casDo' $readOnlyStr>";

                                if (isset($sloupecEditace['zobrazitdatum']) && ((int)$sloupecEditace['zobrazitdatum'] == 1))
                                    echo "<input type='text' class='app_input ronInputHodiny inputDate $classStr' 
                                            size='10' maxlength='10'
                                            data-ron-hodiny-casod='vyroba_casod_{$idRadku}' 
                                            data-ron-hodiny-datumod='vyroba_datumod_{$idRadku}'  
                                            data-ron-hodiny-casdo='vyroba_casdo_{$idRadku}' 
                                            data-ron-hodiny-datumdo='vyroba_datumdo_{$idRadku}' 
                                            data-ron-hodiny-hodiny='vyroba_hodiny_{$idRadku}'
                                            name='vyroba_datumdo_{$idRadku}' value='$datumDo' $readOnlyStr>
                                          <i class='showcalendar inline-glyph glyph-button fa glyph-calendar' data-roncalendar-inputname='vyroba_datumdo_{$idRadku}'></i>";
                                echo
                                "</td>";
                                break;

                            case 'DV' :
                                echo "<td class='vyroba_dv {$sloupecEditace['class']}'>" .
                                        (isset($zaznamVyroby->denniVysledky[$sloupecEditace['kod']])  &&  ($zaznamVyroby->denniVysledky[$sloupecEditace['kod']] instanceof DenniVysledek)
                                            ? $zaznamVyroby->denniVysledky[$sloupecEditace['kod']]->vratText($sloupecEditace['jednotka'] ?: 'H')
                                            : '') .
                                     "</td>";
                                break;

                            case 'MV' :
                                echo "<td class='vyroba_mv {$sloupecEditace['class']}'>" .
                                        (($zaznamVyroby->mesicniVysledky[$sloupecEditace['kod']] instanceof MesicniVysledek)
                                            ? $zaznamVyroby->mesicniVysledky[$sloupecEditace['kod']]->vratText($sloupecEditace['jednotka'] ?: 'H')
                                            : '') .
                                    "</td>";
                                break;

                            case 'VYSLEDEKHODINY' :
                                echo "<td class='vyroba_vysledekhod {$sloupecEditace['class']}'>";
                                if (is_array($zaznamVyroby->vysledkyVyroby)) {
                                    switch ($sloupecEditace['typzobrazeni']) {
                                        case 1 :
                                            $suma = 0;
                                            foreach ($zaznamVyroby->vysledkyVyroby as $vysledekVyroby) {
                                                $suma += $vysledekVyroby->hodiny;
                                            }
                                            echo StringUtils::format('hodiny', $suma);
                                            break;
                                        case 2 :
                                            $first = true;
                                            foreach ($zaznamVyroby->vysledkyVyroby as $vysledekVyroby) {
                                                if (!$first) echo ', ';
                                                echo StringUtils::format('hodiny', $vysledekVyroby->hodiny);
                                                if ($first) $first = false;
                                            }
                                            break;
                                        case 0 :
                                        default :
                                            echo StringUtils::format('hodiny', reset($zaznamVyroby->vysledkyVyroby)->hodiny);

                                            break;
                                    }
                                }
                                echo "</td>";
                        }
                    }
                } ?>

            <?php
                echo "<td>";
                if ($i == $maxIndexRadku) {
                    echo "<input type='button' class='button smallbutton vyroba_add' name='vyroba_add' value='+'>";
                }
                echo "</td>";
                if ($zaznamVyroby->serialVyroba > 0) {
                    echo "<td>";
                    /** @noinspection HtmlUnknownTarget */
                    echo GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete"></i>', TextManager::_('Opravdu_zrusit_radek'),
                        '?menuid=vyroba&action=zrusitradek&record=' . $zaznamVyroby->serialVyroba);
                    echo "</td>";
                }
            ?>

        </tr>
<?php       }

            $lastPP = $zaznamVyroby ->idPP;
        } ?>

        </table>

        <input type="submit" name="zapsat" class="button submitbutton" value="<?= TextManager::_('Zapsat') ?>">
        <?php if (($nastaveniFormulare['podlepuvodni'] == 1) && ($den < $datumZpracovani)) { ?>
        <input type="submit" name="zpracovat" class="button submitbutton" value="<?= TextManager::_('Vyroba_podle_puvodni') ?>">
        <?php } ?>
    </form>
    </div>
<?php } ?>

