<?php
/**
 * pavel.rudol, 18.8.2022 13:32
 */

namespace RonPortal\Vyroba;

use Ron\Controls\BrowserField;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;

class VyrobniCinnostiBrowser extends RonPortalBrowser
{
    public function __construct($database)
    {
        parent::__construct($database, 'vyroba');
        $this->browserUrl = '?menuid=vyrobnicinnosti&';

        $this->title = TextManager::_('Vyroba_seznam_cinnosti');

        $this->code = 'browservyrobnicinnosti';

        $this->sqlFromPart =
            'VyrobniCinnost JOIN S_DruhZpracovaniCinnosti ON S_DruhZpracovaniCinnosti.Kod = VyrobniCinnost.DruhZpracovani ' .
            ' LEFT OUTER JOIN TypVyrobniCinnosti ON TypVyrobniCinnosti.Kod = VyrobniCinnost.Typ ' .
            ' LEFT OUTER JOIN StatusVyrobniCinnosti ON StatusVyrobniCinnosti.Kod = VyrobniCinnost.Status';

        $this->primaryField = 0;

        $this->maxPageRows = 100;

        $this->addField()
            ->setSqlFieldName('VyrobniCinnost.kod')->setHeaderText('Kod')
            ->setCode('col-kod')->setOrdering(true);
        $this->addField()
            ->setSqlFieldName('VyrobniCinnost.popis')->setHeaderText('Popis')->setLink('?menuid=vyrobnicinnosti&formaction=edit')
            ->setCode('col-popis')->setOrdering(true);
        $this->addField()
            ->setSqlFieldName('S_DruhZpracovaniCinnosti.Popis')->setHeaderText('vyroba_druh_zpracovani')
            ->setCode('col-druh-zpracovani')->setOrdering(true);
        $this->addField()
            ->setSqlFieldName('VyrobniCinnost.Aktivni')->setHeaderText('Vyroba_aktivni')
            ->setCode('col-aktivni')->setOrdering(true)->setFormat('anone');
        $this->addField()
            ->setSqlFieldName('TypVyrobniCinnosti.popis')->setHeaderText('vyroba_typ_cinnosti')
            ->setCode('col-typ-cinnosti')->setOrdering(true);
        $this->addField()
            ->setSqlFieldName('StatusVyrobniCinnosti.popis')->setHeaderText('vyroba_status_cinnosti')
            ->setCode('col-status-cinnosti')->setOrdering(true);
        $this->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(VyrobniCinnost.Poznamka)}')
            ->setHeaderColSpan(-1)
            ->setClassName('poznamka')
            ->setCode('col-poznamka');
    }

    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this->database, 'vyrobnicinnosti', true); //new CinnostiFilterForm($this->database, ['menuid' => 'vyrobnicinnosti'], true);

        // filtr druh zpracovani
        $filtrDruh = new FilterFormFilter('druhzpracovani', FilterFormFilter::TYPE_OPTIONS);
        $filtrDruh
            ->setCaption(TextManager::_('vyroba_druh_zpracovani'))
            ->setRowId('filterrow_druhzpracovani')
            ->setSessionName('filterform_druhzpracovani')
            ->setOptionsSql('SELECT Kod, Popis FROM S_DruhZpracovaniCinnosti WHERE ' .
                    ' %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('VyrobniCinnost.DruhZpracovani = ');
        $this ->filterForm ->addFilter($filtrDruh);

        // filtr typ cinnosti
        $filtrTyp = new FilterFormFilter('typcinnosti', FilterFormFilter::TYPE_OPTIONS);
        $filtrTyp
            ->setCaption(TextManager::_('Typ'))
            ->setRowId('filterrow_typcinnosti')
            ->setSessionName('filterform_typcinnosti')
            ->setOptionsSql('SELECT Kod, Popis FROM TypVyrobniCinnosti WHERE ' .
                    ' %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('VyrobniCinnost.Typ = ');
        $this ->filterForm ->addFilter($filtrTyp);

        // filtr status
        $filtrStatus = new FilterFormFilter('statuscinnosti', FilterFormFilter::TYPE_OPTIONS);
        $filtrStatus
            ->setCaption(TextManager::_('vyroba_status_cinnosti'))
            ->setRowId('filterrow_statuscinnosti')
            ->setSessionName('filterform_statuscinnosti')
            ->setOptionsSql('SELECT Kod, Popis FROM StatusVyrobniCinnosti WHERE ' .
                    ' %FILTER% ORDER BY %ORDER%',
                '1 = 1', 'Popis'
            )
            ->addItem('', TextManager::_('_vse_'))
            ->setSqlFilter('VyrobniCinnost.Status = ');
        $this ->filterForm ->addFilter($filtrStatus);

        $filtrAktivni = new FilterFormFilter('aktivni', FilterFormFilter::TYPE_CHECKBOX);
        $filtrAktivni
            ->setCaption(TextManager::_('Vyroba_jen_aktivni'))
            ->setRowId('filterrow_aktivni')
            ->setSessionName('filterform_aktivnicinnosti')
            ->setSqlFilter('VyrobniCinnost.Aktivni = 1');
        $this->filterForm->addFilter($filtrAktivni);
    }

    function renderFilter()
    {
        parent::renderFilter();

        echo "<a href='?menuid=vyrobnicinnosti&formaction=new'>" . TextManager::_('New_record') . "</a>";
    }

}