<?php
/**
 * stanislav.elbl
 * 22.05.2021 23:02
 */

namespace RonPortal\XML\Classes;
use RonPortal\Prava;
use RonPortal\XML\XMLFormElementCommand;

/**
 * Class XMLFormElementFiltrPodlePrav
 * Do promenne se zadanym jmenem (name) vlozi sql filtr pro odfiltrovani zaznamu podle prav uzivatele
 */
class XMLElementFiltrPodlePrav extends XMLFormElementCommand
{
    /** @var string */
    protected $name;
    /** @var string */
    protected $typPrava;
    /** @var string     Jmeno tabulky, ktera se ma v SQL vyrazu filtrovat */
    protected $tabulka;
    /** @var string     Vyraz pro nastaveni pocatku obdobi. */
    protected $obdobiOd;
    /** @var string     Vyraz pro nastaveni konce obdobi. */
    protected $obdobiDo;

    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->typPrava = $this ->getAttributeValue('typprava');
        $this ->tabulka = $this ->getAttributeValue('tabulka');
        $this ->obdobiOd = $this ->getAttributeValue('obdobiod');
        $this ->obdobiDo = $this ->getAttributeValue('obdobido');
    }


    function doCommand()
    {
        $tabulka = ($this ->tabulka) 
            ? $this ->nahradPromenne($this ->tabulka)
            : 'PracovniPomer';
        
        $typPrava = $this ->nahradPromenne($this ->typPrava);

        $obdobiOd = $this ->obdobiOd 
            ? (int)$this -> nahradPromenne($this ->obdobiOd) 
            : null;

        if (!$obdobiOd) {
            $obdobiOd = $this ->xmlForm ->pocatekObdobi ?: time();
        }

        $obdobiDo = $this ->obdobiDo 
            ? (int)$this -> nahradPromenne($this ->obdobiDo) 
            : null;

        if (!$obdobiDo) {
            $obdobiDo = $this ->xmlForm ->konecObdobi ?: time();
        }

        $prava = Prava::vratPrava($typPrava);
        $filtr = $prava ->sqlFiltr($tabulka, $obdobiOd, $obdobiDo);
        $this ->setVariableValue($this ->name, $filtr);
    }
}

/*
//DOC
PAGE filtrpodleprav
<pre>
PRE
  <filtrpodleprav name="jmeno_promenne" typprava="kod_typu_prava" tabulka="jmeno_sql_tabulky" obdobiod="datum_od" obdobido="datum_do" />
PREEND
</pre>

//END
*/