<?php

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\XML\XMLFormElement;

/**
 * Implementace tagu <inputtable> a <inputtablerow>
 * Class XMLElementInputTable
 */
class XMLElementInputTable extends XMLFormElement
{
    public function getHTMLOutput() {
        //
        switch ($this ->elementName) {
            case 'inputtable' :
                if ($this->getBoolAttribute('usetable')) {
                    // jen pokud by bylo potreba prepnout ve starem xml formulari na puvodni verzi s tabulkou. Pokud mozno nepouzivat
                    return
                        '<table class="inputtable">' .
                        $this->getElementsHTMLOutput() .
                        "</table>\n";
                }

                $attrs = [
                    'class' => $this ->getAttributeValue('class') ?: 'form large-form form-30-70'
                ];

                // doplni atributy oznacene prefixem html:
                $attrs = $this->setHTMLAttributes($attrs);
                $attrs_str = $this->getHTMLAttributes($attrs);

                return
                    "<div $attrs_str><div class='form-content'>\n" .
                    $this->getElementsHTMLOutput() .
                    "</div></div>\n";

            case 'inputtablerow' :
                if ($this ->parent ->getBoolAttribute('usetable')) {
                    return
                        '<tr>' .
                        '<td class="formtablecaption">' . StringUtils::fromUTF8($this->getAttributeValue('caption')) . '</td>' .
                        '<td>' . $this->getElementsHTMLOutput() . '</td>' .
                        "</tr>\n";
                }

                // doplni atributy oznacene prefixem html:
                $attrs = $this->setHTMLAttributes([
                    'class' => $this ->getAttributeValue('class') ?: 'form-row'
                ]);
                $attrs_str = $this->getHTMLAttributes($attrs);

                $captionClass = $this ->getAttributeValue('captionclass') ?: 'form-col1 caption';
                $dataClass = $this ->getAttributeValue('dataclass') ?: 'form-col2';

                return
                    "<div $attrs_str>" .
                    "<div class='$captionClass'>" . StringUtils::fromUTF8($this ->getAttributeValue('caption')) . '</div>' .
                    "<div class='$dataClass'>" . $this ->getElementsHTMLOutput() . '</div>' .
                    "</div>\n";

            default :
                return '';
        }
    }
}

/*
//DOC
PAGE inputtable
<pre>
PRE
   <inputtable>
     <inputtablerow caption="Nadpis radku">... obsah ...</inputtablerow>
   </inputtable>
PREEND
</pre>

//END
*/
