<?php

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\XML\XMLFormElementInput;

/**
 * implementace tagu <memo>
 * Class XMLFormElementMemo
 * @package RonPortal\XML\Classes
 */
class XMLElementMemo extends XMLFormElementInput
{
    protected $defaultValue;


    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        //$this ->f_defaultvalue = iconv('utf-8', 'windows-1250', $xmlReader ->value);
        $this->defaultValue = StringUtils::fromUTF8($this->loadXMLValue($xmlReader));
    }


    public function getHTMLOutput()
    {
        $name = $this->nahradPromenne($this->name);
        $editing = $this ->nahradPromenne($this->getAttributeValue('editing')); // yes/no/append/nolog

        if (($editing === 'no' ||  $editing === '0')  &&  $this ->xmlForm ->dataMode()  &&  $this ->variableValue($this ->name) !== null) {
            // datove XML, needitovatelne pole, uz vyplnene
            return '<div class="dotaznik_odpoved">' . StringUtils::longTextToHtml($this ->variableValue($this ->name), true) . '</div>';
        }

        $attrs = ['name' => $name];

        $rows = $this->getAttributeValue('rows');
        $attrs['rows'] = $rows ?: 8;

        $cols = $this->getAttributeValue('cols');
        $attrs['cols'] = $cols ?: 60;

        $attrs['wrap'] = 'virtual';

        if ($this->xmlForm->readOnly) {
            $attrs['readonly'] = 'readonly';
        }

        if ($this->xmlForm->usePostValues && isset($_POST[$name])) {
            $value = $_POST[$name];
        } else {
            $value = null;

            if ($this ->xmlForm ->dataMode()) {
                if ($editing === 'append') {
                    $value = $this ->defaultValue;
                } else {
                    $value = $this->variableValue($name);
                }
            }

            if (!isset($value)) {
                $value = $this->nahradPromenne($this->defaultValue);
            }
        }

        $this->setVariableValue($name, $value);

        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        return "<textarea $attrs_str>" . StringUtils::xmlTextConvert($value) . '</textarea>';
    }


    protected function getPostValue($name)
    {
        if (!isset($_POST[$name])) {
            return '';
        }

        // z formulare prijde jiny konec radku nez z xml v $defaultvalue!
        // musi se to sjednotit, aby potom fungovalo porovnavani
        return str_replace("\r\n", "\n", $_POST[$name]);
    }


    public function writeXMLOutput($xmlWriter)
    {
        $name = $this->nahradPromenne($this->name);
        $editing = $this ->nahradPromenne($this->getAttributeValue('editing')); // yes/no/append/nolog

        $value = $this ->getPostValue($name);
        $defaultvalue = $this->defaultValue; // iconv('utf-8', 'windows-1250', $this->defaultValue);

        $newDefaultValue = $defaultvalue;
        $showEdit = true;
        $logValue = true;

        if ($editing === 'append') {
            // pridava se dalsi text, vychozi hodnota je porad stejna jako v sablone
            if ($value == '') {
                $logValue = false;
            }
            else if (!strcmp($value, $defaultvalue)) {
                $logValue = false;
            }
            // neni co logovat (byla zadana prazdna nebo vychozi hodnota)

            $newDefaultValue = $defaultvalue; // zustava puvodni vychozi hodnota
        } else if ($editing === 'yes') {
            $logValue = (strcmp($value, $defaultvalue)); // loguje pouze v pripade zmeny
            $newDefaultValue = $value; // nova vychozi hodnota
        } else if ($editing === "nolog") {
            // editace je povolena a zadana hodnota se neloguje
            $logValue = false;
            $newDefaultValue = $value;
        } else {
            $showEdit = false;
            $logValue = false;
            // needitovatelne pole neni treba logovat
        }

        // nastavi hodnotu promenne
        $this->setVariableValue($name, $value);

        if ($logValue) {
            //$xmlWriter ->text(questform_getlogname(false));
            //$xmlWriter ->writeElement('posledniodpoved', str_replace("\n", "<BR/>", $value));
            $this->posledniOdpovedXML($xmlWriter, str_replace("\n", '<br/>', StringUtils::xmlTextConvert($value)), false);
        }

        if ($showEdit) {
            $xmlWriter->startElement(($this->prefix ? $this->prefix . ':' : '') . $this->elementName);
            $this->writeXMLAttributes($xmlWriter);
            $xmlWriter->text( StringUtils::toUTF8($newDefaultValue) );
            $xmlWriter->endElement();
        } else {
            $xmlWriter->startElement('longvariable');
            $xmlWriter->writeAttribute('name', $name);
            $xmlWriter->text($value);
            $xmlWriter->endElement();

            $xmlWriter->startElement('div');
            $xmlWriter->writeAttribute('class', 'dotaznik_odpoved');
            $xmlWriter->text(str_replace("\n", "<br/>", StringUtils::toUTF8($value) ));
            $xmlWriter->endElement();
        }
    }


    public function process()
    {
        $name = $this->nahradPromenne($this->name);
        $editing = $this ->nahradPromenne($this->getAttributeValue('editing')); // yes/no/append/nolog - vychozi je yes

        if ($editing !== 'append'  &&  $editing !== 'yes'  &&  $editing != '1'  &&  $editing !== 'nolog'  &&  $editing !== '') {
            if ($this ->variableValue($name) !== null) {
                // neni editovatelny a uz ma vyplnenou hodnotu
                return;
            }
        }

        $value = $this ->getPostValue($name);

        $oldValue = ($editing === 'append')
            ? $this ->defaultValue
            : $this ->variableValue($name);

        $this->setVariableValue($name, $value);
        if ($this ->dataVariable) {
            $this->xmlForm->addDataVariable($name);
        }

        if ($value != $oldValue) {
            // zaloguje zmenu
            $this ->xmlForm ->addLogData($name, $value);
        }
    }
}

/*
//DOC
PAGE memo
<pre>
PRE
  <memo name="jmeno" editing="yes" datavariable="1" cols="60" rows="8">vychozi obsah</memo>
PREEND
</pre>

//END
*/
