<?php
/**
 * stanislav.elbl, 09.06.2021 23:42
 */

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\Prava;
use RonPortal\User;
use RonPortal\XML\XMLFormElementCommand;

/**
 * Class XMLElementPrava
 * Zkontroluje prava uzivatele a vysledek vlozi do promenne se zadanym jmenem
 */
class XMLElementPrava extends XMLFormElementCommand
{
    /** @var string */
    var $typPrava;
    /** @var string */
    var $typPrava2;


    function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->typPrava = $this ->getAttributeValue('typprava');
        $this ->typPrava2 = $this ->getAttributeValue('typprava2');
    }


    function doCommand()
    {
        Prava::nactiPravaUzivatele( $this ->xmlForm ->database);
        Prava::nastavObdobiProPrava(time(), time());

        $prava = null;

        $typprava = $this ->nahradPromenne($this ->typPrava);

        if ( $typprava  &&  Prava::existujePravo($typprava) ) {
            $prava = Prava::vratPrava($typprava);
        } else {
            // typprava2 jen v pripade, ze vubec neexistuje typprava
            $typprava = $this ->nahradPromenne($this ->typPrava2);
            if ( $typprava  &&  Prava::existujePravo($typprava) ) {
                $prava = Prava::vratPrava($typprava);
            }
        }

        if ($prava) {
            if ( $idpp = $this ->nahradPromenne($this ->getAttributeValue('idpp')) ) {
                // kontrola prava na konkretni PP
                $result = $prava ->lzeZobrazitPP($idpp, time(), time());
            } else if ( $rc = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('rc'))) ) {
                // kontrola prava na osobu
                $result = $prava ->lzeZobrazitRC($rc, time(), time());
            } else {
                // jen nastavi hodnotu prava
                $result = $prava ->prava;
            }
        }
        else {
            $result = 0;
        }

        if ($name = $this ->getAttributeValue('name')) {
            $this ->setVariableValue($name, $result);
            if ($prava) {
                $this ->setVariableValue($name.'_priznak', $prava ->priznak);
                $this ->setVariableValue($name.'_priznak2', $prava ->priznak2);
                $this ->setVariableValue($name.'_priznak3', $prava ->priznak3);
            }
        }

        if (!$result) {
            if ($message = $this ->getAttributeValue('errormessage')) {
                $message = $this ->nahradPromenne(StringUtils::fromUTF8($message));
                $this ->xmlForm ->addErrorMsg($message);
            }

            if ( self::getBoolValue($this ->getAttributeValue('exitonerror')) ) {
                $this ->xmlForm ->exit = true;

//                $resultcode = $this ->getAttributeValue('errorresultcode');
//                if ($resultcode) {
//                    $this ->xmlForm ->f_XmlResultCode = $resultcode;
//                }
            }
        }
    }
}


/*
//DOC
PAGE prava
<pre>
PRE
  <prava name="jmeno_promenne" typprava="kod_typu_prava" typprava2="kod_typu_prava"
      idpp="id_pracovniho_pomeru" rc="rc_osoby"
      errormessage="text_chybove_hlasky" exitonerror="0"/>
PREEND
</pre>

//END
*/
