<?php
/**
 * stanislav.elbl, 09.10.2018
 * XML elementy <switch>, <case> a <default>
 */

namespace RonPortal\XML\Classes;

use RonPortal\XML\XMLFormElement;

/**
 * XML element switch - muze mit parametr variable nebo expression
 * Class XMLElementSwitch
 * @package RonPortal\XML\Classes
 */
class XMLElementSwitch extends XMLFormElement
{
    /**
     * @var XMLElementCase[]
     */
    public $cases = [];

    /**
     * @var XMLElementDefault[]
     */
    public $default = null;

    /**
     * @return null|XMLElementCase
     */
    private function findCase()
    {
        $variableName = $this ->getAttributeValue('variable');
        if ($variableName) {
            $value = $this ->variableValue($variableName);
        } else {
            $value = $this ->nahradPromenne($this ->getAttributeValue('expression'));
        }

        foreach ($this ->cases as $case) {
            $caseValue = $case ->getAttributeValue('value');
            if (isset($caseValue)) {
                $caseValue = $this ->nahradPromenne($caseValue);
                if ($value === $caseValue) {
                    return $case;
                }
            }
        }

        return $this ->default;
    }


    function getHTMLOutput()
    {
        $case = $this ->findCase();
        if (!$case) {
            return '';
        }

        return $case ->getElementsHTMLOutput();
    }


    function event()
    {
        $case = $this ->findCase();
        if ($case) {
            $case ->elementsEvent();
        }
    }


    function writeXMLOutput($xmlWriter)
    {
        throw new \Exception("Chyba: mod transform neni v tagu switch podporovan!");
    }


    function process()
    {
        $case = $this ->findCase();
        if ($case) {
            $case ->processElements();
        }
    }
}


/**
 * XML element case - musi mit parametr value
 * Class XMLFormElementCase
 * @package Ron\RonPortal\XML
 */
class XMLElementCase extends  XMLFormElement
{
    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        /** @var XMLElementSwitch $switch */
        $switch = $this ->findParentByName('switch');

        if ($switch) {
            $switch ->cases[] = $this;
        }
    }
}

/**
 * XML element default
 * Class XMLElementDefault
 * @package Ron\RonPortal\XML
 */
class XMLElementDefault extends  XMLFormElement
{
    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        /** @var XMLElementSwitch $switch */
        $switch = $this ->findParentByName('switch');

        if ($switch) {
            $switch ->default = $this;
        }
    }
}

/*
//DOC
PAGE switch
<pre>
PRE
  <switch variable="jmeno-promenne" expression="xml_vyraz">
      <case value="hodnota1">
         ...
      </case>

      <case value="hodnota2">
         ...
      </case>

      <default>
         ...
      </default>
  </switch>
PREEND
</pre>
//END
*/
