<?php
/**
 * stanislav.elbl, 12.10.2018
 */

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\NastaveniUzivatele;
use RonPortal\XML\XMLFormElementCommand;

/**
 * Umoznuje nacteni a zmenu uzivatelskeho nastaveni jakehokoliv uzivatele.
 * Atribut rc urcuje osobu - pokud neni zadan, tak pracuje s aktualni osobou
 * Class XMLFormElementUserSettings
 * @package Ron\RonPortal\XML
 */
class XMLElementUserSettings extends XMLFormElementCommand
{
    function doCommand()
    {
        $rc = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('rc')));

        if ($rc) {
            // pracuje s nastavenim zadaneho uzivatele
            $nastaveni = new NastaveniUzivatele();
            $nastaveni->load($this->xmlForm->database, $rc);
        } else {
            // pracuje s nastavenim aktualniho uzivatele
            $nastaveni = NastaveniUzivatele::loadUserSettings($this ->xmlForm ->database);
        }

        $key = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('key')));
        $variableName = $this ->nahradPromenne($this ->getAttributeValue('name'));

        $defaultValue = $this ->getAttributeValue('defaultvalue');
        if (isset($defaultValue)) {
            $defaultValue = $this ->nahradPromenne(StringUtils::fromUTF8($defaultValue));
        }


        if ($variableName) {
            // nastavi promennou
            if ($key) {
                // vysledna promenna je retezec - hodnota konkretniho nastaveni
                $this ->setVariableValue($variableName, $nastaveni ->getItem($key, $defaultValue));
            } else {
                // vysledna promenna je cele pole s nastavenim
                $this ->setVariableValue($variableName, $nastaveni);
            }
        }

        if ($key) {
            $newValue = $this->getAttributeValue($this ->elementName === 'uloznastaveniuzivatele' ? 'value' : 'newvalue');
            if (isset($newValue)) {
                $newValue = $this ->nahradPromenne(StringUtils::fromUTF8($newValue));

                // nastavi novou hodnotu zadane promenne
                if ($nastaveni->getItem('index') != $newValue) {
                    $nastaveni ->setItem($key, $newValue, $defaultValue);
                    $nastaveni ->save($this ->xmlForm ->database, $this ->getBoolAttribute('log'));
                }
            }
        }
    }
}


/*
//DOC
PAGE portal/usersettings
<pre>
PRE
  <usersettings key="setting_item_name" name="jmeno_promenne" rc="rc_osoby" defaultvalue="vychozi hodnota" newvalue="nova hodnota" log="0"/>
PREEND
</pre>
//END
*/
