<?php
/**
 * stanislav.elbl
 * 22.05.2021 14:21
 */

namespace RonPortal\XML\Classes;

use Ron\StringUtils;
use RonPortal\XML\XMLFormElementCommand;


class XMLElementVariable extends XMLFormElementCommand
{
    var $value;
    var $name;
    var $index;
    protected $format;

    function loadXML($xmlReader)
    {
        parent::loadXML($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->index = StringUtils::fromUTF8($this ->getAttributeValue('index'));
        $this ->format = StringUtils::fromUTF8($this ->getAttributeValue('format'));

        $val = $this ->getAttributeValue('value');
        if (isset($val)) {
            $this ->value = StringUtils::fromUTF8($val);
        } else {
            if (!$this ->isEmpty  ||  $this ->elementName != 'datavariable') {
                $this->value = $this->getTextContent(false);
            }
        }
    }


    function doCommand()
    {
        $name = $this ->nahradPromenne($this ->name);

        if ($this ->elementName == 'datavariable') {
            $this ->xmlForm ->addDataVariable($name);
            // prazdny tag <datavariable name=".." /> nenastavuje hodnotu, jen zajisti ulozeni promenne do dat
            if (!isset($this ->value)) {
                return '';
            }
        }

        $preserveParams = $this ->getBoolAttribute('preserveparams');
        $value = $preserveParams // self::getBoolValue($this ->getAttributeValue('preserveparams'))
            ? $this ->value
            : $this ->nahradPromenne($this ->value);

        if (self::getBoolValue($this ->getAttributeValue('evaluateparams'))) {
            // dodatecne vyhodnoti promenne v textu
            // slouzi pro zpracovani promenne, ktera byla drive nastavena s atributem preserveparams
            $value = $this ->nahradPromenne($value);
        }

        if (!$preserveParams  &&  $this ->format) {
            switch ($this ->format) {
                case 'int' :
                    $value = (int)$value;
                    break;

                case 'float' :
                case 'double' :
                    $value = (double)$value;
                    break;
            }
        }

        $index = (isset($this ->index)  &&  $this ->index !== '')
            ? $this ->nahradPromenne($this ->index)
            : null;

        if ($delimiter = $this ->getAttributeValue('delimiter')) {
            $value = explode($delimiter, $value, $this ->getAttributeValue('explodelimit', PHP_INT_MAX));
        }

        $this ->setVariableValue($name, $value, $index);

        return '';
    }


    function loadData()
    {
        // nacteni dat z datoveho XML
        $this ->doCommand();
        // prida do datovych promennych, aby se znovu ulozila
        $this ->xmlForm ->addDataVariable($this ->name);
    }
}

/*
//DOC
PAGE variable
<pre>
PRE
  <variable name="promenna1" index="1" value="Moje testovaci promenna" preserveparams="0" evaluateparams="0" delimiter=";" explodelimit="10"/>
PREEND
</pre>

PAGE datavariable
<pre>
PRE
  <datavariable name="promenna1" index="1" value="Moje testovaci promenna" preserveparams="0" evaluateparams="0" delimiter=";" explodelimit="10"/>
PREEND
</pre>

//END
*/
