<?php
/**
 * User: stanislav.elbl
 */

namespace RonPortal\XML\Classes;

use Ron\DateUtils;
use Ron\StringUtils;
use RonPortal\RonPortalSettings;
use RonPortal\User;
use RonPortal\XML\XMLFormElement;

/**
 * Implementace xml tagu <vyberrozdeleni>
 * Class XMLElementVyberRozdeleni
 */
class XMLElementVyberRozdeleni extends XMLFormElement
{
    /** @var string */
    protected $name;
    /** @var bool */
    protected $required;
    /** @var int */
    protected $typ;
    /** @var int  0-podle obdobi formulare, 1-aktualne platna, 2-vsechna */
    protected $obdobi;
    /** @var int  0-vsechny, 1-jen vlastni, 2-vlastni a podrizena, 3-podrizena, 4-jen primo podrizena */
    protected $pouzeVlastni;
    /** @var string */
    protected $sqlFilter;
    /** @var string */
    protected $value;


    protected function loadXMLElement($xmlReader)
    {
        parent::loadXMLElement($xmlReader);

        $this ->name = $this ->getAttributeValue('name');
        $this ->required = self::getBoolValue($this ->getAttributeValue('required'), true); // defaultne je true

        $this ->typ = $this ->getAttributeValue('typ');
        if (!$this ->typ) {
            $this ->typ = RonPortalSettings::getValue(RonPortalSettings::PREDAVANI_TYP_ROZDELENI);
        }

        $this ->obdobi = (int)$this ->getAttributeValue('obdobi');
        $this ->pouzeVlastni = (int)$this ->getAttributeValue('pouzevlastni');

        $this ->value = $this ->getAttributeValue('value');

        $this ->sqlFilter = StringUtils::fromUTF8($this ->loadXMLValue($xmlReader) );
    }


    function getHTMLOutput()
    {
        $name = $this ->nahradPromenne($this ->name);
        $typ = (int)$this->nahradPromenne($this->typ);

        if (!$this ->typ) {
            throw new \Exception("Chyba v tagu vyberrozdeleni ({$name}): neni zadan typ rozdeleni");
        }

        $params = $this ->xmlForm ->database ->createParameters();

        $obdobiod = time();
        $obdobido = DateUtils::endOfTheDay(time());

        if ($this ->obdobi == 1) ; // aktualne platna obdobi
        else if ($this ->obdobi == 2) {
            // rozdeleni nejsou filtrovana podle obdobi
            $obdobiod = 0;
            $obdobido = 0;
        }
        if (!$this ->obdobi) {
            // Zobrazi jen platna rozdeleni
            if ($this ->xmlForm->pocatekObdobi) {
                $obdobiod = $this ->xmlForm->pocatekObdobi;
                $obdobido = DateUtils::endOfTheDay($this ->xmlForm->konecObdobi);
            }
            // jinak zustane vychozi nastaveni - aktualni cas
        }

        // Poznamka: puvodne byl filtr na rozdeleni pres EXISTS, jenze tam nefungovaly parametry v ODBC!

        // TODO: pouzit RozdeleniUtils::sqlSeznamRozdeleniOsoby
        if ($this ->pouzeVlastni == 1) {
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis' .
                ' FROM Rozdeleni, RozdeleniOsob ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typ) .
                ' AND RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                ' AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod ' .
                ' AND RozdeleniOsob.RC = ' . $params ->addParameter(User::getRC()) .
                ' AND RozdeleniOsob.PlatnostOd <= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) .
                ' AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) . '  OR  RozdeleniOsob.PlatnostDo IS NULL)';
        } else if ($this ->pouzeVlastni == 2) {
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis' .
                ' FROM Rozdeleni, RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typ) .
                ' AND R_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                ' AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                ' AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                ' AND H.TypPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                ' AND H.KodPodrizene = Rozdeleni.Kod ' .
                ' AND R_Ja.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                ' AND R_Ja.PlatnostOd <= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) .
                ' AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) . '  OR  R_Ja.PlatnostDo IS NULL)';
        } else if ($this ->pouzeVlastni == 3) {
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis' .
                ' FROM Rozdeleni, RozdeleniOsob R_JA, HierarchieRozdeleni H ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typ) .
                ' AND R_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                ' AND H.TypNadrazene = R_Ja.KodTypuRozdeleni ' .
                ' AND H.KodNadrazene = R_Ja.Rozdeleni ' .
                ' AND H.TypPodrizene = Rozdeleni.KodTypuRozdeleni ' .
                ' AND H.KodPodrizene = Rozdeleni.Kod ' .
                ' AND R_Ja.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni' .
                ' AND R_Ja.PlatnostOd <= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) .
                ' AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                ' AND H.KodNadrazene <> H.KodPodrizene';
        } else if ($this ->pouzeVlastni == 4) {
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis' .
                ' FROM Rozdeleni, RozdeleniOsob R_JA' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typ) .
                ' AND R_JA.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
                ' AND R_Ja.RC = ' . $params ->addParameter(User::getRC()) .
                ' AND Rozdeleni.RozdeleniRodice = R_Ja.KodTypuRozdeleni ' .
                ' AND Rozdeleni.Rodic = R_Ja.Rozdeleni ' .
                ' AND R_Ja.PlatnostOd <= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) .
                ' AND (R_Ja.PlatnostDo >= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam(time())) . '  OR  R_Ja.PlatnostDo IS NULL)';
        } else {
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis' .
                ' FROM Rozdeleni ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params->addParameter($typ);
        }

        if ($obdobiod) {
            $sql .=
                ' AND (Rozdeleni.DatumZruseni >= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam($obdobiod)) .
                ' OR Rozdeleni.DatumZruseni IS NULL) ';
        }

        if ($obdobido) {
            $sql .=
                ' AND Rozdeleni.DatumVytvoreni <= ' . $params ->addParameter($this ->xmlForm ->database ->dateToParam($obdobido));
        }

        $sqlFilter = $this ->sqlFilter
            ? trim($this->nahradPromenneSQL($this ->sqlFilter))
            : null;

        if ($sqlFilter) {
            $sql .=
                ' AND ( ' . $sqlFilter . ')';
        }

        $sql .=
            ' ORDER BY Popis ';

        // hodnota predvyplnena nebo z POST
        if ($this ->xmlForm ->usePostValues) {
            // hodnota z POST
            $value = $_POST[$name];
        } else {
            // defaultni hodnota
            $value = null;

            if ($this ->xmlForm ->dataMode()) {
                // ... nebo hodnota zadana uz drive
                $value = $this ->variableValue($name);
            }

            if (!isset($value)) {
                $value = StringUtils::fromUTF8($this->value);
            }
        }

        $this ->xmlForm ->addDebugMessage('vyberrozdeleni, sql: ' . $sql, true);

        $cursor = $this ->xmlForm ->database ->getSqlResult($sql, $params);

        $attrs = ['name' => $name];
        $attrs = $this->setHTMLAttributes($attrs);
        if ($this ->xmlForm ->isError($name)) {
            if (isset($attrs['class'])) $attrs['class'] .= ' dotaznik_errinput';
            else $attrs['class'] = 'dotaznik_errinput';
        }

        $attrs_str = $this->getHTMLAttributes($attrs);

        $result = '<SELECT ' . $attrs_str . '>';

        if (!$value  ||  !$this ->required) { // pokud jeste nebyla zadana hodnota, prida prazdnou hodnotu  -  pridat prazdny radek, kdyz pole neni povinne? ? ?
            $selectedStr = $value ? '' : 'selected="selected"';
            $result .= "<OPTION value='' $selectedStr></OPTION> ";

            $this ->xmlForm ->nastavRozdeleni($name, null, null);
        }

        while ($row = $cursor ->fetch()) {
            $hlpVal = $row[0];

            if ($hlpVal == $value) {
                $selectedStr = 'selected="selected"';
                // nastavi prislusne promenne
                $this ->xmlForm ->nastavRozdeleni($name, $typ, $row[0]);
            } else {
                $selectedStr = '';
            }

            $result .= "<OPTION value='" . StringUtils::htmlTextConvert($row[0]) . "' $selectedStr>" . StringUtils::htmlTextConvert($row[1]) . "</OPTION>";
        }

        $cursor ->close();

        $result .= '</SELECT>';

        return $result;
    }


    function writeXMLOutput($xmlWriter)
    {
        throw new \Exception('XMLElementVyberRozdeleni::writeXMLOutput neni implementovano!');
        //parent::writeXMLOutput($xmlWriter);
    }


    function process()
    {
        $name = $this ->nahradPromenne($this ->name);
        $typ = (int)$this->nahradPromenne($this->typ);

        if ( $this ->required  &&  !$_POST[$name] ) {
            $this ->xmlForm -> addErrorMsg( \Ron\ronText('Chybi_povinna_hodnota'), $name );
        }

        $oldValue = $this ->xmlForm ->getVariableValue($name);
        if (!isset($oldValue)) {
            $oldValue = $this ->value;
        }

        $this ->xmlForm ->nastavRozdeleni($name, $typ, $_POST[$name]);

        if ($oldValue != $_POST[$name]) {
            $this ->xmlForm ->addLogData($name, $_POST[$name] . ' (' . $this ->variableValue($name . '_popis') . ')');
        }

        $this ->xmlForm ->addDataVariable($name);
        $this ->xmlForm ->addDataVariable($name . '_typ');
        $this ->xmlForm ->addDataVariable($name . '_kod');
        $this ->xmlForm ->addDataVariable($name . '_popis');
    }

}

/*
//DOC
PAGE vyberrozdeleni
<pre>
PRE
  <vyberrozdeleni name="ROZDELENI" typ="kod_typu_rozdeleni" obdobi="0" required="0" pouzevlastni="0" value="prednastavena_hodnota">SQL filtr</vyberrozdeleni>
PREEND
</pre>
//END
*/
