<?php
/**
 * stanislav.elbl, 24.05.2021 15:18
 */

namespace RonPortal\XML;

use Ron\Controls\BrowserField;
use Ron\DB\Cursor;
use Ron\DB\Database;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;


class XMLBrowserField extends BrowserField
{
    /** @var RonPortalXMLForm */
    var $xmlForm;
    /** @var string */
    var $xmlText;
    /** @var  string */
    var $xmlHTML;

    /**
     * @param RonPortalXMLForm $xmlForm
     */
    function __construct($xmlForm)
    {
        parent::__construct($xmlForm ->database);
        $this->xmlForm = $xmlForm;

        $this->xmlText = null;
        $this->xmlHTML = '';
    }


    function getLinkUrl($recordId, $rowData)
    {
        if ($this->link) {
            //return str_replace('&', '&#38;', $this->xmlForm->nahradPromenne($this->link, false));
            return $this->xmlForm->nahradPromenne($this->link, false);
        }

        return null;
    }


//    /**
//     * @param string $text
//     * @param string $recordId
//     * @return string
//     */
//    function getLink($text, $recordId, $rowData)
//    {
//        $result = '';
//
//        if ($this->link) {
//            $link = $this->xmlForm->nahradPromenne($this->link, false);
//
//            $link = str_replace('&', '&#38;', $link);
//
//            $result .= '<A ' . $this->xmlHTML . ' href="' . $link . '">';
//        }
//
//        $result .= $text;
//
//        if ($this->link) {
//            $result .= '</A>';
//        }
//
//        return $result;
//    }
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this->xmlText) {
            return $this ->getLink($this->xmlForm->nahradPromenne($this->xmlText, false), $recordId, $rowData);
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



class XMLBrowser
{
    /** @var Database */
    var $database;

    /** @var RonPortalXMLForm   odkaz na xml formular */
    var $xmlForm;
    /** @var string         jmeno pro vytvarene promenne */
    var $name;

    var $fixedCols;

    var $sql;
    /** @var XMLBrowserField[] */
    var $fields;

    /** @var string         css trida pro zobrazeni, defaultne "browser" */
    var $cssClass;
    var $columnCount;

    /** @var string         dodatecny text, ktery se zobrazi u kazdeho zaznamu */
    var $infoLineText;
    /** @var int            sloupec tabulky, na kterem zacina dodatecny text (odsazeni) */
    var $infoLineCol;

    /** @var string         odkaz pro kliknuti na radek (onclick) */
    var $rowClickLink;


    /**
     * @param Database $database
     * @param RonPortalXmlForm $xmlForm
     */
    function __construct($database, $xmlForm)
    {
        $this ->database = $database;
        $this ->xmlForm = $xmlForm;
        $this ->infoLineCol = 1;
    }



    function addField($field)
    {
        $this ->fields[] = $field;
    }


    function getTableHeader($sortFieldNumber)
    {
        if (!$this ->fields) return '';

        $result = '<tr>';

        $this ->columnCount = 0;

        foreach ($this ->fields as $i => $field) {
            if ($poms = $field ->getHeader($i, $sortFieldNumber)) {
                $result .= $poms;
                $this ->columnCount++;
            }
        }
        $result .= "</tr>\n";

        return $result;
    }



    function getTableStart()
    {
        $cssClass = $this ->cssClass ?: 'browser rowOver';

        $fixedCols = null;
        if ($this->fixedCols) {
            $fixedCols = "data-ron-fixed-cols='{$this->fixedCols}'";
            $cssClass .= " fixedCols";
        }

        return "<table class='$cssClass' $fixedCols>";
    }



    function createXMLVariables($values)
    {
        if ($this ->xmlForm ->verze > 1) {
            // od 02-2015 - promenne typu pole uz jsou podporovany primo ve vyrazech (plati pro verzi XML formulare 2)
            $this->xmlForm->variables[$this->name] = $values;
        } else {
            if ($this->xmlForm && $this->name) {
                foreach ($values as $i => $sqlvalue) {
                    $this->xmlForm->variables[$this->name . "[$i]"] = $values[$i];
                }
            }
        }
    }


    /**
     * Odkaz na proklik
     * @return string
     */
    function getClickLink()
    {
        if (!$this ->rowClickLink) return '';

        $link = $this ->xmlForm ->nahradPromenne($this ->rowClickLink, false);

        return $link;
    }


    /**
     * Zobrazeni jednoho radku
     * @param $values
     * @param $className
     * @param $rowNumber
     * @return string
     */
    function getLine($values, $className, $rowNumber)
    {
        if (!$this ->fields) {
            return '';
        }

        $link = $this ->getClickLink();
        //$link = str_replace('&', '&#38;', $link);

        $result =
            '<tr class="' . $className . ($link ? ' clickHref' : '') . '"' .
            ($link ? ' data-ron-href="' . $link . '"' : '') .
            '>';

        //$primaryindex = $this -> GetPrimaryFieldSQLIndex();
        //if ($primaryindex >= 0) $recordId = $values[$primaryindex];
        //else $recordId = '';
        $recordId = '';

        foreach ($this ->fields as $i => $field) {
            $value = (isset($field ->sqlFieldIndex)  &&  $field ->sqlFieldIndex) >= 0
                ? ($values[$field->sqlFieldIndex] ?? null)
                : null;

            $attrs = $field ->getAttributes($value, $rowNumber, $recordId, $values);

            switch ($field ->type) {
                case BrowserField::FIELD_TYPE_HIDDEN :
                case BrowserField::FIELD_TYPE_COMMENT :
                    break;

                case BrowserField::FIELD_TYPE_LINE :
                    $result .=
                        '<td ' . $attrs . '>' .
                        ($rowNumber > 0 ? $rowNumber : '') .
                        '</td>';
                    ;
                    break;

                default :
                    $result .=
                        '<td ' . $attrs . '>' .
                        $field ->getViewText($value, $rowNumber, $recordId, $values) .
                        '</td>';
            }
        }

        $result .= "</tr>\n";

        return $result;
    }


    /**
     * @param string $className
     * @return string
     */
    function getInfoLine($className)
    {
        if (!$this ->infoLineText) {
            return '';
        }

        $detailtext = $this ->xmlForm ->nahradPromenne($this ->infoLineText, false);
        if (!$detailtext) {
            return '';
        }

        $result =
            '<tr class="' . $className . '">';

        // prazdne sloupce na zacatku (odsazeni)
        for ($i = 0; $i < $this ->infoLineCol; $i++) {
            $result .= '<td></td>';
        }

        // spojeny sloupec
        $result .=
            '<td colspan="' . (count($this ->fields) - $this ->infoLineCol) . '">' .
            '<div class="poznamka browserpoznamka">' . StringUtils::longTextToHtml($detailtext, true) . '</div>' .
            '</td>';

        $result .=
            "</tr>\n";

        return $result;
    }


    /**
     * Vraci data pro zobrazeni ze zadane query.
     * @param Cursor $cursor
     * @return string
     */
    function getDBData($cursor)
    {
        $i = 1;

        $result = '';

        while ($row = $cursor ->fetch()) {
            $this ->createXMLVariables($row);

            $class = ($i % 2) ? 'browsercolor1' : 'browsercolor2';

            $result .= $this ->getLine($row, $class, $i);
            $result .= $this ->getInfoLine($class);

            $i++;
        }

        return $result;
    }


    /**
     * @return string
     */
    function getDBDataFromSQL()
    {
        MessageManager::addDebugLog('XMLBrowser::GetDBDataFromSQL: ' . $this ->sql);

        $cursor = $this ->database ->select($this ->sql);

        if ($cursor) {
            $result =
                $this ->getTableStart() .
                '<thead class="floating-table-header">' .
                $this ->getTableHeader(0) . // sortfieldnumber = 0 (prozatim ?)
                "</thead>\n" .
                "<tbody>\n" .
                $this ->getDBData($cursor) .
                "</tbody></table>\n";

            return $result;

        } else {
            MessageManager::addDbErrorMessage($this ->database, $this ->sql, 'Chyba v XMLBrowseru!');
            return '';
        }
    }


    /**
     * @return string
     */
    function getHTML()
    {
        return $this ->getDBDataFromSQL();
    }
}
