<?php

namespace RonPortal\Zamestnanci;

use Ron\DB\Database;


class AdresaOsoby
{
    public $rc;
    public $trvaleBydliste;
    public $kontaktniAdresa;
    public $platnostOd;
    public $platnostDo;
    public $ulice;
    public $mesto;
    public $obec;
    public $obecPopis;
    public $psc;
    public $stat;
    public $nazevStatu;
    public $telefon1;
    public $telefon2;
    public $mobil;
    public $email;
    public $zpusobDopravy;


    /**
     * @param Database $database
     * @param string $rc
     * @param int $platnost
     * @param int|null $trvale
     * @return AdresaOsoby[]
     */
    public static function nactiAdresuOsoby($database, $rc, $platnost = 0, $trvale = null)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT RC, TrvaleBydliste, PlatnostOd, PlatnostDo, Ulice, Mesto, Obec, PSC, Stat," . // 0 - 8
            "   Telefon1, Telefon2, Mobil, ZpusobDopravy, KontaktniAdresa, E_Mail, Stat.Popis, Obec.Popis" . // 9 - 16
            "  FROM AdresaOsoby LEFT OUTER JOIN Stat ON Stat.Kod = AdresaOsoby.Stat LEFT JOIN Obec ON (Obec.Kod = AdresaOsoby.Obec)".
            " WHERE RC = " . $params->addParameter($rc);

        if ($platnost > 0) {
            $sql .=
                ' AND PlatnostOd <= ' .  $params->addDateParameter($platnost) .
                ' AND (PlatnostDo >= ' . $params->addDateParameter($platnost) . ' OR PlatnostDo IS NULL) ';
        }

        if (!is_null($trvale)) {
            $sql .=
                ' AND TrvaleBydliste = ' . $params->addParameter($trvale);
        }

        $sql .=
            ' ORDER BY TrvaleBydliste DESC';

        $cursor = $database->getSqlResult($sql, $params);

        $seznamAdres = [];

        while ($row = $cursor->fetch()) {
            $adresa = new static();
            $adresa->rc = $rc;
            $adresa->nactiAdresuZRadku($database, $row);

            $seznamAdres[] = $adresa;
        }

        $cursor ->close();

        return $seznamAdres;
    }


    /**
     *
     * @param Database $database
     * @param array $row
     */
    private function nactiAdresuZRadku($database, $row)
    {
        $this->trvaleBydliste = (int)$row[1];
        $this->platnostOd = $database->stringToTimestamp($row[2]);
        $this->platnostDo = $database->stringToTimestamp($row[3]);
        $this->ulice = $row[4];
        $this->mesto = $row[5];
        $this->obec = $row[6];
        $this->psc = $row[7];
        $this->stat = $row[8];
        $this->telefon1 = $row[9];
        $this->telefon2 = $row[10];
        $this->mobil = $row[11];
        $this->zpusobDopravy = (int)$row[12];
        $this->kontaktniAdresa = (int)$row[13];
        $this->email = $row[14];
        $this->nazevStatu = $row[15];
        $this->obecPopis = $row[16];
    }
}
