<?php

namespace RonPortal\Zamestnanci;

use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SQLUtils;
use Ron\RonException;
use Ron\TextManager;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;
use Tracy\Debugger;


class NahlasenaZmenaUtils
{
    /**
     * Nastavi stav zadane zmeny
     * @param Database $database
     * @param int $idZmeny
     * @param string|object $logDescription
     * @param int $novyStav
     * @return void
     * @throws DBException
     */
    public static function aktualizujStavZmeny(Database $database, int $idZmeny, $logDescription, $novyStav = NahlasenaZmena::STAV_ZPRACOVANO)
    {
        $rec = new RonPortalDBRecord($database, 'NahlasenaZmena', $logDescription);
        $rec ->addField('Id', 'serial', true);
        $rec ->addStringField('TypZmeny', true);
        $rec ->addStringField('RC', true);
        $rec ->addTimestampField('DatumCas', true);
        $rec ->addIntField('Stav', false, $novyStav);

        ($rec ->addStringField('Zpracoval', false, 15, User::getRC()))
            ->setUpdateLog(true);
        ($rec ->addTimestampField('DatumCasZpracovani', false, time()))
            ->setUpdateLog(true);

        $params = $database ->createParameters();
        $rec ->setWhere('Id = ' . $params ->addParameter($idZmeny), $params);
        if ($rec ->loadRecord()) {
            $rec ->updateRecord();
        }
    }


    /**
     * @param Database $database
     * @param NahlasenaZmena $zmena
     * @param object|string $logDescription
     * @throws RonException
     * @throws DBException
     */
    protected static function aktualizaceOsobnichUdajuOsoby(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        // osobni
        // E:Titul:Titul_pred;E:TitulZa:Titul_za;E:Jmeno:Jmeno;E:Prijmeni:Prijmeni:Po;E:Email:E_mail;E:Telefon:Telefon;E:Mobil:Mobil;soubor:soubor:Pripojit_dokument
        $database ->beginTransaction();
        try {
            $rec = new RonPortalDBRecord($database, 'Osoba', $logDescription);
            $rec ->setXmlEvents('OSOBA_BEFORE_POST', 'OSOBA_AFTER_POST');
            $rec ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];
            $rec->logInfoMessage = 'Uprava osobnich udaju';

            $rec->addStringField('RC', true);

            $rec->addStringField('Jmeno', false, 30, trim($zmena ->hodnoty['Jmeno'] ?? ''));

            // prijmeni je povinne a v pripade zmeny se uklada do seznamu predchozich prijmeni
            $novePrijmeni = trim($zmena ->hodnoty['Prijmeni'] ?? '');
            if ($novePrijmeni === '') {
                throw new RonException(TextManager::_('form.Missing_value') . 'Prijmeni', 'Prijmeni');
            }
            $fPrijmeni = $rec->addStringField('Prijmeni', false, 30, $novePrijmeni);

            $rec->addStringField('Titul', false, 20, trim($zmena ->hodnoty['Titul'] ?? '') ?: null);
            $rec->addStringField('TitulZa', false, 20, trim($zmena ->hodnoty['TitulZa'] ?? '') ?: null);

            $rec->addStringField('eMail', false, 60, trim($zmena ->hodnoty['Email'] ?? '') ?: null);
            $rec->addStringField('Telefon', false, 60, trim($zmena ->hodnoty['Telefon'] ?? '') ?: null);
            $rec->addStringField('Mobil', false, 60, trim($zmena ->hodnoty['Mobil'] ?? '') ?: null);

            $rec ->addUpdateLogFields(); // Update_Timestamp, Update_UserName

            $params = $database->createParameters();
            $rec->setWhere('RC = ' . $params->addParameter($zmena ->rc), $params);

            if (!$rec ->loadRecord()) {
                throw new \Exception('Editovana osoba nebyla nalezena!');
            }

            // puvodni prijmeni si zapamatuje
            $predchoziPrijmeni = $fPrijmeni ->originalValue;

            $rec ->updateRecord();

            if ($novePrijmeni != $predchoziPrijmeni) {
                $date = DateUtils::today();

                // zmena prijmeni - zaznamena predchozi
                $rec2 = new RonPortalDBRecord($database, 'PredchoziPrijmeni', $logDescription);
                $rec2 ->logInfoMessage = 'Uprava osobnich udaju - zmena prijmeni';
                $rec2 ->addStringField('RC', true, 15, $zmena ->rc);
                $rec2 ->addDateField('PlatnostDo', true, $date);
                $rec2 ->addStringField('Prijmeni', true, 30, $predchoziPrijmeni);

                $params2 = $database ->createParameters();
                $rec2 ->setWhere(
                    'RC = ' . $params2 ->addParameter($zmena ->rc) .
                    ' AND PlatnostDo = ' . $params2 ->addDateParameter($date),
                    $params2);

                if (!$rec2 ->loadRecord()) {
                    // v pripade, ze zaznam uz existuje, nedela nic - posledni zadane prijmeni melo platnost az do dnesniho dne
                    // pravdepodobne se jedna o opravu
                    $rec2 ->insertRecord();
                }
            }

            self::aktualizujStavZmeny($database, $zmena ->id, $logDescription);

            $database ->commitTransaction();
        } catch (\Exception  $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }

    /**
     * @param Database $database
     * @param object|string $logDescription
     * @return RonPortalDBRecord
     */
    public static function getRecAdresaOsoby($database, $logDescription)
    {
        $rec = new RonPortalDBRecord($database, 'AdresaOsoby', $logDescription);
        $rec ->setXmlEvents('ADRESA_OSOBY_BEFORE_POST', 'ADRESA_OSOBY_AFTER_POST');

        $rec->addStringField('RC', true, 15);
        $rec->addIntField('TrvaleBydliste', true);
        $rec->addDateField('PlatnostOd', true);
        $rec->addDateField('PlatnostDo', true);
        $rec->addIntField('KontaktniAdresa', true);
        $rec->addStringField('Ulice', true, 60);
        $rec->addStringField('Mesto', true, 40);
        $rec->addStringField('Obec', true, 10);
        $rec->addStringField('PSC', true, 10);
        $rec->addStringField('Stat', true, 10);
        $rec->addStringField('Telefon1', true, 20);
        $rec->addStringField('Telefon2', true, 20);
        $rec->addStringField('Mobil', true, 20);
        $rec->addStringField('E_mail', true, 60);
        $rec->addIntField('ZpusobDopravy', true);
        $rec->addFloatField('VzdalenostDoZamestnani', true);

        return $rec;
    }

    /**
     * Zapise do databaze novou trvalou/prechodnou adresu.
     * @param Database $database
     * @param NahlasenaZmena $zmena
     * @param object|string $logDescription
     * @throws DBException
     */
    protected static function aktualizaceAdresyOsoby(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        // trvale/prechodne
        $trvale = $zmena ->typZmeny === 'trvale' ? 1 : 0;

        $database ->beginTransaction();

        try {
            $platnostOd = DateUtils::dateStringToTimestamp($zmena ->hodnoty['PlatnostOd']);

            // ... a vlozi novy zaznam
            $rec = self::getRecAdresaOsoby($database, $logDescription);
            $rec->logInfoMessage = 'Zapsani nove adresy';
            $rec ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];
            $rec['RC'] = $zmena ->rc;
            $rec['TrvaleBydliste'] = ($trvale);
            $rec['PlatnostOd'] = $platnostOd;
            $rec['PlatnostDo'] = ($zmena ->hodnoty['PlatnostDo'] ?? '') ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['PlatnostDo']) : null;
            $rec['KontaktniAdresa'] = (int)($zmena ->hodnoty['KontaktniAdresa'] ?? 0) ? 1 : 0;
            $rec['Ulice'] = $zmena ->hodnoty['Ulice'] ?? null;
            $rec['Mesto'] = $zmena ->hodnoty['Mesto'] ?? null;
            $rec['Obec'] = $zmena ->hodnoty['Obec'] ?? null;
            $rec['PSC'] = $zmena ->hodnoty['PSC'] ?? null;
            $rec['Stat'] = $zmena ->hodnoty['Stat'] ?? null;
            $rec['Telefon1'] = $zmena ->hodnoty['Telefon1'] ?? null;
            $rec['Telefon2'] = $zmena ->hodnoty['Telefon2'] ?? null;
            $rec['Mobil'] = $zmena ->hodnoty['Mobil'] ?? null;
            $rec['E_Mail'] = $zmena ->hodnoty['E_Mail'] ?? null;
            $rec['ZpusobDopravy'] = (int)($zmena ->hodnoty['DopravaDoZamestnani'] ?? 0);
            $rec['VzdalenostDoZamestnani'] = $zmena ->hodnoty['VzdalenostDoZamestnani'] ?? null;


            $params = $database ->createParameters();
            $rec ->setWhere(
                'RC = ' . $params ->addParameter($zmena ->rc) .
                ' AND TrvaleBydliste = ' . $params ->addParameter($trvale) .
                ' AND PlatnostOd = ' . $params ->addDateParameter($platnostOd),
                $params
            );

            if ($rec ->loadRecord()) {
                // zaznam uz existuje - provede jen update
                $rec ->updateRecord();
            } else {
                // nejdrive ukonci platnost predchoziho zaznamu ...
                $recPrev = self::getRecAdresaOsoby($database, $logDescription);
                $recPrev->logInfoMessage = 'Ukonceni platnosti predchozi adresy';
                $paramsPrev = $database ->createParameters();
                $recPrev ->setWhere(
                    'RC = ' . $paramsPrev ->addParameter($zmena ->rc) .
                    ' AND TrvaleBydliste = ' . $paramsPrev ->addParameter($trvale) .
                    ' AND PlatnostDo IS NULL',
                    $paramsPrev
                );

                if ($recPrev ->loadRecord()) {
                    // zkontroluje, jestli neni platnost noveho zaznamu starsi nez platnost od existujiciho zaznamu
                    if ($recPrev['PlatnostOd'] > $platnostOd) {
                        throw new \Exception(\Ron\ronText('platnost_od_mala'));
                    }
                    // u predchoziho zaznamu nastavi PlatnostDo na predchazejici den
                    $recPrev['PlatnostDo'] = DateUtils::incDayStart($platnostOd, -1);
                    $recPrev ->updateRecord();
                }

                // ... a potom vlozi novy zaznam
                $rec ->insertRecord();
            }

            self::aktualizujStavZmeny($database, $zmena ->id, $logDescription);

            $database ->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }


    /**
     * Zapise do databaze novy prukaz osoby nebo aktualizuje existujici
     * @param Database $database
     * @param NahlasenaZmena $zmena
     * @param object|string $logDescription
     * @throws DBException
     */
    protected static function aktualizacePrukazuOsoby(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        $database ->beginTransaction();

        try {
            $rec = new RonPortalDBRecord($database, 'Prukaz', $logDescription);
            $rec ->setXmlEvents('PRUKAZ_BEFORE_POST', 'PRUKAZ_AFTER_POST');
            $rec ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];
            $rec->addStringField('RC', true, 15, $zmena ->rc);
            $rec->addIntField('TypPrukazu', false, $zmena ->hodnoty['TypPrukazu']);
            $rec->addStringField('CisloPrukazu', false, 20, $zmena ->hodnoty['CisloPrukazu']);
            $rec->addStringField('VydalStat', false, 10, trim($zmena ->hodnoty['VydalStat'] ?? '') ?: null);

            $platnostOd = $zmena ->hodnoty['PlatnostOd']
                ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['PlatnostOd'])
                : DateUtils::today();

            $platnostDo = $zmena ->hodnoty['PlatnostDo']
                ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['PlatnostDo'])
                : null;

            $rec->addDateField('PlatnostOd', false, $platnostOd);
            $rec->addDateField('PlatnostDo', false, $platnostDo);

            $rec->addMemoField('Poznamka', false, $zmena ->hodnoty['vlastnipoznamka'] ?? null);

            $params = $database->createParameters();
            $rec->setWhere(
                'RC = ' . $params->addParameter($zmena ->rc) .
                ' AND TypPrukazu  = ' . $params->addParameter((int)$zmena ->hodnoty['TypPrukazu']) .
                ' AND CisloPrukazu  = ' . $params->addParameter($zmena ->hodnoty['CisloPrukazu']),
                $params);

            $rec ->updateOrInsert();

            self::aktualizujStavZmeny($database, $zmena ->id, $logDescription);

            $database ->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }


    /**
     * Zapise do databaze rodinneho prislusnika (update nebo insert)
     * @param Database $database
     * @param NahlasenaZmena $zmena
     * @param object|string $logDescription
     * @throws \Exception
     */
    protected static function aktualizaceRodinnychPrislusniku(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        // pri editaci musi byt zadano id (RC_RodinnehoPrislusnika) existujici osoby
        if ($zmena ->typZmeny === 'rodinnyprislusnik') {
            if (!$zmena ->hodnoty['ID']) {
                throw new \Exception('Chyba pri editaci rodinneho prislusnika - neni zadano ID osoby');
            }
        }

        $database ->beginTransaction();
        try {
            $novyZaznam = false;
            if ($zmena ->typZmeny === 'novyclenrodiny') {
                // zkusi najit existujiciho clena rodiny (muze byt prirazen jinemu zamestnanci)
                $idRC = RodinnyPrislusnik::najdiIDRodinnehoPrislusnikaDleRC($database, $zmena->hodnoty['RC_RodinnehoPrislusnika']);

                if (!$idRC) {
                    // vygeneruje nove ID
                    $idRC = SQLUtils::getNextRecordSerialCode($database,
                        'RodinnyPrislusnik', 'RC_RodinnehoPrislusnika',
                        "ID######", 1, 999999);
                    $novyZaznam = true;
                }
            } else {
                // pri update uz musi byt zadane
                $idRC = $zmena ->hodnoty['ID'];
            }

            // insert/update editovane osoby v tabulce RodinnyPrislusnik
            $rec = new RonPortalDBRecord($database, 'RodinnyPrislusnik', $logDescription);
            $rec ->setXmlEvents('RODINNY_PRISLUSNIK_BEFORE_POST', 'RODINNY_PRISLUSNIK_AFTER_POST');
            $rec ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];
            $rec ->logInfoMessage = $novyZaznam
                ? 'Prirazeni noveho rodinneho prislusnika'
                : 'Aktualizace rodinneho prislusnika';
            $rec->addStringField('RC_RodinnehoPrislusnika', true, 15, $idRC);  // toto je ID pole, nikoliv skutecne RC
            $rec->addStringField('Prijmeni', false, 30, trim($zmena ->hodnoty['Prijmeni']));
            $rec->addStringField('Jmeno', false, 30, trim($zmena ->hodnoty['Jmeno']));
            $rec->addStringField('RodneCisloRP', false, 15, trim($zmena->hodnoty['RC_RodinnehoPrislusnika'] ?? '') ?: null);
            $rec->addStringField('Titul', false, 20, trim($zmena ->hodnoty['Titul'] ?? '') ?: null);
            $rec->addIntField('ZdravotniStav', false, (int)($zmena ->nacteneHodnoty['ZdravotniStav'] ?? 0));
            $rec->addIntField('DrzitelPrukazuZTTP', false, (int)($zmena ->hodnoty['DrzitelPrukazuZTTP'] ?? 0) ? 1 : 0);

            //datum ZTP/P pouze pokud je Drzitel ZTP/P roven 1
            $rec->addDateField('DatumPriznaniZTTP', false,
                ($rec['DrzitelPrukazuZTTP'] && $zmena ->hodnoty['DatumPriznaniZTTP'])
                    ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['DatumPriznaniZTTP'])
                    : null);

            $rec->addDateField('DatumNarozeni', false,
                ($zmena ->hodnoty['DatumNarozeni'] ?? null) ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['DatumNarozeni']) : null);

            $rec->addIntField('KontaktniOsoba', false, (int)($zmena ->hodnoty['KontaktniOsoba'] ?? 0) ? 1 : 0);
            $rec->addStringField('Telefon', false, 60, trim($zmena ->hodnoty['Telefon'] ?? '') ?: null);
            $rec->addStringField('Email', false, 60, trim($zmena ->hodnoty['Email'] ?? '') ?: null);

            if ($novyZaznam) {
                $rec ->insertRecord();
            } else {
                $params = $database->createParameters();
                $rec->setWhere('RC_RodinnehoPrislusnika  = ' . $params->addParameter($idRC), $params);
                if (!$rec ->loadRecord()) {
                    throw new \Exception('Rodinny prislusnik se zadanym ID nebyl nalezen!');
                }
                $rec ->updateRecord();
            }

            // zapsat nebo aktualizovat noveho clena rodiny
            $rec2 = new RonPortalDBRecord($database, 'ClenoveRodiny', $logDescription);
            $rec2->logInfoMessage = 'Vlozeni noveho clena rodiny';

            $rec2->addStringField('RC', true, 15, $zmena ->rc);
            $rec2->addStringField('RC_RodinnehoPrislusnika', true, 15, $idRC);

            $rec2->addIntField('RodinnyVztah', false, (int)$zmena->hodnoty['RodinnyVztah']);

            $rec2->addDateField('PartnerDatumOd', false,
                ($zmena ->hodnoty['PartnerDatumOd'] ?? null) ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['PartnerDatumOd']) : null);
            $rec2->addDateField('PartnerDatumDo', false,
                ($zmena ->hodnoty['PartnerDatumDo'] ?? null) ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['PartnerDatumDo']) : null);

            $rec2->addIntField('StejnaAdresa', false, (int)($zmena ->hodnoty['StejnaAdresa'] ?? 0) ? 1 : 0);
            $rec2->addIntField('VyzivovanaOsoba', false, (int)($zmena ->hodnoty['VyzivovanaOsoba'] ?? 0) ? 1 : 0);

            $params2 = $database->createParameters();
            $rec2->setWhere(
                'RC = ' . $params2->addParameter($zmena ->rc) .
                ' AND RC_RodinnehoPrislusnika  = ' . $params2->addParameter($idRC),
                $params2);

            $rec2 ->updateOrInsert();

            self::aktualizujStavZmeny($database, $zmena ->id, $logDescription);

            $database ->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }


    /**
     * Zapise do databaze novy rodinny vztah. Predchozi ukonci.
     * @param Database $database
     * @throws \Exception
     * @throws DBException
     */
    protected static function aktualizaceRodinnehoStavuOsoby(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        $platnostOd = DateUtils::dateStringToTimestamp($zmena ->hodnoty['PlatnostOd']);

        $database ->beginTransaction();
        try {
            $rec = new RonPortalDBRecord($database, 'RodinnyStav', $logDescription);

            $rec ->setXmlEvents('RODINNY_STAV_BEFORE_POST', 'RODINNY_STAV_AFTER_POST');
            $rec ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];

            $rec->addStringField('RC', true, 15, $zmena ->rc);
            $rec->addDateField('PlatnostOd', true, $platnostOd);
            $rec->addIntField('KodRodinnehoStavu', false, (int)$zmena ->hodnoty['RodinnyStav']);

            //$rec->addDateField('PlatnostDo', false, $platnostDo);
            //$rec->logInfoMessage = 'Ukonceni platnosti puvodniho rodinneho stavu';
            //$platnostDo = DateUtils::incDay($this ->nacteneHodnoty['PlatnostOd'], -1);


            $params = $database->createParameters();
            $rec->setWhere(
                'RC = ' . $params->addParameter($zmena ->rc) .
                ' AND PlatnostOd = ' . $params ->addDateParameter($platnostOd),
                $params
            );

            if ($rec ->loadRecord()) {
                // zaznam uz existuje - provede jen update
                $rec ->updateRecord();
            } else {
                // zaznam neexistuje - nejdrive ukonci platnost predchoziho zaznamu ...
                $recPrev = new RonPortalDBRecord($database, 'RodinnyStav', $logDescription);
                $recPrev->addStringField('RC', true);
                $recPrev->addDateField('PlatnostOd', true);
                $recPrev->addDateField('PlatnostDo', false, DateUtils::incDay($platnostOd, -1));
                $recPrev->addIntField('KodRodinnehoStavu', true);

                $paramsPrev = $database ->createParameters();
                $recPrev ->setWhere(
                    'RC = ' . $paramsPrev->addParameter($zmena ->rc) .
                    ' AND PlatnostDo IS NULL',
                    $paramsPrev
                );

                if ($recPrev ->loadRecord()) {
                    $recPrev ->updateRecord();
                }

                // ... a vlozi novy zaznam
                $rec ->insertRecord();
            }

            self::aktualizujStavZmeny($database, $zmena ->id, $logDescription);

            $database ->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }


    private static function najdiPracovniPomer($database, $rc)
    {
        // najde aktivni pracovni pomer - prednostne HPP
        $pp = PracovniPomer::najdiPPOsoby($database, $rc, true, time());

        if (!$pp) {
            throw new \Exception(TextManager::_('Nenalezen_platny_pracovni_pomer_pro_zadanou_osobu'));
        }

        return $pp;
    }


    /**
     * @param Database $database
     * @param NahlasenaZmena $zmena
     * @param $logDescription
     * @throws DBException
     */
    protected static function aktualizaceUctuOsoby(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        // ucetosoby
        // E:JmenoUctu:Jmeno_uctu;S_TypBankovnihoUctu:TypUctu:Typ_uctu;E:CisloUctu:Cislo_uctu;Banka:Banka:Banka;
        // C:ZahranicniPlatba:Zahranicni_platba;IBAN:IBAN:IBAN;Mena:KodMeny_ISO:Kod_meny;E:SpecifickySymbol:specificky_symbol;
        // KonstantniSymbol:KonstantniSymbol:konstantni_symbol;E:VariabilniSymbol:Variabilni_symbol;
        // D:PlatnostOd:Platnost_od;D:PlatnostDo:Platnost_do;soubor:soubor:Pripojit_dokument

        // E:CisloUctu:Cislo_uctu;Banka:Banka:Banka;E:VariabilniSymbol:Variabilni_symbol;D:PlatnostOd:Platnost_od;soubor:soubor:Pripojit_dokument

        // najde platny pracovni pomer osoby - prednostne HPP
        $pp = self::najdiPracovniPomer($database, $zmena ->rc);

        $platnostOd = DateUtils::firstDayOfMonth(DateUtils::dateStringToTimestamp($zmena ->hodnoty['PlatnostOd']));

        if ($platnostOd < time()) {
            throw new \Exception(\Ron\ronText('Zmena_bankovniho_uctu_nelze_zpetne'));
        }

        $database ->beginTransaction();
        try {
            // zjisti PlatnostOd posledniho zaznam prirazeni bankovniho uctu (VyplaceniMzdy)
            $params = $database ->createParameters();
            $sql =
                'SELECT MAX(PlatnostOd) FROM VyplaceniMzdy' .
                ' WHERE RC = ' . $params ->addParameter($pp ->rc) .
                ' AND OsobniCislo = ' . $params ->addParameter($pp ->osobniCislo) .
                ' AND DatumNastupu = ' . $params ->addDateParameter($pp ->datumNastupu);
            $cursor = $database ->getSqlResult($sql, $params);
            $row = $cursor ->fetch();
            $cursor ->close();

            $platnostOdPoslednihoZaznamu = $database ->stringToTimestamp($row[0] ?? null);

            if ($platnostOdPoslednihoZaznamu  &&  $platnostOdPoslednihoZaznamu > $platnostOd) {
                // Uz je zadan nejaky budouci ucet => pripadna aktualizace by byla slozita. Napriklad by se musela aktualizovat platnostDo dalsiho predchazejiciho uctu
                throw new \Exception(TextManager::_('Platnost_od_uctu_musi_byt_vetsi_nez_platnost_predchoziho_uctu'));
            }

            // pokud je zadana PlatnostOd v budoucnu a je rovna PlatnostiOd posledniho zaznamu, tak provede pouze editaci existujicich zaznamu
            $puvodniBankovniUcet = null;    // ucet, ze ktereho nacte nektere hodnoty, nebo ktery se primo updatuje
            $jenUpdateZaznamu = false;      // priznak, ze se ma provest jen update zaznamu VyplaceniMzdy a pripadne BankovniUcet

            if ($platnostOdPoslednihoZaznamu  &&  ($platnostOdPoslednihoZaznamu < time()  ||  $platnostOdPoslednihoZaznamu !== $platnostOd)) {
                // stary zaznam je potreba ukoncit
                $recVyplaceniOld = new RonPortalDBRecord($database, 'VyplaceniMzdy', $logDescription);
                $recVyplaceniOld->addStringField('RC', true);
                $recVyplaceniOld->addStringField('OsobniCislo', true);
                $recVyplaceniOld->addDateField('DatumNastupu', true);
                $recVyplaceniOld->addIntField('SerioveCisloUctuVyplaty', true);
                $recVyplaceniOld->addDateField('PlatnostOd', true);
                $recVyplaceniOld->addDateField('PlatnostDo', true);

                $paramsVyplaceniOld = $database->createParameters();
                $recVyplaceniOld->setWhere(
                    'RC = ' . $paramsVyplaceniOld->addParameter($pp->rc) .
                    ' AND OsobniCislo = ' . $paramsVyplaceniOld->addParameter($pp->osobniCislo) .
                    ' AND DatumNastupu = ' . $paramsVyplaceniOld->addParameter($database->dateToParam($pp->datumNastupu)) .
                    ' AND PlatnostOd = ' . $paramsVyplaceniOld->addDateParameter($platnostOdPoslednihoZaznamu),
                    $paramsVyplaceniOld
                );

                if ($recVyplaceniOld->loadRecord()) {
                    $puvodniBankovniUcet = $recVyplaceniOld['SerioveCisloUctuVyplaty'];
                    $platnostDo = DateUtils::incDayStart($platnostOd, -1);

                    if (!$recVyplaceniOld['PlatnostDo'] || $recVyplaceniOld['PlatnostDo'] > $platnostDo) {
                        // ukonci predchozi zaznam - ale jen v pripade, ze uz neni ukoncen s PlatnostiDo mensi, nez je zadana
                        $recVyplaceniOld['PlatnostDo'] = $platnostDo;
                        $recVyplaceniOld->updateRecord();
                    }
                }
            }

            $recVyplaceni2 = new RonPortalDBRecord($database, 'VyplaceniMzdy', $logDescription);
            $recVyplaceni2 ->setXmlEvents('VYPLACENI_MZDY_BEFORE_POST', 'VYPLACENI_MZDY_AFTER_POST');
            $recVyplaceni2 ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];
            $recVyplaceni2->addStringField('RC', true,15, $pp ->rc);
            $recVyplaceni2->addStringField('OsobniCislo', true, 15, $pp ->osobniCislo);
            $recVyplaceni2->addDateField('DatumNastupu', true, $pp ->datumNastupu);
            $recVyplaceni2->addIntField('SerioveCisloUctuVyplaty', true);
            $recVyplaceni2->addDateField('PlatnostOd', true, $platnostOd);
            $recVyplaceni2->addDateField('PlatnostDo', true);

            if ($platnostOdPoslednihoZaznamu  &&  $platnostOdPoslednihoZaznamu === $platnostOd  &&  $platnostOd > time()) {
                // jedna se o existujici budouci zaznam - je mozno editovat
                $paramsVyplaceni2 = $database->createParameters();
                $recVyplaceni2->setWhere(
                    'RC = ' . $paramsVyplaceni2->addParameter($pp->rc) .
                    ' AND OsobniCislo = ' . $paramsVyplaceni2->addParameter($pp->osobniCislo) .
                    ' AND DatumNastupu = ' . $paramsVyplaceni2->addParameter($database->dateToParam($pp->datumNastupu)) .
                    ' AND PlatnostOd = ' . $paramsVyplaceni2->addDateParameter($platnostOdPoslednihoZaznamu),
                    $paramsVyplaceni2
                );

                if (!$recVyplaceni2 ->loadRecord()) {
                    // nenasel zaznam - nemelo by nastat, protoze zaznam byl nacten predchozim sql dotazem
                    throw new \Exception('Chyba pri aktualizaci predchoziho zaznamu VyplaceniMzdy - zaznam nebyl nalezen!');
                }

                $puvodniBankovniUcet = $recVyplaceni2['SerioveCisloUctuVyplaty']; // POZOR! Muze byt null (vyplaceni bylo nastaveno jinak nez na ucet)
                $jenUpdateZaznamu = true;
            }

            // vytvori a naplni (novy) bankovni ucet
            $recUcet = new RonPortalDBRecord($database, 'BankovniUcet', $logDescription);
            $recUcet ->setXmlEvents('BANKOVNI_UCET_BEFORE_POST', 'BANKOVNI_UCET_AFTER_POST');
            $recUcet ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];

            $jmenoUctu = str_replace(
                ['%JMENO%', '%PRIJMENI%', '%TITUL%', '%OSOBNICISLO%', '%CISLOUCTU%'],
                [$pp ->jmeno, $pp ->prijmeni, $pp ->titul, $pp ->osobniCislo, $zmena ->hodnoty['CisloUctu']],
                TextManager::_('Maska_jmena_uctu_vyplaty')
            );

            $recUcet->addField('SerioveCislo', 'serial', true);
            $recUcet->addStringField('JmenoUctu', true, 40, $jmenoUctu);

            $recUcet->addStringField('CisloUctu', false, 20, $zmena ->hodnoty['CisloUctu']);
            $recUcet->addStringField('KodBanky', false, 10, $zmena ->hodnoty['Banka']);
            $recUcet->addIntField('TypUctu', true, 1); // ucet zamestnance
            $recUcet->addStringField('RC', true, 15, $pp->rc);

            $recUcet->addStringField('SpecifickySymbol', true, 20); // pouzije se hodnota z predchoziho uctu
            $recUcet->addStringField('KonstantniSymbol', true, 10); //  -//-
            $recUcet->addStringField('VariabilniSymbol', false, 20, $zmena ->hodnoty['VariabilniSymbol']);

            $recUcet->addIntField('ZahranicniPlatba', !isset($zmena ->hodnoty['ZahranicniPlatba']), (int)($zmena ->hodnoty['ZahranicniPlatba'] ?? 0));
            $recUcet->addStringField('IBAN', !isset($zmena ->hodnoty['IBAN']), 40, $zmena ->hodnoty['IBAN'] ?? null);
            $recUcet->addStringField('KodMeny_ISO', !isset($zmena ->hodnoty['KodMeny_ISO']), 10, ($zmena ->hodnoty['KodMeny_ISO'] ?? null) ?: null);
            $recUcet->addStringField('PlatebniTitul', !isset($zmena ->hodnoty['PlatebniTitul']), 10, $zmena ->hodnoty['PlatebniTitul'] ?? null);

            $recUcet->addInsertLogFields();

            if ($jenUpdateZaznamu  &&  $puvodniBankovniUcet) {
                $recUcet ->addUpdateLogFields();
            }

            if ($puvodniBankovniUcet) {
                // Nacte puvodni bankovni ucet - nacita ho i v pripade, ze bude vkladat novy zaznam,
                // protoze se z nej nacitaji nektere hodnoty (SpecifickySymbol, KonstantniSymbol)
                $paramsUcet = $database ->createParameters();
                $recUcet ->setWhere(
                    'SerioveCislo = ' . $paramsUcet ->addParameter($puvodniBankovniUcet),
                    $paramsUcet
                );

                if (!$recUcet ->loadRecord()) {
                    throw new \Exception('Chyba pri aktualizaci zaznamu BankovniUcet - zaznam nebyl nalezen!');
                }
            }

            // provede update nebo insert bankovniho uctu
            if ($jenUpdateZaznamu  &&  $puvodniBankovniUcet) {
                $recUcet ->updateRecord();
            } else {
                // vlozi novy bankovni ucet
                $recUcet ->insertRecord();
            }

            $recVyplaceni2['SerioveCisloUctuVyplaty'] = $recUcet['SerioveCislo'];

            if ($jenUpdateZaznamu) {
                // provede pouze update
                $recVyplaceni2 ->updateRecord();
            } else {
                // provede insert noveho zaznamu ...
                $recVyplaceni2 ->insertRecord();
            }

            self::aktualizujStavZmeny($database, $zmena ->id, $logDescription);

            $database ->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }



    /**
     * Zapise do databaze novou zdravotni pojistovnu.
     * @param Database $database
     * @throws \Exception
     */
    protected static function aktualizacePojistovnyOsoby(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        $database ->beginTransaction();
        try {
            $platnostOd = DateUtils::dateStringToTimestamp($zmena ->hodnoty['PlatnostOd']);

            // ZdravotniPojistovna:KodZdravotniPojistovny:pojistovna:Pr;D:PlatnostOd:Platnost_od:Pr;D:PlatnostDo:Platnost_do;
            // E:CisloPojistence:Cislo_pojistence:Po;Uctarna:CisloUctarny:Cislo_uctarny:Po;S_TypZamestnanceProZP:TypZamestnance:Typ_zamestnance_ZP:Po;
            // soubor:soubor:Pripojit_dokument

            // ukonci platnost predchoziho zaznamu
            $rec = new RonPortalDBRecord($database, 'PojisteniOsobyU_ZP', $logDescription);
            $rec->logInfoMessage = 'Aktualizace zdravotni pojistovny - ukonceni platnosti predchoziho zaznamu';
            $rec->addStringField('RC', true);
            $rec->addStringField('KodZdravotniPojistovny', true);
            $rec->addDateField('PlatnostOd', true);
            $rec->addDateField('PlatnostDo', false, DateUtils::incDayStart($platnostOd, -1));
            $rec->addStringField('CisloPojistence', true);
            $rec->addStringField('CisloUctarny', true);
            $rec->addIntField('TypZamestnance', true);

            $params = $database->createParameters();
            $rec->setWhere(
                'RC = ' . $params->addParameter($zmena ->rc) .
                ' AND PlatnostDo IS NULL',
                $params);

            if ($rec ->loadRecord()) {
                // existuje predchozi zaznam
                if ($rec['PlatnostOd'] >= $platnostOd) {
                    // platnost od musi byt u predchoziho zaznamu mensi nez zadana hodnota
                    throw new \Exception('Nelze_ukoncit_platnost_pojistovny_k_zadanemu datu');
                }

                $rec ->updateRecord();
            }

            // a vlozi novy zaznam
            $rec2 = new RonPortalDBRecord($database, 'PojisteniOsobyU_ZP', $logDescription);
            $rec2 ->setXmlEvents('POJISTENI_OSOBY_U_ZP_BEFORE_POST', 'POJISTENI_OSOBY_U_ZP_AFTER_POST');
            $rec2 ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];
            $rec2->logInfoMessage = 'Aktualizace zdravotni pojistovny - vlozeni noveho zaznamu';
            $rec2->addStringField('RC', true, 15, $zmena ->rc);
            $rec2->addStringField('KodZdravotniPojistovny', true, 10, $zmena ->hodnoty['KodZdravotniPojistovny']);
            $rec2->addDateField('PlatnostOd', true, $platnostOd);
            //$rec2->addDateField('PlatnostDo', true, $zmena ->hodnoty['PlatnostDo'] ?: null);
            $rec2->addStringField('CisloPojistence', true, 15, $zmena ->hodnoty['CisloPojistence']);
            $rec2->addStringField('CisloUctarny', true, 10, $zmena ->hodnoty['CisloUctarny']);
            $rec2->addIntField('TypZamestnance', true, (int)$zmena ->hodnoty['TypZamestnance']);

            $rec2 ->insertRecord();

            self::aktualizujStavZmeny($database, $zmena ->id, $logDescription);

            $database ->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }


    /**
     * @param Database $database
     * @param object|string $logDescription
     * @param bool $zivotni
     * @param ?PracovniPomer $pp
     * @return RonPortalDBRecord
     */
    public static function getRecSmlouvaPojisteni(Database $database, $logDescription, bool $zivotni = false, ?PracovniPomer $pp = null)
    {
        $rec = new RonPortalDBRecord($database, $zivotni ? 'SmlouvaZivotniPojisteni' : 'SmlouvaPenzijnihoPrip', $logDescription);
        $eventsName = $zivotni ? 'SMLOUVA_ZIVOTNI_POJISTENI' : 'SMLOUVA_PENZIJNIHO_PRIP';
        $rec ->setXmlEvents("{$eventsName}_BEFORE_POST", "{$eventsName}_AFTER_POST");

        $rec->addStringField('RC', true, 15, $pp ->rc ?? null);
        $rec->addStringField('OsobniCislo', true, 15, $pp->osobniCislo ?? null);
        $rec->addDateField('DatumNastupu', true, $pp->datumNastupu ?? null);

        $rec->addDateField('PlatnostOd', true);
        $rec->addDateField('PlatnostDo', true);

        if ($zivotni) {
            $rec->addStringField('Pojistovna', true, 10);
        } else {
            $rec->addStringField('PenzijniFond', true, 10);
        }

        $rec->addStringField('CisloSmlouvy', true, 15);

        $rec->addFloatField('PrispevekZamestnavatele', true);
        $rec->addIntField('SerioveCisloUctu1', true);
        $rec->addFloatField('PrispevekZamestnance', true);
        $rec->addIntField('SerioveCisloUctu2', true);

        if (!$zivotni) {
            $rec ->addMemoField('Poznamka', true);
        }

        return $rec;
    }


    /**
     * Zapise do databaze nove zivotni pojisteni.
     * @param Database $database
     * @param NahlasenaZmena $zmena
     * @param string|object $logDescription
     * @param bool $zivotniPojisteni
     * @throws DBException
     */
    protected static function aktualizacePenzijkoZivotko(Database $database, NahlasenaZmena $zmena, $logDescription, bool $zivotniPojisteni)
    {
        // zivotnipojisteni
        // Pojistovna:Pojistovna:pojistovna:Pr;D:PlatnostOd:Platnost_od:Pr;D:PlatnostDo:Platnost_do;E:CisloSmlouvy:Cislo_smlouvy;
        // E:PrispevekZamestnavatele:prispevek_zamestnavatele;BF:SerioveCisloUctu1:Bankovni_ucet;E:PrispevekZamestnance:prispevek_zamestnance;
        // BZ:SerioveCisloUctu2:Bankovni_ucet;soubor:soubor:Pripojit_dokument

        // najde platny pracovni pomer osoby - prednostne HPP
        $pp = self::najdiPracovniPomer($database, $zmena ->rc);
        $platnostOd = DateUtils::dateStringToTimestamp($zmena->hodnoty['PlatnostOd']);
        $fieldName = $zivotniPojisteni ? 'Pojistovna' : 'PenzijniFond';

        if ($platnostOd < DateUtils::today()) {
            throw new \Exception(TextManager::_('Nelze_provest_zmenu_pripojisteni_zpetne'));
        }

        $database ->beginTransaction();
        try {
            $rec = self::getRecSmlouvaPojisteni($database, $logDescription, $zivotniPojisteni, $pp);
            $rec ->eventsParameters = [
                'NahlasenaZmena' => $zmena ->id,
                'HodnotyZmeny' => $zmena ->hodnoty
            ];
            $rec->logInfoMessage = 'Potvrzeni zmeny zivotniho pojisteni';

            $rec['PlatnostOd'] = $platnostOd;
            $rec['PlatnostDo'] = $zmena->hodnoty['PlatnostDo'] ? DateUtils::dateStringToTimestamp($zmena->hodnoty['PlatnostDo']) : null;

            $rec[$fieldName] = $zmena->hodnoty[$fieldName];
            $rec['CisloSmlouvy'] = $zmena->hodnoty['CisloSmlouvy'];

            $rec['PrispevekZamestnavatele'] = (float)($zmena->hodnoty['PrispevekZamestnavatele'] ?? 0) ?: null;
            $rec['SerioveCisloUctu1'] = (int)($zmena->hodnoty['SerioveCisloUctu1'] ?? 0) ?: null;
            $rec['PrispevekZamestnance'] = (float)($zmena->hodnoty['PrispevekZamestnance'] ?? 0) ?: null;
            $rec['SerioveCisloUctu2'] = (int)($zmena->hodnoty['SerioveCisloUctu2'] ?? 0) ?: null;

            $params = $database->createParameters();
            $rec->setWhere(
                'RC = ' . $params->addParameter($pp->rc) .
                ' AND OsobniCislo = ' . $params->addParameter($pp->osobniCislo) .
                ' AND DatumNastupu = ' . $params->addDateParameter($pp->datumNastupu) .
                ' AND PlatnostOd = ' . $params->addDateParameter($platnostOd) .
                " AND $fieldName = " . $params->addParameter($zmena->hodnoty[$fieldName]),
                $params);

            if ($rec ->loadRecord()) {
                // zaznam uz existuje - provede jen update (oprava drive zadaneho zaznamu)
                $rec ->updateRecord();
            } else {
                // zaznam neexistuje - ukonci pripadny predchozi zaznam a pak provede insert
                // nejdrive se pokusi najit zaznam se stejnym fondem/pojistovnou (napr. pri zmene castky)
                $rec2 = self::getRecSmlouvaPojisteni($database, $logDescription, $zivotniPojisteni);
                $params2 = $database ->createParameters();
                $rec2 ->setWhere(
                    'RC = ' . $params2->addParameter($pp->rc) .
                    ' AND OsobniCislo = ' . $params2->addParameter($pp->osobniCislo) .
                    ' AND DatumNastupu = ' . $params2->addDateParameter($pp->datumNastupu) .
                    ' AND PlatnostDo IS NULL' .
                    " AND $fieldName = " . $params2->addParameter($zmena->hodnoty[$fieldName]),
                    $params2);

                if ($rec2 ->loadRecord()) {
                    // nasel => ukonci platnost
                    $rec2['PlatnostDo'] = DateUtils::incDay($platnostOd, -1);
                    $rec2 ->updateRecord();
                } else {
                    // nanasel => zkusi jeste najit zaznam pro jiny fond (napr. pri zmene fondu)
                    $params2 = $database ->createParameters();
                    $rec2 ->setWhere(
                        'RC = ' . $params2->addParameter($pp->rc) .
                        ' AND OsobniCislo = ' . $params2->addParameter($pp->osobniCislo) .
                        ' AND DatumNastupu = ' . $params2->addDateParameter($pp->datumNastupu) .
                        ' AND PlatnostDo IS NULL',
                        $params2);

                    if ($rec2 ->loadRecord()) {
                        $rec2['PlatnostDo'] = DateUtils::incDay($platnostOd, -1);
                        $rec2 ->updateRecord();
                    }
                }

                // ... a nakonec provede vlozeni noveho zaznamu
                $rec ->insertRecord();
            }

            $rec->updateOrInsert();

            self::aktualizujStavZmeny($database, $zmena->id, $logDescription);

            $database->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }


    /**
     * Zapise do databaze nove vzdelani osoby.
     * @param Database $database
     * @throws \Exception
     */
    protected static function aktualizaceVzdelaniOsoby(Database $database, NahlasenaZmena $zmena, $logDescription)
    {
        // TypVzdelani:TypVzdelani:Typ_vzdelani:Pr;OborVzdelani:OborVzdelani:Obor_vzdelani:Pr;D:DatumUkonceni:Datum_ukonceni:Po;
        // Skola:Skola:Skola;D:DatumOd:Datum_od;D:DatumDo:Datum_do;E:Zamereni:Zamereni;
        // S_ZpusobUkonceniSkoly:VzdelaniUkonceno:Vzdelani_ukonceno:Po;soubor:soubor:Pripojit_dokument
        if (!$zmena ->hodnoty['DatumUkonceni']) {
            throw new \Exception('Neni_zadano_datum_ukonceni');
        }

        $datumOd = $zmena ->hodnoty['DatumOd'] ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['DatumOd']) : null;
        $datumDo = $zmena ->hodnoty['DatumDo'] ? DateUtils::dateStringToTimestamp($zmena ->hodnoty['DatumDo']) : null;

        $datumUkonceni = DateUtils::dateStringToTimestamp($zmena->hodnoty['DatumUkonceni']);

        if ($datumOd  &&  $datumDo  &&  $datumDo < $datumOd) {
            throw new \Exception(\Ron\ronText('vzdelani_chyba_platnostDo_mensi_platnostiOd'));
        }

        $database ->beginTransaction();
        try {
            $rec = new RonPortalDBRecord($database, 'VzdelaniOsoby', $logDescription);
            $rec->logInfoMessage = 'Potvrzeni zmeny vzdelani osoby';

            $rec->addStringField('RC', false, 15, $zmena->rc);

            $rec->addIntField('KodVzdelani', false, (int)$zmena->hodnoty['TypVzdelani']);
            $rec->addStringField('OborVzdelani', false, 10, $zmena->hodnoty['OborVzdelani']);
            $rec->addDateField('DatumUkonceni', false, $datumUkonceni);

            $rec->addStringField('Skola', false, 10, ($zmena->hodnoty['Skola'] ?? null) ?: null);
            $rec->addStringField('Zamereni', false, 40, ($zmena->hodnoty['Zamereni'] ?? null) ?: null);

            $rec->addDateField('DatumOd', false, $datumOd);
            $rec->addDateField('DatumDo', false, $datumDo);

            $rec->addIntField('VzdelaniUkonceno', false, $zmena->hodnoty['VzdelaniUkonceno']);

            $params = $database->createParameters();
            $rec->setWhere(
                'RC = ' . $params->addParameter($zmena->rc) .
                ' AND KodVzdelani  = ' . $params->addParameter((int)$zmena->hodnoty['TypVzdelani']) .
                ' AND OborVzdelani  = ' . $params->addParameter($zmena->hodnoty['OborVzdelani']) .
                ' AND DatumUkonceni  = ' . $params->addDateParameter($datumUkonceni),
                $params);

            $rec->updateOrInsert();

            self::aktualizujStavZmeny($database, $zmena->id, $logDescription);

            $database->commitTransaction();
        } catch (\Exception $e) {
            $database ->rollbackTransaction();
            throw $e;
        }
    }


    /**
     * @param Database $database
     * @param int $idZmeny
     * @param object|string $logDescription
     * @return void
     * @throws \Exception
     */
    public static function potvrdZmenu(Database $database, int $idZmeny, $logDescription)
    {
        $zmena = NahlasenaZmena::nactiZmenu($database, $idZmeny);

        if ($zmena ->stav) {
            throw new \Exception('Zmena uz byla zpracovana nebo zamitnuta!');
        }

        switch ($zmena ->typZmeny) {
            case 'osobni' :
                self::aktualizaceOsobnichUdajuOsoby($database, $zmena, $logDescription);
                break;

            case 'trvale' :
            case 'prechodne' :
                self::aktualizaceAdresyOsoby($database, $zmena, $logDescription);
                break;

            case 'prukaz' :
                self::aktualizacePrukazuOsoby($database, $zmena, $logDescription);
                break;

            case 'novyclenrodiny' :
            case 'rodinnyprislusnik' :
                self::aktualizaceRodinnychPrislusniku($database, $zmena, $logDescription);
                break;

            case 'rodinnystav' :
                self::aktualizaceRodinnehoStavuOsoby($database, $zmena, $logDescription);
                break;

            case 'ucetosoby' :
                self::aktualizaceUctuOsoby($database, $zmena, $logDescription);
                break;

            case 'pojistovna' :
                self::aktualizacePojistovnyOsoby($database, $zmena, $logDescription);
                break;

            case 'zivotnipojisteni' :
                self::aktualizacePenzijkoZivotko($database, $zmena, $logDescription, true);
                break;

            case 'penzijnipripojisteni' :
                self::aktualizacePenzijkoZivotko($database, $zmena, $logDescription, false);
                break;

            case 'vzdelani' :
                self::aktualizaceVzdelaniOsoby($database, $zmena, $logDescription);
                break;

            default :
                throw new \Exception("Zapis zmeny {$zmena ->typZmeny} neni implementovan!");
        }

    }
}
