<?php
/**
 * Roman.Szablatura RON Software 3.10.2017
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\SqlStatement;
use Ron\DB\Database;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterDatumOdDo;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;



/**
 * Class NahlaseneZmenyBrowser
 */
class NahlaseneZmenyBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby */
    var $pravo_nahlaseni;

    /** @var  PravaNaOsoby */
    var $pravo_schvaleni;


    /**
     * NahlaseneZmenyBrowser constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'nahlasenezmeny');

        Prava::nastavObdobiProPrava(time(),time());
        $this->pravo_nahlaseni = Prava::vratPrava('ou_zmeny_nahlaseni');
        $this->pravo_schvaleni = Prava::vratPrava('ou_zmeny_schval');

        $this->code = 'browsernahlasenezmeny';
        $this->tableId = 'browsernahlasenezmeny';
        $this->browserUrl = '?menuid=nahlasenezmeny';
        
        $this->title = \Ron\ronText('nahlasene_zmeny');

        $this->sqlSelectPart =
            "NahlasenaZmena.ID, NahlasenaZmena.Stav, NahlasenaZmena.RC";    //0..2

        $this->primaryField = 0;

        $this->sqlFromPart =
            "(NahlasenaZmena LEFT JOIN TypNahlaseneZmeny ON (NahlasenaZmena.TypZmeny = TypNahlaseneZmeny.Kod)) 
                LEFT JOIN Osoba OsobaZmeny ON (NahlasenaZmena.RC = OsobaZmeny.RC) " .
                "LEFT JOIN Osoba Zpracovatel ON (NahlasenaZmena.Zpracoval = Zpracovatel.RC)" .
                 "LEFT JOIN Osoba Nahlasil ON (NahlasenaZmena.Nahlasil = Nahlasil.RC)";
                  //"LEFT JOIN PracovniPomer PP ON (PP.RC = NahlasenaZmena.RC)";

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCas') ->setHeaderText('datum_hlaseni') ->setFormat('date')
            ->setCode('col-datum')
            ->setOrdering(true) ->setAlign('center');

        $this->defaultSort = '-col-datum';
        
        //$this->addField(new StavZmenyField());
        $this->addField(BrowserField::FIELD_TYPE_SELECT, 1)
            ->setSqlFieldName('NahlasenaZmena.Stav') ->setOrdering(true)
            ->setHeaderText('stav_zmeny')
            ->setCode('col-stavzmeny')
            ->setOptions([
                0 => \Ron\ronText('nezpracovano'),
                1 => \Ron\ronText('zpracovano'),
                2 => \Ron\ronText('odmitnuto')
            ]);

        //$this->addField(new StavZmenyField());
        $this->addField()
            ->setSqlFieldName('TypNahlaseneZmeny.Popis') ->setOrdering(true)
            ->setHeaderText('typ_zmeny')
            ->setCode('col-typzmeny')
            ->setLink('?menuid=nahlasenezmeny'); // . ($backMenuId ? '&back=' . $backMenuId : ''))
        //$this->addField(new TypZmenyField());

//        $this->addField(new ZobrazeniDetailuNahlaseneZmenyField($this->database))
//            ->setAlign('center') ->setHeaderText('detail')
//            ->setCode('col-detail')
//            ->setLink('?menuid=nahlasenezmeny&record=');

        // osoba zmeny
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Prijmeni') ->setCode('col-osoba-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Jmeno') ->setCode('col-osoba-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Titul') ->setCode('col-osoba-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-osoba-jmeno', 'col-osoba-prijmeni', null, 'col-osoba-titul'))
            ->setCode('col-osoba')
            ->setSortSQL('OsobaZmeny.Prijmeni, OsobaZmeny.Jmeno, OsobaZmeny.Titul')
            ->setSortSQLDesc('OsobaZmeny.Prijmeni DESC, OsobaZmeny.Jmeno DESC, OsobaZmeny.Titul DESC')
            ->setHeaderText('Osoba')
            ->setOrdering(true);

        // nahlasil
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Prijmeni') ->setCode('col-nahlasil-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Jmeno') ->setCode('col-nahlasil-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Titul') ->setCode('col-nahlasil-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-nahlasil-jmeno', 'col-nahlasil-prijmeni', null, 'col-nahlasil-titul'))
            ->setCode('col-nahlasil')
            ->setSortSQL('Nahlasil.Prijmeni, Nahlasil.Jmeno, Nahlasil.Titul')
            ->setSortSQLDesc('Nahlasil.Prijmeni DESC, Nahlasil.Jmeno DESC, Nahlasil.Titul DESC')
            ->setHeaderText('zmenu_nahlasil') ->setClassName('poznamka')
            ->setOrdering(true);

        //$this->addField(new ZpracovatelField($this->database));
        // zpracoval
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Prijmeni') ->setCode('col-zpracoval-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Jmeno') ->setCode('col-zpracoval-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Titul') ->setCode('col-zpracoval-titul');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('NahlasenaZmena.Zpracoval') ->setCode('col-zpracoval-rc');

        $this ->addField($f = new JmenoPrijmeniBrowserField($this ->database, 'col-zpracoval-jmeno', 'col-zpracoval-prijmeni', null, 'col-zpracoval-titul'))
            ->setCode('col-zpracoval')
            ->setSortSQL('Zpracovatel.Prijmeni, Zpracovatel.Jmeno, Zpracovatel.Titul')
            ->setSortSQLDesc('Zpracovatel.Prijmeni DESC, Zpracovatel.Jmeno DESC, Zpracovatel.Titul DESC')
            ->setHeaderText('zpracoval') ->setClassName('poznamka')
            ->setOrdering(true);
        $f ->setIndexIfNull('col-zpracoval-rc'); // v tabulce neni vazba - pokud nenajde osobu podle RC, tak zobrazi primo hodnotu pole "zpracoval"

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCasZpracovani')
            ->setCode('col-datumzpracovani')
            ->setHeaderText('datum_zpracovani') ->setFormat('date') ->setOrdering(true) ->setAlign('center');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(NahlasenaZmena.Poznamka)}')
            ->setClassName('poznamka') ->setHeader(TextManager::_('Poznamka') . ': ') ->setHeaderColSpan(-1)
            ->setCode('col-poznamka');
    }

    function setFilterForm()
    {
        $this->filterForm = new RonPortalFilterForm($this ->database, 'nahlasenezmeny', true);

        $this->filterForm->addFilter(
            (new FilterFormFilterDatumOdDo($this ->filterForm))
                ->setPovolitPrazdneDatumDo(true)
                ->setDatumOdDoLinks([
                        'today',
                        'currentMonth'
                    ]
                ),
            true
        );

        ($this ->filterForm ->addFilterPP(true, [Prava::vratPrava('ou_zmeny_nahlaseni'), Prava::vratPrava('ou_zmeny_schval')]))
            ->setVcetneExternich(true)
            ->setDatumOd(time())
            ->setDatumDo(time());

        $this ->filterForm ->addFilterRozdeleni1();
        $this ->filterForm ->addFilterRozdeleni2();

        // typ zmeny
        $filterTyp = new FilterFormFilter('filtrtypzmeny', FilterFormFilter::TYPE_OPTIONS);
        $filterTyp
            ->setSessionName('filtr_kodTypuZmeny') ->setCaption(TextManager::_('typ_zmeny'))
            ->addItem('', TextManager::_('_vse_'))
            ->setOptionsSql('SELECT Kod, Popis FROM TypNahlaseneZmeny WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis');
        $this ->filterForm->addFilter($filterTyp);

        // zpracoval
        $filterZpracoval = new FilterFormFilter('zpracovatel', FilterFormFilter::TYPE_OPTIONS);
        $filterZpracoval
            ->setSessionName('filtr_zpracovatelzmeny') ->setCaption(TextManager::_('zpracoval'))
            ->addItem('', TextManager::_('_vsichni_'))
            ->setOptionsSql(
                "SELECT DISTINCT MAX(PracovniPomer.ID), Coalesce (Zpracovatel.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Zpracovatel.Jmeno, '') Zpracovatel" .
                ' FROM NahlasenaZmena, Osoba Zpracovatel, PracovniPomer' .
                ' WHERE NahlasenaZmena.Zpracoval = Zpracovatel.RC' .
                ' AND Zpracovatel.RC = PracovniPomer.RC' .
                ' AND (%FILTER%)' .
                ' GROUP BY Zpracovatel.Prijmeni, Zpracovatel.Jmeno' .
                ' ORDER BY %ORDER%',
                '1 = 1', 'Zpracovatel'
            );
        $this ->filterForm ->addFilter($filterZpracoval);

        // stav
        $filterStav = new FilterFormFilter('stavzmeny', FilterFormFilter::TYPE_OPTIONS);
        $filterStav
            ->setSessionName('filtr_stavZmeny') ->setCaption(TextManager::_('stav_zmeny'))
            ->addItem('', TextManager::_('_vse_'))
            ->addItem('0', TextManager::_('nezpracovano'))
            ->addItem('1', TextManager::_('zpracovano'))
            ->addItem('2', TextManager::_('odmitnuto'));
        $this ->filterForm ->addFilter($filterStav);

        // odkaz na osobni udaje
        $this ->filterForm ->addFilter((new FilterFormFilter('linkosoba', FilterFormFilter::TYPE_TEXT))
            ->setDefaultValue("<a href='?menuid=osobniudaje'>" . \Ron\ronText('osobni_udaje') . "</a>")
            ->setCaption(false)
        );
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na osobu zmeny pravo
        if ($this->pravo_nahlaseni->lzeZobrazitRC($values[2]) || $this->pravo_schvaleni->lzeZobrazitRC($values[2])){
            return true;
        }
        return false;
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        switch ($values[1]) { // stav
            case 1 :
                $className .= ' zpracovana-zmena';
                break;
            case 2 :
                $className .= ' odmitnuta-zmena';
                break;
            default :
                $className .= ' nezpracovana-zmena';
                break;
        }

        return parent::getRowAttributes($values, $className, $rowNumber);
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);
        $this->renderSeparator();
    }

    /**
     * @return SqlStatement
     * @throws \Ron\DB\DBException
     */
    public function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result = $this ->filterForm ->buildSqlFilter($params);

        $pp = null;
        if ($this ->filterForm ->getFilterValue('osoba')) {
            $pp = PracovniPomer::nactiPP($this ->database, $this ->filterForm ->getFilterValue('osoba'));
        }


        if ($this ->filterForm ->getFilterValue('datumod')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' NahlasenaZmena.DatumCas >= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod'));

            if ($this ->filterForm ->getFilterValue('datumod', 'datumDo')) {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' NahlasenaZmena.DatumCas <= ' . $params ->addDateParameter($this ->filterForm ->getFilterValue('datumod', 'datumDo'));
            }
        }

        if ($typZmeny = $this ->filterForm ->getFilterValue('filtrtypzmeny')) {
            $result .=
                ($result ? ' AND ' : '') .
                'TypNahlaseneZmeny.kod = ' . $params->addParameter($typZmeny);
        }

        $stav = $this ->filterForm ->getFilterValue('stavzmeny');
        if (isset($stav)) {
            $result .=
                ($result ? ' AND ' : '') .
                (!$stav
                    ? '(NahlasenaZmena.DatumCasZpracovani IS NULL OR NahlasenaZmena.Zpracoval IS NULL)'
                    : '') .
                (($stav == 1)
                    ? '(NahlasenaZmena.Stav = ' . $params->addParameter($stav) . ')' .
                      ' AND (NahlasenaZmena.DatumCasZpracovani IS NOT NULL OR NahlasenaZmena.Zpracoval IS NOT NULL)' .
                      ' OR (NahlasenaZmena.Stav =  0 AND (NahlasenaZmena.DatumCasZpracovani IS NOT NULL OR NahlasenaZmena.Zpracoval IS NOT NULL))'
                    : '') .
                (($stav == 2)
                    ? ' NahlasenaZmena.Stav = ' . $params->addParameter($stav)
                    : '');
        }

        if($pp) {
            $result .=
                ($result ? ' AND ' : '') .
                'NahlasenaZmena.RC = ' . $params->addParameter($pp->rc);
        } else {
            foreach ($this ->filterForm ->getRozdeleniFilters() as $rozdeleniFilter) {
                $sqlFiltr =
                    $rozdeleniFilter ->sestavSQLFiltr($this ->database, 'NahlasenaZmena', $params, true);

                if ($sqlFiltr) {
                    $result .=
                        ($result ? ' AND ' : '') .
                        '(' . $sqlFiltr . ')';
                }
            }
        }

        if ($zpracovatel = $this ->filterForm ->getFilterValue('zpracovatel')) {
            try {
                $pracovniPomerZpracovatel = PracovniPomer::nactiPP($this->database, $zpracovatel);

                $result .=
                    ($result ? ' AND ' : '') .
                    'NahlasenaZmena.Zpracoval = ' . $params->addParameter($pracovniPomerZpracovatel->rc);
            } catch (\Exception $e) {
                // zpracovatel neexistuje
                $result .= ' AND 0 = 1';
            }
        }

        return new SqlStatement($result, $params);
    }
}