<?php
/**
 * stanislav.elbl, 30.07.2021 16:23
 */

namespace RonPortal\Zamestnanci;


use Ron\DB\Database;
use Ron\DB\DBException;

class Prukaz
{
    /** @var string */
    public $rc;
    /** @var int */
    public $typPrukazu;
    /** @var string */
    public $typPrukazuPopis;

    /** @var string */
    public $cisloPrukazu;
    /** @var int */
    public $platnostOd;
    /** @var int */
    public $platnostDo;

    /** @var string */
    public $vydalStat;

    /** @var string */
    public $poznamka;


    /**
     *  Nacte seznam vsech prukazu osoby/vsech prukazu vybraneho typu.
     * @param Database $database
     * @param string $rc
     * @param int $typPrukazu
     * @param int $platneKDatu
     * @return Prukaz[]
     * @throws DBException
     * @throws \Exception
     */
    public static function nactiPrukazyOsoby($database, $rc, $typPrukazu = null, $platneKDatu = null)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT Prukaz.TypPrukazu, CisloPrukazu, PlatnostOd, PlatnostDo, VydalStat, ' . $database ->castWideMemo('Poznamka') . ', TypPrukazu.Popis' .
            ' FROM Prukaz JOIN TypPrukazu ON TypPrukazu.Kod = Prukaz.TypPrukazu ' .
            ' WHERE RC = ' . $params->addParameter($rc) .
            ($typPrukazu
                ? ' AND TypPrukazu = ' . $params->addParameter($typPrukazu)
                : ''
            ) .
            ($platneKDatu
                ? ' AND (PlatnostDo >= ' . $params ->addDateParameter($platneKDatu) . ' OR PlatnostDo IS NULL)'
                : ''
            ) .
            ' ORDER BY TypPrukazu, PlatnostOd DESC';

        $cursor = $database->getSqlResult($sql, $params);

        $prukazy = [];

        while ($row = $cursor->fetch()) {
            $prukaz = new static();

            $prukaz->rc = $rc;
            $prukaz->nactiDataZRadku($database, $row);

            if ($typPrukazu > 0) {
                $prukazy[] = $prukaz;
            }
            else {
                $prukazy[$row[0]][] = $prukaz;
            }
        }

        $cursor ->close();

        return $prukazy;
    }


    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiDataZRadku($database, $row)
    {
        $this->typPrukazu = $row[0];
        $this->cisloPrukazu = $row[1];
        $this->platnostOd = $database->stringToTimestamp($row[2]);
        $this->platnostDo = $database->stringToTimestamp($row[3]);
        $this->vydalStat = $row[4];
        $this->poznamka = $row[5];
        $this->typPrukazuPopis = $row[6];
    }
}
